#include "includes.h"
#include "TalComponent.h"


TalComponent::TalComponent (TalCore* const ownerFilter)
: AudioProcessorEditor (ownerFilter)
{
    internalCachedBackgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);

	// Load knob filmstrip image
	Image knobImage = ImageCache::getFromMemory(bmp00129_png, bmp00129_pngSize);

	// Add knobs
	inputDriveKnob = addNormalKnob(22, 43, ownerFilter, knobImage, INPUTDRIVE);
	delayTimeKnob = addNormalKnob(105, 43, ownerFilter, knobImage, DELAYTIME);
	highCutKnob = addNormalKnob(245, 43, ownerFilter, knobImage, HIGHCUT);
	cutoffKnob = addNormalKnob(330, 43, ownerFilter, knobImage, CUTOFF);
	feedbackKnob = addNormalKnob(415, 43, ownerFilter, knobImage, FEEDBACK);
	wetKnob = addNormalKnob(645, 43, ownerFilter, knobImage, WET);
	dryKnob = addNormalKnob(730, 43, ownerFilter, knobImage, DRY);

	// Add meter
	int width = 100;
	int height = 20;
	int distance = 35;
	addAndMakeVisible(meter = new TalMeter(ownerFilter, width, height, distance));
    meter->setBounds(526, 58, width, 2 * height + distance);

	// Combo Box
	addAndMakeVisible(delaySyncComboBox = new ComboBox("delaySyncComboBox"));
	delaySyncComboBox->setBounds(125, 147, 50, 18);
	delaySyncComboBox->setColour(ComboBox::backgroundColourId, Colour((juce::uint8)10, (juce::uint8)46, (juce::uint8)89));
	delaySyncComboBox->setColour(ComboBox::buttonColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
	delaySyncComboBox->setColour(ComboBox::arrowColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
	delaySyncComboBox->setColour(ComboBox::textColourId, Colour::greyLevel(0.8f));
	delaySyncComboBox->addItem("FREE",1);
	delaySyncComboBox->addItem("1/16",2);
	delaySyncComboBox->addItem("1/8",3);
	delaySyncComboBox->addItem("1/4",4);
	delaySyncComboBox->addItem("1/2",5);
	delaySyncComboBox->addItem("1/1",6);
	delaySyncComboBox->addItem("2/1",7);
	delaySyncComboBox->addItem("1/16.",8);
	delaySyncComboBox->addItem("1/8.",9);
	delaySyncComboBox->addItem("1/4.",10);
	delaySyncComboBox->addItem("1/2.",11);
	delaySyncComboBox->addItem("1/1.",12);
	delaySyncComboBox->addItem("2/1.",13);
	delaySyncComboBox->addItem("1/16T",14);
	delaySyncComboBox->addItem("1/8T",15);
	delaySyncComboBox->addItem("1/4T",16);
	delaySyncComboBox->addItem("1/2T",17);
	delaySyncComboBox->addItem("1/1T",18);
	delaySyncComboBox->addItem("2/1T",19);
	delaySyncComboBox->addItem("-",20);
	delaySyncComboBox->addListener (this);

	// Version info
	versionLabel = new Label("Version Info", "V 0.95b");
	versionLabel->setBounds(32, 145, 100, 20);
	versionLabel->setColour(Label::textColourId, Colour((juce::uint8)166, (juce::uint8)166, (juce::uint8)166, 0.8f));
	addAndMakeVisible(versionLabel);

	tabButton = addTabButton(720, 147, ownerFilter, bmp00136_png, bmp00136_pngSize, bmp00137_png, bmp00137_pngSize, -1);

	delayTwiceLButton = addToggleButton(209, 58, ownerFilter, bmp00134_png, bmp00134_pngSize, bmp00135_png, bmp00135_pngSize, DELAYTWICE_L);
	delayTwiceRButton = addToggleButton(209, 93, ownerFilter, bmp00134_png, bmp00134_pngSize, bmp00135_png, bmp00135_pngSize, DELAYTWICE_R);

        updateParametersFromFilter();

	// set our component's initial size to be the last one that was stored in the filter's settings
    setSize (internalCachedBackgroundImage.getWidth(), internalCachedBackgroundImage.getHeight());

    // register ourselves with the filter - it will use its ChangeBroadcaster base
    // class to tell us when something has changed, and this will call our changeListenerCallback()
    // method.
    ownerFilter->addChangeListener (this);

	tooltipWindow.setMillisecondsBeforeTipAppears(200);
}

TalComponent::~TalComponent()
{
    //ImageCache::release (internalCachedBackgroundImage);
    getFilter()->removeChangeListener (this);
    deleteAllChildren();
}

FilmStripKnob* TalComponent::addNormalKnob(int x, int y, TalCore* const ownerFilter, Image knobImage, int parameter)
{
	FilmStripKnob *filmStripKnob;
	addAndMakeVisible(filmStripKnob = new FilmStripKnob(knobImage,
									 knobImage.getHeight() / knobImage.getWidth(),
									 false, parameter));
    filmStripKnob->setBounds(x, y, knobImage.getWidth(), knobImage.getWidth());
	filmStripKnob->addListener (this);
	return filmStripKnob;
}


ImageToggleButton* TalComponent::addToggleButton(int x, int y, TalCore* const ownerFilter,
	const char *imageDataOff, int imageDataOffSize, const char *imamgeDataOn, int imageDataOnSize, int parameter)
{
	Image toggleImgOff = ImageCache::getFromMemory(imageDataOff, imageDataOffSize);
	Image toggleImgOn = ImageCache::getFromMemory(imamgeDataOn, imageDataOnSize);

	ImageToggleButton *toggleButton = new ImageToggleButton("Toggle Button", toggleImgOff, toggleImgOn);
	toggleButton->setBounds(x, y, toggleImgOff.getWidth(), toggleImgOff.getHeight());
	addAndMakeVisible(toggleButton, 0);
	toggleButton->addListener (this);
	return toggleButton;
}

ImageTabButton* TalComponent::addTabButton(int x, int y, TalCore* const ownerFilter,
	const char *imageDataOff, int imageDataOffSize, const char *imamgeDataOn, int imageDataOnSize, int parameter)
{
	Image tabImgOff = ImageCache::getFromMemory(imageDataOff, imageDataOffSize);
	Image tabImgOn = ImageCache::getFromMemory(imamgeDataOn, imageDataOnSize);

	ImageTabButton *tabButton = new ImageTabButton("Tab Button", tabImgOff, tabImgOn);
	tabButton->setBounds(x, y, tabImgOff.getWidth(), tabImgOff.getHeight());
	addAndMakeVisible(tabButton, 0);
	tabButton->addListener (this);
	return tabButton;
}

//==============================================================================
void TalComponent::paint (Graphics& g)
{
    // just clear the window
    g.fillAll (Colour::greyLevel (0.2f));

    g.drawImage (internalCachedBackgroundImage,
                 0, 0, internalCachedBackgroundImage.getWidth(), internalCachedBackgroundImage.getHeight(),
                 0, 0, internalCachedBackgroundImage.getWidth(), internalCachedBackgroundImage.getHeight());
}

//==============================================================================
void TalComponent::changeListenerCallback (ChangeBroadcaster* source)
{
    // this is the filter telling us that it's changed, so we'll update our
    // display of the time, midi message, etc.
    updateParametersFromFilter();
}

void TalComponent::sliderValueChanged (Slider* caller)
{
    TalCore* const filter = getFilter();
	if (caller == inputDriveKnob) filter->setParameterNotifyingHost(INPUTDRIVE, (float)inputDriveKnob->getValue());
	if (caller == highCutKnob) filter->setParameterNotifyingHost(HIGHCUT, (float)highCutKnob->getValue());
	if (caller == cutoffKnob) filter->setParameterNotifyingHost(CUTOFF, (float)cutoffKnob->getValue());
	if (caller == wetKnob) filter->setParameterNotifyingHost(WET, (float)wetKnob->getValue());
	if (caller == dryKnob) filter->setParameterNotifyingHost(DRY, (float)dryKnob->getValue());
	if (caller == feedbackKnob) filter->setParameterNotifyingHost(FEEDBACK, (float)feedbackKnob->getValue());
	if (caller == delayTimeKnob) filter->setParameterNotifyingHost(DELAYTIME, (float)delayTimeKnob->getValue());
	setTooltip(caller);
}

void TalComponent::setTooltip(Slider* slider)
{
	if (slider == feedbackKnob)
	{
		// Set delayTime tooltip
		String text = String::formatted(T("%4.4f"),
			audioUtils.calculateFeedback((float)slider->getValue()));
		slider->setTooltip(text);
	}
	if (slider == delayTimeKnob)
	{
		// Set delayTime tooltip
		String text = String::formatted(T("%4.0f ms"), (float)delayTimeKnob->getValue() * 4000.0f);
		slider->setTooltip(text);
	}
	if (slider == inputDriveKnob
		|| slider == inputDriveKnob
		|| slider == wetKnob
		|| slider == dryKnob)
	{
		// Set dB tooltip
		String text = String::formatted(T("%4.1f dB"), audioUtils.getLogScaledValueInDecibel((float)slider->getValue(), 2.0f));
		slider->setTooltip(text);
	}
}

void TalComponent::comboBoxChanged (ComboBox* caller)
{
    TalCore* const filter = getFilter();
	if (caller == delaySyncComboBox) filter->setParameterNotifyingHost(DELAYTIMESYNC, float(delaySyncComboBox->getSelectedId()-1)/19.0f);
}

void TalComponent::buttonClicked (Button* caller)
{
    TalCore* const filter = getFilter();
	if (caller == delayTwiceLButton)
	{
		float value = caller->getToggleState() ? 1.0f : 0.0f;
		filter->setParameterNotifyingHost(DELAYTWICE_L, value);
	}
	if (caller == delayTwiceRButton)
	{
		float value = caller->getToggleState() ? 1.0f : 0.0f;
		filter->setParameterNotifyingHost(DELAYTWICE_R, value);
	}
	if (caller == tabButton)
	{
		// set new rate
		juce::int64 deltaTimeInMs = tabButton->setClick();
		if (deltaTimeInMs > (juce::int64)0 &&
			deltaTimeInMs < (juce::int64)4000)
		{
			float delayTime = (float)deltaTimeInMs / 4000.0f;
			delayTimeKnob->setValue(delayTime);
		}
	}
}

//==============================================================================
void TalComponent::updateParametersFromFilter()
{
    TalCore* const filter = getFilter();

    // we use this lock to make sure the processBlock() method isn't writing to the
    // lastMidiMessage variable while we're trying to read it, but be extra-careful to
    // only hold the lock for a minimum amount of time..
    filter->getCallbackLock().enter();

    // take a local copy of the info we need while we've got the lock..
	float inputDrive = filter->getParameter(INPUTDRIVE);
	float delayTime = filter->getParameter(DELAYTIME);
	float feedback = filter->getParameter(FEEDBACK);
	float highCut = filter->getParameter(HIGHCUT);
	float cutoff = filter->getParameter(CUTOFF);
	float wet = filter->getParameter(WET);
	float dry = filter->getParameter(DRY);

	float delayTimeTwiceL = filter->getParameter(DELAYTWICE_L);
	float delayTimeTwiceR = filter->getParameter(DELAYTWICE_R);
	float delaySync = filter->getParameter(DELAYTIMESYNC)*19.0f+1.0f;

	//float lfoSync = filter->getParameter(LFOSYNC);

    // ..release the lock ASAP
    filter->getCallbackLock().exit();

	inputDriveKnob->setValue(inputDrive, dontSendNotification);
	delayTimeKnob->setValue(delayTime, dontSendNotification);
	feedbackKnob->setValue(feedback, dontSendNotification);
	highCutKnob->setValue(highCut, dontSendNotification);
	cutoffKnob->setValue(cutoff, dontSendNotification);
	wetKnob->setValue(wet, dontSendNotification);
	dryKnob->setValue(dry, dontSendNotification);

	delayTwiceLButton->setValue(delayTimeTwiceL, dontSendNotification);
	delayTwiceRButton->setValue(delayTimeTwiceR, dontSendNotification);

	delaySyncComboBox->setSelectedId((int)delaySync, dontSendNotification);

	// update tooltips
	setTooltip(inputDriveKnob);
	setTooltip(delayTimeKnob);
	setTooltip(feedbackKnob);
	setTooltip(wetKnob);
	setTooltip(dryKnob);
}

//==============================================================================
// Binary resources - be careful not to edit any of these sections!



// JUCER_RESOURCE: bmp00128_png, 35135, "D:\Code\workspace svn\Tal-Dub-3-VST-Effect-juce\resources\bmp00128.png"
static const unsigned char resource_NewJucerComponent_bmp00128_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,3,68,0,0,0,170,8,6,0,0,0,141,191,143,65,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,126,90,73,68,65,84,120,218,236,125,119,160,37,69,149,247,239,84,247,123,51,204,12,
42,10,51,3,187,159,18,7,19,131,160,174,138,46,168,36,69,146,136,4,5,23,20,49,97,196,64,86,9,10,230,44,139,136,24,65,16,36,43,130,162,40,226,170,132,25,193,64,150,93,153,128,1,152,55,195,204,188,238,58,
223,31,93,93,93,85,93,125,211,187,161,239,189,117,244,49,239,221,216,93,117,234,156,243,59,145,0,48,2,5,10,20,40,80,160,64,129,2,5,10,20,104,12,73,132,37,8,20,40,80,160,64,129,2,5,10,20,40,208,184,82,
12,80,88,133,64,129,2,5,10,20,40,80,160,64,129,2,141,39,32,162,128,135,2,5,10,20,40,80,160,64,129,2,5,10,52,166,20,82,230,2,5,10,20,40,80,160,64,129,2,5,10,52,182,20,82,230,2,5,10,20,40,80,160,64,129,
2,5,10,52,182,20,34,68,129,2,5,10,20,40,80,160,64,129,2,5,26,91,10,53,68,129,2,5,10,20,40,80,160,64,129,2,5,26,95,64,20,82,230,2,5,10,20,40,80,160,64,129,2,5,10,52,174,20,82,230,2,5,10,20,40,80,160,64,
129,2,5,10,52,182,68,0,56,44,67,160,64,129,2,5,10,20,40,80,160,64,129,198,145,66,132,40,80,160,64,129,2,5,10,20,40,80,160,64,1,16,5,10,20,40,80,160,64,129,2,5,10,20,40,208,184,81,252,148,215,254,0,224,
212,120,136,0,228,127,71,64,52,129,188,19,29,197,97,193,2,5,170,34,9,0,107,37,128,245,168,206,70,37,0,19,0,145,58,119,164,222,153,246,232,170,4,16,79,128,98,106,216,62,133,1,112,126,233,72,1,36,8,217,
180,141,136,0,196,128,136,128,200,120,56,1,16,179,241,26,6,152,64,161,125,141,205,107,250,192,148,151,85,235,27,243,92,229,148,26,71,69,178,241,96,18,22,182,41,69,206,46,184,191,207,50,126,79,186,36,147,
38,212,247,50,32,40,251,53,178,247,215,226,13,86,255,17,100,95,218,186,92,30,165,176,223,205,133,156,67,228,200,45,145,253,30,207,2,69,172,63,155,32,244,83,0,64,2,224,4,224,36,85,15,38,125,148,33,17,64,
217,133,208,228,68,118,87,92,92,27,71,51,147,27,236,172,51,179,122,140,156,199,213,43,7,37,163,228,116,10,164,35,160,115,104,2,52,43,10,178,126,88,1,81,102,152,145,71,248,17,64,2,100,196,144,88,22,2,36,
80,160,64,142,34,151,185,113,214,172,52,47,81,103,174,79,98,51,97,48,17,40,106,172,154,89,171,208,0,134,90,50,53,200,1,67,230,154,49,1,49,16,96,80,19,107,45,95,162,68,107,164,236,44,169,227,65,238,105,
138,12,123,93,155,123,129,87,91,3,67,12,197,148,198,154,249,128,209,116,5,90,237,196,224,103,245,121,10,192,202,40,179,242,163,6,136,40,7,42,198,37,242,68,4,76,39,176,144,76,105,223,217,184,79,86,206,
138,8,34,202,174,37,3,218,100,95,158,249,73,81,148,125,191,100,244,206,65,85,177,82,177,50,162,169,242,18,203,82,134,11,231,65,43,199,140,19,9,36,172,129,22,71,145,178,229,24,156,34,3,141,3,2,69,52,17,
41,191,252,116,73,35,13,23,32,10,242,126,248,1,17,124,198,25,89,15,115,42,65,68,160,56,108,120,160,64,126,224,209,74,164,199,245,217,245,26,124,72,5,192,38,188,202,142,171,52,109,160,166,162,51,139,4,
57,158,236,168,248,157,154,236,252,184,146,54,117,242,5,153,70,225,109,91,79,89,116,32,86,62,3,199,48,100,47,211,114,83,152,31,136,1,76,22,73,242,108,122,114,114,57,52,221,131,239,76,236,67,129,4,72,162,
236,58,200,115,38,216,57,41,108,92,98,28,1,73,82,33,163,100,1,136,162,8,136,4,68,7,142,91,138,10,86,236,45,169,136,150,96,80,20,131,34,106,121,69,57,45,214,169,153,61,70,250,61,172,84,147,218,227,116,
2,64,10,70,164,182,41,251,157,162,193,72,41,2,192,49,1,137,48,4,233,244,240,73,54,65,65,198,15,179,86,39,34,48,169,244,15,157,163,160,188,220,42,174,74,4,64,138,160,205,3,5,234,42,165,142,98,239,149,1,
39,117,80,202,140,18,201,252,63,150,63,36,24,144,45,27,123,92,9,45,253,134,124,160,50,77,27,160,61,95,49,25,1,44,252,167,66,103,52,165,13,12,77,242,156,173,113,135,160,145,254,199,98,200,196,4,149,178,
119,103,197,253,210,68,128,35,145,29,163,200,0,66,185,127,72,166,224,216,184,216,60,114,36,34,64,38,141,224,66,219,96,136,21,16,210,203,64,145,25,242,239,29,24,138,144,129,33,209,222,78,22,43,73,96,39,
74,196,240,59,99,178,227,144,218,15,164,145,137,174,6,207,165,177,0,35,54,50,22,69,15,247,160,71,231,44,68,136,134,27,16,101,198,80,145,180,77,32,32,18,224,52,1,145,0,164,58,46,33,77,46,80,160,198,94,
254,88,0,211,49,138,180,185,252,71,58,130,93,54,81,150,178,55,87,206,84,36,25,185,217,70,9,128,52,69,191,83,69,134,147,242,66,8,211,147,77,206,190,5,197,216,208,60,158,246,24,178,30,135,64,41,57,42,207,
138,74,217,54,246,173,220,69,10,124,92,130,147,66,129,10,106,35,194,214,45,227,223,221,73,46,190,86,203,31,54,162,67,108,188,212,8,17,165,211,77,220,11,113,22,29,66,123,209,88,59,85,141,192,204,64,18,
43,38,229,138,123,225,14,246,32,210,30,41,138,24,136,162,206,236,231,168,240,31,112,194,64,68,250,210,178,212,55,255,210,96,58,46,135,88,73,201,174,56,139,14,13,50,190,65,10,20,73,229,192,179,215,124,
24,128,81,220,48,45,61,208,16,236,32,203,44,21,206,58,7,68,128,20,96,153,170,63,9,20,135,8,81,160,214,192,194,40,166,6,53,83,127,68,202,211,41,227,236,135,205,244,185,200,48,210,6,149,198,147,0,152,0,
167,217,89,103,43,107,143,85,84,56,120,213,91,7,67,205,151,138,37,130,35,169,173,179,100,22,141,20,199,135,125,91,144,70,6,191,154,221,126,2,15,219,50,134,109,176,152,72,232,240,137,76,219,144,112,221,
216,109,35,175,212,168,21,194,52,171,168,143,113,205,169,138,4,69,234,90,43,83,229,252,192,154,83,232,160,88,219,58,44,22,96,78,129,52,118,192,15,183,46,35,4,10,32,199,25,143,210,100,4,50,155,69,208,12,
183,120,58,227,117,70,212,20,200,144,32,112,172,128,30,59,247,19,103,181,68,117,73,246,18,177,128,36,86,206,69,12,137,115,131,128,80,78,50,252,128,136,116,116,200,110,237,163,15,110,238,62,9,123,29,168,
1,201,60,8,50,198,221,180,178,60,104,181,14,169,0,164,217,65,206,92,149,102,81,160,168,7,74,64,21,212,18,192,19,148,225,35,2,144,74,163,219,221,184,27,146,194,48,174,83,56,29,19,202,228,125,9,91,24,148,
99,211,40,9,114,66,159,147,72,1,113,211,123,238,242,188,155,77,106,62,46,200,1,66,57,63,27,238,243,177,230,227,6,96,83,255,195,125,0,68,13,128,81,218,12,228,48,160,59,191,181,88,43,150,38,89,202,101,28,
117,12,58,8,89,145,63,38,212,183,38,234,187,147,170,122,79,81,252,27,1,132,40,139,218,168,183,145,175,129,196,76,148,164,132,90,151,92,14,165,89,141,144,80,194,40,18,222,123,66,148,117,249,203,178,228,
84,87,185,72,212,210,188,163,136,138,149,158,30,134,243,22,133,250,250,81,0,68,186,189,101,146,218,160,200,4,66,61,200,139,220,120,46,227,212,221,250,131,252,239,250,59,240,185,155,26,247,12,63,224,217,
41,118,219,170,185,82,248,231,26,224,164,235,226,174,223,107,43,215,216,75,250,202,126,157,27,17,9,128,245,211,192,173,255,203,88,254,120,132,233,20,248,229,253,162,239,107,179,231,34,137,253,158,209,
56,180,126,193,82,81,121,109,141,214,32,135,53,57,148,241,152,99,0,178,174,213,18,192,202,85,192,205,127,38,172,153,150,248,159,255,45,142,91,17,41,138,80,20,2,119,70,71,191,32,193,211,231,55,190,223,
211,175,159,192,63,31,167,226,234,24,192,250,88,189,127,61,158,62,191,113,71,37,251,253,51,249,238,97,48,32,35,163,254,219,116,47,11,27,5,89,142,245,180,2,208,26,135,195,120,40,52,86,80,230,171,80,191,
9,213,121,204,90,48,46,106,241,125,194,198,119,248,116,167,186,113,5,245,17,202,29,228,76,6,100,91,152,233,247,244,26,60,114,197,38,54,219,171,196,40,46,106,150,162,102,182,95,207,123,104,71,29,243,102,
9,28,197,153,65,196,98,50,107,205,45,141,67,29,101,141,17,0,128,56,242,131,159,110,174,166,132,106,222,227,241,97,201,180,0,69,158,123,210,151,21,217,77,180,168,166,50,2,145,138,244,81,222,97,165,255,
221,255,90,190,90,85,239,22,32,209,144,235,166,141,15,190,36,243,21,233,83,163,2,167,68,193,163,25,168,101,146,41,178,118,165,130,124,50,121,164,40,79,109,39,106,220,22,149,19,229,104,208,61,132,213,27,
100,14,175,164,97,112,167,202,160,145,232,95,13,68,136,10,89,214,181,136,108,67,219,104,5,109,63,150,58,134,165,137,155,34,167,53,116,246,126,51,183,60,40,77,195,196,77,149,129,71,164,214,51,49,86,40,
46,225,75,125,76,170,172,190,4,0,79,99,248,106,136,102,114,22,141,52,78,33,138,245,154,174,74,137,99,99,125,243,46,21,117,173,209,16,104,175,102,199,152,61,36,122,23,149,101,48,136,85,83,3,209,251,179,
162,127,73,164,234,247,96,70,171,242,200,80,148,117,171,27,161,234,6,61,31,47,201,239,185,142,209,223,24,152,200,234,135,130,108,31,110,138,57,55,210,146,52,179,217,226,172,207,66,175,115,223,235,20,33,
122,207,139,19,44,218,184,245,207,91,51,13,156,249,139,8,127,95,77,93,187,215,97,142,16,229,118,138,228,204,214,23,2,152,48,150,230,188,91,4,126,255,127,162,231,107,211,139,8,17,27,16,37,143,14,233,123,
85,65,213,201,200,46,183,205,19,229,214,235,215,48,72,0,223,188,85,224,182,7,69,214,177,49,201,163,68,190,150,247,21,41,68,14,205,52,74,115,244,11,166,59,126,255,240,70,136,168,172,204,168,0,50,214,220,
53,163,76,133,172,134,76,185,241,153,218,118,36,151,63,218,53,232,57,128,34,235,254,179,212,57,97,244,58,54,249,126,58,123,101,26,101,41,114,121,255,4,119,17,99,7,128,14,37,200,231,106,144,211,210,251,
140,185,88,145,177,184,137,15,12,185,105,104,117,158,231,212,206,53,137,66,118,50,171,225,191,189,113,236,18,168,63,77,197,242,58,58,230,198,75,17,197,32,149,128,64,163,38,39,132,74,61,78,99,32,205,67,
193,117,225,85,213,213,210,72,159,102,4,231,215,176,146,128,76,179,238,82,156,22,30,236,49,138,12,253,231,22,178,45,48,4,0,115,38,128,87,46,146,99,203,52,190,68,47,129,204,102,153,140,108,48,4,0,111,124,
174,196,127,110,49,156,235,149,194,246,89,195,129,48,18,156,207,186,43,123,27,8,136,84,227,163,152,128,3,158,33,51,175,101,12,104,237,165,165,166,7,240,81,140,108,210,251,100,139,198,81,160,230,20,217,
63,164,234,13,102,85,116,103,50,75,46,242,172,36,43,51,201,53,44,147,210,87,5,165,216,194,150,228,109,182,201,68,144,249,233,75,50,15,4,27,237,151,35,232,238,197,46,46,24,206,5,152,116,126,204,144,162,
10,117,76,168,31,29,250,96,251,51,162,2,19,144,245,177,106,182,147,32,245,62,181,112,22,82,168,243,194,241,64,222,90,7,69,155,213,48,37,153,3,45,77,42,220,10,217,191,52,162,165,222,164,142,0,98,232,46,
130,53,187,58,149,245,225,179,15,66,14,198,48,81,92,108,36,245,245,68,253,125,53,225,237,151,55,246,250,247,58,178,50,119,18,216,115,235,106,67,125,205,116,6,126,124,244,162,167,50,126,255,16,227,79,43,
198,195,220,49,107,167,204,58,136,61,23,73,60,125,19,137,173,55,46,38,128,248,232,208,197,18,127,125,132,240,224,191,134,111,189,114,159,212,191,214,0,31,54,234,199,88,165,172,19,1,59,111,145,69,167,54,
152,112,15,88,177,46,155,204,1,54,153,203,120,120,117,54,224,152,41,50,50,228,216,201,90,81,121,211,148,102,35,219,101,47,26,45,140,35,153,220,171,128,16,252,49,186,162,237,175,103,208,7,123,172,45,179,
107,150,42,176,45,205,33,10,69,68,229,245,208,41,134,162,162,193,71,158,162,24,219,155,229,218,243,67,59,88,56,71,49,102,40,49,181,129,142,9,194,39,149,64,74,141,112,153,209,9,62,199,57,148,119,147,76,
93,224,24,25,78,118,198,232,152,108,236,57,192,52,34,71,46,177,142,130,207,38,31,101,202,75,217,121,2,96,204,2,210,188,90,119,208,148,2,34,182,29,103,57,11,26,179,130,93,30,12,106,160,182,146,88,157,38,
65,160,49,155,178,251,146,205,37,158,60,167,250,249,47,220,28,97,77,131,14,39,175,126,250,120,26,168,38,143,92,123,151,192,231,110,138,113,229,159,132,85,21,227,163,87,108,35,135,250,158,181,113,235,201,
123,250,229,253,2,75,150,217,167,39,183,81,200,0,216,58,205,82,57,104,41,31,77,17,231,181,44,202,250,137,66,125,79,243,221,200,127,140,46,79,250,167,153,225,20,1,179,168,117,91,162,148,165,81,181,55,70,
222,196,180,49,70,133,195,118,122,205,87,114,129,81,84,177,80,230,130,122,240,82,146,31,204,100,72,22,58,231,188,184,144,20,38,251,70,170,96,63,23,34,210,232,111,100,70,200,242,200,143,18,52,84,213,96,
46,207,253,205,65,190,21,213,140,234,206,37,109,190,62,111,114,32,134,123,46,140,240,105,221,124,60,130,19,29,138,199,67,102,16,0,49,1,32,158,68,61,82,153,242,64,2,149,185,86,117,222,229,252,39,168,129,
250,243,215,83,14,188,128,129,24,52,43,14,67,118,3,117,166,174,204,194,153,30,20,116,106,225,50,128,65,208,174,240,98,179,221,156,211,43,161,157,207,36,231,119,134,81,11,158,219,248,169,57,45,189,25,220,
28,71,138,13,53,153,192,46,38,161,138,245,82,74,84,76,130,38,171,155,98,176,147,253,86,162,212,215,201,43,7,178,198,243,17,149,240,18,137,178,99,97,220,100,134,117,174,164,177,101,169,137,112,124,171,
164,16,66,158,186,152,168,151,235,131,57,12,78,170,200,230,223,188,211,107,51,163,86,24,162,192,167,204,163,114,68,178,196,199,185,48,213,51,136,216,216,132,81,48,151,21,192,19,81,214,82,122,216,83,86,
37,32,167,83,213,161,199,73,145,204,255,166,8,52,43,26,59,121,34,215,49,192,235,6,143,90,227,73,251,88,51,138,65,195,17,89,14,159,92,246,51,198,34,176,55,140,22,197,68,86,87,92,195,157,233,101,202,92,
179,70,10,167,92,159,53,77,104,118,13,107,166,129,15,95,31,99,245,250,193,220,71,183,168,89,83,5,95,187,113,61,123,72,121,38,15,220,49,197,75,158,198,150,163,211,109,72,112,190,211,96,193,183,54,12,96,
125,146,233,108,150,9,254,252,119,129,115,126,61,1,196,148,149,26,24,128,132,40,179,133,136,128,87,60,189,251,77,21,82,0,235,211,172,81,68,186,142,85,30,190,225,174,21,132,9,202,42,125,26,241,200,133,
75,253,141,37,44,63,223,132,74,113,49,219,222,203,60,101,171,124,95,51,111,170,144,12,105,83,133,200,254,55,82,133,247,22,37,104,88,40,222,224,168,89,245,253,238,102,37,200,102,55,233,75,17,198,119,187,
80,215,0,70,66,100,7,134,5,56,118,154,55,120,248,97,228,157,40,105,35,239,131,25,181,72,253,27,150,27,250,186,235,95,191,58,51,182,105,44,85,49,30,137,50,15,114,99,150,39,248,51,53,93,94,229,70,232,147,
205,55,36,134,132,30,25,206,42,225,133,161,38,1,80,28,101,179,144,216,25,58,44,98,213,121,173,243,65,180,67,13,127,103,17,120,125,108,180,65,31,16,8,151,156,233,32,179,96,72,119,148,85,188,152,102,109,
105,153,236,167,3,213,12,16,209,132,26,206,53,70,244,188,127,111,220,72,225,230,7,73,167,54,253,125,53,225,230,7,9,47,122,170,95,186,206,153,0,14,120,86,138,111,223,54,134,69,239,18,89,67,14,65,96,38,
252,125,21,33,93,207,72,133,198,10,144,50,211,253,66,129,151,89,19,205,85,218,250,4,72,83,153,125,54,166,179,33,167,80,83,182,115,41,147,170,217,16,42,92,205,4,172,155,214,65,170,174,81,138,76,15,165,
211,156,213,243,88,97,161,230,34,237,250,123,9,151,222,17,121,65,144,23,28,197,206,104,27,206,141,251,113,38,97,48,156,97,88,26,227,131,178,104,1,43,190,104,4,49,88,45,114,67,115,74,219,172,22,72,114,
141,203,188,197,113,36,154,143,86,145,42,12,34,1,172,143,193,147,66,95,191,9,16,56,26,93,207,161,221,106,219,247,2,55,156,97,238,37,151,15,102,203,104,162,11,70,15,90,249,62,119,226,139,167,83,92,92,1,
114,146,138,175,32,199,208,245,229,220,68,202,49,100,62,149,58,96,201,10,48,228,94,165,161,238,68,209,58,211,13,249,97,34,2,16,199,42,72,100,236,27,41,80,4,100,237,184,49,228,41,130,157,156,202,201,24,
188,30,3,4,69,105,54,128,157,226,34,63,158,161,6,158,115,214,176,71,170,77,148,106,86,71,104,180,83,99,64,20,141,223,176,161,253,155,68,17,126,116,151,40,253,253,162,167,86,123,31,95,244,84,198,47,30,
224,129,54,12,104,183,141,249,229,127,18,184,246,174,25,238,61,65,161,30,5,14,30,79,129,100,189,2,42,34,179,77,165,4,19,65,78,196,136,68,225,68,33,216,57,181,185,159,114,253,116,62,166,71,129,33,75,179,
73,195,48,54,170,24,85,135,42,78,34,172,147,192,6,93,96,105,70,54,100,53,77,149,112,107,0,126,204,12,58,247,171,119,219,138,177,219,86,9,110,126,144,240,163,187,132,213,170,221,183,103,235,164,10,36,104,
167,247,180,202,165,3,174,188,51,198,13,247,141,227,112,48,51,34,148,187,66,185,152,200,174,129,145,226,170,20,101,67,212,181,16,185,122,223,53,107,165,158,143,200,155,208,136,204,195,175,65,211,164,57,
47,195,76,221,242,133,162,18,32,153,180,103,71,230,250,124,58,5,79,68,37,165,201,45,152,232,181,7,67,85,99,68,168,234,46,205,57,43,236,128,36,223,234,244,170,238,206,156,219,99,54,55,49,211,52,99,63,203,
86,96,123,19,108,155,77,38,153,253,155,205,62,64,20,23,127,115,234,124,103,82,96,127,243,117,5,47,11,244,103,48,235,224,160,247,200,21,174,199,177,82,137,137,118,204,17,101,169,89,28,139,81,193,127,109,
155,32,60,17,3,235,200,177,23,250,73,202,121,75,81,161,64,114,15,90,74,153,158,34,168,38,73,99,213,196,121,232,104,236,246,102,207,69,141,27,41,152,209,161,156,242,40,81,35,58,224,153,227,215,96,65,68,
40,34,37,82,98,131,73,163,118,131,215,1,114,125,102,60,112,10,172,159,134,148,192,234,199,51,57,33,85,73,204,234,53,132,199,215,74,172,91,3,172,125,28,144,82,102,32,192,155,211,158,27,154,190,8,0,1,105,
154,125,213,12,205,34,9,96,29,3,233,90,0,235,114,119,118,90,124,157,3,142,100,154,1,153,117,108,148,52,24,87,204,10,52,159,186,91,138,231,253,187,108,168,198,99,97,92,123,34,67,5,102,201,10,84,198,113,
36,202,14,120,93,252,19,53,249,169,54,121,245,31,137,231,71,230,67,137,68,209,20,195,52,64,116,39,228,212,177,76,19,165,172,13,23,128,76,237,207,230,84,229,195,39,25,40,74,117,77,110,6,255,19,231,103,
216,118,79,7,127,12,176,104,116,70,43,208,36,27,175,169,202,171,75,80,110,119,222,8,133,116,81,93,18,23,109,175,105,50,243,12,79,100,153,231,152,48,188,34,17,138,199,204,31,81,240,13,60,56,69,191,223,
237,15,98,138,190,82,147,4,197,94,22,63,57,126,36,225,172,183,126,114,166,38,64,221,226,153,81,182,39,106,93,70,166,46,58,79,177,154,136,109,240,29,9,96,178,136,12,241,24,246,121,18,132,44,18,131,65,53,
90,80,245,139,137,2,70,169,44,206,5,75,21,25,42,206,126,171,50,83,34,180,237,238,251,49,219,228,176,31,134,245,14,212,185,40,200,79,109,154,187,41,101,5,152,153,128,149,110,198,70,250,153,30,119,154,255,
221,169,215,114,34,251,236,9,81,106,192,208,142,94,148,121,120,136,215,27,215,148,95,223,132,146,185,84,132,133,84,126,112,169,225,153,146,149,237,22,209,115,174,216,166,185,2,4,142,27,229,197,231,177,
110,204,229,26,58,22,14,73,164,177,216,228,103,199,8,160,9,42,34,149,38,130,205,113,76,105,2,21,50,47,160,167,85,119,177,103,169,99,145,82,227,123,178,128,147,249,30,21,141,40,13,29,85,52,171,168,197,
31,10,40,155,55,12,129,211,78,90,63,180,222,49,48,26,249,236,242,60,50,51,167,145,109,199,69,79,104,178,184,38,195,46,173,138,228,181,2,21,204,177,168,141,34,130,118,116,173,106,13,157,79,54,155,53,8,
15,70,215,221,44,58,105,168,144,243,168,137,188,106,96,141,211,164,186,44,161,179,201,70,67,209,22,13,22,49,157,0,113,148,97,115,53,244,47,151,63,20,141,121,179,150,117,82,233,237,1,131,114,215,73,163,
58,70,10,209,226,125,152,126,159,104,188,247,117,0,150,70,61,169,219,205,8,142,254,143,20,207,217,180,119,134,101,85,131,133,126,52,85,152,105,202,92,39,77,21,128,98,206,223,127,252,63,198,193,219,229,
94,110,114,140,187,12,80,60,248,136,192,231,126,153,135,64,132,6,25,135,237,40,177,227,191,53,14,119,159,255,187,24,75,151,87,75,147,39,111,192,120,223,46,107,49,103,98,194,1,48,170,165,252,4,129,8,56,
233,218,24,107,42,228,229,23,246,75,176,62,5,176,62,239,94,35,148,84,202,90,66,252,125,141,192,199,126,170,82,4,97,252,107,216,8,47,219,66,98,159,103,75,171,43,116,142,209,242,213,203,27,59,52,218,179,
105,0,211,235,160,230,45,100,233,0,110,202,220,40,52,85,120,211,179,255,142,39,205,42,175,193,146,135,55,192,245,15,62,1,179,35,198,155,158,189,18,63,184,123,35,172,88,171,12,64,34,188,253,57,255,196,
109,15,111,128,155,255,182,65,225,109,207,51,128,98,131,79,18,137,18,44,22,208,221,127,118,223,114,10,207,217,228,241,226,101,211,25,223,62,50,21,225,235,119,60,17,165,220,163,168,193,220,162,105,52,0,
243,133,220,153,29,1,111,122,246,67,248,193,221,79,198,138,53,177,199,4,54,66,84,73,57,234,113,236,115,255,137,139,31,220,8,139,55,89,7,1,224,170,251,230,181,116,230,247,222,114,10,27,205,74,241,237,63,
61,209,122,124,199,5,107,241,226,77,215,224,139,183,63,121,70,242,111,246,4,227,240,109,31,193,194,185,217,26,44,95,29,227,162,123,158,128,127,173,141,240,154,109,30,193,230,115,167,1,76,99,109,66,184,
242,190,39,226,193,85,115,180,53,255,154,109,254,5,64,226,146,187,55,2,0,60,117,195,117,120,237,162,71,180,37,255,192,99,177,122,78,20,235,195,147,6,96,236,135,185,48,173,121,0,177,223,64,233,228,42,220,
142,147,141,217,39,169,120,220,209,107,170,86,161,18,83,218,112,181,9,185,3,159,20,16,18,81,225,73,224,4,118,58,97,223,16,80,217,248,100,30,189,90,26,213,149,136,56,75,17,35,70,169,147,207,184,27,205,
4,128,99,2,166,7,61,179,47,117,156,59,34,179,21,38,68,211,148,70,29,44,103,20,195,168,73,128,41,171,12,8,157,233,198,24,16,117,147,158,177,128,123,10,134,128,172,193,194,158,139,210,82,17,125,63,168,149,
65,183,189,160,103,46,96,236,184,153,196,14,155,114,118,232,83,23,12,21,74,247,103,119,155,207,21,109,95,255,231,175,192,142,255,214,248,123,142,120,126,130,155,30,136,176,244,33,129,187,255,81,136,132,
13,98,224,133,79,149,216,117,81,130,57,19,177,241,185,70,216,90,166,224,117,19,248,213,3,2,83,143,251,21,135,68,214,77,14,235,36,50,111,181,25,233,146,142,176,51,215,185,200,231,126,217,22,140,93,23,25,
58,44,246,15,171,125,124,186,181,61,147,235,149,80,28,97,49,248,245,59,54,182,12,224,79,223,178,192,122,254,249,11,30,199,242,53,49,94,242,111,83,184,228,129,217,25,184,69,117,73,187,206,152,35,159,253,
200,10,12,101,13,13,24,192,181,247,205,195,181,10,80,124,232,5,15,227,130,123,55,194,131,143,78,96,193,236,105,188,254,233,143,97,225,220,199,1,16,254,242,207,89,184,234,254,39,99,246,44,224,144,103,60,
130,255,55,55,193,250,132,240,200,186,8,119,254,107,54,30,92,21,227,181,207,126,4,231,46,157,131,71,215,71,120,254,130,213,120,234,19,214,107,3,223,190,167,85,234,158,30,179,158,63,246,185,43,112,221,
3,27,98,233,63,230,224,169,27,174,213,235,177,96,78,130,189,183,124,4,79,154,149,98,249,234,13,212,125,49,214,166,192,44,37,106,118,125,234,106,236,180,233,154,12,204,173,139,112,245,253,27,226,190,71,
39,218,222,143,147,95,240,48,30,89,23,105,144,250,179,255,155,135,155,254,182,1,54,157,151,96,239,205,87,105,176,243,235,101,115,240,211,7,231,90,239,125,177,250,254,211,254,103,19,48,128,35,159,253,47,
188,112,179,199,113,235,202,217,88,56,107,26,231,254,97,35,60,186,30,216,123,139,191,99,241,198,171,240,224,42,129,5,115,128,3,183,89,137,71,214,69,88,155,22,134,247,226,141,31,199,111,151,205,193,47,
31,122,2,22,204,153,198,97,207,248,7,22,204,73,20,128,20,246,241,179,204,9,129,222,181,144,206,11,118,186,119,34,93,105,89,137,133,244,127,34,251,205,186,193,33,89,152,192,20,81,86,228,73,167,46,38,104,
30,137,55,129,80,100,175,1,69,64,196,69,139,97,125,23,189,92,255,170,213,27,143,78,2,164,230,22,147,199,114,11,83,235,114,246,35,32,154,80,118,72,93,234,227,100,118,62,211,198,172,170,235,44,173,78,117,
170,25,67,154,57,241,56,42,142,101,0,70,1,16,117,76,175,95,220,31,143,193,110,91,49,110,188,159,75,117,72,205,104,209,198,205,163,52,62,186,249,65,234,75,135,187,39,207,105,124,125,235,1,36,235,245,0,
29,83,149,3,200,26,2,84,69,120,238,254,7,225,250,187,35,236,182,77,227,61,122,241,230,41,94,188,121,218,130,103,70,148,60,68,15,62,50,141,107,254,20,3,114,34,243,192,76,20,2,69,66,1,161,117,46,16,178,
105,227,57,18,159,217,103,61,236,244,30,24,128,101,34,11,141,171,148,174,216,115,192,214,76,3,127,90,41,90,63,157,235,129,113,237,50,183,96,78,130,197,155,172,193,215,239,216,4,175,218,114,53,22,111,188,
6,75,255,53,87,239,25,251,12,2,55,224,18,9,32,49,234,191,162,168,164,148,200,88,110,161,34,158,123,111,187,10,127,253,71,140,239,254,121,51,44,152,147,224,192,109,30,193,243,255,109,13,102,111,144,125,
244,167,110,221,24,179,34,137,255,122,214,191,178,221,177,242,238,154,221,211,227,234,158,30,193,226,167,172,193,210,127,204,105,44,83,158,250,40,254,250,216,36,190,254,224,19,176,248,41,107,50,144,150,
48,48,205,72,34,194,142,11,214,98,199,77,30,199,185,119,110,132,101,83,49,246,222,114,10,175,217,250,81,124,242,150,141,59,90,247,251,31,155,196,85,247,205,195,139,255,237,113,188,252,223,167,112,211,
223,54,192,222,155,175,194,178,53,19,248,218,29,27,97,163,217,41,14,123,250,163,88,182,122,2,127,252,199,164,126,223,79,31,156,139,159,98,174,229,100,184,247,209,73,44,152,76,176,54,21,120,116,125,182,
62,15,62,54,137,231,43,240,180,98,13,240,229,37,243,53,128,212,235,52,55,193,131,143,77,2,96,172,88,19,227,145,117,17,22,108,176,94,1,34,246,120,98,209,39,147,144,243,255,119,205,32,105,233,115,34,3,244,
228,127,155,75,49,89,216,80,141,12,174,34,13,183,85,48,196,229,127,117,141,14,123,150,156,251,104,158,123,154,112,168,107,202,3,86,52,106,85,210,209,12,249,104,132,137,21,255,115,62,178,194,10,161,37,
125,148,17,190,157,73,1,25,235,46,162,13,143,93,194,198,217,202,199,54,68,26,24,113,76,99,51,136,55,0,162,30,80,179,70,10,0,240,197,223,68,248,211,138,230,34,229,25,11,24,239,124,97,99,195,253,176,231,
164,3,157,41,52,8,154,4,144,8,115,136,78,33,120,154,165,187,1,192,53,127,142,240,248,122,194,62,207,154,169,241,95,206,101,191,245,111,17,46,89,26,225,241,4,0,214,1,152,0,11,81,204,252,89,159,183,117,
75,59,80,43,214,224,32,213,145,78,84,14,38,250,250,45,81,195,153,85,150,108,20,128,212,71,116,252,170,101,95,242,111,171,48,59,78,241,142,231,172,0,226,73,60,105,35,224,174,199,230,96,93,74,94,91,44,215,
131,12,224,105,79,156,198,193,79,127,4,169,204,34,38,95,191,245,201,13,13,138,188,34,34,223,201,5,115,82,92,255,224,134,192,36,97,69,50,129,229,60,137,39,206,99,60,121,78,138,63,255,107,54,30,95,71,120,
28,17,238,92,54,137,116,157,4,166,101,27,247,196,120,199,115,86,2,0,158,52,43,197,93,143,108,128,181,105,53,223,45,156,155,224,151,127,203,162,88,75,255,49,7,187,111,254,184,98,117,70,154,16,38,99,198,
242,53,19,88,54,149,241,202,31,255,49,11,59,108,242,56,54,154,157,226,95,107,219,119,150,60,180,58,251,156,252,243,242,107,88,56,55,193,14,155,60,174,31,219,104,182,159,39,55,157,151,96,175,205,87,225,
250,255,157,135,251,31,157,192,226,141,214,118,217,16,201,63,107,26,229,34,171,94,155,133,170,131,84,162,230,73,245,208,132,178,226,236,102,131,5,197,168,100,220,46,181,184,98,213,131,140,76,139,44,143,
186,56,105,156,38,16,50,63,75,0,144,249,235,7,225,188,201,59,79,198,42,106,53,186,17,163,16,9,106,14,8,57,70,22,193,76,100,209,137,52,201,87,110,16,157,232,84,139,132,52,5,38,162,22,204,9,50,26,248,196,
217,121,19,202,33,162,204,27,142,3,8,30,59,64,212,173,52,176,107,239,234,66,139,105,69,127,90,209,254,53,13,42,157,173,29,234,198,245,101,33,223,24,72,214,117,36,182,111,184,79,224,134,251,38,251,32,156,
82,85,252,110,206,183,73,241,190,43,39,58,252,76,51,129,70,153,214,179,187,148,86,51,73,229,128,148,162,115,254,103,102,123,54,147,247,207,244,187,155,209,226,167,172,193,236,72,170,20,58,2,68,140,189,
159,190,26,139,55,89,139,223,45,223,0,49,217,153,113,174,82,252,235,163,19,248,196,255,108,82,60,17,185,150,161,122,189,51,32,47,79,113,92,189,46,194,51,159,178,14,127,125,116,2,11,230,37,216,108,195,
105,252,223,234,73,76,175,38,108,187,209,90,44,89,54,27,179,56,197,182,27,174,197,111,87,207,197,163,107,34,0,211,120,226,172,4,235,82,129,69,27,173,181,82,192,202,247,148,209,222,91,60,130,237,54,94,
131,223,173,152,139,71,214,69,152,21,103,23,184,104,163,98,2,251,242,213,49,22,109,180,14,15,174,154,133,69,27,173,133,51,84,6,171,19,194,194,57,211,216,116,94,130,101,83,49,158,249,148,117,88,155,80,
71,96,168,138,150,175,142,177,108,205,68,195,122,165,141,102,167,216,123,139,41,60,158,16,190,251,151,39,97,205,52,1,9,176,226,81,129,39,109,150,224,137,147,9,30,93,31,97,254,92,198,138,213,141,249,103,
197,234,24,243,231,38,192,63,128,39,78,166,120,210,172,20,43,30,247,201,134,164,202,244,239,161,252,152,6,146,89,26,164,80,151,63,221,251,183,153,30,71,29,202,22,13,102,184,129,28,51,35,67,92,204,87,1,
156,8,40,103,209,86,29,56,82,251,192,81,15,29,56,2,213,179,147,216,6,67,35,140,26,2,40,106,97,125,242,249,112,250,65,161,186,182,14,170,190,72,165,240,203,72,215,2,121,175,91,39,217,20,245,207,136,168,
92,133,160,74,147,198,173,205,250,88,3,162,110,52,35,104,165,145,66,171,209,161,156,254,115,11,137,67,23,55,246,8,255,115,13,240,241,95,100,13,22,218,109,120,208,14,117,43,101,174,147,116,61,147,214,76,
3,215,222,35,176,126,26,248,199,191,24,75,151,207,252,222,94,182,101,177,198,173,68,142,30,124,68,96,201,223,50,33,184,100,25,53,40,232,79,189,74,59,75,135,235,148,12,107,37,38,220,246,119,194,138,71,
9,17,128,63,61,76,29,207,168,34,1,176,21,187,40,60,92,163,208,84,193,71,179,35,198,243,55,93,131,171,238,123,82,177,182,204,248,229,255,206,197,97,207,250,23,238,126,100,18,49,128,221,254,125,10,187,253,
251,148,174,226,250,241,3,27,226,214,21,179,173,34,245,18,82,114,20,72,21,160,250,193,61,79,192,238,155,175,194,251,95,240,48,0,224,222,127,206,194,111,255,182,1,102,79,48,158,250,132,245,120,199,115,
254,142,71,30,35,172,85,243,178,30,93,63,137,7,30,155,80,141,0,128,7,30,155,108,114,79,25,253,242,111,27,226,176,103,252,3,119,253,107,54,150,62,188,1,118,254,247,41,236,252,239,83,214,251,175,127,240,
137,216,123,203,71,112,236,115,87,216,159,171,108,214,37,15,205,198,147,103,165,56,234,89,255,2,144,69,196,46,185,231,137,222,181,221,252,9,235,113,178,186,39,0,184,237,225,13,116,68,168,17,93,245,192,
134,216,123,243,85,250,189,203,87,199,184,234,129,13,173,40,210,174,255,111,53,54,87,105,111,91,60,121,29,146,117,192,146,135,38,113,253,131,243,240,192,99,147,56,106,187,149,218,176,189,248,174,141,26,
126,223,210,191,111,128,215,46,122,4,219,171,136,212,3,143,77,26,13,40,6,73,92,0,177,245,49,120,162,176,193,187,93,233,199,30,60,64,21,124,219,178,165,168,163,63,21,173,23,221,111,101,99,222,151,57,16,
201,197,85,81,62,72,73,125,110,202,134,33,216,13,32,4,123,206,139,5,134,5,236,252,193,12,199,117,59,181,177,110,70,127,160,230,238,73,54,88,155,35,1,172,37,71,247,247,11,32,169,171,74,179,104,111,195,
202,96,66,150,30,194,92,52,225,141,96,183,164,84,101,73,20,24,161,187,187,20,218,110,7,234,166,2,231,105,169,250,240,143,99,221,139,178,140,226,9,221,133,170,27,242,74,230,93,201,83,119,202,81,59,170,
147,135,127,93,137,128,56,46,53,197,240,21,165,87,249,193,91,221,19,247,253,165,239,75,129,39,210,52,14,123,198,74,252,118,249,92,252,110,197,156,46,26,128,173,26,137,138,201,162,172,133,161,168,147,28,
96,168,25,94,166,225,202,182,129,219,215,245,234,21,205,42,152,99,22,85,242,22,181,187,126,112,68,104,92,13,222,219,250,204,245,0,100,226,236,139,233,208,113,79,85,12,76,10,237,156,241,158,5,7,63,89,153,
115,156,58,149,226,237,158,251,188,129,131,186,190,36,41,34,83,249,125,152,13,30,0,128,35,157,105,148,7,185,130,221,24,168,144,75,38,239,199,104,173,193,72,23,228,181,136,74,53,172,84,165,183,18,216,53,
122,17,217,160,8,237,143,243,168,235,158,212,233,30,66,105,86,160,238,50,119,42,48,184,137,209,131,38,213,129,46,207,30,81,5,158,98,134,150,106,86,75,4,32,205,35,69,211,109,30,239,188,13,249,144,27,159,
81,22,41,226,148,10,103,113,155,176,176,85,48,84,178,246,212,27,183,95,176,22,187,62,109,85,182,5,235,215,226,47,255,156,192,239,86,204,30,192,218,170,110,136,60,89,99,97,48,138,224,220,165,117,234,118,
34,32,153,200,162,69,212,57,112,41,173,97,42,173,212,159,78,86,208,122,109,196,170,222,199,125,38,130,63,253,144,53,38,105,165,205,56,229,243,40,21,91,114,18,1,73,39,250,32,82,133,228,137,110,84,147,93,
142,250,37,6,48,109,132,129,80,128,33,223,2,105,160,38,2,56,26,87,210,229,57,179,1,36,74,79,199,234,119,239,252,67,238,238,183,19,21,205,111,217,102,87,255,193,162,226,181,228,156,175,188,35,55,13,111,
20,180,110,96,40,0,162,64,93,23,56,60,246,67,68,145,9,214,164,240,253,200,89,98,198,67,52,73,228,233,207,237,116,113,226,172,247,183,52,251,125,14,35,40,98,195,170,201,56,141,91,49,56,217,80,34,104,223,
163,110,106,154,92,249,44,89,49,27,183,47,159,173,162,31,73,13,120,93,205,42,138,163,26,42,70,238,145,113,81,83,103,72,34,50,176,225,104,85,110,211,8,39,56,173,182,185,122,117,169,221,237,16,4,204,202,
89,39,50,26,115,170,84,58,9,88,222,115,226,142,190,79,223,67,4,32,153,133,214,135,230,42,57,37,34,213,30,127,210,206,134,139,140,134,14,177,209,246,59,49,68,28,88,117,153,83,67,75,19,88,221,194,57,12,
185,28,107,18,10,87,115,62,165,99,66,157,5,107,186,119,55,129,145,170,77,22,162,168,255,49,208,128,57,95,72,107,119,51,171,213,0,242,230,188,113,198,144,167,132,170,181,168,211,121,12,128,40,80,15,96,
81,32,139,214,49,120,54,205,104,101,8,121,247,156,56,43,184,228,102,17,31,145,89,8,81,150,102,86,12,164,207,115,166,115,175,88,110,216,215,181,84,215,80,19,185,244,143,10,128,226,51,212,204,49,14,172,
60,114,237,26,164,4,64,122,34,123,108,53,210,138,107,0,138,88,227,34,196,131,87,144,58,163,131,221,58,143,49,113,132,72,101,92,69,54,174,104,71,233,91,17,74,138,42,59,35,182,226,20,176,188,176,198,139,
88,34,3,28,17,178,182,244,121,74,26,43,231,73,14,198,84,122,154,121,222,218,58,71,4,240,108,2,203,9,96,125,190,40,178,26,12,229,64,72,221,186,11,16,73,121,150,24,0,167,100,127,86,10,64,24,131,148,173,
161,204,249,80,221,8,129,2,89,195,188,243,7,146,168,224,253,188,22,174,43,242,157,50,158,46,161,30,148,34,150,228,168,112,119,188,125,213,121,31,70,96,196,249,132,19,201,160,137,122,92,125,0,68,129,186,
108,186,18,198,187,15,78,82,120,132,52,208,136,193,73,6,104,102,146,61,71,57,24,72,26,69,123,168,48,46,98,81,148,223,76,2,44,149,71,152,34,93,147,192,50,6,210,40,107,9,90,235,232,30,3,114,58,139,120,101,
195,187,91,82,0,51,153,67,226,114,49,75,229,137,150,169,109,196,13,116,221,20,66,75,0,142,162,74,144,216,55,32,148,26,0,109,236,100,128,234,64,39,205,180,52,168,32,15,53,4,47,236,130,238,148,157,89,168,
228,173,139,51,193,127,51,247,148,107,108,105,252,35,140,252,211,40,50,47,186,152,125,52,3,134,34,117,14,121,182,0,175,83,33,154,146,161,89,128,33,19,8,89,178,207,109,140,18,1,156,176,3,72,85,164,139,
124,115,103,146,66,118,26,253,35,200,236,21,145,22,232,139,34,106,219,153,18,104,8,65,81,254,135,53,182,72,24,217,24,157,126,114,206,104,148,157,55,225,156,163,6,221,34,219,109,158,50,140,60,74,148,167,
178,82,71,14,151,0,136,2,213,222,28,192,216,3,34,147,12,192,146,100,147,96,57,234,92,144,233,182,156,121,96,2,19,40,213,6,209,172,98,168,78,234,8,87,81,6,8,36,0,8,130,68,158,62,83,55,80,228,100,26,231,
163,36,102,13,240,82,74,217,207,92,143,117,50,2,105,253,4,69,69,33,48,143,235,12,225,106,120,19,65,183,205,101,216,57,255,13,103,249,70,198,206,37,50,91,91,243,163,99,251,32,115,7,134,123,145,210,86,92,
171,6,212,146,74,114,98,166,188,68,0,48,43,202,82,216,18,131,75,69,12,68,164,129,80,171,114,145,0,48,71,64,148,42,217,101,12,236,181,4,173,52,228,71,2,36,113,6,164,162,236,58,138,84,100,123,255,152,163,
12,20,137,192,209,163,14,138,172,232,169,254,59,239,129,45,219,144,243,230,176,48,5,206,163,50,8,234,6,144,105,71,251,112,179,115,132,38,157,239,122,33,37,141,99,169,59,233,213,128,31,130,229,26,40,80,
160,64,129,2,5,10,20,40,80,160,177,164,224,251,8,20,40,80,160,64,129,2,5,10,20,40,80,0,68,129,2,5,10,20,40,80,160,64,129,2,5,10,52,110,20,111,188,201,83,71,238,166,116,118,50,219,221,117,66,113,100,247,
215,153,205,228,83,174,26,135,25,168,68,198,84,53,61,64,190,91,243,75,186,189,183,86,85,138,239,234,250,177,231,164,23,75,80,31,239,63,231,107,118,239,51,247,37,213,173,141,185,145,172,46,122,227,241,
202,214,198,183,38,249,247,183,211,26,190,215,107,49,140,242,168,89,207,184,70,36,156,215,247,239,204,12,92,86,193,148,87,220,125,62,34,49,210,235,56,206,118,76,115,214,225,246,206,31,53,110,18,48,144,
134,55,214,45,112,11,50,69,52,238,130,215,233,117,152,127,24,243,149,234,112,180,226,25,181,97,170,49,16,202,138,53,201,59,27,61,80,151,14,153,84,6,97,110,209,83,0,67,237,25,247,69,11,89,107,240,90,77,
174,144,225,67,106,220,0,16,245,10,16,231,235,37,172,238,219,189,150,35,250,47,162,140,183,153,26,0,128,26,2,162,30,48,83,113,167,210,143,226,189,179,215,187,13,14,154,125,166,240,188,127,152,230,111,
85,113,120,163,117,165,234,53,34,2,19,141,137,67,176,23,50,72,245,71,198,56,173,227,120,105,99,110,38,206,153,219,144,33,164,206,220,224,141,124,246,233,179,150,229,114,214,20,130,85,11,184,153,168,19,
219,137,230,32,160,26,29,168,120,228,142,183,227,213,37,181,139,65,136,117,121,153,25,160,82,247,149,176,202,109,25,248,21,61,115,235,34,39,4,25,81,146,252,58,217,105,101,102,69,6,169,71,134,103,1,32,
251,217,5,39,187,93,114,84,139,171,49,169,214,60,214,237,171,211,219,206,212,6,128,233,10,55,26,192,70,180,1,12,234,190,79,157,0,194,38,64,216,178,236,114,62,24,93,67,94,75,29,43,67,161,71,156,175,166,
104,142,210,90,242,136,156,148,110,156,50,233,5,66,249,43,162,6,171,102,186,222,169,233,233,236,199,94,22,178,218,124,192,188,78,217,228,138,76,3,128,204,128,115,251,215,196,74,27,25,233,48,102,199,205,
250,12,102,37,26,77,206,86,145,47,6,133,232,80,207,125,43,1,8,117,182,110,6,195,82,253,242,58,179,137,217,100,205,235,0,81,49,120,147,205,107,239,245,220,108,26,72,248,76,99,192,198,240,160,158,128,91,
237,85,85,92,161,149,171,230,202,91,110,37,130,209,139,117,17,205,207,210,80,236,81,175,128,146,105,94,24,7,151,198,65,70,247,122,175,107,22,198,239,50,16,10,100,140,182,224,18,220,118,100,143,244,203,
165,22,245,120,47,56,181,20,13,42,144,136,113,115,194,185,159,54,184,64,233,19,238,160,19,189,158,245,101,4,41,76,53,81,23,23,99,60,154,98,210,52,214,202,76,83,39,68,58,220,234,56,136,212,206,69,23,121,
13,199,186,69,145,35,248,143,146,36,7,12,51,121,12,212,238,47,91,63,114,141,155,195,187,97,224,123,9,72,149,98,72,212,146,82,174,132,19,102,246,85,73,117,85,169,118,238,225,153,17,142,65,66,21,215,196,
99,46,91,138,127,71,93,231,81,159,191,139,70,112,205,66,158,135,33,251,45,223,142,71,1,186,233,211,68,141,197,31,53,78,38,166,46,156,122,192,136,6,233,204,13,223,197,184,50,130,91,131,205,92,220,103,39,
215,43,60,75,41,169,60,79,105,160,128,104,148,60,71,69,232,156,44,206,200,107,33,52,67,122,60,167,129,90,63,116,186,70,139,4,134,43,55,191,174,42,137,44,207,18,213,244,42,75,138,195,76,161,211,174,181,
252,20,114,247,175,64,229,101,247,83,104,102,183,199,40,70,105,203,33,146,28,92,214,52,121,205,40,75,67,183,139,106,181,88,89,206,226,230,83,26,70,1,171,129,132,93,151,13,212,4,190,81,144,39,174,239,86,
121,16,70,126,101,56,175,97,165,30,175,239,232,70,219,130,61,100,75,25,13,140,124,233,193,68,205,209,37,3,102,52,134,27,188,165,19,206,245,86,203,89,83,185,205,186,39,106,240,77,109,124,179,186,111,73,
173,59,6,90,137,219,155,142,183,110,215,94,181,179,182,35,87,67,164,187,202,73,54,226,116,182,209,73,236,231,233,64,173,234,29,179,150,34,44,226,140,13,24,7,12,13,229,138,146,1,138,88,32,27,29,207,221,
58,209,246,249,29,136,2,175,97,5,104,219,135,151,12,71,6,217,154,200,108,174,195,45,170,202,28,20,145,147,166,165,193,82,163,90,159,153,128,161,102,142,152,113,244,117,27,235,67,118,107,168,81,143,14,
21,90,136,250,243,109,50,107,219,23,52,223,56,240,84,19,228,210,208,4,50,100,32,23,10,222,231,124,18,29,92,95,137,227,173,86,114,85,50,144,102,38,99,140,40,81,158,136,37,218,251,132,50,160,99,243,113,
238,90,189,163,25,18,105,245,19,227,81,57,213,86,164,80,163,29,163,243,153,242,130,154,134,103,160,246,193,80,177,166,33,93,110,236,193,144,19,243,39,67,102,66,70,221,3,69,36,116,67,133,193,105,69,140,
16,207,203,178,63,131,165,225,33,178,138,198,42,84,154,253,154,34,56,100,132,240,152,171,223,215,145,10,109,100,133,184,45,209,199,45,114,205,133,211,128,186,215,42,183,246,156,156,247,220,38,217,199,
117,150,170,132,50,10,109,184,71,252,68,149,140,120,242,100,206,121,197,144,153,81,96,88,166,68,45,247,41,109,20,89,225,166,58,170,219,13,142,156,171,204,129,209,12,186,190,178,121,253,133,225,62,227,
8,81,214,79,141,173,235,52,48,92,195,207,30,137,8,145,123,147,196,108,108,157,209,33,67,20,13,22,66,108,163,131,35,81,50,114,16,86,113,166,128,104,72,193,16,33,115,148,154,87,109,149,156,10,0,50,6,144,
204,240,155,34,173,121,136,26,7,254,123,167,21,229,8,240,57,53,208,226,174,150,37,91,118,154,231,31,38,240,177,77,6,213,115,195,72,127,160,230,138,181,37,50,59,34,81,197,71,118,210,158,123,4,101,10,19,
76,68,52,170,209,33,141,131,88,14,78,15,177,4,40,10,154,112,68,181,115,43,113,230,134,105,107,94,4,239,87,246,62,169,216,180,33,142,139,142,116,84,94,118,81,67,230,87,102,54,96,16,165,244,212,78,210,253,
116,29,186,19,61,155,233,172,35,42,140,147,210,231,55,139,102,197,163,212,49,165,88,2,66,230,157,46,207,163,48,231,8,4,80,52,19,187,48,172,220,204,5,13,233,193,103,195,184,154,121,145,100,149,119,139,
5,171,244,57,238,124,141,204,40,5,13,34,25,74,98,116,210,67,219,76,251,115,35,68,78,185,16,115,249,57,50,1,19,85,53,179,109,23,20,229,45,111,169,58,208,68,21,64,218,218,199,102,214,138,168,120,207,144,
237,49,141,118,10,68,54,3,79,153,162,37,47,124,127,215,154,141,20,34,166,208,176,105,84,249,173,89,79,71,27,24,113,147,119,218,14,139,70,249,54,220,142,95,135,122,209,118,158,156,31,191,140,233,72,59,
234,40,19,119,109,164,134,86,15,218,236,200,86,88,53,250,110,250,5,67,223,101,206,197,174,5,131,249,122,87,144,149,133,30,64,81,123,96,136,186,86,23,18,192,80,35,51,108,24,239,196,215,177,140,137,192,
90,50,117,146,102,153,213,186,16,53,91,193,222,220,151,68,59,195,248,134,230,36,195,202,109,111,21,23,121,100,110,14,80,185,172,46,193,66,129,34,174,2,66,173,128,162,60,173,20,229,232,85,129,188,252,197,
160,44,155,112,140,39,139,223,108,79,45,205,53,146,195,185,213,35,104,160,179,218,27,170,133,30,82,242,129,133,215,208,13,52,252,90,186,213,97,18,26,150,151,154,11,249,228,15,107,244,236,105,146,111,248,
224,184,217,183,249,92,5,61,210,240,100,243,56,249,97,83,59,90,168,200,208,38,173,237,171,251,239,182,249,217,26,32,146,6,70,173,248,137,134,62,66,228,245,221,18,64,136,10,47,18,144,213,33,168,84,58,179,
15,122,16,92,141,73,230,198,1,133,166,156,221,231,220,153,229,224,214,23,226,25,143,69,164,141,152,44,106,219,254,39,179,175,108,101,160,119,54,42,166,165,104,1,9,149,61,129,112,64,80,229,222,83,142,75,
170,154,43,52,91,219,168,184,14,242,143,248,48,77,99,251,241,200,176,17,100,115,78,117,39,177,71,202,251,207,102,166,1,15,201,41,52,83,91,70,222,159,132,182,231,169,244,234,44,57,145,162,64,195,47,33,
155,197,122,188,164,197,69,228,247,5,21,34,167,28,33,114,7,161,235,244,55,118,228,16,247,81,95,25,243,183,76,230,238,32,66,228,117,135,145,253,92,163,120,111,43,223,195,204,6,48,40,154,61,181,98,55,196,
163,112,116,169,145,66,87,40,155,40,235,53,17,66,66,109,162,109,9,195,144,13,11,215,61,18,99,177,158,218,87,38,72,229,245,166,45,188,58,103,60,245,247,192,50,98,200,105,157,58,130,187,99,23,6,121,0,139,
49,113,204,144,157,173,182,91,205,74,141,34,103,79,219,187,190,70,179,14,27,2,50,245,253,76,145,2,228,126,52,77,158,206,79,69,182,133,123,237,117,7,69,102,103,43,26,57,227,220,110,247,207,24,156,112,240,
172,59,83,104,93,59,130,96,168,29,141,94,180,234,246,8,39,170,6,8,246,151,250,210,123,61,136,170,239,118,138,17,29,242,28,59,137,22,102,218,25,75,227,102,118,53,210,180,220,174,148,96,163,201,76,27,66,
112,100,6,179,178,129,12,169,148,247,78,69,115,180,162,217,71,160,22,214,148,194,156,161,222,9,26,42,68,206,168,83,4,53,204,149,163,38,134,140,225,221,182,68,164,40,28,177,125,82,7,153,192,30,7,65,225,
233,12,231,180,230,166,22,64,8,60,10,78,144,202,224,110,27,88,82,75,223,215,234,167,176,40,55,102,246,166,121,122,65,241,176,201,64,227,122,37,129,137,70,170,27,90,68,128,4,53,72,41,26,148,182,84,233,
115,53,207,64,25,132,89,61,148,192,123,38,235,228,141,102,59,50,201,124,185,217,131,186,99,72,214,171,85,48,160,139,7,236,249,228,38,53,113,215,248,128,147,27,171,16,77,53,67,149,195,196,168,111,85,41,
213,196,173,237,225,72,52,85,200,88,39,91,106,115,52,70,94,248,91,4,138,138,196,234,16,214,110,65,104,202,220,48,10,117,67,221,55,181,97,117,61,28,31,45,147,31,80,225,23,111,150,82,48,192,80,62,131,168,
143,231,182,104,94,54,226,83,92,204,189,48,231,17,121,180,121,163,245,231,166,143,181,178,142,145,5,70,186,165,158,168,211,199,72,165,223,201,156,31,229,112,237,173,222,180,17,155,55,72,0,203,58,166,50,
102,145,34,31,0,175,35,40,10,54,80,247,117,70,179,126,151,236,245,95,176,49,68,179,110,187,35,51,185,108,230,72,187,51,185,155,220,35,55,89,31,253,25,70,237,99,187,154,195,250,94,45,26,84,83,133,22,207,
227,232,12,102,101,103,254,130,212,227,75,188,67,172,137,41,68,137,154,48,22,181,92,82,24,168,109,64,68,99,6,134,80,116,165,171,186,247,194,73,230,248,143,168,181,150,153,61,57,3,121,200,189,163,250,167,
97,81,225,64,211,225,108,212,130,194,55,236,111,173,0,217,148,31,174,115,197,223,146,131,132,24,184,196,113,191,95,230,157,34,134,206,57,84,88,49,35,39,197,201,72,141,169,27,40,170,249,240,214,17,24,51,
221,119,25,208,19,75,72,179,47,57,14,23,170,231,138,120,162,67,173,172,77,43,115,209,204,153,175,237,102,5,228,109,248,45,85,150,63,160,228,54,81,115,80,52,244,131,89,139,192,98,97,194,43,77,108,204,167,
101,195,154,106,189,240,145,49,58,13,119,59,151,8,33,58,212,245,69,165,241,12,79,54,19,114,214,96,87,51,25,123,64,181,225,133,158,170,163,209,213,69,9,106,88,71,2,213,201,26,85,123,32,173,138,224,138,
198,114,86,187,126,81,33,197,179,8,85,29,29,85,69,230,220,112,242,129,21,148,29,57,55,75,221,234,187,242,148,224,168,118,114,62,104,243,238,1,164,246,236,211,102,6,102,221,135,221,151,219,86,146,71,204,
195,87,134,202,173,33,28,247,165,237,54,106,208,130,174,64,68,214,171,88,61,199,13,50,115,70,162,169,130,14,57,186,75,203,70,202,11,83,169,89,80,43,11,46,217,11,140,187,34,140,168,206,203,169,215,52,164,
204,117,151,162,177,140,14,181,171,116,216,170,138,31,244,97,25,113,244,106,88,203,92,18,4,213,104,86,123,229,28,64,84,104,70,81,230,125,175,68,36,13,134,106,221,247,70,55,89,144,24,198,72,209,176,59,
246,120,216,206,39,219,115,15,3,13,63,223,81,183,120,214,29,170,90,251,36,70,1,159,112,46,53,67,40,233,2,165,7,12,159,151,172,200,246,104,215,237,232,133,144,68,246,247,150,94,33,1,25,161,170,168,50,166,
33,206,27,203,245,46,145,47,99,81,20,94,26,97,175,149,175,135,58,42,85,53,23,214,153,232,236,96,88,245,114,84,190,202,186,129,33,210,255,25,19,131,176,223,92,75,34,172,104,51,187,19,118,183,158,65,157,
19,210,113,252,81,142,19,23,221,220,124,237,80,141,151,148,229,166,94,22,31,72,112,27,100,56,74,203,115,25,162,206,60,73,84,164,101,112,165,74,174,161,33,147,1,77,49,2,217,32,230,47,228,118,74,170,105,
154,17,215,148,175,131,14,234,221,154,149,138,13,170,10,108,200,55,201,173,174,167,207,159,221,82,4,184,184,162,37,126,121,229,204,33,198,190,230,54,205,198,45,251,215,54,117,116,140,47,4,209,56,11,33,
30,106,1,41,77,214,227,138,5,176,171,180,220,172,117,230,6,19,166,153,13,23,104,214,254,151,141,166,12,173,8,57,233,34,34,54,174,66,161,229,90,29,5,173,232,67,119,185,158,45,176,4,66,136,168,53,197,211,
239,230,39,236,8,227,225,235,48,54,195,27,111,240,60,55,52,83,185,241,135,154,157,183,200,99,23,12,129,129,166,202,223,32,217,236,128,72,53,150,149,162,18,204,14,63,191,154,58,170,174,160,84,106,207,142,
204,25,127,192,224,40,128,160,214,64,55,57,192,166,157,181,244,5,205,109,190,109,248,142,154,74,62,115,200,105,241,112,225,23,226,38,186,192,120,60,155,133,144,255,227,131,44,165,94,123,62,253,96,57,238,
36,123,118,173,125,13,51,180,77,21,138,219,111,84,248,95,40,99,95,81,151,126,23,219,198,151,142,60,233,191,164,71,200,69,89,135,77,15,219,88,44,84,121,50,80,184,193,169,1,40,27,200,154,74,4,209,9,124,
236,140,227,74,143,157,112,226,153,93,19,50,173,172,240,103,62,125,74,233,177,247,29,123,234,88,129,162,129,156,3,153,159,251,113,233,199,84,196,171,169,228,210,81,242,79,3,1,225,81,126,141,122,7,169,
162,126,26,13,195,44,34,232,238,115,108,213,151,113,13,79,143,0,117,152,182,85,55,217,99,78,132,178,235,134,234,238,89,55,13,65,209,176,134,97,166,52,206,250,98,38,0,168,209,227,212,193,251,73,54,58,151,
190,110,149,117,230,97,211,50,53,80,16,155,206,160,102,99,52,220,198,58,92,164,195,59,181,73,69,67,30,251,173,236,177,177,43,6,57,121,160,149,241,221,13,154,43,196,67,233,169,102,55,58,212,224,133,121,
110,60,57,195,250,76,28,197,108,0,96,242,244,131,247,41,188,52,19,110,70,91,94,214,159,67,133,237,0,31,122,118,211,209,178,174,99,57,62,26,212,150,112,185,40,32,80,175,72,12,232,189,129,154,159,129,18,
24,26,7,146,202,88,51,6,164,19,156,8,153,171,252,90,81,164,34,211,108,202,59,110,122,245,134,29,98,102,162,159,178,153,23,121,196,130,49,67,190,161,38,166,86,123,96,8,130,186,219,76,65,212,131,85,103,
190,206,3,4,71,12,200,168,143,117,164,65,95,52,5,52,212,14,74,34,216,178,204,53,153,172,97,246,163,178,74,121,9,138,104,96,203,54,90,60,233,89,44,3,68,113,81,67,90,137,70,89,141,215,33,103,241,75,245,
40,85,54,108,1,134,216,152,163,108,58,234,134,178,169,130,44,165,117,53,247,78,106,64,235,130,72,54,54,91,82,133,62,170,240,130,178,52,71,154,27,143,19,32,141,220,73,179,197,143,14,245,59,5,77,146,1,33,
52,155,8,12,192,55,173,1,81,136,14,153,116,194,137,103,117,209,147,67,109,181,123,122,223,7,78,211,191,127,230,147,39,15,129,55,105,136,33,129,142,180,203,49,93,231,44,157,147,45,254,156,201,72,212,28,
12,69,182,20,29,161,38,139,133,66,37,200,52,151,218,51,49,134,68,155,160,179,106,221,169,107,44,108,202,160,90,236,156,28,102,29,197,58,101,104,38,183,96,102,94,153,6,93,208,23,237,131,33,235,119,203,
81,222,2,64,50,109,202,182,134,79,15,227,158,168,122,68,45,223,58,57,135,13,98,111,110,35,175,60,21,78,3,37,103,248,49,187,27,212,204,98,102,0,81,49,172,149,204,62,4,10,16,13,219,182,72,169,102,91,112,
11,74,135,12,40,79,134,177,195,228,60,47,97,133,241,244,162,166,29,108,114,46,244,82,232,126,193,165,205,115,2,127,76,46,128,45,253,221,143,125,162,145,31,66,217,185,137,209,213,79,19,51,115,218,133,29,
234,162,82,100,163,83,167,132,33,11,198,113,49,164,163,252,228,12,57,148,189,6,4,141,40,15,139,8,224,148,102,192,62,194,112,150,8,99,79,100,123,107,175,16,39,117,169,93,125,157,246,74,242,136,240,206,
12,186,114,91,96,168,133,253,9,250,162,218,98,35,95,139,104,52,2,57,194,111,131,143,124,237,181,25,37,234,69,221,164,233,72,202,83,180,169,144,129,238,192,118,246,113,121,179,232,58,1,156,0,34,214,74,
223,52,121,9,64,60,52,213,150,186,191,129,244,32,135,42,197,32,140,225,38,178,72,7,115,147,19,201,96,116,43,156,71,93,216,100,118,220,56,190,198,236,162,20,134,181,10,143,141,118,116,228,26,113,30,47,
81,167,138,38,251,160,80,63,84,129,20,187,227,97,33,49,243,20,150,48,81,184,107,198,149,62,113,50,151,43,193,132,40,228,43,101,209,157,182,27,75,56,205,108,220,226,161,17,93,94,2,128,24,224,116,162,13,
32,35,44,228,98,117,141,5,84,155,111,115,15,26,124,38,197,5,30,234,230,26,215,72,222,80,158,214,46,121,184,207,23,58,31,12,101,138,45,166,22,182,40,232,139,18,22,245,130,32,50,108,254,162,144,220,176,
215,68,5,196,100,163,211,38,141,248,202,13,162,110,207,252,190,153,192,127,255,148,61,162,162,190,53,30,42,38,150,18,205,235,134,74,73,135,13,90,164,154,109,97,9,189,173,27,224,198,168,59,79,104,116,102,
36,229,24,46,215,135,102,147,58,179,105,157,57,165,183,29,49,43,75,233,160,161,126,168,55,36,186,231,178,13,52,115,115,132,77,158,151,129,239,27,155,13,51,147,117,185,135,79,165,4,187,206,156,81,3,69,
20,1,204,2,44,201,233,134,230,190,82,104,213,35,200,103,244,178,237,241,106,56,24,182,8,189,5,251,119,56,206,23,115,135,67,136,125,221,155,2,117,46,162,184,10,113,166,206,131,105,230,156,32,195,64,31,
187,174,188,92,243,239,109,214,19,208,48,150,89,2,20,235,63,137,128,184,238,19,81,100,110,244,203,118,198,99,9,219,125,210,82,110,225,160,187,73,9,235,18,136,77,47,161,147,59,231,43,226,51,231,27,101,
181,180,13,231,43,1,78,99,9,109,24,134,65,172,237,121,29,218,248,12,178,219,191,83,199,156,18,44,158,78,196,169,100,195,198,100,23,8,133,53,181,23,204,244,96,83,231,171,206,194,139,179,40,26,241,225,196,
4,164,130,212,116,116,225,109,42,69,34,243,76,138,74,100,69,214,118,48,177,138,24,185,160,85,40,241,210,224,243,102,164,153,234,115,54,82,54,109,129,33,63,179,18,96,133,132,221,212,120,106,162,141,164,
80,246,156,122,175,8,250,162,37,29,192,62,48,228,157,214,82,21,5,145,21,230,81,88,227,122,218,109,220,192,2,206,108,93,50,78,72,60,12,195,31,56,117,83,221,90,144,52,45,45,78,157,16,119,130,34,169,152,
212,163,236,151,146,110,246,29,236,10,75,102,32,165,172,61,108,169,51,187,89,186,100,58,55,40,29,176,7,96,196,241,144,54,130,138,120,228,192,175,103,140,148,160,148,202,144,34,169,103,137,57,125,241,3,
85,201,81,154,233,103,152,245,48,24,155,72,105,68,128,204,111,52,42,23,194,139,6,60,91,202,136,119,83,57,9,96,213,184,39,7,86,61,203,70,164,250,112,99,102,167,142,138,55,62,5,203,72,119,183,205,19,68,
168,201,224,120,157,217,21,21,102,93,208,23,237,240,15,154,79,254,164,48,135,113,180,12,183,234,206,115,12,9,134,0,36,215,187,203,92,145,218,210,104,214,80,59,72,177,238,146,193,104,133,71,78,31,117,171,
79,160,187,64,70,159,92,86,221,234,242,249,159,70,93,146,244,206,176,117,211,133,130,244,236,141,70,41,154,103,80,45,174,103,60,192,80,170,179,108,83,195,186,12,252,94,170,95,232,233,0,90,215,196,167,
177,89,119,107,250,66,139,105,130,62,3,152,80,14,182,113,68,125,90,205,193,239,149,212,78,141,116,196,206,173,44,6,85,26,121,147,46,48,242,241,68,132,86,221,151,65,95,232,18,4,178,75,17,236,30,87,102,
195,128,144,41,51,26,187,46,80,110,100,102,74,103,54,236,3,49,4,53,68,93,153,55,192,67,180,129,106,163,216,28,224,69,54,96,50,11,137,200,52,182,13,240,164,230,26,165,102,118,65,158,51,100,185,70,194,161,
15,52,122,164,13,40,70,214,89,38,144,7,12,9,219,102,226,62,121,69,137,44,209,62,46,53,47,212,229,247,143,147,153,155,169,173,116,68,245,149,221,78,216,141,36,82,15,121,170,38,119,62,243,172,137,10,75,
170,212,57,219,29,91,83,234,183,29,230,48,142,158,37,80,5,110,77,38,200,254,141,235,172,140,244,16,219,177,74,107,49,42,142,217,245,170,154,255,154,145,157,200,233,156,199,70,68,41,127,185,106,35,222,
210,4,178,64,30,251,173,43,22,81,55,102,176,132,162,233,214,142,17,229,94,191,176,94,21,128,200,243,120,63,64,81,222,81,147,139,8,182,8,123,52,250,242,111,166,231,121,164,121,132,11,31,5,23,199,49,26,
149,253,171,2,67,10,127,72,117,239,17,186,211,219,215,2,210,142,89,101,47,78,190,216,189,106,39,29,104,192,210,171,193,158,150,231,23,213,123,48,171,6,235,99,24,190,212,81,32,225,1,66,110,234,137,155,
24,235,68,148,48,174,131,38,187,121,168,102,250,254,72,119,87,31,252,245,140,254,209,97,71,208,5,50,193,144,185,30,230,180,112,160,92,145,192,221,61,67,234,251,89,218,3,199,37,143,214,192,214,32,255,58,
63,187,238,183,201,113,146,123,146,244,92,93,49,132,251,215,22,24,202,55,55,79,251,231,172,46,202,24,70,82,33,215,237,59,162,138,215,177,27,236,49,255,102,95,26,85,24,74,63,90,50,171,85,253,95,60,95,219,
148,57,29,255,32,182,211,193,198,6,209,250,250,205,248,18,95,185,153,143,36,156,141,186,168,0,14,22,95,223,124,9,28,248,191,44,90,226,178,56,105,186,76,52,195,117,36,27,140,53,112,87,143,114,59,238,64,
173,59,50,92,83,149,199,102,36,132,234,96,198,194,30,175,49,130,251,44,161,202,54,116,68,154,85,98,139,176,58,90,19,252,137,108,166,169,91,101,33,121,223,192,62,240,19,162,66,35,106,5,180,77,113,109,15,
140,44,73,195,49,2,68,206,239,68,198,160,191,86,129,80,160,250,236,103,225,242,11,198,94,159,13,140,64,6,24,81,163,128,216,156,98,32,28,177,193,13,214,146,59,224,125,20,41,122,33,215,51,80,51,48,196,30,
172,206,128,61,19,102,196,101,150,4,32,68,49,116,117,4,165,178,13,134,242,127,35,221,99,2,200,90,139,147,201,23,158,68,151,188,170,128,224,68,157,188,128,72,122,184,206,105,94,21,104,172,169,150,41,115,
5,24,82,133,134,24,195,26,162,220,136,102,169,164,193,32,167,118,208,152,131,46,154,217,123,115,99,20,33,101,174,47,39,200,242,168,132,181,202,0,137,200,231,162,22,157,150,74,236,228,164,140,88,6,69,212,
6,40,170,40,255,55,90,103,233,54,209,100,26,189,129,70,79,254,117,120,126,21,111,20,134,176,192,88,13,191,148,12,137,108,112,171,152,113,212,148,234,116,103,197,37,89,109,19,243,9,244,69,102,12,179,211,
252,173,66,70,72,53,50,205,173,23,42,131,32,134,191,42,43,232,136,64,10,16,213,205,105,151,167,177,147,206,241,76,199,112,91,140,192,47,153,210,192,29,64,212,15,43,194,141,88,141,159,229,50,227,51,34,
186,87,32,219,149,235,25,241,147,3,25,214,200,139,71,12,67,51,111,240,209,204,48,96,201,142,1,147,206,88,132,228,96,200,28,9,64,35,234,13,15,242,175,117,22,241,78,15,28,219,178,14,46,114,72,103,152,89,
80,55,89,88,204,80,34,199,19,194,74,190,120,92,135,44,253,51,227,84,103,62,72,163,137,142,9,138,74,92,229,153,144,28,34,68,129,114,64,84,55,99,70,230,241,81,166,254,181,129,173,37,73,227,176,146,71,117,
244,19,12,249,234,154,130,0,105,109,253,68,87,193,80,160,230,167,198,72,158,8,100,162,68,229,109,202,109,8,95,103,55,110,242,119,107,230,14,123,45,50,82,79,69,212,28,55,5,26,79,105,89,140,199,99,71,231,
104,83,122,140,14,109,10,48,65,202,104,228,102,25,235,38,186,57,168,177,28,45,210,248,199,132,202,62,105,148,26,145,38,217,226,186,138,6,0,41,80,0,68,189,214,197,45,42,59,169,28,1,153,215,32,13,42,18,
110,12,56,255,189,159,96,196,77,151,11,102,75,91,198,161,160,129,38,59,142,211,73,225,220,169,202,65,201,249,96,162,94,41,25,105,132,222,104,182,141,44,161,162,180,197,157,48,63,48,115,8,104,159,111,0,
67,129,26,104,26,157,38,87,105,168,142,19,40,202,207,43,35,37,66,68,163,49,119,200,154,39,207,212,130,237,211,204,222,105,151,31,164,71,242,4,7,111,160,30,3,34,139,241,155,40,189,2,12,37,129,73,189,166,
130,233,217,96,244,167,174,199,44,58,164,161,86,76,167,125,244,125,3,192,67,212,19,207,222,89,31,63,174,227,247,126,232,248,51,71,211,228,215,249,230,221,139,158,182,202,51,39,127,248,51,67,135,211,219,
167,54,82,229,242,38,10,130,116,253,3,85,240,175,108,112,57,163,202,171,189,160,153,200,132,58,106,187,0,147,157,179,151,198,72,35,116,4,138,186,205,27,173,156,75,233,88,8,149,22,93,101,178,9,247,145,
211,130,157,25,72,3,34,209,91,150,83,30,31,61,100,181,194,99,32,37,0,153,4,6,53,36,196,105,31,253,128,113,112,83,7,152,20,255,158,252,225,79,244,96,205,242,239,202,249,35,47,170,14,53,69,173,173,93,148,
121,199,71,195,26,30,30,84,20,248,177,49,80,225,8,68,164,211,229,170,98,190,210,124,139,148,109,140,62,200,228,134,136,132,215,141,226,202,125,49,174,188,58,250,8,186,115,54,37,128,43,171,138,198,53,250,
155,2,169,64,42,34,8,129,134,103,107,144,252,144,58,191,87,54,75,97,215,140,232,183,131,149,43,0,82,208,31,99,14,136,122,111,26,74,6,152,24,100,244,145,180,170,83,52,24,26,119,32,228,10,28,23,0,249,0,
81,47,234,137,68,241,111,238,237,117,67,125,92,21,57,234,103,141,83,235,212,137,23,127,102,81,165,222,25,17,31,58,254,19,65,114,185,170,141,235,193,51,245,94,168,40,207,226,68,163,113,205,178,146,159,
101,75,70,3,137,198,96,232,67,199,127,34,140,204,29,2,153,112,214,199,63,56,16,45,72,4,176,160,172,239,114,168,241,48,206,92,154,101,208,201,168,173,238,115,253,210,23,222,97,168,37,228,227,51,23,6,181,
199,65,250,4,234,51,32,210,3,198,152,193,198,228,61,70,209,126,21,50,29,115,193,71,222,127,79,254,240,167,97,71,101,220,89,205,212,195,117,19,40,231,123,25,221,95,72,58,94,29,163,40,129,123,5,212,134,
77,129,5,129,219,87,80,212,130,177,62,238,68,70,203,41,209,210,235,1,22,34,51,90,42,139,151,185,244,137,173,112,126,56,29,129,26,130,162,210,51,50,156,111,206,219,198,68,181,155,243,205,246,127,12,211,
133,154,188,73,14,200,254,99,143,36,12,250,35,0,162,30,243,28,179,180,187,198,229,255,138,8,156,14,234,48,212,74,252,27,194,62,50,30,55,59,205,185,135,181,87,233,65,2,222,138,107,34,59,88,196,66,245,185,
140,202,38,142,136,128,148,6,40,232,2,141,219,9,18,4,72,25,148,89,83,113,204,12,38,106,104,72,185,125,226,4,1,50,119,142,112,234,204,15,9,134,196,120,59,32,122,99,148,219,128,72,34,164,79,26,171,173,50,
109,184,102,252,192,57,24,98,195,126,49,1,82,150,127,235,112,142,12,174,145,64,181,57,91,113,63,206,47,129,84,68,40,7,69,249,40,98,70,255,187,166,213,209,156,51,133,130,59,133,140,157,223,123,73,102,4,
8,246,28,4,50,90,58,144,130,57,78,135,24,61,144,30,128,140,132,74,40,54,5,96,100,220,163,79,189,142,94,30,175,11,105,3,245,222,64,11,84,69,41,32,163,34,219,181,137,84,50,127,207,207,182,68,84,204,38,42,
163,45,109,204,182,34,245,2,13,207,217,242,249,210,251,47,73,195,41,55,215,131,89,212,34,74,36,45,64,212,40,26,228,218,122,92,163,125,237,101,182,77,160,97,161,190,212,16,9,34,197,246,100,212,158,72,140,
231,208,85,159,186,33,143,153,208,47,16,228,128,21,18,198,248,120,37,238,164,0,69,246,85,10,82,65,32,7,78,153,251,158,165,218,76,148,173,32,157,106,231,86,49,80,11,230,210,176,8,173,108,221,42,135,174,
4,234,141,193,31,168,13,121,211,158,28,207,165,4,27,3,140,44,108,148,207,27,10,139,60,18,0,104,189,249,55,219,64,215,5,73,221,220,115,237,14,12,61,123,252,171,35,50,27,202,92,158,65,156,57,19,202,112,
105,50,136,244,0,13,246,232,113,174,9,199,7,169,21,136,250,51,152,85,231,4,91,17,161,32,233,202,230,198,32,128,144,241,253,121,190,175,41,216,26,184,123,163,6,2,153,116,100,159,252,85,219,28,57,194,82,
148,161,21,41,193,74,198,115,90,208,202,26,3,36,229,249,82,169,69,65,212,246,105,201,3,245,93,90,9,42,64,17,81,136,134,142,194,49,202,71,96,36,105,1,122,155,185,168,186,13,138,202,221,201,56,156,58,221,
228,136,178,179,70,131,49,231,53,143,112,21,74,202,95,17,121,158,172,227,126,154,67,128,131,34,25,95,146,253,1,68,90,192,241,160,12,254,186,171,32,215,189,146,183,187,238,199,33,205,5,45,161,148,16,65,
37,136,210,208,56,42,129,96,40,76,192,21,79,66,248,235,17,204,75,145,194,190,8,41,212,146,17,178,136,64,29,59,17,25,123,22,210,223,3,30,170,5,9,67,22,75,72,18,93,97,75,50,124,38,157,128,161,224,155,173,
161,54,210,136,72,106,68,196,34,43,234,145,84,142,22,245,28,117,7,48,148,157,95,161,192,144,24,156,74,177,6,171,90,120,130,51,239,8,27,224,194,202,128,175,123,202,99,112,210,143,239,217,42,220,250,113,
223,88,77,170,182,145,129,233,90,48,233,242,9,29,212,99,70,0,178,121,57,206,67,108,191,142,196,204,190,129,155,200,121,13,28,12,219,77,39,86,138,242,115,217,223,164,162,76,185,39,42,173,217,94,10,109,
238,5,163,175,15,230,62,1,18,2,33,109,174,10,12,69,61,147,189,98,6,167,36,80,221,117,82,81,239,203,100,26,194,164,35,130,221,38,61,164,61,52,229,49,116,101,164,179,229,168,17,118,28,132,201,146,131,160,
82,105,176,204,174,187,112,143,134,125,12,84,107,80,20,247,239,220,112,56,20,109,109,144,244,32,216,94,1,33,79,24,135,102,46,112,201,249,133,29,188,96,21,110,11,27,23,149,186,125,179,209,212,65,228,50,
152,42,58,94,161,86,134,113,0,68,125,2,68,36,178,46,104,129,170,23,9,118,235,237,65,219,82,185,61,69,225,128,212,6,58,123,33,8,171,142,97,66,84,203,249,174,50,8,247,110,192,216,48,218,3,198,252,48,26,
128,83,129,171,172,17,102,167,115,92,21,71,4,96,27,168,206,96,40,115,24,198,253,59,77,161,75,76,251,226,71,244,0,20,81,129,70,74,161,31,46,205,13,152,41,24,50,163,68,154,3,68,185,1,3,156,59,174,2,85,250,
215,28,11,73,42,80,82,169,163,93,58,248,179,70,33,99,174,111,203,45,8,156,134,97,142,101,19,55,82,89,177,197,236,183,192,147,129,170,207,17,192,82,0,81,228,159,165,217,99,244,202,218,128,14,231,184,85,
61,219,175,239,209,137,101,37,147,164,226,74,56,236,97,160,225,57,61,113,63,132,78,185,126,40,80,107,212,139,189,81,155,65,78,155,73,179,235,183,96,235,113,119,28,236,12,96,88,67,14,32,84,143,160,205,
77,187,202,184,25,65,213,21,201,250,28,54,2,168,39,215,19,148,140,215,244,39,32,13,177,56,91,188,139,236,52,17,1,17,213,71,254,86,165,211,134,104,106,77,246,70,0,81,12,164,137,35,141,101,10,22,66,207,
9,202,147,131,187,101,164,75,7,22,5,178,87,134,102,184,170,93,176,28,10,83,142,203,48,214,213,127,97,43,3,213,151,204,250,253,84,219,152,253,145,176,33,39,162,94,76,192,30,20,98,34,13,227,101,221,44,55,
164,138,43,162,10,0,101,130,161,134,60,93,71,207,3,85,77,92,15,212,51,49,19,133,62,103,38,24,34,202,178,156,234,218,253,61,168,134,122,105,6,221,49,48,227,32,68,177,71,248,74,169,51,165,88,22,177,156,
148,179,159,78,115,65,244,60,27,153,75,252,16,199,52,55,167,46,170,206,202,218,200,103,112,148,140,7,105,92,112,112,224,5,170,187,228,67,255,0,145,208,135,38,8,184,58,218,237,246,102,57,64,195,0,70,212,
133,175,163,22,47,195,188,28,81,113,121,254,15,247,117,105,24,0,9,210,151,20,236,189,254,81,68,0,68,20,14,182,2,67,195,146,30,39,185,48,198,131,3,97,240,148,183,225,136,115,192,90,213,89,135,25,156,102,
63,57,26,202,247,178,221,164,183,162,195,93,88,255,70,128,168,22,160,168,228,72,117,187,61,72,231,7,65,27,6,170,61,197,204,220,143,179,92,4,37,66,234,92,77,4,172,44,11,39,51,167,141,109,92,193,132,158,
117,21,234,200,155,232,106,7,118,152,173,139,60,214,254,25,17,128,228,172,193,14,119,159,219,251,113,102,135,158,189,199,82,206,24,241,84,206,210,228,136,135,99,21,2,75,215,194,230,214,233,237,172,88,
41,2,64,156,117,180,145,146,0,153,183,226,78,161,243,169,77,216,195,145,37,142,205,121,57,45,3,34,176,35,203,199,220,185,1,100,7,89,102,37,179,237,186,150,187,169,47,220,230,10,58,75,93,235,95,243,129,
20,213,45,32,194,129,15,84,67,64,212,119,183,2,135,78,115,131,165,168,33,34,177,166,132,59,157,17,242,169,63,110,195,132,126,43,109,83,103,86,107,134,65,21,215,147,254,250,224,11,27,144,9,65,0,143,93,
11,238,124,86,73,104,156,16,168,151,44,38,50,80,196,102,209,169,169,207,101,49,43,161,205,129,57,26,12,177,241,89,129,124,167,124,96,36,28,213,203,80,219,157,194,113,70,250,186,113,68,238,102,135,61,14,
52,90,128,168,89,177,125,62,184,152,115,247,79,40,170,24,176,56,53,254,245,84,52,27,99,38,172,129,139,185,174,146,152,217,68,122,55,118,83,85,119,73,21,124,86,26,10,87,210,157,221,23,176,167,159,122,172,
254,253,164,83,62,221,68,85,101,13,43,186,5,134,62,113,230,241,129,109,59,225,242,177,113,190,80,9,12,154,255,14,11,9,154,121,227,150,64,221,56,55,229,199,63,121,230,241,72,149,205,123,220,135,62,174,
64,17,163,186,90,168,125,25,108,141,100,8,29,105,125,43,211,146,147,163,31,250,130,60,32,73,86,249,159,168,98,158,98,214,202,16,97,20,75,160,154,1,34,238,248,152,178,10,173,55,138,24,20,96,136,13,235,
151,155,152,195,129,122,35,198,140,240,187,139,76,132,189,37,2,238,12,95,103,14,91,23,172,22,110,240,152,128,191,227,156,116,223,168,175,69,206,88,41,119,126,213,106,109,85,173,28,129,189,3,244,122,123,
13,129,44,37,29,1,82,10,149,218,195,163,127,166,1,221,29,114,88,163,67,1,8,213,119,15,162,252,135,56,131,43,196,21,93,61,185,48,120,137,91,222,83,169,191,188,78,149,50,245,2,70,98,6,239,237,185,4,34,64,
230,179,159,89,192,142,18,177,129,156,220,135,57,232,180,64,245,145,127,155,111,185,3,55,59,74,228,128,160,226,247,188,218,62,155,90,45,168,12,115,24,128,148,234,83,242,48,41,187,49,130,16,54,237,11,246,
117,165,88,142,46,200,78,76,22,30,168,234,70,102,114,163,171,93,35,134,27,136,192,124,224,181,57,132,206,82,145,178,234,77,64,169,232,169,175,164,52,65,36,116,211,157,96,220,13,210,116,80,156,192,200,
138,189,139,68,203,17,51,85,237,148,36,202,229,112,96,129,64,61,62,95,146,1,150,238,80,206,162,134,141,162,246,128,121,170,229,63,135,206,10,30,221,77,177,55,217,189,118,36,13,179,66,139,92,81,129,205,
172,161,174,97,191,3,213,224,164,49,24,204,70,231,68,151,95,25,72,149,129,199,102,33,190,25,245,81,21,247,110,224,192,54,162,243,106,77,99,248,39,247,199,139,49,134,56,23,229,34,27,46,30,182,186,100,178,
181,61,194,137,248,85,193,85,55,122,212,206,149,229,159,89,138,58,229,249,149,146,242,218,220,226,210,36,42,242,229,6,45,80,213,221,144,0,169,121,47,1,12,213,3,58,144,30,222,59,10,6,22,163,212,132,94,
48,136,8,204,217,191,117,154,51,212,10,96,53,31,176,92,100,212,190,92,9,212,223,243,85,100,164,82,249,228,9,110,27,152,235,140,43,98,115,112,97,32,136,108,61,135,231,106,179,35,173,100,175,155,186,203,
128,237,140,213,114,45,236,119,160,26,0,162,60,122,195,142,33,39,149,197,154,69,197,165,209,169,132,109,147,150,97,161,41,215,222,54,241,83,150,76,228,60,25,242,72,123,104,114,56,77,171,125,51,135,60,
162,214,151,35,204,174,238,235,230,213,154,96,72,61,192,76,101,188,204,141,242,214,3,5,106,197,250,30,17,80,36,178,136,80,6,32,104,104,106,134,36,251,129,16,91,195,63,5,132,10,63,7,80,84,95,195,151,5,
192,68,214,240,162,78,91,189,231,145,117,169,45,230,113,146,241,141,7,79,180,217,155,162,54,160,72,122,74,149,117,9,57,229,47,8,25,66,129,106,4,136,88,102,41,83,82,26,24,197,108,145,205,108,244,56,246,
28,98,229,42,202,188,248,153,167,82,146,255,184,51,81,81,120,164,13,220,112,32,122,42,154,132,7,169,26,251,194,158,66,157,86,118,163,27,13,52,45,39,145,78,34,87,0,217,106,229,89,231,118,157,69,58,40,171,
245,14,220,92,179,237,225,209,188,47,237,246,80,246,99,221,249,78,215,147,82,9,13,149,210,164,50,63,157,104,171,109,115,160,254,155,241,164,25,208,230,77,57,211,79,29,199,129,158,214,172,167,162,239,57,
241,112,175,132,149,9,2,67,223,179,68,208,150,129,106,5,136,204,65,51,121,83,38,206,181,151,21,65,48,67,155,236,28,94,169,97,63,51,235,87,154,138,204,174,163,75,3,16,234,135,24,34,46,227,87,159,66,67,
161,131,90,77,131,155,169,141,194,37,48,100,42,64,86,109,107,204,186,160,58,91,181,121,67,242,44,253,52,24,112,117,211,198,163,134,138,236,206,83,195,64,178,40,40,69,233,226,75,53,35,105,158,235,136,60,
89,40,156,169,49,57,174,58,21,111,92,210,230,68,249,32,11,232,134,20,163,196,246,229,30,10,33,53,50,80,237,0,81,94,247,192,165,70,9,217,3,169,195,197,92,40,49,97,116,19,201,139,227,68,118,192,243,188,
118,184,112,202,154,49,16,14,67,111,181,11,87,198,218,169,202,118,236,147,96,100,179,153,80,169,251,32,0,76,195,8,190,15,193,98,167,153,213,39,68,214,23,34,24,112,245,56,2,2,96,73,35,162,124,163,50,32,
26,38,107,200,245,8,115,11,78,6,38,48,40,0,162,177,163,113,24,226,105,20,217,68,208,5,5,52,22,91,43,3,32,10,84,63,64,196,20,129,220,44,54,51,183,33,55,82,201,40,232,37,165,220,242,220,225,60,226,147,207,
39,80,169,113,82,229,206,177,217,159,155,101,69,187,206,64,93,52,3,53,152,48,131,68,185,248,17,45,136,161,94,137,41,171,20,40,7,214,37,26,226,161,109,121,10,106,24,204,90,31,253,75,52,164,226,198,233,
227,72,182,231,130,168,98,38,87,29,113,16,115,197,89,111,242,102,146,0,162,218,223,103,160,46,242,140,46,88,29,229,77,23,153,189,148,215,93,241,104,195,63,13,125,116,186,92,168,27,15,212,11,187,55,183,
125,59,27,183,17,19,21,41,110,90,123,185,253,149,45,133,108,184,244,117,234,155,178,188,89,26,89,117,84,116,165,227,34,45,47,68,134,250,193,20,101,207,11,25,96,104,240,198,17,70,52,127,56,85,192,95,128,
57,120,181,107,99,122,8,64,178,112,162,221,195,112,148,217,153,142,108,59,167,168,230,247,34,173,54,202,173,152,123,174,145,20,82,81,199,10,8,193,40,184,111,56,160,97,4,116,52,49,64,60,54,153,4,54,24,
10,205,145,2,245,192,193,0,50,10,106,115,112,148,182,7,136,116,9,145,102,79,167,55,51,27,160,72,26,2,138,204,12,87,163,133,54,187,19,108,210,176,87,125,7,68,208,41,143,166,231,105,16,96,168,148,248,32,
49,162,194,144,202,55,29,228,125,125,118,135,0,102,49,92,242,136,61,127,231,209,161,154,243,151,174,67,45,69,133,154,121,134,157,231,85,72,153,41,56,24,198,65,115,113,137,175,71,55,93,142,198,45,99,44,
116,138,13,212,19,138,20,24,42,41,124,245,92,235,58,63,150,170,121,130,5,99,178,196,123,195,37,199,158,212,57,215,212,117,203,241,67,74,220,224,164,78,54,37,188,149,212,184,158,203,191,14,159,29,106,79,
5,3,80,145,215,64,53,58,25,249,164,244,161,217,25,57,180,224,58,91,230,180,69,164,231,156,159,82,116,64,165,205,5,22,30,11,237,101,235,135,81,76,175,23,186,105,2,143,211,190,18,220,65,68,129,2,205,128,
84,80,134,216,252,75,179,24,235,172,10,183,49,156,64,185,47,66,246,119,12,153,100,41,62,78,55,159,194,195,151,43,36,178,57,187,232,165,109,63,31,34,66,53,16,61,140,58,204,104,244,10,124,43,118,62,74,7,
211,232,188,72,245,88,255,64,190,93,106,37,117,142,90,52,222,7,224,82,96,97,167,210,213,222,10,226,54,65,17,123,254,13,197,215,227,117,70,77,219,98,212,246,94,141,194,24,179,162,184,50,22,242,141,103,
15,20,168,67,235,210,248,179,172,189,201,214,235,196,149,35,93,98,134,49,25,154,168,120,222,138,14,81,217,80,40,77,168,206,188,120,129,2,121,205,30,11,255,140,10,24,202,61,13,66,247,153,39,199,31,17,168,
126,114,52,75,157,235,4,148,15,66,121,243,112,159,251,202,51,227,122,137,169,56,71,92,30,13,77,20,206,211,88,233,12,205,7,163,230,96,21,67,126,178,187,37,19,68,208,146,129,58,208,191,238,191,212,88,59,
147,211,200,24,174,138,17,150,147,62,206,16,144,18,58,44,116,219,108,72,213,48,33,111,140,96,69,129,80,6,72,0,58,237,236,16,168,7,162,135,235,117,57,5,26,151,35,34,8,243,54,114,66,7,76,41,180,194,26,10,
145,202,150,151,210,231,129,230,6,224,135,107,114,152,134,6,125,162,148,94,173,211,16,242,141,80,5,177,185,231,142,205,123,205,94,31,206,214,152,157,81,192,51,159,106,216,239,170,224,113,49,134,251,234,
207,88,14,209,223,64,29,130,161,204,232,106,200,62,102,26,157,251,59,151,4,142,80,17,34,19,208,72,233,185,136,212,249,8,23,111,193,48,116,67,8,116,224,76,35,234,147,159,92,216,54,163,82,76,41,108,253,
166,147,77,3,215,15,211,41,145,228,2,244,166,239,8,128,168,163,171,228,138,127,81,76,225,180,7,212,57,249,53,12,10,245,120,99,71,60,82,198,114,174,44,168,22,181,189,3,223,87,171,123,70,0,68,129,218,177,
189,184,212,167,222,42,243,105,67,115,154,54,114,174,134,98,61,111,200,91,189,107,70,128,164,115,97,46,16,26,213,2,200,33,99,24,193,16,53,145,47,133,204,27,165,206,50,133,19,192,4,67,129,134,113,43,219,
81,198,117,81,220,18,96,1,22,245,229,61,59,192,195,77,214,80,69,145,184,193,39,133,232,80,56,159,67,175,151,51,126,22,99,206,202,118,141,88,0,68,129,90,5,66,70,54,7,89,255,148,144,74,187,252,104,254,27,
251,213,106,85,81,177,249,118,51,149,78,32,140,161,172,161,65,50,120,211,109,196,196,184,40,218,205,83,136,133,14,251,142,74,221,113,70,54,56,69,117,147,107,249,156,4,81,46,181,169,137,236,209,248,197,
219,114,91,213,154,146,231,124,149,242,24,130,78,25,111,29,54,90,251,63,206,145,33,75,62,88,127,85,105,209,160,93,3,249,216,130,122,106,239,58,17,162,86,187,186,72,216,53,69,33,93,174,54,70,123,157,162,
67,230,47,140,33,229,141,114,67,17,65,28,216,124,84,156,6,220,172,175,53,215,244,234,107,206,128,58,245,90,26,230,160,186,118,213,133,145,221,123,114,110,139,66,46,234,184,107,179,33,214,27,30,222,14,
84,236,171,116,141,3,95,161,124,136,30,5,114,178,215,242,220,54,114,173,178,46,1,34,182,52,78,179,33,122,140,114,15,111,56,192,40,48,240,96,72,232,154,150,58,84,57,240,48,26,113,190,53,205,249,59,239,
117,31,234,25,70,12,20,13,227,160,159,76,46,115,13,215,51,51,118,170,90,103,219,43,95,217,21,72,129,84,30,146,238,226,129,122,196,79,118,7,148,161,214,203,2,65,111,160,36,105,171,162,240,236,128,163,42,
251,52,208,216,129,35,242,204,77,237,38,32,178,243,116,217,3,134,220,199,124,77,23,66,46,104,45,196,140,93,107,86,3,129,7,167,195,28,15,207,193,211,69,176,217,181,103,143,4,79,223,200,105,102,230,6,220,
91,199,16,5,89,80,142,234,186,176,236,26,54,198,96,239,102,53,65,134,172,96,166,162,229,182,186,225,128,145,198,198,252,81,251,62,140,243,136,8,110,103,185,64,174,152,165,86,173,136,178,28,9,11,58,70,
140,18,161,40,205,233,45,197,76,85,22,172,201,128,228,71,244,185,245,61,146,3,212,134,77,117,100,251,195,53,226,99,93,227,48,116,197,147,69,71,32,53,94,40,136,224,17,61,54,108,117,59,170,82,198,117,140,
197,212,20,95,90,215,39,29,125,145,163,153,234,248,113,22,20,224,74,123,40,36,101,143,225,25,245,182,111,31,6,18,181,234,248,90,59,44,84,213,191,203,43,34,26,149,116,132,102,94,163,237,18,73,81,61,250,
167,203,128,136,184,202,97,231,25,198,234,94,12,147,215,40,111,135,22,204,223,4,231,157,253,89,156,123,254,119,241,195,43,126,132,221,95,190,11,222,115,204,155,241,243,27,127,141,79,126,238,43,214,107,
223,246,230,255,194,115,119,88,140,163,222,126,172,247,179,78,61,249,131,120,238,14,139,75,143,255,252,198,95,227,91,223,187,24,43,86,62,12,0,120,245,190,175,196,81,71,188,222,251,25,183,220,182,20,103,
125,230,203,88,189,122,53,0,224,220,175,124,26,15,45,91,129,83,78,251,4,78,61,249,131,0,128,83,78,251,68,233,125,175,222,247,149,56,228,192,253,241,198,183,189,15,123,236,186,115,229,231,3,192,171,14,
56,172,203,76,67,22,62,109,135,174,189,226,123,222,199,47,189,252,26,252,247,215,191,163,255,62,244,160,87,227,136,195,94,91,249,57,123,238,251,58,44,92,184,0,231,159,243,89,156,243,157,139,113,201,247,
127,88,56,198,60,237,141,243,125,190,234,71,215,225,171,95,251,38,0,96,241,179,159,137,143,159,122,130,119,239,243,215,231,124,210,187,3,152,183,118,212,209,217,177,160,77,213,222,157,255,157,139,113,
193,69,63,172,124,221,62,123,237,142,87,236,249,114,108,189,197,211,244,99,75,254,240,71,92,116,201,21,248,253,173,75,1,0,111,121,211,97,216,99,215,93,112,196,91,222,131,85,171,86,91,159,127,207,253,127,
197,59,222,125,188,126,239,75,119,222,9,199,191,255,24,156,248,145,51,245,251,93,58,244,160,87,227,149,123,188,20,11,230,111,82,201,159,159,56,227,36,0,192,7,79,60,221,123,95,95,58,251,27,0,128,99,222,
122,164,126,78,2,72,82,100,51,215,88,98,217,242,149,56,234,237,239,243,200,58,81,50,249,27,81,149,156,200,249,251,248,83,62,134,165,119,252,177,36,251,76,89,178,243,139,95,136,69,219,108,165,31,187,235,
238,123,113,205,181,63,195,117,63,187,17,121,167,185,171,174,248,30,206,255,222,197,184,196,217,175,252,172,238,185,239,235,42,175,113,219,109,182,196,33,175,221,31,59,189,240,121,250,177,123,238,255,
43,190,241,205,11,244,62,60,111,199,197,56,227,35,199,121,247,230,216,119,191,5,219,111,247,76,188,225,168,119,227,216,119,191,5,123,236,186,11,36,128,148,1,150,156,253,162,82,168,175,250,209,47,240,213,
175,157,171,230,9,53,51,150,178,23,156,122,234,137,120,254,226,103,91,186,73,168,158,38,95,57,251,27,184,242,154,235,42,101,23,0,205,199,173,200,173,156,127,182,223,238,153,165,231,127,253,155,223,227,
156,243,190,139,101,203,87,232,251,222,99,215,93,74,175,91,242,135,63,226,179,95,252,154,126,157,203,127,43,86,62,140,55,28,245,238,174,242,247,62,123,237,142,99,222,122,36,150,252,225,143,248,250,249,
223,195,95,238,190,111,160,50,164,21,217,208,104,93,242,199,92,51,56,1,192,41,128,52,193,27,223,250,110,236,244,194,231,225,168,35,94,239,213,163,231,126,229,211,184,229,182,165,90,167,92,125,233,119,
188,215,58,53,181,26,231,158,255,61,92,247,179,95,120,63,99,211,133,11,240,198,183,190,87,219,12,249,99,85,244,185,47,125,13,75,239,248,163,113,158,127,172,111,224,117,158,253,253,201,79,127,129,239,125,
255,50,205,47,230,122,188,227,189,39,104,185,105,238,243,17,71,191,183,196,95,163,128,142,14,63,244,48,236,189,247,46,120,227,91,223,131,213,83,217,125,47,88,176,0,231,157,243,89,220,117,239,95,241,222,
247,29,175,229,241,78,47,120,9,78,252,224,49,56,254,195,103,2,0,62,254,209,15,170,231,210,146,141,113,252,41,31,195,139,95,244,124,236,253,202,221,43,191,222,180,63,70,153,170,206,193,133,23,95,134,111,
95,240,3,75,247,52,179,141,183,218,226,105,61,180,211,26,185,15,148,151,93,244,206,46,139,115,23,56,75,167,192,141,216,112,161,86,68,136,96,88,140,29,71,1,216,80,133,197,251,95,186,243,78,88,182,98,37,
190,125,193,197,37,47,98,163,56,200,45,183,45,197,201,167,157,165,255,222,106,139,205,241,206,183,190,17,167,127,228,56,188,251,3,39,43,160,147,189,127,175,3,236,141,223,106,139,205,241,241,143,158,128,
195,95,119,32,190,250,181,243,75,87,119,241,15,175,196,153,167,158,136,45,183,120,26,238,189,255,1,235,189,123,189,98,55,92,248,131,31,98,106,245,84,229,231,247,206,136,167,25,121,162,206,255,206,69,248,
158,97,84,237,187,215,238,56,226,176,131,177,112,193,124,124,228,99,159,182,246,105,143,125,15,109,112,37,172,255,205,119,179,232,52,101,95,217,65,7,238,139,135,150,175,192,222,175,220,29,151,94,126,53,
150,175,124,24,75,238,184,19,159,251,210,57,120,207,49,71,227,158,251,238,195,165,234,64,109,181,197,230,234,144,125,71,63,214,51,143,158,14,9,141,155,103,207,61,135,101,122,221,65,175,198,17,135,29,132,
107,127,250,11,188,253,221,199,1,0,54,220,112,46,142,125,231,91,113,198,71,142,195,151,206,62,15,87,92,115,29,126,127,219,18,28,176,223,94,216,113,135,237,240,243,27,111,86,231,249,69,88,190,242,97,108,
189,197,211,176,104,155,45,180,241,182,227,14,219,97,106,106,53,126,119,235,18,239,119,126,242,140,147,177,96,193,38,184,248,210,43,113,197,53,215,53,224,79,219,160,246,221,215,21,215,92,135,43,174,249,
73,97,148,92,113,1,206,191,224,34,92,116,225,37,165,215,182,182,86,173,25,247,238,223,236,200,58,115,205,223,246,230,35,176,207,43,119,199,185,231,127,7,239,249,208,41,0,128,185,115,231,226,29,111,62,
2,239,57,230,205,152,90,179,26,191,254,205,239,51,229,47,37,18,201,144,165,246,239,92,177,22,208,64,231,99,31,57,30,191,254,205,239,241,154,215,31,133,85,171,86,99,195,13,231,226,148,227,222,135,51,62,
114,28,222,121,236,137,248,203,221,247,121,174,23,94,89,252,169,207,159,141,79,125,254,108,48,128,237,22,111,135,51,79,57,30,199,157,242,113,44,189,99,9,204,154,211,188,190,177,145,235,44,255,158,24,2,
15,173,120,24,111,60,250,93,198,183,9,29,173,165,10,217,85,181,87,141,228,150,105,188,127,224,196,211,44,208,248,238,99,142,198,153,167,159,128,119,188,247,120,172,90,181,26,12,96,249,202,135,241,134,
163,222,165,95,183,217,194,5,56,243,244,19,113,244,27,95,95,226,199,67,15,222,31,203,87,62,140,133,243,55,193,62,123,237,166,121,120,166,252,253,145,19,142,197,78,47,124,94,11,247,223,31,218,118,155,45,
113,204,91,143,180,100,67,126,127,103,124,228,56,205,103,141,214,229,161,229,203,173,125,250,228,25,39,3,0,222,127,226,105,70,109,90,218,144,191,125,54,130,79,111,188,77,157,167,165,119,220,137,229,10,
244,0,192,30,47,223,5,155,46,92,128,135,150,175,192,171,247,219,75,219,1,111,210,142,18,224,128,125,95,137,163,142,56,172,164,223,23,106,192,163,190,157,24,167,157,113,50,182,219,226,105,248,193,101,87,
235,125,202,249,234,75,159,57,29,199,127,248,99,74,22,178,118,16,159,121,250,73,214,26,182,34,155,135,82,227,40,5,123,251,159,150,224,144,67,246,194,226,231,110,135,95,43,125,177,211,75,95,132,135,254,
254,48,22,109,243,52,108,185,104,11,220,123,247,125,0,3,139,119,220,14,83,143,175,198,146,59,150,96,241,246,139,1,18,56,238,195,31,199,210,165,75,212,58,217,160,104,201,29,119,226,43,106,15,23,63,251,
153,56,243,212,19,113,220,41,103,96,233,29,127,28,187,120,139,123,14,246,120,249,46,56,234,136,215,99,225,130,249,248,196,231,190,108,241,90,51,219,184,55,118,26,57,160,200,163,203,169,183,118,153,0,51,
136,217,168,143,80,32,72,102,10,55,187,46,233,252,24,224,135,161,94,167,30,111,247,199,188,53,195,29,120,229,53,63,193,161,175,221,31,123,188,124,23,231,61,104,242,89,246,99,247,222,119,63,206,252,244,
23,176,217,194,5,216,115,215,157,237,112,152,243,254,185,115,230,120,30,135,254,204,165,127,184,19,183,220,182,36,243,16,26,175,217,233,5,207,199,19,230,205,195,181,215,255,162,225,231,119,255,7,26,188,
82,167,159,225,185,214,43,174,254,9,206,255,206,247,177,211,11,159,135,109,183,222,162,245,123,2,23,226,40,231,33,150,153,123,207,120,221,30,47,223,5,155,45,92,128,143,127,226,243,152,154,90,141,87,239,
187,151,126,238,39,63,253,57,206,61,255,59,56,234,136,195,176,211,11,158,143,133,243,55,198,241,239,127,23,126,126,227,77,184,244,242,107,122,190,166,4,117,30,122,190,119,53,251,65,227,61,222,108,225,
124,28,113,216,65,56,255,59,23,225,211,159,251,170,126,124,213,99,83,248,200,25,159,194,37,151,93,141,35,14,59,24,27,206,155,131,223,223,178,4,83,83,171,177,237,214,91,233,215,189,228,197,47,192,143,127,
114,3,126,253,155,223,225,165,255,185,147,126,252,197,47,120,30,174,189,254,231,222,239,220,119,175,221,177,253,118,207,196,23,190,124,46,174,184,250,39,22,127,126,236,147,95,192,170,169,41,108,56,111,
78,229,217,111,122,95,150,189,161,188,79,250,39,255,72,114,100,14,26,203,160,70,215,98,245,24,40,95,219,226,103,61,19,251,188,114,119,156,126,214,231,44,94,95,61,53,133,79,124,246,75,56,247,252,239,96,
222,156,57,133,28,150,18,72,18,176,100,72,230,172,25,65,11,103,245,93,111,59,10,75,254,112,39,62,114,198,167,176,234,177,41,189,143,31,56,225,84,92,114,217,213,216,118,155,173,252,215,235,187,31,223,99,
228,88,60,234,90,88,54,57,123,230,223,170,19,157,126,76,50,192,41,88,74,112,42,33,101,171,50,169,69,89,236,217,179,191,220,117,47,78,255,248,103,50,163,253,149,123,84,223,183,210,131,87,253,232,186,210,
153,217,115,215,93,240,131,75,175,196,146,63,220,137,3,95,189,207,140,249,123,179,133,243,241,149,207,159,137,45,183,120,26,222,249,190,19,240,189,239,95,90,11,249,145,71,79,110,185,117,137,245,248,7,
78,56,21,123,236,115,136,230,179,86,214,165,114,79,52,79,53,218,83,180,180,255,155,45,156,111,120,157,139,159,131,94,179,31,174,188,230,39,184,230,199,215,97,159,87,238,142,133,243,55,110,157,167,96,203,
148,61,246,220,29,207,219,238,153,56,254,148,51,172,125,250,203,93,247,226,237,239,250,16,150,63,252,119,28,117,196,235,172,247,254,250,55,191,195,194,77,54,198,177,239,122,75,249,251,48,26,186,134,156,
53,91,122,123,166,47,182,50,244,197,46,74,95,252,242,55,191,195,206,255,185,83,246,30,48,94,182,243,243,112,229,207,126,94,172,117,212,130,60,174,146,191,99,165,223,203,60,155,217,90,223,197,75,119,222,
9,91,109,254,180,182,108,227,238,218,105,173,238,135,114,84,247,112,157,98,102,123,226,43,51,25,105,78,141,162,62,42,50,193,210,19,229,106,29,191,21,29,133,88,255,0,192,175,110,254,45,166,166,86,43,20,
250,0,238,189,239,254,210,123,154,125,102,78,203,150,175,196,67,203,86,96,193,252,249,214,119,92,243,195,114,202,197,149,215,252,4,223,252,238,69,165,207,200,255,190,232,210,43,113,214,105,39,225,155,
223,189,72,167,213,189,114,143,151,225,130,139,47,197,212,212,148,245,218,170,207,255,202,57,223,232,170,103,127,166,165,223,230,154,228,244,187,91,110,7,0,44,218,102,43,252,249,174,123,245,243,63,185,
242,194,210,251,175,189,254,231,248,212,231,190,106,237,165,100,86,209,161,114,110,111,174,116,238,185,239,126,92,112,241,165,120,243,145,135,227,210,203,175,194,242,21,153,167,238,146,203,174,198,130,
249,243,113,210,135,222,131,187,238,186,7,83,171,86,225,172,207,124,169,247,145,54,163,129,2,48,94,121,223,238,57,116,41,79,69,189,225,23,191,242,62,127,195,141,55,225,53,251,191,10,207,125,206,98,220,
112,227,175,113,237,245,63,199,158,187,189,20,103,159,251,45,108,187,104,43,108,189,197,211,240,193,19,62,138,151,237,252,98,28,121,248,33,250,241,121,243,230,226,134,27,111,242,126,230,54,91,111,137,
229,43,30,214,188,232,242,167,239,241,170,115,91,117,95,17,179,242,1,25,187,174,141,122,242,45,20,90,201,89,127,238,14,219,123,207,191,246,243,26,215,147,255,190,253,118,207,196,212,212,106,220,116,243,
255,120,223,119,201,101,87,59,143,72,28,117,216,65,56,234,176,131,128,40,2,64,16,81,49,89,201,119,191,155,109,186,0,11,23,108,130,139,47,189,194,251,252,217,231,126,171,184,78,5,58,88,114,165,204,45,173,
55,216,208,155,70,35,5,80,91,3,86,9,192,166,11,55,193,143,174,184,16,96,64,42,231,220,215,190,253,61,92,122,233,229,128,36,72,0,135,31,118,16,254,235,176,131,74,241,189,55,190,249,93,120,104,217,138,150,
228,86,163,251,249,219,67,203,177,124,197,195,216,116,225,124,253,220,194,5,155,120,63,207,124,13,144,165,194,45,95,241,48,46,191,234,90,252,237,111,203,241,241,211,78,192,62,123,237,158,129,159,14,248,
123,193,252,249,248,210,103,62,134,121,243,230,226,139,95,253,58,254,124,215,189,181,145,31,55,252,226,38,108,187,205,86,56,225,3,239,194,9,31,120,87,230,228,64,150,234,153,223,111,171,235,226,59,115,
230,144,111,243,220,180,98,3,28,117,196,97,56,234,8,59,189,238,174,187,238,193,103,191,248,223,88,182,124,101,225,49,223,245,165,216,108,211,5,56,241,35,87,97,213,212,26,28,114,224,171,241,234,125,95,
85,210,215,85,223,95,60,78,96,102,108,181,205,150,120,104,197,195,149,251,244,203,95,253,6,71,30,126,48,230,205,155,163,223,251,151,187,239,195,207,111,252,53,78,248,224,187,241,208,178,21,248,222,247,
47,109,42,195,134,58,39,65,221,206,149,63,249,57,246,121,229,75,241,173,115,191,133,173,23,109,133,109,183,120,26,78,58,225,163,88,187,243,139,241,166,195,15,193,249,74,95,60,113,222,92,252,250,151,153,
190,96,149,127,123,230,25,199,103,121,149,58,109,142,241,208,178,21,120,211,219,222,83,29,161,31,195,97,207,62,254,89,242,135,59,0,0,91,110,241,52,220,115,223,253,109,217,198,221,177,211,168,200,206,41,
61,110,212,160,230,93,126,123,220,159,36,118,63,62,175,123,149,86,167,32,79,123,109,22,234,34,221,43,148,51,61,34,250,247,111,125,239,251,88,184,112,62,206,60,245,68,188,243,216,227,209,122,55,59,174,
120,204,126,239,43,247,63,36,11,245,109,185,5,222,245,214,55,97,213,234,213,248,202,57,231,85,166,220,0,192,210,63,220,129,91,110,91,130,61,119,219,5,151,94,126,53,22,111,247,44,108,187,205,214,248,248,
167,190,80,250,222,252,243,123,102,192,119,53,120,200,45,61,183,251,222,7,53,191,50,9,20,45,216,237,207,221,99,215,151,105,165,3,48,174,189,254,231,56,244,181,7,224,128,253,108,197,243,149,115,206,195,
115,119,88,140,205,54,219,20,71,188,229,93,125,128,39,162,30,45,250,234,163,170,58,56,95,197,57,251,253,45,183,227,53,251,191,10,219,46,218,18,47,219,121,39,44,249,195,157,88,181,106,10,55,220,248,43,
28,121,248,33,120,217,206,47,194,182,139,182,198,242,21,43,241,151,187,238,241,126,203,212,212,20,230,205,157,211,230,222,115,91,207,101,35,165,139,212,223,12,239,176,191,232,183,212,145,179,250,252,221,
114,219,18,156,244,209,143,91,143,45,222,238,89,56,235,180,147,171,211,1,60,41,123,111,63,250,72,236,179,215,158,250,239,135,150,45,183,20,253,215,190,241,77,92,122,249,143,44,224,241,218,67,94,131,163,
14,63,216,123,109,171,166,86,1,0,230,205,155,215,198,62,55,186,94,251,241,72,40,175,109,105,168,144,17,44,66,107,57,224,203,87,172,196,17,71,29,83,24,51,26,96,101,63,82,2,231,124,243,251,184,244,146,75,
74,198,51,169,220,58,110,67,110,181,166,59,24,203,87,172,196,225,111,58,70,63,187,225,134,115,241,137,51,62,140,35,15,63,4,87,92,125,173,2,158,11,177,231,110,47,197,23,191,250,245,236,60,220,118,59,150,
252,225,78,188,246,128,125,244,107,218,229,239,133,11,54,193,37,151,93,133,223,223,178,4,31,63,237,68,0,208,159,85,7,58,251,220,111,226,236,115,191,137,205,54,93,136,215,29,252,106,0,192,59,223,246,38,
188,114,207,93,241,193,19,63,138,85,171,86,183,180,46,205,236,130,118,245,252,215,190,241,109,92,122,121,230,76,120,195,235,14,194,62,123,237,137,175,127,251,2,44,253,195,157,214,59,15,62,112,63,92,121,
205,181,88,190,34,3,73,182,179,110,101,251,114,146,84,230,77,155,242,244,134,27,111,194,166,155,46,196,145,135,31,140,101,203,150,181,41,155,135,140,242,180,185,219,111,199,161,175,125,21,182,89,180,37,
94,178,243,78,184,245,15,119,98,106,213,20,110,188,241,87,56,202,208,23,15,175,88,137,123,149,190,16,96,8,1,124,224,164,51,176,244,182,37,134,189,154,118,32,207,130,110,111,223,54,238,150,157,150,215,
109,251,122,20,176,210,199,100,57,255,123,102,5,50,75,248,127,124,17,37,54,126,100,150,190,96,62,134,78,162,84,70,126,44,155,185,191,217,223,95,58,251,235,120,232,161,101,56,254,216,119,97,222,220,185,
134,247,177,245,159,5,243,55,193,102,155,46,196,242,21,43,140,174,82,197,243,247,220,123,63,62,246,169,207,99,219,109,182,198,7,223,123,76,41,210,232,126,222,69,151,94,142,67,95,123,0,152,129,151,188,
240,63,112,193,197,151,96,106,106,181,241,30,238,232,58,219,251,201,43,117,178,244,25,238,240,167,88,107,251,241,231,238,152,69,4,254,124,215,221,234,117,249,61,53,250,60,99,239,164,159,143,14,62,112,
63,0,192,55,254,251,11,248,209,101,23,226,7,223,253,58,230,205,155,139,125,246,218,19,11,230,111,98,189,246,161,101,43,240,216,170,85,214,218,246,238,71,142,249,15,87,242,2,179,212,222,234,151,238,188,
147,247,249,231,238,176,189,170,5,186,61,123,253,173,183,97,106,106,53,158,187,195,246,120,241,139,254,3,191,248,213,205,96,150,120,236,177,85,184,233,230,223,98,175,87,236,142,237,23,63,27,191,250,245,
111,43,175,233,207,119,221,131,121,243,230,226,121,59,44,246,62,255,188,29,22,99,209,54,91,130,89,98,106,106,53,230,206,157,91,122,205,162,109,182,204,128,196,67,203,90,230,125,86,233,93,165,199,117,229,
15,131,89,116,144,181,224,200,58,71,246,221,254,135,59,48,111,222,92,236,244,194,255,208,239,249,242,127,127,3,175,216,239,16,188,98,191,67,112,165,50,24,237,44,8,54,174,57,187,206,52,173,62,171,143,61,
182,10,247,220,247,0,118,216,254,217,149,235,254,186,131,15,200,94,155,131,167,13,237,117,149,44,177,197,150,91,224,255,86,60,108,61,198,44,149,1,152,121,244,244,58,74,165,43,32,75,175,175,252,81,198,
82,241,158,98,205,204,136,21,39,9,164,148,197,143,169,159,90,150,91,178,82,14,108,186,112,1,22,46,152,175,162,77,210,200,0,180,215,244,154,107,175,199,188,121,115,149,183,95,226,117,7,21,128,224,186,171,
46,194,117,87,93,132,237,183,123,22,22,46,152,143,125,94,181,71,219,252,205,12,220,190,244,78,124,245,107,231,227,119,183,222,134,47,124,245,92,188,243,109,111,170,60,143,131,252,249,219,67,15,225,147,
159,253,50,62,249,217,47,227,29,239,61,14,91,111,185,57,94,186,243,139,91,94,151,198,123,82,24,179,115,230,204,177,206,215,156,57,115,44,61,111,159,145,236,239,111,126,247,34,92,113,245,181,56,235,180,
147,177,229,22,155,235,199,119,223,245,165,216,108,211,133,216,103,175,61,241,163,203,46,196,143,46,187,16,111,62,242,112,0,192,1,251,190,202,123,142,171,109,153,236,154,239,190,231,94,108,182,96,190,
177,135,246,207,14,219,63,27,183,47,189,19,143,61,182,170,36,127,191,123,225,197,184,246,186,27,240,174,183,191,89,167,35,154,124,61,82,63,144,88,114,251,109,88,51,181,26,207,217,97,123,236,242,162,255,
192,175,27,232,11,176,4,177,196,36,24,49,128,136,50,59,40,147,21,220,134,252,29,183,140,185,242,125,111,247,236,172,145,76,22,29,106,207,54,158,185,157,70,234,71,233,86,197,11,197,79,142,41,88,239,121,
175,121,81,112,201,255,101,148,18,145,239,57,130,233,247,96,243,127,204,168,250,188,70,63,141,124,144,83,171,215,224,140,79,125,30,243,54,156,135,151,237,242,18,143,191,206,223,52,55,255,217,114,203,205,
113,194,251,223,141,135,150,45,199,143,174,251,185,247,59,25,192,178,21,15,227,123,23,95,130,151,237,242,18,28,176,255,94,13,63,243,246,165,127,196,159,239,186,7,31,122,223,49,120,217,46,47,177,62,183,
234,243,187,254,67,69,58,245,76,62,199,247,254,125,247,126,5,222,120,248,161,184,233,230,223,226,207,119,221,211,242,61,165,234,5,211,21,207,239,177,91,166,116,62,116,242,169,216,115,191,67,244,207,127,
29,253,206,76,241,236,183,119,211,253,236,199,143,116,226,91,227,242,211,104,173,255,182,108,25,206,251,246,133,56,242,240,67,240,254,247,188,67,63,62,111,195,121,248,232,73,31,196,145,135,31,130,139,
126,120,37,30,91,53,165,159,251,241,117,55,224,200,195,15,193,194,5,243,241,179,95,252,74,63,254,179,27,111,194,115,22,63,11,91,111,185,57,126,118,227,175,42,191,243,103,191,248,21,238,185,239,1,188,251,
152,163,177,239,222,175,176,248,243,236,47,126,10,111,58,242,48,205,159,183,220,190,20,91,111,185,57,222,122,212,17,250,117,155,110,186,41,14,62,112,127,44,95,177,18,127,186,251,158,206,206,168,91,58,
100,158,63,75,14,54,231,219,70,241,160,92,174,92,113,245,181,56,249,184,247,225,13,175,59,200,122,239,1,251,239,149,213,246,180,120,253,212,224,185,11,127,112,25,158,179,248,89,248,232,73,31,196,188,13,
231,233,125,124,255,123,222,129,235,175,254,1,254,182,108,57,24,192,159,239,186,7,203,87,172,196,193,7,238,143,109,22,109,173,207,197,129,135,28,136,45,183,220,28,191,91,242,135,172,179,156,115,102,204,
20,149,34,61,80,149,2,33,43,7,146,92,188,39,255,140,148,109,89,98,125,110,105,15,24,82,178,189,7,134,99,78,170,215,79,183,120,166,125,123,182,104,209,214,56,229,132,99,177,124,197,74,92,126,245,143,43,
215,125,222,134,243,176,215,158,187,101,198,237,170,41,108,186,233,166,216,115,247,151,225,188,111,95,136,93,95,117,160,245,179,124,197,74,28,116,192,190,109,243,183,123,125,151,95,245,99,156,247,237,
11,113,226,7,223,131,231,238,248,156,129,203,142,247,191,231,29,184,236,251,223,180,238,131,1,108,187,104,107,229,88,187,167,229,117,113,247,68,58,246,199,175,110,254,29,0,224,248,247,191,27,115,230,206,
209,207,29,255,254,172,91,221,109,75,255,216,240,140,252,224,242,171,240,208,178,229,56,241,131,239,209,239,63,228,192,253,241,251,91,151,88,122,105,207,253,178,136,223,62,175,218,19,11,22,108,210,146,
236,0,10,185,112,237,85,63,198,111,151,222,137,179,78,59,25,175,59,228,64,139,175,206,254,226,167,240,156,197,207,194,57,223,248,118,229,103,126,226,115,95,198,61,247,61,128,3,95,189,247,64,244,96,63,
244,141,73,87,93,119,3,142,58,252,16,108,182,96,126,150,154,221,68,95,72,59,190,96,216,171,194,43,155,49,64,155,162,142,186,125,143,221,94,138,163,143,60,28,55,40,57,212,174,109,220,115,59,141,186,103,
231,182,250,19,91,249,185,70,86,28,231,89,35,204,240,207,34,98,127,236,179,221,144,25,27,121,20,86,88,166,120,110,249,242,149,56,253,172,207,226,43,159,59,203,126,143,39,228,247,188,29,159,131,107,47,
183,243,187,175,184,250,90,156,255,157,11,177,122,245,26,231,22,236,207,185,228,135,87,99,199,237,23,227,117,175,125,13,110,95,114,39,238,189,239,129,226,94,157,215,94,124,233,229,56,235,244,83,112,206,
121,223,210,173,34,221,200,163,123,29,57,125,232,164,83,113,251,210,59,103,150,222,213,197,144,239,27,15,63,4,111,60,252,16,35,85,105,53,46,186,244,74,124,247,194,139,75,247,244,211,171,127,80,113,79,
167,227,255,150,45,7,0,28,253,134,67,240,230,67,95,107,153,72,159,249,194,87,113,200,129,251,227,138,171,175,197,237,75,236,123,95,190,124,37,206,57,239,91,56,250,141,111,192,37,151,93,169,107,137,204,
130,236,238,199,233,243,207,47,218,108,67,25,110,52,142,147,31,213,26,187,188,0,0,95,248,202,215,112,249,85,63,198,119,47,184,24,83,171,86,97,175,87,236,110,241,193,237,75,239,192,135,78,58,29,191,191,
245,54,235,125,191,187,229,118,28,248,234,189,113,211,205,191,197,170,199,86,233,199,127,127,203,109,58,5,229,47,127,185,187,225,101,189,229,152,99,241,182,55,31,129,55,30,126,40,222,245,182,163,244,227,
63,248,225,149,186,3,20,0,92,126,229,143,48,111,238,60,236,181,231,203,181,1,1,0,55,221,252,91,124,244,220,111,89,223,95,214,140,70,231,26,159,230,108,84,50,196,190,97,129,21,124,235,43,234,117,100,221,
151,255,251,60,44,95,177,2,47,125,201,78,120,221,193,7,88,169,114,63,191,241,38,124,243,187,23,149,191,223,76,37,99,243,235,253,41,6,55,252,252,151,88,245,216,20,14,61,104,127,92,118,97,177,134,247,220,
247,64,105,31,63,112,194,169,120,203,81,111,192,151,62,123,38,164,90,131,191,45,95,137,179,207,187,16,151,92,124,177,247,72,81,254,0,187,233,214,100,221,47,235,52,191,92,225,80,113,254,242,244,85,201,
158,140,142,60,77,91,226,232,35,15,198,209,71,28,10,55,117,251,247,183,46,197,137,31,57,13,211,107,36,146,148,241,227,203,179,117,19,66,100,237,187,213,235,142,211,247,203,216,110,241,179,241,227,43,127,
160,94,151,93,210,181,215,221,128,175,125,253,124,172,90,53,101,164,174,205,47,201,193,219,151,222,129,79,127,254,108,128,25,175,63,248,0,44,95,177,18,223,53,58,164,230,244,217,47,158,131,179,78,63,9,
251,189,106,79,92,126,213,143,91,230,111,31,79,125,247,130,139,1,6,206,58,237,36,125,70,7,69,159,252,236,151,48,53,85,190,143,156,167,254,242,151,187,241,129,247,30,211,242,186,152,247,204,78,65,252,242,
229,15,227,67,39,157,138,55,189,225,117,184,244,123,69,26,207,93,119,221,131,15,157,116,42,238,189,247,126,239,25,207,105,245,212,106,124,254,203,231,224,172,211,79,193,17,175,63,24,247,220,123,31,54,
219,116,33,142,63,229,244,210,153,61,255,59,23,226,229,187,188,4,175,217,111,111,124,249,191,207,107,104,63,20,231,57,81,141,79,8,39,30,119,10,14,63,228,181,216,107,207,151,91,114,245,199,215,221,128,
143,158,241,41,60,148,167,196,121,100,1,0,124,228,140,79,224,236,47,124,18,11,23,204,247,52,89,24,254,108,57,115,57,111,187,229,118,28,250,234,189,113,227,205,191,197,84,43,250,66,189,241,19,167,157,164,
29,40,153,7,36,123,226,156,243,190,133,75,46,187,172,90,254,142,25,29,125,228,27,112,244,145,111,176,236,188,239,93,116,137,93,155,218,182,109,220,69,59,141,109,53,225,27,252,211,115,158,220,232,201,255,
86,168,77,247,134,72,53,77,176,46,41,212,89,12,150,132,230,18,183,5,109,29,178,83,217,18,56,117,229,21,167,189,163,42,156,163,194,46,3,5,70,27,63,76,232,138,192,146,51,200,144,139,85,142,157,150,100,100,
47,26,135,22,2,193,188,60,234,246,185,110,229,88,1,86,119,78,11,16,17,26,43,207,82,30,57,219,143,55,188,16,110,113,157,242,109,37,11,143,21,35,2,200,186,142,186,201,217,177,59,147,37,95,195,144,212,128,
80,134,188,69,216,198,214,246,216,17,37,62,171,179,209,25,148,165,15,27,231,90,161,33,129,196,68,101,21,75,229,253,239,23,197,133,199,205,195,60,102,231,165,150,25,43,204,101,238,41,24,82,125,216,81,67,
163,157,75,86,101,221,1,145,114,103,83,225,73,239,21,24,98,4,131,106,40,68,180,213,12,77,182,102,91,123,59,108,114,59,167,102,230,188,172,26,65,244,84,145,228,55,203,85,11,215,224,61,249,9,96,106,176,
182,228,200,14,179,19,3,149,239,151,125,42,167,133,2,118,46,244,91,229,222,113,241,93,12,178,110,47,156,227,1,232,148,97,3,67,181,214,127,227,178,114,1,16,213,126,151,217,209,88,3,22,174,177,221,66,210,
211,77,46,8,129,26,25,241,92,123,204,201,146,135,128,23,60,235,200,176,156,199,51,17,226,85,241,212,0,138,234,203,13,214,158,85,165,196,148,17,83,249,101,140,138,222,160,51,137,106,52,145,9,228,130,137,
222,24,38,122,28,3,115,3,32,131,6,17,52,207,96,239,134,107,193,21,167,203,121,127,163,253,154,145,174,42,56,195,12,116,5,143,127,255,206,37,251,67,68,67,101,214,203,32,247,91,131,45,30,127,60,53,145,213,
13,197,76,0,68,195,163,125,153,236,234,133,1,217,75,113,49,74,147,103,8,132,2,245,22,12,65,71,50,168,246,12,62,68,128,8,152,113,196,173,148,106,69,126,99,153,220,199,67,42,78,109,65,81,171,60,236,195,
6,222,77,101,243,91,186,120,70,8,86,75,82,234,249,73,230,38,192,133,80,46,196,227,46,222,179,153,104,206,109,237,85,23,37,113,160,0,134,90,184,9,214,134,125,224,157,38,114,166,162,131,58,183,168,151,185,
49,212,10,52,92,150,238,192,124,254,113,217,163,31,24,168,206,204,194,53,141,14,217,89,13,117,87,96,10,240,51,205,248,224,249,202,165,50,5,232,120,175,137,116,240,192,85,2,161,102,169,70,188,11,95,57,
16,87,28,70,88,129,11,106,244,150,202,154,186,46,128,3,118,74,108,186,121,74,184,85,235,197,115,191,228,156,174,158,20,50,247,225,243,153,116,189,17,115,136,244,246,31,12,13,171,77,34,181,69,207,3,172,
139,24,34,236,216,21,249,61,252,124,51,230,138,120,128,214,99,92,20,146,6,170,59,26,98,174,167,64,181,231,242,14,67,132,168,124,121,212,230,219,184,210,57,205,13,179,163,216,163,5,216,104,234,16,104,48,
142,134,178,66,173,96,10,99,227,173,58,80,110,162,236,43,157,4,93,74,153,227,246,107,92,154,142,232,227,70,22,11,55,189,172,210,107,135,18,16,145,54,108,217,104,90,17,64,81,143,128,144,245,139,28,129,
59,10,48,168,35,67,130,102,40,30,57,0,162,97,221,255,65,158,150,208,1,33,80,160,64,129,2,5,10,20,40,80,160,64,99,75,161,62,52,80,160,64,129,2,5,10,20,40,80,160,64,1,16,5,10,20,40,80,160,64,129,2,5,10,
20,40,208,184,81,252,148,3,47,8,171,48,12,20,205,2,98,170,117,1,190,85,70,195,0,18,6,100,2,53,67,186,102,68,0,162,236,215,137,24,20,217,55,145,93,127,98,204,93,137,96,103,151,38,198,99,249,253,9,245,88,
130,142,51,81,105,50,251,103,150,8,153,231,67,78,121,233,16,81,185,7,3,18,0,137,84,188,147,26,60,169,106,86,42,73,0,136,129,217,66,191,3,77,184,173,81,21,67,233,154,210,84,61,144,95,83,187,117,28,194,
115,69,230,239,230,57,170,122,141,58,71,19,17,40,42,223,35,167,234,143,36,49,158,233,181,140,81,2,98,114,34,91,211,98,62,118,160,54,73,50,128,117,185,94,24,135,140,253,73,96,182,8,213,68,51,176,41,90,
237,52,167,185,105,29,212,160,229,4,161,107,114,157,73,100,63,34,202,58,143,76,14,182,201,84,28,54,100,72,40,77,129,184,222,219,165,13,23,82,163,88,98,2,214,215,22,97,26,98,84,130,19,225,88,93,112,134,
80,38,30,3,46,113,85,61,102,214,221,38,210,104,140,19,128,194,233,28,73,34,0,28,43,118,145,104,67,229,151,165,182,183,184,63,103,205,188,243,92,108,191,142,124,103,22,0,82,213,21,171,196,215,109,153,187,
74,201,17,252,147,83,83,3,20,249,74,88,163,236,71,160,112,82,152,183,38,115,32,105,130,32,245,158,126,56,94,82,153,57,124,98,161,65,81,160,214,57,3,211,0,210,100,134,60,54,116,202,27,72,4,56,14,128,168,
83,155,98,102,112,42,80,189,165,2,103,94,18,138,128,116,176,231,36,152,92,99,35,28,250,108,240,9,12,143,99,38,1,16,165,21,125,151,219,53,178,102,216,70,153,131,16,31,169,243,218,232,208,198,2,88,159,251,
141,211,246,78,120,202,64,68,126,246,91,151,218,87,145,170,239,138,154,124,195,132,200,192,134,140,187,12,138,184,108,32,150,64,145,249,47,233,182,125,222,174,63,156,84,172,118,47,64,145,17,73,142,35,
88,189,243,3,181,44,17,89,131,161,181,227,41,9,82,6,226,0,135,122,201,99,101,35,36,69,136,14,5,106,198,51,166,230,9,128,104,152,54,48,143,26,12,157,92,237,147,247,182,125,20,84,44,108,2,84,167,243,12,
64,133,70,129,223,71,9,24,153,45,133,153,220,163,17,163,181,20,53,81,0,28,228,134,57,101,206,135,156,101,83,15,144,103,2,166,1,70,11,160,168,114,178,108,39,160,168,33,162,115,238,43,87,73,42,18,155,198,
89,20,205,157,175,148,86,129,168,94,156,87,81,254,252,124,24,92,104,147,223,1,24,74,198,116,21,82,21,18,206,78,69,72,157,235,49,40,10,56,104,72,181,100,134,72,168,79,188,194,211,74,31,11,6,4,1,162,45,
64,20,58,116,15,220,128,79,81,251,180,185,178,77,17,171,58,162,58,130,162,20,118,95,145,164,22,96,40,208,104,41,104,107,190,74,106,88,68,169,204,126,154,130,33,178,205,40,78,179,115,149,56,211,30,83,23,
212,56,159,151,18,216,19,85,210,35,95,82,86,111,25,68,221,159,9,110,212,119,39,113,150,118,203,198,34,18,121,94,223,171,115,43,13,185,16,27,245,85,67,40,135,7,5,134,114,190,74,83,140,87,154,156,71,215,
200,72,171,155,86,230,88,181,59,87,44,152,212,106,205,72,169,117,29,233,158,9,58,170,163,221,50,170,27,216,159,28,100,41,81,46,7,135,4,19,183,3,136,130,145,56,248,227,78,224,117,41,48,59,170,189,144,212,
181,9,17,1,28,169,240,117,29,243,232,100,141,64,191,68,30,181,226,4,160,137,192,245,67,109,12,114,49,196,19,18,192,250,4,101,4,147,182,113,154,76,86,49,26,49,232,95,169,130,175,115,131,44,6,166,57,3,25,
214,33,133,170,245,75,93,100,213,107,233,224,1,55,6,40,74,1,164,145,10,220,178,129,42,169,201,191,189,216,205,196,104,174,226,44,117,160,50,239,67,241,148,156,14,6,101,238,9,89,31,1,179,116,128,177,181,
53,108,44,5,2,53,36,209,165,85,11,160,168,183,123,20,1,145,208,9,59,189,228,115,206,235,107,89,2,66,24,136,136,1,142,66,202,220,112,81,230,169,228,181,195,1,138,32,0,74,179,2,228,140,231,18,148,187,80,
213,9,104,115,77,246,56,10,254,135,33,54,125,200,229,168,220,75,174,163,25,205,58,201,53,3,69,21,237,17,184,25,200,90,151,165,231,165,0,16,103,249,1,137,201,119,105,5,96,233,37,48,242,117,136,51,235,170,
210,12,20,129,149,97,210,8,12,201,30,239,172,50,176,38,162,96,156,6,48,212,38,73,0,211,192,186,9,96,182,63,250,99,129,32,227,232,153,41,182,84,161,173,104,204,121,174,122,205,3,160,169,61,197,153,211,
171,191,221,229,200,107,123,5,64,52,172,160,104,8,186,144,17,178,224,16,165,0,34,1,198,164,45,197,210,113,106,189,218,230,218,133,147,57,84,6,32,65,121,159,164,147,182,145,42,48,36,83,131,215,185,131,
147,228,254,110,158,27,225,24,0,205,228,135,122,127,210,200,188,160,6,230,87,183,97,99,51,19,199,76,41,204,91,219,187,233,114,253,160,234,238,119,129,12,10,96,168,49,111,167,84,242,121,177,123,68,19,155,
237,16,219,192,136,91,52,241,198,65,238,122,215,68,194,225,191,200,145,37,173,82,224,225,158,202,83,35,226,222,15,254,37,213,57,129,153,10,117,73,177,42,40,10,77,21,28,81,210,107,47,99,111,60,35,84,247,
149,53,66,161,108,92,56,115,156,61,41,199,61,191,220,220,69,37,32,66,75,223,218,159,53,134,10,122,146,50,86,82,0,130,85,112,131,128,245,110,125,80,167,160,34,245,240,9,187,218,191,131,59,18,45,60,46,123,
36,97,58,93,11,57,56,25,29,81,150,218,17,168,241,174,82,168,104,175,62,199,148,69,105,149,195,132,125,47,73,129,172,11,69,254,88,4,72,145,165,219,249,142,15,149,1,2,249,95,54,82,96,168,51,41,50,211,14,
154,129,186,67,113,182,123,106,172,94,95,121,83,168,239,19,26,7,25,87,69,234,106,164,219,246,213,128,108,60,202,40,153,134,144,145,148,16,52,198,112,12,83,75,110,125,177,147,0,75,2,146,72,53,94,8,30,147,
236,192,70,45,21,221,6,234,31,16,242,85,168,240,52,128,116,58,235,80,67,148,129,160,52,151,163,105,143,28,44,221,140,212,72,180,22,93,10,17,220,190,36,184,143,2,73,192,106,140,17,168,172,1,101,2,172,143,
181,49,88,2,68,58,196,108,232,68,78,179,116,187,89,40,71,144,226,226,136,178,250,155,169,172,119,235,162,83,76,64,211,110,199,61,111,124,221,205,244,77,224,72,237,0,138,234,67,147,208,233,207,145,89,199,
51,0,139,159,84,243,71,6,48,29,3,152,70,76,19,177,58,111,4,50,65,16,17,56,73,139,54,163,35,163,20,205,208,169,217,195,58,45,224,99,237,162,68,177,115,141,0,146,20,136,162,2,24,69,195,167,171,53,74,215,
251,50,206,74,148,53,24,162,136,130,221,85,39,48,148,167,194,89,105,112,80,105,65,100,56,147,228,16,2,9,217,224,49,129,246,210,241,70,212,128,5,3,20,131,38,194,169,108,122,94,82,24,78,129,0,164,203,164,
58,22,34,85,141,86,124,249,151,110,198,132,178,85,152,179,185,101,102,154,5,18,199,182,207,134,91,106,51,71,153,56,131,142,20,113,131,223,221,232,59,181,10,134,242,101,34,143,169,20,168,134,52,43,139,
178,235,172,103,49,240,241,5,89,38,14,1,130,192,211,49,226,60,53,135,132,231,96,82,12,78,71,169,198,35,234,208,47,81,7,48,4,219,253,193,170,13,172,22,22,209,80,130,162,108,91,204,78,116,193,4,15,84,163,
29,97,195,200,19,66,53,72,32,128,215,163,183,45,159,235,2,150,162,49,229,75,97,220,119,172,135,106,6,72,212,68,122,145,219,57,48,80,121,149,166,13,71,67,106,216,37,142,142,47,129,164,42,191,68,90,254,
59,141,80,74,137,141,160,103,150,13,26,20,233,123,49,80,80,91,160,45,7,67,38,74,106,43,216,16,248,179,143,176,163,208,35,10,16,209,160,152,208,115,101,250,151,201,102,93,230,8,35,216,95,148,212,108,28,
99,57,162,40,51,114,180,187,97,24,162,21,137,125,158,167,179,122,6,142,35,136,104,184,142,10,162,124,248,124,29,163,115,253,183,192,25,33,66,84,159,99,198,217,224,2,202,219,91,155,169,197,227,2,224,71,
253,76,18,202,67,153,85,46,69,20,33,228,175,182,104,236,178,114,28,132,70,57,109,158,41,217,226,10,39,30,184,192,13,192,81,14,140,12,180,64,173,181,253,238,54,111,112,213,173,160,240,57,184,67,152,103,
126,141,113,15,100,116,85,186,93,175,26,208,12,59,169,112,16,9,96,178,222,3,173,85,191,133,6,47,160,114,46,234,112,83,90,220,178,240,85,114,213,241,102,19,103,155,184,1,64,138,129,36,5,71,195,213,22,54,
235,70,39,128,196,72,134,214,238,159,113,181,46,2,213,99,27,200,57,139,210,54,48,70,158,70,29,244,229,249,27,249,150,70,42,98,157,117,147,35,10,3,50,243,179,208,48,149,73,165,149,102,6,67,240,190,15,94,
65,120,206,109,42,180,213,55,48,140,159,131,161,84,221,67,68,109,221,97,83,206,178,82,232,68,27,50,172,81,99,45,51,255,48,255,219,151,175,23,168,188,110,2,16,145,119,36,84,206,131,117,90,193,230,25,151,
35,23,33,82,0,35,118,118,98,40,244,62,55,151,52,60,156,73,180,34,206,64,17,235,174,192,62,55,210,24,24,159,60,78,198,246,16,145,244,41,207,113,160,81,55,110,99,32,230,162,26,61,42,215,249,134,25,47,10,
244,72,219,130,209,61,151,242,180,210,68,23,215,5,121,81,43,135,134,145,241,146,136,193,2,162,14,120,207,27,123,201,49,202,180,131,125,92,123,60,245,13,103,205,215,196,4,52,177,251,70,231,113,147,233,
85,68,89,219,90,46,48,26,71,103,110,30,97,55,103,63,25,3,87,61,169,154,117,44,92,105,106,61,143,94,18,19,233,157,32,106,207,231,210,63,198,114,17,90,123,192,128,125,105,201,67,160,220,137,0,76,40,197,
155,15,143,28,59,80,20,60,171,181,217,5,9,53,43,203,24,18,58,118,233,114,163,204,143,145,145,22,46,178,249,66,52,222,60,111,182,107,102,211,25,144,112,145,102,174,244,39,79,24,71,33,128,161,33,1,69,100,
205,65,234,59,187,43,29,95,229,104,247,181,11,111,120,157,194,177,100,83,199,140,242,90,61,249,90,196,21,182,151,249,30,81,126,25,1,72,138,115,0,233,51,163,199,21,20,25,67,180,99,178,250,242,208,16,216,
162,205,195,9,130,128,120,2,72,166,71,96,195,226,44,20,17,123,206,163,229,8,77,7,196,76,0,162,216,184,132,78,174,35,81,229,69,17,200,215,167,32,26,130,185,69,2,96,61,44,43,114,68,228,232,43,220,48,248,
113,192,134,33,231,115,133,216,0,67,178,21,245,28,104,56,78,88,182,143,34,86,226,133,244,244,137,177,118,0,160,168,115,215,170,112,189,4,164,233,16,64,118,56,152,178,20,103,2,144,202,0,134,134,2,20,229,
191,78,104,181,218,143,1,175,228,227,181,132,203,175,138,155,67,8,31,96,66,12,59,59,46,110,240,198,170,242,54,179,240,154,12,164,101,100,18,89,145,42,51,107,78,56,58,65,230,209,167,49,2,66,100,116,236,
16,48,103,174,150,214,174,206,114,182,165,252,42,138,163,172,5,247,80,11,188,24,152,140,65,194,191,33,28,139,12,245,115,210,57,83,20,238,131,246,223,43,34,80,108,20,62,34,238,0,20,229,195,8,20,40,226,
178,101,205,42,99,176,110,76,89,54,51,89,77,16,54,107,138,42,69,226,136,80,38,73,40,216,219,3,35,169,187,202,73,35,84,233,242,90,48,252,134,159,38,128,9,202,154,162,137,0,134,244,191,108,12,25,78,61,96,
72,199,142,34,163,203,105,56,15,67,5,140,100,12,72,242,118,156,235,91,189,156,47,19,215,40,151,166,78,192,80,110,134,85,181,225,78,161,236,9,199,150,80,78,114,175,58,110,116,249,58,153,167,209,172,163,
113,136,18,197,26,208,90,147,26,60,190,67,26,130,59,105,141,119,5,13,161,220,139,50,163,58,206,194,155,141,102,64,81,212,10,22,50,179,30,61,139,81,138,238,184,73,8,58,198,106,159,54,19,12,229,239,136,
73,129,162,78,0,22,103,55,147,86,136,183,184,62,33,8,54,180,49,155,251,19,83,49,160,142,145,77,247,214,202,120,132,211,149,226,16,30,26,180,97,136,52,53,243,54,155,104,241,64,195,231,116,24,18,237,220,
39,210,101,228,102,139,121,54,135,11,167,30,83,52,204,26,26,222,221,166,2,124,112,102,216,247,122,39,75,96,38,166,50,75,57,96,168,233,241,140,252,199,219,2,120,165,47,118,122,137,17,101,179,66,91,176,
248,92,93,161,59,228,70,134,73,152,192,7,219,198,71,129,138,198,251,63,12,208,174,53,134,158,136,193,235,58,49,206,7,121,246,99,208,172,214,90,24,19,0,142,41,51,188,101,85,157,128,145,107,199,73,25,20,
89,136,216,133,199,228,87,202,34,6,98,225,103,160,152,192,169,242,198,181,13,2,216,40,38,26,142,195,201,18,69,119,167,200,88,102,160,64,179,58,185,61,25,77,137,18,236,139,193,82,2,117,110,130,181,60,186,
6,33,23,129,252,56,236,116,201,250,99,55,42,84,101,46,7,97,53,180,66,78,198,133,207,54,234,223,224,86,205,73,81,123,0,164,145,129,205,77,222,95,138,40,229,245,63,49,154,214,90,83,11,0,79,255,46,170,174,
104,212,207,137,177,10,18,67,221,19,42,110,143,149,235,40,193,243,219,48,83,250,162,2,191,180,241,105,34,2,56,34,112,46,44,210,170,227,103,20,220,153,30,180,252,225,40,242,167,187,81,220,210,154,50,27,
207,68,177,42,236,238,16,20,105,224,80,124,55,39,108,181,187,28,212,144,44,43,58,132,124,106,176,231,22,220,50,162,233,17,206,209,77,85,219,244,208,104,174,166,130,63,208,72,88,254,68,89,36,36,68,100,
11,174,214,205,178,90,1,67,129,134,119,183,213,222,166,241,64,140,216,124,228,49,119,1,12,181,108,186,184,67,102,226,214,0,79,203,0,175,116,152,34,247,80,141,240,57,74,51,99,46,141,10,123,205,140,154,
13,145,45,211,50,32,226,52,65,189,82,149,12,14,143,200,6,27,42,138,67,17,117,244,169,164,242,32,121,34,46,2,45,6,158,225,20,69,65,41,12,28,150,38,69,218,92,20,103,215,151,26,7,162,148,83,169,30,72,36,
56,22,168,236,200,63,35,80,228,238,120,164,238,129,85,153,206,224,129,110,14,134,74,94,31,85,70,100,62,193,201,40,43,231,12,192,114,18,129,38,131,234,30,152,68,148,177,17,37,10,198,224,200,41,111,196,
128,76,1,17,141,125,139,12,203,48,213,13,18,218,53,11,3,13,39,40,82,237,163,83,3,169,160,63,109,99,220,105,85,212,107,222,118,100,124,163,72,18,205,68,180,152,31,96,213,112,39,21,119,60,10,250,37,85,204,
147,2,82,168,166,43,198,6,76,14,207,44,183,184,85,198,242,23,142,213,7,27,81,37,160,152,217,161,5,149,3,59,20,1,50,169,144,28,10,20,233,168,75,28,23,107,152,164,89,244,168,69,23,71,150,163,154,33,5,78,
187,33,30,160,129,162,5,44,6,185,181,236,143,12,233,204,141,74,234,2,56,172,43,201,4,44,227,16,37,26,128,139,5,164,26,172,76,135,66,241,209,223,109,26,250,20,143,110,173,68,94,174,153,165,139,19,70,223,
171,29,64,81,145,101,49,184,214,179,125,73,207,67,239,186,156,177,247,148,184,205,235,27,157,165,170,228,191,97,35,131,151,152,129,100,162,148,142,56,12,206,167,214,34,68,105,106,88,168,117,216,52,21,
29,138,122,7,134,154,94,193,44,145,129,148,84,170,150,141,164,80,146,177,253,17,233,73,231,148,0,92,149,158,193,217,26,179,121,64,212,61,17,140,212,178,46,158,224,60,50,68,131,222,70,170,62,48,150,88,
73,141,133,136,144,133,103,245,112,181,81,51,94,243,40,209,232,123,104,235,38,36,155,3,241,64,163,133,128,67,35,117,45,103,169,145,73,233,251,59,208,240,75,95,88,193,162,81,59,8,45,55,104,232,198,173,
179,177,134,58,32,148,119,155,115,135,189,154,182,96,138,225,47,1,112,187,1,51,144,78,234,150,228,195,34,53,226,150,185,170,22,189,128,13,32,4,0,113,4,49,64,207,6,69,128,36,1,36,30,131,60,38,189,100,249,
185,160,196,163,78,146,124,182,67,106,247,194,207,73,0,36,1,142,242,54,167,205,114,222,125,81,19,227,61,105,2,70,4,17,15,190,215,108,171,62,200,76,13,179,181,182,186,6,106,90,121,36,70,70,81,243,80,121,
84,70,17,20,101,71,70,53,87,241,118,155,11,52,244,102,210,68,84,204,101,13,96,200,150,173,169,106,30,36,125,233,115,163,55,170,125,188,73,57,187,147,168,185,105,49,10,252,221,195,239,225,216,81,106,41,
229,221,177,12,213,30,23,199,200,180,207,100,52,34,192,40,39,9,96,29,32,39,11,60,56,4,130,54,102,48,154,245,97,35,17,1,36,193,152,200,82,194,40,157,161,253,73,142,25,68,142,144,141,202,175,23,0,98,59,
141,168,22,16,77,228,51,140,84,164,136,200,139,31,45,103,65,190,118,105,90,238,245,29,137,162,145,157,1,138,4,68,54,35,69,118,201,56,27,50,107,91,215,131,169,235,206,215,72,198,17,48,205,24,185,38,11,
50,1,210,24,28,209,200,26,108,181,133,176,121,175,20,25,1,211,20,0,81,48,147,198,99,85,72,233,40,1,21,129,151,133,1,7,179,13,119,160,209,19,196,105,182,231,113,88,142,78,44,89,87,170,112,108,28,29,144,
194,58,19,213,38,46,48,130,3,93,57,3,69,201,100,214,130,111,162,254,87,27,35,77,139,70,0,77,164,37,77,0,16,49,56,161,138,112,71,59,202,136,140,239,37,213,52,64,253,110,226,164,40,214,198,112,93,251,220,
145,0,120,82,20,152,174,149,11,77,156,38,21,20,103,245,69,80,93,230,156,233,200,44,0,17,11,176,39,225,157,19,217,2,80,82,155,37,98,80,36,106,155,55,207,108,179,128,203,53,76,227,162,146,83,48,71,24,85,
56,196,14,159,215,9,159,235,90,162,18,74,10,158,241,225,167,168,164,102,198,205,246,109,58,244,146,76,132,164,181,92,0,67,35,73,10,244,18,103,193,162,0,136,102,236,94,113,93,253,128,234,175,16,53,180,
204,148,82,28,225,218,232,33,160,56,107,166,148,102,81,160,86,54,61,202,210,214,56,77,139,86,96,237,178,142,136,117,246,27,229,92,50,49,105,8,101,163,1,192,144,8,97,202,109,166,38,218,168,229,44,108,46,
31,32,246,101,185,73,0,177,80,99,145,146,138,139,48,192,208,0,234,173,218,210,214,234,158,124,247,234,85,230,60,176,221,238,253,5,24,147,187,71,201,32,3,84,38,154,148,96,82,225,62,195,222,162,26,156,101,
54,145,81,48,4,71,196,108,49,124,185,169,4,67,100,206,172,6,167,154,70,112,21,90,126,161,46,168,35,223,19,129,165,70,144,51,24,99,223,99,164,39,103,172,217,185,227,8,217,124,36,109,244,141,24,40,226,225,
216,183,54,39,245,84,89,44,194,126,13,193,211,22,141,178,148,178,152,64,84,88,250,190,210,164,81,245,138,19,169,169,208,41,192,113,92,212,15,1,89,234,92,2,32,142,244,4,228,150,58,163,168,26,163,12,20,
197,78,99,7,243,117,5,24,202,7,136,81,221,4,8,217,30,202,86,206,16,197,253,110,197,45,80,213,91,166,23,136,136,147,108,223,70,66,38,42,68,203,41,178,136,38,81,81,124,42,85,83,17,18,224,9,210,19,199,3,
20,9,212,85,73,163,187,220,100,122,138,101,209,229,210,215,62,96,44,27,46,228,7,47,142,179,180,238,52,87,94,228,1,151,129,70,205,138,31,247,38,35,131,58,110,60,11,224,181,185,174,207,109,228,245,35,112,
135,73,214,142,123,8,188,187,255,127,0,72,205,111,51,191,82,219,72,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00128_png = (const char*) resource_NewJucerComponent_bmp00128_png;
const int TalComponent::bmp00128_pngSize = 35135;
































































// JUCER_RESOURCE: bmp00129_png, 113953, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00129.png"
static const unsigned char resource_NewJucerComponent_bmp00129_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,88,0,0,24,16,8,6,0,0,0,198,175,106,156,0,0,0,9,112,72,89,115,0,0,11,19,0,0,
11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,1,178,60,73,68,65,84,120,218,236,221,121,152,92,87,121,39,254,239,123,206,189,183,214,
238,234,238,234,69,173,214,214,90,188,10,35,121,199,193,24,204,22,182,49,152,176,101,66,18,8,67,32,147,76,146,223,144,73,50,89,38,206,50,201,176,100,66,32,24,12,12,201,64,8,4,18,226,0,99,3,70,96,27,27,
140,109,121,147,100,203,218,213,146,90,173,86,239,93,235,189,231,188,191,63,238,173,234,234,86,47,213,173,46,187,173,126,95,63,245,72,86,111,85,223,122,251,220,115,78,213,189,31,98,102,72,53,174,148,68,
32,1,75,192,82,18,176,4,44,1,75,73,192,18,240,42,44,103,165,222,177,91,110,185,133,0,172,1,176,17,64,39,128,22,0,241,232,195,69,0,163,0,206,0,56,6,224,244,157,119,222,185,34,87,76,180,18,86,114,183,220,
114,75,2,64,2,192,101,0,94,1,224,165,0,174,6,208,86,231,183,24,6,240,8,128,31,1,248,1,128,125,0,10,119,222,121,103,97,213,6,28,133,186,13,192,139,1,220,28,133,218,11,64,207,242,233,12,32,136,110,149,223,
60,7,0,205,242,185,6,192,145,40,236,93,0,158,0,112,224,249,10,251,121,9,248,150,91,110,121,53,128,183,0,120,9,128,237,51,134,42,6,176,31,192,129,40,168,83,0,134,0,228,103,4,156,4,144,5,176,54,122,98,182,
1,184,120,70,232,1,128,61,0,126,12,224,27,119,222,121,231,247,46,232,128,111,185,229,150,171,1,252,17,128,151,1,72,215,4,91,6,240,83,0,255,6,224,91,3,3,3,185,147,39,79,94,217,218,218,186,19,192,165,214,
218,94,165,84,39,128,84,244,249,57,107,237,25,165,212,17,0,79,143,140,140,60,214,211,211,179,187,171,171,43,5,224,141,0,222,12,224,90,0,94,77,208,147,0,238,3,240,103,119,222,121,231,35,23,76,192,209,193,
106,29,128,15,0,120,63,128,246,168,75,115,0,250,173,181,255,58,56,56,248,131,124,62,239,12,13,13,221,212,218,218,122,163,82,106,39,128,216,34,127,84,201,90,251,216,200,200,200,253,217,108,246,222,100,
50,25,116,116,116,188,66,41,117,43,128,238,232,201,33,0,103,1,220,1,224,211,0,78,52,250,224,216,208,128,111,185,229,150,22,0,175,6,240,219,209,112,0,0,133,168,147,190,113,228,200,145,35,249,124,254,237,
142,227,188,220,243,188,238,232,215,126,57,42,95,46,151,251,131,32,248,97,50,153,252,231,222,222,222,222,104,72,122,89,116,48,69,52,108,252,111,0,223,187,243,206,59,71,95,112,1,223,114,203,45,107,0,252,
1,128,119,70,93,11,0,187,131,32,248,155,161,161,161,83,163,163,163,191,232,121,222,219,106,30,112,163,170,80,46,151,191,214,210,210,242,127,179,217,236,90,199,113,126,11,192,149,209,199,206,2,248,10,128,
191,184,243,206,59,79,191,96,2,190,229,150,91,60,0,119,70,221,171,1,248,0,62,119,224,192,129,47,91,107,127,193,113,156,91,149,82,29,207,229,193,198,90,59,24,4,193,191,42,165,190,180,109,219,182,159,7,
240,62,0,110,52,235,248,30,128,91,238,188,243,206,242,138,14,248,150,91,110,113,0,236,0,240,121,0,87,68,119,254,120,16,4,127,218,215,215,55,12,224,171,53,139,133,231,171,138,0,222,177,126,253,250,54,199,
113,254,24,192,134,168,9,158,4,240,43,0,30,191,243,206,59,131,21,183,84,142,194,125,101,116,0,185,2,64,9,192,119,70,71,71,127,231,241,199,31,127,41,128,47,174,128,112,17,221,135,47,62,254,248,227,47,29,
29,29,253,29,0,223,137,238,235,21,209,125,127,101,244,88,86,220,82,121,7,128,191,140,238,40,3,248,218,248,248,248,39,7,7,7,63,158,205,102,175,157,99,81,240,124,85,115,54,155,125,239,224,224,224,118,165,
212,111,54,55,55,15,3,248,143,209,125,255,203,104,182,243,200,138,25,34,162,49,247,225,154,112,255,237,200,145,35,127,2,224,115,0,174,89,236,247,243,60,15,29,29,29,232,233,233,193,197,23,95,140,158,158,
30,52,53,53,65,41,133,114,185,140,161,161,33,12,14,14,226,228,201,147,56,118,236,24,134,135,135,145,203,229,96,173,93,202,221,127,24,192,251,122,123,123,255,36,154,63,83,52,92,92,179,28,99,242,121,7,28,
205,22,190,0,224,103,163,95,181,175,29,57,114,228,35,0,190,12,224,242,186,199,42,165,208,221,221,141,119,188,227,29,120,217,203,94,134,201,201,73,244,245,245,161,191,191,31,253,253,253,24,31,31,71,46,
151,67,16,4,80,74,193,113,28,56,142,131,68,34,1,199,113,80,44,22,113,248,240,97,244,245,245,161,88,44,46,246,97,236,5,240,243,189,189,189,191,3,224,109,209,28,252,110,0,239,57,223,217,197,121,5,28,205,
115,255,12,192,7,163,127,250,206,248,248,248,159,14,13,13,125,98,49,157,187,109,219,54,124,228,35,31,65,44,22,195,238,221,187,177,123,247,110,236,217,179,7,167,78,157,130,49,102,206,206,180,214,66,41,
133,68,34,129,116,58,141,100,50,137,88,44,134,92,46,135,190,190,62,148,203,139,106,192,135,179,217,236,111,52,55,55,255,49,128,215,70,255,118,59,128,63,58,159,121,242,146,3,142,86,104,63,7,224,83,209,
60,247,200,232,232,232,239,12,14,14,254,142,227,56,215,213,243,61,58,58,58,112,251,237,183,163,181,181,21,119,221,117,23,238,185,231,30,28,58,116,104,250,206,141,49,211,2,157,235,239,74,133,199,107,199,
113,144,74,165,160,148,194,196,196,4,242,249,124,221,143,41,8,130,135,186,187,187,63,146,78,167,63,18,237,111,156,5,240,107,0,190,190,212,21,223,249,28,228,214,69,43,180,118,0,126,16,4,127,122,240,224,
193,215,69,7,180,5,235,189,239,125,47,222,243,158,247,224,193,7,31,196,87,191,250,85,28,58,116,168,238,49,116,174,112,173,181,40,151,203,8,130,160,58,236,68,193,193,113,22,126,168,142,227,92,59,48,48,
240,186,100,50,249,167,74,169,59,162,199,246,219,0,126,2,160,239,185,158,166,125,160,102,249,251,185,190,190,190,225,108,54,251,182,122,102,11,183,223,126,59,110,189,245,86,220,113,199,29,248,232,71,63,
138,3,7,14,84,67,171,237,216,185,186,119,230,216,61,219,255,91,107,171,95,83,44,22,49,50,50,82,215,111,180,82,234,109,199,142,29,27,142,14,208,136,30,227,7,158,211,33,34,218,21,187,43,122,134,119,31,56,
112,224,55,29,199,249,94,61,243,220,47,125,233,75,112,28,7,159,252,228,39,177,119,239,222,105,65,84,66,213,90,79,11,120,174,161,97,182,241,120,230,223,131,32,168,206,60,202,229,50,214,175,95,95,215,98,
36,155,205,190,186,185,185,249,227,209,178,250,44,128,215,45,101,23,110,169,29,252,71,81,184,133,32,8,254,6,192,47,212,27,174,181,22,31,254,240,135,241,212,83,79,205,26,238,108,127,95,76,231,206,236,98,
165,20,226,241,56,90,91,91,17,4,1,158,126,250,233,186,22,35,131,131,131,191,96,173,253,155,104,115,170,61,122,204,141,31,34,162,205,242,151,69,243,221,251,134,134,134,78,69,91,130,11,14,11,142,227,224,
111,255,246,111,207,57,144,205,172,133,14,108,74,169,69,207,121,147,201,36,122,122,122,16,4,1,158,122,234,169,122,166,141,183,14,15,15,159,138,118,254,24,192,203,162,199,222,184,128,163,151,121,222,130,
112,179,60,7,224,27,163,163,163,191,184,208,198,205,123,223,251,94,108,218,180,9,159,250,212,167,112,224,192,129,89,195,91,168,99,235,233,220,185,14,132,181,33,111,220,184,17,0,112,236,216,177,5,39,57,
185,92,238,23,1,124,35,122,172,105,0,111,137,50,104,88,7,111,139,6,125,7,64,255,145,35,71,142,68,91,142,243,78,197,126,229,87,126,5,95,249,202,87,176,119,239,222,5,59,182,222,217,194,204,78,158,43,232,
218,127,183,214,162,185,185,25,109,109,109,24,31,31,95,112,65,98,173,125,219,248,248,248,17,0,253,209,99,126,73,148,65,195,2,126,49,194,215,208,96,173,253,87,107,237,219,23,218,207,253,204,103,62,131,
7,31,124,16,119,223,125,119,117,250,52,87,135,85,14,110,245,118,238,124,243,226,153,159,83,59,222,119,119,119,195,243,188,105,179,151,57,42,49,52,52,244,118,107,237,191,70,255,191,61,202,96,249,3,142,
126,53,110,142,158,201,242,224,224,224,15,130,32,120,249,66,43,180,214,214,86,124,245,171,95,157,214,45,179,13,13,90,235,186,166,101,181,157,92,239,80,82,251,189,148,82,80,74,161,167,167,7,0,22,92,136,
56,142,243,242,32,8,126,128,240,117,67,7,192,205,139,25,38,22,211,193,9,132,47,173,3,192,79,243,249,188,19,189,204,51,231,3,252,216,199,62,134,239,124,231,59,56,114,228,200,188,223,184,222,206,157,109,
152,88,104,40,169,237,224,218,239,149,78,167,209,220,220,140,35,71,142,204,219,197,65,16,116,159,60,121,210,65,248,162,44,162,12,26,18,240,101,209,242,17,0,254,109,104,104,232,38,204,243,26,90,119,119,
55,18,137,4,238,185,231,158,57,135,134,217,230,188,243,205,22,22,211,185,115,133,91,251,253,178,217,44,0,44,180,103,145,236,234,234,186,9,225,43,222,136,50,184,172,17,1,191,2,225,206,63,3,248,86,107,107,
235,141,243,125,242,187,222,245,46,236,222,189,123,222,41,217,108,157,91,207,84,108,161,133,199,204,249,245,92,79,76,50,153,68,50,153,196,192,192,192,188,15,124,96,96,224,70,0,223,138,30,187,142,178,88,
246,128,43,195,195,254,129,129,129,92,244,210,250,156,251,185,47,127,249,203,177,123,247,238,105,227,222,204,191,207,53,139,88,104,17,49,219,109,174,207,91,104,86,146,78,167,49,62,62,190,208,193,110,103,
34,145,200,33,124,67,76,109,22,203,19,112,180,115,118,117,244,191,7,78,158,60,121,37,230,121,223,66,71,71,7,242,249,124,93,171,166,153,67,195,66,83,177,185,102,11,139,93,124,84,126,78,101,231,109,129,
97,34,166,148,186,18,225,187,141,0,224,234,40,147,101,235,224,53,152,122,35,222,145,232,29,55,115,214,250,245,235,209,215,215,135,190,190,190,186,127,173,235,93,254,206,53,148,212,142,183,139,249,153,
241,120,28,241,120,124,193,189,227,61,123,246,236,68,248,86,46,68,89,172,89,206,128,55,214,252,253,20,128,75,231,61,26,94,118,25,250,251,251,231,28,31,235,89,96,204,182,113,51,243,251,205,12,119,177,85,
249,222,158,231,33,151,203,45,244,233,151,70,143,125,182,76,206,59,224,206,232,79,6,48,100,173,237,93,168,131,251,251,251,23,236,158,185,14,118,115,109,220,204,12,102,102,184,245,44,60,230,58,102,44,212,
193,209,99,30,138,50,168,205,100,89,2,110,169,76,11,1,228,163,55,226,205,89,205,205,205,24,31,31,159,247,65,207,181,192,168,119,252,92,236,230,252,124,43,63,165,212,180,169,228,28,63,179,19,211,223,225,
217,178,156,1,199,107,2,14,48,245,46,199,57,171,142,95,185,115,58,119,174,5,199,204,48,23,59,222,206,55,127,158,109,182,49,71,165,48,253,61,202,241,229,12,120,81,85,46,151,231,12,101,174,5,70,109,39,207,
183,178,155,173,115,231,90,22,47,180,242,171,116,245,34,66,110,216,126,112,101,35,161,242,206,242,121,219,115,100,100,100,222,59,60,91,231,214,243,82,209,98,247,134,23,19,90,29,159,155,171,121,252,181,
153,44,75,192,163,53,1,39,173,181,103,22,88,249,204,217,85,51,231,192,243,117,235,98,199,226,197,174,252,106,135,26,207,243,22,26,203,207,68,91,3,206,140,76,150,37,224,74,160,4,32,27,189,179,124,206,234,
235,235,67,44,22,155,183,75,22,218,135,168,119,67,103,169,157,91,251,253,130,32,88,48,224,232,49,103,49,245,162,238,153,229,12,184,118,251,127,45,128,121,151,104,71,142,28,65,60,30,159,119,21,182,80,231,
214,251,226,230,98,102,11,115,205,203,131,32,64,34,177,224,6,217,211,209,99,159,45,147,243,14,248,52,194,83,165,0,160,119,100,100,228,177,249,62,121,120,120,24,142,227,32,153,76,46,56,78,206,214,189,243,
29,32,151,218,185,115,5,93,153,255,46,212,193,175,122,213,171,30,171,217,77,28,142,50,89,158,128,163,119,181,84,94,178,222,214,211,211,179,27,225,251,208,102,173,137,137,9,148,203,101,36,147,201,5,151,
191,90,235,234,109,230,134,208,124,27,68,139,157,45,204,181,65,84,121,175,219,2,195,74,105,114,114,114,55,166,94,46,122,164,222,119,250,44,102,110,242,163,232,207,139,187,186,186,82,214,218,199,230,235,
150,3,7,14,32,149,74,213,189,113,83,121,15,218,82,166,75,179,125,77,61,67,6,0,248,190,95,221,240,153,167,30,27,24,24,72,33,60,77,172,54,139,101,13,248,7,8,223,177,78,0,222,56,50,50,114,255,124,159,124,
240,224,65,196,227,241,234,175,222,92,27,55,215,93,119,29,174,187,238,186,105,203,223,197,28,240,230,27,183,23,122,178,42,111,179,170,12,101,115,213,181,215,94,123,63,194,211,195,40,202,224,7,141,8,120,
95,205,110,210,155,179,217,236,189,209,210,113,246,137,115,177,136,201,201,201,234,193,110,182,95,229,215,191,254,245,216,188,121,51,46,186,232,34,188,225,13,111,56,175,9,253,98,134,140,202,223,11,133,
66,61,195,67,254,167,63,253,233,189,8,223,59,140,40,131,125,141,8,184,80,243,171,113,109,50,153,12,202,229,114,255,124,95,240,236,179,207,206,186,33,110,173,197,206,157,59,145,201,100,144,78,167,209,213,
213,133,151,190,244,165,184,254,250,235,207,123,227,166,222,105,90,16,4,8,130,0,173,173,173,243,126,126,38,147,233,223,190,125,123,128,240,196,198,202,240,80,88,246,128,163,115,125,119,69,107,113,175,
163,163,227,21,142,227,252,112,161,95,193,124,62,63,235,3,40,22,139,240,60,15,173,173,173,88,191,126,61,182,111,223,190,44,27,55,245,46,48,10,133,2,28,199,89,240,201,152,152,152,248,225,196,196,196,43,
16,158,53,26,0,216,181,152,243,158,23,123,68,121,2,225,185,191,80,74,221,170,148,250,231,133,158,205,161,161,33,228,114,57,148,74,165,234,246,98,229,128,22,143,199,209,222,222,142,141,27,55,162,179,179,
19,15,62,248,224,121,111,220,212,19,116,177,88,132,181,22,153,76,102,161,128,11,151,92,114,201,63,215,188,53,108,79,148,65,221,181,216,247,7,31,64,120,134,228,118,0,221,189,189,189,189,251,247,239,255,
154,231,121,191,56,95,215,85,30,144,231,121,213,131,222,254,253,225,203,91,163,163,163,216,191,127,63,62,253,233,79,159,211,157,75,217,184,89,168,171,131,32,64,169,84,170,103,230,128,166,166,166,175,21,
10,133,94,132,167,226,6,209,99,63,176,152,192,22,253,246,213,232,13,112,255,12,32,3,224,187,3,3,3,31,153,156,156,252,167,133,222,159,150,207,231,81,42,149,144,205,102,145,74,165,234,122,153,126,49,191,
250,243,61,49,181,67,86,161,80,128,235,186,72,167,211,11,77,253,6,51,153,204,187,90,90,90,126,7,192,107,0,140,1,120,251,98,207,216,95,244,164,51,250,1,247,69,83,150,151,101,179,217,181,53,111,45,154,179,
146,201,36,138,197,34,78,158,60,57,235,94,241,114,109,220,204,119,80,43,20,10,208,90,47,24,110,84,255,218,210,210,178,22,225,59,73,9,192,125,75,185,28,194,82,55,65,255,12,225,155,146,19,209,185,191,95,
170,103,251,174,187,187,27,147,147,147,120,250,233,167,49,58,58,218,144,141,155,217,254,30,4,1,114,185,28,180,214,104,110,110,174,107,123,246,77,111,122,211,151,0,252,22,194,119,241,156,141,30,51,158,
147,128,163,119,122,223,17,253,239,149,209,185,191,239,0,48,190,208,215,110,219,182,237,87,203,229,242,221,251,247,239,231,190,190,190,121,127,173,23,187,113,51,215,124,60,151,203,49,128,187,155,155,155,
127,181,142,47,25,7,240,142,39,159,124,242,231,49,117,210,248,29,75,189,198,196,249,108,227,127,58,26,244,1,224,125,235,215,175,111,27,26,26,250,26,166,94,20,156,173,254,235,147,79,62,121,7,51,191,14,
192,127,63,117,234,148,127,232,208,33,76,78,78,46,203,198,205,204,241,118,114,114,18,165,82,201,7,240,223,251,250,250,94,247,228,147,79,222,1,224,191,206,115,255,248,230,155,111,254,218,150,45,91,218,
16,158,44,142,232,49,126,122,201,123,214,231,17,240,9,132,215,91,56,11,192,117,28,231,143,183,110,221,122,87,16,4,63,157,227,243,255,228,201,39,159,252,235,234,35,97,254,43,0,189,163,163,163,207,238,221,
187,23,199,142,29,67,62,159,63,175,141,155,202,139,151,147,147,147,152,152,152,128,49,230,89,173,117,111,95,95,223,95,85,190,79,116,31,254,100,142,251,248,211,190,190,190,187,172,181,127,140,240,76,252,
179,209,99,60,177,212,144,158,171,19,17,63,246,228,147,79,126,104,214,59,64,164,0,188,88,41,117,151,227,56,93,233,116,26,77,77,77,72,167,211,240,60,111,193,119,246,84,246,115,43,251,10,190,239,3,192,128,
82,234,117,90,235,39,142,30,61,58,235,88,114,197,21,87,124,116,70,55,175,172,19,17,107,66,94,232,84,218,207,60,249,228,147,11,158,6,229,186,174,6,176,89,41,245,251,74,169,247,84,78,94,241,60,15,142,227,
192,117,221,115,94,134,242,125,127,218,203,237,74,169,47,40,165,254,18,192,225,190,190,190,5,223,11,112,197,21,87,124,26,192,175,98,165,158,74,91,19,242,92,39,131,255,220,147,79,62,249,199,139,253,126,
137,68,34,165,148,90,175,148,186,2,192,203,149,82,87,42,165,182,0,104,141,134,132,17,0,135,148,82,187,149,82,63,4,240,164,82,170,239,228,201,147,185,197,254,172,43,174,184,226,79,1,124,125,197,158,12,
94,19,242,213,152,186,86,132,2,240,143,0,126,163,145,215,195,89,166,251,221,2,224,19,8,47,103,96,163,112,223,191,92,87,166,90,206,55,3,60,14,224,247,163,59,72,0,206,174,244,112,163,41,231,104,116,48,171,
116,238,239,71,143,101,89,170,81,151,148,249,185,59,239,188,243,247,240,2,170,91,110,185,229,175,0,124,29,203,124,73,25,18,133,160,177,37,151,183,149,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,
128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,34,144,128,37,96,41,9,88,2,150,128,165,36,224,149,88,226,42,55,184,196,85,190,80,3,22,87,185,177,225,138,171,220,160,96,197,85,110,208,
193,74,92,229,6,133,219,2,113,149,197,85,134,184,202,231,95,226,42,63,119,37,174,114,131,75,92,229,231,160,196,85,22,87,121,250,108,65,92,229,70,4,44,174,114,3,3,22,87,185,241,7,57,113,149,27,60,77,19,
87,185,81,67,132,184,202,141,239,96,113,149,27,21,176,184,202,13,12,88,92,101,113,149,231,125,66,196,85,22,87,89,92,101,136,171,44,174,242,188,115,222,249,102,11,226,42,215,148,184,202,139,15,88,92,101,
113,149,197,85,94,210,198,205,108,31,19,87,89,92,229,69,5,44,174,178,184,202,243,111,206,139,171,12,113,149,235,46,113,149,23,31,176,184,202,226,42,139,171,44,174,242,60,37,174,178,184,202,47,96,87,249,
222,199,111,22,87,121,134,171,28,101,178,172,211,180,101,119,149,107,233,49,113,149,23,229,42,19,14,30,60,132,120,60,1,207,139,3,80,80,202,137,126,28,65,41,141,157,59,119,226,117,175,123,29,222,245,174,
119,225,85,175,122,149,184,202,168,203,85,166,234,173,88,44,99,114,50,143,120,60,1,165,52,0,130,181,12,165,28,116,118,118,226,162,139,46,130,231,121,104,107,107,195,213,87,95,141,151,190,244,165,56,159,
90,5,174,114,37,92,5,144,2,72,227,217,131,135,64,218,5,105,15,172,28,144,118,192,164,176,110,253,70,48,8,169,116,26,221,221,221,216,182,109,27,222,240,134,55,44,203,198,77,189,211,180,21,231,42,223,251,
248,205,243,184,202,83,157,11,165,65,228,0,228,160,92,6,242,133,0,90,199,64,112,65,228,130,200,193,129,131,135,225,104,15,173,45,109,88,187,182,7,219,182,109,195,183,191,253,237,101,217,184,169,119,129,
113,62,174,114,148,197,178,119,48,112,142,171,236,124,13,160,2,64,97,215,42,13,192,1,147,11,168,24,160,226,24,30,201,99,178,96,81,244,9,6,30,44,107,12,15,143,227,169,125,251,48,60,50,130,53,221,221,248,
240,135,63,130,31,253,232,71,203,178,113,83,79,208,47,48,87,249,217,175,123,158,247,110,144,2,224,2,202,141,254,12,187,216,146,66,41,0,172,49,8,44,195,115,194,14,127,226,241,61,120,244,209,221,248,228,
39,254,14,33,130,69,0,248,130,115,149,23,213,193,209,175,198,55,16,158,88,157,2,248,45,77,45,107,190,108,65,103,1,39,12,87,121,81,247,38,192,58,1,86,41,176,74,161,68,47,198,240,72,30,165,50,67,57,49,24,
75,32,40,128,194,131,223,114,110,220,44,228,42,199,98,49,196,227,241,133,166,126,131,142,227,252,95,132,239,197,75,69,143,249,27,139,25,30,150,180,93,121,239,227,55,71,174,50,17,64,55,182,181,181,245,
24,246,190,9,21,5,76,49,64,39,192,58,9,168,20,160,19,192,218,47,32,121,205,255,64,57,118,43,78,159,25,65,174,80,14,135,148,40,224,229,220,184,153,239,160,118,190,174,114,244,216,209,208,128,1,128,153,
254,28,192,89,128,18,49,143,127,195,80,231,55,0,167,4,56,0,197,192,81,7,67,37,128,117,159,4,117,49,192,140,206,159,121,31,242,126,7,14,30,60,134,209,177,201,134,108,220,92,16,174,242,125,79,188,226,17,
64,125,54,154,65,188,120,107,111,235,91,75,106,195,111,67,197,114,172,194,144,161,186,128,141,31,3,181,134,225,50,49,248,240,67,88,179,126,211,39,2,235,253,248,224,193,195,124,226,228,169,23,140,171,124,
239,227,55,63,167,174,50,149,252,248,103,25,244,16,64,112,40,248,249,206,142,214,182,225,113,252,16,228,48,98,87,131,55,255,1,168,153,195,9,6,0,62,250,40,138,19,255,239,75,35,116,227,183,154,47,249,195,
255,162,220,182,143,245,247,159,242,15,29,58,252,156,184,202,19,234,67,175,59,62,246,43,47,28,87,249,233,51,215,158,8,172,251,9,132,175,180,186,205,241,220,7,219,187,47,251,73,224,253,135,62,108,121,47,
84,18,225,235,175,12,152,99,143,161,52,254,237,127,31,197,181,223,0,163,12,203,126,230,226,63,252,59,175,237,103,174,26,29,29,59,208,104,87,121,66,125,168,234,42,31,31,251,149,231,212,85,94,114,192,147,
229,22,30,45,174,217,101,172,243,175,0,140,34,187,166,51,83,120,61,197,59,190,128,0,37,138,230,198,124,236,49,248,227,255,254,157,81,190,230,159,152,57,15,182,69,128,75,0,202,169,238,159,239,107,222,252,
123,59,148,78,92,115,246,236,217,129,3,7,14,224,200,145,35,56,115,230,76,53,236,133,134,140,202,169,91,147,147,147,24,29,29,197,216,216,24,74,165,210,128,82,234,74,173,245,165,163,252,219,39,103,222,247,
227,99,191,114,27,128,143,205,248,231,135,179,217,236,111,6,65,240,94,132,103,224,27,132,215,146,248,94,189,59,103,179,254,170,215,115,150,81,203,198,143,159,243,117,58,147,209,80,74,95,212,249,248,134,
108,98,240,51,164,212,43,88,197,252,162,105,190,247,104,233,3,187,147,93,61,255,31,159,126,216,9,198,191,254,253,33,220,116,59,51,231,192,60,14,107,39,192,60,193,198,228,0,20,216,26,223,140,140,248,197,
161,79,43,83,58,216,187,92,174,242,132,250,208,130,239,5,216,144,249,252,130,174,242,189,143,223,60,231,190,239,232,177,223,108,92,192,78,182,93,177,53,14,41,237,54,39,199,154,47,107,255,201,247,161,156,
75,152,92,46,155,244,3,135,134,175,251,23,175,188,235,218,33,122,213,255,5,163,8,182,69,48,231,0,76,178,49,121,48,23,57,240,75,0,2,51,54,102,163,213,70,184,249,122,230,247,150,228,42,231,156,255,182,104,
87,121,67,230,243,115,186,202,247,62,126,243,188,239,70,105,100,192,0,160,116,38,163,200,113,93,210,58,118,89,251,131,215,53,197,199,62,70,164,46,97,104,85,182,201,187,15,14,95,241,215,19,229,204,40,44,
251,0,151,192,92,100,107,11,176,182,4,112,217,150,203,190,157,156,52,83,163,245,115,95,55,237,216,213,130,89,92,229,122,102,13,245,4,124,62,111,6,96,114,92,102,107,44,51,251,199,70,47,217,93,244,227,127,
198,54,120,154,184,68,100,75,103,38,138,77,199,97,237,40,216,142,129,121,156,141,201,129,185,184,82,194,141,22,78,163,120,190,93,229,57,58,184,218,197,80,74,19,145,110,74,140,197,55,181,236,219,153,112,
39,223,252,112,255,27,254,28,0,216,90,6,179,133,181,1,136,2,14,252,128,141,49,43,33,220,25,157,92,117,149,239,125,252,230,186,46,41,211,232,33,162,186,79,25,13,21,138,180,86,28,248,10,52,99,18,203,214,
178,49,22,74,49,172,53,209,152,203,233,27,110,228,201,7,239,199,11,181,26,61,68,32,10,201,70,129,25,91,42,6,182,92,246,109,169,88,98,107,124,182,198,183,165,98,201,150,203,62,128,192,140,140,248,181,225,
98,21,212,178,92,179,39,125,195,141,118,242,193,251,43,187,238,149,154,57,77,226,232,115,87,21,69,190,108,23,69,138,130,19,199,125,153,135,8,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,88,
2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,90,114,173,88,87,249,166,29,187,22,229,42,159,207,155,164,47,248,128,111,218,177,235,188,93,229,155,118,236,58,199,85,
94,236,57,109,23,84,192,87,229,111,11,93,229,29,203,226,42,183,1,120,77,116,171,186,202,55,237,216,85,117,149,159,175,176,157,231,41,220,70,186,202,26,192,214,232,246,11,136,92,229,155,118,236,250,198,
82,78,36,124,65,5,124,85,254,182,165,184,202,59,235,112,149,191,63,143,171,188,35,122,18,223,117,211,142,93,247,1,248,179,165,158,243,182,148,58,223,247,7,35,125,195,141,11,133,186,162,93,229,243,57,56,
62,23,111,192,158,55,224,171,242,183,181,224,5,224,42,71,167,17,188,176,2,190,42,127,219,11,202,85,158,239,116,173,21,23,240,85,249,219,94,144,174,242,66,167,109,61,239,1,95,149,191,237,130,112,149,151,
243,36,152,101,91,42,71,225,94,16,174,242,77,59,118,137,171,124,158,245,156,185,202,206,50,117,175,87,51,44,84,92,229,143,0,248,156,227,56,43,213,85,38,199,113,174,27,26,26,250,196,208,208,208,251,122,
123,123,83,209,252,121,39,128,207,223,180,99,215,53,139,29,147,27,50,6,223,180,99,215,5,237,42,63,95,39,131,87,194,109,193,42,112,149,231,154,39,215,19,240,146,135,136,104,59,241,213,209,60,87,35,116,
149,255,207,224,224,224,199,235,29,22,234,113,149,43,221,93,9,180,54,220,202,159,19,19,19,213,139,242,87,92,229,214,214,214,197,186,202,215,12,12,12,124,92,41,245,145,116,58,125,105,180,191,241,78,0,247,
221,180,99,215,215,151,186,226,19,87,185,54,12,113,149,49,103,119,207,214,237,43,193,85,86,75,28,30,174,142,166,50,64,232,42,127,57,90,68,44,120,205,172,47,125,233,75,104,109,109,197,95,253,213,95,225,
91,223,250,86,93,227,228,249,184,202,149,43,167,148,203,229,105,182,210,124,83,56,0,95,29,31,31,255,50,128,221,209,191,189,63,122,204,207,89,7,139,171,220,168,128,111,218,177,107,213,186,202,209,99,111,
92,192,209,107,103,171,218,85,142,50,104,88,7,139,171,44,174,242,11,212,85,142,126,53,196,85,6,110,94,204,48,33,174,178,184,202,226,42,207,90,226,42,47,62,96,113,149,27,229,42,71,59,103,226,42,215,184,
202,81,38,203,214,193,226,42,139,171,188,248,18,87,25,226,42,207,190,223,39,174,242,162,3,22,87,89,92,229,250,54,136,102,235,106,113,149,151,184,55,44,174,178,184,202,213,18,87,89,92,101,113,149,235,90,
42,207,55,107,168,103,70,80,207,19,118,129,184,202,62,146,201,84,21,76,173,220,86,155,171,188,152,119,246,252,8,225,121,104,23,119,117,117,165,14,29,58,244,152,82,234,250,233,159,66,209,3,67,141,171,92,
233,94,170,233,100,194,171,94,245,74,116,116,132,175,149,62,248,224,131,56,124,248,176,184,202,152,215,85,174,81,105,65,56,120,232,40,98,137,20,188,88,28,32,5,165,93,128,194,110,126,205,107,94,131,142,
142,14,36,147,73,116,118,118,226,230,155,111,22,87,25,115,186,202,231,114,191,164,92,148,74,6,147,147,69,196,227,105,40,29,3,200,133,101,66,34,213,132,182,182,14,36,18,41,180,119,116,96,243,230,205,120,
205,107,94,3,215,117,207,107,66,127,129,186,202,126,127,245,219,80,133,251,117,193,228,130,85,12,7,14,157,0,156,36,72,39,192,228,66,233,24,74,165,0,249,98,17,169,116,19,186,215,172,197,230,205,91,224,
186,238,57,111,83,21,87,185,163,227,21,142,227,222,91,27,110,200,253,198,1,157,2,84,10,101,235,33,95,36,144,147,6,171,36,88,197,193,20,195,254,103,143,192,24,70,71,71,7,140,181,120,203,91,110,93,84,151,
190,144,92,229,197,190,125,181,226,42,239,80,74,223,10,149,248,109,144,121,23,145,147,224,10,152,170,98,0,121,96,229,129,160,49,50,110,225,184,14,0,134,167,195,49,120,239,211,135,176,111,223,62,252,237,
39,63,9,178,1,66,150,125,121,54,110,22,218,131,174,117,149,23,234,94,68,174,114,161,80,248,100,101,223,29,139,116,149,23,123,200,142,92,101,10,0,234,222,180,105,211,230,114,128,59,89,69,80,181,78,128,
117,10,112,154,1,39,3,118,50,48,170,25,165,244,7,81,44,123,40,148,29,24,246,0,114,160,116,248,4,84,14,124,203,181,113,179,80,87,175,120,87,153,153,254,13,192,36,64,41,34,190,37,209,188,245,95,45,187,195,
161,70,155,0,84,26,172,155,0,221,12,78,222,0,222,242,87,160,13,151,161,148,248,121,12,143,251,40,250,4,237,196,97,108,229,160,184,252,27,55,47,104,87,249,190,39,94,113,15,128,251,153,136,136,232,134,150,
150,230,158,0,205,63,128,138,131,84,10,112,82,96,157,6,183,188,15,216,244,38,80,210,128,217,34,113,209,245,240,189,215,226,204,224,4,242,133,32,234,92,85,189,11,226,42,215,222,89,19,251,159,0,134,8,72,
36,227,254,251,74,122,243,247,152,98,62,43,15,172,226,64,215,239,3,221,93,128,178,0,71,216,33,89,180,95,243,54,20,252,22,28,60,116,12,227,99,19,51,126,245,105,89,54,110,46,8,87,249,199,71,94,243,168,101,
231,11,12,5,5,92,190,165,135,126,54,79,23,125,156,221,43,202,88,255,71,161,165,12,11,128,163,255,0,48,192,199,158,68,199,218,77,255,16,112,252,145,131,135,142,240,137,19,167,192,213,253,7,179,168,113,
245,194,118,149,173,165,177,98,251,231,153,245,163,32,32,238,20,222,216,218,146,104,30,141,255,252,72,248,102,42,174,233,72,10,93,229,147,79,163,56,241,131,111,141,242,142,239,53,109,253,205,223,85,94,
246,227,226,42,207,83,7,135,174,62,89,12,18,159,97,240,8,0,39,155,28,126,115,218,252,232,94,155,103,191,178,92,6,20,8,4,62,185,31,197,177,123,239,29,51,151,220,5,182,5,88,91,108,218,242,95,111,143,103,
111,188,102,116,108,236,224,222,125,251,196,85,158,209,94,236,91,143,7,242,27,127,16,24,239,91,12,99,52,149,219,187,83,143,93,26,156,121,228,31,109,9,190,34,21,14,32,39,247,163,52,190,235,39,99,193,214,
127,99,203,99,96,30,7,144,7,163,148,232,126,123,95,102,219,31,239,84,42,46,174,114,109,188,58,147,81,164,181,7,82,241,117,45,71,122,215,52,159,248,184,214,250,103,140,74,6,249,160,229,209,19,133,255,176,
63,213,189,233,231,49,240,148,227,79,220,245,208,176,217,241,127,217,242,24,192,103,97,237,48,91,59,2,230,9,88,155,103,182,101,82,58,152,124,246,207,73,92,229,232,107,106,76,229,56,136,226,9,175,216,182,
189,251,167,119,42,229,108,99,120,92,48,233,71,143,12,189,248,59,177,224,161,23,13,243,117,255,12,182,121,48,79,128,121,140,173,29,7,48,14,99,10,108,130,34,196,85,158,245,92,101,165,210,105,173,60,207,
133,114,98,164,40,177,53,251,196,117,45,201,225,255,169,72,109,3,105,85,10,146,63,56,56,116,233,223,229,202,169,17,48,151,192,156,227,16,174,206,193,152,130,184,202,11,125,97,44,110,153,217,128,173,207,
198,20,251,70,182,60,82,40,198,255,146,109,233,89,178,121,34,155,63,149,43,37,15,130,249,12,91,123,150,45,143,130,121,162,26,110,177,24,172,6,87,121,169,231,42,219,96,232,172,210,153,140,137,230,184,92,
68,194,28,30,190,236,187,155,90,246,157,73,121,227,111,124,236,212,207,252,85,244,91,111,96,140,1,81,0,102,159,77,224,179,49,198,230,243,43,198,85,190,247,241,155,127,251,166,29,187,74,88,164,171,220,
168,131,220,180,151,48,156,108,59,1,208,145,167,172,73,41,5,34,5,102,2,0,102,107,193,48,164,136,153,217,172,54,87,249,124,206,182,103,0,28,12,157,85,0,2,157,201,40,14,124,195,214,18,197,98,213,117,47,
151,74,12,165,24,90,51,57,46,7,35,35,12,113,149,235,175,138,169,108,198,198,76,117,249,54,247,169,81,44,174,242,210,66,230,154,174,150,90,142,89,132,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,
148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,157,111,137,171,188,26,2,22,87,185,1,37,174,114,99,195,21,87,185,65,193,138,171,60,91,137,171,60,123,137,171,44,
174,178,184,202,75,10,88,92,229,6,5,44,174,114,3,151,202,226,42,55,126,154,182,3,226,42,55,38,96,113,149,27,56,6,139,171,220,192,128,197,85,22,87,89,92,101,113,149,231,46,113,149,27,21,176,184,202,141,
239,96,113,149,27,21,176,184,202,13,12,88,92,101,113,149,231,125,66,196,85,22,87,89,92,101,136,171,44,174,242,180,238,21,87,121,170,196,85,22,87,89,92,229,5,23,21,226,42,139,171,12,136,171,92,95,137,171,
12,113,149,103,223,239,19,87,121,209,1,139,171,44,174,114,125,27,68,179,117,181,184,202,75,220,27,22,87,89,92,229,106,137,171,44,174,242,170,113,149,9,195,195,35,112,28,23,201,100,248,174,125,165,52,172,
173,229,119,48,239,120,123,33,185,202,117,189,241,228,222,199,111,230,232,60,180,215,32,114,149,243,249,124,105,106,63,130,106,70,16,194,196,68,14,229,114,25,201,100,26,147,147,181,27,218,12,165,52,174,
191,254,58,180,180,100,224,121,30,78,158,60,137,135,31,126,120,206,95,255,165,236,1,215,187,192,168,12,15,141,116,149,23,51,55,169,128,169,23,119,117,117,165,172,181,143,213,134,90,171,210,90,16,158,61,
120,20,201,116,51,152,28,88,166,42,253,123,229,85,87,99,227,198,141,200,102,179,232,234,234,194,43,94,241,10,172,93,187,182,250,30,52,113,149,171,174,242,232,253,231,112,191,145,74,11,138,225,208,225,
19,136,197,154,224,197,211,128,242,34,250,87,3,68,32,165,144,105,105,69,79,79,15,182,109,219,134,77,155,54,137,171,140,115,92,229,246,251,0,149,7,169,41,238,87,197,34,149,54,137,162,175,48,94,48,136,37,
90,66,149,150,226,176,236,2,228,33,158,72,163,189,189,3,155,54,245,226,178,203,46,67,95,95,223,121,79,244,47,48,87,153,174,141,39,50,92,14,204,105,34,93,229,126,73,37,1,39,13,168,38,64,103,112,232,232,
16,160,155,64,186,9,80,73,144,147,192,163,187,247,32,151,47,161,165,165,21,93,107,186,241,79,95,249,42,142,30,61,182,168,41,218,249,28,236,106,199,222,231,194,85,174,251,221,149,247,62,126,115,33,60,247,
151,126,1,32,175,163,179,227,21,185,66,233,1,38,189,25,42,6,168,68,216,193,20,7,84,12,76,46,202,172,144,43,199,0,71,193,162,0,226,2,128,34,190,242,213,127,195,93,223,190,11,35,163,131,128,173,140,58,92,
119,151,62,31,174,178,82,106,73,174,242,162,126,47,153,233,73,0,123,0,130,214,244,102,118,186,190,13,138,21,161,18,128,74,2,186,25,112,90,192,110,27,216,109,3,82,63,139,209,204,255,64,222,52,163,100,226,
176,72,194,34,14,203,14,70,198,38,1,232,112,252,6,69,211,184,243,223,184,169,39,232,138,171,156,201,100,234,114,149,107,222,26,214,88,87,57,95,76,31,4,212,79,152,40,32,168,53,61,107,187,183,148,76,234,
30,168,4,160,211,96,167,25,236,182,128,157,22,160,227,119,129,205,55,129,155,2,248,173,31,66,49,136,161,224,187,176,28,162,214,202,137,71,7,71,90,214,141,155,133,186,122,69,187,202,143,236,191,166,16,
24,231,155,0,229,24,42,233,58,193,235,84,250,178,123,12,197,199,161,147,128,78,195,166,94,9,94,255,107,64,54,0,115,0,134,1,183,26,148,227,111,196,232,68,128,162,79,80,58,6,99,41,236,96,40,113,149,107,
235,39,39,222,248,125,102,253,0,64,164,21,95,211,220,20,91,235,163,109,55,171,56,184,227,119,65,235,118,0,49,3,203,6,28,221,44,44,226,155,175,128,239,221,132,193,161,9,228,139,1,8,53,212,47,145,184,202,
211,134,10,191,249,163,76,106,132,137,98,45,201,220,91,139,244,162,39,120,211,159,48,181,133,176,33,195,2,204,224,232,6,14,85,179,236,206,55,32,95,206,224,208,161,227,24,27,23,87,249,220,77,134,168,158,
232,191,233,49,223,196,254,17,0,28,101,182,108,232,28,186,178,52,114,230,39,60,75,19,17,69,95,72,4,62,177,23,237,107,54,124,197,112,108,247,193,67,135,249,196,73,113,149,167,77,34,106,255,103,48,223,243,
69,107,245,147,0,35,29,155,188,33,238,255,96,120,236,180,63,110,153,64,80,32,34,16,85,86,122,4,62,121,0,133,177,31,125,111,204,94,178,43,221,251,129,63,208,94,199,223,246,159,234,247,15,29,58,130,201,
137,137,101,217,184,185,32,92,101,46,149,24,128,57,49,186,173,111,178,148,254,123,102,59,70,236,235,206,212,224,13,137,210,163,143,218,2,5,68,10,68,186,122,67,255,17,20,199,30,248,201,120,176,249,123,
17,158,90,104,218,242,91,159,142,183,223,120,237,216,216,168,184,202,51,90,139,217,90,107,172,10,6,38,215,255,176,28,56,223,35,46,91,87,229,50,93,169,167,215,249,167,31,191,155,203,42,32,114,64,112,128,
83,135,80,26,189,239,209,113,127,195,55,217,242,16,152,199,192,60,193,214,22,19,93,111,237,107,222,246,199,59,149,78,94,43,174,114,244,249,58,147,209,0,28,210,78,28,90,167,59,211,167,182,172,111,233,251,
176,118,157,107,13,37,130,92,185,101,207,201,226,43,15,167,187,55,223,194,131,123,117,121,252,251,143,142,6,151,126,133,45,159,5,219,33,88,123,150,153,135,97,237,24,91,147,7,80,18,87,121,6,92,173,146,
73,135,188,152,75,90,39,73,235,148,231,148,178,87,172,121,228,159,73,59,155,45,197,184,24,164,158,56,50,116,217,247,99,102,247,101,35,118,231,191,128,121,50,234,220,81,48,143,178,181,227,176,54,207,190,
95,130,34,95,92,229,89,100,112,40,165,137,148,7,32,65,142,147,236,109,219,119,93,54,57,244,167,74,211,86,64,83,49,72,254,240,224,217,139,62,157,47,167,70,0,46,128,57,207,204,185,10,253,203,204,101,14,
252,178,184,202,179,28,196,205,216,152,133,181,134,77,224,3,40,176,49,185,147,163,27,31,202,23,189,191,98,83,124,150,56,71,100,115,253,249,82,226,32,216,158,102,107,7,163,241,119,52,228,214,185,204,229,
146,47,174,242,60,147,9,51,54,102,117,38,19,48,91,38,210,92,42,199,252,195,195,23,223,189,169,245,233,211,233,216,196,27,31,63,249,146,15,3,214,70,7,11,3,107,3,0,101,246,253,128,217,6,226,42,207,61,68,
212,46,60,168,50,92,192,90,170,186,202,53,151,71,100,182,150,148,182,96,107,197,85,94,100,23,71,157,12,0,118,62,87,153,149,98,18,87,121,233,175,214,0,160,168,43,163,205,138,252,188,79,10,0,188,144,59,
247,185,14,24,43,101,44,93,137,37,87,192,150,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,34,144,128,37,96,41,9,248,121,43,113,
149,87,67,192,23,178,171,124,222,88,212,121,134,186,13,88,22,87,185,182,170,174,50,128,134,186,202,207,137,136,184,196,112,27,233,42,215,134,30,32,114,149,17,158,165,249,189,11,58,224,72,83,89,172,171,
124,105,29,174,242,99,243,184,202,149,160,39,17,154,24,203,230,42,159,119,192,203,17,108,116,176,18,87,185,17,1,71,231,63,136,171,220,136,128,35,41,81,92,229,70,4,124,211,142,93,226,42,55,34,224,136,203,
221,1,113,149,151,127,169,28,133,43,174,114,3,87,114,59,32,174,114,99,134,136,104,204,125,24,211,93,229,63,65,200,54,174,84,87,185,82,15,3,120,223,243,114,42,109,61,1,139,171,220,192,128,197,85,22,87,
89,92,101,113,149,231,46,113,149,27,21,176,184,202,141,239,96,113,149,27,21,176,184,202,13,12,88,92,101,113,149,231,125,66,196,85,22,87,89,92,101,136,171,44,174,242,180,238,21,87,121,170,196,85,22,87,
89,92,229,5,23,21,226,42,139,171,12,136,171,92,95,137,171,12,113,149,103,223,239,19,87,121,209,1,139,171,44,174,114,125,27,68,179,117,181,184,202,75,220,27,22,87,89,92,229,106,45,210,85,62,129,88,44,142,
41,239,243,220,46,17,87,121,122,45,224,42,19,106,241,212,41,87,217,137,156,56,85,67,255,46,188,131,86,239,118,229,42,112,149,167,7,91,1,84,135,71,198,161,181,135,68,42,13,144,134,210,78,120,53,247,232,
115,95,254,242,87,224,214,91,111,197,229,151,95,126,193,187,202,117,5,28,189,171,165,242,110,195,109,107,215,110,120,12,160,210,52,149,86,105,144,114,65,228,97,98,178,136,162,111,144,76,54,65,105,15,76,
78,36,210,42,220,242,230,183,160,187,187,27,173,173,173,184,249,230,155,113,227,141,55,66,107,13,173,245,57,27,66,243,109,16,45,118,182,48,215,6,209,138,116,149,59,58,215,164,173,229,199,1,138,184,223,
80,155,101,21,3,171,56,44,226,56,120,232,36,146,233,86,48,197,192,28,170,181,91,183,93,140,68,60,137,84,42,141,206,174,46,108,221,186,21,31,252,224,7,171,227,176,184,202,128,1,136,20,241,155,70,198,114,
15,66,85,184,95,111,74,165,213,41,192,73,227,208,177,65,56,177,22,120,241,22,64,37,64,58,129,137,201,50,44,20,210,77,205,232,94,179,22,91,183,110,67,185,92,158,22,238,42,119,149,233,72,52,244,254,135,
76,118,227,143,1,167,0,229,161,106,122,234,144,251,133,211,130,146,73,98,188,160,224,36,178,128,211,12,232,20,78,15,142,225,216,241,126,36,147,77,232,90,179,6,169,116,26,191,251,187,191,7,107,249,130,
117,149,235,126,243,95,217,143,21,61,215,127,0,160,173,32,117,77,60,209,74,249,201,242,96,204,137,111,96,149,12,217,95,157,4,69,228,47,72,225,232,233,102,180,55,43,192,1,172,201,131,160,241,211,71,247,
224,201,167,246,160,51,219,140,195,135,15,194,26,63,122,128,188,232,141,155,243,89,96,44,198,85,94,191,126,125,144,203,229,150,228,42,215,221,54,63,222,251,51,69,203,244,195,112,45,78,94,71,123,243,77,
202,109,123,148,163,238,133,19,118,46,59,109,96,55,11,238,252,99,148,55,253,39,228,232,10,192,109,9,187,91,37,1,138,35,95,12,112,248,104,95,228,221,107,212,123,118,193,243,233,42,35,60,107,180,177,174,
178,177,238,147,76,234,105,130,134,235,224,13,190,211,123,31,84,170,204,58,13,235,52,131,157,22,216,214,95,6,111,250,32,144,13,0,229,99,188,245,229,161,171,108,19,48,148,132,69,12,198,134,99,119,117,106,
39,174,114,88,67,147,107,14,89,118,126,202,80,1,72,117,116,118,100,54,21,108,203,35,208,73,160,233,141,224,245,31,4,58,155,0,39,128,101,31,204,1,172,10,224,55,255,18,74,190,135,162,239,32,96,15,32,15,
202,241,162,169,219,242,110,220,44,212,213,43,218,85,62,56,122,85,177,108,226,223,2,169,60,136,18,9,175,120,35,39,94,252,184,93,243,63,2,90,183,29,136,219,144,249,101,3,182,83,127,114,139,65,57,126,51,
70,39,3,148,124,64,57,113,88,51,181,56,17,87,185,166,207,118,159,124,213,15,13,59,15,1,138,92,109,182,167,19,156,245,81,154,96,182,64,116,99,182,176,209,159,21,2,56,182,241,82,248,238,13,24,28,154,68,
161,80,174,25,30,16,185,202,90,92,229,234,214,90,177,253,19,204,52,74,96,47,155,58,251,51,249,83,79,254,180,114,46,205,212,72,90,171,239,132,127,102,95,252,26,20,202,77,56,116,184,15,99,19,147,81,48,188,
172,27,55,47,84,87,121,218,178,112,255,224,149,143,151,130,216,191,16,44,92,93,90,183,46,251,236,154,82,255,153,253,108,107,201,95,53,99,175,2,224,19,79,35,187,166,231,107,129,117,31,59,120,240,16,159,
56,121,2,28,121,202,226,42,99,138,252,5,179,63,48,209,243,21,99,177,143,184,140,166,216,216,139,60,127,247,217,201,193,32,15,40,16,52,20,85,110,42,12,187,255,48,242,163,15,236,26,51,23,239,74,247,190,
255,15,180,215,254,9,113,149,207,125,212,204,214,90,176,13,6,38,122,142,78,20,82,255,8,54,227,154,139,170,43,125,102,123,172,248,212,30,91,36,67,170,98,42,59,32,114,128,254,163,40,142,62,240,211,9,179,
233,187,96,30,143,92,229,219,99,237,55,93,39,174,114,205,179,171,98,113,11,66,192,198,250,214,82,97,96,162,251,7,101,31,187,136,243,214,83,147,77,93,201,103,59,202,3,79,237,66,73,7,138,92,40,114,128,254,
195,40,141,222,183,123,220,95,255,77,182,118,8,204,35,96,30,103,107,139,201,174,183,28,95,213,174,242,44,103,122,42,149,78,107,114,92,143,20,37,160,116,186,189,105,112,219,134,214,227,127,161,29,231,154,
200,85,222,119,170,120,211,209,244,218,205,111,192,224,62,93,26,251,254,238,209,224,210,175,134,104,170,61,35,174,242,252,1,135,142,103,107,171,27,145,191,73,40,149,142,185,126,251,21,107,31,251,50,105,
189,153,225,113,33,72,63,121,100,232,146,93,49,179,251,210,17,179,243,95,0,158,0,243,56,172,29,5,48,34,174,242,220,1,3,21,87,25,112,0,138,145,235,198,1,74,245,102,247,95,215,145,58,251,39,20,185,202,5,
63,113,239,161,161,139,63,147,247,147,35,96,206,131,57,39,174,114,157,99,177,25,27,179,108,140,1,184,196,38,40,128,237,196,169,209,117,63,153,40,197,254,87,232,42,79,146,226,220,169,124,41,254,44,172,
61,197,198,136,171,188,152,128,1,192,78,78,26,149,78,131,148,42,1,176,37,63,238,31,61,123,209,93,155,218,246,245,167,99,147,179,184,202,70,92,229,58,135,136,233,227,241,12,87,25,4,77,52,117,52,18,87,121,
233,53,171,171,12,107,9,226,42,47,75,192,0,128,244,13,55,218,201,7,239,175,219,85,94,45,193,46,91,192,81,200,226,42,47,231,110,154,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,
82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,45,71,137,171,188,26,2,190,144,93,229,231,45,224,171,242,183,133,174,242,142,101,113,149,219,0,188,38,186,85,93,229,155,118,236,
106,168,171,188,98,3,190,42,127,91,35,93,101,13,96,107,116,251,5,68,174,242,77,59,118,45,187,171,188,226,2,190,42,127,219,82,92,229,157,117,184,202,223,159,199,85,222,17,61,137,239,186,105,199,174,101,
117,149,235,169,243,246,228,210,55,220,184,80,168,226,42,55,42,224,171,242,183,181,64,92,229,198,4,124,85,254,54,113,149,27,21,240,85,249,219,196,85,110,68,192,87,229,111,19,87,185,81,75,229,40,92,113,
149,27,56,77,219,1,113,149,27,19,112,52,230,126,30,211,93,229,143,0,248,92,189,128,106,109,61,71,174,50,57,142,115,221,208,208,208,39,134,134,134,222,215,219,219,155,138,230,207,59,1,124,254,166,29,187,
174,89,236,152,220,144,49,88,92,229,6,6,44,174,178,184,202,226,42,139,171,60,119,137,171,220,168,128,197,85,110,124,7,139,171,220,168,128,197,85,110,96,192,226,42,139,171,60,239,19,34,174,178,184,202,
226,42,67,92,101,113,149,167,117,175,184,202,83,37,174,178,184,202,226,42,47,184,168,16,87,89,92,101,64,92,229,250,74,92,101,136,171,60,251,126,159,184,202,139,14,88,92,101,113,149,235,219,32,154,173,
171,197,85,94,226,222,240,5,230,42,79,161,36,3,3,103,160,148,174,254,191,184,202,11,215,28,174,242,12,145,54,186,29,239,59,137,88,44,54,227,223,41,10,189,190,125,136,122,55,116,150,218,185,207,149,171,
92,239,203,246,51,92,101,122,102,90,192,84,209,105,195,127,59,114,244,56,118,238,220,1,210,46,76,224,135,48,31,17,172,53,184,230,154,107,225,186,14,124,223,199,129,3,7,48,58,58,90,247,88,187,152,113,117,
49,251,22,65,16,160,169,169,105,161,12,158,43,87,153,54,143,140,140,62,54,141,251,37,39,146,105,99,0,121,24,25,205,65,235,24,146,169,76,104,199,105,15,150,9,47,185,225,103,176,117,235,86,244,246,246,98,
199,142,29,120,247,187,223,45,174,50,80,235,42,19,0,108,237,232,190,236,73,144,46,17,57,0,92,176,138,131,85,34,36,37,117,10,19,121,70,161,76,136,39,219,64,58,17,5,239,130,148,3,173,61,100,90,90,177,126,
253,122,92,114,201,37,184,250,234,171,197,85,142,58,247,129,232,207,139,91,218,215,102,44,59,79,49,185,33,233,171,18,128,147,6,34,149,214,170,102,28,56,50,136,68,42,11,232,20,44,98,96,21,195,224,217,81,
196,83,73,116,118,118,97,195,134,141,120,209,139,94,84,221,140,95,245,174,178,181,234,135,0,25,38,69,49,199,127,211,200,4,63,18,114,191,137,136,251,109,14,209,84,183,13,112,218,112,244,100,14,42,222,6,
39,150,133,85,41,40,157,66,174,16,32,8,128,150,150,86,172,223,176,1,143,60,250,40,70,71,199,46,104,87,185,222,131,28,21,130,166,103,82,222,196,81,64,109,113,200,190,49,221,118,233,255,129,30,42,178,74,
198,73,165,0,157,4,171,56,160,60,48,52,74,233,119,98,162,252,24,156,68,14,101,67,96,155,195,201,254,17,252,195,255,253,26,238,106,111,198,192,233,19,0,12,140,177,47,8,87,185,183,183,247,247,42,187,137,
104,128,171,76,249,82,83,49,225,229,127,172,160,182,64,169,157,78,188,197,45,23,243,35,174,155,238,102,221,28,193,213,73,160,249,157,224,76,6,42,6,244,217,30,180,57,255,0,56,22,28,40,144,3,192,16,78,159,
25,6,145,3,19,4,53,225,174,114,87,249,153,254,23,23,3,27,187,143,161,3,38,237,118,180,185,55,192,237,220,15,157,2,116,19,56,251,235,224,77,255,9,220,145,2,98,62,44,251,240,201,71,30,23,131,157,12,172,
110,2,171,176,203,73,121,48,150,196,85,158,214,194,177,24,21,131,212,62,134,126,150,161,17,119,131,155,74,122,243,83,220,254,23,150,183,254,54,208,174,192,58,8,169,95,27,192,114,0,139,0,227,205,87,162,
96,210,40,155,24,44,37,97,225,33,96,7,97,59,107,136,171,92,211,94,167,39,54,29,14,216,221,13,82,134,20,181,102,51,122,141,239,149,38,161,57,20,104,217,192,114,0,99,253,48,100,27,128,149,129,223,124,43,
74,129,135,162,175,97,216,3,200,141,92,101,37,174,114,109,13,230,214,23,242,126,250,110,6,242,68,20,79,199,11,87,4,35,7,119,91,191,226,148,112,104,41,115,212,193,108,192,108,128,38,131,114,236,70,140,
78,132,174,178,214,49,113,149,107,106,218,132,122,239,233,235,238,15,140,179,27,108,201,211,165,205,113,117,128,131,201,210,228,57,95,81,53,150,25,12,70,108,195,54,248,206,181,24,60,59,142,124,161,92,
179,135,1,113,149,129,8,125,98,107,65,20,12,231,59,63,3,230,113,69,129,219,158,26,186,60,63,240,204,211,204,225,88,74,80,225,222,68,229,86,83,109,47,126,117,232,42,31,17,87,121,182,71,202,96,24,88,107,
142,12,93,244,88,193,143,125,83,177,143,152,46,116,118,183,28,205,20,7,206,28,3,107,40,165,161,200,65,136,166,106,16,169,106,232,124,242,25,100,215,244,124,61,48,226,42,159,123,164,211,154,97,140,141,
6,251,226,233,241,174,127,10,2,187,95,113,17,153,248,232,214,152,191,231,108,238,172,41,0,14,180,114,160,148,3,69,78,196,255,42,224,212,97,20,71,31,248,225,88,112,209,174,244,166,247,255,161,246,58,196,
85,174,253,193,102,108,204,194,209,150,217,250,96,46,158,205,117,29,157,40,197,255,9,182,52,161,57,175,186,210,131,219,188,194,158,103,184,168,12,145,11,165,66,83,89,145,6,78,31,69,113,244,193,135,199,
252,141,223,5,120,2,224,124,211,150,223,20,87,249,156,3,157,181,6,129,241,217,152,50,27,76,156,25,95,243,131,114,25,63,212,54,103,61,26,75,175,73,30,108,45,15,60,117,47,202,202,16,42,174,242,81,148,70,
239,127,108,220,95,247,77,176,29,132,229,97,113,149,163,154,113,166,39,1,32,149,76,58,228,186,30,105,39,9,165,154,178,169,193,109,27,218,250,254,220,117,157,171,2,74,6,185,114,230,233,83,197,151,29,77,
175,221,252,250,200,85,126,108,52,184,244,43,226,42,47,28,48,48,147,252,117,156,4,72,53,197,220,114,246,138,181,143,253,99,232,42,199,184,16,164,34,87,249,177,75,71,204,14,113,149,23,17,112,216,40,233,
180,38,173,53,105,39,14,162,56,136,82,189,217,103,175,21,87,249,60,87,114,149,241,216,78,78,26,54,198,176,9,138,108,77,30,204,227,179,184,202,253,226,42,47,135,171,12,148,161,180,184,202,203,56,68,76,
59,232,137,171,188,252,29,60,109,199,65,92,229,198,5,12,64,92,229,134,7,28,133,44,174,242,50,206,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,
128,165,36,96,9,88,2,150,146,128,37,96,9,88,106,89,74,92,229,213,16,176,184,202,13,40,113,149,27,27,174,184,202,13,10,86,92,229,218,18,87,121,254,18,87,89,92,101,113,149,151,20,176,184,202,13,10,88,92,
229,6,46,149,197,85,110,252,52,109,7,196,85,110,76,192,226,42,55,112,12,22,87,185,129,1,139,171,44,174,178,184,202,226,42,207,93,226,42,55,42,96,113,149,27,223,193,226,42,55,42,96,113,149,27,24,176,184,
202,226,42,207,251,132,136,171,44,174,178,184,202,16,87,89,92,229,105,221,43,174,242,84,137,171,44,174,178,184,202,11,46,42,196,85,22,87,25,16,87,185,190,18,87,25,226,42,207,190,223,39,174,242,162,3,22,
87,249,249,119,149,167,84,218,208,85,102,212,190,184,33,174,242,252,53,135,171,92,171,210,78,209,57,195,35,163,209,29,174,253,24,205,219,185,226,42,79,5,156,180,150,207,76,11,151,20,64,186,122,27,56,61,
8,210,110,248,255,160,240,50,205,51,66,158,217,201,75,234,142,23,128,171,92,239,171,202,53,174,178,106,87,202,57,10,224,218,144,211,137,164,23,232,240,240,74,132,227,39,78,35,158,72,0,228,128,137,1,246,
1,182,80,74,99,219,182,45,200,102,219,96,140,193,195,15,63,12,107,237,130,139,141,249,126,245,151,122,16,172,253,30,43,200,85,38,48,171,30,86,206,179,4,6,193,1,147,6,83,228,194,69,29,123,228,216,0,94,
252,226,203,65,58,14,27,148,67,122,89,89,188,253,237,111,5,129,17,143,79,105,88,15,61,244,80,221,99,237,98,198,213,197,236,91,60,239,174,242,209,211,189,3,0,13,3,4,6,54,143,142,217,189,33,247,235,1,42,
30,170,180,186,162,210,54,97,120,60,0,116,18,137,84,91,40,32,234,56,24,26,12,133,100,170,9,157,93,107,176,101,203,22,188,239,125,239,19,87,25,0,142,157,238,5,128,71,1,2,41,189,45,221,117,195,126,168,88,
57,12,54,5,142,68,218,202,109,178,228,33,95,214,136,37,219,65,186,9,172,18,112,189,38,88,86,72,167,155,209,189,166,27,91,182,132,128,117,197,84,94,205,174,50,57,217,118,178,236,254,152,73,131,160,182,
166,155,90,218,12,210,251,89,165,0,221,12,118,90,192,78,43,216,205,130,221,118,88,221,138,67,199,125,120,169,14,176,110,130,69,28,229,64,161,239,196,0,226,201,20,218,59,58,145,109,111,199,7,63,248,107,
211,14,120,23,162,171,92,207,24,76,28,248,228,115,236,62,15,190,97,114,116,83,124,226,245,167,10,241,125,217,214,228,139,160,211,128,106,2,235,4,64,30,144,122,37,208,116,49,78,36,9,91,248,171,112,98,19,
240,153,64,90,227,161,135,247,224,233,167,159,198,218,238,44,246,63,189,55,146,31,8,198,4,211,150,191,245,28,228,234,25,183,235,117,149,51,153,204,188,159,119,237,181,215,222,63,56,56,216,56,87,153,141,
161,241,66,118,127,54,125,230,56,160,123,99,78,240,170,120,230,178,111,67,157,45,179,110,246,56,118,9,56,243,38,32,109,1,15,32,248,40,89,194,184,233,129,142,231,225,7,22,108,0,70,128,241,201,60,38,246,
31,134,229,16,244,51,129,184,202,128,181,52,86,236,240,91,83,35,63,85,202,233,85,138,46,211,110,250,158,82,236,149,101,183,107,135,135,166,0,224,0,12,11,88,6,83,56,250,244,171,107,209,162,79,3,78,9,12,
3,5,11,24,3,99,125,16,52,140,241,197,85,6,66,242,119,32,183,169,84,50,201,159,48,145,1,180,147,205,240,21,186,99,187,79,205,6,12,19,97,137,1,44,251,176,214,7,115,0,31,1,10,216,4,214,105,176,74,131,85,
28,172,98,32,229,206,226,42,83,221,93,122,65,186,202,0,144,47,55,63,109,216,57,200,80,72,197,252,23,231,7,15,60,105,13,71,56,170,133,229,176,59,141,45,195,216,50,44,7,152,108,186,12,69,155,66,217,120,
48,72,192,34,22,186,202,106,166,171,188,60,27,55,245,4,189,210,92,229,106,157,158,216,112,36,48,238,83,0,27,165,108,166,57,246,52,252,241,114,129,129,80,159,141,152,95,107,77,21,175,102,178,240,211,175,
67,49,112,81,10,20,12,187,161,171,172,197,85,158,58,200,69,34,237,120,169,53,151,43,53,221,3,70,81,81,224,54,199,243,155,130,209,163,7,217,0,68,4,2,192,96,0,182,10,89,51,27,112,147,133,239,189,4,99,19,
126,141,171,172,170,155,67,54,58,141,97,181,186,202,83,34,45,96,158,61,251,162,251,3,163,158,36,4,148,112,242,107,99,116,188,20,76,250,197,104,61,12,85,221,61,3,106,113,152,216,134,109,40,59,215,96,112,
176,226,42,79,255,241,171,218,85,38,173,153,20,49,216,6,96,46,157,205,181,127,158,173,153,80,40,185,237,169,225,77,249,129,3,7,1,93,85,104,85,132,165,210,180,176,129,182,43,94,141,130,159,198,161,35,199,
197,85,158,54,13,30,27,179,204,108,216,88,31,204,165,227,195,155,30,45,150,156,255,167,184,136,184,206,181,175,201,244,37,74,3,131,39,9,14,148,242,160,148,11,138,232,223,208,85,14,103,9,124,242,25,100,
187,214,125,61,176,158,184,202,181,1,3,0,251,190,5,81,0,230,50,216,78,14,140,119,124,37,8,204,179,10,121,180,196,71,122,189,242,190,193,220,89,83,36,114,160,148,11,173,220,26,87,89,3,253,21,87,121,219,
174,244,198,255,36,174,242,108,195,4,216,26,54,166,4,160,112,54,215,126,112,178,16,251,170,50,133,9,135,115,106,77,211,224,102,175,176,247,25,46,106,163,200,13,187,88,92,229,186,3,182,102,108,204,178,
49,38,236,98,91,98,75,185,129,137,206,239,151,202,252,67,197,147,161,171,156,58,216,82,62,253,212,125,83,174,178,43,174,242,121,184,202,41,16,53,101,211,195,219,54,180,29,255,51,215,213,87,5,42,25,228,
74,153,167,79,22,95,118,172,105,237,230,215,137,171,44,174,242,10,118,149,73,57,228,186,49,113,149,151,113,169,92,57,210,218,201,73,195,108,3,246,253,146,184,202,203,243,162,231,57,211,182,170,171,108,
137,197,85,94,222,33,98,218,65,79,92,229,229,239,224,218,78,22,87,185,129,1,3,16,87,185,225,1,71,33,139,171,188,140,179,8,41,9,88,2,150,128,165,36,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,36,96,
9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,90,150,18,87,121,53,4,44,174,114,3,74,92,229,198,134,43,174,114,131,130,21,87,185,182,196,85,158,191,196,85,22,87,89,92,229,37,5,44,174,
114,131,2,22,87,185,129,75,101,113,149,27,63,77,219,1,113,149,27,19,176,184,202,13,28,131,197,85,110,96,192,226,42,139,171,44,174,178,184,202,115,151,184,202,141,10,88,92,229,198,119,176,184,202,141,10,
88,92,229,6,6,44,174,178,184,202,243,62,33,226,42,139,171,44,174,50,196,85,22,87,121,90,247,138,171,60,85,226,42,139,171,44,174,242,130,139,10,113,149,197,85,6,196,85,174,175,196,85,134,184,202,179,239,
247,137,171,188,232,128,231,112,149,105,142,27,144,203,229,23,220,123,23,87,121,150,98,38,154,30,166,58,231,86,42,251,145,171,92,249,55,138,254,127,238,5,198,133,234,42,215,251,154,92,180,145,64,78,120,
83,225,2,158,34,136,36,98,118,42,29,59,60,60,6,210,78,244,113,6,184,246,210,150,188,170,92,229,122,3,30,141,194,115,24,186,201,2,131,74,57,27,9,26,28,17,59,76,83,234,203,233,129,33,16,121,0,185,96,88,
176,245,1,16,46,189,244,18,244,244,244,160,92,46,97,108,108,12,187,119,239,22,87,57,252,230,250,172,82,22,128,34,134,238,80,42,209,7,162,171,153,52,64,33,80,77,80,97,216,0,142,157,56,139,68,34,13,38,15,
4,3,38,70,215,154,118,188,120,199,14,104,165,144,76,38,144,76,134,219,24,79,60,241,196,5,237,42,215,133,166,250,54,118,28,80,96,34,48,156,117,86,53,31,133,138,129,84,18,208,33,156,106,157,76,164,210,102,
112,244,196,40,156,88,51,148,147,134,129,7,144,135,117,235,54,128,89,33,149,74,99,77,119,55,182,110,13,85,218,197,140,181,139,25,87,23,179,111,17,4,1,18,137,5,55,200,26,230,42,211,222,83,215,157,97,162,
97,192,1,148,222,60,146,79,29,132,74,130,117,19,224,180,128,157,22,144,211,6,184,89,192,205,98,120,146,96,41,141,88,170,29,80,41,144,147,132,23,75,193,245,98,104,203,102,177,126,253,122,92,122,233,165,
115,142,183,171,201,85,38,39,219,78,121,110,35,102,253,56,160,65,228,108,246,90,95,114,146,117,218,103,167,25,172,91,166,184,95,55,11,142,93,130,92,226,55,48,89,114,224,37,59,66,87,153,146,248,201,195,
79,97,96,96,24,205,205,173,232,233,89,143,161,161,97,252,251,191,127,243,130,119,149,23,26,131,137,3,159,72,107,109,216,251,169,163,112,51,148,234,141,39,98,157,166,216,214,167,117,108,51,116,19,16,187,
24,182,233,85,64,210,9,183,162,137,112,116,226,77,240,18,95,130,213,147,0,51,200,6,184,231,7,15,226,201,39,159,196,192,233,19,80,138,163,169,91,200,252,173,90,87,153,141,33,144,162,98,144,126,32,165,11,
6,112,117,107,114,252,101,199,39,182,15,180,174,253,133,205,156,54,64,60,8,93,79,248,209,172,76,161,223,183,216,232,181,64,199,198,17,88,3,69,12,88,198,233,51,195,208,218,3,216,159,53,220,11,205,85,174,
71,195,35,82,74,157,205,173,59,100,225,156,100,114,144,136,149,175,75,172,121,77,192,109,22,136,27,88,54,161,223,201,161,227,105,217,71,201,6,152,12,58,161,99,45,128,219,12,214,73,24,196,160,156,4,64,
26,97,243,18,140,121,97,184,202,8,223,59,12,44,210,85,174,207,244,100,75,185,32,83,10,172,247,56,160,224,104,187,217,230,15,230,252,162,101,102,11,176,1,115,0,99,202,48,166,4,107,203,96,4,56,67,47,130,
213,105,192,105,2,83,10,74,39,1,138,193,176,6,72,195,24,158,22,238,114,108,220,212,187,192,88,140,171,188,125,251,246,0,225,137,141,149,225,97,249,217,223,177,98,182,88,12,82,15,1,100,8,164,91,147,199,
218,41,40,229,67,38,149,67,137,150,3,152,200,85,54,182,12,31,6,69,172,7,84,10,172,147,176,42,1,86,113,16,121,48,70,92,229,169,35,29,41,6,179,157,44,165,247,27,75,71,1,139,166,120,113,107,97,240,200,97,
182,149,5,240,148,66,91,193,171,45,251,200,165,47,66,193,36,80,54,30,44,226,176,112,17,176,22,87,185,230,94,51,91,107,193,204,67,185,238,131,129,113,246,18,172,113,148,159,106,242,142,248,254,164,95,2,
40,116,59,9,85,196,58,188,5,96,50,8,210,175,69,209,119,106,92,101,15,90,199,196,85,6,34,106,210,24,11,176,153,44,55,77,78,22,83,187,216,154,18,161,228,52,199,243,107,130,209,227,199,217,168,208,238,132,
138,54,120,0,192,130,97,97,217,134,174,114,236,250,200,85,102,104,29,13,17,226,42,135,34,45,28,109,43,34,237,193,179,23,223,107,2,126,74,113,137,146,238,100,103,12,39,11,38,87,46,42,229,212,32,169,209,
198,79,205,84,60,182,126,27,202,206,213,179,184,202,20,61,160,213,233,42,207,20,105,125,176,45,156,157,108,251,60,217,242,132,230,162,211,145,30,89,151,31,56,116,132,160,67,141,150,28,40,10,69,218,112,
19,104,33,87,217,174,122,87,185,42,210,130,185,12,160,208,55,178,225,145,66,73,223,165,56,143,184,158,104,91,147,57,17,43,14,156,61,69,112,161,84,12,164,92,80,45,251,43,174,242,252,13,98,198,198,44,251,
190,101,107,124,48,151,192,156,63,61,222,25,186,202,156,67,75,124,100,83,172,188,183,234,42,107,229,65,43,23,90,137,171,92,191,171,236,186,22,64,192,198,20,1,228,134,114,217,131,147,5,47,114,149,39,85,
87,211,217,94,113,149,151,30,240,76,87,185,200,150,38,7,38,58,119,133,174,114,78,92,229,57,119,203,196,85,22,87,89,92,101,113,149,207,255,32,87,123,164,21,87,185,254,18,87,25,226,42,175,216,18,87,121,
5,212,114,92,82,198,2,168,219,85,174,12,9,47,228,206,125,174,3,198,74,25,75,87,98,201,21,176,37,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,36,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,36,
96,9,88,2,150,8,36,96,9,88,74,2,126,222,74,92,229,213,16,240,133,236,42,159,55,22,117,158,161,110,3,150,197,85,174,173,170,171,12,160,161,174,242,115,34,34,46,49,220,70,186,202,181,161,7,136,92,101,132,
103,105,126,239,130,14,56,210,84,22,235,42,95,90,135,171,252,216,60,174,114,37,232,73,132,38,198,178,185,202,43,34,224,232,96,37,174,114,35,2,142,206,127,16,87,185,17,1,71,82,162,184,202,141,8,248,166,
29,187,196,85,110,68,192,17,151,187,3,226,42,47,255,82,57,10,87,92,229,6,174,228,118,64,92,229,198,12,17,209,152,251,48,166,187,202,127,130,144,109,92,169,174,114,165,30,6,240,190,149,116,42,237,108,179,
5,113,149,27,17,176,184,202,226,42,139,171,44,174,242,220,37,174,114,163,2,22,87,249,60,3,110,217,248,241,133,14,112,226,42,55,170,131,197,85,110,96,192,226,42,139,171,60,239,19,34,174,178,184,202,226,
42,67,92,101,113,149,167,117,175,184,202,83,37,174,178,184,202,226,42,47,184,168,16,87,89,92,101,64,92,229,250,234,5,226,42,159,171,210,134,174,242,233,5,187,103,174,131,221,133,228,42,215,187,225,222,
82,153,22,50,83,65,41,221,57,61,220,233,213,212,156,137,92,101,53,141,52,187,128,92,101,23,117,186,202,245,6,28,109,69,82,192,80,6,80,225,187,28,137,166,126,9,8,213,32,217,2,147,185,66,205,19,192,117,
117,238,92,97,175,64,87,217,69,157,239,241,168,51,96,154,186,216,39,59,10,164,41,164,126,21,120,154,254,18,86,161,108,96,172,2,43,23,196,6,224,202,175,242,148,31,183,144,171,92,207,6,208,108,29,253,66,
116,149,137,153,138,97,196,228,24,120,113,144,91,8,169,95,29,50,203,208,53,207,1,99,104,120,2,164,61,16,185,0,20,24,1,64,140,27,94,114,29,210,233,52,242,249,28,14,31,62,140,83,167,78,205,26,232,133,228,
42,215,133,166,50,212,56,64,96,210,14,195,205,24,196,134,160,18,225,72,17,121,113,208,153,232,214,132,83,3,99,96,10,105,51,144,11,102,141,151,220,112,3,54,108,220,136,174,174,46,92,126,249,229,120,231,
59,223,57,173,147,151,244,106,193,5,224,42,147,147,109,39,99,189,33,114,2,16,52,89,120,107,224,100,6,160,16,218,112,42,6,38,15,83,167,86,24,28,63,53,142,100,188,9,160,36,88,41,176,101,116,118,173,133,
235,198,209,218,210,140,206,206,118,116,117,117,225,202,43,175,92,245,174,50,217,82,81,77,250,45,39,9,10,76,14,160,156,117,86,181,159,134,110,2,116,11,216,105,5,156,44,224,69,55,39,139,99,167,45,200,205,
0,78,19,44,194,39,225,232,145,19,72,36,83,232,236,234,194,182,109,23,225,178,203,46,155,115,3,190,222,237,202,23,186,171,28,142,186,174,171,158,61,115,245,89,134,51,202,112,65,228,108,28,47,103,7,160,
155,0,39,3,118,218,192,222,38,112,230,109,176,93,31,2,247,254,30,198,178,239,131,161,20,98,169,118,176,74,129,116,18,143,239,217,143,68,50,133,246,246,14,172,237,233,193,151,191,252,79,8,2,115,193,187,
202,243,14,17,58,147,81,176,150,2,229,41,3,231,41,82,250,70,98,103,163,74,93,252,77,206,92,105,57,189,89,113,50,0,197,66,160,15,240,193,68,40,24,141,201,146,11,55,158,5,242,5,176,97,192,250,248,155,143,
127,6,27,122,58,208,215,119,4,224,74,39,234,121,55,212,151,178,7,92,239,2,163,50,60,52,210,85,94,184,131,181,86,32,104,223,198,31,5,28,40,173,122,60,207,105,55,237,189,6,173,6,240,12,172,173,104,180,1,
216,134,174,231,137,220,75,225,196,91,192,58,13,67,113,176,74,64,57,9,244,157,60,13,144,134,210,186,102,17,130,11,214,85,158,247,187,178,49,196,150,9,32,53,89,106,121,24,164,45,160,169,53,53,121,197,228,
64,97,28,48,224,232,86,129,82,141,245,97,217,224,140,31,0,110,19,180,215,2,86,21,145,214,3,147,11,165,93,212,98,125,51,93,229,122,143,254,11,141,219,245,186,202,201,228,252,231,222,92,123,237,181,247,
35,60,61,172,1,174,114,84,67,249,53,135,12,171,126,6,33,21,43,95,234,122,35,3,214,112,180,136,8,106,160,212,18,140,45,163,108,13,242,166,13,218,107,2,57,105,88,149,128,133,7,173,227,0,52,194,231,109,149,
187,202,164,53,71,11,57,46,5,137,66,96,188,167,0,192,115,252,181,156,63,57,25,148,195,55,191,134,118,167,1,179,31,134,109,203,176,92,198,32,93,12,214,73,176,78,3,148,4,233,36,152,60,24,118,0,82,171,222,
85,230,232,247,215,130,45,79,150,155,115,133,114,252,17,192,26,130,213,45,201,147,105,242,253,82,237,167,87,37,218,72,10,247,17,160,136,181,128,74,70,174,114,50,114,149,93,113,149,1,192,140,141,89,40,
226,112,160,229,32,87,78,63,109,13,250,8,62,154,226,249,117,197,179,199,251,192,10,4,85,181,59,25,28,17,236,161,177,156,75,109,67,209,196,81,54,46,44,60,113,149,103,118,48,251,190,229,208,86,247,207,78,
182,31,240,141,218,75,92,182,174,42,37,210,238,241,82,48,25,148,73,69,236,47,213,116,98,197,187,39,131,32,253,26,20,125,45,174,242,28,109,17,118,48,16,20,202,201,241,201,98,98,23,217,160,168,185,232,52,
199,115,217,96,180,239,4,76,133,252,117,34,242,183,242,236,176,184,202,11,124,156,107,68,90,31,64,225,240,217,109,247,250,1,239,33,148,40,233,78,116,196,112,50,103,114,126,41,116,149,221,16,72,157,193,
253,2,226,42,207,61,21,174,17,105,217,218,50,216,230,134,38,91,62,79,166,52,225,112,193,233,72,143,244,132,174,178,19,185,202,225,77,92,229,69,204,131,201,117,45,44,27,10,79,211,202,135,174,178,186,139,
166,92,101,79,92,229,165,7,28,118,177,9,76,141,171,156,59,61,222,249,21,35,174,242,242,4,92,237,98,113,149,27,51,68,96,94,87,25,226,42,207,255,122,219,185,95,59,199,9,48,226,42,207,168,37,159,74,59,207,
25,70,226,42,55,56,224,176,81,196,85,174,59,96,113,149,197,85,126,78,66,126,110,93,229,58,47,196,33,174,178,184,202,207,127,157,247,53,123,210,55,220,104,39,31,188,191,110,87,121,181,4,187,108,1,71,33,
139,171,188,140,179,8,41,9,88,2,150,128,165,36,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,90,150,18,87,121,53,4,124,33,187,202,
207,91,192,87,229,111,11,93,229,29,203,226,42,183,1,120,77,116,171,186,202,55,237,216,213,80,87,121,197,6,124,85,254,182,70,186,202,26,192,214,232,246,11,136,92,229,155,118,236,90,118,87,121,197,5,124,
85,254,182,165,184,202,59,235,112,149,191,63,143,171,188,35,122,18,223,117,211,142,93,203,234,42,215,83,231,243,254,96,0,64,250,134,27,23,10,85,92,229,70,5,124,85,254,182,22,136,171,220,152,128,175,202,
223,38,174,114,163,2,190,42,127,155,184,202,141,8,248,170,252,109,226,42,55,106,169,28,133,43,174,114,3,167,105,59,32,174,114,99,2,142,198,220,207,99,186,171,252,17,0,159,171,23,80,173,173,231,200,85,
38,199,113,174,27,26,26,250,196,208,208,208,251,122,123,123,83,209,252,121,39,128,207,223,180,99,215,53,139,29,147,27,50,6,139,171,220,192,128,197,85,22,87,89,92,101,113,149,231,46,113,149,27,21,176,184,
202,141,239,96,113,149,27,21,176,184,202,13,12,88,92,101,113,149,231,125,66,196,85,22,87,89,92,101,136,171,44,174,242,180,238,21,87,121,170,196,85,22,87,89,92,229,5,23,21,226,42,207,234,42,215,170,180,
226,42,47,37,224,26,67,152,250,1,117,217,84,160,234,156,63,167,92,101,158,119,252,20,87,121,170,42,204,47,91,86,195,214,114,47,160,0,210,145,133,225,68,127,15,9,157,117,235,54,68,174,114,24,248,108,65,
207,119,176,91,117,174,50,51,181,70,104,106,192,172,139,74,187,157,161,74,75,168,188,219,116,10,46,99,164,155,50,24,27,207,1,164,107,52,218,233,23,194,23,87,185,102,76,32,162,56,0,48,169,32,64,92,129,
188,52,136,66,149,182,134,253,13,95,222,179,48,172,48,153,43,1,164,193,32,16,44,192,83,236,239,92,157,187,26,93,101,2,64,204,74,133,131,137,130,177,9,15,42,78,80,26,4,39,162,127,43,224,147,5,216,32,95,
8,16,176,3,144,7,66,0,230,0,174,235,224,210,75,46,66,119,119,55,6,7,207,224,233,167,159,70,62,159,95,245,174,50,169,116,90,27,184,190,134,1,224,58,1,197,211,172,18,69,82,46,172,138,129,200,139,58,56,234,
78,235,227,204,112,1,172,226,33,251,107,75,32,0,47,189,241,165,232,236,104,67,60,30,199,150,45,155,113,217,101,151,225,139,95,252,226,172,129,94,72,174,242,124,1,147,147,109,39,182,70,89,184,19,26,4,38,
215,49,136,103,45,53,79,104,199,5,84,2,76,49,64,185,213,14,38,235,227,212,153,28,152,146,96,74,130,84,40,136,183,119,116,32,145,72,160,189,189,21,107,187,215,160,173,173,13,127,255,247,127,191,170,93,
101,226,192,39,210,90,151,130,228,136,27,203,129,149,75,32,111,13,187,217,17,214,4,232,20,88,37,0,242,0,183,7,112,47,6,123,151,160,111,98,18,73,247,46,64,167,0,195,8,76,0,102,141,84,42,141,158,181,61,
216,184,113,3,178,217,54,184,174,11,107,237,5,237,42,207,59,68,176,49,4,82,52,94,234,56,157,138,149,194,177,93,57,221,38,113,83,89,55,191,12,28,143,129,99,6,112,109,56,12,135,19,13,156,44,165,112,81,188,
25,112,154,97,140,5,81,128,3,7,143,98,227,198,117,232,232,232,68,123,123,59,126,253,215,127,125,222,16,158,107,87,185,169,169,105,161,172,150,228,42,59,11,60,205,4,130,62,62,118,241,80,87,166,127,12,228,
102,180,82,61,19,133,173,35,237,23,197,192,28,0,48,96,107,17,190,67,40,28,14,70,11,140,32,158,132,151,108,67,49,40,131,148,143,199,159,120,6,251,246,238,65,185,52,9,112,80,157,85,24,19,204,59,59,152,109,
74,181,148,113,116,57,92,229,67,135,14,85,248,157,186,93,229,249,77,207,88,140,136,148,178,86,105,195,222,211,12,13,173,169,139,244,225,99,198,90,48,27,176,13,97,62,107,35,52,213,6,40,153,0,121,223,129,
27,107,9,135,9,10,199,234,146,207,211,68,90,165,20,180,214,213,219,204,13,161,249,54,136,22,59,91,152,107,131,232,249,116,149,107,231,18,84,54,137,39,64,10,90,163,211,117,131,132,45,6,166,130,165,90,246,
97,34,246,215,218,50,12,27,244,79,238,132,19,111,6,235,20,44,18,128,138,65,235,56,172,85,226,42,159,59,24,91,158,44,101,30,13,39,186,68,45,201,194,198,220,217,98,62,92,183,133,80,106,165,131,141,245,97,
109,25,103,141,1,57,41,40,183,9,86,39,170,34,109,232,42,59,16,87,25,145,32,96,12,3,176,195,185,246,131,214,170,51,128,69,58,94,220,232,234,177,33,182,225,242,141,153,171,65,115,36,132,151,109,128,188,
105,129,19,107,2,233,84,228,42,187,161,229,41,174,114,237,103,16,3,48,165,192,155,244,3,189,151,216,32,230,148,58,108,225,244,164,41,131,65,149,89,203,84,200,21,134,253,44,109,11,167,113,58,5,168,138,
171,28,19,87,185,230,158,86,200,95,91,240,83,19,5,223,123,152,16,24,197,190,202,36,6,98,228,7,101,34,5,34,85,157,28,218,154,144,125,246,81,68,55,88,39,192,42,17,185,202,49,113,149,171,199,54,173,25,198,
132,27,12,128,159,47,37,247,5,129,61,161,80,70,115,60,183,166,120,182,239,20,177,174,66,169,213,78,230,112,134,97,97,144,75,109,69,201,36,196,85,158,117,157,49,54,102,225,104,203,209,145,107,40,215,126,
192,4,180,47,116,149,139,241,148,219,87,8,114,129,175,34,86,50,244,59,85,229,171,195,144,201,192,79,189,90,92,229,217,2,6,66,184,26,68,1,128,114,161,156,24,157,40,37,119,17,151,139,154,11,78,38,62,217,
26,140,156,232,131,117,161,107,92,101,2,129,65,225,193,79,92,229,5,166,192,83,174,114,25,64,225,200,217,205,63,244,3,187,71,161,72,73,119,178,35,134,83,121,147,243,75,164,166,66,174,112,191,181,205,41,
174,242,28,205,48,195,85,46,1,152,60,59,209,246,5,50,229,9,135,243,78,71,122,120,109,126,224,208,17,5,7,74,121,226,42,47,101,161,49,205,85,102,46,156,24,89,247,83,113,149,151,47,224,153,174,114,49,116,
149,59,196,85,94,174,128,171,93,44,174,114,99,134,8,136,171,44,174,114,244,4,136,171,12,113,149,197,85,134,184,202,203,87,226,42,63,55,33,139,171,188,18,75,92,229,21,80,226,42,175,244,128,163,144,197,
85,94,198,89,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,45,75,137,171,188,26,2,22,87,185,1,37,174,
114,99,195,21,87,185,65,193,138,171,12,136,171,92,111,137,171,44,174,178,184,202,75,10,88,92,229,6,5,44,174,114,3,151,202,226,42,55,126,154,182,3,226,42,55,38,96,113,149,27,56,6,139,171,220,192,128,197,
85,22,87,89,92,101,113,149,231,46,113,149,27,21,176,184,202,141,239,96,113,149,27,21,176,184,202,13,12,88,92,101,113,149,231,125,66,196,85,22,87,89,92,101,136,171,44,174,242,180,238,21,87,121,170,196,
85,22,87,249,130,113,149,251,175,2,40,54,93,166,173,92,46,156,208,209,209,41,174,242,34,59,120,13,64,109,97,128,250,104,107,107,251,149,213,80,171,50,109,244,119,40,172,95,191,1,199,251,78,160,175,239,
36,166,192,84,113,149,231,220,97,178,86,71,7,55,130,101,61,0,165,46,135,114,194,75,137,147,7,162,88,116,243,64,202,197,37,151,94,142,83,167,78,215,96,80,180,168,213,219,106,115,149,9,68,93,128,2,147,226,
0,241,9,203,78,47,200,171,186,157,211,111,49,172,93,183,1,167,78,159,137,64,85,21,94,193,114,158,93,204,213,236,42,43,157,201,40,102,213,202,138,65,208,65,96,19,129,210,137,206,233,42,237,20,78,66,28,
32,153,202,96,124,44,31,146,58,108,64,196,161,167,106,131,5,23,24,245,142,159,23,130,171,92,5,46,152,116,138,192,96,114,141,111,83,49,56,169,20,224,194,42,23,68,110,24,48,115,164,191,248,40,7,10,19,249,
0,76,30,8,101,48,128,76,83,26,151,93,118,9,218,218,218,112,242,228,9,60,242,200,35,171,222,85,14,175,172,106,45,89,56,154,193,96,184,240,145,74,179,74,41,82,241,72,165,117,163,131,27,131,56,0,91,31,19,
5,134,111,189,240,227,150,64,40,225,229,55,191,18,241,152,131,100,50,129,222,222,77,232,234,234,194,221,119,223,189,186,93,101,246,125,165,18,73,199,114,44,96,10,192,42,230,4,72,100,88,183,6,208,49,32,
226,203,42,80,20,91,31,164,90,112,102,216,192,170,80,172,37,48,214,175,91,11,47,22,67,42,149,66,103,103,22,107,186,58,177,125,251,118,220,125,247,221,179,6,186,26,92,229,80,178,143,197,8,204,78,192,94,
206,37,128,201,213,12,175,205,102,222,7,157,220,6,118,12,216,177,128,195,225,119,210,4,82,10,253,57,31,150,14,130,85,10,96,139,83,253,67,48,134,144,201,52,163,123,77,55,182,110,221,140,76,38,51,109,169,
188,164,87,11,94,0,174,242,156,79,155,206,100,20,2,163,0,232,98,144,30,7,57,96,242,136,148,211,65,45,155,60,116,24,160,37,0,82,1,56,230,131,181,15,68,204,250,169,81,23,112,154,194,128,85,28,229,128,112,
228,104,31,226,137,36,58,187,58,65,164,112,203,45,111,158,119,31,162,222,13,157,165,118,110,237,247,123,62,92,101,138,126,119,21,192,52,86,204,158,110,77,77,130,225,64,105,213,110,252,226,36,113,172,137,
217,192,178,1,115,120,103,67,137,75,97,32,7,164,227,41,144,147,132,49,5,40,242,240,216,227,251,176,251,209,135,1,91,142,120,58,91,119,0,243,205,145,23,51,91,120,62,92,229,57,159,106,14,61,50,0,164,6,114,
235,135,44,156,9,144,134,163,185,125,124,160,127,40,228,37,43,96,106,25,38,186,49,27,140,23,13,44,98,240,18,45,128,74,128,116,2,182,50,124,145,70,200,84,134,183,217,186,119,165,186,202,53,187,137,203,
227,42,87,186,153,45,193,88,247,89,128,224,104,110,5,78,244,87,160,84,27,117,177,101,31,28,169,180,37,107,80,240,53,156,88,115,116,32,140,3,228,129,201,153,38,210,174,106,87,153,180,14,193,62,14,239,101,
41,136,61,9,48,28,101,218,92,39,32,91,180,182,150,151,12,29,79,63,50,150,3,156,206,111,135,27,11,103,18,150,98,0,197,160,117,76,92,229,234,8,49,54,198,48,198,18,17,131,217,76,150,210,187,137,173,37,24,
202,164,74,157,133,225,82,17,53,84,42,106,130,102,14,48,108,12,200,73,194,241,210,128,138,87,167,116,226,42,215,132,12,69,161,74,11,4,35,249,182,3,198,98,144,96,208,20,203,119,107,26,31,1,19,40,26,79,
163,1,35,116,60,57,128,111,2,228,77,6,202,75,71,236,111,12,22,78,228,42,59,171,222,85,174,122,158,28,222,43,191,108,220,9,223,168,189,4,31,113,167,152,181,197,51,19,166,76,33,146,90,107,43,87,69,218,0,
103,105,11,184,210,189,42,9,232,84,228,42,107,113,149,163,123,202,0,27,182,54,40,148,18,163,197,178,247,48,113,96,21,202,42,19,31,112,40,8,202,164,42,34,173,138,34,158,234,226,128,3,148,106,92,229,240,
38,174,242,244,3,157,101,67,132,0,64,57,87,140,239,11,2,211,167,184,140,76,60,215,85,28,60,113,146,216,169,161,37,163,144,57,156,194,137,171,188,24,87,153,185,52,156,111,127,214,4,216,71,40,89,87,21,102,
184,202,142,184,202,139,62,200,97,14,87,217,86,92,229,156,184,202,231,187,208,152,223,85,158,16,87,249,60,3,22,87,89,92,229,5,146,18,87,89,92,101,113,149,197,85,22,87,89,92,229,58,2,6,196,85,110,120,192,
97,163,136,171,92,119,192,226,42,139,171,252,156,132,44,174,242,74,44,113,149,87,64,137,171,188,210,3,142,66,22,87,121,25,103,17,82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,
73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,181,44,37,174,242,106,8,88,92,229,6,148,184,202,141,13,87,92,229,6,5,43,174,50,32,174,114,189,37,174,178,184,202,226,42,47,41,96,113,
149,27,20,176,184,202,13,92,42,139,171,220,248,105,218,14,136,171,220,152,128,197,85,110,224,24,44,174,114,3,3,22,87,89,92,101,113,149,197,85,158,187,196,85,110,84,192,226,42,55,190,131,197,85,110,84,
192,226,42,55,48,96,113,149,197,85,158,247,9,17,87,89,92,101,113,149,33,174,178,184,202,211,186,87,92,229,169,170,186,202,198,58,255,175,181,181,237,198,169,139,219,159,123,19,87,121,209,1,83,37,208,253,
167,6,242,69,165,156,157,83,220,239,244,155,231,197,113,211,77,47,199,238,221,143,161,66,234,136,171,60,79,178,29,45,103,52,128,171,34,149,246,240,153,51,167,174,129,210,49,170,200,180,213,155,3,40,141,
246,206,174,26,87,121,126,194,97,181,187,202,4,128,122,215,29,93,11,168,54,38,5,11,175,175,165,37,123,21,40,22,210,57,51,84,90,80,12,235,214,111,196,177,227,39,209,119,162,191,26,112,232,22,45,109,227,
102,182,143,93,40,174,50,233,76,70,105,240,102,166,80,161,53,156,28,34,229,94,30,50,102,41,144,74,1,58,13,168,20,72,37,1,149,192,197,23,111,199,201,254,51,176,172,192,164,166,141,205,226,42,215,124,172,
122,217,68,173,215,16,20,152,92,46,115,83,201,82,172,23,58,13,214,205,176,78,6,208,45,96,39,3,214,205,128,78,99,77,247,58,244,247,15,1,20,94,5,59,108,94,113,149,103,14,13,128,82,154,148,118,45,59,237,
76,12,166,152,41,217,102,151,220,166,118,40,23,76,49,144,10,53,23,130,5,219,0,196,37,120,201,12,70,38,75,96,21,131,181,37,128,52,182,111,191,28,157,29,89,164,211,105,244,247,159,194,131,15,62,56,235,2,
163,222,241,243,5,239,42,87,200,95,144,114,152,157,52,19,192,20,51,101,78,37,161,155,147,172,227,128,142,131,225,130,35,154,4,92,6,219,34,10,101,141,201,188,5,40,14,38,6,193,96,243,150,173,72,38,60,164,
146,9,108,222,220,139,166,166,38,220,115,207,61,231,116,242,106,113,149,137,141,33,114,92,13,34,215,194,117,152,20,12,197,200,32,209,140,206,63,247,88,59,128,178,96,197,128,98,48,69,244,25,41,140,23,158,
129,111,194,3,32,177,193,101,151,109,139,216,223,52,58,59,219,177,166,171,19,27,55,110,196,61,247,220,179,138,93,101,107,137,194,159,170,13,123,6,228,3,20,211,6,94,6,109,142,75,174,137,168,57,27,170,224,
20,210,57,68,26,131,99,73,88,149,0,83,2,160,0,71,143,159,194,197,23,133,212,111,79,79,15,182,110,217,140,193,193,193,57,59,247,66,114,149,231,253,174,204,150,0,104,159,227,121,134,6,147,171,64,186,201,
154,192,50,66,210,204,218,96,218,159,204,1,206,76,182,195,32,17,177,191,30,38,39,74,56,122,244,56,136,20,50,153,12,242,249,60,254,227,127,252,143,171,194,85,158,179,131,41,22,35,88,38,104,82,165,32,62,
206,40,129,225,144,82,42,195,190,95,230,24,37,152,45,172,245,193,209,129,85,145,3,40,96,96,220,65,179,147,8,231,198,54,14,80,1,143,236,222,131,199,118,63,10,107,74,0,91,24,99,231,220,135,88,204,134,206,
82,15,130,181,223,163,145,174,242,66,158,28,129,153,38,138,45,167,67,154,140,224,104,110,182,126,177,88,11,243,89,27,138,180,214,150,193,28,224,108,222,0,58,9,210,201,240,32,168,92,40,237,134,204,100,
244,35,23,234,220,231,218,85,78,36,22,220,32,91,102,87,185,84,98,54,134,1,96,184,208,117,150,153,38,1,130,171,77,211,196,224,224,88,52,132,68,88,170,31,137,180,225,80,49,89,178,176,228,194,75,164,193,
42,1,165,227,176,92,113,149,149,184,202,83,31,37,158,122,146,213,65,130,133,235,152,102,112,255,233,202,59,174,184,242,95,5,232,227,0,37,107,80,242,85,40,210,234,56,152,60,128,92,48,105,40,237,66,92,229,
233,247,48,0,193,148,141,247,20,216,192,85,229,140,171,3,195,37,107,195,209,136,167,98,174,9,249,76,225,210,200,85,142,195,34,6,80,60,236,100,75,226,42,71,247,140,65,138,35,214,208,159,44,165,30,37,24,
75,28,80,115,162,216,90,24,46,151,66,145,86,213,116,114,52,46,91,131,161,192,128,156,56,180,155,2,84,2,208,137,72,7,23,87,25,0,152,180,102,24,99,217,152,0,204,229,145,92,203,126,107,120,80,193,71,115,
188,208,25,186,202,10,10,10,4,5,2,34,89,57,196,171,125,27,160,96,50,208,94,26,208,73,64,197,97,225,137,171,92,157,236,71,134,28,128,0,204,190,111,220,9,223,96,47,224,35,238,20,218,108,113,112,220,250,
225,194,2,149,29,51,198,148,16,14,51,195,85,14,187,88,92,229,202,177,107,234,175,134,153,253,66,57,49,82,44,123,143,40,246,231,118,149,105,106,168,16,87,185,158,121,112,133,154,156,114,149,247,154,192,
244,41,46,137,171,188,12,1,207,234,42,7,226,42,139,171,44,174,178,184,202,83,247,91,92,101,113,149,231,79,74,92,101,113,149,197,85,22,87,89,92,101,113,149,235,8,24,16,87,185,225,1,135,141,34,174,114,221,
1,139,171,44,174,242,115,18,178,184,202,43,177,196,85,94,1,37,174,242,74,15,56,10,89,92,229,101,156,69,72,73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,
44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,212,178,148,184,202,171,33,96,113,149,27,80,226,42,55,54,92,113,149,27,20,172,184,202,128,184,202,245,150,184,202,226,42,139,171,188,164,128,197,85,110,80,
192,226,42,55,112,169,44,174,114,227,167,105,59,32,174,114,99,2,22,87,185,129,99,176,184,202,13,12,88,92,101,113,149,197,85,22,87,121,238,18,87,185,81,1,139,171,220,248,14,22,87,185,81,1,139,171,220,192,
128,197,85,22,87,121,222,39,68,92,101,113,149,197,85,198,115,227,42,211,79,243,249,162,235,121,177,238,115,69,218,169,7,34,174,242,34,2,182,86,111,7,168,23,32,48,171,111,14,13,141,190,28,80,201,217,101,
90,66,119,247,90,113,149,235,12,152,0,40,34,220,12,144,6,20,231,131,204,119,90,179,217,151,133,255,27,221,168,114,11,67,126,231,187,222,133,71,119,239,198,161,67,135,23,213,185,171,205,85,158,186,116,
23,225,103,152,20,152,156,131,253,103,61,171,148,119,5,84,12,160,232,166,98,145,225,233,194,139,39,113,211,203,94,142,221,187,31,71,72,234,104,113,149,231,10,87,165,211,58,166,139,46,72,95,69,112,96,57,
126,116,108,244,196,245,208,137,24,116,18,213,91,197,201,80,49,116,180,119,99,50,151,199,211,207,60,91,237,232,185,182,40,86,179,171,76,58,147,81,164,181,222,216,241,236,122,192,105,101,242,80,230,212,
64,115,115,243,53,208,77,8,111,25,64,55,3,78,115,149,255,93,187,126,51,142,246,157,194,137,19,167,193,208,0,209,52,183,104,53,185,202,115,154,158,78,182,157,216,26,77,164,220,184,83,216,12,114,97,201,
67,217,102,114,112,74,151,135,42,173,27,13,73,12,192,130,216,7,219,18,182,108,187,20,39,79,13,193,176,3,77,64,184,169,95,9,119,225,253,134,165,184,202,75,161,122,102,186,202,11,92,223,253,82,0,143,213,
252,255,198,104,1,178,180,128,57,240,137,180,214,32,114,65,122,157,37,23,80,49,46,218,102,176,154,92,199,142,3,144,23,129,35,0,108,16,66,5,182,128,108,215,58,156,236,59,10,168,88,200,61,88,131,171,175,
189,22,107,215,116,66,41,133,179,103,7,113,223,125,247,77,59,216,213,134,221,72,87,121,182,78,246,60,175,158,85,93,47,128,123,162,110,34,44,213,85,174,188,92,196,190,175,72,59,97,192,208,89,144,7,75,113,
91,230,166,184,94,243,23,107,88,35,252,89,236,3,92,2,184,8,152,2,200,20,224,120,26,99,19,7,0,138,195,178,69,58,221,140,181,107,123,144,76,120,104,74,167,176,121,115,47,186,187,187,241,229,47,127,121,85,
186,202,4,68,166,103,248,251,239,49,57,41,38,134,165,132,241,57,158,70,22,49,232,240,162,170,225,175,191,7,80,12,10,173,32,210,40,248,135,48,81,96,88,149,128,50,1,54,110,218,80,101,127,215,116,175,193,
154,174,78,244,246,246,226,171,95,253,234,57,211,182,11,205,85,158,239,82,255,4,102,2,160,25,142,203,228,194,194,35,75,110,154,97,136,81,81,104,67,110,146,217,135,141,254,109,188,0,248,214,3,84,168,33,
238,221,119,16,198,0,153,76,6,221,221,107,176,109,219,54,20,10,133,57,151,202,243,173,236,230,114,149,235,221,32,154,173,171,159,115,87,57,84,193,153,160,161,1,104,99,221,128,193,96,114,52,72,39,96,13,
179,50,84,193,249,0,14,137,29,98,40,5,12,79,196,96,41,6,166,56,72,21,64,236,224,153,167,15,160,173,165,9,137,68,18,253,253,253,248,208,135,62,52,103,231,94,72,174,114,29,47,122,146,10,172,147,35,4,32,
40,34,168,120,16,88,86,154,169,98,41,131,109,72,151,41,23,196,132,179,185,86,48,197,163,69,136,7,195,10,123,246,237,199,222,189,79,224,179,159,189,35,252,124,16,140,9,46,120,87,121,214,167,205,140,141,
77,189,68,205,150,125,227,141,69,97,147,210,156,160,192,15,106,221,184,240,22,210,191,204,6,103,39,29,144,19,7,171,144,152,4,185,145,227,57,69,77,46,180,15,81,239,134,206,82,59,183,246,251,61,31,174,50,
67,17,179,101,11,34,158,40,53,159,170,92,189,214,213,38,110,252,114,41,156,65,216,41,1,177,98,44,179,193,112,193,130,116,44,28,77,216,5,200,129,214,177,16,76,141,238,223,106,118,149,167,152,51,182,22,
214,154,145,124,219,25,203,152,36,88,184,218,36,115,103,207,142,79,223,158,172,136,180,22,204,1,114,101,11,38,7,110,60,21,10,136,58,14,195,42,156,148,136,171,60,125,189,1,192,48,83,217,24,58,4,4,240,180,
159,102,123,122,0,160,106,190,21,122,61,244,238,13,202,214,160,236,43,184,94,10,170,234,42,59,226,42,207,210,204,33,173,3,14,202,129,179,135,96,224,169,82,179,171,130,50,151,217,134,236,47,69,225,218,
41,106,210,26,156,41,92,2,55,150,132,141,52,90,113,149,207,189,215,21,87,217,2,40,79,22,83,143,16,251,150,184,76,205,201,82,166,48,92,46,85,176,84,34,53,53,186,112,216,197,161,171,156,128,118,147,226,
42,207,214,186,51,92,229,210,72,190,229,153,208,85,14,208,28,207,119,76,185,202,21,253,144,42,243,138,26,87,185,89,92,229,185,62,48,183,171,92,14,93,229,66,173,171,28,209,189,226,42,47,98,154,86,51,14,
139,171,44,174,242,188,31,19,87,121,25,55,110,22,234,106,113,149,197,85,22,87,121,57,3,22,87,89,92,229,5,146,18,87,89,92,101,113,149,197,85,126,1,185,202,243,156,229,41,174,242,140,90,210,169,180,11,145,
191,226,42,55,54,224,176,81,196,85,174,59,96,113,149,197,85,126,78,66,126,238,92,229,69,92,43,66,92,101,113,149,159,255,18,87,121,165,7,28,133,44,174,242,50,206,34,164,36,96,9,88,2,150,146,128,37,96,41,
9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,106,89,74,92,229,213,16,176,184,202,13,40,113,149,27,27,174,184,202,13,10,86,92,229,197,88,114,
128,184,202,207,107,192,226,42,55,48,96,113,149,27,24,176,184,202,13,10,88,92,229,6,46,149,197,85,110,252,52,109,7,196,85,110,76,192,226,42,55,112,12,22,87,185,129,1,139,171,44,174,178,184,202,226,42,
207,93,226,42,55,42,96,113,149,27,223,193,226,42,55,42,96,113,149,27,24,176,184,202,226,42,207,251,132,188,192,92,101,124,195,90,188,3,160,196,108,42,109,165,196,85,174,179,94,114,249,3,73,64,189,18,32,
7,160,242,192,153,209,251,130,192,188,124,166,70,59,245,39,137,171,140,69,168,180,150,98,41,0,47,5,20,24,234,225,92,137,227,94,44,177,102,186,72,235,84,13,79,165,28,124,244,163,226,42,215,173,210,106,
10,182,51,169,77,76,26,190,77,125,103,116,100,236,229,80,94,178,42,210,42,183,170,210,66,57,88,179,182,7,241,68,2,247,220,243,125,4,129,153,179,123,87,179,171,92,85,105,1,56,164,240,74,130,163,1,151,135,
139,235,31,104,105,107,127,41,84,173,74,155,2,162,139,224,131,60,188,227,29,239,196,238,221,143,225,208,225,163,152,75,165,93,237,174,114,85,165,37,199,141,41,162,159,97,114,97,41,113,100,112,178,57,166,
221,212,229,83,50,109,116,115,210,32,149,130,23,111,194,13,63,243,82,60,250,216,83,96,132,210,128,82,161,54,32,174,114,101,51,58,219,78,80,74,147,118,92,77,38,198,202,217,105,41,6,159,155,250,75,185,211,
47,129,206,184,112,90,193,78,27,216,105,3,156,22,64,103,192,78,19,178,29,235,49,62,89,196,51,251,15,131,200,1,215,28,248,22,90,84,172,22,87,153,108,169,168,136,72,3,240,214,52,31,223,8,114,51,80,113,20,
57,51,158,74,39,175,100,183,21,236,182,1,110,59,224,182,131,221,44,216,105,5,235,12,186,214,110,194,177,227,3,232,59,117,22,76,46,136,52,44,87,71,29,113,149,163,123,20,170,78,68,110,194,203,111,142,84,
90,20,77,179,239,116,254,210,53,236,186,161,70,107,6,128,224,12,80,238,7,252,227,32,51,130,245,189,23,225,196,233,81,48,123,32,88,128,2,92,125,205,213,216,220,187,17,108,13,70,70,70,240,248,227,143,163,
191,191,127,81,27,55,23,138,171,124,142,74,171,149,237,102,242,96,41,193,101,155,214,200,232,14,36,12,0,2,115,23,128,46,16,93,1,144,2,7,26,45,107,20,250,251,190,25,30,240,216,64,43,198,250,245,27,194,
49,175,185,25,235,215,175,199,246,237,219,241,145,143,124,100,214,57,240,5,237,42,3,17,154,26,24,5,87,105,16,185,68,42,27,6,28,99,159,227,49,114,172,174,40,136,149,203,254,50,20,8,10,74,3,74,141,96,108,
194,132,174,178,45,99,205,154,44,180,118,145,74,165,208,29,177,191,21,199,226,66,119,149,213,92,29,12,173,21,192,4,102,135,136,18,128,6,195,51,6,110,2,202,82,136,243,5,48,214,135,177,62,172,245,67,110,
3,6,197,226,56,114,69,11,86,49,176,114,113,252,68,63,252,192,84,217,223,45,91,182,32,159,207,159,51,77,155,107,193,177,2,93,101,224,124,92,101,14,153,22,68,99,48,49,148,102,210,96,56,196,80,94,213,42,
138,176,84,107,203,176,214,175,154,158,249,146,69,192,14,194,119,143,58,32,56,184,231,158,31,160,255,244,0,60,215,195,145,35,71,240,159,255,243,127,158,115,169,124,193,187,202,211,251,153,148,101,237,
87,254,14,34,23,204,204,108,35,87,57,0,66,88,7,96,130,130,198,88,222,3,195,141,102,16,46,152,52,38,38,198,241,143,255,248,21,252,227,151,190,8,176,57,167,115,47,84,87,121,225,123,192,204,134,157,201,112,
236,32,82,4,199,26,203,85,98,50,2,250,194,155,129,101,139,209,66,6,76,110,200,254,194,129,181,10,12,93,51,15,158,2,83,87,163,171,204,118,114,210,114,224,51,179,181,96,182,126,224,14,135,7,51,75,154,172,
3,191,246,168,100,67,67,174,134,255,29,201,57,208,78,8,165,66,57,128,210,32,37,174,242,204,145,129,97,217,2,176,19,165,244,73,130,5,177,133,171,3,207,250,229,50,209,116,149,150,97,171,112,234,72,145,65,
142,7,82,30,44,107,0,33,92,18,62,38,113,149,163,89,23,49,40,100,127,199,242,173,103,172,229,73,66,0,207,9,226,185,179,67,227,132,169,255,42,95,82,225,38,39,203,6,128,3,47,158,4,43,15,164,189,144,153,20,
87,185,146,84,5,174,102,3,102,223,50,74,198,224,16,96,224,233,114,138,237,233,126,32,98,38,163,136,171,96,42,44,124,99,80,10,8,174,151,134,82,21,87,89,139,171,60,235,211,30,98,125,126,232,42,7,240,84,
169,217,153,230,42,215,108,69,50,87,15,116,131,133,75,224,196,18,226,42,207,115,175,25,90,219,217,92,229,204,12,87,185,130,87,87,134,9,113,149,235,24,131,23,227,42,43,40,113,149,23,219,193,226,42,135,
37,174,50,196,85,22,87,121,174,46,22,87,89,92,229,69,47,42,196,85,62,207,141,155,249,14,106,226,42,47,195,198,141,184,202,226,42,207,126,176,19,87,121,137,123,214,245,4,92,237,98,113,149,27,51,68,64,92,
101,113,149,163,39,64,92,101,136,171,44,174,50,196,85,94,190,18,87,249,185,9,89,92,229,149,88,226,42,175,128,18,87,121,165,7,28,133,44,174,242,50,206,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,
128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,106,89,74,92,229,213,16,176,184,202,13,40,113,149,27,27,174,184,202,13,10,86,92,101,113,149,235,47,113,
149,197,85,22,87,121,73,1,139,171,220,160,128,197,85,110,224,82,89,92,229,198,79,211,118,64,92,229,198,4,44,174,114,3,199,96,113,149,27,24,176,184,202,226,42,139,171,44,174,242,220,37,174,114,163,2,22,
87,185,241,29,44,174,114,163,2,22,87,185,129,1,159,235,42,171,127,27,29,29,255,37,165,116,199,124,67,175,184,202,117,46,74,140,117,46,6,232,37,145,142,218,127,248,104,223,49,207,139,253,28,166,129,124,
211,9,96,113,149,235,255,60,34,226,29,0,109,7,20,2,235,252,187,101,122,7,72,39,166,68,218,202,245,128,167,2,23,87,121,129,174,69,116,41,175,109,155,79,52,3,116,51,147,118,152,116,185,127,208,252,196,26,
220,4,154,210,104,137,60,144,114,35,2,88,97,219,182,139,208,210,210,138,175,126,245,159,197,85,158,43,224,202,53,121,92,93,78,147,210,47,5,92,24,36,158,200,151,227,25,55,222,212,85,81,105,195,203,135,
87,204,12,23,90,123,248,200,71,62,138,239,124,231,187,56,114,228,232,188,211,227,213,234,42,171,42,156,170,180,155,240,10,87,128,220,13,76,30,10,166,237,254,201,220,196,203,161,211,137,90,145,22,58,21,
137,3,9,116,117,175,131,23,143,227,158,93,63,140,92,229,115,61,207,213,238,42,195,150,138,10,12,7,204,49,215,49,55,51,121,154,85,130,79,231,54,239,107,206,100,175,133,211,138,105,55,157,137,8,224,36,222,
250,214,183,227,145,221,79,226,240,225,99,211,49,147,5,58,119,181,184,202,170,106,42,43,229,66,169,152,82,234,122,166,56,2,78,159,156,40,37,50,78,44,189,153,221,108,168,209,70,42,45,220,48,100,55,222,
138,171,175,127,9,30,123,124,31,64,78,120,45,97,93,113,149,245,170,114,149,231,220,1,97,99,136,28,87,131,200,85,138,83,164,156,23,89,138,161,100,51,195,1,111,122,21,247,188,195,225,114,30,8,246,3,197,
103,1,115,6,176,69,64,229,209,210,230,97,108,188,136,253,207,30,7,195,5,81,168,18,120,94,12,229,114,113,222,69,197,82,93,229,249,14,140,243,13,37,181,174,242,60,215,117,175,117,149,47,65,213,85,254,77,
94,74,192,161,74,235,186,10,150,29,16,185,237,169,211,235,161,156,102,166,56,138,182,57,159,89,123,249,21,104,13,0,118,0,92,6,198,101,128,33,112,89,129,138,26,237,235,134,112,236,196,0,78,158,30,1,148,
11,178,1,174,184,226,50,108,219,178,17,90,17,10,133,2,142,28,57,130,39,158,120,98,209,203,223,133,134,146,243,113,149,231,187,112,254,158,61,123,118,246,246,246,206,116,149,251,151,58,68,16,172,37,82,
74,129,57,150,244,38,55,49,121,176,42,129,146,77,25,229,233,86,203,166,106,122,50,251,96,229,3,137,0,212,106,176,182,183,29,253,103,198,97,57,34,39,161,176,109,219,86,56,142,139,166,230,12,182,110,221,
138,87,191,250,213,243,110,238,44,214,85,94,74,205,116,149,23,168,75,17,190,79,174,82,27,151,58,68,144,206,100,170,151,73,4,145,235,104,94,3,184,96,196,184,108,99,138,93,138,35,34,38,67,75,138,195,203,
136,3,96,34,52,197,9,3,39,39,67,149,214,150,160,181,11,48,77,99,127,187,186,186,230,156,3,95,240,174,114,56,254,42,68,42,173,163,200,102,152,28,88,114,217,88,215,37,135,137,217,34,244,75,76,20,176,3,173,
66,62,85,99,18,227,249,0,76,46,44,8,108,128,161,225,17,172,235,89,131,174,174,46,108,217,178,185,250,64,87,163,171,60,21,52,51,1,80,138,16,103,40,48,180,181,80,14,19,147,173,72,47,236,195,218,0,108,163,
63,193,240,203,19,40,20,13,64,46,66,246,87,225,129,7,30,194,190,167,159,129,82,132,190,190,62,188,249,205,111,94,21,174,242,220,175,163,176,37,210,46,0,16,67,169,240,55,67,1,32,135,104,138,151,100,54,
64,104,212,130,88,131,217,162,228,51,140,173,92,40,91,131,73,161,88,44,225,238,187,190,131,187,239,250,127,48,65,57,116,177,49,183,171,92,207,6,208,108,29,189,210,92,101,53,207,139,39,204,149,59,205,84,
170,249,124,197,54,188,84,37,87,28,207,136,154,180,48,96,88,76,20,92,48,57,96,138,28,207,10,10,69,106,214,78,93,237,174,178,53,86,77,32,28,77,73,43,38,54,149,235,173,115,149,55,3,115,149,156,28,47,54,
135,11,140,40,100,107,1,158,182,154,59,151,254,93,82,119,188,208,93,229,136,240,54,126,224,12,135,252,147,33,173,172,134,111,12,136,170,194,75,24,117,216,197,0,99,52,175,161,29,7,76,78,56,14,71,96,201,
20,53,9,113,149,171,174,50,179,153,44,55,157,36,4,32,54,112,85,224,90,191,92,14,193,84,2,213,122,158,17,96,61,90,96,40,29,3,170,174,178,35,174,242,204,46,158,225,42,15,176,229,28,193,192,115,252,88,238,
236,208,56,17,69,118,92,141,102,24,65,125,147,190,5,72,35,22,79,130,149,43,174,242,57,225,226,92,87,57,48,56,8,4,21,87,249,116,197,85,166,121,93,229,148,184,202,11,254,158,206,238,42,151,196,85,62,223,
105,154,184,202,213,18,87,25,226,42,139,171,60,219,129,174,178,245,35,174,178,184,202,243,126,76,92,229,121,94,166,95,142,5,198,204,161,65,92,229,243,220,184,17,87,89,92,229,169,251,45,174,178,184,202,
243,39,37,174,178,184,202,226,42,139,171,44,174,178,184,202,117,4,12,136,171,220,240,128,195,70,17,87,185,238,128,197,85,22,87,249,57,9,89,92,229,149,88,226,42,175,128,18,87,121,165,7,28,133,44,174,242,
50,206,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,106,89,74,92,229,213,16,176,184,202,13,40,113,
149,27,27,174,184,202,13,10,86,92,101,113,149,235,47,113,149,197,85,22,87,121,73,1,139,171,220,160,128,197,85,110,224,82,89,92,229,198,79,211,118,64,92,229,198,4,44,174,114,3,199,96,113,149,27,24,176,
184,202,226,42,139,171,44,174,242,220,37,174,114,163,2,22,87,185,241,29,92,117,149,125,223,126,28,80,239,134,184,202,203,18,48,189,236,197,63,124,13,64,47,3,136,1,186,127,104,120,244,180,82,234,45,179,
137,180,51,135,5,113,149,231,9,22,128,218,121,233,158,20,160,110,5,84,26,80,57,195,222,55,199,198,39,126,73,105,183,3,213,235,8,43,204,148,105,197,85,94,56,92,82,233,180,142,57,249,75,160,232,134,80,166,
117,206,28,62,62,209,31,243,18,111,33,242,66,145,54,50,51,42,42,45,160,208,209,209,41,174,114,61,225,42,207,115,137,104,39,65,95,10,184,40,153,244,119,1,243,118,86,137,68,85,164,85,49,32,82,105,43,23,
9,189,253,211,183,227,129,7,31,196,221,119,127,71,92,229,249,194,237,72,157,200,104,69,175,100,138,57,172,98,254,201,225,244,30,11,231,103,224,164,35,145,54,13,168,84,40,211,70,180,195,182,109,23,71,174,
242,215,81,44,150,170,99,179,184,202,81,192,21,56,21,32,207,213,166,5,74,95,111,149,7,159,155,158,41,154,100,135,151,200,182,195,105,169,145,105,51,97,208,58,5,229,36,241,23,255,243,47,112,247,119,191,
143,35,71,143,207,123,0,92,141,174,114,245,162,190,164,29,23,128,215,148,24,187,2,202,91,199,148,192,184,223,245,68,177,152,191,129,157,150,24,59,21,153,182,13,112,218,0,221,2,232,52,58,187,55,66,187,
9,236,250,193,253,48,134,171,222,134,184,202,53,221,139,192,40,16,185,80,58,145,240,202,47,99,138,105,75,73,30,200,247,156,108,218,246,167,47,225,206,63,7,55,189,14,97,200,237,96,183,13,236,182,130,157,
12,94,255,198,55,97,247,227,251,112,248,240,169,80,129,169,118,46,205,219,185,23,154,171,188,16,251,171,192,236,130,56,161,21,95,205,20,135,207,205,131,69,63,213,145,104,67,26,42,0,248,74,176,217,9,148,
9,40,40,32,247,4,220,210,247,176,99,231,78,124,231,174,187,66,149,22,46,20,49,172,97,172,95,223,141,147,39,79,78,187,94,112,61,174,114,189,188,195,124,20,240,92,174,242,153,51,103,22,26,219,119,38,18,
137,92,161,80,216,143,144,253,61,127,87,25,214,18,56,100,127,137,56,161,181,186,204,82,12,37,147,30,103,181,254,106,80,13,154,74,0,226,4,36,20,40,251,98,52,167,175,196,232,196,32,14,28,238,175,178,147,
128,143,183,191,237,109,32,10,73,180,195,135,15,227,161,135,30,186,224,93,101,53,215,248,75,174,27,41,36,112,219,82,67,107,161,156,52,83,28,5,147,42,54,117,118,175,11,175,49,23,132,55,14,192,85,165,214,
160,45,101,113,226,212,8,250,7,198,192,112,0,82,184,252,242,237,96,2,82,233,52,186,187,187,241,218,215,190,246,156,29,174,243,113,149,23,235,124,86,215,196,53,174,242,124,181,103,207,158,157,8,223,202,
5,76,185,202,231,113,144,179,76,164,72,129,200,77,121,185,141,128,3,134,11,223,196,44,121,78,154,43,174,178,45,195,216,242,148,80,203,6,29,105,139,51,67,147,176,236,0,42,84,17,55,109,218,136,120,60,142,
108,54,139,141,27,55,98,251,246,237,184,252,242,203,23,181,113,243,66,116,149,103,237,96,149,78,171,80,53,13,81,78,87,7,237,161,209,233,178,111,93,192,81,14,195,128,173,15,195,126,21,175,14,85,90,139,
230,4,227,236,240,100,52,115,208,96,11,156,30,56,131,116,42,141,238,53,221,216,178,101,11,122,122,122,176,123,247,238,89,15,118,141,116,149,231,122,13,112,161,14,142,92,229,33,76,93,31,174,243,252,198,
96,132,236,47,17,57,74,113,51,67,193,66,179,101,173,148,3,50,108,80,161,127,67,112,82,131,200,128,153,225,210,56,242,133,0,128,134,101,128,136,240,240,195,187,65,48,88,215,211,141,145,145,17,188,255,253,
239,159,117,129,81,239,248,249,66,113,149,23,98,127,9,204,164,136,99,209,222,130,101,16,49,129,192,21,149,214,212,62,33,161,99,100,114,40,148,252,115,54,125,126,242,147,135,240,147,31,255,8,96,3,99,130,
5,117,218,185,198,207,21,224,42,187,168,243,61,30,75,217,15,38,32,68,161,0,91,35,33,50,24,97,55,251,129,5,91,170,145,104,85,132,243,205,189,192,152,233,42,215,179,249,190,208,178,120,101,187,202,0,56,
74,197,50,249,209,191,68,142,206,204,217,9,131,153,163,235,177,50,114,37,55,114,138,20,56,234,96,158,103,129,177,58,93,101,82,92,125,188,86,143,131,67,87,89,145,37,54,108,105,26,150,90,19,52,24,19,197,
102,144,138,246,135,41,28,135,33,174,114,77,227,158,227,42,235,161,176,7,45,57,202,42,248,246,28,87,25,21,87,153,67,87,57,196,81,157,106,200,68,74,92,229,105,13,60,171,171,28,156,227,42,79,223,32,11,71,
228,208,85,246,0,229,70,174,178,43,174,242,57,131,106,157,174,50,97,166,74,59,155,171,236,138,171,60,45,92,136,171,60,163,196,85,94,202,2,3,16,87,121,81,27,55,226,42,139,171,60,85,226,42,135,37,174,50,
196,85,22,87,121,206,151,229,196,85,22,87,121,177,139,10,113,149,207,115,227,102,190,131,154,184,202,203,176,113,35,174,178,184,202,179,31,236,196,85,110,192,43,26,181,7,59,113,149,27,23,240,212,88,44,
174,242,226,95,192,20,87,89,92,101,113,149,197,85,62,255,49,120,230,139,162,226,42,215,89,226,42,67,92,229,21,91,226,42,175,128,18,87,121,165,7,28,133,44,174,242,50,206,34,164,36,96,9,88,2,150,146,128,
37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,106,89,74,92,229,213,16,176,184,202,13,40,113,149,27,27,174,184,202,13,10,86,92,101,
113,149,235,47,113,149,197,85,22,87,121,73,1,139,171,220,160,128,197,85,110,224,82,89,92,229,198,79,211,118,64,92,229,198,4,44,174,114,3,199,96,113,149,27,24,176,184,202,226,42,139,171,44,174,242,220,
53,195,85,62,49,34,174,242,249,7,76,0,212,141,87,220,123,45,64,239,143,206,7,223,253,236,129,35,95,9,151,156,212,188,208,117,40,197,85,158,255,115,73,103,50,74,41,254,67,64,181,3,170,80,44,171,191,35,
165,222,13,210,241,233,34,237,20,80,82,27,174,184,202,115,116,173,206,100,148,110,109,117,175,223,112,247,107,152,212,203,152,52,91,184,15,14,142,242,176,86,222,155,137,92,144,114,128,42,239,48,61,100,
113,149,231,9,184,114,77,136,158,230,131,77,80,244,86,130,147,2,220,124,209,180,238,202,77,140,189,155,156,100,59,171,88,120,141,96,138,77,83,105,197,85,94,224,227,149,112,73,105,183,45,53,116,153,34,
231,58,75,49,199,80,226,236,177,211,24,141,37,18,111,128,78,1,186,9,164,146,213,171,93,51,185,32,114,209,222,185,6,239,121,239,123,241,149,175,124,21,123,247,238,91,176,99,235,157,45,204,236,228,23,162,
171,60,117,213,78,134,3,182,113,71,219,43,161,157,139,89,37,144,243,179,63,86,202,127,51,116,38,6,167,21,208,25,176,147,1,84,83,24,178,14,137,135,191,251,228,39,241,224,3,63,198,221,119,127,23,65,96,48,
27,253,91,169,85,231,42,87,224,84,34,242,154,147,99,45,142,230,155,152,226,142,165,100,112,106,60,219,7,114,119,194,109,3,59,109,161,76,235,180,1,78,6,172,155,0,149,194,230,45,151,160,41,211,130,175,126,
253,27,40,150,202,16,87,121,70,247,178,239,43,114,93,7,74,197,92,229,103,149,86,215,88,74,160,196,45,71,185,249,173,59,220,141,31,206,218,204,207,129,189,144,254,69,164,210,194,201,64,57,77,248,163,255,
241,7,248,238,247,238,197,177,99,39,193,164,107,128,146,165,117,238,11,209,85,158,115,121,19,93,128,78,69,112,170,215,146,28,221,78,202,235,182,42,129,177,124,251,225,82,169,105,103,124,189,33,100,95,
4,14,182,3,121,2,38,143,1,147,223,6,65,35,219,209,6,210,49,252,224,190,135,224,7,168,138,49,83,118,6,175,8,87,121,124,124,124,33,48,181,214,85,174,188,119,227,50,212,105,105,204,217,193,108,76,116,21,
41,114,1,36,18,94,241,6,75,158,50,148,224,179,249,236,72,106,173,110,181,28,128,173,15,86,1,144,246,193,221,235,192,91,126,13,188,238,15,113,243,107,127,21,143,63,121,16,71,143,13,132,215,107,39,5,6,97,
205,154,174,121,59,119,213,184,202,225,101,186,16,6,76,20,115,181,221,193,20,71,217,166,71,202,65,172,37,25,183,14,87,201,201,240,39,19,52,72,1,110,11,225,146,139,20,30,184,239,104,104,25,177,7,71,25,
92,119,205,213,88,215,179,6,181,236,239,108,179,136,11,201,85,158,151,253,165,112,34,235,40,69,9,199,161,139,152,98,40,155,196,36,156,53,23,1,161,151,97,172,15,99,74,85,149,150,217,32,21,183,152,200,149,
113,228,232,64,24,58,8,87,93,117,37,214,246,244,32,145,72,162,179,179,19,175,126,245,171,113,195,13,55,44,155,171,188,216,197,199,108,174,242,60,85,235,42,3,85,87,249,124,14,114,150,137,173,85,0,220,76,
98,120,45,17,165,25,46,74,38,94,74,183,119,101,129,48,220,138,76,203,214,135,181,62,192,140,76,130,49,48,56,142,129,193,209,232,224,166,80,42,251,112,28,7,173,173,173,88,191,126,61,46,190,248,98,92,124,
241,197,139,94,254,206,53,148,188,208,92,101,48,91,66,232,42,59,77,177,201,158,144,253,117,80,182,94,160,92,47,97,173,1,115,80,149,105,45,66,37,156,193,104,79,51,134,71,115,33,167,19,45,149,79,158,56,
9,215,243,208,210,210,130,13,27,54,96,243,230,205,85,246,183,222,141,155,11,202,85,38,199,165,138,171,236,104,63,203,208,96,104,54,86,51,92,165,25,83,164,14,34,176,154,35,238,172,57,206,24,29,153,172,
153,247,50,6,7,135,240,211,135,30,198,248,248,56,70,70,70,240,107,191,246,107,171,220,85,102,27,13,193,208,90,217,76,4,160,178,97,5,165,161,56,98,127,195,144,25,42,194,162,0,32,238,228,48,86,244,163,111,
51,197,165,238,219,247,52,246,237,125,178,202,254,206,182,192,168,119,252,124,161,184,202,106,129,221,231,240,247,155,224,69,155,106,22,28,162,2,161,169,92,121,50,35,99,185,114,179,185,105,43,183,57,231,
218,179,232,180,245,108,220,172,0,87,25,104,160,171,92,219,230,225,173,246,194,138,12,4,1,215,92,104,113,246,229,177,184,202,168,186,202,12,70,57,74,81,69,139,176,40,187,26,168,47,60,50,2,96,228,75,110,
4,170,86,60,101,113,149,103,27,31,102,113,149,237,44,174,50,205,232,105,198,68,169,73,92,229,121,2,22,87,121,246,239,249,124,186,202,84,211,191,226,42,215,19,176,184,202,51,74,92,101,113,149,197,85,158,
121,175,197,85,142,74,92,101,136,171,44,174,242,108,7,186,232,175,226,42,139,171,60,255,199,196,85,94,198,141,155,133,186,90,92,101,113,149,197,85,94,206,128,197,85,22,87,121,129,164,196,85,22,87,89,92,
101,113,149,197,85,22,87,185,142,128,1,113,149,27,30,112,216,40,226,42,215,29,176,184,202,226,42,63,39,33,139,171,188,18,75,92,229,21,80,226,42,175,244,128,163,144,197,85,94,198,89,132,148,4,44,1,75,192,
82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,45,75,137,171,188,26,2,22,87,185,1,37,174,114,99,195,21,87,185,65,193,138,171,
44,174,114,253,37,174,178,184,202,226,42,47,41,96,113,149,27,20,176,184,202,13,92,42,139,171,220,248,105,218,14,136,171,220,152,128,197,85,110,224,24,44,174,114,3,3,22,87,89,92,229,23,164,171,76,0,40,
48,238,122,71,7,85,87,217,247,205,159,29,60,120,248,245,217,108,219,181,83,123,54,115,215,106,113,149,157,197,6,235,100,219,137,3,159,180,226,15,0,244,146,72,229,249,66,223,137,254,177,108,182,253,231,
80,189,136,101,237,230,216,244,186,253,246,219,177,126,253,122,220,113,199,29,184,251,238,187,167,141,153,245,92,122,124,182,238,158,175,219,139,197,34,124,223,71,107,107,235,130,143,49,114,149,255,189,
183,183,247,115,209,208,87,113,149,255,96,41,1,171,197,132,171,51,25,197,214,56,87,111,216,117,61,20,254,83,196,239,60,241,236,145,241,111,40,197,255,8,229,52,163,122,177,208,153,252,111,88,226,42,207,
17,174,74,167,53,0,135,72,121,174,246,255,128,224,102,1,183,56,81,202,252,189,34,251,11,172,226,113,80,12,80,30,136,188,144,0,166,218,11,127,146,184,202,243,133,75,90,107,114,220,216,139,187,239,251,89,
165,156,27,44,197,216,71,211,163,103,198,147,5,199,241,94,63,37,210,38,193,42,6,38,47,180,254,84,120,41,219,79,137,171,60,123,192,58,147,81,149,112,91,19,131,45,174,107,111,177,228,37,173,74,22,199,253,
174,71,252,226,200,219,224,101,90,173,147,1,116,6,208,105,132,12,112,120,129,124,192,197,47,191,231,61,216,184,113,35,62,245,169,219,113,224,192,193,89,195,91,141,174,114,245,162,146,164,29,23,204,177,
108,106,96,187,210,250,106,86,9,39,64,243,104,255,72,34,136,37,82,47,35,167,13,228,180,69,42,109,75,85,165,37,149,64,123,231,90,252,210,47,253,18,190,242,149,175,97,239,222,167,103,93,49,175,90,87,185,
194,254,2,240,160,116,34,230,250,59,72,121,91,45,37,49,94,234,216,235,117,253,167,119,34,251,187,46,187,107,192,110,59,224,181,129,221,54,132,206,114,19,88,37,241,209,143,125,24,15,252,248,97,124,247,
187,223,71,96,108,117,60,22,87,25,53,236,47,145,155,246,198,91,60,199,190,148,41,174,13,165,204,96,254,226,18,154,156,181,88,155,129,221,246,235,176,221,191,13,78,191,18,236,180,130,157,102,64,55,97,227,
230,139,144,74,103,240,47,255,242,45,20,203,126,213,51,18,87,185,198,245,4,179,3,192,115,116,208,170,53,174,176,148,64,209,100,250,145,222,186,65,199,13,89,246,1,229,3,205,62,184,231,10,240,250,95,3,218,
126,27,240,186,241,91,191,245,107,248,254,15,126,130,163,125,225,111,216,116,177,96,241,157,251,66,116,149,231,12,88,165,211,186,114,145,35,16,197,91,18,35,151,147,118,214,88,138,97,188,212,114,170,92,
78,111,132,10,66,118,210,134,55,230,0,136,5,64,151,139,214,171,254,11,72,187,248,209,131,143,193,24,5,142,230,195,77,205,25,92,122,233,165,152,137,69,61,95,174,50,128,133,230,226,181,174,50,48,229,42,
159,231,16,97,45,133,42,162,114,0,184,169,120,225,90,80,76,49,197,121,36,159,30,79,174,113,211,108,13,172,13,96,108,25,198,250,176,214,68,96,148,197,75,46,102,236,125,230,56,142,159,56,83,93,112,92,127,
253,245,120,221,107,95,139,29,59,118,224,214,91,111,197,37,151,92,50,231,129,237,66,113,149,213,66,179,52,48,187,0,197,61,39,184,194,146,139,178,77,140,249,198,77,147,199,202,112,24,46,91,31,108,203,176,
28,128,217,64,147,197,182,174,49,236,63,112,2,128,6,41,7,241,68,28,235,214,245,192,113,28,100,90,90,112,209,69,219,240,214,183,190,117,214,89,196,66,139,136,249,174,190,58,243,243,22,154,149,164,211,105,
140,143,143,47,116,176,219,153,72,36,114,8,223,16,83,25,38,150,28,240,148,171,172,148,2,145,171,136,227,90,99,43,66,246,55,71,78,231,58,34,27,169,180,1,44,251,85,157,150,153,145,140,51,10,37,31,199,251,
206,128,35,24,170,92,14,55,96,82,233,20,186,187,187,177,109,219,54,92,127,253,245,226,42,3,208,153,228,72,151,82,42,205,112,80,14,220,82,42,219,217,18,73,104,85,109,128,171,236,175,69,115,28,24,30,158,
196,217,193,17,84,28,251,202,21,174,155,154,210,232,234,234,68,111,111,47,158,121,230,153,69,47,127,231,26,74,86,170,171,60,231,110,26,179,37,82,90,129,72,167,189,137,30,134,3,38,13,223,58,129,114,189,
24,87,197,151,208,82,166,154,171,36,118,52,1,227,99,57,88,182,112,96,81,185,130,226,215,191,254,175,184,241,198,235,144,201,52,225,199,15,62,136,127,248,135,127,88,212,198,205,124,174,242,82,168,158,153,
174,242,2,215,119,191,20,192,99,53,255,191,49,90,128,44,58,224,115,92,101,215,49,217,48,66,205,198,58,6,174,214,224,32,98,127,43,18,98,136,247,17,128,76,130,48,54,60,137,10,168,202,214,86,208,8,220,251,
195,123,113,239,15,118,1,224,57,231,192,141,116,149,103,235,100,207,243,234,89,213,245,2,184,39,234,22,66,157,174,242,220,79,59,91,170,78,83,149,105,14,93,101,98,19,114,203,138,107,175,235,57,99,203,55,
225,228,80,44,150,106,134,6,27,253,125,225,5,70,189,227,231,243,236,42,3,13,115,149,81,179,157,192,53,150,114,109,210,156,71,169,228,3,140,104,232,152,189,196,85,94,98,153,192,98,230,139,169,226,42,207,
58,74,44,224,42,207,81,249,146,91,163,115,213,74,91,52,103,231,138,171,60,155,171,76,231,126,121,197,85,14,33,168,25,174,50,213,254,88,113,149,235,116,149,43,223,98,186,171,172,157,89,92,101,37,174,242,
84,3,139,171,60,179,196,85,22,87,89,92,229,89,158,118,113,149,1,113,149,23,191,192,168,12,15,226,42,47,98,227,102,177,47,45,137,171,188,192,2,227,124,166,105,226,42,47,113,227,166,222,5,134,184,202,75,
220,184,169,39,104,113,149,207,99,227,102,161,174,22,87,89,92,101,113,149,151,51,96,113,149,197,85,94,32,41,113,149,197,85,22,87,89,92,101,113,149,197,85,174,35,96,64,92,229,134,7,28,54,138,184,202,117,
7,44,174,178,184,202,207,73,200,226,42,175,196,18,87,121,5,148,184,202,43,61,224,40,100,113,149,151,113,22,33,37,1,75,192,18,176,148,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,
18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,203,82,226,42,175,134,128,197,85,110,64,137,171,220,216,112,197,85,110,80,176,226,42,139,171,92,127,137,171,44,174,178,184,202,75,10,88,92,229,6,5,44,
174,114,3,151,202,226,42,55,126,154,182,3,226,42,55,38,96,113,149,27,56,6,139,171,220,152,14,38,0,116,253,229,15,182,0,234,15,1,126,117,116,64,219,53,58,150,251,20,64,159,3,184,238,112,103,186,202,95,
252,226,23,151,236,42,95,116,209,69,75,113,149,47,7,240,185,241,241,241,223,104,110,110,110,67,232,42,191,26,192,31,220,180,99,215,31,45,117,158,188,148,128,171,50,109,74,13,42,71,155,215,2,120,71,232,
74,242,209,225,145,220,63,12,157,61,253,55,142,163,175,153,190,181,48,123,137,171,60,75,184,145,76,171,55,119,237,219,72,138,126,139,201,105,7,224,23,203,250,127,29,62,178,247,245,217,182,230,107,42,23,
9,157,10,248,220,211,232,86,139,171,188,40,246,87,165,211,154,28,215,37,82,94,194,205,127,64,145,190,22,112,17,112,242,203,39,78,23,138,217,246,142,183,64,185,68,228,206,170,210,86,234,246,219,111,199,
173,183,222,138,59,238,184,3,31,253,232,71,167,201,132,245,92,44,127,182,238,158,173,219,43,95,83,44,22,49,50,50,82,215,227,140,92,229,97,0,159,139,254,173,226,42,163,81,1,87,195,85,158,231,130,57,118,
121,215,67,55,104,77,191,108,41,134,128,146,79,239,239,75,236,114,84,241,211,208,201,166,80,165,141,215,168,180,78,116,141,225,240,26,105,226,42,207,246,172,86,194,13,175,137,152,72,197,115,191,195,20,
107,101,149,44,13,21,186,255,221,81,133,119,64,55,199,160,67,106,114,74,165,141,1,228,70,23,171,83,248,226,151,190,40,174,242,204,143,87,216,95,128,60,16,197,47,233,120,244,103,73,187,215,178,74,112,145,
91,159,25,206,53,43,199,139,191,140,221,54,176,211,10,56,45,97,200,42,13,82,9,128,98,96,114,241,201,79,126,2,142,118,241,183,127,251,9,28,58,116,120,222,31,186,154,92,101,176,239,43,144,210,0,188,116,
124,34,155,112,253,55,128,226,73,131,116,105,180,216,253,44,101,126,249,93,170,233,85,105,184,89,176,155,157,82,105,157,102,176,78,1,42,142,119,191,251,23,177,126,195,70,124,234,211,159,193,129,3,135,
102,13,111,181,186,202,68,174,171,136,194,238,109,75,14,94,166,29,181,195,82,66,251,200,76,14,149,119,100,189,181,238,58,108,126,21,184,231,67,64,250,181,96,167,13,236,182,130,117,51,160,211,104,235,88,
139,119,253,252,59,241,181,175,125,3,251,246,238,175,17,8,176,96,184,23,188,171,172,210,233,176,123,137,92,0,137,164,87,186,66,105,119,147,85,73,140,149,218,142,121,237,219,174,129,10,69,68,78,5,224,117,
219,193,107,63,0,110,122,19,224,52,3,58,133,219,110,251,239,248,201,67,143,225,187,247,220,55,195,85,62,119,90,185,234,92,229,16,78,101,7,68,94,34,86,104,137,185,193,245,150,98,218,80,210,12,23,214,27,
138,233,52,115,0,182,81,200,28,0,201,0,88,187,13,188,254,119,208,115,201,7,145,72,53,225,223,190,121,15,138,229,0,17,153,38,174,114,237,240,16,94,120,18,158,167,253,22,199,193,118,166,24,74,65,250,44,
37,215,183,43,207,80,213,244,180,126,68,255,134,215,111,87,73,139,95,249,143,27,112,255,3,143,227,120,223,0,42,215,10,6,8,107,214,116,45,169,115,95,136,174,178,179,240,240,0,13,192,107,73,140,94,172,181,
234,12,224,97,172,212,52,224,155,244,150,152,10,47,38,101,57,0,192,32,86,128,2,20,43,52,167,45,136,39,240,208,195,123,17,88,64,65,99,237,218,110,92,185,243,114,36,19,30,8,140,221,187,119,99,255,254,253,
115,206,28,158,11,87,121,124,124,28,229,114,121,190,11,54,215,186,202,149,247,110,92,134,58,45,141,5,93,101,82,218,1,145,151,142,231,174,177,112,149,165,24,143,229,19,19,137,78,47,97,173,137,68,218,154,
238,229,80,63,188,114,51,112,232,240,41,156,60,117,166,122,13,225,68,50,17,110,204,164,210,232,233,233,193,59,222,241,14,180,180,180,204,122,96,91,77,174,178,3,192,243,156,224,69,76,46,202,54,62,238,27,
29,87,30,148,101,51,197,253,194,68,212,14,67,19,176,165,115,18,135,142,156,138,30,32,65,41,66,123,54,139,120,60,134,108,123,22,235,215,175,199,246,237,219,209,219,219,187,122,93,229,232,226,191,174,34,
196,180,198,22,64,163,108,220,28,233,206,181,128,13,53,52,54,225,198,14,79,233,179,201,88,184,193,114,234,212,153,232,223,194,143,17,128,100,34,137,174,174,46,244,246,246,162,167,167,7,143,60,242,200,
162,166,98,47,52,87,121,206,49,24,150,9,138,9,85,87,153,210,1,28,148,3,175,148,202,118,118,86,24,135,208,126,177,213,107,182,19,128,230,4,97,108,108,18,103,7,135,161,56,52,230,172,13,240,147,135,30,66,
16,228,145,78,39,224,58,26,191,248,139,191,184,232,229,239,66,67,201,249,184,202,243,93,56,127,207,158,61,59,123,123,123,103,186,202,253,75,13,120,62,87,217,87,174,231,49,76,132,67,85,182,34,163,145,150,
8,29,77,132,201,137,124,21,182,6,130,106,135,255,244,161,159,226,167,63,249,9,0,11,99,130,69,109,220,188,16,93,101,53,215,12,98,46,87,57,176,202,134,174,114,104,190,84,236,228,218,109,201,150,36,48,49,
62,9,192,130,96,193,198,86,158,181,89,247,134,103,78,211,26,233,42,207,245,26,224,66,187,122,145,171,60,84,115,231,27,227,42,91,171,120,186,171,60,61,48,2,33,225,228,195,101,40,87,174,9,26,142,209,83,
23,194,231,106,247,138,171,188,160,171,60,219,43,169,51,93,229,217,63,79,92,229,37,150,184,202,117,6,44,174,242,180,18,87,89,92,101,113,149,107,26,88,92,229,153,37,174,178,184,202,226,42,207,242,180,139,
171,12,136,171,188,248,5,70,101,120,16,87,121,17,27,55,139,125,105,73,92,229,37,108,220,212,59,77,19,87,121,137,27,55,245,46,48,196,85,94,226,198,77,61,65,139,171,124,30,27,55,11,117,181,184,202,226,42,
139,171,188,156,1,139,171,44,174,242,2,73,137,171,44,174,178,184,202,226,42,139,171,44,174,114,29,1,3,226,42,55,60,224,176,81,196,85,174,59,96,113,149,197,85,126,78,66,22,87,121,37,150,184,202,43,160,
196,85,94,233,1,71,33,139,171,188,140,179,8,41,9,88,2,150,128,165,36,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,90,150,18,87,
121,53,4,44,174,114,3,74,92,229,198,134,43,174,114,131,130,21,87,89,92,229,250,75,92,101,113,149,197,85,94,82,192,226,42,55,40,96,113,149,27,184,84,22,87,185,241,211,180,29,16,87,121,89,135,136,234,69,
129,95,188,245,49,175,37,61,250,208,84,184,252,111,135,143,28,187,141,96,63,11,96,165,186,202,149,122,24,192,251,86,210,169,180,213,107,112,233,76,70,93,181,254,123,107,18,78,254,115,8,169,220,18,160,
190,126,248,72,223,199,8,165,47,162,234,42,47,60,213,20,87,185,38,220,202,201,135,151,180,255,180,53,230,148,255,128,73,189,10,32,195,76,63,28,26,51,159,37,50,119,128,213,140,112,231,14,89,92,229,154,
112,85,58,173,1,104,207,49,110,83,98,252,181,68,250,237,4,173,45,232,248,224,8,190,54,62,210,247,49,87,187,87,129,76,116,29,119,27,93,76,223,158,19,180,184,202,51,195,77,38,29,229,121,14,64,222,134,150,
167,54,57,154,127,131,201,107,99,80,48,86,104,254,116,223,201,199,127,54,219,146,188,146,217,2,28,0,48,160,72,139,137,206,112,142,190,21,175,26,87,217,89,76,231,86,194,133,214,137,150,228,200,251,161,
188,171,152,92,20,131,212,183,251,135,173,202,102,59,94,7,4,20,250,70,62,96,125,48,151,163,144,81,13,249,246,219,111,199,250,245,235,113,199,29,119,224,238,187,239,158,54,102,214,115,233,241,217,186,123,
190,110,47,22,139,240,125,31,173,173,173,11,62,214,200,85,254,247,222,222,222,207,1,248,32,166,92,229,63,88,74,192,117,177,191,211,100,90,165,146,91,179,123,110,112,28,245,46,86,73,148,57,115,248,192,
233,246,39,19,206,216,255,128,147,73,193,105,5,116,6,208,149,171,174,38,194,171,96,71,151,249,250,162,184,202,179,76,197,148,210,80,202,3,81,28,64,58,147,24,255,77,168,120,139,161,116,249,76,126,211,67,
177,182,91,222,193,110,155,203,110,22,236,180,1,110,11,160,51,224,74,200,20,3,43,23,127,255,15,95,128,49,22,31,254,240,71,240,212,83,123,196,85,174,12,13,236,251,154,136,60,0,137,173,29,251,94,173,181,
115,165,85,41,206,155,182,99,147,188,179,221,93,219,179,141,183,252,33,184,237,87,67,250,215,201,130,221,86,64,55,131,85,120,237,224,191,254,235,143,64,41,23,159,248,228,167,112,232,208,145,121,127,232,
106,113,149,41,122,84,68,174,235,128,148,155,140,229,218,82,177,210,107,161,98,241,0,105,127,180,212,126,74,101,122,111,32,215,39,120,62,208,213,2,94,247,1,112,243,107,193,58,19,201,180,105,188,253,29,
111,71,79,207,122,220,113,199,23,112,240,224,225,234,69,242,87,189,171,28,142,189,142,6,195,1,144,104,75,14,93,226,104,122,145,81,9,237,115,58,63,238,111,205,56,45,42,197,28,186,158,150,3,112,60,0,119,
95,12,238,254,13,32,190,19,45,109,157,120,203,91,222,132,127,249,198,93,216,247,204,1,76,93,175,114,234,154,193,171,213,85,6,27,67,208,90,33,132,171,99,201,88,97,187,114,156,245,140,56,198,138,205,253,
177,236,230,139,65,81,184,54,148,105,153,3,48,7,160,38,3,218,252,70,252,215,15,253,62,30,217,189,15,63,184,247,199,8,76,120,81,124,158,163,195,86,147,171,92,29,30,192,236,16,145,155,240,242,173,113,47,
184,150,201,211,134,226,118,180,216,21,192,211,113,102,3,99,125,24,91,134,169,210,191,33,247,208,217,98,17,243,114,184,235,187,15,160,88,50,85,216,36,30,79,96,227,198,13,211,186,119,181,185,202,168,14,
15,128,3,34,215,213,65,147,163,113,57,35,134,98,144,28,162,196,218,86,229,89,178,54,0,91,127,74,167,173,18,192,140,183,221,0,60,242,232,51,56,121,114,74,165,185,246,154,107,240,31,222,244,70,220,112,195,
75,240,198,55,190,17,93,93,93,23,180,171,60,103,7,87,134,7,210,33,251,219,154,28,185,68,107,234,176,112,49,94,76,157,49,166,41,203,100,170,161,78,241,146,225,69,153,155,146,128,167,243,120,252,201,253,
48,54,92,89,117,100,219,177,126,195,122,184,142,131,76,75,11,182,111,191,28,175,125,237,107,231,156,57,60,23,174,50,128,133,230,226,181,174,50,48,229,42,159,231,24,108,45,193,24,21,94,253,30,177,116,44,
119,53,147,163,152,60,30,207,199,39,226,29,94,140,107,200,95,176,141,46,144,31,46,217,95,180,129,208,215,55,16,202,180,204,32,54,72,55,37,64,96,52,53,55,161,187,187,27,219,182,109,195,59,222,241,142,89,
15,108,171,195,85,86,74,69,174,178,235,57,193,139,24,46,202,54,54,238,91,237,41,143,84,5,170,230,40,220,10,159,168,21,97,219,154,60,250,142,159,6,193,64,145,129,82,140,35,135,143,160,84,46,161,185,41,
141,158,158,181,216,178,101,11,246,236,217,35,174,178,34,196,28,141,173,128,131,178,241,114,164,59,187,129,138,2,110,107,118,200,194,11,139,167,98,10,108,44,250,79,157,1,113,0,130,65,5,80,253,230,191,
127,19,67,67,67,104,107,109,197,61,247,220,131,223,253,221,223,93,212,84,236,130,117,149,73,81,138,161,81,14,220,200,85,14,189,140,202,160,64,224,104,150,64,200,36,21,38,38,38,49,52,52,4,205,1,96,3,176,
241,1,132,20,229,231,62,247,121,124,238,115,159,61,103,143,248,66,116,149,231,28,34,152,45,65,209,12,87,217,169,113,149,237,57,174,50,0,16,41,116,164,129,252,100,14,28,205,139,167,186,157,171,151,20,95,
104,129,177,88,87,121,41,53,211,85,94,160,46,69,248,62,185,74,109,92,242,16,49,183,171,172,230,112,149,43,112,117,184,74,107,73,17,198,199,39,167,246,131,107,232,223,154,171,216,206,185,192,16,87,249,
28,87,185,118,108,153,207,85,174,60,104,43,174,114,180,251,188,204,174,242,252,34,184,184,202,117,150,184,202,245,110,246,136,171,92,91,226,42,139,171,44,174,114,77,3,139,171,60,179,196,85,22,87,89,92,
229,89,158,118,113,149,1,113,149,23,191,192,168,12,15,226,42,47,98,227,102,177,47,45,137,171,188,132,141,155,122,167,105,226,42,47,113,227,166,222,5,134,184,202,75,220,184,169,39,104,113,149,207,99,227,
102,161,174,22,87,89,92,101,113,149,151,51,96,113,149,197,85,94,32,41,113,149,197,85,22,87,89,92,101,113,149,197,85,174,35,96,64,92,229,134,7,28,54,138,184,202,117,7,44,174,178,184,202,207,73,200,226,
42,175,196,18,87,121,5,148,184,202,43,61,224,40,100,113,149,151,113,22,33,37,1,75,192,18,176,148,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,
4,44,1,75,192,82,203,82,226,42,175,134,128,197,85,110,64,137,171,220,216,112,197,85,110,80,176,226,42,139,171,92,127,137,171,44,174,178,184,202,75,10,88,92,229,6,5,44,174,114,3,150,202,147,15,222,79,147,
15,222,175,54,156,250,180,199,172,94,5,144,184,202,203,52,77,83,0,224,100,219,137,3,159,218,154,135,119,18,225,47,1,188,8,32,6,232,235,163,99,147,127,55,56,120,246,127,139,171,188,184,33,162,122,250,22,
251,190,162,88,140,186,154,250,18,91,179,143,221,15,240,118,0,204,76,223,60,124,244,204,159,105,158,184,29,224,171,235,145,105,107,107,53,187,202,85,223,147,180,214,228,184,206,214,236,238,181,237,201,
83,159,210,100,95,9,160,100,172,254,247,131,199,75,159,140,225,228,103,24,193,37,224,138,74,59,251,53,223,171,191,14,23,168,171,188,164,179,237,85,60,238,0,228,173,205,28,206,174,203,28,250,67,71,153,
119,51,20,44,59,247,159,26,105,250,100,48,177,231,143,9,229,43,192,6,20,93,125,170,226,28,205,22,242,76,87,121,247,238,221,75,118,149,99,177,216,82,92,101,32,188,156,193,111,52,55,55,255,49,66,87,25,0,
110,7,48,167,171,188,156,39,34,78,117,174,227,186,0,185,218,65,50,155,30,124,29,105,125,139,37,79,27,118,78,245,143,164,239,41,77,28,184,205,213,241,203,193,26,224,0,204,62,96,131,240,250,150,108,34,234,
33,28,46,196,85,174,9,184,122,134,189,82,46,148,74,174,105,58,182,45,230,6,239,39,149,104,181,112,205,96,174,243,223,206,158,125,246,149,109,77,77,151,133,65,150,195,155,45,134,91,13,85,82,57,188,159,
239,125,239,123,196,85,174,52,80,116,62,178,130,229,138,76,219,156,77,15,190,71,107,239,69,134,226,152,44,183,62,56,82,222,217,222,210,54,120,3,195,18,177,15,182,101,144,45,0,208,209,117,212,16,134,172,
24,159,250,228,223,98,195,134,117,226,42,87,247,15,125,95,129,225,144,86,46,136,82,27,219,14,95,23,115,249,45,86,165,80,228,214,147,71,134,46,29,77,92,116,253,219,177,238,79,98,28,187,20,236,100,1,167,
5,112,50,128,211,4,86,41,64,133,151,28,255,251,191,255,66,228,42,255,47,124,235,91,223,94,245,174,178,210,153,140,34,215,85,164,148,11,80,2,68,233,182,212,232,7,160,227,205,1,210,254,153,92,207,179,169,
158,235,110,2,2,160,217,7,111,250,57,216,214,95,6,59,173,96,167,22,176,142,227,142,59,110,71,16,48,62,252,209,191,198,83,79,237,21,87,25,136,224,84,82,26,128,7,162,228,150,236,179,175,208,142,190,194,
82,146,11,166,165,191,192,91,90,84,90,55,115,68,78,178,10,128,142,38,112,247,175,130,189,75,193,170,9,172,18,248,243,63,251,35,144,210,248,212,237,159,197,161,195,71,107,142,157,231,214,170,114,149,201,
117,67,180,143,200,75,196,10,45,169,120,241,213,80,94,220,32,25,140,22,51,103,157,150,141,151,145,83,145,105,131,170,74,139,148,1,111,124,3,184,237,55,241,198,55,253,44,186,214,118,227,243,255,231,159,
112,240,208,209,240,66,117,84,185,226,42,213,213,177,245,116,238,92,195,74,109,200,43,206,85,70,96,20,105,229,2,136,103,147,103,47,210,26,151,51,197,149,207,201,194,132,191,33,173,155,84,140,57,136,68,
218,72,166,101,3,134,5,41,139,76,111,18,175,123,221,43,241,173,111,255,16,251,15,28,169,249,145,180,224,112,112,225,187,202,190,175,160,72,115,8,87,123,201,88,241,114,199,209,235,44,98,24,43,54,13,196,
178,155,54,130,44,140,245,193,214,135,97,63,18,106,131,170,241,249,222,87,51,158,218,179,31,63,122,224,81,152,192,86,195,77,165,211,231,12,17,171,207,85,142,26,153,136,220,68,172,208,18,247,202,215,48,
185,218,82,204,142,22,58,2,242,28,207,114,104,41,91,246,195,235,5,35,146,105,1,180,55,3,73,183,128,31,252,240,17,148,74,83,179,133,155,111,126,57,222,240,186,215,225,109,111,123,43,174,185,230,154,106,
184,171,219,85,86,126,179,171,113,57,195,67,49,72,12,171,68,119,134,60,75,33,181,81,195,254,70,251,14,4,224,63,92,77,216,179,247,48,78,159,62,3,66,120,125,164,171,174,122,49,218,178,173,72,38,19,232,94,
187,22,175,125,237,107,176,117,235,214,186,131,186,128,93,229,209,75,212,116,87,185,21,48,209,120,107,170,220,111,69,121,105,74,16,146,94,17,123,247,30,132,9,74,213,101,50,155,0,49,207,67,54,219,134,13,
235,215,85,119,203,196,85,142,231,174,154,114,149,99,227,21,87,57,188,148,56,215,224,169,4,34,133,237,27,20,250,251,207,98,160,255,52,136,253,232,85,163,80,13,143,199,98,104,239,104,199,230,205,155,177,
121,243,102,28,62,124,120,81,83,177,213,227,42,195,86,49,212,176,121,67,89,192,81,10,219,214,20,112,242,68,63,136,125,40,24,168,136,157,124,236,177,221,216,247,116,184,200,72,167,211,248,173,223,250,173,
115,86,88,23,146,171,236,204,59,255,157,225,42,219,89,92,229,74,239,86,108,74,2,33,25,83,128,45,227,116,255,25,16,151,65,40,131,16,0,145,123,255,227,7,127,140,7,238,15,231,239,51,135,134,165,186,202,243,
29,24,231,27,74,106,93,229,121,174,235,94,235,42,95,130,170,171,252,155,188,244,89,132,101,2,207,233,42,55,79,185,202,53,112,53,133,35,74,38,169,144,155,204,97,232,236,80,56,44,112,173,171,108,97,231,
152,247,158,143,171,188,88,231,179,186,38,174,113,149,231,171,61,123,246,236,68,248,86,46,96,202,85,94,250,16,81,175,171,60,181,119,62,229,42,119,166,9,185,201,28,96,253,240,86,153,101,192,138,171,140,
101,114,149,39,198,39,1,4,211,93,101,136,171,92,211,194,13,112,149,141,184,202,51,119,159,197,85,158,42,113,149,235,217,32,154,173,171,197,85,94,226,222,176,184,202,226,42,139,171,92,243,61,197,85,174,
119,159,120,230,247,19,87,89,92,101,113,149,151,54,77,19,87,185,82,226,42,47,118,129,81,25,30,196,85,94,196,198,205,98,95,90,18,87,121,9,27,55,245,78,211,196,85,94,226,198,77,189,11,12,113,149,151,184,
113,83,79,208,226,42,159,199,198,205,66,93,45,174,178,184,202,226,42,47,103,192,226,42,139,171,188,64,82,226,42,139,171,44,174,178,184,202,226,42,139,171,92,71,192,128,184,202,13,15,56,108,20,113,149,
235,14,88,92,101,113,149,159,147,144,197,85,94,137,37,174,242,10,40,113,149,87,122,192,81,200,226,42,47,227,44,66,74,2,150,128,37,96,41,9,88,2,150,146,128,37,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,
165,36,96,9,88,74,2,150,128,37,96,41,9,88,2,150,128,165,150,165,196,85,94,13,1,139,171,220,128,18,87,185,177,225,138,171,220,160,96,197,85,22,87,185,254,18,87,89,92,101,113,149,151,20,176,184,202,203,
24,48,106,174,7,115,217,166,189,78,182,249,236,14,165,236,103,81,117,149,249,184,239,155,63,59,113,226,248,16,196,85,94,244,50,91,233,76,70,171,116,90,175,237,30,138,183,54,141,190,70,41,124,6,160,43,
0,42,1,244,221,145,145,220,239,61,241,196,19,47,5,72,92,229,197,116,108,133,253,5,160,200,113,117,79,182,239,42,237,240,95,48,244,229,0,152,89,125,99,104,204,255,244,232,240,224,135,179,217,246,107,0,
166,169,105,238,243,94,43,214,85,158,126,234,22,195,33,165,220,166,196,104,114,123,199,79,190,71,100,47,3,17,27,171,191,115,240,184,249,72,140,143,126,148,224,239,228,232,18,227,225,21,179,237,130,33,
175,86,87,121,154,169,12,165,60,82,58,182,174,229,208,186,158,230,163,127,163,20,191,140,161,202,190,141,223,115,224,100,252,11,41,222,247,151,100,75,91,129,80,165,37,84,248,95,59,107,200,226,42,215,158,
8,174,148,71,164,18,237,77,103,214,108,104,57,248,187,158,107,223,14,210,84,10,18,143,158,24,109,255,39,93,120,234,3,138,11,23,1,62,200,150,193,54,20,106,169,22,149,170,9,89,92,229,154,43,23,17,133,120,
159,210,104,234,104,26,120,141,227,234,215,91,149,208,198,198,6,79,140,119,63,129,210,51,255,133,156,212,38,176,7,176,15,166,2,64,69,192,42,176,45,86,47,192,8,166,200,85,254,148,184,202,149,161,1,12,7,
196,49,144,74,117,164,79,111,77,121,165,247,176,74,100,44,37,204,233,92,247,3,147,241,183,221,218,210,226,180,97,228,175,192,214,7,217,34,96,92,128,156,234,62,14,71,151,133,127,207,123,126,9,239,249,229,
95,198,143,127,124,97,187,202,245,76,211,66,153,86,107,77,74,185,32,21,135,82,233,206,230,193,119,147,19,187,212,170,52,198,74,93,123,114,222,43,122,51,235,84,27,58,13,184,231,191,129,189,117,53,50,109,
10,80,73,16,197,0,114,240,137,191,253,91,188,229,205,111,193,103,63,251,89,124,244,163,31,155,38,19,214,115,177,252,217,186,123,182,110,175,124,77,177,88,196,200,200,72,125,141,20,186,202,195,0,62,23,
253,91,197,85,70,163,2,70,52,29,115,162,193,63,181,161,229,200,117,49,143,223,100,85,10,69,211,114,166,111,116,67,62,214,157,125,49,171,72,166,77,251,224,13,191,0,78,191,18,208,205,96,213,20,94,24,84,
197,240,153,59,110,71,166,165,21,255,235,35,31,195,183,190,245,255,196,85,158,114,149,181,11,162,24,72,165,179,233,209,247,65,121,77,6,201,224,76,174,243,88,186,123,199,139,129,0,108,67,157,150,109,0,
118,2,160,231,114,112,246,215,163,171,174,38,240,191,255,247,255,68,96,44,254,250,127,127,2,79,237,217,39,174,50,80,227,42,19,121,0,18,155,219,15,220,228,56,250,69,76,73,206,7,153,51,69,172,111,166,148,
147,180,85,153,54,68,83,57,186,118,37,218,53,120,237,239,227,191,253,183,15,2,164,113,199,103,191,136,35,71,142,69,166,154,184,202,83,174,50,224,37,188,66,107,58,86,124,21,43,47,110,40,17,140,21,155,134,
220,204,134,94,210,6,255,63,123,255,29,31,201,113,222,137,255,159,167,170,187,39,207,32,135,197,38,108,226,146,203,176,164,150,89,20,41,74,20,37,81,148,21,108,137,146,236,251,234,28,100,235,252,243,247,
236,11,95,249,78,62,57,159,207,103,203,246,89,62,219,119,166,207,178,45,157,28,101,203,178,149,69,81,34,181,204,203,184,76,155,19,176,8,3,96,6,19,187,187,170,126,127,116,207,96,144,7,88,0,11,236,62,143,
141,23,119,169,37,128,253,76,161,186,170,166,251,121,43,237,66,27,183,78,255,6,61,132,131,11,218,91,223,4,116,119,119,225,243,95,248,18,78,28,63,93,199,75,216,85,110,116,149,137,34,109,241,209,61,210,
194,85,134,162,194,213,177,74,193,239,75,200,148,116,116,109,228,214,201,223,0,243,131,49,72,70,13,238,218,167,240,173,111,31,196,209,163,39,231,24,181,134,93,229,154,171,156,136,86,246,89,150,220,162,
201,65,190,146,28,138,180,111,219,108,72,213,201,95,19,130,213,83,128,42,240,129,59,8,71,142,30,199,147,79,62,15,229,7,222,242,222,43,118,227,157,239,188,23,15,60,240,1,188,245,173,111,173,127,189,203,
219,85,118,202,173,81,199,61,96,200,22,26,17,61,81,238,240,201,182,28,163,253,105,225,214,219,240,16,208,150,34,100,162,85,28,60,248,28,170,213,74,120,244,170,177,163,127,43,82,169,20,210,233,52,174,189,
246,26,252,232,143,254,40,187,202,182,244,82,129,171,108,135,174,114,79,154,108,61,35,220,250,106,18,130,4,222,121,189,192,235,175,159,194,240,208,16,200,184,128,246,17,141,72,196,19,113,36,19,113,116,
118,118,162,191,191,31,247,223,127,127,211,63,226,151,170,171,28,153,238,42,199,135,149,74,183,130,130,54,226,141,211,2,33,8,55,21,149,72,69,171,120,229,229,163,208,94,25,100,60,16,60,148,138,5,140,143,
101,145,74,37,209,219,219,131,221,187,119,35,30,143,195,243,188,69,87,11,151,178,171,236,36,163,197,3,134,108,97,200,49,185,82,36,31,11,93,229,90,191,118,212,84,101,18,32,72,236,219,42,48,60,148,197,200,
249,112,244,154,42,200,4,167,106,223,250,230,55,145,207,229,208,211,221,141,84,42,133,143,127,252,227,75,90,138,109,36,87,217,90,116,56,24,99,129,200,118,44,255,106,131,40,92,29,201,251,90,58,228,144,
208,58,60,231,13,195,13,15,52,33,165,196,21,189,21,28,121,249,60,200,84,33,200,11,143,43,61,8,17,252,217,7,31,252,83,60,248,224,159,204,185,138,88,232,130,182,88,51,231,133,40,224,249,92,229,225,225,225,
197,230,246,235,99,177,88,177,92,46,191,134,128,253,125,227,133,140,224,121,93,101,204,235,42,135,32,101,136,71,37,34,2,208,10,231,7,135,0,184,161,173,236,129,230,56,15,94,41,87,121,169,155,143,185,92,
229,5,170,209,85,6,234,174,242,133,172,34,86,200,85,38,227,5,199,150,202,67,237,69,9,92,101,51,231,161,251,98,219,223,249,166,18,118,149,217,85,158,118,246,203,174,242,236,23,131,93,229,133,86,28,236,
42,131,93,229,166,139,93,229,102,15,123,216,85,110,44,118,149,217,85,102,87,185,97,0,179,171,60,179,216,85,102,87,153,93,229,57,94,118,118,149,1,118,149,151,190,193,168,77,15,236,42,47,225,224,102,169,
111,45,177,171,188,140,131,155,102,151,105,236,42,47,243,224,166,217,13,6,187,202,203,60,184,105,38,104,118,149,47,224,224,102,177,81,205,174,50,187,202,236,42,175,100,192,236,42,179,171,188,72,82,236,
42,179,171,204,174,50,187,202,236,42,179,171,220,68,192,0,187,202,171,30,112,48,80,216,85,110,58,96,118,149,217,85,94,147,144,217,85,94,143,197,174,242,58,40,118,149,215,123,192,97,200,236,42,175,224,
42,130,139,3,230,128,57,96,46,14,152,3,230,226,128,57,96,14,152,139,3,230,128,57,96,46,14,152,3,230,128,185,56,96,14,152,139,3,230,128,57,96,46,14,152,3,230,128,185,86,164,216,85,190,28,2,102,87,121,21,
138,93,229,213,13,151,93,229,85,10,150,93,101,118,149,155,47,118,149,47,37,87,185,112,240,145,105,93,53,110,187,250,209,30,219,242,255,51,160,63,8,118,149,151,31,112,109,179,98,181,119,144,241,61,50,74,
209,142,142,87,156,190,150,99,127,15,152,183,162,238,42,211,159,6,174,178,255,17,118,149,155,156,191,209,160,211,82,36,66,109,241,225,200,142,182,23,174,141,88,165,63,2,204,213,0,20,25,115,166,234,233,
95,63,119,238,76,150,140,255,127,131,205,194,69,221,19,108,8,87,89,160,166,36,2,150,136,197,157,84,36,159,234,111,125,229,109,142,244,254,144,96,93,77,176,170,218,56,223,25,158,48,159,122,241,197,151,
239,32,152,207,2,34,218,108,127,158,85,172,117,239,42,11,0,16,241,184,69,150,109,147,16,54,140,113,182,180,29,59,224,216,254,47,129,156,189,26,48,190,118,190,62,52,30,253,243,242,196,179,191,208,222,209,
177,159,140,34,131,26,152,218,184,228,189,40,181,190,93,101,17,143,91,228,68,108,18,34,10,33,34,49,187,148,185,102,211,19,95,18,36,118,3,194,184,126,244,209,35,131,169,255,147,52,47,252,127,194,148,175,
52,161,68,0,237,7,211,177,105,244,55,102,135,124,121,187,202,211,195,141,119,165,6,182,111,109,57,245,155,210,194,173,6,142,87,246,19,79,28,63,223,242,149,52,29,254,55,100,202,155,97,60,192,84,1,93,9,
63,188,32,100,221,248,28,162,97,87,25,141,207,40,91,86,20,66,36,90,226,99,155,183,183,159,248,15,142,141,247,26,138,162,236,167,94,29,40,28,120,218,182,211,239,20,133,191,234,32,227,194,232,42,72,149,
0,93,6,233,18,140,46,135,244,131,87,247,149,119,239,222,117,217,187,202,181,167,236,37,64,14,132,136,146,48,169,174,244,240,61,182,45,239,209,34,38,60,157,156,24,40,238,61,47,122,15,124,88,36,200,50,195,
255,25,24,251,12,160,75,0,217,128,146,104,148,11,72,43,116,116,182,227,15,255,231,103,208,214,118,121,187,202,4,4,120,31,57,17,73,66,68,0,36,58,18,163,87,36,163,238,15,27,17,75,43,74,233,193,98,207,225,
138,115,227,77,169,152,178,140,1,208,73,208,214,207,130,70,255,8,32,43,252,36,10,48,193,161,216,191,250,127,62,132,127,253,209,15,227,241,199,31,199,111,252,198,95,227,216,177,163,151,173,171,76,117,153,
54,128,83,35,32,145,236,78,143,60,32,45,123,143,166,4,38,170,237,71,170,206,13,155,146,189,50,105,224,79,209,147,45,62,76,207,79,193,200,20,140,76,4,171,52,225,224,183,63,253,27,120,247,187,223,133,63,
121,240,207,240,233,223,254,29,118,149,107,174,178,1,34,32,138,111,105,61,117,99,44,162,222,169,41,134,178,78,103,7,38,122,220,72,119,71,63,200,111,80,105,125,24,248,64,82,193,108,254,56,32,123,0,17,193,
239,124,250,215,144,74,103,240,233,79,127,6,255,242,47,95,131,219,224,200,53,59,98,23,154,143,103,254,122,67,185,202,20,244,236,74,182,39,199,127,220,136,72,74,81,194,31,45,180,159,77,244,92,189,27,228,
7,50,173,118,161,234,50,109,208,17,155,98,26,102,219,79,224,23,127,233,147,240,149,193,31,252,207,63,197,75,47,191,18,244,183,156,71,166,189,92,93,229,232,142,142,163,119,217,150,188,218,80,212,20,189,
212,72,5,125,73,145,176,163,74,7,42,184,50,110,29,81,213,70,213,215,185,31,123,15,32,168,140,191,248,203,47,226,212,201,51,51,118,115,211,175,31,151,185,171,92,126,139,17,118,212,167,184,159,175,36,199,
172,204,150,173,144,83,236,111,29,172,14,217,95,0,184,117,47,208,147,152,196,63,252,227,55,113,242,212,153,96,21,1,51,175,10,190,220,145,59,223,180,210,24,242,134,113,149,61,29,173,20,252,77,49,43,101,
217,186,70,174,215,60,229,154,240,73,132,68,132,112,199,149,6,7,31,123,22,199,143,159,2,133,13,80,174,187,246,106,220,117,215,155,240,161,7,62,128,219,110,187,109,201,171,133,153,35,249,146,118,149,77,
45,220,16,239,11,104,141,192,247,124,207,45,132,211,167,206,224,208,161,23,161,253,10,96,124,100,210,73,236,218,189,3,189,61,61,72,165,211,184,229,150,155,113,219,109,183,177,171,60,159,171,172,103,210,
191,97,59,197,246,164,68,123,194,195,83,79,189,8,175,82,12,232,73,227,97,71,255,102,16,105,36,147,9,116,119,119,97,231,206,157,120,243,155,223,204,174,242,124,174,242,212,193,77,48,45,4,244,175,196,59,
110,16,56,126,236,12,70,135,206,131,76,37,56,147,48,30,206,156,62,5,219,178,208,146,73,99,115,120,160,83,46,151,155,10,234,178,118,149,195,104,65,36,145,138,73,164,162,46,94,123,245,24,180,95,4,233,10,
72,87,161,85,21,35,195,67,56,124,248,37,36,83,73,244,246,246,194,247,125,252,206,239,252,206,162,171,133,141,232,42,91,11,204,240,129,171,44,229,156,174,114,124,171,19,49,198,109,112,149,195,215,139,130,
209,123,245,86,137,145,225,17,12,159,31,4,153,106,125,122,32,227,3,80,120,238,217,103,113,242,248,49,76,76,140,79,91,166,93,44,87,185,182,186,152,171,134,134,134,238,232,239,239,255,40,128,223,194,148,
171,252,232,5,95,228,234,174,50,16,186,202,214,52,87,217,96,182,171,76,36,33,165,192,158,158,10,6,207,133,174,50,2,246,55,112,149,53,4,33,80,187,194,112,155,113,149,103,206,161,243,117,95,157,249,231,
22,91,149,36,147,73,228,243,249,197,46,118,215,199,98,177,34,130,27,98,106,211,196,178,167,136,21,112,149,37,200,132,174,178,97,87,121,238,85,196,178,92,101,98,87,185,217,41,98,121,174,50,177,171,220,
204,20,193,174,242,156,47,6,187,202,11,173,56,216,85,6,187,202,77,23,187,202,205,30,246,176,171,220,88,236,42,179,171,204,174,114,195,0,102,87,121,102,177,171,204,174,50,187,202,115,188,236,236,42,3,236,
42,47,125,131,81,155,30,216,85,94,194,193,205,82,223,90,98,87,121,25,7,55,205,46,211,216,85,94,230,193,77,179,27,12,118,149,151,121,112,211,76,208,236,42,95,192,193,205,98,163,154,93,101,118,149,217,85,
94,201,128,217,85,102,87,121,145,164,216,85,102,87,153,93,101,118,149,217,85,102,87,185,137,128,1,118,149,87,61,224,96,160,176,171,220,116,192,236,42,179,171,188,38,33,179,171,188,30,139,93,229,117,80,
236,42,175,247,128,195,144,217,85,94,193,85,4,23,7,204,1,115,192,92,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,173,72,
177,171,124,57,4,204,174,242,42,20,187,202,171,27,46,187,202,171,20,44,187,202,236,42,55,95,107,198,254,206,36,127,107,117,203,190,131,45,142,229,190,149,200,252,108,48,29,24,128,93,229,229,177,191,50,
147,17,70,41,130,214,68,145,8,93,219,243,221,158,164,51,241,9,65,250,135,130,81,107,0,224,144,231,169,255,145,29,27,29,200,77,140,255,8,187,202,139,252,183,152,231,17,130,214,216,104,244,202,174,39,62,
79,164,223,28,92,116,140,103,12,125,246,200,209,211,95,128,42,60,96,89,226,189,66,16,187,202,139,133,43,146,73,89,147,186,96,140,21,115,10,177,237,173,175,236,111,137,142,255,14,145,222,11,64,25,99,206,
149,170,214,239,156,31,60,63,42,145,125,16,70,71,131,182,184,13,173,201,215,182,214,189,171,60,21,110,208,31,49,2,32,22,177,43,173,59,218,94,123,123,42,86,252,29,35,157,189,90,196,92,215,164,31,31,24,
111,249,189,87,95,63,246,70,41,42,127,12,114,162,181,126,18,141,189,34,214,184,214,189,171,76,13,190,103,132,72,68,65,34,209,215,114,230,166,68,212,253,121,146,177,93,6,210,148,189,216,193,193,137,150,
127,70,241,153,143,181,183,183,93,65,218,37,99,100,128,168,194,15,219,169,224,98,141,228,117,235,42,139,250,156,11,68,136,68,12,66,36,35,182,215,113,237,166,103,255,111,240,40,173,99,202,126,242,133,19,
217,189,15,37,213,35,239,19,40,108,35,93,99,127,203,245,127,146,241,96,140,63,167,175,204,174,178,109,59,16,34,70,66,166,218,83,163,187,183,182,158,254,53,219,182,15,248,20,243,139,110,203,43,3,149,55,
157,76,110,218,241,78,140,188,44,145,255,135,48,212,34,160,10,32,85,12,164,90,93,5,140,27,98,214,6,66,128,93,101,145,76,74,178,108,135,4,197,64,34,145,137,77,108,221,214,126,230,63,68,35,244,46,37,18,
40,122,45,167,7,138,87,30,143,109,186,238,78,17,17,18,0,204,224,49,80,254,203,128,46,0,126,30,228,231,64,170,0,163,139,32,237,194,24,15,187,119,237,192,111,253,214,111,178,171,12,64,6,244,142,112,2,87,
121,228,158,136,67,111,86,34,33,170,38,147,63,95,220,49,110,119,95,115,39,28,45,53,20,96,8,212,187,29,134,238,7,114,255,12,146,42,64,75,66,188,164,163,179,5,127,240,153,79,163,189,173,5,95,251,26,187,
202,83,174,50,81,162,35,49,178,43,25,171,126,80,203,88,202,167,164,30,202,119,28,119,163,215,238,77,68,149,12,26,211,33,108,123,43,33,122,182,3,230,93,64,254,139,48,170,2,162,18,62,242,145,7,240,209,31,
249,33,60,245,228,227,248,111,255,237,111,112,252,216,145,105,125,123,22,89,211,206,26,197,181,95,111,92,87,89,90,114,202,85,166,100,119,122,248,67,150,101,237,209,136,99,162,210,118,178,234,92,221,153,
232,146,49,99,252,80,165,117,167,51,148,189,59,96,82,239,1,9,27,191,252,203,159,192,59,222,249,54,252,159,63,251,75,252,238,239,254,62,142,30,61,202,174,50,4,201,208,85,78,108,105,61,125,99,44,162,223,
161,41,142,138,78,141,157,207,117,86,163,221,29,125,6,30,180,118,3,60,85,123,208,122,6,69,217,183,23,159,250,165,95,66,42,149,194,239,127,230,79,241,213,175,125,179,193,85,166,166,71,236,66,243,241,204,
95,111,68,87,57,209,158,156,248,49,35,35,41,69,49,127,164,208,118,46,209,189,111,135,129,130,214,30,148,9,248,95,109,124,24,168,26,159,10,0,248,217,247,0,198,27,193,131,127,250,5,188,250,202,107,225,40,
155,123,196,94,230,174,178,184,218,32,102,138,94,106,164,138,77,9,145,176,34,90,215,60,229,218,168,53,211,246,15,31,186,147,96,121,195,248,219,191,255,42,78,159,57,135,26,253,59,253,156,125,225,169,225,
50,115,149,99,53,87,121,11,132,15,101,26,232,223,144,39,169,241,103,7,118,9,244,166,74,248,250,215,30,197,217,51,103,16,52,115,85,216,189,123,7,246,238,221,131,253,251,247,179,171,60,219,85,142,85,10,
254,166,184,149,178,236,169,139,89,163,76,27,52,111,78,68,36,110,223,11,60,243,204,75,56,125,234,36,72,87,65,218,195,21,187,183,227,186,107,247,225,134,253,251,113,195,13,251,241,209,143,254,63,75,90,
45,204,28,201,151,156,171,156,171,36,134,34,237,219,250,12,213,70,174,95,183,17,137,68,0,147,144,196,187,111,22,24,28,24,192,139,207,29,134,242,74,117,33,209,104,31,82,18,146,201,56,122,123,123,112,224,
192,129,121,1,235,249,70,238,37,237,42,231,234,174,178,154,38,130,7,110,137,132,32,11,237,41,11,157,9,31,207,62,243,18,188,106,30,66,151,64,186,10,165,170,136,69,108,68,35,17,116,118,117,98,71,127,63,
174,188,242,202,69,127,196,103,142,228,102,167,146,75,192,85,86,83,23,41,18,117,87,249,237,215,75,156,56,126,22,163,67,131,32,93,2,76,37,20,106,61,60,247,252,179,240,188,42,58,59,59,176,101,203,22,104,
173,113,240,224,193,69,131,218,104,174,178,53,239,25,133,82,68,150,45,72,136,105,174,178,95,115,149,77,186,31,20,144,14,122,150,171,108,33,21,179,144,142,185,120,250,181,163,208,94,1,66,87,64,218,133,
82,193,113,37,140,194,159,127,246,207,113,228,245,215,0,152,105,225,174,165,171,156,207,231,225,186,238,66,13,155,27,93,229,218,189,27,87,225,130,217,223,154,171,76,52,167,171,28,235,116,34,245,169,161,
126,96,68,117,87,249,154,173,18,35,195,217,6,87,185,90,31,189,53,153,86,74,129,39,158,120,2,79,60,241,196,146,150,98,171,225,42,47,84,67,67,67,119,0,248,231,240,199,180,230,42,175,192,58,120,201,174,50,
213,93,229,221,243,186,202,1,147,20,188,40,193,7,187,202,43,234,42,43,176,171,204,174,242,172,98,87,121,25,197,174,50,216,85,6,187,202,23,176,138,96,87,121,90,177,171,220,204,1,209,92,163,154,93,229,101,
158,13,179,171,204,174,50,187,202,13,159,147,93,229,102,207,137,103,126,62,118,149,217,85,102,87,121,121,203,52,118,149,107,197,174,242,82,55,24,181,233,129,93,229,37,28,220,44,245,173,37,118,149,151,
113,112,211,236,50,141,93,229,101,30,220,52,187,193,96,87,121,153,7,55,205,4,205,174,242,5,28,220,44,54,170,217,85,102,87,153,93,229,149,12,152,93,101,118,149,23,73,138,93,101,118,149,217,85,102,87,153,
93,101,118,149,155,8,24,96,87,121,213,3,14,6,10,187,202,77,7,204,174,50,187,202,107,18,50,187,202,235,177,216,85,94,7,197,174,242,122,15,56,12,153,93,229,21,92,69,112,113,192,28,48,7,204,197,1,115,192,
92,28,48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,215,138,20,187,202,151,67,192,236,42,175,66,177,171,188,186,225,178,171,188,220,10,133,
90,96,246,227,243,230,77,215,125,231,0,145,249,5,176,171,220,124,213,92,229,70,250,55,124,142,3,70,41,74,56,147,180,175,247,201,190,136,93,249,9,64,255,4,2,161,150,93,229,165,188,72,0,200,106,239,32,93,
173,8,178,109,81,187,25,123,119,231,179,153,182,216,249,187,165,240,127,70,64,133,13,45,76,25,48,143,0,20,186,202,133,31,98,87,121,145,112,107,58,45,12,44,16,36,9,105,109,111,61,220,219,157,56,253,115,
82,168,247,18,116,91,240,85,244,243,85,23,127,48,54,54,118,118,114,226,252,135,29,199,250,65,192,176,171,188,104,184,36,44,88,210,38,34,7,210,178,99,214,100,242,218,158,199,31,20,164,110,71,232,42,251,
218,250,235,163,39,115,127,39,253,129,247,90,210,189,79,144,233,88,75,254,119,99,186,202,241,184,69,78,196,166,224,185,217,104,196,114,211,125,45,199,247,183,39,71,126,77,18,118,3,66,41,35,134,243,165,
228,255,25,201,78,140,69,205,209,255,10,237,69,96,60,80,168,199,92,4,99,121,3,185,202,78,36,24,181,66,196,109,203,107,223,218,126,236,109,109,201,252,175,145,136,238,246,69,202,47,233,142,151,78,143,247,
253,237,169,179,3,183,69,228,196,175,66,36,35,36,98,0,69,96,200,14,123,75,76,117,1,92,163,90,255,174,114,253,65,196,48,92,0,169,77,153,115,55,165,99,229,159,19,50,182,93,147,99,10,110,250,197,161,210,
254,195,82,20,31,104,109,29,236,132,169,18,180,3,163,194,38,72,58,232,237,3,210,13,228,195,154,213,154,185,202,75,13,88,52,160,213,78,208,28,130,82,142,229,117,117,164,115,191,40,100,108,155,79,49,83,
116,51,199,206,85,110,205,38,54,239,120,128,36,89,230,116,17,168,62,15,82,181,182,138,6,4,13,3,5,210,6,134,52,96,166,154,219,173,145,171,76,150,101,221,156,205,102,63,147,205,102,127,188,191,191,63,17,
174,159,175,7,240,167,119,238,127,232,198,165,206,201,23,58,7,207,245,48,120,186,61,153,221,189,181,125,224,215,44,219,190,222,167,148,154,116,91,78,12,150,247,103,147,155,250,111,36,39,104,48,108,170,
4,58,251,87,32,127,0,228,231,0,63,7,168,28,160,138,129,185,108,170,16,48,232,237,237,198,7,63,248,1,118,149,73,80,12,160,100,58,145,223,190,189,237,204,191,139,68,197,125,138,210,40,248,109,131,131,133,
29,67,177,77,251,246,195,129,168,253,216,19,4,80,146,160,193,7,1,127,28,228,141,129,252,9,144,202,195,232,18,118,237,216,140,255,254,155,191,142,104,36,130,103,159,101,87,217,2,201,8,9,147,234,78,13,223,
237,68,232,46,77,73,225,154,244,228,112,113,75,222,233,190,106,63,108,45,130,22,49,65,243,79,65,18,148,0,76,247,79,128,6,126,31,36,34,0,217,104,239,232,194,239,254,246,167,208,217,209,130,111,124,253,
235,120,232,219,223,12,93,229,185,1,147,203,197,85,142,2,136,119,36,70,119,37,99,238,7,141,136,166,124,74,232,243,249,182,83,110,100,223,238,120,68,139,160,205,154,31,196,75,193,225,152,48,4,145,50,64,
231,191,131,57,255,75,120,255,251,222,133,127,245,161,119,224,249,231,158,193,167,63,253,105,156,56,246,58,180,242,154,93,211,206,26,197,181,95,95,10,174,178,19,184,202,35,15,88,150,220,163,41,129,137,
74,235,41,215,185,170,35,209,37,35,198,184,208,198,13,117,218,80,74,172,131,82,6,162,221,224,223,253,199,95,193,91,223,250,70,252,229,231,255,14,159,249,204,31,227,216,209,99,97,104,196,174,242,148,171,
172,2,87,89,165,198,206,231,58,170,145,238,142,30,141,144,253,213,222,20,228,55,99,9,246,255,254,0,33,19,27,196,131,127,250,87,248,246,183,31,14,231,201,185,157,141,249,70,236,66,243,241,204,95,95,2,174,
114,235,185,68,247,85,253,6,126,29,172,14,194,13,164,174,160,207,123,208,225,239,199,238,17,160,202,32,190,240,133,47,227,245,215,143,64,43,63,108,235,213,200,158,205,86,186,150,50,114,103,142,226,75,
196,85,182,35,70,215,192,234,70,33,177,134,167,10,188,247,86,129,40,198,240,149,127,249,14,206,157,61,19,40,93,240,145,201,36,113,224,192,245,232,234,236,196,124,4,218,204,95,95,182,174,114,112,97,11,
53,30,18,245,158,105,215,239,144,216,156,169,224,225,239,60,129,129,179,167,67,25,209,197,53,251,174,192,189,111,187,27,187,118,236,196,189,247,222,131,3,7,14,52,243,23,95,240,247,11,77,37,151,128,171,
92,147,105,3,25,81,8,11,201,136,141,55,94,41,240,226,243,175,224,236,233,147,32,83,6,76,5,48,46,218,90,146,144,146,144,78,39,209,211,219,139,247,190,247,61,104,111,111,111,106,181,48,115,36,95,70,174,
50,234,244,164,32,11,247,223,44,49,52,56,128,23,95,56,12,227,77,130,84,9,164,43,208,126,5,93,221,237,72,38,226,232,238,233,198,174,157,59,176,127,255,126,100,179,217,166,71,238,101,234,42,19,4,89,83,174,
114,210,199,115,135,14,195,171,228,1,85,14,194,85,101,16,60,156,59,123,26,201,68,28,61,221,93,216,185,115,231,52,203,130,93,229,37,184,202,35,67,131,16,166,4,97,202,128,169,130,180,11,24,133,239,63,250,
40,14,30,252,62,206,156,57,131,47,127,249,203,184,231,158,123,22,13,138,93,229,57,92,101,227,21,64,170,12,232,10,180,170,212,93,101,229,251,33,251,251,248,156,43,7,118,149,155,118,149,43,16,13,42,120,
163,171,60,223,213,159,93,101,118,149,151,29,48,187,202,179,139,93,101,118,149,217,85,158,246,166,38,187,202,179,95,12,118,149,23,90,113,176,171,12,118,149,155,46,118,149,155,61,236,97,87,185,177,216,
85,102,87,153,93,229,134,1,204,174,242,204,98,87,153,93,101,118,149,231,120,217,217,85,6,216,85,94,250,6,163,54,61,176,171,188,132,131,155,165,190,181,196,174,242,50,14,110,154,93,166,177,171,188,204,
131,155,102,55,24,236,42,47,243,224,166,153,160,217,85,190,128,131,155,197,70,53,187,202,236,42,179,171,188,146,1,179,171,204,174,242,34,73,177,171,204,174,50,187,202,236,42,179,171,204,174,114,19,1,3,
236,42,175,122,192,193,64,97,87,185,233,128,217,85,102,87,121,77,66,102,87,121,61,22,187,202,235,160,86,162,249,143,6,208,180,171,92,155,18,54,242,200,189,160,41,130,107,101,139,59,96,115,192,28,48,23,
7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,7,204,17,112,192,28,48,23,7,124,209,106,214,141,39,203,16,17,87,165,54,130,171,188,218,183,78,173,
100,152,151,172,171,188,18,166,231,133,132,186,27,88,17,87,185,177,234,174,50,128,85,117,149,215,10,77,157,243,243,98,129,167,187,223,116,221,119,222,74,100,222,131,213,113,149,27,191,174,143,208,85,70,
240,148,230,55,55,92,192,53,246,23,0,10,7,31,17,0,96,181,119,144,241,61,50,74,81,208,226,36,168,91,119,126,235,6,75,86,255,51,128,59,0,36,1,211,140,171,124,101,19,174,242,179,11,184,202,181,160,11,8,76,
140,21,115,149,215,44,224,154,169,44,51,25,97,60,47,160,127,73,72,0,176,45,95,238,238,120,102,83,38,154,253,49,65,222,191,6,208,30,124,81,83,4,204,160,214,138,93,229,102,166,3,145,76,74,146,82,130,132,
36,33,108,192,88,189,233,211,109,61,201,211,119,69,173,226,199,136,244,27,2,5,209,148,141,49,7,141,161,47,157,60,121,242,120,165,52,254,62,203,146,119,58,142,205,174,242,188,225,198,227,150,136,70,45,
32,212,186,132,136,244,164,78,111,238,75,159,252,105,219,242,222,73,64,43,160,161,13,29,46,85,172,7,199,39,38,207,85,242,199,222,231,88,250,7,96,84,108,234,1,163,85,89,105,109,112,87,185,70,255,18,57,
144,50,6,80,34,98,85,50,87,247,30,250,140,109,233,155,12,132,0,224,87,188,232,87,142,159,51,95,119,188,35,119,219,178,240,102,1,183,141,66,119,110,102,219,241,213,168,13,237,42,3,136,144,148,113,219,242,
91,55,101,206,190,161,51,53,250,41,33,69,63,96,107,207,56,99,217,66,230,159,39,114,165,124,146,142,126,28,170,100,195,84,1,93,6,116,96,106,172,85,200,13,155,148,141,227,42,3,136,144,16,9,219,242,218,183,
182,158,124,91,71,42,247,41,178,98,253,190,104,241,11,170,235,216,169,220,13,207,140,152,143,222,31,79,244,252,148,145,25,219,88,25,64,36,131,86,95,34,2,19,54,241,88,67,95,121,3,186,202,68,233,158,244,
249,155,50,137,242,79,147,21,221,166,40,110,242,110,235,241,225,226,174,179,78,239,254,183,100,98,194,66,249,135,96,206,156,5,105,7,6,18,84,35,34,40,104,61,179,198,190,242,154,185,202,23,250,40,109,28,
68,25,219,242,123,175,233,123,249,179,194,138,110,85,148,52,147,126,203,153,193,226,181,163,137,222,109,215,81,36,64,61,9,2,102,66,128,70,254,39,200,159,0,252,9,144,159,3,169,73,24,93,2,233,106,56,85,
104,56,142,181,22,174,114,173,158,2,240,227,235,225,81,218,57,93,229,182,196,216,158,173,29,3,191,106,217,206,245,62,165,85,222,109,59,51,84,185,170,152,232,221,118,21,108,67,245,54,228,33,98,130,161,
113,136,137,207,131,252,49,192,27,3,84,30,80,5,72,184,232,238,233,192,7,63,240,126,220,249,166,59,216,85,6,40,153,142,231,183,111,111,63,243,239,35,81,235,157,138,82,152,244,219,135,206,23,182,103,163,
155,174,188,10,182,17,218,248,128,49,160,48,92,34,9,50,54,232,204,195,160,210,99,32,47,11,248,19,216,185,173,29,255,237,215,127,1,241,152,131,103,159,123,22,207,30,58,132,151,94,122,145,93,229,238,244,
240,221,78,68,220,169,41,46,170,38,93,24,46,246,229,237,158,43,175,130,173,66,250,55,68,83,73,66,16,32,133,8,250,236,119,191,5,56,241,36,90,219,90,241,223,126,229,23,208,211,145,198,119,190,243,45,124,
231,161,111,225,248,209,163,179,86,19,151,175,171,28,117,31,48,34,150,242,41,161,135,242,109,167,252,232,85,187,109,71,11,165,221,80,166,213,245,165,138,33,17,138,93,0,197,128,119,252,224,167,240,192,
219,170,120,253,229,23,241,135,159,249,71,156,60,254,42,180,106,238,71,251,242,113,149,109,185,91,83,60,116,149,247,182,199,59,165,163,77,16,110,163,146,88,163,208,106,255,247,163,111,35,220,185,127,2,
95,252,226,87,241,191,254,247,103,113,236,248,177,122,104,236,42,207,235,42,119,246,104,120,97,184,94,216,32,52,240,99,2,64,53,232,53,252,163,247,8,180,89,35,248,194,95,125,9,223,125,248,81,120,213,74,
157,63,155,111,187,204,174,114,163,171,28,74,224,117,163,43,188,184,9,33,241,145,187,4,34,106,20,95,250,210,55,113,252,232,17,104,85,129,209,46,8,26,145,72,4,202,247,234,33,179,171,60,167,171,236,67,27,
5,3,29,248,4,36,2,223,83,88,184,255,38,137,164,200,227,91,223,124,4,231,207,157,2,233,50,200,84,177,103,247,118,220,119,223,219,240,222,31,120,23,222,241,142,123,103,133,203,174,242,60,174,50,40,160,39,
133,176,112,125,191,141,45,45,46,190,255,200,83,56,127,238,52,160,74,128,174,192,168,10,118,239,220,138,116,42,129,116,58,137,107,174,189,22,63,245,83,63,185,236,145,59,223,180,210,24,242,198,119,149,
77,205,246,148,32,97,33,17,177,113,251,94,194,225,151,94,197,185,211,39,64,186,8,50,37,144,169,0,218,69,34,25,69,50,17,71,103,87,39,118,244,111,199,189,247,222,219,212,106,97,230,72,190,124,92,101,10,
71,175,8,92,229,119,223,108,97,100,232,60,94,122,254,48,140,151,15,2,214,21,104,191,12,192,67,118,116,4,201,228,20,253,219,219,219,219,244,200,189,204,93,101,27,237,41,11,29,9,31,207,29,122,9,94,37,23,
76,13,170,12,163,74,32,184,128,246,241,141,175,127,29,99,217,44,122,123,123,161,181,198,3,15,60,208,244,106,161,217,169,228,146,117,149,223,113,189,133,147,39,166,187,202,132,42,160,171,128,241,131,179,
110,99,240,231,127,254,231,248,192,7,62,128,123,239,189,23,39,79,158,92,48,168,141,230,42,207,59,130,141,82,4,41,5,73,57,205,85,214,53,87,89,165,91,65,65,27,241,133,92,229,35,161,171,140,154,171,236,151,
131,55,121,67,87,89,202,128,57,241,60,111,209,213,194,106,184,202,0,22,59,179,104,116,149,129,41,87,249,2,231,224,21,115,149,207,179,171,188,224,112,184,96,87,185,194,174,242,188,235,95,118,149,107,197,
174,50,187,202,236,42,79,123,83,147,93,229,217,47,6,187,202,11,173,56,216,85,6,187,202,77,23,187,202,205,30,246,176,171,220,88,236,42,179,171,204,174,114,195,0,102,87,121,102,177,171,204,174,50,187,202,
115,188,236,236,42,3,236,42,47,125,131,81,155,30,216,85,94,194,193,205,82,223,90,98,87,121,25,7,55,205,46,211,216,85,94,230,193,77,179,27,12,118,149,151,121,112,211,76,208,236,42,95,192,193,205,98,163,
154,93,101,118,149,217,85,94,201,128,217,85,102,87,121,145,164,216,85,102,87,153,93,101,118,149,217,85,102,87,185,137,128,1,118,149,87,61,224,96,160,176,171,220,116,192,236,42,179,171,188,38,33,179,171,
188,30,139,93,229,117,80,23,220,252,39,121,219,29,186,112,240,145,166,93,229,203,37,216,21,11,56,12,121,93,204,165,235,177,184,3,54,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,113,
192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,28,1,7,204,1,115,113,192,23,173,172,245,250,141,109,4,87,121,195,4,124,41,187,202,171,22,112,13,82,157,239,127,63,112,197,83,177,120,180,176,139,
246,155,107,113,225,174,114,27,128,183,133,31,117,87,249,206,253,15,173,170,171,124,81,2,174,209,191,50,147,17,141,236,47,133,55,4,154,106,213,220,186,235,27,111,177,132,122,55,128,91,0,186,186,129,255,
173,133,121,33,174,178,4,176,43,252,248,97,132,174,242,157,251,31,90,113,87,121,77,3,158,73,255,2,16,100,217,146,132,16,198,104,34,18,226,186,222,239,94,31,183,243,255,129,200,220,110,64,9,50,194,10,111,
105,115,1,51,159,171,124,125,19,174,242,183,23,112,149,247,35,104,110,255,161,59,247,63,180,162,174,114,51,181,162,174,242,44,250,151,200,150,66,91,155,91,142,108,233,140,159,251,17,91,86,63,66,70,183,
2,198,16,116,201,64,159,87,202,252,227,200,240,200,119,42,229,28,101,179,163,111,106,109,109,97,87,121,174,165,158,136,199,45,146,150,132,16,14,73,25,129,16,145,246,196,112,71,119,242,204,93,201,72,241,
163,66,248,215,134,95,176,170,52,61,229,249,214,87,207,13,12,30,243,138,167,238,147,82,221,238,88,86,15,160,227,43,116,147,230,37,227,42,79,133,91,163,127,133,136,130,40,222,153,28,218,214,215,114,230,
223,56,150,127,15,17,165,13,8,74,91,199,38,74,241,191,207,79,86,7,116,225,229,183,70,100,241,173,48,213,40,153,154,61,87,123,216,104,69,87,92,27,214,85,158,22,46,66,250,23,68,73,199,246,59,246,245,60,
255,187,182,77,55,24,114,132,129,165,10,213,196,99,167,134,82,207,70,205,171,215,58,52,118,189,52,149,116,64,255,134,252,175,241,234,166,231,106,60,186,177,161,93,101,18,20,3,137,184,109,121,173,61,233,
243,55,117,101,198,62,41,164,181,77,83,76,251,58,150,31,158,108,125,178,226,188,169,53,218,214,115,0,39,126,153,72,87,0,85,4,84,1,208,197,58,1,1,227,173,133,175,188,65,92,229,134,145,27,132,235,183,111,
105,61,253,182,206,244,196,127,38,25,219,230,83,139,154,244,59,206,158,206,95,121,36,103,189,243,54,167,167,253,70,196,60,50,221,191,0,99,181,192,88,105,64,38,1,17,5,9,7,16,51,125,229,85,171,53,117,149,
151,21,112,176,90,8,132,196,112,90,72,133,174,242,191,17,118,108,171,18,105,147,119,59,78,15,22,246,140,202,238,3,111,72,246,137,36,132,7,173,125,160,205,135,73,220,13,200,84,216,238,43,14,35,28,0,54,
76,67,15,159,85,14,185,230,42,255,199,124,62,255,43,0,254,54,252,145,169,185,202,251,47,214,58,120,10,79,69,253,130,150,178,45,175,187,51,61,254,159,133,29,219,170,40,101,242,94,219,192,249,202,222,74,
124,211,182,253,100,43,17,180,150,49,97,63,31,1,116,92,11,84,158,0,116,5,68,101,128,108,64,72,192,4,175,183,99,59,232,236,108,71,95,223,166,213,116,149,201,178,172,155,179,217,236,103,178,217,236,143,
247,247,247,39,194,245,243,245,0,254,244,206,253,15,221,184,212,57,249,66,231,224,38,93,229,246,115,231,43,123,203,137,222,173,123,96,41,210,90,133,205,244,1,65,50,108,210,97,1,227,22,196,200,167,1,111,
20,228,141,65,152,2,122,186,82,248,192,251,239,199,157,111,186,29,197,34,187,202,51,92,229,52,38,253,246,161,193,194,182,137,88,223,149,87,24,233,139,58,158,10,19,116,7,20,54,100,173,19,138,176,65,39,
14,130,10,95,193,246,190,4,126,229,83,63,141,84,194,193,139,207,63,135,231,158,59,132,195,47,189,136,129,129,115,236,42,79,119,149,55,21,156,158,90,184,46,148,153,162,127,53,1,210,232,105,63,16,233,29,
119,226,147,15,92,133,205,221,73,60,246,232,35,120,228,123,223,198,137,99,175,133,82,215,236,150,182,151,185,171,220,122,218,143,94,181,203,114,212,84,184,218,15,174,29,181,86,228,84,67,13,4,238,188,154,
240,158,155,52,78,29,31,196,231,254,252,107,56,125,226,21,24,191,216,212,242,236,114,116,149,79,187,206,222,182,120,167,116,140,241,234,225,154,58,131,134,144,254,149,16,36,240,193,59,5,110,223,85,192,
215,191,254,45,124,238,47,254,6,39,79,156,8,158,21,55,154,93,229,121,92,229,74,163,171,28,240,147,161,171,28,6,43,201,130,20,54,62,124,151,68,123,100,2,95,254,167,111,224,177,131,143,193,115,139,128,118,
65,245,105,97,246,46,142,93,229,105,174,114,35,90,77,161,169,28,180,249,250,224,29,18,113,51,142,111,124,245,97,156,60,126,4,240,75,48,126,25,48,85,220,120,224,58,28,56,176,31,55,223,124,83,56,122,23,
149,89,230,253,253,37,238,42,55,232,136,225,178,76,144,141,251,111,178,145,146,121,124,247,225,199,48,116,238,20,72,21,1,29,232,180,111,125,243,27,177,99,199,86,92,181,247,74,220,112,195,126,220,118,219,
173,11,142,94,118,149,131,217,44,108,235,101,99,255,14,7,91,91,93,60,254,253,103,112,254,236,73,144,153,4,153,34,224,23,145,74,58,104,239,72,35,30,139,161,187,167,19,187,118,238,192,187,223,125,255,178,
70,238,124,211,74,99,200,151,128,171,172,167,187,202,81,27,111,220,75,120,249,197,215,112,238,244,9,8,93,128,80,69,144,46,193,232,10,38,243,99,208,190,135,84,42,142,238,174,78,236,218,181,11,55,220,112,
195,162,171,133,153,35,249,50,114,149,105,110,87,249,133,151,96,188,28,72,7,83,131,246,203,32,120,128,246,113,242,228,73,164,83,73,116,119,119,163,187,187,27,159,252,228,39,155,26,185,27,193,85,182,22,
154,30,106,3,121,186,171,28,173,187,202,145,86,203,49,186,186,168,171,252,205,71,2,87,89,168,18,160,67,87,217,84,1,237,65,41,23,79,60,254,56,178,163,195,232,237,237,193,201,147,39,113,230,204,153,85,117,
149,1,160,175,175,15,39,78,156,64,169,84,90,176,27,107,232,42,255,180,227,56,63,23,190,199,119,119,184,241,40,95,208,8,94,53,87,217,84,102,185,202,82,18,78,156,56,142,71,30,121,100,214,82,138,93,229,101,
185,202,106,150,171,60,215,166,130,93,229,101,187,202,193,187,22,236,42,179,171,92,43,118,149,217,85,102,87,121,198,26,152,93,229,153,197,174,50,187,202,236,42,207,58,125,102,87,121,170,216,85,110,230,
128,104,174,81,205,174,242,50,207,134,217,85,102,87,153,93,229,134,207,201,174,114,179,231,196,51,63,31,187,202,236,42,179,171,188,188,101,26,187,202,181,98,87,121,169,27,140,218,244,192,174,242,18,14,
110,150,250,214,18,187,202,203,56,184,105,118,153,198,174,242,50,15,110,154,221,96,176,171,188,204,131,155,102,130,102,87,249,2,14,110,22,27,213,236,42,179,171,204,174,242,74,6,204,174,50,187,202,139,
36,197,174,50,187,202,236,42,179,171,204,174,50,187,202,77,4,12,176,171,188,234,1,7,3,133,93,229,166,3,102,87,153,93,229,53,9,153,93,229,245,88,236,42,175,131,98,87,121,189,7,28,134,204,174,242,10,174,
34,184,56,96,14,152,3,230,226,128,57,96,46,14,152,3,230,128,185,56,96,14,152,3,230,226,128,57,96,14,152,139,3,230,128,185,56,96,14,152,3,230,226,128,57,96,14,152,107,69,138,93,229,141,28,240,98,244,111,
173,110,221,247,253,168,99,87,163,96,87,121,105,193,90,237,29,164,171,21,49,147,254,5,128,93,45,207,68,219,147,231,119,74,225,93,43,200,220,5,208,237,128,97,87,185,217,17,43,146,73,105,180,18,100,217,
22,9,33,140,82,130,72,16,0,220,208,247,173,187,28,89,121,151,32,117,19,25,92,9,144,85,111,145,27,116,187,99,87,121,190,112,69,50,41,3,26,2,22,180,177,66,193,192,34,199,177,174,232,120,230,250,150,232,
232,207,72,242,111,134,17,113,130,180,12,17,0,227,194,224,41,24,252,147,54,244,229,145,225,243,133,129,129,211,215,179,171,140,186,171,44,106,163,150,72,88,176,100,32,212,146,136,8,97,98,221,201,179,219,
186,211,231,62,20,177,170,239,39,66,11,160,13,140,41,195,168,97,207,167,175,140,140,230,191,91,41,143,155,92,246,212,173,173,153,196,109,66,208,126,192,176,171,220,184,212,155,65,255,70,0,196,91,226,227,
61,221,169,193,55,167,98,197,15,75,129,171,12,4,12,224,250,202,122,169,236,70,191,55,52,50,113,74,23,143,188,201,22,147,111,112,164,234,4,252,24,106,130,215,133,221,252,126,105,186,202,104,160,127,219,
19,217,254,190,214,179,63,25,177,205,221,16,50,101,96,195,83,145,179,217,98,219,163,133,178,200,82,249,133,235,99,98,226,0,116,209,33,93,134,209,21,64,187,171,33,211,94,26,174,114,141,254,13,140,79,191,
235,234,77,47,127,218,178,197,117,154,162,66,35,170,243,213,212,75,231,178,125,231,163,93,183,222,24,177,34,41,113,254,151,45,168,18,72,21,166,248,95,93,6,105,23,198,120,43,238,43,95,34,174,178,223,222,
157,30,186,185,43,61,246,159,132,229,108,213,148,208,174,73,20,134,38,91,95,117,157,235,91,163,61,221,187,131,30,28,2,230,236,75,192,228,63,67,248,121,192,207,1,186,22,116,5,48,46,160,87,205,87,222,184,
174,242,230,214,211,247,116,166,199,63,33,236,216,86,159,50,106,210,239,24,60,155,219,121,182,232,220,121,77,164,171,125,55,16,64,126,218,120,48,237,87,1,50,5,35,19,48,50,14,144,3,34,59,108,129,176,170,
59,248,141,233,42,119,167,135,110,110,73,148,62,46,237,216,86,69,105,147,119,219,207,157,47,108,159,176,122,246,239,77,116,137,152,161,26,135,22,246,115,143,40,152,204,135,0,17,7,137,72,216,83,173,161,
143,207,234,2,214,27,207,85,238,74,143,253,167,186,171,236,182,13,158,175,92,81,73,244,109,189,18,150,47,180,86,245,190,16,1,224,103,5,236,70,91,6,38,231,4,163,87,88,128,14,194,181,29,7,29,29,93,216,188,
169,59,116,149,55,177,171,108,57,206,245,62,101,84,222,109,31,24,42,95,81,137,247,108,221,101,44,159,166,232,95,13,132,144,148,20,14,68,216,176,3,89,27,52,252,171,16,126,22,61,237,54,222,255,3,111,193,
157,119,220,140,114,185,128,179,103,207,224,252,224,192,134,118,149,173,37,79,13,150,109,147,160,40,128,88,58,150,235,219,220,58,240,83,182,109,93,167,144,212,69,175,101,120,184,184,185,16,223,180,229,
10,99,121,164,180,7,109,220,58,253,27,244,186,52,117,200,186,118,138,176,217,185,6,63,255,179,119,161,53,101,227,181,87,94,196,231,191,240,87,120,249,240,11,24,60,119,14,74,121,211,186,161,204,88,25,204,
114,149,247,236,217,179,28,87,121,31,128,7,243,249,252,207,164,211,233,54,4,174,242,61,0,62,121,231,254,135,254,203,114,215,201,75,9,120,105,174,178,229,11,165,221,6,64,53,128,164,106,61,124,68,216,153,
42,21,19,248,233,251,4,182,180,189,29,207,62,253,36,158,56,248,93,156,58,254,106,64,161,25,191,190,130,96,87,121,14,87,89,155,169,169,161,214,142,156,40,80,18,137,36,110,190,66,226,237,215,27,156,63,243,
10,254,233,11,223,193,185,83,175,2,126,30,52,75,166,53,115,142,220,153,163,184,246,235,203,198,85,214,58,160,127,131,11,155,8,16,63,17,64,126,239,185,69,226,214,93,101,60,250,189,131,248,187,191,253,39,
156,57,117,28,70,85,97,140,15,165,220,96,147,97,204,52,153,150,93,229,89,174,242,212,188,75,100,65,132,174,253,7,239,176,208,29,159,196,55,191,254,93,60,253,228,83,240,171,57,144,46,131,76,21,93,29,45,
216,189,171,127,214,6,131,93,229,57,92,229,0,120,16,83,244,175,176,241,67,111,180,145,20,19,248,246,55,31,193,153,19,71,0,63,15,163,10,32,93,193,205,55,93,139,55,223,117,59,110,60,240,6,188,247,189,239,
9,71,239,252,211,30,187,202,53,87,89,216,144,194,193,187,110,116,208,98,23,240,200,195,79,96,120,224,20,72,231,65,186,0,82,37,196,162,132,109,91,122,97,91,2,237,109,45,216,179,103,55,62,246,177,159,152,
119,244,178,171,28,204,102,211,92,229,109,109,129,171,60,52,112,2,66,231,65,106,18,198,159,4,116,25,157,237,41,16,25,164,146,113,116,247,116,99,207,238,93,184,239,190,251,150,60,114,231,187,16,54,134,
124,233,187,202,106,18,66,77,130,116,9,80,101,16,170,56,121,226,40,10,147,121,100,90,50,216,220,183,9,59,119,238,196,161,67,135,216,85,94,170,171,172,189,124,131,171,92,2,193,5,116,240,226,60,241,248,
99,56,123,230,52,58,59,59,113,240,224,65,252,242,47,255,242,162,35,119,35,184,202,98,161,233,161,54,144,167,187,202,118,221,85,38,219,114,130,198,172,11,187,202,207,29,10,92,101,210,133,57,93,101,24,131,
209,209,17,252,197,95,252,5,238,187,239,62,124,226,19,159,88,85,87,89,8,129,190,190,190,224,173,143,69,54,34,161,171,252,29,0,110,56,138,239,94,202,52,177,182,174,178,158,223,85,158,111,197,192,174,50,
187,202,203,156,131,217,85,174,23,187,202,236,42,179,171,60,59,100,118,149,167,21,187,202,203,123,43,159,93,101,118,149,217,85,94,254,42,130,93,229,105,197,174,242,98,187,186,249,70,53,187,202,77,30,232,
44,118,180,217,236,217,197,60,197,174,50,187,202,236,42,55,12,96,118,149,103,22,187,202,236,42,179,171,60,199,203,206,174,50,192,174,242,210,55,24,181,233,129,93,229,37,28,220,44,245,173,37,118,149,151,
113,112,211,236,50,141,93,229,101,30,220,52,187,193,96,87,121,153,7,55,205,4,205,174,242,5,28,220,44,54,170,217,85,102,87,153,93,229,149,12,152,93,101,118,149,23,73,138,93,101,118,149,217,85,102,87,153,
93,101,118,149,155,8,24,96,87,121,213,3,14,6,10,187,202,77,7,204,174,50,187,202,107,18,50,187,202,235,177,216,85,94,7,197,174,242,122,15,56,12,153,93,229,21,92,69,112,113,192,28,48,7,204,197,1,115,192,
92,28,48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,240,42,86,225,224,35,20,18,149,151,124,89,107,17,38,230,121,140,115,70,200,211,238,173,
96,87,121,9,193,202,76,70,24,165,8,90,83,35,253,11,4,183,86,237,237,125,62,154,138,229,34,142,40,94,41,132,190,11,48,183,131,93,229,5,195,21,181,96,33,132,132,175,4,89,182,104,188,49,176,51,113,38,214,
147,58,185,35,106,21,175,178,68,245,77,2,234,86,192,108,71,208,22,5,51,110,20,98,87,185,113,212,202,76,70,24,207,19,208,70,66,192,38,219,14,58,36,9,33,97,140,181,175,231,137,59,226,118,225,237,22,121,
55,16,244,30,130,176,130,56,53,200,104,3,152,215,0,58,10,152,227,96,87,121,122,184,34,153,148,0,36,217,182,5,18,14,9,233,128,40,10,32,218,223,254,250,245,109,241,225,159,180,164,127,3,65,198,141,17,178,
214,216,202,104,253,172,111,196,63,87,125,251,203,19,163,3,185,145,193,151,175,109,109,109,185,30,48,236,42,215,93,229,120,220,34,105,73,8,225,144,148,81,16,69,72,152,100,71,114,184,191,55,125,254,67,
81,219,189,31,68,105,144,48,198,152,138,209,102,188,234,89,223,29,30,115,15,250,149,9,175,48,246,234,254,214,20,29,16,228,95,3,163,34,75,52,61,47,43,87,57,10,32,153,142,231,54,117,167,207,223,149,138,
86,63,100,73,186,66,147,5,109,108,207,85,145,227,133,74,236,249,108,206,59,79,149,35,215,218,24,187,34,34,43,173,80,229,40,116,25,100,2,15,233,2,100,218,75,222,85,78,181,39,70,251,251,90,6,127,42,226,
224,46,35,34,41,77,81,84,85,116,120,180,216,126,184,162,119,145,149,222,125,131,173,34,73,26,253,164,128,42,2,170,16,18,192,101,64,151,1,120,43,37,211,94,138,174,178,215,189,111,211,171,191,109,219,214,
117,154,98,194,167,184,206,87,50,71,7,198,187,242,209,206,235,247,70,50,145,164,180,9,70,17,232,232,239,2,106,18,228,231,0,21,18,60,186,4,210,213,134,81,172,47,120,46,188,4,93,229,72,232,42,167,138,231,
39,219,78,184,206,85,109,209,158,174,205,132,41,177,128,72,0,163,26,148,253,3,144,63,14,248,19,128,202,3,126,1,164,203,33,96,189,226,190,242,165,226,42,167,213,164,223,62,116,38,223,63,80,114,110,187,
34,210,213,177,217,24,15,74,79,241,191,218,40,152,22,9,35,227,65,243,36,114,80,235,119,105,230,232,134,178,66,117,137,184,202,94,199,192,96,97,251,132,213,125,237,238,120,23,69,180,169,66,169,42,148,174,
66,41,55,20,100,52,32,53,76,250,1,64,68,96,132,29,54,19,157,222,14,119,21,106,227,187,202,67,229,43,42,129,78,235,11,93,39,40,195,135,186,33,161,13,133,237,103,12,144,201,0,227,97,31,76,18,112,236,8,58,
122,90,208,215,219,142,43,246,236,194,230,190,94,118,149,231,118,149,107,244,111,168,36,26,83,199,83,165,136,132,190,178,5,1,27,52,112,30,157,242,47,241,238,183,223,136,59,239,184,30,126,165,136,193,129,
51,56,127,126,0,231,7,7,145,207,231,54,164,171,188,148,128,69,232,42,59,36,40,6,80,50,29,207,111,223,222,126,230,223,71,162,214,59,21,165,49,233,183,15,13,22,182,77,196,54,237,109,160,127,93,232,58,65,
73,161,12,110,67,144,3,41,29,8,178,177,169,213,198,191,121,187,68,71,42,135,147,199,94,195,171,135,159,195,171,47,191,128,193,129,211,80,190,11,173,231,230,39,231,114,149,35,145,200,114,92,101,32,104,
103,240,51,233,116,250,83,8,92,101,0,248,35,0,243,186,202,43,249,32,226,50,93,101,55,164,127,107,174,50,133,29,80,2,227,51,25,149,248,87,111,22,232,107,45,225,149,195,47,226,217,167,30,195,217,147,175,
2,186,8,210,85,76,119,149,13,187,202,243,185,202,83,206,81,16,172,16,54,222,176,75,226,238,171,129,225,193,227,248,203,127,126,28,231,207,28,129,241,38,32,234,75,179,198,71,153,231,111,184,127,153,187,
202,186,190,222,21,100,135,115,174,131,119,223,108,227,214,157,85,60,253,228,51,248,151,127,250,6,6,79,31,7,252,34,200,84,225,251,161,53,103,52,148,63,53,53,176,171,188,128,171,44,234,35,215,193,7,222,
104,163,39,62,137,239,60,244,125,60,247,244,211,80,213,49,144,154,4,116,9,189,93,25,92,121,69,63,110,187,237,38,220,122,235,205,115,134,203,174,242,156,174,114,48,122,63,240,70,27,41,145,195,195,223,254,
62,206,158,120,29,240,115,128,55,1,232,34,96,202,184,241,13,87,227,134,235,247,97,247,174,157,184,254,250,235,112,203,45,55,163,25,41,113,230,239,47,91,87,57,99,21,240,253,239,61,137,225,129,147,16,58,
7,161,242,65,184,186,132,158,206,12,226,113,7,201,68,2,61,61,157,216,189,107,39,222,252,230,187,216,85,110,214,85,222,218,234,226,137,131,207,224,252,185,227,16,58,15,161,38,161,253,60,160,75,32,83,193,
192,185,83,32,99,208,210,146,194,166,222,30,236,220,185,19,91,182,108,89,210,200,157,239,66,216,24,242,37,235,42,191,242,210,116,87,57,24,185,129,105,175,253,0,150,122,246,217,103,144,136,199,208,219,
219,139,246,246,118,124,242,147,159,156,119,142,157,57,146,217,85,158,203,85,14,233,223,218,185,196,107,175,189,138,223,251,189,223,195,175,255,250,175,227,150,91,110,65,54,155,93,112,228,178,171,188,
4,87,89,74,130,82,62,198,199,199,103,169,224,243,173,22,22,153,67,231,12,159,93,229,69,130,98,87,153,93,229,38,231,96,118,149,235,197,174,50,187,202,236,42,207,14,153,93,229,105,197,174,242,242,222,202,
103,87,153,93,101,118,149,151,191,138,96,87,121,90,177,171,188,216,174,110,190,81,205,174,114,147,7,58,139,29,109,54,123,118,49,79,177,171,204,174,50,187,202,13,3,152,93,229,153,197,174,50,187,202,236,
42,207,241,178,179,171,12,176,171,188,244,13,70,109,122,96,87,121,9,7,55,75,125,107,137,93,229,101,28,220,52,187,76,99,87,121,153,7,55,205,110,48,216,85,94,230,193,77,51,65,179,171,124,1,7,55,139,141,
106,118,149,217,85,102,87,121,37,3,102,87,153,93,229,69,146,98,87,153,93,101,118,149,217,85,102,87,153,93,229,38,2,6,216,85,94,245,128,131,129,194,174,114,211,1,179,171,204,174,242,154,132,204,174,242,
122,44,118,149,215,65,177,171,188,222,3,14,67,102,87,121,5,87,17,92,28,48,7,204,1,115,113,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,92,28,48,7,204,1,115,113,192,
28,240,229,82,214,90,125,161,133,248,223,25,117,73,221,2,176,166,174,114,35,255,11,0,53,2,216,84,171,6,141,119,255,100,71,205,182,158,19,216,222,115,162,27,236,42,47,30,172,72,38,37,217,182,32,34,9,18,
4,10,180,40,34,65,0,208,211,62,20,201,68,71,236,116,44,123,133,77,213,55,137,45,222,173,0,222,0,118,149,231,13,87,52,6,11,3,11,218,88,144,100,147,16,18,66,200,76,124,44,209,157,60,189,35,238,228,175,140,
200,210,45,130,188,155,8,58,192,243,140,192,140,94,107,236,42,55,142,218,186,171,28,244,45,176,131,38,251,178,118,179,182,179,167,227,197,55,37,34,147,111,181,165,123,141,32,181,139,140,45,97,130,86,12,
100,60,99,136,142,26,173,142,16,112,156,72,159,3,244,40,216,85,158,199,85,14,26,68,36,64,20,223,214,118,226,13,237,137,177,159,176,164,186,6,36,99,64,92,26,24,24,82,190,86,230,101,215,151,223,200,87,210,
95,43,76,78,140,21,134,14,94,209,146,17,251,72,251,215,107,237,111,23,130,216,85,158,229,42,3,49,146,72,118,36,70,119,117,167,135,62,16,139,168,119,65,88,41,3,203,104,99,121,90,83,174,232,58,207,101,115,
244,162,114,11,213,74,238,229,29,45,177,202,85,18,133,43,160,171,206,18,101,218,203,210,85,238,235,78,13,223,157,138,85,62,40,45,107,143,166,8,52,34,94,217,139,15,228,171,137,147,249,234,46,88,233,171,
174,181,242,207,219,78,229,239,28,232,130,19,128,169,69,24,93,14,101,90,111,57,190,242,165,239,42,183,197,179,59,54,183,14,252,148,19,161,59,141,136,166,12,98,40,171,228,232,104,177,245,68,213,108,141,
88,153,254,221,118,198,137,9,41,96,242,2,116,238,55,64,42,15,248,249,64,166,85,1,100,2,227,94,168,175,124,169,187,202,113,225,83,66,79,84,90,78,158,159,232,44,71,59,175,222,233,100,156,168,144,141,44,
132,4,78,124,25,84,121,25,240,199,65,254,4,72,229,195,81,92,89,17,95,249,210,115,149,237,200,86,141,164,174,154,84,105,168,208,118,210,181,175,108,143,118,119,244,214,90,146,27,99,64,84,131,83,37,48,110,
129,70,126,31,228,143,53,24,203,133,112,154,112,27,172,142,11,222,79,92,42,174,114,70,77,250,237,67,103,243,219,7,203,206,45,123,34,93,237,189,202,184,80,186,10,95,85,160,180,27,24,203,90,5,61,47,147,
6,16,17,64,56,0,89,225,135,12,227,108,118,151,221,84,93,34,174,178,219,62,48,88,216,54,97,117,95,187,51,218,73,142,50,149,192,84,214,85,232,70,169,182,54,183,58,6,72,188,165,33,204,240,131,86,197,86,190,
196,92,101,237,66,153,41,20,42,104,170,45,1,210,13,16,59,128,212,21,64,14,176,109,11,237,29,157,232,235,222,129,43,118,111,197,230,190,110,164,211,73,72,65,236,42,207,118,149,3,64,213,212,166,2,32,152,
119,201,130,12,155,35,17,89,176,200,66,107,210,198,219,182,63,135,59,110,217,10,227,151,48,58,116,14,35,195,3,24,62,63,128,201,252,68,232,42,123,27,202,85,182,150,60,53,88,182,77,130,162,0,98,233,88,174,
111,115,235,192,79,217,182,117,157,66,82,23,189,150,225,225,226,230,66,48,114,195,112,67,87,185,209,248,36,4,20,26,133,13,59,186,210,18,63,124,183,68,87,34,143,115,167,5,190,252,229,175,227,232,107,47,
96,120,224,20,148,95,130,86,115,95,228,230,114,149,247,236,217,179,28,87,121,31,128,7,243,249,252,207,164,211,233,54,4,174,242,61,0,62,121,231,254,135,254,203,114,215,201,75,9,248,2,93,229,218,180,16,
52,105,14,26,118,56,72,69,109,252,224,237,18,155,90,92,28,125,237,37,124,229,249,67,24,60,115,36,68,84,139,160,144,254,13,90,245,139,16,176,158,10,183,246,79,118,149,235,244,175,21,136,224,194,193,245,
59,109,188,233,74,129,145,243,103,240,197,111,61,139,161,115,199,195,77,70,41,88,154,233,16,149,50,186,241,84,109,90,184,141,163,184,246,235,203,216,85,14,187,82,73,7,247,223,236,224,214,93,46,158,63,
244,28,190,241,213,111,227,252,217,163,129,80,171,39,97,188,34,108,203,128,224,65,169,106,125,106,216,168,174,114,83,83,132,241,60,65,182,45,13,16,161,105,174,114,178,238,42,167,119,119,110,215,166,186,
128,171,236,132,244,111,160,211,126,239,59,79,97,224,244,113,192,207,129,252,128,162,36,93,193,77,183,92,139,109,91,123,33,72,131,160,241,249,207,125,62,236,72,181,48,96,189,152,171,92,155,151,231,130,
88,103,46,225,16,184,202,247,164,211,233,155,195,109,245,199,194,227,206,167,87,122,4,175,176,171,236,32,37,114,248,238,119,190,143,179,39,95,7,188,113,24,111,2,164,38,65,186,132,76,218,198,182,173,189,
112,108,129,150,76,10,91,182,108,198,91,222,242,230,57,119,111,236,42,47,228,42,159,59,9,161,114,16,42,23,92,208,76,112,176,211,221,153,129,32,133,84,34,134,77,155,122,177,107,231,14,188,245,173,111,153,
53,247,178,171,92,59,196,153,215,85,14,208,234,186,171,172,43,80,126,25,175,188,124,24,110,181,130,76,75,6,155,251,54,97,247,238,221,120,237,181,215,154,30,185,51,167,140,185,66,190,204,92,229,10,200,
132,206,167,241,1,163,240,194,243,207,129,96,208,222,222,142,199,31,127,28,159,253,236,103,231,156,99,103,142,228,245,234,42,91,77,93,220,12,44,162,41,87,217,39,7,185,114,98,40,210,190,109,171,33,63,88,
49,52,233,42,139,105,174,114,165,193,85,14,116,218,215,95,127,29,191,254,235,191,62,107,20,174,180,171,92,40,20,112,228,200,17,236,219,183,111,161,46,131,177,108,54,251,129,100,50,249,69,33,196,39,48,
229,42,191,112,161,35,248,162,185,202,75,89,45,44,50,135,206,25,62,187,202,139,4,197,174,50,187,202,77,94,228,216,85,174,23,187,202,236,42,179,171,60,59,100,118,149,167,21,187,202,203,123,43,159,93,101,
118,149,217,85,94,254,42,130,93,229,105,197,174,242,98,187,186,249,70,53,187,202,77,30,232,44,118,40,223,236,217,197,60,197,174,50,187,202,236,42,55,12,96,118,149,103,22,187,202,236,42,179,171,60,199,
203,206,174,50,192,174,242,210,55,24,181,233,129,93,229,37,28,220,44,245,173,37,118,149,151,113,112,211,236,50,141,93,229,101,30,220,52,187,193,96,87,121,153,7,55,205,4,205,174,242,5,28,220,44,54,170,
217,85,102,87,153,93,229,149,12,152,93,101,118,149,23,73,138,93,101,118,149,217,85,102,87,153,93,101,118,149,155,8,24,88,69,87,57,121,219,29,245,161,153,253,199,219,47,25,87,121,169,79,219,107,149,203,
65,36,147,138,72,184,193,189,107,26,85,207,214,163,133,150,95,232,76,140,254,146,144,238,174,132,44,93,187,179,181,48,126,44,123,197,255,130,209,243,185,202,158,49,218,215,185,194,172,94,49,237,239,249,
126,17,192,171,225,199,223,44,244,13,93,40,186,218,223,223,255,59,8,92,229,247,96,186,171,124,193,79,218,47,101,14,158,249,166,232,138,186,202,23,211,249,188,228,93,229,245,128,168,94,210,174,242,133,
254,5,46,166,203,204,174,242,58,40,118,149,215,123,192,97,200,236,42,175,224,42,130,139,3,230,128,57,96,46,14,152,3,230,226,128,57,96,14,152,139,3,230,128,57,96,46,14,152,3,230,128,185,56,96,14,152,139,
3,230,128,57,96,46,14,120,221,148,181,214,95,48,148,19,129,133,31,140,187,100,222,222,95,75,87,185,254,211,82,231,127,1,212,8,224,224,231,105,234,6,149,194,193,71,46,137,27,84,172,181,10,182,22,42,217,
118,208,85,219,178,5,140,38,216,182,0,130,6,32,70,107,77,130,140,209,74,165,218,140,217,183,233,137,46,187,240,221,173,66,168,14,204,231,42,239,191,204,93,229,6,161,86,144,101,75,104,99,145,109,217,65,
163,97,41,64,36,50,209,108,52,97,229,34,29,137,179,59,163,86,225,54,73,238,205,68,106,63,25,211,134,208,226,88,228,190,150,203,215,85,6,32,201,137,4,200,73,240,97,131,40,146,116,38,147,237,137,193,93,
201,72,225,138,168,85,188,217,18,238,126,1,191,143,0,9,19,144,201,134,124,144,17,0,180,1,200,15,187,58,215,190,111,118,149,201,178,109,24,99,145,16,17,0,17,16,197,65,34,182,179,227,181,55,165,162,133,
183,88,82,237,147,66,111,7,108,9,88,97,31,120,207,64,171,19,198,168,227,208,214,113,130,127,150,160,70,8,106,18,128,79,100,12,216,85,14,92,101,18,20,5,201,8,128,56,136,146,91,90,79,31,232,72,142,253,152,
101,225,26,8,59,98,16,149,10,2,6,70,249,190,56,94,118,157,39,198,75,233,239,85,202,149,97,119,252,241,190,214,120,113,151,208,249,235,181,46,111,17,64,39,160,227,225,84,193,174,114,131,171,156,10,92,229,
225,7,98,17,253,14,35,236,148,161,168,209,176,61,95,59,165,201,74,236,232,120,177,111,84,36,246,198,203,131,95,177,211,246,43,93,210,140,111,37,85,176,161,75,97,191,119,23,208,181,6,163,243,206,197,151,
161,171,28,203,245,117,167,135,239,78,198,220,15,90,150,220,163,41,6,69,49,191,228,37,134,242,149,196,80,193,219,106,89,169,93,59,173,68,36,110,69,5,209,152,0,134,127,17,80,249,192,55,82,147,48,186,4,
210,110,232,43,55,13,167,94,78,174,178,184,211,136,104,74,35,134,138,78,141,141,22,90,207,84,176,57,102,165,183,110,179,211,118,132,36,0,19,244,29,70,69,130,78,253,46,224,79,128,212,68,40,211,22,1,85,
2,26,122,191,47,241,78,217,203,197,85,110,61,117,62,215,81,141,118,236,219,110,167,237,40,201,169,160,8,34,100,40,37,112,242,235,160,242,139,128,159,5,249,19,83,136,117,115,211,196,188,117,9,187,202,233,
210,208,100,235,41,215,217,219,30,237,238,236,209,240,67,223,51,48,52,16,90,115,68,129,28,131,49,64,140,254,1,200,27,133,241,199,32,252,28,76,72,161,153,11,8,184,161,46,53,87,121,219,96,57,114,203,238,
72,87,123,143,50,85,104,85,133,210,141,174,178,15,109,252,169,185,53,110,193,132,253,216,106,141,61,216,85,110,202,85,174,66,169,208,85,54,30,140,246,166,154,236,55,252,20,81,28,32,217,1,132,13,204,87,
169,46,85,87,217,175,211,16,34,124,109,41,108,172,95,131,34,8,4,171,229,126,180,169,1,108,234,236,196,21,59,123,176,165,175,3,45,233,56,164,36,120,110,149,93,229,185,92,229,41,227,211,132,23,53,25,82,
148,1,232,39,201,70,38,97,225,77,87,218,184,109,207,56,164,41,99,34,59,136,241,209,65,140,12,159,67,97,114,28,165,194,100,232,42,19,187,202,211,93,229,153,198,167,4,133,61,37,164,112,208,145,178,240,190,
219,44,116,36,74,24,25,60,135,111,127,235,117,156,56,250,50,70,6,79,64,123,5,104,85,170,123,115,141,23,57,118,149,231,114,149,67,134,82,8,27,201,168,131,247,220,108,161,59,237,225,248,209,215,241,221,
195,47,225,252,217,227,32,21,32,170,129,48,230,65,136,32,83,173,12,102,114,104,236,42,207,114,149,131,81,123,253,14,7,183,239,37,140,12,157,195,63,63,252,60,134,7,79,3,222,36,72,7,128,42,233,50,160,221,
112,228,234,6,251,136,93,229,69,92,229,8,222,117,83,4,183,236,114,241,210,115,47,224,219,95,123,8,67,103,143,129,188,113,8,149,131,246,38,32,80,193,173,55,95,135,55,220,112,53,122,122,58,3,143,67,207,
221,95,99,189,187,202,77,45,211,140,231,9,8,146,38,56,114,108,112,149,227,117,87,57,210,221,217,163,225,53,229,42,247,38,10,120,228,225,199,240,194,161,167,161,171,163,16,106,28,164,2,184,26,186,132,91,
110,188,26,91,182,116,99,239,21,187,113,255,125,239,192,85,87,237,173,175,147,151,235,42,187,174,139,51,103,154,26,132,53,87,249,255,2,56,20,254,187,143,221,185,255,161,3,171,17,240,42,186,202,175,1,222,
24,140,55,6,10,249,95,237,229,209,215,221,138,45,91,122,144,136,197,208,211,221,137,254,29,219,241,3,63,240,110,44,7,150,98,87,57,12,150,66,72,21,166,138,193,129,83,16,198,32,149,138,163,167,183,27,87,
236,217,131,157,59,119,78,155,123,217,85,14,102,179,166,93,101,232,50,148,95,174,11,181,39,78,28,69,58,149,68,79,119,55,250,250,250,240,7,127,240,7,77,141,220,153,83,198,92,33,175,181,171,108,45,118,113,
131,175,4,217,214,92,174,114,177,224,111,138,199,83,150,173,76,101,186,171,76,98,150,171,124,248,249,133,92,229,74,184,82,8,214,184,7,15,30,132,148,2,103,206,156,193,161,67,135,112,232,208,161,37,99,126,
141,47,64,227,146,174,230,42,143,141,141,161,82,169,44,216,36,52,116,149,63,159,78,167,7,195,183,164,106,174,242,11,43,50,69,212,47,110,6,22,48,229,42,107,114,144,171,36,134,34,237,219,250,12,5,135,53,
205,186,202,52,175,171,172,27,116,90,131,71,30,121,4,15,62,248,32,14,29,58,52,239,200,189,16,87,217,113,28,28,57,114,100,177,169,38,150,205,102,63,160,181,254,98,248,251,154,171,124,193,83,4,187,202,83,
107,99,118,149,217,85,102,87,121,206,159,69,118,149,195,98,87,153,93,101,118,149,103,135,204,174,242,180,98,87,121,25,197,174,50,216,85,6,187,202,23,176,138,96,87,121,90,177,171,188,216,174,110,190,81,
205,174,114,147,7,58,139,189,157,212,236,217,197,60,197,174,50,187,202,236,42,55,12,96,118,149,103,22,187,202,236,42,179,171,60,199,203,206,174,50,192,174,242,210,55,24,181,233,129,93,229,37,28,220,44,
245,173,37,118,149,151,113,112,211,236,50,141,93,229,101,30,220,52,187,193,96,87,121,153,7,55,205,4,205,174,242,5,28,220,44,54,170,217,85,102,87,153,93,229,149,12,152,93,101,118,149,23,73,138,93,101,118,
149,217,85,102,87,153,93,101,118,149,155,8,24,96,87,121,201,1,179,171,204,174,242,234,22,187,202,107,19,50,187,202,171,53,69,92,72,177,171,188,14,138,93,229,245,30,112,24,50,187,202,43,184,138,224,226,
128,57,96,14,152,139,3,230,128,185,56,96,14,152,3,230,226,128,57,96,14,152,139,3,230,128,57,96,46,14,152,3,230,226,128,57,96,14,152,171,249,178,46,230,23,95,128,0,54,64,253,205,84,14,120,25,161,214,3,
13,111,38,68,141,1,38,41,13,16,60,8,25,254,217,13,253,86,255,69,113,149,69,50,41,161,117,64,0,107,19,52,33,181,66,122,71,144,49,74,105,217,218,106,140,231,105,17,137,234,141,108,44,175,173,171,236,121,
34,104,217,216,112,139,149,52,225,247,64,2,68,6,70,27,146,150,111,161,106,246,108,126,186,35,105,79,244,201,205,110,59,65,167,169,244,157,104,216,234,138,93,229,57,93,101,39,18,74,50,112,40,120,228,222,
6,9,59,233,228,227,17,171,28,111,143,159,223,153,140,76,220,104,139,202,27,36,252,107,8,126,139,129,6,153,69,179,99,87,185,238,42,7,208,73,52,30,41,181,180,39,134,119,37,156,210,149,49,187,124,163,101,
121,215,72,248,61,193,147,229,54,140,14,158,161,35,227,193,16,12,25,21,186,202,4,4,240,50,187,202,13,174,114,52,12,54,14,18,201,29,29,71,238,76,70,203,119,219,82,237,19,18,91,8,142,52,112,160,0,16,124,
163,149,62,103,140,58,101,140,127,10,218,63,43,224,158,39,120,57,9,183,66,228,107,130,150,68,38,6,118,149,103,187,202,237,201,241,31,183,45,186,218,8,59,106,40,46,52,44,104,72,229,250,214,64,177,26,125,
121,188,152,120,209,117,253,17,93,124,181,45,237,20,122,132,30,189,14,126,238,109,18,213,118,24,47,6,163,12,160,139,90,43,118,149,49,205,85,30,121,32,22,81,239,48,34,146,50,20,53,138,162,190,167,35,229,
201,74,252,236,68,169,187,64,241,109,9,175,220,218,147,216,20,75,91,163,39,36,229,254,183,128,154,12,209,212,192,218,88,4,78,189,140,93,229,168,91,135,165,20,197,252,162,151,26,201,87,18,163,5,191,47,
106,165,251,55,91,113,59,42,29,34,33,131,46,86,102,92,130,206,255,26,224,231,2,227,200,207,3,186,8,210,213,208,87,158,187,139,107,67,93,134,174,114,8,75,141,20,90,207,85,177,41,97,101,182,110,182,82,150,
67,66,7,56,109,67,139,114,84,37,232,212,31,128,252,49,144,63,30,250,202,97,163,253,165,251,202,23,213,85,190,144,179,136,89,174,50,73,153,4,81,198,182,188,222,173,29,3,191,26,137,90,239,52,34,153,242,
41,163,199,42,93,39,142,143,110,25,246,146,55,237,137,108,234,223,33,211,198,209,166,10,95,85,67,161,214,13,90,209,104,5,138,104,64,118,1,20,9,91,130,133,205,59,104,89,223,110,204,113,156,127,85,40,20,
190,112,236,216,177,219,143,28,57,242,111,1,252,17,0,15,1,2,245,113,0,127,118,231,254,135,156,245,52,7,47,213,85,62,237,58,123,219,34,221,157,61,6,62,106,176,84,173,7,60,234,184,73,96,124,10,178,65,67,
231,64,185,191,5,220,44,200,31,3,84,46,4,172,171,128,94,150,16,94,171,75,210,85,222,21,233,106,239,209,166,10,165,42,240,117,21,53,53,177,14,157,212,195,14,206,122,76,116,219,2,95,254,130,106,77,93,229,
229,124,162,0,79,37,33,195,11,218,28,174,114,202,228,221,182,129,225,226,230,66,164,231,202,43,44,71,9,101,170,117,204,79,55,200,49,193,220,107,213,27,236,83,205,66,138,82,56,82,107,31,115,55,121,94,102,
213,92,229,171,133,16,255,54,157,78,143,1,248,8,166,92,229,143,1,120,250,98,76,17,51,31,165,141,131,168,197,182,188,222,107,250,94,249,51,81,71,171,219,6,207,151,175,40,198,123,182,236,52,150,154,225,
42,171,250,180,80,115,143,26,133,90,65,18,182,180,209,18,183,208,93,248,107,236,222,26,195,150,77,105,180,101,162,176,45,64,121,101,140,141,101,49,58,50,130,115,231,206,94,136,171,92,171,167,0,252,248,
106,61,74,123,145,92,101,170,155,246,141,144,106,58,46,113,211,78,11,55,244,79,194,166,42,138,185,17,76,142,15,97,108,116,0,229,194,24,74,165,28,148,87,129,20,8,93,101,201,174,242,252,174,114,237,130,
102,161,35,101,227,190,27,37,90,99,85,140,14,157,198,19,143,157,196,153,147,71,144,29,58,13,237,229,160,253,66,160,212,154,144,141,128,1,160,217,85,94,204,85,78,68,108,188,251,38,11,93,41,31,199,143,29,
199,99,175,188,134,161,129,211,117,134,50,248,168,64,194,3,168,17,76,53,236,42,47,230,42,239,223,225,224,246,43,8,35,67,131,248,218,35,47,98,120,240,44,140,95,0,233,2,132,42,1,166,28,218,202,53,26,205,
175,179,106,236,42,55,235,42,63,255,2,30,250,250,119,48,124,246,40,200,205,66,248,99,48,238,120,200,255,22,113,221,213,187,241,190,247,189,19,183,222,250,134,240,89,242,217,118,7,187,202,243,184,202,143,
62,252,24,94,60,244,12,116,117,56,216,18,171,137,128,92,87,147,128,42,226,3,239,191,15,87,236,237,71,75,58,137,155,14,188,1,111,188,253,214,112,244,46,141,231,185,76,93,229,131,161,171,156,133,241,198,
195,112,39,235,74,109,95,79,27,4,41,164,18,9,244,116,119,98,215,238,157,120,239,123,223,211,212,200,157,57,138,47,83,87,249,196,28,174,114,1,208,21,192,184,56,115,230,36,8,10,233,116,2,155,54,245,96,215,
206,157,232,237,237,101,87,121,121,174,114,177,193,85,118,131,11,154,81,56,113,252,56,82,201,4,122,123,123,209,218,218,138,79,124,226,19,139,142,220,153,83,198,92,33,179,171,28,94,196,14,30,60,136,39,
158,120,28,215,92,115,13,187,202,171,225,42,3,6,158,231,177,171,204,174,242,50,71,48,187,202,168,109,82,216,85,94,108,163,193,174,242,28,87,127,118,149,217,85,94,118,192,236,42,207,46,118,149,217,85,102,
87,121,218,59,23,236,42,207,126,49,216,85,94,104,197,193,174,50,216,85,110,186,216,85,110,246,176,135,93,229,198,98,87,153,93,101,118,149,27,6,48,187,202,51,139,93,101,118,149,217,85,158,227,101,103,87,
25,96,87,121,233,27,140,218,244,192,174,242,18,14,110,150,250,214,18,187,202,203,56,184,105,118,153,198,174,242,50,15,110,154,221,96,176,171,188,204,131,155,102,130,102,87,249,2,14,110,22,27,213,236,42,
179,171,204,174,242,74,6,204,174,50,187,202,139,36,197,174,50,187,202,236,42,179,171,204,174,50,187,202,77,4,12,176,171,188,228,128,217,85,102,87,121,117,139,93,229,181,9,153,93,229,213,154,34,46,164,
216,85,94,7,197,174,242,122,15,56,12,153,93,229,21,92,69,112,113,192,28,48,7,204,197,1,115,192,92,28,48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,113,192,28,48,7,204,213,124,93,
52,87,121,1,83,121,174,218,176,239,70,95,84,87,89,102,50,194,40,69,193,3,204,97,218,225,61,21,166,90,53,104,184,159,98,163,210,191,214,69,13,85,27,73,150,45,130,14,87,36,1,128,72,4,97,71,34,26,6,138,4,
25,163,149,146,153,140,10,49,235,13,37,134,175,185,171,12,64,144,101,135,224,9,73,18,194,6,201,0,65,105,184,38,72,73,106,107,230,181,246,116,100,164,39,98,149,90,5,188,148,132,103,19,105,109,138,223,169,
16,153,113,176,171,220,224,42,107,35,201,182,37,9,105,35,32,128,109,4,38,135,19,179,139,169,136,229,38,218,18,163,187,147,145,220,27,28,89,190,78,146,123,165,48,126,134,80,235,119,25,222,79,109,26,30,
238,159,42,118,149,137,200,49,64,4,66,56,0,98,177,72,185,165,61,62,186,39,30,41,93,21,179,221,3,182,165,174,22,66,119,1,36,96,130,39,242,131,182,229,30,72,187,134,12,249,128,231,27,2,200,232,176,251,147,
97,87,185,209,85,38,162,4,72,36,66,87,249,45,82,98,159,101,209,102,131,136,52,36,161,32,0,104,163,124,26,86,218,12,41,173,207,65,251,131,48,238,121,137,74,214,50,149,130,20,101,215,66,69,19,169,168,32,
221,6,152,94,172,99,87,121,69,216,223,198,112,69,50,41,41,232,120,100,145,148,81,144,104,116,149,111,108,79,78,252,152,109,225,106,35,34,81,67,142,48,112,160,200,214,85,63,50,50,89,137,158,202,149,226,
39,61,79,148,141,217,186,61,21,57,77,178,252,93,135,188,108,171,48,249,20,233,114,20,186,98,96,220,130,86,222,176,16,116,2,208,47,143,143,143,63,187,128,171,12,4,207,25,23,16,152,24,43,230,42,175,9,251,
187,100,87,89,70,82,6,177,208,85,142,85,114,149,196,249,92,185,163,42,98,125,73,207,203,180,199,187,162,113,25,181,164,20,2,230,124,21,24,253,181,105,244,47,84,105,62,95,153,93,229,233,174,114,114,172,
224,111,138,217,233,237,125,50,110,57,194,9,155,153,35,240,143,136,36,144,179,64,131,255,13,8,233,95,225,231,97,66,2,205,24,127,122,115,254,233,197,174,178,149,217,178,197,74,89,54,40,104,162,20,224,202,
83,230,28,9,9,84,44,208,233,63,0,121,217,208,87,30,7,84,33,4,172,171,205,0,214,192,101,228,42,159,156,230,42,167,96,43,83,129,175,42,80,170,10,213,32,213,106,163,96,180,6,108,13,80,208,224,110,170,85,
163,152,241,173,47,186,211,190,60,93,101,85,235,138,109,212,148,79,75,34,148,187,236,186,175,140,19,95,1,85,94,4,121,163,161,16,158,95,174,16,94,171,203,200,85,214,110,48,90,107,115,233,140,118,229,0,
0,231,138,134,16,117,216,147,184,126,6,180,156,65,117,169,187,202,94,72,162,213,104,158,90,247,192,224,97,37,65,141,77,150,4,224,92,9,170,247,130,87,211,195,94,126,200,151,131,171,92,91,45,212,88,52,171,
46,39,218,82,34,29,179,208,153,182,208,159,178,176,57,253,125,116,100,44,196,108,13,173,171,152,24,31,65,118,100,24,3,231,206,226,212,169,147,236,42,207,235,42,135,36,154,32,11,146,108,164,98,22,246,111,
151,184,102,75,17,22,121,168,20,198,80,202,143,34,63,118,30,229,98,22,213,210,4,180,95,130,16,6,182,164,208,85,142,178,171,60,159,171,76,34,8,182,61,105,225,237,111,144,200,68,171,200,14,159,195,11,207,
158,195,185,211,39,49,54,124,6,198,207,67,123,1,69,73,186,2,152,106,48,85,4,207,52,66,8,98,87,121,78,87,153,108,36,162,54,238,191,209,66,103,202,195,241,99,39,241,228,171,71,48,52,120,14,164,139,16,186,
4,82,37,24,85,128,132,11,192,135,54,30,234,162,129,14,126,26,180,54,236,42,47,228,42,15,159,31,196,55,30,125,9,195,131,231,130,48,85,25,48,165,64,75,212,101,64,151,96,75,13,79,5,33,7,225,78,237,224,214,
187,171,220,84,192,50,147,9,206,115,231,112,149,253,25,174,178,50,213,133,93,229,16,80,221,220,82,197,75,207,191,134,151,158,127,9,170,154,3,84,17,194,148,160,189,18,36,185,32,83,197,213,251,118,227,202,
189,59,65,164,48,150,205,226,161,111,127,11,213,74,25,141,126,199,66,174,114,125,225,91,169,192,243,60,180,182,182,46,62,13,6,174,242,63,245,247,247,63,24,110,66,106,174,242,39,151,189,27,91,244,29,199,
85,117,149,71,65,126,22,66,141,131,252,137,224,96,71,23,224,88,62,246,238,217,14,75,26,100,82,73,92,119,205,62,188,253,222,123,166,141,222,198,98,87,121,30,87,25,94,22,164,243,32,61,229,42,147,169,160,
92,202,193,18,6,169,68,12,155,54,245,96,231,174,29,184,255,254,119,45,58,114,103,142,98,118,149,213,100,168,129,79,185,202,202,171,0,198,71,165,82,68,42,157,66,79,119,23,118,237,220,89,159,71,217,85,14,
102,179,101,186,202,225,142,205,104,156,56,113,2,4,131,238,238,110,104,173,241,241,143,127,156,93,229,149,114,149,1,224,233,167,159,194,161,67,207,212,219,119,177,171,188,194,174,242,204,48,231,11,110,
230,136,101,87,153,93,101,118,149,107,197,174,242,60,47,8,187,202,236,42,131,93,229,169,98,87,153,93,101,118,149,103,172,129,217,85,158,89,236,42,179,171,204,174,242,172,211,103,118,149,167,138,93,229,
197,118,117,243,141,106,118,149,155,60,208,89,40,248,165,156,93,204,83,236,42,179,171,204,174,114,195,0,102,87,121,102,177,171,204,174,50,187,202,115,188,236,236,42,3,236,42,47,125,131,81,155,30,216,85,
94,194,193,205,82,223,90,98,87,121,25,7,55,205,46,211,216,85,94,230,193,77,179,27,12,118,149,151,121,112,211,76,208,236,42,95,192,193,205,98,163,154,93,101,118,149,217,85,94,201,128,217,85,102,87,121,
145,164,216,85,102,87,153,93,101,118,149,217,85,102,87,185,137,128,1,118,149,151,28,48,187,202,236,42,175,110,177,171,188,54,33,179,171,188,90,83,196,133,20,187,202,235,160,216,85,94,239,1,135,33,179,
171,188,130,171,8,46,14,152,3,230,128,185,56,96,14,152,139,3,230,128,57,96,46,14,152,3,230,128,185,56,96,14,152,3,230,226,128,57,96,46,14,152,3,190,164,202,90,47,223,200,2,12,176,1,0,102,127,151,23,232,
204,48,73,102,50,65,91,71,132,143,47,32,120,102,175,225,5,48,172,210,54,25,108,35,1,76,182,45,160,13,129,132,32,171,97,230,82,74,203,214,86,109,170,85,67,182,173,107,97,55,222,44,120,217,7,188,184,171,
108,44,18,66,64,66,26,99,168,110,43,195,104,8,219,0,80,20,139,251,198,247,124,217,218,106,160,181,10,239,232,89,247,183,11,92,92,87,57,108,157,75,82,90,8,238,40,151,36,200,2,130,142,54,221,137,179,237,
153,104,182,39,106,21,210,22,85,227,130,92,91,192,87,100,84,145,72,143,195,152,33,33,212,9,176,171,188,128,171,12,68,99,78,57,237,88,110,186,53,62,190,39,25,41,220,16,177,170,215,90,194,221,77,80,105,
64,131,140,15,24,31,100,76,248,200,185,1,25,3,144,14,127,48,12,187,202,243,187,202,149,125,81,199,189,209,150,122,159,20,232,52,36,4,16,133,70,20,48,10,65,43,27,215,144,241,20,116,213,7,0,50,144,134,140,
69,134,194,39,208,217,85,158,225,42,31,189,43,25,41,223,45,45,179,207,178,228,102,67,17,169,97,195,135,12,219,55,90,99,190,18,99,190,66,86,43,61,106,180,63,66,112,199,44,148,115,54,138,5,91,20,171,150,
40,91,22,42,41,65,170,27,80,219,177,78,93,101,107,165,195,173,143,90,219,182,72,80,20,36,231,112,149,233,106,35,156,168,161,136,240,225,64,83,68,87,252,104,118,178,26,31,204,21,35,195,158,146,26,212,179,
41,221,185,117,155,180,163,251,204,196,73,87,228,30,4,169,113,139,116,81,195,47,22,181,42,15,11,242,79,64,235,195,227,227,163,223,238,235,235,125,102,30,87,121,63,130,230,246,31,186,115,255,67,43,234,
42,55,83,235,192,85,78,14,77,148,59,60,17,235,77,249,42,221,26,239,136,196,68,84,136,122,211,37,146,192,136,4,134,63,5,242,39,66,95,121,114,46,95,153,93,229,185,92,101,43,189,109,179,140,91,182,116,64,
193,69,171,182,157,171,105,137,18,152,176,64,67,191,5,120,227,128,63,54,221,87,214,30,230,128,83,217,85,174,185,202,134,20,76,77,238,50,97,55,109,154,66,253,4,89,64,209,2,13,212,124,229,177,80,167,45,
132,128,181,187,16,96,13,92,100,87,249,66,230,224,89,174,50,40,0,84,109,203,235,222,218,49,240,171,182,109,93,167,41,46,124,74,232,137,74,235,233,243,185,142,74,180,227,170,61,145,180,29,9,120,158,74,
16,174,169,245,94,10,198,110,208,166,38,104,53,99,72,131,44,83,239,32,8,106,236,116,213,120,124,49,111,192,53,87,249,29,19,19,19,95,20,66,252,219,221,187,119,127,24,193,163,178,53,87,121,215,157,251,31,
250,129,149,122,178,104,37,78,211,104,174,112,3,250,247,236,29,215,244,189,242,103,150,227,92,175,40,141,138,105,43,156,203,245,188,58,110,174,139,166,119,239,223,107,183,138,136,161,10,124,93,129,210,
85,40,237,162,230,126,214,44,164,224,99,174,97,96,230,109,169,187,232,95,84,136,78,199,113,126,210,178,172,111,158,56,113,226,107,190,239,127,12,83,45,195,223,14,224,169,59,247,63,116,96,37,201,223,229,
6,188,194,174,178,15,3,53,237,147,139,250,252,27,118,22,148,4,104,47,220,112,52,194,40,115,247,37,94,164,214,212,85,94,86,192,34,153,148,36,45,73,68,78,109,206,157,238,42,167,77,222,109,31,24,44,108,155,
176,186,175,221,25,237,164,64,74,84,85,104,29,248,202,129,127,164,167,95,208,132,85,111,133,32,133,93,15,89,8,1,18,45,193,5,205,132,65,35,212,12,204,5,185,202,255,49,159,207,255,10,128,191,13,95,165,154,
171,188,255,98,45,211,86,216,85,174,193,38,65,219,3,17,118,76,177,165,133,84,68,162,53,41,177,165,205,194,230,22,137,54,231,101,196,157,42,132,169,160,144,27,197,248,216,16,6,7,78,227,244,201,19,24,27,
203,162,88,44,176,171,188,144,171,44,66,158,231,154,45,18,123,251,74,176,224,163,92,200,161,84,24,71,97,98,4,213,210,56,188,242,4,180,95,128,77,30,44,59,176,149,227,177,72,232,42,151,217,85,174,77,7,141,
174,178,32,11,29,73,27,247,222,32,145,142,84,49,58,124,30,175,190,56,128,129,179,103,48,62,122,30,198,47,4,22,146,46,130,116,9,208,229,96,131,97,92,104,223,11,93,229,8,187,202,139,187,202,167,240,228,
171,71,49,124,126,0,66,55,176,191,126,9,100,2,30,77,235,114,40,38,250,225,147,185,6,90,107,118,149,155,117,149,161,66,18,77,151,65,186,10,152,10,180,170,192,177,13,122,186,59,112,242,196,209,48,92,175,
110,33,5,58,56,187,202,139,187,202,186,4,210,69,104,175,12,33,188,0,79,245,43,184,233,192,181,216,190,181,15,150,77,184,229,166,107,240,213,175,252,11,198,199,198,235,219,99,118,149,155,117,149,253,113,
144,159,3,169,9,64,77,130,84,48,231,238,218,185,25,145,136,68,38,149,196,142,237,219,240,254,247,189,119,174,179,7,118,149,23,119,149,243,80,117,87,185,12,229,79,157,150,53,218,202,119,223,253,230,5,71,
238,204,81,204,174,114,131,171,76,225,10,33,112,149,61,64,43,76,140,143,33,157,78,162,167,187,11,59,250,251,225,186,46,187,202,225,108,118,193,174,50,160,113,232,208,33,104,173,17,141,70,113,228,200,17,
252,232,143,254,232,188,35,119,230,148,49,87,200,236,42,79,35,127,21,6,7,7,240,155,191,249,155,236,42,175,150,171,60,223,72,101,87,153,93,229,133,71,48,187,202,168,109,82,216,85,158,111,163,193,174,50,
187,202,236,42,135,197,174,50,187,202,236,42,207,88,3,179,171,60,179,216,85,102,87,153,93,229,89,167,207,236,42,79,21,187,202,139,237,234,230,27,213,236,42,55,121,160,179,80,240,75,57,187,152,167,216,
85,102,87,153,93,229,134,1,204,174,242,204,98,87,153,93,101,118,149,231,120,217,217,85,6,216,85,94,250,6,163,54,61,176,171,188,132,131,155,165,190,181,196,174,242,50,14,110,154,93,166,177,171,188,204,
131,155,102,55,24,236,42,47,243,224,166,153,160,217,85,190,128,131,155,197,70,53,187,202,236,42,179,171,188,146,1,179,171,204,174,242,34,73,177,171,204,174,50,187,202,236,42,179,171,204,174,114,19,1,3,
236,42,47,57,96,118,149,217,85,94,221,98,87,121,109,66,102,87,121,181,166,136,11,41,118,149,215,65,177,171,188,222,3,14,67,102,87,121,5,87,17,92,28,48,7,204,1,115,113,192,28,48,23,7,204,1,115,192,92,28,
48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,92,28,48,7,124,73,213,122,115,149,155,105,149,181,161,156,229,117,229,42,79,19,107,195,162,72,132,76,181,90,123,136,195,144,148,102,35,209,191,235,202,
85,54,74,5,221,174,148,154,186,233,87,27,80,36,98,136,132,49,90,107,24,173,54,18,253,123,241,93,101,41,37,164,37,65,36,201,22,18,32,25,118,29,12,30,65,10,91,42,192,24,77,82,42,24,225,129,200,55,193,231,
66,195,253,24,230,178,12,120,81,87,89,8,155,136,236,144,1,182,195,199,148,108,65,70,182,197,135,59,83,209,137,158,136,85,78,219,84,141,75,114,165,128,231,19,249,5,65,254,40,148,58,175,182,208,241,19,103,
183,15,140,76,116,169,203,42,224,37,185,202,145,114,139,45,253,84,107,108,252,138,100,180,120,192,145,238,213,150,165,119,19,153,100,208,43,72,129,194,150,184,100,124,144,241,64,198,131,17,26,142,81,184,
106,251,225,49,224,240,51,128,121,4,236,42,79,185,202,109,177,209,43,18,209,202,85,81,199,189,209,146,216,39,37,58,13,164,0,197,161,235,79,140,26,16,124,67,198,211,64,85,73,72,3,136,224,243,26,143,12,
105,144,161,54,192,220,131,192,31,98,87,121,46,87,217,192,129,130,5,77,182,241,180,157,115,125,43,239,43,57,233,43,202,107,173,115,48,106,82,194,205,73,148,243,14,21,75,17,145,247,28,154,140,72,42,181,
11,184,91,4,169,29,96,87,121,33,87,57,54,150,175,196,135,242,229,232,184,167,164,5,209,221,147,234,232,217,36,236,232,14,178,109,71,72,75,168,49,223,195,240,47,184,101,149,47,25,111,114,84,152,234,9,163,
189,195,19,19,217,135,186,186,54,61,181,169,59,30,149,194,127,39,216,85,158,229,42,251,34,214,155,86,42,213,26,237,140,68,132,67,162,214,68,165,174,37,66,194,140,74,208,240,47,129,252,241,186,76,27,240,
103,85,64,171,170,214,254,179,227,227,99,236,42,207,116,149,173,184,101,147,99,8,164,130,150,140,225,247,84,115,149,107,216,20,198,44,208,200,127,7,220,49,144,63,6,82,121,24,85,4,153,42,204,20,96,13,176,
171,60,135,171,108,130,102,251,65,203,114,154,198,81,10,97,7,77,237,38,108,136,161,223,3,249,163,48,222,24,132,159,91,12,176,6,216,85,246,235,14,71,157,233,9,3,38,216,48,212,208,163,88,76,89,7,4,1,67,
77,117,249,102,87,89,213,161,169,41,174,7,211,14,42,40,236,50,72,97,187,220,169,54,142,77,255,69,47,119,87,217,64,135,163,86,132,141,61,166,204,207,154,175,76,68,128,113,97,180,15,50,10,4,189,156,163,
136,53,117,149,151,243,137,2,60,149,132,12,47,104,115,184,202,41,147,119,219,6,134,139,155,11,145,158,43,175,176,28,37,148,169,214,189,57,109,26,70,32,97,186,82,91,55,237,37,132,176,234,191,38,146,48,
36,0,93,14,152,30,51,101,32,153,165,31,67,212,92,229,171,133,16,255,54,157,78,143,1,248,8,166,92,229,143,1,88,145,165,220,250,113,149,195,145,43,132,132,37,44,36,35,18,153,132,64,111,70,162,55,45,145,
137,142,33,97,101,97,81,9,229,201,44,38,199,135,48,116,254,12,206,158,58,142,177,177,97,20,11,147,208,218,95,206,153,207,165,238,42,7,98,162,32,137,100,212,194,53,91,37,118,247,148,33,140,143,82,97,18,
229,98,14,133,220,56,220,114,14,126,37,7,163,138,176,168,10,199,82,112,164,70,44,230,192,182,8,213,74,25,199,143,31,99,87,121,46,87,185,61,105,225,237,55,88,72,69,170,24,29,26,194,145,195,231,49,112,110,
0,227,217,97,24,85,130,241,75,33,170,26,186,202,186,2,210,85,104,85,129,32,131,88,212,65,50,25,71,60,30,99,87,121,62,87,249,196,177,227,120,242,213,163,24,57,63,24,66,126,85,200,80,74,20,166,10,232,10,
140,170,194,152,10,72,123,208,170,130,224,9,93,133,201,201,10,138,197,73,0,134,93,229,197,92,101,154,70,162,185,32,85,1,105,23,201,164,131,109,91,251,241,226,11,207,65,43,55,100,41,85,232,42,7,13,118,
217,85,94,212,85,14,164,68,65,1,207,163,253,10,8,62,110,58,112,29,250,250,54,33,26,181,112,205,190,29,56,252,210,75,120,246,208,33,4,58,237,244,174,253,236,42,207,229,42,171,177,208,85,158,0,169,92,232,
42,23,128,112,206,237,219,212,137,68,204,70,38,149,196,246,173,91,240,198,55,222,214,176,45,158,18,20,217,85,158,207,85,86,185,41,87,89,21,65,166,2,229,151,64,161,171,44,5,144,74,197,177,105,83,15,118,
237,218,129,59,222,120,251,188,35,119,230,40,102,87,121,78,87,185,28,108,36,116,224,216,103,71,71,144,78,5,182,242,206,157,59,103,77,1,205,20,187,202,243,184,202,128,198,183,191,253,45,156,57,115,6,90,
107,60,247,220,115,120,224,129,7,230,28,185,141,197,174,114,147,174,114,173,254,226,47,254,2,159,251,220,231,166,133,199,174,50,187,202,236,42,179,171,204,174,50,187,202,203,155,131,217,85,174,23,187,
202,236,42,179,171,60,59,100,118,149,167,21,187,202,203,40,118,149,193,174,50,216,85,190,128,85,4,187,202,211,138,93,229,197,118,117,243,141,106,118,149,155,60,208,89,40,248,165,156,93,204,83,236,42,179,
171,204,174,114,195,0,102,87,121,102,177,171,204,174,50,187,202,115,188,236,236,42,3,236,42,47,125,131,81,155,30,216,85,94,194,193,205,82,223,90,98,87,121,25,7,55,205,46,211,216,85,94,230,193,77,179,27,
12,118,149,151,121,112,211,76,208,236,42,95,192,193,205,98,163,154,93,101,118,149,217,85,94,201,128,217,85,102,87,121,145,164,216,85,102,87,153,93,101,118,149,217,85,102,87,185,137,128,1,118,149,151,28,
48,187,202,236,42,175,110,177,171,188,54,33,179,171,188,90,83,196,133,20,187,202,235,160,216,85,94,239,1,135,33,179,171,188,130,171,8,46,14,152,3,230,128,185,56,96,14,152,139,3,230,128,57,96,46,14,152,
3,230,128,185,56,96,14,152,3,230,226,128,57,96,46,14,120,109,203,90,207,223,92,72,168,205,170,141,244,190,222,70,112,149,73,102,50,4,160,14,164,54,4,191,238,223,11,92,79,174,50,137,100,178,118,223,234,
84,239,54,109,40,104,56,1,200,76,198,64,8,99,170,85,67,182,173,201,178,141,159,29,53,235,57,232,245,227,42,75,43,104,25,104,52,65,90,18,48,4,18,130,236,134,174,31,6,10,68,134,162,49,5,24,101,180,82,50,
147,81,235,153,254,93,87,174,50,0,171,129,4,150,16,66,194,24,2,9,10,123,56,170,240,195,133,49,30,193,120,6,218,19,201,164,10,239,115,91,119,33,175,55,87,217,9,72,96,97,19,25,167,53,62,214,149,138,228,
123,109,233,182,88,194,79,72,225,89,4,223,35,227,23,1,149,133,81,103,43,126,236,248,169,145,61,103,170,153,140,169,221,146,117,89,4,188,100,87,89,120,233,214,196,196,21,201,72,241,128,99,249,215,72,105,
118,10,66,178,54,77,7,28,154,170,155,202,48,46,132,241,144,114,38,208,185,237,204,56,140,122,6,6,223,55,6,15,9,161,94,2,187,202,211,93,101,91,98,159,144,212,105,96,9,67,17,24,88,240,235,203,116,99,12,
180,33,227,43,65,174,17,166,44,133,17,66,107,170,181,203,107,53,164,223,74,208,111,37,50,191,0,118,149,103,186,202,81,105,96,195,135,13,67,142,113,181,147,119,149,83,240,148,44,121,74,86,180,162,178,9,
48,163,178,48,94,193,166,74,193,161,201,74,84,228,225,136,92,194,161,98,183,32,179,29,48,187,0,195,174,242,220,174,114,124,44,95,137,15,231,75,145,188,167,165,77,178,171,39,209,222,221,35,109,39,98,44,
219,146,150,32,8,73,6,210,184,89,84,203,231,63,121,62,175,75,103,225,151,95,201,231,243,127,155,105,217,252,120,111,135,43,226,86,238,94,34,125,63,96,216,85,158,238,42,247,164,149,74,183,70,59,157,136,
112,72,52,94,171,168,161,249,51,193,2,178,18,52,244,139,32,127,12,240,3,154,7,170,92,213,218,125,97,60,155,253,94,123,123,203,195,241,120,212,99,87,121,202,85,118,200,49,64,104,123,78,107,91,30,118,183,
18,225,67,141,130,44,32,107,131,134,255,43,224,103,3,196,90,77,2,170,20,54,223,87,0,116,201,117,171,236,42,7,174,178,143,26,21,161,209,96,116,6,143,63,2,68,13,76,154,5,34,27,148,37,80,246,119,0,119,52,
12,56,23,4,28,8,225,152,225,43,179,171,172,235,248,105,173,169,232,20,75,73,70,4,129,209,84,43,115,152,106,195,46,123,209,159,240,139,234,42,47,119,4,79,163,127,27,93,229,238,244,208,205,93,233,177,255,
36,236,200,86,141,164,174,154,116,105,104,50,240,230,34,221,157,61,6,62,140,246,66,251,72,213,63,208,96,203,82,131,12,30,60,142,27,60,255,44,68,48,146,49,156,3,101,255,55,200,31,5,188,9,64,229,67,26,205,
157,213,35,126,142,170,0,248,224,150,45,91,218,44,203,250,20,130,167,58,37,130,39,140,126,12,75,120,8,166,153,17,188,78,92,229,154,195,129,80,167,109,8,54,164,215,107,15,149,19,68,96,114,152,26,118,226,
135,14,82,211,215,170,203,217,85,158,234,98,37,234,33,203,134,112,195,35,139,16,61,49,186,10,210,30,130,86,230,141,13,251,23,13,251,50,118,149,73,194,146,18,137,136,68,38,38,208,149,18,232,76,9,164,34,
57,196,100,14,54,185,112,43,5,20,198,207,98,116,248,12,6,206,28,195,196,216,32,138,133,28,180,239,46,54,61,204,85,151,151,171,124,245,86,137,61,61,101,64,249,40,22,10,40,21,38,81,152,204,193,45,79,194,
171,22,64,170,20,186,202,62,28,233,35,17,19,129,171,92,45,226,196,177,99,56,115,230,52,42,149,37,239,138,47,31,87,121,100,104,8,175,31,30,194,224,185,1,140,143,141,194,168,10,180,42,131,116,53,244,148,
203,1,191,174,171,208,170,4,1,31,209,168,133,84,34,134,120,60,138,61,123,118,179,171,60,211,85,238,72,122,56,121,188,209,85,174,2,166,10,161,171,208,170,12,203,120,128,118,161,85,5,4,23,38,252,53,180,
7,13,133,194,100,17,165,66,14,129,171,44,217,85,158,229,42,63,114,24,35,231,207,5,96,170,46,135,214,92,21,100,92,24,21,48,148,164,131,144,175,185,122,47,142,31,59,130,201,124,21,181,71,160,5,17,0,13,173,
21,92,87,177,171,60,191,171,92,134,246,139,161,171,236,67,251,213,96,133,160,21,174,222,119,5,118,239,217,129,104,212,198,53,251,118,226,200,145,215,241,196,99,143,65,16,166,93,216,216,85,94,208,85,30,
159,225,42,7,35,90,171,34,122,123,219,144,140,57,161,173,188,25,247,190,237,173,104,228,41,217,85,110,218,85,46,133,174,114,49,156,42,92,192,248,72,37,19,136,39,162,232,234,234,192,142,29,219,177,255,
186,107,231,28,185,51,71,49,187,202,179,92,229,106,232,42,251,117,87,25,70,225,252,224,185,105,182,114,36,18,89,210,95,148,93,229,89,174,114,237,76,33,24,165,143,62,250,8,94,120,225,5,104,173,49,50,50,
130,251,238,187,111,214,200,109,44,118,149,151,232,42,107,173,241,213,175,126,21,95,253,234,87,217,85,102,87,121,241,128,217,85,158,90,27,179,171,204,174,50,187,202,115,254,44,178,171,28,22,187,202,236,
42,179,171,60,59,100,118,149,167,21,187,202,203,40,118,149,193,174,50,216,85,190,128,85,4,187,202,211,138,93,229,197,118,117,243,141,106,118,149,155,60,208,89,40,248,165,156,93,204,83,236,42,179,171,204,
174,114,195,0,102,87,121,102,177,171,204,174,50,187,202,115,188,236,236,42,3,236,42,47,125,131,81,155,30,216,85,94,194,193,205,82,223,90,98,87,121,25,7,55,205,46,211,216,85,94,230,193,77,179,27,12,118,
149,151,121,112,211,76,208,236,42,95,192,193,205,98,163,154,93,101,118,149,217,85,94,201,128,217,85,102,87,121,145,164,216,85,102,87,153,93,101,118,149,217,85,102,87,185,137,128,1,118,149,151,28,48,187,
202,236,42,175,110,177,171,188,54,33,179,171,188,90,83,196,133,20,187,202,235,160,216,85,94,239,1,135,33,179,171,188,130,171,8,46,14,152,3,230,128,185,56,96,14,152,139,3,230,128,57,96,46,14,152,3,230,
128,185,56,96,14,152,3,230,226,128,57,96,46,14,120,109,107,221,186,202,51,76,229,153,79,200,24,96,99,248,202,235,221,85,38,0,168,115,192,8,239,184,71,240,152,217,70,240,149,215,159,171,60,195,84,54,70,
83,208,160,169,161,148,210,178,181,85,155,106,213,64,8,35,34,81,189,94,125,229,245,233,42,131,36,9,33,12,180,32,33,69,227,141,44,128,81,16,182,1,160,54,130,175,188,126,93,101,34,155,132,8,30,164,11,244,
40,10,31,192,51,193,61,202,198,3,224,194,192,35,34,119,189,250,202,235,209,85,142,212,218,57,102,226,227,93,169,72,161,207,182,188,54,73,126,82,144,178,9,90,25,99,202,198,232,172,210,98,176,236,198,143,
159,207,111,61,165,200,22,0,170,114,157,249,202,235,216,85,198,78,33,40,105,194,103,233,130,210,65,255,75,168,160,31,166,113,1,51,132,45,45,175,231,72,251,207,106,99,190,111,52,190,173,140,245,146,48,
213,226,99,135,111,47,93,146,1,95,144,171,76,150,48,136,194,64,194,135,5,67,2,128,48,38,108,167,66,228,27,130,43,133,169,146,48,101,18,70,0,26,25,18,250,46,203,120,119,25,225,255,188,109,42,39,17,184,
202,223,6,187,202,115,185,202,145,188,171,236,162,235,219,21,79,89,174,210,228,107,4,243,46,65,187,18,94,217,166,74,213,17,5,21,149,121,59,74,185,180,77,232,21,208,253,128,150,128,217,9,232,157,8,140,
56,118,149,231,112,149,29,146,93,189,137,246,174,110,97,59,14,108,41,133,20,4,34,34,18,48,90,152,170,47,84,53,167,70,198,135,127,231,80,177,128,135,34,137,221,143,117,38,243,213,182,232,153,219,109,81,
188,151,128,27,193,174,242,108,87,57,214,233,68,200,33,17,16,192,65,227,81,83,83,62,41,92,80,32,120,248,60,240,149,255,11,200,27,7,84,206,83,94,241,240,196,216,232,163,45,173,153,135,19,17,170,116,119,
181,188,73,8,188,23,236,42,55,184,202,102,202,86,158,226,127,17,134,43,67,225,192,10,122,176,141,26,208,200,127,7,121,99,64,205,87,14,240,212,146,91,45,159,247,125,239,225,120,60,250,215,236,42,55,184,
202,166,1,44,153,234,173,61,133,87,7,46,82,176,170,163,225,33,208,248,231,1,111,52,80,194,107,128,245,116,95,153,93,101,29,182,30,55,48,117,143,163,182,83,152,34,128,169,142,172,214,203,159,167,249,222,
244,49,195,174,242,148,171,220,40,43,135,255,12,93,14,170,79,13,118,40,42,90,192,137,175,131,42,207,129,188,44,224,79,132,83,68,216,121,123,126,95,249,114,118,149,41,188,128,77,25,116,193,115,207,206,
180,96,131,254,64,2,240,78,215,57,52,132,108,251,84,158,243,94,187,216,85,110,212,100,80,191,184,5,59,58,17,250,202,198,151,32,53,14,132,68,26,105,15,102,154,175,188,96,200,107,230,42,47,53,224,41,60,
21,225,106,33,156,115,131,105,161,30,238,224,80,249,138,74,160,211,250,243,187,202,52,191,171,28,119,4,50,113,137,142,36,161,45,78,136,219,147,136,136,73,72,248,112,93,27,229,74,18,19,145,173,24,26,212,
200,143,21,80,42,248,245,21,200,148,209,177,240,64,177,44,235,230,108,54,251,153,108,54,251,227,253,253,253,9,4,79,123,94,15,224,79,239,220,255,208,141,43,49,39,175,91,87,217,40,31,197,66,17,197,66,1,
147,249,73,84,43,37,120,213,18,160,42,176,200,171,187,202,241,40,16,177,52,188,106,1,167,78,28,197,153,51,167,80,41,23,177,68,192,250,114,116,149,7,49,62,150,133,86,46,140,174,6,224,137,246,194,127,86,
0,83,133,241,75,32,120,136,69,128,84,220,65,44,238,96,247,238,157,40,21,39,113,230,204,105,118,149,231,117,149,67,246,87,24,23,240,93,0,30,200,120,208,170,38,38,250,48,170,2,24,23,198,120,40,228,171,40,
21,130,126,210,150,20,72,36,226,236,42,55,231,42,251,48,42,160,209,200,40,144,242,96,160,176,117,115,15,206,15,22,81,173,86,161,149,7,33,106,140,132,130,171,60,248,126,48,114,217,85,94,208,85,86,208,126,
5,84,123,150,220,247,145,201,36,241,214,183,190,21,209,136,13,99,20,142,29,59,138,199,14,62,30,238,218,12,0,141,64,166,96,87,185,9,87,185,182,105,40,65,123,69,192,84,112,221,181,123,16,143,74,164,146,
113,108,221,188,9,247,189,227,237,117,44,80,107,53,45,92,118,149,151,228,42,7,224,84,123,107,6,241,120,20,93,93,237,216,182,125,43,174,185,102,31,106,124,79,45,220,198,169,129,93,229,37,184,202,48,10,
231,206,157,67,34,30,67,79,119,23,118,237,220,25,54,153,107,238,218,195,174,242,34,174,50,96,240,248,227,143,225,200,145,35,136,70,163,200,100,50,248,240,135,63,204,174,242,74,186,202,66,8,124,249,203,
95,198,151,190,244,165,250,84,192,174,50,187,202,243,79,15,181,129,204,174,50,187,202,243,254,25,118,149,217,85,102,87,153,93,101,118,149,217,85,158,29,50,187,202,211,138,93,229,101,20,187,202,96,87,25,
236,42,95,192,42,130,93,229,105,197,174,242,98,187,186,249,70,53,187,202,77,30,232,44,20,252,82,206,46,230,41,118,149,217,85,102,87,185,97,0,179,171,60,179,216,85,102,87,153,93,229,57,94,118,118,149,1,
118,149,151,190,193,168,77,15,236,42,47,225,224,102,169,111,45,177,171,188,140,131,155,102,151,105,236,42,47,243,224,166,217,13,6,187,202,203,60,184,105,38,104,118,149,47,224,224,102,177,81,205,174,50,
187,202,236,42,175,100,192,236,42,179,171,188,72,82,236,42,179,171,204,174,50,187,202,236,42,179,171,220,68,192,0,187,202,75,14,152,93,101,118,149,87,183,216,85,94,155,144,217,85,94,173,41,226,66,138,
93,229,117,80,236,42,175,247,128,195,144,217,85,94,193,85,4,23,7,204,1,115,192,92,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,188,182,181,81,93,229,90,205,
221,233,131,3,158,55,208,69,93,229,250,255,32,165,81,185,92,237,189,192,218,127,167,57,224,249,3,93,220,85,54,154,26,250,88,64,102,50,10,66,24,227,121,58,12,28,235,45,232,117,235,42,7,190,142,33,18,50,
236,236,1,144,180,97,180,6,17,5,189,44,140,84,70,107,77,150,237,195,104,37,146,73,21,34,214,2,235,4,177,94,215,174,50,130,71,250,37,106,143,246,3,32,33,2,110,32,112,123,60,146,178,26,52,142,23,30,72,120,
70,43,191,1,177,190,244,3,190,64,87,185,59,233,76,246,217,150,223,38,133,78,9,50,17,130,214,218,160,162,141,200,249,138,206,151,220,228,201,209,98,247,25,173,69,145,128,42,136,170,38,248,122,168,221,117,
116,73,6,124,161,174,178,37,177,43,104,231,104,133,141,149,68,125,193,64,208,65,131,103,120,32,51,130,254,182,35,121,163,253,23,181,214,143,123,190,124,168,236,198,94,240,148,83,56,146,219,156,15,127,
114,204,37,21,240,74,184,202,26,22,12,130,38,30,6,210,24,80,48,53,64,131,160,132,48,30,17,170,36,116,25,36,42,105,65,184,93,138,202,237,142,172,252,108,194,153,56,109,180,122,180,231,186,99,15,25,67,207,
73,225,191,198,174,242,44,87,217,41,186,190,93,245,180,229,41,45,148,210,181,128,141,47,160,125,41,60,207,17,85,29,17,69,25,21,249,120,132,114,109,22,153,62,24,45,9,186,159,132,238,39,163,63,4,210,236,
42,47,224,42,119,77,185,202,16,36,130,134,176,100,200,40,159,76,197,51,149,146,171,198,115,103,95,127,193,146,71,254,60,21,41,231,122,18,199,175,138,201,177,59,36,252,235,0,114,0,179,31,236,42,207,227,
42,135,45,116,231,234,176,77,52,5,252,97,220,2,157,251,79,190,95,45,28,207,231,178,79,38,19,169,135,227,78,37,215,219,41,111,177,132,255,110,192,176,171,60,183,171,172,67,184,186,22,47,213,31,108,148,
245,231,240,44,96,216,7,101,255,123,136,167,142,3,170,80,246,42,147,67,158,87,249,110,44,22,249,235,29,219,183,108,3,244,123,192,174,242,20,118,82,235,238,106,194,86,228,181,78,131,212,96,35,213,156,101,
12,188,12,202,125,185,33,224,154,175,236,1,70,149,93,183,250,119,45,45,233,63,103,87,185,193,85,54,211,206,112,168,54,26,106,195,2,211,236,229,202,235,11,156,7,33,230,56,246,143,20,10,133,183,179,171,
60,203,85,174,253,58,232,108,85,163,211,72,212,116,112,9,210,54,232,248,255,2,249,217,169,17,236,7,240,148,9,245,196,25,187,102,118,149,167,185,202,13,31,82,68,194,127,218,83,15,153,187,2,164,43,32,93,
157,3,172,214,115,29,73,92,222,174,114,109,158,13,24,96,81,151,193,81,23,194,197,212,197,14,18,166,34,1,93,10,32,20,83,173,155,159,152,182,250,152,85,151,183,171,44,165,64,194,145,72,199,5,58,18,132,214,
56,33,102,21,96,81,17,48,62,170,85,27,101,175,29,147,229,20,178,213,99,152,116,124,148,189,114,160,215,106,63,236,23,180,232,17,196,229,233,42,95,179,85,98,87,119,25,208,62,10,133,34,74,147,5,228,243,
5,84,43,101,184,110,5,70,121,144,66,193,22,26,142,229,35,30,1,34,150,130,239,78,226,244,201,99,24,56,123,28,213,242,36,140,241,103,241,105,11,212,229,229,42,31,121,121,8,3,231,6,49,49,62,22,184,202,202,
5,153,176,125,186,241,195,81,234,194,168,64,153,137,69,128,68,76,34,30,179,176,123,87,63,138,197,28,206,158,62,13,215,173,52,123,206,115,57,185,202,231,1,19,192,212,66,123,64,200,254,194,40,152,218,69,
204,4,20,37,193,131,209,30,10,149,42,74,121,5,24,23,150,69,72,196,162,104,109,109,193,228,100,30,165,82,211,15,224,95,206,174,114,240,184,51,84,176,94,214,218,135,101,17,148,87,115,149,13,200,248,80,202,
135,235,43,248,225,200,173,65,170,236,42,55,229,42,7,203,175,72,196,198,93,119,222,142,142,142,118,128,12,94,62,252,50,158,121,230,233,112,7,88,163,127,77,216,100,41,88,133,178,171,188,152,171,236,23,
1,83,6,153,50,174,186,114,39,186,58,91,145,136,71,209,211,217,142,251,222,241,54,68,108,11,90,249,8,30,65,175,25,203,236,42,55,239,42,235,16,156,50,10,155,55,117,33,26,117,208,209,209,138,29,59,251,177,
127,255,181,168,86,203,245,145,27,76,13,236,42,47,209,85,214,33,219,227,97,100,100,4,137,120,12,155,122,123,176,125,219,54,108,217,178,165,169,165,24,187,202,11,186,202,83,235,217,23,95,124,1,249,124,
30,109,109,109,104,107,107,195,199,63,254,241,57,191,121,118,149,151,233,42,23,139,69,60,248,224,131,120,240,193,7,97,89,86,189,235,20,187,202,171,224,42,55,182,244,98,87,153,93,229,233,197,174,50,106,
155,20,118,149,231,122,65,216,85,102,87,25,236,42,79,21,187,202,236,42,179,171,60,99,13,204,174,242,204,98,87,153,93,101,118,149,103,157,62,179,171,60,85,236,42,47,182,171,155,111,84,179,171,220,228,129,
206,66,193,47,229,236,98,158,98,87,153,93,101,118,149,27,6,48,187,202,51,139,93,101,118,149,217,85,158,227,101,103,87,25,96,87,121,233,27,140,218,244,192,174,242,18,14,110,150,250,214,18,187,202,203,56,
184,105,118,153,198,174,242,50,15,110,154,221,96,176,171,188,204,131,155,102,130,102,87,249,2,14,110,22,27,213,236,42,179,171,204,174,242,74,6,204,174,50,187,202,139,36,197,174,242,218,184,202,230,141,
95,122,199,216,190,207,178,171,188,26,174,178,121,227,151,234,174,242,216,190,207,178,171,188,146,174,178,127,235,23,103,185,202,99,251,62,203,174,242,133,186,202,230,141,95,90,244,94,128,182,195,31,101,
87,25,75,112,149,233,77,95,94,178,171,220,118,248,163,236,42,99,21,139,93,229,85,46,118,149,215,38,100,118,149,87,171,46,196,101,102,87,121,29,20,187,202,235,61,224,48,100,118,149,87,112,21,193,197,1,
115,192,28,48,23,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,197,1,175,109,93,18,174,242,122,126,159,111,195,186,202,0,160,11,193,93,65,51,94,140,117,245,198,
234,198,113,149,167,122,8,213,123,89,200,84,90,67,144,49,158,167,17,222,26,160,114,57,29,126,238,117,17,244,198,113,149,73,214,122,8,193,104,109,136,200,64,144,50,218,104,178,157,224,182,90,165,180,108,
109,213,181,155,91,10,7,31,185,232,211,199,198,114,149,131,70,121,32,203,50,1,245,2,69,100,60,32,236,219,40,165,7,99,60,3,237,137,100,82,233,66,65,93,236,144,55,136,171,172,218,165,80,105,65,198,6,0,109,
168,170,53,229,61,101,141,22,221,196,169,177,98,251,128,49,162,12,80,25,70,149,73,90,146,164,116,145,76,226,98,135,188,113,92,229,198,254,64,70,133,182,114,17,64,22,59,245,241,130,82,250,101,165,233,233,
178,235,124,111,178,156,121,193,83,114,98,4,155,115,181,144,113,145,110,140,217,96,174,50,133,125,219,180,16,193,236,64,4,15,68,149,164,20,213,155,44,83,185,41,106,231,126,178,53,54,114,22,90,61,190,189,
245,240,183,141,49,207,86,253,248,171,207,30,68,241,98,140,226,141,227,42,27,81,119,40,9,198,8,82,218,22,30,108,89,21,49,89,140,70,68,33,109,83,190,83,0,146,132,222,38,160,183,73,114,223,111,160,94,113,
68,229,224,155,174,123,248,139,135,112,199,55,54,108,192,107,225,42,215,26,127,144,129,209,62,76,213,215,126,197,245,11,3,67,163,103,201,156,248,94,212,46,140,116,199,207,245,165,237,161,235,108,82,123,
97,164,13,232,107,136,244,149,128,249,224,27,74,191,252,61,0,191,250,76,252,23,217,85,158,237,42,215,250,20,7,141,60,68,40,34,214,159,195,51,22,244,24,10,249,35,159,122,44,26,141,31,140,202,210,72,95,
91,225,234,136,44,188,141,140,223,5,232,4,96,216,85,158,219,85,110,188,121,83,212,149,153,218,211,164,66,132,207,72,15,91,160,225,95,0,249,19,85,183,156,29,245,220,210,247,35,78,236,111,118,108,77,245,
74,114,239,7,204,29,96,87,121,202,85,14,2,198,180,5,64,48,130,67,188,154,236,16,84,13,125,229,51,135,64,147,95,5,249,89,192,159,168,233,180,229,170,91,254,135,76,58,245,231,237,109,45,61,182,45,254,45,
216,85,158,233,42,55,198,59,213,60,211,52,28,249,24,109,64,149,167,26,122,178,213,43,230,216,242,195,197,66,238,158,220,68,246,31,132,0,187,202,51,93,229,58,92,93,107,231,72,50,100,43,237,58,193,78,21,
11,116,246,143,64,222,40,200,27,171,17,236,32,93,9,58,49,76,89,31,236,42,79,119,149,237,240,225,242,8,132,136,64,136,104,192,174,83,163,175,44,64,101,1,210,85,80,72,163,205,108,198,223,80,236,42,207,116,
149,167,116,112,212,127,93,107,237,88,155,143,81,124,30,208,1,134,2,227,162,134,161,152,185,125,101,118,149,167,185,202,49,66,196,42,64,162,24,52,53,170,218,40,121,25,76,150,35,152,168,10,20,93,129,74,
233,107,48,170,16,32,214,117,228,111,65,188,122,77,92,101,107,137,83,67,128,167,218,182,21,174,115,147,109,241,236,142,173,29,3,191,42,236,200,86,159,82,42,239,182,173,144,171,92,69,161,80,68,113,178,
128,201,177,2,114,149,10,92,215,133,86,10,130,12,44,9,196,164,65,187,3,56,81,13,117,195,118,12,156,61,137,243,3,99,168,250,129,16,78,80,13,12,197,188,65,223,8,224,255,158,56,113,226,195,253,253,253,69,
4,15,131,95,11,224,75,119,238,127,232,95,47,103,117,177,236,128,47,158,171,28,132,133,112,106,33,232,58,3,76,240,17,117,128,68,148,16,143,10,236,234,239,69,185,24,195,217,51,39,224,86,117,131,56,179,96,
93,222,174,50,133,157,4,141,10,78,209,106,48,42,25,31,6,62,74,101,23,149,188,66,214,84,97,9,131,68,220,65,75,75,26,133,188,102,87,121,97,87,89,193,168,192,72,38,163,2,17,28,38,104,10,91,219,144,104,31,
66,24,32,244,63,93,163,224,123,147,128,158,34,128,125,223,99,87,121,126,87,57,120,158,92,107,21,62,91,14,64,43,188,241,246,91,176,105,83,15,78,158,60,133,39,159,124,50,88,146,25,31,130,8,208,225,159,15,
55,37,236,42,47,232,42,23,161,253,2,160,74,128,169,192,168,18,222,120,251,1,236,216,177,25,233,116,28,183,220,124,3,254,211,207,255,28,180,114,33,194,185,89,55,140,94,118,149,151,224,42,7,182,178,135,
150,76,18,182,69,104,109,73,97,235,214,62,236,221,187,7,219,182,246,133,23,51,221,160,131,179,171,188,100,87,25,70,161,92,42,32,17,143,161,171,179,3,187,119,237,196,190,171,174,10,71,231,252,75,50,118,
149,155,116,149,133,32,28,61,122,4,137,68,2,221,221,221,232,235,235,195,231,62,247,185,89,161,178,171,188,76,87,185,22,206,111,252,198,111,224,192,129,3,120,238,185,231,234,141,238,217,85,94,97,87,249,
233,167,159,174,183,245,98,87,153,93,229,233,197,174,114,80,236,42,207,243,130,176,171,204,174,50,216,85,158,42,118,149,217,85,102,87,121,198,26,152,93,229,153,197,174,50,187,202,236,42,207,58,125,102,
87,121,170,216,85,94,108,87,55,223,168,102,87,121,145,121,149,93,101,118,149,103,12,92,118,149,231,250,156,236,42,47,54,170,217,85,102,87,153,93,229,240,247,236,42,179,171,204,174,50,187,202,181,101,61,
187,202,203,88,166,177,171,188,204,131,155,102,55,24,236,42,47,243,224,166,153,160,217,85,190,128,131,155,197,70,53,187,202,236,42,179,171,188,146,1,179,171,204,174,242,34,73,177,171,204,174,50,187,202,
236,42,179,171,204,174,114,19,1,3,236,42,47,57,96,118,149,217,85,94,221,98,87,121,109,66,102,87,121,181,138,93,229,13,94,236,42,175,247,128,195,144,217,85,94,193,85,4,23,7,204,1,115,192,92,28,48,7,204,
197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,188,182,117,73,184,202,192,250,181,149,47,5,87,25,152,110,43,175,43,204,122,99,186,202,193,159,54,34,153,52,68,65,167,187,
245,106,43,111,88,87,57,124,17,20,180,209,36,132,70,68,4,79,63,42,165,69,60,174,200,182,117,45,232,198,27,11,47,201,128,87,196,85,14,62,130,209,45,101,160,251,1,10,194,248,0,60,10,158,72,245,65,228,146,
109,251,198,243,252,26,98,29,222,189,99,54,60,154,58,79,184,43,230,42,7,221,175,0,24,184,74,139,156,175,236,236,100,53,121,46,87,106,29,212,134,130,118,218,64,5,198,84,200,178,170,198,24,23,66,64,102,
50,168,221,174,117,201,4,188,58,174,114,227,83,253,62,8,101,244,152,49,24,115,170,224,251,230,117,79,137,231,202,110,244,224,68,169,229,37,95,217,99,185,114,102,28,36,42,38,120,129,17,2,214,122,195,7,
188,178,174,178,5,3,17,186,202,34,236,48,161,5,145,38,130,34,50,62,8,94,82,10,247,6,203,84,110,72,68,114,31,109,79,140,14,106,165,158,242,21,125,215,87,226,208,88,177,253,229,115,249,93,147,0,106,115,
178,217,144,1,175,150,171,236,107,161,181,22,141,176,167,145,164,33,133,34,71,184,86,68,86,34,142,172,36,29,81,104,133,41,9,162,74,159,69,165,62,91,86,222,5,229,191,150,176,199,158,216,146,126,253,239,
31,215,111,255,198,90,79,21,27,204,85,158,154,33,180,130,214,158,86,85,207,45,15,142,100,179,80,231,158,114,172,202,68,71,108,172,45,19,29,221,17,33,189,77,192,88,36,212,62,203,232,43,140,80,239,127,99,
255,151,190,167,53,253,218,179,184,227,201,181,10,120,195,184,202,102,150,76,91,147,17,195,86,10,144,48,21,225,23,207,251,227,17,187,240,28,229,254,254,249,77,153,177,205,9,59,123,171,52,165,14,192,139,
147,81,4,232,81,192,176,171,60,167,171,108,26,58,109,131,194,112,173,6,219,51,104,173,128,172,52,222,169,255,47,235,85,75,207,74,43,249,79,219,122,116,58,42,199,239,22,240,110,3,116,44,156,33,216,85,158,
238,42,155,105,199,18,181,214,9,130,44,144,8,116,218,224,209,93,27,56,251,18,40,247,119,32,47,11,168,92,181,90,46,255,75,34,149,249,203,206,22,106,139,58,250,255,7,152,235,195,79,198,174,114,240,143,198,
38,252,97,175,98,163,97,136,160,161,33,77,208,140,212,16,0,101,64,229,135,129,169,165,93,36,226,232,247,121,197,115,111,58,149,171,252,35,1,63,187,103,119,255,3,96,87,121,218,151,154,246,149,9,13,190,
50,228,180,17,140,130,13,26,252,12,200,203,214,70,112,192,167,233,42,76,32,32,84,0,60,176,101,203,230,86,118,149,133,221,224,42,7,93,82,166,124,229,8,36,57,33,95,105,133,205,153,4,168,120,58,236,27,31,
64,40,100,106,15,124,214,57,138,40,96,254,130,93,229,89,174,50,213,134,111,195,40,23,83,43,10,146,128,150,192,228,63,3,186,12,210,161,147,132,169,233,167,161,216,85,158,203,85,22,166,4,165,53,74,21,11,
69,55,141,201,138,141,92,149,80,246,8,174,22,208,69,9,242,243,128,42,194,212,109,80,63,148,190,102,15,20,118,149,107,174,242,120,1,185,114,21,174,235,66,169,64,60,20,130,144,16,132,54,71,192,142,26,248,
94,5,195,213,20,70,100,5,174,91,156,18,194,141,187,152,175,204,174,242,148,171,60,61,32,173,53,72,24,68,29,129,88,84,32,30,17,232,223,146,65,181,104,112,238,220,56,60,191,26,172,54,204,162,190,50,187,
202,6,26,70,77,169,95,218,232,240,71,74,163,92,81,168,228,21,198,181,7,75,42,196,99,18,173,153,24,10,249,114,192,153,53,106,97,243,23,187,202,208,65,80,90,155,250,181,46,16,104,81,39,37,60,207,195,100,
213,0,198,131,16,193,166,37,112,149,229,162,33,179,171,172,107,91,230,96,186,233,233,238,193,182,237,91,16,113,28,60,250,232,247,161,181,27,244,149,54,26,74,251,193,159,55,26,149,74,57,116,149,91,22,159,
6,47,111,87,57,212,20,117,5,251,174,220,129,183,221,115,59,174,216,189,21,123,118,111,197,125,239,188,27,130,124,192,248,80,202,133,104,64,86,163,209,8,92,183,202,174,114,179,174,178,49,30,182,109,237,
131,20,6,153,84,2,155,55,245,224,45,119,223,137,190,222,46,192,40,4,111,141,234,58,96,205,174,242,18,93,101,65,6,158,91,65,34,30,67,79,79,23,118,236,216,142,235,174,189,6,231,206,157,197,76,38,184,86,
236,42,47,193,85,6,12,178,217,81,196,227,113,116,118,118,162,191,191,31,227,227,227,152,78,6,179,171,188,108,87,89,107,141,67,135,14,65,8,129,83,167,78,225,155,223,252,38,254,254,239,255,126,190,145,200,
174,242,133,184,202,127,253,215,127,141,191,253,219,191,101,87,153,93,229,25,197,174,114,80,236,42,207,243,130,176,171,204,174,50,216,85,158,42,118,149,217,85,102,87,121,198,26,152,93,229,153,197,174,
50,187,202,236,42,207,58,125,102,87,121,170,216,85,94,108,87,55,223,168,102,87,121,145,121,149,93,101,118,149,103,12,92,118,149,231,250,156,236,42,47,54,170,217,85,102,87,153,93,229,240,247,236,42,179,
171,204,174,50,187,202,181,101,61,187,202,203,88,166,177,171,188,204,131,155,102,55,24,236,42,47,243,224,166,153,160,217,85,190,128,131,155,197,70,53,187,202,236,42,179,171,188,146,1,179,171,204,174,242,
34,73,177,171,204,174,50,187,202,236,42,179,171,204,174,114,19,1,3,236,42,47,57,96,118,149,217,85,94,221,98,87,121,109,66,102,87,121,181,138,93,229,13,94,236,42,175,247,128,195,144,217,85,94,193,85,4,
23,7,204,1,115,192,92,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,188,182,197,174,242,229,18,240,10,186,202,192,58,122,115,245,210,114,149,1,64,8,163,11,
5,213,8,89,95,204,176,47,29,87,217,166,160,77,54,9,35,51,25,101,60,79,55,218,202,184,84,217,223,133,66,93,33,87,89,1,80,68,228,129,200,131,209,62,140,84,20,181,166,217,202,97,208,107,62,87,111,96,87,89,
230,61,95,102,11,110,234,92,174,212,58,164,141,168,0,166,2,146,193,99,14,128,75,182,237,1,112,13,180,39,146,73,21,78,29,216,176,42,237,92,163,118,117,93,101,3,66,14,90,159,46,40,133,163,174,111,189,80,
172,196,158,156,40,183,30,246,149,53,94,112,211,19,208,170,76,150,45,0,84,69,50,137,181,14,121,195,185,202,225,250,65,16,153,208,85,86,32,233,39,133,244,246,199,109,111,127,60,154,255,225,142,212,216,
121,165,240,130,235,139,71,171,158,253,92,182,216,253,210,120,185,115,2,64,85,102,50,102,45,233,223,141,233,42,3,144,194,64,10,37,109,225,89,142,229,69,34,178,146,112,68,57,67,166,34,4,85,55,217,178,188,
201,177,43,247,36,34,147,71,91,98,35,79,123,158,248,210,243,131,111,250,154,49,218,88,237,29,102,173,0,235,13,238,42,43,85,245,220,106,113,52,59,97,212,224,171,182,244,10,173,241,66,38,29,153,232,141,
202,124,175,4,73,33,112,133,133,210,78,71,148,239,191,101,235,191,28,244,148,253,235,79,159,190,251,241,112,190,89,245,128,55,164,171,28,236,64,66,237,128,4,4,9,24,35,0,151,116,113,216,43,56,86,225,20,
138,167,206,118,198,95,178,211,206,200,62,11,249,22,161,75,81,97,170,100,224,101,161,241,39,74,211,31,91,210,59,205,174,242,172,46,175,168,119,173,154,178,149,67,95,153,44,144,144,128,182,224,87,164,81,
99,254,128,30,251,147,191,234,109,45,91,105,123,232,128,141,201,55,0,94,148,140,15,192,60,6,24,118,149,103,187,202,83,71,19,20,138,7,160,192,236,144,194,9,157,229,16,176,30,200,3,217,223,244,170,229,210,
35,118,180,229,111,186,210,165,88,42,146,251,40,25,255,186,176,11,247,170,187,202,23,18,240,130,174,242,190,77,175,254,246,60,174,114,191,29,186,202,53,224,164,121,87,185,33,96,66,29,77,165,16,164,162,
208,77,18,194,2,180,5,113,228,127,130,252,243,48,222,24,224,230,198,93,183,242,21,144,245,185,61,253,233,247,10,82,255,26,48,118,184,97,249,38,128,37,187,202,171,25,240,197,115,149,27,191,137,250,84,33,
3,198,50,124,70,90,8,11,200,219,16,131,191,11,242,178,48,126,22,194,207,195,232,34,72,87,42,90,211,71,182,108,238,202,216,182,248,47,96,87,185,193,85,174,127,132,190,50,57,193,135,176,194,167,251,229,
84,232,147,135,16,244,115,82,32,19,118,222,14,166,158,40,193,253,179,231,159,103,87,121,14,87,121,238,195,96,10,123,199,215,91,218,84,36,80,122,60,100,43,171,161,67,167,26,47,148,233,246,246,182,127,61,
50,50,178,143,93,229,57,92,229,98,197,66,177,154,70,174,226,32,87,1,42,62,224,154,176,69,13,9,32,239,129,212,148,62,131,80,10,167,240,226,89,107,32,192,174,242,60,174,114,208,42,87,32,33,36,90,29,11,34,
74,240,61,23,163,222,221,24,83,4,111,252,15,65,42,15,168,66,131,175,236,1,240,103,145,106,96,87,185,193,85,110,152,20,130,166,75,193,82,205,113,108,196,98,54,98,206,113,108,179,129,106,151,135,161,193,
18,188,80,89,52,186,230,43,179,171,188,176,171,172,85,168,250,80,56,149,6,65,87,170,26,213,2,33,103,20,44,169,17,139,8,180,164,8,5,99,80,46,86,67,227,222,107,80,195,230,12,154,93,101,212,251,251,152,122,
127,54,13,64,144,8,118,124,218,135,7,5,85,81,129,171,12,5,232,42,148,95,69,208,15,122,225,199,249,216,85,174,191,219,57,53,221,200,112,117,225,68,28,84,202,165,64,164,173,249,29,164,1,120,40,151,11,240,
92,23,173,173,153,197,206,119,86,220,85,110,106,138,8,47,108,210,0,17,154,230,42,39,235,174,114,122,119,231,118,93,95,138,45,236,42,103,172,2,30,125,248,105,12,156,62,1,227,79,134,50,109,25,208,229,224,
159,164,194,17,235,162,198,247,214,244,46,163,117,120,12,22,236,230,126,228,135,63,82,255,62,79,159,62,131,239,126,247,187,48,33,247,11,163,33,96,16,141,56,40,22,242,56,115,38,143,45,91,54,47,246,215,
173,185,202,247,164,211,233,155,17,116,158,250,216,157,251,31,250,135,229,116,158,218,32,174,178,59,203,85,22,164,112,205,213,123,33,160,145,74,198,177,169,183,19,119,191,249,141,104,107,203,128,72,3,
218,135,86,126,29,174,110,109,109,129,239,123,236,42,207,239,42,251,211,92,101,173,92,116,180,181,32,26,115,208,209,209,138,237,219,183,226,234,125,87,97,83,79,103,184,20,155,78,6,179,171,188,68,87,185,
6,253,197,162,17,116,118,180,99,251,182,109,216,186,117,11,94,122,233,165,169,105,101,198,124,123,49,92,101,177,232,197,205,87,130,164,152,203,85,174,20,252,77,113,43,101,217,186,126,112,19,186,202,181,
203,79,131,171,252,202,75,11,185,202,193,136,69,200,255,214,2,157,10,201,204,217,79,237,224,193,239,227,233,167,159,70,38,147,65,50,153,196,71,62,242,17,44,100,212,53,186,202,249,124,62,0,173,22,168,208,
85,62,1,96,48,188,94,213,92,229,149,185,200,213,47,110,6,22,209,148,171,236,147,131,92,57,49,20,105,223,182,213,144,31,172,24,154,116,149,197,188,174,114,141,254,157,251,253,200,185,48,63,223,247,241,
232,163,143,226,209,71,31,157,25,76,253,207,204,231,42,23,10,5,28,57,114,4,251,246,237,91,168,203,96,44,155,205,126,32,153,76,126,81,8,241,9,76,185,202,23,12,87,179,171,60,181,54,102,87,153,93,101,118,
149,231,252,89,100,87,57,44,118,149,217,85,102,87,121,118,200,236,42,79,43,118,149,151,81,236,42,131,93,101,176,171,124,1,171,8,118,149,167,21,187,202,139,237,234,230,27,213,236,42,47,50,175,178,171,204,
174,242,140,129,203,174,242,92,159,147,93,229,197,70,53,187,202,236,42,179,171,28,254,158,93,101,118,149,217,85,102,87,185,182,172,103,87,121,25,203,52,118,149,151,121,112,211,236,6,131,93,229,101,30,
220,52,19,52,187,202,23,112,112,179,216,168,102,87,153,93,101,118,149,87,50,96,118,149,217,85,94,36,41,118,149,217,85,102,87,153,93,101,118,149,217,85,110,34,96,128,93,229,37,7,204,174,50,187,202,171,
91,236,42,175,77,200,236,42,175,86,177,171,188,193,139,93,229,245,30,112,24,50,187,202,43,184,138,224,226,128,57,96,14,152,139,3,230,128,185,56,96,14,152,3,230,226,128,57,96,14,152,139,3,230,128,57,96,
46,14,152,3,230,226,128,215,182,216,85,190,92,2,94,105,87,121,189,132,126,73,186,202,36,165,185,216,220,239,69,11,120,53,92,101,16,25,178,157,224,121,92,41,53,148,210,34,30,87,181,176,11,7,31,209,225,
136,214,151,100,192,107,225,42,195,24,15,150,229,195,24,109,0,143,132,168,221,203,172,68,60,174,201,182,117,120,23,143,185,36,216,223,134,112,215,200,85,166,42,0,23,68,46,17,185,0,220,160,163,62,185,112,
132,11,163,213,197,176,149,47,9,87,217,232,51,69,63,112,149,95,44,84,19,79,143,151,90,95,245,148,53,81,174,198,38,64,84,38,99,42,128,112,73,74,23,241,56,233,82,201,95,171,144,55,168,171,108,4,145,38,130,
38,50,62,72,170,132,144,254,117,49,219,187,46,30,155,252,112,71,42,55,236,251,120,169,226,201,39,202,110,236,133,177,82,199,225,130,155,25,131,82,16,209,40,32,132,169,141,100,172,242,29,73,27,211,85,38,
64,80,208,160,201,22,190,237,88,94,196,145,110,194,17,149,180,48,85,65,150,219,99,203,106,79,196,169,190,57,21,45,157,108,143,143,62,87,246,172,127,121,117,228,13,95,131,82,16,145,104,125,165,177,97,2,
94,107,87,217,40,163,61,79,171,170,231,186,197,209,108,222,232,243,199,108,233,151,51,241,114,42,19,153,236,142,88,133,14,105,202,82,160,184,211,17,149,237,142,61,249,182,155,54,127,243,201,98,37,241,
91,135,135,110,62,8,33,180,213,222,97,252,236,232,170,250,202,214,138,78,9,241,184,69,36,36,64,83,174,178,48,179,92,101,61,229,42,23,167,185,202,38,221,31,219,234,68,226,211,92,101,21,32,37,141,25,16,
64,22,9,216,36,4,108,59,221,178,41,1,108,238,53,46,244,196,168,91,46,186,133,33,51,49,112,170,45,118,82,102,34,99,219,108,228,146,146,84,82,8,245,150,116,108,226,134,155,182,124,237,179,101,47,254,199,
199,71,174,58,145,3,173,234,141,139,27,211,85,14,54,26,33,252,23,152,29,245,165,51,73,144,145,80,174,52,170,168,138,254,248,201,39,59,34,79,101,91,162,131,123,162,52,190,151,116,41,34,76,21,218,168,39,
141,54,191,231,43,249,245,199,15,223,54,182,156,240,46,97,87,185,182,139,38,212,218,139,33,12,183,134,173,6,15,65,6,252,175,59,224,157,53,185,207,126,161,45,145,215,237,177,193,119,91,166,116,37,224,129,
140,63,10,224,175,1,253,107,236,42,207,254,214,107,219,142,105,66,120,237,41,83,73,118,40,46,218,192,241,111,65,79,126,171,224,86,202,223,211,136,255,245,21,91,202,119,91,84,250,48,25,101,7,72,157,97,
87,121,161,195,180,186,115,15,9,17,134,90,83,23,169,104,129,206,253,118,160,126,121,99,128,202,87,125,29,253,169,205,61,177,104,204,174,126,2,48,91,0,195,174,242,124,31,66,56,193,211,253,225,115,209,83,
92,112,56,194,39,158,107,192,84,52,0,29,177,204,232,239,31,126,241,208,173,99,227,197,159,7,240,13,176,171,60,219,85,110,188,240,77,141,123,17,190,56,97,107,155,162,0,202,7,65,198,3,77,199,168,82,237,
109,233,31,201,142,14,238,21,162,231,103,91,50,241,44,216,85,110,112,149,203,22,10,110,26,249,178,131,137,170,65,217,51,240,52,96,106,115,114,56,93,32,119,4,164,75,161,182,88,169,83,106,20,172,76,200,
178,228,77,227,99,195,255,99,124,204,176,171,60,203,85,150,18,9,97,33,99,219,160,136,132,235,249,200,122,55,99,66,3,202,8,152,178,0,77,254,51,224,79,2,186,20,250,73,53,95,89,53,250,202,236,42,79,185,202,
181,11,27,193,64,64,27,1,146,22,28,39,104,114,20,113,206,98,179,77,112,189,52,70,242,155,160,252,9,64,21,67,237,182,2,163,189,249,124,101,118,149,205,52,131,35,112,149,5,44,192,151,168,86,8,110,33,184,
94,75,73,136,69,45,180,224,41,148,172,10,202,110,30,80,101,152,80,8,55,240,231,243,149,217,85,158,114,149,17,30,95,10,104,3,72,97,1,74,64,27,13,101,12,138,21,19,218,159,62,72,21,225,185,37,88,82,131,234,
96,202,220,187,99,118,149,235,239,118,234,122,123,115,9,1,232,176,3,172,54,16,2,128,111,130,175,79,26,70,151,81,45,231,80,112,43,104,105,73,77,67,170,230,156,6,87,216,85,110,106,29,108,60,79,64,144,52,
64,4,211,92,229,120,221,85,142,116,119,246,104,120,77,185,202,189,137,2,30,125,248,49,188,120,232,25,232,234,40,72,141,65,248,227,32,127,28,164,114,64,77,170,213,69,64,23,161,253,98,224,209,169,82,131,
77,87,170,127,188,241,214,107,112,215,29,215,227,234,171,182,96,115,111,11,180,95,128,64,5,208,21,72,120,136,68,36,170,110,17,103,78,159,66,35,84,53,79,213,92,229,255,11,224,80,248,239,62,118,231,254,
135,14,172,70,192,235,210,85,158,98,213,93,220,124,211,181,216,177,99,11,118,244,111,198,155,239,188,13,247,221,247,22,72,114,1,83,133,86,85,16,41,68,29,11,173,153,12,124,223,101,87,185,57,87,89,7,237,
39,140,66,60,26,129,37,12,50,233,4,182,244,245,226,134,235,174,69,79,119,71,120,196,48,165,220,198,227,49,244,245,109,130,239,123,236,42,55,231,42,7,255,222,243,92,196,227,49,244,116,119,97,251,246,173,
184,246,218,171,49,56,56,0,173,253,89,255,45,187,202,75,114,149,131,176,95,121,229,101,68,163,81,180,183,183,163,191,191,31,207,62,251,108,195,69,204,176,171,124,33,174,178,214,26,217,108,22,191,253,219,
191,141,3,7,14,224,216,177,99,24,31,31,103,87,121,53,92,229,167,159,126,26,227,227,227,115,190,32,236,42,179,171,204,174,50,216,85,158,254,191,179,171,204,174,50,187,202,13,197,174,50,187,202,236,42,79,
123,231,130,93,229,217,47,6,187,202,11,173,56,216,85,6,187,202,77,23,187,202,77,6,204,174,242,180,98,87,153,93,101,118,149,27,6,48,187,202,51,139,93,101,118,149,217,85,158,227,101,103,87,25,96,87,121,
233,27,140,218,244,192,174,242,18,14,110,150,250,214,18,187,202,203,56,184,105,118,153,198,174,242,50,15,110,154,221,96,176,171,188,204,131,155,102,130,102,87,249,2,14,110,22,27,213,236,42,179,171,204,
174,242,74,6,204,174,50,187,202,139,36,197,174,50,187,202,236,42,179,171,204,174,50,187,202,77,4,12,176,171,188,228,128,217,85,102,87,121,117,139,93,229,181,9,153,93,229,213,42,118,149,55,120,177,171,
188,222,3,14,67,102,87,121,5,87,17,92,28,48,7,204,1,115,113,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,7,204,197,1,115,192,92,28,240,218,22,187,202,151,75,192,236,42,175,126,160,171,
229,42,3,23,241,221,236,203,194,85,190,152,136,245,165,233,42,27,227,147,112,124,16,249,141,97,135,80,234,154,6,125,169,185,202,94,232,42,87,97,140,103,140,241,32,165,75,150,244,141,231,249,178,181,85,
212,110,221,218,208,236,111,227,168,93,99,87,249,165,66,37,241,244,120,169,229,85,79,219,249,178,27,31,135,49,85,0,101,114,200,51,193,163,104,110,35,96,141,141,196,254,206,28,181,107,238,42,59,254,117,
241,88,225,67,237,233,201,17,95,225,112,197,181,158,44,85,99,47,141,149,218,95,45,85,19,99,68,40,131,132,104,4,172,103,46,247,214,117,192,23,223,85,174,166,9,174,16,150,215,237,88,110,119,196,169,222,
153,138,142,159,105,79,142,189,88,172,70,190,126,116,228,170,111,130,72,64,235,105,128,245,106,134,124,105,185,202,194,119,211,241,106,58,29,45,118,70,173,82,187,68,69,146,93,217,30,145,165,45,17,171,
120,247,27,162,143,125,48,87,78,255,143,163,217,171,15,66,41,144,148,90,102,50,171,10,88,95,34,174,114,111,232,42,27,157,207,186,213,162,59,153,53,185,161,151,91,162,3,50,19,205,247,218,100,199,45,89,
136,91,84,188,163,61,49,118,77,38,242,157,47,76,148,90,255,247,233,137,189,199,92,95,214,80,142,85,1,172,47,9,87,121,138,167,12,27,53,25,11,202,21,70,149,85,197,31,63,253,122,198,121,53,215,18,201,246,
198,101,118,155,52,5,135,116,9,208,222,51,190,194,103,38,203,233,175,191,58,124,96,84,229,114,106,169,1,95,22,174,114,240,255,98,90,200,83,182,178,4,140,5,149,215,69,53,126,242,96,146,14,101,187,226,39,
111,114,40,183,131,116,9,100,220,49,99,212,223,84,125,231,215,159,124,233,166,1,204,213,123,247,2,3,190,144,41,98,65,87,121,107,199,192,175,206,227,42,239,137,132,174,178,50,149,192,43,90,150,171,60,117,
6,100,160,195,167,251,5,200,104,128,130,255,94,138,192,13,181,91,172,132,149,216,251,214,194,217,228,145,241,115,19,159,223,221,227,95,27,19,229,251,8,104,3,233,159,136,89,165,254,235,118,61,251,158,231,
143,94,95,89,47,83,196,58,113,149,245,180,111,169,238,43,55,8,96,34,124,40,29,174,13,58,246,107,128,55,238,85,252,244,175,244,182,249,42,19,29,253,41,130,215,71,70,73,192,188,0,24,118,149,103,154,202,
66,56,1,80,77,114,106,94,158,235,69,25,31,9,191,121,109,199,112,250,231,143,29,121,238,134,145,49,252,138,49,244,45,176,171,60,143,171,108,204,180,11,97,237,69,66,72,173,213,254,91,83,38,80,254,175,27,
4,4,47,209,222,98,191,63,63,126,162,95,136,77,255,190,51,35,70,1,245,97,176,171,60,221,85,206,149,109,228,42,6,37,223,192,213,83,77,161,235,47,74,237,197,154,120,21,164,171,117,63,41,84,191,200,150,120,
67,97,252,212,111,79,142,201,143,237,232,239,141,131,93,229,154,171,44,65,82,34,33,36,50,142,3,68,45,84,61,131,108,245,58,228,140,129,50,0,68,112,209,51,147,18,152,252,118,192,171,169,114,40,222,122,13,
150,146,62,64,240,255,242,248,137,83,31,217,209,191,141,93,229,169,41,66,66,67,64,8,27,118,36,142,72,52,134,72,100,16,155,164,133,170,159,198,88,244,118,24,33,96,70,255,25,80,121,192,159,132,209,37,144,
158,26,197,13,23,248,125,4,253,39,236,42,79,115,149,131,159,0,8,137,74,85,194,45,56,128,176,32,164,133,104,36,130,116,225,48,202,110,12,110,233,40,140,46,79,145,150,161,16,78,80,211,145,21,118,149,231,
118,149,141,161,160,7,155,150,208,70,64,145,68,169,34,16,180,64,86,128,95,132,242,138,176,200,11,125,251,240,243,214,145,171,169,77,12,187,202,245,229,175,174,55,134,38,72,4,173,36,5,72,19,200,178,194,
63,18,174,171,85,25,213,226,4,10,110,9,45,153,56,40,100,137,103,154,74,245,105,144,93,229,154,77,87,12,93,229,34,224,79,66,123,121,236,217,217,133,158,246,40,140,155,131,208,5,144,158,132,208,69,68,108,
13,183,146,195,217,83,199,103,132,59,103,177,171,220,232,42,7,70,93,5,31,248,161,251,112,235,205,215,225,158,183,220,134,55,221,126,61,72,151,130,159,6,184,136,57,6,45,153,4,60,191,130,87,94,57,220,16,
238,188,211,42,187,202,53,87,217,104,31,111,121,203,93,72,196,28,164,146,113,244,245,118,226,222,123,238,66,60,102,1,198,13,161,42,31,241,168,131,190,222,110,248,94,53,116,149,23,190,102,177,171,28,254,
123,33,128,206,142,54,196,227,49,116,117,181,99,199,142,109,184,238,186,107,208,222,154,158,19,175,222,182,109,43,0,195,174,114,179,174,178,214,10,175,189,246,42,18,33,94,189,107,231,78,116,119,119,225,
212,169,147,245,23,160,241,243,173,181,171,44,154,186,184,25,88,192,148,171,172,201,65,174,146,24,138,180,111,235,51,228,7,203,177,38,93,101,154,215,85,214,13,58,109,115,152,95,173,135,218,161,67,135,
240,204,51,207,160,173,173,13,131,131,131,120,215,187,222,53,237,5,168,189,40,141,174,178,227,56,56,114,228,200,98,45,26,99,217,108,246,3,90,235,47,134,191,175,185,202,23,28,240,134,115,149,191,243,157,
239,224,3,31,248,0,126,248,135,127,24,67,67,67,236,42,179,171,204,174,114,173,216,85,158,235,127,103,87,153,93,101,118,149,27,138,93,101,118,149,217,85,158,246,206,5,187,202,179,95,12,118,149,23,90,113,
176,171,12,118,149,155,46,118,149,155,61,236,97,87,185,177,216,85,102,87,153,93,229,134,1,204,174,242,204,98,87,153,93,101,118,149,231,120,217,217,85,6,216,85,94,250,6,163,54,61,176,171,188,132,131,155,
165,190,181,196,174,242,50,14,110,154,93,166,177,171,188,204,131,155,102,55,24,236,42,47,243,224,166,153,160,217,85,190,128,131,155,197,70,53,187,202,236,42,179,171,188,146,1,179,171,204,174,242,34,73,
177,171,204,174,50,187,202,236,42,179,171,204,174,114,19,1,3,236,42,47,57,96,118,149,217,85,94,221,98,87,121,109,66,102,87,121,181,138,93,229,13,94,236,42,175,247,128,195,144,217,85,94,193,85,4,23,7,204,
1,115,192,92,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,1,115,113,192,28,48,23,7,188,182,197,174,242,229,18,48,187,202,171,31,40,187,202,43,29,236,170,187,202,193,159,209,141,97,
55,222,100,120,201,4,124,209,92,101,24,5,109,148,108,109,213,198,243,52,73,105,106,119,242,172,85,208,151,188,171,76,18,190,81,218,35,203,246,97,180,18,241,184,38,219,214,13,228,175,217,144,1,95,116,87,
89,217,147,101,55,54,14,192,37,75,84,2,190,64,84,65,194,3,193,23,201,36,173,182,72,187,106,1,95,108,87,185,35,61,57,226,41,28,174,184,206,83,165,106,244,112,182,212,241,122,185,26,155,128,49,37,18,162,
106,148,170,18,9,87,36,147,216,48,236,239,92,163,246,226,186,202,94,119,196,113,239,76,68,115,103,219,146,185,195,197,74,244,91,199,178,123,30,130,49,5,34,10,140,30,173,133,136,199,189,213,4,172,47,41,
87,217,18,190,155,137,87,51,233,104,169,51,106,149,219,4,85,165,176,43,219,164,85,217,226,216,165,187,174,143,61,243,194,88,177,229,143,79,141,237,120,2,68,121,2,74,20,141,154,6,192,122,125,6,188,174,
92,229,49,183,90,116,243,227,58,63,242,122,75,244,188,204,68,39,187,109,42,199,164,44,198,44,81,186,165,75,102,175,202,196,198,254,33,91,232,252,179,243,147,91,142,40,159,26,1,107,96,137,50,237,98,117,
105,185,202,16,245,85,157,114,201,152,178,114,189,137,51,167,83,246,201,66,75,116,162,61,38,39,122,45,20,108,161,75,80,170,250,98,213,179,254,119,182,208,249,213,129,252,206,33,93,46,185,225,84,209,116,
192,151,184,171,28,126,249,186,171,220,40,210,202,186,76,11,35,160,10,186,234,143,159,126,37,70,71,199,187,18,231,246,68,105,188,79,154,2,140,46,143,27,165,190,52,86,234,248,141,35,35,215,157,34,33,125,
63,59,218,52,195,126,25,184,202,1,167,22,184,202,8,90,107,146,15,131,160,127,133,17,26,82,216,176,210,86,68,198,119,237,119,243,253,249,35,131,135,191,215,223,254,212,241,148,93,189,77,144,219,42,133,
251,35,29,241,129,45,249,84,219,15,14,229,250,10,88,97,130,253,146,113,149,27,255,75,2,213,71,48,9,27,178,254,96,122,208,238,177,252,250,227,127,211,234,60,51,218,25,63,243,30,203,20,187,133,169,74,3,
245,146,86,248,241,177,124,219,179,47,159,220,231,174,212,8,190,68,92,229,224,153,104,106,248,235,4,156,84,56,221,52,206,227,121,32,230,127,237,7,206,159,125,253,202,193,177,228,31,106,99,61,2,160,10,
152,171,165,80,127,220,145,25,185,155,93,101,76,119,149,103,93,16,77,96,42,77,125,190,176,157,130,2,104,228,11,32,232,72,91,74,221,85,205,31,233,26,48,219,126,177,47,83,25,17,194,255,65,192,92,67,132,
255,138,224,185,57,118,149,107,174,114,209,215,240,180,14,123,153,54,184,30,161,171,84,239,177,153,205,129,252,115,53,44,133,108,233,237,243,39,95,250,212,209,92,203,207,236,222,70,17,2,238,7,12,187,202,
51,93,101,19,177,80,245,128,172,123,53,114,198,192,215,181,159,12,212,109,37,20,5,40,247,87,1,86,165,3,219,3,198,3,193,189,54,130,115,255,235,248,73,231,71,118,109,111,159,4,204,15,130,93,229,70,87,217,
9,93,229,56,34,145,97,244,90,17,84,252,20,38,34,215,67,137,224,167,196,104,1,26,249,23,144,46,128,116,9,208,229,186,150,24,172,94,252,189,210,84,255,104,34,23,249,127,91,50,241,86,176,171,28,186,202,36,
1,178,3,87,185,20,129,129,3,105,71,225,68,99,72,21,143,194,53,155,81,109,191,19,122,180,4,170,28,6,233,128,182,132,42,133,4,176,55,69,0,195,28,24,25,30,248,93,75,246,176,171,60,221,85,150,144,210,1,140,
3,99,44,40,207,70,165,106,195,144,128,160,115,48,99,7,161,221,73,72,242,130,145,171,74,32,93,130,209,46,160,67,64,37,116,149,44,75,178,171,92,75,152,194,158,107,20,190,217,97,84,240,6,136,20,14,140,150,
65,167,5,33,64,158,11,183,56,9,175,90,66,38,237,0,186,18,134,91,5,224,79,183,240,216,85,110,116,149,67,83,89,23,0,85,128,241,39,32,245,36,160,38,96,188,49,72,157,3,249,227,16,42,135,136,44,193,45,141,
226,220,233,35,193,212,48,43,220,105,91,113,118,149,3,87,217,159,114,149,117,5,2,21,220,114,227,62,220,118,211,62,220,126,203,213,128,154,132,246,114,16,166,128,136,229,33,147,182,224,85,39,241,234,203,
47,0,240,0,237,55,80,109,179,166,87,118,149,27,93,101,99,60,220,117,231,109,216,179,103,59,246,236,217,142,253,215,238,197,143,252,240,123,131,57,55,28,177,137,168,192,166,222,118,120,94,25,47,190,248,
2,230,195,89,235,193,176,171,60,229,42,195,40,180,102,210,112,44,129,246,214,12,182,109,217,132,219,111,189,25,173,45,201,112,41,230,194,104,15,241,168,131,109,91,250,0,237,135,44,240,130,11,3,118,149,
167,88,95,3,199,113,66,188,186,3,253,253,219,113,213,149,123,49,62,150,133,160,41,181,81,107,31,233,116,42,116,149,115,236,42,207,248,145,157,245,251,198,30,106,167,78,157,68,38,147,65,111,111,47,118,
238,220,129,127,252,199,127,128,16,168,191,0,90,251,245,95,247,246,246,172,153,171,108,45,52,61,212,6,242,116,87,57,90,119,149,35,173,150,99,116,117,81,87,249,155,143,4,174,178,104,202,85,70,195,40,157,
115,84,205,217,118,241,241,199,31,199,185,115,231,208,223,223,143,51,103,206,224,196,137,19,245,61,223,92,14,71,95,95,31,78,156,56,129,82,169,180,96,55,214,208,85,254,105,199,113,126,46,188,205,224,238,
112,227,81,190,160,17,188,17,93,229,51,103,206,224,225,135,31,174,135,203,174,50,187,202,236,42,3,236,42,179,171,204,174,242,92,33,179,171,60,173,216,85,94,70,177,171,12,118,149,193,174,242,5,172,34,216,
85,158,86,236,42,207,55,149,44,52,101,176,171,204,174,50,102,221,113,195,174,242,194,1,179,171,60,247,231,100,87,121,177,81,205,174,50,187,202,236,42,135,191,103,87,153,93,101,118,149,217,85,174,45,235,
217,85,94,198,50,141,93,229,101,30,220,52,187,193,96,87,121,153,7,55,205,4,205,174,242,5,28,220,44,54,170,217,85,102,87,153,93,229,149,12,152,93,101,118,149,23,73,138,93,101,118,149,217,85,102,87,153,
93,101,118,149,155,8,24,96,87,121,201,1,179,171,204,174,242,234,22,187,202,107,19,50,187,202,171,85,236,42,111,240,98,87,121,189,7,28,134,204,174,242,10,174,34,184,56,96,14,152,3,230,226,128,57,96,46,
14,152,3,230,128,185,56,96,14,152,3,230,226,128,57,96,14,152,139,3,230,128,185,56,224,181,45,118,149,47,151,128,217,85,94,253,64,217,85,94,233,96,217,85,94,229,80,217,85,94,153,112,217,85,94,205,81,187,
46,92,101,73,85,0,46,12,185,176,225,1,240,69,50,169,194,187,59,105,53,131,190,212,93,229,167,139,149,232,225,177,82,199,235,101,55,54,14,162,50,17,85,1,84,65,194,147,173,173,126,195,205,136,192,122,102,
127,231,26,181,23,207,85,246,187,109,203,237,142,56,222,93,137,104,254,76,123,50,255,114,161,26,251,246,241,209,93,15,7,178,9,138,36,168,98,130,135,43,221,134,209,188,226,33,95,74,174,242,113,75,248,213,
25,174,178,16,182,187,77,90,149,45,237,78,233,206,76,236,217,23,179,133,150,7,79,79,244,63,1,141,2,193,148,76,240,184,116,163,18,190,254,2,94,119,174,178,23,184,202,153,232,144,149,137,22,186,109,81,142,
90,84,142,73,81,186,185,171,37,119,85,38,254,252,151,135,242,29,159,27,45,244,28,209,198,130,129,15,18,162,186,26,182,242,165,233,42,27,1,237,9,168,178,239,250,19,103,207,38,237,211,165,76,116,178,45,
110,229,186,36,74,150,208,69,24,223,125,165,232,70,62,59,156,239,250,70,182,216,125,14,64,217,104,229,169,241,113,175,217,139,222,101,234,42,135,210,34,5,80,10,25,9,85,80,174,55,113,246,100,20,167,115,
93,201,161,109,81,145,235,18,122,18,164,43,121,223,243,255,101,184,208,243,91,103,199,119,156,48,202,175,0,240,195,139,158,94,137,128,47,49,87,57,104,16,109,194,145,76,198,135,32,11,50,101,59,34,222,191,
199,159,220,82,58,58,244,250,11,91,90,95,29,204,56,234,106,41,84,218,177,221,15,244,165,79,246,229,42,109,31,202,23,211,158,241,61,181,146,115,240,37,230,42,215,26,69,83,125,186,160,6,176,138,200,2,193,
66,117,48,251,66,220,253,238,201,222,196,169,91,45,202,181,11,93,20,80,222,43,21,223,254,169,115,217,45,79,15,12,182,87,86,106,4,95,2,174,114,237,169,254,112,14,198,84,223,160,186,130,107,66,102,199,104,
128,52,162,61,93,215,142,91,239,127,227,233,241,190,127,114,253,216,51,6,194,37,97,174,140,217,133,207,244,119,188,126,247,85,219,15,179,171,60,211,85,166,58,82,173,167,68,176,122,195,60,192,80,16,59,
25,3,18,6,45,142,213,230,103,95,121,203,89,181,249,51,91,91,75,35,17,170,188,3,192,62,75,186,191,218,158,30,29,198,148,252,178,166,1,175,95,87,89,5,13,241,12,130,3,182,224,66,137,240,69,147,65,191,255,
90,223,138,2,129,198,127,19,150,112,183,155,202,11,63,121,228,92,207,39,175,236,147,66,2,247,2,102,191,16,230,79,216,85,158,195,85,30,117,175,70,206,104,248,90,77,253,132,132,50,66,208,223,77,0,190,0,
134,254,60,212,20,61,144,41,239,73,152,151,127,227,232,169,182,159,220,179,85,76,80,208,163,135,93,229,233,174,114,2,145,88,28,81,103,20,49,43,138,138,159,70,46,114,21,148,104,240,61,16,40,53,24,126,9,
228,103,65,186,18,216,202,240,65,186,186,43,98,78,254,118,54,215,253,115,157,25,100,0,188,13,236,42,55,184,202,174,21,184,202,20,129,176,227,112,162,73,36,203,167,225,154,77,168,182,222,0,136,32,96,51,
170,64,133,135,66,91,57,144,24,73,87,131,157,28,188,235,39,70,143,253,247,168,232,253,116,42,21,217,11,118,149,67,87,25,22,164,176,1,237,192,32,2,229,71,80,117,131,176,73,12,65,76,188,0,191,245,99,16,
54,64,163,191,21,218,202,133,80,181,173,17,192,30,12,52,44,33,110,28,30,30,184,55,30,223,254,171,82,210,255,2,187,202,161,171,108,44,24,109,131,140,132,144,49,24,237,4,127,53,225,192,248,26,238,201,159,
135,239,185,72,37,172,192,179,83,165,122,192,48,110,128,169,4,27,28,18,130,126,240,244,233,147,95,90,41,87,185,169,41,34,188,176,73,3,68,104,154,171,156,172,187,202,233,221,157,219,117,125,41,182,176,
171,156,177,10,120,244,225,167,49,112,250,4,140,63,25,202,180,229,224,71,86,149,1,82,225,136,117,1,232,112,13,27,130,8,193,240,109,232,162,45,0,35,97,140,12,143,154,171,208,170,6,88,91,144,202,32,66,30,
74,197,113,156,31,43,162,183,167,45,152,22,234,63,37,211,117,112,192,212,92,229,123,210,233,244,205,8,58,79,125,236,206,253,15,253,195,114,58,79,109,112,87,217,155,230,42,215,16,235,84,84,3,126,14,198,
29,131,208,1,92,157,73,40,120,149,44,94,127,229,89,24,93,12,195,157,242,68,27,133,48,176,171,92,115,149,253,186,171,92,67,172,239,123,199,93,120,223,123,222,134,247,253,192,91,112,213,222,205,225,5,173,
128,152,227,163,167,43,5,175,154,199,225,23,159,13,40,224,57,248,181,122,48,236,42,55,186,202,26,48,10,239,120,199,189,232,104,107,65,50,30,193,142,254,45,120,199,219,222,28,194,213,21,144,174,32,17,21,
216,178,185,11,90,187,56,117,234,248,92,35,183,177,216,85,110,116,149,133,0,186,58,219,16,79,68,209,213,213,142,157,59,183,227,246,219,111,134,160,16,174,54,62,148,95,69,58,25,71,91,75,6,249,137,9,84,
42,165,5,143,125,217,85,214,83,239,85,214,86,25,169,100,2,155,122,123,176,107,231,14,164,146,137,250,20,163,85,176,178,1,52,122,123,187,225,56,54,142,28,121,125,213,93,101,177,208,244,80,27,200,211,93,
101,187,238,42,147,109,57,181,195,242,133,92,229,231,14,5,174,50,53,229,42,155,134,81,58,231,168,154,183,237,226,119,191,251,93,228,243,121,116,117,117,161,82,169,224,222,123,239,69,13,173,110,156,98,
132,32,244,245,5,230,72,105,254,230,33,193,50,43,112,149,191,3,192,13,71,241,221,75,153,38,46,41,87,249,196,137,19,248,195,63,252,67,252,224,15,254,32,238,191,255,126,118,149,87,203,85,174,5,203,174,50,
187,202,236,42,135,197,174,50,187,202,236,42,207,88,3,179,171,60,179,216,85,102,87,153,93,229,153,67,153,93,229,134,98,87,121,190,169,100,161,41,131,93,101,118,149,49,235,142,27,118,149,23,14,152,93,229,
185,63,39,187,202,139,141,106,118,149,217,85,102,87,57,252,61,187,202,236,42,179,171,204,174,114,109,89,207,174,242,50,150,105,236,42,47,243,224,166,217,13,6,187,202,203,60,184,105,38,104,118,149,47,224,
224,102,177,81,205,174,50,187,202,236,42,175,100,192,236,42,179,171,188,72,82,236,42,179,171,204,174,50,187,202,236,42,179,171,220,68,192,0,187,202,75,14,152,93,101,118,149,87,183,216,85,94,155,144,217,
85,94,173,98,87,121,131,23,187,202,235,61,224,48,100,118,149,87,112,21,193,197,1,115,192,28,48,23,7,204,1,115,113,192,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,197,1,175,109,177,
171,124,185,4,204,174,242,234,7,202,174,242,74,7,203,174,242,42,135,122,49,93,229,181,26,213,151,173,171,220,48,170,245,134,12,120,93,187,202,218,40,17,143,171,48,232,218,69,145,93,229,21,112,149,43,0,
170,36,80,133,148,46,140,86,171,73,254,174,120,192,235,217,85,158,172,198,191,125,98,116,231,195,36,80,4,80,34,99,42,70,153,170,112,28,15,241,56,233,82,201,15,127,68,244,186,12,120,125,186,202,174,16,
118,117,155,180,171,91,58,156,210,157,153,216,243,47,101,11,45,127,122,102,124,235,83,0,38,201,178,138,70,169,42,57,145,170,0,16,134,44,86,50,228,203,192,85,46,6,174,178,44,199,44,81,185,169,167,101,226,
202,76,108,242,43,67,249,206,191,26,45,117,30,1,228,36,140,22,112,34,104,8,153,86,106,186,184,28,92,229,115,73,251,76,41,29,45,180,37,236,201,78,97,74,150,52,37,40,223,123,189,80,118,62,63,60,217,253,
205,137,82,235,57,24,83,52,90,87,140,239,185,13,66,248,130,33,179,171,44,36,200,88,240,139,190,231,143,159,61,19,193,185,66,103,50,219,23,19,249,118,97,138,32,93,158,244,92,245,205,129,252,166,223,31,
46,108,58,6,165,10,75,177,149,47,75,87,25,68,48,161,227,73,38,192,82,100,194,178,101,108,251,14,127,114,115,229,248,240,209,163,155,90,78,101,91,34,180,235,255,207,222,125,135,89,150,157,229,161,127,191,
181,247,62,185,82,87,87,234,234,156,38,39,205,104,52,26,105,148,144,16,72,136,96,236,11,182,47,182,9,6,129,177,49,215,1,155,36,99,99,99,204,5,33,192,32,131,192,112,49,57,8,33,144,16,72,163,81,107,52,26,
205,244,4,205,244,228,158,238,233,220,213,21,79,62,123,239,181,214,119,255,88,251,156,58,149,227,233,233,174,250,62,61,245,168,186,187,166,187,234,61,235,172,189,210,222,63,79,113,87,42,93,253,230,61,
189,175,237,42,135,125,223,93,231,76,76,240,13,179,229,205,234,135,183,144,171,220,62,200,104,62,141,59,153,113,39,112,138,82,129,115,149,199,38,94,75,71,79,93,24,201,95,186,37,165,102,186,149,45,41,182,
225,43,165,90,215,15,159,159,62,240,149,114,189,80,33,63,136,245,228,132,93,174,155,232,84,11,94,214,85,238,205,215,190,223,185,202,93,198,25,159,187,42,58,115,243,225,220,128,151,106,122,115,182,61,220,
182,183,127,235,89,152,212,212,100,155,143,47,86,75,172,85,218,86,223,108,91,72,106,123,159,13,247,152,49,226,214,216,192,83,64,122,104,231,254,202,229,175,217,121,174,252,212,227,35,185,103,134,115,126,
227,136,71,241,145,222,204,196,207,230,118,206,252,167,211,211,55,253,237,84,101,231,166,92,236,182,148,171,220,52,145,102,195,110,182,108,247,144,38,130,130,101,15,74,89,116,141,4,5,91,127,243,3,151,
46,7,191,191,43,247,240,100,193,171,189,133,8,55,166,253,218,79,30,220,241,204,229,201,226,219,191,242,122,12,211,174,97,87,217,45,172,89,118,154,34,147,77,250,245,217,110,166,189,95,39,98,248,121,242,
149,223,253,190,211,99,189,31,190,97,176,196,105,85,123,171,34,190,45,237,215,126,237,224,206,23,238,59,89,217,83,191,154,1,95,195,174,114,144,184,202,183,160,104,13,180,109,142,72,76,242,36,110,110,189,
27,168,245,204,99,2,95,137,161,42,127,184,179,155,74,223,255,242,165,209,159,184,105,164,50,157,162,198,123,1,190,117,180,247,213,63,27,188,245,204,119,61,114,226,173,23,175,90,192,215,182,171,156,111,
115,149,123,48,147,186,1,90,181,1,214,68,201,191,235,160,85,158,86,80,83,191,232,156,15,91,223,221,197,207,252,251,177,233,61,63,190,171,175,146,83,132,119,2,252,238,192,215,63,250,246,59,31,252,113,113,
149,35,31,81,45,3,80,6,20,228,16,100,186,80,168,95,64,136,93,8,123,110,6,188,246,209,137,2,42,30,104,226,215,28,189,102,27,32,142,161,184,126,83,125,230,169,31,47,250,35,31,222,209,197,135,1,236,7,236,
183,1,248,130,184,202,42,0,108,26,22,41,176,206,32,140,74,128,151,1,188,41,120,229,87,96,122,190,21,170,47,105,249,117,5,186,252,107,137,37,90,113,4,48,135,96,142,17,120,246,206,169,137,51,239,238,202,
237,254,175,129,135,95,133,184,202,243,93,229,0,228,103,1,174,195,232,20,148,159,5,140,66,116,254,35,224,137,175,65,118,239,93,80,231,126,3,48,211,32,83,77,0,235,122,226,43,199,32,54,228,145,254,230,115,
231,206,252,197,193,253,123,127,19,224,15,66,92,101,229,2,102,5,165,210,128,110,192,162,10,207,203,128,57,13,101,21,82,76,40,141,253,54,42,231,127,31,67,3,30,192,33,96,26,173,23,149,109,3,176,45,95,185,
155,96,126,191,88,44,126,109,79,79,215,189,16,87,57,4,217,122,226,42,87,1,91,129,178,37,144,153,1,199,147,80,102,26,25,85,68,87,166,142,184,118,18,175,190,244,20,160,103,0,83,2,108,13,108,235,206,91,66,
220,46,59,102,38,38,174,252,99,113,149,91,174,242,108,216,176,117,12,15,118,225,141,111,56,138,254,110,5,50,69,64,23,145,13,234,24,234,247,17,213,199,241,194,137,39,18,126,189,61,92,139,118,176,85,92,
229,57,174,178,73,196,175,24,111,186,247,46,124,237,123,222,142,27,111,216,143,111,254,198,175,197,61,119,222,152,136,225,85,100,83,6,187,135,187,97,227,50,206,158,121,25,100,67,247,239,89,51,47,96,0,
226,42,47,116,149,251,119,244,225,166,27,142,32,29,120,232,239,235,197,129,125,187,240,182,7,238,5,37,120,53,235,58,186,11,1,250,122,114,40,206,76,162,94,175,180,117,11,11,151,68,197,85,158,231,42,79,
78,78,0,96,116,117,229,177,107,215,16,14,31,58,136,111,252,192,251,192,172,97,77,8,99,26,96,214,24,30,234,71,202,39,188,242,242,203,115,72,224,69,74,92,229,249,235,107,99,99,99,232,234,234,194,174,145,
17,28,62,124,8,31,249,200,71,90,125,55,216,66,145,107,237,163,163,35,0,44,106,181,234,178,11,102,27,117,149,151,28,166,53,199,190,88,193,85,182,43,186,202,175,205,113,149,193,13,240,26,93,229,149,248,
222,246,223,255,235,191,254,107,84,171,85,100,50,25,124,244,163,31,197,67,15,61,52,239,107,92,107,45,20,242,232,238,238,194,233,211,167,113,203,45,183,44,249,162,53,93,229,3,7,14,60,6,135,165,54,93,229,
250,70,2,38,54,134,200,15,20,41,53,199,85,214,77,87,153,187,15,128,92,192,203,185,202,199,19,87,153,218,92,101,234,176,171,124,236,216,49,60,244,208,67,171,114,149,75,165,18,162,40,90,238,129,205,237,
174,242,91,49,235,42,63,188,177,62,88,92,229,86,137,171,44,174,178,184,202,11,67,22,87,121,78,137,171,188,142,18,87,25,226,42,67,92,229,13,140,34,196,85,158,83,226,42,47,55,195,19,87,89,92,101,113,149,
33,174,242,234,91,174,184,202,226,42,139,171,188,238,46,34,217,205,16,87,121,182,196,85,94,207,4,3,16,87,121,77,11,55,226,42,139,171,220,54,27,19,87,25,128,184,202,235,158,96,136,171,188,206,133,155,213,
4,45,174,242,6,22,110,86,106,213,226,42,139,171,44,174,242,102,6,44,174,178,184,202,43,36,37,174,178,184,202,226,42,139,171,44,174,178,184,202,171,8,24,16,87,121,205,1,139,171,44,174,114,103,75,92,229,
171,19,178,184,202,157,42,113,149,175,243,18,87,249,90,15,56,9,89,92,229,77,28,69,72,73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,73,192,87,183,
196,85,222,46,1,139,171,220,249,64,197,85,222,236,96,197,85,238,112,168,226,42,111,78,184,226,42,119,178,213,138,171,220,193,86,123,141,185,202,51,32,170,163,233,42,43,138,1,232,54,87,185,35,173,121,203,
187,202,133,165,92,101,162,144,141,105,144,31,104,149,203,197,157,106,205,219,194,85,78,5,225,158,157,233,57,174,242,113,0,85,242,253,42,216,54,16,4,52,175,53,99,179,66,222,62,174,178,170,103,253,116,
227,222,97,191,120,83,79,246,185,79,95,46,13,254,225,100,181,255,36,91,85,33,143,60,54,166,65,164,34,85,40,96,51,67,222,22,174,114,62,56,87,239,201,84,250,114,65,101,192,231,154,175,184,6,173,205,203,
149,122,234,143,198,202,131,15,150,106,61,231,1,84,192,220,224,56,14,153,173,78,66,222,48,251,187,205,92,229,139,213,129,194,212,104,198,43,239,80,92,133,50,245,114,24,241,67,23,102,70,127,109,178,186,
243,36,216,150,217,154,26,172,141,176,10,91,89,92,229,150,171,236,37,174,242,104,120,106,252,213,211,195,61,231,103,122,51,254,126,223,243,186,210,153,234,251,247,238,56,187,171,18,117,255,96,24,7,150,
148,103,160,60,203,214,112,235,31,216,64,87,177,197,93,229,230,99,232,125,247,168,5,21,64,193,71,99,108,226,98,58,122,110,124,168,107,226,64,138,138,5,223,150,21,155,198,169,137,74,239,143,95,156,217,
253,104,24,103,166,87,35,132,111,111,87,57,121,198,60,177,5,145,5,147,133,199,22,80,140,244,224,206,93,213,137,183,238,56,95,122,238,197,225,252,201,222,156,111,118,251,136,15,14,228,39,254,83,198,47,
255,236,107,19,71,63,93,179,217,152,117,108,94,143,81,196,245,225,42,99,97,63,110,192,128,5,148,34,228,7,252,12,55,238,186,237,226,101,255,216,72,238,68,185,59,104,220,170,188,232,112,87,186,244,239,246,
239,120,254,210,243,151,239,254,82,115,101,239,106,6,124,157,184,202,26,76,182,205,88,194,236,139,145,60,17,203,35,5,47,235,121,217,209,59,222,118,225,210,142,79,7,244,41,155,247,106,183,43,197,55,246,
102,166,127,161,47,59,241,246,105,236,12,151,185,161,103,211,3,190,230,93,229,201,248,22,204,24,131,216,106,48,76,210,21,53,1,235,249,239,2,6,136,65,105,229,117,237,58,248,245,167,79,222,248,75,135,250,
42,211,57,191,122,63,145,189,241,166,193,175,252,94,37,44,124,207,83,211,183,159,187,106,1,95,235,174,114,58,53,129,97,63,131,186,238,65,49,117,20,154,52,44,205,2,214,201,90,231,236,176,46,209,217,48,
254,188,87,48,95,252,123,151,102,70,126,121,95,95,49,72,81,253,62,34,251,206,66,106,230,71,238,187,229,145,31,127,244,185,251,167,58,29,240,117,230,42,119,163,208,184,132,6,118,33,234,62,10,235,205,118,
21,4,106,61,173,144,72,129,47,189,10,42,125,28,10,149,125,182,250,196,247,76,121,195,191,62,212,205,123,8,216,171,200,254,131,148,31,61,244,246,59,31,252,19,113,149,91,174,114,14,97,92,6,171,44,200,47,
66,85,94,3,119,127,61,84,143,157,51,140,163,228,73,44,84,250,164,147,25,109,136,64,53,110,168,204,156,188,191,39,59,244,11,217,32,252,57,0,59,137,248,95,195,61,86,70,92,101,66,10,228,55,128,184,6,75,153,
4,165,78,33,190,244,59,224,233,183,32,123,224,134,217,54,115,233,20,80,250,43,231,125,54,249,75,27,145,143,250,215,93,188,112,241,47,14,238,235,254,109,2,255,115,108,208,85,94,213,185,136,22,61,9,164,
49,199,85,206,181,92,229,244,208,192,176,69,188,42,87,121,36,95,193,195,15,125,25,207,62,249,4,108,56,1,50,83,80,122,26,164,167,157,34,219,148,106,109,21,176,85,88,93,5,76,29,48,53,112,92,113,159,219,
90,139,249,133,169,128,227,162,227,126,227,105,112,228,184,95,165,39,225,233,73,164,120,26,209,248,199,49,249,212,131,110,157,255,226,73,80,233,207,65,102,6,208,165,22,96,205,28,1,28,23,60,76,126,108,
166,88,255,3,0,79,38,99,188,239,125,251,157,15,222,211,137,22,220,236,123,21,41,175,249,204,174,197,92,229,163,107,115,149,191,130,43,23,29,160,10,93,6,113,29,100,27,208,186,14,178,49,200,99,24,29,37,
99,91,110,117,13,104,77,165,29,179,100,109,114,145,226,216,245,199,70,3,42,134,66,0,232,8,22,85,40,149,65,90,165,193,41,139,241,201,143,227,204,103,62,143,253,163,30,216,198,206,20,109,241,191,141,164,
91,210,96,182,153,233,137,115,223,222,83,216,255,17,207,195,71,49,235,42,127,211,166,183,224,235,207,85,110,182,236,50,96,138,32,61,3,232,41,100,189,10,6,122,44,194,226,87,241,210,243,199,65,122,10,208,
69,231,219,91,23,50,181,134,117,12,165,240,45,83,211,147,226,42,207,119,149,97,35,16,135,120,227,27,110,198,237,183,30,196,205,55,236,2,233,18,96,138,200,250,117,236,26,76,193,134,19,56,127,230,69,215,
181,216,106,210,114,155,58,110,203,187,27,168,85,43,223,129,13,186,202,254,138,23,55,109,20,5,254,98,174,114,181,162,119,229,114,93,126,96,184,49,215,85,110,77,34,102,93,229,231,190,186,156,171,220,72,
90,233,92,60,122,254,211,0,23,199,252,220,118,154,51,150,221,127,255,77,223,252,141,232,233,238,65,144,78,35,149,206,128,88,227,185,23,94,130,181,64,87,22,232,41,120,152,158,186,132,129,29,41,100,82,129,
155,68,59,215,30,237,152,96,226,42,255,94,119,119,247,37,184,7,228,55,93,229,103,54,165,139,184,222,92,101,128,241,214,183,222,143,222,158,46,20,186,114,24,30,220,137,35,135,246,227,238,187,111,5,235,
26,88,215,64,182,129,145,193,110,4,190,193,43,47,61,15,99,67,192,234,57,96,118,91,137,171,60,127,125,13,96,248,190,135,222,158,110,140,238,222,133,27,142,30,65,62,155,73,214,65,34,16,105,16,25,236,26,
217,9,102,141,90,165,178,228,139,10,108,220,85,94,178,5,123,61,61,138,60,127,69,87,153,87,116,149,207,207,113,149,93,171,93,155,171,188,92,55,49,255,243,99,199,142,97,108,108,12,221,221,221,216,187,103,
15,6,6,118,226,127,254,207,95,129,53,49,92,247,237,214,38,10,249,28,186,187,242,56,125,250,213,54,29,124,97,53,93,101,0,143,37,191,213,116,149,55,212,7,95,215,174,242,39,63,249,73,76,78,78,226,19,159,
248,4,142,29,59,214,246,194,216,121,174,242,14,148,74,69,113,149,151,250,154,165,186,18,107,45,142,29,59,134,99,199,142,45,251,194,136,171,188,200,175,197,85,22,87,121,149,163,8,113,149,231,148,184,202,
235,40,113,149,33,174,50,196,85,222,192,40,66,92,229,57,37,174,242,82,93,201,114,93,134,184,202,226,42,99,193,137,27,113,149,151,15,88,92,229,197,255,78,113,149,87,106,213,226,42,139,171,44,174,114,242,
107,113,149,197,85,22,87,89,92,229,230,176,94,92,229,117,12,211,196,85,94,231,194,205,106,39,24,226,42,175,115,225,102,53,65,139,171,188,129,133,155,149,90,181,184,202,226,42,139,171,188,153,1,139,171,
44,174,242,10,73,137,171,44,174,178,184,202,226,42,139,171,44,174,242,42,2,6,196,85,94,115,192,226,42,139,171,220,217,18,87,249,234,132,44,174,114,167,74,92,229,235,188,196,85,190,214,3,78,66,22,87,121,
19,71,17,82,18,176,4,44,1,75,73,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,82,18,240,213,45,113,149,183,75,192,226,42,119,62,80,113,149,55,59,88,113,149,59,28,170,
184,202,155,19,174,184,202,157,108,181,226,42,119,176,213,138,171,44,174,178,184,202,155,237,42,79,84,250,126,235,252,244,158,199,33,174,114,103,92,229,17,191,120,83,111,182,252,233,203,165,129,63,154,
172,246,191,202,86,149,196,85,222,20,87,185,218,151,11,202,3,62,215,124,226,58,76,172,95,41,55,50,127,116,165,180,243,115,165,122,207,5,0,21,88,219,104,221,222,80,171,233,149,186,11,113,149,23,186,202,
181,129,194,212,174,140,87,217,161,184,10,178,141,114,20,218,47,92,152,25,249,181,201,234,206,83,96,46,183,238,33,209,113,212,188,135,100,169,144,197,85,158,239,42,87,70,195,83,87,94,125,109,184,231,66,
169,55,19,236,245,85,165,43,157,169,189,111,111,255,197,209,114,212,251,195,81,236,3,74,185,86,71,202,182,117,23,226,42,47,239,42,39,110,168,10,64,240,17,142,141,95,78,197,47,77,12,21,38,247,165,169,148,
87,92,86,108,194,51,227,165,190,255,114,169,56,242,104,172,131,201,246,155,36,245,228,132,93,44,228,109,238,42,219,54,87,217,180,148,90,165,2,164,7,119,14,87,39,123,119,156,47,191,120,106,48,119,182,171,
224,99,216,247,120,223,64,215,196,143,166,189,218,47,158,157,58,248,153,48,246,53,91,54,54,108,216,141,12,221,182,190,171,220,228,215,216,2,100,19,111,137,145,235,79,165,16,222,118,244,242,152,247,244,
80,142,235,221,41,125,200,247,244,254,158,92,249,95,141,218,83,99,167,166,110,254,34,56,142,40,8,84,178,238,177,46,130,125,235,186,202,112,23,205,89,192,154,97,89,67,89,192,16,224,41,130,151,73,169,236,
174,91,238,188,116,169,235,184,194,227,232,14,26,135,148,106,28,238,47,140,255,244,165,82,249,253,117,155,15,89,199,145,215,211,163,76,177,104,58,221,130,175,59,87,153,154,160,31,108,75,165,117,47,174,
51,151,137,61,168,148,82,249,93,7,238,185,120,209,251,188,226,135,202,133,160,126,155,79,141,35,183,15,63,250,177,177,242,240,15,156,158,184,241,53,176,213,87,163,5,95,63,174,178,233,193,76,112,4,154,
60,176,231,39,47,174,105,33,174,115,22,141,192,80,41,168,252,174,131,111,31,59,63,253,135,169,124,81,249,170,122,171,34,243,150,161,252,217,31,246,169,254,147,47,95,186,117,188,211,51,185,235,199,85,78,
229,17,164,187,208,229,93,70,136,93,104,116,31,6,121,177,27,14,38,253,126,123,223,222,26,119,120,228,123,166,250,181,19,197,252,31,142,246,78,13,250,192,136,167,204,183,236,200,94,126,40,159,218,247,199,
101,80,199,2,190,254,92,229,168,4,86,121,144,95,130,87,61,11,116,191,27,232,242,221,133,174,9,88,147,231,222,73,80,224,144,64,23,63,133,32,122,116,32,138,227,219,74,233,222,223,218,145,41,253,123,130,
221,225,41,251,47,111,25,121,236,75,143,78,189,233,76,199,2,190,254,92,229,58,44,37,218,44,103,161,47,255,9,108,241,77,72,239,217,55,59,118,102,2,41,5,84,61,208,229,63,4,226,179,0,135,20,160,248,166,241,
9,254,171,238,93,254,31,5,20,254,223,10,230,222,116,96,254,57,196,85,110,119,149,39,160,236,148,115,149,205,4,2,158,65,52,249,183,152,121,254,120,114,65,117,147,14,148,125,208,165,143,129,244,69,168,230,
187,199,54,114,25,62,249,223,38,103,232,79,1,155,60,70,209,254,243,183,221,241,249,123,58,17,176,242,122,122,20,5,129,34,229,5,180,180,171,124,96,109,174,242,35,56,255,218,75,64,60,9,196,147,32,83,4,217,
18,140,46,1,198,89,159,70,87,147,174,34,106,211,106,221,223,15,142,161,48,75,253,146,173,67,161,105,36,87,160,184,12,232,25,23,180,158,70,26,51,40,4,101,232,233,191,195,133,71,255,214,61,75,104,42,0,93,
252,37,64,79,131,116,209,225,213,166,10,178,13,192,198,233,234,244,203,223,18,107,245,43,0,215,225,8,246,31,239,72,11,190,190,92,229,134,107,217,166,150,252,157,69,192,76,3,102,10,25,175,132,254,66,136,
104,226,227,120,245,211,63,3,26,251,41,40,61,9,138,157,175,12,227,94,92,182,33,192,49,60,213,120,255,244,212,244,121,128,191,152,76,17,197,85,118,174,114,156,80,194,73,208,186,228,90,168,158,70,214,171,
96,168,143,97,43,95,198,197,51,207,3,241,148,131,173,77,17,100,28,67,236,252,59,13,176,217,217,168,77,254,35,128,196,85,158,239,42,51,20,222,254,182,55,99,255,129,67,184,50,54,137,43,19,51,56,254,228,
179,176,236,161,144,33,116,229,98,76,79,190,134,157,221,26,153,148,151,204,48,147,23,7,198,189,35,152,97,217,252,253,98,177,248,123,61,61,93,226,42,55,93,101,192,226,173,111,185,15,7,15,236,67,202,83,
56,122,244,0,190,246,61,15,96,71,79,218,189,99,76,5,131,59,210,240,169,138,147,47,63,11,163,43,238,133,182,17,128,184,21,110,242,174,201,78,77,77,252,3,113,149,231,28,73,115,143,44,15,124,31,189,125,221,
216,51,58,130,155,110,56,140,163,71,246,129,108,8,15,33,20,66,236,26,236,3,76,13,245,202,52,192,161,123,184,172,53,11,0,107,113,149,23,249,188,86,171,34,151,203,98,112,96,39,14,28,216,143,91,110,190,9,
167,78,190,226,150,45,147,81,72,62,159,66,161,144,193,169,211,39,97,117,60,47,216,217,239,101,163,174,242,146,45,152,141,33,120,158,34,207,155,227,42,219,166,171,108,186,251,64,238,49,226,203,185,202,
175,36,174,50,218,92,101,116,216,85,62,126,252,56,198,199,199,209,223,223,143,253,251,247,225,47,255,242,19,24,27,187,148,160,129,113,235,162,216,223,215,3,88,131,40,106,96,25,29,188,221,85,6,102,93,229,
13,94,228,154,174,178,195,143,23,184,202,185,189,169,52,115,180,130,171,60,190,10,87,121,113,119,110,163,174,242,239,254,238,239,226,119,127,247,119,209,223,223,143,201,201,201,57,93,72,243,223,204,229,
178,200,229,178,24,27,187,140,125,251,150,38,58,199,198,198,30,56,112,224,192,63,3,240,115,152,117,149,31,222,248,56,120,11,184,202,179,225,162,53,202,104,94,20,149,34,20,10,121,113,149,197,85,22,87,121,
237,93,132,184,202,11,74,92,101,113,149,197,85,158,223,148,197,85,110,43,113,149,151,234,74,150,235,50,196,85,22,87,25,11,78,220,136,171,188,124,192,226,42,47,254,119,138,171,188,82,171,22,87,89,92,101,
113,149,147,95,139,171,44,174,178,184,202,226,42,55,135,245,226,42,175,99,152,38,174,242,58,23,110,86,59,193,16,87,121,157,11,55,171,9,90,92,229,13,44,220,172,212,170,197,85,22,87,89,92,229,205,12,88,
92,101,113,149,87,72,74,92,101,113,149,197,85,22,87,89,92,101,113,149,87,17,48,32,174,242,154,3,22,87,89,92,229,206,150,184,202,87,39,100,113,149,59,85,226,42,95,231,37,174,242,181,30,112,18,178,184,202,
155,56,138,144,146,128,37,96,9,88,74,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,37,96,41,9,88,2,150,146,128,175,110,137,171,188,93,2,22,87,185,243,129,138,171,188,217,193,138,171,220,
225,80,197,85,222,156,112,197,85,238,100,171,21,87,185,131,173,86,92,101,113,149,197,85,190,42,174,178,231,43,0,177,184,202,109,45,120,19,93,229,50,124,191,6,107,235,68,42,82,185,28,37,228,239,166,116,
25,226,42,151,118,62,88,170,247,92,4,80,90,196,85,198,114,33,139,171,188,172,171,92,3,217,122,57,10,249,11,231,167,71,254,215,84,173,255,85,88,91,1,184,206,150,235,171,9,89,92,101,118,102,156,151,247,
23,113,149,103,246,250,202,239,74,103,170,239,219,187,243,210,104,249,82,207,191,139,99,255,50,152,137,148,5,82,105,40,0,109,221,5,95,205,46,226,58,117,149,125,16,2,231,42,71,47,77,13,117,77,237,77,83,
41,167,184,170,172,110,156,29,47,245,253,183,203,165,225,175,196,218,159,2,115,149,173,109,44,39,132,139,171,188,168,171,108,145,26,236,31,174,79,190,105,199,249,210,75,103,6,243,231,10,121,95,13,250,
1,246,14,116,79,255,72,224,133,191,122,110,122,239,223,198,38,165,9,48,240,60,227,245,244,112,243,198,157,171,49,138,184,206,92,101,211,10,26,228,250,120,165,44,50,253,65,10,93,183,28,186,116,89,189,56,
152,83,97,79,154,247,121,62,239,235,203,87,126,192,216,11,147,103,166,14,62,12,64,147,231,27,102,203,173,254,104,141,33,111,31,87,153,109,171,199,102,203,240,20,195,75,167,84,118,215,77,55,142,93,202,
191,168,212,137,115,221,190,217,227,249,250,192,64,215,196,79,92,46,237,250,191,66,157,110,56,249,213,210,122,109,229,45,238,42,199,96,219,30,116,115,84,18,183,158,4,235,5,74,229,118,237,191,233,210,69,
245,52,178,79,53,186,83,209,65,223,11,15,220,54,242,228,47,159,159,25,253,127,46,151,246,156,4,169,152,117,108,58,221,130,175,75,87,217,122,26,100,53,24,78,107,68,171,107,113,110,156,101,15,94,160,40,
63,178,239,142,43,23,202,199,82,84,163,188,87,61,228,251,245,123,71,187,95,251,23,196,230,63,95,42,239,175,179,142,59,58,147,187,238,92,229,130,119,25,17,118,161,209,125,8,214,139,97,173,231,4,48,54,173,
75,170,187,190,186,11,34,165,61,149,26,190,243,173,19,227,209,231,210,249,233,62,5,181,211,247,27,239,27,46,156,61,54,94,25,249,195,200,90,234,84,192,215,161,171,92,6,171,60,224,151,225,85,207,131,187,
223,5,175,43,6,89,175,205,190,107,95,2,117,217,249,89,229,219,174,55,222,51,85,126,241,175,134,243,211,255,88,1,125,25,191,250,189,71,118,62,113,236,185,179,119,158,234,88,192,215,167,171,92,5,108,55,
128,42,244,216,199,193,229,123,144,30,29,133,181,179,6,19,193,221,102,221,122,92,122,157,160,166,62,222,53,93,173,150,122,83,217,79,231,188,202,7,136,236,221,61,153,201,239,134,184,202,109,174,114,60,
9,101,167,64,122,2,158,158,66,138,139,136,39,191,128,226,203,207,181,190,143,230,131,155,154,19,17,158,241,128,115,31,6,133,207,166,10,244,202,7,175,204,100,254,198,50,61,71,176,80,20,127,231,155,15,125,
246,13,157,8,248,250,116,149,173,115,149,17,77,66,153,73,164,105,6,249,160,12,61,115,12,99,79,127,217,93,3,84,0,82,62,200,6,192,216,52,104,252,127,182,188,81,216,122,96,42,207,189,171,30,5,31,99,231,42,
247,251,94,248,163,29,105,193,215,165,171,108,171,32,83,1,76,201,177,190,122,10,25,85,198,142,124,136,120,242,239,112,246,209,47,67,81,0,106,4,160,243,15,65,205,252,94,130,87,151,65,166,6,216,6,124,42,
189,179,56,83,185,192,204,143,36,235,9,15,188,237,142,207,191,123,51,3,190,126,93,101,219,112,221,138,45,187,208,116,17,100,166,144,241,202,216,217,29,195,76,253,62,198,158,124,12,116,250,103,64,181,47,
131,244,20,160,103,92,200,9,236,10,219,216,161,235,103,255,30,51,125,2,224,42,128,2,17,127,243,90,93,101,181,226,197,77,27,69,174,179,154,239,42,55,42,122,87,206,239,242,131,217,181,133,196,85,6,45,112,
149,95,56,177,156,171,92,111,13,161,218,241,210,89,202,151,23,121,158,26,39,150,178,73,62,71,171,165,51,235,214,72,196,117,29,21,64,23,97,163,73,20,210,53,116,165,42,40,159,254,31,136,170,23,65,241,100,
27,96,221,12,55,4,217,24,108,234,223,84,42,150,78,1,124,9,96,31,179,174,242,230,140,34,146,89,155,199,12,159,104,214,85,214,148,66,177,158,31,75,247,239,219,203,164,221,136,97,149,174,178,90,210,85,110,
210,191,139,175,169,172,206,85,118,173,219,26,11,16,225,27,190,225,125,232,223,57,128,106,165,129,231,95,60,137,103,79,188,2,80,128,157,189,132,202,204,20,78,191,60,134,163,71,15,66,145,77,102,125,113,
75,6,119,190,178,201,206,76,95,250,123,61,221,123,254,92,41,245,35,152,117,149,55,12,87,95,215,174,50,96,241,222,175,125,55,6,119,238,64,62,155,197,129,253,123,240,117,239,125,7,216,84,29,63,204,85,12,
237,76,195,198,211,168,87,174,184,81,136,187,184,129,56,106,134,11,192,194,247,189,183,139,171,60,239,115,223,247,48,58,186,203,217,202,131,253,56,124,104,63,222,250,230,55,161,171,144,6,217,6,96,234,
200,103,21,10,57,15,175,189,250,60,88,215,65,214,157,91,105,15,23,0,180,142,197,85,94,172,235,24,31,31,71,46,151,195,174,145,97,28,58,116,16,189,189,221,168,150,103,96,76,232,46,164,54,194,142,190,28,
172,141,16,70,149,100,36,98,22,3,172,55,228,42,47,247,136,83,231,42,187,5,217,5,174,114,118,32,149,158,189,176,45,229,42,79,174,194,85,94,216,5,44,22,238,98,95,179,84,87,98,173,197,51,207,60,131,124,62,
143,157,59,119,130,8,120,199,59,222,209,220,239,111,125,228,50,105,228,50,41,140,93,186,180,164,12,14,56,87,25,192,95,37,127,208,116,149,55,97,170,220,116,149,137,18,87,57,51,199,85,182,118,121,87,249,
228,11,139,187,202,176,140,38,85,183,90,87,121,181,188,67,243,235,206,156,57,131,15,125,232,67,243,94,140,182,11,97,210,221,20,10,57,92,185,114,5,214,238,90,238,239,190,43,155,205,86,235,245,250,75,0,
110,132,184,202,203,142,141,230,140,167,243,249,28,148,34,113,149,197,85,22,87,121,245,93,132,184,202,139,190,24,226,42,47,55,226,16,87,25,226,42,175,186,196,85,94,101,192,226,42,207,41,113,149,197,85,
22,87,185,173,1,139,171,60,191,196,85,22,87,89,92,229,69,94,118,113,149,1,113,149,215,62,193,104,118,15,226,42,175,97,225,102,173,91,75,226,42,175,99,225,102,181,195,52,113,149,215,185,112,179,218,9,134,
184,202,235,92,184,89,77,208,226,42,111,96,225,102,165,86,45,174,178,184,202,226,42,111,102,192,226,42,139,171,188,66,82,226,42,139,171,44,174,178,184,202,226,42,139,171,188,138,128,1,113,149,215,28,176,
184,202,226,42,119,182,196,85,190,58,33,139,171,220,169,18,87,249,58,47,113,149,175,245,128,147,144,197,85,222,196,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,
4,44,1,75,73,192,18,176,148,4,124,117,75,92,229,237,18,176,184,202,157,15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,139,171,44,174,178,184,202,
16,87,121,185,18,87,185,233,42,51,151,217,218,6,128,80,92,229,246,127,126,131,174,114,24,242,177,11,211,35,31,157,170,238,56,5,230,178,184,202,115,87,35,176,62,87,249,98,169,55,51,189,215,87,126,87,38,
83,253,250,61,59,47,141,150,227,158,127,27,235,192,131,181,226,42,207,13,184,121,69,218,128,171,140,170,178,113,120,246,74,169,239,103,198,74,67,95,137,181,63,9,182,181,121,45,89,92,229,117,184,202,103,
7,243,231,243,249,192,27,244,131,242,222,129,238,233,31,73,57,87,249,179,113,236,49,17,12,101,50,12,165,184,13,175,238,248,40,98,11,184,202,6,153,254,84,10,93,183,28,188,116,89,189,60,152,87,186,39,69,
187,189,0,123,123,242,181,31,136,205,165,153,115,211,123,31,6,179,102,173,45,121,158,77,240,106,172,117,100,177,141,92,101,3,235,30,116,3,182,22,30,49,84,42,80,217,93,55,30,189,124,57,127,146,232,197,
139,221,129,221,229,249,188,127,176,103,234,199,198,42,35,223,17,69,126,76,158,103,216,90,205,81,104,215,51,108,219,94,174,114,50,174,38,102,64,49,216,2,158,79,42,63,178,247,200,229,75,120,137,51,47,156,
233,73,217,61,190,167,247,221,50,252,204,47,156,155,222,253,31,38,202,131,175,144,82,17,130,192,36,120,245,154,102,122,219,203,85,134,105,217,31,108,53,172,2,200,18,60,159,40,55,188,239,232,248,197,234,
87,125,10,189,174,32,28,13,130,240,13,187,123,207,125,191,101,250,111,83,149,129,250,122,241,234,237,225,42,171,24,150,149,155,17,178,158,157,188,176,133,133,6,89,15,94,224,169,244,208,173,119,76,142,
155,167,178,94,165,75,81,189,55,229,151,223,51,92,56,255,72,177,190,243,15,13,43,197,235,176,149,183,143,171,92,80,32,219,124,228,216,44,94,221,156,208,48,44,84,214,83,182,247,182,27,174,204,92,252,242,
104,87,249,93,138,84,119,62,93,249,167,163,221,47,127,233,236,212,209,151,40,157,166,101,110,178,92,127,192,215,183,171,92,131,190,242,23,224,242,61,8,118,13,131,172,130,97,157,32,87,201,112,48,25,161,
16,0,175,43,93,104,84,223,210,91,9,175,124,185,59,40,190,213,83,230,246,129,252,197,239,56,59,117,244,199,231,53,186,77,120,126,112,179,191,187,46,93,229,105,144,158,76,92,229,18,226,169,47,162,252,234,
75,240,188,12,60,149,118,15,110,162,230,211,177,188,214,147,188,149,2,50,59,134,239,185,82,30,121,218,88,255,85,128,145,242,26,255,232,142,145,47,220,181,216,113,129,141,6,124,125,187,202,9,96,157,198,
12,242,65,21,102,230,43,24,63,241,20,124,149,114,33,123,105,120,173,199,143,185,49,55,79,41,224,244,127,38,170,63,121,91,177,222,245,231,196,28,18,219,190,92,80,250,183,235,94,79,88,182,245,94,183,174,
114,25,208,69,71,254,234,25,164,189,50,250,242,33,226,233,135,113,241,233,103,147,11,109,243,193,166,62,16,250,192,249,103,64,227,63,15,216,6,82,52,117,87,185,210,184,160,173,58,14,48,19,153,183,220,183,
251,175,190,102,51,3,190,206,93,229,90,210,125,148,0,83,4,197,211,200,168,10,250,187,34,152,233,191,198,212,171,65,235,1,122,152,246,65,231,127,3,84,125,208,97,215,166,2,50,213,110,47,124,229,221,90,123,
159,34,216,26,192,121,223,211,31,184,231,134,199,197,85,118,174,114,3,176,141,164,235,40,3,182,8,142,39,145,15,106,200,251,37,148,95,249,215,136,167,2,208,233,71,160,198,127,30,20,79,130,244,12,72,151,
102,125,101,61,245,238,114,165,246,42,195,94,38,102,31,176,247,229,50,149,195,155,214,69,180,46,110,12,31,152,117,149,45,165,80,108,228,199,210,253,251,70,153,116,178,120,179,58,87,153,150,116,149,109,
155,78,187,58,204,111,161,171,236,90,182,53,238,5,247,200,192,234,58,122,187,83,64,92,134,141,103,0,61,141,254,110,3,165,39,112,246,216,63,4,151,62,5,196,19,45,29,28,166,148,116,51,13,128,195,76,117,230,
228,251,141,225,191,72,190,159,91,137,248,246,205,8,248,186,119,149,21,17,238,191,255,62,252,163,111,255,86,124,211,55,190,7,223,244,13,95,3,152,42,148,173,64,217,18,6,250,0,219,184,140,70,249,60,40,158,
13,151,76,37,185,88,54,64,28,35,240,236,91,226,72,127,30,224,40,209,193,197,85,86,138,112,248,240,65,28,57,124,16,221,93,57,244,245,116,227,182,219,110,196,59,30,184,199,181,78,83,65,62,29,35,159,142,
113,230,212,115,224,120,198,117,35,186,9,88,187,112,153,13,116,28,14,95,190,244,26,1,44,174,114,251,215,236,218,53,2,128,81,40,228,177,107,215,48,110,186,241,8,222,251,181,239,4,235,90,18,114,21,125,221,
62,108,92,66,212,152,73,90,109,2,88,115,19,176,182,0,108,110,104,104,224,109,16,87,121,238,159,191,242,202,43,72,165,82,232,235,235,195,222,61,187,113,227,13,71,241,169,79,125,42,25,105,68,0,71,200,101,
8,217,12,97,236,210,89,128,219,131,157,123,177,221,136,171,172,86,108,14,110,136,150,184,202,254,28,87,153,177,188,171,124,233,194,226,174,178,34,110,237,62,172,214,85,110,255,124,185,167,175,54,235,194,
133,11,120,230,153,103,90,40,234,135,63,252,97,124,241,216,23,90,67,64,171,35,16,44,242,249,20,138,197,41,88,29,47,8,182,173,219,186,43,155,205,86,1,188,212,214,77,172,123,45,98,73,87,217,174,201,85,142,
23,184,202,232,160,171,188,88,107,126,228,145,71,240,240,195,15,47,216,36,109,13,235,96,145,207,102,160,136,17,69,141,228,185,238,139,94,11,218,93,229,27,209,114,149,127,136,215,63,138,216,178,174,50,
207,249,200,100,210,200,100,210,136,162,112,217,37,6,113,149,215,81,226,42,67,92,101,136,171,188,129,81,132,184,202,115,74,92,229,165,186,146,229,186,12,113,149,197,85,198,130,19,55,226,42,47,31,176,184,
202,139,255,157,226,42,175,212,170,197,85,22,87,89,92,229,228,215,226,42,139,171,44,174,178,184,202,205,97,189,184,202,235,24,166,137,171,188,206,133,155,213,78,48,196,85,94,231,194,205,106,130,22,87,
121,3,11,55,43,181,106,113,149,197,85,22,87,121,51,3,22,87,89,92,229,21,146,18,87,89,92,101,113,149,197,85,22,87,89,92,229,85,4,12,136,171,188,230,128,197,85,22,87,185,179,37,174,242,213,9,89,92,229,78,
149,184,202,215,121,109,198,147,255,44,128,85,187,202,205,46,102,35,45,231,122,170,5,93,132,212,230,150,48,15,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,
1,75,192,82,18,240,181,95,11,246,228,214,129,69,117,170,214,228,42,163,67,162,225,106,106,185,221,229,13,111,122,22,238,127,96,83,190,201,141,186,202,243,254,219,171,230,42,175,180,121,43,174,242,213,
238,34,174,102,176,226,42,119,56,84,113,149,55,39,220,107,218,85,238,116,139,222,246,174,114,210,162,59,214,154,197,85,78,36,218,78,181,102,113,149,219,92,229,78,180,102,113,149,23,113,149,43,143,124,
113,211,198,208,226,42,47,225,42,111,86,151,33,174,242,10,174,242,74,33,87,30,249,226,178,83,101,113,149,87,225,42,47,23,242,74,1,139,171,188,10,87,185,242,200,23,213,122,187,139,237,234,42,143,165,162,
151,38,157,171,92,206,41,84,156,171,92,236,253,239,99,229,225,166,171,92,157,239,42,47,118,225,235,68,11,222,10,174,242,80,125,242,77,125,237,174,178,23,148,247,14,244,20,255,125,224,199,31,61,63,189,
231,239,98,237,243,34,45,121,205,163,11,113,149,47,171,151,7,242,74,247,166,176,91,5,216,219,155,175,125,48,54,99,197,243,51,123,190,4,11,75,100,13,60,223,170,66,97,93,120,245,182,115,149,221,131,250,
231,184,202,55,92,185,156,63,73,120,233,114,79,10,195,94,192,251,7,187,167,254,227,88,101,248,187,226,216,143,161,148,134,82,150,148,195,171,147,86,108,59,17,240,22,112,149,147,187,247,57,6,200,5,175,
124,80,110,120,239,225,177,203,56,9,188,120,174,59,224,81,223,183,123,111,25,121,225,231,206,77,238,250,177,201,106,255,203,228,121,150,173,213,28,199,140,14,202,224,91,192,85,214,73,15,109,220,131,152,
154,182,160,15,202,141,236,61,116,229,66,245,69,175,96,198,186,252,120,56,8,226,59,70,123,47,124,80,91,245,115,197,90,111,216,142,87,87,30,249,226,170,167,211,91,223,85,238,114,174,178,97,5,101,9,150,
103,241,106,203,26,176,174,127,247,124,165,50,195,55,223,56,49,110,95,76,23,26,133,52,69,221,169,84,249,157,67,93,151,30,47,55,122,254,216,90,10,215,131,87,111,27,87,217,47,40,24,16,144,124,95,205,177,
181,11,218,128,172,134,74,121,74,245,222,116,240,202,244,196,179,163,221,213,187,125,85,239,234,206,86,254,225,96,215,197,71,47,23,71,75,164,214,142,87,111,19,87,249,19,224,242,221,8,118,13,57,188,218,
18,44,230,134,204,108,1,101,225,229,131,76,88,189,189,175,20,22,79,244,165,42,183,123,10,55,15,119,157,255,246,203,165,221,63,5,102,111,173,120,245,54,113,149,139,136,167,191,132,242,171,39,225,169,20,
148,74,67,81,202,93,7,218,158,221,198,204,32,197,200,12,15,31,158,142,110,191,28,155,244,121,134,66,218,15,191,245,134,129,39,239,92,100,245,112,195,1,111,1,87,121,26,105,42,33,231,87,97,74,143,99,226,
133,231,19,184,218,173,253,171,228,66,235,246,89,169,185,142,4,175,251,200,27,167,106,3,95,98,80,68,196,189,189,153,137,127,185,238,245,132,101,91,239,117,239,42,207,128,244,52,178,94,5,125,185,8,122,
230,43,184,242,220,203,240,188,20,124,47,13,213,84,194,147,71,144,53,201,76,31,233,174,74,29,147,177,241,78,0,150,61,210,111,122,195,232,103,223,209,118,30,100,195,143,24,223,2,174,114,53,177,58,103,64,
122,6,25,175,130,29,133,8,102,230,65,20,207,5,173,119,150,106,62,223,13,30,208,240,192,231,79,0,151,126,202,15,226,231,239,172,71,153,47,130,185,14,182,185,148,215,248,134,195,189,79,100,54,171,5,95,231,
174,178,27,137,40,52,146,160,139,176,241,52,114,65,13,57,191,130,202,233,159,131,9,131,214,200,134,76,0,140,151,65,231,62,10,148,255,10,100,106,240,205,248,61,149,106,252,26,216,142,19,172,175,72,223,
219,95,184,124,104,211,186,136,235,219,85,214,240,40,78,90,114,13,86,151,192,209,52,200,204,96,71,87,12,21,143,227,194,151,127,11,108,2,96,50,0,157,254,40,212,244,239,128,244,52,84,11,175,174,166,116,
229,133,183,197,26,127,237,102,134,246,38,79,197,226,42,55,93,101,102,55,93,222,55,58,136,92,26,32,91,133,178,37,40,83,68,127,151,134,169,60,140,240,217,95,4,93,249,47,137,12,62,233,46,142,186,216,186,
38,164,85,245,238,48,194,23,156,171,12,95,17,191,227,205,183,124,41,179,225,113,112,115,236,139,21,92,101,187,162,171,252,218,28,87,25,220,0,175,209,85,110,127,2,224,114,128,223,220,207,45,6,118,14,225,
45,111,125,43,186,123,122,193,76,56,245,218,5,28,251,226,227,0,34,228,82,105,100,189,10,206,159,252,44,14,31,62,224,250,118,171,65,236,196,90,88,247,125,106,29,13,92,25,59,195,221,251,122,31,7,236,91,
0,188,37,21,132,25,56,91,110,253,45,248,122,119,149,173,53,184,233,166,27,208,219,211,133,174,124,22,187,70,6,241,181,95,243,54,12,236,44,184,25,163,41,161,183,96,97,162,73,196,181,241,68,6,47,182,70,
52,176,201,69,23,58,59,50,144,121,51,24,127,153,188,75,196,85,110,62,211,242,224,193,3,200,229,178,24,24,28,192,254,125,123,113,199,29,183,160,187,144,118,221,148,173,33,155,50,200,248,33,174,92,62,13,
232,10,96,170,179,50,120,115,68,99,53,198,199,46,222,111,153,62,153,252,160,226,42,55,63,127,236,177,199,80,40,20,48,60,52,136,35,71,14,99,215,174,17,188,252,210,243,0,71,96,93,135,135,8,249,12,161,52,
125,9,172,171,32,219,72,100,240,40,153,208,180,70,49,119,102,179,233,10,214,225,42,171,101,199,191,243,92,101,172,201,85,54,11,92,101,234,160,171,188,88,139,63,113,226,4,142,29,59,134,238,238,110,84,171,
21,220,121,199,29,174,65,88,3,34,11,230,24,185,92,0,32,70,24,85,193,28,129,109,236,22,149,230,2,214,233,192,199,93,152,125,64,254,61,119,215,126,138,54,114,145,115,174,178,90,210,85,30,92,232,42,211,162,
174,178,183,136,171,204,29,112,149,151,234,62,30,126,248,97,60,252,240,195,243,194,103,183,3,13,32,147,10,144,73,5,136,26,53,100,82,62,150,186,43,225,196,137,19,119,29,56,112,96,190,171,124,105,221,93,
196,214,117,149,103,135,114,214,26,40,5,164,82,62,170,213,10,230,146,235,11,70,53,226,42,175,157,231,113,223,127,42,21,36,50,56,150,185,230,139,171,188,236,136,67,92,101,136,171,188,234,18,87,121,181,
139,61,226,42,183,151,184,202,226,42,139,171,220,214,128,197,85,158,95,226,42,139,171,188,133,92,229,39,114,31,18,87,121,53,23,66,113,149,87,217,151,174,117,130,209,236,30,196,85,94,101,137,171,44,174,
242,194,11,93,242,169,184,202,243,92,229,39,114,31,18,87,121,185,160,197,85,222,192,194,205,74,173,90,92,229,107,220,85,94,75,247,176,170,229,74,113,149,103,93,229,39,114,31,18,87,89,92,229,235,212,85,
126,34,247,33,113,149,197,85,22,87,121,133,190,120,155,186,202,171,93,57,91,124,3,88,92,229,101,93,229,39,114,31,90,118,221,119,179,159,120,178,173,92,229,39,114,31,90,145,254,221,236,231,69,108,27,87,
121,53,225,110,102,31,60,127,83,116,203,184,202,79,228,62,52,3,113,149,59,30,242,15,223,93,251,169,150,171,252,68,238,67,226,42,95,43,213,201,167,78,137,171,220,161,46,98,78,21,238,127,192,86,30,249,226,
170,93,229,237,18,236,166,5,156,132,188,212,147,227,182,125,9,243,32,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,95,251,229,95,171,223,216,188,
7,32,175,232,42,95,171,123,125,254,53,22,232,186,93,229,121,47,8,95,43,161,139,171,188,213,2,22,87,249,42,134,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,139,171,44,174,178,184,202,
16,87,121,185,18,87,89,92,229,230,204,87,92,101,113,149,55,177,5,139,171,220,193,22,44,174,114,135,71,17,91,223,85,214,151,75,231,139,123,31,134,97,38,130,184,202,27,119,149,115,175,18,94,185,220,147,
194,176,10,176,127,176,103,230,63,140,85,71,190,43,102,223,169,83,74,89,2,177,184,202,235,118,149,247,29,26,187,76,39,25,47,157,239,73,241,46,63,176,123,111,25,121,241,231,206,78,140,252,228,84,45,113,
149,141,54,201,154,115,115,145,136,55,61,224,173,233,42,51,224,131,114,195,123,14,143,95,172,190,228,147,185,210,229,155,161,32,48,119,236,238,187,248,125,154,189,159,47,213,122,66,144,138,201,243,141,
215,211,99,155,55,82,110,102,192,91,204,85,54,109,174,114,114,121,245,65,233,225,155,142,78,140,243,201,84,33,42,100,72,119,165,82,165,183,15,117,93,121,178,28,246,140,3,94,200,238,249,113,107,106,197,
219,211,85,182,152,139,87,51,129,44,37,174,242,13,251,174,204,76,189,188,171,187,113,75,224,133,93,93,217,250,183,239,204,143,61,62,94,26,170,17,169,152,227,56,246,122,122,212,106,91,241,182,115,149,97,
221,208,207,114,130,177,2,201,5,208,61,225,91,229,83,233,176,122,83,95,177,81,59,185,35,221,56,234,123,141,163,195,221,99,223,62,94,30,254,105,48,251,240,60,197,58,166,53,205,198,86,170,173,229,42,59,
133,86,81,144,236,70,33,121,238,177,27,105,128,44,50,195,131,187,167,163,35,229,208,100,175,88,74,33,27,132,31,56,208,255,242,93,80,238,102,119,172,65,7,223,6,174,242,12,210,84,66,62,168,193,148,159,192,
212,11,47,180,94,108,213,246,124,183,230,216,154,147,201,74,208,125,224,198,201,90,255,115,150,189,24,68,221,59,114,87,190,15,204,62,145,82,212,118,179,251,134,91,240,245,239,42,187,119,70,198,171,160,
55,27,34,46,30,199,196,11,167,18,180,218,109,5,186,169,244,220,135,232,121,93,233,124,200,71,85,100,210,167,64,138,125,79,191,225,150,225,175,60,176,204,225,152,53,7,188,69,92,229,10,160,75,32,61,131,
172,87,197,142,174,24,166,244,37,148,47,4,240,84,38,121,198,91,202,61,72,79,37,207,222,36,2,121,132,212,142,131,119,85,26,217,103,152,185,65,64,46,23,84,190,110,32,127,46,187,166,153,217,138,23,183,235,
222,85,174,187,126,90,151,192,113,17,185,160,142,156,95,69,237,220,199,96,99,31,158,151,106,61,72,111,206,99,122,13,129,234,233,66,165,225,93,102,203,211,96,235,249,20,191,97,184,235,181,131,107,57,174,
181,133,93,101,147,184,202,141,150,171,108,19,138,189,175,160,65,122,26,99,79,254,37,154,198,135,74,78,14,144,241,129,9,11,58,249,97,208,196,143,41,85,127,241,182,70,236,127,129,96,64,48,71,51,126,245,
102,14,67,222,104,11,222,2,174,50,18,87,57,76,66,110,0,166,2,197,21,40,93,196,142,174,24,182,250,36,226,178,15,133,0,136,2,96,188,10,122,237,55,64,147,191,226,200,118,83,69,74,205,220,80,107,248,143,16,
155,152,96,124,95,133,111,187,113,228,171,153,13,119,17,94,79,143,34,207,95,209,85,230,21,93,229,243,115,92,101,215,106,215,230,42,47,22,238,242,12,15,37,45,217,192,24,13,226,24,195,131,61,80,92,79,244,
217,34,178,126,29,25,85,194,165,47,253,56,236,133,139,80,175,125,24,106,250,99,9,96,61,13,74,240,106,27,23,251,138,83,151,98,182,246,41,134,133,130,121,115,46,93,206,108,116,170,76,108,12,145,31,40,114,
61,127,203,85,214,77,87,153,187,15,128,220,234,213,114,174,242,241,196,85,166,54,87,153,174,146,171,108,45,227,206,59,239,194,221,119,191,17,76,132,70,67,227,243,199,30,195,133,75,83,32,214,232,206,50,
202,51,47,66,95,254,5,164,211,105,183,172,105,227,22,92,77,182,14,216,48,51,178,35,188,83,115,240,87,1,217,123,1,222,159,245,203,55,22,238,127,96,99,150,209,86,112,149,251,251,119,224,158,123,222,128,
76,54,133,29,125,189,56,122,228,0,222,112,199,141,110,214,104,202,200,4,13,164,85,5,19,151,94,1,244,20,40,158,74,0,235,82,50,116,172,3,182,142,169,241,51,247,132,58,248,36,177,101,128,61,165,236,59,54,
103,28,124,157,187,202,71,142,56,25,167,171,144,199,174,93,195,56,122,244,48,222,243,238,183,1,214,77,100,20,215,144,77,69,40,77,159,3,71,211,64,50,139,132,174,128,173,11,215,53,136,232,182,124,154,138,
0,39,174,50,191,101,35,1,111,25,87,249,177,199,30,131,82,10,61,61,61,216,61,186,11,55,222,112,20,31,255,248,199,1,142,224,33,2,76,29,185,12,3,166,138,168,49,13,152,106,107,77,132,108,3,128,155,246,131,
77,186,144,105,220,14,224,100,242,125,175,218,85,94,122,20,97,153,220,179,41,23,117,149,187,23,186,202,88,212,85,166,69,92,101,219,1,87,121,41,231,243,207,254,236,207,240,234,171,175,226,210,165,75,248,
193,31,252,65,124,254,193,207,1,214,180,166,220,233,20,33,29,88,68,141,178,235,123,57,76,166,251,122,14,96,253,220,115,207,222,5,240,169,228,175,110,186,202,235,95,15,102,182,238,240,243,146,174,178,89,
179,171,76,29,116,149,151,186,0,78,78,78,226,99,31,251,88,219,127,155,140,88,216,77,82,20,128,32,80,168,84,139,232,42,100,23,25,242,181,142,15,220,8,224,169,182,47,216,135,117,194,213,91,220,85,230,57,
83,109,192,34,21,120,136,26,245,121,223,163,157,19,178,184,202,75,14,134,150,235,191,103,215,55,180,142,177,140,10,46,174,178,184,202,226,42,47,214,75,136,171,220,86,226,42,139,171,44,174,114,91,3,22,
87,121,126,137,171,44,174,178,184,202,139,188,236,226,42,3,226,42,175,125,130,209,236,30,196,85,94,101,137,171,44,174,242,194,11,93,243,108,143,184,202,226,42,47,251,103,226,42,111,226,194,205,74,173,
90,92,101,113,149,197,85,222,204,128,197,85,22,87,121,133,164,196,85,22,87,89,92,101,113,149,197,85,22,87,121,21,1,3,226,42,175,57,96,113,149,197,85,238,108,137,171,124,117,66,22,87,249,90,45,113,149,
95,231,18,87,249,90,15,56,9,89,92,229,77,28,69,72,73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,73,192,87,183,196,85,222,46,1,139,171,220,249,64,
197,85,222,236,96,197,85,238,112,168,226,42,111,78,184,226,42,119,178,213,138,171,220,193,86,43,174,178,184,202,226,42,139,171,12,113,149,151,43,113,149,197,85,110,206,124,197,85,22,87,121,19,91,176,184,
202,29,108,193,226,42,119,120,20,177,173,92,101,128,45,137,171,220,33,87,217,122,134,148,50,204,108,196,85,222,176,171,140,87,25,124,190,39,133,93,126,96,247,222,60,242,226,255,123,110,98,228,39,166,170,
59,94,129,82,22,90,91,54,90,92,229,245,187,202,123,15,141,95,172,189,228,147,189,210,229,235,161,84,96,110,223,221,119,233,251,180,245,126,161,84,239,137,17,248,26,158,103,149,82,107,234,42,196,85,94,
232,42,159,74,21,162,90,134,76,33,149,46,191,125,176,123,226,233,114,212,243,39,108,84,8,214,49,121,158,89,75,87,33,174,114,18,50,129,64,41,165,168,239,232,222,177,233,233,147,163,221,209,141,190,138,
187,186,178,245,255,107,103,110,252,137,241,242,64,157,60,47,102,107,53,235,216,108,102,31,188,5,93,101,36,67,197,102,200,6,224,24,30,43,120,185,116,42,172,220,184,99,166,209,120,109,71,58,60,232,251,
209,209,161,238,241,111,31,175,12,253,119,167,13,114,72,65,160,1,152,100,129,139,55,124,46,98,107,185,202,41,39,210,38,35,151,38,189,195,172,97,173,235,167,51,67,59,135,167,163,3,245,134,205,79,90,74,
35,147,214,239,219,187,227,181,187,65,148,38,79,5,68,228,37,93,230,134,84,218,86,223,123,189,187,202,41,42,207,186,202,47,190,148,152,202,169,100,102,167,90,211,113,134,105,53,142,160,103,223,161,201,
106,255,105,139,116,76,202,235,218,153,159,248,110,183,157,165,124,54,198,237,43,110,198,201,158,173,224,42,171,196,85,238,203,197,136,139,79,96,226,133,215,230,192,213,115,66,78,30,7,233,21,130,76,136,
189,233,186,206,93,100,248,236,123,246,246,163,59,159,125,0,128,151,60,157,105,195,1,111,29,87,217,148,64,186,136,140,87,197,142,130,134,41,127,5,149,203,193,44,94,157,60,223,109,214,5,97,64,1,169,29,
251,14,151,26,185,215,44,251,49,8,153,124,186,252,238,238,244,116,126,45,58,248,246,112,149,147,144,57,158,65,46,104,32,23,212,80,59,255,7,128,14,146,199,65,166,146,161,227,236,179,144,9,128,202,165,178,
17,221,5,203,94,17,32,207,247,226,219,134,187,206,29,88,243,146,227,138,23,183,173,224,42,199,101,216,184,8,50,37,244,230,13,148,153,193,248,179,159,119,215,7,47,5,165,210,174,95,78,158,32,235,30,30,77,
240,122,246,220,94,109,248,95,37,88,120,164,15,231,82,197,155,54,35,224,45,231,42,59,97,182,6,101,203,80,182,132,190,130,134,173,191,0,93,115,71,50,220,163,32,155,75,154,30,20,60,112,73,193,47,189,28,
84,194,244,51,128,209,196,236,165,188,250,155,135,242,175,165,87,139,87,47,57,14,110,142,125,177,130,171,108,87,116,149,95,155,227,42,131,27,224,53,186,202,237,79,0,92,14,240,155,251,185,133,53,128,77,
222,84,198,48,60,82,96,235,30,86,154,241,21,210,202,224,202,19,191,137,221,111,251,30,16,49,152,0,68,0,42,10,92,252,36,40,122,9,28,151,82,53,221,31,218,2,63,175,160,111,247,40,122,99,79,102,60,184,164,
70,196,85,118,71,216,20,140,141,113,199,157,183,225,194,165,73,76,78,87,147,37,103,66,87,198,71,117,226,69,216,70,0,79,51,168,252,18,80,253,156,131,94,77,61,121,199,149,83,67,61,233,67,161,246,63,27,248,
230,118,192,238,237,206,78,222,64,158,119,121,99,51,185,166,171,236,240,227,5,174,114,110,111,42,205,28,173,224,42,143,175,194,85,94,220,157,219,12,87,25,32,220,120,227,81,188,225,238,123,144,201,230,
240,70,10,240,234,169,179,248,252,177,227,128,6,50,94,6,1,42,152,252,202,191,193,208,80,111,210,51,218,100,236,221,0,146,110,165,60,57,125,83,220,119,231,175,119,229,47,255,48,195,122,1,133,111,35,63,
120,104,53,11,62,91,218,85,86,138,112,215,93,119,33,151,205,160,167,171,128,93,195,131,120,239,215,190,19,41,79,131,227,18,148,173,32,235,87,81,154,120,17,28,78,128,226,137,54,122,125,38,1,172,171,32,
91,190,33,229,171,41,134,61,73,108,160,40,126,179,158,156,88,213,1,21,127,217,241,239,60,87,217,174,201,85,142,23,184,202,232,160,171,188,84,107,206,102,51,232,238,238,194,206,129,1,236,218,53,138,129,
129,65,68,97,213,29,141,179,22,217,32,2,235,41,196,245,43,72,167,211,179,231,222,90,27,4,49,96,227,212,96,215,216,13,186,97,94,33,226,35,0,238,222,55,124,26,147,120,96,3,163,136,45,226,42,63,249,228,147,
200,231,243,24,30,26,196,225,67,7,49,50,50,148,216,245,33,96,106,72,251,26,41,21,34,170,79,56,21,60,153,85,58,25,188,1,178,17,128,24,39,95,126,226,22,2,78,37,223,247,142,253,195,167,135,54,212,7,111,21,
87,249,153,103,158,65,111,111,47,114,185,28,46,92,184,128,31,251,177,159,108,94,198,193,22,240,160,16,120,17,170,229,73,116,229,125,128,185,41,44,181,246,251,146,223,59,74,100,159,106,251,129,87,229,42,
251,75,236,92,172,194,85,214,115,79,220,208,60,87,121,106,109,174,114,123,216,155,233,42,91,107,241,224,131,15,226,193,7,31,108,115,149,201,157,195,176,22,80,10,129,15,52,26,37,192,246,181,29,1,152,123,
157,176,214,238,103,230,207,18,181,122,68,113,149,23,239,203,155,129,37,147,26,107,160,192,208,81,56,123,141,152,55,101,79,166,229,131,68,92,133,184,202,171,225,121,218,102,124,138,220,58,198,130,133,
37,59,191,1,137,171,188,210,12,111,241,86,109,146,144,151,94,7,217,72,137,171,188,250,175,21,87,89,92,101,113,149,219,26,176,184,202,243,75,92,101,113,149,197,85,94,228,101,23,87,25,16,87,121,181,19,140,
185,37,174,242,26,74,92,101,113,149,23,94,232,154,103,123,196,85,22,87,121,217,63,19,87,121,211,22,110,196,85,22,87,121,65,55,33,174,178,184,202,226,42,139,171,188,108,137,171,44,174,178,184,202,226,42,
55,75,92,101,113,149,197,85,22,87,121,237,125,240,252,77,81,113,149,87,89,226,42,67,92,229,107,186,196,85,126,157,75,92,229,107,61,224,36,100,113,149,55,113,20,33,37,1,75,192,18,176,148,4,44,1,75,73,192,
18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,37,1,95,221,18,87,121,187,4,44,174,114,231,3,21,87,121,179,131,21,87,185,195,161,138,171,188,57,225,138,171,220,201,86,43,174,114,7,91,173,
184,202,226,42,139,171,44,174,50,196,85,94,174,196,85,22,87,185,57,243,21,87,89,92,229,77,108,193,226,42,119,176,5,139,171,220,225,81,132,184,202,29,12,88,92,101,113,149,197,85,190,14,92,101,51,148,10,
138,183,143,246,93,250,160,182,222,207,139,171,220,17,87,89,23,210,233,242,219,6,187,39,158,42,135,221,127,202,86,69,226,42,111,186,171,172,187,186,50,181,111,235,207,79,62,53,81,222,217,32,207,139,217,
104,147,108,79,217,205,106,193,91,211,85,78,190,191,185,174,50,65,229,50,169,168,114,195,142,153,70,120,118,71,38,222,239,5,209,145,225,238,43,223,54,81,25,248,89,0,117,82,94,132,0,218,235,233,81,201,
234,155,221,112,23,145,92,216,60,6,210,52,199,85,46,180,92,229,238,35,3,251,109,107,40,182,188,171,220,227,87,240,240,67,199,113,241,236,105,176,46,39,50,109,29,176,117,247,255,100,146,22,27,1,176,46,
220,230,24,55,97,204,102,159,162,173,92,192,172,160,188,52,16,135,176,84,131,242,243,96,29,195,67,14,41,228,81,153,254,18,26,245,219,209,117,232,112,219,235,164,91,221,133,115,149,9,228,121,72,15,13,12,
79,159,223,255,82,46,168,77,229,84,184,35,155,46,126,253,158,190,179,159,61,55,181,231,24,136,66,144,138,86,171,131,111,51,87,249,201,54,87,57,65,82,225,248,29,78,158,26,107,147,11,98,208,179,119,255,
68,117,199,121,67,89,205,42,213,181,179,48,245,221,32,149,5,145,91,227,14,2,181,194,153,141,213,157,236,217,90,174,114,228,92,229,23,19,87,89,205,134,204,173,144,221,223,169,242,65,58,196,104,182,22,231,
175,88,74,179,231,211,109,135,119,190,244,246,166,14,14,109,212,106,116,240,109,229,42,167,189,26,118,20,52,108,233,49,84,46,39,166,50,5,173,231,109,186,150,156,44,56,41,139,244,142,61,123,75,141,220,
101,70,90,147,242,210,133,76,245,93,133,84,185,0,184,217,208,106,116,240,109,229,42,35,158,65,46,21,34,27,212,81,187,248,199,224,56,213,26,50,182,195,213,205,245,17,202,6,233,72,221,16,104,27,84,25,202,
243,61,125,75,127,254,210,97,16,17,173,82,7,223,166,174,178,133,50,37,76,190,240,72,98,220,55,67,110,127,20,164,203,219,235,30,61,88,14,51,39,1,5,79,241,254,66,186,114,3,136,20,243,198,232,245,45,226,
42,219,121,174,114,21,100,42,80,92,70,111,222,192,54,94,129,105,52,31,5,153,106,61,212,212,45,107,186,81,74,144,75,231,26,56,58,193,238,237,229,101,252,234,155,122,50,147,153,85,47,222,44,245,7,94,79,
143,34,207,95,209,85,230,21,93,229,243,115,92,101,215,106,215,230,42,47,22,238,242,12,15,181,90,178,49,26,196,17,140,105,64,113,8,178,85,144,46,35,27,132,72,171,42,38,158,253,115,56,169,198,61,52,218,
243,82,179,143,232,37,5,75,138,66,222,147,215,70,157,2,44,124,21,221,153,247,139,105,34,181,33,71,131,216,24,130,231,41,242,188,57,174,178,109,186,202,166,187,15,228,30,35,190,156,171,252,74,226,42,163,
205,85,198,85,115,149,53,192,22,198,196,80,100,0,110,192,234,42,96,43,32,83,66,33,29,194,54,78,193,134,126,50,108,115,207,63,86,42,121,230,102,195,3,77,41,116,211,19,126,61,76,125,133,88,67,65,143,238,
204,159,63,196,238,149,220,0,251,219,116,149,221,130,236,2,87,57,59,144,74,207,94,216,150,114,149,39,87,225,42,47,236,2,22,11,119,177,175,89,170,43,153,75,81,186,11,32,113,140,93,195,59,157,151,108,170,
128,46,33,227,213,224,115,17,165,211,231,92,183,64,1,168,17,0,147,0,94,251,91,208,153,15,3,87,62,132,198,196,23,134,166,107,221,199,128,152,9,198,203,248,149,183,144,34,78,206,200,109,96,170,220,116,149,
137,18,87,57,51,199,85,182,118,121,87,249,228,11,139,187,202,176,140,38,85,183,90,87,121,181,188,195,66,10,152,241,174,119,190,29,7,14,28,134,37,194,133,11,87,240,119,159,251,50,98,237,158,211,159,241,
52,102,78,255,119,236,232,254,105,80,227,111,64,118,210,173,129,112,148,188,235,170,80,182,186,7,164,174,192,218,211,96,123,208,35,125,47,51,27,242,131,21,159,85,180,229,93,229,119,189,235,157,56,120,
240,0,178,185,12,178,217,60,246,239,219,143,48,140,241,208,177,199,192,6,200,250,140,233,168,4,61,246,51,72,103,178,104,130,170,224,24,212,10,185,30,236,234,190,56,202,172,79,1,230,32,145,185,51,71,83,
92,158,164,21,3,222,242,174,114,169,84,130,239,251,232,235,235,195,222,61,187,113,243,77,55,160,175,175,144,184,201,85,164,253,16,1,202,136,107,99,201,212,125,10,172,167,156,12,174,103,220,140,208,214,
112,233,204,19,135,97,205,41,64,131,152,119,220,178,235,43,131,216,8,189,206,108,9,138,150,113,149,237,154,93,101,116,208,85,94,174,50,153,12,250,122,123,176,127,255,62,28,58,116,0,143,126,249,17,144,
141,97,117,5,100,107,8,168,142,106,249,18,160,139,128,158,134,210,37,55,49,49,85,176,173,185,9,139,45,237,39,232,243,196,110,24,24,168,112,239,186,233,245,173,228,42,31,63,238,30,189,158,203,229,176,103,
207,30,252,147,239,248,14,92,60,127,54,57,55,225,86,27,125,47,66,163,86,5,108,161,109,67,181,57,43,77,118,64,172,222,67,48,127,139,38,160,169,204,206,141,173,7,179,37,151,239,66,87,89,121,80,118,5,87,
185,220,230,42,171,215,217,85,126,236,177,199,112,252,248,241,228,191,113,239,52,107,226,100,214,105,65,208,208,97,5,100,27,115,79,89,240,236,17,0,5,12,16,76,25,128,6,56,192,42,93,101,127,249,77,184,171,
239,42,47,22,246,230,186,202,60,123,104,133,9,138,8,138,172,91,102,181,241,50,179,67,228,146,5,19,13,247,124,119,113,149,151,158,249,205,46,16,89,171,161,200,9,155,115,187,177,185,171,120,0,131,136,215,
124,164,85,92,229,5,203,160,139,125,0,16,87,89,92,101,113,149,231,52,96,113,149,231,151,184,202,226,42,139,171,188,200,203,46,174,50,32,174,242,106,38,24,11,75,92,229,53,148,184,202,226,42,47,188,208,
53,207,246,136,171,44,174,242,178,127,38,174,242,134,22,110,196,85,22,87,121,217,110,66,92,101,113,149,197,85,22,87,121,249,21,105,113,149,197,85,110,253,153,184,202,226,42,139,171,12,113,149,197,85,22,
87,121,141,125,240,252,77,81,113,149,87,89,226,42,67,92,229,107,186,196,85,126,157,75,92,229,107,61,224,36,100,113,149,55,113,20,33,37,1,75,192,18,176,148,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,
148,4,44,1,75,192,82,18,176,4,44,37,1,95,221,18,87,121,187,4,44,174,114,231,3,189,110,93,229,230,89,146,197,206,71,136,171,188,21,90,176,184,202,157,13,87,92,229,78,182,90,113,149,59,216,106,197,85,22,
87,89,92,101,113,149,33,174,242,114,37,174,242,38,184,202,205,90,108,162,33,174,242,38,184,202,203,5,44,174,178,184,202,215,190,171,188,217,45,88,92,229,14,143,34,196,85,238,96,192,226,42,139,171,44,174,
178,184,202,203,118,13,226,42,139,171,124,181,92,229,100,253,133,55,28,240,214,117,149,205,82,174,114,255,92,87,121,124,174,171,236,177,241,122,122,108,243,158,235,13,119,17,91,215,85,70,18,178,129,101,
128,102,93,229,161,185,174,114,249,235,247,244,157,253,220,185,169,61,199,24,136,160,40,230,56,54,88,133,14,46,174,50,40,49,153,154,120,117,12,134,134,223,179,103,255,120,117,199,69,77,57,205,94,170,171,
191,48,243,221,32,149,39,165,82,164,188,128,130,96,195,236,175,107,189,91,216,85,158,125,182,155,2,51,195,178,113,63,131,141,225,229,131,116,132,209,92,45,46,140,51,210,28,248,116,235,193,157,175,188,
29,128,195,171,73,121,9,251,43,174,242,2,87,185,252,56,42,151,221,83,10,221,163,32,219,30,205,155,132,204,164,17,236,216,189,187,212,200,79,58,29,220,207,20,50,245,119,101,83,181,238,100,170,233,109,148,
253,221,186,174,178,95,71,253,226,159,128,117,144,248,157,126,219,83,99,185,245,144,105,149,245,83,17,29,74,199,54,85,103,74,169,192,179,183,236,44,76,28,1,144,34,144,183,26,126,125,27,185,202,21,247,
239,155,50,250,10,22,202,86,48,245,210,87,64,104,62,64,207,119,107,33,173,229,34,6,200,194,235,25,217,83,14,115,231,25,1,148,135,61,249,84,229,38,55,75,130,7,203,148,244,195,107,110,193,91,200,85,142,
18,87,217,93,0,201,86,161,184,130,222,188,133,105,156,130,13,131,86,87,209,124,60,111,251,143,31,228,130,116,221,238,175,48,148,1,148,151,77,53,238,41,164,74,121,0,104,83,95,215,222,130,183,142,171,28,
207,186,202,136,64,166,10,232,10,50,65,132,140,106,96,234,133,207,192,45,10,166,90,50,173,106,174,230,1,48,32,138,104,176,16,233,224,34,0,248,94,116,91,218,175,103,155,144,211,74,190,253,54,113,149,181,
115,149,109,3,214,212,28,251,107,202,200,103,98,216,232,2,56,246,146,161,91,186,53,141,87,202,79,158,49,79,232,31,233,27,170,134,169,23,8,12,143,236,72,127,238,242,161,53,109,251,44,241,94,220,66,174,
114,115,170,237,70,24,176,53,192,84,144,241,27,240,81,70,249,220,21,183,76,154,12,41,149,10,64,112,3,5,192,67,45,204,118,79,87,11,207,0,134,193,70,21,210,51,111,4,216,174,198,86,86,43,54,7,55,68,75,92,
101,127,142,171,204,88,222,85,190,116,97,113,87,89,17,183,102,102,171,117,149,219,63,95,238,233,171,115,191,142,90,59,30,204,6,123,70,135,192,28,58,188,90,151,161,108,25,25,85,67,245,252,111,3,54,104,
173,238,53,125,123,68,30,48,237,129,198,79,123,0,77,88,99,47,0,26,129,106,220,13,134,97,99,108,251,121,228,213,78,149,183,148,171,12,0,239,120,199,219,112,224,224,65,80,162,131,253,214,111,255,177,27,
255,234,10,210,94,10,28,197,176,145,15,143,20,184,193,160,198,25,160,250,40,200,92,1,153,42,200,148,213,64,223,13,59,216,234,51,68,102,183,130,190,205,167,200,198,42,96,51,61,109,215,179,22,225,92,101,
181,164,171,60,184,208,85,166,69,93,101,111,17,87,153,59,224,42,47,213,125,248,190,143,195,135,15,33,149,74,35,147,205,35,95,232,198,63,252,182,111,194,31,254,241,39,193,4,164,189,44,60,174,33,124,238,
39,145,207,103,64,42,97,216,172,113,203,166,166,1,216,42,138,99,207,14,117,143,100,206,16,197,111,33,232,222,163,131,199,119,158,56,127,207,217,149,90,240,150,119,149,239,188,243,78,0,64,46,151,197,174,
145,97,28,61,114,8,223,254,109,223,2,182,33,56,174,128,108,5,62,23,81,47,190,10,232,73,32,74,62,244,36,72,79,3,102,6,48,101,40,59,49,12,171,207,83,50,91,45,4,51,163,42,157,89,113,53,77,45,177,115,177,
10,87,121,254,137,155,121,174,114,105,109,174,242,74,45,119,35,174,178,82,10,61,61,61,24,221,189,11,71,143,30,198,39,63,249,151,128,13,193,214,181,78,159,234,8,107,87,64,122,106,206,7,244,12,72,59,192,
26,186,56,164,16,93,102,24,38,54,240,84,212,175,39,39,214,77,175,163,237,158,136,5,174,50,121,80,188,130,171,220,104,115,149,241,58,187,202,159,254,244,167,49,54,54,6,163,53,126,237,87,127,21,31,251,141,
223,128,181,33,200,70,201,58,116,21,113,125,26,48,101,192,20,221,135,118,11,80,205,181,18,15,97,63,33,46,18,27,237,38,84,182,123,53,219,70,219,194,85,62,119,238,28,126,229,87,126,5,191,244,75,191,52,123,
144,197,50,88,1,30,124,40,14,147,177,113,109,126,35,115,199,1,92,119,152,245,16,53,0,171,1,4,68,200,20,238,127,160,253,150,132,171,119,0,123,41,87,153,218,250,222,149,92,229,229,250,231,229,90,244,202,
60,15,183,132,112,5,134,209,33,72,25,0,17,192,97,242,2,204,94,91,90,215,26,128,137,180,109,107,40,27,223,244,100,107,221,64,122,49,87,121,153,9,226,124,87,153,22,117,149,121,201,150,219,121,87,185,77,
7,7,3,214,56,30,222,154,101,214,55,80,37,88,183,63,229,170,177,177,128,23,184,202,26,68,109,174,178,79,10,164,22,188,142,203,186,202,173,159,77,181,250,225,215,207,85,230,214,62,159,181,26,169,148,191,
196,14,80,107,241,233,10,17,103,33,174,242,122,92,229,24,169,84,128,101,84,112,113,149,197,85,22,87,121,110,167,35,174,242,156,18,87,121,181,203,158,226,42,139,171,44,174,50,196,85,22,87,89,92,229,100,
87,67,92,101,113,149,197,85,22,87,121,217,110,66,92,101,113,149,197,85,22,87,121,217,18,87,89,92,101,113,149,197,85,110,150,184,202,107,119,149,155,181,25,79,60,17,87,121,141,1,139,171,44,174,114,103,
75,92,229,171,19,178,184,202,215,106,117,242,169,83,226,42,119,168,139,152,83,226,42,119,56,224,36,100,113,149,55,113,20,33,37,1,75,192,18,176,148,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,
44,1,75,192,82,18,176,4,44,37,1,95,221,18,87,121,187,4,44,174,114,231,3,189,110,93,229,107,42,96,113,149,175,98,168,226,42,111,78,184,226,42,119,178,213,138,171,220,193,86,43,174,178,184,202,226,42,139,
171,12,113,149,151,43,113,149,197,85,110,206,124,197,85,22,87,121,19,187,8,113,149,59,216,130,197,85,238,240,40,66,92,229,14,6,44,174,178,184,202,226,42,139,171,188,108,215,32,174,178,184,202,226,42,95,
191,174,114,67,92,229,77,117,149,235,45,87,121,119,223,249,207,159,159,222,253,16,195,173,37,139,171,188,33,87,57,118,174,114,165,175,229,42,239,236,154,250,78,0,5,82,42,61,207,85,86,27,9,120,27,185,202,
182,205,85,214,206,85,166,209,92,45,238,26,183,200,112,224,171,91,15,236,60,245,14,56,87,57,213,230,42,175,187,5,111,51,87,153,146,89,161,134,229,24,76,49,130,29,163,123,138,141,252,180,161,172,102,74,
101,186,210,181,119,101,83,245,94,0,41,48,251,27,101,127,183,188,171,108,117,162,32,162,217,85,180,61,154,215,185,202,65,68,7,179,177,205,52,152,82,202,243,249,230,254,252,196,17,40,149,34,79,5,208,70,
173,151,253,109,93,220,182,174,171,92,197,244,75,143,65,193,77,120,220,147,188,41,89,29,77,22,162,200,192,239,29,222,85,106,228,47,91,164,224,251,106,119,46,93,187,25,64,0,184,57,253,74,252,250,54,115,
149,171,137,171,92,69,111,193,194,134,175,193,132,205,126,120,22,174,118,33,187,150,236,101,253,84,157,119,135,150,2,203,240,189,108,16,189,49,27,84,187,92,199,109,105,37,126,125,155,185,202,161,11,218,
84,145,9,98,164,85,136,153,151,30,156,181,148,91,4,251,44,91,97,1,210,212,95,8,117,122,156,201,67,224,155,91,210,126,84,72,22,188,188,245,94,228,182,168,171,28,193,154,170,155,49,154,42,242,25,13,19,95,
6,98,247,142,107,233,224,80,201,74,31,129,20,208,51,212,211,95,105,100,206,0,10,74,217,193,29,249,137,195,88,229,217,234,237,229,42,115,12,178,161,155,204,152,42,210,126,132,0,53,84,46,78,38,19,33,247,
80,211,230,131,77,155,11,253,141,56,147,155,169,101,207,36,63,168,42,164,139,119,131,20,145,231,17,181,61,0,106,237,227,224,45,230,42,91,19,58,44,149,27,110,74,110,171,72,123,33,106,151,63,225,246,239,
146,217,166,215,212,193,155,11,255,105,223,83,94,174,110,12,174,128,45,82,94,253,14,176,93,55,181,179,164,171,140,53,185,202,102,129,171,76,29,116,149,151,119,62,221,133,143,216,32,229,43,144,109,64,113,
29,100,170,72,123,117,112,60,3,27,59,172,186,217,130,21,249,104,18,22,158,82,24,217,115,211,62,99,120,140,96,224,81,116,147,82,214,48,91,203,97,184,174,181,136,45,227,42,3,192,61,247,220,131,35,71,142,
32,151,235,130,182,132,19,207,159,196,227,79,190,4,216,20,82,30,193,179,6,113,201,135,159,165,100,113,41,121,99,54,20,16,122,224,70,136,137,210,83,52,84,176,23,72,197,183,19,155,158,189,61,47,245,159,
158,56,58,3,165,214,183,216,195,108,221,225,231,37,93,101,179,102,87,153,58,232,42,47,117,1,84,74,225,134,27,110,64,62,159,71,62,95,64,190,208,131,195,71,143,224,241,39,190,10,27,43,40,31,240,56,70,253,
226,231,144,165,119,3,209,139,64,252,18,40,190,224,222,125,214,245,215,158,225,20,114,189,39,129,24,4,141,238,244,248,48,169,27,78,98,5,219,222,95,98,231,98,21,174,178,158,123,226,134,230,185,202,83,107,
115,149,219,195,222,76,87,217,90,139,108,54,131,66,33,143,193,161,33,12,14,14,97,112,104,24,41,159,17,70,117,176,6,124,142,17,143,255,127,80,193,67,96,213,228,216,56,105,28,110,49,137,226,184,15,156,187,
76,54,98,176,161,180,95,235,99,102,67,126,176,236,51,227,150,94,174,100,75,201,48,111,129,171,172,60,40,187,130,171,92,110,115,149,213,235,236,42,143,143,143,99,116,116,20,67,67,131,56,124,248,48,140,
49,8,107,101,176,74,185,201,145,169,35,174,91,32,206,131,84,128,89,153,214,58,221,194,134,80,28,119,121,104,76,18,107,13,232,64,129,187,96,173,105,62,42,114,93,163,136,215,195,85,94,105,88,54,191,117,
174,102,232,246,217,207,126,22,47,191,252,50,8,192,243,207,63,135,119,191,251,107,193,208,32,27,185,201,7,215,96,227,233,4,172,158,118,75,157,122,6,208,211,80,201,130,20,217,122,198,231,70,5,136,53,96,
225,33,14,54,109,193,125,173,117,173,185,202,213,106,21,191,247,123,191,135,223,251,189,223,115,215,111,203,110,198,233,17,88,55,160,208,104,205,240,154,7,95,154,87,58,55,20,53,0,51,123,170,30,161,117,
198,162,109,137,111,221,139,61,238,110,158,197,93,229,101,106,190,171,140,69,93,229,165,251,224,206,186,202,220,122,251,195,26,48,98,176,117,211,105,178,141,228,162,230,118,86,200,134,32,219,0,56,2,56,
170,248,104,88,98,235,3,12,102,110,172,102,235,126,27,187,202,179,56,138,53,17,130,20,18,77,72,181,117,102,118,118,77,194,218,43,68,38,227,224,106,6,17,166,215,59,209,216,38,174,242,236,132,71,199,33,
82,190,215,90,79,110,46,224,187,95,155,100,97,159,78,43,178,59,220,17,78,0,226,42,175,118,221,194,64,235,24,217,108,166,109,134,57,255,255,25,128,125,30,224,145,182,255,84,92,229,213,116,19,11,93,229,
197,135,159,226,42,139,171,44,174,242,252,239,90,92,229,164,196,85,134,184,202,226,42,47,118,161,107,158,237,17,87,89,92,229,101,255,76,92,229,101,182,233,55,99,130,49,191,107,16,87,121,157,253,234,106,
38,21,226,42,111,160,95,93,233,162,38,174,242,186,23,110,196,85,134,184,202,43,92,236,196,85,94,231,154,245,106,2,110,181,98,113,149,59,211,69,64,92,101,113,149,147,23,64,92,101,136,171,188,98,87,33,174,
114,7,250,224,249,155,162,226,42,175,178,196,85,134,184,202,215,116,137,171,252,58,151,184,202,215,122,192,73,200,226,42,111,226,40,66,74,2,150,128,37,96,41,9,88,2,150,146,128,37,96,9,88,74,2,150,128,
37,96,41,9,88,2,150,128,165,36,96,9,88,74,2,190,186,37,174,242,118,9,88,92,229,206,7,42,174,242,102,7,43,174,114,135,67,21,87,121,115,194,21,87,185,147,173,86,92,229,14,182,90,113,149,197,85,22,87,89,
92,101,136,171,188,92,137,171,44,174,114,115,230,43,174,178,184,202,155,216,69,136,171,220,193,22,44,174,114,135,71,17,226,42,119,48,96,113,149,197,85,22,87,89,92,229,101,187,6,113,149,197,85,22,87,89,
92,229,121,227,77,113,149,197,85,22,87,249,122,116,149,49,154,23,87,121,221,174,114,48,235,42,99,9,87,185,127,116,119,203,85,86,226,42,175,209,85,254,211,89,87,185,173,171,224,228,1,211,179,174,242,129,
108,108,179,13,70,90,121,62,110,222,145,155,56,10,162,244,60,87,121,93,1,111,31,87,153,188,121,58,184,235,42,152,52,252,222,225,209,82,35,63,102,41,5,223,247,246,228,51,141,155,1,164,152,225,67,209,138,
221,196,54,118,149,57,113,149,19,102,18,243,31,48,109,192,214,192,203,248,169,26,143,106,139,148,101,242,85,38,21,189,49,155,174,247,146,82,238,233,167,226,42,47,227,42,123,17,102,94,254,2,8,94,235,217,
199,115,200,10,88,88,98,104,218,209,213,208,153,73,70,128,192,179,183,164,188,184,11,64,64,74,169,149,250,225,109,232,42,215,146,73,77,21,133,140,133,141,199,192,113,178,24,79,179,162,66,34,110,1,196,
232,25,236,238,43,135,185,75,12,15,158,194,64,95,110,234,136,107,120,240,28,167,176,158,62,120,11,187,202,48,110,204,156,242,99,248,20,162,122,101,122,22,173,110,246,199,173,104,24,13,147,206,20,171,233,
43,12,143,153,148,42,164,43,111,0,145,199,204,4,207,83,94,79,15,173,181,5,207,54,135,45,227,42,219,196,85,142,128,100,228,162,184,142,180,23,161,126,229,179,152,125,216,127,219,163,210,155,139,253,41,
165,200,203,178,182,222,20,64,72,251,225,237,110,61,219,253,131,203,93,232,182,145,171,108,220,174,6,199,128,141,156,130,104,106,200,248,49,160,43,96,237,185,25,94,107,150,231,207,194,213,68,24,24,57,
48,162,13,77,3,12,79,197,71,19,110,143,215,59,209,112,174,50,47,233,42,119,47,116,149,177,168,171,76,139,184,202,182,3,174,242,210,206,103,50,142,54,166,213,247,83,2,161,178,173,35,240,66,120,92,133,174,
54,145,234,160,21,244,236,67,255,21,202,245,158,126,109,104,2,108,160,160,187,134,242,231,250,147,183,111,243,56,237,218,22,123,182,138,171,12,0,35,35,35,184,239,190,251,208,219,219,15,120,30,158,127,
225,20,190,252,216,11,96,93,133,242,3,120,236,33,156,158,70,182,191,15,12,5,166,217,133,170,230,206,180,23,100,10,182,110,39,28,41,98,209,147,153,28,190,92,220,253,34,140,118,99,194,37,90,243,150,119,
149,1,224,29,239,120,123,11,174,206,230,10,216,179,103,31,142,63,249,28,66,93,3,76,0,159,9,241,204,87,64,244,13,110,236,108,216,169,25,154,0,173,0,237,129,107,167,34,134,30,39,142,24,172,41,227,87,186,
1,24,248,158,53,211,211,118,237,203,149,91,200,85,86,74,161,80,40,96,231,192,0,134,135,71,48,56,52,140,209,225,126,156,62,51,6,38,15,100,24,241,212,167,129,151,39,64,60,3,80,208,92,233,114,19,34,83,131,
50,33,168,103,247,20,113,108,136,181,239,19,242,32,210,220,8,23,55,141,87,179,30,252,122,184,202,139,133,189,81,87,249,213,87,95,197,190,125,251,48,52,52,136,67,135,14,162,171,171,27,167,78,189,2,80,26,
138,61,192,196,176,113,17,104,188,8,82,169,54,252,170,57,180,171,3,182,225,251,232,47,146,13,53,177,246,21,25,159,117,172,147,103,219,95,221,3,216,215,154,171,252,232,163,143,98,116,116,20,71,143,30,197,
149,177,49,124,227,55,126,51,136,99,144,82,176,113,13,202,70,128,142,65,122,202,181,94,242,220,154,10,219,217,21,58,196,54,64,181,66,137,109,175,64,6,74,113,210,61,240,186,2,102,107,157,43,183,152,171,
188,204,12,124,190,171,76,139,186,202,188,100,203,237,132,171,252,39,127,242,39,248,147,63,249,19,55,64,178,0,148,130,98,183,214,204,182,6,50,13,144,41,131,91,124,155,91,157,83,108,192,208,128,53,213,
64,85,67,226,216,115,71,2,168,218,182,143,181,142,128,183,172,171,108,220,11,109,1,195,17,64,10,54,174,35,240,44,216,86,209,118,51,19,208,58,225,105,96,173,189,226,171,186,207,208,62,49,131,136,166,147,
61,57,187,214,113,240,54,112,149,185,181,78,1,107,16,235,58,2,159,91,18,248,252,15,112,4,69,250,180,143,70,23,177,117,175,14,243,216,70,38,26,219,192,85,230,214,58,178,142,66,100,51,62,216,198,96,142,
192,28,38,31,17,216,198,128,213,128,181,207,41,50,67,205,76,149,50,167,87,115,186,103,155,187,202,140,40,106,0,176,72,5,190,123,183,55,119,85,218,84,112,192,226,107,190,230,29,79,2,102,127,242,194,136,
171,188,216,90,197,210,174,50,181,237,245,241,130,125,64,128,195,106,181,252,4,196,85,198,154,186,12,64,92,229,85,175,1,47,181,86,177,92,137,171,188,134,18,87,89,92,229,133,23,186,230,138,135,184,202,
226,42,47,251,103,226,42,47,179,77,191,25,19,140,249,93,131,184,202,235,236,87,87,122,7,136,171,44,174,178,184,202,155,29,240,220,139,157,184,202,29,9,88,92,101,113,149,197,85,22,87,185,89,226,42,139,
171,44,174,178,184,202,107,239,131,231,111,138,138,171,188,202,18,87,25,226,42,95,211,37,174,242,235,92,226,42,95,235,1,39,33,139,171,188,137,163,8,41,9,88,2,150,128,165,36,96,9,88,74,2,150,128,37,96,
41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,41,9,248,234,150,184,202,219,37,96,113,149,59,31,168,184,202,155,29,172,184,202,29,14,85,92,229,205,9,87,92,229,78,182,90,113,149,59,216,106,197,85,
22,87,89,92,101,113,149,33,174,242,114,37,174,178,184,202,205,153,175,184,202,226,42,111,98,23,33,174,114,7,91,176,184,202,29,30,69,136,171,220,193,128,197,85,22,87,89,92,101,113,149,151,237,26,196,85,
22,87,89,92,101,113,149,231,141,55,197,85,22,87,89,92,101,113,149,23,191,184,137,171,44,174,50,58,227,42,107,113,149,59,235,42,155,165,92,229,91,208,238,42,187,163,8,107,238,34,182,169,171,76,201,226,
220,92,87,185,206,163,218,34,109,153,2,149,73,69,247,100,83,245,62,162,166,42,181,252,1,69,113,149,95,62,150,184,202,94,219,3,163,185,245,44,122,3,139,152,250,186,27,58,51,197,8,224,123,184,37,240,226,
174,228,92,135,79,158,239,37,221,196,218,90,240,214,119,149,107,109,174,178,123,238,113,19,125,77,122,125,183,236,73,22,221,131,221,125,165,70,238,10,195,135,231,97,160,47,63,115,3,136,82,228,121,62,60,
79,37,163,137,53,178,191,91,214,85,142,90,174,114,218,143,225,171,8,213,43,51,80,205,135,253,55,247,247,154,162,2,24,161,78,165,75,245,204,180,165,128,25,190,42,164,171,119,195,29,247,114,167,62,151,225,
215,183,161,171,28,38,174,178,187,14,100,124,141,198,248,67,128,157,251,136,244,22,190,194,12,164,148,130,202,120,177,77,21,65,30,82,94,116,43,56,57,144,184,194,219,106,27,186,202,58,113,149,35,192,214,
145,242,99,192,214,1,227,181,193,213,205,71,164,171,100,99,0,216,49,180,123,56,210,94,137,65,240,125,123,68,41,56,70,193,209,235,235,24,69,108,89,87,217,177,109,206,142,11,145,246,52,60,52,160,107,206,
187,107,142,128,220,32,161,217,67,2,181,168,187,71,27,175,236,126,76,46,236,200,142,237,108,133,213,246,156,184,85,119,17,204,150,160,104,25,87,217,174,217,85,70,7,93,229,85,45,90,49,59,134,152,35,48,
55,47,120,13,120,104,32,42,150,220,133,174,249,120,244,57,143,72,87,80,65,38,163,13,149,154,235,22,93,153,153,225,132,131,91,243,122,240,150,115,149,173,53,120,255,251,191,1,253,59,7,96,44,112,250,181,
75,120,228,177,19,78,59,180,53,248,108,16,151,79,64,209,91,221,172,153,189,214,117,165,185,241,74,129,74,89,107,139,4,205,4,67,105,191,209,237,118,80,137,57,12,121,237,23,185,217,123,34,22,184,202,228,
65,241,10,174,114,163,205,85,198,235,236,42,191,247,189,239,197,232,232,46,244,245,245,98,207,238,221,120,215,187,30,64,33,159,2,91,55,154,32,83,134,46,63,7,80,210,69,168,185,255,79,240,160,60,95,129,
77,153,56,182,4,131,128,26,57,0,134,72,173,127,61,120,171,184,202,221,221,221,200,229,114,216,57,48,128,193,193,33,244,239,28,196,225,131,123,240,212,87,95,1,121,117,128,9,220,56,5,84,60,119,58,136,147,
119,165,5,96,21,216,42,192,232,216,67,84,4,135,6,172,61,207,53,115,195,238,237,180,62,246,119,189,117,173,185,202,103,206,156,193,145,35,71,48,60,52,136,125,251,247,163,191,127,39,158,124,226,56,148,159,
135,213,85,144,33,64,215,64,103,62,4,168,12,136,82,201,180,137,1,196,128,105,128,77,35,242,186,247,151,60,14,153,56,130,130,239,150,239,216,154,228,158,145,181,203,224,91,197,85,62,126,252,56,14,29,58,
132,155,111,190,25,108,45,190,251,123,190,39,25,50,134,174,165,106,3,24,6,233,105,48,165,65,202,79,174,39,238,162,76,28,130,109,92,75,81,181,70,28,122,196,49,136,80,1,145,110,195,171,215,216,130,183,152,
171,252,7,127,240,7,248,131,63,248,131,132,185,84,128,242,0,77,128,178,48,113,3,62,148,91,160,167,192,121,247,80,201,137,32,147,12,51,237,68,90,149,226,102,192,138,104,130,173,137,87,194,171,253,69,27,
110,165,98,85,161,192,20,4,139,186,202,113,108,13,2,82,179,174,114,243,168,210,172,171,220,179,152,171,76,190,91,31,128,89,214,85,94,221,130,142,218,192,69,144,97,117,12,229,3,100,25,58,42,35,8,178,110,
87,133,188,214,2,188,226,228,36,16,27,40,246,78,167,168,156,38,142,137,97,1,99,46,183,61,108,218,174,121,20,177,117,93,229,182,209,143,53,48,54,130,142,234,200,166,224,150,82,237,236,78,8,219,42,216,84,
147,223,143,159,243,168,214,207,208,32,54,48,160,83,171,217,85,222,166,174,242,236,76,50,106,212,193,136,16,164,8,224,69,100,112,14,1,14,241,214,183,222,247,132,66,180,135,220,142,203,212,233,243,251,
47,46,115,50,124,217,128,183,137,171,236,190,107,173,35,52,143,176,192,26,183,137,106,227,182,15,13,88,19,22,139,213,199,21,153,131,73,60,79,212,111,254,86,179,17,25,188,109,108,181,149,93,101,70,28,71,
200,231,179,110,79,111,206,108,115,206,199,83,229,153,243,25,128,111,72,26,237,23,215,116,27,192,18,223,245,54,115,149,121,201,143,123,239,125,227,23,61,165,223,7,113,149,215,54,18,17,87,121,133,9,198,
122,46,118,205,18,87,121,153,86,42,174,178,184,202,171,10,88,92,101,113,149,177,166,174,65,92,101,113,149,197,85,222,236,128,231,94,236,196,85,238,72,192,226,42,139,171,44,174,178,184,202,205,18,87,89,
92,101,113,149,197,85,94,123,31,60,127,83,84,92,229,85,150,184,202,16,87,249,154,46,113,149,95,231,18,87,249,90,15,56,9,89,92,229,77,28,69,72,73,192,18,176,4,44,37,1,75,192,82,18,176,4,44,1,75,73,192,
18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,73,192,87,183,196,85,222,46,1,139,171,220,249,64,197,85,222,236,96,197,85,238,112,168,226,42,111,78,184,226,42,119,178,213,138,171,220,193,86,43,174,178,184,
202,226,42,139,171,12,113,149,151,43,113,149,197,85,110,206,124,197,85,22,87,121,19,187,8,113,149,59,216,130,197,85,238,240,40,66,92,229,37,170,119,223,71,22,180,98,113,149,197,85,22,87,89,92,229,77,8,
88,92,101,113,149,197,85,22,87,121,169,18,87,89,92,101,113,149,197,85,94,226,226,38,174,178,184,202,184,90,174,178,47,174,114,39,93,229,52,60,113,149,215,231,42,147,242,219,92,101,94,194,85,78,89,43,174,
242,250,92,229,249,173,56,217,85,157,117,149,85,95,119,67,103,167,128,20,2,113,149,215,232,42,235,43,137,171,60,87,83,112,11,161,6,32,139,158,129,238,190,82,35,55,110,225,67,121,52,208,151,155,185,209,
245,195,226,42,99,81,87,217,134,128,169,35,237,107,248,20,162,54,62,211,122,222,113,51,167,214,222,32,27,52,116,42,93,170,165,139,76,41,102,10,84,33,83,189,39,153,209,42,113,149,23,115,149,109,3,156,92,
100,51,190,65,125,252,24,192,222,172,223,217,122,216,127,178,241,154,34,197,94,38,21,217,116,137,225,33,229,235,91,1,242,93,54,228,173,245,34,183,69,93,229,100,87,163,57,3,132,107,201,105,223,36,226,161,
151,12,215,212,220,93,193,100,87,187,111,112,116,40,50,65,5,240,225,121,124,136,136,83,110,227,65,92,101,55,140,51,201,69,213,26,247,98,219,8,196,33,2,207,192,83,49,76,93,205,74,10,109,93,69,243,191,175,
199,93,221,177,241,106,12,5,69,40,244,229,166,6,157,215,174,212,114,19,142,109,231,42,3,198,41,91,112,99,100,133,16,62,34,68,229,114,178,110,210,22,50,205,98,88,94,144,74,199,198,107,52,127,214,174,116,
105,23,152,137,20,169,68,7,95,245,69,110,149,174,242,252,19,55,243,92,229,210,218,92,229,149,90,238,198,92,229,217,153,31,37,33,187,23,222,117,19,30,133,48,213,87,146,195,42,65,178,16,228,207,238,21,130,
192,126,224,91,67,245,230,21,61,240,162,30,144,162,182,97,45,173,109,185,146,45,37,253,247,2,87,89,121,80,118,5,87,185,220,230,42,171,215,217,85,6,128,119,191,251,221,24,30,25,69,177,84,198,133,75,19,
120,226,169,151,65,20,130,109,3,100,9,186,122,18,68,247,56,167,80,81,50,113,34,40,229,1,68,240,124,69,12,219,96,88,102,88,242,149,206,39,87,248,245,239,104,108,21,87,249,222,123,239,197,193,131,7,144,
205,229,177,123,247,110,220,118,123,26,199,159,120,30,80,97,50,249,96,176,190,224,128,106,5,48,83,194,180,163,117,140,0,202,35,176,169,19,107,67,48,202,83,177,239,250,27,24,40,226,57,143,150,236,244,1,
236,107,205,85,222,183,111,31,50,153,12,250,251,251,49,48,48,136,29,253,3,120,195,93,55,227,233,103,79,131,227,170,27,27,107,134,34,31,204,4,75,42,105,24,201,209,2,119,139,8,19,199,21,133,136,137,99,16,
224,174,218,68,188,110,184,122,171,184,202,165,82,9,135,14,29,194,208,224,0,246,31,56,128,129,129,65,188,122,242,149,100,58,172,192,90,1,218,130,207,189,12,246,178,32,47,11,80,6,160,52,64,129,251,190,
13,194,128,26,21,197,117,231,42,51,85,1,68,0,76,155,109,207,171,15,120,11,185,202,159,249,204,103,176,103,207,30,220,121,231,157,24,216,185,19,255,226,7,255,37,74,51,211,96,149,2,41,130,141,13,124,142,
128,153,223,4,169,172,11,86,37,223,151,213,0,71,208,90,95,78,245,238,106,40,110,40,112,4,192,155,76,182,87,76,66,96,172,186,5,111,73,87,249,99,31,251,24,62,246,177,143,181,232,97,11,5,47,89,255,211,141,
26,252,116,30,208,51,32,85,119,200,107,235,208,161,59,192,66,214,63,159,81,37,192,134,164,56,6,56,117,62,185,147,201,144,31,172,157,94,111,185,202,170,233,42,23,91,174,114,24,71,145,66,16,180,92,101,44,
116,149,251,178,179,174,178,106,186,202,186,177,100,55,177,154,190,118,53,47,192,202,60,79,115,134,150,156,143,208,117,196,81,9,249,66,26,164,75,128,106,238,114,52,3,182,110,246,105,115,47,164,84,49,175,
216,173,4,54,116,246,20,27,29,3,48,205,135,78,175,101,38,183,133,93,101,158,221,165,102,141,48,170,1,182,129,180,167,103,183,152,116,201,237,128,152,34,96,202,128,41,227,174,59,111,127,60,69,213,33,226,
8,128,158,62,51,126,244,28,27,99,86,218,186,223,166,174,50,181,182,152,116,220,0,145,3,110,40,89,101,131,173,187,29,234,214,71,61,28,159,138,31,85,212,216,207,208,0,155,39,66,147,137,219,142,80,173,41,
224,121,99,171,173,234,42,187,92,226,40,66,62,155,129,130,109,29,9,112,27,2,201,135,13,1,232,103,108,227,180,34,214,135,137,45,192,248,82,91,203,93,23,189,190,77,92,229,16,90,199,200,229,50,73,95,107,
103,119,180,109,242,193,6,247,220,253,134,99,57,191,248,94,119,44,136,13,51,30,44,220,255,128,221,136,12,190,141,92,101,90,112,20,96,222,71,237,248,227,95,121,136,200,126,32,105,176,167,149,50,39,214,
116,35,203,162,179,177,109,235,42,207,85,193,123,122,186,47,221,122,235,205,49,192,226,42,175,118,107,73,92,101,113,149,197,85,222,148,128,91,227,97,113,149,197,85,22,87,25,226,42,175,124,177,19,87,185,
35,1,139,171,44,174,178,184,202,226,42,55,75,92,229,245,187,202,192,194,71,26,136,171,188,137,174,242,98,1,139,171,44,174,114,103,75,92,229,171,19,178,184,202,215,106,109,118,31,60,191,37,139,171,220,
129,46,98,78,137,171,220,225,128,147,144,197,85,222,196,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,124,117,75,92,229,237,18,176,
184,202,157,15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,139,171,44,174,178,184,202,16,87,121,185,18,87,121,19,93,101,96,227,119,25,45,27,46,182,
185,171,188,88,192,226,42,139,171,124,253,184,202,155,213,130,197,85,238,240,40,66,92,229,14,6,44,174,178,184,202,226,42,139,171,188,108,215,32,174,178,184,202,226,42,139,171,60,111,188,41,174,178,184,
202,226,42,139,171,188,248,197,77,92,101,113,149,33,174,178,184,202,139,119,15,205,134,44,174,178,184,202,243,190,70,92,229,107,196,85,230,37,93,101,183,232,41,174,242,250,93,101,91,71,218,143,17,80,212,
230,42,47,100,131,154,174,114,113,41,87,217,24,37,174,242,98,174,114,92,3,113,132,180,111,80,31,255,162,163,242,200,91,128,174,52,93,101,180,92,101,63,113,149,17,128,121,69,198,96,27,187,202,110,188,156,
246,13,96,67,176,246,146,14,174,217,77,36,251,38,236,214,66,250,6,71,71,34,19,84,1,15,158,135,67,138,224,188,10,34,111,61,236,239,150,119,149,221,132,38,66,224,27,248,74,195,132,73,111,216,214,77,112,
219,255,234,113,161,59,210,65,131,225,67,41,42,244,228,166,7,147,229,91,90,43,251,155,116,65,91,215,85,118,70,81,4,107,67,16,52,60,210,136,203,149,217,189,192,230,57,139,214,200,143,161,130,84,42,54,126,
196,164,192,80,232,74,87,70,221,155,150,20,243,218,250,224,45,239,42,51,116,210,85,68,32,142,225,145,134,174,157,110,109,241,55,169,138,102,60,12,2,2,207,51,150,52,160,24,80,8,252,120,7,136,20,145,82,
228,7,148,220,71,178,122,25,188,237,158,136,5,174,50,121,80,188,130,171,220,104,115,149,241,186,187,202,237,211,106,39,110,177,118,239,44,216,8,138,35,152,198,249,230,27,54,241,64,252,182,35,89,4,229,
41,178,32,205,201,144,201,35,93,72,200,218,245,239,104,108,21,87,89,41,133,251,239,191,31,163,163,163,152,46,150,241,248,241,19,152,42,186,214,75,136,65,136,220,210,105,178,118,13,226,228,208,55,181,57,
77,68,201,90,172,5,172,82,132,160,181,139,186,145,229,202,245,212,82,174,114,123,223,187,146,171,188,124,139,92,126,182,55,191,43,185,239,190,251,112,243,205,55,97,112,104,8,119,222,113,7,190,239,123,
191,19,204,6,4,13,107,26,128,109,0,166,222,178,237,149,10,160,90,107,198,126,235,124,6,193,70,4,205,196,6,4,235,142,127,186,97,81,243,190,145,181,181,224,173,226,42,31,58,116,8,153,76,6,59,7,6,28,92,189,
99,0,7,246,237,198,153,11,211,110,210,97,8,192,0,8,30,72,49,20,83,155,170,158,108,75,89,197,30,226,58,33,242,220,104,72,213,29,24,64,12,163,221,226,199,118,117,149,173,181,200,229,114,24,28,28,192,129,
4,174,190,112,225,44,88,7,128,159,130,213,49,188,84,63,216,120,32,223,161,192,60,79,6,183,154,76,138,26,161,226,144,136,35,48,55,93,101,24,248,158,53,211,211,171,166,215,183,156,171,252,224,131,15,226,
219,190,237,219,48,60,52,132,161,161,33,252,139,31,252,33,132,245,42,200,207,3,182,14,221,96,248,120,24,244,242,171,96,175,15,8,246,130,82,35,128,63,8,120,67,238,103,136,227,241,148,170,24,197,117,34,
142,96,108,254,18,54,34,131,111,37,87,249,210,165,75,248,197,95,252,69,252,194,47,124,216,53,8,229,14,105,131,99,176,110,64,71,117,228,243,25,7,87,219,16,48,83,64,152,168,146,220,148,193,11,79,102,6,130,
64,113,3,224,8,245,40,119,170,41,131,47,103,219,111,51,87,57,153,213,233,216,29,173,178,17,162,70,9,172,43,8,84,195,109,190,234,105,80,60,9,196,19,64,60,1,138,167,64,241,52,14,236,223,251,100,134,138,
221,176,13,128,227,226,229,210,222,51,0,34,102,54,42,157,89,114,251,126,27,186,202,14,73,85,228,38,28,58,118,139,255,138,235,32,93,118,88,181,158,1,233,41,144,158,2,244,52,96,138,241,248,20,127,57,160,
242,136,226,16,100,245,83,134,189,144,141,142,97,173,209,147,19,107,150,193,103,191,195,45,231,42,155,217,217,39,27,68,81,29,249,140,130,66,8,182,85,64,87,90,26,120,243,131,77,253,185,192,156,11,21,215,
15,16,199,176,204,95,98,29,135,235,149,193,155,223,245,22,118,149,221,234,90,20,54,160,117,3,217,92,202,173,250,37,235,196,110,55,58,81,193,109,13,119,222,113,203,195,59,50,231,222,2,196,196,208,134,45,
62,7,64,111,68,6,223,6,174,178,65,189,94,67,243,132,27,172,110,147,193,219,116,112,27,213,158,124,242,201,135,2,85,125,47,177,1,177,125,205,176,127,98,195,50,248,246,112,149,99,244,245,245,184,45,255,
150,12,174,231,232,224,61,221,133,203,183,220,184,191,65,176,111,76,22,170,30,86,28,86,55,44,131,183,159,237,217,250,174,50,183,29,102,153,43,132,151,203,165,135,234,213,226,219,0,78,37,171,67,159,123,
182,231,63,212,86,125,125,88,118,177,103,91,186,202,115,100,240,250,141,55,30,253,35,165,240,45,201,31,138,171,188,82,171,22,87,89,92,101,113,149,55,51,96,113,149,197,85,94,33,41,113,149,197,85,22,87,
89,92,101,113,149,197,85,94,69,192,128,184,202,107,14,88,92,101,113,149,59,91,226,42,95,157,144,197,85,190,86,75,92,229,215,185,196,85,190,214,3,78,66,22,87,121,19,71,17,82,18,176,4,44,1,75,73,192,18,
176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,4,44,37,1,75,192,82,18,240,213,45,113,149,183,75,192,226,42,119,62,80,113,149,55,59,88,113,149,59,28,170,184,202,155,19,174,184,202,157,108,181,
226,42,119,176,213,138,171,44,174,178,184,202,226,42,67,92,229,229,74,92,101,113,149,155,51,95,113,149,197,85,222,196,46,66,92,229,14,182,96,113,149,59,60,138,16,87,185,131,1,139,171,44,174,178,184,202,
226,42,47,219,53,136,171,44,174,178,184,202,226,42,207,27,111,138,171,44,174,178,184,202,226,42,47,126,113,19,87,89,92,101,136,171,44,174,242,226,221,67,179,33,139,171,44,174,242,188,175,17,87,89,92,101,
113,149,197,85,190,218,174,114,90,92,229,77,118,149,121,174,171,60,28,153,84,21,240,225,123,56,44,174,50,86,225,42,35,66,224,91,248,202,204,115,149,103,71,36,110,216,214,114,149,67,134,7,82,148,239,201,
77,15,65,92,229,249,63,211,60,87,217,204,119,149,213,156,93,108,71,118,186,144,85,144,74,197,214,143,153,60,48,124,20,82,229,209,100,187,90,92,229,217,214,222,28,35,152,36,104,13,226,24,190,178,208,245,
51,173,159,97,118,23,155,91,155,171,206,85,86,134,225,49,131,16,248,122,71,178,55,233,137,171,156,76,165,103,151,61,141,91,56,130,78,206,25,107,216,240,210,236,69,174,133,17,206,182,102,242,160,140,165,
230,49,90,120,202,118,37,77,126,253,203,149,175,135,171,188,210,176,172,253,215,235,227,121,92,215,70,108,93,11,78,22,244,89,207,36,23,185,89,126,189,117,254,13,4,114,183,135,180,222,130,138,56,157,100,
4,44,211,69,116,228,0,246,82,174,50,181,245,189,43,185,202,203,245,207,203,181,232,197,186,146,124,62,143,187,238,122,3,134,134,134,112,254,226,21,60,126,252,89,196,214,81,156,214,70,80,28,131,88,183,
78,2,41,242,221,97,44,66,107,134,231,174,247,205,77,60,11,2,89,119,169,178,137,224,107,214,30,240,86,113,149,223,253,238,119,99,96,96,0,217,92,30,135,14,31,197,45,183,222,142,223,248,205,63,118,23,96,
229,236,56,162,148,235,129,149,239,182,243,185,121,209,118,231,219,200,18,43,88,77,48,138,96,97,217,107,0,176,68,196,204,188,142,46,98,129,171,108,65,104,115,149,91,247,77,96,193,137,155,37,93,101,106,
255,103,105,78,75,94,215,233,195,85,2,81,3,3,3,200,229,114,24,24,24,192,193,131,7,240,158,175,121,39,130,128,96,19,152,207,154,6,188,244,104,235,28,155,151,108,18,52,61,16,34,5,214,196,158,138,99,197,
49,17,107,88,86,69,183,210,191,188,12,190,40,154,106,43,21,203,58,110,178,226,11,92,101,196,214,184,11,1,181,253,21,115,93,101,111,49,87,89,249,173,175,95,206,85,94,205,231,171,229,121,148,82,168,215,
235,40,20,10,24,25,25,198,161,67,7,113,248,240,65,52,234,213,100,113,63,130,137,202,8,178,119,56,25,188,121,88,80,37,135,5,155,8,86,204,141,148,10,45,33,36,32,134,54,116,217,237,143,193,128,173,49,197,
226,162,103,108,151,252,46,91,174,50,55,93,101,221,114,149,109,28,69,212,60,156,71,152,51,189,108,186,202,202,155,117,149,209,116,149,237,252,110,98,165,171,255,202,99,228,149,186,12,173,53,94,122,233,
37,0,192,208,224,32,250,251,251,241,193,239,255,23,0,12,200,198,176,186,6,29,22,17,196,47,128,167,61,160,238,131,172,187,171,172,105,219,43,242,96,35,51,157,86,85,207,173,8,70,168,69,133,215,64,20,178,
251,134,150,124,188,175,191,228,220,114,129,171,124,174,74,158,201,55,93,229,238,222,145,124,243,106,219,218,38,106,119,149,115,206,85,110,196,137,171,28,123,110,139,189,57,152,231,38,152,170,151,29,29,
204,255,245,122,96,169,199,30,123,12,143,62,250,40,0,66,42,149,65,35,210,32,74,129,57,66,220,176,96,13,4,245,63,7,93,126,16,164,114,9,147,57,8,100,142,2,254,13,64,42,135,189,3,149,103,226,203,165,81,183,
220,170,75,19,213,161,115,96,142,65,164,91,183,241,174,82,6,103,0,196,113,108,41,72,37,174,50,133,218,224,164,242,244,29,137,171,124,10,216,61,178,152,171,76,243,92,229,72,101,192,118,214,85,246,84,0,
171,77,219,254,154,183,236,130,250,90,41,246,229,253,79,130,214,49,20,193,29,105,229,24,58,106,0,150,64,182,2,138,25,240,26,128,74,187,93,233,240,37,0,159,0,217,80,159,153,184,239,179,7,186,139,63,172,
56,132,181,250,89,107,169,10,182,49,216,154,165,250,223,149,135,105,214,90,248,126,203,85,206,166,244,29,115,92,101,95,41,144,74,90,35,47,112,149,179,233,9,88,149,1,33,5,69,25,40,47,134,213,177,27,181,
43,36,147,144,217,245,224,206,187,202,156,124,174,192,100,65,86,35,142,106,200,165,243,80,92,5,140,118,251,132,42,152,115,109,177,150,78,121,193,116,209,167,202,40,113,8,107,213,163,176,54,100,107,99,
54,198,36,119,253,219,181,141,34,182,186,171,108,13,194,168,6,29,85,145,205,32,113,148,155,50,248,244,156,143,27,143,30,120,108,56,127,234,102,178,117,34,142,76,172,189,7,65,20,130,160,19,118,125,205,
195,180,45,236,42,115,107,147,180,94,171,128,200,192,131,118,96,181,113,219,73,115,117,240,74,253,196,137,23,31,202,122,83,15,16,71,0,199,103,235,81,246,25,182,166,157,93,183,107,30,7,111,109,87,153,161,
117,12,29,71,232,235,233,6,145,117,187,29,28,129,108,195,29,128,73,100,240,158,66,106,236,232,193,254,162,135,250,29,64,12,182,230,225,216,164,42,237,30,200,186,90,112,251,217,158,173,233,42,215,224,251,
94,219,121,10,227,174,89,28,205,10,225,28,163,84,46,126,193,132,151,239,35,54,41,98,163,1,126,240,149,83,187,75,109,192,213,250,100,240,100,87,99,139,187,202,221,11,78,10,53,79,9,37,167,139,234,71,15,
31,250,35,95,233,111,76,118,97,78,48,211,211,112,119,28,173,106,57,79,92,229,57,231,41,120,206,217,138,174,174,194,159,70,97,117,31,192,35,73,234,95,246,148,126,105,45,119,26,137,171,188,212,60,75,209,
184,239,171,223,1,236,55,67,92,101,113,149,1,136,171,188,242,197,78,92,229,142,4,44,174,178,184,202,226,42,139,171,220,44,113,149,197,85,22,87,89,92,229,181,247,193,243,55,69,197,85,94,101,137,171,12,
113,149,175,233,18,87,249,117,46,113,149,175,245,128,147,144,197,85,222,196,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,124,117,
75,92,229,237,18,176,184,202,157,15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,139,171,44,174,178,184,202,16,87,121,185,18,87,89,92,229,230,204,
87,92,101,113,149,55,177,139,16,87,185,131,45,88,92,229,14,143,34,196,85,238,96,192,226,42,139,171,44,174,178,184,202,203,118,13,226,42,139,171,44,174,178,184,202,243,198,155,226,42,139,171,44,174,178,
184,202,139,95,220,196,85,22,87,25,226,42,139,171,188,120,247,208,108,200,226,42,139,171,60,239,107,196,85,22,87,89,92,101,113,149,197,85,22,87,89,92,229,69,92,229,185,116,155,184,202,107,114,149,129,
89,87,89,195,154,200,185,202,202,32,174,84,91,108,241,156,17,159,184,202,107,225,121,208,230,42,187,157,14,98,13,159,24,186,118,166,117,74,8,171,114,149,77,63,196,85,94,202,85,118,51,63,78,134,139,68,
6,54,186,60,199,81,158,141,202,125,61,121,80,134,147,99,180,164,224,50,73,80,83,94,231,40,98,107,186,202,104,217,69,110,177,136,221,162,190,169,180,8,77,162,185,93,42,39,215,151,228,147,166,171,28,128,
153,154,247,141,172,123,185,114,61,181,148,171,220,222,247,174,228,42,47,223,34,151,159,237,45,63,243,115,11,71,196,236,176,76,163,65,100,147,157,19,114,112,106,115,194,148,140,141,104,110,115,222,188,
227,171,91,197,85,222,179,103,15,110,190,249,22,100,115,5,76,205,148,240,224,231,191,60,119,81,137,45,72,101,147,179,25,212,182,196,64,179,39,56,45,152,192,12,88,229,80,66,63,4,17,179,49,188,62,184,122,
129,171,172,65,212,230,42,251,148,112,147,88,112,226,102,73,87,185,213,6,84,171,31,190,26,174,242,123,223,251,94,0,64,54,155,199,209,163,55,0,236,227,243,95,124,50,233,147,221,46,134,151,29,6,145,107,
193,138,188,230,113,194,100,71,198,131,213,96,69,198,42,24,34,199,80,150,192,172,215,3,87,111,41,87,249,158,123,238,1,0,116,119,119,99,104,120,8,135,14,29,196,119,126,215,63,113,93,67,178,6,109,227,16,
126,230,168,235,24,148,223,34,127,61,114,162,13,193,3,180,213,129,138,65,9,92,29,27,127,98,118,81,197,216,228,174,211,237,231,42,63,253,244,211,80,74,161,167,167,7,163,163,163,184,225,134,35,168,86,42,
201,73,31,13,54,33,116,92,69,170,107,71,235,254,199,166,12,174,84,170,197,5,113,164,43,129,23,42,66,12,64,163,26,229,207,128,57,102,203,214,205,253,193,107,185,200,45,226,42,115,149,96,208,116,149,169,
121,178,28,11,23,72,42,177,5,200,185,202,172,18,87,153,147,123,9,73,181,237,8,211,162,173,119,51,93,229,40,138,240,226,139,47,34,147,201,96,100,120,24,93,93,93,248,190,239,253,254,100,153,53,70,20,86,
0,93,131,151,153,61,234,234,182,186,124,40,229,39,45,218,195,200,142,234,249,172,87,205,36,174,114,101,170,218,127,17,64,12,102,179,237,93,229,99,199,142,225,161,135,190,128,125,251,246,227,204,217,243,
80,202,75,216,223,8,38,114,43,109,116,250,191,0,93,239,7,229,239,0,178,22,72,49,72,37,199,3,172,135,75,103,79,31,223,147,175,124,131,226,16,198,216,231,153,169,14,182,49,136,116,34,131,175,227,92,196,
22,115,149,207,156,57,3,107,57,121,223,186,77,210,56,172,35,147,245,221,86,127,229,175,128,218,231,28,92,13,15,200,220,3,100,222,4,246,184,162,195,241,241,160,80,26,32,110,192,88,117,28,160,58,136,98,
24,99,151,131,171,183,153,171,60,59,225,33,107,16,135,21,232,168,130,76,160,65,186,8,138,167,65,241,20,40,158,0,233,9,80,249,211,160,43,63,142,61,193,255,249,242,80,238,194,168,226,26,17,135,166,30,165,
142,193,154,186,187,63,218,91,246,132,207,246,116,149,173,129,177,49,234,181,50,8,237,59,207,51,128,158,74,66,158,4,233,73,144,158,9,79,190,122,254,145,238,96,236,14,226,58,96,163,243,229,122,207,51,0,
34,54,58,110,59,211,181,166,128,183,168,171,60,59,185,208,113,3,113,220,64,95,119,14,138,67,7,86,155,10,160,219,101,240,25,116,101,237,196,238,161,236,120,64,229,27,149,141,0,107,30,141,141,55,3,112,196,
198,172,104,43,111,67,87,185,169,130,215,225,123,0,41,3,226,8,176,13,23,114,243,176,138,118,31,229,210,244,151,210,246,204,173,100,195,128,56,212,198,242,231,198,171,187,139,54,138,226,182,115,194,235,
232,131,177,149,93,229,58,172,213,232,233,238,130,2,187,163,87,28,187,221,23,27,182,116,112,178,245,250,190,189,7,254,56,237,85,190,22,136,193,48,47,48,243,83,171,13,119,165,128,183,184,171,156,107,109,
239,183,86,215,172,78,14,47,70,96,142,144,43,228,62,206,122,124,132,88,15,18,27,13,203,143,132,58,247,226,90,144,171,109,238,42,243,156,115,110,179,50,184,129,34,30,247,21,253,142,71,209,7,0,155,7,108,
5,176,127,254,212,11,183,86,11,247,63,96,87,123,143,134,184,202,139,92,0,147,182,245,241,190,222,174,97,128,31,112,3,125,62,246,100,254,39,254,118,173,247,201,137,171,188,120,53,62,240,129,247,255,46,
145,253,33,136,171,188,66,82,226,42,139,171,44,174,178,184,202,226,42,139,171,188,138,128,1,113,149,215,28,176,184,202,226,42,119,182,196,85,190,58,33,139,171,124,173,150,184,202,175,115,137,171,124,173,
7,156,132,44,174,242,38,142,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,224,171,91,226,42,111,151,128,197,85,238,124,160,226,42,
111,118,176,226,42,119,56,84,113,149,55,39,92,113,149,59,217,106,197,85,238,96,171,21,87,89,92,101,113,149,197,85,134,184,202,203,149,184,202,226,42,55,103,190,226,42,139,171,188,137,93,132,184,202,29,
108,193,226,42,119,120,20,33,174,114,7,3,22,87,89,92,101,113,149,197,85,94,182,107,16,87,89,92,101,113,149,197,85,158,55,222,20,87,89,92,101,113,149,197,85,94,252,226,38,174,178,184,202,16,87,89,92,229,
197,187,135,102,67,22,87,89,92,229,121,95,35,174,178,184,202,226,42,139,171,44,174,178,184,202,226,42,139,171,252,250,185,202,188,132,171,236,139,171,188,212,231,74,181,236,47,180,92,101,152,85,187,202,
218,42,155,156,244,20,87,121,126,55,225,92,101,59,207,85,182,137,171,108,97,163,177,5,59,215,152,237,137,65,30,148,181,243,92,229,38,134,39,174,178,154,55,211,75,214,45,96,224,76,62,118,235,21,173,117,
102,204,237,42,230,92,190,148,77,206,26,164,0,136,171,188,120,23,227,166,228,138,220,59,129,218,215,239,231,28,38,90,120,237,90,235,153,214,229,23,123,220,221,60,139,187,202,203,212,124,87,25,139,186,
202,115,91,110,39,93,229,69,71,109,243,126,65,94,190,173,147,107,39,216,147,95,187,203,12,19,172,179,142,25,145,27,11,216,117,110,25,109,33,87,249,158,123,238,193,192,192,32,130,84,6,39,79,158,198,51,
207,157,74,206,71,216,102,67,130,151,29,106,133,219,156,242,183,31,145,101,13,235,57,65,156,192,22,198,250,69,52,181,234,53,194,213,108,43,21,171,10,5,166,32,88,212,85,142,99,107,16,144,154,117,149,155,
71,149,102,93,229,158,197,92,101,242,147,183,167,89,214,85,94,11,146,186,82,240,247,220,115,15,238,188,243,78,40,165,144,205,230,113,211,205,183,98,98,250,143,112,225,114,5,110,66,233,102,146,126,246,
224,172,178,155,0,132,4,154,21,198,98,107,60,101,136,96,136,96,160,77,48,217,92,84,97,29,243,182,117,149,155,191,215,221,221,141,209,209,81,28,57,114,8,183,223,118,203,236,174,8,27,88,19,33,200,119,39,
129,206,158,47,86,45,97,204,131,141,163,122,74,69,62,177,203,161,28,22,46,36,219,31,150,72,173,67,68,220,34,174,114,161,80,64,38,147,65,127,127,63,246,237,219,139,155,110,188,161,205,145,51,136,162,58,
216,70,240,178,205,181,20,31,78,22,115,30,147,219,204,81,216,217,85,153,76,123,141,180,91,83,225,74,177,190,227,34,128,152,148,90,51,92,61,235,200,185,229,178,216,50,66,109,112,18,208,72,92,229,203,238,
38,204,133,174,50,207,115,149,149,202,128,105,214,85,86,94,128,38,115,166,148,130,231,121,173,143,246,69,155,249,139,56,75,77,149,87,106,213,15,62,248,32,206,156,57,131,254,254,126,140,142,142,34,12,27,
248,155,207,252,29,84,114,175,157,137,27,32,27,66,37,75,47,74,185,131,44,42,241,152,154,55,154,142,93,56,255,114,202,107,20,136,67,104,205,47,91,203,117,0,154,177,118,184,122,238,79,177,5,92,229,143,127,
252,227,248,248,199,255,2,123,246,236,195,153,115,23,0,229,186,47,112,236,216,223,238,187,64,85,15,156,231,4,105,36,119,88,222,13,47,128,6,105,19,149,102,82,170,220,71,54,68,108,212,211,96,14,221,25,5,
152,229,224,234,109,229,42,159,59,119,206,109,107,89,11,88,157,184,202,101,100,248,4,232,252,79,131,94,253,85,208,229,11,64,37,0,89,191,181,173,191,127,216,76,239,204,77,246,121,92,39,197,145,173,71,153,
71,0,56,87,153,173,73,238,157,230,181,12,211,102,93,101,102,77,68,225,116,173,247,197,157,221,229,113,229,235,161,238,76,109,160,26,151,166,193,253,89,5,15,12,213,218,213,176,176,160,166,171,12,231,42,
219,168,12,112,29,214,36,150,167,117,187,182,32,130,209,87,203,85,78,54,70,73,1,100,96,109,132,122,173,14,226,12,8,85,192,40,119,130,179,248,167,64,41,112,141,38,251,118,32,247,6,126,249,92,249,233,189,
233,137,131,100,235,96,27,93,154,169,245,158,0,81,8,34,205,113,108,109,173,102,215,220,130,183,172,171,156,172,142,153,168,1,29,85,209,219,21,64,217,90,75,6,71,19,173,142,38,161,138,127,138,92,241,63,
79,246,224,51,79,167,105,102,191,226,58,172,177,143,199,54,152,132,49,17,199,177,78,24,56,96,173,50,120,251,217,158,45,231,42,235,24,181,122,5,190,178,32,68,201,1,149,230,73,160,166,10,62,5,196,83,168,
148,102,158,42,224,204,30,197,53,159,184,161,181,161,47,148,106,61,51,204,28,173,196,174,175,188,216,179,229,92,101,215,77,52,26,117,88,19,161,187,59,11,15,218,157,105,51,53,192,150,65,186,8,152,68,7,
55,197,176,127,96,247,95,228,131,201,55,147,173,3,38,126,73,27,245,36,72,53,64,208,43,177,235,43,5,188,69,93,229,24,97,216,64,62,151,77,208,234,216,237,186,88,119,120,144,109,205,181,102,83,69,46,155,
254,235,180,189,212,227,113,125,167,226,80,91,54,95,153,170,246,63,207,214,196,109,100,144,93,239,98,207,22,118,149,83,200,100,210,109,231,31,154,50,120,152,136,224,33,20,162,9,32,248,221,140,55,243,53,
64,156,99,232,42,44,255,217,249,201,125,69,82,158,94,205,217,224,85,45,87,110,93,87,185,13,173,110,201,224,38,57,173,25,3,172,255,162,191,135,119,40,196,247,19,107,34,182,199,30,61,251,117,127,107,43,
21,163,39,39,120,181,120,245,54,119,149,23,145,193,221,20,186,241,190,175,255,250,223,245,85,244,47,0,155,5,120,194,90,250,105,83,44,154,36,216,85,223,198,37,174,242,130,176,185,4,224,219,159,123,238,
233,111,7,248,13,201,239,253,250,83,133,31,127,108,61,207,144,16,87,121,222,207,234,92,229,131,125,16,87,89,92,101,113,149,33,174,178,184,202,16,87,89,92,229,205,62,23,33,174,242,26,74,92,101,136,171,
124,77,151,184,202,175,115,137,171,124,173,7,156,132,44,174,242,38,142,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,9,88,74,2,150,128,165,36,224,171,91,226,
42,111,151,128,197,85,238,124,160,226,42,111,118,176,226,42,119,56,84,113,149,55,39,92,113,149,59,217,106,197,85,238,96,171,21,87,89,92,101,113,149,197,85,134,184,202,203,149,184,202,226,42,55,103,190,
226,42,139,171,188,137,93,132,184,202,29,108,193,226,42,119,120,20,33,174,114,7,3,22,87,89,92,101,113,149,197,85,94,182,107,16,87,89,92,101,113,149,197,85,158,55,222,20,87,89,92,101,113,149,197,85,94,
252,226,38,174,178,184,202,16,87,89,92,229,197,187,135,102,67,22,87,89,92,229,121,95,35,174,178,184,202,226,42,139,171,44,174,178,184,202,226,42,139,171,44,174,178,184,202,226,42,111,200,85,110,46,94,
137,171,188,30,87,217,206,186,202,182,186,138,125,29,64,92,229,181,240,60,164,96,45,131,136,220,17,130,14,212,246,116,149,91,147,34,204,142,126,218,92,229,57,71,182,154,193,183,126,230,181,185,202,106,
153,254,97,158,171,236,76,225,150,171,220,186,111,2,11,78,220,44,233,42,83,251,63,75,115,90,242,186,90,199,154,217,51,130,203,195,5,103,109,114,88,144,1,63,61,188,68,151,224,78,21,177,129,245,20,59,140,
146,45,140,245,74,152,227,42,175,126,79,110,75,185,202,74,41,188,227,29,239,68,16,4,184,50,49,133,39,159,60,145,4,141,228,200,188,130,53,128,151,219,55,123,244,144,40,153,217,205,30,139,69,108,140,167,
180,231,92,101,70,172,61,113,149,1,224,246,219,111,199,225,195,135,112,228,200,17,188,243,29,239,192,127,252,209,127,223,214,69,16,44,3,134,25,65,62,63,119,60,15,106,29,232,38,82,176,113,20,6,74,251,196,
6,4,141,74,212,117,161,41,196,110,107,87,249,13,111,120,3,50,153,12,118,36,174,242,45,55,223,60,231,2,172,181,6,152,161,210,73,172,68,160,230,253,143,173,163,175,10,59,242,149,153,148,31,167,157,211,103,
43,197,90,223,24,0,179,237,93,229,122,189,142,92,46,135,193,193,65,28,61,114,4,55,221,120,3,210,233,84,107,205,216,232,24,68,6,202,111,158,39,246,91,59,225,77,25,156,89,225,202,197,11,231,211,94,35,15,
196,48,6,39,45,83,3,204,49,91,107,151,115,149,213,138,239,83,135,245,197,145,246,79,16,52,230,184,202,201,113,211,185,45,120,214,85,246,211,89,88,149,129,69,10,160,12,148,151,73,46,44,30,148,231,205,94,
112,128,142,185,202,79,61,245,20,0,96,104,112,16,195,35,195,248,229,95,249,53,68,81,228,198,193,214,64,199,33,210,125,255,0,202,83,173,235,4,37,59,206,158,74,57,35,47,34,107,226,90,37,165,234,61,138,99,
68,218,127,6,224,6,136,154,107,175,235,56,120,210,238,42,19,69,149,70,254,120,46,91,249,135,132,72,245,228,194,158,210,84,20,102,135,188,108,107,129,132,103,103,62,77,87,121,79,226,42,91,157,5,188,6,96,
66,48,5,240,148,5,90,225,242,2,87,121,45,74,237,114,253,182,82,10,47,190,248,34,158,127,254,5,220,126,251,237,184,112,113,12,19,19,69,64,101,0,24,196,113,13,38,6,242,195,35,0,105,119,203,129,2,136,85,
235,98,71,228,97,100,103,84,153,152,168,116,19,55,8,28,218,106,163,251,49,0,49,216,106,144,226,245,192,213,179,174,178,49,26,204,225,116,173,247,69,107,120,92,65,163,59,83,27,240,168,52,13,118,71,80,231,
239,4,112,211,85,54,206,85,134,151,3,146,150,236,121,105,0,190,91,128,34,130,49,87,195,85,102,60,243,204,179,24,31,191,50,59,226,209,33,194,90,25,196,13,208,233,143,0,99,49,16,186,46,65,41,247,65,228,
3,214,199,107,47,87,206,116,167,103,118,41,14,193,70,95,158,169,247,61,231,216,55,24,176,53,203,157,182,220,70,174,242,236,137,120,64,35,214,117,196,81,5,221,121,130,103,47,131,166,126,17,116,238,215,
64,23,158,113,112,181,117,39,64,115,190,199,189,153,137,179,25,175,56,162,184,14,99,240,140,54,254,52,128,168,9,87,47,213,255,46,215,69,112,91,199,58,199,85,78,167,226,119,40,114,174,114,67,31,141,40,
237,165,136,221,5,143,97,231,186,202,112,174,178,242,74,128,201,2,42,11,230,134,115,149,117,99,214,85,102,187,166,86,186,126,87,89,185,203,177,142,209,168,213,224,171,172,59,77,164,125,128,25,196,17,80,
249,28,168,242,5,0,1,208,253,79,80,245,11,23,123,227,175,164,189,116,213,35,110,152,72,167,31,174,68,221,51,96,19,207,131,171,215,177,216,179,165,92,229,100,21,77,199,168,55,170,176,38,68,87,33,5,15,117,
192,148,65,122,26,20,79,1,209,20,40,158,0,197,151,129,201,159,141,179,245,63,250,195,174,96,252,22,197,53,176,141,78,134,113,240,52,172,169,3,216,48,92,189,5,93,101,118,174,114,163,134,124,54,5,133,24,
108,235,109,50,248,12,72,79,129,147,143,84,74,29,203,208,152,239,163,212,75,182,174,173,49,199,39,171,67,39,64,20,178,209,241,134,225,234,214,120,120,203,184,202,33,234,245,26,210,169,20,210,105,31,64,
12,178,113,34,131,55,207,177,21,161,116,17,158,173,78,107,147,249,147,238,96,236,30,101,107,25,197,81,45,142,213,39,166,235,3,51,172,227,144,141,49,171,57,31,188,13,93,101,133,66,33,215,26,7,59,25,60,
2,217,208,157,204,76,116,112,107,227,79,13,116,85,179,1,202,119,43,14,201,90,253,200,87,47,189,237,111,154,225,182,141,28,54,20,240,22,117,149,219,118,99,184,137,85,55,207,33,135,32,219,104,188,245,129,
183,255,159,156,63,245,207,128,56,195,136,39,99,19,252,87,102,27,1,208,107,113,240,182,177,171,60,15,172,110,30,41,176,186,100,45,253,227,115,175,30,255,22,98,125,7,185,123,56,126,227,248,217,119,61,218,
78,174,175,246,54,46,113,149,231,236,45,90,126,231,59,223,241,167,135,14,238,238,81,100,190,51,57,191,241,101,99,233,163,166,88,180,122,114,194,174,150,92,95,83,192,173,86,188,165,93,101,6,192,143,157,
63,127,246,83,204,250,39,0,14,0,158,0,248,195,190,23,159,93,107,176,107,221,147,19,87,121,157,37,174,178,184,202,226,42,139,171,188,201,231,34,196,85,94,67,137,171,12,113,149,175,233,18,87,249,117,46,
113,149,175,245,128,147,144,197,85,222,196,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,124,117,75,92,229,237,18,176,184,202,157,
15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,139,171,44,174,178,184,202,16,87,121,185,18,87,89,92,229,230,204,87,92,101,113,149,55,177,139,16,
87,185,131,45,88,92,229,14,143,34,196,85,238,96,192,226,42,139,171,44,174,178,184,202,203,118,13,226,42,139,171,44,174,178,184,202,243,198,155,226,42,139,171,44,174,178,184,202,139,95,220,196,85,22,87,
25,226,42,139,171,188,120,247,208,108,200,226,42,139,171,60,239,107,196,85,22,87,89,92,101,113,149,197,85,22,87,89,92,101,113,149,197,85,222,146,174,50,181,142,5,136,171,188,66,173,207,85,166,246,28,196,
85,94,105,232,182,94,87,153,23,157,219,138,171,188,234,153,223,172,171,172,196,85,94,186,95,93,11,207,211,246,189,52,71,12,201,199,172,171,188,196,226,130,184,202,203,45,16,113,235,71,117,110,82,98,40,
89,55,122,103,38,248,153,225,57,231,219,102,225,106,213,114,149,149,178,180,86,87,121,81,217,206,86,42,150,117,204,201,172,96,129,171,140,216,26,119,197,165,182,191,98,174,171,236,45,230,42,43,191,245,
245,203,185,202,171,249,124,125,60,15,181,126,108,182,0,37,144,171,101,130,151,221,55,119,169,148,48,123,100,139,8,136,141,241,149,85,107,117,149,151,220,244,108,185,202,170,233,42,23,91,174,114,24,71,
145,66,16,180,92,101,44,116,149,251,178,179,174,178,106,186,202,186,49,175,155,224,53,245,181,171,234,87,231,181,234,190,190,62,220,118,219,237,72,167,51,40,85,170,248,210,35,79,130,136,192,228,129,73,
129,89,193,48,33,147,207,1,92,159,123,70,147,102,15,120,219,56,138,2,165,131,181,186,202,75,139,136,11,92,229,115,85,242,76,190,233,42,119,247,142,228,155,174,114,107,155,168,221,85,206,57,87,185,17,39,
174,114,236,185,45,246,230,130,10,55,53,90,189,236,232,96,254,175,215,218,114,191,245,91,191,21,0,144,205,229,145,78,231,16,4,121,124,254,216,147,110,80,12,15,218,48,0,15,42,221,250,41,90,195,53,55,117,
118,239,190,190,124,165,68,38,74,92,101,78,92,101,54,32,226,109,235,42,223,123,239,189,0,128,66,161,128,161,161,33,28,57,114,24,111,127,231,219,161,60,15,238,14,93,130,177,238,239,39,31,115,86,4,91,86,
41,57,181,241,202,197,139,23,83,94,156,119,108,60,94,181,140,176,185,133,190,109,93,229,87,94,121,5,74,41,244,245,245,97,116,215,40,14,31,62,212,214,138,220,69,76,107,131,84,239,215,37,215,224,230,98,
79,243,208,182,251,64,4,107,117,189,145,82,141,110,114,174,242,179,72,208,212,149,92,229,229,222,111,179,174,50,16,85,26,249,227,196,177,37,142,168,39,23,246,212,167,162,176,233,190,207,95,226,107,186,
202,148,184,202,72,68,90,80,10,76,1,148,231,163,29,235,155,239,42,175,229,130,183,92,191,61,61,61,141,227,199,143,35,151,203,97,120,100,24,61,61,61,248,205,223,252,237,230,209,126,196,113,4,107,98,164,
7,251,231,156,136,87,202,79,206,77,4,80,202,195,80,95,84,239,201,213,187,20,135,164,16,219,74,35,255,56,152,35,182,102,69,87,121,201,62,184,229,42,51,107,34,10,167,107,189,47,238,236,46,143,43,95,15,117,
103,106,3,213,184,52,13,238,207,42,120,96,168,214,174,134,133,5,53,93,101,56,87,217,70,101,128,235,176,38,177,60,173,219,181,5,17,140,238,172,171,124,252,248,113,28,63,254,4,70,71,247,224,252,133,75,96,
149,1,169,60,172,213,8,227,26,40,123,16,148,118,195,54,69,158,59,40,228,134,24,238,215,214,195,185,87,167,199,118,165,203,67,132,8,198,216,43,51,245,222,151,0,68,109,174,178,197,118,119,149,47,92,184,
144,140,115,45,24,6,90,55,96,226,58,10,61,119,129,106,109,221,130,242,221,97,153,68,117,204,248,30,23,130,169,137,180,95,217,73,28,66,107,156,136,76,48,227,2,110,245,191,188,214,128,185,253,108,79,187,
171,172,56,182,10,206,85,38,173,35,119,232,199,107,155,185,181,185,202,236,92,101,246,178,96,213,252,112,235,212,198,208,172,171,188,236,141,46,43,111,39,173,101,107,137,172,133,213,49,26,181,50,20,69,
80,149,143,131,206,253,28,112,241,21,160,230,183,14,203,80,34,131,215,75,166,18,232,23,61,143,107,158,226,208,132,177,255,104,61,204,206,128,217,245,193,203,116,15,43,47,246,108,25,87,217,204,94,31,172,
70,216,168,192,234,6,186,114,10,138,107,128,46,129,202,159,4,93,252,40,232,252,9,80,213,133,12,235,163,96,39,30,239,9,166,246,41,91,131,53,241,107,181,40,253,12,128,186,27,65,44,223,61,172,20,240,22,115,
149,221,186,110,172,35,132,97,13,217,140,7,197,13,119,38,206,148,64,241,52,72,79,1,149,207,128,46,252,191,192,169,207,34,93,140,207,115,241,243,147,1,138,5,197,117,99,180,125,122,170,54,240,60,152,27,
204,28,173,212,61,172,24,112,107,60,188,149,92,229,90,5,233,64,33,147,38,119,116,203,84,1,93,2,18,25,156,154,39,62,107,15,86,170,99,255,223,31,244,102,46,29,245,80,73,131,27,181,122,28,252,117,165,209,
53,233,250,95,107,86,98,215,87,181,92,185,117,92,229,24,245,122,13,158,82,200,231,179,201,94,97,152,200,224,149,54,25,220,5,109,116,116,108,71,190,108,51,52,125,35,217,58,177,137,31,123,113,252,238,191,
1,115,3,224,168,13,174,182,27,9,120,139,185,202,10,221,221,133,214,89,96,119,78,34,108,147,193,203,238,144,160,41,133,55,222,244,134,63,234,77,157,255,70,178,181,52,113,56,93,109,228,127,14,198,212,1,
142,108,20,197,171,189,17,102,27,186,202,220,218,194,130,213,109,50,120,163,41,131,151,181,205,124,176,62,249,216,187,124,174,221,164,56,132,49,252,219,207,141,189,233,17,16,133,237,225,110,22,92,189,
5,93,229,197,100,240,24,176,49,191,237,129,183,124,252,192,158,158,140,79,181,127,4,247,120,159,199,234,113,238,163,204,54,98,29,175,89,113,220,198,174,242,162,50,248,227,231,207,157,253,148,143,242,143,
16,155,128,88,79,176,229,95,60,53,126,243,233,245,168,224,107,217,147,219,194,174,114,171,53,63,222,183,99,231,191,102,219,248,167,0,239,73,238,25,251,35,109,188,207,20,199,201,172,71,5,79,214,60,196,
85,134,184,202,226,42,139,171,188,137,231,34,196,85,94,67,137,171,12,113,149,175,233,18,87,249,117,46,113,149,175,245,128,147,144,197,85,222,196,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,
176,148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,124,117,75,92,229,237,18,176,184,202,157,15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,
42,139,171,44,174,178,184,202,16,87,121,185,18,87,89,92,229,230,204,87,92,101,113,149,55,177,139,16,87,185,131,45,88,92,229,14,143,34,196,85,238,96,192,226,42,139,171,44,174,178,184,202,203,118,13,226,
42,139,171,44,174,178,184,202,243,198,155,226,42,139,171,44,174,178,184,202,139,95,220,196,85,22,87,25,226,42,139,171,188,120,247,208,108,200,226,42,139,171,60,239,107,196,85,22,87,89,92,101,113,149,197,
85,22,87,89,92,101,113,149,197,85,22,87,89,92,101,113,149,151,248,245,82,67,49,113,149,197,85,94,229,98,207,150,119,149,149,184,202,107,233,139,215,234,42,187,99,18,75,185,202,115,143,201,138,171,188,
108,203,157,219,38,9,238,118,60,86,110,232,105,217,131,159,221,215,26,146,205,142,116,212,34,174,178,93,147,171,188,228,119,217,114,149,185,233,42,235,150,171,108,227,40,162,230,225,60,194,130,193,249,
76,157,161,188,89,87,25,77,87,217,46,221,77,172,166,175,93,85,191,186,76,171,102,102,48,121,128,106,122,118,238,70,27,203,30,252,124,118,118,55,38,153,52,53,127,182,185,174,178,94,147,171,188,244,194,
234,2,87,153,171,4,131,166,171,76,205,147,229,88,184,64,82,137,45,64,206,85,102,149,184,202,156,220,75,72,170,109,71,152,22,109,189,155,233,42,223,123,239,189,248,167,255,244,159,226,125,239,127,63,230,
144,235,228,3,110,83,2,80,62,40,197,173,205,207,217,1,103,243,212,60,161,47,95,41,165,252,56,77,48,96,203,85,231,42,99,123,187,202,183,223,126,59,238,188,243,78,116,117,117,225,182,91,111,193,191,253,
119,255,6,202,243,65,94,0,242,82,0,249,48,172,64,94,0,248,201,180,184,21,238,236,173,9,128,135,43,23,47,92,74,121,81,30,208,208,6,39,197,85,182,22,247,222,123,47,50,153,12,118,14,12,96,239,190,125,184,
231,238,187,113,228,240,145,230,4,181,21,112,170,247,29,32,154,123,252,187,121,116,150,200,3,34,182,54,110,68,238,54,177,150,171,28,131,72,175,52,227,217,210,174,242,169,83,167,144,203,229,48,56,56,136,
195,135,14,225,240,145,67,40,150,74,201,46,178,66,108,44,172,37,164,250,123,230,180,91,55,114,240,90,167,227,7,123,163,176,59,215,232,38,142,136,88,219,74,152,63,14,71,191,185,155,207,213,58,250,224,150,
171,108,140,6,115,56,93,235,125,209,26,30,87,208,232,206,212,6,60,42,77,131,221,17,212,249,59,1,220,116,149,141,115,149,225,229,128,164,37,123,94,26,128,239,22,160,136,96,76,231,92,229,231,158,123,14,
141,70,3,3,59,119,98,96,112,16,191,241,177,223,198,229,177,241,100,168,196,8,195,24,20,244,131,130,182,49,114,235,150,131,68,167,101,15,231,78,149,39,187,51,181,1,133,24,198,240,248,116,173,239,69,48,
71,108,140,134,49,54,185,57,157,215,116,116,202,20,139,86,229,114,134,84,106,142,171,28,248,209,80,198,175,239,176,165,241,151,109,60,176,139,130,196,85,230,230,241,38,219,58,142,52,65,135,208,171,46,
57,87,217,214,64,94,22,108,34,88,14,221,102,128,110,119,149,121,205,174,242,74,19,140,177,177,49,252,226,47,126,4,159,248,196,39,49,54,62,137,74,53,118,47,54,0,151,141,70,215,129,111,5,169,24,100,221,
17,175,230,114,80,211,88,78,43,197,190,63,51,157,241,171,183,16,135,136,13,158,139,141,95,118,135,48,160,217,104,99,203,53,187,246,81,68,219,217,158,235,221,85,62,125,250,53,84,42,149,214,117,194,26,141,
122,163,10,223,223,1,149,81,173,139,153,82,126,2,13,58,112,144,200,67,88,49,141,192,156,246,61,110,40,226,200,52,34,255,177,122,148,155,102,230,120,158,171,188,54,25,60,217,213,216,34,174,114,178,70,197,
22,214,196,8,195,42,88,55,144,79,77,128,94,251,43,112,185,217,72,252,150,12,78,228,3,236,35,29,143,191,212,157,42,141,16,26,176,90,159,171,133,217,19,96,14,193,28,181,117,15,235,90,139,216,34,174,50,183,
46,190,128,133,209,17,162,122,5,217,52,185,35,4,141,167,64,151,126,25,116,233,101,80,195,79,14,5,186,70,147,137,109,213,86,158,42,6,84,201,121,182,110,180,225,103,167,106,253,47,2,168,131,92,247,176,210,
17,214,109,226,42,187,73,68,24,53,80,175,151,145,74,17,210,129,117,71,170,116,201,157,238,44,125,6,116,254,163,192,216,12,16,5,128,9,184,58,118,246,145,222,244,196,46,31,149,0,54,108,84,195,244,103,106,
97,126,10,108,99,142,227,21,187,135,85,45,87,110,57,87,153,44,242,185,192,157,139,179,53,119,48,48,158,6,233,73,80,60,9,154,250,95,160,87,127,26,124,233,202,43,121,122,114,50,231,77,237,83,182,74,198,
232,39,79,142,223,252,119,0,234,204,28,81,16,152,213,220,171,177,77,92,229,24,213,106,5,158,2,186,186,10,32,235,208,106,152,26,200,84,0,83,4,226,25,144,110,202,224,211,241,206,252,163,127,180,51,125,234,
94,143,43,41,216,112,166,88,239,254,8,128,10,152,27,238,230,147,214,230,226,134,2,158,123,177,187,222,93,229,174,174,15,186,107,131,118,33,219,122,130,86,39,33,235,105,64,23,171,13,221,253,159,131,198,
211,183,167,168,120,144,108,13,90,155,63,56,57,121,235,35,176,182,54,79,5,23,87,121,214,85,254,139,175,63,91,252,206,223,0,248,223,182,66,230,24,176,141,54,25,188,194,247,190,241,142,79,239,25,98,147,
85,19,239,119,7,6,227,39,102,106,125,191,222,82,193,27,13,189,233,112,117,171,21,95,231,174,114,225,254,7,236,217,226,119,126,24,224,159,106,134,12,110,222,134,208,0,76,248,228,249,243,151,255,38,239,
141,125,80,113,232,19,71,83,218,224,151,207,206,220,248,42,27,221,176,81,180,236,141,225,27,217,147,219,50,174,114,18,242,79,1,252,11,238,192,118,75,6,63,94,232,29,250,55,1,38,254,1,33,30,101,196,134,
217,252,113,185,222,253,153,72,123,17,0,189,30,131,116,155,187,202,223,245,191,0,254,94,0,207,49,210,223,113,240,192,158,127,3,216,191,15,231,42,127,166,174,115,223,243,196,185,247,92,110,195,8,23,132,
43,174,242,138,33,255,179,255,204,80,127,118,240,192,190,15,1,244,205,73,67,122,102,166,210,251,166,175,158,188,43,194,10,15,26,21,87,121,133,154,235,42,115,203,85,126,117,240,95,53,10,131,29,222,182,
95,114,55,95,92,229,85,151,184,202,16,87,249,154,46,113,149,95,231,18,87,249,90,15,184,109,8,35,174,242,38,141,34,164,36,96,9,88,2,150,146,128,37,96,41,9,88,2,150,128,165,36,96,9,88,2,150,146,128,37,96,
9,88,74,2,150,128,165,36,224,171,91,226,42,111,151,128,197,85,238,124,160,226,42,111,118,176,226,42,119,56,84,113,149,55,39,92,113,149,59,217,106,197,85,238,96,171,21,87,89,92,101,113,149,197,85,134,184,
202,203,149,184,202,226,42,55,103,190,226,42,139,171,188,137,93,132,184,202,29,108,193,226,42,119,120,20,33,174,114,7,3,22,87,89,92,101,113,149,197,85,94,182,107,16,87,89,92,101,113,149,197,85,158,55,
222,20,87,89,92,101,113,149,197,85,94,252,226,38,174,178,184,202,16,87,89,92,229,197,187,135,102,67,22,87,89,92,229,121,95,35,174,178,184,202,226,42,139,171,44,174,178,184,202,226,42,139,171,44,174,178,
184,202,226,42,139,171,188,196,175,151,26,138,137,171,44,174,242,42,23,123,196,85,94,164,57,139,171,188,72,171,95,198,85,38,111,17,87,121,225,49,89,113,149,151,109,185,115,127,92,74,134,175,172,218,216,
223,57,174,50,183,190,94,92,229,21,186,140,185,174,178,77,92,101,5,80,0,80,0,134,47,174,242,70,92,229,214,223,193,140,230,253,140,150,189,86,192,218,122,128,10,128,148,109,237,200,204,14,56,55,238,42,
251,75,76,220,137,227,216,82,144,74,92,101,10,181,193,73,229,233,59,18,87,249,20,176,123,100,49,87,153,230,185,202,145,202,128,237,172,171,236,169,0,86,155,182,253,53,111,217,5,245,213,175,1,47,252,154,
251,239,191,31,7,14,28,68,20,69,152,152,154,193,231,30,252,34,72,165,96,85,10,240,210,128,74,193,88,31,202,75,131,124,219,218,176,229,217,121,233,28,87,121,79,79,116,72,92,229,164,246,236,217,131,155,
111,190,25,249,124,14,187,247,236,193,91,223,242,22,188,229,205,247,187,229,149,68,5,103,74,193,176,143,160,239,1,128,146,251,65,184,173,255,77,54,15,16,177,53,45,87,89,35,210,254,9,108,119,87,249,208,
161,67,0,128,238,238,110,140,12,143,224,192,193,131,120,219,59,223,158,92,112,155,221,131,130,101,31,193,142,174,214,202,96,115,219,203,29,173,117,167,227,7,122,163,176,39,23,38,174,114,108,43,141,252,
113,0,209,182,118,149,207,157,59,135,84,42,133,238,158,94,236,217,179,7,71,143,28,193,229,75,87,192,73,87,197,236,33,138,1,74,15,0,254,236,154,10,181,254,231,238,239,0,60,156,63,85,158,238,206,84,7,21,
98,88,195,227,211,181,222,23,193,28,174,198,85,94,242,167,106,26,114,0,230,184,202,64,132,140,95,223,97,235,227,37,27,55,23,169,147,27,225,57,153,187,243,172,171,204,42,227,92,229,164,21,51,165,221,5,
143,20,140,225,57,225,174,213,85,94,174,203,120,245,213,87,113,226,196,115,200,229,178,24,26,30,134,239,251,248,205,255,253,127,146,31,217,131,102,183,105,147,221,251,238,228,13,152,12,199,104,86,168,
37,242,144,241,20,231,252,153,98,198,175,239,32,68,137,171,28,204,113,149,231,8,12,171,60,155,198,109,29,235,28,87,57,157,138,223,161,200,185,202,13,125,52,162,180,151,34,118,93,5,195,206,117,149,225,
92,101,229,149,0,147,5,84,22,204,13,231,42,235,198,172,171,204,118,77,173,116,181,19,140,135,30,122,8,15,61,116,12,125,59,250,49,53,85,118,221,84,50,241,136,226,24,170,235,32,84,186,217,246,220,89,14,
75,201,190,162,114,59,227,141,138,142,50,250,172,175,130,80,17,199,166,17,165,30,175,71,217,105,102,219,220,251,194,82,23,184,149,23,123,182,132,171,204,152,156,154,106,69,96,173,69,20,54,96,13,144,59,
252,1,168,246,219,24,84,114,218,71,5,238,72,44,43,164,194,43,103,122,50,229,33,133,16,90,155,243,181,48,243,28,128,136,8,26,150,77,2,25,174,107,45,98,139,184,202,104,219,55,180,48,58,68,20,213,145,73,
27,208,100,13,108,103,143,25,120,42,72,78,129,186,35,85,169,216,134,182,246,98,37,160,90,150,184,97,141,193,115,147,181,157,47,3,104,176,177,49,124,207,110,115,87,153,219,214,128,45,226,168,129,70,173,
138,192,183,72,249,6,52,245,17,208,153,191,6,170,205,6,146,200,224,228,131,172,143,218,149,115,207,247,102,138,59,61,212,124,216,168,81,109,100,254,174,30,102,103,192,28,129,72,47,55,254,93,245,114,229,
245,239,42,187,159,61,214,49,234,245,10,148,50,200,103,61,119,146,222,150,65,209,11,160,75,255,11,24,47,131,172,235,119,137,2,160,102,74,25,58,57,147,245,102,70,20,215,200,104,243,204,171,147,71,63,15,
230,154,27,162,89,179,18,187,190,154,128,183,142,171,92,41,195,35,70,87,33,211,38,209,86,64,205,35,174,211,191,13,122,237,79,129,146,91,164,26,204,94,254,194,206,236,249,163,62,42,1,76,163,52,85,237,253,
40,152,43,238,94,102,27,115,28,55,187,7,187,145,128,231,94,236,174,75,87,185,142,106,181,194,32,252,109,190,123,199,191,114,27,177,145,59,132,168,221,25,98,232,25,39,131,135,47,0,231,255,83,88,127,249,
209,63,214,227,127,211,155,161,233,81,101,171,8,99,250,248,153,169,131,95,1,115,149,141,141,65,208,43,93,220,214,18,240,22,112,149,233,199,106,61,63,249,254,75,248,190,95,55,236,255,24,108,243,0,74,21,
48,101,144,158,113,50,120,60,197,55,223,120,232,145,129,236,211,19,93,193,229,251,61,174,192,152,232,217,201,202,192,255,6,80,106,170,224,205,214,187,154,187,140,182,137,171,252,241,159,49,197,162,133,
181,230,50,127,247,47,88,246,126,166,121,124,170,117,118,216,20,193,186,250,252,197,11,19,159,235,75,191,246,205,138,171,30,219,250,116,24,251,191,126,185,188,231,21,88,91,99,107,157,67,186,134,251,228,
182,147,171,204,73,159,169,47,209,247,254,20,131,127,201,157,83,107,36,71,106,27,207,248,185,61,31,234,9,206,190,219,231,234,144,178,117,195,198,124,98,178,50,240,105,163,225,200,245,117,56,164,219,202,
85,158,255,208,166,209,212,175,255,42,17,127,55,193,127,49,196,232,247,29,222,155,254,65,79,153,111,4,144,54,172,62,55,81,219,245,3,39,39,223,112,145,117,172,217,24,179,88,184,226,42,47,82,237,15,203,
27,197,47,255,39,67,249,191,56,184,127,240,39,136,248,3,201,162,196,137,147,147,119,61,48,86,222,83,231,48,228,230,131,236,176,14,184,122,91,186,202,237,76,230,174,91,14,255,66,58,136,126,9,208,31,112,
123,70,244,172,177,222,7,199,138,163,21,242,61,54,181,90,243,73,174,235,122,226,201,186,216,223,38,249,139,56,6,124,207,146,242,204,197,153,221,143,102,252,242,207,118,165,75,255,78,41,190,81,177,117,
174,50,108,204,214,198,0,213,1,110,192,218,144,153,227,228,73,34,175,27,253,219,100,50,79,224,129,169,187,107,63,53,158,180,220,103,152,241,163,83,165,29,79,109,150,156,43,174,50,218,93,101,254,83,34,
126,250,236,174,15,234,194,174,205,249,187,197,85,222,96,137,171,252,58,151,184,202,215,122,192,237,23,12,72,173,123,38,39,37,1,75,192,18,176,148,4,44,1,75,73,192,18,176,4,44,37,1,75,192,18,176,148,4,
44,1,75,73,192,18,176,4,44,37,1,191,126,37,174,242,118,9,88,92,229,206,7,42,174,242,102,7,43,174,114,135,67,21,87,121,115,194,21,87,185,147,173,86,92,229,14,182,90,113,149,197,85,22,87,89,92,101,136,171,
188,92,137,171,44,174,114,115,230,43,174,178,184,202,155,216,69,136,171,220,193,22,44,174,114,135,71,17,226,42,119,48,96,113,149,197,85,22,87,89,92,229,101,187,6,113,149,197,85,22,87,89,92,229,121,227,
77,113,149,197,85,22,87,89,92,229,197,47,110,226,42,139,171,12,113,149,197,85,94,188,123,104,54,100,113,149,197,85,158,247,53,226,42,139,171,44,174,178,184,202,226,42,139,171,44,174,178,184,202,226,42,
139,171,44,174,178,184,202,75,252,122,169,161,152,184,202,226,42,175,114,177,71,92,229,69,154,179,184,202,139,180,122,113,149,87,221,98,87,94,81,19,87,121,197,126,118,181,203,149,157,113,149,103,39,77,
226,42,175,170,229,82,50,49,229,230,176,181,205,85,78,45,226,42,183,31,161,221,184,171,172,150,152,184,59,71,206,45,151,197,150,17,106,131,147,128,70,226,42,95,118,55,97,46,116,149,121,158,171,172,84,
6,76,179,174,178,242,130,86,31,172,148,130,231,121,173,143,246,69,155,249,139,56,75,77,149,87,215,170,61,119,54,216,115,199,7,200,75,131,188,140,115,149,57,128,242,51,32,223,182,22,41,151,114,149,83,94,
148,95,171,171,236,175,248,62,245,253,150,171,156,77,233,59,230,184,202,190,82,32,229,78,150,183,86,173,102,93,229,108,122,2,86,101,64,72,65,81,6,202,139,97,117,236,92,101,133,100,18,50,187,30,188,153,
174,242,190,125,251,240,158,247,188,7,0,112,101,124,10,159,248,196,39,221,125,119,240,64,42,5,86,105,176,202,192,216,0,65,239,91,147,238,180,253,150,5,213,218,60,152,227,42,179,184,202,0,128,219,111,191,
29,0,208,213,221,131,219,110,187,21,255,234,135,126,8,80,62,152,2,48,165,1,149,134,182,62,44,210,8,118,20,220,1,148,214,134,43,181,206,38,139,171,188,68,232,67,67,67,40,20,10,24,24,24,192,129,3,7,240,
174,119,189,19,233,76,14,160,20,64,41,88,164,16,25,15,42,61,8,246,77,235,80,56,183,142,123,187,159,13,172,112,254,84,105,186,59,83,19,87,185,189,46,93,186,132,174,174,46,12,13,13,225,240,225,67,24,30,
30,70,163,30,187,115,16,202,145,235,150,3,100,118,191,165,181,86,130,182,112,155,247,118,164,61,133,156,95,44,57,87,57,94,179,171,188,220,246,108,235,108,79,187,171,172,56,182,10,206,85,38,173,35,119,
232,199,107,155,185,181,185,202,236,92,101,246,178,96,213,252,112,235,212,198,208,172,171,188,236,141,46,88,211,80,172,89,127,253,215,127,141,233,233,105,12,14,14,162,80,232,194,119,126,247,15,0,164,192,
228,195,178,66,20,3,42,187,31,148,154,93,13,196,188,112,73,249,8,171,58,242,245,57,95,33,84,196,177,109,68,169,227,206,85,102,113,149,127,233,151,126,25,239,127,255,7,112,215,93,111,194,137,19,47,128,
73,129,25,136,98,3,139,0,153,253,111,75,94,86,90,208,114,21,249,32,246,16,132,227,231,187,51,149,33,197,17,180,54,231,170,13,113,149,219,126,127,225,187,66,27,141,40,214,200,140,254,107,120,62,181,118,
97,144,132,170,84,144,124,164,16,196,38,178,181,87,170,41,85,207,16,55,172,209,120,126,170,182,243,101,215,255,138,171,220,58,114,213,92,166,140,163,8,141,122,29,41,159,16,100,218,90,110,27,90,173,200,
29,179,133,245,80,27,191,112,186,39,83,238,247,184,238,147,141,26,149,48,243,217,230,51,40,196,85,158,179,79,196,208,58,70,163,86,133,71,22,217,52,129,46,254,12,48,97,230,116,9,42,33,215,21,249,64,93,
215,210,124,166,148,243,139,131,196,53,210,218,60,123,106,226,200,23,0,212,193,44,174,114,251,231,177,142,81,171,150,161,148,65,33,31,128,184,14,152,58,104,242,87,128,11,207,129,244,236,13,57,164,220,
147,110,122,253,203,95,221,153,27,219,231,84,240,176,60,89,233,253,24,216,86,192,28,178,53,226,42,55,171,209,104,56,87,153,241,96,182,48,242,163,100,67,23,174,45,187,99,173,149,207,129,206,254,31,80,197,
221,152,67,54,64,120,105,242,171,60,243,88,46,227,205,12,42,91,69,24,169,79,158,157,57,240,21,0,85,0,33,32,174,114,34,213,150,17,134,141,24,240,126,178,218,251,35,223,114,133,190,237,183,98,206,254,119,
36,231,211,154,42,56,69,231,65,23,254,7,112,197,195,173,251,237,84,143,253,226,233,222,244,216,173,202,86,192,58,124,225,74,105,224,119,97,109,197,105,188,54,102,99,140,184,202,137,171,172,188,220,161,
90,207,143,254,119,104,19,195,218,240,10,254,254,47,107,27,252,74,235,204,176,158,1,197,147,64,52,9,59,246,147,175,189,250,229,143,255,121,127,250,212,91,20,87,60,178,245,98,53,74,255,239,241,202,208,
73,48,87,1,10,65,16,87,185,221,85,46,242,7,207,155,98,209,50,91,205,214,198,48,166,126,133,190,237,191,89,67,191,73,166,2,152,34,160,139,96,93,125,217,250,35,31,233,245,94,188,35,69,197,126,101,171,70,
199,230,83,227,229,161,191,181,6,101,102,219,96,163,227,102,95,214,137,59,61,29,150,154,203,197,8,2,34,143,188,82,173,231,252,5,101,126,109,239,142,179,187,130,160,124,119,206,55,187,71,114,39,202,23,
46,237,248,116,211,85,142,138,159,123,170,216,114,149,249,10,216,78,39,58,109,131,217,198,42,61,18,247,221,246,171,79,86,94,254,233,221,38,60,121,160,82,169,252,199,90,173,246,157,227,227,227,27,118,149,
53,128,2,220,35,7,154,223,59,101,50,12,99,248,50,190,253,71,135,237,239,100,148,173,254,99,86,233,147,53,58,240,99,71,250,206,125,103,74,53,238,128,177,202,50,29,187,82,217,255,115,19,149,161,139,204,
38,132,181,81,98,43,91,172,225,134,240,109,227,42,183,63,180,137,60,207,35,183,167,229,13,243,111,253,120,76,35,159,58,188,215,251,119,158,50,239,5,51,49,171,231,79,140,223,247,158,114,189,183,198,214,
198,32,232,165,44,82,113,149,147,42,220,255,0,55,91,178,215,211,195,204,150,97,45,21,6,239,248,133,129,252,197,159,39,138,222,155,236,25,61,215,208,133,127,89,170,118,23,193,238,174,206,229,224,234,205,
234,131,23,116,21,204,86,115,28,135,108,77,13,204,165,139,51,187,31,45,135,233,159,101,211,120,153,184,66,138,171,206,85,182,246,34,27,51,158,116,17,51,176,182,198,188,62,39,126,51,66,110,142,138,204,
244,116,12,64,191,124,233,214,113,98,125,133,88,19,177,121,214,104,250,177,203,51,187,30,103,29,71,0,180,173,213,116,243,29,182,158,239,115,219,185,202,205,144,19,232,213,0,160,99,95,125,219,15,191,253,
206,135,26,214,226,207,166,203,189,79,159,59,63,168,129,10,54,227,251,19,87,121,131,37,174,242,235,92,226,42,95,235,1,183,247,107,144,218,148,81,132,148,4,44,1,75,192,82,18,176,4,44,37,1,75,192,18,176,
148,4,44,1,75,192,82,18,176,4,44,1,75,73,192,18,176,148,4,124,117,75,92,229,237,18,176,184,202,157,15,84,92,229,205,14,86,92,229,14,135,42,174,242,230,132,43,174,114,39,91,173,184,202,29,108,181,226,42,
139,171,44,174,178,184,202,16,87,121,185,18,87,89,92,229,230,204,87,92,101,113,149,55,177,139,16,87,185,131,45,88,92,229,14,143,34,196,85,238,96,192,226,42,139,171,44,174,178,184,202,203,118,13,226,42,
139,171,44,174,178,184,202,243,198,155,226,42,139,171,44,174,178,184,202,139,95,220,196,85,22,87,25,226,42,139,171,188,120,247,208,108,200,226,42,139,171,60,239,107,196,85,22,87,89,92,101,113,149,197,
85,22,87,89,92,101,113,149,197,85,22,87,89,92,101,113,149,151,248,245,82,67,49,113,149,197,85,94,229,98,143,184,202,139,52,103,113,149,23,105,245,226,42,175,186,197,174,188,162,38,174,242,138,253,236,
106,151,43,197,85,190,166,92,101,96,113,87,217,23,87,121,243,93,229,12,148,151,1,84,186,205,85,54,29,113,149,213,138,239,83,135,245,197,145,246,79,16,52,230,184,202,201,113,211,185,45,120,214,85,246,211,
89,216,68,163,5,185,31,202,38,183,34,43,207,75,46,62,11,233,223,245,92,236,22,239,98,8,3,3,131,176,204,201,168,38,0,35,104,185,202,80,25,24,4,8,250,222,154,28,14,52,109,171,124,180,184,171,140,181,185,
202,254,50,223,245,172,171,76,20,85,26,249,227,185,108,229,31,18,34,213,147,11,123,74,83,81,152,29,242,178,173,5,18,158,93,226,107,186,202,123,18,87,217,234,44,224,53,0,19,130,41,128,167,44,208,10,151,
23,184,202,107,81,106,151,106,213,31,248,192,7,48,50,50,2,128,240,220,11,47,225,75,143,28,119,239,34,149,2,84,26,240,178,136,109,10,150,210,72,247,229,97,185,209,182,244,138,5,174,178,63,209,238,42,23,
230,186,202,108,183,151,171,124,223,125,247,97,100,100,4,185,92,14,3,3,131,248,134,247,191,31,195,35,187,146,123,204,83,128,202,129,41,131,200,250,80,233,33,192,211,201,249,225,217,157,25,113,149,151,
121,65,114,185,28,82,169,20,118,244,239,196,238,61,123,112,244,134,27,113,224,192,65,176,10,220,247,163,50,208,72,131,145,70,122,244,190,217,101,215,121,59,51,226,42,47,179,86,145,201,100,176,115,231,
78,28,56,112,0,123,247,238,193,11,47,158,2,40,13,144,59,133,25,105,31,148,61,0,4,166,109,227,22,173,123,237,148,184,202,75,95,236,30,122,232,33,60,253,244,87,209,215,215,139,122,189,129,239,255,129,31,
198,244,116,9,32,55,177,136,181,2,83,22,233,61,247,36,23,55,110,187,13,167,185,145,211,116,149,175,156,239,206,84,215,237,42,251,203,30,75,43,22,173,215,215,103,217,216,152,60,10,167,106,59,95,238,47,
148,158,79,5,225,158,166,171,28,87,119,199,94,222,11,44,249,32,178,109,11,103,201,205,133,202,185,202,92,253,125,88,40,164,224,158,102,224,121,105,24,27,45,120,237,219,91,231,70,92,229,207,126,246,179,
248,236,231,62,239,222,45,72,185,181,6,242,160,173,66,204,64,122,247,255,13,229,199,48,134,103,241,234,214,145,3,31,74,249,8,34,29,155,218,171,245,160,171,158,33,27,46,238,42,79,79,111,103,87,25,109,67,
65,133,56,214,8,163,24,126,239,215,33,232,71,219,140,173,237,126,186,4,174,38,27,160,58,126,225,108,119,166,178,195,111,185,202,217,207,137,171,188,232,90,24,67,27,141,70,35,132,242,242,200,140,222,217,
250,26,213,118,163,162,71,169,228,168,109,0,174,199,141,52,46,84,242,65,121,128,184,65,177,182,39,78,77,28,126,8,64,67,92,229,214,121,53,183,168,174,181,70,173,90,133,167,24,133,76,4,58,243,27,224,186,
106,221,139,231,37,71,108,155,135,196,193,62,186,233,242,43,253,185,241,221,30,87,125,178,141,166,171,92,101,107,197,85,110,223,229,108,52,26,168,85,203,204,240,30,246,243,251,62,2,14,1,115,25,116,254,
215,128,138,7,130,123,215,41,229,100,112,176,143,112,108,226,52,87,78,164,178,170,212,175,184,142,122,232,125,234,220,244,190,199,1,212,8,8,87,115,113,91,75,192,215,177,171,92,65,35,108,196,76,217,255,
90,235,249,254,239,158,182,111,253,227,208,244,252,14,25,119,64,155,46,127,20,152,241,91,135,21,9,62,142,236,50,149,188,249,234,133,222,244,196,17,197,85,24,29,191,60,86,26,248,195,68,5,111,48,115,212,
54,246,229,77,9,184,213,138,175,67,87,25,193,200,141,245,158,31,250,121,48,215,1,148,167,236,125,191,19,153,204,159,54,209,106,117,229,23,129,113,6,145,15,14,61,125,225,196,115,143,13,100,206,223,234,
113,69,145,173,149,42,141,212,255,153,168,238,124,9,64,149,141,9,193,182,213,122,87,115,51,226,22,118,149,253,187,99,245,230,91,170,193,63,57,207,238,230,145,6,91,91,6,243,244,164,185,239,215,98,173,254,
18,122,6,164,39,65,227,255,13,246,204,75,19,250,194,241,223,47,216,199,135,83,52,221,173,108,217,198,177,253,187,43,165,193,191,99,75,101,88,219,104,210,191,171,85,193,87,26,7,47,182,211,113,29,185,202,
127,110,26,143,124,81,121,65,0,16,116,210,181,145,219,189,82,102,194,190,237,127,236,228,191,245,3,154,126,31,40,123,190,50,245,165,95,61,56,60,243,190,172,87,57,66,28,43,163,241,229,139,197,125,31,153,
169,247,95,0,219,122,179,107,104,90,164,171,13,109,203,187,202,149,71,190,216,250,158,201,243,3,16,101,146,143,28,17,101,7,232,175,254,101,29,7,191,120,100,164,252,93,41,191,241,86,192,144,181,252,242,
179,151,222,244,205,245,56,87,132,181,33,91,219,88,234,185,17,226,42,55,103,164,61,61,218,89,202,176,68,164,225,121,17,51,215,76,238,13,191,124,227,142,23,126,202,87,209,91,97,97,153,241,98,57,220,241,
239,235,97,102,18,176,17,91,27,207,123,188,140,184,202,237,213,110,41,195,218,255,159,189,251,137,141,162,138,227,0,254,125,111,102,246,95,75,187,133,22,194,255,42,130,213,16,35,81,49,144,88,9,6,47,92,
76,184,233,1,245,226,159,171,55,99,162,7,18,189,155,232,201,68,79,106,98,98,136,70,176,49,138,16,9,32,9,8,202,63,17,80,178,84,232,223,101,103,103,230,205,204,123,63,15,51,187,221,150,182,172,182,75,132,
254,126,39,8,61,188,126,243,227,205,219,183,111,222,71,83,168,34,138,227,0,198,120,48,166,50,226,46,189,42,225,15,74,82,2,38,60,19,70,246,59,87,71,215,29,131,16,30,233,56,160,56,10,155,189,32,105,174,
115,240,164,85,197,221,228,42,55,88,202,64,34,232,26,19,248,90,56,142,20,150,37,143,92,121,246,141,45,189,95,87,35,157,251,242,242,216,67,167,42,170,67,145,242,72,56,142,169,125,91,252,95,199,185,224,
92,229,198,155,96,237,37,221,130,226,72,52,190,80,88,231,137,71,134,169,153,174,101,87,121,134,110,158,26,246,212,223,237,127,115,243,223,221,236,42,223,9,174,152,93,229,22,23,51,15,28,48,7,204,197,1,
115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,240,255,191,216,85,94,40,1,179,171,220,250,64,217,85,158,239,96,217,85,110,113,168,236,42,207,79,184,236,42,
183,178,107,217,85,110,97,215,178,171,204,174,50,187,202,236,42,131,93,229,217,138,93,101,118,149,107,159,124,217,85,102,87,121,30,167,8,118,149,91,216,193,236,42,183,120,21,193,174,114,11,3,102,87,153,
93,101,118,149,217,85,158,117,106,96,87,153,93,101,118,149,217,85,158,178,222,100,87,153,93,101,118,149,217,85,158,254,225,198,174,50,187,202,96,87,153,93,229,233,167,135,90,35,179,171,204,174,242,148,
159,97,87,153,93,101,118,149,217,85,102,87,153,93,101,118,149,217,85,102,87,153,93,101,118,149,217,85,158,225,239,51,45,197,216,85,102,87,185,201,205,30,118,149,167,105,103,118,149,167,233,122,118,149,
155,238,216,219,239,168,177,171,124,219,121,182,217,237,74,118,149,217,85,102,87,153,93,229,38,31,118,211,79,49,34,185,191,255,22,87,57,11,178,242,119,196,85,158,237,255,219,132,171,12,132,110,208,118,
92,80,100,4,133,162,179,160,58,253,209,80,77,96,169,147,183,248,106,174,178,72,93,101,164,34,45,68,6,36,28,72,203,70,35,214,55,213,85,254,55,15,188,153,186,186,191,191,31,219,182,109,171,239,243,81,122,
142,141,100,38,149,105,243,136,40,3,35,114,176,187,218,146,35,84,116,235,198,85,205,85,238,44,52,186,202,109,147,93,101,41,23,150,171,188,115,231,78,244,245,245,161,175,175,15,187,95,220,157,238,243,38,
15,52,145,134,75,34,135,72,59,51,186,202,146,93,229,233,135,222,223,223,95,71,171,151,46,91,134,141,27,55,98,199,142,29,105,247,102,211,113,20,160,145,131,22,121,100,87,110,78,142,79,213,142,12,80,237,
155,25,139,93,229,217,246,30,58,58,139,88,189,106,53,54,108,232,195,99,155,55,39,221,43,106,221,155,135,50,25,200,66,47,96,199,245,83,160,181,99,7,245,151,103,164,197,174,242,212,127,59,120,240,32,194,
48,68,177,88,196,138,21,43,176,124,249,114,236,221,187,15,36,18,21,156,144,131,210,14,72,46,66,118,229,163,105,247,78,156,126,79,86,14,233,239,68,54,28,117,163,52,23,87,121,86,246,87,151,203,6,182,101,
72,155,8,68,106,212,235,190,16,199,56,35,160,76,205,85,142,171,113,84,63,177,222,16,114,221,85,22,137,171,28,68,22,84,44,161,41,89,127,90,86,118,202,199,231,57,110,220,52,116,245,199,31,127,130,253,251,
7,112,228,232,49,188,244,242,171,248,237,204,69,64,102,1,153,65,140,12,98,114,144,89,246,28,96,77,175,130,39,251,232,54,156,40,142,140,247,135,231,72,63,39,48,131,171,124,155,35,172,247,172,171,60,48,
48,128,61,123,222,197,47,167,207,164,58,174,141,72,75,132,177,128,221,185,13,118,215,196,7,11,74,87,243,181,115,204,86,218,189,213,161,210,213,186,171,172,195,160,162,10,236,42,79,222,7,147,245,249,61,
54,4,21,106,88,237,91,145,91,179,126,210,143,38,225,90,16,34,121,181,65,72,7,20,196,42,139,82,181,205,169,244,8,36,174,242,229,225,117,7,192,174,242,173,127,142,181,129,231,43,72,103,41,242,27,158,158,
244,237,139,128,53,241,106,67,138,189,74,216,104,199,224,165,37,133,145,149,22,249,182,208,65,101,196,45,126,196,174,242,45,15,116,130,10,2,120,85,151,32,178,39,100,102,209,23,84,58,55,177,207,32,210,
143,208,41,183,46,101,6,2,14,130,235,195,37,114,207,103,115,178,178,88,146,7,95,89,251,174,142,173,97,87,121,146,171,172,20,92,183,2,165,130,200,200,197,239,123,133,93,111,149,227,245,223,7,227,63,29,
192,224,165,250,82,76,214,199,155,129,16,54,122,123,180,159,143,207,14,21,115,163,189,18,30,226,56,190,112,253,102,207,167,32,114,65,164,216,85,174,185,202,174,139,88,235,139,58,255,248,147,65,199,11,
31,2,228,1,84,41,71,107,7,130,241,195,63,227,239,43,233,251,122,233,75,226,194,1,66,75,95,59,119,254,92,79,219,245,251,109,114,165,212,126,197,245,179,159,15,87,187,47,130,168,74,90,7,236,42,167,174,178,
144,133,205,170,243,181,77,42,187,253,47,50,38,72,84,92,26,5,153,161,155,209,170,175,212,248,161,19,24,188,146,204,191,112,128,80,106,53,120,250,199,69,56,219,149,21,229,118,203,84,141,10,233,135,27,55,
123,18,87,25,148,172,28,216,85,150,239,106,177,230,178,234,126,157,200,104,27,100,164,32,17,16,81,85,192,140,129,164,69,134,168,28,174,253,172,56,246,45,178,230,153,77,232,121,88,123,131,127,126,179,34,
255,107,111,193,30,95,101,25,95,70,81,116,188,84,94,243,65,217,47,150,200,104,15,198,4,20,69,17,187,202,105,213,61,101,67,142,112,156,44,164,44,8,41,59,0,116,65,202,34,132,232,0,196,162,46,235,228,46,
101,109,58,123,223,146,115,219,243,182,251,136,64,40,72,235,75,167,174,109,122,94,69,206,48,140,113,137,140,15,64,213,60,210,198,177,46,100,87,153,116,185,108,100,123,123,12,99,164,32,242,147,135,137,
32,97,140,130,16,46,132,40,40,241,192,39,235,186,142,191,146,183,252,71,64,154,72,155,223,71,170,221,111,171,40,51,4,163,147,121,23,20,2,248,87,83,195,92,206,69,220,21,174,114,141,193,52,174,155,88,202,
68,33,140,241,64,52,78,134,70,72,235,33,24,115,205,83,185,11,146,170,215,4,185,130,180,127,161,26,100,222,43,141,175,62,10,50,149,164,115,73,213,230,93,252,7,86,243,158,118,149,219,183,62,69,238,225,67,
48,158,23,75,41,9,66,26,225,56,201,22,172,148,1,25,99,1,176,78,150,182,188,249,196,170,239,198,93,213,254,245,149,177,7,79,170,40,231,35,217,224,138,72,107,51,151,70,88,16,174,114,227,141,132,144,210,
18,201,94,164,36,163,229,212,177,38,219,107,70,67,74,106,102,172,236,42,55,116,178,46,151,141,189,164,155,40,142,52,233,80,76,55,214,250,5,207,99,99,181,181,238,156,46,167,155,143,139,233,208,244,142,
249,204,23,160,221,201,106,233,88,111,123,49,29,215,252,22,51,15,28,48,7,204,197,1,115,192,28,48,23,7,204,1,115,192,92,28,48,7,204,197,1,115,192,28,48,87,179,245,207,0,109,151,227,237,26,241,115,226,0,
0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00129_png = (const char*) resource_NewJucerComponent_bmp00129_png;
const int TalComponent::bmp00129_pngSize = 113953;























































// JUCER_RESOURCE: bmp00132_png, 3712, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00132.png"
static const unsigned char resource_NewJucerComponent_bmp00132_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,3,155,73,68,65,84,120,218,236,89,193,107,218,80,24,255,101,216,26,139,25,90,61,68,
241,16,102,145,130,25,20,204,165,160,116,23,165,135,210,67,15,235,105,184,211,232,117,255,192,14,189,236,216,29,203,110,101,224,232,96,61,200,14,163,189,172,180,224,14,41,12,170,32,98,193,131,216,48,180,
202,18,154,152,6,178,131,77,58,107,236,172,109,179,29,242,3,133,247,158,239,51,239,251,189,223,247,125,121,143,200,229,114,58,28,252,55,112,1,192,218,87,15,244,243,51,232,93,17,208,84,199,43,182,50,48,
9,194,77,129,152,154,198,230,162,220,35,68,63,63,131,46,253,132,174,252,2,180,174,227,36,91,9,113,3,23,242,101,195,115,73,72,87,236,145,33,119,160,95,40,142,147,108,4,49,65,66,7,128,9,207,21,33,208,84,
64,235,246,200,208,28,66,236,132,14,128,152,232,154,169,194,53,234,68,46,70,99,121,126,6,0,240,246,227,119,200,170,214,55,190,182,52,135,112,192,139,189,163,26,14,138,117,248,41,18,175,87,56,179,29,13,
249,144,205,176,3,118,219,162,2,190,34,224,160,88,55,251,150,231,103,192,197,104,188,217,58,180,124,150,215,43,28,206,36,5,91,187,69,203,241,108,134,69,52,228,179,28,59,57,237,12,204,51,254,47,95,168,
130,175,8,125,99,41,54,130,116,130,25,106,235,211,183,50,100,85,51,253,83,170,53,177,189,95,30,176,31,13,249,176,177,195,143,150,212,111,11,46,70,247,57,48,26,242,193,79,145,35,205,221,218,45,226,228,
180,99,182,211,9,198,92,240,159,54,239,10,43,199,91,193,79,145,224,98,52,218,162,130,20,27,25,32,196,192,245,205,17,14,120,145,205,176,72,39,24,228,11,85,179,63,206,4,145,150,24,236,29,213,198,122,238,
71,183,157,192,87,4,112,49,122,128,160,97,11,249,27,246,142,106,224,43,2,146,79,35,255,36,100,164,216,8,218,162,130,124,161,106,146,51,10,60,147,174,161,254,73,177,145,145,237,220,89,33,165,90,19,113,
38,104,146,224,167,72,196,153,32,54,118,120,164,216,241,156,218,104,73,224,98,52,252,20,137,182,120,63,57,44,26,242,97,61,155,28,232,255,51,44,25,4,228,11,85,156,156,118,112,114,218,25,170,18,43,91,124,
69,24,80,66,169,214,132,172,106,88,158,159,65,163,37,161,209,146,30,150,16,0,56,60,174,35,206,4,77,181,240,21,225,222,28,105,103,200,50,212,97,16,112,120,92,71,54,195,90,42,222,8,89,225,128,23,203,243,
51,144,85,173,47,84,93,87,253,180,151,68,54,195,98,243,203,143,135,13,89,198,206,8,7,188,136,134,124,224,98,52,74,181,230,157,156,23,14,120,33,171,154,173,164,26,234,240,83,36,214,179,73,172,103,147,102,
209,113,147,210,27,45,9,219,251,101,132,3,94,172,46,204,14,253,93,190,80,69,91,84,176,186,48,59,52,188,221,155,66,100,85,67,169,214,196,243,103,179,104,180,164,190,36,125,91,164,19,12,184,24,61,118,18,
188,75,238,144,85,13,239,62,243,125,21,163,81,45,221,148,23,219,162,130,195,227,58,210,9,6,169,86,196,178,24,145,85,13,219,251,101,188,76,179,8,51,193,145,55,155,107,220,5,29,20,235,99,169,227,122,233,
219,104,73,150,229,230,56,57,130,175,8,102,24,25,150,67,0,96,99,135,55,115,199,245,242,157,175,8,136,51,193,27,43,46,99,253,79,194,62,36,159,70,134,110,200,182,168,96,123,191,140,181,165,185,209,95,20,
115,185,156,254,234,67,29,186,40,64,63,111,59,47,134,182,31,157,144,32,166,252,32,40,26,239,95,68,198,203,33,14,30,144,159,222,247,36,224,114,95,157,171,56,176,245,44,11,46,119,143,3,131,16,194,77,1,23,
242,213,185,138,3,27,37,225,6,65,62,238,113,96,18,50,53,221,27,156,240,56,247,33,182,19,114,117,31,2,92,222,135,108,46,202,0,60,151,31,7,255,6,189,59,145,223,3,0,29,37,157,14,59,13,208,137,0,0,0,0,73,
69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00132_png = (const char*) resource_NewJucerComponent_bmp00132_png;
const int TalComponent::bmp00132_pngSize = 3712;

// JUCER_RESOURCE: bmp00133_png, 5709, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00133.png"
static const unsigned char resource_NewJucerComponent_bmp00133_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,11,104,73,68,65,84,120,218,236,153,91,140,93,213,121,199,127,223,90,123,239,115,206,
92,140,61,166,120,6,99,192,1,115,113,77,28,67,72,9,9,56,54,4,229,162,186,180,168,9,74,105,20,181,85,163,180,137,218,170,74,111,145,250,144,74,233,75,68,212,86,114,46,82,171,170,82,156,86,164,161,208,152,
38,74,212,208,64,108,39,196,118,68,137,49,96,27,98,108,108,6,123,236,241,140,231,156,125,89,235,235,195,90,123,159,115,198,131,75,170,62,228,33,123,180,117,206,156,181,246,90,223,250,255,191,251,150,29,
59,118,40,63,191,126,102,174,4,224,47,31,125,23,0,74,133,167,68,169,0,15,226,17,20,17,16,145,161,7,85,135,121,92,60,62,60,22,239,129,239,97,13,240,26,62,195,125,161,110,44,254,105,112,155,122,76,9,107,
3,24,35,136,128,17,48,166,255,187,2,222,15,238,167,23,200,190,212,153,6,215,34,238,83,175,165,244,101,191,24,54,195,242,10,96,80,53,8,9,134,20,9,52,240,233,109,143,199,111,128,167,192,83,160,20,40,21,
138,67,240,88,11,214,10,214,244,193,232,31,44,108,108,68,48,38,8,94,31,126,232,96,17,40,91,3,52,0,128,247,224,188,14,17,211,7,80,135,126,171,201,52,113,35,23,199,209,72,138,4,89,19,11,137,149,32,143,8,
104,88,191,114,138,139,178,123,175,17,108,105,192,174,21,196,123,109,200,181,6,82,27,230,212,4,233,128,188,126,17,185,225,249,154,172,32,95,131,155,130,247,2,42,40,22,213,4,197,97,200,48,100,125,11,9,
150,81,160,228,241,179,4,169,16,163,88,11,89,42,100,169,33,181,97,229,210,41,149,211,120,168,32,116,98,133,52,25,6,42,98,129,8,36,70,26,32,107,96,157,87,156,235,19,226,125,32,168,38,164,172,194,62,58,
64,108,98,133,100,64,14,231,20,23,1,172,101,104,101,134,86,34,136,1,65,240,26,214,49,21,148,149,167,242,96,44,65,110,35,36,73,32,207,249,154,180,32,191,53,113,189,52,16,97,163,245,5,133,236,107,206,160,
133,86,94,169,92,56,139,139,243,250,99,224,28,84,78,80,103,81,82,12,138,7,4,211,39,36,184,169,34,90,73,142,82,34,84,136,129,52,21,58,109,195,104,219,210,206,194,210,69,169,228,149,54,154,148,38,66,150,
24,178,36,8,92,186,0,236,16,33,86,72,173,52,7,17,2,0,69,92,71,233,19,67,180,142,110,238,201,203,62,224,214,4,112,178,52,8,159,23,158,188,82,196,245,229,24,105,27,198,59,150,78,203,68,0,131,44,221,194,
211,43,60,121,233,41,171,0,120,146,8,173,212,208,78,131,178,84,78,251,227,4,178,218,153,97,164,101,72,18,33,181,195,138,20,180,62,172,111,163,149,149,78,131,34,249,240,233,124,223,141,229,101,184,21,193,
171,197,199,65,19,124,209,176,133,132,187,68,41,241,148,24,113,88,171,100,153,97,180,147,240,193,205,151,243,177,247,175,3,224,183,62,183,135,215,102,11,42,23,204,190,149,8,15,254,238,70,214,78,142,241,
208,19,71,121,100,247,203,76,140,183,249,219,143,221,194,87,30,255,9,143,238,57,198,91,174,185,132,63,253,245,95,188,192,175,190,122,182,199,183,246,157,224,225,93,199,112,209,125,253,254,47,175,227,61,
183,76,241,206,63,254,79,48,142,42,2,158,24,97,164,109,249,234,167,126,137,19,51,61,62,190,125,31,228,158,162,10,227,173,204,48,54,2,159,251,232,70,222,188,118,249,146,126,124,239,11,103,248,196,231,247,
133,249,169,97,180,45,252,225,189,235,184,123,211,36,95,124,236,16,143,236,57,78,94,40,94,149,44,17,30,216,186,134,223,188,107,237,146,107,29,56,58,203,246,175,63,207,124,183,98,235,198,85,124,248,238,
55,241,131,231,78,243,224,215,14,82,121,200,75,165,40,61,127,112,239,245,108,186,102,57,191,246,87,187,16,81,186,10,133,243,40,138,98,130,251,162,234,19,2,30,197,13,16,19,98,136,53,144,38,74,167,109,104,
103,125,65,182,221,54,197,191,237,62,74,89,133,184,177,233,218,21,172,90,209,6,96,180,13,203,199,133,101,163,97,110,59,131,75,70,97,164,21,254,255,236,191,62,195,211,47,206,128,8,222,43,31,184,99,45,15,
108,93,75,59,19,254,253,251,47,163,10,173,52,62,219,242,148,222,33,210,119,73,237,44,184,23,107,160,157,121,74,231,240,234,163,133,24,218,89,112,161,7,143,205,178,253,235,7,240,170,244,10,207,185,133,
138,249,174,163,87,120,146,68,73,173,97,188,3,235,86,119,184,123,211,36,167,207,245,184,239,157,87,240,189,103,79,176,144,59,170,10,172,133,86,60,247,31,125,105,55,105,98,176,6,170,74,153,156,24,225,247,
222,191,158,223,216,114,37,255,242,221,35,140,118,194,188,183,93,191,146,223,126,207,213,60,188,235,37,186,133,208,43,132,86,26,112,234,180,60,206,7,239,130,248,16,103,176,40,46,36,81,16,28,151,226,135,
110,112,136,56,196,56,172,245,100,137,146,69,144,126,248,194,52,119,110,152,100,197,152,97,197,184,97,249,152,225,174,183,76,241,212,243,175,2,33,222,140,181,133,145,150,68,45,132,145,182,52,32,27,163,
120,245,168,6,33,254,249,187,135,248,175,103,78,240,190,91,175,96,124,68,24,235,72,19,171,178,84,73,19,197,218,112,167,137,210,74,99,48,54,1,172,36,241,36,86,177,54,124,166,137,54,9,70,150,40,137,85,140,
241,77,214,104,173,210,74,161,211,130,177,142,240,171,183,175,97,102,46,103,231,83,47,113,233,178,54,239,187,117,138,75,70,13,203,70,133,241,142,144,37,125,197,106,103,208,201,132,118,11,150,143,165,16,
93,113,56,95,144,121,239,161,105,222,189,105,53,247,220,60,197,196,184,101,98,220,146,69,151,216,110,65,154,120,18,235,145,69,152,107,36,36,233,135,208,254,29,254,66,16,168,181,209,198,128,252,194,43,
51,108,184,106,130,173,27,167,216,245,236,171,172,92,214,102,227,218,149,252,205,35,63,98,243,77,171,177,70,98,18,32,67,90,93,251,253,196,66,154,244,179,26,85,225,216,169,121,54,111,152,226,242,137,14,
167,206,245,176,193,157,146,37,18,247,238,199,144,172,9,176,48,218,54,148,149,197,26,135,247,194,72,43,248,123,107,224,186,213,151,240,217,223,185,253,2,55,179,125,231,179,124,123,255,113,178,212,112,
245,170,17,110,191,113,146,199,158,122,137,227,167,231,120,241,213,115,220,179,233,10,246,28,60,73,89,73,36,53,200,253,215,31,121,251,5,107,237,63,60,205,119,158,126,153,86,42,13,62,207,29,155,33,47,42,
238,125,251,155,152,62,187,192,145,147,231,154,216,218,201,12,243,54,88,188,152,11,113,31,32,228,245,47,141,193,87,99,96,85,133,221,7,79,178,225,170,149,252,240,133,105,110,187,126,21,123,15,77,51,51,
215,107,130,87,147,142,190,129,75,151,202,147,47,82,207,132,115,135,3,102,137,161,149,153,160,62,190,78,79,3,48,135,78,204,242,247,223,60,64,81,121,206,247,28,231,22,28,221,194,53,177,168,149,24,222,251,
214,43,153,153,235,177,255,240,52,198,8,187,15,188,194,135,182,220,192,157,27,166,120,226,153,147,81,49,195,245,39,255,176,11,107,132,53,191,48,202,175,220,118,13,221,188,226,225,221,71,26,76,234,196,
163,114,240,205,125,71,89,49,222,230,195,119,221,192,246,157,255,141,17,65,162,66,213,41,246,235,93,23,29,214,152,46,22,85,63,112,246,10,207,158,131,39,153,156,24,229,170,203,150,113,203,186,203,248,209,
145,83,44,228,193,228,138,74,89,232,121,122,165,139,2,6,31,94,148,190,249,191,78,103,67,70,226,185,226,210,49,22,242,138,227,167,23,232,21,190,201,96,138,42,248,220,16,236,67,186,89,249,190,54,25,9,36,
100,137,33,77,4,49,18,50,53,31,178,180,249,158,227,124,207,209,205,61,189,194,197,84,61,16,123,217,242,14,239,88,63,201,196,120,155,79,221,255,54,254,252,3,183,242,161,45,55,0,240,174,155,86,163,196,148,
58,102,66,51,115,37,175,205,22,60,253,226,89,182,239,252,49,151,175,28,229,190,119,92,75,55,247,116,115,215,156,111,33,119,204,117,29,15,61,113,152,211,115,57,31,188,243,58,218,153,109,234,18,239,99,253,
226,47,82,169,199,26,186,185,5,1,53,104,204,203,187,133,167,23,55,60,223,243,156,56,147,179,255,240,41,238,223,124,29,71,167,231,217,119,104,134,50,10,222,45,28,179,11,21,173,24,116,242,210,115,190,23,
210,87,162,181,213,5,22,170,220,191,249,90,54,223,52,197,67,79,30,97,174,235,240,158,102,173,188,84,138,138,38,203,42,141,82,86,3,149,125,212,58,107,12,18,73,106,106,4,47,148,149,82,186,254,158,38,22,
149,94,133,247,190,117,13,231,123,21,127,246,143,63,32,47,43,140,8,170,112,199,134,73,30,216,178,142,91,215,173,226,59,79,159,32,26,21,231,22,28,170,225,249,51,243,231,121,244,251,63,225,254,59,175,225,
200,141,115,124,99,239,49,186,69,144,121,174,235,152,153,171,56,223,243,124,233,63,158,229,19,219,54,112,211,213,43,121,109,182,71,94,133,116,217,123,185,0,243,186,146,73,136,69,201,224,173,49,39,118,
94,41,43,161,155,67,55,215,40,152,231,244,172,227,171,79,30,229,246,27,39,121,242,199,175,113,106,214,225,162,143,90,232,41,103,231,61,237,84,35,65,202,236,121,207,66,124,254,147,247,189,121,72,35,14,
159,152,227,243,59,159,227,91,251,95,9,26,172,144,151,97,172,151,67,89,10,165,51,81,109,132,188,8,100,172,191,114,5,95,254,228,150,161,181,190,177,247,21,190,248,216,115,120,133,245,87,46,231,11,31,191,
99,73,45,252,232,223,237,97,235,198,203,249,194,99,207,243,242,116,62,212,146,121,248,123,199,217,184,246,82,222,189,105,13,143,236,57,78,175,8,114,159,157,247,56,31,172,82,4,254,233,219,47,177,126,205,
4,247,220,188,134,167,158,63,211,156,239,220,130,231,212,57,71,150,120,206,204,207,243,224,215,14,240,153,143,220,140,87,101,161,7,69,101,168,188,162,42,23,224,14,32,59,118,236,208,191,120,244,22,60,93,
60,61,60,57,158,18,145,42,102,53,66,43,19,58,153,161,211,178,88,19,218,13,117,111,202,235,96,113,20,10,63,23,43,248,212,74,83,24,218,69,133,97,109,45,101,21,14,218,152,115,51,86,23,134,190,49,239,36,17,
70,219,134,177,182,37,75,77,44,46,181,113,101,170,138,145,144,241,24,35,141,21,57,55,92,241,15,246,164,156,31,40,181,53,100,111,117,32,175,207,85,57,101,161,240,161,51,1,136,17,18,3,157,150,109,50,66,
141,56,212,133,174,181,66,98,164,137,27,149,83,206,247,28,11,61,165,91,64,81,90,188,75,48,180,48,180,49,116,248,204,182,189,181,133,36,241,78,17,124,205,21,26,219,0,228,130,119,66,89,74,223,221,244,219,
68,212,110,93,36,58,14,13,19,236,0,33,73,236,11,129,12,245,178,202,202,244,9,25,232,35,41,66,89,9,149,51,13,144,170,66,23,65,125,104,227,132,253,165,9,170,53,174,189,92,22,53,22,165,33,187,238,10,104,
220,171,114,23,54,39,147,129,181,3,41,66,94,14,247,212,172,9,242,37,49,137,8,107,11,101,229,81,250,65,92,98,134,26,148,207,146,151,161,125,130,183,17,239,180,193,191,113,89,134,52,54,185,106,48,12,170,
22,245,30,87,129,154,184,89,217,55,109,65,26,173,170,117,79,16,6,245,176,238,162,134,230,98,63,117,174,123,126,62,2,57,104,33,48,220,92,244,131,13,71,3,120,193,85,210,111,30,50,216,117,13,187,15,90,240,
96,103,121,112,77,223,16,50,108,57,70,66,115,82,68,26,115,173,123,97,186,168,89,90,85,97,254,80,70,186,104,94,221,192,84,250,201,9,222,32,4,66,12,25,66,134,33,29,140,33,9,134,12,31,251,42,67,213,99,116,
9,42,23,111,177,255,111,233,234,112,251,125,160,219,251,83,180,223,85,161,242,65,219,203,178,223,245,101,160,179,58,188,167,32,139,82,236,165,246,28,218,67,192,59,89,50,227,28,156,231,5,220,128,183,24,
122,29,240,58,175,38,194,26,161,253,46,106,163,101,100,145,148,100,184,14,9,63,26,124,211,87,241,168,250,126,145,160,111,184,92,88,186,214,248,63,94,178,104,161,160,133,255,191,47,133,150,218,227,141,
206,211,159,242,252,33,135,53,81,245,135,223,135,52,132,124,122,219,227,63,127,85,247,51,114,253,207,0,38,45,191,173,163,217,175,15,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00133_png = (const char*) resource_NewJucerComponent_bmp00133_png;
const int TalComponent::bmp00133_pngSize = 5709;

// JUCER_RESOURCE: bmp00134_png, 3135, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00134.png"
static const unsigned char resource_NewJucerComponent_bmp00134_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,19,0,0,0,19,8,6,0,0,0,114,80,54,204,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,1,90,73,68,65,84,120,218,228,147,33,72,3,97,20,199,127,202,20,217,194,174,12,174,221,
194,202,149,115,224,194,89,198,197,91,24,12,86,86,60,12,134,37,131,150,25,236,150,13,227,133,229,113,101,112,96,112,69,24,43,91,56,97,201,25,12,94,145,193,149,91,216,16,139,6,241,195,121,42,19,47,8,190,
244,248,190,143,223,123,239,255,255,222,90,167,211,121,38,166,72,0,28,156,93,252,26,212,110,148,89,39,198,248,187,176,196,170,15,21,57,141,101,110,163,200,105,113,214,237,79,184,28,221,253,28,86,175,20,
8,194,185,48,171,152,87,176,76,13,64,0,5,44,35,37,169,87,10,204,31,159,104,57,35,0,142,106,58,25,41,133,237,122,156,216,87,75,240,193,216,199,50,53,146,91,27,81,205,130,112,129,237,122,100,101,137,170,
161,82,53,84,178,178,132,237,122,248,211,89,164,211,146,158,3,192,187,125,248,124,204,32,92,208,116,134,156,238,23,1,104,57,163,47,65,37,61,23,185,143,104,150,145,82,34,127,63,194,155,20,123,166,70,106,
107,147,166,51,140,20,74,124,116,172,94,217,161,219,159,0,96,153,26,65,56,199,159,206,40,233,57,170,134,202,96,236,211,234,141,190,255,26,138,156,230,184,182,203,96,236,11,119,94,225,5,108,215,163,106,
168,194,197,98,94,17,128,155,251,64,24,38,96,254,116,198,225,121,111,169,146,237,94,139,124,149,253,253,39,187,25,255,162,183,27,229,88,96,47,3,0,219,57,114,101,181,186,214,75,0,0,0,0,73,69,78,68,174,
66,96,130,0,0};

const char* TalComponent::bmp00134_png = (const char*) resource_NewJucerComponent_bmp00134_png;
const int TalComponent::bmp00134_pngSize = 3135;

// JUCER_RESOURCE: bmp00135_png, 3580, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00135.png"
static const unsigned char resource_NewJucerComponent_bmp00135_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,19,0,0,0,19,8,6,0,0,0,114,80,54,204,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,3,23,73,68,65,84,120,218,172,148,79,104,93,69,20,198,127,231,204,220,123,223,123,73,
17,91,181,105,139,137,169,143,162,80,196,128,88,208,32,66,74,55,198,168,155,226,194,118,33,136,80,112,39,184,16,93,20,116,225,70,93,85,68,20,55,1,177,34,180,20,10,46,12,173,138,8,82,73,161,180,53,104,
138,138,74,83,108,155,127,239,221,123,231,28,23,239,229,53,184,148,206,48,204,172,190,249,241,125,156,79,102,103,103,157,219,180,34,192,155,39,158,4,192,169,49,42,156,26,48,28,3,54,255,37,8,10,40,66,68,
201,144,158,4,71,103,230,250,47,192,40,49,74,156,18,167,198,73,56,134,136,163,34,196,0,65,149,168,138,74,192,45,97,102,152,57,110,217,45,178,30,81,137,211,237,223,125,58,49,66,128,34,42,141,92,104,228,
129,34,6,4,112,23,202,82,232,86,66,85,42,34,125,70,209,138,168,21,72,181,73,44,161,106,20,153,208,204,149,161,70,160,85,8,205,76,201,162,99,201,232,148,198,202,90,98,69,106,84,149,40,34,132,144,8,49,145,
69,67,130,227,56,136,19,213,137,17,90,185,243,240,253,67,188,242,204,30,198,71,134,6,14,126,249,205,85,102,191,250,7,48,130,246,201,178,224,20,133,211,106,56,205,134,16,67,192,1,17,129,190,111,175,29,
124,144,171,55,58,188,252,222,15,108,105,102,76,238,221,206,115,147,59,49,19,62,62,117,19,13,27,100,209,104,239,202,121,235,197,54,221,58,241,193,201,75,64,226,200,204,3,108,221,82,240,206,241,243,188,
244,254,183,196,16,184,163,153,3,129,31,47,95,227,217,199,119,50,220,84,52,56,244,114,118,4,88,186,94,114,236,228,21,198,71,90,28,154,26,227,208,254,113,198,182,15,241,209,233,139,44,254,181,76,50,167,
78,9,115,35,153,51,185,247,46,0,190,191,112,147,170,54,220,189,31,128,64,12,194,202,90,205,231,115,87,56,124,96,55,0,95,156,93,96,233,198,58,69,166,0,228,81,25,110,4,14,60,178,131,39,30,186,135,99,39,
22,185,244,219,26,101,29,55,196,4,17,6,231,206,225,98,96,112,35,15,168,64,35,87,138,76,25,187,123,152,23,166,218,180,138,140,79,78,255,194,252,175,29,214,75,165,91,250,45,50,16,146,9,35,91,11,158,126,
108,23,103,231,255,6,140,169,137,81,46,254,190,204,106,119,153,167,30,189,151,233,125,163,156,91,184,198,103,95,47,114,125,21,186,37,116,43,39,153,244,196,68,0,87,118,239,104,242,234,193,251,56,51,191,
196,241,51,127,96,36,134,154,57,135,247,239,225,195,83,151,153,222,55,10,192,68,123,27,19,237,109,3,250,159,22,86,57,242,238,159,189,9,112,135,170,22,206,253,220,97,250,245,243,20,121,77,8,9,167,230,141,
79,47,224,56,89,16,158,127,251,59,242,160,4,13,36,83,214,187,202,234,122,96,101,45,210,41,115,220,33,186,59,117,10,172,119,149,50,9,162,62,24,114,85,71,5,98,128,44,26,65,29,21,199,61,82,215,66,85,9,101,
165,212,117,216,240,204,73,117,164,182,12,169,12,195,240,77,187,39,32,168,130,138,32,34,40,2,4,60,5,146,69,60,101,61,49,119,48,139,189,6,232,13,18,142,226,68,188,95,72,255,173,32,33,32,68,132,12,37,67,
9,155,211,4,37,71,80,140,240,191,250,108,80,65,71,103,230,110,75,211,254,59,0,64,12,109,123,4,149,204,244,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00135_png = (const char*) resource_NewJucerComponent_bmp00135_png;
const int TalComponent::bmp00135_pngSize = 3580;

// JUCER_RESOURCE: bmp00136_png, 3298, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00136.png"
static const unsigned char resource_NewJucerComponent_bmp00136_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,1,253,73,68,65,84,120,218,236,153,61,107,219,80,24,70,143,138,100,75,1,21,156,201,
67,198,96,58,120,212,18,104,160,83,200,148,193,67,87,175,89,253,7,58,116,201,152,53,107,22,23,58,100,200,84,200,82,112,32,29,148,169,30,138,137,75,6,99,155,130,99,81,153,72,178,5,234,32,75,194,142,220,
150,226,170,114,185,7,52,232,19,233,61,60,122,47,247,74,205,102,51,64,144,27,100,128,227,15,26,193,227,3,129,103,131,63,21,85,201,212,64,1,169,168,35,109,109,115,118,232,132,66,130,199,7,130,201,55,2,
247,59,248,158,40,82,166,66,138,48,115,230,59,218,92,136,103,135,50,28,139,96,230,138,34,101,136,164,168,4,0,138,150,8,193,159,130,239,133,50,124,33,36,75,2,64,82,188,184,85,200,121,124,73,163,82,230,
104,111,119,229,249,177,237,114,122,97,2,80,210,85,26,53,99,225,216,175,158,211,31,77,184,186,189,167,59,176,242,217,212,243,134,217,25,98,118,134,241,254,219,250,75,174,110,239,105,181,123,79,174,221,
175,238,48,182,93,74,186,138,81,41,47,220,23,113,122,97,50,182,147,228,215,15,170,188,126,245,130,147,119,159,114,247,237,207,54,57,238,145,132,86,187,71,119,96,177,95,221,249,237,123,175,63,247,242,59,
236,221,84,162,116,152,157,33,99,219,165,126,80,77,77,73,163,102,164,254,246,132,144,191,144,142,203,155,59,0,186,3,43,78,201,178,144,229,95,214,209,222,110,44,174,63,154,8,33,235,74,71,84,220,229,198,
189,170,151,68,180,218,61,140,74,153,146,174,10,33,235,76,71,90,163,111,212,140,212,148,44,203,116,166,62,95,251,98,148,181,214,222,145,54,234,186,188,185,139,123,201,170,30,210,31,77,120,255,241,11,206,
212,23,66,254,132,55,231,215,79,138,190,138,238,192,90,184,254,103,73,201,35,27,61,236,253,31,9,19,34,23,64,46,38,243,42,130,204,144,20,53,156,96,148,11,137,16,169,168,195,204,73,230,85,4,25,70,162,136,
164,62,15,29,196,66,182,182,195,147,138,38,214,67,50,23,146,172,135,192,124,61,228,236,208,1,180,249,38,248,55,132,107,34,63,6,0,125,200,214,56,47,46,119,153,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00136_png = (const char*) resource_NewJucerComponent_bmp00136_png;
const int TalComponent::bmp00136_pngSize = 3298;

// JUCER_RESOURCE: bmp00137_png, 4269, "D:\Code\workspace svn\Tal-Dub-3-Effect-juce\resources\bmp00137.png"
static const unsigned char resource_NewJucerComponent_bmp00137_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,5,200,73,68,65,84,120,218,236,153,61,172,28,213,21,199,127,231,204,204,238,91,32,64,
136,240,51,6,155,56,82,130,108,146,240,101,12,8,28,35,170,196,178,76,1,33,161,137,210,1,21,5,66,40,41,40,16,72,52,52,110,41,144,40,76,68,16,5,32,16,2,196,119,164,135,21,176,144,133,48,145,120,198,160,
240,236,152,152,151,24,191,221,153,123,207,161,184,119,246,195,134,130,130,241,43,56,171,163,187,90,205,220,153,57,255,249,159,143,255,202,158,61,123,156,31,108,213,88,9,112,255,51,55,2,224,4,140,6,39,
0,134,99,64,55,120,137,128,228,85,85,210,154,189,53,7,204,210,234,158,124,218,124,234,7,145,180,71,187,143,228,125,218,243,205,210,241,238,147,253,186,49,65,80,64,17,74,148,10,73,48,240,192,174,215,242,
55,192,168,49,106,156,26,39,224,196,78,1,81,50,16,154,214,178,128,178,16,84,101,12,138,153,19,13,162,57,230,179,96,186,79,130,157,30,27,84,161,42,4,213,4,16,238,56,16,98,218,163,9,142,89,218,215,165,43,
80,18,32,66,129,80,226,68,148,30,74,111,194,144,196,140,26,103,148,215,38,131,210,29,32,237,91,173,42,148,149,208,175,148,126,37,84,165,80,170,96,238,52,209,9,1,130,165,64,2,20,154,2,111,25,168,104,41,
176,34,9,208,126,149,0,41,84,112,79,199,132,152,192,64,28,162,99,193,113,75,79,219,29,32,37,66,133,230,40,39,214,100,64,82,154,170,51,75,70,157,3,34,2,158,243,139,168,80,149,202,160,175,12,250,5,115,61,
161,87,42,230,206,168,113,234,198,104,98,10,124,161,208,43,19,136,102,80,7,35,196,4,150,42,244,43,101,208,83,122,149,80,164,231,165,137,206,168,54,78,140,12,52,226,35,203,0,91,199,128,68,36,95,81,17,140,
98,150,33,201,27,156,102,170,142,68,188,11,64,50,59,80,65,84,185,109,251,6,254,250,199,77,223,122,252,210,177,33,127,126,100,129,126,37,92,188,102,192,238,187,182,112,228,203,33,119,236,222,75,157,25,
84,40,236,186,118,158,59,119,252,252,148,243,23,151,142,243,216,75,139,188,250,254,127,48,143,52,22,105,204,176,232,157,192,225,20,8,142,2,142,226,20,57,222,180,53,196,114,205,8,51,206,184,142,124,255,
102,46,68,19,130,9,127,127,235,32,47,237,251,140,115,207,44,249,209,25,37,143,223,179,141,167,223,62,200,211,255,56,196,176,49,98,116,6,125,56,107,174,224,182,237,235,249,226,127,67,214,156,59,199,173,
219,214,241,226,187,255,166,110,28,17,24,164,180,204,67,127,123,143,229,19,53,0,117,99,220,177,99,19,247,253,126,19,11,7,14,51,172,35,101,19,16,233,230,229,35,195,144,136,32,25,140,8,57,206,153,33,54,
227,16,51,24,221,0,210,22,83,115,33,152,210,68,136,38,68,23,220,115,69,23,71,196,40,11,71,197,41,84,248,233,218,1,55,108,158,231,217,133,69,46,221,112,30,191,219,114,33,239,124,180,196,48,197,158,126,
47,229,169,94,5,115,25,28,149,148,206,94,248,231,167,244,123,208,171,156,162,112,16,3,177,14,10,187,35,153,43,9,140,233,184,143,25,226,51,158,62,214,121,81,55,135,96,70,48,33,68,35,68,35,78,181,83,42,
80,106,250,82,149,194,142,45,235,57,118,124,196,254,79,142,114,124,88,115,251,246,75,184,233,178,11,120,251,131,37,0,6,189,244,232,247,222,114,197,41,215,58,186,60,100,174,74,221,92,161,142,170,119,242,
172,109,9,111,249,49,27,123,38,109,239,233,54,207,109,173,59,132,224,212,193,24,53,201,33,119,71,25,156,66,133,249,31,15,184,110,211,90,158,123,103,17,199,89,60,188,204,226,210,50,191,249,229,133,236,
253,232,8,78,98,17,192,238,103,246,177,252,213,40,165,172,224,236,220,186,145,235,55,175,229,217,133,67,28,89,254,111,154,89,144,213,51,24,174,10,64,242,128,134,59,117,52,134,181,81,149,70,169,97,156,
251,135,181,81,40,120,161,252,246,202,245,0,236,220,186,145,157,91,55,206,236,181,229,23,107,88,248,240,240,152,93,43,35,227,248,74,196,129,38,56,207,239,61,196,53,151,204,243,147,179,7,196,232,233,69,
192,87,19,32,50,227,233,109,209,156,190,172,243,155,50,19,130,165,224,141,130,103,134,144,219,93,225,252,115,230,184,241,215,235,120,242,205,143,121,241,221,207,40,243,100,15,240,224,159,174,102,219,230,
117,188,254,254,231,105,214,0,70,193,25,54,142,121,154,65,110,190,236,34,190,26,6,22,14,28,165,14,78,176,116,77,58,96,73,106,121,53,199,248,100,207,128,76,14,74,238,20,51,197,167,123,83,44,42,77,16,134,
35,201,169,6,86,134,160,234,236,220,186,129,165,99,43,60,254,202,193,44,175,180,82,139,240,232,11,255,226,47,127,248,21,151,255,108,158,97,157,0,185,255,246,171,102,118,255,120,233,255,60,252,228,126,
142,124,217,80,55,66,12,10,238,157,164,173,52,0,22,64,113,74,220,167,24,50,25,229,211,192,50,45,64,116,79,101,55,161,9,130,136,226,38,92,125,247,203,168,100,246,10,60,248,196,129,177,118,149,242,127,186,
71,81,231,149,125,95,240,198,254,215,81,73,19,250,83,111,125,78,136,173,102,229,196,232,212,33,249,137,145,176,50,82,66,4,247,22,148,239,127,14,97,28,235,214,139,156,145,198,12,41,199,163,188,96,57,89,
77,6,152,174,77,60,77,222,161,17,48,33,70,25,139,143,100,205,203,29,236,164,30,85,37,75,45,133,228,246,54,173,109,152,221,5,179,36,193,52,193,25,53,208,4,37,70,7,243,28,146,110,165,19,201,226,98,43,48,
150,137,31,85,22,185,218,129,69,199,61,242,233,96,8,14,30,133,152,5,195,24,79,77,38,109,87,214,118,197,45,96,101,33,20,58,209,175,102,212,226,20,247,172,121,37,80,162,129,155,143,171,102,247,90,86,15,
161,135,82,205,50,68,233,97,57,121,181,211,227,233,4,4,192,99,238,208,37,75,43,39,73,237,211,4,241,169,116,23,100,86,206,159,62,55,117,84,19,249,29,207,204,243,174,170,229,172,218,43,89,233,157,97,72,
98,73,15,65,177,177,174,98,167,15,16,78,186,172,127,243,93,124,83,0,91,16,191,147,142,214,109,66,254,214,255,67,198,128,60,176,235,53,126,176,213,97,95,15,0,165,191,61,154,200,132,84,173,0,0,0,0,73,69,
78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00137_png = (const char*) resource_NewJucerComponent_bmp00137_png;
const int TalComponent::bmp00137_pngSize = 4269;

















// JUCER_RESOURCE: bmp00138_png, 3644, "D:\Code\workspace svn\Tal-Dub-3-VST-Effect-juce\resources\bmp00138.png"
static const unsigned char resource_NewJucerComponent_bmp00138_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,3,87,73,68,65,84,120,218,236,153,65,104,34,103,20,199,127,179,76,116,38,104,209,196,
131,138,208,161,138,4,116,79,206,161,130,97,247,98,216,147,135,61,236,210,67,241,86,122,221,99,47,57,228,210,99,174,161,55,41,88,232,33,135,176,135,210,189,20,20,210,131,11,11,49,80,36,22,11,131,74,209,
85,170,100,212,12,76,15,198,49,38,217,236,54,217,204,90,152,63,12,204,124,111,120,223,155,247,127,239,125,111,190,79,40,22,139,38,14,150,6,34,192,183,191,200,152,167,111,49,199,3,48,38,142,87,108,101,
192,133,224,246,34,172,174,177,247,68,159,18,98,158,190,197,28,254,141,57,250,7,140,177,227,36,91,9,113,195,153,126,254,32,159,19,50,30,76,201,208,251,152,103,35,199,73,54,66,88,145,48,1,86,228,57,33,
24,19,48,198,83,50,12,135,16,59,97,2,194,202,216,90,42,30,124,44,197,106,60,200,78,62,67,52,228,187,86,190,153,140,176,147,207,220,203,71,189,120,170,178,147,207,144,77,41,87,100,9,37,192,78,62,115,167,
185,163,33,223,141,223,246,49,241,192,174,72,40,85,53,182,11,229,123,157,67,141,7,145,93,226,149,49,125,98,252,191,186,44,59,176,153,140,144,77,41,108,23,202,60,127,180,65,120,221,195,238,126,101,65,238,
247,74,28,28,158,16,94,247,144,75,199,8,175,123,0,232,13,70,148,170,26,149,90,251,157,250,235,173,62,107,30,137,132,18,176,222,11,175,123,136,134,124,148,170,26,155,201,200,66,196,103,83,202,59,245,95,
150,31,55,58,11,115,221,198,190,165,35,228,34,42,181,54,249,173,36,209,144,143,122,171,143,223,43,145,77,41,236,238,87,144,93,34,249,173,36,199,141,14,123,47,223,88,81,158,75,199,232,13,70,212,91,253,
27,245,110,38,35,150,99,212,120,144,74,173,205,104,60,207,16,191,87,226,217,227,13,122,131,17,223,255,244,59,250,196,32,151,142,89,250,155,221,33,207,30,111,208,236,14,217,46,148,45,123,102,184,139,125,
75,75,72,189,213,167,217,29,162,198,131,212,91,125,203,137,189,193,136,132,18,64,118,137,168,241,32,106,60,120,37,50,223,71,72,230,97,132,104,200,103,233,223,221,175,144,252,60,176,144,29,178,75,164,112,
120,98,149,178,131,195,19,18,74,128,132,18,192,239,149,144,93,34,175,94,55,0,208,39,6,165,170,198,243,71,27,0,124,17,246,221,218,190,165,37,100,230,188,92,58,70,180,230,179,28,119,17,123,47,223,208,236,
14,255,147,78,125,98,80,62,210,80,227,65,154,221,33,245,86,159,222,224,126,186,198,219,216,183,84,139,250,117,132,232,19,131,252,86,210,202,14,128,63,155,125,171,140,248,189,210,66,135,246,33,93,78,245,
175,14,9,37,64,230,97,132,242,145,118,109,118,206,244,207,26,128,217,253,113,163,195,113,163,179,32,151,93,226,194,250,115,87,251,108,207,144,139,245,118,134,203,209,63,67,249,72,35,155,82,40,85,181,133,
40,47,252,90,37,151,142,241,226,169,106,141,189,122,221,248,160,114,208,27,140,168,212,218,214,250,116,157,252,231,223,254,32,155,82,248,238,171,47,173,177,131,195,19,235,253,203,242,139,122,238,106,223,
123,127,20,139,197,162,249,205,143,26,230,160,141,121,218,115,126,12,109,239,115,37,132,85,63,130,55,200,15,95,71,62,93,201,114,112,83,201,18,93,32,186,231,251,42,14,108,221,203,66,116,79,57,152,17,34,
184,189,112,166,207,247,85,28,216,152,18,110,4,233,179,41,7,22,33,171,107,83,225,138,236,156,135,216,78,200,252,60,4,206,207,67,246,158,232,128,124,126,57,248,52,152,158,137,252,59,0,106,159,154,224,177,
186,221,55,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00138_png = (const char*) resource_NewJucerComponent_bmp00138_png;
const int TalComponent::bmp00138_pngSize = 3644;

// JUCER_RESOURCE: bmp00139_png, 5344, "D:\Code\workspace svn\Tal-Dub-3-VST-Effect-juce\resources\bmp00139.png"
static const unsigned char resource_NewJucerComponent_bmp00139_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,100,0,0,0,19,8,6,0,0,0,102,197,168,45,0,0,0,9,112,72,89,115,0,0,11,19,0,0,11,
19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,4,103,65,77,65,0,0,177,142,124,251,81,147,0,0,0,32,99,
72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,9,251,73,68,65,84,120,218,236,153,91,140,85,213,25,199,127,223,218,123,159,115,102,
6,228,34,140,220,20,24,110,10,162,18,137,151,22,172,86,154,20,107,176,168,209,216,155,182,77,172,105,155,216,86,109,211,62,216,70,155,38,237,83,155,154,190,88,47,209,134,250,210,214,104,219,104,20,42,
245,2,94,170,82,68,188,128,23,64,80,134,65,16,230,204,156,189,247,90,95,31,214,218,251,156,153,57,3,4,251,224,131,123,178,50,39,89,107,175,245,173,239,255,221,254,223,150,53,107,214,40,159,62,159,152,
39,6,184,245,161,11,1,80,114,28,25,74,14,56,20,7,140,142,151,8,72,203,239,226,81,109,191,182,117,78,71,89,119,172,79,113,182,8,72,203,225,58,108,211,225,115,170,199,118,118,241,154,180,185,155,30,225,
158,199,32,57,130,1,12,66,140,33,65,60,12,220,182,234,137,240,11,112,164,56,82,148,20,37,71,177,71,5,196,0,198,8,198,64,100,4,1,156,42,78,193,185,166,18,164,5,57,213,22,197,200,241,93,74,36,156,103,32,
14,231,75,216,203,185,66,225,254,220,104,216,92,238,20,117,224,156,30,241,236,226,110,136,127,223,24,105,190,163,254,125,199,241,3,34,68,8,49,138,197,80,193,80,105,122,136,247,140,20,165,17,254,103,1,
148,209,1,17,1,140,16,197,66,108,132,56,242,66,91,167,228,86,177,225,210,0,38,40,198,136,180,0,230,215,180,110,95,90,165,52,61,160,152,118,174,9,166,215,16,36,137,161,18,11,213,68,202,189,115,171,228,
54,40,85,32,138,188,108,170,144,229,74,154,107,144,209,27,207,16,203,215,230,249,113,36,126,4,99,51,6,114,11,185,85,127,7,7,214,105,91,249,142,236,129,5,32,49,66,130,9,90,246,94,19,0,241,97,42,13,94,210,
56,38,64,12,32,198,16,69,66,181,226,21,19,71,66,102,149,70,234,72,115,69,140,23,48,50,66,53,49,68,145,23,188,0,35,179,94,57,173,161,65,68,188,34,141,52,45,59,40,32,15,96,139,64,28,27,170,21,67,87,205,
208,89,141,136,35,176,14,26,153,35,183,193,16,140,80,137,132,56,246,72,15,166,142,70,166,164,97,141,211,38,24,78,117,8,32,213,0,118,18,238,101,196,223,45,205,28,185,11,242,88,109,43,159,58,80,70,243,192,
2,16,139,4,31,51,8,142,104,168,135,248,145,161,100,45,121,196,162,163,122,136,96,34,67,165,98,168,85,35,174,190,96,6,63,186,124,1,55,221,249,50,255,126,101,31,164,174,20,184,154,24,174,191,228,20,190,
119,233,92,150,221,180,174,180,228,52,88,172,128,15,59,120,139,174,196,134,36,162,12,115,206,41,141,76,25,72,29,141,204,95,34,73,252,185,99,187,132,251,111,62,155,238,241,53,30,222,184,139,123,30,127,
155,70,26,18,100,164,92,120,198,100,126,184,250,84,0,86,223,254,36,245,134,210,72,149,204,54,193,45,148,106,67,12,138,13,84,43,80,75,12,213,68,88,58,111,60,191,248,218,98,110,189,127,51,207,189,190,31,
27,188,35,43,0,25,38,95,154,43,4,79,106,7,135,18,33,40,6,80,12,74,20,244,77,145,67,92,200,25,249,144,65,153,71,70,62,138,16,153,136,36,142,168,86,160,146,248,195,107,21,165,86,113,88,103,17,241,86,152,
36,202,95,158,122,155,127,62,191,131,177,93,133,5,193,96,138,7,36,120,4,64,37,22,106,21,161,18,55,61,36,205,161,222,112,136,241,114,58,167,36,177,161,90,129,49,181,8,227,189,157,139,151,76,97,237,166,
93,236,63,156,161,78,137,99,97,229,210,41,212,27,57,157,213,152,73,39,8,135,7,161,145,249,61,243,156,50,71,168,147,210,75,162,72,168,37,62,20,86,43,194,184,46,47,219,184,46,97,242,56,193,169,15,205,89,
238,149,62,92,62,235,28,214,57,144,118,94,98,202,168,163,1,28,197,66,208,115,240,16,55,100,128,13,96,140,14,8,226,147,106,20,25,170,9,36,1,218,36,134,36,86,226,88,203,131,107,21,229,218,21,179,185,118,
197,92,174,249,245,191,248,193,151,23,49,251,164,177,124,247,15,27,24,72,61,24,87,126,118,22,83,38,116,112,247,99,111,176,240,148,113,92,183,98,62,167,76,30,3,64,239,193,65,254,250,204,59,60,176,254,93,
92,8,117,113,172,196,145,82,173,120,64,183,239,57,200,196,177,53,46,88,220,205,250,205,123,112,170,244,76,57,129,83,79,30,207,83,175,238,102,217,194,105,140,31,227,65,156,51,117,34,171,207,159,197,172,
147,198,150,251,255,227,249,29,172,125,121,55,170,112,102,207,68,174,90,222,195,204,110,127,254,230,119,250,2,32,134,147,38,196,204,152,52,134,43,150,245,112,242,164,246,242,229,214,145,57,135,58,215,
38,190,104,112,252,2,140,86,189,151,30,162,67,134,255,59,182,178,55,50,74,18,65,18,73,80,190,208,85,51,168,42,121,236,223,237,168,26,146,196,207,143,233,48,108,124,237,3,206,93,208,205,103,78,155,196,
75,219,251,152,50,161,147,111,92,60,151,159,220,243,44,211,79,172,114,203,229,103,176,101,199,126,254,248,232,43,56,7,103,245,76,230,59,43,79,101,103,111,63,107,55,125,0,105,145,108,253,144,144,47,94,
126,107,47,23,157,49,157,151,182,239,197,169,178,124,209,20,94,220,182,23,27,98,209,152,142,136,238,241,29,220,176,242,52,246,29,26,228,199,119,111,96,160,145,115,205,133,243,184,110,197,124,234,141,148,
93,251,14,243,253,75,23,242,94,95,63,63,189,119,3,39,116,38,124,243,11,167,1,208,85,139,152,122,98,141,27,46,89,200,214,93,251,185,235,209,45,228,86,71,200,231,141,204,199,25,29,1,135,163,40,87,100,152,
222,91,0,249,56,124,160,40,13,37,196,109,67,45,49,88,171,101,242,171,196,134,128,23,113,36,108,219,125,128,93,251,14,243,249,51,167,241,250,174,3,92,118,222,76,158,126,245,125,14,15,164,156,51,191,155,
142,106,204,210,121,221,44,157,215,61,228,172,5,51,198,177,126,243,94,178,80,202,182,198,125,84,121,113,123,47,231,159,58,149,5,211,199,243,94,95,63,75,231,117,115,199,195,47,115,250,172,73,101,225,48,
127,250,56,58,170,49,15,62,250,22,89,238,168,36,134,135,54,190,197,146,57,147,88,58,111,50,83,39,116,210,81,141,89,183,105,39,73,44,100,185,229,217,215,223,231,178,243,230,144,68,194,130,233,227,168,85,
98,150,244,116,179,164,167,189,124,198,200,199,35,134,199,251,168,250,42,169,40,1,203,40,105,66,101,99,65,141,79,216,197,108,110,21,65,120,102,235,251,92,181,124,46,11,103,78,96,217,162,41,252,252,79,
207,123,5,135,160,123,231,35,175,240,193,129,58,185,85,234,13,75,223,161,140,189,7,210,102,133,131,143,225,131,169,197,169,175,176,246,31,106,240,228,150,221,156,53,103,50,147,199,119,178,109,207,65,222,
235,27,96,193,12,143,218,64,234,176,86,203,152,112,36,66,168,218,60,167,184,91,102,149,40,148,212,247,173,123,149,157,189,135,201,242,118,242,29,63,227,53,45,124,180,28,82,150,102,102,196,92,49,84,5,231,
132,60,167,172,237,1,212,121,111,137,141,33,137,76,176,22,193,134,139,12,52,148,122,195,241,216,75,187,169,55,114,110,92,181,152,245,155,247,176,163,183,78,125,208,241,210,182,62,234,141,156,149,75,103,
83,77,18,234,13,199,178,69,211,184,235,198,207,177,120,214,196,80,225,248,145,230,74,255,160,195,133,138,231,96,191,101,195,107,189,156,62,243,68,150,47,154,198,35,47,236,228,163,186,101,32,245,128,124,
84,183,108,122,251,67,234,141,156,213,231,247,144,196,17,105,166,172,58,175,135,142,74,204,11,111,238,227,185,55,122,25,104,228,124,233,156,30,68,12,34,17,231,46,152,26,100,119,188,242,206,135,12,164,
57,43,206,154,57,170,124,138,215,77,59,189,21,122,149,182,122,13,30,210,92,228,135,18,13,73,62,163,85,211,214,25,178,92,72,51,33,245,85,27,183,92,121,198,136,181,223,254,237,134,146,172,29,232,119,68,
198,215,246,127,123,102,7,95,189,168,135,63,175,127,151,253,135,188,98,15,246,167,252,242,129,255,114,253,23,231,115,203,21,75,0,232,31,204,185,111,237,118,158,218,210,71,154,10,89,238,43,48,84,232,55,
224,212,123,222,190,131,150,15,15,245,179,110,211,30,78,159,57,129,167,95,237,195,57,232,31,244,128,236,59,104,217,127,168,159,223,63,188,149,171,151,207,230,55,223,58,175,76,202,247,62,254,38,27,183,
246,161,10,119,252,125,43,87,45,159,205,237,95,63,7,128,215,118,30,128,19,225,96,191,101,103,239,32,191,123,112,11,95,185,104,14,55,95,222,94,62,155,155,192,12,101,132,246,188,145,71,64,52,66,239,0,178,
102,205,26,253,217,67,103,227,24,192,49,136,163,113,76,60,196,8,36,177,161,163,98,232,172,26,186,106,17,157,85,79,20,11,38,94,144,165,146,252,137,39,134,5,239,40,216,181,210,108,181,8,129,148,37,30,52,
207,176,29,245,212,113,120,192,82,111,248,176,83,148,202,181,138,63,59,137,164,220,179,8,225,54,16,52,193,3,104,157,98,196,135,83,19,74,234,162,43,224,153,182,150,121,49,142,252,59,70,252,158,74,96,230,
26,10,9,241,173,20,215,70,190,70,230,200,66,212,24,94,246,122,136,162,178,143,101,168,98,168,97,232,224,87,171,254,83,120,72,92,82,121,193,133,74,185,73,96,218,123,8,224,132,60,23,6,16,80,67,150,27,34,
83,196,96,175,76,27,88,121,113,217,34,188,22,204,92,85,2,32,30,24,1,226,200,143,98,47,235,132,44,55,12,102,144,231,6,235,130,18,3,127,176,121,147,141,27,145,50,15,20,125,173,2,16,167,126,222,51,239,102,
206,240,196,176,185,62,50,82,42,189,88,167,128,181,173,32,106,217,110,25,46,159,179,126,67,209,118,241,101,104,235,68,66,115,177,104,48,198,62,145,36,161,201,85,36,60,83,214,200,163,166,63,245,249,194,
90,159,113,6,156,144,101,62,161,75,115,73,217,38,209,214,110,171,142,222,65,45,188,199,43,69,202,4,155,135,86,133,181,45,29,91,241,50,56,43,152,108,100,135,87,219,49,229,160,100,35,67,215,57,109,26,75,
49,223,238,46,174,104,165,13,235,34,183,202,135,163,100,226,71,239,101,85,16,42,24,146,214,28,18,99,168,224,66,95,165,96,143,71,229,33,128,90,200,29,56,35,100,249,208,139,210,98,249,173,205,59,61,74,139,
87,130,149,183,54,24,139,22,69,43,120,193,73,113,118,232,153,199,212,186,151,225,21,227,208,230,98,235,167,133,118,159,11,90,231,135,203,7,140,226,29,35,187,189,18,58,189,67,60,196,123,73,5,193,224,202,
190,202,209,137,97,243,195,64,51,7,216,255,195,71,26,61,82,89,58,188,204,80,142,88,198,30,73,236,227,157,63,114,115,125,52,32,142,254,61,164,4,228,182,85,79,124,250,169,238,19,242,252,111,0,230,42,5,241,
103,185,176,206,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00139_png = (const char*) resource_NewJucerComponent_bmp00139_png;
const int TalComponent::bmp00139_pngSize = 5344;
