#include "includes.h"
#include "TalComponent.h"

TalComponent::TalComponent (TalCore* const ownerFilter)
: AudioProcessorEditor (ownerFilter)
{
    Image backgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
	Image knowWhiteImage = ImageCache::getFromMemory(bmp00129_png, bmp00129_pngSize);
	Image knowRedImage = ImageCache::getFromMemory(bmp00130_png, bmp00130_pngSize);
	Image buttonImage = ImageCache::getFromMemory(bmp00132_png, bmp00132_pngSize);
	Image splashImage = ImageCache::getFromMemory(bmp00133_png, bmp00133_pngSize);

    MemoryInputStream mis(electronichighwaysign, electronichighwaysignSize, false);
    this->typeface = new CustomTypeface(mis);
    Font fontDigital(typeface);
    fontDigital.setSizeAndStyle(13.0f, 0, 0.7f, 0.0f);

    this->lowShelfGainSlider = addSlider(this, 21, 39, ownerFilter, knowWhiteImage, 186, LOWSHELFGAIN);
    this->highShelfGainSlider = addSlider(this, 84, 39, ownerFilter, knowWhiteImage, 186, HIGHSHELFGAIN);

    this->decayTimeSlider = addSlider(this, 145, 39, ownerFilter, knowWhiteImage, 186, DECAYTIME);
    this->stereoWidthSlider = addSlider(this, 208, 39, ownerFilter, knowWhiteImage, 186, STEREO);
    this->preDelaySlider = addSlider(this, 271, 39, ownerFilter, knowWhiteImage, 186, PREDELAY);

    this->drySlider = addSlider(this, 358, 39, ownerFilter, knowRedImage, 186, DRY);
    this->wetSlider = addSlider(this, 421, 39, ownerFilter, knowRedImage, 186, WET);

    this->realStereoModeButton = addNormalButton(this, 455, 2, ownerFilter, buttonImage, false, REALSTEREOMODE);
    this->powerButton = addNormalButton(this, 540, 2, ownerFilter, buttonImage, false, POWER);

	// Version info
	this->versionLabel = new Label(T("Version Info"), T("V 0.96"));
    this->versionLabel->setFont(fontDigital);
	this->versionLabel->setBounds(20, 3, 100, 20);
	this->versionLabel->setColour(Label::textColourId, Colour((juce::uint8)200, (juce::uint8)200, (juce::uint8)200, 1.0f));
	addAndMakeVisible(this->versionLabel);

	this->infoLabel = new Label(T("Info text"), T(""));
    this->infoLabel->setFont(fontDigital);
	this->infoLabel->setBounds(100, 3, 100, 20);
	this->infoLabel->setColour(Label::textColourId, Colour((juce::uint8)200, (juce::uint8)200, (juce::uint8)200, 1.0f));
	addAndMakeVisible(this->infoLabel);

    this->talMeter = new TalMeter(ownerFilter, 77, 225, 20);
    this->talMeter->setBounds(500, 30, 77, 254);
    addAndMakeVisible(this->talMeter);

    this->splashScreen = new TalSplashScreen(splashImage, 0, 311, 329, 51);
    addAndMakeVisible(this->splashScreen);

    updateParametersFromFilter();

    setSize(backgroundImage.getWidth(), backgroundImage.getHeight());

    // register ourselves with the filter - it will use its ChangeBroadcaster base
    // class to tell us when something has changed, and this will call our changeListenerCallback()
    // method.
    ownerFilter->addChangeListener (this);
}

TalComponent::~TalComponent()
{
    getFilter()->removeChangeListener(this);
    deleteAllChildren();
}

ImageToggleButton* TalComponent::addNormalButton(Component *component, int x, int y, TalCore* const ownerFilter, const Image buttonImage, bool isKickButton, int parameter)
{
	ImageToggleButton *imageToggleButton;
	component->addAndMakeVisible(imageToggleButton = new ImageToggleButton("Toggle Button", buttonImage, false, isKickButton, parameter));
    imageToggleButton->setBounds(x, y, buttonImage.getWidth(), buttonImage.getHeight() / 2);
	imageToggleButton->addListener(this);
	return imageToggleButton;
}

ImageSlider* TalComponent::addSlider(Component *component, int x, int y, TalCore* const ownerFilter, const Image sliderImage, int height, int parameter)
{
	ImageSlider *imageSlider;
	component->addAndMakeVisible(imageSlider = new ImageSlider(sliderImage, height, parameter));
	imageSlider->setBounds(x, y, /*sliderImage->getWidth()*/ 70, height + sliderImage.getHeight());
	imageSlider->addListener (this);
	return imageSlider;
}

//==============================================================================
void TalComponent::paint (Graphics& g)
{
    // just clear the window
    g.fillAll (Colour::greyLevel (0.9f));

    Image backgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
    g.drawImage (backgroundImage,
                 0, 0, backgroundImage.getWidth(), backgroundImage.getHeight(),
                 0, 0, backgroundImage.getWidth(), backgroundImage.getHeight());
}

void TalComponent::sliderDragStarted (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getFilter();
        filter->beginParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderDragEnded (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getFilter();
        filter->endParameterChangeGesture(values["index"]);
    }
}

void TalComponent::changeListenerCallback (ChangeBroadcaster* source)
{
    // this is the filter telling us that it's changed, so we'll update our
    // display of the time, midi message, etc.
    updateParametersFromFilter();
}

void TalComponent::sliderValueChanged (Slider* caller)
{
    TalCore* const filter = getFilter();

    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        filter->setParameterNotifyingHost(values["index"], (float)caller->getValue());
    }

    this->updateInfo(caller);
}

void TalComponent::updateInfo(Slider* caller)
{
    if (caller == preDelaySlider) this->infoLabel->setText(juce::String((int)(audioUtils.getLogScaledValue((float)preDelaySlider->getValue()) * 1000.0f)) + T(" ms"), dontSendNotification);
    if (caller == lowShelfGainSlider) this->infoLabel->setText(juce::String(lowShelfGainSlider->getValue(), 4), dontSendNotification);
    if (caller == highShelfGainSlider) this->infoLabel->setText(juce::String(highShelfGainSlider->getValue(), 4), dontSendNotification);

    if (caller == stereoWidthSlider) this->infoLabel->setText(juce::String(stereoWidthSlider->getValue(), 4), dontSendNotification);
    if (caller == decayTimeSlider) this->infoLabel->setText(juce::String(decayTimeSlider->getValue(), 4), dontSendNotification);

    if (caller == drySlider) this->infoLabel->setText(juce::String(audioUtils.getLogScaledValueInDecibelFilter((float)drySlider->getValue(), 2), 4) + T(" dB"), dontSendNotification);
    if (caller == wetSlider) this->infoLabel->setText(juce::String(audioUtils.getLogScaledValueInDecibelFilter((float)wetSlider->getValue(), 2), 4) + T(" dB"), dontSendNotification);
}

void TalComponent::buttonClicked (Button* caller)
{
    TalCore* const filter = getFilter();

    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        filter->setParameterNotifyingHost(values["index"], (float)caller->getToggleState());
    }
}

//==============================================================================
void TalComponent::updateParametersFromFilter()
{
    TalCore* const filter = getFilter();

    // we use this lock to make sure the processBlock() method isn't writing to the
    // lastMidiMessage variable while we're trying to read it, but be extra-careful to
    // only hold the lock for a minimum amount of time..
    filter->getCallbackLock().enter();

    // take a local copy of the info we need while we've got the lock..
	float roomSize = filter->getParameter(DECAYTIME);
	float preDelay = filter->getParameter(PREDELAY);

	float lowShelfGain = filter->getParameter(LOWSHELFGAIN);
	float highShelfGain = filter->getParameter(HIGHSHELFGAIN);

	float stereo = filter->getParameter(STEREO);

	float dry = filter->getParameter(DRY);
	float wet = filter->getParameter(WET);

	float realStereoMode = filter->getParameter(REALSTEREOMODE);
	float power = filter->getParameter(POWER);

    // ..release the lock ASAP
    filter->getCallbackLock().exit();

	this->decayTimeSlider->setValue(roomSize, dontSendNotification);
	this->preDelaySlider->setValue(preDelay, dontSendNotification);
	this->lowShelfGainSlider->setValue(lowShelfGain, dontSendNotification);
	this->highShelfGainSlider->setValue(highShelfGain, dontSendNotification);
	this->stereoWidthSlider->setValue(stereo, dontSendNotification);

	this->drySlider->setValue(dry, dontSendNotification);
	this->wetSlider->setValue(wet, dontSendNotification);

    this->realStereoModeButton->setToggleState(realStereoMode > 0.0f, dontSendNotification);
    this->powerButton->setToggleState(power > 0.0f, dontSendNotification);
}

//==============================================================================
// Binary resources - be careful not to edit any of these sections!


static const unsigned char temp1[] = {120,156,237,93,119,124,21,197,22,190,116,130,138,128,210,164,163,128,60,68,16,164,223,157,43,130,5,20,41,162,136,136,130,235,21,145,77,112,4,41,
  82,68,68,121,74,111,34,177,208,213,80,21,81,90,118,22,84,202,163,136,72,81,164,248,20,4,169,74,83,4,203,59,223,153,220,13,252,146,188,108,
  96,137,55,154,63,252,157,143,59,179,103,206,204,156,57,109,118,227,109,86,216,236,249,116,247,184,174,102,249,59,187,118,121,162,119,231,190,229,239,237,
  218,37,46,16,8,212,29,223,46,84,62,240,39,161,28,129,218,197,219,134,226,98,107,23,223,98,208,63,7,89,0,143,23,46,231,88,45,198,55,8,
  185,0,77,102,44,253,83,80,119,219,218,222,230,134,144,11,78,108,44,236,88,104,2,48,195,57,137,227,22,17,23,75,143,4,86,86,25,173,172,103,
  219,55,21,46,184,243,214,159,12,235,240,22,51,25,12,248,126,178,178,208,25,192,12,231,74,122,28,157,15,22,172,235,36,63,142,223,93,192,79,185,
  143,3,160,179,25,206,157,218,227,231,13,97,185,130,225,41,23,232,199,243,164,246,120,74,153,117,231,188,231,204,244,124,206,41,68,53,195,249,146,58,
  143,105,126,143,120,255,155,131,74,238,218,157,143,65,221,233,243,141,230,135,182,40,57,247,169,18,198,138,197,182,2,157,154,247,53,253,195,53,3,194,
  220,99,123,247,58,250,145,121,117,243,42,231,120,75,166,178,71,227,17,12,170,149,126,91,112,143,31,59,57,2,143,128,50,15,128,245,21,149,42,191,
  96,166,224,81,240,8,134,141,200,97,134,243,39,73,70,123,25,164,33,29,108,106,144,126,106,104,141,105,254,153,112,1,154,204,100,241,35,124,92,198,
  145,145,206,27,26,178,184,194,69,164,117,197,143,204,199,157,96,100,198,231,45,1,214,196,93,164,200,170,37,139,31,227,187,6,23,8,148,234,244,144,
  136,11,95,150,180,48,43,11,228,19,3,215,188,170,100,255,114,181,24,96,240,97,167,135,40,57,114,106,87,209,172,82,23,5,74,141,250,7,210,18,
  187,111,175,22,98,64,206,4,155,31,49,95,108,98,131,7,168,12,223,245,190,1,208,235,160,52,238,121,240,29,91,94,43,114,24,135,142,29,182,65,
  235,78,175,161,248,135,92,9,143,41,187,89,55,163,231,117,52,74,254,179,115,13,12,59,241,209,60,60,188,25,123,213,224,69,162,76,76,14,199,106,
  87,127,189,40,83,121,173,178,182,173,250,92,92,63,154,64,232,177,117,188,85,86,179,252,15,139,233,235,243,56,172,176,93,151,230,114,172,239,19,231,
  136,151,62,201,237,88,203,183,222,46,118,148,136,87,242,167,38,117,89,61,231,140,168,32,62,233,60,77,201,86,229,139,138,99,49,51,20,196,253,189,
  199,12,37,235,119,125,211,0,40,191,160,146,49,172,232,72,37,91,140,95,210,232,145,98,82,129,134,239,186,73,255,80,170,211,78,123,64,206,170,70,
  235,125,131,109,126,132,180,34,17,60,64,229,216,66,65,6,163,134,116,18,7,170,61,103,203,142,173,94,16,234,202,237,54,40,243,0,232,59,238,126,
  85,115,88,31,81,114,81,111,37,115,174,138,21,155,122,14,85,185,135,119,16,21,250,140,163,109,190,60,105,51,182,173,106,37,238,172,84,197,145,175,
  62,87,144,193,187,101,55,26,165,58,149,118,72,53,134,26,5,102,20,113,6,141,28,98,60,219,62,183,35,105,44,227,155,161,135,213,87,195,149,49,
  186,203,151,74,54,44,146,83,228,173,179,92,229,173,115,139,248,240,30,91,89,0,151,63,57,72,89,152,132,11,236,27,30,85,86,153,152,198,194,5,
  224,100,117,222,249,112,50,224,166,35,241,155,146,1,63,222,231,189,71,146,193,91,207,38,42,185,63,60,70,208,73,80,180,136,52,197,109,74,110,239,
  254,161,88,243,193,33,181,226,245,133,162,231,117,36,230,172,213,31,8,200,61,97,201,20,193,19,185,109,210,48,158,89,100,170,102,108,233,38,247,138,
  81,67,202,59,146,180,142,65,171,242,19,196,43,181,138,59,146,54,75,76,59,80,208,1,189,251,253,156,250,7,82,39,133,30,180,45,138,31,201,211,
  230,63,10,60,64,101,75,163,44,131,101,47,252,110,112,143,229,91,87,65,3,21,40,243,0,0,83,244,224,81,240,8,134,141,200,97,134,175,72,58,
  35,5,147,182,101,228,212,3,198,232,230,5,29,11,160,193,221,87,227,40,28,48,106,14,59,109,31,89,244,181,209,233,215,253,182,36,21,197,120,54,
  78,0,61,100,75,154,176,65,63,218,115,159,122,205,232,219,107,131,45,47,27,250,178,113,116,192,26,155,212,192,160,125,178,161,142,70,204,138,215,237,
  113,39,22,4,105,53,108,57,104,100,238,32,49,75,124,188,240,228,70,244,223,50,9,243,84,50,62,46,72,39,55,88,50,254,104,144,122,12,9,146,
  90,27,35,118,156,101,42,73,209,97,14,140,183,31,248,150,134,109,155,40,105,173,197,181,226,121,27,116,115,195,83,182,5,64,6,70,89,103,215,58,
  2,138,194,96,230,51,79,42,9,64,19,85,99,11,37,178,130,202,7,254,88,34,242,116,111,163,38,119,252,64,144,25,80,242,88,229,185,226,230,51,
  205,213,248,109,9,226,203,132,59,20,111,64,157,93,141,213,211,253,38,139,178,135,235,41,217,224,238,241,162,126,215,50,106,137,61,74,180,222,151,83,
  201,214,185,134,99,24,27,246,144,206,182,77,170,60,82,208,220,109,210,116,209,233,215,46,182,92,252,245,107,34,97,111,208,166,179,201,84,86,107,48,
  159,52,247,22,27,54,21,43,36,223,88,248,153,232,114,178,168,2,253,106,184,80,22,64,204,238,235,72,89,48,157,211,21,118,40,158,215,8,117,92,
  79,167,71,227,43,28,6,55,239,202,65,251,119,101,210,190,193,56,124,246,67,53,199,34,155,148,12,42,118,40,224,72,178,140,162,248,177,24,103,71,
  137,107,196,186,80,62,71,210,175,100,246,114,59,180,104,70,241,45,1,199,26,119,162,150,209,225,149,162,240,24,115,130,43,171,20,114,44,28,198,213,
  159,159,36,45,111,51,199,48,150,237,131,26,98,111,97,213,158,55,126,106,242,145,194,9,98,103,73,123,209,200,5,180,54,202,66,19,128,36,51,98,
  144,5,80,80,12,154,181,146,179,103,205,50,200,168,169,49,213,87,26,164,148,54,142,41,92,84,34,76,59,148,105,25,235,29,233,45,171,8,76,152,
  61,112,77,49,209,160,198,48,91,146,81,165,229,110,103,99,98,160,60,67,82,30,237,202,93,128,38,121,176,96,47,6,100,75,224,192,109,82,150,177,
  130,230,98,147,55,18,96,111,65,19,72,217,18,45,50,187,130,52,46,209,218,184,110,154,128,80,144,146,204,234,53,112,86,98,76,245,219,148,220,144,
  231,67,65,118,94,129,242,12,97,39,92,192,147,255,232,166,133,12,36,0,153,4,133,157,134,69,103,110,163,134,252,87,45,250,101,170,222,197,184,203,
  55,136,185,247,94,233,88,117,118,173,32,151,123,181,99,65,168,3,213,254,84,242,243,49,163,217,150,224,172,243,86,205,28,219,131,55,17,19,3,53,
  99,143,196,183,134,246,57,114,250,250,23,25,116,57,57,65,28,222,178,71,73,88,38,90,88,182,80,61,30,126,85,255,64,155,194,38,100,194,27,197,
  21,206,20,123,51,240,96,175,246,105,211,146,12,102,142,61,97,112,143,132,189,137,6,25,7,5,202,60,0,18,114,172,82,232,241,139,216,203,158,92,
  116,108,149,203,137,200,97,134,11,101,32,84,75,39,134,244,30,31,22,78,26,116,221,67,159,11,88,21,11,160,207,123,229,216,255,55,250,180,233,36,
  114,53,69,146,250,208,57,23,31,15,164,35,65,251,98,96,89,173,126,193,101,198,194,110,121,177,5,141,69,149,185,87,105,64,203,147,72,174,100,185,
  177,185,225,51,182,85,246,240,34,184,111,219,66,152,67,198,203,176,96,216,249,23,210,53,49,98,71,63,219,130,39,228,167,0,200,90,58,22,57,77,
  177,116,10,113,198,145,167,32,135,68,189,42,73,12,56,231,50,149,155,43,11,138,220,123,195,15,26,24,203,42,107,99,235,2,110,162,165,229,206,102,
  44,58,145,186,40,139,76,187,225,2,150,3,77,0,102,44,108,19,243,62,126,56,94,243,6,96,150,207,228,235,151,12,184,9,106,160,121,163,19,179,
  156,49,191,127,50,96,222,104,98,222,225,171,147,228,223,187,253,67,129,19,109,145,29,72,6,245,94,34,214,205,15,181,101,32,201,166,27,0,136,138,
  72,185,28,73,187,17,132,201,2,189,162,197,47,138,127,8,61,191,69,145,177,55,72,185,148,204,149,176,215,64,204,2,133,2,181,180,102,17,123,138,
  244,146,65,204,138,82,142,5,17,0,244,49,224,159,0,234,44,95,172,228,201,5,249,25,96,111,31,157,182,94,73,10,156,140,251,158,222,173,64,75,
  46,250,69,255,112,102,127,126,7,61,16,33,242,35,96,18,97,102,134,139,102,64,151,61,233,105,177,52,115,139,11,58,10,102,184,248,57,6,223,149,
  240,194,143,157,25,46,145,196,16,139,192,142,131,142,147,241,69,187,63,148,133,179,255,188,60,171,172,187,223,191,85,188,177,144,130,2,0,178,197,134,
  214,247,8,224,38,184,73,242,213,202,250,186,206,116,177,250,200,239,36,106,73,111,73,152,247,197,188,38,137,33,41,112,35,50,167,172,201,141,254,108,
  48,86,89,55,252,249,134,96,128,200,147,130,118,173,200,240,227,214,175,185,191,16,176,7,172,69,165,155,108,214,77,223,213,220,164,159,2,31,51,92,
  202,53,21,95,240,47,240,10,6,3,10,152,5,121,67,101,125,210,185,162,32,94,218,207,33,234,230,95,216,103,223,243,96,21,36,5,10,7,194,96,
  49,192,7,192,12,151,78,226,140,128,148,67,125,0,178,45,54,11,230,130,28,123,126,182,45,82,74,195,5,232,108,134,203,36,61,78,198,88,80,74,
  160,215,103,194,27,19,149,5,171,126,116,192,157,8,15,198,234,41,35,116,217,121,150,214,144,76,186,193,224,242,39,3,186,9,230,155,59,67,120,114,
  199,202,130,243,135,67,231,62,156,113,80,22,99,188,220,114,138,178,38,223,50,75,48,40,86,112,28,71,152,102,184,108,146,24,244,56,135,169,86,161,
  237,95,144,251,28,99,51,67,0,51,92,46,169,15,166,51,172,40,245,193,238,48,192,83,72,41,204,112,249,192,132,83,37,67,113,225,10,154,198,78,
  75,124,73,12,221,210,218,182,0,150,239,47,70,187,221,122,113,50,64,147,201,157,222,44,49,84,113,167,253,115,23,235,54,23,160,73,119,122,187,189,
  173,59,253,116,244,168,110,115,1,154,116,167,132,159,79,232,78,103,86,22,113,184,205,5,104,210,157,182,151,162,104,8,96,224,190,234,186,205,5,104,
  210,157,58,60,89,83,119,10,212,105,166,219,92,128,38,51,92,49,105,150,181,127,41,175,135,5,224,209,126,88,248,124,50,208,195,146,132,186,19,0,
  183,145,240,201,64,119,2,3,150,13,128,69,2,3,23,104,217,102,255,158,168,59,1,112,27,24,184,64,119,2,3,158,0,0,203,13,6,46,224,9,
  48,39,238,4,192,109,96,224,2,61,203,107,147,102,105,132,202,235,189,4,224,45,4,35,23,232,189,4,35,238,4,192,109,167,198,30,77,6,186,19,
  24,144,88,138,57,237,41,52,88,51,112,1,154,52,39,238,4,192,109,96,224,2,221,105,254,201,123,12,218,33,101,1,208,198,40,107,109,141,98,194,
  5,104,210,195,113,39,0,110,195,40,46,208,157,176,207,220,9,128,219,142,157,90,148,12,116,39,136,194,154,10,192,10,10,81,92,160,53,117,94,252,
  9,221,9,128,219,126,155,88,36,228,2,221,9,162,176,58,3,176,22,67,20,23,104,117,198,40,220,9,128,219,48,138,11,116,39,204,156,70,210,75,
  64,3,56,60,115,23,160,73,15,199,202,7,192,109,24,197,5,201,7,195,61,61,220,134,153,187,64,119,130,40,220,9,128,219,32,138,11,116,39,204,
  156,59,1,176,138,99,230,46,208,157,32,10,171,49,128,119,93,199,40,46,208,157,192,160,99,176,166,230,148,82,215,209,148,154,174,131,129,11,180,174,
  95,151,166,221,194,10,184,64,171,49,22,156,53,20,128,6,210,58,231,2,173,161,158,116,29,163,112,39,0,110,195,40,46,200,192,129,72,105,75,193,
  192,5,126,107,104,74,171,140,81,92,144,65,53,102,149,73,93,141,51,160,161,158,244,42,165,19,0,3,23,100,64,249,176,134,220,9,128,219,176,134,
  46,72,203,157,96,20,23,104,229,171,148,164,124,158,108,40,70,227,78,0,220,134,209,92,144,138,134,178,166,96,45,93,144,1,189,194,40,220,9,128,
  219,48,138,11,146,117,157,245,10,128,213,9,11,238,130,180,28,121,26,42,147,174,149,193,164,120,229,1,120,193,49,41,23,36,219,134,116,181,193,211,
  30,98,20,222,67,0,222,58,140,226,2,237,81,49,10,119,74,221,20,233,141,174,156,17,143,234,41,132,202,100,109,240,228,118,51,217,20,249,103,64,
  60,233,149,103,199,148,158,54,120,179,13,85,146,84,230,200,3,218,91,91,0,236,80,131,99,23,36,3,61,1,180,177,108,0,44,18,218,92,160,101,
  67,27,15,11,192,163,161,205,5,122,216,235,147,134,69,204,202,250,5,144,150,73,74,233,197,160,160,46,208,170,131,112,148,55,28,32,45,27,129,54,
  222,112,128,180,54,220,83,212,237,41,216,247,20,26,84,61,39,216,231,165,0,248,127,135,54,221,165,192,90,242,82,0,164,21,77,122,114,158,231,153,
  75,94,1,180,185,32,3,75,225,37,109,8,255,235,156,40,41,221,89,226,16,241,44,83,183,21,201,78,33,221,20,240,239,29,182,120,138,153,207,219,
  195,126,87,223,168,25,184,128,247,48,92,45,35,219,227,105,229,49,57,94,10,0,94,1,210,119,225,2,189,20,23,184,242,169,164,241,158,86,30,203,
  196,157,0,184,13,203,228,130,84,10,2,105,173,188,167,69,189,33,35,199,31,109,85,231,87,16,220,169,218,241,254,134,69,171,158,12,208,164,59,241,
  246,0,120,207,12,210,216,67,79,85,150,125,247,118,52,184,211,216,94,55,216,220,230,2,52,153,225,234,73,179,204,170,27,30,190,49,163,86,58,221,
  117,243,109,155,194,53,206,89,220,243,227,48,172,169,11,46,42,150,78,99,113,211,117,14,88,211,109,205,146,34,96,86,125,172,169,11,208,100,134,107,
  94,92,220,154,74,10,237,41,213,241,148,103,123,74,80,252,43,23,121,242,69,41,79,17,70,113,65,6,124,145,39,99,231,169,18,228,95,253,198,83,
  152,236,41,179,242,45,76,246,22,201,221,228,187,141,240,47,60,190,68,181,240,180,92,159,167,122,179,167,90,137,167,44,166,214,57,22,144,215,11,128,
  151,9,26,227,130,228,42,113,38,90,25,79,41,180,231,106,94,186,137,175,103,151,247,247,61,135,181,163,192,157,164,85,6,241,228,78,60,57,1,255,
  114,141,191,183,54,220,156,145,234,167,231,42,85,186,153,231,121,113,86,74,111,159,74,122,154,242,114,40,249,68,251,115,57,228,41,110,240,164,50,158,
  238,125,60,133,4,254,21,204,61,105,67,157,127,132,109,240,28,214,165,91,98,200,228,11,62,79,86,6,109,124,45,151,122,193,92,95,203,101,174,149,
  225,21,231,78,0,220,134,133,118,129,86,190,186,89,93,249,60,229,57,254,153,162,104,172,185,121,190,5,244,199,123,213,203,136,202,120,138,34,61,59,
  166,75,80,0,72,25,18,112,1,192,191,43,176,204,62,209,245,179,79,116,52,23,200,255,222,145,109,3,223,175,123,81,102,96,149,1,96,77,193,235,
  90,46,208,42,67,171,166,181,1,192,103,109,72,197,202,64,20,238,4,192,109,16,197,5,186,19,68,225,78,0,222,3,30,148,69,93,112,81,42,3,
  6,46,184,244,42,147,242,165,65,94,39,238,4,144,214,59,119,13,207,41,81,249,115,71,224,233,222,213,83,245,233,188,123,87,79,161,102,90,111,98,
  158,87,125,226,90,51,24,184,32,149,234,19,183,129,129,11,244,237,76,163,236,245,202,200,122,113,167,71,22,182,213,86,230,163,201,197,181,188,46,64,
  147,25,169,143,26,60,220,177,83,245,108,190,188,114,1,154,204,112,208,247,55,4,60,221,124,120,190,160,114,147,162,75,255,134,128,113,78,117,243,252,
  186,3,68,114,65,42,117,135,139,210,47,127,235,14,174,225,229,101,130,188,46,72,37,211,252,127,139,122,190,58,195,103,186,32,149,76,243,255,197,165,
  233,217,249,176,72,179,174,12,185,93,112,81,53,220,84,86,62,165,203,75,37,0,242,116,178,61,101,43,41,163,111,76,202,5,90,9,67,23,247,230,
  125,38,87,170,51,217,29,95,104,4,119,75,20,164,15,151,254,226,19,242,78,243,229,21,248,104,188,248,244,175,248,245,23,100,43,233,190,221,238,159,
  174,55,206,200,29,157,127,239,157,102,242,253,125,52,86,244,253,171,6,95,224,247,23,238,235,162,145,23,72,245,18,108,233,157,84,209,239,123,235,141,
  90,83,92,128,38,143,119,191,183,70,241,221,111,38,27,218,44,93,121,77,215,208,102,178,121,140,194,91,134,112,19,223,43,62,23,88,110,76,229,197,
  50,207,106,156,238,213,169,39,13,245,100,175,178,108,105,175,233,95,99,212,50,249,75,142,44,235,226,60,233,149,167,75,235,204,181,50,225,219,162,216,
  89,250,23,243,121,82,190,104,245,131,254,24,181,40,244,94,222,110,212,110,207,72,201,197,147,99,242,207,231,100,242,139,233,254,106,195,165,186,95,77,
  229,42,194,63,109,184,227,175,41,144,68,99,250,153,201,111,134,251,247,173,204,223,59,82,187,51,171,151,53,178,108,210,24,141,49,159,127,153,165,39,
  243,216,236,239,247,197,65,42,218,224,169,194,154,242,186,45,149,247,72,189,124,133,234,163,109,104,238,187,247,242,47,65,243,84,131,244,239,35,97,79,
  201,80,38,135,154,119,101,36,212,188,64,211,157,74,60,234,223,59,230,158,142,152,127,166,219,147,45,245,116,88,253,205,159,211,141,108,61,105,195,221,
  25,189,235,205,196,2,156,127,202,119,158,94,177,166,160,205,5,25,120,9,34,165,54,164,242,54,149,111,217,73,184,69,118,156,117,105,46,56,51,241,
  175,12,120,210,6,79,175,44,120,178,13,153,28,140,221,147,213,53,52,147,175,37,51,89,141,61,125,201,146,101,51,129,150,81,240,38,73,86,178,161,
  89,54,121,204,228,242,70,171,139,43,199,70,99,113,254,146,150,99,255,2,3,18,101,31,206,132,91,251,110,138,252,115,150,254,221,93,122,122,209,39,
  219,94,101,182,242,181,249,107,130,176,75,164,87,105,189,58,27,141,229,216,104,84,62,79,241,85,38,107,232,189,89,61,82,243,100,212,162,81,249,178,
  172,81,107,155,209,119,239,179,100,113,62,10,99,25,111,185,215,125,89,253,68,103,231,94,62,70,219,158,84,230,254,140,254,61,248,191,237,82,120,123,
  173,222,83,105,202,147,207,241,228,78,188,216,208,112,59,223,255,255,21,158,78,116,74,217,160,41,46,184,20,31,239,100,98,158,147,201,69,193,104,140,
  71,61,25,144,7,162,184,216,236,233,19,84,79,39,218,147,54,120,42,236,122,250,0,43,10,205,99,184,253,197,197,126,169,216,6,79,25,166,127,182,
  212,211,61,97,86,245,94,225,7,255,17,47,84,123,254,244,205,83,54,231,233,219,236,172,88,250,244,166,50,29,254,154,151,182,60,253,9,11,79,127,
  157,162,118,134,255,47,10,105,89,229,243,254,240,68,90,119,189,23,248,210,214,5,190,151,28,126,40,105,123,60,185,186,11,244,98,169,92,65,120,250,
  243,7,231,89,64,254,211,27,48,161,46,208,127,179,164,227,197,233,87,42,214,198,147,126,101,217,183,206,50,249,165,192,135,207,241,24,233,254,81,12,
  79,126,209,191,55,38,188,156,162,240,35,89,252,69,169,112,167,192,243,53,191,16,58,164,154,23,127,110,233,10,163,185,96,94,188,231,197,77,25,215,
  47,63,247,47,104,112,92,31,238,28,197,33,187,167,84,48,251,3,248,168,252,0,254,209,40,14,65,207,211,208,236,15,224,179,63,128,191,72,231,105,
  102,127,0,255,207,248,0,254,177,236,15,224,163,217,197,121,210,171,104,252,0,62,28,197,206,210,191,152,207,147,242,69,171,31,252,135,127,0,255,120,
  246,7,240,217,31,192,187,218,208,37,251,3,248,236,15,224,163,58,82,123,34,171,151,53,178,108,210,24,141,49,95,38,95,131,119,205,254,0,62,154,
  63,128,127,50,251,3,248,104,254,0,190,91,246,7,240,217,31,192,187,218,96,101,241,123,157,11,72,134,178,210,7,240,177,217,113,86,246,7,240,81,
  253,1,124,92,86,215,208,236,15,224,179,110,38,208,61,251,35,140,44,127,97,16,141,229,141,167,178,63,128,143,246,47,60,163,236,45,208,176,204,254,
  0,62,251,3,248,191,202,94,61,157,253,1,124,52,223,93,70,99,124,149,201,26,218,35,171,71,106,158,140,90,52,42,95,150,53,106,61,179,63,128,
  143,230,47,90,158,201,234,39,58,59,247,202,236,239,230,122,101,127,0,159,229,63,128,239,157,253,1,124,246,7,240,127,85,189,175,79,20,23,155,179,
  63,128,247,209,83,244,205,254,0,62,138,189,87,248,217,127,196,11,213,217,31,192,251,152,46,244,203,200,137,246,244,62,22,38,199,123,8,144,214,183,
  237,158,172,114,74,31,157,202,205,191,39,131,235,233,26,55,229,202,95,232,103,235,253,163,37,69,78,229,118,225,188,20,57,173,255,85,189,167,20,217,
  139,166,134,7,248,254,1,188,127,47,148,121,138,220,162,240,219,246,240,192,115,236,124,186,239,207,120,82,29,255,82,26,79,74,232,101,229,195,207,5,
  106,23,223,34,226,98,7,228,76,48,238,250,236,50,199,26,191,45,65,184,192,124,177,137,109,161,9,192,140,253,106,184,18,177,85,139,57,214,246,54,
  55,24,46,40,25,127,116,153,133,38,0,51,60,136,56,182,133,50,22,223,98,4,2,129,65,22,192,227,133,203,57,86,139,241,13,66,46,64,147,25,
  75,255,20,212,221,6,163,144,11,78,108,44,236,88,104,2,48,195,207,251,206,113,112,210,172,119,237,206,39,6,228,172,106,72,13,18,140,229,91,87,
  25,15,215,62,99,200,19,27,199,24,59,74,92,35,48,65,234,33,172,65,35,115,27,45,141,178,66,222,245,217,58,99,219,170,207,13,208,17,59,250,
  25,252,195,184,19,181,140,21,175,47,100,42,243,159,157,203,96,194,146,41,70,222,58,183,24,178,85,249,9,196,249,118,163,65,141,97,76,229,184,19,
  11,130,0,160,36,73,35,254,225,167,38,117,237,65,35,135,24,160,248,129,193,232,46,95,26,155,27,158,74,100,9,177,192,17,145,205,240,11,73,147,
  40,83,121,173,40,19,147,195,145,0,149,114,255,168,30,123,98,41,83,249,209,77,11,25,180,232,253,158,152,57,246,132,146,199,42,207,21,37,23,253,
  162,64,95,187,237,55,253,3,173,155,115,235,21,239,137,118,165,114,57,114,232,203,11,196,244,245,121,28,208,165,83,242,234,31,66,143,93,230,52,249,
  233,109,166,178,250,119,227,24,220,184,242,21,177,250,243,147,74,118,93,250,130,56,93,97,135,250,162,221,64,49,229,218,169,202,122,234,195,247,53,168,
  214,96,190,152,154,247,53,101,209,58,49,144,35,167,118,21,13,106,92,166,182,238,105,47,104,105,108,73,171,32,86,86,25,109,220,243,96,21,209,235,
  224,49,67,158,92,144,31,243,19,143,23,158,204,11,47,183,173,106,197,128,118,206,24,184,166,152,144,229,23,204,12,78,124,52,143,0,85,87,110,55,
  248,135,132,189,137,80,196,224,61,15,190,99,200,152,21,165,12,108,224,232,46,237,152,202,29,37,226,13,189,163,241,188,111,252,67,159,247,190,49,94,
  125,110,164,81,115,216,105,67,182,222,55,216,192,210,130,146,128,130,127,168,48,161,52,107,5,168,44,48,163,136,0,128,18,144,86,4,173,217,179,254,
  37,72,14,37,251,151,171,37,182,119,175,163,190,26,46,196,180,3,35,149,69,51,211,243,238,219,107,131,193,160,179,179,213,224,37,25,83,253,54,94,
  27,185,53,96,138,185,247,94,233,44,186,122,54,83,89,103,249,98,6,183,220,241,177,168,216,161,128,222,206,121,117,243,58,145,253,53,195,67,146,118,
  124,250,250,23,105,245,150,7,37,0,84,102,230,51,79,138,82,157,118,26,210,57,222,82,236,60,91,72,144,234,8,168,187,69,122,40,90,149,47,42,
  100,226,227,61,233,56,127,98,128,214,157,94,195,224,31,136,93,240,240,22,147,169,124,251,129,206,12,176,55,111,63,240,109,80,150,110,114,175,232,244,
  235,254,224,187,101,91,50,149,115,70,84,96,0,74,27,151,200,63,92,43,158,183,31,157,118,55,83,230,1,48,66,61,205,198,131,37,36,166,137,17,
  145,205,216,35,139,190,54,88,120,0,8,63,247,169,215,12,22,190,76,229,230,198,202,42,133,4,173,90,144,133,223,222,102,78,144,133,39,96,64,120,
  80,22,30,0,178,62,241,229,112,166,116,60,134,50,216,220,240,25,131,133,111,150,255,97,3,178,226,152,177,240,164,80,141,0,64,89,120,0,200,74,
  182,195,96,225,193,3,32,98,240,88,66,8,31,17,217,12,191,24,104,59,170,10,153,162,41,215,86,8,241,193,5,128,188,77,54,151,9,65,237,228,
  254,193,215,132,160,178,93,78,22,101,42,191,171,89,152,193,139,255,189,34,52,247,169,18,134,188,113,101,1,126,4,148,121,0,144,16,137,195,138,22,
  12,145,1,74,228,71,168,171,13,30,160,204,20,160,89,254,178,33,18,36,145,135,197,35,17,57,204,216,245,21,149,96,110,0,96,159,175,196,34,193,
  18,181,255,246,61,1,1,232,104,48,149,45,141,183,24,76,120,99,162,96,137,118,158,29,205,143,128,50,15,0,176,39,221,21,44,17,30,129,0,224,
  193,18,129,41,192,85,131,23,9,150,8,195,226,145,136,28,102,236,77,173,247,25,204,13,0,236,113,242,88,34,24,55,8,128,221,99,137,250,246,106,
  193,0,210,176,68,232,30,249,143,121,0,128,61,158,103,137,240,8,4,0,15,150,8,76,1,112,230,88,34,12,139,71,34,114,152,225,151,146,206,15,
  172,93,213,30,43,148,4,168,59,125,190,154,51,98,42,83,216,5,6,100,169,233,12,125,164,36,233,147,160,41,169,155,207,52,23,63,118,114,148,4,
  32,75,167,114,174,138,21,205,15,181,85,114,76,243,123,196,101,67,99,20,206,64,249,5,149,12,235,174,207,238,194,210,8,62,235,12,158,109,223,148,
  155,100,157,93,141,69,142,61,63,219,56,55,252,48,90,192,13,148,217,127,218,180,36,143,71,230,218,56,59,151,4,32,41,12,72,132,83,194,34,66,
  93,1,64,121,18,0,165,155,108,230,30,160,18,142,8,0,246,244,230,51,107,149,132,16,237,74,125,194,163,144,121,33,153,73,178,92,9,123,21,89,
  45,113,199,35,191,42,89,239,165,202,180,209,121,29,208,210,149,201,248,0,144,198,58,45,122,215,36,93,41,228,200,199,158,104,40,42,244,185,202,57,
  58,224,78,166,146,228,103,208,121,231,195,226,243,49,212,3,182,29,143,128,50,15,128,149,5,242,57,48,40,60,10,214,14,195,98,169,88,142,28,123,
  122,177,96,88,110,150,116,255,224,120,1,209,177,33,60,23,236,16,64,100,203,204,240,80,159,54,241,147,206,211,20,41,145,160,83,160,216,212,145,48,
  202,110,214,77,144,215,208,63,208,194,178,168,173,247,229,84,48,78,212,242,165,13,74,255,217,73,246,179,173,13,63,120,217,208,151,109,30,246,88,204,
  12,155,204,59,83,73,50,48,0,37,94,250,7,114,29,137,232,1,202,143,0,84,236,240,10,76,110,34,75,74,6,206,134,14,241,35,24,174,197,248,
  37,228,232,218,11,142,54,122,111,120,66,224,32,129,66,179,249,135,134,69,114,10,50,120,226,185,217,87,8,73,66,9,132,46,95,38,220,193,20,10,
  194,128,24,235,30,164,139,252,8,40,243,0,0,211,21,175,223,44,106,14,235,99,176,94,98,88,50,131,90,14,168,16,4,131,78,145,57,76,100,37,
  131,232,3,215,188,202,148,20,181,46,3,80,126,4,0,211,71,15,94,15,60,2,0,30,100,107,53,83,44,33,70,225,53,5,32,25,212,177,152,235,
  244,54,64,14,242,17,81,113,62,160,202,160,236,156,1,200,94,243,249,32,135,171,207,71,245,239,244,249,0,229,243,1,128,243,193,39,8,199,1,143,
  128,50,15,128,117,33,191,207,199,191,3,251,168,83,92,236,115,179,71,136,35,241,20,246,34,56,96,0,29,168,223,181,140,99,33,30,229,95,96,85,
  25,96,41,114,37,92,231,88,253,130,141,4,128,25,126,57,80,105,127,37,114,117,100,171,66,52,57,5,103,15,191,101,47,254,186,68,136,66,173,68,
  73,209,10,123,182,26,239,230,208,174,142,98,77,118,44,191,136,189,20,171,198,5,37,133,133,20,238,13,75,4,229,51,2,64,143,169,110,131,142,137,
  206,206,125,138,118,60,87,104,88,209,145,138,44,8,83,30,5,32,50,172,25,91,170,83,233,16,148,66,2,212,120,247,121,85,104,123,17,166,136,137,
  25,236,94,241,135,8,223,120,189,146,143,20,59,38,202,30,94,100,131,61,197,140,182,164,121,9,196,214,207,182,207,29,2,149,175,62,87,144,1,92,
  43,251,120,48,165,72,208,142,140,98,178,97,153,118,160,160,99,189,91,118,163,129,211,96,33,90,100,64,170,204,225,178,213,244,163,241,226,230,93,20,
  115,227,244,143,216,113,86,221,20,215,157,169,36,181,101,128,61,60,80,237,79,37,243,116,111,35,10,214,60,170,64,43,239,255,94,255,240,214,179,137,
  10,170,67,58,169,36,121,15,156,94,133,144,23,20,60,24,32,231,184,227,145,190,74,194,125,209,126,42,80,10,125,244,15,71,226,55,177,146,83,122,
  160,216,251,209,217,118,104,113,113,198,29,89,110,227,141,12,16,139,126,153,144,207,145,71,226,91,179,200,144,148,69,167,128,88,208,214,57,49,187,103,
  240,124,205,216,31,59,133,68,219,81,251,149,4,56,116,236,176,170,179,188,190,158,196,128,239,203,113,95,168,208,152,234,121,28,137,72,156,150,223,65,
  84,12,202,222,22,128,204,170,65,24,59,219,196,160,172,80,33,3,234,240,206,91,74,38,236,13,26,240,117,8,213,65,41,196,206,201,206,15,222,240,
  233,126,147,245,176,103,215,58,42,34,135,25,235,28,223,40,86,44,54,148,4,160,73,219,200,51,105,89,108,249,241,192,217,136,48,141,37,246,40,166,
  108,177,1,64,73,193,109,169,65,81,181,236,133,1,226,200,162,7,21,103,46,212,93,81,126,198,148,153,2,68,70,49,99,199,84,95,73,10,84,15,
  22,96,165,232,216,234,5,213,191,220,2,166,242,134,63,223,96,208,58,215,112,49,126,91,85,37,145,211,144,137,180,65,57,101,3,160,253,164,136,117,
  2,83,137,12,26,224,95,247,47,198,177,176,153,41,173,141,29,25,197,12,191,18,136,189,124,3,29,214,122,47,189,205,199,206,66,88,193,39,242,227,
  129,213,52,128,249,228,38,228,165,12,160,90,234,202,146,142,69,231,68,31,214,88,90,53,12,101,75,0,68,183,159,253,48,27,251,98,240,164,145,144,
  66,47,65,229,172,213,181,25,64,83,16,60,73,164,152,136,150,143,14,88,195,84,34,31,3,160,19,203,17,183,133,24,154,65,82,72,102,35,173,64,
  72,103,147,113,93,98,32,230,70,226,1,42,47,127,114,16,3,104,23,71,230,139,126,153,10,176,12,148,83,62,0,178,218,54,214,9,193,144,92,89,
  96,40,146,21,182,182,185,135,119,80,18,89,221,188,186,47,41,50,227,224,163,117,137,150,72,129,190,244,201,167,250,135,23,234,29,80,253,130,203,72,
  151,254,80,18,9,225,153,253,249,29,168,14,168,36,70,12,200,62,136,97,167,115,59,178,195,43,19,56,141,90,252,245,107,58,93,166,244,156,1,105,
  172,232,121,29,245,248,186,206,116,177,176,91,94,135,124,20,63,106,253,54,122,174,6,36,174,152,220,241,3,101,125,159,56,135,1,114,78,113,255,125,
  91,21,12,129,221,236,144,146,129,192,32,206,160,225,67,56,147,198,145,1,232,186,52,151,120,242,205,51,144,249,19,227,155,161,135,21,40,229,131,250,
  7,210,13,69,234,68,153,229,27,74,30,58,102,145,36,67,224,169,180,251,91,177,216,166,248,164,184,138,108,175,25,30,150,20,253,132,239,186,137,34,
  134,206,65,9,64,107,20,204,127,182,58,83,137,180,20,0,89,44,108,175,196,121,131,241,197,182,98,221,37,50,40,50,224,148,248,44,105,132,117,183,
  144,22,145,10,43,9,125,32,127,165,98,118,95,39,200,197,160,62,80,93,80,204,175,192,29,84,194,104,1,128,238,155,60,93,255,112,252,112,188,194,
  163,52,101,101,181,206,117,57,36,179,89,50,18,118,105,68,84,51,60,60,80,101,238,189,228,82,122,54,110,17,34,99,103,91,131,70,54,13,65,203,
  44,148,44,0,36,202,12,48,33,5,102,140,101,42,225,144,0,160,253,36,116,35,174,7,224,124,131,110,41,99,41,254,1,174,143,236,32,253,112,24,
  171,102,176,131,133,130,177,163,197,65,4,40,19,179,130,189,184,213,102,223,173,33,23,172,123,232,115,101,29,168,22,98,32,223,122,54,24,10,126,245,
  163,186,162,69,189,208,117,21,78,43,114,33,55,161,180,228,188,220,178,60,83,11,78,198,5,155,122,14,85,86,209,106,165,137,221,80,37,255,109,85,
  101,144,115,85,245,16,45,183,194,26,132,16,96,78,186,190,118,136,76,137,178,174,26,92,47,25,144,19,82,22,154,0,184,47,37,19,234,231,14,213,
  67,181,110,238,168,228,224,122,85,67,152,33,216,131,242,128,164,35,26,192,10,90,7,11,254,12,89,6,201,213,71,126,167,101,170,136,82,69,8,84,
  6,191,170,193,128,76,120,136,34,168,160,172,241,110,99,2,185,19,207,236,191,147,23,223,140,221,186,103,187,230,2,64,65,157,146,0,214,237,223,168,
  181,125,183,35,209,87,146,214,85,196,253,152,139,15,13,40,157,179,23,25,32,209,207,91,39,224,72,132,140,88,167,227,135,175,17,241,151,29,84,114,
  217,11,191,27,196,135,66,233,67,6,102,42,183,6,246,144,102,76,86,20,224,162,80,161,36,10,47,176,156,112,81,172,161,55,181,126,156,77,55,178,
  76,182,229,67,223,94,198,160,239,184,109,58,144,208,162,14,89,22,17,217,12,143,240,189,40,56,210,119,142,163,124,231,56,218,119,142,99,2,227,200,
  252,196,197,194,128,146,7,208,150,180,90,131,249,10,81,50,168,68,213,18,0,81,9,135,220,48,23,119,191,255,177,2,69,225,143,127,184,243,214,159,
  20,236,126,199,86,164,37,240,47,100,7,28,108,57,106,90,214,200,169,7,140,30,141,41,44,129,43,129,222,128,94,51,128,162,33,128,43,127,222,161,
  80,124,1,229,225,0,16,174,215,186,121,23,217,160,152,25,228,142,118,195,109,51,101,9,1,34,34,155,225,177,231,76,98,132,58,174,187,224,97,248,
  180,117,161,213,138,75,176,200,5,161,85,152,12,123,176,89,171,63,64,84,185,19,174,76,129,118,51,127,208,63,208,1,227,56,5,84,82,68,201,0,
  117,33,76,83,214,239,250,166,1,27,81,161,207,56,166,188,0,0,160,183,247,207,233,240,15,152,56,87,39,80,221,3,15,0,184,182,219,38,93,230,
  176,132,40,180,70,68,54,195,227,2,253,40,216,136,139,133,63,226,157,0,128,172,196,132,169,188,189,255,96,6,200,18,121,39,74,117,122,136,19,5,
  80,222,9,0,136,56,99,126,127,193,59,129,204,1,59,129,74,14,239,196,198,121,147,4,239,68,175,131,146,79,48,40,239,4,0,22,190,248,150,65,
  76,121,56,0,114,197,188,152,20,168,190,204,11,127,219,164,97,76,89,66,128,136,200,102,214,86,163,241,231,236,0,171,17,0,30,70,48,195,106,52,
  236,244,16,177,119,251,135,234,206,91,159,226,217,89,112,217,172,70,175,212,154,136,26,154,2,101,53,2,128,214,76,59,48,146,169,68,66,6,80,252,
  216,203,188,71,114,233,148,151,4,180,134,92,62,83,73,81,42,3,80,86,35,0,76,188,245,190,193,186,72,12,30,0,93,78,78,16,172,70,144,16,
  106,20,17,217,204,218,103,96,130,239,246,237,213,0,105,15,237,41,22,153,52,92,17,199,182,13,93,64,26,173,44,52,1,152,225,137,129,66,219,219,
  208,240,235,43,10,74,113,227,208,103,75,67,23,176,175,71,19,128,25,126,45,48,247,222,81,196,25,199,100,238,83,37,28,153,43,225,49,113,227,202,
  2,14,10,47,160,28,27,1,32,73,90,251,80,33,135,93,97,159,113,197,28,220,82,128,74,187,89,55,6,36,47,47,9,71,216,16,156,171,22,79,
  247,43,231,64,93,65,145,242,49,64,133,156,210,48,71,142,45,20,164,233,95,205,195,129,242,225,7,40,185,168,183,160,80,140,38,63,41,48,241,209,
  219,104,62,136,110,56,42,238,95,174,86,8,217,32,40,197,222,202,2,160,9,240,37,70,104,107,192,84,239,253,86,59,180,117,79,123,37,15,22,172,
  27,162,156,81,81,52,69,105,127,11,101,1,148,137,105,172,172,150,70,217,100,128,38,114,224,149,185,111,55,243,250,16,73,161,40,226,170,26,162,229,
  64,104,203,236,45,0,220,56,89,148,167,135,40,150,83,214,186,80,62,148,176,249,146,132,203,31,86,225,196,255,136,154,195,78,107,80,177,195,43,74,
  2,208,54,33,116,16,44,3,0,15,29,190,235,253,100,192,50,80,74,141,44,139,146,71,71,96,34,18,209,30,162,37,80,112,179,0,184,206,2,176,
  169,103,62,103,246,172,89,58,190,7,128,161,177,74,55,217,172,65,225,196,152,16,29,118,101,33,8,227,95,176,134,0,102,44,98,211,247,126,59,161,
  172,30,141,71,48,224,12,134,43,57,1,147,169,133,157,119,1,248,176,46,244,222,240,132,202,87,162,158,64,136,205,201,46,214,108,212,144,78,122,110,
  0,60,165,58,187,86,24,46,224,185,81,14,142,250,184,194,13,21,175,47,146,183,102,149,186,40,80,22,19,128,7,164,4,217,0,128,177,102,64,145,
  154,193,233,13,174,50,139,111,9,56,184,72,99,241,161,107,46,224,57,98,102,60,199,112,252,57,85,36,10,187,21,23,134,96,37,185,84,196,0,79,
  113,19,236,10,3,84,7,40,155,84,92,114,194,47,102,236,201,5,255,22,92,44,2,192,19,168,176,66,235,101,158,54,205,56,15,173,223,181,140,78,
  76,113,243,7,128,178,11,172,182,196,229,28,46,136,80,122,1,229,154,63,0,74,10,148,128,25,52,114,209,100,64,198,210,230,84,21,0,217,184,129,
  27,30,8,199,215,77,148,116,27,184,212,67,177,15,148,236,83,23,6,160,116,254,150,241,15,168,113,162,148,71,89,183,109,33,133,34,99,101,75,228,
  224,244,156,194,154,146,210,43,137,53,68,197,0,116,228,212,174,250,135,27,87,190,162,48,236,254,193,241,138,179,46,202,22,201,62,94,37,64,37,238,
  48,1,176,239,148,229,40,217,217,185,79,80,46,204,214,30,148,175,4,1,96,71,228,107,239,98,169,165,6,168,3,226,80,90,40,25,242,233,164,4,
  65,208,250,40,28,126,30,15,201,51,18,50,216,26,78,204,224,52,1,80,32,229,187,87,228,127,168,172,129,114,249,12,0,133,17,196,227,36,149,178,
  176,37,92,236,70,141,40,79,247,47,184,182,65,14,220,230,221,163,181,178,35,219,105,134,95,135,165,38,5,193,165,39,146,200,164,187,192,28,10,137,
  26,167,153,184,136,65,222,73,203,106,112,152,143,114,0,50,83,20,150,57,85,165,253,97,0,202,201,44,0,82,70,90,1,131,115,72,56,109,20,244,
  144,84,32,187,228,187,94,164,155,160,60,77,0,78,72,137,41,103,168,216,108,0,146,197,224,28,54,102,197,235,40,55,43,178,60,40,212,144,229,126,
  35,176,230,131,67,36,59,185,58,232,180,45,223,105,244,21,23,46,145,240,115,37,19,174,25,0,69,126,210,56,27,73,38,74,212,164,126,185,117,138,
  2,189,66,69,11,150,155,60,132,190,78,220,81,34,222,6,253,120,96,53,21,185,95,84,232,49,97,201,20,190,151,48,232,72,83,60,149,87,128,114,
  141,2,224,142,71,250,146,31,124,81,201,135,98,135,115,153,27,85,39,154,135,146,11,187,173,18,69,38,47,71,225,79,144,112,156,76,241,213,50,5,
  61,108,9,44,154,153,54,9,216,26,98,102,35,85,163,196,207,230,18,5,201,222,8,23,203,172,223,52,85,36,79,137,48,114,92,29,37,221,70,173,
  198,94,240,241,183,172,231,102,44,165,192,184,102,80,178,93,253,245,226,170,240,60,148,96,68,185,141,243,116,56,131,160,1,34,162,7,234,178,15,233,
  170,43,0,57,100,46,159,255,113,255,235,74,162,118,133,64,131,82,108,125,1,83,166,242,90,142,60,112,7,255,105,211,73,138,239,6,30,174,253,172,
  2,157,61,235,95,250,7,100,165,56,105,180,149,137,252,86,5,182,18,177,1,236,134,68,77,130,146,212,32,20,110,123,155,57,137,60,44,69,33,118,
  68,14,51,252,166,239,209,195,91,190,115,156,28,168,127,247,213,196,81,188,181,75,116,218,89,214,145,0,251,7,95,227,144,225,20,175,111,42,238,200,
  95,115,127,129,186,165,147,167,205,127,152,178,187,2,232,23,92,166,123,108,239,254,33,63,2,202,60,0,182,4,174,117,144,55,87,43,93,89,63,210,
  123,195,245,204,3,148,153,2,188,190,233,75,81,121,127,37,61,44,30,137,200,97,198,146,63,209,220,0,192,30,134,137,199,67,113,15,2,96,233,89,
  34,148,131,0,72,207,116,15,148,131,241,8,40,243,0,0,251,175,235,92,171,37,194,35,16,0,60,88,34,48,5,192,33,96,137,48,44,30,137,200,
  97,198,226,202,28,101,26,4,138,33,188,141,82,181,71,142,80,215,165,148,25,52,58,117,90,156,154,240,167,106,86,105,191,176,111,248,78,89,27,215,
  77,35,240,168,6,100,113,108,9,128,186,35,74,118,180,25,65,220,20,193,64,25,88,10,148,70,45,0,156,105,11,182,211,5,92,53,133,1,220,220,
  240,84,16,213,76,58,50,65,62,93,184,221,2,37,99,102,91,0,100,162,148,5,255,85,101,238,102,37,201,46,178,95,37,173,53,138,76,14,56,18,
  78,236,221,28,121,29,184,59,92,174,194,5,48,128,62,115,249,73,223,138,83,216,55,115,108,15,225,2,158,50,202,245,40,70,226,80,113,197,29,111,
  153,152,47,230,66,232,38,86,31,249,93,191,118,242,115,159,159,81,96,23,51,159,57,168,172,38,155,223,228,203,50,139,162,119,118,191,146,150,134,139,
  51,160,187,87,252,161,127,0,251,61,87,173,209,149,48,0,30,24,107,13,96,134,167,4,136,137,136,11,79,77,122,1,110,226,234,73,90,247,1,234,
  142,111,135,215,230,242,134,92,160,117,127,226,234,146,130,218,109,235,225,124,251,147,65,147,14,77,29,11,77,0,102,120,90,70,94,217,244,244,138,171,
  167,247,58,51,249,83,237,76,254,52,198,211,87,47,158,190,217,244,252,2,111,186,111,173,250,246,183,255,194,211,125,87,194,25,190,115,156,233,59,199,
  183,125,231,248,142,239,28,223,245,157,99,130,239,28,103,249,206,113,182,239,28,231,248,206,113,110,0,239,149,196,197,110,234,57,84,116,51,159,80,156,
  29,185,96,227,188,73,202,66,19,128,25,158,231,251,240,243,125,231,248,158,239,28,223,247,157,227,2,223,57,126,224,59,199,133,190,115,252,208,119,142,
  31,249,206,113,145,239,28,23,251,206,113,137,239,28,151,250,206,113,153,239,28,19,125,231,104,251,206,81,249,206,209,241,157,227,114,223,57,174,240,157,
  227,199,190,115,252,196,119,142,159,250,206,113,165,239,28,87,249,206,113,181,239,28,215,248,206,241,63,190,115,92,235,59,199,117,190,115,92,239,59,199,
  13,190,115,252,204,119,142,27,125,231,248,185,239,28,55,249,206,241,11,223,57,110,246,157,227,22,223,57,110,245,157,227,54,223,57,126,233,59,199,175,
  124,231,184,221,119,142,95,251,206,113,135,239,28,119,250,206,113,151,239,28,119,251,206,241,27,223,57,254,215,119,142,223,250,206,241,59,223,57,238,241,
  157,227,94,223,57,126,239,59,199,125,190,115,220,239,59,199,31,124,231,120,192,119,142,7,125,231,120,200,119,142,135,125,231,120,196,119,142,71,125,231,
  248,163,239,28,127,242,157,227,49,223,57,30,247,157,227,9,223,57,158,244,157,227,41,223,57,254,236,59,199,95,124,231,120,218,119,142,191,250,206,241,
  140,239,28,207,250,206,241,55,223,57,254,238,59,199,63,124,231,248,167,239,28,169,107,224,127,51,0,85,123,0,0};
const char* TalComponent::electronichighwaysign = (const char*) temp1;






















// JUCER_RESOURCE: bmp00128_png, 48600, "E:/Code/tal/tal-reverb-3-juce/resources/bmp00128.png"
static const unsigned char resource_TalComponent_bmp00128_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,76,0,0,1,107,8,6,0,0,0,35,37,214,208,0,0,0,9,112,72,89,
  115,0,0,15,97,0,0,15,97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,1,52,151,73,68,65,84,120,218,236,
  253,107,176,101,201,117,30,136,125,43,115,159,122,116,85,161,159,168,174,110,0,141,238,106,52,30,36,208,64,131,2,53,122,80,212,208,24,143,169,71,
  112,108,135,35,198,176,77,91,97,43,38,28,19,166,127,56,194,166,29,225,240,56,28,214,48,236,31,246,104,98,60,33,115,60,148,127,8,18,41,145,
  18,99,164,161,71,130,20,34,91,34,64,145,64,3,141,198,163,209,85,93,143,238,122,118,117,189,159,247,236,204,229,31,249,216,153,185,51,247,235,236,
  115,238,169,238,123,128,219,117,239,185,231,238,189,87,230,202,149,95,174,199,183,232,255,244,127,249,143,249,252,109,66,246,197,26,90,41,243,61,1,26,
  26,74,105,247,163,253,15,0,5,48,24,171,189,194,103,96,128,25,204,0,179,253,30,12,115,11,243,62,252,207,229,75,17,249,111,64,160,224,95,251,
  59,34,108,223,139,173,204,86,118,39,175,17,58,26,135,162,240,20,200,14,242,178,18,0,18,100,63,179,9,217,9,16,128,208,2,26,170,247,163,21,
  75,48,49,136,5,52,106,48,51,20,43,64,51,88,48,22,188,0,22,12,82,18,12,5,146,50,209,27,202,204,185,155,107,177,197,115,238,244,92,219,
  57,111,244,222,127,31,174,141,220,124,55,74,239,199,129,236,34,37,33,214,44,55,77,255,51,102,176,214,94,207,157,206,155,255,55,99,80,190,134,149,
  156,40,210,125,191,222,105,136,236,12,104,96,169,30,64,215,122,229,245,187,33,51,49,220,150,120,155,18,218,209,64,175,74,99,108,199,173,89,83,104,
  214,84,56,190,27,29,38,30,101,71,9,132,170,170,0,33,218,235,169,75,183,220,218,113,27,157,213,41,99,67,96,229,166,237,210,137,89,134,145,177,
  245,47,158,166,15,6,210,104,232,90,163,230,229,104,81,143,29,102,84,231,111,19,126,253,59,85,225,30,75,179,104,44,72,50,223,203,196,72,206,53,
  192,129,242,17,131,236,66,39,216,127,173,114,83,246,158,92,48,222,212,252,134,8,108,32,131,249,215,255,220,97,240,105,70,153,6,126,132,96,54,206,
  70,110,39,95,243,180,221,11,157,252,53,189,188,20,200,13,2,147,92,81,4,154,48,12,139,142,191,37,59,231,2,16,108,140,146,174,64,154,1,174,
  205,88,72,2,80,1,2,48,255,81,0,164,253,62,150,191,25,41,97,190,23,110,206,69,56,138,67,39,100,252,0,209,184,207,147,5,6,68,218,252,
  27,232,179,91,7,221,151,162,70,215,41,167,227,4,118,27,27,141,153,63,26,39,63,141,86,118,16,107,171,231,193,65,140,117,176,206,237,6,79,93,
  247,109,228,4,17,152,236,220,219,77,141,75,155,186,31,11,7,146,150,192,78,93,66,180,227,45,58,175,233,179,131,246,10,142,109,137,102,175,103,104,
  173,2,29,95,143,220,247,20,219,209,208,174,16,129,33,226,245,198,35,54,223,49,159,109,125,164,255,179,94,175,136,1,90,152,67,131,123,250,214,129,
  179,116,248,118,186,148,223,55,24,98,184,62,240,26,117,103,208,231,135,234,206,152,57,235,248,60,175,114,109,30,185,20,134,126,158,205,190,161,96,246,
  142,190,179,81,178,230,127,249,21,61,96,198,149,54,23,214,118,143,138,158,96,61,104,52,134,49,233,226,230,150,39,202,63,10,167,66,166,32,35,253,
  218,74,87,67,102,166,7,130,37,247,123,78,141,7,71,240,145,182,73,118,119,98,179,202,76,186,6,148,253,151,235,242,188,67,4,150,157,35,249,27,
  249,146,147,36,111,243,233,41,51,239,45,176,148,232,47,115,91,71,56,220,44,131,3,199,22,234,123,170,143,148,145,41,222,36,195,241,224,104,126,169,
  235,247,189,47,209,124,77,245,152,108,179,45,97,228,109,98,56,198,156,232,85,4,42,74,199,12,222,254,181,164,25,164,21,128,218,157,197,59,116,164,
  173,67,196,233,184,101,236,203,222,235,33,121,181,64,204,232,151,216,78,87,34,183,12,98,235,124,106,23,53,89,5,14,191,226,141,134,243,64,140,183,
  79,225,91,27,8,199,192,49,220,240,41,243,149,206,27,33,55,14,91,6,22,57,4,190,198,195,2,93,7,39,225,246,38,152,151,61,62,109,81,215,
  166,188,181,27,91,44,111,118,78,153,253,87,12,2,131,241,202,233,54,111,241,230,150,147,33,146,131,59,244,157,91,64,217,207,189,3,143,131,101,151,
  198,105,137,15,16,104,138,244,130,179,186,71,217,241,205,0,118,206,128,249,45,212,43,63,239,254,113,21,160,53,136,151,86,8,29,217,200,230,43,163,
  87,145,254,180,117,114,239,245,176,173,133,213,94,162,23,144,109,220,120,112,178,225,37,160,33,90,228,92,6,30,220,246,88,165,64,138,182,122,82,203,
  158,48,202,130,203,100,35,225,116,243,224,57,245,102,246,249,142,231,43,244,152,232,198,107,162,20,8,10,77,140,134,19,227,213,55,118,219,234,101,105,
  235,50,101,116,53,221,248,169,181,17,114,162,15,188,221,94,53,78,159,53,63,85,148,243,174,181,60,37,57,207,220,88,79,128,48,185,46,226,225,7,
  76,148,221,212,147,49,227,140,183,41,5,236,204,45,48,222,142,0,108,155,142,113,162,95,26,80,75,192,123,154,18,157,99,180,14,100,89,189,122,104,
  14,33,123,175,226,129,200,67,154,105,235,92,116,42,157,148,27,7,76,45,227,200,92,60,73,118,41,115,186,145,100,129,215,182,45,244,208,131,212,242,
  46,165,96,137,139,11,154,138,99,195,91,233,105,161,226,6,31,60,183,226,200,216,181,167,156,51,160,26,109,47,21,111,239,156,163,224,17,36,230,66,
  72,57,220,216,146,53,196,253,160,108,59,55,244,220,161,38,152,187,174,67,79,198,203,52,121,141,63,236,152,137,19,208,192,109,80,73,37,32,144,130,
  7,46,28,62,182,18,47,112,230,160,109,237,169,170,129,101,13,96,105,14,94,169,199,140,11,192,50,8,205,53,160,113,47,44,247,48,30,35,154,245,
  109,177,141,20,115,153,5,178,23,148,27,62,113,21,66,104,89,175,16,119,158,2,202,39,108,110,109,42,219,179,208,51,121,86,156,3,147,253,227,215,
  246,50,117,156,150,118,83,137,57,5,123,161,161,70,123,44,88,32,206,191,40,128,166,86,152,110,139,141,92,112,218,39,230,130,225,70,199,198,22,135,
  11,182,62,87,47,245,50,101,194,30,81,40,58,245,144,114,9,84,37,215,24,29,150,251,32,188,210,48,116,162,75,220,85,60,195,29,182,146,91,215,
  220,182,176,92,191,167,89,131,150,202,20,52,121,111,117,10,156,74,33,183,76,142,211,30,10,121,248,188,76,46,20,35,101,227,117,26,248,170,6,35,
  178,221,80,250,150,119,169,189,25,82,214,92,52,101,53,190,224,195,189,199,108,43,137,26,163,178,13,106,159,51,96,212,50,126,212,242,202,228,124,44,
  230,59,251,9,230,100,144,204,111,152,9,187,46,118,80,238,77,92,202,61,139,13,125,219,83,226,74,133,201,163,194,70,118,128,136,19,77,9,126,191,
  53,94,22,238,1,189,156,245,200,113,66,197,65,110,28,200,87,236,131,200,150,81,83,168,251,91,2,148,40,221,208,11,96,169,164,239,65,85,151,213,
  234,96,205,179,127,239,151,255,212,19,32,75,43,241,27,127,124,13,199,142,84,248,197,207,125,36,186,214,197,91,53,46,222,170,241,242,211,7,1,13,
  104,165,193,0,94,63,127,23,71,15,87,56,122,184,49,172,223,248,201,45,92,190,173,240,213,151,14,227,232,145,216,132,154,223,213,248,218,43,143,249,
  247,190,254,157,235,157,67,113,244,72,133,175,190,116,4,0,112,249,86,141,111,252,228,22,94,126,230,0,0,224,245,11,247,1,192,255,124,244,72,133,
  163,71,22,94,37,190,241,147,155,184,124,171,198,215,126,230,9,124,253,219,87,189,252,255,254,151,31,199,111,126,231,125,252,15,94,121,188,161,171,176,
  95,191,243,221,107,45,160,64,12,124,246,216,1,124,230,232,65,255,238,143,47,223,199,143,47,222,199,231,142,237,199,103,143,29,180,5,26,132,31,93,
  124,128,31,94,52,207,245,11,47,125,4,71,63,178,15,32,137,127,254,230,77,92,190,181,4,0,124,237,43,79,54,242,255,241,21,28,61,178,192,87,
  63,251,104,164,226,151,111,45,241,141,31,223,136,126,119,249,214,14,190,241,227,27,70,230,143,61,130,207,63,123,8,0,240,198,185,59,120,253,220,93,
  0,192,87,63,251,24,142,126,196,84,221,126,227,71,55,112,249,214,14,190,250,217,199,240,250,185,59,120,239,230,3,0,192,191,253,233,35,248,193,249,
  59,120,239,214,14,254,251,175,60,233,149,229,31,190,246,158,45,150,170,13,85,128,168,209,84,240,134,102,134,219,122,197,166,106,206,172,173,116,63,217,
  158,215,95,251,185,103,162,159,127,227,213,11,0,128,99,143,238,195,47,190,252,148,209,247,235,15,240,123,175,191,143,87,62,105,244,238,181,51,183,0,
  192,255,12,0,95,122,238,72,116,157,239,158,185,133,99,143,238,199,177,199,246,249,247,126,239,123,87,112,241,198,131,135,8,48,137,6,56,65,89,103,
  144,4,212,176,100,240,106,24,40,147,48,86,100,3,27,8,71,174,144,196,187,52,228,228,77,222,96,122,184,196,240,124,45,28,129,168,237,59,21,54,
  158,160,124,133,75,105,243,104,131,191,18,48,96,196,148,16,187,181,216,201,128,125,85,0,11,41,141,4,27,143,39,21,231,60,222,46,83,237,137,126,
  191,109,70,142,219,244,25,185,242,103,42,2,39,4,7,4,206,102,189,209,12,76,105,243,123,64,242,107,120,8,88,138,244,221,31,10,168,117,143,151,
  159,57,128,47,62,115,0,66,8,144,16,120,229,252,125,220,122,160,241,202,179,7,112,120,191,192,177,35,11,156,120,127,7,111,93,121,128,99,71,36,
  190,122,252,0,46,223,174,161,181,134,82,10,23,111,238,224,171,47,29,198,161,125,132,59,15,180,7,53,223,60,115,199,188,191,95,248,247,1,224,155,
  167,239,224,229,103,14,224,229,103,26,224,241,198,51,247,61,240,201,189,14,237,19,248,51,207,31,194,209,195,21,190,241,19,179,113,125,254,217,131,17,
  96,114,63,191,252,204,193,232,158,151,111,45,241,205,157,59,248,218,207,60,238,1,19,192,6,48,125,251,10,126,250,216,1,48,51,62,119,116,63,126,
  112,225,174,229,38,66,124,80,177,127,243,153,163,7,240,231,142,31,198,251,119,26,142,154,43,183,107,124,246,233,131,248,185,227,71,240,222,93,101,57,
  153,8,239,221,174,241,200,254,10,63,253,236,65,60,125,100,31,136,4,222,187,181,196,55,79,107,188,248,228,62,188,252,236,35,141,252,246,251,95,122,
  249,49,156,186,178,227,223,63,121,229,62,14,237,151,120,249,99,143,224,229,143,25,249,46,222,172,112,242,202,3,220,121,160,240,249,103,15,249,247,205,
  239,150,184,124,107,137,23,62,122,0,95,253,236,71,112,249,102,141,111,158,186,5,220,2,190,250,217,71,113,249,214,14,222,187,249,0,4,224,47,126,
  250,35,120,239,214,14,62,122,88,226,167,158,121,4,172,13,104,252,236,211,7,241,227,139,119,1,13,144,80,118,3,213,246,95,14,86,75,10,154,218,
  54,116,91,14,219,45,192,244,231,159,193,119,207,222,110,128,206,115,183,241,214,229,59,120,229,185,35,120,229,185,195,0,128,11,31,217,135,183,46,221,
  243,160,200,1,166,16,36,253,226,203,79,53,96,136,129,139,215,119,240,139,47,63,137,195,7,36,110,223,87,30,120,189,250,147,218,255,252,48,120,94,
  35,175,146,210,163,182,192,30,192,164,76,210,173,86,27,20,168,224,57,42,198,151,75,198,152,60,104,138,55,11,138,23,66,203,3,179,123,115,153,47,
  141,142,55,12,202,132,220,66,143,77,118,17,51,27,7,67,112,74,218,125,79,11,3,138,178,114,82,174,162,79,0,84,140,32,7,115,206,104,60,44,
  1,88,118,160,203,120,156,182,200,203,210,11,38,186,230,157,98,89,189,126,135,99,96,129,5,119,232,199,174,122,144,131,156,144,40,44,215,5,150,56,
  1,130,20,121,150,140,103,209,172,237,255,251,95,125,6,255,238,255,251,52,152,12,31,211,31,252,135,159,194,95,248,207,78,226,87,254,209,121,188,242,
  241,131,248,107,95,121,2,191,242,143,206,155,141,230,43,143,225,27,39,239,226,55,254,228,58,128,26,168,77,222,220,127,251,165,143,224,215,191,245,190,
  7,47,191,246,151,142,225,242,109,83,166,254,235,223,186,218,2,67,255,248,127,254,60,254,202,255,231,148,127,204,127,252,191,120,1,127,229,191,56,85,
  28,143,83,239,239,224,215,255,240,10,190,246,51,79,224,215,191,249,126,239,248,253,250,31,94,193,235,23,238,1,12,252,218,95,253,152,127,150,112,120,
  220,80,254,159,127,239,60,192,26,95,255,229,23,240,127,253,111,46,68,197,4,148,73,142,255,215,111,223,196,239,190,126,221,2,156,199,241,231,143,155,
  13,246,213,147,183,240,15,191,119,29,76,192,127,239,149,39,240,23,62,117,4,159,127,246,17,252,253,215,174,227,141,139,247,0,8,252,141,95,122,14,
  151,111,45,241,55,126,233,19,248,43,255,249,79,252,117,255,241,255,242,51,248,63,252,238,59,56,117,101,7,191,250,187,103,163,41,124,249,99,143,224,
  171,159,125,20,191,250,143,206,248,159,255,131,63,127,20,175,159,187,7,0,248,213,127,116,214,122,172,62,138,175,126,238,49,124,253,223,188,135,95,127,
  245,34,94,124,114,63,190,254,199,239,225,212,123,247,147,224,3,71,94,162,255,227,95,250,4,190,246,95,252,216,31,184,255,246,255,244,37,252,207,254,
  246,79,98,175,53,229,124,245,57,144,100,14,179,156,28,182,182,17,56,253,202,223,249,137,255,254,15,254,247,95,198,255,250,239,252,4,191,248,242,147,
  254,253,87,158,59,130,95,249,119,62,142,215,206,220,46,94,227,247,94,191,130,223,120,245,124,52,4,191,248,242,147,248,79,255,217,59,30,96,253,205,
  255,241,103,112,241,198,3,255,243,118,191,156,69,180,120,70,41,3,150,213,112,79,80,213,139,198,180,218,136,119,169,1,13,201,138,239,12,91,112,231,
  192,132,160,41,221,68,182,233,164,64,217,164,117,148,249,83,74,57,3,214,115,66,204,118,81,151,22,126,232,103,219,197,133,238,194,4,132,224,223,212,
  187,100,136,39,13,171,55,163,59,230,28,134,165,130,176,172,31,143,216,91,185,187,70,174,13,14,41,3,24,162,207,112,33,241,54,58,5,115,172,227,
  105,232,117,27,14,8,153,156,181,182,71,185,43,231,140,218,235,55,51,14,0,227,123,231,239,53,159,99,224,47,252,103,39,253,85,110,61,208,120,235,
  74,19,78,184,120,75,5,110,123,97,85,77,227,237,171,59,184,21,120,145,78,218,159,79,38,239,123,143,74,2,160,222,232,240,46,249,103,217,209,56,
  249,126,243,44,151,111,213,248,234,167,143,224,242,167,77,136,238,242,45,83,240,112,114,31,225,214,78,115,128,61,249,254,125,220,218,81,201,61,24,111,
  92,184,27,120,47,129,31,94,188,23,143,123,38,212,251,94,2,188,174,220,94,70,12,234,78,223,174,220,174,1,8,156,121,255,1,238,236,40,175,111,
  111,95,121,128,91,59,26,111,156,191,23,93,231,175,252,231,111,226,133,167,14,224,228,251,237,113,184,245,64,227,228,149,230,253,215,207,221,197,235,231,
  206,198,225,59,0,95,255,227,247,162,199,61,249,190,185,151,123,189,125,229,1,254,253,175,60,137,219,15,150,56,125,229,30,78,95,185,143,219,15,20,
  126,104,199,193,69,47,12,88,98,64,80,224,85,74,129,99,96,59,179,94,38,110,167,122,108,209,43,244,46,185,159,111,61,80,120,235,82,51,47,183,
  238,155,159,211,112,218,197,27,59,157,215,126,235,210,93,220,10,188,73,233,207,219,255,178,92,76,10,134,190,102,228,171,26,116,3,189,185,13,36,103,
  22,243,73,189,125,137,207,49,32,104,54,72,103,100,183,51,29,150,56,245,180,37,30,135,33,196,149,5,79,66,124,26,223,146,60,38,196,156,66,17,
  188,179,156,56,77,251,133,69,199,220,83,98,204,218,158,197,56,108,183,251,103,29,68,228,146,232,6,13,220,1,30,60,88,224,152,117,153,226,80,228,
  54,157,132,67,125,206,17,141,182,223,203,172,111,202,129,166,198,155,8,2,94,63,127,15,255,147,159,121,12,16,70,143,254,246,159,220,240,87,57,178,
  95,224,165,167,246,251,159,143,29,169,240,202,199,14,226,152,205,75,250,238,249,187,120,237,220,29,60,255,209,10,191,180,248,136,247,228,220,217,209,184,
  179,163,240,226,19,251,112,232,243,205,251,46,183,233,243,54,223,200,189,210,159,115,175,35,251,4,94,124,178,121,22,151,27,117,244,72,133,151,159,57,
  224,127,126,241,201,3,56,244,121,105,1,20,112,231,1,227,206,3,237,239,225,198,237,167,143,29,140,14,98,159,123,250,64,111,185,252,83,135,36,62,
  115,244,0,158,58,212,180,28,250,201,229,251,248,204,211,7,241,217,167,15,226,163,135,22,54,97,150,240,163,75,15,240,243,47,125,4,135,246,221,245,
  105,20,199,159,220,135,35,251,132,15,31,166,242,253,153,231,143,52,161,196,155,38,127,233,206,3,133,59,15,52,190,246,149,143,90,160,104,114,152,46,
  223,90,226,243,207,30,242,239,135,57,76,102,28,246,227,143,246,53,30,231,227,79,237,195,165,27,59,248,217,23,14,227,206,131,37,158,127,114,63,14,
  239,23,248,220,177,3,0,24,255,221,47,61,233,243,184,126,247,123,87,109,231,0,97,56,154,88,20,247,162,6,52,101,66,253,46,71,114,203,66,252,
  95,122,238,112,148,199,244,218,217,91,56,178,95,226,165,167,155,121,57,114,192,252,236,66,105,46,119,233,216,163,77,126,210,43,159,60,226,127,254,238,
  153,91,120,237,204,45,188,244,244,35,56,252,179,149,7,90,183,31,168,135,36,28,151,78,239,180,103,174,182,69,6,138,188,73,200,122,151,90,96,137,
  219,231,205,216,61,147,228,51,133,138,237,79,218,188,21,158,22,202,242,203,36,70,45,155,207,132,204,25,221,133,159,130,228,215,64,246,216,3,181,123,
  155,40,101,128,33,152,0,41,236,41,144,65,74,88,20,41,179,158,8,70,193,183,158,252,104,100,70,19,150,219,109,35,199,229,82,238,86,149,78,87,
  123,148,112,45,68,197,12,177,151,41,12,69,238,46,112,10,100,164,148,173,187,36,91,227,21,163,240,124,207,49,104,10,15,12,78,183,255,206,183,175,
  225,255,246,87,159,5,9,1,33,4,128,27,29,30,38,3,66,94,113,249,52,183,106,0,119,112,250,122,141,231,15,87,56,118,216,152,203,191,245,173,
  171,184,124,91,225,228,213,29,188,248,196,62,255,254,55,79,223,5,160,102,243,48,125,227,214,45,11,154,22,145,135,233,197,39,15,224,216,17,147,168,
  252,183,190,249,30,46,223,170,155,123,88,93,49,30,166,70,103,126,120,241,30,16,17,226,182,117,234,202,237,37,62,243,209,253,248,204,71,247,3,32,
  252,235,83,183,241,227,75,247,241,228,161,5,240,52,227,51,79,239,7,17,225,213,183,239,224,71,23,238,225,204,115,247,113,103,167,225,69,43,121,152,
  156,124,151,111,45,125,110,211,201,125,247,173,156,75,124,243,212,109,252,7,127,222,0,163,147,87,204,28,189,126,238,110,156,12,126,51,246,124,228,60,
  76,255,252,71,215,240,111,127,250,8,14,45,4,78,95,185,143,59,247,21,126,120,225,30,8,192,103,158,62,232,199,230,119,223,0,192,50,240,38,154,
  124,65,127,188,162,212,186,196,222,122,19,150,219,62,207,82,232,81,122,37,200,69,250,149,191,243,19,124,234,233,131,157,30,38,7,140,82,15,147,3,
  82,23,175,239,0,184,133,183,46,221,197,75,79,63,130,103,236,231,255,230,63,59,251,16,37,125,219,112,156,6,160,167,185,75,58,0,147,203,95,218,
  108,136,162,85,166,90,106,141,192,40,158,204,57,50,164,141,59,149,60,124,72,150,194,174,234,126,185,23,79,153,51,137,11,161,12,138,147,221,179,222,
  148,237,93,232,198,171,84,1,34,120,70,225,98,206,20,132,18,50,115,78,113,74,63,69,190,135,237,124,181,194,113,84,2,86,249,144,85,219,241,98,
  250,240,113,107,170,195,3,3,239,178,204,40,200,146,228,51,69,135,137,14,93,15,188,164,113,214,98,115,221,255,237,63,190,96,224,19,9,252,254,127,
  248,18,126,254,255,245,118,209,195,244,218,185,123,248,141,63,190,22,42,36,94,122,226,32,254,191,127,116,25,95,123,229,49,124,253,181,235,56,117,213,
  108,42,47,62,177,15,95,127,237,122,43,135,105,78,15,211,215,191,125,13,95,251,153,199,241,181,47,63,142,175,127,231,26,94,124,242,0,190,254,237,
  171,62,135,41,119,15,178,30,38,159,226,192,140,207,61,189,63,111,71,131,3,233,83,135,42,188,121,249,62,254,171,55,174,71,86,230,169,67,18,63,
  190,116,223,39,124,191,250,214,77,128,8,159,124,98,63,14,189,211,132,188,94,120,114,31,14,119,120,152,0,180,114,152,0,224,212,149,251,62,87,233,
  229,143,61,130,95,251,247,158,195,175,254,163,179,248,198,143,111,248,138,185,175,125,229,163,248,218,207,126,20,95,255,55,239,101,61,76,47,88,143,210,
  111,252,225,37,252,71,127,249,227,248,220,177,3,248,147,211,55,241,185,167,141,135,233,215,254,127,239,128,192,248,47,127,249,37,148,8,111,155,253,35,
  6,78,190,42,46,244,50,113,232,177,230,173,242,222,126,233,185,195,248,11,255,241,119,226,241,239,241,48,229,44,202,107,103,110,197,57,76,0,94,122,
  250,17,252,198,171,231,241,215,126,238,89,252,198,171,231,113,226,210,221,135,200,181,164,86,242,46,57,120,221,113,253,13,186,218,56,111,76,41,23,68,
  225,110,226,69,106,245,160,74,255,109,231,79,236,214,182,26,61,67,246,228,215,14,201,80,7,217,92,202,73,67,9,1,93,171,234,112,55,57,84,56,
  105,8,42,133,77,238,54,164,98,77,158,133,200,130,165,104,28,57,51,207,220,222,32,98,79,222,110,183,254,105,39,184,71,77,167,57,101,113,207,177,
  93,183,185,102,40,217,12,90,155,194,46,207,121,43,244,198,156,229,92,203,129,37,164,250,155,107,5,19,112,59,253,218,95,62,150,220,158,39,26,74,
  137,191,245,173,171,248,133,151,14,227,133,39,246,109,116,200,190,241,147,91,190,122,110,168,247,142,19,160,237,49,64,95,95,185,12,112,117,9,212,175,
  158,188,5,128,241,115,159,58,98,223,143,63,95,90,78,191,246,75,207,21,159,248,133,167,14,224,175,255,249,163,173,247,191,250,217,71,91,121,76,67,
  245,234,111,127,243,18,78,95,73,147,193,209,60,115,13,16,148,239,95,217,120,5,50,29,19,38,49,198,127,240,95,127,243,159,189,131,255,206,203,79,
  226,83,79,63,242,16,61,245,184,4,239,17,30,38,222,37,206,187,54,117,127,238,84,68,165,30,10,233,249,51,240,52,133,220,60,193,81,34,56,53,
  236,122,38,44,66,146,56,202,182,140,24,178,201,115,156,248,218,202,83,226,194,251,91,240,170,1,44,194,196,91,7,150,84,2,150,10,115,30,132,26,
  195,156,29,255,142,251,253,46,79,119,43,31,173,103,19,47,130,134,224,32,204,225,186,165,148,54,194,205,247,118,16,89,150,229,201,113,172,149,230,61,
  161,81,176,242,53,193,19,88,30,165,251,248,31,125,249,81,128,108,14,211,31,95,237,124,182,87,62,22,123,71,190,123,254,62,132,52,9,224,167,174,
  238,224,141,11,247,241,103,158,127,196,134,162,128,95,120,233,112,228,221,249,198,79,110,225,235,175,93,75,120,152,174,117,222,243,232,145,10,191,240,233,
  35,56,122,184,194,87,63,125,164,5,142,46,223,170,219,149,112,201,235,235,223,190,134,255,225,151,159,0,180,2,179,198,111,125,247,106,98,207,7,180,
  150,73,67,254,54,148,11,187,110,222,191,189,196,143,47,221,197,103,159,62,132,207,29,59,128,63,56,113,19,63,245,244,65,252,212,51,143,128,72,224,
  251,231,239,226,210,173,37,190,254,39,87,35,30,166,215,207,223,181,114,46,240,181,175,60,229,47,127,249,214,18,175,159,187,27,229,48,1,200,230,48,
  1,38,143,201,128,169,199,112,244,35,251,240,11,159,125,12,23,111,214,184,124,107,39,122,248,51,87,77,50,58,131,241,15,95,123,31,255,222,151,158,
  240,242,115,111,15,209,176,218,148,91,94,38,227,193,13,146,190,183,37,196,223,243,186,112,227,1,94,59,123,27,127,237,231,158,245,98,254,222,235,239,
  227,226,141,157,54,223,210,217,91,248,210,115,71,108,56,238,217,230,253,160,18,238,196,165,187,248,238,153,91,248,185,207,60,142,219,247,213,67,20,150,
  19,43,121,152,186,115,152,72,154,164,184,53,27,89,234,65,103,49,111,8,58,54,206,88,225,211,68,239,56,33,56,141,80,239,210,46,202,93,32,40,
  211,141,30,232,5,14,105,56,46,151,236,28,142,205,110,184,147,169,195,203,146,247,24,13,156,243,180,34,174,5,138,195,36,225,93,50,114,9,248,75,
  25,190,41,42,171,207,121,25,219,211,109,64,83,134,119,42,10,89,185,240,243,110,133,16,184,224,57,42,245,233,74,189,133,57,67,144,75,230,111,254,
  238,239,190,118,13,255,241,95,126,214,243,48,253,111,254,171,75,128,45,34,232,203,97,114,239,189,125,125,7,183,107,0,82,226,27,111,221,198,95,255,
  183,158,192,161,125,178,149,195,4,24,30,166,175,127,231,58,126,237,47,53,73,183,191,250,79,46,116,142,202,161,125,134,15,234,242,237,26,47,60,185,
  207,131,164,20,52,229,170,228,26,192,116,21,127,227,47,63,3,214,10,154,53,254,163,127,242,110,228,85,254,209,165,251,17,175,87,174,201,236,149,59,
  203,44,208,54,239,155,191,248,241,197,251,120,242,240,62,60,117,168,194,31,188,117,19,191,252,167,143,226,249,167,14,128,136,240,95,126,235,42,222,187,
  181,196,215,255,248,10,126,237,151,62,209,200,255,187,103,241,194,83,7,162,28,38,192,240,48,125,227,199,55,146,28,166,7,248,198,31,223,240,0,203,
  121,153,190,241,163,27,62,233,251,133,143,30,192,229,155,59,56,118,100,129,67,7,4,112,203,228,48,221,190,175,188,45,57,115,229,62,238,60,80,248,
  135,175,93,193,175,254,187,31,179,94,55,141,55,47,222,49,159,81,214,187,36,201,158,212,66,66,195,224,224,25,133,202,11,118,100,203,171,228,0,147,
  155,244,234,79,174,227,87,190,250,113,0,192,91,151,238,225,247,94,53,20,22,97,162,55,96,66,113,199,30,53,33,92,79,100,105,121,152,194,170,184,
  223,123,253,125,252,175,254,157,79,224,240,1,25,166,6,110,55,88,146,108,105,183,212,164,60,38,250,235,255,187,95,227,95,255,78,149,63,238,187,235,
  105,61,169,4,111,212,230,201,12,130,54,157,216,157,87,136,117,203,245,78,217,16,91,17,94,153,171,147,187,11,53,255,146,109,93,74,4,64,250,28,
  135,226,165,86,243,37,116,124,66,123,185,1,29,245,42,34,232,78,118,235,188,232,20,20,113,184,146,224,80,118,0,36,192,16,96,18,118,44,68,121,
  209,211,52,185,202,31,167,96,206,141,188,196,218,202,42,64,11,6,80,129,152,224,114,151,40,201,203,162,164,213,1,71,84,3,118,43,38,87,54,76,
  150,131,199,233,130,240,255,186,49,232,148,141,38,204,49,117,127,190,145,215,233,124,172,231,148,227,33,226,76,194,59,197,122,14,42,204,185,29,11,199,
  69,196,36,187,231,60,188,211,24,249,105,192,39,89,123,249,155,49,112,186,16,108,232,172,51,222,166,12,204,164,96,157,131,236,190,71,193,220,218,57,
  183,242,26,249,69,32,219,16,93,102,99,255,84,61,144,147,142,203,216,126,142,207,22,126,108,244,202,141,113,99,87,193,28,241,46,197,128,85,23,110,
  21,36,70,80,152,36,17,218,81,171,87,254,189,112,77,241,96,81,6,13,0,231,63,27,217,18,247,189,179,167,78,254,214,225,148,77,229,36,96,216,
  158,61,97,242,162,213,141,206,217,14,111,75,208,232,88,243,158,232,88,83,99,250,238,77,212,7,94,77,119,6,93,136,71,124,158,87,185,54,143,92,
  10,67,63,207,13,174,209,218,232,189,214,3,15,182,192,47,191,162,187,114,152,92,207,21,189,254,70,148,156,233,17,199,156,129,63,125,96,41,119,90,
  77,219,45,4,158,57,236,110,51,222,172,103,141,209,106,64,154,149,135,187,148,34,217,191,58,22,7,237,86,191,177,36,127,169,73,198,99,235,213,92,
  162,221,35,205,141,141,78,198,177,6,161,238,244,208,149,42,15,119,39,143,169,156,55,71,57,221,15,192,83,154,131,149,231,42,234,122,15,187,151,187,
  22,110,88,140,226,243,166,222,69,42,228,162,101,251,231,113,219,94,164,242,210,7,54,21,165,228,189,139,101,30,110,243,218,137,225,237,188,57,238,205,
  41,221,244,24,148,27,173,39,249,126,90,131,89,129,213,18,168,107,160,214,48,141,121,19,155,200,156,241,142,34,201,161,219,123,109,255,139,0,84,38,
  114,38,228,36,92,211,211,124,215,118,244,37,120,55,246,38,20,189,27,161,143,1,55,57,247,51,131,132,245,98,8,89,96,16,223,141,13,180,124,127,
  202,158,60,10,77,86,147,198,164,237,228,224,64,94,30,115,154,89,39,104,12,158,117,169,65,90,245,0,12,7,170,106,251,197,222,91,87,14,233,37,
  132,125,188,203,242,114,252,156,109,34,65,198,144,246,32,241,53,227,235,83,230,116,187,29,27,91,142,205,62,127,98,164,108,206,73,18,206,101,116,92,
  131,103,56,32,108,63,207,12,33,19,230,229,142,164,238,84,199,210,47,116,245,241,76,237,18,183,129,197,198,129,120,70,63,184,176,54,66,32,100,29,
  13,204,12,214,10,88,214,0,47,65,62,25,156,11,149,157,153,117,197,123,200,233,161,1,78,212,3,127,198,3,166,16,52,109,104,161,231,208,123,111,
  162,55,119,24,3,180,78,158,166,52,182,50,76,175,2,64,101,190,223,29,176,196,69,185,210,42,169,97,96,49,53,94,109,47,83,185,74,138,55,42,
  119,153,168,212,229,23,212,222,189,110,66,149,42,240,64,193,84,112,234,240,82,165,113,226,142,158,101,187,109,228,50,243,201,61,237,112,178,186,142,36,
  108,157,91,99,156,36,153,239,134,103,145,91,155,27,37,207,83,238,165,151,123,230,28,63,91,64,6,202,169,135,96,69,145,165,253,143,196,150,190,74,
  201,219,249,222,156,253,7,135,180,130,51,175,179,148,93,199,187,13,26,115,186,131,214,58,107,99,47,6,150,10,208,203,38,148,151,218,98,46,183,174,
  218,123,61,44,47,97,61,77,179,2,166,208,88,136,201,106,28,127,193,119,189,246,9,183,68,221,134,144,75,39,83,46,224,165,194,162,38,43,135,176,
  13,23,201,28,97,77,57,123,21,38,133,108,118,161,183,66,78,156,63,197,21,55,144,1,196,134,133,198,173,187,225,93,163,34,104,116,134,201,150,127,
  42,101,129,147,178,140,188,170,156,67,98,137,232,8,185,208,75,59,84,177,107,70,142,123,64,114,180,241,113,119,69,104,145,134,130,219,70,126,151,55,
  54,202,121,122,178,132,156,165,67,79,15,248,230,62,221,194,192,46,1,5,164,100,243,190,32,5,160,183,16,49,49,103,19,185,179,128,178,165,78,25,
  91,194,220,121,16,201,245,3,204,217,175,205,29,190,18,143,109,34,103,185,7,103,91,118,6,131,125,249,185,206,0,77,228,105,48,176,29,85,168,123,
  175,17,187,144,168,44,30,24,246,170,134,93,88,152,92,166,170,50,177,222,49,134,70,10,219,183,199,42,95,90,140,198,166,178,9,138,155,138,60,70,
  63,130,239,237,140,18,220,200,125,47,164,185,143,84,54,149,216,182,0,96,9,8,189,97,71,11,71,99,65,37,219,212,218,28,251,60,76,65,37,71,
  210,38,165,93,53,232,140,223,230,225,67,228,233,202,84,239,144,6,192,181,117,155,219,146,120,178,137,224,194,130,94,82,237,147,53,49,8,232,108,129,
  18,211,15,0,187,193,51,16,3,187,110,96,211,9,30,34,234,12,251,223,68,175,56,67,43,64,118,99,216,172,220,185,176,114,174,127,100,38,164,154,
  243,8,80,218,14,167,12,7,211,170,200,113,213,77,238,234,210,84,215,184,14,247,106,163,6,99,188,94,21,54,249,225,220,66,109,54,245,180,34,49,
  174,54,230,150,254,109,250,240,21,218,86,202,130,196,210,97,37,233,26,192,4,168,29,16,45,108,231,20,25,52,109,15,47,147,86,36,239,65,166,135,
  203,203,36,0,177,236,78,254,30,239,97,154,146,203,36,128,133,8,238,32,0,44,0,90,152,127,253,151,73,46,39,41,16,85,80,13,58,121,54,201,
  159,148,38,193,166,116,199,82,216,80,143,251,140,72,54,110,109,89,165,121,131,11,189,203,251,83,242,30,197,68,134,205,87,215,198,26,3,147,136,188,
  146,219,155,213,38,78,195,173,231,40,157,26,53,131,161,109,142,129,6,180,173,44,243,114,235,66,142,87,225,68,156,241,94,108,22,42,113,43,127,41,
  29,151,118,143,185,56,76,229,191,90,64,35,36,171,228,188,247,42,201,235,216,188,7,164,148,80,155,18,110,182,231,168,165,239,156,175,252,161,208,59,
  199,93,107,97,236,138,21,198,126,73,9,200,197,150,241,152,133,45,117,56,34,239,204,218,131,196,171,221,254,42,216,221,46,111,255,224,19,237,38,64,
  83,159,39,181,93,72,145,22,81,176,98,75,116,104,138,80,168,104,203,144,13,133,239,189,30,18,47,147,195,55,67,60,76,207,30,102,252,245,47,215,
  131,22,36,107,134,134,134,90,106,48,235,14,168,36,65,11,5,7,129,72,72,235,50,200,173,113,13,93,51,148,170,193,138,177,84,75,207,84,235,216,
  106,89,199,192,129,91,30,151,252,96,144,141,195,85,146,32,72,219,18,88,13,146,218,208,111,144,225,217,208,100,8,201,8,2,4,17,181,108,93,231,
  6,226,198,19,218,112,132,120,153,157,252,104,39,97,14,16,59,62,239,184,19,53,9,227,109,178,95,230,123,97,126,22,194,70,72,197,70,120,137,88,
  55,178,50,107,51,191,161,252,104,198,193,143,85,178,35,74,84,254,160,47,22,53,136,164,245,166,193,119,23,119,157,214,243,50,155,146,104,97,223,27,
  227,150,93,105,206,173,188,241,124,231,101,119,36,123,92,242,176,36,158,22,163,210,13,173,64,35,115,42,191,0,9,251,253,44,114,83,191,237,176,128,
  215,205,57,180,54,62,174,80,223,89,219,61,153,155,57,239,3,242,94,118,39,35,26,57,145,200,111,27,240,54,58,63,93,118,70,13,181,84,80,181,
  218,29,160,192,233,248,234,64,183,50,122,21,178,127,219,112,27,23,193,14,26,90,1,191,150,130,159,187,108,137,255,234,177,37,60,139,224,70,135,2,
  123,2,157,172,171,112,255,176,186,196,60,196,187,214,216,205,125,114,1,85,105,84,216,7,146,34,179,174,82,189,162,100,77,108,161,207,137,103,159,152,
  93,122,246,25,158,87,3,59,181,50,7,242,142,215,177,195,140,106,231,238,77,60,53,194,105,4,13,44,23,75,212,203,50,200,146,11,137,133,92,24,
  86,84,203,142,90,2,231,172,53,180,96,40,174,161,148,194,146,150,46,138,12,182,12,170,44,56,48,164,246,223,1,251,58,17,97,223,254,125,0,63,
  176,128,200,120,200,164,85,122,86,12,77,218,0,60,0,36,200,122,93,9,188,230,38,122,12,35,151,214,86,78,232,70,110,112,180,105,48,5,27,200,
  144,253,169,129,140,30,56,25,249,221,59,100,207,203,6,210,10,251,190,32,177,17,125,215,86,86,237,129,161,246,198,91,163,1,13,209,124,39,10,238,
  79,117,76,16,74,160,170,42,75,224,72,246,180,231,230,219,74,205,141,252,241,207,22,52,109,32,127,141,29,72,118,27,89,176,185,57,121,67,176,196,
  33,51,113,175,109,39,135,21,154,195,2,57,32,28,204,191,48,223,11,182,6,158,27,45,89,159,249,116,114,7,64,49,35,123,3,156,66,160,136,206,
  3,12,249,77,189,153,107,132,50,135,114,235,102,51,23,118,28,86,209,97,85,43,40,181,251,21,116,204,214,142,12,209,171,200,201,196,5,110,163,152,
  91,139,16,147,159,34,28,215,112,156,69,243,189,105,116,188,238,53,149,232,85,42,55,199,118,52,58,124,49,15,88,82,4,98,64,176,4,17,97,65,
  139,64,110,68,224,208,203,207,214,142,210,202,162,237,14,94,218,150,7,220,160,192,186,214,88,234,37,8,4,109,65,147,240,17,40,130,178,85,178,247,
  110,2,213,27,111,252,112,154,193,168,53,20,199,160,73,88,110,3,73,114,212,98,119,202,91,47,151,208,209,194,78,78,66,220,128,141,150,75,52,25,
  147,74,86,118,209,198,27,74,244,61,91,96,228,78,169,76,208,27,42,33,110,156,39,225,169,39,144,151,67,159,10,183,251,33,117,45,118,70,86,110,
  14,64,68,100,20,2,35,184,41,3,223,54,92,237,185,54,191,207,231,150,69,115,77,18,84,137,196,214,83,196,75,72,65,193,65,52,6,201,191,179,
  77,108,209,211,18,203,172,25,109,47,90,110,174,251,236,19,69,82,182,127,14,230,123,140,220,60,231,156,7,115,159,159,247,116,157,151,159,128,145,204,
  51,208,189,214,163,239,231,169,138,213,172,161,106,61,221,35,205,115,193,209,212,142,20,236,71,4,18,198,222,155,210,229,21,216,147,120,108,189,94,173,
  41,204,159,54,156,246,246,50,241,68,231,116,106,138,247,84,66,128,43,216,3,37,217,38,215,201,152,80,251,251,149,100,91,87,138,4,51,120,238,89,
  88,41,194,207,83,223,26,110,173,6,240,101,42,173,188,37,209,90,197,128,73,42,207,44,50,217,165,64,210,158,212,220,87,37,32,132,24,5,150,226,
  69,237,158,166,16,47,231,252,98,241,191,182,120,67,74,137,197,98,97,66,46,209,168,100,46,32,130,141,11,10,26,106,195,184,121,72,251,143,14,176,
  84,72,113,106,59,173,17,159,218,183,244,116,208,2,12,25,158,168,56,56,105,190,20,107,212,142,137,158,243,171,120,35,18,247,24,185,84,28,230,238,
  21,205,163,247,54,78,254,38,183,129,240,42,166,108,237,128,178,115,190,210,180,156,226,241,129,219,99,208,26,251,213,165,35,22,222,155,181,155,96,9,
  133,16,83,156,110,180,10,88,106,107,15,111,72,190,62,157,108,228,95,95,34,5,9,225,55,208,92,161,10,38,173,174,61,176,180,45,96,9,0,164,
  39,178,228,102,174,51,175,233,128,137,8,85,85,65,72,105,190,32,71,27,142,116,121,11,72,144,164,200,229,27,6,150,226,25,226,168,162,88,10,3,
  148,194,176,18,151,12,72,242,27,13,109,170,214,119,139,147,167,208,50,139,203,219,97,33,39,190,84,54,220,173,68,107,95,168,93,32,185,244,76,153,
  146,243,82,206,133,153,191,161,139,120,77,160,113,228,216,113,7,62,158,4,89,184,56,144,40,32,134,45,3,75,249,117,154,78,57,247,141,211,16,3,
  206,51,5,36,4,32,167,208,173,48,207,109,54,58,6,103,254,66,22,238,194,167,64,129,46,98,13,247,45,28,170,70,123,151,58,24,1,4,132,73,
  9,233,152,55,158,83,182,61,176,180,113,176,212,245,174,130,138,120,107,87,78,90,9,99,216,43,77,0,155,28,34,129,160,178,142,96,243,123,24,16,
  137,82,5,147,181,88,44,76,71,241,174,209,232,120,143,235,224,25,216,103,131,172,13,51,80,166,71,23,17,65,146,1,140,114,97,193,39,241,112,89,
  10,224,176,235,236,197,249,237,105,35,216,176,236,29,72,140,6,119,203,216,16,41,176,143,63,15,90,209,60,163,212,147,192,18,15,155,11,158,180,148,
  6,63,87,238,144,48,183,30,112,196,32,221,158,119,30,36,76,55,200,30,108,88,121,222,236,13,26,107,66,153,215,176,142,122,238,55,24,132,115,239,
  239,185,56,61,73,40,144,215,105,55,186,198,146,87,187,112,162,83,36,237,129,157,121,77,171,99,15,44,109,11,88,114,216,67,132,133,48,34,254,195,
  10,91,246,50,201,168,2,16,140,90,213,22,186,152,112,159,73,254,102,215,147,213,240,59,81,199,168,16,60,3,81,242,38,64,128,214,26,74,235,232,
  18,66,74,16,19,20,207,159,203,228,18,146,153,216,36,160,58,170,6,97,18,192,5,155,42,61,48,67,84,166,90,79,47,181,71,186,196,84,240,130,
  113,66,7,98,4,228,84,228,148,159,106,107,94,49,113,37,143,179,107,30,52,41,85,67,136,125,241,155,109,133,104,23,176,208,106,198,104,156,164,220,
  9,156,198,163,126,42,63,23,149,69,44,137,205,27,152,229,92,168,189,207,19,208,89,212,67,137,68,161,158,71,148,99,9,79,217,92,27,183,4,168,
  166,254,160,208,166,192,18,163,85,97,56,196,159,24,242,41,229,116,139,203,131,31,203,55,115,46,36,175,107,44,59,150,30,147,73,239,48,88,137,243,
  42,134,213,237,200,30,88,218,46,207,82,49,28,103,63,178,251,128,41,61,253,4,121,76,66,200,198,203,66,150,224,146,200,112,136,173,112,59,199,7,
  163,148,138,146,23,133,144,13,25,221,12,29,58,77,229,153,52,137,228,150,250,201,116,250,32,84,139,42,40,35,15,9,225,98,227,37,170,38,249,76,
  43,13,13,13,33,4,132,182,158,56,101,198,76,107,67,164,167,88,181,55,144,236,24,20,0,196,46,205,125,155,180,186,156,252,203,177,117,143,230,117,
  169,151,88,136,69,214,142,117,1,135,174,106,206,57,54,192,14,18,250,14,15,209,64,235,142,142,124,189,4,56,68,115,191,9,176,196,60,122,156,70,
  231,159,245,98,97,70,192,89,153,128,74,154,102,170,2,188,90,161,66,93,213,221,160,105,3,96,137,39,121,95,184,120,24,161,104,4,123,144,1,175,
  221,68,76,24,75,30,119,30,9,37,20,97,226,58,117,124,112,6,240,189,7,150,182,2,44,13,249,195,93,7,76,93,77,16,72,4,134,62,76,240,
  46,94,133,186,189,76,129,110,107,165,147,165,223,4,175,20,180,133,80,211,213,141,64,16,11,105,128,141,123,105,216,22,54,67,12,78,187,42,80,132,
  201,135,194,46,52,247,99,224,70,172,109,251,16,234,219,88,105,250,230,177,242,38,218,66,19,67,218,187,100,108,75,48,245,12,128,20,39,129,230,60,
  76,98,172,88,41,53,214,179,196,125,11,156,71,155,12,20,97,33,10,242,109,16,40,13,28,163,62,58,24,158,60,14,5,35,211,16,224,143,58,43,
  244,61,135,36,227,141,168,185,222,37,207,82,218,94,104,200,31,113,127,195,132,12,35,122,219,187,194,136,74,82,55,4,150,120,84,136,119,194,166,202,
  37,152,200,5,235,106,255,59,224,240,181,7,150,182,15,44,105,232,104,191,214,158,205,31,129,7,106,215,144,210,16,21,79,155,133,150,178,88,74,63,
  163,152,248,192,154,91,118,70,105,13,13,5,41,228,10,60,34,150,221,71,74,15,144,132,48,33,69,87,73,216,169,125,67,79,68,220,102,217,213,66,
  131,5,67,26,46,255,225,37,233,219,229,102,68,41,175,138,7,28,175,57,55,183,45,183,212,138,80,97,133,4,239,105,41,105,220,210,240,108,74,60,
  15,44,159,158,51,127,107,244,24,113,49,52,215,159,83,215,145,157,155,245,206,241,44,9,200,60,112,213,103,155,242,174,27,44,49,247,234,13,175,32,
  87,111,111,94,238,2,43,60,175,188,204,211,5,25,187,78,107,54,149,183,156,7,13,179,220,103,15,44,109,149,103,201,57,35,124,46,108,102,251,223,
  53,192,196,61,110,38,206,76,62,143,74,94,236,248,173,26,96,9,100,46,158,57,36,25,124,24,225,222,208,95,101,175,193,121,62,1,31,212,147,29,
  7,109,206,109,74,188,153,74,185,132,27,166,225,79,233,218,203,71,212,180,177,59,25,168,81,94,140,113,213,104,188,170,169,232,188,95,201,123,26,133,
  244,194,14,42,61,7,131,206,188,119,222,96,53,28,143,3,171,253,169,232,3,96,192,26,211,92,214,161,43,83,158,33,165,14,24,113,226,66,17,143,
  14,254,219,97,197,25,107,5,75,93,71,147,17,167,19,142,74,49,237,191,10,168,85,157,41,38,153,46,229,30,88,194,67,17,134,211,133,78,38,187,
  231,97,26,114,222,46,184,92,179,134,191,167,78,61,242,182,72,235,110,203,92,85,58,90,2,215,99,83,80,243,101,129,20,4,117,158,53,165,144,38,
  177,123,0,123,52,15,49,64,156,3,75,237,51,183,128,128,208,38,105,93,216,255,13,173,144,219,128,207,33,179,167,115,193,227,144,49,203,60,216,186,
  67,41,141,165,218,201,254,49,175,34,228,202,96,169,35,225,187,47,249,153,203,203,133,11,27,37,247,88,188,57,57,137,74,99,196,204,163,21,172,111,
  45,23,223,231,246,128,173,194,148,179,237,96,169,116,191,209,158,163,1,142,250,194,177,16,93,125,250,166,12,1,175,113,44,251,36,41,221,203,0,38,
  238,199,146,60,80,182,61,176,180,189,96,73,160,161,38,210,91,7,152,186,140,31,167,252,115,109,211,151,134,235,121,104,174,35,251,208,88,232,237,16,
  66,24,24,229,60,114,18,88,200,5,22,210,242,59,9,130,100,7,69,242,110,28,33,4,152,24,146,101,76,117,48,234,188,215,149,56,202,61,135,65,
  123,90,210,9,74,230,62,68,178,233,233,94,37,60,214,97,221,221,183,26,80,75,53,237,81,214,230,89,234,241,6,13,134,181,61,159,25,96,184,215,
  66,243,199,60,106,60,218,24,145,123,97,223,16,136,207,204,3,54,178,245,240,80,9,91,229,186,105,176,196,35,156,44,131,189,112,195,200,194,214,122,
  124,94,247,186,227,129,74,26,121,153,120,252,120,236,129,37,60,92,9,222,2,91,232,97,26,64,160,200,211,116,167,232,101,114,189,159,164,148,113,18,
  117,242,121,13,101,65,145,245,60,177,233,143,39,164,132,132,244,121,78,66,72,15,158,196,66,66,74,243,59,200,33,155,10,119,2,151,33,45,96,34,
  144,232,188,98,73,146,184,200,244,127,95,231,169,184,119,114,194,13,141,51,91,63,39,35,48,184,90,44,254,73,67,99,185,92,66,41,149,189,206,96,
  255,218,92,70,59,131,17,7,229,55,113,215,88,14,128,225,115,55,171,156,85,119,186,188,127,92,4,87,60,250,57,10,107,109,102,30,42,225,251,168,
  109,222,179,52,212,245,177,14,207,25,175,67,94,230,121,199,170,215,179,84,214,44,173,245,100,73,246,192,18,30,186,106,56,143,142,146,165,188,171,85,
  114,89,156,16,244,3,234,12,38,13,160,60,161,164,198,152,53,251,242,123,41,165,73,142,22,100,60,111,238,195,26,134,87,133,68,92,12,17,2,32,
  217,128,40,104,6,22,128,132,28,63,253,60,96,131,100,228,67,147,233,102,171,3,160,100,252,138,32,123,216,173,117,29,215,76,109,35,21,83,214,11,
  50,86,233,211,250,114,243,141,214,26,208,53,100,37,163,30,80,233,96,20,75,207,87,222,232,166,148,65,115,183,55,138,59,196,46,188,23,85,160,206,
  169,3,60,33,204,53,34,82,87,58,208,51,133,69,239,20,113,79,97,152,166,204,186,241,19,0,22,102,253,13,223,96,215,1,150,198,109,138,131,7,
  138,211,156,201,121,122,242,77,2,75,125,68,146,220,109,120,187,195,112,249,138,98,93,43,136,74,22,105,220,226,245,103,140,201,30,88,122,200,192,146,
  54,224,216,29,122,4,4,180,208,65,83,222,93,223,25,57,3,12,66,48,197,217,104,221,168,147,132,50,95,81,191,49,101,146,163,149,212,54,47,9,
  77,9,161,176,172,159,212,51,184,196,128,156,194,10,62,134,161,150,187,247,213,112,165,186,254,179,210,130,37,155,118,229,99,178,189,68,64,27,242,46,
  13,90,175,220,189,195,142,220,156,53,140,210,51,243,240,97,152,201,179,196,125,205,100,7,219,133,85,152,203,87,169,149,154,107,140,186,122,158,229,119,
  56,238,121,244,252,134,196,81,129,195,208,168,211,28,158,23,195,208,143,201,94,166,113,96,105,124,88,123,50,131,124,231,64,77,239,219,56,213,179,212,
  223,14,102,232,161,117,248,186,208,8,122,86,142,25,178,61,176,244,240,120,150,4,124,53,123,8,154,132,52,249,192,187,3,152,186,170,104,184,203,72,
  118,101,195,102,10,173,237,91,90,183,21,157,193,208,172,97,2,106,194,36,123,11,134,22,186,127,172,169,192,251,52,73,175,74,27,69,90,38,30,86,
  223,101,142,182,254,27,123,230,182,132,159,154,117,111,226,119,183,71,111,222,57,231,194,189,71,29,122,57,254,26,10,42,180,214,168,85,109,90,225,244,
  55,179,91,207,70,135,17,109,76,185,36,23,143,50,15,209,234,152,137,89,97,220,24,149,231,164,139,93,100,80,130,126,114,208,26,134,11,103,46,118,
  208,249,191,22,35,205,235,100,207,82,41,92,191,187,39,225,222,167,216,124,130,119,62,205,161,59,24,215,66,77,166,96,136,7,162,139,61,176,244,240,
  128,165,1,122,35,118,115,41,245,45,248,126,253,227,94,5,96,205,77,153,57,183,123,142,113,88,239,33,2,218,232,78,143,58,173,223,164,172,64,57,
  194,182,31,30,17,192,146,251,41,14,214,189,200,7,114,225,48,74,137,254,60,8,107,247,141,24,91,131,87,115,109,128,19,115,190,66,108,45,96,105,
  192,169,151,187,65,116,123,28,185,251,216,205,3,239,139,9,149,114,99,62,95,172,96,31,152,111,52,124,11,156,52,11,115,231,244,144,173,148,29,227,
  101,154,84,13,55,152,111,96,133,29,145,39,90,53,158,112,89,94,19,95,22,231,61,75,37,31,29,119,128,9,128,161,106,13,165,85,241,38,205,225,
  100,15,44,125,144,192,210,174,1,166,246,38,144,26,80,110,109,160,227,142,198,22,36,40,198,82,229,93,168,181,174,125,37,153,219,135,4,4,132,32,
  96,205,62,23,46,63,114,251,196,28,60,9,115,167,239,96,240,13,179,33,169,121,233,107,218,167,56,238,182,96,171,22,170,149,174,95,106,51,194,108,
  122,21,170,90,129,89,55,149,85,29,55,209,218,132,245,52,244,202,251,73,219,83,200,131,15,21,125,218,196,125,251,230,170,97,217,161,201,184,60,172,
  144,159,71,25,193,12,95,206,112,13,106,173,170,89,192,82,142,243,140,194,124,194,53,121,150,6,129,36,204,162,171,83,159,156,135,110,156,107,247,44,
  205,123,47,214,61,116,45,123,96,233,161,5,75,90,217,156,37,109,122,184,106,104,104,181,219,57,76,73,50,243,42,246,34,140,55,134,55,232,108,160,
  235,66,58,138,189,113,39,34,8,72,144,150,32,65,227,123,139,205,114,18,43,238,168,173,207,85,85,5,165,106,236,219,183,175,125,54,242,94,37,110,
  231,56,111,108,142,135,45,128,129,91,126,244,90,44,22,32,130,5,184,67,237,96,59,148,161,237,137,177,174,21,52,171,110,207,162,50,95,106,169,176,
  212,75,240,138,118,101,216,251,177,30,72,41,65,68,168,170,170,211,248,245,237,119,60,247,156,142,188,4,103,79,11,197,29,206,235,187,16,100,114,242,
  184,35,111,38,204,93,226,65,10,55,251,56,16,200,180,118,90,131,103,105,86,15,153,123,94,34,8,65,216,183,111,49,236,162,60,116,165,174,23,44,
  113,241,228,153,251,12,183,82,3,0,96,255,190,125,144,82,152,102,187,185,191,25,139,43,215,101,99,247,192,210,198,60,75,166,231,135,242,135,228,221,
  247,48,77,137,31,103,36,174,42,137,147,39,79,224,228,201,19,45,128,211,215,97,200,241,21,213,170,246,94,3,34,219,235,141,214,39,243,144,49,41,
  98,75,251,14,17,225,250,245,107,56,117,234,20,206,157,123,215,47,246,214,146,82,93,237,53,121,86,227,213,127,157,2,52,226,254,46,38,225,229,22,
  139,10,111,159,58,137,83,167,222,134,214,106,192,170,232,9,75,177,6,107,13,93,155,101,226,23,137,134,207,89,112,250,161,108,120,151,21,176,92,46,
  87,170,134,210,80,45,86,242,60,55,76,163,23,247,238,223,195,137,147,39,240,238,185,119,177,216,183,175,211,123,50,1,169,174,21,44,49,134,48,28,
  228,239,81,85,21,206,159,63,135,147,39,79,66,21,19,111,135,129,138,118,35,100,222,232,56,204,238,89,90,233,128,6,111,247,206,156,57,131,211,103,
  206,96,223,98,49,65,87,38,246,13,92,73,182,145,247,204,12,193,162,170,112,229,253,43,56,123,246,44,238,223,191,215,20,200,96,0,93,154,202,157,
  111,246,60,75,15,43,88,26,50,194,187,150,244,205,93,78,149,28,193,88,193,254,95,185,114,5,117,93,163,174,107,92,185,242,158,183,134,181,170,145,
  165,32,8,243,121,28,201,35,115,190,239,218,204,202,207,93,63,112,137,141,57,15,118,170,170,194,197,139,23,1,0,55,110,220,48,155,62,66,98,75,
  142,169,69,120,237,105,221,163,199,141,71,121,212,154,191,186,123,247,14,84,109,54,205,203,151,47,67,6,4,161,67,67,0,185,156,33,6,163,174,21,
  148,50,95,75,181,52,174,88,11,156,60,88,10,109,166,82,6,56,21,93,83,153,187,43,6,20,32,148,48,30,43,165,202,189,75,130,81,58,112,224,
  0,46,92,184,0,0,184,115,231,14,238,220,185,221,15,139,230,98,58,231,185,152,118,198,110,172,230,167,251,247,239,225,206,157,59,0,128,11,23,46,
  96,177,88,172,108,248,25,107,30,7,91,45,23,53,224,94,27,88,234,174,60,236,186,138,148,2,239,189,247,158,81,205,186,198,189,251,247,166,143,208,
  152,22,153,60,87,72,184,99,251,228,246,90,10,63,176,172,107,220,188,121,211,219,146,197,162,26,128,148,216,46,99,85,172,242,158,213,213,180,7,150,
  54,3,150,184,31,56,109,30,48,69,9,222,93,105,159,220,125,2,101,19,158,184,123,247,174,255,205,205,155,55,177,216,183,240,185,19,68,38,172,86,
  98,11,225,53,177,253,14,159,181,52,23,99,56,193,220,125,111,212,26,217,9,78,86,187,160,107,53,80,198,25,161,20,247,107,250,160,132,223,240,233,
  146,143,222,186,117,203,127,127,247,238,93,235,93,27,17,254,27,1,138,21,26,0,213,53,39,74,41,3,168,84,126,56,252,56,43,64,105,21,124,153,
  4,82,23,35,47,27,95,224,238,221,59,168,235,198,187,178,179,179,211,218,22,251,202,240,39,105,250,28,97,56,12,35,19,64,6,200,26,125,191,239,
  191,127,240,224,1,148,170,199,27,71,94,49,60,185,149,158,37,238,26,182,65,175,197,98,17,217,209,219,183,111,207,106,16,215,22,134,27,92,96,90,
  30,163,7,15,238,71,127,247,96,103,103,16,160,112,239,215,90,129,91,250,204,243,169,203,30,88,218,8,88,210,181,246,85,212,219,227,97,26,84,214,
  49,124,26,133,20,254,212,9,0,117,109,67,107,4,195,182,29,220,147,123,142,214,66,9,108,178,40,55,173,78,226,14,143,26,243,88,53,228,164,138,
  131,237,73,210,48,145,67,218,6,189,147,156,146,211,13,60,207,185,232,187,224,31,15,65,75,101,175,222,164,187,107,147,44,232,64,147,82,230,95,173,
  98,4,229,230,165,229,221,183,160,73,65,65,233,58,154,191,146,56,247,239,223,79,64,232,26,134,117,6,207,18,247,129,53,238,215,63,7,14,253,90,
  15,60,200,229,65,226,249,248,174,183,14,44,205,87,241,150,142,237,184,221,139,59,109,249,166,18,188,121,205,243,214,129,59,204,61,52,230,127,237,129,
  165,245,131,37,102,159,182,162,161,173,19,135,27,146,235,228,53,154,233,251,240,225,195,56,124,248,240,180,193,182,141,66,153,25,154,217,87,39,249,242,
  110,251,190,249,168,246,236,213,30,84,68,92,77,156,77,202,62,124,228,176,73,224,86,20,108,58,166,196,222,124,62,248,158,204,247,32,194,130,22,168,
  246,87,62,89,211,123,167,72,96,142,220,111,35,94,32,175,110,203,111,190,98,217,29,120,74,1,223,237,192,203,2,152,16,221,19,143,63,1,144,245,
  138,236,40,63,62,78,238,106,159,132,36,9,69,26,164,141,55,138,73,3,58,148,55,150,127,6,139,21,201,174,195,249,246,191,179,97,81,43,123,211,
  80,182,253,47,0,92,189,250,126,116,135,35,135,15,27,189,137,230,218,124,223,154,123,65,158,114,193,14,142,253,185,145,93,80,155,140,116,114,227,15,
  210,16,76,32,33,192,138,189,215,143,109,117,166,102,229,231,221,233,58,51,67,176,225,5,147,178,2,200,252,254,78,230,228,255,196,147,79,160,33,47,
  167,162,190,123,57,69,34,175,95,7,225,87,223,134,56,92,223,117,75,191,57,153,111,14,42,142,116,70,215,141,61,72,231,252,145,131,7,113,232,208,
  97,47,175,251,199,240,70,154,115,160,73,71,17,5,249,205,7,132,149,223,20,15,208,112,183,212,128,151,130,130,94,106,159,7,55,91,219,95,103,71,
  209,140,35,88,91,58,168,116,156,93,120,158,163,131,68,184,158,238,5,222,37,247,122,252,241,199,155,49,13,248,221,96,245,138,204,47,90,54,53,210,
  41,144,157,131,192,150,172,202,147,234,83,40,96,109,40,183,100,246,54,52,168,124,229,16,128,4,227,112,229,253,43,209,245,15,30,56,128,131,7,15,
  198,242,122,59,209,188,225,100,148,36,32,164,4,105,64,238,147,209,190,1,178,164,193,147,120,141,205,67,190,251,238,185,61,176,180,70,176,212,68,14,
  134,133,228,70,3,166,151,94,250,20,190,244,165,47,78,158,62,23,222,208,90,67,43,19,146,208,246,103,165,109,242,109,240,197,204,13,67,115,242,117,
  251,246,109,252,209,31,125,43,126,190,227,159,194,161,67,135,204,98,178,160,74,8,225,21,57,253,222,125,45,170,125,88,236,95,96,81,85,88,236,219,
  7,33,132,247,200,204,81,45,231,38,167,36,123,238,203,131,203,224,123,119,173,55,223,124,19,63,252,209,15,27,208,112,228,8,190,242,179,127,202,124,
  86,27,82,78,77,218,112,194,176,132,172,204,23,217,42,35,1,97,24,77,73,128,72,64,67,155,62,88,86,102,39,255,28,47,39,179,82,202,200,107,
  101,106,201,206,12,238,153,115,0,56,121,242,68,75,39,15,29,62,28,1,196,210,188,135,115,46,132,49,118,225,207,179,207,57,43,223,142,71,215,230,
  251,165,174,219,115,157,145,155,152,32,42,97,254,37,129,55,79,188,137,31,253,248,71,209,61,254,244,159,254,217,150,220,157,50,147,253,183,178,115,108,
  199,96,238,57,119,243,221,210,119,247,229,244,64,41,47,119,215,156,191,117,226,173,232,250,31,251,248,199,241,153,207,124,38,43,123,42,55,185,185,77,
  230,93,6,243,239,228,95,117,111,82,80,16,90,224,193,242,1,32,96,0,147,101,154,159,207,241,192,173,113,101,107,63,179,54,164,99,108,153,25,39,
  78,156,104,141,239,87,126,246,43,89,157,138,198,56,163,95,110,76,137,168,165,91,115,172,41,111,71,18,29,202,201,142,96,61,165,242,187,113,124,245,
  213,87,125,94,160,209,171,143,225,83,159,250,84,175,94,193,201,45,173,220,36,113,96,177,31,226,128,196,254,106,31,100,85,205,162,87,127,243,63,249,
  79,247,192,210,26,192,146,219,151,70,197,85,120,195,33,57,78,226,76,33,234,15,3,114,140,60,161,94,74,52,152,35,219,83,54,105,87,179,206,254,
  29,115,190,251,185,182,20,4,10,10,92,235,181,200,222,120,200,56,34,54,43,115,149,183,101,236,34,24,100,54,51,42,164,217,100,37,75,8,45,64,
  130,60,131,185,97,254,6,132,144,102,35,177,30,180,176,17,49,175,195,101,94,96,39,206,201,221,37,51,119,52,75,237,27,171,236,56,115,194,7,86,
  34,179,92,117,131,179,140,30,10,186,77,85,158,241,166,57,176,85,239,152,48,179,210,121,202,3,86,229,231,13,223,175,149,194,142,90,162,214,202,228,
  93,36,58,190,14,153,179,161,133,144,158,61,179,38,7,233,78,193,30,244,233,203,74,97,203,17,175,165,90,58,23,211,56,131,60,197,166,228,194,67,
  201,87,170,87,41,112,232,27,243,226,231,115,115,153,126,191,70,123,194,169,45,237,88,79,67,117,103,136,94,133,118,28,218,126,89,138,18,48,131,105,
  61,50,239,129,165,249,192,146,203,5,229,145,249,24,163,61,76,23,46,92,28,238,147,207,60,127,115,242,209,208,218,158,126,2,207,130,9,219,104,239,
  110,205,134,111,252,41,171,110,157,62,46,92,188,0,41,43,31,146,105,254,37,239,94,142,195,79,198,45,79,146,80,201,5,22,84,65,238,175,162,211,
  196,92,222,6,29,120,208,184,112,250,9,101,7,24,90,135,139,180,113,171,95,189,122,53,186,254,189,123,247,112,242,228,219,62,64,161,137,77,7,23,
  69,198,211,36,9,146,44,135,15,85,208,66,67,202,10,130,200,132,179,20,3,162,113,55,11,59,54,157,178,15,236,249,228,230,155,25,209,124,167,243,
  238,66,56,126,147,229,84,238,124,120,228,194,133,139,168,164,244,157,115,195,57,47,205,187,16,46,220,154,57,57,90,217,103,153,115,210,77,248,85,51,
  180,74,100,71,16,158,53,136,182,217,8,236,102,32,152,192,2,120,255,202,149,214,245,223,60,241,166,15,65,153,249,181,178,75,35,39,147,209,47,18,
  4,233,249,197,4,68,69,168,170,133,151,53,252,119,213,24,116,228,25,77,61,135,28,140,69,240,61,16,132,227,57,168,92,45,68,196,222,127,255,125,
  188,245,214,9,31,137,36,52,33,246,38,76,68,94,222,70,167,173,252,129,126,123,79,212,76,188,107,218,38,242,19,3,74,55,33,185,85,214,81,26,
  150,114,54,34,111,67,26,221,10,211,32,194,240,85,19,170,3,174,190,127,181,117,15,111,75,124,72,46,78,99,240,97,185,52,196,43,218,99,75,62,
  228,57,7,80,66,236,145,13,117,201,247,138,180,97,222,16,28,250,170,225,248,176,122,239,222,189,150,94,53,189,66,169,9,199,37,54,37,178,45,68,
  216,39,246,129,246,17,164,148,168,68,213,232,153,211,191,121,130,177,123,96,105,6,176,164,237,97,145,39,36,175,142,6,76,23,47,94,244,229,236,67,
  1,83,203,123,224,188,74,193,247,177,129,12,226,137,73,5,89,24,135,247,36,107,214,72,185,231,219,217,89,58,141,111,229,119,80,99,97,155,56,124,
  240,175,20,21,228,66,196,27,238,76,43,61,202,223,8,189,107,89,217,51,201,234,193,230,145,130,197,123,247,238,227,196,137,147,254,103,10,250,203,105,
  104,211,43,47,148,55,52,4,108,18,150,25,166,195,182,0,153,78,240,43,130,37,127,170,15,65,79,199,124,167,204,238,217,228,237,2,96,90,238,236,
  36,185,2,20,141,1,92,111,245,22,144,206,1,235,249,72,184,152,217,228,32,57,153,117,34,51,123,206,135,182,252,137,188,20,180,236,113,205,132,223,
  250,241,137,38,103,71,34,104,194,108,231,90,3,74,153,66,136,202,45,119,34,67,197,32,195,13,129,0,230,149,245,157,19,175,82,14,240,246,206,55,
  23,27,84,248,215,213,247,175,226,218,181,235,193,148,19,226,111,195,28,148,96,222,211,141,48,148,127,34,128,41,26,102,197,126,158,198,28,46,199,120,
  240,210,131,84,227,177,231,242,142,153,22,154,100,238,127,242,196,201,112,8,227,49,78,85,196,142,49,35,51,198,222,30,205,188,166,192,1,57,111,232,
  241,10,217,182,203,107,201,189,255,96,231,65,244,206,181,107,215,112,237,234,181,182,144,97,94,31,181,247,150,133,172,252,186,163,192,0,15,150,123,47,
  193,123,237,96,73,35,204,39,156,86,233,83,97,29,175,158,222,112,147,147,31,121,128,38,100,254,132,146,239,186,94,138,21,36,139,249,137,43,11,161,
  164,174,137,227,46,198,218,98,23,36,74,5,207,55,1,101,6,107,179,201,50,204,73,208,132,49,141,87,2,51,129,165,105,231,40,46,26,128,124,143,
  38,206,206,46,59,49,152,203,243,153,85,139,97,186,50,124,231,4,32,184,208,238,100,64,35,96,251,158,178,149,28,45,35,0,51,111,80,48,158,182,
  224,35,166,240,193,200,82,67,249,196,85,214,26,130,37,88,40,128,76,3,234,85,115,120,102,169,49,205,121,166,75,45,107,162,105,154,54,103,254,175,
  156,178,60,148,164,148,232,180,12,157,15,198,29,207,84,26,210,204,251,69,198,237,53,118,75,88,173,221,73,188,135,16,114,220,108,93,66,218,223,51,
  172,39,55,131,169,198,28,166,247,192,210,218,193,210,42,158,165,245,1,166,193,141,116,199,152,215,50,3,54,101,14,0,131,54,62,110,86,137,83,118,
  247,150,134,134,128,76,118,221,21,189,75,107,176,192,148,253,222,72,81,4,138,108,92,21,74,219,106,52,16,4,73,131,186,165,241,72,144,36,95,57,
  199,29,157,209,71,61,238,74,11,184,253,183,212,11,126,156,65,75,70,34,152,247,108,15,48,251,159,121,236,124,160,183,58,25,131,33,20,12,45,15,
  27,199,143,156,203,225,179,149,161,148,122,3,82,53,176,121,85,164,204,195,41,210,0,170,201,73,223,67,55,233,209,196,19,220,61,231,233,52,246,233,
  125,233,211,115,130,151,240,32,212,233,93,154,141,12,20,221,27,1,183,149,138,7,140,173,199,61,52,238,208,185,242,66,31,176,191,112,39,151,218,184,
  155,83,143,126,1,0,113,91,118,6,32,156,183,182,8,224,123,198,108,15,44,61,52,96,201,56,31,118,5,44,109,228,236,150,215,37,46,24,152,112,
  115,210,189,38,104,6,207,210,80,230,169,49,156,189,195,6,66,91,222,16,109,75,248,221,6,10,5,8,73,29,125,94,167,47,19,46,0,130,82,229,
  122,217,6,142,240,91,141,216,144,152,231,215,217,98,119,24,158,181,187,91,214,74,230,73,97,51,29,151,3,111,130,82,202,112,153,141,108,247,50,120,
  70,184,140,166,153,57,195,188,62,105,159,24,240,124,156,191,255,140,214,41,44,58,217,148,103,105,87,0,204,136,3,238,122,60,104,243,181,118,206,104,
  101,241,243,46,93,67,74,217,135,135,246,192,210,110,129,37,13,212,186,158,13,44,205,11,152,198,130,165,30,78,193,81,8,210,121,129,10,92,31,60,
  114,18,88,205,68,68,54,198,59,51,165,129,188,147,185,229,14,225,78,160,24,42,15,115,76,114,233,12,125,139,105,125,229,48,92,233,67,195,38,157,
  123,101,231,236,229,120,244,32,175,171,23,84,123,110,250,250,231,77,121,170,81,103,237,44,80,53,37,235,140,25,230,155,71,170,64,223,223,23,244,157,
  209,193,176,204,43,220,127,226,20,179,225,23,72,200,99,215,224,89,226,210,229,198,120,47,131,31,162,241,157,74,243,201,43,24,180,241,62,181,150,55,
  172,132,33,120,156,29,205,231,119,53,227,46,176,34,5,199,30,88,154,13,44,101,71,82,3,53,215,126,31,231,89,216,137,231,2,76,67,193,82,33,
  247,164,244,78,31,49,184,130,26,16,59,25,18,250,107,43,221,18,203,21,157,57,220,115,66,202,128,154,146,145,201,122,87,168,189,232,7,194,13,34,
  83,49,215,120,34,66,146,55,248,240,17,151,45,242,68,245,230,1,191,230,126,176,209,49,231,77,184,107,72,243,80,222,88,169,249,112,55,86,155,92,
  143,75,198,189,4,148,7,141,105,119,98,117,93,215,235,111,29,196,221,235,164,79,191,87,118,175,48,230,207,89,226,158,150,68,27,245,44,113,202,161,
  81,184,66,169,208,97,204,14,188,254,149,196,243,106,95,214,140,218,180,164,210,182,0,81,9,84,84,153,180,133,1,96,118,15,44,109,0,44,177,113,
  114,56,15,121,93,215,168,117,61,136,136,114,172,29,153,163,118,122,240,253,120,212,76,112,230,175,242,72,114,245,163,118,254,163,90,169,105,158,166,33,
  155,64,207,17,151,121,85,211,208,243,87,85,187,130,35,44,87,110,54,248,25,19,188,59,100,54,181,121,180,226,198,204,24,234,158,228,201,139,102,186,
  223,97,202,253,98,122,131,153,242,71,120,192,249,159,17,245,173,155,111,152,56,51,87,125,135,26,158,237,57,214,177,233,114,55,244,157,29,44,13,238,
  232,179,22,252,194,109,156,153,228,24,172,43,20,55,207,56,78,56,220,115,3,150,4,4,32,87,155,188,61,176,52,47,88,130,54,197,90,121,142,190,
  249,192,146,221,54,199,189,62,245,169,79,225,165,151,94,156,180,200,13,7,147,99,105,205,176,91,115,129,245,88,115,64,239,239,248,71,24,183,239,222,
  194,91,63,254,9,132,22,190,122,232,197,23,143,99,255,254,3,13,127,72,194,195,35,68,67,93,111,248,87,200,51,1,71,172,181,149,249,183,194,2,
  114,33,124,172,122,159,216,87,94,48,133,65,55,204,214,57,185,21,180,237,89,211,200,206,13,71,85,166,117,138,25,75,198,59,239,190,139,51,103,78,
  251,123,28,62,124,24,95,254,242,151,145,182,197,8,91,63,52,114,155,127,235,165,241,34,120,246,111,207,130,44,18,110,26,154,110,92,109,133,154,170,
  53,24,26,170,86,32,77,216,209,59,128,2,136,9,75,94,2,138,13,161,162,107,161,162,185,85,122,239,228,255,23,255,226,95,68,119,121,241,83,47,
  226,192,254,253,40,181,192,73,229,110,24,160,41,195,254,237,152,159,87,63,75,24,86,98,13,214,49,19,49,60,179,53,199,156,100,209,124,155,132,82,
  22,12,181,52,46,191,115,231,206,225,221,119,223,137,238,241,229,47,191,226,193,84,212,242,7,8,120,166,2,249,3,198,239,150,236,193,188,187,151,32,
  129,253,139,253,16,149,24,108,56,99,253,214,141,222,7,235,187,97,56,183,92,58,89,22,234,102,131,249,167,255,244,191,137,238,241,236,199,158,197,199,
  63,254,241,98,107,142,102,190,219,115,30,177,158,203,144,165,218,240,84,77,125,213,186,6,150,192,82,47,203,161,130,149,193,18,7,122,165,61,231,83,
  195,232,109,89,254,149,14,214,146,142,137,30,147,182,41,231,207,159,199,185,243,113,11,142,159,249,83,63,227,180,170,105,135,146,232,150,31,223,144,65,
  94,80,134,1,92,122,150,245,85,170,41,74,58,229,117,40,89,83,254,253,130,220,204,192,247,191,255,186,225,94,114,122,245,236,179,248,248,51,31,135,
  38,179,159,72,146,144,11,233,243,149,76,23,128,70,246,136,77,222,190,239,108,72,180,174,104,154,61,249,237,223,254,157,135,27,44,97,30,176,164,107,
  211,181,194,239,189,77,55,165,34,199,217,220,96,105,18,96,58,113,226,4,78,156,56,49,41,193,59,219,63,204,163,238,54,15,81,204,221,18,107,7,
  3,88,236,91,216,131,77,51,96,39,78,190,141,229,206,18,212,58,148,83,208,95,12,73,159,53,68,4,135,109,162,50,195,140,45,33,187,79,23,93,
  97,184,132,128,179,155,135,136,35,254,165,92,227,96,149,52,117,189,117,251,54,190,253,237,239,100,253,205,148,225,161,114,114,215,74,53,67,36,200,22,
  18,53,190,158,62,30,145,174,185,175,68,5,22,166,197,71,60,247,13,169,158,50,157,15,161,107,221,230,236,137,188,33,229,59,157,60,113,18,59,15,
  118,34,177,41,39,127,194,65,133,12,7,83,150,151,103,133,205,13,89,238,173,204,124,123,17,217,71,238,132,176,6,66,193,111,122,233,235,59,223,121,
  45,27,103,160,172,222,51,122,57,168,50,115,94,85,102,147,24,188,214,35,14,166,12,215,148,78,153,238,187,56,115,242,85,70,231,206,157,199,249,243,
  23,27,153,178,107,61,153,247,168,112,48,79,74,56,135,103,169,174,235,22,211,249,156,175,136,220,53,180,143,222,123,161,11,78,187,252,216,230,54,150,
  111,255,201,183,209,170,242,162,140,93,73,167,39,228,40,154,202,73,212,225,89,226,80,103,50,158,160,152,131,137,139,249,90,238,173,157,132,135,233,220,
  185,243,184,240,238,5,83,41,109,193,94,108,83,226,31,34,123,18,218,215,240,189,117,249,51,31,6,176,52,194,233,164,97,192,48,233,72,123,98,64,
  84,143,241,131,206,11,150,24,83,105,5,86,169,134,155,92,228,214,78,150,53,167,38,157,189,126,127,1,108,99,44,104,128,7,69,107,101,78,18,69,
  154,130,129,120,153,231,26,175,97,57,58,40,134,185,172,220,162,177,175,190,116,118,32,149,66,31,88,34,73,241,35,216,210,238,112,12,5,132,1,76,
  164,59,22,44,15,214,199,88,236,148,98,33,201,2,107,13,207,204,252,75,173,144,38,247,78,167,207,149,16,2,144,28,129,244,90,213,35,102,56,249,
  76,23,23,85,159,231,164,86,168,170,169,132,158,165,176,212,208,133,64,147,126,31,169,176,43,141,47,165,0,78,152,246,174,167,151,82,54,225,204,53,
  230,44,77,14,103,12,188,82,175,158,241,204,235,165,7,44,13,123,222,21,10,58,4,32,33,60,75,62,37,180,20,33,215,82,64,195,52,136,114,97,
  15,44,229,223,86,80,166,200,42,194,20,156,183,150,131,49,208,122,192,146,85,145,13,130,165,17,27,255,32,59,51,9,124,76,7,54,69,134,80,230,
  121,22,73,230,249,198,84,248,243,132,146,165,74,196,205,33,153,199,144,29,20,192,82,149,38,69,114,199,70,110,142,169,82,74,80,69,179,50,2,15,
  158,255,149,243,197,250,39,136,71,44,75,10,193,146,223,133,199,157,214,178,21,142,165,2,166,1,66,47,7,36,129,247,141,125,127,233,62,143,56,101,
  240,208,165,29,62,192,124,102,100,130,14,204,108,41,51,191,29,80,48,193,41,216,236,211,76,202,150,230,108,52,193,123,236,56,242,240,95,83,195,238,
  11,1,1,170,18,79,52,166,221,59,236,111,183,7,150,10,96,137,21,184,30,152,39,187,5,96,105,60,96,154,1,44,77,75,252,46,159,45,133,209,
  244,178,160,60,238,225,120,236,138,27,148,224,205,163,246,240,190,50,97,158,217,194,187,252,141,240,115,237,113,237,184,156,48,215,168,68,101,192,18,117,
  39,111,183,73,243,204,81,77,174,84,131,208,7,80,120,6,254,150,249,214,73,9,2,24,207,18,98,176,52,221,110,207,178,105,187,43,184,156,183,113,
  155,93,183,1,154,143,202,116,134,29,128,87,174,243,4,107,83,173,179,141,60,75,188,234,125,147,124,125,158,99,10,103,1,75,52,253,49,8,38,175,
  109,77,125,232,247,192,82,249,109,86,60,171,226,174,27,44,141,3,76,107,36,165,228,150,192,133,35,127,216,75,13,77,233,110,91,217,167,144,149,141,
  224,72,162,113,158,165,97,68,142,60,203,82,155,106,167,133,48,253,144,164,52,149,32,90,104,159,16,11,209,104,74,132,182,43,11,146,68,101,18,102,
  101,193,67,212,19,135,242,125,235,84,206,243,206,179,40,24,111,202,186,241,192,126,61,57,176,84,88,142,10,10,80,83,142,185,99,215,65,127,53,161,
  35,129,91,151,82,242,236,147,50,159,55,100,200,85,20,20,88,107,204,249,234,179,50,156,210,104,12,218,16,186,189,75,140,2,101,238,232,70,193,235,
  4,75,147,88,75,99,125,230,97,41,25,227,13,14,207,74,54,252,65,3,75,38,189,224,225,2,75,192,208,28,166,141,48,120,143,87,0,151,12,54,
  148,153,120,229,136,187,52,9,204,32,204,227,222,159,128,225,120,116,61,252,144,44,45,51,50,154,76,147,86,144,245,58,89,47,135,20,210,156,156,133,
  54,121,32,204,64,69,16,218,84,29,141,26,84,238,86,122,1,1,69,245,112,144,211,82,241,180,151,94,223,172,231,127,63,169,35,14,243,56,217,19,
  3,168,161,109,229,160,48,32,201,57,153,106,64,245,48,71,231,27,55,140,92,13,153,110,34,237,13,70,183,189,95,43,140,9,103,14,69,227,150,73,
  79,251,35,164,122,48,182,187,228,136,117,61,35,175,211,112,251,62,207,65,171,223,131,51,98,67,162,25,100,29,149,115,48,196,22,115,214,99,150,227,
  83,202,175,165,17,73,111,188,134,217,255,128,37,120,63,76,97,184,113,128,105,29,96,137,135,59,16,114,33,50,197,182,180,180,179,19,120,144,240,155,
  219,27,184,159,11,143,18,192,33,32,103,2,75,188,142,213,213,200,220,33,155,249,93,216,64,207,17,215,57,15,7,69,165,213,222,160,72,242,9,218,
  83,224,88,247,97,172,169,126,209,208,217,62,104,33,51,249,20,131,185,198,204,168,172,1,224,94,34,173,114,127,68,118,12,209,122,38,237,40,45,130,
  46,192,216,117,185,154,77,174,71,223,120,172,76,38,198,99,35,234,129,152,28,214,207,149,44,195,32,0,53,148,74,1,106,190,22,43,67,214,205,28,
  124,68,131,55,201,34,70,232,2,15,195,209,211,16,207,210,172,190,45,107,36,37,203,81,151,100,234,2,83,182,98,209,87,199,181,50,195,247,192,18,
  128,186,94,62,148,96,9,232,11,201,173,213,179,52,30,52,48,59,176,164,86,108,93,210,99,200,51,239,9,178,21,114,83,23,51,143,239,175,212,114,
  137,175,156,52,192,29,207,196,61,103,179,134,203,10,176,188,44,98,76,146,54,183,188,131,230,191,26,10,10,74,41,147,4,104,249,150,4,9,159,27,
  197,212,95,9,48,165,229,243,108,71,68,94,197,148,173,218,196,142,51,250,194,157,27,35,15,252,183,244,170,185,182,249,7,19,198,176,163,159,220,60,
  7,144,121,207,36,131,192,18,44,183,22,230,11,197,205,71,213,153,219,113,39,150,4,142,176,63,99,242,212,102,33,165,20,133,247,68,230,62,238,144,
  40,144,148,78,174,48,202,60,195,82,254,16,128,37,197,245,60,139,110,23,192,82,183,135,169,0,150,158,121,230,24,142,29,59,54,225,0,206,17,73,
  37,107,13,237,200,214,252,191,218,51,77,155,47,221,240,246,88,174,158,7,106,7,208,70,59,235,90,225,196,137,183,162,122,134,163,71,63,138,170,170,
  16,242,175,16,96,189,38,134,200,209,125,239,73,238,132,176,239,7,255,218,239,97,201,201,170,170,194,66,44,0,137,168,170,108,140,220,158,68,78,167,
  132,107,220,33,123,76,88,233,120,87,222,191,114,5,103,207,158,241,178,31,56,112,0,47,188,240,188,229,80,10,201,58,209,146,219,17,25,134,164,134,
  33,121,165,35,100,163,136,188,178,181,75,244,102,192,69,178,107,51,119,188,52,178,215,90,129,149,145,89,105,21,121,154,152,109,232,179,2,4,8,181,
  82,208,181,242,57,20,223,189,126,45,154,243,167,143,62,13,89,73,228,184,182,90,95,130,60,240,27,50,239,189,128,176,208,92,57,36,168,140,231,157,
  19,50,65,78,230,189,77,82,234,254,189,122,245,125,156,139,249,5,241,252,39,159,143,170,122,40,226,18,203,125,161,69,226,217,204,183,209,1,39,123,
  247,254,36,32,23,2,85,181,104,207,185,82,141,94,103,116,189,33,21,76,215,120,76,86,25,50,206,127,251,59,223,246,91,61,3,120,252,241,199,241,
  228,147,79,34,199,171,230,198,128,2,226,74,10,230,51,252,190,33,29,108,72,8,49,226,16,160,148,134,178,13,139,231,110,224,91,90,83,158,248,179,
  101,63,205,120,194,141,103,65,175,124,209,182,117,241,95,189,118,13,231,207,159,143,96,212,11,207,63,15,164,107,201,42,88,170,83,222,182,88,219,19,
  141,109,203,150,76,119,224,54,164,198,13,209,171,35,184,101,176,183,39,204,108,14,89,76,198,142,48,251,195,152,89,151,128,144,166,236,244,204,153,51,
  145,94,61,241,196,19,120,226,137,39,16,114,123,249,245,100,247,145,80,167,226,49,72,137,98,99,249,135,29,46,99,176,244,71,223,250,163,15,78,24,
  14,220,127,208,218,98,176,84,6,76,29,158,165,91,183,110,3,184,56,254,0,206,161,129,228,12,56,200,176,252,34,101,103,5,118,234,7,134,248,142,
  131,69,28,156,155,238,220,185,235,217,165,173,138,55,138,159,97,5,70,100,88,27,112,129,128,205,85,10,1,89,85,168,50,205,22,135,185,235,115,114,
  167,27,69,135,236,129,177,116,119,189,119,255,126,36,123,93,215,184,126,237,122,96,228,26,242,190,150,220,5,195,231,89,145,137,12,137,101,96,24,166,
  25,185,88,238,90,47,161,149,70,93,171,96,94,219,178,75,8,212,162,198,66,86,96,13,104,54,33,170,122,169,208,110,169,2,220,190,115,59,193,45,
  20,240,82,230,153,160,145,1,18,230,115,194,111,0,212,203,206,91,182,22,45,189,6,131,117,0,10,161,205,207,246,223,102,190,253,5,192,212,172,29,
  0,120,112,255,65,203,55,112,253,250,245,104,48,40,36,13,12,117,94,152,246,51,160,60,144,116,227,212,176,99,247,207,121,37,42,208,34,174,48,138,
  245,59,96,233,47,253,203,33,89,171,35,237,76,202,64,56,246,139,16,128,123,247,238,225,234,213,107,109,146,210,64,223,189,172,86,126,64,120,192,232,
  129,163,217,5,61,144,22,246,80,49,116,147,170,119,150,150,77,122,253,217,157,206,94,32,210,37,109,117,68,155,115,164,215,171,208,110,106,52,220,161,
  33,233,165,121,61,184,119,191,165,87,215,174,95,107,121,163,188,221,64,192,40,79,221,122,21,235,88,99,147,199,87,74,114,35,151,235,2,224,8,80,
  5,131,136,192,170,177,41,36,154,116,12,102,152,110,2,218,120,174,153,217,63,115,93,215,45,189,122,255,253,171,94,175,162,29,134,26,114,219,208,166,
  54,105,12,161,157,105,235,26,141,204,245,156,94,174,176,165,96,201,166,111,240,196,54,81,187,13,150,192,57,192,212,19,134,187,125,251,54,110,223,190,
  61,105,243,140,88,190,115,140,215,89,102,96,68,223,43,174,125,165,206,98,177,240,138,220,60,223,157,36,70,74,201,71,218,12,199,97,235,20,164,39,
  116,187,104,22,139,69,54,71,118,176,236,110,211,44,177,124,103,88,174,219,223,55,63,43,21,55,30,174,235,26,215,252,230,153,202,140,24,64,33,150,
  187,197,108,14,19,135,15,129,196,212,179,114,202,234,174,149,130,82,38,169,91,39,50,59,110,69,33,5,152,140,119,41,220,49,89,3,53,106,235,74,
  111,158,233,206,237,59,216,89,102,230,60,248,79,244,125,131,160,35,182,103,106,189,71,147,195,112,145,46,7,30,147,49,140,246,173,96,155,110,55,31,
  190,126,227,122,158,140,57,133,17,9,203,121,202,114,141,136,217,126,216,124,11,8,200,42,240,200,100,100,158,194,112,94,92,93,193,115,221,187,119,31,
  15,66,118,247,220,236,83,252,125,150,221,60,25,7,162,225,96,73,179,130,90,42,204,245,26,18,214,103,78,0,15,167,52,127,156,13,161,133,227,156,
  13,81,37,115,126,253,218,245,214,136,70,67,25,40,92,43,231,176,192,246,29,234,213,148,48,92,120,96,228,40,180,102,138,69,40,73,204,230,104,255,
  200,107,86,189,92,38,122,117,15,119,239,222,67,62,120,73,109,249,193,177,126,181,108,206,80,150,243,15,112,130,183,227,151,155,33,81,126,55,193,82,
  59,242,203,43,112,24,205,102,29,242,85,31,209,134,19,254,138,40,99,44,167,77,8,151,12,84,166,157,218,218,200,238,38,255,1,13,211,230,53,241,
  181,140,125,233,33,11,34,253,86,0,66,11,243,213,25,15,228,185,205,195,40,176,180,214,81,165,130,34,174,96,100,121,14,249,10,68,157,189,153,123,
  60,109,184,41,253,91,70,41,125,171,115,188,86,209,1,165,55,11,150,54,98,126,55,240,90,53,103,137,11,224,189,51,247,124,144,221,196,136,202,229,
  57,201,59,63,248,96,9,185,253,251,33,3,75,49,96,202,128,37,214,188,230,149,21,182,59,233,78,250,100,230,134,70,189,79,82,158,195,52,152,5,
  65,32,136,164,17,43,207,100,165,184,67,246,241,27,63,15,146,103,172,82,242,60,252,141,241,51,202,238,167,86,214,11,85,170,128,36,65,62,33,124,
  232,112,243,64,67,53,186,13,203,128,39,232,26,249,238,244,236,245,111,137,43,237,195,238,32,49,160,220,155,123,230,132,135,111,105,3,133,224,9,171,
  125,132,98,107,140,228,198,218,16,88,234,157,138,77,159,252,144,108,152,115,131,64,206,86,147,250,249,228,53,149,43,165,33,179,57,54,133,15,48,88,
  122,216,61,75,49,96,42,129,37,0,74,171,249,152,107,121,238,191,227,121,87,116,176,16,96,115,164,100,21,119,154,222,212,201,108,86,14,197,94,222,
  68,222,152,96,44,0,65,178,103,81,114,224,137,82,209,99,70,224,117,211,199,228,57,90,52,240,110,104,2,18,151,49,207,123,174,152,221,99,48,130,
  142,128,121,158,209,227,53,233,192,154,192,210,116,10,147,185,188,163,43,206,51,207,184,25,112,105,115,229,105,103,233,94,70,16,46,68,96,120,58,245,
  193,135,4,44,81,31,209,219,22,131,37,3,152,18,176,196,138,161,116,109,58,7,7,95,202,86,253,236,46,48,224,105,189,37,11,191,43,122,64,236,
  47,164,144,81,226,239,246,144,118,142,24,0,30,86,239,186,190,238,80,156,40,156,13,169,101,188,67,2,2,130,155,214,32,238,51,57,206,70,87,125,
  181,49,176,58,17,8,20,203,249,231,10,13,13,50,146,60,101,121,244,190,132,160,153,198,122,170,111,141,71,186,163,56,107,56,183,179,55,28,198,49,
  93,143,216,186,86,245,138,140,37,128,224,217,192,82,215,80,233,68,54,94,147,101,216,11,195,77,6,75,68,24,219,162,111,155,192,18,0,84,46,212,
  133,160,0,41,199,39,162,133,225,62,146,163,202,110,103,86,87,197,35,6,167,68,47,214,38,172,139,63,100,233,238,100,220,173,122,110,122,201,117,58,
  54,166,44,19,234,28,171,153,101,22,128,84,18,90,232,56,153,137,25,144,12,33,164,1,78,118,193,80,104,167,180,245,52,145,128,102,71,114,201,101,
  169,230,16,97,69,227,219,207,169,51,255,70,220,69,81,185,234,252,18,168,204,131,185,171,7,4,42,144,11,206,160,202,91,7,150,48,9,106,238,250,
  161,143,215,19,114,86,8,24,232,135,52,111,158,64,132,95,212,173,61,176,52,238,110,15,41,88,2,128,74,89,206,148,222,151,94,29,52,173,146,163,
  161,160,90,254,212,217,2,114,9,19,178,227,206,112,155,249,90,246,134,41,157,129,199,236,148,3,31,154,231,222,88,134,222,76,2,11,94,128,193,88,
  170,165,175,120,209,172,13,163,122,1,184,43,93,251,50,110,154,216,116,179,191,245,193,60,158,37,46,90,196,217,99,211,27,187,130,176,116,27,14,56,
  77,125,136,33,123,218,124,221,93,203,141,99,6,73,144,209,129,85,72,42,55,157,224,189,242,6,55,1,88,248,2,215,240,215,147,250,13,173,23,144,
  241,42,58,59,246,196,178,7,150,30,106,176,4,0,213,168,176,134,6,14,30,57,136,143,28,62,2,73,178,105,204,58,64,135,67,178,177,22,239,80,
  230,125,151,12,205,108,8,42,53,20,180,210,97,54,95,86,41,15,29,122,196,151,167,199,37,157,49,47,81,139,35,196,243,49,1,84,9,236,167,125,
  160,138,32,72,6,60,77,19,64,162,149,79,167,114,235,128,139,39,149,29,200,144,23,114,148,147,116,231,110,76,237,80,85,21,30,123,236,177,168,111,
  150,39,243,179,99,225,57,105,8,25,46,34,17,147,59,82,76,110,57,30,69,177,157,239,60,247,150,182,237,109,88,51,84,173,160,180,2,49,65,147,
  130,20,149,225,235,208,140,90,155,38,141,130,164,33,158,91,170,128,35,198,205,249,33,147,115,215,42,225,77,184,136,178,92,49,34,30,143,136,151,171,
  33,1,29,59,231,161,174,135,244,10,33,15,145,27,163,148,115,139,153,147,132,85,179,30,238,222,187,219,186,215,163,143,126,36,100,32,106,122,1,6,
  20,2,34,224,98,138,185,136,66,50,79,68,250,80,74,168,23,36,26,58,143,220,156,183,184,198,184,231,203,24,22,14,105,69,162,140,90,243,222,245,
  27,215,163,59,29,56,112,0,143,60,242,72,232,246,242,180,16,110,233,199,244,32,33,23,85,66,98,42,218,235,160,203,130,106,173,81,235,37,120,190,
  34,185,65,7,41,182,156,74,158,146,131,251,248,173,26,162,202,156,78,185,91,221,191,127,175,173,87,143,61,22,140,107,232,91,68,204,197,212,193,241,
  213,226,58,243,58,56,1,52,57,142,174,130,78,9,33,236,153,151,208,226,244,10,244,170,17,223,124,243,254,251,239,199,122,117,240,32,14,30,56,224,
  215,78,243,180,129,94,81,155,126,196,233,17,136,204,218,75,120,189,188,142,141,4,75,239,190,251,238,7,10,44,9,41,160,235,110,222,178,141,130,37,
  138,241,77,223,245,170,177,122,123,248,145,67,120,234,169,167,0,0,11,185,128,92,200,128,146,131,58,54,144,134,233,216,127,95,100,249,214,126,179,93,
  46,151,166,29,138,210,126,209,56,78,23,165,84,6,48,29,130,148,178,97,1,73,89,189,211,205,34,32,26,115,76,191,213,190,10,251,229,126,80,213,
  48,181,122,22,224,177,96,41,35,111,155,233,60,47,123,139,187,42,48,122,74,215,25,192,244,104,107,243,232,98,62,118,204,206,205,191,13,227,49,5,
  50,143,149,59,4,76,17,179,123,32,243,82,47,193,181,249,189,170,77,187,27,214,12,77,26,130,76,72,78,161,134,82,170,33,111,212,12,13,6,159,
  181,108,224,118,142,59,231,28,101,118,222,144,237,59,101,124,166,128,13,125,108,243,151,104,30,237,252,54,223,179,101,43,78,136,91,45,243,117,4,174,
  2,254,46,112,190,1,239,99,143,62,214,61,231,25,0,108,236,185,200,50,156,155,239,51,192,193,110,48,66,24,192,148,211,9,199,112,206,33,163,63,
  151,216,190,219,186,206,41,39,91,8,160,78,163,5,152,30,123,236,177,152,75,205,147,3,90,230,229,232,0,32,58,216,221,227,245,223,55,231,117,93,
  99,185,92,142,199,65,51,120,79,162,49,13,89,228,217,140,45,162,195,105,10,90,131,195,151,110,198,246,250,117,100,244,234,209,22,79,93,196,77,39,
  226,3,135,179,35,72,8,112,103,97,251,182,0,49,212,45,63,6,14,89,107,147,178,65,68,190,73,122,170,87,126,239,8,90,66,221,184,121,35,186,
  213,193,131,7,240,248,227,79,128,66,32,238,15,78,249,125,164,213,45,33,98,253,22,94,199,12,139,254,56,101,104,3,166,135,219,179,68,32,200,74,
  198,123,119,192,229,85,238,13,59,29,44,17,132,137,138,5,231,74,167,3,209,74,23,113,111,84,13,13,161,5,148,86,230,160,2,140,7,76,151,46,
  93,194,165,75,151,12,96,18,11,64,210,40,240,16,181,61,232,33,173,212,172,45,88,10,136,199,2,194,187,220,73,247,242,229,247,176,92,46,19,194,
  186,240,116,4,196,196,117,198,51,81,81,5,45,53,164,144,16,44,1,153,158,32,48,45,12,217,34,243,43,17,118,166,4,134,105,15,185,216,149,171,
  84,12,152,238,223,191,143,211,167,79,183,225,51,245,17,249,133,6,177,109,24,49,209,179,22,203,92,152,111,7,124,107,229,55,83,8,110,60,76,96,
  64,153,254,129,128,33,10,108,37,176,18,227,242,229,203,216,89,46,51,128,61,38,240,164,44,129,33,181,218,31,132,94,170,49,178,115,135,190,231,8,
  28,83,210,206,44,97,101,56,237,25,192,116,230,236,217,236,169,137,90,71,40,202,108,128,153,121,47,205,185,3,76,149,128,36,57,66,223,145,37,238,
  108,145,179,114,38,100,223,225,109,185,118,237,26,110,92,191,145,56,43,2,169,51,68,181,232,144,63,4,216,32,234,12,237,232,165,130,26,25,146,155,
  45,103,41,67,246,218,69,90,201,25,123,50,228,94,103,78,159,201,56,130,210,241,77,200,43,163,182,11,41,63,222,248,245,212,242,48,161,45,43,11,
  0,154,65,90,64,67,153,220,71,219,82,41,37,251,205,69,120,119,150,15,98,189,186,122,21,87,223,191,150,153,181,188,110,101,109,106,20,221,136,237,
  201,7,157,103,41,239,196,201,204,185,204,127,206,229,161,57,224,148,13,125,247,8,232,192,172,57,60,139,194,103,242,213,122,225,179,10,152,8,154,20,
  2,186,54,207,82,97,133,151,210,26,82,202,121,38,62,9,253,45,81,131,149,94,169,146,163,191,96,166,9,235,41,161,177,112,27,181,8,187,153,207,
  236,109,31,189,241,14,116,217,15,76,132,31,147,187,49,75,190,244,208,223,8,235,178,133,40,26,122,78,23,161,59,253,73,2,150,185,39,142,127,30,
  38,207,52,169,215,146,117,194,243,222,196,75,54,86,68,119,26,35,138,40,54,198,221,155,199,97,132,1,251,123,49,23,141,198,143,9,77,0,47,155,
  6,75,99,245,128,135,238,185,51,202,27,21,20,48,54,144,179,132,104,99,213,208,16,16,43,130,6,94,131,133,192,135,34,103,169,229,57,226,124,79,
  78,153,32,38,78,168,180,125,14,177,70,92,28,212,7,150,170,230,218,60,207,226,244,7,69,93,107,136,85,116,85,187,46,243,74,205,166,253,218,133,
  107,20,231,217,123,49,189,38,164,84,76,28,78,30,175,102,163,214,180,179,206,20,211,45,89,80,94,171,196,153,9,76,168,6,164,13,133,202,202,44,
  46,155,49,170,117,194,193,100,213,213,123,133,172,171,188,66,133,133,92,152,60,138,85,158,140,167,241,81,173,167,70,105,245,202,188,89,138,160,195,185,
  146,211,18,189,121,149,207,76,209,105,30,63,22,220,1,236,210,151,24,113,72,220,134,4,111,30,49,86,3,156,123,57,159,77,91,20,158,95,159,179,
  147,106,55,213,152,249,159,87,95,56,179,204,64,112,49,94,165,228,233,33,12,195,5,161,71,173,39,230,44,9,243,37,13,207,76,111,190,52,209,122,
  192,146,123,85,85,181,154,135,201,12,134,106,225,237,16,128,104,104,176,228,158,86,22,205,192,186,206,245,99,52,139,250,78,84,185,28,78,74,254,90,
  51,148,212,45,240,52,203,249,136,103,118,21,100,100,223,132,183,104,93,128,209,187,243,93,117,13,35,210,31,67,35,64,96,87,122,35,96,19,153,237,
  229,92,162,115,37,76,119,114,101,170,42,57,60,210,81,247,51,49,120,90,82,63,166,159,216,185,251,129,230,31,248,160,38,218,55,38,165,34,201,198,
  202,228,130,67,185,229,135,250,165,104,246,213,83,144,151,214,166,234,243,130,165,49,6,127,64,197,33,205,245,40,107,52,54,227,153,241,121,128,253,153,
  203,171,214,179,156,105,166,73,222,50,176,196,156,201,39,30,240,64,202,146,18,119,97,3,183,55,72,54,251,178,146,10,18,210,23,9,53,104,134,215,
  10,150,252,109,230,244,12,249,239,29,89,142,223,245,8,168,184,211,3,160,17,128,165,17,130,208,96,197,47,243,47,21,227,164,91,197,51,211,150,159,
  6,62,50,79,25,149,57,229,247,121,42,156,156,230,83,139,210,108,144,174,9,170,115,177,59,213,97,48,100,160,71,20,97,96,155,123,83,153,69,168,
  149,30,184,39,117,112,115,173,205,7,52,237,170,180,41,61,77,6,142,136,58,115,151,214,189,53,150,214,249,112,206,169,34,118,12,169,253,103,155,248,
  221,242,44,77,128,230,69,59,58,138,126,163,211,21,52,51,151,91,129,230,65,8,97,189,78,220,242,70,148,200,54,105,212,72,209,128,183,147,17,229,
  153,231,109,219,195,112,193,152,152,42,65,17,55,104,142,39,173,183,160,136,4,53,77,199,3,112,37,133,132,214,26,66,138,33,93,84,103,1,75,0,
  86,11,201,181,111,82,34,226,99,168,165,54,21,111,75,157,69,154,170,86,253,202,48,210,183,223,199,72,235,72,15,93,99,215,6,55,241,102,140,27,
  143,36,211,31,18,3,231,178,167,98,87,73,235,70,114,234,48,91,175,164,182,229,192,214,6,85,84,13,186,17,145,233,55,215,27,66,226,217,156,228,
  155,240,73,12,59,25,243,244,123,204,209,127,139,49,146,253,164,212,94,133,87,27,47,158,89,39,221,107,169,151,187,4,150,120,252,175,153,215,170,196,
  19,57,211,103,3,242,185,29,109,90,85,111,251,124,183,242,244,124,8,114,150,90,222,244,176,81,173,175,56,165,162,119,41,91,59,161,219,215,115,17,
  131,240,90,166,50,122,115,96,137,231,243,48,13,127,41,40,168,29,7,82,108,201,167,226,105,70,126,108,22,53,197,96,73,144,128,38,203,107,2,70,
  53,39,126,236,121,88,94,203,138,110,39,57,99,43,28,101,60,205,46,112,83,57,193,163,193,132,57,221,232,206,6,210,211,216,192,121,46,249,123,122,
  164,16,81,99,144,214,17,162,67,190,18,110,183,152,252,67,47,228,170,162,148,231,52,249,5,243,172,11,100,171,60,75,60,131,126,116,113,186,102,13,
  205,122,138,101,134,102,0,9,87,61,178,107,118,46,95,13,183,178,239,122,139,219,157,184,194,47,165,84,68,49,84,74,113,112,182,220,135,226,116,234,
  176,176,111,43,221,132,228,32,99,224,69,192,220,164,148,67,60,177,213,238,45,92,142,218,177,172,127,179,138,233,215,132,20,222,107,97,194,60,114,75,
  67,112,227,15,153,132,45,234,123,199,211,193,46,73,91,102,154,227,204,24,176,119,154,190,116,12,157,43,74,40,160,201,1,233,78,243,159,94,57,227,
  249,164,145,23,165,73,191,92,225,179,51,159,206,51,127,183,74,135,185,126,204,55,237,56,209,85,224,178,85,12,222,220,239,90,94,71,109,220,250,100,
  219,162,78,158,165,56,239,44,163,242,240,37,120,19,25,96,93,201,170,241,58,21,65,118,12,150,148,45,236,33,77,62,53,70,8,1,40,155,42,227,
  192,18,209,106,154,49,3,88,154,4,152,142,31,63,142,227,199,95,24,245,55,134,192,208,144,79,42,109,24,158,77,126,137,121,95,69,100,119,236,179,
  234,185,196,20,109,153,92,119,30,236,224,212,169,147,209,189,158,127,254,147,216,183,127,95,92,118,78,33,177,152,249,185,18,21,72,10,136,202,124,191,
  16,11,200,125,18,16,228,171,181,164,48,108,230,166,138,75,66,74,129,177,59,153,82,78,94,35,167,86,58,146,57,149,221,144,206,217,239,145,35,57,
  52,167,240,243,231,207,71,164,102,135,15,31,194,23,190,240,5,52,36,157,176,124,58,177,220,198,77,106,137,212,220,247,182,74,77,10,35,167,251,217,
  201,44,132,28,237,230,54,50,168,64,126,221,188,231,228,85,58,144,55,144,223,202,169,116,237,61,144,100,147,78,24,140,223,255,151,255,50,186,215,167,
  94,60,142,125,251,247,7,76,230,1,97,161,22,134,149,89,152,196,21,145,144,148,198,95,228,229,142,231,28,163,101,87,118,158,221,124,235,128,108,208,
  143,5,135,4,166,150,183,72,48,32,200,20,33,212,53,88,193,179,196,95,186,120,17,23,46,156,143,238,245,249,207,255,116,48,215,148,37,40,21,193,
  207,126,222,173,126,135,164,122,66,54,58,47,165,180,4,160,14,86,8,200,125,2,251,170,253,101,109,183,69,27,78,190,240,251,230,75,181,116,189,197,
  132,175,83,34,75,198,239,255,254,239,71,247,58,246,204,49,60,251,204,179,177,236,148,151,187,33,164,20,193,122,22,129,204,177,222,75,217,54,208,204,
  12,181,84,168,235,122,90,91,148,25,242,187,67,125,82,129,14,133,227,27,147,134,102,72,44,19,27,10,102,92,188,120,9,151,46,93,140,238,247,242,
  203,47,119,142,109,68,216,24,18,161,138,216,158,152,207,57,59,42,205,88,143,92,83,172,57,216,55,172,237,100,142,116,11,96,243,44,218,112,179,41,
  104,83,212,96,9,97,219,157,36,12,135,213,27,111,124,63,98,251,126,246,217,103,240,76,170,87,8,201,79,27,146,206,212,150,182,191,210,253,67,246,
  20,150,180,149,228,183,255,254,111,111,53,88,202,125,164,207,179,148,123,63,252,93,127,247,145,221,1,75,147,0,211,219,111,191,141,183,223,126,123,188,
  167,189,197,88,221,16,24,182,136,252,236,123,4,50,76,174,65,88,34,172,124,90,44,218,143,127,250,244,105,44,151,181,119,217,9,107,36,161,9,154,
  204,34,210,0,42,33,161,5,27,64,96,189,75,13,105,99,233,251,137,196,149,25,194,202,148,188,50,149,189,57,36,38,101,169,246,189,168,228,30,192,
  237,219,183,241,189,239,189,142,144,76,206,251,43,50,223,187,182,6,41,137,95,139,200,113,42,113,101,106,156,80,150,157,91,164,157,169,79,155,59,211,
  77,78,156,120,219,48,47,167,140,243,137,220,186,214,208,80,62,227,183,77,214,137,24,112,6,110,101,30,59,231,197,185,119,147,216,180,0,2,24,144,
  4,73,194,53,181,177,227,160,13,187,183,134,221,232,218,222,141,55,222,120,3,89,238,161,36,252,28,207,127,42,127,252,187,220,156,147,52,7,139,62,
  195,18,182,120,73,231,56,36,168,141,137,90,185,69,50,216,210,255,228,117,241,194,69,92,186,120,41,146,139,83,137,169,172,255,49,121,45,162,245,93,
  210,119,5,5,157,201,193,220,156,103,41,214,35,14,200,28,145,180,147,9,199,147,11,137,160,93,227,251,250,235,175,103,198,172,73,191,79,249,66,169,
  68,98,25,142,49,134,219,146,52,193,155,19,59,232,196,117,96,159,163,124,77,238,33,173,108,116,50,101,108,63,127,254,2,206,157,59,223,146,187,21,
  165,160,130,252,145,237,161,76,139,174,15,142,103,105,140,97,116,243,231,139,120,52,44,131,118,129,3,196,184,161,218,161,184,93,4,75,147,0,211,156,
  81,164,124,100,130,59,16,233,152,4,105,110,78,151,16,86,82,83,69,33,109,204,116,213,228,192,57,15,151,93,198,107,156,193,237,114,4,247,59,137,
  243,159,24,155,228,195,232,79,185,95,231,225,60,255,188,130,4,36,9,212,54,172,226,170,190,250,170,36,251,238,31,86,237,169,14,134,53,9,9,150,
  112,205,21,193,196,208,181,6,36,32,32,91,85,91,4,3,162,148,214,235,11,73,20,203,204,226,170,49,41,228,200,249,25,104,216,120,158,120,17,143,
  253,224,232,246,28,118,174,118,49,12,23,166,179,241,170,99,177,50,223,216,250,50,36,121,196,216,249,202,184,49,23,230,17,99,52,50,154,253,97,9,
  195,77,1,75,0,32,148,240,157,27,178,96,73,36,64,148,182,7,44,129,119,33,233,123,51,110,108,142,194,79,78,131,5,4,180,208,13,57,34,86,
  232,184,190,41,89,120,248,53,182,130,9,97,93,221,194,87,189,10,219,80,23,193,151,186,58,210,187,85,159,79,72,25,44,116,2,43,142,90,215,56,
  160,100,184,164,156,39,192,124,47,42,209,106,23,18,159,105,201,144,182,169,53,205,213,128,19,63,117,236,4,147,55,110,198,134,18,160,25,204,206,91,
  144,38,124,99,48,229,253,50,233,221,184,105,176,52,207,66,226,233,75,154,230,93,177,179,128,165,74,172,107,160,198,161,169,15,65,53,220,28,96,9,
  48,109,174,184,36,160,8,200,137,177,29,57,75,233,155,219,13,152,166,232,185,213,227,180,105,108,227,229,19,83,246,140,141,9,212,95,75,55,19,196,
  96,128,104,102,136,181,238,13,130,231,249,195,176,253,10,247,204,67,223,232,200,128,35,202,93,219,228,33,73,211,205,222,134,165,4,137,134,73,156,99,
  79,82,223,171,34,137,90,50,184,238,144,143,39,36,138,247,1,7,106,10,36,114,225,4,158,60,11,67,137,7,70,82,110,116,97,161,21,84,157,53,
  183,128,245,238,39,120,135,238,18,158,127,201,140,62,149,13,28,224,14,131,59,28,44,17,68,69,150,215,111,115,236,249,41,12,160,149,175,252,225,1,
  75,10,10,82,229,171,157,5,139,184,37,138,87,3,222,42,176,180,181,128,105,85,181,150,144,89,176,52,187,141,152,120,45,110,181,224,224,209,6,151,
  55,248,244,131,64,37,115,207,227,243,160,211,24,143,209,222,33,18,119,108,164,89,151,112,42,167,176,161,60,203,52,187,52,77,235,32,33,32,72,22,
  153,204,4,11,211,24,52,204,97,90,193,172,74,146,160,106,68,243,210,228,0,220,234,149,70,25,56,232,62,40,77,8,78,144,92,207,26,229,137,93,
  124,38,130,218,81,174,36,180,147,189,211,144,237,86,85,195,173,180,49,12,61,180,81,206,213,180,171,158,37,193,52,192,134,4,82,50,15,146,61,222,
  238,41,67,220,57,151,105,253,224,130,37,119,16,117,163,231,192,146,202,228,96,10,182,142,141,140,29,53,182,110,123,192,210,102,1,211,156,36,105,61,
  155,143,182,137,178,93,57,74,131,117,125,70,27,193,172,161,88,245,183,137,153,185,12,56,2,60,235,176,121,107,13,195,141,200,133,106,29,251,122,132,
  229,30,83,41,128,5,45,162,197,188,192,98,254,49,26,60,32,73,210,236,104,215,168,5,74,28,146,191,137,38,233,29,176,201,231,114,126,106,5,110,
  123,150,86,209,130,193,109,149,87,116,53,105,165,163,170,157,237,2,75,155,12,249,241,90,15,145,195,199,142,198,115,45,173,99,78,120,149,113,248,224,
  123,150,194,53,227,26,118,167,7,84,223,218,68,244,223,112,27,192,210,214,122,152,86,215,101,83,206,63,60,169,59,246,59,0,180,190,142,19,107,70,
  150,254,233,199,134,104,214,8,150,120,54,233,198,32,219,252,157,188,183,64,193,86,97,106,127,18,138,60,22,2,88,136,197,202,92,248,209,233,150,251,
  32,192,72,157,16,9,185,101,223,201,28,194,151,72,67,216,68,110,10,170,196,6,248,107,120,29,202,207,107,210,14,158,152,179,228,62,94,243,246,130,
  165,13,68,191,105,13,55,43,174,210,1,99,71,32,144,28,25,26,230,245,142,81,102,235,248,208,131,165,192,216,250,87,203,187,20,126,76,7,252,75,
  91,12,150,38,1,166,67,135,14,227,240,225,67,163,254,70,235,134,71,41,228,91,65,194,47,212,254,178,83,208,162,34,48,239,183,78,219,204,120,228,
  145,71,60,113,214,162,90,216,205,129,60,210,13,75,135,41,199,49,34,16,115,141,4,92,70,142,135,99,140,189,102,54,92,75,170,54,156,66,138,21,
  72,147,113,41,139,178,236,97,233,48,130,50,117,55,6,119,238,222,141,228,150,85,133,71,31,125,212,244,161,109,184,3,34,89,203,114,39,28,43,130,
  2,249,27,217,91,26,197,221,222,52,195,179,213,112,160,232,80,70,157,114,163,52,115,221,46,203,71,84,94,124,237,250,181,72,246,67,135,15,129,153,
  189,167,196,111,251,142,150,223,241,113,129,160,80,131,180,249,156,32,130,38,155,95,4,199,183,98,188,44,149,172,176,175,218,23,205,185,47,137,29,48,
  231,94,215,117,155,3,134,117,204,131,163,179,243,221,148,70,187,241,184,235,230,60,0,64,143,61,249,56,216,245,113,36,2,105,227,65,210,196,13,37,
  6,8,21,9,72,81,129,246,153,51,122,60,183,37,14,35,17,81,121,48,117,23,73,132,115,14,214,208,140,128,67,173,176,238,125,169,60,60,231,146,
  215,245,96,19,189,113,243,70,52,231,7,14,236,199,193,131,143,248,185,54,116,24,113,249,118,72,9,34,136,10,92,85,109,126,33,18,64,189,172,205,
  188,140,222,61,215,7,150,204,88,34,175,83,156,209,167,136,206,3,145,78,133,30,191,123,247,238,183,60,150,143,61,246,104,227,5,160,196,163,233,40,
  72,130,113,55,99,109,236,39,32,140,201,37,97,198,93,144,129,234,162,68,207,210,95,205,66,48,124,71,108,233,54,156,205,240,99,194,218,83,113,68,
  244,53,89,189,106,238,121,245,218,213,68,175,14,224,192,129,3,129,71,23,109,42,142,80,110,255,25,202,112,86,229,116,172,127,115,14,127,12,185,246,
  30,22,176,228,15,160,214,38,43,228,193,146,243,226,251,67,172,143,10,109,39,88,2,120,10,113,229,11,120,249,229,47,140,250,27,165,44,225,155,37,
  49,84,90,67,41,75,106,232,136,13,237,151,10,127,246,27,76,64,240,23,24,223,187,247,238,225,7,63,120,35,88,200,132,79,190,240,73,60,114,240,
  32,136,200,108,120,146,64,149,49,134,21,42,84,85,101,98,224,25,162,49,41,13,185,152,35,88,115,63,203,202,16,89,86,85,213,159,67,98,23,168,
  134,33,40,92,214,75,212,170,70,125,127,137,165,90,66,41,5,214,22,85,51,55,92,20,26,96,101,255,206,17,179,133,114,235,6,116,128,25,111,159,
  58,133,183,223,62,233,229,62,124,232,16,94,126,249,11,209,2,21,126,3,16,1,121,31,121,89,67,185,253,191,78,230,204,215,216,57,207,125,105,247,
  189,37,161,75,231,157,3,130,199,80,230,112,83,56,125,250,116,52,231,47,30,63,142,67,135,14,249,141,209,203,28,200,78,68,96,1,168,186,134,36,
  105,201,27,9,213,162,66,181,88,152,156,36,75,220,72,146,80,201,5,22,139,5,246,237,223,103,250,215,113,179,176,187,136,247,152,217,232,186,5,201,
  42,145,215,143,131,214,197,175,116,206,221,151,159,243,64,7,95,249,226,151,192,74,27,99,45,8,76,12,193,4,176,0,136,61,48,90,84,11,136,133,
  192,98,177,48,223,75,9,33,8,82,86,205,28,59,29,168,36,100,37,13,144,212,205,9,208,97,197,144,21,223,87,241,101,230,60,148,55,148,191,83,
  118,43,119,58,231,103,206,158,137,230,252,216,177,99,56,254,194,11,126,147,114,115,237,171,99,237,252,167,235,219,19,40,6,58,31,126,9,8,212,84,
  103,249,150,118,251,85,7,250,228,215,76,48,190,156,218,207,140,29,13,199,184,25,219,179,56,123,246,76,116,175,47,124,225,11,30,16,165,235,201,143,
  115,97,124,201,141,107,48,214,225,247,85,53,110,219,33,105,42,79,37,11,211,90,203,147,32,199,58,85,146,221,219,146,140,94,253,209,183,190,133,203,
  151,47,123,189,122,230,217,103,240,194,11,47,180,237,168,8,136,57,137,178,123,71,100,67,19,221,18,86,238,177,84,54,255,207,255,199,127,242,240,121,
  150,2,143,32,119,161,193,216,245,15,45,52,72,195,19,230,110,27,88,218,88,72,142,153,71,187,95,7,187,189,41,223,15,75,88,222,24,173,77,62,
  83,101,21,90,11,227,37,40,133,49,120,200,61,115,0,193,113,75,232,198,141,79,220,40,140,175,210,147,118,19,50,148,156,0,27,207,23,147,101,121,
  214,241,115,228,194,89,131,120,109,136,50,41,61,141,87,142,121,123,154,14,68,207,194,133,58,65,238,175,6,162,130,135,188,201,101,22,128,52,64,192,
  17,154,146,229,233,18,16,134,237,93,26,182,119,185,176,198,142,165,7,4,98,76,124,142,135,231,94,241,152,4,255,12,96,119,85,108,126,35,211,132,
  37,148,97,26,182,239,59,222,49,225,193,14,55,148,27,228,143,241,113,47,40,11,192,164,27,0,25,27,67,239,118,159,64,57,48,104,220,6,206,185,
  255,190,167,58,129,211,107,23,238,27,118,71,223,246,23,79,25,219,220,152,230,236,40,81,103,56,109,172,30,79,121,185,131,154,241,78,49,72,197,205,
  92,121,226,90,156,101,239,217,124,164,244,161,1,75,46,162,163,89,71,100,211,137,139,60,1,77,12,37,20,42,174,194,52,203,173,1,75,147,0,211,
  229,203,151,241,250,235,223,31,29,146,107,121,77,130,83,180,249,93,227,74,109,66,26,92,116,227,131,155,62,52,225,235,189,247,174,24,52,111,105,233,
  217,178,77,85,162,106,54,144,86,248,33,127,50,165,140,23,170,211,195,228,60,139,172,205,73,159,132,241,42,105,5,245,160,70,109,65,149,168,9,90,
  232,198,179,228,79,62,77,235,2,93,10,91,218,201,187,113,227,70,116,235,251,15,30,224,236,153,179,109,198,114,203,232,109,158,29,217,176,139,232,56,
  153,135,99,48,106,206,221,105,47,51,231,145,247,200,159,116,131,57,71,208,42,36,97,202,206,105,245,165,75,151,32,109,31,163,148,173,218,135,94,162,
  246,6,150,25,154,76,30,143,36,105,218,228,4,109,66,40,56,57,142,5,127,105,139,138,200,115,22,132,233,124,184,50,104,5,162,195,176,138,27,7,
  59,235,55,174,95,111,221,239,204,233,51,229,57,167,116,28,68,116,98,142,231,157,60,136,12,231,126,124,232,93,71,45,43,154,57,182,242,142,213,117,
  206,27,228,235,215,175,227,244,233,211,45,150,122,178,161,160,182,135,192,234,186,247,22,52,242,82,162,239,90,41,79,110,186,77,176,200,36,160,115,210,
  254,36,109,37,165,219,237,64,180,11,247,234,40,76,229,246,255,27,215,111,180,238,120,246,236,59,45,166,116,63,198,54,180,38,16,235,150,72,244,44,
  55,198,100,91,166,12,125,9,41,80,209,2,154,66,27,98,101,76,244,43,92,87,217,189,35,92,79,246,180,122,239,254,253,88,175,174,93,195,41,155,
  251,214,98,254,15,108,71,206,142,150,247,18,106,90,17,245,81,42,124,80,192,146,161,233,134,84,18,53,215,217,235,197,213,167,236,247,80,37,245,228,
  194,168,117,130,165,73,128,233,210,165,75,184,116,233,210,120,96,30,108,124,185,246,40,161,18,167,255,34,140,195,7,40,63,231,218,125,239,189,247,176,
  172,107,67,45,32,5,88,232,160,60,186,208,26,33,108,131,145,182,5,177,187,48,173,208,26,197,201,187,212,53,180,210,88,80,101,189,73,105,235,136,
  36,95,39,144,159,147,22,33,105,107,148,7,247,31,224,204,217,179,86,178,38,230,62,76,110,4,249,62,169,204,193,216,140,56,169,241,44,237,96,144,
  156,78,242,247,191,116,241,50,150,245,178,229,14,70,148,123,16,126,159,254,174,171,69,10,141,157,240,118,30,86,161,29,12,82,125,207,122,66,154,223,
  113,166,199,210,233,51,103,162,182,210,173,134,222,160,8,72,182,229,69,118,222,75,222,172,94,217,221,243,219,3,80,218,206,3,57,249,211,249,13,243,
  166,10,183,186,126,253,6,110,220,184,25,72,24,207,126,123,206,59,230,219,201,29,200,191,147,180,204,88,155,215,98,132,135,187,165,83,193,120,34,26,
  190,220,216,166,227,27,216,150,204,45,207,156,57,147,172,163,182,142,149,244,171,213,86,196,142,45,15,42,43,64,196,255,229,43,172,146,188,62,100,242,
  179,192,201,90,202,21,19,36,0,60,181,27,215,174,93,195,213,171,215,146,53,4,159,157,147,147,61,109,13,19,131,76,196,185,117,31,96,82,202,116,
  14,253,183,156,9,205,137,208,201,192,131,189,152,187,9,150,114,78,177,237,112,29,207,109,164,184,132,106,55,247,210,104,218,91,104,232,76,72,96,55,
  221,188,67,86,223,252,212,1,188,65,121,120,248,82,95,141,191,225,161,139,211,228,255,156,87,249,107,158,167,175,9,239,210,176,111,87,53,28,175,73,
  24,158,174,42,19,22,247,224,4,0,50,27,172,180,161,243,193,15,192,188,22,123,194,152,209,78,125,88,192,146,155,74,101,0,162,20,178,163,88,164,
  173,25,52,129,10,102,19,96,105,163,128,105,211,27,128,130,194,82,45,125,190,144,210,10,75,109,146,174,21,212,122,90,77,104,4,221,179,51,131,109,
  221,178,130,104,130,161,217,48,56,141,44,198,102,121,150,38,201,59,34,67,112,45,219,221,46,130,37,222,38,208,182,233,219,207,44,47,3,217,158,113,
  15,37,88,154,56,69,4,90,83,203,168,129,96,169,146,227,60,187,57,176,180,42,151,221,202,252,25,253,134,252,131,12,150,194,235,181,82,103,68,240,
  1,1,215,226,181,217,187,149,154,216,228,98,189,96,9,120,104,120,152,166,52,97,160,102,240,21,217,211,138,73,100,212,90,67,99,9,82,166,233,169,
  172,68,214,64,48,155,28,35,146,35,57,141,92,46,147,104,114,11,76,5,165,73,238,110,27,66,94,239,201,102,220,144,101,158,139,176,17,59,179,235,
  242,238,222,166,205,197,75,173,175,245,67,137,19,135,118,113,28,70,109,36,107,240,44,213,186,238,202,206,127,232,193,210,36,175,207,92,107,100,64,133,
  49,48,130,205,62,181,146,188,206,9,153,247,122,31,26,176,148,235,29,167,3,176,132,184,32,138,64,166,170,94,213,144,50,0,206,91,0,150,30,14,
  192,52,19,235,117,54,20,103,189,79,180,4,32,184,33,69,211,0,73,1,214,12,178,28,32,21,170,44,105,90,233,30,218,92,220,120,178,104,13,11,
  145,55,52,246,180,250,35,205,121,170,27,106,16,114,237,81,214,66,70,186,77,97,184,221,236,188,204,243,4,97,119,139,164,17,0,136,147,245,250,80,
  130,37,30,63,111,212,222,20,195,28,50,154,168,104,254,147,35,228,145,36,71,137,186,93,7,180,92,107,157,76,1,226,135,0,44,121,186,145,180,255,
  165,0,160,155,4,111,206,196,184,164,148,150,42,67,25,208,212,91,50,183,25,176,180,221,128,137,87,82,129,241,96,74,147,39,105,35,16,152,117,84,
  130,175,180,113,19,10,97,18,201,41,141,102,38,173,88,82,240,36,198,114,224,237,246,14,58,178,43,49,111,70,168,45,107,12,200,187,46,206,28,23,
  217,168,103,105,107,114,245,218,175,90,45,109,83,28,250,192,122,150,120,67,3,206,19,215,155,130,66,53,100,91,90,83,206,210,116,35,241,240,48,120,
  115,198,43,71,189,12,255,188,242,250,54,155,97,226,215,236,74,41,22,13,7,92,25,72,111,14,44,109,49,96,98,108,65,247,164,100,125,50,20,76,
  169,188,132,128,38,13,193,162,33,51,140,91,214,55,138,96,217,103,5,85,224,135,37,96,53,55,88,250,32,58,95,230,12,195,173,252,28,180,187,186,
  178,170,203,136,86,29,206,174,49,40,255,46,4,138,203,122,105,60,76,180,25,0,179,121,176,196,131,193,146,163,15,96,230,213,22,208,20,113,196,176,
  235,174,43,248,203,115,217,139,45,6,75,62,167,200,126,68,10,153,109,53,92,2,75,197,207,142,56,53,231,62,170,85,130,158,234,102,129,42,161,50,
  160,105,179,96,137,167,0,166,227,199,143,227,248,241,23,70,253,141,82,26,218,179,179,42,223,208,210,243,212,40,29,115,184,32,96,182,14,219,0,232,
  184,149,198,206,206,14,78,159,57,29,221,235,147,159,124,14,251,246,237,139,168,0,66,246,103,18,38,78,42,44,63,143,225,205,176,220,51,210,112,132,
  144,77,204,22,82,64,136,144,165,214,180,208,112,196,135,158,75,73,25,246,99,177,16,30,44,105,101,228,81,75,133,37,150,16,108,248,57,52,44,211,
  183,231,170,113,236,180,33,95,77,78,246,144,147,136,113,225,194,5,156,59,127,206,203,125,232,208,35,248,233,207,127,62,162,234,167,180,173,139,104,216,
  121,189,204,194,113,208,72,43,175,25,3,247,179,20,142,169,86,140,156,115,213,158,247,104,190,67,86,222,132,171,39,226,232,209,9,71,15,240,7,175,
  254,65,116,175,23,95,60,142,125,251,247,123,26,136,88,238,128,165,152,66,121,211,47,25,233,128,97,194,150,168,114,9,168,220,117,128,226,136,193,222,
  235,187,78,101,206,200,238,89,237,51,237,114,96,230,252,252,249,243,209,253,190,248,197,151,131,22,13,9,247,84,42,183,39,235,36,207,120,29,177,21,
  123,29,24,194,238,206,217,57,119,115,109,88,167,141,204,13,11,181,253,55,208,115,29,178,185,235,68,215,3,125,127,245,213,87,163,123,29,123,250,105,
  60,243,204,51,141,236,34,230,87,243,235,155,2,121,35,78,49,227,69,134,54,45,138,156,14,72,41,32,101,5,33,104,45,39,129,201,151,100,70,173,
  148,95,79,90,43,195,105,166,66,93,82,193,152,34,90,79,89,91,98,199,247,242,165,75,158,237,218,177,51,127,225,11,159,183,192,33,163,87,225,248,
  90,91,66,132,252,122,178,250,38,3,125,235,100,250,150,240,4,178,206,99,223,236,27,206,166,196,178,42,21,236,29,206,126,134,109,184,146,239,195,182,
  68,63,248,225,15,113,245,234,85,127,251,103,158,121,6,207,28,59,86,88,83,129,13,165,134,203,203,200,39,173,158,37,118,36,180,165,78,175,70,188,
  254,193,223,255,7,243,131,37,29,124,68,180,147,178,43,81,117,130,165,220,207,217,176,153,142,200,82,198,173,129,94,183,247,230,193,210,36,15,211,219,
  111,191,141,183,223,126,123,244,33,52,228,209,232,228,97,10,249,91,252,147,182,185,106,0,198,162,106,119,142,63,115,230,172,229,214,160,232,196,132,62,
  30,162,145,60,76,13,131,116,195,49,177,88,44,2,93,209,128,98,232,90,67,9,133,5,87,182,119,28,50,124,60,108,61,149,49,81,95,202,73,20,
  241,48,37,196,122,119,238,220,193,247,191,255,70,244,124,141,151,168,155,135,41,236,13,213,201,195,52,66,213,242,243,157,151,157,35,217,99,153,57,41,
  83,207,29,194,79,158,124,27,203,101,157,56,197,242,60,84,20,113,197,244,243,17,17,141,15,207,52,60,76,156,225,34,234,230,27,203,205,185,63,249,
  102,120,152,190,247,189,215,51,58,158,204,47,242,60,76,232,225,97,234,76,190,205,49,153,23,185,167,208,230,223,10,194,42,156,225,9,138,214,122,102,
  200,47,94,186,132,75,151,46,55,210,198,196,63,201,244,83,139,135,73,145,54,237,79,10,60,76,219,232,89,26,198,195,212,182,155,241,154,202,112,49,
  81,123,3,124,253,245,55,2,11,135,44,23,83,202,195,20,241,44,17,218,127,235,251,60,230,245,138,164,5,182,44,218,121,85,17,161,43,58,120,189,
  194,53,149,241,108,112,236,219,93,46,227,36,155,11,23,46,224,252,185,243,153,245,20,219,148,246,112,80,180,195,135,107,15,67,214,211,110,229,44,101,
  66,98,53,153,246,81,12,246,29,44,74,33,49,109,123,248,229,126,223,0,177,105,96,41,212,147,152,98,98,119,192,82,4,152,4,100,144,119,179,75,
  65,22,222,174,48,220,16,119,126,8,225,180,54,12,165,90,52,60,75,90,106,163,76,106,245,83,39,207,247,248,43,7,174,120,45,15,220,163,19,187,
  223,195,101,247,60,12,27,211,236,1,186,176,209,106,184,249,71,173,182,222,9,106,53,211,249,160,133,225,86,185,215,84,114,144,21,2,90,58,248,83,
  29,132,231,194,3,215,156,179,176,53,185,4,91,150,224,173,128,58,201,214,86,38,140,226,129,81,218,238,196,53,188,143,90,38,173,226,89,10,95,34,
  4,79,187,7,150,60,96,18,77,179,44,43,44,109,94,155,182,14,44,77,92,243,142,67,66,24,165,146,66,90,207,64,222,67,51,59,103,18,205,252,
  119,185,253,100,87,13,201,118,129,249,49,98,240,140,163,176,107,160,141,249,225,25,247,174,141,185,235,126,68,27,209,228,15,74,130,119,60,118,171,61,
  176,86,218,116,104,64,92,88,179,93,96,41,19,47,42,216,73,90,101,53,206,8,150,24,38,85,192,239,243,26,113,174,109,242,217,18,144,210,82,231,
  111,73,136,156,2,206,51,53,21,44,185,42,59,65,98,107,192,18,0,84,2,18,27,123,21,165,122,216,96,81,65,159,148,50,70,67,193,52,109,37,
  167,159,18,163,202,228,182,200,215,176,107,112,101,226,102,178,237,9,222,187,127,145,221,7,47,188,225,251,133,180,18,117,18,174,255,80,128,165,53,215,
  147,1,150,146,97,229,107,25,26,151,154,235,38,143,102,29,158,165,149,70,229,225,169,134,115,175,90,215,113,132,67,4,206,145,4,52,245,141,54,171,
  230,244,231,66,140,173,190,111,132,168,24,101,146,103,73,166,97,190,221,7,75,166,45,109,56,96,58,24,76,77,193,96,242,250,86,219,76,60,75,219,
  129,61,154,167,86,90,153,120,60,137,76,162,220,54,200,103,99,254,52,156,211,151,55,248,104,91,131,39,182,6,44,241,44,207,240,112,144,82,206,63,
  153,181,82,62,108,176,86,112,182,5,96,105,83,171,150,231,209,172,242,129,135,121,11,121,150,54,112,173,153,61,75,67,187,90,140,165,14,112,57,117,
  46,149,167,1,56,38,178,98,246,65,128,69,128,200,116,249,122,190,200,132,210,60,183,237,0,75,0,80,65,182,133,16,16,30,129,106,161,177,190,246,
  107,31,140,48,220,230,141,224,220,150,105,155,156,27,188,93,115,186,162,199,97,207,179,180,123,2,51,216,86,171,246,233,250,122,168,25,62,28,97,56,
  172,135,213,226,97,200,89,122,152,194,112,225,75,251,141,126,50,88,202,1,45,197,10,82,180,195,168,206,3,165,243,241,63,15,150,242,137,246,171,129,
  165,134,2,97,117,176,100,0,19,96,24,57,185,17,44,42,39,86,113,227,216,61,176,52,220,8,146,164,89,122,195,205,62,238,60,252,68,56,239,249,
  113,15,44,61,4,216,120,45,27,252,166,195,112,174,173,145,238,224,151,89,167,130,239,62,88,218,0,173,227,58,56,192,8,128,98,195,0,45,102,28,
  145,173,104,119,194,147,126,61,117,167,84,74,117,59,59,244,234,96,41,253,64,83,25,71,30,36,249,144,157,238,1,75,35,110,56,20,44,105,214,16,
  133,42,239,41,215,172,36,26,55,24,216,130,167,224,37,165,244,45,68,102,42,123,250,192,36,120,119,25,38,1,17,17,124,109,77,166,214,110,183,208,
  120,8,176,210,54,249,72,118,245,25,30,210,48,156,98,101,250,55,238,210,98,248,32,131,37,192,240,45,17,220,129,194,120,5,114,227,77,254,108,63,
  252,121,92,43,170,117,128,165,217,178,34,121,29,107,109,102,183,194,154,122,195,13,185,82,218,226,188,203,179,52,229,134,99,193,146,201,95,236,62,56,
  13,11,237,177,245,48,245,40,167,92,72,96,105,4,127,250,233,163,56,122,244,232,56,3,166,148,39,35,140,200,234,180,78,72,251,28,161,90,67,80,
  169,153,129,22,145,157,49,10,90,171,22,113,229,71,63,250,20,132,141,159,82,200,165,67,20,253,108,72,215,44,25,89,64,78,214,144,251,53,100,127,
  36,204,103,162,102,128,61,134,73,66,66,87,12,210,48,13,127,29,25,163,35,153,227,230,123,39,179,142,228,119,180,240,49,129,161,227,28,185,118,245,
  90,68,92,185,255,192,1,60,247,220,39,172,156,13,225,92,86,110,74,136,13,133,240,99,33,146,239,73,136,136,68,110,204,156,107,75,88,104,228,181,
  242,217,239,217,19,54,114,52,6,94,86,71,48,167,19,62,39,48,174,191,126,61,186,215,211,79,63,109,128,61,197,68,123,126,44,130,57,118,242,70,
  243,239,8,230,130,49,241,4,124,114,92,231,116,102,246,164,149,169,204,90,247,207,185,255,94,115,196,83,197,204,184,118,253,26,206,95,136,137,43,63,
  249,201,79,38,114,35,32,115,20,254,61,55,6,34,34,224,139,101,247,122,79,13,177,229,152,151,86,170,37,103,168,235,109,34,69,251,189,37,89,68,
  72,206,170,99,30,171,215,191,255,253,232,94,143,61,250,24,30,127,252,49,32,32,106,101,0,181,229,213,73,137,44,155,53,47,18,27,16,144,121,82,
  64,108,57,107,242,247,60,0,198,147,84,50,71,235,42,36,162,116,246,212,144,53,166,4,168,58,225,190,50,246,228,198,141,235,158,184,178,209,171,231,
  128,132,179,172,114,107,76,54,182,80,82,101,198,158,52,88,49,136,45,97,172,37,11,117,77,199,205,123,70,183,186,8,81,9,2,114,33,0,65,77,
  9,187,35,59,77,247,142,208,166,102,200,110,35,91,194,241,122,114,188,96,103,223,121,7,184,214,220,255,241,199,30,199,99,143,61,26,112,241,161,77,
  98,153,126,137,144,204,83,120,29,139,254,13,200,45,199,168,198,55,191,249,173,135,2,44,117,126,84,15,184,30,205,7,150,74,96,183,4,154,134,130,
  37,96,4,113,165,148,18,90,105,220,190,125,7,192,229,113,11,93,55,236,171,126,67,0,183,89,103,147,47,183,1,53,64,65,55,24,133,57,107,211,
  238,220,185,99,63,99,201,213,18,34,74,180,152,176,219,192,66,100,128,70,200,104,59,68,137,36,4,106,82,128,0,164,174,160,168,6,49,5,50,235,
  162,220,45,217,173,188,28,196,245,239,63,184,31,221,175,174,107,220,184,113,179,169,204,163,134,52,173,33,230,107,100,71,86,78,107,216,2,230,100,17,
  48,133,143,115,78,232,60,195,174,206,0,35,7,22,2,6,222,24,36,33,33,57,77,231,252,54,180,14,15,188,118,158,133,211,3,91,160,92,48,120,
  8,65,100,50,22,34,156,243,1,102,35,100,88,134,59,8,228,100,117,4,173,161,236,78,70,205,64,160,235,78,230,7,247,239,183,238,121,227,198,245,
  230,44,239,167,220,14,132,104,198,66,80,89,254,214,188,163,1,86,163,231,60,96,168,215,86,62,205,28,19,15,118,232,59,114,186,159,121,221,191,127,
  31,215,175,223,104,108,12,49,184,182,204,251,136,201,55,27,2,86,1,18,40,108,126,22,72,5,191,159,5,18,205,230,236,105,143,153,142,14,23,102,
  189,133,140,248,30,32,132,68,142,28,147,194,50,24,247,51,122,117,253,250,141,128,208,150,192,4,147,159,98,59,32,72,163,92,177,237,100,50,125,54,
  89,216,181,103,0,82,205,10,130,141,110,193,130,246,210,24,85,162,2,85,113,63,63,167,83,8,101,14,214,145,223,27,180,249,55,60,100,112,96,59,
  195,181,228,228,175,19,226,202,251,247,239,225,218,53,70,204,59,217,120,210,220,58,19,126,115,105,186,11,148,215,87,0,172,70,166,66,172,2,150,164,
  144,45,22,239,141,131,165,142,235,9,41,188,126,205,13,150,8,41,37,193,106,158,37,175,159,195,253,176,128,80,2,119,238,220,198,157,59,183,71,123,
  246,217,123,75,66,38,224,100,97,119,49,31,15,100,250,190,123,247,222,140,76,223,9,235,183,51,162,67,153,121,133,57,233,138,90,64,113,109,248,69,
  10,76,223,60,128,233,59,34,201,76,146,249,84,93,227,198,205,27,49,96,79,152,201,83,22,90,119,122,242,32,210,25,129,8,92,0,152,178,129,4,
  100,115,69,166,111,111,236,56,203,82,28,25,184,80,13,146,215,237,219,119,12,99,47,37,178,142,97,250,14,198,44,148,153,38,244,212,235,99,250,70,
  39,211,55,183,27,79,135,96,50,183,177,33,207,244,13,42,204,57,58,24,206,163,207,78,152,115,39,127,134,233,59,47,107,202,232,157,215,247,22,96,
  122,112,31,15,30,60,0,180,33,217,115,173,25,56,153,239,102,195,75,45,106,51,235,89,166,239,45,12,195,117,49,125,183,236,36,39,91,99,86,167,
  236,55,25,113,111,220,184,17,31,253,131,214,39,72,172,107,84,213,20,50,201,179,107,5,197,161,98,101,54,177,198,19,32,42,209,46,83,111,201,142,
  2,211,55,183,63,159,147,61,4,76,117,12,152,238,221,191,143,187,119,239,37,236,230,212,138,204,153,80,99,194,120,222,98,55,167,32,116,137,193,45,
  150,230,10,195,13,237,13,23,163,12,52,85,117,107,6,75,2,98,141,158,165,140,252,201,28,246,94,51,212,27,79,17,52,226,37,165,156,186,210,215,
  123,248,26,241,26,103,11,105,53,35,104,123,198,41,40,104,152,190,114,80,109,82,175,21,158,166,231,89,199,13,242,236,121,86,52,94,39,58,47,178,
  174,220,171,15,8,117,192,32,111,119,215,95,242,60,25,208,107,207,89,210,134,91,70,107,109,56,102,244,238,217,147,77,129,165,206,123,110,64,112,193,
  148,180,167,152,43,228,99,15,85,115,10,193,51,15,203,150,243,44,117,125,36,221,179,187,198,89,10,9,159,211,188,46,176,36,204,125,214,13,150,178,
  90,54,18,44,49,140,125,81,90,153,138,67,168,241,189,228,128,145,44,224,115,241,44,77,36,31,159,117,143,229,9,205,28,116,211,111,71,91,183,241,
  230,184,66,183,52,195,155,103,163,140,155,233,89,182,133,58,96,58,195,62,175,170,245,51,114,68,174,67,7,52,180,205,233,11,214,85,144,28,193,187,
  180,20,62,20,212,1,96,80,37,214,114,93,14,188,33,66,228,189,75,163,70,100,171,104,241,119,23,44,25,7,1,161,170,42,212,117,61,8,148,18,
  5,252,139,3,64,201,88,207,146,68,38,143,107,13,96,41,205,87,34,69,136,176,160,206,3,5,231,153,84,170,142,174,165,106,229,240,222,240,83,93,
  150,211,97,37,101,91,197,130,109,24,8,240,60,157,175,244,250,72,173,182,206,46,12,29,71,218,173,231,219,154,222,112,171,94,133,166,47,15,94,243,
  6,79,19,197,13,254,78,105,213,48,12,79,177,5,31,32,82,202,77,86,195,185,59,138,74,128,38,216,91,34,130,208,98,48,88,154,107,22,120,173,
  19,60,104,65,77,250,213,220,96,41,124,9,41,134,173,55,215,72,87,204,15,150,198,126,96,53,207,18,101,52,205,124,74,135,5,41,90,251,98,10,
  102,227,69,10,193,82,107,28,7,235,129,48,238,61,115,2,160,201,3,245,65,39,165,236,251,164,55,12,235,28,8,126,248,199,113,253,226,108,2,44,
  13,91,39,132,135,255,181,202,10,161,30,31,196,48,138,128,45,144,119,163,237,78,54,244,18,232,245,250,244,234,47,81,55,88,178,215,31,15,154,86,
  163,14,40,129,64,26,15,133,214,48,175,243,86,195,57,79,147,171,38,78,191,164,144,65,186,142,169,80,36,162,135,20,44,37,115,168,0,200,32,42,
  24,228,226,185,190,175,67,250,213,1,24,25,146,179,241,71,210,212,137,194,166,129,37,194,118,238,244,195,125,249,27,125,250,206,22,15,52,232,239,138,
  25,43,132,141,153,133,161,162,242,164,193,200,252,154,48,75,120,134,87,126,102,236,46,59,188,75,104,156,161,124,126,109,57,75,182,25,235,202,15,246,
  129,34,165,156,105,219,238,83,80,187,70,4,68,211,50,107,34,224,162,48,193,156,1,14,146,191,197,100,206,165,185,120,150,54,180,87,20,126,181,105,
  158,37,67,15,81,13,123,62,1,144,182,197,20,172,199,221,186,212,191,151,20,42,84,216,84,187,19,23,154,227,220,65,128,133,73,151,9,62,45,33,
  161,58,122,201,84,147,214,128,16,230,79,25,65,217,226,214,104,237,124,22,146,121,140,163,96,248,248,209,58,188,76,60,171,209,158,115,143,161,137,109,
  14,62,240,57,75,187,118,70,152,15,73,240,26,71,77,65,77,79,6,230,189,118,39,43,59,151,92,78,145,152,110,37,72,16,4,75,104,215,203,47,
  218,179,204,181,221,105,223,165,43,244,231,49,205,149,179,196,163,150,106,183,196,187,159,224,221,7,150,70,95,207,3,101,6,105,138,104,109,98,45,160,
  248,94,220,185,168,1,201,157,184,114,46,176,52,228,154,66,154,228,115,102,83,156,213,215,119,111,52,96,58,124,248,48,142,28,57,108,248,46,236,160,
  50,0,189,52,89,98,185,10,48,165,57,33,83,11,40,6,58,56,137,140,45,42,112,17,21,14,199,143,60,242,136,109,178,217,148,119,82,202,25,2,
  116,243,101,4,229,230,13,31,143,37,2,28,74,124,5,68,228,132,142,143,6,96,200,170,178,156,76,37,185,3,250,133,92,57,62,128,187,119,239,68,
  247,146,85,133,71,31,125,52,34,49,244,234,44,194,210,241,128,140,45,203,67,147,146,92,206,192,195,148,155,227,144,151,41,224,95,42,82,77,4,84,
  11,215,111,92,75,116,242,80,208,88,149,58,233,1,40,226,163,18,9,249,92,63,15,19,247,202,221,200,222,112,229,164,178,235,128,95,172,145,57,164,
  218,64,72,35,96,199,227,222,189,187,173,251,61,246,216,99,141,41,111,241,142,185,241,32,207,139,19,113,144,161,75,238,254,57,231,220,156,7,220,106,
  134,115,169,33,26,140,231,215,22,64,160,89,211,17,15,147,98,236,232,37,96,195,113,183,110,223,138,238,181,127,255,126,28,60,112,208,29,155,147,98,
  202,49,220,99,141,220,130,26,238,170,28,181,0,175,176,121,205,225,89,42,114,153,69,196,182,5,30,166,208,171,24,241,123,101,120,152,4,240,248,19,
  143,131,8,144,162,178,97,155,120,93,121,174,179,64,159,16,240,120,121,206,37,251,59,189,52,240,151,168,1,72,30,140,201,88,216,74,84,45,59,234,
  248,187,180,182,92,91,58,120,175,197,241,101,46,166,147,125,195,131,136,32,111,239,234,245,216,150,28,60,112,0,7,246,31,8,208,81,35,103,188,167,
  4,122,132,14,30,38,80,203,198,140,1,75,239,190,243,238,174,128,37,2,217,188,39,3,23,77,79,89,242,227,230,100,115,94,72,87,144,17,123,108,
  242,55,172,85,109,188,92,180,25,176,68,178,227,79,105,220,34,174,26,48,169,26,84,9,120,182,236,244,117,228,200,97,28,59,118,172,53,33,74,25,
  116,86,235,186,117,146,136,152,190,45,128,240,44,208,193,251,8,72,255,90,204,199,45,182,235,54,23,17,0,28,58,116,200,51,172,182,152,159,69,194,
  116,29,130,4,180,153,142,35,38,96,34,195,250,140,49,155,103,194,102,109,147,204,32,9,11,170,44,169,99,194,122,172,219,236,188,158,207,38,0,77,
  58,33,36,171,170,10,143,90,134,218,84,246,22,49,99,192,112,30,177,93,71,204,199,13,211,185,152,200,244,29,178,61,115,48,239,49,67,47,103,0,
  70,56,231,136,201,247,152,113,38,157,243,195,135,162,230,141,45,182,111,65,158,104,78,136,120,131,164,2,211,179,155,123,19,243,31,199,244,173,236,105,
  186,33,108,77,152,221,35,246,235,16,84,33,203,242,237,64,115,206,24,61,234,89,137,115,44,231,101,118,251,28,43,113,196,246,62,97,206,195,68,202,
  120,238,185,71,110,246,92,109,6,16,104,168,29,141,37,47,155,141,254,92,188,113,28,56,112,0,31,241,7,132,152,75,202,131,64,196,12,254,68,33,
  251,121,202,196,28,50,156,15,201,211,220,176,107,144,17,177,93,183,59,5,112,230,144,18,175,169,200,134,56,204,192,12,224,122,4,150,36,73,60,241,
  212,147,38,183,197,3,134,84,175,2,246,248,192,166,184,124,24,175,83,118,204,1,134,174,25,76,140,74,46,76,143,177,133,237,93,106,19,76,20,20,
  132,22,168,22,241,102,26,179,198,231,59,70,228,153,243,57,218,67,144,0,73,48,112,51,1,226,7,14,30,196,227,143,62,6,36,124,101,209,161,42,
  3,130,252,193,210,201,76,73,247,128,192,174,140,121,69,128,105,195,12,222,33,119,159,32,9,37,148,15,209,81,66,76,36,32,90,209,98,221,65,241,
  205,96,195,157,70,107,6,75,93,168,76,52,111,16,17,42,174,192,146,59,139,219,170,208,253,166,108,190,128,108,70,161,245,186,112,225,34,46,92,184,
  88,116,73,107,173,253,13,67,34,63,180,200,236,218,100,126,238,196,51,148,188,113,81,181,29,100,239,189,247,30,150,1,25,89,76,224,24,18,218,21,
  136,43,17,19,247,133,100,118,169,178,243,192,13,52,245,20,53,232,61,33,109,228,54,89,99,232,101,8,165,79,1,211,131,251,247,113,246,204,217,70,
  153,67,210,53,106,147,57,166,94,136,50,97,103,60,54,99,60,76,224,210,220,163,69,100,200,129,124,233,156,71,179,158,25,244,75,23,47,91,178,210,
  100,161,251,19,34,55,36,133,93,196,141,45,61,232,59,17,150,231,60,34,237,244,178,235,96,147,66,134,196,178,253,61,2,167,0,103,0,211,153,211,
  103,146,121,141,231,57,38,210,139,127,31,17,86,102,136,27,75,115,206,29,238,181,144,89,185,181,158,179,36,165,25,130,65,6,106,85,199,122,144,188,
  110,220,184,129,155,55,110,70,58,30,70,26,115,68,173,113,46,219,112,226,202,109,169,134,107,236,66,60,118,72,114,132,202,196,149,237,112,86,196,79,
  19,36,120,159,57,117,58,191,133,82,66,90,153,25,223,136,28,214,101,145,4,107,145,9,237,208,155,70,113,207,65,168,251,145,7,41,241,152,69,204,
  182,5,66,212,36,117,100,153,16,87,94,191,118,13,215,222,191,218,162,233,176,60,156,217,124,79,138,22,23,197,186,23,236,63,174,125,207,86,135,225,
  178,224,198,70,49,72,66,73,85,70,87,212,158,83,227,153,106,223,80,41,101,14,185,76,94,31,54,21,134,91,229,208,83,12,201,105,40,227,130,67,
  79,245,66,26,151,22,198,205,90,47,21,30,186,215,136,14,245,43,153,75,154,248,55,43,218,232,40,171,131,214,60,142,60,237,249,230,25,187,208,178,
  109,1,117,192,216,210,253,117,228,54,205,148,210,51,63,76,160,214,201,115,178,206,229,24,164,105,13,249,106,31,16,234,0,78,198,95,232,57,154,222,
  82,124,208,161,118,168,39,58,245,235,145,35,50,57,85,54,89,84,29,107,140,87,181,217,133,164,167,135,25,44,185,151,204,145,8,182,188,54,49,32,
  38,97,188,135,185,185,214,185,254,110,187,12,150,90,57,89,193,95,136,188,235,108,140,145,202,221,80,152,50,197,177,42,77,131,151,227,26,12,202,58,
  211,87,231,66,87,52,249,33,86,38,54,92,81,39,214,122,235,18,25,56,111,82,43,70,92,133,214,165,31,67,238,79,147,47,197,131,85,128,55,38,
  89,247,147,237,129,165,210,80,164,91,98,127,181,218,154,108,134,40,124,95,0,75,97,212,103,158,110,127,60,225,83,227,15,38,31,4,176,52,250,122,
  2,38,204,239,26,41,203,252,159,14,75,24,95,47,88,114,161,106,199,234,93,250,11,209,128,165,145,12,115,125,237,78,228,212,69,71,43,88,1,158,
  193,74,245,56,10,176,187,47,10,229,100,238,241,174,108,118,219,218,21,158,165,210,61,131,247,105,6,152,49,85,177,104,14,114,76,154,121,124,214,73,
  4,206,188,218,205,120,125,122,86,124,164,161,27,220,182,130,165,145,10,158,5,75,196,51,153,135,164,63,224,80,208,52,0,44,205,10,125,121,69,49,
  71,232,238,135,18,44,245,124,32,252,141,86,218,20,139,237,178,103,73,105,229,123,82,150,254,34,82,83,41,37,164,20,253,173,59,6,246,134,147,83,
  217,91,199,26,116,238,234,131,181,98,242,230,58,218,41,200,82,168,160,191,120,53,83,154,52,147,33,95,149,152,136,71,95,127,37,0,218,41,247,150,
  240,121,173,195,195,213,9,150,199,220,127,120,143,62,222,160,70,133,249,67,140,9,186,190,174,176,227,7,42,12,23,223,195,31,154,181,169,62,155,30,
  22,69,177,65,246,74,35,210,25,134,163,30,28,156,1,240,37,27,202,125,157,24,121,38,19,176,7,150,28,64,49,213,179,48,61,86,179,53,253,235,
  7,75,174,80,39,215,240,155,51,216,62,70,249,130,32,132,132,128,253,74,65,207,64,176,36,76,185,197,10,182,109,117,203,71,43,76,58,77,211,223,
  65,47,73,114,117,35,178,105,64,180,150,19,63,109,252,81,135,2,185,185,183,221,141,247,52,92,113,211,165,177,99,193,60,223,84,80,208,90,111,141,
  96,52,119,255,15,7,88,74,108,181,16,81,226,52,175,122,7,234,208,252,177,174,154,33,57,75,84,186,62,207,166,55,92,132,107,180,231,89,26,248,
  1,9,9,14,182,62,98,10,138,189,118,207,179,84,210,203,220,37,132,167,165,15,254,23,1,168,9,96,105,86,27,70,27,49,83,235,118,20,68,198,
  137,38,197,68,134,229,161,196,27,221,132,254,79,107,4,75,209,89,173,215,17,152,171,60,217,224,68,205,10,104,54,221,252,132,87,190,245,40,239,31,
  207,9,14,41,185,220,6,116,184,211,240,126,192,122,195,113,230,96,171,29,215,145,41,197,135,66,220,232,120,200,192,174,35,151,116,110,46,100,94,195,
  31,22,116,117,15,44,165,251,120,187,66,144,67,174,57,32,241,50,109,190,26,142,186,230,207,22,49,8,191,104,38,110,140,125,143,226,195,114,52,131,
  55,129,230,91,56,195,123,147,206,116,211,160,116,119,165,245,58,32,124,82,218,111,104,238,129,158,221,192,211,244,79,77,8,9,174,5,111,237,70,187,
  19,218,240,223,77,146,151,146,91,142,163,44,160,53,31,164,30,74,176,52,122,109,243,224,62,205,60,50,177,115,158,252,206,185,192,18,175,91,51,6,
  253,197,104,18,154,15,73,206,82,118,186,29,13,137,14,56,33,55,4,150,168,111,166,2,90,138,97,76,223,193,98,126,241,197,227,56,126,252,120,227,
  94,75,154,51,58,38,214,218,245,154,83,192,178,222,129,82,26,90,43,223,25,88,105,213,16,89,170,148,224,80,199,164,107,5,102,219,157,157,29,156,
  62,253,118,244,168,207,191,240,73,236,219,183,175,64,48,214,16,173,193,178,25,27,114,49,75,92,39,27,130,49,243,59,105,223,51,185,93,38,60,57,
  174,190,212,203,173,181,161,93,95,152,144,92,35,179,178,255,178,151,93,7,242,234,68,118,199,67,114,254,252,121,156,11,200,252,14,29,58,132,47,124,
  225,101,207,104,222,34,91,11,136,42,13,129,154,240,223,155,47,43,43,153,113,144,94,118,67,222,56,154,196,80,153,57,214,86,126,165,227,121,110,228,
  214,137,252,236,79,187,72,73,60,237,137,228,15,126,255,247,163,123,125,234,197,227,216,191,127,127,76,214,232,88,135,173,71,207,145,112,82,40,51,9,
  144,20,144,233,24,136,102,206,165,148,24,155,248,30,201,173,98,89,149,214,96,213,144,240,25,66,199,134,208,149,19,34,207,80,238,11,231,207,227,252,
  249,115,209,70,247,165,47,125,49,96,33,206,207,55,69,122,158,204,185,251,222,143,131,25,3,57,145,184,50,94,231,118,222,147,174,224,225,58,215,17,
  137,37,163,94,46,161,150,170,33,192,181,250,254,205,111,254,97,116,175,163,71,143,226,233,99,199,90,60,105,158,148,84,180,153,203,221,24,8,105,214,
  187,244,132,149,194,175,115,167,239,68,155,244,10,14,219,76,85,102,92,13,137,163,106,119,96,15,153,177,189,29,213,89,86,240,75,151,46,225,194,165,
  11,225,76,226,139,95,248,34,200,54,92,151,84,65,84,13,227,191,8,216,188,29,217,107,172,87,162,165,99,82,10,144,91,83,3,43,168,27,30,63,
  195,237,23,235,80,160,91,42,163,83,110,77,117,177,235,3,248,193,27,111,224,234,213,171,254,158,207,62,243,44,158,121,246,153,162,94,53,44,248,93,
  107,138,18,91,98,100,150,85,135,94,21,84,224,239,255,214,223,255,64,129,37,2,129,137,61,1,230,40,153,214,0,150,156,135,139,10,25,111,9,137,
  217,0,192,148,156,124,78,158,124,27,39,79,190,109,218,26,136,62,227,169,162,135,202,145,56,166,164,149,41,145,97,243,8,109,34,195,197,98,209,186,
  239,233,211,103,176,92,46,227,51,45,181,207,185,41,105,101,142,192,48,164,190,207,17,218,13,219,63,173,204,130,33,169,114,212,109,94,230,54,121,97,
  32,59,39,45,13,18,227,25,190,238,220,185,131,239,125,239,245,86,139,136,60,129,97,129,180,51,71,102,24,142,199,200,211,112,58,223,28,181,119,201,
  16,25,14,152,243,220,235,228,201,147,216,217,89,38,242,134,243,218,254,222,207,37,21,72,44,199,206,121,134,108,180,220,218,38,47,59,163,91,102,206,
  132,73,190,251,221,239,89,78,25,110,19,150,70,234,31,203,153,215,1,88,146,65,74,8,95,71,120,156,91,50,163,64,86,154,33,177,4,204,6,87,
  235,38,17,185,64,94,121,249,242,101,92,190,252,94,219,79,146,18,43,182,136,5,17,133,121,57,71,92,187,73,120,52,198,179,228,171,207,56,33,170,
  68,150,20,52,180,35,209,125,147,49,205,77,243,119,191,247,93,63,86,166,89,107,74,142,218,94,103,177,205,161,22,105,99,248,239,88,207,82,212,30,
  43,144,189,69,216,153,146,87,114,238,54,141,190,213,1,225,45,0,92,184,112,30,231,206,157,43,184,225,168,233,164,65,137,119,194,173,157,244,179,20,
  135,165,246,60,75,104,83,47,112,24,207,153,24,9,152,24,134,211,208,38,17,73,8,40,151,128,94,88,39,253,128,201,105,155,14,26,35,10,97,46,
  33,218,0,201,191,84,207,228,110,178,233,232,24,183,53,229,231,115,14,11,42,96,105,241,51,249,30,195,20,96,215,58,181,206,18,166,216,150,244,162,
  181,236,136,107,11,221,12,156,243,153,26,205,110,44,193,155,80,172,24,231,28,103,21,175,103,170,91,163,230,155,109,175,15,50,241,220,227,104,183,102,
  73,18,53,106,176,106,186,58,155,100,110,134,102,181,162,17,206,103,196,166,204,215,180,234,136,204,29,134,227,121,108,17,1,157,157,6,202,111,15,88,
  151,31,34,176,68,68,89,242,74,15,178,109,47,199,174,20,161,117,49,120,51,117,123,150,250,1,211,144,156,37,221,160,52,173,84,194,87,208,8,93,
  73,105,194,116,181,218,216,182,79,157,8,104,19,187,104,48,22,130,252,41,105,24,19,21,183,88,120,199,62,42,173,97,52,119,27,44,117,235,14,141,
  211,137,62,181,24,156,228,54,83,230,6,181,190,153,125,116,105,204,39,138,167,97,94,113,162,40,111,180,93,27,5,36,68,186,188,85,80,123,119,193,
  82,112,224,18,48,97,46,14,218,131,144,106,66,71,77,171,37,19,150,85,48,41,16,83,94,10,42,102,120,222,56,117,192,76,96,201,15,223,200,53,
  230,128,233,94,53,220,202,158,37,71,144,26,23,21,48,32,169,105,201,182,65,176,100,112,138,213,7,167,22,29,45,153,170,161,70,81,64,64,65,199,
  97,1,15,152,84,134,237,130,182,196,74,13,63,125,115,102,83,157,245,149,134,49,93,131,162,14,130,182,220,71,138,79,184,238,118,39,179,152,178,88,
  152,225,166,107,192,39,51,225,184,94,144,178,202,0,242,26,219,174,36,107,120,208,227,209,112,149,160,194,57,152,214,44,111,223,250,136,26,12,243,230,
  104,52,136,249,161,241,44,9,8,136,74,182,180,71,120,0,101,239,17,40,143,84,18,130,154,42,164,193,108,236,12,112,205,80,164,76,14,206,110,131,
  165,150,25,88,245,105,104,133,69,186,130,143,238,67,154,224,109,162,44,70,87,181,210,6,133,18,118,5,44,101,15,134,220,125,157,106,144,81,20,174,
  143,138,136,144,161,22,10,172,187,23,54,201,198,203,35,33,161,80,239,214,153,174,172,189,107,198,118,204,12,34,130,102,13,193,54,233,109,44,179,122,
  250,73,154,32,212,24,148,69,171,25,120,78,55,250,85,247,62,238,26,162,117,192,243,62,87,58,175,40,200,72,48,184,214,215,144,77,131,215,178,50,
  135,135,164,199,138,49,34,76,201,27,180,66,51,140,99,41,25,159,187,238,33,205,104,72,72,160,6,20,169,193,160,137,136,108,55,122,238,169,26,166,
  1,239,114,246,161,167,245,134,211,131,231,146,231,154,191,17,199,238,61,207,82,225,77,155,87,40,165,152,46,222,28,97,184,96,156,181,210,131,174,35,
  134,24,69,206,30,6,149,169,250,42,212,147,42,168,142,109,108,196,81,56,251,57,154,147,97,96,245,102,139,131,14,207,77,62,152,98,61,239,138,94,
  117,123,220,8,191,202,198,182,246,206,207,111,130,58,96,51,109,87,86,83,214,85,120,150,70,63,15,141,90,40,27,145,127,83,11,111,29,60,75,36,
  104,192,150,216,145,93,36,131,183,211,143,81,254,79,73,172,193,68,142,0,75,131,232,101,121,205,70,188,87,107,246,194,112,195,63,190,121,158,165,252,
  225,194,84,116,67,15,187,76,85,90,204,46,143,192,121,150,194,124,37,128,193,3,168,18,148,3,85,8,42,164,86,60,139,175,110,62,169,5,190,250,
  63,57,195,194,179,227,167,180,2,137,180,27,50,207,176,88,187,4,152,202,129,53,45,153,120,150,109,161,21,139,92,189,37,198,182,240,44,177,215,238,
  169,225,201,21,116,126,23,228,237,221,211,236,60,27,239,245,60,12,229,173,144,236,46,100,8,172,3,44,9,33,192,196,145,139,217,67,36,110,83,182,
  149,14,131,82,74,152,34,102,54,133,162,65,69,168,20,113,249,187,20,98,98,159,155,14,123,50,107,206,210,148,192,254,176,191,201,125,138,167,106,230,
  220,96,73,247,233,247,30,88,234,189,132,157,96,134,30,1,152,74,158,165,16,36,217,239,159,126,250,40,142,30,61,58,10,241,56,110,12,99,20,45,
  55,72,192,23,194,150,147,134,217,149,219,135,156,52,13,15,141,9,103,193,150,213,215,80,74,225,212,169,183,35,93,57,122,244,163,118,193,91,79,20,
  81,84,30,223,112,213,136,132,99,195,112,246,56,14,166,248,123,199,63,50,158,117,210,241,131,56,94,16,214,12,182,246,71,64,38,188,83,13,255,142,
  142,74,242,217,39,108,58,254,144,171,87,223,199,185,119,27,30,166,3,7,246,227,249,231,63,9,87,30,78,33,45,130,45,19,111,201,29,114,19,57,
  46,170,240,123,106,248,69,134,148,215,71,16,144,203,243,172,117,195,139,146,202,174,147,57,135,31,135,166,108,250,181,239,94,139,238,251,204,177,99,16,
  82,198,101,242,36,154,141,0,109,46,46,199,163,66,225,124,135,243,30,252,236,101,31,176,233,49,208,200,169,116,60,14,246,123,29,241,44,37,186,238,
  228,109,205,57,227,218,213,171,56,159,148,60,191,240,194,11,158,6,161,69,13,65,13,103,76,139,139,76,136,226,252,19,89,142,162,145,57,61,236,229,
  227,102,222,147,121,214,173,121,103,40,86,168,119,204,122,230,68,239,93,78,193,27,63,248,65,116,175,71,31,123,12,143,61,250,152,49,53,174,100,93,
  144,95,243,20,200,143,112,190,237,207,17,151,142,16,177,62,136,45,201,191,204,184,4,137,201,90,109,106,173,173,104,92,3,254,37,36,107,202,177,121,
  187,144,220,181,27,87,113,249,242,229,232,118,159,126,233,37,115,15,226,150,94,137,112,13,149,116,43,253,55,228,108,26,171,87,206,142,106,157,172,31,
  221,236,39,9,119,153,227,3,108,214,20,34,254,37,71,113,113,230,204,153,232,94,143,63,241,4,30,127,236,49,132,148,20,222,158,248,53,70,209,190,
  96,222,18,17,231,89,243,179,104,141,199,24,160,244,205,63,252,102,143,119,174,63,189,99,15,44,245,92,98,68,98,94,213,23,134,211,74,71,19,249,
  244,209,167,241,133,151,63,63,26,52,40,165,12,249,152,37,32,83,202,145,25,154,127,221,98,80,74,65,147,134,94,6,6,192,110,66,21,36,106,81,
  155,133,177,100,220,189,123,183,53,8,79,31,61,138,71,30,121,36,6,11,118,243,115,138,158,35,88,115,68,118,66,74,72,71,214,40,27,34,59,247,
  53,246,229,228,86,74,153,133,94,107,104,178,1,186,37,71,21,65,78,126,191,241,116,144,206,105,29,39,218,31,56,112,0,207,63,255,188,95,212,20,
  108,246,41,248,105,125,57,249,146,239,101,85,65,8,129,170,170,38,109,158,117,93,27,153,234,26,202,205,173,157,239,80,23,178,95,161,204,161,236,0,
  94,251,238,107,145,236,199,158,57,134,71,14,29,106,131,223,204,166,232,8,25,61,161,156,157,231,112,206,101,242,181,210,156,7,223,135,250,222,37,123,
  105,206,157,252,97,251,155,23,142,191,16,207,185,155,247,96,206,195,49,240,242,135,243,29,232,120,40,255,216,3,130,107,98,153,126,233,14,217,149,214,
  216,89,62,192,242,65,109,200,108,157,220,78,118,123,221,55,126,240,131,200,39,247,232,163,143,226,147,207,127,50,123,232,161,12,32,242,132,148,158,72,
  80,196,178,18,69,63,111,150,184,114,192,249,211,130,22,214,12,98,130,170,107,104,109,192,38,192,205,248,6,246,53,2,167,186,105,125,194,90,99,89,
  215,0,27,234,129,147,111,183,233,103,94,248,212,139,182,126,168,25,91,88,34,88,233,128,145,20,237,181,84,178,39,51,234,149,46,172,39,85,88,83,
  93,235,233,242,229,203,145,94,61,241,196,19,56,126,252,120,254,48,157,209,169,144,152,82,8,1,89,73,8,50,178,211,130,80,201,69,36,251,88,189,
  202,2,166,189,48,220,124,96,105,116,85,221,16,151,213,20,119,97,159,135,150,50,184,88,26,146,180,138,42,84,162,130,68,163,96,66,8,160,34,72,
  72,72,81,1,139,254,32,68,20,146,39,234,79,87,204,16,246,205,219,52,213,140,159,80,38,121,94,67,99,169,107,44,151,75,232,218,0,211,210,130,
  162,68,174,14,203,218,124,13,144,187,40,99,39,209,220,8,35,159,140,231,208,113,141,230,33,149,169,99,190,187,174,221,146,101,194,115,173,48,24,197,
  123,246,205,75,58,231,157,249,71,57,153,2,195,223,247,55,84,26,171,137,107,189,111,61,145,245,100,183,218,33,4,243,93,234,23,79,61,54,37,36,
  75,204,205,125,235,111,39,146,211,110,4,44,9,120,22,119,247,188,12,134,76,146,139,138,50,83,60,15,85,207,97,192,207,155,27,100,199,238,221,80,
  191,70,33,33,42,216,155,185,71,147,122,244,124,200,123,185,103,27,218,125,42,123,15,144,57,196,219,136,178,134,134,38,94,143,61,73,30,180,77,214,
  187,7,150,214,5,150,154,144,220,136,71,57,121,242,109,92,186,116,105,148,130,107,27,102,243,110,84,251,179,103,56,214,12,33,76,211,77,79,53,226,
  220,171,196,129,235,177,137,138,184,176,135,135,124,118,23,57,117,234,84,210,167,54,97,240,77,78,14,0,129,4,64,16,65,91,9,4,174,253,176,205,
  2,97,108,107,148,198,35,100,195,78,196,32,101,203,167,45,128,98,98,84,168,0,10,92,232,64,20,138,74,153,110,239,222,189,19,221,233,246,237,219,
  248,238,119,191,23,179,58,39,70,172,37,183,117,37,83,24,178,76,218,0,144,117,193,79,169,184,247,45,93,194,211,109,26,122,202,189,135,144,37,188,
  153,220,146,142,159,56,113,2,90,115,194,234,28,178,60,55,178,57,47,35,92,104,54,106,39,18,204,121,137,229,124,244,156,187,57,205,203,239,195,5,
  72,228,119,12,205,220,156,178,239,222,187,219,186,219,107,223,121,173,109,200,109,24,18,14,103,6,33,58,36,97,170,52,116,149,254,59,250,64,16,204,
  185,63,217,131,163,112,9,160,97,138,109,155,176,202,178,222,177,27,78,194,102,93,48,72,151,46,94,194,141,27,55,131,141,195,111,227,17,216,143,116,
  61,92,235,54,204,18,201,43,130,112,54,104,5,131,60,95,222,151,169,38,50,45,69,140,167,157,227,70,185,54,140,237,66,184,105,40,187,177,35,205,
  168,114,194,53,115,239,254,189,214,125,255,228,143,255,164,173,251,161,94,5,225,41,10,108,9,34,27,130,38,100,229,198,124,82,243,113,183,111,184,118,
  47,169,221,48,57,40,198,164,134,191,243,18,7,100,224,58,138,174,223,188,117,43,186,211,185,115,231,240,254,149,247,19,76,22,176,255,39,235,202,135,
  43,189,156,65,8,24,212,0,77,49,3,24,223,243,44,173,9,44,153,249,201,117,82,232,4,76,41,241,164,206,148,172,220,185,115,7,119,238,220,25,
  252,176,146,204,41,38,234,13,229,55,254,38,182,44,164,128,96,1,136,1,237,81,236,183,11,185,104,8,50,201,128,144,59,119,238,250,214,40,137,53,
  109,76,106,79,91,148,124,123,144,233,61,208,153,209,2,7,238,125,98,64,19,163,98,9,170,16,44,114,196,212,255,209,63,220,234,38,94,215,10,215,
  175,95,207,182,131,233,107,139,18,183,12,9,193,98,243,253,148,165,16,181,187,73,230,155,3,171,197,141,5,183,191,215,25,189,46,223,233,246,237,219,
  216,217,41,207,121,35,87,247,188,183,228,95,197,182,5,121,35,185,118,40,221,45,97,60,250,136,6,90,103,170,43,175,93,187,158,193,53,113,155,16,
  206,201,156,120,84,41,227,161,152,42,183,151,191,79,102,110,128,112,189,172,145,104,123,231,220,63,120,240,0,15,30,60,64,218,175,35,76,234,165,180,
  45,80,219,245,213,2,90,147,229,78,79,11,51,129,37,223,154,196,62,150,3,71,208,73,219,153,98,171,144,246,6,145,14,107,78,228,107,215,174,117,
  251,100,40,225,172,75,90,208,68,192,213,254,151,39,233,85,166,229,144,230,200,182,102,191,207,202,109,53,35,72,89,241,61,79,237,235,254,189,123,184,
  123,231,110,162,75,169,236,73,222,80,90,185,109,185,134,104,128,167,126,229,106,56,218,3,75,179,120,150,68,155,54,41,247,18,93,143,226,154,10,78,
  141,64,44,22,11,136,74,64,84,65,30,80,37,13,31,8,73,159,144,59,249,37,102,177,74,155,123,113,219,77,203,118,245,170,74,181,24,10,121,204,
  35,175,82,17,199,211,134,100,50,85,14,37,5,138,225,15,41,209,40,141,159,68,30,252,249,241,189,210,198,47,86,90,229,143,139,33,149,161,195,195,
  211,77,203,186,23,195,90,22,47,239,130,108,235,0,75,32,115,156,149,82,164,133,101,91,214,103,8,221,213,136,107,96,240,206,221,142,215,32,204,202,
  212,1,189,116,76,123,212,1,91,1,150,96,137,51,5,32,42,129,74,84,197,246,44,162,123,99,158,142,38,68,210,145,218,37,90,66,0,194,198,206,
  133,113,90,2,48,20,4,26,186,136,240,218,238,225,246,147,11,18,137,160,148,27,25,3,226,4,161,18,21,42,89,217,16,205,154,173,73,143,33,229,
  218,82,53,140,197,10,189,155,231,42,91,42,97,181,165,48,82,138,117,50,103,103,61,79,133,11,172,248,28,206,95,54,89,171,10,247,167,185,199,123,
  238,77,190,56,7,212,59,73,52,129,16,177,199,55,51,78,81,231,192,181,51,130,37,146,100,219,145,36,135,39,158,120,186,217,216,201,112,179,237,78,
  120,200,154,79,190,103,154,48,33,51,195,178,61,176,180,69,96,201,58,133,132,253,159,193,40,121,192,84,241,0,211,67,68,195,89,97,237,127,74,158,
  41,1,1,37,116,132,194,41,224,168,16,16,3,39,62,223,99,77,46,36,22,88,56,247,141,137,247,251,208,34,53,229,210,210,117,103,55,156,36,174,
  207,221,90,188,74,190,169,39,138,173,225,24,64,205,53,36,203,110,47,17,175,10,101,19,37,154,208,30,100,181,180,127,238,158,207,21,200,136,74,30,
  157,45,44,18,47,142,13,141,45,171,160,233,183,163,178,18,96,158,41,78,39,51,199,206,108,62,35,73,162,182,33,199,209,42,64,107,116,98,109,26,
  44,185,11,43,123,156,165,30,6,239,217,230,106,181,195,89,201,39,62,184,237,206,4,176,52,76,0,158,73,244,129,158,165,65,124,112,123,96,105,91,
  192,210,216,87,213,119,215,33,49,103,2,1,146,33,32,61,66,235,250,180,249,12,39,148,228,129,207,171,111,239,28,104,56,152,76,88,81,6,212,182,
  173,98,25,151,39,32,42,147,72,25,222,113,173,59,45,101,55,71,165,20,164,172,86,243,102,209,16,67,215,238,156,183,121,134,106,123,95,230,1,13,
  67,104,240,130,166,65,242,39,225,65,167,87,171,230,46,13,6,15,153,191,228,233,132,124,101,187,77,121,249,157,17,153,81,199,251,237,113,223,230,149,
  200,187,82,199,152,204,238,181,46,89,215,192,134,207,204,80,90,65,192,228,54,78,109,197,211,15,192,135,2,102,26,61,244,107,109,164,59,72,53,120,
  38,124,184,199,224,189,7,150,26,120,210,185,62,28,249,94,241,99,68,134,14,64,44,140,87,73,140,91,208,46,132,70,160,194,223,230,220,249,171,122,
  65,2,224,228,170,29,42,50,249,85,115,134,231,120,252,25,38,234,10,78,19,140,85,231,8,210,74,168,103,45,7,118,230,149,247,180,73,141,74,50,
  185,22,171,14,247,74,44,221,51,13,110,22,120,210,58,92,84,171,111,94,235,2,51,189,247,234,240,145,108,3,88,138,112,135,36,91,102,108,223,211,
  165,128,39,13,158,98,90,245,217,187,58,201,110,32,12,215,233,112,26,162,145,188,154,95,149,114,39,243,1,218,180,7,150,30,50,176,68,237,67,140,
  24,246,119,121,238,9,73,210,103,151,143,125,105,202,228,43,245,118,155,231,17,45,48,198,159,154,154,80,221,120,39,242,170,96,105,250,106,157,107,107,
  163,13,130,165,20,184,81,119,155,154,117,231,105,172,61,119,106,51,80,148,67,15,227,232,254,128,97,165,220,204,240,100,192,248,42,169,166,5,154,87,
  104,22,184,50,56,95,51,88,98,91,13,231,250,93,233,90,103,1,211,238,218,167,17,157,224,136,198,93,167,4,150,168,103,155,216,248,169,175,95,195,
  246,192,210,195,231,89,146,174,99,53,154,10,205,170,123,77,153,15,9,18,230,164,195,192,241,23,95,192,241,227,199,1,9,44,96,188,74,26,186,23,
  56,105,173,160,148,54,255,214,10,75,189,68,189,172,161,107,141,165,94,26,46,38,16,132,144,65,187,136,230,223,134,111,195,149,212,50,30,220,127,128,
  183,78,188,21,221,231,165,151,62,133,253,7,14,216,245,41,18,110,33,202,80,247,7,108,223,76,80,194,176,13,19,59,70,87,195,129,98,114,178,166,
  5,173,156,220,90,155,118,25,90,43,207,60,171,52,131,221,239,52,67,66,128,164,0,9,52,92,62,41,183,10,3,231,207,189,139,119,222,125,199,223,
  227,240,225,195,248,226,23,191,152,112,163,196,220,74,194,115,43,217,22,40,89,230,111,105,127,39,45,147,175,180,227,52,222,30,27,153,148,151,217,201,
  170,66,22,222,66,11,141,44,87,145,45,85,255,231,255,252,159,71,247,250,244,167,95,194,254,253,251,173,151,50,149,61,109,111,99,255,117,44,197,148,
  178,158,219,57,183,223,19,137,9,123,17,55,12,196,58,96,31,86,77,107,7,173,108,139,7,213,180,5,202,203,14,195,225,4,195,17,243,238,185,119,
  163,123,253,204,207,124,57,40,135,207,204,121,75,215,195,22,32,109,166,226,134,189,89,78,154,115,102,157,215,247,22,3,115,188,190,107,173,160,150,134,
  196,213,209,112,52,92,84,140,63,120,245,213,232,94,199,158,57,134,103,142,61,211,240,141,69,124,64,150,59,204,234,66,212,238,39,98,247,167,150,238,
  203,65,115,190,185,93,87,74,233,237,170,211,17,5,13,104,13,22,0,52,188,46,165,107,42,228,63,107,217,209,160,253,204,249,115,231,113,254,194,249,
  232,190,63,251,179,95,233,214,171,144,167,205,174,163,120,124,201,219,14,167,83,78,199,218,138,53,192,63,169,77,136,50,214,41,14,254,109,244,172,212,
  130,39,210,43,109,32,233,235,223,255,62,222,127,255,125,127,159,143,127,252,99,120,246,217,103,141,173,23,177,78,181,214,20,229,116,72,120,54,116,33,
  155,49,49,108,242,34,47,118,199,58,251,173,223,252,173,61,176,180,11,96,201,21,94,16,19,116,221,56,118,170,18,72,114,47,17,148,181,130,128,83,
  167,78,227,212,169,211,147,54,17,255,111,192,201,163,149,50,10,28,37,127,39,92,76,33,79,79,48,22,139,125,109,188,247,214,137,19,216,217,169,179,
  7,154,208,115,212,230,226,105,56,121,0,134,178,185,85,67,184,136,250,78,110,156,146,81,6,114,105,52,242,49,3,21,73,144,12,50,15,184,197,78,
  99,65,88,204,140,124,251,246,109,124,231,59,223,77,100,166,70,6,70,150,111,9,57,222,41,247,239,20,242,194,140,236,45,254,173,228,103,127,2,139,
  230,62,57,151,117,212,84,255,228,39,111,121,30,166,28,39,81,145,131,137,81,150,27,24,221,63,175,91,223,209,226,32,3,231,249,114,218,235,157,179,
  107,19,0,190,253,237,239,180,184,98,98,103,93,156,183,87,228,96,138,200,30,135,249,95,58,101,79,248,198,208,226,98,74,230,220,126,68,213,117,76,
  92,89,184,209,197,11,23,113,241,226,165,236,195,196,132,131,200,115,147,133,68,132,232,231,159,218,164,103,41,188,7,193,180,32,105,14,164,25,158,186,
  132,127,137,163,184,88,194,73,148,177,39,57,153,255,205,191,249,227,178,47,142,218,153,113,190,135,114,202,249,150,232,214,42,158,37,205,121,254,165,70,
  38,157,68,7,185,213,187,59,149,61,229,97,122,247,221,115,56,123,246,157,238,141,212,237,83,148,177,179,201,26,244,31,35,154,182,152,246,192,210,230,
  193,82,56,221,236,246,10,179,39,139,196,7,236,251,245,0,182,153,229,76,185,5,197,40,175,173,168,43,123,167,184,227,98,148,144,251,117,69,166,71,
  180,164,32,130,172,196,120,209,121,206,186,139,238,28,11,74,239,199,29,107,146,121,220,56,172,150,234,212,35,14,199,63,211,192,63,43,37,24,241,164,
  12,166,89,132,227,21,22,1,241,248,103,230,85,158,129,167,172,209,205,249,88,186,90,94,180,195,110,60,224,97,168,115,195,31,35,247,110,129,165,124,
  40,103,122,238,21,184,64,196,56,82,46,42,84,195,205,178,130,6,230,44,209,152,39,232,170,227,9,105,68,152,103,83,126,158,117,109,238,129,165,221,
  0,75,204,166,85,89,37,43,203,31,73,198,195,196,172,145,182,114,218,212,75,8,209,102,241,14,65,0,122,107,139,64,25,69,95,53,119,91,192,244,
  174,99,165,166,211,28,164,43,60,179,234,27,214,89,251,131,164,97,229,28,148,220,151,134,27,183,12,107,100,235,219,153,139,205,199,25,36,26,96,56,
  51,32,176,55,185,153,18,233,86,72,85,155,55,103,105,88,253,14,117,142,33,149,47,85,218,109,104,26,116,226,185,230,57,39,191,4,42,85,65,113,
  221,134,11,156,217,249,120,212,163,175,54,83,27,4,75,174,67,130,115,156,140,45,166,201,41,193,60,243,182,198,220,167,193,9,222,169,124,43,232,219,
  216,185,229,161,43,102,48,199,192,30,88,218,82,176,148,1,5,237,101,88,217,146,118,165,212,96,238,165,129,110,155,89,204,50,117,160,249,201,204,211,
  193,134,26,94,95,144,128,8,218,18,204,231,89,26,113,196,165,254,83,76,63,211,72,161,142,134,202,207,55,231,190,51,182,10,108,200,62,62,108,206,
  105,220,177,143,128,89,108,18,218,245,11,173,211,236,56,183,203,56,211,66,227,128,216,172,64,177,15,208,245,84,115,73,72,64,50,100,37,81,81,85,
  214,247,25,8,70,183,21,44,145,53,206,225,87,219,3,79,182,127,154,238,133,66,196,195,106,24,25,125,164,163,28,199,154,54,12,150,250,15,193,52,
  126,161,174,184,127,228,175,199,195,78,239,123,96,233,225,1,75,13,102,178,201,209,149,73,54,54,135,60,1,6,163,86,245,188,160,169,87,221,169,227,
  212,56,28,100,116,123,245,105,148,247,201,37,135,206,119,154,238,242,106,245,21,64,175,41,104,66,155,154,223,242,214,73,43,62,12,13,138,52,174,206,
  79,206,51,141,65,239,141,6,0,229,236,232,113,39,52,95,73,15,86,225,89,26,175,111,134,174,132,54,72,59,74,91,0,150,220,123,38,181,91,121,
  27,148,190,180,111,54,219,127,3,94,109,34,218,35,50,55,117,192,228,79,14,11,195,165,7,173,89,215,111,31,88,154,209,179,196,174,145,240,30,88,
  218,21,176,228,1,83,168,13,172,52,54,253,162,200,13,193,235,185,3,149,222,203,121,95,226,159,76,229,218,170,96,105,216,118,173,161,215,54,4,51,
  66,172,49,26,56,62,135,170,227,179,52,185,95,204,186,249,158,186,255,144,152,7,66,225,146,117,167,113,195,197,3,150,64,214,35,181,46,207,210,52,
  156,46,43,137,74,84,195,245,99,188,104,211,61,75,115,53,159,231,124,78,144,16,134,182,37,215,4,29,26,32,221,183,73,243,106,253,12,115,113,207,
  193,96,137,134,107,17,79,89,107,166,2,174,86,166,210,26,26,205,87,207,109,20,155,138,109,165,212,108,135,197,112,231,162,149,13,73,3,140,66,176,
  4,5,64,101,230,123,15,44,173,21,44,41,40,40,173,154,28,166,240,15,20,116,236,237,89,23,66,226,238,99,243,160,52,30,154,230,105,24,107,248,
  132,20,16,144,208,208,80,90,77,10,7,12,33,232,39,7,170,124,110,211,132,144,74,16,58,95,137,191,151,102,209,192,142,201,204,180,205,32,198,104,
  91,208,49,231,212,26,147,233,109,83,198,39,120,83,15,136,224,206,183,131,46,56,61,155,84,137,247,136,86,197,207,179,120,150,120,132,45,72,25,191,
  57,56,214,249,14,2,122,132,80,99,105,17,104,100,34,244,10,182,186,183,221,137,138,143,179,14,52,181,66,115,46,191,137,58,60,75,43,250,76,86,
  18,155,50,134,105,50,88,34,11,148,20,52,43,216,78,58,214,219,166,253,120,18,27,34,98,97,35,37,0,80,67,249,150,51,28,192,144,180,218,184,
  203,158,118,43,217,192,166,225,171,132,225,228,158,103,105,55,192,18,43,163,83,0,92,210,55,251,95,118,233,246,186,176,210,176,101,178,70,47,200,192,
  85,47,132,0,52,108,66,234,68,161,11,3,96,174,205,3,6,109,72,70,244,120,71,68,219,207,182,234,241,153,7,110,157,101,240,176,90,36,100,74,
  169,31,101,81,24,207,54,14,51,108,176,93,79,207,125,139,119,150,174,94,211,229,35,116,245,142,137,222,74,91,165,144,32,15,154,52,244,8,13,31,
  240,123,26,33,23,33,58,80,50,243,232,84,177,193,189,225,116,96,27,44,48,168,181,225,169,18,90,100,134,205,13,222,28,253,211,134,126,58,115,58,
  163,117,121,150,24,186,86,101,47,188,59,88,144,241,198,40,169,26,95,77,157,122,227,26,242,10,173,117,177,247,233,48,32,216,163,99,170,1,254,195,
  87,91,142,184,97,15,44,109,18,44,229,94,149,98,131,160,194,63,116,124,25,18,162,197,157,241,210,75,159,194,75,47,189,52,250,161,180,82,88,242,
  210,16,65,169,128,196,79,43,79,172,214,16,24,54,68,134,134,124,205,145,174,5,60,70,0,30,220,191,143,55,223,124,51,126,190,79,127,26,7,246,
  31,240,33,136,148,180,50,34,47,20,100,136,197,72,228,73,28,101,72,230,104,9,58,151,150,64,141,181,5,79,106,64,210,37,161,214,53,148,210,17,
  113,35,59,185,148,37,252,35,141,138,37,72,10,79,50,167,181,227,178,177,95,214,208,190,243,238,187,56,123,246,172,191,199,145,35,71,240,202,43,175,
  32,228,65,201,203,157,144,25,10,209,65,96,41,44,145,223,52,234,103,182,124,86,45,242,70,142,73,12,57,32,50,12,201,246,90,4,142,118,164,255,
  217,63,253,167,209,237,62,243,153,207,224,192,129,253,8,121,150,6,17,87,82,223,156,247,81,102,119,232,123,68,218,169,3,98,61,142,101,231,144,180,
  51,36,23,228,152,172,149,129,119,223,125,23,239,190,27,115,196,124,229,103,191,98,245,28,94,126,55,223,100,41,59,138,115,30,18,120,78,153,115,110,
  0,131,7,16,118,3,210,202,120,99,185,69,88,169,253,24,12,89,231,218,146,12,126,227,27,223,104,14,23,16,120,230,217,103,60,193,96,200,159,229,
  230,60,36,176,164,144,172,51,90,247,109,114,197,49,27,167,148,230,200,223,235,165,24,249,74,159,193,121,149,180,208,80,59,42,38,112,36,13,9,9,
  210,166,181,20,115,66,98,233,199,86,231,215,19,3,231,206,189,139,115,231,207,69,247,252,183,254,204,159,246,92,85,158,167,46,36,111,76,72,112,99,
  59,26,140,107,178,174,198,218,18,13,27,58,91,2,203,229,210,146,28,27,217,148,151,47,212,43,229,229,37,215,124,221,217,210,148,3,14,140,239,127,
  255,251,184,122,245,42,52,52,4,4,62,254,220,39,240,241,143,125,172,104,71,5,25,25,34,210,206,104,77,53,228,203,204,46,58,33,26,242,231,0,
  68,41,182,57,106,36,76,161,131,144,22,28,53,243,255,15,126,251,31,172,236,169,218,3,75,43,130,37,203,195,164,161,81,9,37,80,103,60,38,18,
  50,158,96,251,122,235,173,19,120,235,173,19,211,31,46,36,112,76,201,237,60,161,95,112,26,8,54,202,148,126,96,223,190,69,251,249,126,242,19,79,
  98,24,97,243,48,95,169,131,180,178,49,190,225,198,75,197,19,161,94,234,198,51,151,153,219,198,43,165,108,47,26,130,98,109,100,21,214,133,172,133,
  113,15,8,187,120,82,98,186,140,139,93,235,120,206,110,221,186,133,63,249,147,111,103,206,39,5,153,97,155,19,119,145,86,2,19,192,18,103,21,50,
  37,49,108,230,187,17,142,51,196,123,60,96,161,189,249,230,155,216,217,217,41,156,241,154,19,48,5,121,107,45,121,147,127,87,230,165,200,234,118,50,
  175,69,34,199,224,4,204,150,16,208,178,104,167,175,63,246,4,131,92,168,248,76,228,207,202,219,232,186,155,243,213,60,75,211,101,142,163,221,156,119,
  53,106,224,194,197,11,184,112,254,34,18,145,91,94,15,202,122,64,130,243,186,104,116,98,95,181,128,218,81,81,8,44,107,63,37,53,54,146,140,33,
  85,181,26,110,252,187,170,112,2,176,228,251,74,146,241,254,187,70,229,142,232,87,10,97,108,140,173,90,227,156,253,76,54,153,220,126,147,83,245,111,
  125,243,91,131,60,74,241,248,166,182,150,27,157,10,111,52,198,179,228,194,110,218,120,150,56,71,94,201,25,66,206,228,103,46,8,239,137,43,181,153,
  243,119,207,188,131,179,167,207,116,68,217,26,121,35,239,124,139,180,146,226,29,72,217,144,154,202,29,178,25,84,217,40,70,54,238,182,7,150,250,192,
  146,211,123,26,209,43,103,140,103,201,217,131,202,161,220,12,98,194,250,211,152,184,4,231,114,121,134,19,92,239,157,145,150,113,89,30,133,193,21,11,
  209,156,2,117,147,180,173,53,35,101,194,55,6,142,205,41,158,5,180,98,104,40,195,69,37,122,50,150,178,225,140,241,145,166,120,76,231,38,173,225,
  41,74,16,253,168,39,4,15,114,194,81,95,191,169,213,249,77,103,15,195,149,216,101,250,71,131,87,108,183,71,235,11,195,77,48,144,52,38,108,53,
  101,144,163,173,76,152,164,114,123,83,185,144,198,163,209,17,245,99,205,32,65,13,56,25,83,80,209,89,178,44,188,215,42,122,101,76,180,36,9,225,
  11,73,66,175,201,132,16,27,245,148,100,78,106,241,56,67,24,78,195,246,27,229,50,128,29,149,157,208,175,75,122,224,156,242,152,137,167,102,30,219,
  206,9,246,191,227,33,221,183,246,192,82,22,248,120,178,237,96,0,101,37,167,131,37,13,211,38,77,197,88,164,202,45,179,181,82,9,184,100,88,159,
  152,218,157,233,221,206,132,233,219,62,166,164,58,83,247,183,189,165,134,194,125,99,19,13,1,233,78,19,214,218,73,11,138,36,42,99,220,136,65,146,
  204,41,50,8,171,7,77,81,90,242,210,208,245,50,164,60,124,110,48,60,87,3,219,1,249,94,171,25,148,30,92,188,42,110,226,2,41,229,208,52,
  46,66,187,20,188,35,105,125,16,115,227,106,133,137,131,230,185,169,199,162,1,19,215,214,233,108,106,19,119,232,54,83,175,7,164,175,213,171,16,2,
  18,54,119,80,52,192,193,131,22,93,246,52,177,253,27,134,9,249,244,13,42,33,239,33,12,237,135,20,178,1,106,34,57,188,170,198,251,100,90,52,
  136,166,181,78,129,22,169,192,149,235,61,150,131,212,153,198,101,188,49,208,218,184,90,46,153,17,203,85,151,24,149,57,33,253,13,238,204,171,112,226,
  83,48,191,72,61,236,148,72,217,51,54,118,217,187,61,160,189,175,198,9,222,146,228,30,88,26,9,150,116,205,145,142,12,161,117,24,138,111,82,176,
  4,244,52,223,93,207,49,146,59,55,215,252,90,30,176,233,116,24,170,242,194,24,82,213,192,163,194,51,156,120,233,12,17,95,210,83,108,40,30,152,
  2,24,104,56,167,81,150,240,59,242,198,77,239,159,23,94,163,81,208,182,64,241,222,88,250,253,192,129,104,161,157,41,73,236,180,154,110,79,240,36,
  13,49,49,196,131,210,75,135,75,69,249,103,162,177,242,113,19,34,114,6,139,138,172,227,217,146,184,194,128,112,235,228,207,64,190,48,162,111,83,47,
  156,143,194,166,182,164,40,178,134,82,74,179,209,233,242,4,41,173,140,167,169,163,88,195,133,34,211,86,56,130,68,244,92,174,162,203,85,227,164,236,
  222,97,190,84,5,25,145,119,114,132,81,169,91,89,70,110,92,52,80,211,76,53,145,37,79,33,50,135,65,150,189,124,98,147,192,82,199,42,226,21,
  141,168,134,50,115,80,44,234,27,206,179,212,93,156,192,99,206,184,123,96,105,130,142,40,165,90,30,219,161,96,73,185,170,74,236,34,96,42,158,120,
  162,141,51,206,90,201,87,103,143,88,4,125,68,44,212,133,25,198,149,11,242,168,1,152,128,134,188,45,228,81,222,37,242,39,175,188,52,43,116,8,
  25,237,89,74,231,59,239,69,156,234,158,26,38,71,41,42,181,82,113,232,224,113,160,129,18,183,193,5,141,144,101,44,172,154,18,134,99,102,203,79,
  18,220,67,132,39,229,126,239,40,103,221,107,92,244,132,240,138,60,244,2,2,36,164,73,20,134,134,100,50,132,189,153,75,74,105,60,62,100,229,82,
  90,101,61,77,33,104,10,159,77,7,61,202,88,192,36,102,167,211,33,156,225,215,30,72,117,201,231,55,0,231,93,234,116,208,15,93,77,156,159,238,
  17,204,218,233,105,156,107,134,162,26,146,42,64,217,188,175,1,4,146,13,200,224,113,202,74,221,187,248,96,207,90,109,114,140,36,201,22,97,49,131,
  7,175,47,13,219,110,140,114,27,117,134,58,128,87,90,156,31,58,176,196,96,116,21,171,231,214,208,96,176,100,11,7,242,246,35,247,38,9,172,252,
  98,52,137,136,174,82,67,171,206,70,153,60,32,197,119,68,11,221,169,156,209,99,44,198,10,234,49,198,39,144,35,2,232,59,105,199,201,153,195,195,
  112,35,198,141,167,50,103,55,16,173,156,75,213,74,91,95,195,184,206,52,127,204,19,238,75,5,240,48,143,108,93,99,70,57,217,187,14,22,37,176,
  228,114,125,252,23,3,245,144,134,177,227,141,185,8,148,155,195,210,249,16,120,136,180,194,44,94,67,21,42,144,52,124,250,66,8,72,162,60,135,81,
  120,202,20,0,75,151,100,45,179,151,118,160,137,4,249,103,136,114,150,8,166,146,172,2,168,34,200,74,54,172,205,129,19,66,146,28,182,250,120,72,
  206,82,249,208,24,59,215,185,8,64,134,114,180,41,42,135,205,148,165,1,96,205,189,164,151,26,186,209,35,61,206,254,103,207,212,99,188,210,241,131,
  52,124,123,35,237,8,113,123,157,244,130,165,192,35,178,71,74,57,204,179,164,88,245,120,32,17,121,151,198,164,25,21,61,198,0,42,33,132,77,172,
  115,111,84,208,98,6,182,111,178,202,162,108,194,158,213,36,233,215,40,59,91,18,141,152,193,41,54,12,198,101,158,150,24,216,12,211,233,97,17,166,
  241,89,44,171,113,155,164,62,151,216,165,78,153,83,72,41,183,106,152,49,153,139,218,118,166,156,165,17,142,150,116,195,24,138,75,74,88,128,214,60,
  14,189,190,164,2,41,101,214,171,74,40,243,116,173,0,24,121,98,206,146,171,250,76,159,93,244,141,42,149,247,104,112,38,123,207,187,158,25,176,213,
  165,93,48,177,43,12,194,146,65,36,253,241,204,175,181,130,46,72,72,44,177,196,66,44,194,55,161,150,42,201,167,106,64,147,176,8,76,8,1,173,
  20,64,100,12,119,114,3,41,164,57,201,130,65,154,70,243,0,205,206,238,197,220,51,231,153,35,143,182,161,56,214,232,98,203,101,48,20,20,164,146,
  150,59,82,249,49,128,128,61,76,79,244,44,97,104,130,119,23,127,29,199,139,206,69,69,85,155,159,169,217,27,45,248,180,162,11,146,73,25,115,192,
  209,53,0,44,101,143,171,123,96,169,243,101,200,164,7,84,167,242,124,93,2,171,136,123,196,198,205,5,196,12,151,54,167,4,195,129,161,77,62,140,
  176,204,42,10,134,81,156,77,197,24,179,105,201,194,214,16,75,41,77,82,116,203,203,50,52,249,153,122,27,50,80,215,236,209,166,13,87,199,210,46,
  36,192,206,71,104,158,75,94,90,3,88,74,38,109,40,144,24,108,244,10,62,150,174,79,114,65,87,214,225,97,235,29,150,65,108,232,93,27,27,141,
  124,248,1,82,79,145,79,4,44,202,182,79,197,160,82,233,174,28,69,234,216,3,131,13,202,155,173,156,119,68,186,19,147,75,148,166,65,254,84,33,
  133,217,236,173,12,2,2,88,148,65,19,11,235,43,100,235,237,114,212,6,33,169,34,48,58,81,53,156,19,14,101,158,220,104,185,59,39,180,115,84,
  28,80,210,193,117,144,193,6,201,188,213,201,230,166,116,154,35,54,62,12,55,154,55,181,24,27,230,172,199,43,116,38,196,99,158,236,35,202,134,163,
  131,196,252,88,191,24,46,33,157,51,59,21,147,1,220,195,167,240,195,13,150,36,73,176,4,164,146,89,90,31,31,173,154,96,195,242,121,137,230,231,
  202,112,174,176,55,42,212,85,22,193,192,225,35,135,113,228,200,225,34,72,114,177,91,87,230,23,222,216,57,141,52,43,240,146,161,201,144,139,41,71,
  115,111,133,19,36,44,208,2,164,20,9,151,79,131,26,29,95,76,248,184,71,142,28,49,191,75,92,11,142,200,46,250,222,17,146,69,68,135,49,217,
  99,196,195,180,130,122,120,146,202,22,33,99,243,5,79,78,201,141,49,10,221,186,129,50,221,190,115,59,186,254,162,170,240,196,19,79,68,59,70,200,
  7,66,158,91,202,144,250,33,148,19,150,136,13,13,33,91,72,126,56,69,161,195,143,115,66,200,168,91,50,195,19,189,53,36,157,177,219,49,228,225,
  186,118,245,253,232,30,31,57,242,17,232,195,186,33,154,115,149,90,150,78,201,211,239,17,128,128,212,16,169,252,9,201,105,231,66,224,161,115,158,206,
  123,32,43,107,235,52,74,101,102,52,118,186,25,131,59,119,239,180,238,241,228,147,79,182,0,19,181,198,128,2,242,193,204,252,103,190,250,202,120,151,
  122,9,174,219,191,55,36,133,18,172,217,148,80,179,37,144,100,50,243,142,246,220,155,249,117,99,226,44,73,28,194,185,121,227,186,223,96,24,140,131,
  7,15,226,192,193,131,144,176,132,178,142,91,73,146,201,59,138,100,38,200,125,21,42,146,8,185,213,98,162,203,252,58,111,177,64,219,202,57,199,189,
  228,158,135,2,191,149,20,4,33,165,231,77,10,67,3,74,155,48,159,35,108,117,50,9,33,243,148,2,153,117,231,116,72,41,119,24,181,45,64,180,
  37,250,12,236,133,27,239,144,207,14,1,249,47,24,184,119,239,94,235,118,79,61,245,84,203,61,79,176,85,73,164,173,141,133,31,223,136,44,22,194,
  122,93,40,210,179,254,186,197,126,99,98,108,99,172,71,156,147,215,126,46,5,164,41,183,221,213,171,87,163,91,28,60,120,16,7,14,28,200,186,165,
  41,41,136,9,57,236,156,124,146,140,55,209,113,117,185,228,126,242,234,195,17,160,116,155,115,152,47,37,89,66,72,163,115,239,188,243,206,30,88,42,
  1,27,178,152,69,229,43,35,89,113,115,136,10,29,131,212,125,24,40,62,41,219,164,239,190,83,86,200,0,126,228,200,97,60,243,244,51,141,235,59,
  240,72,105,109,192,210,82,47,141,59,186,112,29,205,12,86,26,218,245,5,210,150,173,213,133,90,216,42,146,96,8,22,193,223,198,196,135,202,146,142,
  133,67,245,145,143,28,129,148,85,196,124,156,50,63,71,70,82,164,12,174,142,25,218,177,182,10,191,185,78,143,218,88,160,224,25,157,141,177,244,224,
  193,127,175,45,184,40,128,168,128,168,205,184,243,155,87,181,88,24,192,148,18,51,38,4,156,20,178,94,183,190,15,254,29,201,122,220,117,34,214,90,
  123,217,180,157,231,134,213,155,61,91,111,22,68,166,36,136,204,56,153,16,43,126,228,35,71,32,43,217,34,102,116,243,150,26,238,146,204,158,185,215,
  254,188,170,220,204,129,188,158,217,154,91,243,238,230,62,202,249,11,115,31,252,252,235,150,89,120,226,201,39,90,243,29,25,114,209,6,255,229,57,111,
  62,87,60,16,89,150,254,90,215,197,22,63,194,110,2,154,2,176,168,27,6,206,136,213,91,167,7,133,102,158,221,230,167,88,3,167,140,76,6,72,
  17,14,28,60,128,39,189,190,7,50,167,99,96,229,173,100,133,170,146,94,190,74,84,241,122,175,132,215,147,200,126,58,239,184,51,52,214,172,41,40,
  159,115,163,181,97,137,214,142,185,25,2,114,33,91,225,61,15,152,108,104,39,245,94,44,228,162,233,153,215,177,193,132,54,196,229,218,120,198,235,214,
  26,138,193,122,67,14,220,176,200,95,125,255,106,107,42,159,124,234,201,104,92,53,43,128,9,74,215,166,87,91,8,132,156,142,161,177,171,126,77,37,
  76,243,163,54,237,204,175,226,245,196,65,199,132,178,220,225,186,10,9,47,25,140,91,183,111,69,183,57,248,200,65,60,254,248,227,141,255,136,242,76,
  231,233,65,11,246,231,102,77,153,134,237,68,2,213,66,102,193,129,76,60,175,62,84,9,233,153,209,219,128,105,15,44,133,7,39,34,50,224,50,164,
  0,225,92,56,20,73,209,129,57,228,48,115,243,183,68,96,151,71,151,1,75,140,92,149,92,15,46,184,112,225,34,46,92,184,88,62,1,88,50,198,
  150,43,51,242,12,185,148,132,224,68,148,176,0,19,1,66,186,77,144,19,246,95,243,218,183,216,215,186,254,249,115,23,176,179,220,105,15,111,120,98,
  104,177,62,115,11,92,53,94,25,154,45,12,215,80,11,228,153,143,35,102,243,136,242,152,91,250,164,147,138,157,123,247,238,225,196,137,19,62,212,68,
  105,195,200,144,249,185,143,245,58,199,242,189,66,248,41,4,202,232,149,29,197,249,70,161,66,240,220,185,115,9,211,119,55,27,49,82,48,157,1,151,
  115,133,225,178,12,231,136,91,158,148,100,230,164,201,51,167,199,31,34,188,245,147,19,5,177,75,108,215,229,121,239,93,254,46,180,162,234,172,171,219,
  108,32,128,16,178,201,195,227,160,250,53,101,249,102,29,25,203,182,205,108,123,8,136,8,76,140,235,215,110,224,198,245,27,32,144,229,65,106,207,121,
  152,234,40,237,38,22,205,115,226,129,157,242,210,141,207,0,122,169,252,225,81,20,42,239,82,93,30,214,130,134,51,105,124,156,213,155,72,135,90,140,
  223,220,218,160,216,130,234,84,175,222,252,241,155,173,245,20,38,37,83,235,168,31,235,157,27,219,198,22,77,232,70,91,248,149,14,100,143,25,189,57,
  113,254,114,134,9,54,30,75,207,244,109,95,87,175,94,197,149,247,222,43,111,134,97,56,142,209,78,168,15,101,37,134,172,164,231,230,139,100,202,13,
  71,13,116,71,175,247,192,18,50,225,76,34,106,40,124,96,42,52,187,178,138,24,230,16,38,29,217,52,107,19,42,47,132,225,66,53,26,116,140,238,
  10,75,229,192,86,41,7,138,50,249,50,157,222,173,41,245,225,52,238,34,196,140,76,68,122,122,179,233,81,83,61,109,113,116,93,183,196,157,82,218,
  14,179,92,76,43,130,132,110,105,104,88,237,22,229,175,50,182,98,158,178,130,110,134,66,161,60,22,67,137,28,184,167,83,250,138,134,106,204,39,180,
  73,130,173,213,178,156,23,64,22,44,117,94,181,84,172,145,75,132,201,231,25,57,96,230,140,159,200,84,151,69,121,34,62,212,71,179,39,29,154,52,
  111,1,193,4,81,137,166,82,143,251,109,215,84,176,212,165,9,52,100,110,185,115,53,229,175,168,26,240,159,85,201,76,113,215,74,250,54,40,193,155,
  70,133,243,198,206,236,240,235,149,61,103,81,206,146,66,146,119,150,124,124,15,44,205,16,208,104,108,102,216,191,178,104,211,88,39,7,215,50,88,26,
  12,152,134,132,155,194,19,207,224,129,139,184,72,98,16,37,101,21,25,65,162,129,138,60,96,17,180,27,77,83,97,131,93,7,88,202,45,180,228,95,
  46,240,106,210,204,54,97,136,121,88,27,235,59,181,231,132,114,160,138,70,62,199,228,20,238,238,207,242,90,70,183,131,61,98,36,201,64,193,187,20,
  235,70,183,124,212,5,150,180,134,134,42,23,80,5,121,27,4,2,217,62,15,76,93,154,74,147,103,74,10,25,209,12,112,87,5,20,140,97,212,208,
  179,145,83,40,168,86,187,19,1,27,198,22,129,101,101,172,226,146,30,240,167,52,233,92,57,246,145,20,171,118,205,252,96,249,86,15,197,149,223,30,
  11,150,194,99,59,97,4,187,217,136,91,186,92,92,130,208,210,0,37,215,75,78,78,191,222,30,88,26,0,150,18,84,227,242,13,57,3,181,203,21,
  181,121,176,180,50,96,98,197,65,191,31,107,116,101,185,250,164,73,142,116,126,114,192,228,196,5,124,40,100,18,231,136,230,48,109,148,128,173,44,90,
  154,233,116,62,207,193,37,191,153,208,184,203,211,208,15,148,193,5,17,205,216,238,100,69,38,165,41,189,92,104,216,252,13,1,162,243,123,150,250,64,
  106,14,45,79,87,55,46,57,240,104,77,250,76,161,51,135,98,16,199,60,219,130,145,66,2,28,48,201,83,110,32,105,222,69,236,83,55,229,48,121,
  214,5,150,104,212,226,25,224,165,201,223,132,97,168,97,24,186,243,243,52,106,185,174,6,150,168,231,195,67,130,12,4,147,251,34,73,56,31,225,168,
  225,164,210,154,74,169,3,24,43,234,199,30,88,26,5,150,122,159,99,224,132,20,234,122,68,254,184,48,92,16,151,252,168,131,110,221,36,9,82,20,
  72,216,82,28,16,132,250,5,9,44,100,5,90,244,121,9,86,237,36,63,228,210,180,178,205,27,189,247,240,128,146,249,81,199,201,161,165,230,25,15,
  219,220,60,75,83,140,63,245,77,86,241,233,163,159,120,48,96,220,164,135,173,228,41,152,80,231,77,93,231,248,158,19,52,51,186,18,43,220,41,76,
  21,188,75,62,241,213,114,16,185,131,14,15,62,12,100,214,220,132,78,201,225,1,75,64,154,16,89,208,227,49,242,248,140,166,96,136,45,230,168,198,
  192,98,194,125,6,122,150,250,223,11,195,112,237,121,81,254,127,186,120,45,173,52,84,64,169,76,61,247,236,103,101,153,193,179,84,36,165,76,79,73,
  84,156,20,105,99,186,36,169,33,61,29,4,154,250,42,253,154,113,142,170,186,167,52,210,219,3,75,227,192,82,160,198,173,48,28,217,220,37,173,76,
  17,137,229,201,226,92,187,243,142,34,232,106,242,195,193,84,122,80,109,201,223,168,41,245,7,76,34,161,132,52,213,52,145,58,153,138,52,230,128,215,
  155,128,5,87,134,88,14,20,37,126,135,188,25,227,156,30,169,119,137,6,184,26,214,232,89,74,207,252,45,62,16,238,113,17,244,61,43,245,108,43,
  35,242,119,70,246,207,155,60,110,125,109,175,70,133,225,168,112,66,94,1,177,241,236,244,128,221,243,202,19,58,97,141,200,217,163,226,131,81,193,153,
  98,12,75,154,24,27,130,20,1,75,124,43,250,12,47,15,18,108,12,155,80,216,161,156,109,72,144,20,25,242,76,155,200,44,64,113,178,188,88,193,
  89,131,145,96,105,29,158,165,9,166,38,103,63,148,33,195,107,54,136,48,247,3,218,84,108,169,32,49,159,230,112,222,205,21,134,27,96,67,184,231,
  57,152,102,138,98,180,111,24,230,107,134,28,65,81,85,220,30,88,154,223,179,36,218,96,73,232,166,234,212,179,183,43,46,251,93,123,24,99,34,192,
  196,10,232,107,152,172,131,178,59,173,45,249,36,101,92,154,150,99,98,33,22,80,186,134,242,221,159,205,131,75,18,134,71,68,48,132,144,13,255,211,
  96,162,109,154,168,212,195,129,195,74,189,197,70,24,53,210,73,255,107,107,1,40,89,124,81,231,45,90,113,72,214,40,24,247,25,240,33,214,143,187,
  26,49,15,147,141,187,92,115,212,51,44,27,243,44,133,247,27,218,103,110,5,220,68,35,180,91,117,140,83,202,92,173,37,168,162,94,176,212,219,115,
  119,160,232,138,85,116,154,175,28,135,145,104,59,171,200,187,177,7,10,243,176,129,165,194,225,203,209,64,64,59,155,225,136,53,17,36,113,55,128,201,
  149,89,179,98,203,177,197,229,133,149,245,204,211,48,111,215,74,96,105,98,130,119,150,29,55,179,183,173,98,241,8,249,106,70,218,99,240,94,59,88,
  42,249,18,165,176,85,180,3,198,105,0,189,158,8,44,208,0,0,209,228,44,57,176,148,186,194,114,7,127,33,36,132,32,8,17,228,55,121,94,61,
  195,253,210,95,254,68,35,108,91,57,201,176,13,151,40,27,200,97,172,236,100,25,169,85,171,34,160,194,223,118,216,176,150,141,91,123,24,174,59,228,
  210,14,130,210,234,195,128,222,97,104,191,55,167,103,137,250,128,90,161,253,207,36,45,232,10,135,37,97,56,30,118,237,86,127,171,136,170,136,253,38,
  195,165,14,114,204,19,214,234,208,241,101,227,129,46,22,75,81,91,209,105,69,189,222,118,207,18,217,67,21,51,244,210,114,21,121,34,80,242,9,200,
  217,51,118,161,231,89,52,116,60,179,85,88,25,44,77,155,208,26,117,52,6,211,193,210,48,220,194,176,189,23,247,192,210,102,192,146,128,39,255,164,
  196,211,55,21,44,25,15,83,152,89,174,209,100,243,231,142,17,10,56,254,252,11,120,254,248,243,173,11,45,228,194,244,79,130,54,140,181,108,122,55,
  73,109,168,203,181,210,198,248,90,30,38,7,186,88,187,159,25,196,4,77,134,20,143,107,215,5,60,37,38,179,166,146,129,251,247,239,225,71,63,250,
  97,244,28,63,245,83,159,51,76,173,136,73,236,66,146,74,34,97,114,46,130,127,133,39,107,12,137,27,165,7,121,115,188,194,50,199,240,139,181,33,
  52,108,136,43,27,130,195,134,128,205,201,109,230,235,204,153,51,56,125,250,180,191,246,71,62,114,4,95,249,202,207,70,132,157,109,6,231,156,220,129,
  188,225,56,72,49,186,125,113,215,82,87,202,201,169,61,145,165,151,61,36,160,211,141,236,49,105,103,195,227,243,79,254,201,63,137,231,252,167,127,10,
  7,14,28,28,39,123,36,47,129,132,244,223,207,66,216,233,231,220,145,179,234,72,231,35,50,79,110,136,76,67,50,207,148,159,236,236,217,179,56,123,
  246,76,116,253,63,247,231,254,108,116,8,113,107,168,97,175,23,38,84,5,227,237,21,85,67,172,231,230,218,205,191,180,173,60,192,150,124,118,217,46,
  201,77,61,201,62,165,91,154,251,136,202,121,155,173,140,42,55,231,141,204,218,202,156,103,193,111,214,250,127,253,95,199,115,254,137,79,62,135,103,159,
  121,22,68,166,77,130,148,210,23,139,144,236,214,247,80,102,177,2,73,171,111,64,108,189,233,174,105,249,232,182,223,19,246,17,55,150,33,129,165,182,
  122,165,234,26,247,31,60,0,52,67,89,194,212,136,16,215,177,100,219,107,157,123,247,93,156,63,127,46,186,254,159,249,179,127,54,246,210,69,29,17,
  26,242,83,35,119,64,214,72,200,142,237,108,225,47,54,118,212,117,148,247,99,128,208,118,36,246,51,145,221,237,119,36,8,223,251,238,247,112,233,189,
  75,254,242,207,61,247,28,62,246,177,143,69,133,42,33,113,165,211,165,72,118,103,71,41,175,91,225,90,49,45,61,24,82,200,40,68,71,50,223,97,
  227,183,254,222,111,238,129,165,145,96,41,90,203,28,120,203,87,4,75,0,80,45,117,221,24,65,141,230,95,97,218,146,68,198,81,0,111,191,125,10,
  39,78,158,104,63,100,37,76,188,208,41,35,101,220,243,174,197,133,99,217,212,1,123,177,207,155,201,144,56,102,6,157,25,216,183,175,77,92,249,195,
  31,254,24,59,59,59,205,41,152,99,119,10,181,190,143,9,236,114,4,142,243,30,36,195,214,5,33,171,49,50,178,167,242,55,206,246,116,35,187,117,
  235,22,190,245,173,111,181,142,132,101,18,195,102,44,40,236,173,53,145,200,175,255,224,196,113,34,100,78,118,29,143,67,50,2,197,155,252,240,7,63,
  194,131,7,15,58,19,222,219,68,157,13,184,138,74,141,103,152,243,18,201,160,227,177,105,84,91,7,223,151,73,74,209,209,23,233,15,255,245,31,122,
  121,195,43,176,214,38,212,77,109,130,214,70,221,203,228,141,10,10,122,169,179,224,200,1,3,215,43,45,235,36,77,218,25,181,230,189,69,58,136,182,
  206,119,204,249,249,179,231,241,238,217,115,102,19,47,122,36,115,235,189,91,238,135,202,179,20,141,37,71,207,186,172,235,136,32,54,179,154,188,128,185,
  67,225,55,255,240,15,227,177,100,106,45,47,110,81,130,196,164,149,0,151,73,43,87,244,44,113,210,83,145,185,229,210,201,102,21,49,115,84,218,95,
  47,235,40,66,242,206,217,119,112,250,212,233,66,196,58,148,47,78,240,46,143,65,243,8,14,48,229,249,150,246,60,75,179,120,150,210,131,133,210,208,
  82,207,2,150,12,180,241,28,43,38,113,211,187,38,53,160,148,206,156,108,10,174,173,128,154,220,32,254,188,247,180,29,250,38,207,149,48,38,152,50,
  104,95,235,139,236,140,103,127,91,207,139,186,6,106,108,181,222,72,114,198,25,116,153,231,16,181,99,33,172,78,193,149,15,194,173,99,154,7,87,55,
  49,119,11,57,178,218,49,110,155,161,252,73,188,105,15,161,243,44,205,165,245,180,204,221,163,57,157,187,83,28,15,108,100,74,185,111,168,92,247,72,
  3,214,104,22,44,77,210,133,135,16,44,245,60,52,9,42,20,131,210,248,254,204,25,254,212,210,168,55,152,161,99,94,54,145,179,148,209,40,150,220,
  48,66,59,110,36,142,61,19,194,51,194,15,187,33,143,52,10,81,178,253,30,88,90,43,88,106,40,248,231,1,75,64,152,244,173,77,213,155,15,205,
  1,144,16,49,104,26,16,55,132,54,73,150,97,197,92,108,229,216,230,245,54,236,140,130,5,52,41,235,101,26,177,13,82,254,61,115,184,225,78,107,
  155,51,0,243,144,99,246,27,201,73,127,22,165,186,140,65,122,93,185,76,249,36,248,49,94,22,30,51,89,220,69,46,24,234,70,71,249,220,32,209,
  71,16,34,186,69,189,6,207,82,238,17,121,128,14,148,147,221,75,91,10,121,131,162,180,106,72,217,92,206,138,27,3,17,124,218,254,71,86,237,208,
  171,210,170,55,167,195,144,64,138,204,230,58,141,161,190,115,244,198,176,190,15,5,6,67,185,9,183,25,44,17,50,237,78,154,182,83,179,224,199,76,
  133,9,175,114,192,156,9,44,81,184,31,247,84,219,18,76,175,64,153,22,22,33,6,75,238,95,77,218,49,27,6,31,32,251,51,151,206,227,200,114,
  167,101,60,229,53,215,198,219,4,185,7,150,214,5,150,156,119,9,58,163,48,211,192,146,55,161,33,104,202,188,235,87,97,145,98,60,252,152,232,51,
  87,77,227,194,222,149,156,201,217,108,45,94,230,246,247,92,44,227,200,110,172,27,171,135,226,114,114,111,31,181,35,245,40,17,15,164,30,240,223,109,
  146,103,105,101,0,73,73,180,39,153,243,17,248,113,54,194,198,169,99,17,61,47,141,219,204,184,27,54,40,165,92,231,69,255,199,218,241,164,185,234,
  86,247,21,184,31,194,207,43,165,80,204,143,68,83,121,197,14,148,137,241,96,137,6,253,162,103,206,185,115,33,229,39,156,70,206,229,195,228,89,10,
  30,153,155,6,126,24,226,93,229,220,134,20,210,185,12,134,48,61,43,97,86,207,82,79,155,155,192,193,166,201,228,199,150,174,204,212,132,141,25,236,
  57,197,132,104,72,45,43,84,168,132,196,66,84,54,36,45,122,214,106,207,243,41,64,161,222,3,75,107,2,75,64,211,121,160,89,11,171,129,165,216,
  195,84,128,82,38,129,155,61,231,73,136,198,83,0,229,204,174,127,208,158,243,1,121,101,118,204,185,140,113,76,44,153,1,77,193,210,32,91,73,235,
  143,194,113,87,173,107,108,157,252,105,201,59,101,74,220,77,5,101,162,158,74,176,18,184,90,135,103,169,203,211,196,99,230,59,254,92,121,17,117,1,
  101,202,128,151,21,251,203,141,241,44,21,136,135,218,94,181,14,111,18,243,8,224,200,201,121,200,88,14,71,218,39,165,136,66,119,161,247,168,219,187,
  196,32,165,125,21,202,90,193,69,175,225,92,109,229,210,67,11,150,18,207,146,79,143,51,94,198,41,215,138,242,205,50,222,108,238,56,184,110,22,44,
  117,219,134,172,108,214,206,250,200,134,12,183,58,209,210,45,159,56,44,218,207,68,36,32,221,129,196,38,221,143,87,3,13,82,4,37,85,204,207,180,
  7,150,102,1,75,197,39,229,213,158,174,202,88,195,24,60,7,15,156,86,146,132,76,186,236,98,195,60,65,207,109,117,143,63,52,180,92,172,41,96,
  232,88,32,67,55,125,66,182,81,237,4,236,176,146,23,165,44,9,101,251,223,240,0,124,65,157,104,145,81,236,97,183,73,207,18,250,166,179,237,103,
  247,234,66,195,93,74,52,192,211,179,202,92,143,242,44,5,130,246,77,99,118,108,146,7,141,146,180,149,138,67,202,204,113,130,115,180,196,181,111,237,
  161,181,241,60,105,210,147,228,213,208,16,76,43,77,118,215,26,240,254,175,28,61,67,135,87,181,171,55,64,92,8,65,15,61,88,170,185,6,171,240,
  250,97,120,187,52,94,229,177,109,126,166,78,191,17,237,22,88,226,97,202,22,22,139,40,165,124,245,183,68,80,161,70,227,142,206,156,113,44,8,136,
  200,75,103,66,128,20,115,93,21,54,239,166,42,125,15,44,205,13,150,26,39,207,60,96,9,224,60,23,188,110,26,38,249,151,204,125,84,152,126,78,
  66,150,21,176,95,181,135,185,203,71,108,133,153,143,180,189,12,211,90,15,172,14,150,168,212,87,109,32,81,231,74,124,158,204,35,61,111,107,2,75,
  197,164,224,174,68,224,145,158,192,158,14,6,180,10,216,94,121,44,104,166,207,152,109,161,174,235,168,17,237,160,4,122,13,79,106,56,148,131,134,147,
  251,106,165,251,243,177,168,167,167,75,79,91,20,158,52,70,227,215,238,195,10,150,12,209,36,214,160,137,25,176,52,165,157,227,46,36,120,55,30,165,
  228,177,85,172,103,115,84,196,10,8,147,134,34,77,162,125,37,43,67,113,1,25,20,38,132,255,139,95,82,202,205,131,165,25,44,217,135,205,179,228,
  254,186,26,180,169,41,195,231,161,161,241,244,211,79,227,217,103,159,49,58,215,193,93,34,133,244,68,151,84,181,85,197,185,50,119,244,210,148,64,179,
  48,241,119,102,104,54,28,27,28,149,96,7,60,26,246,209,234,186,198,15,126,240,70,116,221,79,124,226,19,134,245,55,45,27,15,56,121,16,124,47,
  18,94,17,145,252,59,55,48,208,150,107,136,67,238,33,78,185,67,50,252,67,182,36,221,93,242,242,229,75,56,117,234,109,127,221,71,30,57,136,207,
  126,246,51,193,190,31,148,76,59,25,97,22,53,129,188,124,13,175,138,8,248,84,104,118,214,78,39,147,155,119,207,185,212,226,222,73,56,83,194,146,
  97,187,233,252,225,31,254,235,232,218,207,61,247,156,55,60,17,45,2,5,92,92,41,143,140,227,76,9,231,222,202,63,251,70,25,204,117,56,255,254,
  75,91,200,194,218,175,1,248,249,111,86,251,229,247,46,227,244,233,83,209,193,230,83,199,95,52,121,23,137,190,55,44,10,198,15,108,170,166,68,163,
  239,68,81,69,16,9,154,4,159,165,144,144,251,100,43,172,224,76,76,51,199,33,31,142,213,117,167,223,154,219,242,71,70,147,241,205,111,126,51,186,
  246,71,63,250,81,124,244,169,143,122,160,219,80,129,52,50,35,229,226,138,230,57,248,222,233,251,26,195,138,115,131,37,102,134,218,81,168,107,133,90,
  47,155,245,21,114,183,105,142,233,89,18,221,114,151,187,242,254,21,188,251,238,187,209,157,62,251,185,207,38,41,101,110,108,155,117,228,249,135,67,59,
  98,199,86,216,247,196,220,235,41,176,37,109,27,106,117,43,164,176,104,113,122,89,47,16,19,228,66,226,228,201,147,209,181,143,30,61,138,39,159,122,
  178,129,140,148,254,155,236,37,186,233,167,40,164,136,236,10,107,142,90,131,69,180,49,193,245,133,148,168,68,213,206,29,230,144,54,100,70,24,196,171,
  105,230,214,131,37,118,237,156,230,5,75,121,192,36,140,162,105,112,148,252,233,78,161,79,63,125,20,63,253,249,159,54,68,119,1,58,118,174,197,136,
  81,147,77,47,151,197,98,1,45,53,36,27,16,37,132,128,174,13,225,216,131,157,7,96,214,166,26,143,0,93,91,226,74,107,104,133,237,19,149,110,
  50,0,112,243,230,205,214,227,127,226,19,31,199,145,35,71,50,196,133,109,80,224,72,198,100,72,52,38,165,151,109,29,60,76,142,104,77,41,85,38,
  177,76,136,28,115,96,66,39,121,10,143,60,242,8,62,243,153,79,71,128,193,201,236,129,97,34,183,8,8,11,157,220,173,121,157,241,21,201,172,148,
  1,225,17,121,167,246,128,66,7,96,34,156,115,102,206,0,166,79,116,206,121,3,14,40,75,170,39,3,217,215,97,224,163,57,78,228,30,51,231,45,
  31,133,2,94,250,244,167,226,249,118,64,17,240,134,219,207,55,145,159,103,63,38,129,252,83,245,93,46,36,14,44,14,88,208,213,150,221,207,123,78,
  231,3,16,221,53,231,57,192,244,185,207,125,174,37,183,200,173,245,194,87,56,239,15,195,43,28,75,104,115,96,92,170,101,164,91,14,68,232,220,122,
  210,58,62,124,218,241,125,243,205,55,91,247,250,236,103,63,211,178,37,72,14,153,235,214,171,161,99,145,179,33,145,78,217,127,17,200,92,81,5,45,
  24,23,47,94,140,1,211,211,71,35,189,242,242,39,123,135,57,88,113,68,0,91,45,22,118,76,200,147,55,171,157,198,78,47,177,52,213,139,73,247,
  35,18,132,133,92,96,177,88,180,198,170,13,152,246,192,82,223,107,93,96,169,1,76,150,14,64,8,1,201,194,55,95,212,105,211,74,221,222,0,3,
  68,21,185,247,133,16,222,77,9,0,66,9,40,97,62,47,73,250,235,45,170,5,106,213,144,190,144,4,136,42,40,161,65,150,193,149,52,131,237,9,
  208,119,67,207,210,22,36,27,71,24,106,33,138,51,118,82,210,186,53,120,85,186,66,20,20,122,125,130,175,180,84,149,147,147,111,151,241,105,93,59,
  184,126,68,208,153,220,55,74,124,94,227,24,164,115,18,201,237,158,139,205,92,251,42,190,48,199,100,204,156,39,247,8,221,243,209,215,6,100,239,146,
  59,219,91,45,227,237,72,159,141,131,28,141,146,236,225,191,45,89,147,191,91,73,118,5,240,130,251,3,166,110,141,133,95,204,126,254,61,31,215,128,
  57,207,61,51,37,99,156,206,127,110,222,137,54,77,186,54,17,32,192,176,166,47,151,6,32,9,24,59,221,178,115,225,26,79,214,83,206,150,148,54,
  168,222,241,161,60,65,35,109,192,142,20,215,107,78,167,18,249,201,122,151,88,50,42,33,123,175,27,218,209,150,94,137,252,65,75,144,161,234,16,82,
  64,237,107,170,78,23,88,248,189,213,209,62,184,38,245,144,67,28,188,123,96,169,247,26,154,215,6,150,192,64,229,123,174,72,64,42,227,158,23,44,
  140,167,40,147,218,112,242,237,183,113,241,210,229,65,183,19,11,129,133,88,100,31,196,185,135,217,186,225,67,119,124,218,10,3,161,75,213,135,186,56,
  27,50,123,227,141,31,216,14,219,237,190,117,46,162,140,86,136,38,227,190,159,28,142,227,222,95,151,188,7,33,251,119,94,254,230,34,183,111,223,138,
  46,123,243,230,77,252,235,128,245,217,250,157,225,182,12,242,49,251,244,251,180,149,136,240,174,246,217,150,2,167,115,206,77,136,81,183,231,219,135,98,
  185,129,6,93,169,57,175,191,254,6,180,214,160,24,17,7,174,116,100,231,29,94,118,225,255,157,39,172,194,189,115,238,194,3,97,168,32,10,27,112,
  12,141,0,224,206,157,59,205,59,214,224,126,235,91,223,106,151,143,167,115,109,67,115,89,29,8,67,13,163,141,174,245,48,73,137,170,170,10,31,15,
  67,67,129,252,108,171,246,130,80,179,47,137,71,24,134,206,223,243,204,233,51,120,239,242,149,36,119,155,130,176,100,78,86,55,247,193,120,116,39,205,
  173,164,239,235,8,69,213,181,10,170,147,221,184,233,150,30,249,92,54,118,225,206,32,164,159,112,55,221,187,127,175,117,175,87,255,224,95,181,144,104,
  11,72,4,97,207,230,189,96,157,5,63,243,68,91,57,200,142,130,179,97,108,39,171,171,147,19,108,188,62,66,18,32,141,133,187,126,253,122,116,217,
  83,167,78,225,194,133,11,205,222,209,90,83,240,118,211,91,153,32,84,9,130,79,105,160,14,43,234,168,57,0,196,213,166,123,96,105,142,197,183,22,
  176,228,61,76,174,89,163,18,22,36,17,138,92,44,183,111,223,193,237,219,119,26,80,36,68,145,159,73,44,68,54,183,33,30,164,68,177,57,40,3,
  13,91,39,132,189,18,172,65,216,191,191,221,26,229,230,205,27,120,240,96,39,106,4,193,97,123,12,167,252,160,236,105,20,171,156,60,7,78,122,218,
  38,130,115,237,34,56,44,68,111,207,126,26,146,91,46,151,184,114,229,253,204,132,83,155,182,191,163,61,4,17,173,176,172,250,199,100,112,171,140,64,
  228,190,214,40,55,110,220,48,173,81,34,176,20,74,209,209,34,35,51,22,43,129,37,46,151,55,55,34,119,201,30,207,119,168,5,46,20,19,134,230,
  226,57,143,1,99,12,163,198,181,6,225,17,178,185,208,75,191,49,12,129,127,252,125,68,29,148,162,227,204,109,239,220,189,139,59,119,238,150,185,155,
  90,109,42,154,223,51,13,44,160,88,65,231,215,66,29,0,64,89,62,45,99,115,185,61,45,209,189,51,237,103,226,133,229,189,28,233,235,202,149,43,
  25,215,93,78,197,82,239,82,219,99,187,206,4,239,40,84,205,57,242,142,88,94,33,101,148,227,25,70,55,0,224,206,237,59,184,121,227,102,193,165,
  149,12,6,231,201,142,137,250,57,169,54,154,224,189,7,150,102,1,75,128,77,51,83,181,105,142,27,126,173,118,194,54,213,3,147,12,18,21,150,31,
  197,250,79,131,171,136,114,171,190,187,68,108,157,96,105,48,40,33,228,229,95,209,203,69,43,95,97,61,99,146,123,80,26,56,95,101,225,120,248,135,
  215,10,150,250,30,62,95,29,24,234,174,102,155,179,209,87,209,70,227,231,122,21,176,52,106,73,39,171,150,178,240,30,157,116,1,147,148,57,87,29,
  251,176,241,44,217,127,28,149,203,152,188,43,42,82,176,140,154,188,206,63,35,206,47,167,77,86,195,245,85,198,102,129,245,212,85,194,125,77,168,246,
  192,210,174,128,37,172,15,44,89,192,20,204,188,30,120,25,209,128,34,61,129,180,43,143,12,168,23,60,181,57,108,74,44,182,212,42,133,237,4,30,
  180,218,218,89,169,221,73,4,14,104,32,122,234,248,12,115,193,48,204,1,183,86,31,19,42,77,109,166,196,60,219,141,170,208,14,167,213,112,149,185,
  12,148,71,236,26,188,78,29,232,160,188,78,169,141,180,226,153,122,233,77,144,183,79,182,62,19,64,125,247,164,12,91,56,141,188,36,13,7,141,15,
  41,88,242,230,87,136,136,173,189,219,44,12,64,159,197,60,165,129,122,85,224,29,216,4,88,26,220,4,137,76,229,118,184,147,100,235,66,135,128,115,
  238,211,203,61,176,244,65,243,44,37,208,7,131,26,123,70,6,82,116,243,183,72,33,61,57,222,56,179,94,96,92,226,225,107,153,49,124,21,208,56,
  11,49,31,88,234,184,95,59,152,50,226,89,168,180,107,80,225,251,193,182,96,222,49,161,238,237,156,135,32,165,210,101,19,214,107,26,135,25,214,14,
  150,250,30,165,69,14,25,52,206,29,118,97,26,124,227,164,161,250,52,217,102,43,50,227,149,206,45,93,107,121,237,60,228,27,2,75,217,207,142,209,
  237,185,188,119,61,81,39,30,203,4,190,138,103,169,235,209,195,147,135,94,53,229,128,90,21,110,43,31,68,247,192,210,108,96,73,89,47,252,186,192,
  18,71,166,78,205,231,109,112,21,114,189,234,71,3,141,31,151,221,174,156,124,245,59,102,104,26,9,230,236,96,105,4,122,40,8,19,203,205,25,214,
  218,190,253,50,185,230,28,149,45,125,99,66,211,88,198,41,241,60,229,231,156,90,30,182,169,51,205,107,214,129,172,132,165,9,83,147,174,54,108,238,
  87,5,75,43,200,203,189,72,157,138,235,156,71,204,234,195,30,134,43,129,212,210,129,52,151,57,68,5,79,16,15,177,163,69,187,60,102,97,173,43,
  12,55,252,164,239,122,198,81,114,165,94,185,105,232,243,237,121,150,118,3,44,105,214,80,90,79,178,147,99,192,18,0,8,9,49,13,44,233,50,88,
  26,206,227,67,121,157,143,78,202,92,4,13,99,182,218,190,147,24,141,159,245,89,192,210,168,52,140,193,127,52,86,26,218,253,156,165,214,115,80,97,
  250,58,188,133,204,29,226,83,81,239,104,146,185,154,224,89,162,18,104,140,239,91,148,147,59,54,54,26,2,149,70,204,253,64,217,36,228,56,30,163,
  92,110,75,39,3,253,48,87,71,73,124,166,28,144,158,145,65,100,87,60,75,99,172,12,245,110,2,147,16,12,231,103,40,15,60,54,227,89,42,95,
  207,53,162,214,62,133,100,144,197,211,193,62,199,125,177,136,61,176,180,91,158,37,182,115,53,180,99,193,84,176,100,206,41,51,122,150,74,254,155,241,
  199,226,48,31,135,242,9,192,69,99,58,20,84,229,63,48,200,144,206,230,89,42,155,126,202,122,29,186,146,51,168,99,3,153,41,79,103,205,158,22,
  30,176,63,82,31,88,202,125,158,230,155,163,217,189,47,212,115,95,114,134,128,7,15,10,141,24,189,108,1,69,135,108,2,2,18,2,82,154,47,177,
  16,227,150,118,6,158,210,136,191,238,4,75,165,207,103,129,244,60,222,212,221,0,75,181,94,66,67,173,120,199,241,25,223,67,163,125,163,131,161,179,
  130,165,238,167,212,172,253,216,21,109,67,192,41,168,169,207,115,65,123,96,105,55,193,146,134,7,195,235,6,75,0,80,169,204,141,56,116,16,37,202,
  242,226,139,199,113,252,248,241,142,83,52,121,230,104,192,144,91,122,98,174,208,11,101,91,167,104,173,108,75,16,199,212,218,208,219,179,102,40,85,251,
  126,73,181,174,61,147,184,96,129,157,251,59,120,253,251,223,139,238,255,242,203,95,192,193,131,7,155,62,62,148,240,130,120,166,90,209,80,248,11,71,
  103,239,72,200,214,75,186,230,184,83,180,147,87,107,147,167,162,185,221,70,2,156,180,137,49,239,159,122,251,109,156,60,113,194,95,243,209,71,31,197,
  159,253,115,127,54,162,241,15,249,165,68,192,179,100,216,120,221,24,24,121,27,242,181,53,19,206,57,38,98,118,236,188,28,200,30,180,116,72,249,85,
  184,161,154,248,237,223,249,237,104,83,255,210,151,94,198,129,3,7,3,62,20,228,219,223,100,231,156,54,195,246,204,136,101,214,58,208,243,144,229,186,
  145,93,179,198,242,193,210,244,138,3,227,204,153,51,56,115,246,76,116,217,159,251,185,63,239,145,126,58,231,17,121,105,151,190,91,254,169,210,134,228,
  235,93,69,179,73,56,79,178,132,176,156,54,61,115,94,208,119,215,198,35,109,145,19,210,140,48,128,223,249,157,223,105,72,45,153,241,194,11,47,224,
  133,23,94,104,233,123,155,145,186,172,239,142,127,107,246,147,207,60,208,41,56,65,179,217,16,52,60,179,119,104,182,27,91,18,180,28,114,99,11,4,
  45,105,146,150,67,118,19,59,123,230,12,222,121,231,108,116,219,159,255,139,63,239,237,57,69,250,149,126,101,116,203,234,21,245,217,209,149,59,96,112,
  98,67,122,228,14,248,207,156,206,188,246,39,175,225,210,197,139,94,175,94,124,225,69,124,236,99,31,131,118,237,134,172,220,82,74,80,5,72,89,121,
  153,5,145,73,36,23,2,146,130,238,9,150,203,110,142,215,111,254,221,191,183,7,150,186,60,75,192,196,194,179,241,96,9,0,42,41,227,144,92,45,
  53,100,192,128,234,91,157,176,65,221,39,79,190,141,147,39,223,30,245,104,66,52,124,76,26,166,107,180,139,191,103,121,121,56,32,175,11,71,199,246,
  250,212,208,96,169,113,112,223,193,214,189,94,255,254,247,45,39,15,101,14,148,253,92,52,157,148,2,51,123,150,134,113,18,37,218,225,166,66,213,209,
  181,110,220,184,142,63,248,253,87,139,222,165,28,15,17,35,207,65,181,118,207,82,32,103,8,132,82,206,41,46,28,185,211,167,252,222,119,191,135,251,
  247,31,116,112,242,180,199,32,157,243,77,121,150,90,28,92,57,121,57,156,103,5,173,85,139,107,198,189,94,253,87,255,170,195,219,82,150,155,209,102,
  105,206,201,38,76,178,204,44,237,114,116,107,141,23,116,61,227,116,161,228,25,79,159,58,141,83,111,159,46,56,141,168,245,151,179,50,218,239,114,24,
  78,67,67,215,170,251,153,50,100,183,25,10,38,107,159,219,90,243,251,255,242,247,219,154,196,5,183,76,202,199,68,3,105,111,231,242,44,121,112,141,
  130,78,133,55,108,184,171,34,14,65,59,88,39,79,157,196,79,222,252,9,106,212,193,26,136,195,206,37,207,20,245,53,119,223,243,44,205,6,150,84,
  192,118,63,222,185,52,13,44,1,64,37,132,244,39,71,6,176,8,92,65,145,155,107,5,251,162,109,227,222,200,229,185,136,147,22,41,221,32,211,166,
  17,228,78,169,108,27,200,138,14,55,92,30,44,77,115,226,174,7,44,13,13,99,240,132,191,25,34,20,207,145,240,61,117,33,248,22,13,100,141,97,
  123,174,93,14,59,187,223,141,26,140,254,208,229,164,202,192,57,147,134,125,90,77,89,48,98,238,212,4,90,85,167,168,45,27,217,57,17,13,119,200,
  64,38,226,49,235,32,109,166,197,110,202,155,39,12,30,116,80,33,23,53,122,157,171,148,156,211,67,186,219,57,75,66,11,104,145,116,98,72,219,233,
  228,54,129,85,232,251,121,140,194,237,66,24,142,185,199,120,54,107,137,173,126,57,79,119,230,148,96,198,24,77,199,8,87,25,62,196,238,236,129,165,
  53,123,150,92,184,116,195,96,9,8,154,239,230,62,36,32,226,102,186,51,190,212,82,27,5,148,0,177,229,21,161,182,210,19,167,73,87,100,64,19,
  8,172,105,20,0,26,77,132,184,33,176,212,238,39,150,17,60,220,64,178,207,83,174,254,75,61,12,99,42,140,214,9,30,184,103,174,7,35,72,162,
  241,12,159,180,121,121,195,125,141,61,52,201,160,24,106,152,174,181,214,25,80,217,169,213,133,143,116,204,61,115,188,65,137,102,115,144,36,231,157,103,
  230,201,135,129,190,185,140,170,225,168,123,14,30,86,176,212,108,232,178,156,199,148,81,23,94,225,48,70,60,131,189,88,27,88,42,27,138,246,33,188,
  109,115,180,226,166,255,105,130,136,132,110,82,20,52,116,227,101,162,50,122,220,3,75,235,1,75,10,10,168,195,246,73,51,40,222,132,66,136,106,240,
  16,173,1,55,41,215,234,129,149,105,152,40,76,188,88,100,67,36,4,81,9,232,90,67,104,7,154,6,192,6,202,123,28,178,21,251,180,75,158,37,
  202,27,185,220,210,15,55,156,201,96,128,86,52,125,51,130,165,162,239,36,231,101,26,139,135,178,33,73,4,39,205,245,122,23,39,235,6,143,223,216,
  58,193,242,4,217,230,0,74,45,95,146,7,28,125,82,141,247,42,122,255,88,134,86,130,74,11,254,97,4,75,19,148,171,203,115,215,91,136,82,224,
  17,229,44,84,231,126,173,91,107,53,92,247,7,56,123,250,68,227,165,112,120,201,87,199,37,4,4,26,32,193,201,182,168,34,189,106,181,2,219,3,
  75,179,121,150,244,74,215,92,29,44,1,128,232,251,144,20,50,202,105,90,203,203,157,166,149,169,72,88,106,155,216,232,150,95,224,118,19,149,0,45,
  196,232,106,188,97,190,168,225,213,66,179,120,150,70,115,18,241,136,246,32,237,205,130,231,56,37,174,193,179,52,10,229,13,104,63,66,99,158,129,104,
  237,242,198,247,165,238,231,230,121,66,111,101,121,251,101,155,43,1,190,229,89,162,82,168,52,223,26,102,168,90,12,227,89,122,120,115,150,134,90,153,
  126,190,185,220,168,240,36,176,212,190,41,247,175,167,53,130,165,62,127,15,15,176,21,113,50,61,69,235,65,10,105,138,23,148,134,182,73,248,10,202,
  56,18,148,5,88,202,20,57,69,121,191,123,96,105,165,245,226,19,246,235,25,79,17,60,89,53,251,163,178,222,77,185,46,204,196,25,196,174,205,191,
  174,242,67,41,229,17,191,39,144,90,12,224,123,162,1,160,168,244,222,46,228,44,181,150,114,232,9,226,196,53,52,178,203,7,119,52,133,162,137,115,
  181,170,103,41,111,230,59,158,175,147,119,128,38,206,253,46,122,150,82,134,12,46,211,217,119,73,221,23,138,229,94,218,202,100,205,211,204,57,75,61,
  109,114,122,231,147,202,188,150,156,59,75,208,132,185,126,200,192,18,103,55,129,164,160,159,250,22,124,71,131,41,238,134,87,187,75,29,128,97,132,118,
  212,175,99,233,101,148,173,180,115,207,226,114,101,52,43,255,19,43,54,95,53,7,21,158,104,42,241,234,61,176,52,167,103,73,37,189,109,77,6,179,
  24,216,97,96,62,176,4,240,176,91,10,172,169,228,218,133,151,50,33,49,82,6,52,41,82,32,73,241,242,46,176,93,115,223,102,83,238,14,178,235,
  96,169,211,209,146,101,175,166,88,110,234,218,48,75,251,8,13,243,178,172,197,179,212,215,213,179,236,237,107,207,57,245,195,225,177,173,81,54,18,134,
  163,120,239,74,248,198,24,236,217,137,211,235,240,64,125,27,77,74,41,86,228,82,27,8,150,134,249,129,218,219,114,168,239,60,116,225,61,236,57,75,
  201,181,180,210,241,5,86,37,165,164,97,118,148,231,85,254,249,192,82,107,146,199,53,98,141,101,111,64,18,131,205,225,193,209,98,216,4,113,14,170,
  60,247,114,150,214,11,150,162,212,78,183,175,57,26,28,221,7,154,230,5,75,131,60,76,238,37,231,6,77,5,16,16,253,171,0,40,187,139,136,204,
  126,218,235,108,24,224,97,160,45,241,44,21,159,153,51,39,37,26,188,59,148,206,144,52,121,174,230,183,153,189,179,196,35,105,246,10,30,6,30,234,
  101,89,51,88,162,129,58,71,90,76,88,228,249,94,89,180,38,217,166,131,37,26,235,14,232,188,97,105,249,180,238,50,52,227,123,139,195,112,26,58,
  160,167,192,72,61,31,225,86,142,192,82,191,237,164,93,3,75,40,131,37,90,125,254,226,74,186,246,250,202,181,139,105,247,52,219,3,75,163,193,18,
  123,240,97,162,73,98,12,29,200,252,96,9,8,170,228,134,190,250,136,43,167,106,125,74,98,200,220,144,218,177,0,4,19,4,73,227,10,149,230,239,
  239,63,184,135,239,125,239,187,209,117,94,121,229,139,56,120,240,145,6,145,58,82,59,65,9,121,165,0,2,66,59,71,102,183,201,87,72,96,153,37,
  173,116,242,115,202,87,196,56,121,242,36,222,58,241,150,191,214,99,143,61,134,159,251,185,159,243,94,139,22,97,103,52,6,194,202,29,18,248,209,230,
  228,134,6,152,176,83,239,128,89,67,176,4,35,33,173,44,16,237,129,129,223,252,173,223,140,174,247,229,47,191,130,131,7,15,122,30,148,136,188,209,
  145,237,137,102,222,73,132,99,32,176,65,209,227,57,231,134,120,47,157,239,229,178,70,189,99,137,90,173,224,167,78,157,194,233,211,167,163,235,253,197,
  159,255,249,134,95,139,16,112,78,5,250,222,49,239,173,61,80,18,22,114,1,89,201,21,61,77,156,151,61,34,105,213,249,249,246,4,131,205,70,250,
  91,191,245,91,177,29,58,126,28,199,95,124,49,144,213,141,65,230,43,33,85,36,209,51,231,107,38,166,100,214,214,46,137,17,122,99,188,30,74,43,
  232,157,124,57,188,27,211,134,8,56,209,43,7,4,173,62,185,247,78,159,62,141,51,103,99,226,202,95,248,234,127,203,99,88,99,27,40,34,134,165,
  132,12,182,209,41,106,136,27,121,83,107,42,32,174,244,54,36,176,157,94,238,216,142,2,140,63,250,163,63,194,229,203,151,253,181,142,31,127,1,207,
  63,255,60,194,188,58,39,107,91,118,178,186,212,200,30,218,81,191,150,2,234,158,177,107,234,239,253,221,191,247,225,245,44,217,189,219,37,210,75,33,
  161,180,50,99,233,212,95,111,14,44,13,6,76,26,218,120,122,48,141,184,114,200,105,58,71,222,200,129,75,142,164,97,247,246,251,135,4,14,30,60,
  208,186,206,107,223,253,30,30,60,216,249,255,179,247,238,241,113,21,247,221,255,103,102,118,37,173,108,217,150,228,187,37,136,101,203,134,20,12,54,36,
  92,210,240,180,96,8,134,52,33,16,218,167,188,154,52,52,33,20,18,12,20,26,146,182,207,239,245,250,181,77,74,211,240,0,166,161,33,144,240,60,
  105,95,36,45,9,37,92,98,18,108,2,132,155,185,153,24,130,45,201,150,108,36,203,23,105,37,235,126,217,157,153,231,143,57,231,236,185,238,158,189,
  106,181,154,33,142,206,158,61,123,206,204,156,185,188,231,251,253,206,247,27,40,117,8,116,92,89,92,105,115,168,6,103,175,3,231,10,215,207,187,169,
  115,5,115,226,196,9,252,250,185,231,173,178,72,151,164,197,235,188,208,91,7,165,150,44,65,72,203,206,194,215,145,161,148,161,150,230,187,223,222,141,
  201,201,73,95,233,18,129,45,158,24,224,235,172,180,84,146,37,207,59,247,56,104,77,149,159,39,5,184,228,142,59,249,173,194,159,127,225,133,12,75,
  126,179,220,193,78,28,237,137,70,104,78,59,228,178,81,195,133,113,90,153,233,77,28,56,208,137,253,251,59,3,164,75,46,21,109,54,14,43,103,137,
  129,119,50,145,12,184,42,157,195,74,233,251,11,191,133,226,115,59,119,6,74,150,72,128,129,183,163,110,103,68,178,228,156,51,252,42,223,173,198,77,
  10,103,61,118,118,118,161,163,125,63,124,29,112,16,71,233,157,229,246,201,31,137,144,208,154,25,45,89,10,215,247,25,101,16,16,158,247,86,10,88,
  2,178,80,201,21,196,232,59,76,229,146,0,113,50,87,187,232,56,227,134,134,46,40,140,7,9,132,165,98,172,22,11,153,72,230,146,132,218,75,150,
  73,20,237,168,217,153,130,37,3,100,100,128,155,4,191,237,225,225,119,8,166,201,3,65,209,202,155,93,219,144,105,51,45,33,66,100,26,25,96,201,
  175,46,145,209,102,45,91,100,200,221,102,41,76,139,207,246,149,19,228,220,237,103,9,44,129,35,205,206,53,18,208,219,101,184,14,147,161,13,147,48,
  238,61,102,76,13,71,10,124,191,204,176,148,238,126,50,41,242,203,142,134,37,255,69,93,32,182,20,23,150,76,23,117,161,146,87,39,91,88,88,114,
  111,177,119,79,250,18,128,36,18,132,19,229,236,18,217,88,242,4,12,199,36,37,205,153,105,88,242,212,83,230,184,194,33,96,137,228,54,13,149,2,
  150,82,158,115,124,232,86,166,45,188,239,230,31,146,249,205,19,223,58,153,1,88,202,100,147,69,252,6,163,204,173,93,250,223,40,171,119,111,122,63,
  150,185,216,92,228,82,143,36,56,142,52,9,221,159,145,201,107,103,102,233,210,76,192,82,70,203,253,52,35,190,204,162,189,17,153,126,213,226,107,239,
  231,119,94,6,86,56,153,113,88,10,170,78,146,182,42,178,89,131,201,172,170,86,22,112,16,209,176,148,117,137,139,0,75,225,37,76,34,239,218,45,
  224,100,36,157,110,209,77,224,49,254,17,223,129,148,164,237,1,217,218,239,20,92,178,68,194,248,8,32,222,63,142,178,103,177,235,222,22,106,75,186,
  101,216,37,128,37,11,148,124,99,91,72,132,138,66,111,47,123,186,225,47,176,62,72,193,219,102,88,88,242,138,128,188,80,107,5,61,117,215,153,244,
  55,233,149,126,204,152,109,217,104,30,101,205,226,89,233,251,27,9,94,28,184,250,185,247,29,231,40,93,154,41,201,82,218,85,95,240,160,205,109,234,
  120,153,166,105,103,146,61,121,250,189,95,221,186,16,42,184,79,145,178,112,74,73,114,121,101,153,202,78,2,140,222,3,61,223,170,220,68,34,145,220,
  138,168,97,169,44,97,41,212,16,41,32,242,147,46,201,28,196,251,196,57,147,16,247,204,98,196,146,225,130,123,202,199,147,220,250,94,16,17,40,101,
  153,73,155,37,223,58,242,83,77,145,52,59,92,72,166,193,55,157,116,137,24,145,190,57,146,60,137,36,146,144,73,89,34,88,74,69,12,87,134,171,
  246,136,160,25,61,209,33,180,31,38,191,122,40,208,142,200,252,97,41,67,222,73,6,79,219,46,80,148,105,38,195,116,235,5,199,40,96,134,142,51,
  162,173,147,108,212,53,178,32,14,53,124,207,21,221,110,120,182,168,225,100,106,241,42,33,51,220,159,100,198,8,89,216,94,157,205,130,183,88,176,148,
  22,230,130,198,212,76,166,11,233,96,201,247,6,50,244,34,92,195,82,238,176,148,82,203,149,14,150,128,12,70,223,34,95,209,82,182,149,107,211,41,
  145,192,174,165,190,33,130,64,80,97,237,162,34,210,140,249,195,145,224,9,48,193,212,14,35,102,219,253,102,95,121,231,176,53,170,36,155,62,60,209,
  81,101,14,27,78,72,186,241,2,224,202,216,209,236,212,50,41,145,148,9,80,16,48,22,201,183,11,24,155,4,148,65,55,161,128,105,187,204,40,51,
  234,93,130,8,213,232,5,21,32,82,137,182,157,241,67,179,41,59,241,135,7,146,65,66,85,22,176,148,190,17,100,42,191,244,227,228,116,134,91,65,
  198,222,52,147,146,187,24,176,100,155,196,124,6,206,84,176,88,25,162,153,103,233,201,117,54,194,82,86,247,247,15,70,75,178,126,101,126,54,134,36,
  164,164,165,196,176,20,212,1,36,194,92,149,22,57,195,229,207,21,147,49,77,189,104,88,202,83,178,68,1,136,210,194,82,70,96,162,160,70,136,146,
  18,73,150,124,155,113,192,196,73,84,200,20,19,150,164,125,196,37,102,140,31,2,206,185,33,124,161,202,112,61,164,79,31,64,137,190,205,237,187,132,
  169,29,122,4,0,101,5,116,123,46,51,132,197,36,161,150,80,190,223,123,142,56,144,32,9,64,248,239,14,19,66,66,32,129,8,34,176,108,234,165,
  170,75,102,156,8,90,57,57,198,244,68,42,160,165,61,212,146,128,64,20,81,112,243,59,97,251,53,33,129,101,15,11,140,18,225,252,85,21,218,252,
  63,87,88,34,46,19,35,79,188,101,74,32,69,122,104,36,190,207,202,79,138,42,164,176,188,125,147,116,62,157,243,24,64,51,131,160,121,133,12,185,
  176,240,74,6,156,117,77,102,63,44,1,224,146,103,15,75,246,10,203,87,178,228,231,38,46,148,52,165,20,176,228,211,52,164,207,89,219,184,146,22,
  150,72,0,28,250,174,61,188,185,225,224,96,146,165,15,45,163,97,41,139,129,9,16,84,88,247,161,176,5,160,46,1,44,101,4,38,19,154,120,182,
  145,119,165,204,119,152,112,182,122,207,60,74,172,78,40,109,181,47,3,122,54,231,28,136,0,148,123,61,131,186,59,187,72,8,79,121,37,148,255,152,
  84,71,16,136,178,104,65,97,137,16,226,216,49,38,93,203,108,239,4,147,205,14,65,83,90,40,45,88,10,116,100,40,128,36,146,30,187,53,110,4,
  244,99,140,121,234,204,126,7,33,68,176,100,82,0,156,166,84,166,20,20,84,18,24,206,116,45,113,185,204,32,122,144,97,37,45,126,19,104,142,210,
  197,226,72,150,130,27,185,2,116,158,37,52,202,204,210,69,184,228,183,62,74,121,123,104,20,9,105,235,111,5,148,44,249,208,162,249,170,157,147,29,
  201,48,221,86,74,32,221,16,176,196,57,68,136,237,20,158,234,13,1,171,210,35,85,9,178,125,154,105,215,1,33,46,205,192,218,169,107,130,47,200,
  4,75,25,229,80,18,144,92,130,19,238,8,225,165,97,41,61,44,217,199,155,208,37,46,17,44,201,48,192,228,150,48,205,159,63,15,243,230,205,15,
  126,184,204,239,101,59,29,215,73,219,216,156,114,218,104,65,19,241,222,99,225,194,69,142,23,108,223,133,21,97,76,201,154,8,81,234,34,16,16,73,
  64,25,85,54,79,60,252,238,32,37,176,98,106,114,137,248,108,117,20,54,149,166,84,78,229,8,85,207,242,107,208,82,42,213,24,103,73,229,200,81,
  74,112,40,95,19,52,169,164,99,246,118,59,60,50,228,184,67,85,85,21,22,55,46,134,20,82,57,225,100,202,121,156,32,42,160,177,128,80,156,32,
  82,78,232,178,149,67,16,16,176,8,179,156,180,217,231,77,142,36,36,135,175,93,89,208,155,103,132,130,130,33,73,147,144,92,90,1,46,65,93,113,
  47,108,19,156,4,208,215,127,220,241,136,250,134,122,44,20,194,107,235,102,243,187,69,108,254,182,8,161,197,240,50,17,208,184,101,90,150,178,59,105,
  76,57,19,84,193,38,19,124,218,209,192,71,199,70,61,218,153,101,203,150,186,86,207,196,1,70,202,129,39,108,210,65,9,2,154,10,185,226,170,7,
  74,41,24,163,174,166,206,60,126,162,10,81,57,66,166,28,8,74,219,198,5,187,99,65,243,255,7,79,12,58,30,59,127,126,29,230,205,159,239,31,
  112,150,164,156,45,154,139,145,20,228,203,34,189,231,92,239,44,67,157,78,136,4,68,82,134,190,151,132,114,224,232,152,240,92,227,169,121,245,248,196,
  132,231,145,203,150,47,115,182,35,215,202,195,225,20,152,144,252,251,83,129,96,201,114,238,43,108,241,246,236,229,117,249,118,59,222,223,239,108,87,117,
  243,81,83,83,227,0,34,146,114,230,230,116,136,108,223,152,145,206,22,138,40,7,150,146,102,63,171,119,187,28,138,86,50,44,9,41,124,54,3,121,
  46,2,168,185,224,102,37,133,165,80,18,166,148,243,60,149,230,205,155,111,12,210,69,24,119,92,158,126,33,161,6,85,216,29,253,165,156,26,38,147,
  94,231,85,139,22,45,66,36,18,177,28,214,17,71,35,55,65,193,238,1,156,130,194,165,210,203,161,201,49,166,64,194,2,37,163,211,10,42,28,210,
  154,104,52,106,77,76,38,80,73,33,149,100,71,38,17,65,4,156,112,229,13,152,171,115,140,50,144,164,114,167,192,167,57,104,148,58,118,202,0,64,
  85,180,10,141,139,27,212,42,221,242,140,110,60,195,244,80,75,109,30,123,77,175,200,89,150,147,82,170,254,129,66,50,213,200,33,12,41,158,204,222,
  135,15,12,64,48,213,191,130,72,48,80,68,162,17,95,135,158,18,192,123,239,189,235,184,69,67,67,131,1,195,176,13,222,118,143,231,62,30,122,243,
  144,50,201,34,180,119,179,125,75,41,193,37,7,79,112,112,33,12,201,166,225,144,239,184,247,201,75,151,46,245,74,75,137,179,236,212,244,102,78,82,
  239,156,48,162,98,146,185,68,14,148,165,130,90,83,80,72,160,240,30,240,141,247,40,133,240,241,196,236,138,211,101,28,239,221,251,190,115,225,86,55,
  31,203,151,45,247,150,55,195,59,47,248,182,134,18,24,54,114,33,144,224,9,72,46,67,231,41,229,225,219,21,251,204,238,44,85,154,208,208,231,185,
  227,242,101,203,157,48,96,120,205,135,171,158,131,250,84,225,164,74,217,188,25,105,43,187,171,95,57,36,139,230,57,130,193,161,33,231,252,54,127,62,
  150,44,94,2,211,76,128,184,156,189,250,151,61,243,194,147,16,130,104,52,26,208,151,100,78,192,84,73,176,148,148,73,128,195,33,141,11,186,143,37,
  196,241,201,95,49,97,41,20,48,185,211,177,99,199,112,236,216,49,255,229,114,129,38,157,180,158,128,109,139,134,234,234,42,207,111,15,29,58,100,121,
  250,246,217,112,237,104,220,233,38,205,98,250,89,50,129,67,216,108,121,76,221,172,105,188,78,41,129,228,10,72,36,147,202,43,182,144,134,88,48,2,
  73,157,146,191,209,225,81,188,255,222,251,169,129,202,199,227,181,229,225,58,15,181,148,9,155,146,193,240,186,158,131,196,193,175,173,216,213,36,84,34,
  66,163,161,165,151,157,7,14,96,98,34,216,211,183,123,224,42,240,148,153,127,95,176,95,111,218,147,73,105,237,78,149,102,208,79,159,39,191,251,238,
  187,190,37,77,45,24,212,251,114,171,82,5,56,4,247,169,83,234,220,41,87,112,3,239,128,178,91,255,47,100,168,58,62,122,228,40,142,244,30,245,
  41,179,79,93,24,250,169,217,168,134,51,79,113,206,211,76,96,50,24,76,3,116,64,246,243,148,121,39,241,61,191,253,173,187,227,120,30,101,133,64,
  177,207,113,28,169,112,70,37,148,44,121,47,10,46,187,99,162,230,206,69,247,177,163,71,113,184,167,7,190,234,11,159,234,200,38,75,132,144,20,16,
  104,53,28,178,125,177,14,83,17,233,221,148,86,108,88,10,5,76,233,246,171,21,3,150,224,25,2,101,224,55,233,126,101,229,203,99,195,65,10,152,
  191,220,64,193,180,243,177,67,19,21,52,229,10,65,0,10,162,141,78,159,116,246,137,36,73,130,38,93,182,87,112,171,19,3,108,89,100,190,30,47,
  140,60,241,28,43,42,204,243,5,1,151,220,187,218,8,181,181,37,123,111,209,51,9,75,65,77,158,144,84,12,37,78,224,27,59,44,168,148,142,106,
  32,54,233,103,150,3,97,177,97,201,103,142,203,218,208,63,212,187,158,229,176,148,254,189,133,181,145,115,253,34,155,10,182,105,51,41,161,14,192,114,
  47,64,178,178,119,45,26,44,5,21,80,134,44,108,6,88,202,241,45,219,37,125,4,185,171,136,43,221,192,91,58,22,244,106,71,181,3,142,74,172,
  134,115,173,39,11,3,1,133,132,37,18,232,226,151,100,246,41,227,129,37,146,85,75,47,186,7,111,23,228,120,142,69,186,241,57,101,21,157,148,201,
  84,123,20,210,217,200,125,89,201,102,5,90,8,117,84,161,97,201,109,72,46,149,107,2,79,161,72,56,88,242,123,94,190,194,165,98,193,18,9,240,
  159,69,34,4,132,17,80,48,35,80,110,230,2,16,199,82,39,253,245,148,145,112,35,128,44,129,222,73,230,236,122,48,109,8,164,194,197,128,157,57,
  88,10,246,131,39,51,62,130,248,218,45,147,240,85,109,212,41,5,5,141,164,135,37,115,65,53,243,176,228,87,246,208,251,76,51,60,84,230,21,129,
  69,36,133,150,44,249,37,158,106,235,66,8,8,206,13,41,56,87,187,213,133,233,168,122,230,96,41,20,48,249,137,107,75,35,89,178,15,252,193,208,
  228,251,75,31,88,242,184,113,244,237,236,165,131,165,224,201,211,233,51,199,119,119,156,185,243,77,186,194,88,16,4,187,56,14,136,217,86,156,247,151,
  91,29,184,223,115,66,36,61,208,228,235,20,130,120,125,8,249,171,29,139,104,183,84,72,231,141,246,141,72,68,217,27,177,48,241,128,252,187,15,104,
  144,239,37,194,12,255,75,182,115,148,56,39,213,98,75,150,252,182,115,101,59,159,87,154,100,201,17,151,89,102,235,12,40,228,36,96,86,178,204,8,
  75,144,0,88,186,24,94,229,7,75,30,209,88,65,58,187,204,239,22,134,132,142,228,136,241,149,10,75,158,93,213,166,29,90,82,56,213,187,51,12,
  75,64,88,27,38,95,233,102,177,96,201,255,249,78,155,108,146,234,232,82,134,147,54,144,34,230,175,136,171,115,25,228,136,208,138,131,103,47,1,201,
  60,161,228,40,102,145,165,168,3,215,228,73,8,20,52,1,106,151,137,27,138,131,110,83,214,78,41,179,235,112,246,215,31,65,68,169,200,141,206,16,
  58,116,45,241,46,20,24,34,190,174,5,36,0,102,186,21,40,137,26,78,166,237,240,36,205,59,79,27,2,73,202,2,233,100,103,64,178,68,82,176,
  100,25,230,231,12,75,153,221,50,16,215,22,123,226,130,37,74,168,7,150,72,0,44,49,198,148,239,59,25,96,202,81,50,88,50,219,187,12,108,198,
  210,65,167,225,30,148,143,7,111,70,152,97,239,165,97,41,29,48,73,63,159,16,101,0,75,202,247,83,186,130,8,225,239,184,178,4,176,68,72,136,
  208,155,36,92,152,140,76,182,75,101,11,75,190,18,182,160,58,9,1,75,101,42,89,74,7,14,68,245,168,240,121,8,80,59,230,170,142,43,21,44,
  121,84,39,196,185,81,129,48,229,175,138,152,66,129,32,120,176,223,135,26,222,213,51,36,74,137,75,178,68,74,10,75,129,147,80,38,73,114,58,88,
  42,72,15,152,57,53,92,86,49,230,50,125,69,114,105,217,196,215,102,201,175,193,186,193,136,49,6,22,97,233,99,247,150,0,150,210,185,218,46,214,
  12,150,9,150,72,33,90,78,133,217,44,49,176,244,37,46,19,88,114,173,45,61,216,7,33,132,87,36,92,74,201,146,143,80,37,220,234,60,253,41,
  231,246,215,242,131,37,207,246,231,156,221,156,100,118,117,89,110,176,20,20,205,68,132,137,103,104,122,113,39,33,219,69,89,73,150,72,218,73,147,184,
  69,74,174,176,130,150,63,28,195,207,24,137,16,48,202,210,186,4,32,32,74,45,71,24,136,93,61,55,99,146,37,248,23,48,155,6,83,1,6,222,
  142,247,227,120,127,185,170,225,210,250,107,119,193,179,2,36,63,155,37,191,42,79,87,219,17,22,1,137,16,151,152,48,59,88,98,132,169,127,198,127,
  70,206,212,69,20,160,17,26,48,139,201,224,197,72,142,239,46,147,116,41,8,150,8,81,182,136,26,150,210,8,71,88,249,195,18,16,164,146,19,64,
  82,36,188,119,42,49,44,229,30,91,44,156,116,169,156,37,75,30,143,215,89,150,157,248,122,125,38,165,123,127,185,214,129,75,45,39,13,245,140,138,
  79,231,118,162,104,251,153,12,136,54,155,163,116,169,36,113,3,125,138,46,252,118,250,248,216,159,145,20,38,169,213,107,68,57,228,180,59,118,149,161,
  236,198,136,165,154,160,132,21,92,157,25,22,150,252,61,83,167,182,204,201,204,34,38,39,44,145,2,89,235,205,16,44,89,11,37,105,11,64,110,168,
  185,60,59,151,195,76,2,153,226,242,33,229,111,139,128,164,183,93,37,36,35,44,57,164,7,166,134,151,75,8,34,124,54,168,72,223,122,102,118,223,
  106,246,118,47,205,150,79,172,124,11,35,18,129,82,97,6,135,155,42,181,100,137,68,204,221,174,90,13,151,238,231,12,204,114,212,92,174,176,20,8,
  76,150,227,69,159,9,112,205,218,22,180,172,105,41,217,140,34,132,116,121,255,78,13,140,19,19,227,120,231,157,221,142,235,55,157,181,17,177,88,12,
  150,55,235,0,39,107,229,158,130,203,173,42,126,255,254,253,232,232,104,183,174,175,111,168,199,199,63,126,1,82,190,11,137,211,35,237,44,41,187,148,
  210,218,245,103,186,48,112,123,195,254,201,79,126,226,104,126,231,158,243,81,196,106,107,29,14,73,253,202,93,176,178,23,168,237,139,164,114,47,193,193,
  65,37,85,19,34,145,182,193,61,245,222,121,50,137,3,237,157,104,235,104,183,38,44,202,40,46,189,252,19,96,52,162,28,78,130,41,35,113,163,14,
  132,20,170,220,84,237,180,243,132,5,50,62,115,33,82,164,102,156,99,148,129,148,194,29,186,143,163,65,105,123,215,166,45,207,143,126,244,35,71,181,
  175,91,183,30,173,235,90,189,239,28,133,125,231,114,134,27,137,76,166,252,113,113,194,189,54,77,25,156,197,170,254,35,188,78,65,165,4,53,0,187,
  243,64,39,14,30,58,228,240,166,255,169,43,62,229,172,207,160,62,5,175,106,46,168,221,36,147,73,199,78,39,201,4,24,101,170,15,112,14,187,231,
  12,187,100,45,202,162,193,186,16,115,119,149,225,200,151,75,110,153,146,120,218,149,203,17,174,148,18,47,191,252,50,142,28,61,106,107,87,235,176,186,
  101,117,250,177,36,147,119,115,169,84,232,148,170,62,73,163,52,163,193,124,80,250,241,143,127,92,217,176,228,164,166,84,236,209,50,132,165,96,9,19,
  2,124,176,16,224,192,129,78,116,30,232,242,118,16,91,199,45,180,201,173,148,158,17,2,0,80,229,227,184,242,237,183,118,99,106,106,202,225,168,209,
  45,254,43,111,155,37,231,253,36,252,203,238,246,244,61,56,56,136,157,59,159,243,151,44,33,59,135,141,114,6,235,192,130,9,25,16,49,78,154,64,
  41,172,65,117,215,107,175,99,98,114,34,160,220,193,18,198,156,203,46,11,63,112,153,161,122,28,237,93,202,140,57,219,241,236,206,192,242,101,165,66,
  40,11,53,156,12,157,141,246,246,118,180,181,181,251,246,147,220,33,169,76,36,75,97,175,144,254,30,246,165,79,29,152,159,184,224,30,187,36,63,239,
  202,79,60,254,68,96,155,42,191,221,112,222,11,4,68,106,43,58,224,242,244,109,171,63,215,56,218,222,222,142,247,127,247,59,111,221,145,112,90,10,
  66,136,203,110,80,75,150,50,38,34,149,175,65,86,222,176,228,0,38,187,113,183,176,59,31,179,57,218,178,55,55,179,81,216,207,113,206,139,0,35,
  110,69,148,185,131,70,250,238,136,37,105,26,248,172,130,37,179,12,210,175,236,65,102,240,33,119,202,149,57,44,73,4,249,207,33,206,109,208,194,144,
  136,164,133,162,50,135,37,195,160,93,80,14,74,88,42,100,133,148,150,115,83,98,109,112,144,161,203,151,85,248,202,178,128,37,32,39,175,73,50,173,
  187,214,138,134,37,19,118,56,231,182,69,107,170,59,164,38,110,27,152,80,145,179,180,195,61,222,151,43,44,1,202,5,130,25,126,138,11,111,8,160,
  236,236,58,52,44,21,85,178,100,1,68,121,195,146,5,76,150,189,146,57,9,5,12,114,38,52,49,202,140,184,100,41,241,171,16,34,96,11,124,1,
  144,137,248,45,182,73,224,230,185,176,13,188,172,97,41,112,10,33,105,252,241,144,172,58,124,185,194,146,255,75,247,150,81,64,128,10,10,46,57,36,
  151,32,44,119,219,165,146,195,146,91,26,203,1,193,56,144,76,133,150,144,144,144,76,34,66,34,185,117,154,89,5,75,25,94,123,88,88,34,121,154,
  214,206,34,88,114,75,136,146,156,91,221,195,15,150,0,128,145,72,238,147,38,41,84,103,42,46,44,57,178,76,21,30,42,129,0,81,110,6,68,54,
  221,40,36,44,65,195,82,222,125,37,219,126,194,96,83,241,22,31,150,0,32,146,20,9,7,36,249,171,3,212,0,79,100,176,77,0,23,162,136,68,
  98,195,6,153,105,61,153,31,52,148,19,44,133,5,135,146,140,113,165,132,165,44,228,13,2,66,217,2,81,1,38,153,191,49,112,153,193,146,223,164,
  44,164,33,150,54,37,186,74,175,146,147,29,17,153,165,176,148,173,148,73,230,218,216,43,8,150,236,167,189,170,53,25,216,62,82,1,173,43,23,150,
  204,111,204,216,157,28,92,185,231,34,225,84,191,36,231,133,183,134,165,66,140,186,233,174,176,36,239,212,80,171,202,240,253,36,159,60,81,42,152,97,
  24,24,12,75,142,85,177,84,161,56,28,147,150,144,144,69,182,2,119,59,87,43,139,84,108,88,202,103,30,200,98,197,93,142,176,100,190,116,18,226,
  121,4,202,65,145,189,93,18,82,192,119,80,168,118,39,213,78,33,41,36,56,23,72,242,36,132,224,16,194,84,169,72,95,192,207,102,176,35,97,203,
  90,150,176,148,25,248,210,193,117,246,61,165,114,96,41,219,39,17,223,173,135,149,7,75,182,25,22,140,48,68,104,4,160,198,134,160,52,197,37,46,
  103,175,36,146,114,219,65,92,97,135,156,187,10,53,44,229,36,89,114,69,126,74,11,75,134,235,11,98,108,236,98,212,25,88,188,88,176,4,0,17,
  46,121,86,147,131,132,97,91,33,213,36,37,164,244,39,188,34,140,194,142,149,81,22,175,166,40,59,195,74,4,75,22,56,228,226,95,102,54,195,146,
  189,12,210,41,93,36,62,131,161,57,148,165,138,93,160,178,23,88,13,39,172,48,201,210,136,184,29,236,177,62,167,118,27,102,85,92,198,176,228,43,
  5,129,191,155,86,153,229,194,64,195,82,250,58,246,173,235,74,128,37,215,73,51,208,57,136,59,56,187,225,245,220,5,75,148,49,213,222,152,63,140,
  232,64,186,249,193,146,153,71,6,6,78,120,248,161,131,72,107,200,163,148,90,27,32,138,5,75,128,123,151,92,200,156,74,41,145,228,73,195,129,121,
  41,3,115,134,88,249,144,10,131,165,208,208,68,178,94,20,150,61,44,57,38,71,25,8,5,68,154,134,27,229,11,75,246,231,74,33,213,138,42,131,
  68,151,49,102,132,248,32,254,229,206,240,121,182,194,146,31,43,251,110,226,200,98,97,160,97,201,255,18,223,54,67,178,8,230,55,203,96,9,80,206,
  46,237,65,112,237,110,2,34,145,104,106,39,50,231,150,154,51,208,207,146,134,165,252,36,75,182,196,83,203,199,244,137,2,18,194,35,229,11,10,199,
  83,40,88,114,2,83,200,202,93,182,108,25,150,45,95,134,146,37,233,52,142,149,54,221,97,146,115,236,126,251,173,84,7,151,18,107,214,180,128,177,
  136,99,48,152,13,126,151,194,52,252,84,7,144,232,237,237,69,123,219,62,107,86,153,63,111,30,206,56,115,131,163,1,153,245,66,242,106,34,51,95,
  110,119,135,221,185,227,217,20,72,72,137,150,53,171,17,173,170,2,161,64,148,84,129,70,136,191,58,111,38,203,47,128,36,79,166,34,111,103,200,13,
  33,20,81,18,129,140,2,84,170,253,143,189,189,135,209,214,182,207,209,55,206,254,200,217,14,41,139,253,175,156,153,151,86,152,225,217,92,28,24,127,
  127,245,203,103,28,239,124,213,170,149,88,177,114,101,106,208,180,149,61,91,88,42,106,117,228,115,149,44,102,159,74,29,31,57,114,4,251,247,119,56,
  198,209,115,206,59,199,86,183,246,49,101,166,59,82,126,176,100,250,99,226,137,36,56,23,120,239,253,247,112,236,200,81,43,54,103,83,83,19,150,47,
  95,174,170,66,24,133,103,200,96,75,152,39,44,165,249,249,203,47,189,84,209,176,100,1,78,72,19,104,203,203,123,80,180,52,73,33,109,70,217,133,
  132,165,20,48,101,81,185,203,150,47,195,25,103,108,152,113,112,144,82,98,104,104,72,213,157,13,170,214,172,89,131,133,11,23,122,28,86,206,246,228,
  118,188,38,133,176,202,46,1,204,155,63,15,103,156,113,134,167,204,149,4,139,230,241,142,29,207,58,202,190,182,117,45,234,235,235,65,24,65,148,68,
  193,162,12,145,72,36,184,143,148,184,74,56,231,224,9,110,108,113,230,30,27,64,127,96,34,168,138,84,129,69,153,85,238,55,222,144,46,47,232,192,
  71,76,96,170,208,246,110,150,253,151,191,124,198,241,206,87,173,90,133,179,63,242,145,138,43,247,76,140,37,111,190,241,134,103,28,61,247,220,115,43,
  110,44,49,203,155,156,78,66,64,96,58,57,141,158,99,61,202,7,154,209,174,154,155,155,178,111,87,133,10,93,88,32,96,154,109,146,37,9,9,46,
  69,230,197,14,5,64,82,14,87,221,247,99,140,129,11,14,2,106,205,143,133,132,37,149,5,41,103,77,99,15,106,196,238,216,198,149,56,136,122,6,
  46,43,60,129,247,154,74,26,224,252,60,181,251,53,121,34,137,21,251,80,74,9,14,14,135,59,49,72,112,201,213,249,82,121,169,55,188,120,67,216,
  236,171,104,184,119,194,24,83,241,167,92,158,149,229,28,105,239,25,223,185,171,157,107,88,42,204,88,34,125,174,169,148,177,196,146,18,24,26,136,170,
  72,149,229,147,42,168,93,133,171,208,210,47,194,42,10,150,76,239,245,33,28,219,166,180,38,36,152,123,4,138,2,75,41,9,83,22,69,63,112,224,
  0,142,30,61,86,236,215,30,234,43,234,51,249,188,249,230,91,16,57,236,150,74,255,188,153,14,144,224,204,135,4,48,60,60,236,248,122,112,112,16,
  191,252,229,175,92,11,157,66,5,25,150,5,43,72,78,78,200,50,252,232,141,215,223,0,231,2,12,20,136,18,229,183,136,150,246,61,103,44,171,48,
  0,74,114,136,68,122,71,48,52,66,85,56,8,107,72,81,105,196,245,206,1,224,137,39,158,116,188,109,233,214,153,200,82,244,82,89,164,155,7,111,
  255,222,183,119,31,122,186,15,91,5,39,37,105,202,225,84,103,178,72,35,67,78,54,75,33,230,160,209,241,81,207,233,159,62,250,83,199,16,146,118,
  130,74,243,192,25,177,89,10,1,1,102,234,235,235,115,92,241,187,223,253,14,93,157,93,240,12,161,178,88,157,103,142,251,89,146,225,239,69,9,45,
  192,228,154,95,137,34,217,62,102,116,116,12,163,163,99,197,68,228,208,151,214,212,84,35,26,141,34,145,72,56,192,97,114,114,106,70,242,83,52,96,
  242,201,131,112,185,244,159,158,78,224,88,26,144,157,45,6,222,185,60,47,62,48,128,201,137,201,20,112,48,234,112,34,55,147,6,222,246,196,193,33,
  146,220,233,28,214,13,75,148,250,134,170,240,187,99,52,26,69,239,225,222,224,43,102,145,129,183,223,251,72,119,231,225,145,17,12,13,13,231,4,222,
  115,213,192,59,40,205,175,155,239,57,215,211,211,147,55,220,148,131,129,119,166,47,146,174,113,116,120,104,24,241,254,120,134,159,107,3,239,130,192,82,
  182,47,85,134,0,88,74,210,140,175,249,215,17,45,72,155,158,33,56,153,156,156,194,130,5,11,172,207,11,22,44,80,1,30,43,28,150,204,201,210,
  158,170,171,171,231,12,44,217,203,26,141,70,61,210,5,193,133,21,35,74,66,5,228,132,175,50,171,116,176,4,0,76,210,12,157,145,164,237,145,181,
  181,243,28,159,171,162,85,115,6,150,230,207,175,115,125,158,175,97,169,64,157,78,74,233,24,79,2,199,146,10,131,37,191,178,186,199,85,13,75,37,
  132,37,0,233,182,40,9,211,57,118,154,126,66,65,161,254,71,29,1,156,97,139,123,107,238,138,140,144,72,86,97,130,100,24,96,42,87,88,50,211,
  226,197,139,29,199,201,36,175,120,88,2,0,22,137,88,176,24,141,70,81,87,87,55,103,36,75,13,245,13,214,241,202,149,43,49,97,147,46,217,161,
  105,154,39,192,19,28,76,48,112,41,192,185,0,183,135,75,47,33,44,5,62,135,42,169,82,132,70,64,163,12,140,4,119,201,68,34,225,105,239,115,
  69,178,84,95,95,111,29,55,53,53,97,124,108,92,195,82,33,96,9,18,163,163,163,104,90,213,100,157,91,180,104,209,156,128,37,0,168,141,213,90,
  144,164,230,143,164,134,165,25,130,37,175,205,152,143,102,37,233,239,124,64,186,197,64,20,54,73,147,116,242,24,1,192,0,193,132,11,170,50,151,138,
  92,188,249,18,153,103,159,155,49,56,145,0,170,170,162,24,26,58,1,206,57,26,26,26,49,61,157,168,120,88,178,160,137,81,28,239,59,142,21,43,
  86,98,122,106,122,78,192,18,160,0,67,74,129,161,225,33,44,89,188,20,83,83,83,193,207,165,202,81,29,34,170,3,49,70,193,165,0,37,20,68,
  160,224,198,154,153,96,41,193,141,246,41,96,117,86,74,105,104,3,211,218,218,90,244,30,57,140,37,139,151,64,8,5,129,115,65,13,199,24,67,36,
  202,48,56,56,136,37,139,151,98,108,108,76,195,82,1,96,201,222,174,6,79,12,98,225,194,133,72,76,39,148,19,192,10,135,37,11,16,27,234,113,
  236,216,49,44,90,184,8,227,227,227,26,150,74,8,75,82,74,213,214,136,17,171,86,202,80,63,102,17,102,217,213,5,229,89,8,1,33,56,24,12,
  19,7,230,227,26,67,0,73,145,12,93,170,64,96,154,13,176,84,78,249,41,53,44,21,53,31,101,110,179,148,203,115,25,101,144,84,130,17,170,164,
  76,73,105,1,75,161,118,0,133,145,44,9,112,117,29,55,226,33,49,82,184,103,86,176,205,82,78,53,162,97,41,20,44,21,10,110,102,35,44,133,
  187,175,134,165,98,74,150,132,20,16,66,216,128,41,196,143,153,10,117,147,110,197,43,12,159,119,140,177,192,49,94,112,97,68,92,16,161,222,99,68,
  195,146,134,165,74,135,37,192,8,208,40,96,136,107,101,246,187,232,10,0,75,169,190,206,0,86,224,103,106,88,210,176,164,97,73,195,210,44,131,37,
  0,74,218,79,137,130,151,176,63,22,42,84,77,166,48,227,20,105,164,247,134,81,18,53,254,51,1,202,132,39,191,199,70,52,44,105,88,170,116,88,
  114,174,40,184,138,131,104,0,139,21,86,69,2,146,164,70,14,1,14,201,213,142,59,74,104,193,96,137,22,130,148,52,44,105,88,210,176,164,97,169,
  2,96,41,215,31,75,169,36,83,128,41,105,242,129,37,74,131,23,198,18,22,80,57,198,103,70,65,37,133,16,18,130,10,64,56,123,76,68,195,146,
  134,165,185,2,75,214,51,69,74,4,75,169,225,106,95,24,162,97,155,215,51,2,181,234,33,148,4,58,156,148,37,40,155,134,37,13,75,26,150,52,
  44,85,50,44,9,240,236,251,9,135,101,192,93,200,254,34,9,64,24,81,126,253,140,123,115,33,32,165,76,1,147,134,37,13,75,115,2,150,96,192,
  146,1,77,66,8,215,142,10,231,4,67,50,10,125,53,44,105,88,210,176,164,97,73,195,82,62,111,132,130,130,19,91,20,134,16,253,68,26,133,72,
  5,73,46,0,44,5,92,200,40,133,128,80,242,40,13,75,26,150,230,10,44,5,44,111,210,63,128,104,88,210,176,164,97,73,195,146,134,165,98,192,
  18,96,132,226,161,36,135,126,82,184,254,146,169,117,83,80,80,13,75,26,150,230,52,44,133,72,110,41,147,134,37,13,75,26,150,52,44,105,88,66,
  65,109,150,40,104,217,194,82,42,143,26,150,52,44,105,88,10,134,37,10,135,132,73,195,146,134,37,13,75,26,150,52,44,161,224,6,222,36,32,40,
  100,38,88,82,190,156,68,209,97,169,52,192,164,97,73,195,210,44,133,37,74,9,104,166,152,116,26,150,52,44,105,88,210,176,164,97,169,32,187,225,
  220,155,107,202,69,178,100,205,9,172,64,219,156,53,44,105,88,170,36,88,34,32,42,144,35,209,176,164,97,73,195,146,134,37,13,75,197,134,37,0,
  14,123,210,114,131,37,72,128,210,40,69,148,69,81,112,112,210,176,164,97,105,150,194,146,9,76,166,143,14,41,125,6,49,13,75,26,150,52,44,105,
  88,210,176,84,56,88,146,185,193,146,175,42,175,8,176,4,152,42,57,10,208,40,45,28,52,105,88,210,176,52,139,97,201,239,185,150,219,126,9,72,
  41,52,44,105,88,210,176,164,97,73,195,82,1,251,157,36,50,107,88,50,6,103,48,123,16,221,34,193,82,10,152,204,15,81,90,136,183,174,97,73,
  195,82,101,192,146,225,204,210,62,17,201,164,0,145,4,178,72,45,86,195,146,134,37,13,75,26,150,230,26,44,153,139,82,146,214,172,218,31,150,28,
  114,158,34,194,146,7,152,52,44,105,88,210,176,164,18,167,41,41,146,16,220,112,146,38,85,176,92,130,180,14,45,53,44,21,1,150,242,238,11,26,
  150,52,44,105,88,42,87,88,50,79,11,136,128,216,111,254,63,34,25,183,48,23,14,150,0,91,104,148,68,34,161,97,73,195,146,134,37,243,185,194,
  25,193,90,112,174,194,168,144,34,53,87,13,75,25,7,109,13,75,26,150,52,44,85,46,44,1,0,163,12,16,0,167,220,50,129,8,252,145,13,172,
  184,20,42,148,73,17,97,9,48,36,76,26,150,52,44,105,88,242,62,215,10,157,98,110,121,37,197,161,37,13,75,26,150,52,44,105,88,154,235,176,
  164,6,89,0,12,96,96,198,120,27,4,75,202,71,30,101,20,72,22,180,117,167,189,140,10,46,242,125,235,26,150,52,44,85,28,44,217,159,35,68,
  241,140,188,53,44,105,88,210,176,164,97,73,195,82,16,161,4,125,145,242,247,194,24,11,148,46,21,18,150,0,128,114,193,53,44,105,88,210,176,148,
  233,57,162,240,13,87,195,146,134,37,13,75,26,150,52,44,249,145,137,84,16,228,179,86,101,80,128,68,37,115,152,77,20,27,150,0,9,242,232,163,
  143,150,44,156,156,78,58,205,214,68,65,65,35,20,44,202,80,21,169,210,21,162,147,78,58,233,84,132,36,132,64,66,36,128,4,144,224,9,72,72,
  80,73,45,56,98,148,129,18,10,2,2,14,174,220,33,17,128,69,25,40,212,121,117,138,4,218,156,10,33,32,146,2,130,10,5,101,66,121,25,167,
  80,118,170,36,154,242,195,103,79,145,145,145,17,180,183,119,20,93,138,81,209,84,54,211,82,165,89,88,222,178,46,187,79,249,40,40,162,145,40,16,
  133,111,71,42,219,178,150,210,193,102,145,86,183,165,207,179,44,187,198,90,56,169,82,233,106,83,206,88,254,10,160,249,144,229,215,83,252,110,39,203,
  41,79,121,119,33,231,55,2,2,72,2,9,145,132,148,194,130,38,74,13,159,145,134,59,1,70,152,57,72,103,13,102,66,10,48,97,222,200,15,178,
  82,190,161,54,157,117,22,34,35,35,163,120,243,205,183,138,62,0,107,53,92,17,243,161,213,112,133,123,110,192,115,40,165,96,140,21,246,153,90,13,
  7,173,134,43,53,48,149,171,26,46,124,101,104,53,28,42,212,102,41,253,183,66,10,128,19,21,16,29,254,155,112,138,161,134,51,255,191,165,165,37,
  75,38,211,176,164,97,105,14,194,18,0,48,70,11,251,76,13,75,26,150,52,44,105,88,210,176,148,197,19,73,90,41,82,78,176,36,194,193,146,217,
  15,35,26,150,52,44,105,88,74,255,28,74,41,242,117,192,164,97,73,195,146,134,37,13,75,26,150,114,239,47,20,246,96,232,50,127,88,2,210,0,
  152,55,76,11,79,242,144,18,38,13,75,26,150,230,42,44,33,127,85,156,134,37,13,75,26,150,52,44,105,88,42,96,127,145,118,201,79,142,176,132,
  240,176,4,161,252,85,70,66,188,117,13,75,26,150,230,36,44,129,66,195,146,134,37,13,75,26,150,52,44,149,19,44,33,101,191,84,10,88,50,157,
  23,11,136,12,192,164,97,73,195,210,92,133,37,0,81,22,213,176,164,97,73,195,146,134,37,13,75,101,4,75,57,182,238,220,36,75,0,132,249,62,
  69,58,19,42,13,75,26,150,230,48,44,229,107,228,173,97,73,195,146,134,37,13,75,26,150,102,55,44,185,141,194,233,108,133,165,108,124,225,104,88,
  170,44,88,34,132,128,16,128,48,181,107,194,108,11,238,191,185,194,18,24,64,41,43,76,89,53,44,105,88,146,233,7,97,13,75,26,150,52,44,149,
  17,44,73,128,195,63,2,74,100,182,193,146,25,49,158,129,41,39,83,70,178,10,200,53,44,85,44,44,25,239,93,80,1,74,25,168,160,202,121,153,
  177,121,66,114,115,71,155,113,75,145,219,115,152,134,37,13,75,197,128,37,1,43,46,161,5,245,68,253,51,107,91,74,233,25,172,35,136,164,54,105,
  106,88,210,176,164,97,169,104,176,196,37,183,22,52,220,103,101,19,41,87,88,250,251,127,252,255,241,195,135,126,136,131,93,7,157,176,100,68,50,54,
  7,157,111,222,249,77,112,206,113,96,255,1,172,89,187,6,59,119,236,196,142,157,59,0,14,252,253,55,255,30,63,124,232,135,232,178,221,3,0,254,
  225,155,127,143,95,63,247,107,60,183,243,215,51,14,75,95,250,210,95,160,179,235,32,158,219,249,156,231,60,0,60,244,224,15,212,231,235,190,8,0,
  248,80,203,106,28,236,236,114,124,7,0,45,45,171,241,23,198,53,7,59,187,240,161,150,213,248,225,131,63,64,103,103,151,245,221,223,126,227,239,28,
  207,104,105,89,141,47,94,247,69,252,192,184,174,212,176,244,173,59,191,137,174,206,46,172,110,89,13,0,214,241,223,220,241,55,206,235,254,249,91,214,
  247,107,215,172,197,175,159,255,53,118,62,187,211,146,143,94,124,217,197,248,195,139,254,16,16,64,215,193,78,172,62,185,5,95,251,250,29,32,20,216,
  124,225,133,248,80,203,106,124,255,123,15,58,202,182,249,226,205,184,232,226,139,240,141,175,125,163,40,101,221,114,217,165,216,114,217,22,0,192,254,142,
  253,88,219,186,22,91,191,178,85,125,119,249,22,180,182,182,98,109,235,90,199,247,102,218,250,149,173,216,246,221,109,142,243,246,227,237,191,216,14,0,
  88,219,186,22,173,173,173,0,128,142,142,14,180,182,182,98,251,211,219,241,139,167,127,1,0,184,239,254,251,172,243,246,107,0,120,174,115,222,227,23,
  120,250,169,95,148,4,150,110,249,171,91,0,0,173,235,140,60,182,119,160,117,93,43,238,249,223,247,88,199,183,252,213,45,214,177,121,13,0,220,115,
  215,221,234,30,183,221,234,123,15,243,248,110,227,186,114,128,37,1,1,10,10,33,4,8,37,224,146,171,201,150,120,23,123,102,74,34,9,198,156,11,
  196,48,211,201,186,245,235,113,219,95,223,134,246,182,118,172,91,191,14,0,112,253,151,190,236,184,252,182,175,221,14,0,214,247,230,181,119,125,251,59,
  104,111,107,83,247,248,218,237,184,254,139,215,57,158,100,158,191,235,219,223,65,91,91,91,217,193,210,237,119,124,13,237,109,109,120,226,241,159,59,126,
  241,215,119,124,13,0,240,47,119,254,51,30,250,63,63,68,219,190,54,172,63,101,61,0,56,142,159,120,252,231,0,128,245,167,156,226,251,61,0,124,
  233,11,127,81,145,176,116,197,149,159,193,250,83,78,193,63,127,235,159,2,127,252,240,127,252,8,251,246,238,195,41,167,158,2,0,142,227,199,31,251,
  111,0,192,41,167,158,138,59,191,249,45,199,19,175,184,242,74,92,113,229,103,240,133,63,251,28,254,207,127,252,59,246,237,221,139,83,78,61,213,184,
  71,234,56,117,143,83,112,231,63,126,171,104,176,36,32,0,65,12,169,146,176,214,48,1,10,58,27,48,149,155,100,201,119,233,159,130,165,53,107,215,
  160,161,161,1,47,60,255,2,132,16,24,232,31,192,225,195,135,17,141,70,241,137,139,47,193,51,207,254,42,79,207,57,40,171,144,39,111,189,245,54,
  234,27,234,241,161,150,213,120,235,173,183,29,223,109,220,180,17,0,44,232,26,28,24,68,103,151,130,166,133,139,22,225,196,137,19,101,43,89,234,236,
  236,68,103,103,103,42,223,157,157,184,232,226,139,208,101,0,220,162,69,139,176,115,199,78,235,251,174,174,46,68,162,17,92,114,233,37,248,213,51,191,
  194,165,151,93,138,129,248,0,158,255,245,243,160,156,34,62,16,199,193,253,135,176,101,243,165,232,232,220,15,106,184,205,223,188,249,34,116,118,118,162,
  235,64,23,54,95,188,25,171,215,172,46,90,89,183,24,121,50,193,102,32,62,128,142,142,14,108,185,124,11,246,183,239,199,254,246,253,214,57,11,96,
  140,107,237,169,163,163,3,29,29,29,142,123,88,64,96,3,31,0,136,199,227,216,223,190,31,0,112,217,229,151,89,48,100,62,207,125,77,71,135,130,
  138,134,134,6,215,61,58,32,1,92,246,201,203,240,139,188,160,41,188,100,41,30,143,163,227,169,142,212,113,123,7,214,173,91,135,198,198,70,196,227,
  113,0,192,107,175,190,138,142,246,118,235,154,198,198,70,92,254,71,151,227,181,87,94,179,206,181,63,101,124,223,31,71,123,123,234,184,156,96,9,128,
  138,95,101,73,65,195,245,57,158,228,74,194,74,125,192,137,164,159,78,94,121,229,21,180,27,80,115,222,199,206,7,36,240,234,203,175,164,234,191,191,
  31,79,25,223,247,247,199,209,222,166,192,160,113,113,163,171,254,42,64,178,228,74,109,251,246,161,109,223,62,171,30,204,227,182,125,109,142,115,235,79,
  57,197,1,82,90,178,4,180,237,221,139,182,189,123,1,0,125,253,253,216,103,28,219,225,41,211,19,247,237,221,135,125,123,85,157,247,247,245,89,199,
  251,222,223,139,83,62,124,106,113,36,75,66,45,66,82,117,45,67,183,195,72,89,194,146,159,205,146,25,63,198,72,107,214,174,193,154,53,107,240,111,
  223,253,55,199,117,155,47,221,140,139,46,220,140,103,159,221,105,12,38,164,96,121,152,201,250,217,253,246,110,180,180,172,6,46,82,199,246,116,214,89,
  155,208,217,213,133,231,118,184,164,84,215,125,17,114,245,106,143,244,170,92,96,201,148,26,117,30,232,244,72,148,76,72,106,105,105,193,131,15,60,152,
  154,27,8,193,197,159,184,24,151,92,162,128,105,243,39,54,227,123,255,250,61,188,190,235,117,199,115,255,229,95,190,13,236,80,171,118,2,130,139,47,
  222,140,103,159,221,129,174,174,46,92,116,201,69,169,129,206,88,78,16,227,3,101,52,235,88,113,110,53,220,150,203,182,224,190,123,238,195,174,215,118,
  57,174,219,246,221,109,216,142,237,10,80,204,240,141,151,195,146,12,185,211,254,246,253,22,36,57,158,35,165,5,76,38,24,153,233,178,203,47,195,150,
  203,183,88,231,59,58,58,44,137,140,59,93,118,249,101,88,187,110,45,182,221,189,205,186,175,9,75,151,93,158,15,48,101,167,134,27,136,15,224,233,
  167,158,246,72,158,90,209,106,157,127,237,213,215,28,247,107,93,183,14,183,220,118,11,218,219,82,160,245,244,147,79,231,214,34,139,108,179,100,170,225,
  236,3,182,186,44,203,193,95,2,130,11,7,40,81,74,65,36,108,14,253,188,201,14,71,183,253,245,237,158,115,241,120,28,79,254,252,73,199,3,111,
  251,218,237,88,183,126,157,235,124,37,168,225,220,192,212,102,65,146,47,80,25,127,63,117,133,146,54,217,129,169,178,97,41,243,237,21,236,236,245,189,
  167,2,38,153,241,137,251,246,238,197,190,247,247,250,126,103,2,83,33,96,73,66,66,112,225,0,36,153,67,185,35,5,170,187,130,195,210,193,206,46,
  76,78,78,102,253,115,170,172,128,65,41,69,52,90,5,150,75,160,212,50,181,89,154,152,152,180,212,113,246,212,123,228,8,6,7,6,61,231,31,52,
  84,118,45,45,171,81,202,84,148,221,112,4,142,149,181,99,226,8,122,46,85,39,89,148,225,111,191,254,119,184,112,243,133,248,242,13,95,198,55,238,
  248,6,46,218,124,17,46,218,124,145,207,74,61,143,178,26,101,218,223,177,31,227,19,227,158,107,77,149,156,27,22,122,98,61,94,88,114,221,35,23,
  155,165,142,142,14,76,140,79,132,110,115,229,103,179,20,174,113,117,119,119,251,74,66,202,2,150,146,41,201,146,123,0,207,173,58,165,227,59,33,56,
  40,83,17,218,195,220,241,174,111,127,7,235,214,175,199,3,63,248,62,238,250,246,93,89,149,177,88,33,215,139,1,75,254,247,147,30,88,26,31,31,
  15,245,243,120,127,63,186,63,248,96,78,192,82,127,95,31,98,181,177,140,176,52,62,62,150,87,158,247,237,221,139,241,177,241,180,249,248,160,182,54,
  47,88,114,130,82,126,176,148,51,48,149,66,178,244,161,150,213,168,169,169,177,164,75,96,225,243,101,250,207,137,214,68,17,137,70,17,101,81,92,240,
  7,31,199,238,183,119,99,96,112,176,236,96,169,190,126,17,46,188,232,66,0,18,131,3,131,168,111,168,199,194,250,69,24,26,60,225,184,46,22,171,
  193,135,124,224,103,229,138,21,22,92,154,234,185,250,134,122,0,74,141,101,170,228,54,109,218,232,56,191,122,77,203,140,195,210,234,150,213,150,13,147,
  89,246,157,59,118,162,171,171,11,45,171,91,140,29,113,68,169,99,57,179,156,73,82,215,182,127,247,115,9,35,70,68,107,169,38,20,70,28,109,136,
  48,83,166,68,32,11,18,221,92,58,64,165,169,169,9,27,206,216,96,65,81,67,99,131,175,20,169,161,177,1,77,77,77,158,243,107,91,215,98,195,
  25,27,172,123,196,227,3,104,108,104,240,149,40,153,82,130,198,198,70,0,41,53,93,107,107,171,235,30,234,26,199,61,228,204,195,82,67,99,3,46,
  255,228,229,182,60,54,160,163,189,221,82,199,1,192,185,231,157,107,149,47,222,31,71,227,226,70,60,253,212,211,136,199,227,104,110,110,70,109,109,45,
  26,23,59,191,127,237,149,215,16,143,247,87,12,44,249,174,155,165,10,74,106,170,158,115,185,95,99,99,35,254,232,211,127,164,38,169,254,126,44,94,
  188,24,237,109,237,232,239,239,71,60,222,143,39,159,120,82,169,242,0,188,242,242,43,56,255,99,231,163,177,113,49,158,252,249,147,232,239,239,47,123,
  201,210,199,62,246,49,244,219,212,109,102,90,127,202,122,108,220,180,209,26,51,227,253,253,104,92,188,216,163,118,3,128,198,197,139,209,124,210,73,115,
  64,178,36,177,120,201,98,156,116,210,201,105,127,124,202,169,167,96,227,89,103,97,227,89,103,57,218,141,105,123,180,111,239,94,244,247,247,225,138,43,
  175,84,82,164,189,123,113,197,149,87,2,112,218,56,109,58,251,44,108,58,251,44,11,144,22,47,89,130,199,127,246,24,0,96,241,146,37,56,233,228,
  147,242,146,44,113,199,194,58,63,88,202,9,152,74,237,103,137,194,105,232,29,38,95,230,78,169,8,141,40,41,3,5,46,189,244,82,28,250,224,3,
  12,158,56,17,44,73,152,33,201,82,253,162,69,86,167,53,13,182,1,120,128,41,76,58,235,172,77,144,6,136,0,74,229,101,170,228,54,157,181,201,
  99,96,61,211,146,165,150,150,22,127,163,111,10,180,174,105,117,120,219,38,140,64,18,233,80,153,113,206,51,62,215,108,59,213,172,26,148,81,107,23,
  156,233,45,150,128,128,18,106,236,190,163,217,151,213,85,166,237,79,111,199,150,203,183,120,140,190,253,128,41,93,178,27,134,155,246,78,110,96,218,114,
  249,22,199,247,118,99,110,0,88,187,206,107,24,238,190,199,76,75,150,26,27,27,113,238,121,231,170,60,58,140,190,219,209,186,110,157,1,76,231,121,
  140,190,237,198,220,126,247,232,104,107,15,6,166,18,218,44,21,11,150,82,210,86,9,25,225,233,199,200,52,247,107,92,188,216,2,34,211,208,219,52,
  250,6,128,39,127,254,4,110,55,140,195,77,96,2,128,239,252,243,119,202,27,150,12,141,208,249,191,255,49,135,173,146,19,154,188,70,221,126,192,148,
  2,84,189,27,46,5,77,167,122,140,190,21,12,73,101,207,180,23,248,63,255,241,239,120,252,49,24,192,244,25,60,254,216,127,227,241,199,30,179,238,
  123,202,169,167,120,140,190,31,255,217,99,217,162,141,243,82,9,72,34,11,14,75,89,3,211,76,56,165,164,148,102,220,29,18,38,95,44,202,192,34,
  20,145,72,68,25,78,186,109,155,102,80,13,215,217,213,229,216,241,6,164,118,197,101,155,30,180,221,199,125,15,199,51,164,68,203,154,22,124,49,199,
  231,20,2,150,0,96,231,142,157,232,60,208,105,217,45,185,237,153,76,160,9,108,2,1,174,3,82,224,67,44,64,122,230,151,207,88,247,99,96,41,
  191,77,204,112,85,65,242,135,37,59,52,185,1,105,219,119,183,97,251,47,182,135,6,167,95,60,189,93,217,48,165,105,155,55,221,120,147,101,139,116,
  211,141,55,249,230,99,91,251,182,34,247,232,252,212,112,29,237,29,184,231,127,223,147,54,79,247,220,117,183,101,183,116,207,93,119,123,174,106,111,111,
  47,234,110,56,9,25,60,14,201,84,91,76,138,100,58,121,80,193,97,201,50,111,74,74,223,209,92,202,204,47,163,189,173,13,119,125,251,95,112,219,
  215,110,71,123,91,59,238,250,246,119,60,247,255,206,183,191,131,245,235,215,227,251,63,120,16,95,254,139,235,242,30,5,75,103,224,157,254,62,79,60,
  254,243,180,54,76,69,154,240,102,53,44,153,167,31,127,236,191,45,67,239,92,23,23,143,255,236,191,177,111,239,183,242,27,149,220,146,93,8,8,46,
  11,14,75,74,128,51,195,176,68,0,80,66,173,127,110,88,202,164,138,11,155,47,10,10,98,20,151,49,102,249,115,154,105,88,42,185,132,167,64,229,
  45,4,44,185,193,105,209,162,69,184,232,226,139,28,171,116,127,70,18,16,194,208,75,243,28,202,102,24,203,50,202,148,244,138,170,54,72,66,116,135,
  48,176,180,229,242,45,104,104,108,40,76,205,134,40,79,71,71,7,118,189,186,11,151,93,126,153,37,133,9,243,62,102,155,159,165,129,120,28,79,63,
  245,52,206,57,239,92,75,154,148,117,139,12,11,75,82,130,35,9,14,14,206,57,68,82,128,39,213,95,135,170,205,140,154,144,20,51,6,75,86,242,
  113,75,192,185,243,228,121,31,59,31,235,79,89,135,167,158,120,210,144,190,165,238,242,202,203,175,160,191,191,31,127,244,233,79,161,113,113,227,172,178,
  89,146,249,141,136,26,150,10,114,165,247,91,83,253,118,197,149,87,42,192,10,48,240,14,209,186,195,193,146,16,16,162,56,176,20,26,152,138,9,75,
  166,61,10,141,24,255,136,146,2,69,34,17,176,104,24,223,35,33,129,201,165,106,177,252,154,84,8,44,21,66,154,55,83,176,4,0,59,159,221,137,
  250,134,250,140,134,216,105,219,67,208,115,252,126,194,0,86,165,36,141,97,119,197,133,117,74,185,229,178,45,104,108,104,44,89,123,234,104,239,192,174,
  215,118,89,254,157,102,31,44,201,80,35,190,185,19,238,188,243,207,195,121,231,159,87,60,88,2,192,9,87,82,155,164,146,210,152,255,9,243,191,164,
  64,50,145,84,127,147,73,95,245,91,73,97,9,64,82,38,149,251,1,219,123,22,73,103,190,206,255,216,249,88,183,126,61,158,252,249,147,136,187,236,
  143,94,125,249,21,196,251,227,170,253,54,46,158,189,176,84,224,1,89,171,225,114,131,37,5,76,143,25,192,244,25,60,254,179,199,124,36,82,133,131,
  37,8,64,160,120,176,4,132,80,201,21,3,6,136,177,151,219,138,4,111,219,1,69,35,74,5,119,184,231,48,206,56,253,12,108,249,196,165,198,132,
  151,154,245,222,124,227,77,252,242,153,95,1,0,110,252,202,13,0,128,195,135,123,177,106,213,74,188,241,198,155,184,237,86,165,111,191,237,214,219,241,
  145,143,126,4,159,248,202,37,184,237,54,117,124,209,245,23,226,239,254,230,127,169,103,17,2,65,12,163,95,89,144,166,155,53,188,4,237,112,235,61,
  114,196,241,249,178,79,42,227,222,131,157,93,214,177,185,237,251,193,7,127,128,21,43,86,88,42,184,222,35,71,176,114,197,10,235,120,98,98,82,249,
  53,114,117,252,137,241,9,116,117,118,133,223,73,85,96,88,234,234,236,194,196,196,132,3,154,118,62,187,19,215,93,127,29,222,122,243,45,229,156,242,
  215,192,13,215,223,0,68,128,99,71,142,97,217,138,101,120,231,173,119,240,117,195,225,228,215,238,184,3,31,253,232,71,240,149,175,222,8,0,56,210,
  123,4,43,154,86,224,175,110,254,43,0,192,71,207,253,40,98,61,49,207,179,135,6,135,240,193,193,15,114,43,107,134,1,116,235,87,182,226,156,115,
  207,193,214,203,213,174,184,158,158,30,52,53,53,133,218,37,39,109,82,163,13,27,54,88,54,74,135,187,15,99,85,243,42,0,192,174,87,119,33,30,
  143,35,214,237,44,215,77,55,222,164,158,123,235,86,108,187,123,155,255,61,154,140,123,188,182,203,106,63,55,223,122,179,35,159,187,94,219,133,175,222,
  240,213,146,192,82,79,79,143,195,184,219,125,251,137,137,113,203,102,201,188,255,221,119,221,141,166,230,38,220,122,219,173,120,244,191,30,69,119,119,55,
  98,177,24,110,189,237,22,0,64,119,119,15,154,155,155,172,227,159,254,215,163,217,193,18,146,10,150,210,193,142,76,73,59,211,215,82,105,195,157,72,
  72,36,147,73,16,73,48,54,60,134,125,109,251,112,229,103,175,194,73,77,39,129,49,106,83,183,169,187,116,127,208,237,48,220,110,111,107,195,13,215,
  223,128,63,254,147,63,198,153,27,207,196,127,254,228,63,173,203,199,199,199,45,187,166,242,134,37,137,127,185,243,159,209,124,210,73,248,235,175,223,129,
  159,60,242,99,0,176,142,205,221,110,109,251,218,176,113,211,70,124,234,138,79,27,117,241,129,101,216,253,202,75,47,225,229,151,94,182,96,169,191,175,
  223,187,115,172,66,97,169,191,191,31,177,67,135,112,199,223,254,141,239,175,238,252,230,183,176,111,239,62,108,58,107,19,174,184,242,51,0,36,62,56,
  244,129,97,160,13,188,244,155,223,224,165,23,127,99,93,175,252,43,121,65,105,223,222,189,216,116,246,89,184,226,170,207,0,128,227,30,191,121,241,55,
  232,235,235,71,109,237,33,124,227,239,254,214,55,31,255,244,15,223,52,250,171,0,19,20,66,22,222,102,201,125,65,36,159,123,88,33,36,12,150,113,
  139,127,61,171,124,105,218,143,16,128,165,84,98,118,169,129,121,124,240,224,65,12,14,12,98,98,210,59,153,199,7,6,172,227,3,7,14,168,137,103,
  96,16,147,147,19,24,176,125,167,206,15,216,174,25,64,87,87,202,163,53,165,20,148,82,72,33,13,227,97,153,95,103,205,65,210,115,164,247,8,134,
  124,28,75,30,233,61,226,251,121,114,114,210,1,88,102,30,38,39,39,173,178,13,14,12,98,202,216,53,55,56,48,168,190,243,113,71,96,158,207,197,
  125,67,33,36,75,157,157,157,152,156,152,244,61,127,226,196,9,229,21,89,80,236,239,220,15,70,25,6,135,84,89,226,158,119,60,104,217,62,197,7,
  226,14,8,27,136,15,248,62,123,32,62,128,253,251,247,23,28,150,236,247,183,59,157,180,231,201,125,157,231,57,82,98,127,251,126,196,227,169,178,216,
  143,221,109,220,221,222,77,231,148,206,123,72,196,227,3,150,171,2,59,164,184,243,233,11,48,69,145,44,1,61,221,221,136,187,223,145,116,66,125,123,
  123,187,231,254,230,249,137,241,9,244,116,247,24,112,101,212,85,127,220,113,156,21,44,201,36,36,151,5,233,248,51,25,27,78,66,98,120,124,24,251,
  246,238,197,224,192,9,140,143,143,217,164,236,169,187,184,93,50,72,235,189,244,120,194,177,140,143,143,163,109,95,123,217,195,82,170,141,140,163,109,223,
  62,76,24,174,3,236,199,230,231,120,127,191,229,90,192,126,220,223,223,239,144,44,5,239,6,172,60,201,82,127,95,191,85,127,65,247,84,187,224,236,
  245,213,151,58,238,115,214,85,127,95,191,229,144,210,121,143,125,232,239,51,127,39,209,215,215,143,241,241,49,11,178,204,228,112,253,224,151,25,97,152,
  105,216,29,150,21,1,150,0,128,220,123,239,54,249,228,19,79,230,84,241,140,166,182,119,251,217,155,8,206,173,78,71,41,51,194,38,17,228,163,101,
  43,90,108,56,9,72,46,145,148,201,146,193,82,129,4,188,37,123,94,33,97,41,155,68,65,129,8,252,183,78,203,226,42,68,117,108,184,226,193,82,
  185,197,134,171,20,88,74,215,143,40,37,190,134,24,230,116,227,81,79,235,216,112,133,233,47,58,54,92,33,90,183,191,235,0,33,64,0,8,41,139,
  10,75,159,251,243,63,247,151,48,229,242,98,253,182,99,123,206,201,130,54,153,194,194,10,1,72,132,128,38,221,162,61,13,75,51,9,75,230,96,14,
  78,64,152,116,238,110,212,176,164,97,169,80,187,225,164,204,28,166,100,22,195,146,234,71,28,66,192,218,89,170,0,138,26,195,31,85,22,90,212,22,
  28,88,195,146,134,165,50,135,37,171,109,23,25,150,82,139,142,60,96,128,11,238,235,113,185,24,147,104,81,97,201,150,88,132,33,18,137,120,118,236,
  105,88,154,25,88,178,143,135,2,220,182,143,90,195,146,134,165,2,186,14,32,133,233,116,229,10,75,126,23,8,168,157,125,92,168,93,167,16,0,146,
  6,80,105,88,210,176,52,75,96,137,81,90,18,88,242,0,83,46,47,150,11,94,49,176,100,38,66,136,181,99,79,195,82,25,192,18,76,23,247,82,
  195,146,134,165,194,195,18,0,14,158,217,207,82,5,193,146,253,27,225,250,15,60,216,165,135,121,47,1,1,14,174,97,73,195,210,140,194,146,121,1,
  13,25,26,40,223,254,66,11,10,37,21,0,75,22,52,25,129,88,53,44,205,60,44,57,78,104,88,210,176,84,4,88,2,15,184,168,194,97,201,239,
  164,80,86,180,62,59,0,37,132,16,224,130,67,242,96,71,158,26,150,52,44,149,18,150,0,64,18,20,188,255,249,165,72,57,195,82,125,67,131,21,
  251,108,98,98,2,177,88,12,7,246,171,29,111,13,13,13,168,137,213,160,247,112,175,227,55,13,198,111,14,236,63,128,53,107,215,96,98,124,2,177,
  88,141,227,30,128,218,61,150,110,183,17,96,72,154,8,5,151,162,228,117,96,6,204,157,152,152,68,44,86,131,129,193,19,24,52,226,224,181,180,172,
  70,111,239,17,207,238,182,150,150,213,214,14,186,154,154,26,107,27,236,196,248,132,99,75,172,159,55,237,153,130,165,250,250,122,207,59,238,60,208,233,
  120,110,203,154,22,181,235,73,2,11,235,22,96,112,104,208,218,93,182,182,117,173,227,189,218,143,205,247,27,139,197,124,191,7,84,216,146,98,195,210,
  170,166,85,152,152,152,176,242,108,222,185,105,149,218,226,223,211,163,118,121,153,49,229,98,181,49,203,213,131,223,119,128,218,229,102,223,97,23,139,197,
  208,212,220,100,181,119,233,122,239,61,221,61,104,108,108,244,221,5,103,222,219,124,86,161,96,41,86,27,67,83,83,147,163,206,59,218,219,29,247,107,
  50,182,255,199,98,42,200,230,248,196,56,106,109,199,241,120,28,205,77,205,104,183,126,151,30,150,204,58,176,151,49,86,27,67,83,115,51,218,219,218,
  29,23,199,106,107,177,114,229,10,124,112,168,27,147,201,9,47,244,204,65,88,114,174,166,169,218,169,138,148,202,35,83,232,32,13,75,26,150,74,14,
  75,37,144,44,57,128,41,159,20,216,129,10,32,89,250,200,71,207,198,37,159,184,4,0,44,0,50,125,44,125,228,163,103,99,205,154,53,184,255,187,
  255,230,248,157,249,155,219,110,189,29,55,126,229,6,235,119,246,123,0,192,175,158,249,21,126,249,204,47,51,75,154,24,5,146,34,215,158,152,115,249,
  255,194,240,169,100,198,87,123,110,231,115,216,185,67,197,132,251,226,117,95,196,15,30,252,1,58,93,174,2,190,120,221,23,241,156,113,141,61,168,173,
  59,110,220,223,126,253,111,203,70,178,180,233,236,77,150,179,74,51,159,223,184,195,233,255,227,186,47,127,73,185,69,32,192,250,245,235,241,194,175,95,
  192,246,95,168,240,34,55,221,124,147,21,171,205,4,32,243,248,153,95,60,99,65,149,223,247,0,112,243,87,111,46,186,100,233,170,207,94,133,142,142,
  14,108,127,122,187,67,178,116,229,213,42,24,229,182,187,85,232,18,243,115,107,107,171,181,213,223,239,59,0,158,152,113,77,205,77,216,122,203,86,43,
  142,154,130,147,212,241,189,119,223,139,203,46,191,12,29,29,29,150,255,37,43,127,87,95,101,93,83,72,201,82,83,83,19,110,249,171,91,28,249,248,
  202,95,222,232,184,253,103,255,248,106,85,46,159,60,119,180,119,224,233,39,159,198,45,183,221,130,27,175,191,49,148,100,233,234,63,185,26,237,109,29,
  120,250,201,167,172,211,77,205,205,184,245,182,91,113,195,151,111,112,92,188,114,229,10,124,249,47,175,199,253,255,250,93,107,33,166,97,201,150,132,1,
  77,148,135,114,238,170,97,73,195,210,76,193,146,44,98,127,41,40,48,89,206,39,11,12,75,55,126,229,6,188,248,226,111,44,32,50,87,169,55,126,
  229,6,188,241,198,155,161,38,242,251,255,245,254,212,239,190,122,35,222,120,253,13,11,146,252,156,69,250,86,33,201,185,39,230,244,179,141,155,54,98,
  197,202,21,248,225,131,63,80,82,2,67,194,180,186,101,53,190,116,221,23,61,49,231,252,158,247,246,91,111,227,253,223,189,143,88,109,12,27,55,109,
  68,125,125,61,126,16,244,187,25,132,165,235,174,191,14,47,191,244,50,30,250,254,67,214,59,174,137,197,240,229,235,175,195,155,111,190,133,183,223,122,
  27,0,240,224,3,15,226,180,15,159,134,21,39,173,192,67,223,127,200,33,25,188,239,222,251,124,37,76,55,221,236,140,171,118,223,189,247,249,74,152,
  138,13,75,190,207,9,120,198,99,143,62,134,85,77,171,208,218,218,138,199,30,125,204,243,253,225,238,195,86,28,186,88,44,134,173,183,110,197,99,143,
  62,134,158,158,30,244,116,247,96,219,221,247,98,124,124,2,173,235,90,113,213,213,87,225,103,143,254,204,33,97,202,103,208,206,167,86,126,250,232,163,
  150,4,233,179,127,252,89,64,2,63,253,175,159,90,223,247,116,119,227,233,39,159,246,149,48,153,199,97,36,75,87,255,201,213,120,244,63,31,69,99,
  99,35,110,189,253,86,60,250,159,143,162,187,59,88,106,38,32,252,39,76,13,75,150,90,142,82,162,97,73,195,82,217,75,150,24,165,72,6,249,129,
  44,16,44,229,13,76,190,210,165,2,218,44,13,12,12,120,84,110,248,68,248,123,217,87,141,7,246,31,192,225,195,135,189,247,203,144,15,2,130,72,
  36,130,100,50,153,77,79,204,187,11,118,118,118,57,234,98,209,162,69,88,189,122,117,168,231,153,170,59,243,119,19,43,38,202,74,13,231,206,171,233,
  148,211,186,163,61,60,138,148,232,60,208,137,134,198,6,176,8,115,168,208,76,137,145,61,53,52,54,32,22,139,97,127,199,126,196,7,148,90,38,214,
  19,243,92,87,106,88,10,243,12,83,37,214,209,209,225,163,30,83,176,103,122,189,54,213,104,202,115,183,68,79,119,15,218,141,239,76,233,140,223,61,
  74,13,75,110,88,59,227,140,13,136,213,214,162,117,221,58,244,244,116,163,167,187,27,221,221,61,54,149,155,75,74,213,220,228,84,227,5,192,82,99,
  99,163,227,89,166,100,201,17,156,214,86,18,14,110,121,243,214,176,228,159,4,56,104,136,41,66,195,210,236,133,165,188,103,130,50,82,195,81,16,87,
  104,148,194,194,146,122,70,153,194,210,154,181,107,60,146,0,0,184,255,187,255,134,55,94,127,35,235,242,7,221,47,212,45,72,86,61,49,175,58,168,
  111,168,199,138,21,43,60,121,120,251,237,221,254,210,165,12,207,171,175,175,199,202,149,43,203,18,150,86,183,172,182,222,137,253,142,223,127,224,65,37,
  93,34,18,81,26,5,141,80,44,110,92,140,21,43,87,100,188,231,57,231,156,131,155,110,190,9,247,221,123,31,94,127,237,117,52,54,52,90,225,64,
  102,10,150,172,177,62,196,51,98,181,177,80,241,224,122,122,122,176,237,238,109,56,231,188,115,112,229,103,175,114,148,37,86,27,11,31,132,183,104,176,
  228,63,104,255,244,191,126,138,215,94,121,13,183,220,118,11,154,154,154,209,212,220,140,198,197,193,113,247,106,99,49,180,174,91,151,246,17,87,255,201,
  213,104,106,110,198,221,223,185,27,128,10,135,114,247,119,238,198,185,231,159,139,171,255,68,169,252,38,19,147,214,111,146,92,133,62,209,54,75,25,238,
  69,145,114,55,160,97,169,34,97,73,230,222,216,202,207,102,137,20,79,178,148,149,132,41,147,161,95,161,97,169,24,171,247,124,242,17,58,0,176,44,
  113,24,222,89,190,27,238,161,239,63,132,21,43,86,88,54,76,189,189,189,88,185,114,37,190,255,192,131,42,116,13,161,42,72,46,24,12,55,241,105,
  211,77,55,223,132,231,127,253,188,165,126,115,127,7,0,135,123,84,60,181,93,187,118,97,215,107,175,151,6,150,108,233,170,207,94,133,158,158,30,236,
  122,109,87,9,95,154,77,98,211,212,228,137,29,215,212,220,228,85,217,149,96,55,156,61,79,183,222,118,171,1,59,221,104,110,110,198,163,255,245,40,
  122,186,187,179,46,183,244,138,73,208,221,221,141,127,189,231,62,92,254,71,159,4,0,252,252,191,31,199,167,63,115,5,106,106,106,240,111,223,189,31,
  135,123,122,53,44,5,124,41,168,176,108,153,66,57,1,215,176,164,97,9,51,99,224,77,41,85,209,69,138,1,75,50,11,96,50,51,227,0,7,82,
  92,88,58,176,255,64,96,252,173,82,194,146,227,101,8,81,116,144,24,28,24,68,117,77,13,74,157,102,194,117,64,231,129,78,44,90,180,200,115,190,
  165,165,5,131,131,3,24,29,25,181,206,69,105,52,16,220,27,26,27,208,208,208,0,64,169,113,15,247,28,206,98,64,46,5,44,165,142,86,53,175,
  202,174,93,207,4,160,151,16,150,130,239,145,221,72,31,171,141,161,117,253,58,43,238,84,109,109,45,106,99,181,24,27,31,195,196,216,56,58,58,59,
  112,234,105,31,70,77,77,141,181,241,99,114,114,82,169,237,53,44,249,127,105,198,250,164,196,138,5,106,158,211,176,164,97,169,156,96,201,146,0,49,
  134,100,146,231,251,192,192,203,35,89,181,15,226,227,123,163,72,146,165,108,84,104,197,134,37,0,160,44,13,48,21,208,207,82,125,67,61,86,174,88,
  129,82,166,153,244,179,244,214,91,111,227,45,195,184,219,124,206,63,125,251,159,176,243,217,157,120,225,249,23,82,231,153,241,207,39,157,115,206,57,184,
  244,178,75,173,29,111,126,201,148,58,205,117,63,75,61,61,61,174,93,114,18,55,223,122,75,241,96,41,100,158,76,163,239,92,71,250,166,230,38,92,
  120,209,133,150,45,212,186,181,235,144,20,73,236,219,167,130,126,74,46,241,243,255,126,28,107,214,174,193,13,95,185,17,183,223,122,91,150,99,250,28,
  131,37,0,224,198,166,30,98,192,134,0,8,35,254,62,111,52,44,105,88,66,121,184,14,96,148,130,103,112,188,154,107,213,70,242,108,133,197,159,168,
  93,201,179,75,174,4,176,164,22,91,1,198,223,37,114,74,185,105,211,70,108,58,107,147,181,163,172,18,96,233,203,215,95,135,39,159,124,202,50,250,
  182,63,199,238,52,212,172,127,70,189,196,148,78,13,151,126,76,159,221,176,180,170,169,9,87,93,125,21,118,189,182,43,203,29,112,233,7,237,98,75,
  150,174,250,227,207,162,182,182,22,247,220,117,15,186,123,186,179,107,145,105,212,112,61,221,10,186,82,18,166,24,62,245,201,79,35,145,72,32,201,147,
  121,13,64,115,18,150,76,102,50,85,28,12,0,5,40,103,32,17,162,97,73,195,82,89,194,18,160,252,39,50,74,45,231,171,50,215,182,233,115,121,
  238,192,36,100,238,91,238,67,100,61,147,74,110,96,96,0,53,62,170,171,129,129,1,175,79,149,52,247,203,166,10,9,33,136,208,8,146,34,89,52,
  88,10,163,146,235,234,236,178,28,27,186,207,219,119,200,1,106,23,90,117,111,117,217,193,82,186,231,248,149,35,62,16,71,236,112,44,176,45,4,169,
  225,226,3,113,212,244,196,102,20,150,14,119,31,182,28,41,186,143,221,105,98,124,194,242,193,100,79,214,239,92,146,165,158,238,30,207,110,184,137,241,
  212,110,58,183,36,39,229,236,82,58,206,231,55,69,249,15,218,19,19,227,142,124,152,87,141,143,143,91,146,160,238,238,110,196,251,227,129,247,25,159,
  24,87,187,228,50,192,82,79,119,143,205,49,165,132,228,18,123,215,237,245,214,239,132,86,195,229,52,18,152,97,28,153,150,44,105,88,42,95,88,178,
  173,176,93,158,234,243,135,165,156,128,73,66,2,210,80,205,217,116,218,133,150,106,220,255,221,127,195,39,46,189,4,183,221,254,87,22,240,216,29,87,
  154,19,210,93,247,220,229,248,254,87,207,252,10,247,255,235,253,158,251,249,169,248,114,106,248,12,160,130,66,72,94,200,110,104,165,183,223,222,13,188,
  189,27,223,252,167,127,180,224,97,117,203,106,60,183,227,57,75,186,244,208,247,31,194,69,155,47,194,87,111,254,170,227,26,63,135,148,65,187,228,202,
  1,150,190,255,192,131,216,124,241,69,216,106,24,100,91,142,43,191,246,13,80,70,45,137,210,119,238,253,14,14,117,30,2,0,220,123,223,189,120,102,
  251,51,150,227,202,181,173,107,113,229,85,87,6,58,174,148,0,154,154,86,97,219,191,222,235,249,30,0,182,126,101,107,209,37,75,63,251,233,207,208,
  186,174,21,247,221,127,31,182,221,163,28,81,154,199,118,160,216,122,171,202,75,107,107,171,117,108,58,174,92,213,188,10,171,154,86,225,207,62,255,103,
  0,128,95,60,253,11,151,163,201,84,10,218,37,103,122,221,118,191,125,211,69,65,161,37,75,177,88,45,90,215,181,226,150,219,110,181,242,163,28,80,
  166,82,115,115,51,154,155,155,240,249,47,124,30,128,242,4,110,238,138,235,104,111,199,211,79,62,141,214,117,235,112,235,237,183,250,238,150,187,251,174,
  187,45,239,222,118,88,226,146,99,229,42,111,187,143,217,108,152,52,44,133,31,5,181,205,146,134,165,89,3,75,48,66,155,89,118,199,133,129,37,0,
  32,247,222,187,77,62,241,196,147,25,239,65,9,85,106,18,163,17,6,26,128,23,0,150,204,212,16,20,26,197,214,17,204,193,207,252,62,40,228,201,
  154,181,107,208,123,184,215,146,50,201,188,58,162,128,224,34,189,17,120,158,117,225,14,141,50,56,48,232,144,186,56,66,138,24,33,48,252,124,45,213,
  215,215,163,166,166,6,71,142,28,41,75,201,82,125,253,34,212,215,123,67,163,0,64,52,26,181,160,200,124,111,177,88,12,3,3,3,161,67,163,72,
  227,55,181,177,24,32,221,206,16,225,43,205,41,36,44,89,19,181,17,186,196,84,159,153,199,118,169,103,218,208,40,171,86,169,178,24,142,40,7,226,
  3,158,16,39,238,103,185,165,76,10,152,198,157,191,147,192,42,35,60,73,246,170,189,244,131,182,10,141,210,236,168,115,183,191,37,21,26,69,250,58,
  171,28,31,55,66,163,52,55,99,124,124,28,181,181,181,158,199,116,119,119,171,144,47,227,19,136,199,251,149,48,132,115,72,41,177,210,8,61,211,123,
  56,37,201,171,169,137,97,213,170,149,190,82,104,13,75,153,175,98,140,129,16,162,97,73,195,82,217,194,146,163,15,115,9,65,132,26,150,137,226,22,
  153,238,101,202,224,187,125,238,11,95,8,15,76,128,114,12,101,2,146,181,107,174,72,176,84,204,137,188,80,129,116,19,137,68,241,243,80,169,129,116,
  67,60,199,4,166,130,60,83,7,210,197,156,8,164,107,192,82,62,157,78,195,82,250,171,34,44,162,97,73,195,210,172,138,13,103,191,84,64,24,42,
  230,176,126,216,212,23,159,251,194,23,178,112,92,41,37,132,20,16,182,64,180,146,228,245,154,103,53,44,153,19,122,40,31,85,26,150,114,122,142,224,
  162,48,207,212,176,84,209,176,36,33,32,147,18,201,100,82,195,82,145,97,201,233,241,65,195,146,134,165,217,5,75,74,240,67,189,236,34,195,61,136,
  134,108,133,190,247,8,235,208,177,18,97,201,76,140,49,208,40,45,124,30,230,56,44,21,236,153,26,150,42,27,150,164,0,79,114,112,240,188,59,157,
  134,165,112,247,149,153,252,100,105,88,210,176,84,166,176,100,38,98,247,43,41,195,63,40,68,84,197,212,143,8,33,74,127,93,42,72,41,115,88,178,
  160,9,204,179,13,94,195,82,254,176,196,193,211,134,101,208,176,164,97,137,115,94,144,78,167,97,41,252,125,57,79,102,13,39,26,150,52,44,149,11,
  44,41,240,33,160,140,170,104,18,89,60,136,230,50,137,134,129,131,185,2,75,22,52,81,150,54,170,183,134,165,28,158,35,128,4,79,64,128,103,255,
  76,13,75,168,116,53,156,134,165,210,195,146,249,65,8,145,251,156,169,97,73,195,210,12,194,146,253,34,66,8,40,101,161,31,68,115,158,68,137,134,
  165,130,231,65,195,146,127,226,89,62,83,195,18,230,130,26,78,195,210,204,192,146,9,41,92,24,6,246,217,188,86,13,75,26,150,202,4,150,236,44,
  147,18,118,164,255,53,101,81,166,220,223,135,152,112,164,148,86,3,77,103,191,52,103,97,201,39,124,135,134,37,104,155,37,13,75,133,131,165,164,150,
  44,205,52,44,57,160,137,115,112,193,195,181,9,13,75,26,150,202,13,150,96,196,117,103,36,212,175,35,53,209,26,181,211,139,2,34,33,32,101,241,
  124,11,85,52,44,65,169,229,24,24,4,21,224,9,174,97,73,195,146,134,165,2,193,146,16,2,68,32,189,113,183,134,165,146,193,146,103,33,13,103,
  156,81,13,75,26,150,102,3,44,101,251,118,44,215,74,20,20,148,146,66,54,153,57,5,75,206,74,165,144,180,116,207,171,120,88,98,26,150,230,50,
  44,41,98,146,26,150,202,16,150,236,64,107,99,15,13,75,26,150,102,29,44,121,52,109,62,63,141,56,38,28,10,143,189,72,216,93,113,26,150,156,
  121,96,140,41,81,181,134,165,226,66,140,134,165,138,134,37,1,1,8,153,217,133,137,134,165,25,131,37,179,94,149,7,101,151,51,99,13,75,26,150,
  102,1,44,217,128,199,51,214,75,135,48,196,136,13,103,74,70,24,99,32,198,127,140,49,80,74,125,163,196,107,88,74,159,7,10,10,22,101,26,150,
  52,44,105,88,202,19,150,50,186,123,211,176,52,163,176,100,94,206,33,156,117,172,97,73,195,210,108,130,165,16,87,68,78,156,56,129,51,206,60,195,
  250,54,33,166,13,41,19,73,169,66,132,205,235,50,49,182,208,51,154,147,151,235,34,34,87,217,37,158,228,72,36,19,69,47,110,121,213,94,225,115,
  67,41,69,148,70,129,72,120,103,169,179,161,114,229,92,236,59,210,11,69,68,16,192,176,79,18,66,164,246,238,18,0,220,144,114,243,16,246,75,133,
  128,165,89,86,127,229,242,16,243,23,132,17,181,88,132,90,108,11,38,210,186,91,153,189,245,40,43,187,9,20,1,110,202,61,37,68,18,124,58,25,
  152,243,241,241,113,68,70,38,18,24,192,34,107,128,18,72,66,48,110,27,177,204,142,0,112,46,192,40,5,24,64,193,64,192,10,152,93,130,138,75,
  76,64,8,142,132,72,148,190,168,164,114,234,154,18,67,210,105,14,188,148,101,95,17,179,178,121,145,89,210,109,114,204,4,145,0,212,6,9,33,132,
  26,127,184,2,38,202,24,24,40,184,16,214,221,133,16,134,208,66,150,241,152,34,103,225,99,100,65,155,2,97,4,81,68,65,40,83,247,246,243,219,
  39,203,188,126,11,154,141,50,132,6,57,139,242,154,54,139,133,147,58,38,19,9,240,104,50,240,138,177,169,36,34,189,163,4,15,190,109,15,166,24,
  129,114,175,44,96,25,52,9,219,119,86,219,167,80,34,40,82,184,158,54,99,227,28,41,210,120,75,84,229,9,18,236,75,136,20,32,159,89,95,70,
  242,124,36,201,163,122,115,172,84,70,1,74,84,27,36,97,219,29,201,191,142,115,169,203,92,97,149,100,89,207,36,215,247,73,194,95,76,10,217,111,
  237,23,8,128,112,99,149,198,129,100,50,117,141,125,140,33,182,119,200,133,49,64,146,194,180,181,108,234,34,43,117,144,204,98,44,207,225,218,80,115,
  69,30,215,230,58,161,74,91,189,70,35,106,220,179,22,55,52,195,109,178,81,37,21,248,90,235,178,44,223,177,44,210,181,185,180,7,89,168,182,147,
  173,30,80,230,127,109,46,16,36,11,221,118,146,64,18,128,100,129,125,254,243,27,69,144,172,148,166,26,184,221,203,0,173,76,213,89,145,229,35,10,
  44,153,174,137,188,146,48,38,76,46,116,93,84,68,178,193,146,16,222,119,45,168,99,205,230,11,80,129,192,99,140,95,84,215,242,140,73,0,4,7,
  168,80,127,117,210,105,214,76,215,36,227,108,158,126,162,79,63,131,161,82,237,143,10,251,18,76,104,210,212,148,223,40,12,67,188,79,116,117,84,18,
  9,11,17,48,190,24,99,12,231,10,148,205,215,78,8,192,140,127,196,24,228,172,152,80,154,146,202,34,37,37,144,72,34,235,96,144,58,233,52,83,
  41,196,252,28,201,242,142,182,193,204,252,75,140,65,74,79,98,105,235,205,46,154,230,9,93,37,57,85,35,53,212,113,58,205,122,72,50,199,139,48,
  243,41,49,129,89,184,152,40,10,48,219,88,148,52,23,39,134,181,184,78,51,191,198,225,73,245,194,168,6,89,157,202,150,238,149,36,148,100,246,246,
  157,161,21,219,196,218,212,14,77,246,159,113,189,138,200,38,81,45,105,202,45,17,128,68,53,152,207,250,148,72,193,76,88,117,141,57,134,73,195,5,
  10,113,47,70,204,190,21,177,173,235,236,82,39,232,118,51,147,208,68,133,158,35,116,42,99,88,74,168,230,233,27,159,146,102,9,76,214,128,196,156,
  131,148,101,231,68,161,197,224,25,41,73,215,83,33,170,80,167,89,152,132,243,88,216,225,133,5,168,227,194,36,99,60,50,213,221,148,165,64,201,92,
  200,57,236,221,164,134,166,153,90,232,8,143,49,154,78,58,149,199,226,45,153,0,184,4,164,72,173,204,40,113,224,143,57,214,48,176,48,42,185,8,
  156,142,81,92,3,150,94,189,233,84,18,96,98,208,246,114,179,49,113,127,120,18,60,247,97,131,75,23,28,25,81,175,9,87,42,58,46,2,4,26,
  186,253,148,62,73,245,46,44,41,147,94,249,232,84,110,11,57,159,177,129,82,99,147,27,51,174,101,160,140,132,181,97,162,105,110,174,97,41,251,201,
  159,42,67,86,157,116,154,83,3,147,225,50,64,24,106,252,156,181,52,46,247,4,246,197,27,215,210,140,242,107,2,134,148,73,179,146,78,229,56,23,
  11,225,227,54,196,102,139,68,13,104,162,34,91,163,111,13,73,133,161,89,157,178,175,194,36,192,170,160,44,123,35,186,62,102,77,50,253,186,17,231,
  216,33,96,136,193,141,177,41,151,46,66,121,234,126,230,189,192,117,119,43,203,241,47,98,123,201,154,156,116,154,113,82,50,254,48,128,38,128,164,29,
  154,12,59,99,41,29,139,49,202,244,204,51,67,227,135,30,209,115,99,78,225,239,53,88,167,50,78,166,147,81,17,188,224,202,170,59,16,99,195,92,
  42,84,19,184,52,36,24,66,107,221,202,121,204,163,81,93,15,58,149,31,52,161,10,136,36,12,193,180,93,18,74,124,151,127,58,149,36,25,106,130,
  132,86,25,228,87,135,26,152,102,239,192,36,109,159,121,70,46,114,252,196,74,198,9,97,216,198,128,66,73,29,225,20,100,233,84,158,11,30,237,193,
  87,167,178,75,4,202,69,9,15,104,184,212,24,121,132,158,125,74,186,210,214,187,69,242,75,90,122,48,203,161,201,54,220,120,92,107,24,91,82,136,
  221,237,70,152,208,54,90,90,59,107,136,41,105,186,148,208,239,76,167,114,132,38,102,251,231,191,232,211,192,164,211,236,105,207,122,117,90,1,208,4,
  195,21,0,53,226,3,166,88,201,227,226,77,102,19,87,75,167,242,103,38,1,76,39,148,223,27,13,77,58,149,229,36,147,30,137,180,74,174,100,137,
  67,75,151,242,152,104,25,51,28,17,106,104,154,221,201,102,252,203,108,148,36,141,208,39,82,218,124,53,105,82,170,64,106,178,105,230,180,155,1,157,
  202,114,101,238,219,110,57,23,26,152,74,186,186,210,41,103,94,82,33,81,244,224,90,49,47,148,16,231,170,142,24,134,225,196,102,231,68,252,182,251,
  234,84,9,204,164,66,218,232,197,143,78,229,152,204,118,105,134,99,18,128,76,64,36,120,233,128,105,203,25,75,176,229,140,37,0,128,222,193,41,172,
  172,175,182,142,1,56,62,91,199,39,166,176,114,81,234,248,137,183,143,227,80,255,4,198,166,130,37,53,219,62,119,170,117,252,189,231,186,67,94,79,
  66,95,95,138,116,221,31,52,97,205,146,24,0,224,232,208,52,150,47,172,2,0,236,254,96,4,27,79,170,179,174,179,127,183,253,221,56,186,250,198,
  241,65,124,202,113,175,59,63,187,54,117,253,240,52,150,47,48,238,213,61,138,141,205,243,141,243,9,44,95,16,181,93,151,250,188,253,189,56,186,250,
  38,192,24,197,245,23,172,180,190,223,254,110,28,109,71,199,51,150,229,206,171,214,88,199,15,191,114,4,221,3,83,56,127,237,2,108,62,165,193,147,
  167,163,195,211,0,224,248,188,124,65,21,34,209,8,250,70,57,86,54,212,0,18,56,50,34,240,196,111,227,120,191,119,12,219,254,52,85,190,222,19,
  211,88,185,72,253,246,205,131,35,56,251,67,117,158,243,0,240,189,23,143,162,111,100,26,125,35,169,152,126,107,151,198,176,245,194,149,0,136,163,221,
  1,192,155,135,70,112,246,201,198,189,134,166,176,114,97,181,117,252,196,158,65,28,234,159,196,216,116,241,129,120,203,239,45,194,150,223,91,100,228,105,
  12,103,159,60,47,85,246,161,4,86,26,109,225,137,119,7,241,219,158,113,244,141,6,199,44,220,246,217,147,83,229,251,96,12,103,159,52,207,184,207,
  180,117,159,222,161,4,158,120,239,4,14,13,76,5,150,111,237,146,106,108,189,96,169,186,126,88,61,111,229,194,168,45,79,81,227,25,227,120,179,123,
  28,239,31,157,180,129,18,131,83,53,99,196,129,99,134,87,110,189,200,168,220,5,144,228,134,196,88,234,133,144,78,179,38,149,12,152,150,47,170,198,
  153,39,47,0,0,44,136,141,163,101,105,173,117,12,192,241,57,232,248,249,88,4,140,166,223,6,99,62,67,253,166,240,215,231,150,178,3,176,53,75,
  98,56,173,73,193,76,93,205,36,78,94,92,3,0,232,31,77,88,231,221,223,189,117,112,24,71,79,120,87,108,142,235,227,147,56,185,209,123,47,251,
  121,247,231,183,14,141,224,232,208,52,170,35,212,113,253,75,29,225,154,206,105,77,243,108,249,141,128,209,105,44,173,171,182,206,215,197,153,245,172,186,
  184,202,191,253,115,203,146,90,80,198,208,61,144,192,201,139,171,1,10,212,29,79,96,65,141,186,246,204,230,84,249,22,212,76,162,101,137,250,109,223,
  72,194,250,206,126,94,125,102,24,158,112,14,210,117,213,204,118,61,115,92,223,55,154,192,153,205,243,140,54,194,208,98,212,249,130,24,195,243,237,195,
  96,172,52,91,179,150,47,168,194,153,70,189,245,141,38,173,99,149,231,41,180,44,86,32,183,235,224,40,170,163,25,218,125,83,109,170,124,99,73,235,
  243,130,26,102,221,103,65,205,20,158,223,207,210,246,137,186,106,154,250,173,1,235,45,141,169,223,155,199,125,163,73,180,247,77,185,126,109,66,147,244,
  158,55,205,9,180,22,187,130,147,222,210,168,147,6,38,223,116,244,196,20,222,57,52,108,73,145,134,39,146,14,9,147,253,179,117,124,194,123,204,69,
  122,17,189,249,12,243,158,225,174,39,161,175,207,62,137,212,238,184,144,131,255,129,190,9,135,20,105,100,82,213,65,247,192,36,222,235,25,245,253,238,
  248,72,2,19,9,239,3,28,215,15,79,99,196,168,207,238,193,41,235,187,163,195,9,235,188,251,243,241,145,105,76,76,115,76,115,233,188,126,50,25,
  170,44,239,245,140,89,199,35,147,170,126,143,143,76,89,231,85,158,184,67,194,100,255,60,54,37,65,25,69,223,40,199,240,84,18,160,192,209,193,105,
  12,79,42,105,198,59,221,195,128,164,0,81,146,164,97,35,95,135,226,147,120,167,123,212,146,48,13,219,242,59,60,201,49,149,116,74,47,70,166,184,
  113,189,146,48,13,79,166,234,82,221,107,204,146,42,13,27,249,235,29,82,215,113,94,26,181,209,209,225,105,188,99,212,219,161,248,148,117,108,74,115,
  204,60,31,27,73,96,42,145,161,221,247,140,219,202,55,101,125,238,29,154,182,238,99,222,51,93,159,24,153,18,169,223,26,18,38,247,239,1,224,208,
  192,180,163,78,253,193,137,235,249,116,46,36,1,128,17,253,146,117,154,5,13,213,216,213,201,1,9,89,58,96,218,254,219,62,108,255,109,159,239,119,
  219,62,255,97,75,210,179,235,64,55,158,216,125,28,239,31,30,13,117,223,107,47,104,194,181,23,172,2,0,60,251,94,28,223,123,174,219,250,237,182,
  207,157,106,221,247,129,231,186,241,194,190,1,244,12,76,150,86,194,36,120,86,176,228,150,48,189,117,176,23,219,247,244,25,234,47,130,255,220,117,204,
  247,55,215,156,183,28,127,189,69,169,89,158,223,55,136,231,247,13,226,205,131,195,14,9,211,79,31,239,68,239,208,20,122,7,21,152,4,221,203,111,
  210,218,208,52,191,40,18,166,254,182,4,30,126,229,104,122,245,30,53,140,190,77,187,22,75,132,31,193,233,205,243,64,57,5,97,192,35,187,142,163,
  103,112,10,61,6,132,255,232,213,227,190,99,242,182,255,185,198,146,38,61,240,226,17,236,238,30,67,53,35,14,9,211,247,94,60,130,247,143,140,91,
  147,252,143,94,59,238,155,181,23,111,223,96,29,127,237,177,131,232,25,156,70,207,137,169,226,244,161,223,157,192,246,223,157,176,62,127,233,99,75,173,
  227,71,222,252,0,239,31,153,72,3,37,206,180,245,167,135,82,125,232,220,37,150,148,168,111,44,137,71,222,138,227,181,174,209,148,164,39,77,218,223,
  55,133,173,63,235,182,234,121,219,85,205,56,115,149,186,215,174,67,125,120,226,189,19,46,53,92,186,100,170,232,220,78,46,181,29,83,197,77,68,18,
  134,74,78,155,209,234,84,174,176,4,37,236,176,133,78,209,202,227,82,36,173,86,200,61,81,91,19,229,92,57,44,180,82,18,130,11,8,189,69,185,
  130,146,161,86,214,35,83,133,143,137,38,28,107,24,214,105,246,164,178,192,251,14,155,116,225,216,208,20,198,51,24,93,175,93,86,139,186,26,149,245,
  222,19,147,120,228,213,35,152,74,8,116,30,31,71,223,200,116,106,69,190,167,31,187,15,141,168,149,243,200,52,214,44,173,197,146,186,42,12,140,37,
  208,63,50,141,177,41,94,100,149,92,110,19,185,93,37,119,124,100,218,87,213,230,78,199,135,167,45,149,89,247,192,164,165,50,179,171,228,70,38,147,
  152,78,228,150,167,145,41,94,20,149,92,247,224,20,38,166,51,149,79,26,212,41,85,32,79,34,0,68,1,2,188,219,61,10,74,149,203,1,63,85,
  155,111,123,59,158,170,223,99,195,9,140,79,113,76,51,226,81,201,133,81,181,153,170,58,0,161,159,95,168,100,87,201,101,82,157,165,75,74,213,167,
  250,224,161,248,84,104,41,149,111,221,218,236,148,142,141,36,49,158,147,49,188,169,158,99,42,94,156,230,225,10,92,192,39,128,36,83,33,41,160,67,
  166,232,84,174,237,148,59,152,190,44,128,169,117,121,173,77,37,55,132,222,12,42,141,173,151,156,236,80,181,61,181,251,184,71,213,6,192,161,2,116,
  171,238,126,246,198,81,188,127,120,180,68,70,223,217,37,167,74,110,24,71,67,168,120,150,46,168,178,126,211,63,154,192,1,3,10,156,70,226,17,140,
  228,184,3,176,174,154,21,71,37,55,150,192,59,85,105,182,23,155,59,165,236,44,96,70,72,33,192,233,171,230,35,26,81,240,228,103,204,237,219,222,
  150,198,44,245,219,174,174,17,244,14,77,123,84,114,207,183,15,41,99,238,68,250,123,153,198,224,230,239,76,251,166,82,36,167,209,55,203,185,237,42,
  99,242,148,74,174,189,111,50,247,190,188,164,218,166,146,27,67,239,80,46,162,34,195,218,219,140,61,38,4,52,53,85,232,100,148,4,192,18,0,49,
  199,19,109,211,164,83,57,36,67,155,33,189,35,147,78,197,25,13,160,157,85,22,185,65,3,160,148,234,49,182,34,147,41,117,208,33,194,43,30,154,
  56,7,164,185,50,209,42,58,157,202,99,110,1,247,174,150,245,80,84,160,180,101,195,98,108,217,176,4,128,196,225,248,40,150,214,69,193,165,64,111,
  124,194,90,197,3,94,191,67,166,31,165,163,67,211,216,190,167,15,221,3,78,105,210,25,39,213,225,247,215,213,99,98,154,167,245,195,244,246,161,17,
  220,191,179,27,61,131,83,24,28,79,224,196,184,23,212,182,156,222,136,250,121,81,76,39,69,90,63,76,15,191,116,4,241,209,4,250,109,126,124,14,
  244,77,224,111,126,118,0,0,48,145,16,184,229,226,102,124,246,44,229,87,235,225,151,143,160,123,96,18,227,211,2,171,151,212,224,122,67,146,119,116,
  120,26,255,247,101,195,160,155,0,93,253,19,152,152,118,14,136,203,23,86,225,198,63,92,137,137,41,225,235,135,105,229,130,26,8,8,236,238,25,197,
  171,251,135,241,193,224,116,224,59,248,212,25,141,104,156,31,197,84,82,164,245,195,180,99,239,9,60,181,103,0,125,35,9,244,14,77,99,116,146,99,
  221,178,152,227,94,127,121,193,10,140,77,115,112,145,222,15,211,3,191,57,138,131,253,83,24,159,230,56,208,63,153,113,119,154,78,217,36,35,116,10,
  162,106,66,181,2,184,234,84,89,208,100,216,50,49,0,36,106,64,147,94,5,233,52,83,176,20,60,198,104,96,42,80,82,126,166,234,32,165,64,109,
  68,98,85,67,4,60,193,49,47,162,58,190,233,47,169,174,127,210,255,184,102,18,47,25,42,43,123,170,175,141,134,246,195,116,224,248,132,5,39,65,
  121,12,229,135,201,71,117,55,54,197,177,199,178,135,34,158,235,77,117,80,93,117,196,225,99,233,165,142,33,236,113,236,120,116,102,46,22,165,30,63,
  76,45,75,230,129,130,162,115,112,12,39,215,43,96,234,31,75,224,157,154,0,213,157,49,190,174,92,84,29,202,15,211,35,175,247,57,118,211,249,165,
  108,252,48,189,127,100,220,169,138,211,99,125,97,161,9,68,141,84,73,67,53,167,153,169,50,161,201,48,77,84,253,39,170,59,146,78,51,0,75,92,
  217,201,6,12,50,179,210,232,187,227,152,237,250,225,169,80,134,182,71,135,82,126,160,14,245,79,88,207,40,148,209,183,242,51,53,98,72,152,198,48,
  48,198,32,18,220,230,91,40,105,73,77,28,199,134,241,180,233,83,137,11,57,227,126,152,194,24,135,251,25,115,3,192,200,84,210,233,99,201,199,56,
  60,157,31,38,202,40,38,147,20,148,81,28,30,154,194,216,164,132,128,192,225,19,9,76,78,75,181,21,153,166,242,246,238,97,35,31,132,86,188,31,
  38,119,170,92,163,111,119,34,106,168,98,211,182,221,85,229,147,174,59,183,1,107,26,82,18,204,7,94,139,163,107,96,26,119,94,190,2,95,127,250,
  72,224,239,212,247,189,0,128,213,141,85,184,254,188,197,142,239,15,196,167,208,21,159,198,230,117,117,142,243,59,218,135,177,163,125,196,63,47,231,55,
  98,77,99,202,75,253,3,175,246,161,171,127,26,119,254,209,42,207,181,15,188,218,135,205,235,22,224,193,87,148,173,231,234,197,213,216,188,110,1,118,
  180,15,227,250,243,150,120,243,210,63,133,205,235,83,54,159,59,218,134,176,163,109,24,215,125,108,9,214,216,22,80,15,188,124,28,93,253,185,184,214,
  16,202,102,208,242,54,16,193,150,223,107,80,222,237,109,77,91,185,215,24,4,0,108,249,189,122,227,156,250,124,211,31,174,192,246,247,6,177,255,120,
  56,59,188,109,127,210,226,57,183,245,39,7,108,158,255,213,38,145,251,158,83,239,105,203,105,245,216,114,154,241,204,119,7,177,253,189,1,108,57,77,
  69,45,216,254,222,128,113,141,243,115,224,179,255,116,45,182,254,120,63,0,21,105,96,203,233,13,184,111,103,143,250,188,44,134,45,167,55,98,251,187,
  113,108,189,168,41,149,143,29,61,214,239,183,156,222,136,45,167,55,170,103,237,137,99,251,187,113,85,7,155,155,208,186,76,217,17,110,219,209,141,253,
  199,38,130,243,240,103,235,176,245,63,218,61,231,215,46,171,197,214,139,155,173,249,246,190,103,187,177,101,195,98,227,89,253,234,249,27,26,83,121,217,
  224,108,187,219,247,244,99,251,111,251,131,159,107,139,200,177,245,223,247,166,206,127,254,84,152,47,123,235,143,246,58,242,179,229,140,37,184,239,87,135,
  28,247,81,209,67,100,160,203,162,240,201,216,88,196,13,120,103,188,124,129,41,91,163,239,214,101,181,54,175,225,17,252,127,87,168,198,241,236,123,113,
  92,124,90,234,37,118,30,79,121,10,183,31,247,141,36,80,91,109,120,138,46,144,209,247,246,61,253,86,67,2,18,74,255,41,72,184,185,192,149,210,
  249,97,250,220,249,43,172,235,126,250,134,233,87,41,184,190,242,246,195,20,120,223,121,33,37,76,222,38,22,232,135,201,84,191,48,179,105,26,254,90,
  192,85,44,57,34,128,8,83,149,102,192,193,239,173,136,129,82,10,198,24,30,217,213,105,73,142,54,158,52,31,95,250,248,10,75,194,100,249,85,154,
  240,223,221,231,246,244,93,142,126,152,220,169,114,141,190,131,250,74,84,237,170,226,54,127,77,214,206,244,153,83,133,174,105,168,194,209,209,36,142,143,
  38,1,72,156,247,161,90,140,77,11,156,182,162,38,240,55,215,108,90,132,61,71,38,112,205,166,122,60,242,246,32,198,166,5,246,28,153,192,210,249,
  17,108,88,17,195,142,142,17,28,31,73,98,105,157,234,63,123,142,216,119,206,6,239,80,93,211,88,131,163,35,9,235,154,243,78,174,195,216,212,48,
  78,91,25,195,35,111,13,58,219,124,21,115,192,85,93,21,197,154,198,106,236,170,162,88,90,23,197,142,182,97,219,51,19,88,90,167,212,246,123,122,
  39,140,126,92,133,13,43,99,222,103,174,174,195,216,148,192,241,145,233,80,245,119,205,71,22,227,145,55,250,82,208,36,40,144,76,0,148,227,232,192,
  56,118,127,0,108,60,105,1,32,41,118,247,140,89,139,208,141,205,243,176,220,8,189,179,177,121,30,118,119,143,161,117,73,13,94,170,9,31,163,110,
  183,177,16,186,246,252,165,120,248,21,53,30,46,95,88,133,143,183,46,192,238,238,212,98,115,203,105,245,56,58,148,192,242,133,85,234,55,82,93,183,
  241,164,249,150,89,132,213,159,92,159,253,210,181,191,191,28,187,187,71,113,237,239,47,199,195,47,29,69,93,13,67,235,210,152,99,60,106,93,26,195,
  75,213,12,203,23,86,97,251,187,6,140,157,222,96,29,31,29,154,118,44,158,77,136,26,157,82,230,27,0,240,241,117,139,48,58,201,113,116,200,127,
  76,58,243,164,58,223,243,163,147,28,71,78,76,99,227,201,117,216,190,71,129,216,242,69,174,114,218,194,71,1,176,158,9,137,180,27,149,174,189,96,
  21,118,219,28,76,95,123,193,42,60,252,194,97,92,251,63,92,231,255,71,19,30,126,161,7,203,23,85,227,234,115,150,99,133,235,121,27,63,180,0,
  91,206,92,140,221,7,135,243,4,37,56,199,19,195,81,101,217,2,83,5,202,151,181,218,32,151,196,40,82,251,16,12,239,207,102,48,86,98,14,130,
  230,150,115,21,135,74,106,35,209,57,148,136,1,212,18,150,147,75,102,244,181,36,159,49,104,58,48,48,141,29,237,163,232,26,152,54,36,78,245,152,
  87,69,241,222,145,96,240,220,176,34,134,175,63,125,4,119,94,190,2,143,96,16,199,71,146,120,228,173,65,172,110,172,194,216,180,176,224,102,243,186,
  58,236,24,25,49,36,74,153,203,119,32,62,137,29,237,35,150,132,231,186,243,23,99,94,21,193,123,189,19,120,228,205,184,227,218,213,139,171,112,192,
  22,123,114,100,90,224,64,124,10,35,211,2,175,30,28,245,92,191,121,253,2,236,104,155,182,64,106,243,250,5,88,90,23,85,207,108,27,70,151,33,
  93,188,238,247,151,96,94,53,5,70,194,213,223,134,85,181,120,228,13,159,241,83,8,236,62,52,136,221,135,71,112,116,40,1,16,138,237,191,75,213,
  195,242,5,85,234,60,76,187,199,49,116,244,77,98,36,11,9,169,9,73,27,155,231,225,225,151,143,89,210,158,249,213,204,146,42,173,93,26,179,164,
  74,71,135,166,177,253,189,65,0,18,91,126,175,193,200,195,180,75,155,145,25,20,55,54,207,199,214,31,239,199,182,63,93,139,135,1,140,76,114,135,
  187,147,145,41,245,121,100,138,227,55,29,67,120,248,165,35,150,212,9,6,48,237,254,96,4,203,223,173,114,192,202,218,101,49,108,127,55,110,73,149,
  110,218,220,132,249,53,12,24,10,144,82,127,224,255,146,142,14,77,225,209,55,142,97,116,42,105,9,2,142,158,112,149,211,6,69,219,79,76,217,4,
  6,233,155,234,198,147,23,56,165,74,159,59,21,15,227,176,58,255,163,189,214,143,183,125,254,84,60,12,96,126,53,195,138,69,213,14,77,148,2,211,
  106,79,62,114,135,37,211,185,180,112,14,55,178,12,129,41,31,149,220,123,135,71,241,218,129,33,203,15,147,221,189,192,224,88,2,245,243,212,10,164,
  58,74,45,233,66,49,84,114,169,100,236,250,200,103,0,78,227,135,233,145,215,142,90,199,213,81,138,213,141,49,44,158,31,197,145,19,83,24,155,18,
  24,119,249,52,42,148,31,38,119,122,196,38,157,74,10,137,245,203,106,49,205,37,170,24,193,227,187,251,49,62,205,113,98,44,137,94,159,149,77,118,
  126,152,204,176,25,246,184,99,204,114,104,185,231,240,8,34,180,26,32,78,85,219,145,161,105,60,252,242,81,0,4,131,227,9,44,153,31,197,135,87,
  212,98,42,33,12,227,108,225,80,203,249,169,228,180,31,166,28,250,114,81,84,114,238,246,96,151,90,69,1,154,80,35,25,79,206,200,248,181,166,161,
  10,187,170,169,237,115,53,118,85,143,7,74,152,158,250,210,106,124,242,161,46,0,192,215,159,238,197,83,95,106,193,39,31,236,116,72,121,82,210,216,
  236,134,232,53,141,53,216,85,53,102,251,92,141,93,85,12,167,173,140,121,165,170,105,36,76,246,243,169,188,68,93,159,171,108,207,28,117,229,33,92,
  164,134,167,110,92,143,79,222,223,150,254,162,100,18,141,181,64,148,153,146,69,130,107,207,95,106,0,207,113,75,66,116,237,249,75,179,150,48,89,146,
  22,155,123,16,83,186,227,254,60,234,154,151,76,149,219,181,31,91,158,149,132,233,197,59,206,196,5,255,252,14,0,96,235,143,247,227,197,59,206,196,
  205,63,222,31,40,97,50,207,239,63,54,129,151,170,79,96,219,53,173,216,250,72,135,239,179,90,151,213,226,165,246,161,192,207,97,37,76,74,115,192,
  44,213,158,41,97,186,246,227,43,177,124,81,21,30,126,177,215,35,97,10,155,236,176,4,0,103,26,27,106,204,191,0,240,226,255,58,199,58,222,127,
  108,220,146,64,57,234,255,183,125,88,190,168,58,199,124,24,99,165,228,134,95,63,115,61,206,12,23,112,254,209,57,102,189,74,110,215,129,33,71,200,
  147,23,246,13,4,136,1,155,172,223,20,67,37,151,130,165,68,222,210,165,116,126,152,30,121,53,5,76,119,94,189,214,186,238,255,190,212,139,61,221,
  163,158,16,35,133,242,195,228,1,166,215,82,192,116,205,185,203,112,205,57,203,0,0,207,183,157,192,147,239,244,59,243,65,220,3,111,38,63,76,44,
  205,100,105,54,91,85,201,103,52,47,64,20,17,128,57,253,48,29,29,154,182,86,140,32,192,139,127,125,134,117,151,175,253,172,203,99,244,237,86,201,
  105,63,76,57,246,229,98,170,228,188,34,73,27,52,77,1,168,2,100,50,229,187,171,132,233,186,115,27,48,54,165,158,187,163,99,20,7,250,167,11,
  118,239,205,173,117,216,176,66,77,154,233,236,151,172,188,156,183,4,99,198,34,100,71,219,8,14,24,59,111,237,118,76,95,127,242,48,14,196,167,176,
  163,125,4,215,157,191,4,59,218,135,113,225,186,5,120,224,213,62,212,85,81,172,94,92,141,59,63,213,100,72,173,166,240,224,203,125,150,84,105,131,
  1,95,123,122,39,176,163,109,8,27,86,198,112,221,199,150,166,202,191,111,200,33,185,10,74,119,94,113,18,254,230,231,221,225,42,65,0,9,36,148,
  27,2,154,253,180,229,107,171,244,159,157,57,189,15,183,29,211,92,74,166,122,206,109,179,180,101,67,163,181,131,59,147,253,146,227,189,124,238,84,220,
  252,239,251,156,231,62,127,42,110,254,209,94,220,251,249,83,139,84,10,67,253,38,141,99,41,140,48,61,174,75,80,166,192,84,25,137,3,72,204,200,
  96,93,57,85,40,140,57,48,221,182,98,243,60,53,254,95,187,18,211,201,128,39,42,1,110,108,10,40,97,55,60,96,168,226,198,166,149,65,213,158,
  35,19,24,75,136,64,149,220,123,71,38,177,193,146,62,73,199,117,166,90,204,76,199,71,146,216,115,100,194,176,143,74,111,191,164,224,70,221,107,89,
  93,20,199,70,18,42,47,83,2,239,245,78,96,143,43,63,99,83,2,7,226,147,88,221,88,101,229,191,171,127,10,171,23,87,227,221,222,9,116,26,
  106,189,227,35,9,235,239,158,94,155,116,191,119,12,199,71,18,214,51,199,166,164,173,30,210,167,13,171,106,177,231,240,56,246,28,30,15,39,5,29,
  154,50,38,56,115,147,76,2,203,23,70,177,209,182,96,57,58,148,64,71,181,191,74,110,183,77,18,236,151,236,146,98,83,29,230,254,108,170,218,142,
  14,77,99,247,7,99,150,253,84,182,42,185,119,62,24,197,198,147,230,59,62,167,83,201,249,157,15,122,86,199,177,113,140,76,37,3,63,123,243,18,
  12,223,35,147,220,161,205,57,122,98,90,253,27,154,242,168,195,118,31,26,177,213,79,56,21,217,198,147,23,96,247,161,97,203,110,233,157,67,35,198,
  185,17,236,62,52,108,109,210,82,121,73,122,84,114,169,231,135,149,172,155,6,143,6,36,1,106,190,166,198,220,35,204,121,220,184,76,150,41,48,149,
  203,46,185,117,203,107,113,122,211,124,36,133,68,231,241,9,180,44,139,1,224,134,106,47,37,94,183,171,250,222,236,26,198,161,254,113,12,79,36,10,
  230,163,50,108,104,148,7,158,63,140,58,67,252,188,184,46,138,143,175,175,199,89,171,23,224,80,255,36,186,226,19,232,181,237,132,3,128,53,203,98,
  56,189,105,62,166,184,196,161,190,73,156,108,108,157,63,49,150,192,162,121,41,113,187,253,243,59,31,140,224,248,240,180,195,39,147,59,169,176,44,134,
  138,109,96,50,99,168,19,187,74,110,100,146,227,227,173,11,113,214,201,117,56,49,149,64,148,70,209,176,176,22,0,117,212,243,224,120,18,245,181,17,
  0,18,131,227,73,188,121,112,24,61,131,9,188,219,59,97,241,85,58,149,216,205,63,57,96,29,175,91,22,195,153,205,243,48,149,148,120,243,224,136,
  218,169,231,82,201,157,179,186,14,167,175,154,167,218,66,255,164,229,74,32,149,15,117,252,47,191,58,140,190,145,4,166,146,162,232,126,152,54,54,207,
  179,164,74,157,253,147,14,149,220,186,165,53,56,125,101,45,146,66,226,205,15,198,50,170,214,174,61,55,181,11,106,124,154,91,42,185,225,9,142,115,
  78,158,143,83,151,197,48,56,206,241,102,247,24,250,70,147,129,245,186,124,65,4,91,62,188,200,170,15,187,159,172,121,85,12,23,182,46,192,57,39,
  207,71,103,124,10,7,250,167,209,115,98,186,72,181,99,72,29,35,36,21,48,179,68,105,77,67,21,30,217,125,194,6,36,170,13,4,169,228,78,91,
  81,131,111,93,190,194,247,59,63,149,220,241,209,164,97,211,148,185,109,173,105,172,193,35,111,197,113,205,89,13,120,226,224,152,5,59,167,173,140,225,
  235,79,30,246,121,30,243,168,225,234,170,40,230,87,81,143,13,211,210,186,168,3,216,76,227,239,53,141,53,120,228,205,126,236,57,172,62,223,249,233,
  102,229,134,164,55,24,134,190,245,233,230,204,170,56,199,179,13,213,83,82,0,148,99,249,2,130,63,255,232,98,92,123,94,42,0,245,195,175,30,15,
  84,201,153,182,74,97,36,197,153,84,114,219,223,27,84,6,219,231,47,247,85,191,101,82,201,157,121,210,124,220,123,210,90,231,123,72,99,244,237,119,
  62,232,89,197,86,201,153,18,164,107,47,168,195,181,31,95,137,135,127,211,107,129,219,195,47,30,206,170,223,220,251,185,83,112,193,63,190,158,202,203,
  201,117,184,247,243,167,224,130,127,216,101,124,94,96,203,75,4,173,203,107,189,117,189,168,58,11,96,50,236,28,37,129,21,122,201,132,38,80,132,89,
  101,85,132,74,174,58,50,150,241,25,203,23,86,135,82,201,181,24,141,241,217,247,250,241,7,31,94,168,0,171,111,18,45,141,243,32,192,65,41,195,
  193,254,41,235,186,233,164,196,224,216,164,82,197,20,104,144,14,27,26,165,203,6,86,215,156,183,28,87,108,82,147,224,243,251,6,49,149,20,232,29,
  156,114,170,228,98,17,203,231,209,243,251,6,241,7,167,40,177,242,33,151,31,38,251,231,68,82,128,11,153,22,152,84,88,22,187,138,109,52,195,224,
  151,82,201,213,197,24,62,212,88,3,16,224,131,193,105,48,202,176,118,105,29,88,21,67,87,223,36,90,150,168,122,238,236,83,190,148,4,4,14,30,
  153,196,209,161,73,244,12,38,112,102,211,124,203,48,60,93,104,20,251,142,151,107,63,182,204,82,191,77,39,5,120,55,60,161,81,236,126,152,158,221,
  123,2,23,159,186,200,2,21,19,158,58,251,39,241,189,23,143,226,253,163,165,241,195,116,102,211,60,92,107,108,247,126,118,223,144,75,37,23,65,203,
  226,234,84,155,28,79,102,0,166,148,72,253,217,182,97,75,37,183,160,134,89,247,233,236,159,194,209,145,132,1,162,254,247,89,177,32,138,107,207,81,
  59,83,59,13,201,72,139,49,241,46,168,97,214,241,179,109,195,152,74,142,20,17,152,144,26,244,72,249,185,30,112,39,211,134,9,144,120,234,75,45,
  5,191,255,215,159,60,108,168,224,6,44,176,241,75,234,187,56,174,57,171,17,95,127,178,39,212,189,31,121,179,31,215,156,189,24,215,156,189,24,143,
  188,217,159,117,222,158,186,97,125,86,176,228,77,73,240,169,4,126,248,74,15,126,244,250,9,0,212,178,105,154,45,201,180,97,2,148,77,83,168,5,
  211,73,243,113,237,239,175,176,236,151,102,50,61,252,98,111,94,191,127,241,239,62,234,128,37,171,94,254,225,245,226,102,92,64,169,117,153,207,249,144,
  203,50,157,114,77,18,42,136,36,184,142,128,146,75,50,192,130,18,21,254,130,73,150,71,176,30,237,228,78,39,59,56,149,71,186,211,37,73,74,231,
  151,41,40,217,109,152,128,112,118,76,128,242,179,244,233,211,22,89,118,139,110,95,76,126,18,39,51,217,109,152,0,101,199,100,218,40,1,202,7,211,
  230,245,11,45,191,76,217,216,48,133,182,91,74,51,233,37,5,71,36,201,144,114,220,36,172,247,126,211,31,172,112,24,104,111,251,117,111,104,191,76,
  128,242,119,180,253,189,1,108,251,159,107,0,168,13,36,219,158,235,197,232,20,199,150,211,234,177,237,127,182,24,210,213,42,108,127,47,229,139,105,219,
  159,170,235,119,127,48,150,209,7,147,231,153,199,140,103,254,105,107,234,153,59,123,80,87,205,176,118,89,12,219,174,105,181,206,153,201,238,135,233,232,
  137,105,108,127,55,142,109,59,186,113,245,71,150,226,90,195,165,202,246,61,113,116,28,79,175,246,220,246,103,235,28,159,183,254,71,27,214,46,171,197,
  77,155,155,177,124,97,53,110,186,184,25,247,61,155,254,157,217,109,152,76,41,84,144,29,147,233,131,201,225,139,233,71,123,113,243,143,246,57,252,48,
  221,108,248,97,90,187,172,22,55,125,226,100,149,151,75,78,182,124,49,109,57,99,9,182,156,177,216,40,255,84,56,95,76,150,80,131,229,212,244,200,
  117,119,220,41,31,124,187,28,184,137,228,57,247,229,227,78,63,224,119,196,112,102,229,169,96,98,123,230,84,202,217,85,46,147,56,41,64,62,179,190,
  140,228,249,72,146,71,245,218,63,24,128,196,40,82,59,224,2,26,51,33,62,75,2,26,238,193,249,212,113,46,117,73,114,172,123,146,101,61,147,92,
  223,39,9,127,49,41,100,191,37,185,183,45,18,182,14,147,106,231,75,34,25,178,254,66,214,133,148,105,128,102,190,97,103,196,109,227,2,112,205,198,
  69,94,233,204,219,39,112,205,166,69,120,228,237,19,214,181,215,108,170,183,220,8,44,173,51,252,48,25,64,180,97,69,141,103,135,155,178,71,154,240,
  213,68,108,94,87,167,242,98,168,226,54,175,91,128,61,71,38,176,121,221,2,31,73,81,60,245,204,149,49,203,93,192,210,186,168,231,250,227,35,9,
  227,158,210,146,88,153,198,223,75,235,162,41,149,153,1,76,230,181,105,135,235,244,39,28,105,195,74,37,1,85,54,79,18,27,140,141,5,123,122,39,
  1,80,108,252,208,124,128,50,44,95,16,115,132,121,2,148,83,203,76,118,69,215,158,191,204,216,85,171,210,242,133,85,62,174,4,148,59,128,51,79,
  154,103,101,247,157,238,81,236,54,108,146,76,9,181,121,206,81,54,91,241,76,223,75,238,207,203,23,86,97,203,233,13,41,0,120,119,192,56,103,64,
  209,208,148,229,131,201,148,56,157,105,216,66,189,115,104,212,114,45,176,229,244,70,155,250,44,110,148,93,250,86,247,181,31,95,233,121,15,15,255,166,
  23,203,23,86,91,142,41,85,216,174,126,11,136,118,31,82,207,217,120,114,157,117,35,187,166,6,82,153,186,152,215,185,31,122,237,5,94,39,170,15,
  191,160,224,221,190,19,238,225,23,20,28,46,95,84,109,56,168,180,129,145,84,126,152,204,157,117,239,28,26,78,239,143,73,10,5,215,73,131,19,152,
  29,160,2,108,30,93,125,254,243,27,69,37,1,19,79,181,9,146,45,65,134,5,38,225,90,193,10,99,87,156,200,29,76,230,50,48,69,163,70,189,
  210,204,32,172,129,73,3,83,168,58,148,42,238,92,34,81,50,96,10,164,1,89,164,107,67,49,70,30,215,166,189,190,64,16,148,37,48,57,47,145,
  254,47,179,138,65,133,84,161,25,174,13,186,119,150,239,88,22,233,218,92,218,131,44,84,219,145,89,53,147,192,235,101,62,247,150,89,118,5,25,240,
  165,205,86,73,26,243,180,48,62,231,8,76,179,64,37,231,134,20,191,74,18,1,96,195,10,152,15,35,40,159,4,64,42,92,255,102,78,36,178,88,
  6,204,12,136,50,228,167,17,214,187,227,116,210,73,39,219,124,48,205,129,42,164,160,73,167,57,154,132,235,216,216,4,194,133,49,183,113,219,87,217,
  205,113,145,146,53,230,92,36,19,102,97,29,147,164,116,2,76,32,144,22,16,150,164,205,101,186,16,150,211,196,217,31,61,221,144,220,72,99,15,165,
  121,76,137,10,79,144,138,59,81,184,231,49,211,241,32,205,162,221,228,218,126,116,154,155,227,165,54,40,156,179,208,148,224,74,197,79,52,48,205,217,
  54,224,72,70,48,93,251,144,32,165,181,97,46,219,20,41,125,1,178,133,37,219,246,63,7,64,25,210,30,110,124,102,180,56,210,7,41,148,231,110,
  187,180,165,146,124,45,49,106,52,38,158,134,210,11,156,40,11,241,142,164,179,209,235,253,9,58,133,110,95,68,111,194,152,203,243,37,5,116,196,164,
  185,154,76,65,10,51,34,110,8,64,24,225,180,236,102,29,57,182,143,200,204,183,110,18,48,81,139,212,74,145,114,231,247,150,144,137,167,145,58,248,
  25,107,103,155,55,158,39,244,149,243,164,18,192,68,196,253,110,72,1,202,111,179,133,161,217,136,203,77,59,37,6,45,93,210,41,124,115,163,208,196,
  164,147,78,115,141,148,97,104,131,108,130,21,129,130,142,5,37,0,38,251,142,50,153,6,108,204,235,146,169,130,114,27,244,80,187,222,209,30,251,197,
  103,242,149,194,53,191,102,11,77,201,212,125,56,175,64,94,178,217,39,113,238,173,31,46,11,7,138,196,152,192,168,48,154,27,205,226,135,84,131,146,
  78,57,166,242,247,197,164,83,49,134,182,48,210,107,157,42,51,113,155,233,76,26,168,34,134,16,160,60,85,114,246,252,218,236,142,28,59,217,56,192,
  73,138,12,73,128,176,193,4,37,41,157,226,53,75,8,194,50,72,42,194,74,53,76,82,77,86,190,104,87,202,226,49,137,249,142,24,0,146,139,33,
  166,134,37,157,116,210,41,219,33,67,143,27,115,43,137,212,59,55,67,157,100,92,79,81,175,240,165,188,128,137,56,120,68,101,154,91,188,4,145,38,
  70,146,105,221,238,183,107,139,115,32,194,144,191,145,183,13,148,120,6,31,34,179,159,146,156,219,104,165,105,1,71,10,247,174,35,0,184,177,125,147,
  152,70,222,218,184,64,167,210,52,111,157,230,104,162,185,44,146,117,154,221,176,100,64,132,44,190,68,185,180,18,38,194,156,129,49,77,141,91,70,202,
  243,17,159,153,82,38,198,66,244,30,153,158,74,193,13,123,41,97,228,73,58,24,79,167,92,216,152,25,187,85,4,188,187,226,244,96,166,147,78,58,
  21,99,254,20,54,245,191,78,149,13,74,134,10,142,24,199,60,164,208,36,143,121,189,196,173,202,176,75,161,34,229,83,202,50,208,202,161,20,210,189,
  147,202,176,134,15,212,233,217,28,78,138,68,234,152,23,194,176,89,47,205,213,43,166,254,118,101,22,37,19,56,253,102,69,53,64,233,84,216,49,134,
  48,128,9,109,247,61,103,199,48,189,171,182,178,147,185,251,205,248,103,186,197,65,50,196,244,102,211,174,228,32,144,42,125,171,34,204,176,255,22,174,
  137,180,144,228,233,103,248,199,225,48,10,227,133,246,49,52,199,65,41,109,74,170,247,33,12,91,53,134,20,56,35,129,194,58,24,213,73,39,6,176,
  132,218,78,44,53,53,205,45,193,131,180,73,29,244,34,172,98,23,69,230,60,238,231,161,61,12,82,228,136,29,51,135,225,57,24,92,249,195,38,183,
  169,229,120,154,154,17,182,75,244,32,90,178,151,44,160,60,163,11,211,151,21,113,121,107,48,35,71,155,46,12,116,210,169,16,41,10,80,105,91,129,
  234,52,119,134,29,161,36,140,26,152,42,59,241,210,247,235,18,183,40,153,218,41,39,4,138,179,237,151,102,160,74,13,75,197,111,78,194,5,198,240,
  17,98,113,151,71,102,189,5,92,167,66,38,195,134,14,84,115,248,156,28,135,244,56,95,241,243,204,12,244,235,200,202,249,18,215,109,74,250,195,77,
  33,115,36,36,36,231,16,68,217,22,112,41,32,29,147,100,110,207,34,132,128,81,153,210,196,129,248,222,137,27,210,14,49,87,194,38,144,156,191,204,
  173,9,27,161,8,76,239,14,140,83,21,3,83,0,194,8,140,44,173,54,149,162,39,74,25,24,165,160,140,26,62,84,138,84,17,164,226,94,98,25,
  149,139,148,244,103,217,60,64,138,36,18,60,9,36,225,26,111,138,184,40,196,108,123,204,12,72,224,100,241,126,72,40,5,163,20,140,77,41,175,239,
  51,158,255,50,148,112,202,89,148,87,159,44,10,206,145,16,220,144,50,201,16,131,76,254,229,90,62,95,226,255,13,0,178,136,9,104,235,169,33,233,
  0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00128_png = (const char*) resource_TalComponent_bmp00128_png;
const int TalComponent::bmp00128_pngSize = 81772;

// JUCER_RESOURCE: bmp00129_png, 5438, "E:/Code/tal/tal-reverb-3-juce/resources/bmp00129.png"
static const unsigned char resource_TalComponent_bmp00129_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,70,0,0,0,51,8,6,0,0,0,42,193,126,246,0,0,0,9,112,72,89,115,0,0,15,97,0,0,15,
97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,10,105,73,68,65,84,120,218,212,154,203,111,221,198,21,198,191,51,67,82,126,196,145,172,196,50,42,197,150,100,89,168,221,58,64,243,
232,186,73,129,22,240,34,109,150,69,55,5,26,160,203,62,22,169,157,180,139,252,19,93,117,213,77,223,113,80,180,40,186,15,106,167,65,224,244,225,72,206,195,168,228,198,70,236,218,150,101,217,137,238,229,
204,156,46,56,36,135,115,103,72,94,91,93,152,194,104,134,115,121,121,57,63,126,231,49,67,18,90,182,19,79,62,249,226,11,47,124,227,59,139,139,139,47,78,78,78,130,0,128,8,84,52,0,102,48,0,102,6,23,155,109,
27,24,195,108,107,24,99,152,77,209,103,216,64,107,13,99,12,140,49,108,219,108,140,129,214,186,170,181,214,172,141,129,177,109,163,53,116,113,60,107,173,161,108,255,173,91,183,62,94,185,112,225,47,171,
171,43,63,99,230,127,99,135,54,138,244,79,157,60,121,242,141,83,167,95,121,238,208,161,67,216,218,218,194,205,155,183,144,43,85,193,129,133,96,140,129,97,70,49,112,83,246,177,97,3,46,160,128,153,217,249,
172,168,11,128,204,134,125,176,204,64,121,254,162,13,0,12,6,24,198,158,203,20,228,115,173,85,158,231,138,87,87,87,62,253,243,159,254,248,218,163,123,246,254,226,210,250,154,249,191,128,249,238,75,47,189,
251,242,203,63,254,210,96,48,192,63,255,117,1,55,111,222,26,57,70,107,109,239,188,134,209,6,218,84,42,40,128,112,1,0,149,138,170,177,53,126,157,80,40,144,136,138,34,4,4,9,8,33,32,164,128,16,18,137,148,
16,82,34,73,18,16,141,92,178,97,54,119,181,214,195,245,245,53,121,230,247,191,251,222,238,52,251,3,0,253,32,128,164,223,241,236,179,95,126,237,213,159,252,244,91,131,225,16,127,123,235,109,220,189,119,
111,228,75,53,0,211,128,193,101,159,85,16,187,80,74,72,246,15,224,202,20,235,127,20,184,109,100,205,183,248,76,8,49,114,115,137,104,66,8,193,147,147,147,185,214,250,107,43,239,93,248,229,158,221,123,242,
233,169,41,154,158,154,226,141,205,219,15,14,230,7,63,252,209,27,199,142,31,223,245,206,59,231,49,28,14,131,95,178,126,0,134,107,48,198,232,194,159,148,102,197,12,54,5,0,195,46,132,166,138,172,142,70,
88,212,140,60,69,57,144,60,62,89,146,200,252,240,225,121,121,241,226,197,187,172,245,223,203,51,77,79,77,97,99,243,54,143,3,198,199,255,220,177,99,199,166,174,94,189,138,237,237,237,40,148,194,79,184,
80,172,147,229,82,69,165,98,106,101,177,163,34,227,58,105,174,149,198,198,249,172,84,97,195,87,21,231,139,250,5,162,221,19,89,38,190,120,226,196,215,1,100,0,38,108,201,150,230,23,210,165,249,5,186,95,
48,56,112,96,6,55,110,220,140,126,161,114,184,134,225,56,78,59,0,83,58,85,123,92,253,185,113,76,171,86,140,95,154,0,217,170,206,120,231,139,193,49,134,211,44,203,196,190,125,251,158,40,129,56,128,82,0,
189,225,140,128,81,90,33,207,85,20,10,143,220,109,174,84,208,216,183,102,84,29,63,242,253,48,36,119,191,4,226,130,47,161,69,174,143,210,52,165,153,153,153,57,7,72,102,161,184,112,196,216,96,180,82,109,
63,92,153,72,49,6,83,93,116,237,59,202,11,247,32,152,0,20,123,206,210,245,212,240,17,86,22,106,83,139,109,105,150,137,36,73,201,129,226,194,41,21,212,9,103,84,49,93,96,156,11,175,239,172,219,143,96,127,
51,78,59,231,171,224,212,202,66,29,183,170,16,95,158,27,30,88,127,203,210,148,146,36,33,207,132,50,15,78,10,32,105,51,171,196,239,200,149,194,141,255,94,195,198,198,134,55,8,56,14,151,107,199,26,51,143,
50,73,115,79,48,234,45,189,104,100,163,78,225,72,1,34,8,18,54,191,33,16,9,8,65,69,142,35,196,72,116,218,220,220,196,226,226,60,165,105,5,134,34,249,90,201,29,75,243,11,249,165,245,53,238,4,163,148,194,
245,235,215,112,253,218,53,60,76,219,220,220,28,150,142,44,34,77,83,225,40,134,60,24,126,205,0,12,0,213,13,38,87,184,124,249,50,214,215,214,30,42,48,251,247,239,199,236,236,44,210,218,148,210,88,96,117,
128,24,0,102,105,126,193,248,89,114,192,148,114,44,47,47,227,241,199,30,123,168,192,204,206,206,22,206,183,233,99,98,48,216,171,141,111,82,62,24,82,185,194,242,242,50,180,82,15,21,152,36,73,160,148,70,
154,166,144,82,194,42,134,35,197,216,177,151,181,246,77,170,1,70,8,33,114,149,99,123,144,7,115,25,173,53,148,82,208,74,65,105,13,173,21,236,114,64,115,238,84,133,103,227,133,100,118,179,212,106,249,194,
157,64,18,89,231,90,78,36,237,100,82,10,9,41,37,100,146,64,6,39,148,121,161,152,36,113,77,201,120,48,180,7,194,56,253,102,105,126,65,151,170,169,194,245,210,252,130,32,64,228,121,110,231,56,193,148,187,
24,140,173,169,156,224,21,173,42,210,216,67,234,11,183,117,115,190,83,67,169,224,0,213,185,202,99,168,154,129,123,17,43,170,156,148,172,98,146,50,44,123,69,58,181,180,12,202,90,132,242,24,1,34,145,231,
121,49,233,139,129,169,6,225,214,205,80,91,135,225,209,240,91,21,31,74,165,26,114,160,208,232,228,177,117,34,9,36,137,164,68,74,242,64,200,0,144,96,41,115,155,196,170,133,0,8,34,18,121,174,16,75,44,171,
245,18,99,96,202,65,152,250,66,217,214,2,128,33,178,233,89,1,134,1,16,55,83,24,170,148,228,44,47,32,0,68,212,109,209,2,5,0,164,76,176,190,190,254,153,7,67,71,64,136,64,77,0,88,56,9,144,32,128,238,220,
185,211,53,131,45,46,84,8,199,47,136,192,93,21,13,181,184,166,33,196,232,221,175,139,104,168,139,220,4,79,136,234,119,67,219,193,131,51,144,137,68,150,101,255,137,168,195,7,225,154,144,112,192,160,1,230,
240,220,19,231,86,87,222,219,84,42,143,207,33,44,136,202,65,218,139,46,218,194,1,102,213,85,238,195,219,119,29,174,16,246,28,78,134,91,157,143,170,223,44,28,115,92,49,75,71,142,176,148,146,206,188,254,
250,71,17,53,132,32,132,250,43,48,229,65,116,238,236,217,159,15,135,67,12,135,131,168,98,132,19,61,202,193,86,197,135,70,206,49,158,50,132,53,13,81,165,254,182,79,56,96,169,25,161,136,194,115,191,227,
199,62,143,153,131,51,248,235,155,111,126,118,253,147,79,222,142,12,216,133,17,170,9,0,45,205,47,144,4,128,233,169,169,210,22,147,173,173,59,239,111,108,108,124,117,97,241,200,1,102,3,41,71,215,89,171,
125,102,84,14,196,93,156,36,52,35,139,231,160,69,181,182,91,43,166,156,3,53,65,75,72,33,32,164,13,213,82,142,44,109,166,105,138,103,158,126,10,203,203,71,177,178,178,130,87,78,159,58,147,8,249,97,32,28,
235,192,190,14,236,107,0,186,156,97,86,229,209,71,246,109,125,244,193,251,223,102,230,95,125,229,249,231,143,239,221,171,145,166,25,146,68,58,114,46,6,34,165,40,242,151,204,230,45,168,39,145,161,85,220,
202,175,0,163,185,75,99,33,156,32,132,116,22,196,5,164,76,172,98,10,242,123,247,238,193,129,3,143,99,110,118,14,89,150,226,220,217,179,120,245,244,169,51,172,244,121,119,60,61,10,66,253,100,193,164,206,
50,96,85,52,225,251,79,61,253,204,55,247,79,79,127,238,232,209,101,155,88,73,72,105,147,44,41,33,147,242,110,38,213,93,149,82,56,109,105,147,51,123,231,203,182,144,54,113,243,219,205,82,42,171,246,109,
181,57,94,185,114,5,171,43,43,120,247,252,249,15,127,251,155,95,191,245,232,35,251,62,0,48,4,48,240,202,48,176,191,29,57,118,0,96,88,130,201,188,53,210,70,217,30,12,166,239,125,122,239,11,32,74,9,148,
18,33,37,80,138,186,206,8,148,216,58,5,80,127,230,183,155,251,137,211,151,0,32,48,148,77,99,21,128,156,193,57,24,185,237,107,236,11,33,238,237,217,189,251,138,55,192,29,1,147,120,83,241,208,44,148,119,
77,76,220,221,53,49,241,15,11,111,151,191,208,236,129,204,58,218,89,100,101,77,90,25,107,11,97,232,149,65,164,246,175,159,3,227,225,192,49,104,25,59,132,51,163,244,39,90,8,244,199,142,139,77,212,56,226,
4,117,169,8,91,6,246,14,110,59,3,86,182,248,142,211,63,167,255,123,60,230,88,130,253,73,128,40,119,20,19,90,211,136,20,29,1,34,28,199,199,182,191,116,124,238,241,185,99,86,218,131,229,255,70,108,89,129,
3,16,59,65,38,129,245,138,216,201,204,3,130,16,158,231,119,87,207,164,7,198,120,32,74,64,58,80,66,225,56,84,184,39,28,115,105,125,173,82,140,105,89,204,233,3,193,4,64,200,0,20,87,210,198,153,195,248,10,
226,128,201,41,107,98,185,3,204,55,183,16,48,211,179,184,160,26,138,185,31,8,49,24,210,14,192,207,23,56,2,69,180,28,163,2,202,201,35,128,98,144,250,170,171,252,221,226,217,245,198,230,109,76,79,77,137,
142,121,69,159,244,26,1,24,136,56,191,152,67,142,65,136,21,213,241,153,242,84,215,85,235,141,205,219,141,21,60,221,162,14,223,84,100,64,45,185,7,10,17,5,232,128,90,68,0,160,239,107,116,96,32,125,20,212,
71,73,85,41,163,116,226,93,76,236,11,33,83,161,128,67,13,41,197,68,6,42,3,78,57,164,170,190,138,82,61,224,40,15,116,200,23,53,95,3,177,230,132,128,169,144,103,42,49,147,161,142,199,21,109,38,20,26,184,
234,105,46,190,99,238,99,102,161,243,171,75,235,107,58,246,148,192,4,66,108,204,167,80,203,19,190,182,59,30,90,32,162,142,103,63,125,225,13,35,230,214,101,94,101,59,252,226,144,85,13,143,49,27,245,223,
127,10,165,218,161,149,122,19,24,168,110,113,138,33,85,248,225,219,87,69,236,179,160,25,186,106,9,190,81,181,177,121,155,173,73,245,1,66,29,243,143,88,114,216,101,70,121,71,116,82,1,0,177,50,140,124,207,
85,152,242,95,71,75,34,126,161,188,208,62,126,36,230,67,76,192,124,100,196,233,250,206,23,129,249,86,95,152,227,132,248,210,183,112,231,59,120,142,73,153,0,144,182,183,145,56,18,85,184,135,90,250,14,180,
75,41,195,14,167,60,12,152,144,233,245,114,98,192,223,96,135,161,152,158,209,168,79,162,215,5,103,216,6,200,247,43,232,57,216,234,9,101,181,152,52,90,18,175,29,123,226,231,175,216,223,79,116,27,39,204,
183,69,171,78,40,189,192,56,112,146,49,160,180,61,211,161,72,98,71,99,128,49,61,192,68,243,160,46,40,189,193,56,79,43,67,48,210,30,80,100,75,78,212,229,212,251,248,41,213,97,146,85,221,247,109,241,222,
96,28,56,178,5,138,236,48,163,88,86,141,72,84,138,101,204,198,123,117,163,15,156,96,244,217,17,48,1,211,234,251,38,129,108,129,50,206,76,156,35,203,7,109,96,84,31,211,217,17,48,14,32,57,134,9,197,252,
75,76,49,232,177,130,216,229,136,245,56,42,217,49,48,158,121,201,49,192,132,214,111,208,145,57,143,19,250,239,27,200,142,129,241,0,245,121,155,64,180,76,55,186,124,12,183,128,49,59,1,100,199,193,4,32,
197,178,102,234,121,13,220,162,162,70,189,83,48,220,237,127,3,0,196,86,110,19,77,74,139,1,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00129_png = (const char*) resource_TalComponent_bmp00129_png;
const int TalComponent::bmp00129_pngSize = 5438;

// JUCER_RESOURCE: bmp00130_png, 5374, "E:/Code/tal/tal-reverb-3-juce/resources/bmp00130.png"
static const unsigned char resource_TalComponent_bmp00130_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,70,0,0,0,51,8,6,0,0,0,42,193,126,246,0,0,0,9,112,72,89,115,0,0,15,97,0,0,15,
97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,10,41,73,68,65,84,120,218,212,154,77,108,29,87,21,199,255,231,222,121,207,95,81,242,98,55,77,148,56,216,200,105,17,105,43,117,65,
164,208,32,19,62,90,171,148,136,74,108,88,178,105,186,15,11,42,64,42,91,86,44,16,5,196,130,138,37,168,124,136,5,144,6,149,69,21,26,210,164,93,80,17,67,140,236,132,64,226,210,216,177,155,196,126,115,239,
61,44,222,204,188,51,247,221,59,51,118,82,68,71,58,154,175,55,243,238,249,205,255,156,123,238,157,33,212,47,29,0,143,227,255,116,33,162,183,153,121,245,190,223,183,226,220,244,115,207,157,252,238,39,142,
28,121,118,110,110,14,68,84,178,124,97,102,112,111,129,115,14,217,154,157,115,3,107,107,29,91,107,97,173,133,117,54,223,230,204,122,219,198,178,177,38,91,91,182,198,160,191,111,216,24,203,198,152,194,
222,126,235,98,123,121,121,249,183,103,94,61,253,29,102,126,235,3,5,115,232,208,161,175,158,58,245,181,159,60,249,212,28,236,234,10,222,63,127,30,119,23,46,151,46,98,6,216,90,192,57,56,118,128,181,96,
199,96,118,96,231,196,54,51,179,203,1,2,12,48,228,54,152,197,141,57,223,32,48,136,0,82,128,34,144,82,0,136,153,25,12,134,213,218,221,222,213,73,223,219,183,111,115,19,228,230,231,47,37,191,249,245,175,
78,237,28,29,251,233,194,210,162,253,32,192,28,127,229,23,191,124,237,240,225,195,88,251,195,25,172,158,62,61,248,11,102,56,107,193,194,208,83,6,216,185,158,171,153,122,80,0,233,185,204,131,177,0,2,245,
156,39,2,148,2,73,211,26,164,53,148,214,80,73,2,80,185,201,46,73,248,218,199,62,254,254,202,196,68,247,202,210,146,122,233,251,223,251,202,244,228,193,63,2,176,247,2,72,251,7,190,254,194,11,175,29,63,
254,217,206,202,43,63,199,250,235,175,7,47,98,151,43,164,15,162,128,146,237,51,51,216,49,144,169,69,66,237,91,31,21,197,147,72,15,94,110,74,149,79,59,71,187,222,93,30,114,35,35,60,116,112,210,36,73,107,
246,63,55,110,188,12,128,198,59,29,26,239,116,176,114,107,149,183,10,70,121,251,207,62,253,244,23,166,55,22,46,227,206,133,11,97,40,204,61,32,217,26,206,245,1,137,227,61,96,14,112,92,40,168,56,95,88,22,
114,18,170,184,95,204,66,203,222,203,127,31,29,38,82,79,60,113,108,239,149,107,215,190,4,96,40,179,246,204,212,180,190,39,48,207,60,243,197,199,71,71,71,113,251,236,217,248,21,17,16,133,10,196,118,15,
160,235,131,44,169,37,55,161,48,255,156,227,32,172,160,35,198,208,248,245,235,67,187,58,29,154,61,126,252,17,9,38,131,211,154,153,154,166,109,129,217,187,111,47,140,49,232,206,95,66,68,46,5,132,226,169,
139,112,98,239,56,60,103,75,79,94,134,147,252,77,6,164,80,84,224,158,224,112,100,140,45,223,104,183,219,45,181,99,108,236,81,1,69,2,106,12,71,149,197,192,48,38,158,175,100,35,81,72,191,236,52,92,8,74,
207,121,242,44,239,97,242,243,165,100,45,33,136,227,197,255,135,156,81,10,237,86,75,237,121,240,193,157,2,74,219,183,153,169,105,181,69,48,14,198,26,84,144,41,245,56,185,99,37,85,112,64,41,249,147,207,
82,45,103,144,139,243,240,146,179,216,231,64,136,34,18,78,68,132,86,171,77,90,107,157,65,105,9,32,45,105,117,112,6,193,152,8,24,70,36,71,148,159,38,124,167,243,167,27,186,14,125,37,245,33,149,127,75,222,
61,88,254,102,0,140,162,86,187,69,90,107,149,1,24,18,48,124,56,73,85,88,37,33,48,103,134,135,163,161,20,114,120,160,59,14,108,179,44,14,67,69,148,172,79,196,54,101,221,117,105,219,171,190,243,165,187,
118,11,79,182,218,42,3,211,22,127,65,225,71,13,158,153,154,54,11,75,139,92,15,38,53,248,199,230,230,253,26,200,108,239,186,24,232,138,101,118,118,22,251,15,28,64,171,213,34,173,148,15,70,62,27,25,209,
46,91,155,26,48,22,169,73,113,245,234,85,124,216,150,205,205,77,16,17,181,90,165,80,10,170,196,7,51,51,53,237,22,150,22,93,20,140,181,61,197,156,56,113,226,67,7,102,98,98,2,138,8,173,86,66,170,31,74,62,
16,39,214,190,117,227,138,177,150,82,147,226,163,67,67,209,30,137,3,73,146,69,105,95,21,6,28,43,253,189,252,66,222,112,96,32,191,120,231,0,32,89,93,197,200,250,58,90,7,14,34,11,165,196,3,18,50,155,13,
139,244,204,212,180,150,99,171,2,204,204,212,52,89,103,41,77,13,246,191,241,70,184,232,53,6,206,24,176,49,189,65,164,49,189,250,69,140,155,100,13,194,94,239,83,74,168,18,66,238,176,82,0,41,80,54,154,38,
165,122,131,74,49,144,36,173,65,73,210,27,80,250,121,97,106,10,201,236,167,73,105,77,89,40,113,13,144,68,238,103,33,197,190,98,180,181,78,153,52,173,154,20,2,17,129,133,99,240,123,10,230,226,124,209,3,
49,151,158,46,11,40,236,245,50,68,225,251,18,170,123,164,188,125,73,146,144,86,138,132,211,33,40,137,84,139,48,147,55,47,201,213,2,64,89,107,169,91,1,198,111,44,11,153,115,190,79,36,234,51,238,157,139,
205,115,228,78,138,117,201,100,8,41,239,119,97,48,148,104,77,170,31,74,193,176,241,76,121,107,39,11,188,2,76,154,166,112,1,153,74,197,20,243,38,2,16,149,156,204,238,72,4,70,217,105,246,166,18,16,176,208,
127,248,199,130,115,40,35,163,156,36,9,222,124,243,252,205,12,140,174,49,9,69,1,80,121,209,167,68,5,172,174,44,45,173,165,105,138,181,7,246,68,21,83,76,32,229,57,33,111,104,9,142,26,112,190,52,159,82,
7,68,94,43,126,79,226,191,130,131,200,199,30,131,214,26,27,119,55,110,214,168,66,85,108,211,128,98,174,93,189,250,187,191,205,207,227,95,147,7,97,35,170,41,205,176,81,102,89,131,185,152,129,35,16,169,
98,90,178,10,0,13,0,81,37,136,36,30,6,114,80,129,101,232,192,1,116,142,30,197,252,252,60,253,245,157,119,206,133,212,16,48,242,182,243,253,66,58,4,128,246,238,217,243,207,139,23,47,252,236,182,214,88,
60,244,80,16,142,108,40,169,114,15,82,60,81,121,142,6,33,34,55,18,219,226,26,40,239,119,254,253,189,165,53,49,129,201,147,39,89,107,141,31,254,224,165,191,140,12,15,175,214,192,160,154,125,232,241,78,
39,207,224,9,128,228,221,229,229,243,239,173,220,252,204,254,71,30,221,179,190,123,28,35,155,27,104,123,67,132,126,227,56,94,127,128,250,57,180,80,3,60,53,120,243,187,33,203,187,232,172,187,246,195,168,
115,244,147,248,200,243,207,163,181,107,23,125,251,197,23,255,125,238,236,159,94,214,74,117,69,194,181,222,118,104,191,116,108,229,214,170,165,108,248,45,39,116,134,110,223,185,243,192,228,212,212,143,
62,255,212,220,145,29,59,118,96,196,164,216,185,190,142,225,110,23,42,119,74,160,46,124,207,97,160,31,30,138,114,64,170,191,159,41,73,101,10,80,66,89,74,137,181,238,239,147,210,25,204,222,253,135,15,78,
98,236,161,135,145,140,141,97,109,109,13,223,250,230,55,174,95,56,247,231,31,39,73,178,6,96,211,179,110,224,88,213,241,52,8,38,219,31,190,118,227,250,151,143,29,251,212,231,58,187,119,63,188,123,124,124,
159,86,26,90,107,40,173,160,149,130,82,26,58,107,188,210,58,59,214,223,87,74,149,143,73,203,10,185,254,49,31,150,7,205,203,71,68,132,165,197,197,245,165,197,197,229,87,79,255,254,146,51,246,34,128,13,
207,217,238,22,33,229,215,119,43,193,52,60,30,59,86,181,29,154,93,203,43,213,52,111,156,231,72,55,224,88,55,226,216,118,192,200,243,221,36,50,2,245,95,1,113,197,240,61,52,106,229,216,72,214,43,184,108,
86,109,146,56,159,31,179,226,183,242,250,42,11,181,149,43,166,29,16,243,63,169,184,57,111,195,121,223,129,80,57,238,3,65,118,78,137,107,108,166,28,227,65,178,30,176,24,56,215,192,135,152,79,88,88,90,228,
164,161,211,46,242,199,174,102,228,42,33,228,99,17,37,128,176,128,162,196,253,173,128,98,4,164,80,47,226,42,64,185,72,251,107,149,167,178,209,100,76,242,91,129,16,234,2,165,165,194,100,14,217,200,236,
174,216,222,20,191,233,122,112,76,0,146,13,180,163,10,150,171,9,211,98,116,237,182,112,35,219,0,134,9,20,80,165,25,140,204,242,146,157,2,42,178,158,114,82,15,78,90,243,32,108,164,221,33,223,252,125,36,
94,99,66,161,208,84,17,38,82,97,202,25,8,223,105,85,1,198,85,132,84,218,80,69,54,144,163,124,72,54,16,106,3,96,154,128,112,129,156,17,43,187,201,83,9,7,242,142,252,61,123,138,242,29,148,64,186,158,146,
252,100,189,157,112,179,249,68,149,6,128,149,91,171,24,239,116,16,27,138,123,3,46,10,56,79,53,175,42,66,243,174,182,34,100,252,117,19,51,219,88,15,28,203,191,140,72,188,39,26,147,162,142,132,75,76,29,
28,128,97,197,100,81,72,45,20,184,198,121,42,13,133,85,55,2,210,4,20,228,135,156,220,46,189,41,40,62,143,200,84,67,53,35,208,42,133,80,69,161,200,1,72,161,80,49,145,100,27,83,137,31,78,221,200,53,166,
38,244,82,0,169,252,142,198,175,124,165,42,244,22,20,130,200,203,172,80,149,171,61,181,168,72,175,229,223,195,207,29,85,225,214,173,9,49,255,62,214,127,175,84,250,160,70,228,26,191,193,42,160,142,170,
215,140,126,117,233,2,33,229,2,146,14,197,127,157,117,27,228,157,42,21,149,114,75,244,83,179,149,91,171,156,133,20,34,97,67,222,220,54,123,61,10,55,80,79,12,74,44,156,182,99,166,2,90,233,222,190,90,66,
161,148,47,166,65,143,195,145,100,89,21,62,218,171,93,84,224,222,177,225,72,172,23,171,202,75,221,58,21,197,62,96,12,126,155,150,133,20,71,128,80,205,104,188,201,136,218,119,46,230,104,85,40,116,27,134,
87,55,162,30,147,133,16,26,131,17,33,197,13,62,150,230,134,80,92,77,213,108,107,106,140,244,62,228,156,210,249,208,231,31,117,206,66,188,186,85,254,215,72,242,227,27,177,246,45,127,5,170,188,48,138,149,
0,117,95,37,216,72,152,218,192,52,69,45,212,42,40,141,192,8,56,201,22,160,36,117,47,182,42,242,87,44,119,113,197,80,165,42,12,7,122,163,58,40,141,193,136,215,184,137,7,168,9,20,29,120,161,85,85,195,84,
37,96,87,83,161,215,229,40,219,4,202,150,192,8,64,186,65,248,132,192,132,122,163,38,138,225,154,169,15,211,0,142,217,234,231,243,219,250,22,44,11,45,93,3,70,85,116,209,170,102,224,201,53,170,137,13,41,
66,249,166,177,74,238,25,140,167,30,93,161,150,88,110,217,174,98,184,225,36,89,176,204,255,159,129,9,0,210,219,0,83,165,24,52,152,77,28,176,123,1,114,95,193,120,33,166,42,192,80,131,174,186,105,158,25,
80,206,118,66,38,182,252,119,0,0,162,76,176,89,175,232,11,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00130_png = (const char*) resource_TalComponent_bmp00130_png;
const int TalComponent::bmp00130_pngSize = 5374;

// JUCER_RESOURCE: bmp00132_png, 4303, "E:/Code/tal/tal-reverb-3-juce/resources/bmp00132.png"
static const unsigned char resource_TalComponent_bmp00132_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,34,0,0,0,38,8,6,0,0,0,161,214,73,212,0,0,0,9,112,72,89,115,0,0,15,97,0,0,15,
97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,5,250,73,68,65,84,120,218,212,151,219,142,92,71,21,134,191,181,86,117,247,204,120,12,141,61,56,177,226,72,182,18,7,135,27,27,225,
72,225,138,112,67,204,21,190,225,62,60,2,143,144,39,128,71,8,15,128,146,8,238,227,92,147,11,115,16,68,137,15,227,177,35,50,24,247,140,156,209,120,186,123,215,90,92,84,237,238,221,115,240,65,34,78,40,169,
212,123,215,174,195,95,171,254,245,215,223,66,167,156,92,91,123,231,194,133,215,127,122,234,212,169,179,170,74,169,134,170,16,17,100,119,60,103,220,29,207,78,246,242,156,59,109,238,121,161,95,174,109,
94,219,178,59,187,187,187,219,163,209,131,143,71,163,209,123,192,54,128,84,12,195,31,95,126,227,163,55,223,252,201,165,149,149,21,246,246,246,136,136,2,196,20,83,67,68,102,11,230,156,235,115,46,11,229,
92,64,229,250,173,243,92,250,228,25,160,156,51,17,144,155,134,123,95,220,219,254,252,243,207,126,6,92,23,128,215,126,112,225,253,159,191,125,229,234,100,60,230,238,198,58,187,187,187,124,221,197,44,17,
30,220,189,187,177,126,227,230,141,31,25,112,233,237,43,191,248,93,74,137,219,183,110,176,183,183,199,243,40,17,142,154,113,252,248,119,134,15,191,122,184,169,231,207,191,118,117,101,101,133,209,232,1,
147,201,132,231,89,34,28,75,137,181,181,239,255,82,191,59,28,94,116,119,190,122,248,144,231,93,220,29,51,99,48,24,92,210,149,229,149,97,206,153,32,248,38,138,153,177,52,88,26,38,143,194,238,163,112,236,
236,236,176,116,243,230,124,160,42,150,140,100,134,213,20,23,17,130,192,61,102,153,212,228,121,134,0,236,189,242,10,171,171,171,7,230,79,102,168,41,201,221,113,119,34,14,71,210,52,13,103,182,183,1,16,
17,250,253,30,75,131,62,131,126,159,126,191,71,95,21,51,193,3,166,158,153,52,13,227,201,132,241,120,194,120,50,33,103,7,224,211,166,57,50,34,166,70,138,170,11,60,197,209,8,160,34,136,8,162,130,153,146,
146,209,235,165,170,19,142,170,204,251,136,60,249,104,82,66,77,81,143,192,59,33,124,44,203,219,90,31,60,202,113,180,32,34,130,136,249,150,158,102,78,51,67,103,17,121,204,209,164,148,184,55,28,206,34,98,
102,152,41,102,70,194,80,15,100,154,33,2,15,37,91,162,233,11,217,122,52,131,165,217,188,41,165,163,129,204,56,242,152,163,89,93,93,133,139,23,23,218,114,173,79,82,29,235,206,243,152,136,152,42,234,30,
53,34,124,51,233,155,18,77,206,232,104,235,193,117,207,249,169,136,245,191,46,189,94,15,51,67,224,154,254,231,254,253,191,140,199,227,35,207,240,235,44,39,78,174,97,102,220,186,117,115,93,119,118,118,
222,187,247,197,189,117,17,101,48,88,126,126,32,78,156,228,212,169,23,248,247,230,38,27,27,27,239,38,128,79,255,249,143,95,31,95,61,254,209,218,218,26,203,203,199,104,242,148,240,168,90,0,136,32,200,161,
239,72,17,186,133,119,58,58,34,243,220,143,8,204,140,225,240,123,12,79,156,96,107,52,226,147,79,254,252,46,176,222,37,198,165,87,95,61,255,219,147,39,215,222,154,185,51,43,191,54,115,106,243,182,246,221,
102,239,135,181,149,212,84,81,60,124,193,56,109,126,249,229,250,223,255,246,215,223,108,110,110,126,0,208,37,198,245,173,237,173,223,191,240,226,139,235,207,98,21,155,102,250,12,86,177,140,41,86,113,244,
241,104,244,224,90,87,181,159,104,21,85,203,174,190,53,86,81,1,137,232,252,70,151,2,4,66,0,46,130,3,33,114,228,13,182,223,42,38,224,210,229,203,111,92,157,78,38,220,190,117,227,128,75,83,192,194,73,30,
36,2,171,0,52,10,8,217,119,23,121,5,146,17,178,8,141,40,141,8,190,15,84,206,13,102,137,51,103,94,62,59,218,26,189,147,142,178,138,10,36,119,250,225,164,8,82,120,5,226,24,53,42,204,193,196,12,136,224,81,
128,52,34,52,20,32,83,85,166,162,228,14,160,174,85,76,135,89,197,20,65,191,130,232,181,149,10,132,10,68,14,63,26,15,193,17,26,132,28,194,20,165,65,153,228,32,105,48,17,101,162,90,64,187,211,235,21,171,
152,246,91,197,20,193,192,157,126,100,250,81,193,80,129,136,147,112,76,2,107,129,200,60,34,30,148,104,132,210,32,52,161,36,130,105,123,156,30,136,148,117,90,48,102,198,210,210,62,171,168,17,244,188,68,
160,5,145,31,237,112,101,165,143,138,99,117,175,26,13,226,13,68,70,240,106,60,132,178,156,17,98,184,244,112,73,148,81,202,31,119,39,244,151,87,235,145,8,225,5,76,178,162,61,11,86,209,162,16,50,181,188,
192,241,220,240,171,211,47,17,52,120,52,120,179,135,55,123,228,60,198,125,74,196,20,188,114,69,12,213,62,102,3,212,150,208,222,18,98,125,148,196,159,62,91,39,225,100,164,242,76,208,170,180,7,172,162,82,
162,162,139,251,3,156,40,246,24,39,227,146,113,205,100,105,200,76,9,207,88,40,138,128,24,82,29,75,97,75,25,175,237,252,4,26,160,82,192,183,86,49,205,173,98,44,176,191,213,187,88,208,61,89,168,130,32,161,
181,151,182,45,229,155,200,190,113,28,50,215,161,86,177,8,144,203,156,249,25,1,75,252,225,95,247,81,201,168,120,201,129,152,118,118,93,249,17,197,187,69,76,8,50,46,83,92,30,225,36,60,20,44,213,57,33,139,
144,165,232,205,161,86,177,17,193,68,107,70,104,33,239,242,49,174,133,211,67,72,225,244,68,49,77,165,143,180,220,40,124,245,168,26,18,37,99,166,109,69,73,203,199,152,136,206,116,165,145,121,214,88,33,
235,220,42,6,48,17,65,84,139,68,182,12,239,138,20,138,101,175,96,153,165,99,25,223,233,23,66,166,128,152,82,196,108,66,209,144,73,85,219,174,85,76,163,173,7,215,79,159,62,253,86,107,21,93,132,177,216,
92,33,189,136,211,52,132,132,146,170,38,88,248,17,18,95,229,189,130,110,16,166,162,179,58,209,57,136,174,85,76,93,171,56,157,22,137,247,42,56,77,8,61,17,122,161,179,148,182,122,207,180,89,208,165,118,
212,60,241,150,7,173,188,215,8,52,170,109,160,23,172,226,157,59,119,202,237,123,225,245,31,222,62,119,246,220,217,156,51,227,241,163,131,183,110,5,144,162,128,176,88,188,107,186,25,113,240,210,59,120,
225,181,86,241,165,51,47,115,119,99,131,15,63,124,255,220,51,89,197,146,85,117,97,41,137,170,29,139,136,180,127,55,21,251,127,181,138,255,29,0,55,233,176,217,24,42,121,48,0,0,0,0,73,69,78,68,174,66,96,
130,0,0};

const char* TalComponent::bmp00132_png = (const char*) resource_TalComponent_bmp00132_png;
const int TalComponent::bmp00132_pngSize = 4303;














































static const unsigned char temp2[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,74,0,0,1,106,8,6,0,0,0,229,103,117,50,0,0,0,9,112,72,89,
  115,0,0,15,97,0,0,15,97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,1,156,81,73,68,65,84,120,218,236,
  253,213,147,100,89,150,230,137,254,246,57,74,198,204,12,110,110,14,225,76,225,17,30,156,193,145,89,153,89,93,85,210,211,115,175,92,153,151,43,50,
  15,247,95,232,191,227,62,204,21,105,145,158,153,146,174,174,202,140,204,96,38,135,136,112,38,99,102,102,83,56,112,31,14,171,170,153,155,67,56,238,
  207,69,93,193,84,15,236,179,206,222,223,94,123,173,111,137,23,78,191,96,34,33,33,33,33,33,33,33,33,145,1,69,54,129,132,132,132,132,132,132,
  132,132,36,74,18,18,18,18,18,18,18,18,146,40,73,72,72,72,72,72,72,72,72,162,36,33,33,33,33,33,33,33,33,137,146,132,132,132,132,132,
  132,132,132,36,74,18,18,18,18,18,18,18,18,146,40,73,72,72,72,72,72,72,72,72,162,36,33,33,241,216,67,8,129,16,66,54,132,132,132,132,
  68,26,66,178,115,148,144,120,246,72,81,240,125,224,29,0,230,29,100,104,205,44,95,120,28,250,18,211,148,250,185,18,18,18,15,152,40,201,38,144,
  144,120,246,72,146,199,105,50,73,147,105,166,147,39,0,51,99,59,233,156,36,27,225,202,66,101,238,116,132,89,190,35,118,248,91,239,252,182,227,75,
  146,76,73,72,72,72,162,36,33,33,113,7,146,228,189,206,36,79,100,33,27,226,142,36,195,249,125,54,207,146,67,192,210,9,148,127,123,222,239,196,
  54,36,106,103,228,199,219,95,54,130,38,36,105,146,144,144,144,68,73,66,66,98,43,50,35,92,130,228,189,23,108,181,114,230,231,18,30,177,48,183,
  36,51,59,221,86,250,54,239,103,233,46,147,240,152,129,253,249,159,133,48,125,175,133,36,75,18,18,18,146,40,73,72,72,130,36,2,100,201,9,222,
  86,20,225,190,246,123,151,182,34,33,166,121,103,47,204,78,182,149,109,19,254,175,59,127,191,211,103,91,19,57,239,189,119,204,206,107,143,48,73,178,
  36,33,33,33,137,146,132,132,132,143,96,136,0,73,82,20,101,91,178,180,21,73,50,183,97,59,2,2,94,165,108,219,219,138,151,100,46,153,121,196,
  200,255,121,182,207,178,29,155,159,40,153,166,137,97,120,30,49,211,20,146,44,73,72,72,220,17,106,99,99,227,127,149,205,32,33,241,180,147,36,143,
  4,89,196,200,34,72,225,112,136,35,71,142,240,202,43,175,178,171,163,131,141,141,117,214,215,215,108,2,165,184,100,234,208,225,35,180,181,181,51,49,
  62,102,125,46,172,207,61,194,101,61,84,251,55,39,78,158,66,211,82,36,18,113,162,209,40,71,142,28,35,30,223,36,153,76,248,182,29,124,28,58,
  116,136,182,182,118,38,39,199,3,159,55,52,54,240,252,233,23,152,157,153,65,215,53,84,213,251,155,243,250,224,161,195,152,166,201,230,230,134,123,204,
  22,65,19,8,197,35,111,89,90,70,26,135,132,132,196,182,144,30,37,9,137,103,135,46,185,228,193,33,54,57,57,57,116,236,238,100,116,108,132,229,
  165,37,94,126,249,21,46,92,56,199,240,200,176,75,176,76,195,32,47,47,143,188,188,60,66,33,21,33,4,186,110,224,143,83,10,46,139,9,142,30,
  61,70,113,81,33,223,125,255,45,53,181,53,188,244,242,203,44,45,47,176,186,182,2,166,137,16,10,134,97,184,191,87,20,133,104,52,74,126,126,30,
  170,170,4,182,23,141,68,40,44,44,36,18,81,73,36,44,98,100,24,6,149,149,85,84,84,84,208,213,213,133,174,165,16,2,66,33,213,254,173,227,
  41,178,246,161,227,120,156,20,192,192,217,181,105,154,89,189,88,18,18,18,18,146,40,73,72,60,235,180,73,8,66,161,16,66,8,70,134,135,153,157,
  157,38,55,47,151,3,7,14,177,188,188,196,129,131,135,136,69,115,184,113,227,26,154,150,66,215,117,138,75,138,57,113,226,20,169,84,138,91,55,111,
  176,171,99,55,103,127,250,145,186,186,122,42,42,42,185,116,233,87,162,209,8,137,120,156,202,170,106,84,69,161,162,162,194,141,17,42,41,46,225,228,
  169,211,132,67,33,122,251,122,232,233,238,226,228,169,211,20,21,22,81,90,86,198,64,127,31,251,247,63,199,226,210,18,179,51,211,28,57,122,156,201,
  137,113,180,84,138,88,44,198,222,125,207,81,90,90,198,245,107,87,104,110,110,229,244,233,23,49,76,147,220,156,92,150,150,23,105,104,104,224,208,225,
  163,0,252,252,243,121,18,241,4,39,79,62,143,105,154,108,108,110,242,195,247,223,177,177,177,105,147,35,73,146,36,36,36,238,12,169,204,45,33,241,
  76,195,196,48,116,84,85,5,33,24,31,27,37,20,14,115,232,240,17,76,19,22,22,23,216,187,111,63,209,72,212,93,218,186,117,235,6,21,229,21,
  84,85,85,83,82,82,66,99,99,35,77,205,205,32,64,216,94,170,197,165,69,22,22,230,121,238,192,65,66,161,16,35,35,67,132,212,16,167,78,157,
  102,116,116,152,95,47,254,76,91,91,59,47,188,248,18,197,197,197,156,61,247,3,211,211,83,132,66,33,42,171,170,201,203,203,67,40,130,250,250,122,
  76,76,132,16,36,83,41,6,6,250,216,88,95,227,192,129,67,76,77,77,210,213,125,155,161,161,1,170,170,171,169,169,169,229,240,145,99,92,185,114,
  137,174,174,91,156,60,121,138,138,138,10,202,43,42,248,246,219,111,40,45,45,165,165,181,213,183,12,41,175,190,132,132,132,36,74,18,18,18,219,66,
  160,8,133,84,42,5,166,73,75,75,27,171,171,43,20,22,22,81,88,80,136,105,24,140,12,15,187,228,98,207,158,125,236,233,220,71,74,211,8,133,
  67,220,188,121,131,163,199,78,144,151,151,71,111,79,23,0,177,88,14,166,105,50,52,52,200,201,147,207,147,72,36,152,157,153,161,176,176,136,130,130,
  2,166,167,166,88,152,159,39,149,74,81,91,91,199,242,210,18,27,235,27,172,174,172,32,156,248,34,4,170,162,162,40,42,2,129,166,105,52,55,183,
  112,248,208,81,116,93,71,13,133,48,77,3,77,211,72,38,147,152,166,181,223,72,56,204,244,212,36,179,51,211,68,34,81,34,145,8,43,203,203,108,
  110,110,144,72,196,81,85,233,68,151,144,144,184,59,200,94,67,66,226,25,130,183,220,100,101,129,105,154,134,26,10,209,209,177,155,229,149,106,154,154,
  154,249,225,135,239,168,168,168,164,190,190,129,213,213,85,38,39,39,40,44,42,36,20,14,83,92,92,130,110,232,196,227,155,132,195,17,70,70,134,120,
  237,181,55,88,92,92,96,125,125,29,128,72,52,134,174,105,44,45,46,96,24,38,171,43,43,68,34,17,20,85,165,127,160,159,99,199,79,178,190,190,
  74,42,149,228,194,133,179,28,61,114,156,3,7,15,211,212,220,194,232,200,16,43,43,43,236,238,220,67,69,101,165,27,143,164,233,26,197,197,197,152,
  152,36,146,9,12,195,96,109,125,149,162,162,34,26,234,27,49,12,131,165,197,5,34,225,8,47,190,248,18,106,40,204,200,200,48,107,107,235,32,252,
  217,111,134,251,90,66,66,66,98,71,211,201,23,95,124,81,118,25,18,18,79,251,141,190,77,214,91,107,107,43,229,229,229,24,134,201,216,216,8,11,
  11,243,40,138,194,238,221,157,228,231,23,48,48,208,143,97,232,132,195,33,146,201,36,45,45,109,36,147,9,166,167,167,209,117,157,119,222,121,159,111,
  190,249,146,233,233,105,192,36,47,47,159,188,188,124,22,22,230,41,41,41,97,121,121,137,252,252,2,20,69,176,184,184,72,103,231,94,34,145,8,3,
  3,125,172,173,173,81,87,87,79,89,89,57,43,43,43,204,207,207,177,177,177,78,103,231,94,116,93,103,113,113,129,149,149,101,10,10,10,89,94,94,
  166,185,185,5,211,52,89,90,90,100,106,106,146,93,187,118,179,177,177,65,50,153,96,121,121,5,211,52,232,236,220,75,42,149,162,171,235,22,170,26,
  162,184,184,132,177,177,49,170,170,170,89,91,91,101,105,105,25,93,55,124,114,1,230,142,244,161,36,36,36,158,209,254,243,204,139,103,100,239,32,33,
  241,140,16,37,39,227,77,8,220,20,122,85,85,60,29,37,197,175,131,36,172,184,35,156,239,11,223,239,173,0,159,151,95,121,141,229,165,37,46,93,
  250,53,144,109,102,237,51,112,4,190,99,217,238,24,183,214,51,10,122,130,252,250,72,96,98,186,73,120,142,183,204,4,76,195,244,222,219,196,200,48,
  76,76,211,176,159,145,68,73,66,66,98,91,200,165,55,9,137,103,4,142,26,181,97,88,164,199,48,12,132,176,210,229,93,143,147,97,107,15,137,116,
  47,148,149,178,175,40,216,191,183,182,121,249,210,175,172,173,173,101,37,56,65,129,72,51,203,231,233,140,201,180,151,6,179,215,131,75,39,53,134,197,
  144,124,175,131,132,199,251,190,39,52,233,144,36,71,157,219,79,236,36,36,36,36,36,81,146,144,120,102,73,146,159,128,120,100,9,12,116,61,83,149,
  219,255,222,33,65,138,162,0,10,86,130,156,130,170,170,172,172,172,248,200,16,1,181,235,173,73,147,71,140,130,223,177,69,43,85,21,93,215,125,223,
  23,1,242,227,39,73,134,225,120,134,252,143,116,130,149,89,198,196,89,114,147,144,144,144,144,68,73,66,66,34,64,88,44,162,98,145,37,39,77,222,
  9,242,118,188,60,233,49,77,166,169,128,48,17,166,73,65,65,49,123,246,236,69,8,152,156,156,96,98,98,220,191,135,45,201,82,54,210,228,39,79,
  249,249,249,236,218,213,129,26,10,113,251,214,13,74,75,203,88,93,93,101,101,101,57,243,247,62,146,228,95,74,3,92,226,228,39,99,102,96,89,46,
  251,231,18,18,18,18,217,32,229,1,36,36,158,25,146,228,247,202,56,164,198,139,219,177,30,70,218,195,71,68,116,235,179,230,230,22,234,234,234,89,
  88,88,224,149,87,94,167,164,164,20,128,218,186,122,10,10,10,92,194,85,82,82,66,85,85,149,75,182,98,177,40,149,85,149,228,228,228,144,151,159,
  71,121,121,133,123,108,225,112,152,211,47,156,161,184,164,132,197,197,5,76,19,138,138,138,201,205,203,163,172,172,156,170,170,106,74,75,75,0,200,203,
  203,167,170,170,10,108,79,147,67,146,12,195,68,183,143,209,58,47,195,254,204,31,155,20,44,142,43,73,146,132,132,196,157,32,61,74,18,18,207,32,
  89,178,150,179,220,79,51,20,170,157,165,58,171,4,136,98,255,198,10,144,214,13,131,229,149,37,230,231,231,88,89,93,65,13,169,28,57,114,140,226,
  226,18,84,85,229,210,229,95,9,169,33,14,28,60,140,174,107,204,206,206,48,60,52,200,233,23,206,176,186,186,74,97,97,17,243,115,115,84,84,84,
  112,227,198,53,250,251,251,237,229,48,171,84,202,244,212,36,241,248,38,177,88,140,88,52,70,97,65,1,173,173,237,228,228,228,112,238,220,79,84,87,
  215,34,132,160,178,170,154,75,23,47,5,138,222,250,51,217,188,115,217,186,29,36,36,36,36,238,4,233,81,146,144,144,112,61,44,153,15,51,24,235,
  99,152,232,154,70,117,117,13,175,189,254,59,150,151,150,16,8,90,90,90,249,229,151,243,172,172,44,179,127,223,1,78,156,124,158,222,158,46,110,92,
  191,70,125,93,3,205,45,173,228,229,229,211,219,211,77,44,22,99,125,99,157,169,169,73,90,90,219,9,133,66,164,82,41,206,157,251,145,133,249,5,
  222,125,239,247,84,87,215,80,81,89,69,94,126,30,3,3,253,152,192,216,216,40,237,187,58,168,174,174,33,39,55,151,3,7,14,82,84,84,20,88,
  78,75,247,24,101,198,47,153,146,36,73,72,72,72,162,36,33,33,145,29,110,218,191,251,62,248,240,127,238,163,81,129,128,104,69,85,153,24,159,224,
  155,175,191,164,160,160,128,226,226,98,82,154,198,234,234,42,107,107,107,132,66,33,98,177,24,243,243,243,172,174,174,144,210,82,68,35,81,150,151,150,
  88,93,93,97,105,105,137,149,149,101,150,151,151,93,41,2,211,52,89,95,91,227,236,217,31,89,90,90,162,169,185,5,45,149,34,190,25,167,169,169,
  133,84,50,201,197,139,191,144,151,151,207,210,242,18,189,189,61,124,249,229,23,108,108,108,100,100,201,73,72,72,72,60,72,200,165,55,9,137,103,136,
  36,101,18,33,145,246,29,210,72,148,192,11,240,182,62,55,12,131,148,150,98,118,118,150,100,42,69,50,153,98,117,117,133,215,95,127,147,156,220,28,
  46,95,186,72,94,126,62,175,188,250,58,241,205,77,86,86,86,24,27,27,165,164,180,12,69,81,49,13,195,221,135,166,105,24,134,129,170,170,28,58,
  116,132,146,146,18,242,243,242,232,186,125,139,150,150,22,242,11,10,232,220,189,135,141,205,13,246,236,217,199,64,127,31,29,187,59,73,36,18,76,78,
  76,144,74,37,93,141,39,43,123,47,72,240,156,115,150,203,111,18,18,18,247,220,119,158,57,35,5,39,37,36,158,21,146,228,39,72,126,57,0,255,
  223,130,130,145,158,192,164,98,215,97,203,205,205,37,26,141,176,190,190,78,65,65,62,186,174,97,154,38,245,245,13,44,46,46,178,188,188,4,64,69,
  69,37,209,104,140,241,241,81,84,85,37,22,203,97,125,125,157,156,156,28,18,137,36,0,225,112,136,181,181,117,76,211,164,184,184,152,234,234,106,102,
  102,102,108,53,239,60,132,16,228,231,23,16,141,70,209,117,131,137,137,49,114,114,242,168,168,168,100,102,102,134,249,249,121,12,195,184,163,210,118,144,
  19,201,140,55,9,9,9,73,148,36,36,36,210,136,146,159,36,57,234,220,22,97,18,89,213,178,253,36,202,35,75,138,175,12,138,240,189,246,182,237,
  144,172,108,194,145,217,148,176,253,196,204,175,32,238,39,111,66,8,151,4,249,51,216,156,76,61,231,181,95,75,105,39,18,1,146,44,73,72,72,220,
  9,33,129,92,223,151,144,120,234,137,18,206,157,238,17,27,167,222,155,159,40,109,23,239,99,145,36,91,91,201,94,146,19,8,155,135,8,119,21,207,
  217,151,200,88,214,19,150,80,36,194,210,100,66,248,244,155,130,251,247,142,214,219,166,34,192,48,65,17,2,161,88,196,72,40,222,118,20,177,51,209,
  73,247,159,79,239,105,187,210,41,18,18,18,207,56,81,146,77,32,33,241,148,147,36,225,247,30,17,32,73,126,207,208,86,158,37,124,148,197,239,137,
  202,128,153,253,99,255,154,158,39,14,46,48,133,29,67,228,227,39,225,80,24,211,52,209,117,61,64,222,84,85,5,32,149,210,220,223,41,248,149,192,
  205,180,12,189,204,140,61,203,35,37,210,106,189,73,178,36,33,33,177,61,212,166,166,166,255,42,155,65,66,226,217,32,74,78,156,145,67,146,14,30,
  60,196,243,167,95,160,181,165,149,154,154,26,55,230,199,250,142,247,61,231,119,214,182,148,0,177,114,189,80,62,143,82,78,78,140,87,94,125,157,197,
  197,5,226,241,56,229,229,21,156,122,254,52,19,227,99,24,110,48,183,176,11,238,122,143,99,199,79,144,159,159,207,194,194,124,32,67,175,99,247,30,
  90,91,219,152,154,154,176,215,208,236,101,63,85,184,199,122,236,216,113,54,227,155,36,147,9,223,54,149,192,246,29,222,22,228,67,210,171,46,33,33,
  177,53,164,60,128,132,196,51,67,152,252,15,139,96,180,180,182,50,59,51,205,185,115,63,81,84,84,196,203,175,188,74,44,199,82,208,174,174,169,118,
  83,253,11,139,138,168,173,171,37,22,139,82,81,81,65,105,89,169,165,182,109,171,108,215,214,213,145,19,203,177,137,15,228,230,230,241,220,115,7,105,
  107,109,71,0,109,109,237,236,223,127,144,72,36,138,162,40,84,86,85,83,81,89,137,170,90,100,44,47,47,143,218,218,90,74,75,203,136,70,99,132,
  194,97,212,80,8,53,20,34,20,14,147,19,139,145,155,151,79,56,18,161,162,178,138,242,138,10,84,85,225,212,201,211,188,246,218,27,20,20,228,179,
  178,186,140,174,107,68,34,17,234,234,234,40,43,47,71,85,21,98,177,40,37,37,197,212,215,215,147,151,151,155,17,99,229,180,141,132,132,132,68,54,
  200,165,55,9,137,103,150,56,9,12,93,103,99,99,131,205,205,13,126,58,251,3,239,190,251,1,101,101,101,52,52,52,81,91,91,71,119,215,109,66,
  161,16,157,123,246,178,178,178,76,73,113,41,211,51,83,148,149,149,243,221,183,95,179,111,255,115,148,149,150,177,185,185,137,162,170,124,251,205,87,108,
  110,110,16,142,68,152,158,158,162,160,176,208,82,216,142,197,152,153,153,34,28,14,113,240,208,97,234,106,235,81,20,133,193,193,1,70,71,135,121,229,
  213,55,88,88,152,167,182,182,142,225,161,65,94,121,249,53,46,95,190,136,170,170,236,217,187,143,217,217,25,82,201,4,177,104,148,214,214,54,42,43,
  42,233,238,238,162,164,164,132,218,186,122,122,251,122,104,105,105,101,115,115,147,166,166,102,74,138,75,136,198,98,92,190,116,145,220,188,124,158,219,255,
  28,11,11,11,24,166,201,151,95,124,206,250,250,70,160,198,157,92,113,147,144,144,216,10,210,163,36,33,241,12,195,95,210,68,75,105,14,133,98,102,
  102,154,141,141,13,170,170,171,41,40,40,96,114,114,130,179,63,253,72,56,18,230,135,239,191,101,97,126,158,242,138,10,98,177,24,221,221,93,124,246,
  217,199,152,134,65,123,251,46,0,162,145,8,115,115,179,172,175,175,243,252,243,47,50,63,63,199,226,226,2,229,21,21,180,181,182,243,233,167,31,113,
  246,220,143,52,52,54,114,250,133,51,140,12,15,242,227,15,223,49,56,216,79,36,18,33,18,137,184,26,75,225,80,216,97,118,24,134,193,236,236,12,
  241,120,156,138,202,74,198,199,199,184,124,233,87,166,38,39,200,137,229,208,216,216,68,113,113,9,159,125,246,49,23,206,159,229,224,161,195,228,196,98,
  76,77,77,241,201,39,31,81,84,88,72,113,113,177,143,44,74,27,144,144,144,144,68,73,66,66,98,11,146,20,14,135,81,236,64,233,19,39,78,177,
  48,63,79,115,83,51,251,247,31,96,109,109,149,80,40,132,170,170,232,154,134,170,42,164,82,41,59,54,72,65,32,48,116,131,104,44,70,94,94,30,
  145,104,148,68,34,129,105,66,65,65,33,27,27,27,44,47,47,113,240,208,17,230,230,102,49,12,131,156,88,14,8,40,46,46,166,176,176,16,195,48,
  72,38,147,228,216,218,76,78,73,18,85,85,41,42,46,166,164,180,148,156,156,28,20,161,160,235,58,199,79,156,162,161,190,145,181,181,85,84,85,37,
  28,14,83,84,84,140,26,10,161,168,42,186,174,19,14,135,201,207,47,160,168,168,136,100,210,210,107,210,117,221,85,21,23,66,241,181,129,180,3,9,
  9,137,237,17,146,113,140,18,18,79,57,132,247,112,202,197,58,41,242,75,203,75,180,183,183,83,82,82,66,78,110,14,191,252,114,129,146,146,82,202,
  202,203,137,197,98,44,46,46,146,74,165,48,77,19,77,215,89,92,180,150,175,86,86,86,88,223,88,199,48,77,218,219,119,81,94,94,193,236,236,12,
  131,67,3,152,152,164,52,141,249,249,121,22,22,230,185,125,251,38,203,203,86,217,146,165,229,101,110,221,186,193,169,231,95,192,52,77,186,186,110,50,
  55,55,199,139,47,190,204,241,19,207,3,176,184,180,72,178,231,54,135,14,29,102,125,125,157,233,153,105,54,54,55,64,192,202,202,50,85,85,213,32,
  96,105,105,137,137,137,81,94,122,233,85,42,43,43,89,92,92,96,104,104,16,77,75,241,194,139,103,0,248,245,215,159,41,41,41,67,55,116,76,96,
  110,110,142,100,42,97,183,133,105,61,111,149,173,39,33,33,33,1,136,151,94,122,73,206,169,36,36,158,230,155,60,45,235,205,175,178,157,151,151,75,
  113,113,49,138,162,176,178,178,228,42,102,151,151,151,99,24,6,171,171,171,182,7,198,68,211,52,98,177,24,155,155,155,182,82,182,206,59,239,188,207,
  248,248,40,163,163,163,44,46,46,144,74,165,0,147,72,36,106,139,68,234,168,170,138,97,232,196,98,49,91,69,91,167,188,188,28,77,211,89,90,90,
  192,48,76,242,242,242,173,98,185,235,235,108,110,110,162,105,26,229,229,21,164,82,41,54,54,214,237,243,80,72,38,19,148,149,149,163,105,26,27,27,
  235,164,82,41,74,237,24,41,195,48,72,36,18,246,241,87,144,72,196,89,90,90,34,28,182,150,238,54,55,227,196,98,57,108,110,110,90,50,3,174,
  68,0,100,83,243,150,144,144,144,0,16,47,191,244,178,236,25,36,36,158,17,178,228,79,149,247,75,5,128,39,40,233,87,197,78,231,13,233,49,61,
  187,59,59,153,157,153,97,97,97,193,37,28,254,125,250,21,189,189,253,41,246,62,140,128,154,182,95,109,219,175,129,116,55,72,47,85,226,215,83,218,
  106,31,217,4,42,37,36,36,36,28,200,172,55,9,137,103,8,126,50,99,201,25,25,182,232,162,192,52,245,109,149,185,179,225,214,205,91,91,18,12,
  107,155,194,38,70,6,160,32,132,73,99,83,51,191,251,221,219,220,184,126,149,243,231,207,98,24,134,91,138,196,255,252,32,72,75,122,57,147,108,165,
  78,36,36,36,36,36,81,146,144,144,176,213,167,45,53,106,143,44,57,226,141,254,194,185,59,39,75,219,169,89,59,68,201,217,190,67,78,10,10,10,
  168,171,171,103,118,118,38,80,175,205,121,214,245,7,75,148,252,132,41,232,101,242,222,111,69,246,36,36,36,36,36,81,146,144,120,38,72,146,105,19,
  23,220,146,29,214,107,111,137,77,136,157,16,133,59,145,40,211,253,158,181,236,166,216,164,68,100,120,119,60,133,110,107,255,134,97,186,36,201,202,82,
  99,135,251,218,158,28,102,146,38,239,183,146,36,73,72,72,72,162,36,33,33,145,65,150,32,72,152,2,84,72,100,39,26,214,231,102,218,119,179,19,
  39,139,116,249,203,156,224,198,41,249,213,176,131,165,80,188,227,76,247,246,236,132,204,108,255,103,115,203,239,73,146,36,33,33,33,137,146,132,132,68,
  128,20,248,9,83,58,249,217,138,55,56,30,168,173,62,75,39,64,78,109,56,69,81,80,85,21,85,181,158,173,152,37,124,127,243,188,59,222,107,19,
  195,240,147,57,51,35,88,252,126,248,141,36,71,18,18,18,146,40,73,72,72,60,112,162,224,121,163,28,82,100,218,193,224,184,25,115,170,106,17,164,
  80,72,37,20,178,200,145,245,90,117,63,3,203,195,20,142,132,80,116,129,162,233,25,94,37,33,76,123,121,206,242,78,5,151,206,36,217,145,144,144,
  120,88,68,73,10,173,73,72,72,220,45,97,194,90,198,195,93,86,19,62,15,145,245,8,133,84,194,225,80,128,36,89,106,218,214,252,76,85,21,162,
  209,16,186,174,160,169,10,106,72,65,73,41,150,234,183,34,208,52,29,97,8,59,208,219,64,64,32,16,93,66,66,66,226,225,16,37,9,9,9,137,
  123,134,95,151,201,35,72,14,73,138,68,66,46,89,10,135,67,168,170,66,36,98,9,64,134,66,42,57,57,81,52,77,71,215,117,82,41,29,69,209,
  124,186,78,2,77,51,208,117,103,95,134,157,165,39,75,143,72,72,72,60,68,162,36,164,75,73,66,66,226,94,40,146,16,40,46,73,178,98,144,194,
  33,149,80,88,37,18,14,17,137,134,137,68,194,22,89,10,133,8,133,85,66,170,138,192,201,118,51,201,205,137,162,105,6,154,174,19,14,105,36,85,
  21,85,81,81,132,66,42,165,1,154,93,115,69,96,26,32,48,144,21,71,36,36,36,30,42,81,146,77,32,33,33,113,47,36,201,129,98,19,38,85,
  85,40,46,41,166,190,174,142,112,36,76,56,108,121,145,194,97,203,195,228,4,113,151,87,84,1,144,159,95,64,125,67,179,195,153,208,52,29,77,211,
  72,38,53,82,41,13,77,211,153,95,88,100,124,108,28,195,52,81,20,75,151,73,152,118,157,54,9,9,9,9,73,148,36,36,36,30,119,194,36,132,
  64,40,22,81,82,20,133,215,223,120,147,146,146,82,87,142,32,93,110,64,85,173,110,167,166,166,129,242,242,170,64,230,156,63,80,219,52,225,195,191,
  254,79,38,39,38,80,21,129,97,199,68,73,72,72,72,72,162,36,33,33,241,100,17,38,64,32,136,111,174,211,219,211,197,75,47,191,126,199,223,40,
  138,74,52,154,179,229,223,71,71,135,24,30,30,68,40,34,83,151,64,66,66,66,226,33,65,145,77,32,33,33,113,183,200,94,219,205,250,239,210,229,
  95,152,155,155,185,175,237,27,134,193,249,115,63,160,233,26,50,142,82,66,66,226,81,66,202,3,72,72,72,220,59,97,178,255,33,188,164,125,77,75,
  113,245,234,47,188,254,250,123,247,188,221,190,190,46,38,167,198,221,125,248,163,146,156,253,73,72,72,72,60,12,72,143,146,132,132,196,253,145,37,211,
  43,120,235,60,122,123,187,152,153,153,184,167,237,37,147,9,110,222,188,140,166,233,89,235,179,73,72,72,72,72,162,36,33,33,241,132,145,36,171,168,
  173,97,24,232,186,65,42,149,226,202,149,11,247,164,158,61,48,208,197,204,204,180,43,52,105,109,215,35,97,214,126,37,105,146,144,144,120,56,144,58,
  74,18,18,18,247,4,97,175,183,9,4,166,97,98,26,38,134,110,61,116,205,96,108,116,148,177,177,65,26,26,90,119,188,205,245,245,85,122,122,110,
  145,74,105,232,154,225,110,15,19,76,3,48,101,143,37,33,33,241,112,33,61,74,18,18,18,247,4,191,119,199,239,77,178,244,144,116,18,137,4,183,
  111,95,69,215,180,29,111,115,100,164,159,233,169,41,82,73,205,86,236,54,208,117,221,174,249,38,61,73,18,18,18,146,40,73,72,72,60,97,112,150,
  197,116,221,180,75,145,88,162,145,201,164,198,216,232,8,35,163,3,59,218,206,234,234,50,221,93,55,72,36,83,36,147,41,82,41,139,112,25,134,137,
  174,251,203,151,72,178,36,33,33,241,240,32,117,148,36,36,36,118,4,177,133,150,145,247,185,67,150,12,183,118,91,34,153,162,167,231,6,181,181,141,
  68,163,177,109,183,63,60,212,203,236,236,44,90,74,11,120,146,28,130,36,4,182,56,101,246,227,144,4,74,226,97,219,254,227,2,105,251,146,40,73,
  72,72,60,6,3,131,95,101,59,253,239,138,34,2,127,183,188,64,58,186,166,51,57,49,198,232,232,0,237,237,123,183,220,223,234,234,50,183,187,174,
  161,233,186,237,61,50,2,219,54,77,187,124,137,48,51,6,135,173,198,8,57,120,72,60,40,251,223,206,246,31,37,49,146,182,255,176,136,146,140,140,
  148,144,144,200,50,64,56,175,21,69,4,6,11,239,59,206,231,94,41,19,69,21,182,146,182,245,103,221,48,209,13,131,238,238,235,212,215,183,16,139,
  101,87,226,238,233,185,206,250,250,58,134,97,56,202,76,32,64,40,10,138,106,34,20,213,183,244,230,14,7,110,198,157,53,48,136,192,123,231,56,229,
  160,33,113,63,228,200,249,60,248,222,179,253,135,75,144,164,237,63,26,162,36,33,33,33,193,86,3,130,227,45,82,16,194,33,77,193,65,194,255,59,
  171,222,155,69,154,172,142,219,10,240,94,88,152,99,104,168,135,206,206,131,25,251,93,92,156,163,191,191,203,94,106,51,221,193,64,8,97,239,79,197,
  52,13,236,154,186,25,218,74,142,52,1,248,101,4,144,131,134,196,61,145,36,199,126,211,109,223,153,12,100,179,253,135,71,148,50,109,223,145,205,144,
  182,255,27,18,37,153,108,43,33,33,7,9,255,107,199,131,164,40,2,69,81,92,114,228,45,175,109,191,4,167,8,197,250,14,2,76,97,199,27,25,
  116,119,93,165,177,177,157,220,220,188,192,111,110,223,186,76,34,145,64,215,13,76,195,146,2,80,132,181,239,144,10,134,48,49,77,37,99,192,240,102,
  211,166,75,176,12,97,164,13,26,193,89,182,28,48,36,118,106,251,66,40,168,234,206,109,255,97,146,165,108,182,111,42,94,246,169,180,253,7,76,148,
  100,19,72,72,200,129,194,63,147,246,15,18,66,120,100,201,63,80,108,23,195,164,40,10,138,245,37,107,25,205,206,90,91,93,93,163,187,251,42,135,
  15,159,118,127,51,57,57,194,216,248,48,186,166,99,232,158,160,164,80,4,170,237,66,18,194,220,178,147,119,72,145,170,58,130,151,2,195,48,17,194,
  17,171,180,102,222,206,210,132,28,48,36,182,179,125,199,222,61,219,183,222,223,201,246,31,53,97,114,8,146,16,166,157,244,96,221,115,66,128,97,72,
  178,36,137,146,132,132,196,3,37,73,22,49,82,80,85,235,225,145,39,43,254,72,224,124,79,117,186,236,192,246,156,88,165,144,170,162,42,42,138,162,
  96,26,22,145,233,235,189,77,75,75,39,197,197,165,24,134,78,87,215,53,18,241,164,69,104,132,181,205,80,200,34,54,138,208,81,77,211,173,92,226,
  117,242,194,93,210,51,49,81,237,129,66,81,60,162,164,235,214,57,57,178,2,214,96,34,7,12,137,157,218,190,247,58,221,246,17,130,144,170,96,45,
  65,63,108,59,18,54,9,210,173,72,190,44,182,111,61,27,128,245,172,40,146,44,73,162,36,33,33,113,95,112,6,2,43,190,72,73,123,22,40,170,
  130,170,40,84,84,86,82,90,90,70,69,121,5,121,121,249,68,34,17,155,38,153,105,93,185,21,204,173,56,131,75,154,87,202,52,173,140,182,84,42,
  69,99,99,59,85,85,13,8,4,134,187,100,230,121,130,178,117,234,66,8,52,77,99,115,99,131,165,165,69,230,230,230,152,153,153,33,30,143,187,101,
  79,44,146,164,187,68,78,14,24,18,219,217,190,162,8,84,85,117,159,253,182,175,8,65,89,121,57,101,165,229,148,87,84,80,144,159,79,36,26,181,
  20,233,31,50,81,18,88,118,29,143,111,178,188,188,196,236,236,28,179,179,51,172,175,175,219,247,141,137,174,11,116,93,0,214,125,38,109,95,18,37,
  9,9,137,251,152,81,103,35,73,161,144,61,80,216,4,169,173,173,157,189,251,246,211,212,212,66,73,73,233,3,59,134,104,52,70,91,91,231,125,111,
  39,145,136,51,50,50,76,119,247,109,110,221,186,193,230,70,220,71,146,182,30,48,36,164,237,251,73,146,227,65,245,30,42,13,141,141,236,223,127,128,
  230,166,22,202,43,42,31,187,115,209,52,75,212,181,167,183,139,91,55,111,176,178,178,226,91,38,244,2,207,117,93,146,165,251,178,153,215,94,123,77,
  182,150,132,196,51,56,80,120,203,13,222,64,225,144,36,85,85,41,40,200,231,213,87,223,160,179,115,47,57,57,185,0,12,15,15,50,57,57,206,202,
  202,10,201,100,50,107,167,235,239,160,253,157,182,95,107,201,249,142,27,136,106,250,131,176,217,178,248,173,179,175,80,40,68,94,94,62,21,21,21,52,
  53,181,146,155,235,29,223,247,223,127,203,224,96,191,27,68,238,149,66,177,68,44,157,253,100,219,190,132,180,125,85,85,201,201,137,113,230,204,203,236,
  223,127,136,130,130,2,0,198,199,71,153,152,24,99,101,101,133,68,34,110,199,4,9,238,214,132,238,45,196,201,19,117,85,213,16,57,57,185,148,149,
  149,209,216,216,76,97,97,17,0,147,147,227,156,61,251,3,183,110,221,116,109,93,211,60,251,247,23,153,150,182,47,61,74,18,18,207,92,199,127,183,
  29,159,127,160,72,247,36,169,170,74,77,77,13,239,191,255,7,106,107,235,49,12,131,203,151,127,229,218,181,203,44,44,44,176,177,177,145,81,123,45,
  125,215,153,242,2,254,160,88,124,242,1,86,231,239,197,17,5,179,213,178,13,48,254,193,46,22,139,146,155,155,71,71,71,39,199,143,159,162,169,169,
  133,127,252,199,42,190,250,234,115,46,95,190,232,251,181,105,15,54,138,27,228,42,7,138,103,211,246,253,246,233,44,9,59,94,164,80,72,165,180,180,
  148,119,223,253,61,173,173,237,0,220,186,117,131,171,87,47,50,55,55,199,198,198,186,235,173,244,219,232,78,246,127,55,210,2,217,38,8,206,179,115,
  15,133,195,17,242,242,242,104,106,106,230,216,177,83,212,212,212,241,254,251,127,164,180,180,140,179,103,127,76,219,150,147,208,32,92,219,151,94,165,157,
  67,109,105,105,249,175,178,25,36,36,158,156,193,33,221,181,190,213,223,182,202,76,11,146,23,143,28,89,68,73,165,186,186,154,127,252,199,127,161,170,
  170,154,217,217,25,254,250,215,127,227,210,165,95,89,94,94,34,145,72,250,102,167,166,251,236,145,27,51,144,133,99,189,54,2,165,77,252,94,30,171,
  128,174,243,222,243,252,108,189,77,211,38,105,214,107,77,211,136,199,227,76,76,140,209,221,125,155,220,220,92,234,234,26,104,111,239,96,99,99,157,233,
  233,73,223,128,225,39,77,65,157,25,137,39,203,238,239,197,246,253,223,75,247,36,133,66,10,161,80,136,210,210,98,254,244,167,127,162,169,169,133,149,
  149,101,62,249,228,67,126,249,229,28,75,75,139,164,82,150,237,7,211,243,73,211,46,194,213,245,114,236,214,179,61,255,235,76,79,148,255,247,65,34,
  22,252,174,183,77,131,68,34,193,236,236,52,93,93,183,81,20,149,198,198,38,90,90,218,16,2,70,71,71,124,19,4,51,112,12,18,119,233,81,146,
  58,74,18,18,79,222,12,122,171,84,101,127,199,156,77,112,206,185,223,21,97,205,164,67,206,76,218,206,80,203,207,203,227,247,191,255,35,101,101,229,
  140,142,14,243,247,191,253,133,165,165,69,119,128,208,117,3,195,180,82,254,77,211,14,101,221,110,86,234,204,132,217,126,22,237,110,107,135,219,51,13,
  91,54,192,16,152,10,160,42,172,44,47,243,209,223,63,100,101,101,153,231,159,63,195,91,111,189,199,242,242,18,125,189,189,86,123,24,38,166,1,166,
  48,49,49,80,132,34,103,212,79,152,221,223,201,246,253,36,35,195,246,237,247,170,227,69,82,20,203,254,21,203,254,115,98,49,222,125,247,15,212,213,
  53,48,55,55,203,71,31,253,133,217,217,105,139,91,152,86,161,65,129,240,106,15,218,89,101,126,241,199,172,30,33,225,212,41,20,32,76,20,69,1,
  187,102,161,216,198,163,228,254,213,169,111,232,19,98,197,244,125,142,32,17,143,243,253,119,95,178,178,178,196,75,47,189,206,153,51,175,178,188,188,204,
  149,203,151,48,13,13,83,81,49,21,251,30,192,64,17,96,72,211,223,185,71,169,181,165,85,122,148,36,36,158,144,193,34,125,201,44,248,236,45,111,
  121,94,35,175,144,108,102,0,183,149,186,175,134,66,238,204,250,119,111,190,77,71,199,30,230,230,102,249,235,95,254,205,37,73,254,135,229,33,178,158,
  77,159,87,201,121,232,122,208,171,20,124,100,251,204,200,136,161,216,250,97,184,138,223,30,185,242,15,132,38,35,35,195,228,231,231,83,91,91,79,109,
  109,29,55,110,92,71,211,82,25,51,235,108,203,123,18,79,174,237,251,151,165,28,225,72,191,23,41,232,81,178,146,23,44,111,146,74,40,20,226,249,
  211,47,114,228,200,49,54,54,214,249,235,95,254,141,185,217,25,55,131,211,33,47,126,229,120,191,103,201,177,69,247,97,152,62,239,144,233,250,117,64,
  248,150,253,68,214,192,107,235,165,216,162,13,210,207,201,202,44,85,236,247,211,83,147,96,154,212,215,55,82,87,87,79,111,111,55,155,155,155,25,158,
  42,233,81,149,68,73,66,226,169,29,44,130,153,58,193,12,29,55,165,217,30,4,156,14,215,83,19,22,105,65,172,138,47,219,71,165,169,185,153,215,
  95,127,11,128,191,125,248,63,153,154,154,244,8,146,97,88,162,144,134,129,166,101,35,62,30,153,9,46,147,101,18,157,236,15,115,199,143,192,0,101,
  120,218,74,126,47,195,228,196,4,77,77,45,84,86,86,97,98,50,52,52,136,97,107,50,249,183,33,241,100,216,190,223,139,148,205,246,157,207,210,201,
  71,54,229,109,103,217,45,20,114,236,95,165,178,178,130,183,222,126,143,104,52,202,231,159,127,196,216,232,176,75,76,156,224,233,100,50,225,218,141,162,
  40,228,228,228,162,233,58,134,174,187,158,213,112,56,76,52,18,37,149,74,89,158,87,215,35,228,57,68,115,114,114,169,168,168,160,176,176,136,141,141,
  141,192,241,133,195,17,114,115,115,109,98,31,108,131,242,242,114,42,43,171,0,72,36,18,238,249,149,149,151,83,81,81,137,105,154,36,83,9,102,102,
  166,40,175,168,164,186,186,150,104,52,70,79,79,87,198,50,182,244,164,74,162,36,33,241,212,14,22,233,4,39,20,82,9,135,67,118,167,111,189,247,
  107,32,9,161,100,16,137,116,81,73,69,181,150,32,94,126,249,85,234,234,234,185,121,227,26,63,255,124,222,243,30,217,36,201,138,35,50,51,72,82,
  182,78,248,183,124,164,199,47,249,22,246,220,115,212,117,141,68,98,147,221,157,123,41,44,44,166,235,246,77,146,137,68,214,210,38,18,143,191,237,251,
  101,44,156,120,58,199,19,100,197,23,169,89,38,11,233,1,208,65,97,73,85,85,80,67,42,33,85,225,248,137,83,116,116,116,50,52,212,207,185,179,
  223,187,222,22,33,4,207,29,56,196,203,175,188,193,216,216,136,75,108,74,74,74,120,239,189,223,51,53,57,97,105,24,217,199,186,127,255,1,78,158,
  122,158,190,190,94,180,84,202,178,76,225,9,85,230,229,230,241,199,63,254,35,77,205,205,212,84,215,112,224,224,97,86,87,87,88,93,91,65,40,130,
  195,71,142,241,234,43,111,48,50,58,76,60,190,137,16,16,10,133,57,125,250,69,142,30,59,65,85,85,53,245,13,141,12,14,14,0,38,167,78,189,
  200,225,35,199,169,172,172,166,182,174,129,241,241,81,146,201,4,171,171,43,236,218,181,155,170,170,106,122,122,110,219,58,75,153,229,79,164,249,239,12,
  33,25,162,36,33,241,36,140,22,160,168,194,19,128,84,173,224,211,226,226,66,132,128,120,60,238,206,90,29,2,227,45,151,9,151,224,152,166,233,46,
  207,41,142,86,146,42,40,43,47,163,161,177,9,128,95,47,94,176,74,143,96,162,27,30,89,50,76,19,221,208,221,24,13,107,178,188,117,79,155,25,
  172,106,222,209,99,150,185,141,45,178,127,176,99,56,76,211,226,73,2,132,97,61,176,133,46,187,186,187,120,126,102,154,170,170,106,218,119,117,112,241,
  226,47,40,170,64,24,150,114,184,97,130,48,173,120,39,137,199,219,147,42,20,207,246,157,192,235,226,226,98,146,73,39,77,223,139,203,243,47,19,43,
  186,238,222,11,30,233,178,149,183,109,82,149,95,144,239,102,184,93,185,114,17,195,52,92,238,29,14,135,105,106,106,161,184,184,152,61,123,246,49,55,
  55,139,34,32,18,141,80,93,83,67,52,22,179,142,205,180,52,138,10,10,11,169,170,170,66,81,84,132,111,249,207,217,94,52,22,165,166,182,150,47,
  191,252,140,233,233,73,14,31,62,198,27,111,188,197,127,255,239,255,63,98,177,28,154,154,154,169,170,174,161,169,169,133,165,165,5,12,3,58,118,239,
  102,239,190,231,248,250,235,207,89,92,92,32,28,14,99,152,58,187,59,247,176,119,223,126,190,251,238,43,150,150,22,137,68,34,164,82,73,0,38,38,
  198,24,27,27,165,173,109,23,157,123,246,49,61,51,141,48,108,1,88,197,178,127,211,185,127,36,238,8,69,54,129,132,196,147,225,77,242,207,174,85,
  85,37,28,86,216,179,103,47,255,219,255,246,255,230,200,145,35,20,22,230,147,147,19,37,55,55,74,110,110,140,220,220,40,177,88,132,104,52,66,36,
  18,34,28,14,17,137,132,108,47,148,106,123,159,172,129,163,170,170,154,210,210,50,38,38,198,88,92,92,200,200,90,243,222,103,63,182,108,15,39,70,
  42,51,174,34,219,239,216,226,145,125,25,197,239,13,74,95,254,243,60,70,6,189,189,93,0,52,52,52,185,49,29,78,145,223,237,178,163,36,30,63,
  178,228,149,213,177,108,247,133,23,94,228,127,253,95,255,95,116,116,236,34,63,63,151,220,220,168,107,255,57,57,17,114,114,34,68,163,97,194,225,176,
  107,247,126,9,12,199,235,84,90,90,74,93,93,61,139,139,11,86,240,182,143,164,215,214,214,163,40,10,63,253,244,29,205,205,109,129,24,192,100,50,
  137,165,107,228,72,12,8,12,67,39,153,76,162,40,108,113,31,8,226,241,56,203,203,139,44,45,45,114,245,234,69,242,243,11,136,70,163,84,85,85,
  3,112,238,220,15,180,181,237,34,26,141,161,40,10,249,249,5,40,138,194,220,220,12,203,203,75,204,205,205,98,154,6,121,121,5,8,33,152,157,157,
  113,143,93,211,52,247,248,7,7,123,1,104,108,108,242,221,99,10,210,228,37,81,146,144,120,74,7,11,124,75,15,10,225,176,181,228,166,170,10,197,
  197,37,188,243,206,31,248,167,127,250,47,60,247,220,65,74,75,139,201,203,139,185,100,41,39,39,66,44,22,33,22,11,19,137,132,93,210,228,12,28,
  138,162,80,86,86,14,88,41,197,201,100,202,37,35,254,224,85,231,56,130,49,78,217,31,233,218,73,94,16,110,246,135,63,192,214,139,177,218,234,187,
  65,157,166,173,164,9,12,195,180,83,164,161,164,164,148,112,56,156,86,248,87,236,136,240,61,138,135,68,54,155,243,108,223,33,62,245,245,141,252,249,
  207,255,153,247,223,255,19,109,109,109,20,21,21,144,151,23,35,47,47,70,78,142,69,156,28,187,143,70,211,109,223,34,76,37,37,37,86,32,244,244,
  36,155,155,27,129,165,217,250,250,38,22,22,230,185,124,249,87,20,69,161,190,190,193,181,79,139,188,137,140,184,64,255,61,226,217,107,240,117,110,110,
  30,5,5,5,236,219,119,128,181,181,85,116,93,163,174,174,129,197,197,5,126,249,229,28,69,69,69,148,148,148,160,40,130,190,190,30,214,214,86,249,
  211,159,254,153,61,123,246,17,141,70,17,66,161,167,231,54,107,107,107,252,225,15,255,72,103,231,94,194,225,72,32,184,124,114,114,2,195,48,40,42,
  42,38,47,47,63,16,231,37,109,236,238,32,229,1,36,36,30,195,217,115,250,103,138,176,139,204,170,42,225,80,152,112,40,68,52,18,38,28,82,221,
  239,85,86,214,240,198,27,239,49,53,53,78,95,223,45,70,71,134,88,93,91,35,149,76,145,76,106,164,82,214,50,132,223,41,228,100,203,228,231,91,
  234,195,43,203,203,150,167,198,73,63,246,253,83,21,53,144,106,157,121,168,158,122,176,71,180,140,192,12,221,31,82,148,158,226,189,181,212,65,80,212,
  210,191,220,231,45,197,9,59,133,219,222,188,105,29,243,218,234,42,166,105,18,141,70,201,207,203,39,149,76,217,114,2,4,206,241,113,137,85,242,213,
  253,205,122,76,79,115,76,85,54,219,119,72,72,72,85,9,169,42,225,80,136,72,56,76,52,18,65,117,201,138,66,107,235,46,90,90,218,24,30,238,
  167,183,231,22,227,227,99,36,18,9,146,182,237,107,118,34,130,105,122,182,135,16,168,138,160,176,192,82,182,94,93,93,69,215,116,203,142,128,104,36,
  66,109,109,61,211,83,147,52,212,55,97,24,58,187,119,239,101,98,124,12,197,241,114,9,129,34,20,132,98,17,116,43,38,208,254,204,231,135,176,125,
  152,152,134,129,170,170,28,57,114,156,245,245,53,138,138,138,249,254,187,175,8,135,34,52,53,181,48,57,57,65,77,117,29,138,162,208,210,210,206,220,
  236,12,235,107,171,124,248,215,127,227,232,177,147,156,57,243,42,29,187,58,249,252,243,143,88,91,93,227,175,127,249,55,142,30,59,193,11,47,188,194,
  174,246,78,190,252,242,19,123,25,222,178,253,68,34,78,56,28,38,63,63,159,213,149,21,235,184,132,37,141,161,8,19,67,234,42,237,140,40,201,38,
  144,144,120,252,6,137,244,217,159,27,184,29,10,185,51,234,72,36,76,40,156,121,11,87,87,215,81,93,93,199,228,228,40,131,131,61,12,14,246,178,
  190,190,73,42,165,217,153,107,158,254,145,105,130,170,42,68,163,81,0,55,198,33,211,155,101,211,17,191,134,141,240,116,153,210,199,56,103,102,171,40,
  34,67,125,59,64,0,221,101,176,96,59,248,85,187,253,179,228,32,33,203,222,94,126,232,186,102,215,120,179,226,90,84,85,205,186,124,248,56,16,144,
  116,13,44,167,82,188,127,153,241,105,84,83,206,70,150,253,129,216,86,208,181,231,69,141,68,66,68,163,33,84,223,36,193,250,174,66,115,243,46,26,
  26,90,24,29,29,100,96,160,155,225,161,1,18,137,148,101,251,186,17,40,95,227,108,63,226,218,126,34,112,45,202,202,202,201,207,47,96,163,112,157,
  142,221,123,208,52,203,235,3,254,9,128,216,214,19,22,212,86,178,190,111,24,6,35,35,195,76,79,77,50,61,61,129,166,165,168,174,169,163,168,168,
  152,249,185,57,58,58,58,73,36,18,180,182,180,113,225,252,79,128,21,131,248,211,143,223,113,253,218,21,254,233,159,255,11,123,247,29,224,210,197,159,
  137,199,55,249,233,199,239,185,121,227,26,255,248,159,254,51,251,246,31,224,226,175,23,48,77,72,105,214,253,46,20,107,146,229,19,40,8,220,111,50,
  161,65,18,37,9,137,39,110,160,240,19,164,96,182,143,151,229,102,13,24,65,143,82,58,106,106,26,168,174,174,167,173,173,147,254,254,46,250,251,122,
  72,218,234,194,134,238,197,31,9,161,184,179,115,211,52,80,132,64,207,168,211,166,96,218,1,212,206,32,230,73,16,144,49,104,100,100,169,97,90,2,
  145,102,208,91,144,78,112,210,151,211,156,148,254,116,226,148,238,93,9,46,165,137,45,7,46,85,85,50,136,157,16,34,48,120,62,66,170,148,225,137,
  11,202,33,60,125,165,39,178,165,239,251,175,167,91,94,199,142,75,114,200,82,56,18,118,175,101,58,84,53,68,115,243,46,234,106,155,104,107,29,166,
  175,255,54,35,195,67,182,87,201,204,168,121,102,145,8,75,251,8,97,121,125,76,211,164,177,169,133,217,217,25,62,251,244,99,116,67,167,176,160,128,
  63,253,249,95,168,173,173,103,101,101,5,128,141,205,141,128,79,198,178,37,157,120,124,51,112,93,157,235,103,253,221,160,191,191,151,153,233,105,87,3,
  169,189,109,55,99,99,163,124,249,229,167,150,135,183,160,144,127,254,167,255,133,178,242,10,102,103,102,220,45,45,47,47,177,177,177,78,52,18,69,119,
  133,47,77,150,150,22,217,88,95,39,22,205,177,61,91,120,137,14,100,215,227,150,171,111,146,40,73,72,60,145,131,70,122,12,141,191,22,149,63,174,
  40,108,199,90,168,33,229,142,219,172,170,170,163,188,188,138,93,187,246,210,219,123,147,161,161,62,52,123,134,237,102,3,57,51,120,167,30,27,65,146,
  4,6,173,173,237,180,180,182,90,250,49,90,138,201,137,9,122,123,187,172,44,33,160,177,177,153,93,187,58,44,79,146,80,88,94,94,226,236,217,179,
  236,217,187,155,100,34,73,127,127,95,134,71,73,85,21,142,28,57,70,60,17,167,187,235,22,249,249,5,236,219,183,159,91,183,111,177,180,184,132,105,
  154,228,23,20,176,167,115,15,183,110,223,102,105,113,209,37,14,66,8,10,10,172,64,216,197,197,133,52,162,25,20,238,179,226,70,156,148,113,37,205,
  91,246,232,137,71,186,55,201,242,196,89,228,210,26,212,157,225,78,60,149,117,186,210,109,223,203,204,180,20,180,195,97,149,144,61,65,8,135,67,68,
  194,33,151,220,111,133,112,36,66,83,243,46,170,107,26,152,222,53,78,111,207,13,38,38,199,130,226,169,186,225,217,190,237,2,50,77,147,156,156,92,
  90,91,119,113,243,198,53,214,214,86,49,12,147,141,245,77,102,103,166,217,187,247,57,190,253,238,107,84,53,196,91,111,189,199,218,218,26,33,85,229,
  202,213,203,108,108,172,83,85,101,149,0,210,116,29,77,75,241,203,207,23,24,31,31,7,172,140,188,72,36,66,52,18,67,211,116,20,69,16,206,137,
  177,171,99,55,63,124,255,45,27,27,155,152,166,201,250,250,6,139,139,11,116,116,236,33,26,201,225,249,231,95,96,102,102,154,210,210,82,76,211,228,
  226,197,95,104,107,109,231,200,209,227,204,206,206,80,86,86,142,80,20,46,95,190,184,101,65,105,73,142,238,135,40,201,134,147,144,120,228,51,234,244,
  129,194,10,136,86,210,52,95,20,66,97,149,112,196,158,89,71,84,216,225,13,172,170,33,42,42,106,40,45,173,96,215,174,253,116,117,93,97,98,194,
  10,220,214,52,61,40,104,169,224,166,207,43,8,12,195,42,251,112,230,165,151,81,85,171,51,206,207,47,224,216,241,227,28,63,113,146,143,62,250,11,
  107,107,171,236,219,183,143,214,214,93,220,190,125,211,202,187,23,130,112,68,229,224,193,67,172,173,173,49,52,60,232,155,219,90,231,89,86,86,202,107,
  175,191,65,34,145,96,104,168,159,148,150,164,99,247,30,138,75,74,248,226,139,79,209,117,131,227,199,79,208,220,220,194,149,171,151,173,180,102,211,202,
  250,59,122,244,24,175,190,250,58,195,195,67,252,207,255,249,63,208,117,205,77,127,22,138,176,101,2,124,30,44,91,55,71,24,6,66,193,126,88,223,
  125,92,60,74,254,101,70,195,48,65,183,82,213,133,98,13,178,152,230,211,37,103,32,238,108,251,161,144,138,106,123,147,212,144,234,222,3,59,29,187,
  162,209,24,141,141,109,212,212,52,48,57,57,66,87,215,53,230,230,102,220,229,56,87,197,219,150,153,48,49,73,105,41,126,250,233,123,70,71,71,72,
  105,186,77,78,117,190,250,250,75,242,243,243,217,220,220,224,179,207,62,162,176,176,200,38,182,42,171,171,171,76,76,140,243,233,167,159,16,141,90,129,
  213,186,174,179,190,177,142,110,88,133,116,87,215,86,249,251,223,63,100,98,114,2,77,215,17,6,132,52,157,175,190,250,156,190,190,62,82,118,214,154,
  16,240,233,103,31,17,141,198,152,158,158,226,118,215,77,138,139,75,232,31,232,231,246,237,155,22,121,27,55,201,205,203,167,184,184,152,193,193,126,186,
  186,110,179,190,190,102,183,99,186,116,135,79,70,67,1,12,225,190,151,28,64,122,148,36,36,158,16,194,132,75,84,156,129,194,203,32,83,92,143,82,
  56,172,186,106,194,170,170,98,216,29,240,78,97,17,166,106,42,42,222,102,106,106,140,238,238,107,76,77,141,99,154,70,6,97,179,30,166,235,153,81,
  20,193,200,200,48,93,93,183,80,20,133,235,215,47,241,167,63,253,51,175,191,254,38,127,255,251,127,16,141,198,44,193,190,115,223,185,169,250,78,42,
  182,183,236,21,36,74,251,246,29,96,108,108,148,194,194,34,58,58,118,211,219,219,205,47,191,156,229,205,55,223,227,215,95,47,96,154,38,123,247,238,
  227,203,47,63,39,149,74,185,30,163,72,36,194,202,202,50,61,61,93,228,229,229,17,14,135,48,12,61,16,100,30,204,140,3,195,208,92,17,78,207,
  219,228,16,68,243,17,93,247,108,245,202,60,21,114,33,64,215,241,41,158,227,86,127,127,122,38,9,91,219,190,99,63,142,55,213,145,181,184,23,219,
  15,135,35,52,54,182,83,87,215,204,200,200,0,221,221,215,152,159,159,13,216,190,115,13,54,54,54,185,125,251,166,91,160,217,90,162,22,76,76,76,
  184,219,187,121,243,134,107,91,254,107,119,241,226,47,91,10,154,110,110,198,185,113,227,70,224,190,95,91,91,231,234,213,171,25,199,59,50,50,234,190,
  62,127,254,124,154,60,134,201,226,226,18,231,207,159,115,239,47,135,92,90,223,147,117,12,37,81,146,144,120,202,188,74,206,50,148,95,49,59,20,82,
  169,168,168,176,220,234,54,209,240,199,40,133,195,42,37,37,229,247,188,223,234,234,122,170,171,235,25,27,27,36,18,137,249,200,133,149,25,227,16,26,
  225,171,117,229,175,173,101,154,38,23,46,156,229,221,119,255,64,56,28,38,149,74,145,151,87,64,89,89,5,166,105,18,143,199,89,89,89,9,164,71,
  123,75,97,150,87,168,169,169,153,43,87,46,82,93,93,195,238,221,157,244,247,247,48,60,60,200,212,212,4,71,142,28,99,115,51,206,204,204,52,189,
  189,221,129,101,178,68,34,78,79,79,55,165,165,101,180,183,239,74,35,28,193,84,123,39,235,173,185,185,149,213,213,21,20,69,193,48,112,99,180,156,
  229,199,199,5,206,0,59,61,61,197,210,210,146,253,169,129,105,42,152,166,145,17,235,245,52,216,126,186,90,124,40,164,82,87,87,71,97,97,17,66,
  152,46,57,242,180,192,20,10,10,138,239,105,159,170,26,162,165,165,131,250,250,102,250,251,187,136,68,162,25,237,31,20,108,53,182,20,62,77,39,185,
  233,175,189,143,124,213,222,68,240,111,217,18,33,178,37,38,108,5,195,48,109,146,233,76,64,148,180,2,193,178,143,149,68,73,66,226,41,240,38,165,
  151,40,201,201,137,209,208,208,200,107,175,253,142,250,250,134,223,116,255,245,245,45,238,107,191,22,140,83,100,54,179,51,247,2,176,215,214,172,244,251,
  220,220,60,146,201,56,123,247,238,183,2,194,21,133,241,241,81,126,254,249,188,207,83,165,248,74,67,64,117,117,13,209,104,140,201,201,113,214,215,87,
  121,235,173,15,200,201,201,97,115,115,147,159,126,250,142,63,253,233,95,216,220,220,228,223,255,253,127,108,217,225,103,15,132,15,6,197,235,186,70,65,
  65,33,191,251,221,251,79,148,93,244,244,116,241,217,103,31,51,55,55,23,8,230,54,205,32,121,125,26,108,223,31,139,23,141,70,104,107,107,231,221,
  119,63,160,184,184,228,55,219,119,56,28,161,179,243,64,22,146,228,47,240,236,21,108,206,230,5,115,238,7,127,184,244,86,138,244,91,101,134,110,69,
  182,182,146,206,72,207,166,179,38,3,206,223,85,192,75,4,144,120,64,68,73,234,40,73,72,60,194,129,194,214,92,81,157,217,180,162,162,42,42,197,
  69,197,188,249,230,59,84,87,215,48,54,54,66,42,149,202,8,240,22,64,110,94,1,69,69,247,63,152,104,90,138,80,40,108,39,202,8,87,135,72,
  4,210,102,124,159,216,154,69,177,104,12,33,4,137,248,38,145,112,148,137,137,113,174,94,185,4,192,250,198,90,96,27,138,176,11,143,216,203,94,77,
  141,205,148,150,150,242,210,153,87,81,20,149,162,162,34,26,26,154,233,233,190,205,252,252,60,243,115,115,196,19,113,22,22,230,237,227,177,179,190,178,
  232,30,89,199,236,28,151,179,79,91,87,73,40,164,82,41,102,103,167,72,38,147,129,229,56,71,51,202,124,68,122,50,254,234,244,126,148,149,150,211,
  209,209,201,216,216,40,63,254,240,61,134,98,21,0,54,133,98,55,189,241,212,216,190,226,100,183,217,182,159,155,147,203,233,211,103,40,46,46,97,114,
  98,156,120,34,110,217,188,189,188,36,236,251,160,176,176,152,188,188,130,251,58,6,93,215,136,199,227,228,229,229,123,118,110,63,76,195,106,115,67,55,
  124,164,195,204,144,178,224,174,109,71,100,20,166,13,122,128,50,137,149,227,133,218,210,43,135,106,155,190,137,105,151,240,9,152,149,41,216,238,159,132,
  244,40,73,72,60,57,3,135,163,230,171,8,194,225,48,186,174,243,197,231,31,19,79,108,162,40,138,173,163,164,216,10,219,97,234,27,90,56,126,252,
  165,123,222,231,204,204,4,189,61,183,168,173,109,164,165,181,3,211,78,155,118,211,251,241,119,232,206,123,207,179,177,119,223,115,44,46,46,144,72,38,
  9,135,35,44,45,45,50,54,54,98,125,207,150,5,64,128,161,27,129,153,116,56,172,210,216,216,196,237,219,183,232,237,237,34,28,10,19,141,197,104,
  109,105,167,171,235,22,166,97,178,177,185,65,34,17,183,182,99,250,11,240,250,6,148,180,248,144,192,204,220,38,64,170,170,178,188,180,200,55,223,124,
  110,121,192,12,235,115,67,183,107,216,233,78,86,217,163,242,166,120,222,68,103,204,58,113,226,52,199,143,63,79,40,20,242,41,156,27,1,175,217,83,
  181,252,230,243,82,58,117,12,1,190,255,254,107,22,22,230,173,250,110,97,219,246,195,150,237,239,217,115,136,221,105,30,161,157,194,52,13,70,71,7,
  233,234,186,70,85,85,61,7,15,30,119,101,40,252,229,112,12,211,179,189,76,239,142,121,231,243,74,83,144,247,39,111,248,179,211,252,222,165,244,103,
  235,187,119,178,35,195,93,14,55,205,32,241,146,144,68,73,66,226,169,25,44,2,65,212,182,48,157,21,143,147,32,190,25,71,85,21,180,144,130,166,
  133,108,33,69,29,93,215,239,122,95,166,105,178,180,52,79,79,247,117,198,198,134,216,216,220,164,172,172,42,224,101,241,113,35,247,181,170,168,132,195,
  17,20,69,37,55,47,151,195,135,142,210,222,190,155,15,63,252,159,24,186,69,138,138,138,138,41,46,46,65,8,5,77,211,88,88,88,192,52,160,176,
  176,144,146,146,50,132,16,108,110,110,82,94,94,65,81,81,9,63,252,248,29,83,147,19,56,177,21,175,188,242,6,5,249,69,246,224,24,66,215,194,
  129,146,36,25,131,136,237,137,112,102,252,233,229,76,156,177,204,48,77,226,155,113,54,55,226,1,29,29,107,105,69,183,244,154,30,25,49,86,92,114,
  236,40,165,59,215,213,180,11,190,58,101,49,116,227,105,181,255,96,28,155,105,19,147,100,50,65,34,17,39,165,9,194,90,8,45,165,98,68,117,12,
  83,15,144,151,187,33,72,83,147,86,18,195,244,204,36,241,205,4,165,165,85,174,141,100,216,145,91,63,144,192,132,33,72,152,178,159,143,115,149,45,
  47,148,125,95,43,130,144,26,194,48,141,0,185,247,215,45,204,92,78,221,126,127,138,130,155,0,96,37,37,72,146,244,224,137,146,244,186,73,72,60,
  74,55,146,47,157,29,95,122,187,215,217,170,33,75,236,209,48,13,208,65,81,44,79,136,166,235,110,205,169,157,192,48,12,86,86,150,232,237,189,206,
  240,112,63,137,68,2,77,211,173,96,102,103,214,235,251,103,152,190,119,166,201,196,228,4,77,77,205,252,254,15,127,68,85,67,108,108,172,241,63,254,
  199,255,205,204,204,148,93,43,107,138,163,71,79,240,187,55,223,67,81,20,214,214,86,249,232,163,15,25,29,27,230,232,209,19,188,243,238,7,168,170,
  194,204,204,44,83,83,147,12,14,245,51,61,61,141,102,147,130,177,241,49,166,166,39,41,44,42,98,118,110,150,249,249,121,18,137,4,186,161,187,51,
  124,255,160,2,176,188,188,204,204,236,140,229,25,50,77,4,158,23,203,30,50,188,129,88,17,190,109,249,131,117,245,71,54,176,40,166,64,197,196,68,
  65,65,1,5,20,59,160,222,34,130,126,169,3,111,176,21,230,83,224,82,18,164,217,188,240,210,215,157,76,56,85,117,189,152,186,161,163,152,2,77,
  215,81,117,197,213,63,218,9,52,45,197,220,220,52,61,61,215,25,31,31,65,211,44,105,0,77,211,221,157,153,62,123,15,164,206,11,239,190,240,79,
  108,182,91,174,53,77,235,58,186,4,71,128,26,82,57,112,224,32,167,79,159,166,171,235,54,95,127,253,181,175,140,142,109,215,89,98,207,132,239,222,
  204,122,95,155,160,248,142,208,244,181,109,122,91,251,207,71,202,3,72,143,146,132,196,19,227,73,202,246,58,155,23,200,17,29,212,117,29,77,19,118,
  246,214,157,103,213,150,7,105,129,193,193,110,250,250,110,17,143,39,60,79,138,225,137,24,58,51,90,135,72,152,102,112,201,235,147,79,62,38,22,139,
  17,137,132,137,199,227,110,58,190,131,115,231,206,241,243,207,23,236,207,4,166,105,144,74,105,246,231,63,227,23,76,180,182,171,7,188,87,115,115,115,
  252,235,191,254,95,246,192,166,241,197,23,159,97,24,38,154,102,4,189,68,62,92,185,114,153,171,87,175,96,154,150,20,65,112,233,33,56,179,118,82,
  189,157,115,247,19,165,71,17,248,234,44,51,165,81,167,45,234,253,165,123,42,158,190,123,192,203,84,76,183,123,195,110,23,211,213,252,178,18,3,118,
  74,144,166,232,235,235,98,112,176,39,40,54,105,164,47,171,153,59,94,178,186,83,16,125,182,130,203,185,185,185,52,53,53,161,235,58,165,165,101,40,
  138,112,179,47,183,91,74,189,155,125,221,249,184,101,22,156,36,74,18,18,79,252,160,145,61,59,198,26,200,253,68,65,179,7,130,237,151,222,230,231,
  103,24,30,238,163,183,215,170,52,238,47,227,224,185,235,149,140,37,7,111,144,242,149,33,49,13,214,215,215,217,216,8,150,153,240,81,145,45,143,35,
  149,74,237,104,16,50,77,205,119,28,4,6,178,108,191,183,200,26,190,148,232,224,210,67,48,110,137,12,146,228,120,212,30,85,246,152,159,208,133,66,
  94,137,11,158,145,98,165,119,30,228,179,197,8,233,110,169,144,173,219,213,96,106,106,140,129,129,30,122,123,187,108,113,73,61,64,144,178,217,251,131,
  54,3,63,9,220,216,216,224,239,127,255,144,215,94,123,157,146,146,82,55,171,244,126,84,214,131,186,97,65,85,122,71,111,75,81,164,203,72,18,37,
  9,137,167,128,28,165,19,35,255,179,227,5,177,62,115,60,50,86,113,215,173,136,210,252,252,12,67,67,189,12,14,246,178,180,180,228,86,80,247,167,
  58,123,169,250,2,205,86,4,206,86,52,214,33,106,233,49,26,134,33,182,61,143,108,179,217,157,204,150,179,13,94,219,13,36,150,238,76,166,110,76,
  122,48,172,181,220,22,36,73,78,85,249,71,227,81,218,202,107,164,102,37,122,207,10,97,242,159,178,183,228,154,233,233,212,180,236,222,212,137,137,17,
  6,6,186,25,28,236,103,115,211,42,6,237,47,138,27,76,173,183,72,188,101,251,161,0,65,245,107,113,249,3,178,239,135,20,43,138,87,128,218,179,
  77,51,112,60,247,178,139,96,173,68,145,69,200,52,155,20,129,236,123,119,76,148,100,106,160,132,196,35,28,32,210,211,116,77,95,170,187,211,161,217,
  105,202,86,112,169,192,80,132,149,185,101,152,104,169,32,81,90,92,156,163,167,251,38,35,35,3,44,47,175,184,131,68,42,165,145,210,116,171,24,46,
  118,104,130,240,2,136,227,155,113,0,114,114,114,51,142,204,159,46,157,54,215,191,171,78,124,171,65,102,171,14,60,240,249,22,226,121,174,102,146,255,
  159,253,94,17,130,88,52,134,162,88,129,229,137,120,2,67,183,179,221,52,235,97,248,150,97,30,54,20,69,128,105,216,97,34,2,67,152,40,2,2,
  142,18,51,155,149,60,29,105,221,129,179,112,194,115,124,233,249,206,249,187,25,147,66,96,26,158,237,167,11,133,78,76,140,208,213,117,157,201,137,113,
  54,54,214,73,166,116,203,238,237,73,66,96,153,90,8,91,150,64,176,185,97,217,126,44,150,3,120,1,245,138,112,228,11,12,76,97,149,244,185,219,
  115,115,238,103,225,251,204,21,154,76,251,124,251,146,206,91,223,87,214,49,218,18,35,246,179,64,144,19,203,33,26,141,146,72,36,72,37,19,129,123,
  217,223,214,146,3,72,143,146,132,196,19,15,19,210,50,185,132,157,218,174,160,219,157,255,210,210,34,215,174,93,100,112,160,151,141,141,13,119,137,73,
  211,236,193,34,165,7,130,150,189,146,7,150,18,242,210,242,34,0,229,229,21,132,84,213,42,22,154,150,222,236,204,240,239,62,102,194,220,214,131,180,
  29,113,10,112,165,59,168,35,7,72,147,197,4,41,43,175,0,96,99,125,157,141,141,13,95,121,16,35,144,6,254,40,60,55,86,57,18,51,163,4,
  134,153,133,17,62,171,179,127,195,47,180,105,171,71,41,138,129,97,120,17,223,19,19,99,92,189,242,43,147,147,99,196,227,113,215,246,83,54,81,114,
  188,135,126,15,163,95,224,114,105,201,178,253,146,146,82,98,177,24,235,235,27,40,66,96,248,236,255,126,150,201,28,41,13,119,233,60,253,122,155,230,
  93,95,231,204,184,46,17,184,95,133,109,251,150,100,199,18,43,43,43,129,123,72,122,147,36,81,146,144,120,202,152,146,157,205,226,186,207,173,94,78,
  215,13,150,22,151,249,241,199,239,184,118,245,34,155,241,184,251,253,96,12,142,142,166,25,246,178,5,62,189,26,19,85,181,222,207,207,207,145,74,165,
  104,108,108,38,22,203,33,149,210,236,108,29,145,37,22,137,187,242,16,109,113,74,247,244,187,108,3,70,102,229,121,199,171,4,45,45,173,0,204,204,
  78,219,154,56,143,159,198,76,182,32,117,137,204,246,49,77,43,133,95,215,45,130,57,53,57,201,167,159,252,141,174,174,155,214,132,193,254,78,246,229,
  85,211,93,106,182,8,146,149,105,40,132,96,105,105,145,245,245,117,234,234,234,41,44,40,100,115,99,19,83,81,80,84,19,213,244,19,105,37,195,243,
  184,211,235,230,15,216,118,234,215,89,63,189,59,226,146,30,36,238,216,187,247,80,220,12,194,166,166,22,132,16,86,1,96,77,115,143,253,81,77,12,
  158,108,162,36,189,110,18,18,143,14,98,155,135,255,59,224,165,236,155,134,219,177,222,238,186,69,79,111,183,175,142,154,25,200,92,243,151,96,240,151,
  97,80,20,197,77,205,54,76,193,204,204,52,211,211,147,212,215,55,210,216,212,204,237,219,55,48,17,168,88,133,108,21,67,96,162,128,48,124,153,58,
  247,19,132,154,109,25,77,236,248,183,126,146,164,170,10,138,170,160,134,172,103,235,33,40,40,44,164,181,181,157,84,42,197,208,80,191,237,149,48,108,
  217,3,35,152,10,254,56,93,123,182,120,222,206,70,158,54,219,79,179,127,139,36,25,54,209,53,0,157,159,127,249,217,174,143,24,84,90,183,98,250,
  244,128,237,59,68,75,81,44,117,120,103,31,134,34,88,90,94,102,116,116,136,206,206,125,180,180,182,49,59,55,19,176,125,195,52,65,152,160,27,246,
  113,121,194,142,2,177,165,183,211,47,231,224,63,223,68,50,129,26,15,5,174,173,80,0,83,88,75,175,89,98,249,211,75,152,4,75,30,169,132,156,
  98,217,33,5,85,21,68,163,17,90,91,219,1,232,235,235,133,108,146,31,143,210,246,165,71,73,66,66,226,55,152,87,219,157,115,230,114,145,110,47,
  147,57,240,103,170,249,151,152,60,101,96,111,9,194,34,83,130,245,245,117,250,251,123,169,171,107,224,212,169,23,232,239,239,33,145,72,102,4,72,91,
  29,180,95,44,79,60,84,239,76,122,249,136,96,165,121,213,29,56,21,69,225,216,177,147,228,230,230,49,62,62,202,208,208,112,32,139,207,11,80,151,
  150,245,248,123,148,210,51,48,189,152,50,93,15,6,90,59,65,251,233,34,165,222,146,147,192,145,26,16,194,218,102,50,153,160,167,167,155,93,187,58,
  57,124,248,24,93,93,183,88,94,94,206,168,167,38,132,17,8,38,223,206,19,228,95,178,118,38,49,14,177,249,245,215,95,80,20,213,150,7,80,92,
  145,72,71,170,67,81,204,29,111,87,81,172,2,194,206,125,96,41,184,11,14,28,56,76,121,121,5,203,203,75,244,245,245,4,213,246,93,207,170,180,
  45,73,148,36,36,158,210,65,195,79,128,156,165,52,111,106,104,102,12,44,193,204,49,191,222,144,129,97,88,29,179,97,152,92,189,122,133,125,251,14,
  80,89,89,197,201,147,167,249,241,199,239,236,237,42,118,234,186,163,69,36,182,200,70,251,173,123,94,145,70,146,210,102,213,33,197,141,61,169,175,111,
  228,192,129,195,0,156,59,247,19,186,30,204,122,74,87,68,150,120,220,237,222,33,19,134,47,11,212,90,134,243,219,126,250,242,146,233,83,219,182,108,
  198,250,155,83,242,195,218,158,66,111,111,15,99,99,35,52,53,181,240,194,11,47,241,233,167,127,119,109,223,251,173,64,215,119,106,251,34,35,198,207,
  121,191,182,182,26,176,95,103,59,14,65,242,47,175,239,212,246,157,201,130,162,8,202,202,202,57,118,236,36,138,162,112,238,220,143,108,110,198,183,36,
  142,18,146,40,73,72,60,101,179,106,2,34,144,214,192,159,109,64,33,48,195,78,239,20,157,129,195,155,85,91,131,197,234,234,42,63,253,244,61,31,
  124,240,71,78,156,56,205,210,210,34,55,110,92,195,170,70,110,43,69,11,37,48,171,222,110,255,15,206,139,180,69,208,182,127,233,205,87,10,164,188,
  188,130,223,253,238,29,98,177,24,215,175,95,161,167,167,59,48,80,108,85,14,69,226,177,181,126,223,4,193,244,45,37,111,29,47,180,85,28,78,240,
  30,178,74,126,232,186,65,60,190,201,217,179,223,83,85,85,195,129,3,135,89,88,152,231,231,159,207,17,44,130,107,218,228,42,187,71,41,93,77,59,
  27,73,242,147,30,127,124,210,86,114,24,91,217,190,223,163,228,120,146,20,69,80,80,80,192,155,111,190,75,81,81,49,3,3,125,92,191,126,53,205,
  171,108,100,120,151,36,118,64,148,100,106,160,132,196,163,195,78,42,122,7,210,244,237,103,43,69,58,123,134,88,70,231,151,182,132,16,144,32,48,77,
  55,221,218,48,76,110,221,188,65,77,117,45,39,78,62,207,239,222,120,135,144,26,230,234,213,75,8,76,12,33,48,21,19,195,80,210,102,234,230,195,
  111,53,97,213,121,83,68,48,144,181,178,170,154,183,223,126,143,138,138,74,198,198,70,249,230,235,47,221,10,240,166,129,85,147,206,77,65,127,180,189,
  223,246,87,94,164,125,39,251,47,158,126,219,39,216,38,246,181,179,174,227,246,182,47,16,153,165,64,92,169,11,48,133,233,62,27,186,201,208,224,16,
  103,207,254,192,107,175,253,142,151,94,122,13,85,85,57,127,238,39,91,106,66,113,39,20,166,226,159,36,152,119,182,83,59,41,66,164,17,38,47,155,
  207,190,15,239,102,155,182,237,91,19,5,235,185,164,180,148,223,189,241,14,141,141,205,204,205,205,242,213,151,159,145,76,164,172,182,50,172,115,117,227,
  159,76,57,242,75,143,146,132,196,83,236,90,114,50,140,211,151,21,50,191,154,93,204,209,138,77,114,130,142,76,192,147,2,208,129,239,190,251,154,72,
  36,194,161,195,71,121,243,173,119,169,172,170,226,194,133,179,172,174,172,160,235,186,187,124,161,170,230,67,21,70,244,102,231,222,128,225,44,65,68,99,
  49,246,237,125,142,211,167,207,144,155,151,199,228,228,56,31,127,244,87,214,214,214,172,192,94,159,39,194,148,203,15,79,162,217,187,207,134,175,220,206,
  189,216,190,187,52,109,122,90,76,206,178,218,207,23,206,17,139,70,121,254,244,25,206,156,121,149,202,202,106,126,252,241,91,150,22,23,209,52,221,93,
  166,222,74,253,61,221,163,228,39,75,254,247,14,33,218,206,155,116,167,109,58,177,79,177,88,148,182,182,93,188,120,230,21,138,139,75,88,92,88,224,
  147,143,255,198,236,236,172,109,251,134,180,125,73,148,36,36,158,173,1,195,180,215,225,238,135,164,184,157,178,97,98,96,216,65,177,86,39,156,76,166,
  248,236,179,143,89,91,91,229,232,177,147,28,58,116,148,221,29,123,184,113,243,26,131,131,253,44,47,45,145,76,38,220,56,17,255,64,246,219,18,37,
  255,172,90,16,14,135,201,205,203,165,190,174,145,125,251,159,163,170,170,6,211,52,233,235,235,225,139,207,63,102,113,113,201,83,35,215,141,172,75,134,
  114,192,120,114,72,18,102,166,30,215,253,216,190,161,27,174,151,199,175,242,253,221,119,223,176,182,190,206,11,47,156,97,247,238,61,180,182,182,115,243,
  230,53,250,251,123,89,92,88,176,10,53,235,58,65,125,176,108,182,106,121,58,173,36,59,79,219,43,176,244,150,181,54,161,185,197,246,130,182,31,203,
  137,81,83,83,199,222,189,207,209,208,208,8,192,216,216,40,159,127,254,49,83,147,83,94,214,159,47,235,85,218,254,189,18,37,233,127,147,144,120,116,
  216,169,60,128,176,189,63,233,85,192,205,187,79,241,53,29,225,60,28,201,1,16,166,129,110,216,219,213,173,207,127,248,241,59,198,39,198,56,126,252,
  20,13,13,77,28,63,126,138,227,199,79,177,190,190,198,242,242,50,154,150,74,59,208,135,19,204,13,86,41,136,188,188,124,74,74,74,221,191,206,204,
  76,113,253,250,85,46,93,250,149,100,50,229,205,166,13,3,221,208,49,76,75,26,192,73,145,126,228,125,223,93,93,251,45,126,243,204,216,254,22,237,
  192,189,219,190,16,182,119,202,208,45,50,99,219,191,9,252,250,235,207,76,77,77,112,226,196,41,154,155,219,56,116,232,40,135,14,29,101,115,115,131,
  229,229,37,146,201,228,29,108,63,88,220,215,139,53,50,237,96,114,43,222,47,27,97,201,190,252,22,148,7,200,205,205,163,180,180,204,221,238,252,252,
  28,183,111,223,224,215,95,127,182,4,103,13,3,205,181,127,203,163,234,151,6,144,227,190,244,40,73,72,72,236,112,102,109,24,78,38,144,145,54,51,
  182,102,184,253,253,125,12,15,15,209,218,218,78,107,107,27,149,149,213,148,149,149,81,91,91,247,200,143,63,149,74,49,55,55,203,204,204,52,35,35,
  67,12,12,244,179,180,180,228,46,51,120,207,58,186,110,102,73,21,151,51,106,105,251,118,220,142,93,208,217,18,132,180,236,127,116,116,148,241,241,49,
  90,91,219,104,105,105,163,186,186,134,178,178,114,170,171,107,31,249,241,107,154,198,226,226,2,179,179,51,140,142,14,51,56,216,207,236,236,92,134,118,
  154,245,58,88,4,91,218,190,36,74,18,18,18,59,28,40,156,212,100,175,4,150,97,119,194,94,12,146,149,101,100,210,211,211,69,111,111,55,249,249,
  5,228,231,23,16,137,68,92,161,191,157,10,69,62,168,227,246,19,165,205,205,141,0,57,242,103,182,5,9,147,41,7,10,137,172,182,159,73,150,76,
  187,128,173,229,185,236,235,235,99,96,160,159,188,188,60,242,243,243,137,218,245,3,183,179,125,39,46,201,210,53,242,178,52,171,170,170,57,125,250,21,
  102,102,166,56,127,254,7,116,93,11,20,157,118,36,63,238,100,255,154,166,177,185,185,193,234,234,42,137,68,210,71,132,210,137,146,153,177,228,38,33,
  137,146,132,132,196,125,144,37,75,245,219,196,52,21,119,134,109,45,3,88,66,118,171,171,43,172,173,173,6,2,75,157,129,225,183,63,94,247,149,69,
  235,12,191,102,84,58,65,202,20,219,124,20,133,111,37,158,60,178,228,217,190,233,214,105,51,77,133,181,181,53,214,215,215,119,100,251,158,108,69,80,
  186,194,82,21,135,120,124,131,137,137,209,109,237,53,187,237,91,246,31,36,87,193,223,166,79,26,164,118,152,36,74,18,18,18,191,201,128,225,175,204,
  174,0,6,138,162,216,202,198,15,135,24,237,148,52,249,75,182,4,95,103,39,73,114,160,144,216,153,237,91,37,76,132,240,23,182,189,27,219,247,60,
  174,14,89,138,70,163,214,150,21,149,80,40,68,50,153,180,137,215,78,109,50,40,74,105,186,37,141,60,187,79,87,239,151,36,233,1,16,37,169,166,
  32,33,241,232,176,157,142,78,80,73,137,59,106,206,220,207,49,184,142,26,55,195,200,254,212,36,109,223,100,61,202,135,217,139,152,190,253,153,166,233,
  197,245,154,118,222,117,154,78,146,63,222,221,250,190,120,140,175,125,230,53,221,233,247,158,14,219,207,118,254,217,237,79,60,160,99,200,180,125,2,182,
  111,217,149,253,222,209,98,18,98,91,219,87,132,146,118,175,100,30,177,34,44,125,38,132,129,137,64,17,86,29,58,177,67,219,119,103,12,105,118,111,
  26,184,90,105,166,173,149,230,104,74,201,241,254,222,160,200,38,144,144,144,112,102,190,89,62,124,66,142,89,248,20,148,229,181,148,184,63,250,244,48,
  177,19,39,207,221,216,116,80,5,92,66,18,37,9,9,137,7,70,142,188,0,84,225,213,146,18,78,209,81,241,200,59,225,173,246,235,47,23,225,175,
  129,149,94,23,107,75,50,40,241,204,219,190,223,110,156,215,153,15,118,108,67,174,226,54,120,202,219,152,129,172,255,244,88,164,29,175,138,165,105,42,
  57,226,147,138,34,16,118,81,92,127,17,94,105,251,247,15,169,163,36,33,241,168,39,175,219,105,197,220,233,123,247,113,255,110,61,80,40,110,177,205,
  224,123,197,29,52,252,191,127,152,253,175,16,128,41,92,61,28,76,176,4,138,173,37,12,235,25,208,5,8,29,116,171,220,138,34,172,248,19,127,92,
  202,99,123,237,197,61,126,239,105,177,253,135,112,254,119,178,253,244,26,106,214,119,148,140,76,207,173,2,185,189,55,142,90,153,101,115,137,100,2,0,
  93,215,209,116,13,159,178,209,29,207,203,177,125,215,197,97,154,246,210,157,79,79,77,55,221,54,179,98,244,120,60,109,255,137,35,74,18,18,18,207,
  52,188,129,66,65,85,189,65,34,61,107,39,63,63,159,186,186,122,42,42,42,41,40,40,36,28,14,63,146,148,99,33,64,215,13,54,54,214,89,92,
  156,103,116,116,148,249,249,57,187,12,133,83,142,197,176,43,203,27,222,128,161,200,1,67,98,43,219,247,178,212,178,217,126,78,78,46,117,117,117,84,
  85,85,219,242,24,209,59,78,16,178,21,196,205,205,205,3,160,162,162,146,215,94,123,59,80,92,119,167,165,69,12,195,100,115,115,131,197,197,5,38,
  39,199,153,154,154,178,131,185,45,187,119,202,177,120,246,47,109,95,18,37,9,9,137,123,158,81,103,146,36,149,80,200,27,40,66,33,149,234,234,26,
  142,28,57,70,91,91,7,57,57,57,143,157,11,63,149,74,49,49,49,198,149,43,151,232,235,235,33,153,76,218,37,89,172,135,127,192,176,178,139,228,
  64,33,109,63,27,73,82,81,85,37,96,255,229,229,229,28,62,124,148,142,142,189,228,229,229,185,250,73,247,3,211,52,201,201,201,163,163,99,239,125,
  111,75,211,52,102,102,166,184,118,237,10,93,93,183,136,199,227,246,100,193,239,237,146,100,73,18,37,9,9,137,123,26,40,28,34,97,13,20,65,146,
  20,10,169,228,231,23,112,242,228,243,28,59,118,18,69,81,72,38,19,204,207,207,50,51,51,195,242,242,34,137,68,226,145,28,191,105,66,56,28,34,
  63,191,128,242,242,10,74,75,203,104,106,106,161,169,169,133,161,161,1,190,255,254,27,198,199,199,237,115,196,55,99,55,236,115,183,82,166,229,96,33,
  109,223,91,110,179,108,63,20,82,81,85,149,156,156,28,142,28,57,202,169,83,47,18,137,68,72,165,82,44,45,45,50,59,107,217,126,60,190,105,219,
  149,175,110,91,230,222,112,120,149,23,55,228,127,31,172,187,230,47,93,146,205,44,157,98,186,170,106,149,239,41,45,45,167,172,172,156,218,218,122,106,
  107,235,57,120,240,48,63,254,248,29,3,3,253,1,157,39,63,89,18,66,10,79,74,162,244,148,223,220,119,154,165,200,118,120,80,3,241,227,218,150,
  34,48,83,116,2,152,239,229,184,131,129,207,142,247,200,35,73,21,21,149,188,245,214,187,52,52,52,145,76,38,232,237,237,166,171,235,22,163,163,195,
  164,82,169,71,170,112,237,31,236,20,69,80,81,81,201,238,221,123,232,236,220,71,115,115,43,213,213,53,124,241,197,167,220,188,121,29,77,179,74,82,
  248,143,211,48,12,87,27,71,226,89,237,83,241,121,82,61,187,87,85,149,226,226,34,126,247,187,119,104,111,239,192,48,12,122,123,187,232,233,185,205,
  200,200,32,241,120,34,64,104,182,211,231,74,39,101,78,208,181,16,138,75,122,188,223,249,139,227,146,133,124,137,192,113,59,68,168,164,164,148,246,246,
  14,58,58,246,82,83,83,199,159,255,252,47,124,251,237,151,92,188,248,11,66,104,190,73,130,55,81,112,108,95,78,20,238,130,40,73,93,133,167,99,
  112,223,106,63,79,250,141,240,40,150,121,30,102,91,102,234,199,100,170,196,100,87,154,81,80,4,152,62,97,200,108,199,157,237,152,133,173,217,34,16,
  168,138,130,170,40,132,84,21,85,81,9,133,84,42,43,42,248,253,31,254,76,69,69,37,179,179,51,156,253,233,123,122,251,122,48,125,226,141,150,200,
  221,163,35,74,78,37,118,211,16,76,79,77,51,59,59,75,119,87,23,39,78,62,207,158,61,251,120,251,237,247,81,21,149,107,215,174,34,172,42,191,
  65,61,28,197,116,227,153,30,213,61,34,238,240,111,167,223,121,98,239,237,123,182,253,123,63,123,129,151,205,169,8,159,237,171,42,33,53,68,73,73,
  49,239,189,247,7,234,27,26,89,89,89,230,231,11,63,209,213,117,211,82,194,54,13,91,155,203,81,235,182,178,216,182,186,15,156,188,55,139,220,120,
  201,7,66,216,158,77,159,51,202,76,215,47,195,244,121,171,124,203,104,166,165,141,36,236,73,206,202,242,18,151,46,254,66,95,111,15,135,15,31,227,
  185,3,135,121,237,181,55,9,133,194,156,63,247,147,231,10,49,221,122,215,8,44,219,151,144,30,165,199,118,128,223,110,224,247,223,108,233,223,187,19,
  95,240,223,167,91,109,231,73,35,77,153,109,176,85,69,238,173,219,243,94,6,194,199,187,45,189,37,3,211,180,98,43,178,217,144,211,121,59,199,236,
  124,158,238,141,113,188,73,138,61,171,206,207,203,227,157,119,127,79,69,69,37,147,19,227,124,242,201,223,88,88,152,15,40,94,187,100,9,103,160,48,
  31,190,77,8,75,176,207,16,2,197,110,131,217,217,105,62,255,236,35,214,214,86,57,126,252,20,175,191,241,38,43,171,43,12,14,14,160,6,202,81,
  8,116,93,206,168,159,213,62,217,201,98,115,18,23,28,111,106,44,22,229,245,55,222,162,190,161,145,133,133,121,190,252,226,35,102,102,102,236,123,206,
  75,237,119,61,63,100,6,97,187,94,34,159,55,72,32,236,108,57,203,254,188,250,111,217,251,249,244,254,39,248,29,95,140,157,176,150,254,132,128,213,
  213,101,190,255,254,43,150,87,150,57,125,250,37,94,56,125,134,149,149,101,110,221,188,97,221,183,138,95,161,222,68,81,132,44,233,115,87,68,73,58,
  148,30,202,0,239,39,59,91,15,240,247,126,49,68,218,64,31,116,235,122,251,125,82,6,134,160,190,143,8,116,42,59,77,75,119,75,14,112,119,30,
  162,135,218,150,91,201,2,100,72,3,8,43,243,221,4,97,10,132,41,80,28,229,221,224,17,251,102,184,194,237,28,211,211,161,17,160,168,10,138,42,
  92,146,164,134,20,94,120,241,12,181,181,117,44,44,204,243,247,143,254,194,242,242,18,134,105,162,27,153,5,102,51,11,109,254,150,182,149,169,247,100,
  8,172,178,42,118,41,7,21,5,51,149,226,135,31,190,37,22,139,241,220,115,135,120,227,141,183,248,63,255,207,255,198,218,218,26,138,105,162,152,78,
  58,182,37,49,96,62,202,137,181,224,206,169,239,59,249,206,147,235,82,218,161,237,63,160,243,119,250,16,97,123,100,20,219,246,67,214,68,225,240,145,
  163,180,183,119,176,177,177,206,23,95,124,204,220,220,12,138,234,76,76,84,140,68,194,78,247,183,182,19,82,173,18,36,30,105,50,221,165,188,68,34,
  229,222,15,194,246,124,42,138,192,48,65,85,20,242,243,173,236,183,141,141,245,52,18,103,73,112,164,82,154,77,206,188,137,65,44,150,67,78,78,14,
  107,107,235,104,90,210,251,60,39,135,220,220,28,86,87,87,184,118,237,34,145,72,132,19,39,78,243,218,107,111,50,57,57,193,220,220,28,138,97,96,
  162,184,246,111,24,86,231,38,39,11,210,163,244,216,12,248,119,30,228,211,3,2,183,174,72,189,189,7,196,116,9,130,55,203,121,178,111,4,127,251,
  249,215,250,131,222,37,177,237,96,189,149,135,40,91,59,251,219,210,121,237,197,179,60,218,182,244,7,160,154,166,23,231,96,205,86,133,239,250,251,139,
  101,154,118,0,167,255,124,210,227,147,172,247,13,13,141,116,118,238,3,224,155,111,190,176,72,146,97,162,235,186,91,145,220,41,184,233,196,105,108,221,
  182,191,29,121,118,226,51,156,37,13,203,51,16,220,247,247,223,127,67,85,85,53,149,149,213,28,59,118,156,239,191,255,14,69,177,234,213,89,49,74,
  65,17,65,57,88,60,43,253,49,89,52,147,172,56,183,3,7,14,3,112,246,236,119,204,207,207,184,246,182,111,223,65,14,31,62,198,135,31,254,59,
  179,179,211,0,20,23,151,242,234,171,111,242,205,55,95,48,51,51,237,110,127,239,222,125,180,181,237,226,227,143,255,238,6,124,59,48,12,200,207,207,
  229,247,191,255,19,170,170,146,74,165,80,20,193,207,63,159,99,114,210,74,62,56,124,248,24,7,15,30,229,223,255,253,255,102,97,97,193,93,178,59,
  122,244,20,237,237,29,104,90,138,68,34,206,231,159,127,140,166,165,56,122,244,4,173,173,187,208,117,157,120,124,147,111,190,249,140,75,151,46,80,85,
  85,77,83,83,43,39,79,62,207,167,159,126,132,97,40,24,134,227,73,178,250,10,73,146,36,81,122,236,60,33,233,131,188,191,195,207,28,228,179,101,
  82,108,237,69,73,31,220,173,193,17,59,203,193,75,9,125,50,219,48,61,67,69,248,68,15,51,213,114,183,86,186,245,136,227,246,94,168,204,182,196,
  158,129,61,14,109,233,144,27,199,141,47,132,55,11,245,159,191,229,253,201,94,145,220,63,72,120,237,170,178,119,239,126,114,115,243,184,117,235,6,131,
  131,3,110,37,114,231,161,105,6,134,161,7,138,109,62,42,146,225,217,132,213,14,161,16,104,154,247,247,205,205,13,126,249,229,2,239,189,247,7,246,
  238,125,142,75,151,126,101,121,121,197,23,76,107,186,153,71,18,207,218,196,53,189,63,81,216,181,171,131,146,146,82,70,70,134,232,239,239,245,121,95,
  85,234,234,26,8,135,35,180,183,119,184,68,73,85,85,74,74,74,8,135,67,129,62,38,39,39,135,162,162,98,251,30,244,98,148,156,236,183,80,40,
  66,73,73,41,159,127,254,49,227,227,163,156,62,125,134,87,95,125,147,127,253,215,255,70,52,26,163,186,186,150,220,220,60,234,235,27,89,92,92,192,
  52,5,29,29,123,216,183,239,57,62,249,228,67,230,230,102,41,40,40,36,145,72,208,217,185,151,206,206,253,124,249,229,39,204,207,207,80,80,80,136,
  166,233,24,134,193,149,43,191,82,83,83,199,158,61,251,248,249,231,115,204,206,206,186,36,201,121,150,25,112,59,135,44,97,242,16,61,33,101,101,165,
  228,230,230,6,116,59,130,2,103,170,251,153,147,178,234,255,76,85,21,170,170,170,233,236,220,67,107,107,27,209,104,20,85,85,51,4,210,84,213,235,
  8,252,100,35,157,132,60,73,157,154,16,130,80,72,165,180,180,140,88,44,134,170,42,148,148,20,81,80,144,79,73,73,9,165,165,165,110,230,138,243,
  200,203,203,101,247,238,14,162,209,72,32,171,197,121,84,86,86,177,103,207,94,90,91,219,136,68,34,110,91,122,237,255,248,180,101,122,231,238,156,79,
  113,113,17,37,37,37,68,163,97,98,177,8,177,88,132,104,52,76,52,26,33,22,11,19,141,134,9,135,67,246,185,251,109,45,168,66,92,82,82,66,
  67,67,19,166,105,114,249,242,175,129,106,228,14,97,50,12,221,38,95,102,96,249,45,59,225,52,3,15,47,198,233,254,30,193,109,26,105,158,46,239,
  120,13,195,164,187,251,54,115,115,214,0,210,222,222,225,18,74,143,108,203,186,112,207,178,87,201,121,228,229,229,209,218,218,14,192,245,235,87,208,180,
  148,251,221,186,186,122,84,85,229,252,249,31,105,108,108,246,221,243,38,134,161,187,50,19,126,187,55,12,125,219,123,33,149,74,145,76,166,48,12,131,
  171,87,47,145,155,155,75,56,28,161,162,162,18,128,159,127,62,75,115,115,27,225,112,24,192,213,46,219,216,216,192,48,12,150,151,151,220,207,157,73,
  129,174,91,159,235,186,134,105,154,140,141,141,48,57,57,65,56,28,161,179,115,111,154,194,184,8,40,236,75,72,162,244,88,120,70,156,206,249,249,231,
  79,211,216,216,228,14,196,170,170,16,137,132,3,3,88,230,67,216,15,133,221,187,59,57,121,242,20,213,213,213,84,84,84,16,14,135,3,36,41,20,
  82,105,105,105,161,166,166,246,169,184,33,252,105,240,138,34,8,135,195,188,240,194,139,84,84,84,160,40,10,207,61,119,128,142,142,221,148,150,150,82,
  89,89,233,18,164,112,56,132,170,42,20,22,22,112,232,208,97,242,242,114,3,109,174,40,130,93,187,118,241,226,139,47,82,89,89,73,85,85,165,91,
  174,195,223,246,173,173,109,84,87,87,63,54,109,233,79,105,14,133,20,194,225,16,29,29,187,249,227,31,255,204,129,3,7,41,40,200,35,55,55,74,
  110,110,140,156,28,63,105,178,30,145,72,152,80,40,100,101,248,132,130,4,187,162,162,146,210,210,50,166,167,39,153,155,155,179,59,117,63,249,48,92,
  239,218,221,146,92,239,216,183,58,39,2,245,170,188,14,157,140,135,127,240,241,15,66,233,158,51,103,192,234,235,235,65,85,45,175,64,230,178,173,28,
  44,158,85,79,146,255,125,113,113,49,85,85,53,172,172,44,51,59,59,229,122,146,13,195,164,174,174,129,185,185,57,110,220,184,138,162,40,212,212,212,
  225,15,146,10,214,24,84,124,159,111,93,47,78,8,65,56,28,34,28,14,115,232,208,81,214,215,215,208,117,141,154,154,90,22,23,23,185,116,233,23,
  10,10,10,40,46,46,65,8,232,239,239,99,101,101,153,15,62,248,19,251,246,61,71,40,100,121,177,250,250,122,89,89,89,230,173,183,62,160,179,115,
  31,138,162,250,164,0,96,112,176,15,128,134,134,166,52,47,188,44,154,123,183,144,75,111,191,249,32,239,221,28,145,72,132,80,72,117,217,125,115,115,
  11,141,141,77,24,134,193,173,91,215,169,171,107,160,167,167,155,166,166,102,146,201,36,227,227,99,236,221,187,143,171,87,175,80,80,80,200,254,253,251,
  185,122,245,10,67,67,131,8,33,168,174,174,166,184,184,152,174,174,46,58,59,247,176,185,185,201,75,47,189,202,242,242,18,31,127,252,119,86,87,215,
  92,247,106,122,54,197,19,214,154,118,59,42,68,163,81,10,10,10,88,92,92,32,63,191,128,100,50,233,186,182,11,10,10,56,112,224,32,166,9,137,
  68,156,129,129,1,66,161,16,39,78,156,34,153,76,114,229,202,37,214,214,214,236,182,124,142,235,215,175,49,56,56,0,64,121,121,5,85,85,213,220,
  186,117,131,142,142,221,172,175,175,243,202,43,175,177,186,186,194,151,95,126,206,252,252,194,35,104,203,116,146,17,204,150,113,136,118,125,125,35,117,117,
  13,140,143,143,208,223,223,205,192,64,15,154,166,161,105,6,154,166,147,74,105,164,82,26,154,166,7,178,223,252,49,74,101,101,101,8,33,24,29,29,
  65,211,146,62,25,128,96,108,147,115,254,119,234,108,131,226,121,194,183,223,237,38,20,233,219,200,92,78,221,234,183,153,179,119,235,24,70,71,71,56,
  117,10,74,74,74,8,133,84,116,93,223,209,241,75,60,173,158,164,32,65,22,2,138,139,139,9,133,66,76,76,140,177,177,177,233,218,91,36,18,161,
  178,178,134,217,217,105,42,43,171,209,52,157,93,187,58,153,152,24,115,109,39,88,124,57,115,114,151,110,187,186,174,163,170,42,199,143,63,207,250,250,
  26,121,121,185,124,243,205,231,132,66,97,26,26,154,153,154,154,164,180,180,12,69,81,105,106,106,101,118,118,134,181,181,21,62,251,236,239,60,247,220,
  33,142,30,61,65,115,115,43,95,126,249,25,171,171,43,124,246,217,71,28,56,112,136,67,135,142,209,212,212,202,183,223,126,78,60,30,71,8,152,156,
  28,199,48,12,138,138,138,137,197,114,72,165,86,221,242,38,114,130,112,151,68,73,234,40,61,240,33,221,125,118,244,106,20,161,88,169,204,186,1,118,
  214,67,69,69,37,251,246,237,167,187,235,54,197,37,37,28,62,116,148,205,120,156,150,150,86,154,154,154,221,155,185,188,188,2,129,160,162,162,130,248,
  102,156,233,169,105,20,91,176,172,180,180,140,186,186,122,122,122,122,104,109,105,163,187,187,139,201,137,113,150,150,151,209,82,26,138,16,152,40,8,91,
  107,71,60,65,109,40,132,167,172,226,104,254,8,172,120,156,3,7,14,81,83,83,75,99,99,19,51,211,83,20,23,23,83,92,92,66,73,73,25,166,
  9,67,67,131,188,252,242,107,76,76,140,35,132,96,104,112,128,206,206,61,180,181,181,115,253,218,85,10,242,243,49,13,147,225,161,33,183,45,75,138,
  139,105,110,110,166,171,235,22,109,173,237,116,119,119,49,59,51,205,236,220,44,241,205,248,111,214,150,25,234,72,66,113,207,223,117,251,10,197,45,126,
  233,16,28,175,212,130,234,18,134,250,250,38,59,46,225,57,122,122,110,50,52,212,71,50,153,34,149,180,136,82,74,211,49,116,51,141,180,88,179,225,
  252,188,124,0,86,87,87,92,173,33,210,142,11,97,160,168,194,71,94,179,147,21,143,176,88,100,41,91,10,181,63,32,219,63,161,72,39,91,193,180,
  107,115,11,18,13,86,14,160,167,199,227,156,195,234,202,10,166,105,18,141,198,200,205,205,35,153,72,217,109,108,160,8,5,3,243,145,233,17,101,87,
  6,18,91,212,132,125,250,117,148,130,231,148,169,35,197,125,156,125,240,30,115,180,148,20,4,10,5,249,133,0,150,103,71,211,92,155,43,41,46,35,
  63,191,128,120,60,206,158,61,251,49,13,157,218,218,122,132,155,109,234,219,178,0,195,244,70,0,197,231,189,193,137,141,52,173,123,89,215,13,6,6,
  122,153,153,158,102,117,117,153,205,205,13,170,171,107,40,42,42,102,105,105,137,189,123,159,35,149,76,210,220,220,202,175,191,92,0,96,99,125,131,11,
  231,207,210,221,117,155,127,248,227,127,162,179,115,47,87,175,92,98,99,125,157,11,231,207,210,215,219,205,251,31,252,137,142,142,189,92,189,114,17,132,
  96,109,117,149,100,34,65,56,28,166,176,160,144,245,181,117,12,76,171,47,179,207,223,144,49,122,210,163,244,216,117,12,138,176,68,203,176,102,49,134,
  97,48,52,52,72,201,242,18,167,95,56,67,111,95,15,199,142,158,96,100,116,152,252,188,124,118,239,238,164,235,246,109,107,125,122,125,131,88,78,14,
  249,5,249,36,147,113,119,48,50,12,107,123,134,189,246,189,190,177,206,242,242,18,169,84,234,169,105,55,191,71,67,215,52,110,222,184,206,244,204,20,
  209,104,20,69,181,170,195,135,195,97,84,69,101,114,114,156,185,217,25,214,215,215,80,85,149,205,205,77,70,71,71,40,175,168,112,215,244,147,169,20,
  161,80,136,210,210,82,22,22,22,220,246,115,130,47,117,67,39,145,76,16,143,199,89,90,122,136,165,58,68,58,221,182,7,73,197,211,75,18,138,37,
  18,233,45,51,170,129,77,168,106,136,202,202,26,74,75,43,232,236,220,79,87,215,53,70,134,135,136,39,18,164,82,58,186,174,187,194,145,78,187,42,
  138,66,36,26,5,32,149,76,100,59,28,247,123,46,121,83,108,226,70,166,40,158,165,195,103,233,206,56,246,153,77,78,96,171,140,208,108,222,33,79,
  63,102,107,79,65,182,165,81,221,176,130,91,21,91,88,48,189,80,169,156,89,63,123,83,217,116,132,35,17,171,95,72,36,2,54,215,212,220,202,220,
  220,44,95,124,241,49,152,130,220,220,92,254,240,15,255,137,218,186,6,86,86,150,45,137,10,221,178,45,75,34,64,179,229,7,44,239,183,162,40,96,
  154,22,129,50,77,76,95,2,202,196,248,56,147,147,19,174,71,170,173,125,55,227,227,99,124,251,237,23,8,4,185,121,121,252,233,79,255,76,121,121,
  37,179,179,211,40,138,130,174,235,44,46,46,176,177,177,78,36,28,245,250,68,221,96,113,105,145,245,245,117,98,209,152,75,244,52,77,71,211,53,107,
  162,101,107,55,121,247,153,240,121,137,37,89,186,51,81,146,253,196,111,115,31,250,180,62,76,220,41,52,77,77,77,8,1,154,29,116,183,111,255,126,
  74,75,203,24,29,29,97,117,117,141,194,162,34,150,111,44,147,72,36,57,116,232,48,223,124,243,53,38,38,211,51,83,204,207,207,113,226,196,73,122,
  122,186,137,199,55,137,199,55,201,179,9,85,89,89,41,154,174,97,24,6,13,13,214,18,94,50,153,180,133,119,182,236,35,30,219,54,52,237,244,85,
  171,13,77,247,60,76,96,51,190,201,250,250,186,117,126,182,71,36,30,143,179,182,182,74,93,125,3,138,170,82,80,80,224,166,129,43,118,28,142,45,
  164,203,252,252,60,83,211,83,188,240,226,25,110,220,184,142,97,24,108,110,110,144,151,151,71,123,251,46,202,203,43,0,136,39,226,52,53,53,51,52,
  52,68,60,30,255,109,218,50,139,142,140,21,234,224,168,79,91,46,124,53,164,164,121,147,84,55,72,59,235,141,29,10,81,94,94,205,11,47,84,209,
  222,62,78,79,207,13,166,166,198,217,220,140,103,4,61,91,219,180,182,99,226,105,204,248,245,154,172,101,63,107,249,56,26,141,96,24,166,235,226,247,
  72,154,66,40,20,178,99,53,176,83,150,227,118,236,132,153,65,222,157,243,113,50,135,44,66,163,218,169,211,193,229,191,80,40,132,97,24,104,254,212,
  54,155,176,57,68,72,216,250,80,194,109,67,2,193,183,150,22,149,247,192,176,181,169,44,137,243,135,63,96,108,167,157,116,167,239,61,141,58,74,226,
  14,231,206,125,156,191,171,157,228,217,133,117,237,61,27,49,76,19,195,38,74,185,185,185,212,215,55,208,221,221,197,250,250,6,166,105,178,177,185,193,
  204,204,20,109,109,187,56,127,254,39,76,211,228,228,169,23,88,93,93,65,85,67,220,188,121,157,141,141,13,138,139,75,120,229,213,215,237,82,63,6,
  87,174,92,102,122,122,202,38,77,134,69,162,84,213,34,241,166,32,26,141,209,216,216,204,133,11,103,89,91,179,116,149,214,214,215,89,88,152,167,185,
  165,149,80,56,196,225,195,199,152,156,156,160,178,178,10,16,92,185,122,137,198,166,102,246,237,59,192,244,244,36,21,21,149,168,170,194,141,155,215,48,
  76,211,106,26,167,0,180,35,203,157,246,16,10,96,60,5,118,36,61,74,79,182,23,36,189,158,207,245,235,87,169,175,111,164,178,178,138,225,225,65,
  126,250,233,71,246,238,221,199,220,220,28,55,111,222,32,28,14,243,253,247,223,50,55,55,7,152,172,173,173,186,3,6,192,79,63,253,192,238,221,157,
  212,212,212,178,178,178,204,245,235,215,81,20,149,178,178,114,46,92,56,207,252,252,28,107,107,107,236,218,213,65,78,78,174,175,46,209,147,25,159,228,
  111,67,195,48,73,38,83,92,189,122,133,197,197,69,12,195,160,175,175,151,84,42,105,121,68,34,17,22,22,172,32,228,162,162,98,155,56,173,113,243,
  230,117,116,93,103,104,104,208,39,30,105,114,225,194,57,218,219,119,81,87,87,207,234,234,42,215,175,95,229,202,149,75,148,150,150,241,243,207,231,153,
  157,157,101,97,97,129,189,123,247,17,139,229,216,177,11,191,125,91,102,139,159,113,2,209,157,160,209,96,208,186,122,135,237,9,170,171,235,169,174,174,
  103,98,98,152,193,193,30,166,167,39,220,42,227,78,41,3,39,171,47,24,232,234,143,135,18,28,62,124,148,198,198,70,119,214,189,190,190,70,95,95,
  143,171,35,179,123,247,30,246,238,221,79,34,145,64,81,20,22,23,23,249,246,219,175,56,124,248,40,137,68,146,107,215,174,4,180,169,156,24,169,55,
  222,120,147,120,124,147,115,231,126,34,47,47,159,211,167,95,228,250,245,43,76,76,76,96,24,38,149,149,85,28,58,116,132,179,103,127,100,105,105,9,
  176,50,126,90,90,172,186,110,137,68,130,91,183,110,176,178,178,156,33,123,144,46,215,17,148,68,112,226,53,144,105,210,207,210,92,118,203,18,69,184,
  202,243,169,148,206,229,203,23,25,25,25,70,211,52,219,62,116,206,158,253,129,252,252,2,54,54,54,249,241,199,239,40,41,41,117,239,81,93,183,86,
  8,126,250,233,123,114,114,114,109,175,142,134,166,121,181,17,215,214,214,249,246,219,175,152,153,153,33,149,210,237,251,217,224,252,249,179,244,246,118,147,
  74,105,238,49,126,247,221,215,68,163,49,102,102,102,153,156,156,160,184,184,152,185,185,89,190,254,250,75,214,214,86,153,158,86,41,45,157,180,63,159,
  227,187,239,190,97,125,125,205,149,12,201,166,188,45,61,168,146,40,61,134,3,189,23,11,98,24,48,52,52,204,240,240,112,32,11,226,252,249,115,238,
  160,152,72,36,24,24,232,119,127,191,178,178,146,81,56,241,214,173,91,220,186,117,203,253,78,79,79,15,208,227,126,103,109,109,157,185,185,179,25,169,
  212,79,122,27,154,38,164,82,41,6,6,6,220,193,110,124,124,60,144,234,93,90,90,102,119,90,42,61,61,93,172,172,172,176,188,188,2,152,76,79,
  79,7,60,20,0,93,93,93,116,117,117,165,181,165,183,95,211,52,57,119,238,209,180,101,186,146,182,170,42,190,140,55,107,201,45,20,82,239,42,32,
  185,182,182,137,218,218,38,198,198,6,25,25,233,99,114,114,140,100,50,133,174,27,62,249,131,204,76,29,103,197,237,224,193,67,196,227,155,252,242,203,
  5,242,242,242,104,110,110,229,15,127,248,51,159,126,250,55,38,39,39,104,107,107,167,184,184,132,43,87,46,2,176,190,190,78,40,164,178,107,87,7,
  171,171,43,220,188,121,29,191,86,152,170,10,106,107,235,216,191,255,0,166,105,112,227,198,85,18,137,56,133,133,133,156,62,125,134,191,252,229,223,0,
  56,117,234,121,114,115,243,216,220,220,68,181,151,90,43,42,42,56,112,224,32,243,243,115,180,183,183,211,209,209,193,191,253,219,191,146,76,38,50,178,
  140,156,253,57,82,8,170,106,216,58,50,66,10,240,73,248,123,27,183,143,88,95,95,227,198,141,107,174,22,153,83,74,100,116,116,204,189,79,174,94,
  189,154,209,79,1,92,184,112,62,45,190,206,251,91,60,30,231,202,149,43,238,123,195,16,172,173,173,115,249,242,165,140,9,226,208,208,176,251,217,15,
  63,124,159,113,180,243,243,243,252,248,227,247,46,81,115,164,76,130,154,106,217,181,227,36,95,146,68,233,145,15,236,94,135,107,7,249,25,38,138,226,
  44,45,56,153,67,102,22,181,238,59,215,129,219,94,183,198,185,217,177,7,247,157,255,254,241,37,73,153,109,232,4,73,90,157,154,215,57,172,174,174,
  177,176,176,200,220,220,60,19,19,227,25,231,186,29,225,73,175,11,183,85,91,110,213,1,62,64,138,228,179,5,203,86,52,45,229,122,148,66,33,79,
  99,43,20,82,2,154,47,59,69,125,125,11,53,53,13,76,77,141,49,52,212,195,232,232,48,186,174,101,120,93,210,99,126,82,41,43,19,115,124,124,
  20,69,81,24,24,232,229,141,55,222,226,165,151,94,227,223,254,237,255,68,8,193,200,200,32,183,110,93,115,7,28,85,181,98,43,52,77,183,227,172,
  188,107,42,132,176,18,26,186,111,83,90,90,74,103,231,94,174,95,191,194,247,223,127,205,127,250,79,255,153,234,234,26,116,93,167,174,174,158,127,253,
  215,255,43,112,140,115,115,179,124,248,225,127,144,176,131,85,255,247,255,253,255,67,93,93,157,155,17,26,20,38,245,8,167,71,150,76,219,166,12,31,
  97,146,100,233,89,37,72,206,253,28,92,150,246,107,136,25,59,234,171,179,245,13,254,76,211,251,177,45,255,190,253,125,152,23,59,232,100,177,18,152,
  16,74,72,162,244,216,15,244,134,129,175,19,14,14,254,65,61,23,210,2,77,197,182,55,224,86,37,36,156,207,29,183,107,102,61,174,39,135,108,250,
  219,208,175,52,238,252,61,93,155,100,125,125,157,161,161,161,45,83,191,211,133,223,50,219,208,220,65,91,254,54,141,153,61,168,217,68,85,85,26,27,
  91,136,199,55,125,34,152,142,14,146,160,164,164,236,158,246,167,170,33,234,234,154,169,168,168,161,185,121,146,72,36,150,230,81,242,142,203,47,170,23,
  14,135,3,90,71,215,174,93,230,247,191,255,71,98,177,24,201,100,130,202,202,106,87,152,111,109,109,149,153,153,25,87,190,193,43,181,98,109,47,26,
  141,80,83,83,199,143,63,126,75,101,101,21,109,109,237,220,188,121,149,181,181,21,186,187,111,115,252,248,41,226,241,56,221,221,93,44,45,45,4,148,
  236,173,248,52,111,169,48,149,74,162,105,41,95,59,6,237,195,153,93,91,30,41,255,160,168,160,40,134,171,114,110,202,250,87,207,124,159,237,23,72,
  181,150,167,245,128,202,118,112,66,67,192,19,148,141,128,57,223,183,108,43,219,62,253,30,45,238,224,253,17,91,244,85,166,29,175,104,197,64,61,233,
  43,9,146,40,61,163,100,201,159,17,228,184,249,173,103,103,38,32,2,94,166,59,205,90,252,229,53,178,149,146,240,180,100,204,167,162,13,157,90,102,
  30,137,50,51,136,102,186,39,36,219,236,47,157,36,57,222,162,108,30,166,71,209,150,217,8,202,43,175,188,185,165,14,145,63,19,237,94,16,137,68,
  169,171,107,14,120,93,210,179,193,182,178,67,33,132,29,127,97,16,10,133,209,52,141,138,138,74,246,239,63,136,170,170,140,143,143,49,59,59,235,22,
  238,117,178,131,156,253,212,215,55,162,40,10,107,107,107,24,134,193,129,3,71,40,46,46,97,113,113,129,95,126,57,199,191,252,203,255,131,68,34,206,
  127,255,239,255,109,219,193,227,244,233,23,89,90,90,100,106,106,50,141,120,122,89,65,197,197,37,52,54,54,115,251,246,77,251,90,170,246,189,169,96,
  24,138,175,244,139,36,75,207,86,255,156,253,51,175,0,180,231,85,218,138,180,4,251,246,108,247,178,215,215,59,132,60,125,178,155,125,226,182,29,89,
  50,51,238,79,93,183,146,46,172,125,61,220,58,140,146,40,73,220,151,71,196,63,208,250,213,133,253,238,88,203,232,149,64,153,12,103,246,110,9,133,
  37,217,220,220,12,8,233,165,151,144,72,175,113,150,237,198,120,242,150,221,130,53,220,252,129,192,214,115,144,52,89,5,98,189,186,74,206,239,35,145,
  8,57,57,57,108,110,110,90,217,107,1,242,227,165,176,103,39,73,143,190,45,87,87,87,208,117,205,245,200,248,137,76,36,18,37,55,55,255,190,182,
  239,148,98,80,85,117,75,210,189,149,86,82,97,97,49,0,155,155,235,68,163,49,250,251,123,249,238,187,47,93,233,10,255,111,211,107,27,214,212,212,
  81,88,88,196,75,47,189,134,170,170,228,230,230,82,87,215,192,194,194,2,137,68,130,233,233,41,18,137,4,154,150,242,5,226,123,215,69,8,216,191,
  255,0,157,157,123,249,240,195,127,71,211,180,140,227,180,60,145,6,209,104,148,151,94,122,3,211,132,174,174,27,246,121,27,168,170,234,91,166,48,124,
  3,165,36,75,207,34,89,50,205,160,7,217,89,130,243,247,181,129,76,202,45,39,60,194,231,137,53,3,181,41,253,191,79,95,210,207,214,159,59,219,
  202,238,169,194,45,14,45,132,19,107,248,244,76,146,31,27,162,36,5,39,127,171,187,47,203,108,220,204,50,131,177,61,33,142,64,158,16,158,192,223,
  190,189,251,120,227,119,111,114,251,214,45,62,254,248,35,12,211,64,177,133,205,132,115,51,108,81,164,213,127,93,221,27,252,73,188,214,91,180,163,219,
  193,185,105,195,78,10,172,0,211,17,148,179,244,76,14,31,58,194,171,175,189,206,15,63,124,199,79,63,253,132,161,235,190,108,99,75,112,112,187,53,
  253,223,178,45,51,36,244,76,199,22,132,237,13,209,248,228,147,191,176,182,186,226,214,162,11,133,173,140,183,112,72,165,169,105,23,71,143,189,120,143,
  3,132,193,226,194,28,3,3,221,148,87,84,211,220,188,203,234,172,13,219,22,125,199,227,120,133,116,77,179,108,14,147,72,52,198,209,163,39,25,31,
  31,69,215,77,84,197,82,190,214,82,154,117,10,134,233,138,239,233,154,238,19,178,180,8,94,99,67,19,191,252,114,158,238,174,91,132,66,33,142,30,
  59,73,75,179,37,10,26,24,160,236,227,33,77,131,105,87,123,7,71,143,30,231,163,191,255,213,18,98,117,60,138,174,8,160,45,212,105,95,183,112,
  56,204,107,175,189,73,72,85,185,113,227,26,132,45,189,8,211,48,49,13,123,73,28,203,131,32,20,243,55,39,75,98,155,127,254,124,248,237,191,247,
  228,98,107,33,73,17,56,251,7,33,183,185,163,86,52,125,199,100,122,54,103,26,78,63,235,210,155,59,120,90,211,72,12,94,127,84,84,84,68,65,
  65,1,107,107,107,44,47,47,99,98,218,162,167,105,147,48,159,154,189,43,149,178,213,185,41,214,241,225,62,139,140,62,106,251,118,148,28,64,122,148,
  30,19,207,72,186,155,52,211,219,20,44,186,170,40,130,88,44,70,126,126,1,121,249,121,110,32,172,229,252,48,183,220,254,211,58,139,216,170,29,253,
  49,91,170,106,7,57,250,50,61,20,69,144,155,155,67,52,26,37,47,55,23,85,17,110,39,228,213,69,50,31,185,215,40,125,6,232,63,47,45,165,
  161,235,150,134,174,71,212,44,2,98,154,198,61,237,111,126,126,134,145,145,126,134,135,250,136,199,55,41,40,44,241,217,163,233,238,203,137,237,49,77,
  75,155,169,174,174,129,214,214,57,43,61,191,181,13,69,40,156,253,233,123,12,221,42,4,218,208,208,196,222,125,7,80,20,133,68,60,65,119,207,109,
  116,195,160,181,173,141,165,229,37,84,85,101,117,101,133,72,36,66,44,39,135,254,190,62,86,86,86,17,2,122,122,186,121,255,189,63,80,94,94,201,
  196,196,56,145,72,36,173,148,138,215,62,187,118,117,240,79,255,252,47,244,247,245,81,82,82,74,77,109,45,227,99,99,204,206,206,120,223,181,191,175,
  170,42,43,43,203,204,204,76,210,222,222,201,153,151,94,71,81,85,174,92,185,132,26,50,9,57,36,217,149,104,50,236,216,66,233,89,122,54,189,75,
  153,225,12,217,60,74,91,39,214,248,75,152,120,100,169,125,215,46,142,29,59,78,78,142,85,24,253,167,31,127,164,187,187,203,42,184,170,224,11,3,
  184,187,126,221,178,85,255,210,157,249,104,116,193,158,106,162,36,201,228,67,118,144,100,206,70,28,33,52,69,181,68,243,20,85,65,81,21,119,82,169,
  235,58,138,106,149,208,48,49,172,66,26,166,8,10,32,62,131,237,152,49,203,178,61,74,194,22,22,180,218,209,83,67,55,49,173,207,81,236,201,162,
  97,137,42,146,41,164,99,154,15,169,109,51,132,224,76,239,217,153,157,10,129,97,173,7,89,159,25,160,24,2,221,208,45,177,196,187,34,72,211,140,
  140,12,48,54,54,200,198,198,58,154,102,123,80,124,158,23,71,116,207,176,73,167,97,183,205,149,43,151,169,175,175,167,161,177,25,211,52,24,25,25,
  161,167,231,54,155,155,27,0,116,117,223,198,4,42,42,170,16,66,176,185,185,9,192,141,27,215,232,236,220,67,85,85,181,187,20,58,55,55,203,133,
  11,231,152,155,159,115,151,62,199,199,199,249,249,151,243,232,134,129,166,235,220,190,125,203,170,81,167,235,129,193,75,8,129,26,10,113,233,210,37,18,
  137,4,13,141,86,172,211,194,194,60,14,77,246,30,38,138,162,144,74,37,248,229,151,159,88,94,94,226,216,177,231,121,241,197,87,17,66,225,226,197,
  159,49,66,10,134,169,90,34,124,186,176,248,146,176,60,90,138,248,13,201,210,78,133,36,159,118,193,73,184,179,248,230,253,158,191,216,65,59,166,139,
  48,110,177,47,243,14,101,63,132,16,174,200,176,35,104,106,221,94,38,55,110,92,103,105,105,145,195,135,143,240,198,239,126,199,204,236,52,139,139,139,
  30,187,242,245,233,59,181,55,107,127,119,104,159,180,118,204,248,190,132,244,40,61,22,125,130,16,89,63,243,170,166,43,129,202,245,78,53,123,203,43,
  162,216,25,13,206,114,76,112,92,247,175,169,167,199,70,61,43,237,232,100,97,165,183,161,106,171,114,251,219,49,179,40,164,21,35,246,184,180,99,176,
  108,7,118,64,169,145,86,92,86,183,74,40,24,59,59,182,165,165,121,6,6,186,24,27,27,98,109,109,205,21,155,244,178,49,211,75,141,4,179,127,
  76,211,224,242,229,139,92,187,118,149,112,56,66,42,149,4,76,215,70,193,211,165,242,2,204,45,178,209,221,221,69,119,119,87,160,71,206,230,197,91,
  91,91,115,245,98,116,221,224,242,229,75,89,179,55,133,128,27,55,174,115,253,250,53,223,181,21,190,50,13,138,61,195,246,218,81,85,173,174,238,234,
  213,95,81,20,193,145,35,167,120,225,133,151,80,20,133,243,231,207,18,10,5,39,48,78,240,174,37,233,33,61,75,207,88,111,189,109,29,195,123,233,
  167,132,16,12,14,14,186,122,104,215,174,93,165,181,181,141,194,194,34,22,23,23,125,177,170,247,86,116,123,187,98,211,153,158,41,105,195,146,40,61,
  230,131,123,122,141,41,47,221,91,241,165,125,123,3,189,243,157,112,56,132,16,186,111,144,183,170,160,107,26,118,134,67,166,219,246,105,235,216,183,111,
  71,37,80,40,54,216,142,30,81,10,135,85,87,145,57,216,193,24,248,131,121,31,85,59,250,131,245,253,68,88,215,13,155,24,5,11,114,106,154,190,
  237,246,86,86,150,232,237,189,193,208,80,63,107,107,235,110,170,179,23,160,138,189,93,111,95,233,117,217,76,211,112,63,215,245,36,169,84,202,109,127,
  167,29,131,215,71,15,16,76,175,253,252,210,22,108,75,68,211,51,13,211,131,91,13,67,15,76,18,44,162,100,218,3,81,118,189,44,69,81,49,12,
  131,139,23,47,160,105,58,39,78,188,192,243,207,159,65,8,56,119,238,39,192,186,159,66,105,189,162,36,75,79,145,39,58,235,146,212,214,253,202,131,
  33,93,233,122,121,80,95,223,192,218,218,154,45,123,225,100,60,223,91,159,232,207,148,77,151,197,144,144,68,233,137,28,232,253,49,72,126,93,156,112,
  88,117,7,120,103,144,15,133,252,3,124,8,85,85,208,52,221,29,52,173,135,238,122,6,60,47,192,211,189,62,237,180,159,223,147,224,111,71,171,253,
  130,100,9,172,90,100,145,72,24,93,55,208,52,171,124,128,166,9,132,48,16,66,195,48,132,75,34,28,221,166,135,219,142,102,224,181,179,107,199,163,
  228,39,29,14,241,200,70,148,76,211,176,203,139,220,162,191,191,155,213,213,53,107,9,75,243,108,37,152,94,172,184,98,142,86,93,182,236,132,197,35,
  111,102,70,39,125,55,3,213,78,73,146,127,95,193,98,186,94,12,154,147,249,24,36,85,233,94,43,145,230,153,131,139,23,47,160,235,58,167,78,157,
  225,244,233,151,1,133,115,231,126,220,49,89,122,90,61,182,79,55,73,50,179,218,152,67,186,29,175,99,54,33,224,251,189,230,254,190,191,170,170,134,
  195,135,143,240,211,79,63,178,190,190,126,199,123,100,167,132,44,184,31,255,228,69,220,225,30,147,144,68,233,177,26,224,189,250,86,222,50,145,87,5,
  222,41,114,234,20,61,13,217,189,181,170,42,196,98,214,0,175,170,170,59,200,123,3,189,238,211,253,112,130,80,159,78,61,24,255,224,236,120,145,210,
  235,159,5,201,166,234,22,142,117,218,81,211,12,155,116,58,42,215,14,105,114,72,168,95,9,252,97,7,117,167,123,115,176,61,57,134,207,51,98,96,
  154,214,82,161,83,27,202,239,65,234,239,239,162,175,175,139,245,245,117,82,41,205,126,232,104,154,150,177,84,231,100,138,109,110,90,178,9,57,57,57,
  100,115,205,103,6,189,139,172,223,187,19,113,218,42,205,249,110,52,95,130,90,100,248,150,228,50,211,171,195,225,8,138,98,77,48,18,137,132,21,20,
  111,159,203,197,139,63,147,76,166,120,241,197,151,57,125,250,140,189,12,247,35,144,66,207,226,168,243,147,165,167,209,99,251,140,208,165,12,91,139,199,
  19,0,196,98,177,12,114,100,137,5,243,192,234,1,22,23,151,240,218,107,111,112,253,250,53,110,221,186,145,118,127,221,125,95,232,159,56,250,201,152,
  21,11,24,35,28,14,147,76,38,73,38,19,25,247,154,20,165,148,68,233,49,29,224,131,177,72,222,224,174,18,14,135,237,103,139,32,169,170,181,76,
  228,13,240,81,52,77,39,20,210,209,52,133,84,74,65,136,148,187,93,175,170,186,129,174,155,79,117,161,207,173,219,49,100,63,50,73,39,64,40,164,
  18,141,70,8,133,44,143,146,166,233,164,82,90,32,206,198,89,206,52,77,195,85,210,117,150,167,30,93,167,226,196,42,5,245,93,172,242,27,214,185,
  205,205,205,48,56,216,75,111,111,23,235,235,150,128,163,166,25,238,57,58,36,208,191,28,230,116,176,170,170,176,188,188,4,88,245,242,172,37,42,51,
  99,9,34,184,36,249,224,218,34,187,87,233,206,19,14,135,80,90,97,74,153,74,237,214,249,88,69,75,55,55,55,88,91,219,176,175,171,183,223,203,
  151,127,193,48,116,206,156,121,149,83,167,94,32,28,14,243,195,15,223,144,76,102,47,13,35,201,210,147,236,81,74,87,193,54,221,201,5,64,73,73,
  41,209,104,20,195,216,196,48,188,58,128,158,71,233,222,39,158,166,105,82,94,94,193,251,239,255,158,153,153,105,46,93,250,149,130,130,2,86,86,214,
  48,77,253,174,183,153,78,232,178,149,237,41,43,43,35,28,142,176,186,186,194,218,218,218,83,81,251,83,18,165,167,152,32,5,189,73,153,21,224,163,
  209,48,145,72,136,72,36,28,88,58,114,98,102,192,36,39,39,138,174,235,182,103,64,71,85,83,246,182,82,190,89,188,102,15,160,34,67,70,224,105,
  111,71,167,253,162,209,176,75,150,188,118,52,93,143,67,78,78,20,195,48,72,165,44,2,225,212,78,11,186,215,61,53,240,199,161,29,253,222,156,244,
  88,157,249,249,57,126,254,249,28,55,110,92,101,109,109,21,175,62,157,225,146,35,235,89,119,151,103,157,115,181,150,36,173,115,95,88,152,39,149,74,
  209,212,212,66,44,22,101,109,77,203,90,92,54,123,41,157,173,9,205,157,8,208,189,117,218,94,57,136,224,128,17,180,21,128,198,198,86,151,72,234,
  186,150,85,47,235,210,165,95,209,52,141,87,95,125,147,163,71,79,160,170,42,223,126,251,165,36,75,79,45,97,114,234,16,90,175,151,150,150,136,199,
  55,169,173,173,163,160,160,128,120,60,110,167,246,91,73,1,94,98,128,184,39,181,107,167,47,121,241,197,151,168,170,170,102,99,99,157,127,248,135,63,
  35,132,224,203,47,63,103,98,98,34,195,195,122,55,158,36,127,13,67,103,181,66,81,4,141,141,77,110,93,196,100,50,25,88,70,151,246,42,137,210,
  99,236,81,242,130,183,203,202,202,56,120,240,16,225,112,24,85,197,205,208,114,226,105,76,19,183,142,87,105,105,5,123,246,28,114,7,2,127,13,34,
  77,179,226,149,122,122,122,24,24,24,240,13,4,79,103,224,105,208,19,162,82,93,93,205,254,253,251,3,237,23,10,121,25,131,166,9,229,229,85,0,
  84,85,213,5,74,126,56,237,168,105,186,253,90,167,183,183,151,193,193,65,59,158,229,241,171,42,31,140,21,130,222,222,62,250,250,250,124,53,205,130,
  138,194,186,238,197,37,249,227,156,156,118,176,126,103,48,61,61,197,236,236,52,181,181,245,52,52,52,209,221,125,27,176,74,123,88,245,209,172,1,195,
  202,14,52,220,54,217,206,115,185,147,38,219,170,166,225,118,223,75,79,134,240,6,135,224,64,145,159,95,64,75,75,27,169,84,146,161,161,65,52,205,
  8,232,50,57,80,20,193,213,171,87,208,117,157,55,222,120,135,67,135,142,162,40,10,223,126,251,165,187,44,35,201,210,147,79,142,156,103,175,92,141,
  181,188,190,188,188,196,200,200,16,29,29,123,104,106,106,97,126,126,62,240,187,244,223,56,158,169,173,174,185,63,134,210,191,157,179,103,127,226,231,159,
  47,16,137,132,221,196,130,165,165,165,45,38,24,226,142,147,69,32,35,195,215,201,242,141,70,99,180,180,180,3,208,223,223,231,243,6,27,146,44,221,
  11,81,146,170,156,191,241,192,142,167,146,42,176,148,130,85,69,193,52,13,154,155,91,168,173,173,191,227,54,138,138,74,41,42,42,221,242,239,186,174,
  211,215,219,139,170,40,232,66,65,17,153,106,175,79,83,59,42,66,184,237,152,76,38,216,183,239,0,5,5,133,119,220,70,121,121,53,229,229,213,91,
  254,125,109,109,149,158,238,110,84,69,197,80,76,12,97,218,186,221,191,237,121,221,73,45,215,83,200,246,84,170,221,26,84,54,81,246,136,161,231,81,
  74,175,134,238,198,172,97,90,202,238,194,196,208,77,84,5,54,55,226,12,244,247,81,91,91,207,241,227,207,51,56,208,79,50,153,34,164,98,237,59,
  100,29,135,174,27,152,138,146,81,103,240,222,189,67,119,231,81,244,15,36,206,128,225,18,100,213,242,32,58,246,113,232,208,17,114,115,115,153,24,31,
  99,120,104,8,67,55,221,229,199,64,96,175,162,128,106,114,243,198,13,116,77,231,205,183,222,229,192,129,195,40,138,194,55,95,127,73,60,30,119,175,
  135,98,43,127,235,8,139,48,42,230,61,147,165,157,168,69,7,245,148,121,42,149,185,183,87,138,126,112,186,228,126,213,109,75,193,218,18,102,76,37,
  53,250,251,122,233,232,216,195,209,163,39,233,237,233,98,109,109,205,114,240,168,184,215,221,155,112,108,109,251,233,30,78,127,226,206,244,212,84,128,228,
  56,118,168,8,5,161,152,246,209,121,219,78,247,30,251,189,180,126,175,176,162,216,182,175,88,125,162,34,20,246,236,217,71,101,101,21,203,203,75,244,
  246,246,88,234,243,1,149,123,187,130,129,28,162,119,232,81,146,45,245,91,247,6,238,179,80,156,103,193,230,230,6,55,111,94,163,166,166,238,190,83,
  81,111,221,186,198,204,236,148,39,108,246,52,10,137,101,17,158,83,84,65,60,190,193,197,139,23,120,229,149,223,221,247,46,122,123,187,152,155,159,177,
  68,28,237,235,228,138,207,253,150,231,117,39,49,61,159,72,156,37,0,105,96,152,38,186,207,67,100,103,228,7,102,190,94,209,95,95,199,235,120,136,
  108,33,73,5,107,59,138,170,112,227,230,117,246,236,221,79,85,85,53,71,143,157,228,194,133,159,64,40,132,4,8,29,87,200,211,48,12,20,223,210,
  219,195,156,153,102,95,130,245,102,210,138,106,189,175,111,104,96,223,190,131,0,92,188,244,51,154,174,161,27,6,186,161,7,60,74,66,216,2,130,182,
  208,159,37,154,105,242,187,223,189,195,254,253,7,81,20,133,175,191,254,156,205,205,4,8,208,45,153,101,203,62,12,75,111,201,17,165,188,107,178,180,
  67,17,196,167,94,112,242,97,9,110,218,215,218,158,38,96,152,206,189,36,232,235,239,99,100,100,136,198,198,102,78,156,60,205,55,223,124,129,130,226,
  238,71,183,83,201,20,51,40,159,177,149,71,41,221,235,153,94,168,219,186,121,236,73,144,157,132,32,20,211,222,254,214,219,245,219,190,159,40,121,246,
  175,80,94,94,193,225,195,71,45,219,191,248,51,201,100,194,58,95,195,19,144,181,108,94,14,207,59,39,74,18,15,209,253,27,116,175,14,14,246,49,
  49,49,74,93,93,227,61,111,115,115,115,157,174,174,27,25,133,93,159,37,38,218,223,223,195,254,253,7,220,37,182,123,193,218,218,42,55,110,92,73,
  187,94,230,111,238,41,217,185,237,120,1,212,206,146,161,147,234,158,253,187,108,217,153,123,131,186,233,46,163,25,134,193,234,234,10,191,252,114,158,183,
  222,122,143,19,39,158,103,105,105,145,238,238,91,88,21,201,189,206,217,210,114,218,190,218,249,111,71,148,130,51,106,127,96,191,67,154,138,139,139,57,
  115,230,85,242,242,242,232,234,186,69,87,87,151,187,84,237,100,135,58,237,98,253,62,232,97,234,234,186,141,97,24,188,249,230,187,236,221,251,28,138,
  162,242,213,87,159,177,177,177,153,230,201,117,158,173,101,56,255,54,229,178,198,227,219,7,123,201,16,158,160,234,230,230,38,191,254,122,129,154,154,90,
  14,29,58,202,226,226,2,87,174,92,180,21,235,189,229,126,207,35,185,181,237,103,179,81,47,134,78,248,150,171,29,29,179,116,237,178,237,183,11,94,
  224,182,63,131,90,81,20,114,115,115,57,115,230,21,74,75,203,25,25,25,226,250,245,171,110,127,225,45,219,155,50,176,91,18,165,199,237,198,52,179,
  8,238,89,55,130,166,105,116,117,93,163,186,186,206,21,151,188,91,244,245,117,177,176,48,239,235,4,30,175,1,254,65,183,163,159,112,58,157,74,50,
  153,224,198,141,43,188,252,242,155,247,236,157,235,234,186,206,250,250,154,59,213,123,28,219,46,253,218,102,11,76,78,175,81,149,78,158,60,91,20,238,
  54,132,48,92,33,211,235,215,175,81,89,89,197,145,35,199,121,253,245,183,8,133,84,110,222,188,110,15,24,102,6,81,202,86,159,238,183,244,36,249,
  103,236,206,128,225,169,114,11,170,170,170,120,245,213,223,81,93,93,203,236,236,52,223,124,243,165,79,58,195,200,88,62,177,218,33,221,102,84,122,122,
  186,209,52,157,183,222,122,143,206,206,189,132,66,42,95,126,249,41,171,171,235,89,143,207,139,1,19,146,44,61,230,253,136,159,44,9,97,197,34,234,
  186,160,191,191,159,243,231,127,226,204,153,87,121,229,149,55,8,133,66,92,186,244,11,150,184,175,226,18,147,108,181,7,179,47,147,101,166,236,7,28,
  74,38,89,9,82,54,2,227,183,127,107,153,61,211,246,75,74,74,120,249,229,215,105,105,105,99,101,101,153,175,191,254,130,68,34,25,136,81,244,148,
  246,165,45,72,162,244,24,207,102,156,155,204,41,11,49,50,50,196,212,212,24,117,117,77,247,224,5,89,161,183,247,150,207,248,205,167,94,72,204,43,
  210,26,188,241,199,198,134,152,158,30,167,186,186,254,174,183,185,178,178,68,79,207,173,64,201,14,175,83,125,188,72,167,215,145,102,6,118,111,229,89,
  202,70,56,253,226,145,142,103,201,33,75,223,127,255,13,225,112,132,231,158,59,200,239,126,247,46,229,229,149,220,184,113,149,185,185,57,31,97,53,239,
  186,120,231,131,34,74,233,131,144,162,8,162,209,40,29,29,123,56,126,252,20,197,197,37,204,204,76,241,209,71,31,178,186,186,234,10,77,58,215,214,
  121,237,156,127,118,65,62,133,129,129,62,62,254,248,67,222,126,251,61,218,219,119,35,132,194,23,95,124,188,5,89,178,202,183,75,178,244,36,76,90,
  253,68,41,168,122,127,254,252,57,34,145,24,39,79,62,207,153,51,175,82,90,90,206,213,171,151,152,158,158,180,239,19,211,45,64,187,221,100,36,157,
  208,123,100,9,159,189,153,62,21,254,173,39,30,119,178,125,85,85,217,181,171,147,99,199,78,186,113,73,159,124,242,55,102,102,102,236,100,14,61,16,
  167,40,61,73,146,40,61,214,36,41,88,63,203,112,235,109,221,188,121,217,246,42,133,238,234,166,31,26,234,101,121,121,25,195,208,221,25,115,250,108,
  231,105,187,33,156,142,197,79,56,117,221,32,145,72,210,219,123,139,242,242,42,66,161,240,93,109,243,246,237,43,196,227,113,223,114,150,25,200,110,121,
  28,219,32,91,133,243,123,159,89,27,246,50,146,213,1,39,18,73,190,250,234,51,86,86,150,57,125,250,12,71,142,28,167,165,165,149,193,193,1,70,
  71,135,153,153,153,182,117,89,140,45,6,140,7,77,146,200,50,91,135,104,52,70,73,73,41,13,13,77,180,182,182,83,87,87,111,123,89,123,248,246,
  219,175,88,88,88,112,179,254,188,218,118,70,32,110,203,81,96,247,164,56,108,159,146,237,224,29,30,30,228,227,143,255,198,219,111,191,71,91,219,46,
  84,245,3,62,251,236,35,86,86,86,179,120,47,45,178,228,87,199,151,100,233,241,245,42,57,10,252,86,41,30,123,64,12,193,15,63,124,203,250,250,
  42,47,188,240,50,251,246,61,71,67,67,35,67,67,3,140,140,12,49,61,61,101,147,111,125,219,248,188,116,175,103,250,103,126,162,148,30,243,228,151,
  222,200,168,251,109,111,35,28,142,82,92,92,68,125,125,35,173,173,237,212,215,55,162,170,42,99,99,35,124,243,205,151,76,77,77,6,236,222,223,183,
  221,79,191,33,137,146,196,67,240,132,8,223,140,214,112,83,211,167,167,39,25,29,29,160,185,185,227,46,188,73,203,116,119,223,72,27,0,140,167,222,
  171,228,95,54,242,159,187,174,235,174,87,169,174,174,121,199,219,155,155,155,98,100,100,208,150,90,208,51,58,149,199,209,155,118,191,30,156,204,153,181,
  83,10,196,192,209,45,77,36,146,156,61,251,35,147,147,99,156,60,249,2,13,13,77,148,148,148,177,111,223,1,18,137,56,155,155,155,62,145,211,135,
  15,69,17,196,98,49,162,209,28,114,115,115,109,207,224,50,191,254,122,129,155,55,175,179,177,177,25,184,199,210,73,146,231,53,76,175,247,151,137,209,
  209,97,62,250,232,175,188,251,238,239,105,110,110,229,237,183,223,231,179,207,62,98,121,121,37,163,212,9,60,253,234,248,79,139,87,73,184,65,248,206,
  117,247,88,201,197,139,191,50,49,49,193,169,83,167,105,111,239,224,192,129,195,236,217,179,143,120,220,178,253,84,42,117,87,4,63,221,51,228,236,255,
  94,38,25,206,36,33,22,139,145,155,155,135,16,130,141,141,13,46,95,254,133,171,87,47,219,53,29,211,165,65,204,172,75,134,18,146,40,61,118,55,
  167,223,213,235,40,67,91,202,201,58,93,93,215,168,171,107,38,28,142,236,104,123,253,253,93,172,173,173,251,202,83,232,105,179,6,243,41,238,228,188,
  27,223,17,225,76,165,52,226,241,36,125,125,93,84,84,212,16,137,68,119,180,189,222,222,155,108,108,108,184,109,104,105,42,25,207,132,139,218,33,73,
  214,160,238,145,4,75,153,220,68,85,77,6,6,6,25,31,31,167,161,161,145,221,187,247,80,95,223,72,126,126,193,142,228,24,30,198,241,39,147,9,
  134,135,7,233,233,233,166,183,183,219,86,37,55,3,3,133,127,176,200,182,92,233,180,131,245,58,59,89,154,152,24,231,163,143,254,202,59,239,124,64,
  83,83,11,239,190,251,7,62,249,228,175,44,45,249,201,146,144,100,233,137,180,125,167,126,160,142,179,172,175,170,42,19,19,227,252,237,111,127,161,174,
  174,150,221,187,247,210,216,216,76,97,97,17,249,249,5,247,225,29,21,247,53,201,241,255,86,211,82,76,76,140,209,219,219,67,79,79,23,203,203,75,
  129,68,15,79,31,46,232,41,151,222,36,73,148,30,11,108,23,80,236,95,126,243,151,152,152,159,159,103,96,160,139,221,187,15,220,113,251,203,203,139,
  244,246,222,118,235,120,57,131,123,182,32,189,157,10,250,61,185,237,104,248,200,146,206,216,216,48,51,51,227,212,215,183,222,113,251,83,83,163,140,143,
  143,186,117,208,130,3,234,211,211,142,119,38,157,65,143,138,159,36,90,153,65,113,250,250,250,232,239,239,39,28,14,81,90,90,74,94,94,62,161,80,
  216,21,157,124,120,199,237,29,255,230,230,6,75,75,139,172,175,111,224,72,33,4,3,182,141,45,61,73,254,107,232,111,7,69,33,80,87,207,127,153,
  39,38,38,248,248,227,191,241,206,59,239,81,95,223,192,251,239,255,145,143,62,250,43,139,139,75,247,229,89,218,89,2,130,120,128,21,237,31,175,251,
  59,179,184,114,166,184,168,191,208,235,131,182,125,231,154,59,65,253,126,27,51,12,131,161,161,97,134,135,71,8,133,84,74,74,74,200,207,207,39,28,
  142,222,209,246,173,204,52,197,13,190,22,194,42,99,114,252,248,11,204,205,205,112,233,210,5,219,43,43,2,201,25,91,245,45,254,227,138,199,55,89,
  94,94,98,117,117,45,96,251,126,98,228,191,15,100,17,92,73,148,30,187,1,61,155,56,88,240,230,241,214,168,157,37,51,77,211,24,28,236,161,177,
  177,157,156,156,220,109,247,215,221,125,149,68,34,17,8,74,117,148,98,157,204,12,107,134,180,179,82,19,143,203,160,127,183,237,232,239,64,252,94,132,
  238,238,27,84,86,214,109,235,85,210,117,141,161,161,30,123,9,41,93,91,71,248,116,74,196,19,215,142,247,67,150,172,184,35,111,89,78,85,77,76,
  83,177,107,170,9,146,201,20,211,211,51,8,49,147,101,128,123,120,100,201,159,41,20,244,22,5,201,209,157,72,210,118,3,103,182,107,58,57,57,193,
  199,31,255,157,183,223,126,143,154,154,58,222,127,255,143,124,242,201,135,204,205,205,19,10,165,103,231,153,174,116,128,159,44,249,237,59,91,17,86,39,
  139,41,253,243,96,221,49,2,36,239,201,36,72,217,132,25,69,198,125,158,126,238,206,251,96,97,100,243,190,108,223,185,70,224,197,221,41,138,95,149,
  94,33,149,50,153,157,157,99,110,110,238,14,182,191,85,89,17,197,45,78,187,185,185,97,47,249,59,118,170,163,235,230,29,87,4,178,217,126,80,88,
  214,200,200,114,75,207,80,149,222,164,187,135,34,155,224,222,111,246,244,142,206,211,179,16,1,105,121,71,231,197,42,173,97,189,246,167,186,91,131,188,
  201,242,242,18,3,3,183,183,221,239,236,236,36,227,227,195,54,41,240,110,40,231,198,244,246,27,148,182,15,133,130,159,251,59,164,12,33,180,199,180,
  29,131,37,74,130,237,232,184,204,117,221,96,126,126,134,241,241,193,59,120,147,198,152,152,24,205,232,152,188,76,18,197,45,170,155,89,34,64,60,118,
  237,120,63,100,201,155,61,155,25,106,222,254,14,216,95,30,39,40,209,240,40,143,63,232,93,12,102,184,221,153,36,109,215,14,206,128,227,143,95,51,
  12,131,169,169,73,62,250,232,175,204,204,76,81,93,93,195,7,31,252,137,202,202,202,192,125,238,183,151,116,18,144,94,196,212,95,122,194,25,84,253,
  147,131,244,191,249,69,6,211,181,122,158,148,254,210,211,1,10,158,159,191,205,188,254,64,73,107,163,76,82,117,47,231,127,103,219,55,210,82,234,131,
  217,105,59,141,39,242,19,193,72,36,98,159,191,213,167,100,203,140,187,91,219,207,118,172,193,152,36,73,146,36,81,122,196,158,143,244,65,93,81,252,
  3,172,74,40,164,248,10,180,250,133,241,68,96,54,228,172,41,143,142,14,176,186,186,180,197,141,97,208,211,115,157,84,74,203,8,54,78,239,80,156,
  98,177,161,80,102,199,29,172,143,245,232,6,250,59,145,163,244,118,116,206,201,79,150,130,5,91,61,215,115,79,207,13,18,137,120,214,253,166,82,73,
  250,251,173,165,75,107,22,103,186,30,185,244,14,59,189,29,253,199,240,184,180,227,131,36,77,217,50,112,188,142,246,225,72,1,220,105,112,240,189,203,
  56,86,47,91,49,59,153,219,238,152,183,34,75,94,93,69,47,134,109,118,118,150,143,62,250,144,137,137,49,187,42,252,63,80,85,85,149,197,118,51,
  73,144,255,225,39,61,254,71,38,81,16,25,19,175,7,69,24,30,6,65,202,188,207,131,109,147,78,132,156,83,72,111,195,173,218,243,126,207,63,104,
  251,102,150,218,112,119,246,40,223,137,48,57,222,166,96,219,100,247,166,239,212,246,131,199,79,150,227,151,36,233,65,64,214,122,187,71,130,228,148,115,
  16,129,217,145,146,38,46,182,205,210,145,106,213,228,17,194,173,77,129,105,152,172,173,173,49,60,220,199,254,253,199,50,246,63,62,62,194,220,220,52,
  186,102,184,53,123,4,194,170,29,167,88,175,179,221,20,1,55,45,184,117,127,220,89,148,157,7,255,176,10,124,110,215,142,126,242,115,167,118,116,126,
  171,8,5,129,2,246,146,145,161,155,172,173,174,50,48,208,197,158,61,135,178,122,147,230,231,166,173,184,4,251,156,157,250,96,138,93,115,41,189,29,
  189,230,216,190,29,205,180,37,188,59,181,227,157,42,89,137,59,126,235,254,239,95,119,95,34,179,158,152,91,165,208,81,40,182,235,206,249,191,251,80,
  239,65,159,77,7,171,89,120,199,105,98,184,71,159,93,37,105,187,209,8,247,158,180,234,129,89,207,86,237,68,195,174,243,102,32,66,130,249,185,121,
  62,253,228,239,188,249,214,187,212,215,55,242,193,7,127,228,147,143,255,198,228,228,132,181,127,33,80,132,129,34,140,52,237,38,207,142,45,79,175,138,
  170,168,190,122,93,86,189,58,199,46,85,69,177,235,142,121,125,133,243,16,152,89,9,238,189,46,73,253,150,19,74,132,85,167,209,239,97,11,169,42,
  33,53,100,63,43,110,189,50,220,218,122,138,123,254,166,138,87,171,77,245,250,187,96,124,207,221,157,127,122,77,69,167,38,167,103,221,202,150,85,246,
  156,99,201,102,96,233,247,15,102,230,221,226,222,75,88,82,221,238,221,108,138,109,109,63,253,189,176,219,201,179,251,123,180,125,137,173,137,146,108,201,
  123,188,233,179,40,163,102,115,175,103,247,160,224,35,86,86,237,40,167,118,151,174,235,12,13,245,82,87,215,66,73,73,89,192,11,50,56,216,69,50,
  153,66,55,172,90,95,78,61,50,161,8,20,85,32,20,101,91,101,102,127,157,34,195,48,17,78,161,84,195,14,60,21,191,125,25,134,108,237,168,170,
  34,208,30,206,251,187,106,71,187,111,53,157,118,52,12,134,134,122,104,104,104,37,63,223,203,208,74,38,19,116,119,95,39,229,100,183,217,51,69,167,
  29,85,20,76,197,220,113,59,234,186,105,23,70,53,16,38,24,34,168,144,126,199,118,220,73,173,175,157,124,239,65,144,86,197,218,145,85,47,205,247,
  80,109,50,43,132,87,179,16,30,77,189,49,251,26,11,187,100,136,69,178,77,223,0,97,34,76,223,48,101,152,150,223,220,220,185,8,164,137,87,60,
  24,211,14,113,55,177,211,163,236,115,214,33,36,96,97,113,145,79,63,253,136,183,223,126,143,250,250,70,222,123,255,15,124,242,201,223,25,31,31,69,
  24,2,221,173,143,39,2,75,128,254,90,93,174,247,41,164,216,182,44,2,215,93,81,21,183,206,159,98,42,168,120,181,186,210,151,134,12,195,42,134,
  253,176,38,61,59,154,8,185,226,136,65,15,144,187,164,29,178,107,244,169,86,173,178,160,112,150,115,254,166,117,111,2,33,84,183,239,115,9,168,184,
  251,243,207,218,167,167,217,190,80,184,39,219,183,174,143,112,184,183,221,207,216,147,82,27,134,105,56,87,50,80,203,209,101,57,219,50,37,187,174,167,
  233,205,26,132,98,19,47,103,226,233,179,93,153,121,121,191,68,73,226,174,7,119,127,208,101,250,179,63,246,192,191,30,159,105,160,153,110,115,43,19,
  78,39,30,223,100,112,176,139,194,194,83,110,105,147,209,209,1,102,102,38,93,149,85,191,215,66,81,148,128,231,200,127,220,233,42,211,142,208,154,162,
  56,129,175,166,207,83,98,4,8,193,131,190,177,182,171,172,237,119,201,167,199,112,236,164,29,179,45,99,174,175,175,211,219,123,131,67,135,158,119,255,
  62,52,212,195,242,242,130,27,127,226,108,206,219,159,178,163,118,116,6,38,175,29,133,221,126,166,251,252,184,43,52,223,201,174,253,75,28,254,137,128,
  19,79,161,40,202,35,57,47,171,140,132,129,147,73,100,217,191,237,254,193,127,237,252,106,217,119,103,215,233,223,75,207,12,116,246,17,10,193,210,210,
  18,31,127,252,33,239,188,243,1,13,13,77,188,247,222,239,249,228,147,191,49,54,54,234,42,62,27,134,146,17,164,155,89,212,212,107,127,191,71,211,
  178,123,65,122,164,132,16,86,233,13,199,6,189,146,28,60,178,218,115,119,154,80,166,63,103,134,36,88,159,249,175,181,19,80,109,154,158,16,168,179,
  84,110,181,237,221,159,255,86,19,54,255,177,121,241,101,202,61,217,126,112,41,76,248,202,220,176,133,82,182,121,151,109,141,125,220,74,192,222,173,190,
  213,191,175,223,174,79,151,68,73,34,75,7,45,210,110,156,236,65,134,37,37,165,20,23,23,83,92,92,66,110,110,30,161,80,104,75,67,247,103,170,
  121,157,8,132,195,17,91,211,197,234,25,98,177,92,58,58,14,160,40,74,32,59,226,78,169,159,134,97,16,143,111,178,180,180,196,210,210,34,243,243,
  243,104,154,102,123,67,132,93,231,203,17,28,116,84,175,127,219,27,203,239,9,74,143,89,72,111,199,146,146,18,138,139,75,201,205,205,37,20,10,109,
  145,94,157,189,29,193,36,55,55,15,195,48,92,194,25,137,196,216,179,231,176,235,13,218,169,80,167,97,24,108,108,108,176,188,188,200,210,210,18,243,
  243,243,54,105,53,237,125,123,237,232,124,255,73,32,75,119,99,215,69,69,150,77,151,148,148,146,159,159,127,207,245,9,31,4,12,195,180,237,122,145,
  165,165,69,22,22,22,208,52,205,29,56,45,91,22,118,121,138,123,179,235,76,178,100,218,100,37,168,181,20,10,193,202,202,42,159,126,250,119,94,127,
  253,45,90,91,219,121,231,157,15,248,226,139,79,24,25,25,218,178,54,158,99,183,170,106,21,242,45,40,40,164,168,168,136,188,188,124,106,107,27,0,
  104,104,104,38,149,74,17,143,199,237,116,240,21,150,150,150,3,222,26,167,176,177,16,222,115,122,41,158,135,97,127,119,67,188,253,239,139,138,10,41,
  42,42,166,180,180,140,194,194,34,138,139,139,41,43,43,7,224,232,209,83,44,45,205,179,186,186,198,252,252,156,123,173,173,235,33,220,62,204,127,254,
  86,230,218,206,206,63,243,56,253,9,27,170,107,251,133,133,133,174,237,231,229,229,19,14,135,119,212,199,249,175,19,64,97,97,17,0,197,197,165,28,
  63,126,26,77,211,220,99,219,105,161,105,211,132,68,194,178,7,171,79,95,32,153,76,96,154,138,61,1,244,236,222,177,85,73,150,36,81,122,40,55,
  127,182,106,205,254,108,40,69,81,104,104,104,160,189,125,55,205,205,45,238,141,254,160,80,91,219,72,109,109,227,125,109,99,109,109,141,209,209,33,122,
  123,123,232,235,235,69,8,13,195,80,16,66,247,105,199,24,59,74,133,127,16,237,232,116,72,254,44,54,69,81,104,110,110,102,215,174,78,26,26,26,
  41,45,45,123,160,237,216,220,188,235,190,183,177,186,186,194,200,200,48,61,61,93,244,247,247,249,58,108,17,208,224,113,200,210,227,216,39,221,141,93,
  215,213,213,209,209,177,135,198,198,38,42,42,42,31,187,115,89,95,95,99,116,116,196,189,30,169,84,202,190,22,186,111,230,109,248,200,206,206,7,138,
  224,192,226,120,13,179,147,165,229,229,21,62,255,252,99,222,124,243,93,90,91,219,121,243,205,119,249,234,171,207,24,25,25,202,18,32,12,209,104,132,
  134,134,102,154,154,154,169,174,174,161,162,162,42,99,153,185,174,174,129,186,186,6,251,216,117,102,103,103,152,156,156,96,100,100,136,225,225,33,91,38,
  228,209,15,144,217,73,146,178,101,240,182,162,8,234,235,27,105,110,110,161,177,177,137,234,234,218,172,1,205,141,141,205,52,54,54,187,239,151,150,22,
  153,152,24,99,104,104,128,193,193,65,54,55,55,178,158,191,115,31,166,159,127,230,49,139,140,201,129,223,246,171,170,170,216,189,123,143,123,140,15,10,
  5,5,133,28,56,112,228,190,183,19,143,199,25,27,27,161,175,175,155,190,190,62,226,241,77,95,95,36,2,94,80,73,150,238,195,190,63,248,224,3,
  217,90,59,90,42,242,92,179,254,172,171,80,72,37,55,55,151,19,39,78,177,111,223,115,228,230,230,1,48,61,61,201,236,236,12,171,171,171,219,150,
  122,240,167,134,250,151,154,156,229,142,138,138,74,150,150,22,221,12,46,47,171,199,200,146,221,147,222,25,88,203,18,57,57,185,148,149,149,187,245,128,
  116,93,103,112,208,170,148,237,212,4,242,151,239,120,208,74,174,153,237,168,164,165,224,91,29,83,97,97,1,207,63,127,134,142,142,221,196,98,57,0,
  76,78,142,51,51,51,205,198,198,186,43,206,182,93,59,42,10,105,241,77,16,14,135,41,41,41,103,121,57,123,59,26,70,112,22,151,173,29,85,85,
  37,55,55,143,242,242,10,234,234,26,16,66,144,74,165,232,237,237,230,194,133,179,204,207,207,161,105,70,214,218,98,219,181,163,101,67,94,102,159,245,
  172,82,86,86,198,31,254,240,103,74,74,74,248,63,254,143,255,47,139,139,139,246,53,210,221,253,104,154,17,88,86,122,208,118,29,139,197,120,254,249,
  23,232,236,220,71,126,126,62,0,51,51,83,204,204,76,219,118,157,10,92,143,135,149,0,224,188,207,205,181,236,186,174,174,129,80,40,132,97,24,174,
  93,79,78,78,164,101,172,221,159,93,103,91,54,78,151,225,112,50,34,115,115,115,121,253,245,183,216,181,107,55,27,27,235,124,251,237,151,12,15,15,
  4,200,210,238,221,123,233,236,220,71,77,77,157,235,153,91,90,90,96,110,110,150,181,181,85,146,201,4,154,166,163,170,214,117,200,207,47,160,172,172,
  130,226,226,18,0,116,93,103,124,124,140,91,183,174,115,243,230,245,180,250,145,186,79,218,224,225,212,127,188,155,137,80,109,109,45,135,15,31,165,169,
  169,197,237,47,151,151,151,152,158,158,100,105,105,145,141,141,117,226,241,56,166,105,18,14,135,201,201,201,165,176,176,144,242,242,74,106,106,234,220,125,
  142,143,143,114,227,198,53,110,220,184,22,208,81,243,215,209,220,170,116,71,186,237,43,138,149,157,236,216,126,56,28,226,248,241,147,236,219,119,192,109,
  243,133,133,57,166,167,39,89,89,89,33,149,210,50,4,39,253,205,154,53,241,68,224,150,79,113,190,147,45,43,45,125,233,223,254,118,192,83,21,141,
  198,40,45,45,163,182,182,158,104,212,210,139,27,29,29,230,231,159,207,49,52,52,232,158,191,211,103,56,239,165,58,183,244,40,237,232,102,190,23,227,
  200,238,154,181,110,168,202,202,74,222,120,227,109,183,32,103,87,215,77,174,95,191,106,223,240,27,246,50,151,55,224,102,217,122,70,188,142,63,62,39,
  26,141,146,72,36,50,148,91,29,163,207,126,83,101,110,59,39,39,135,194,194,66,218,218,58,56,124,248,40,237,237,29,84,86,86,243,221,119,95,209,
  221,125,59,99,22,13,138,59,59,75,159,129,220,75,59,6,59,166,32,73,10,133,20,106,106,106,121,227,141,183,168,170,170,1,224,218,181,203,244,244,
  116,49,63,63,207,198,198,186,61,192,221,123,59,70,34,81,219,61,125,183,237,232,117,80,14,233,44,42,42,100,215,174,78,14,30,60,194,222,189,251,
  169,173,173,227,171,175,62,99,112,112,192,39,85,96,218,113,37,194,94,146,187,255,142,201,153,1,123,157,48,247,149,221,180,157,93,151,151,151,241,198,
  27,111,211,208,208,228,218,245,205,155,215,89,88,152,223,214,174,127,171,190,55,93,185,57,221,174,91,91,219,57,116,232,40,109,109,187,168,170,170,230,
  219,111,191,162,171,235,182,219,54,254,85,66,191,93,239,20,233,158,37,48,3,222,67,71,57,91,81,4,241,248,38,95,127,253,25,134,161,179,123,247,
  94,94,123,237,45,190,255,254,75,134,134,250,41,45,45,231,196,137,23,168,171,107,64,85,85,214,214,86,233,238,190,197,240,240,32,203,203,203,196,227,
  113,82,169,100,198,254,35,145,8,209,104,140,194,194,34,154,155,91,217,187,119,63,141,141,77,212,214,214,177,107,215,110,206,158,253,129,233,233,105,31,
  161,20,62,175,151,241,155,122,19,182,34,73,233,30,154,72,36,204,241,227,39,57,120,240,8,121,121,22,241,190,125,251,38,189,189,93,204,207,207,217,
  118,149,218,114,31,14,97,106,106,106,97,239,222,231,168,171,107,160,186,186,134,246,246,14,190,255,254,27,230,231,231,81,20,129,166,233,248,203,201,100,
  243,166,56,19,83,199,246,67,33,143,44,229,231,23,240,250,235,111,178,107,215,110,0,6,7,251,185,113,227,42,115,115,179,129,99,204,182,156,154,189,
  223,203,156,188,249,175,129,63,43,57,147,48,109,77,186,34,145,24,5,5,249,52,54,54,115,240,224,81,26,26,154,40,47,175,228,252,249,31,185,124,
  249,162,111,91,94,129,111,167,16,176,244,42,73,143,82,214,25,232,78,58,194,173,6,146,224,236,209,155,57,86,86,86,242,254,251,127,160,172,172,130,
  229,229,37,190,253,246,75,70,71,135,73,38,83,182,209,7,211,86,179,237,199,127,227,56,157,205,86,226,99,206,54,179,107,220,100,155,253,146,33,112,
  167,40,10,229,229,229,60,255,252,25,218,218,118,145,72,36,248,242,203,79,185,125,251,166,207,83,161,251,100,240,239,174,230,217,86,199,18,140,85,8,
  122,79,106,107,107,121,255,253,127,160,168,168,152,153,153,105,126,248,225,27,38,38,198,108,189,168,204,244,231,187,105,71,191,130,247,131,110,199,138,138,
  74,94,121,229,117,234,234,26,136,199,55,249,232,163,15,25,28,236,183,75,211,104,174,199,199,175,20,157,126,236,119,227,81,242,207,16,253,30,146,7,
  109,215,101,101,165,252,254,247,127,166,188,188,130,165,165,69,190,255,254,107,70,70,134,92,187,222,201,245,248,109,61,188,100,21,110,44,45,45,229,197,
  23,95,161,173,109,23,186,174,241,217,103,31,115,235,214,141,12,187,246,43,23,223,237,177,167,15,86,233,154,91,225,112,200,126,182,60,65,47,189,244,
  26,187,119,239,35,145,136,211,215,215,77,83,83,43,249,249,5,108,108,172,115,229,202,37,110,221,186,206,250,250,218,150,74,224,217,239,37,133,130,130,
  2,246,238,125,142,35,71,142,19,141,70,89,95,95,227,187,239,190,230,214,173,155,105,197,128,245,109,61,43,247,211,63,110,231,157,76,215,64,43,42,
  42,226,181,215,222,164,173,205,90,250,238,237,237,230,226,197,11,204,206,206,186,5,102,183,139,17,244,147,5,33,4,249,249,121,236,223,127,144,195,135,
  143,17,139,229,176,178,178,204,215,95,127,65,95,95,175,155,204,145,238,221,205,166,59,231,8,241,134,66,214,117,203,207,207,231,131,15,254,129,250,250,
  70,54,55,55,56,123,246,123,250,251,123,72,38,147,25,194,148,153,199,188,253,4,46,157,52,101,107,219,173,238,169,116,205,37,191,253,1,228,231,23,
  112,252,248,41,246,238,61,128,105,154,252,240,195,55,252,250,235,207,110,137,44,199,30,156,62,253,94,108,255,89,134,186,123,247,238,255,250,52,147,164,
  96,246,148,146,54,160,110,45,84,230,125,238,136,189,169,1,1,194,188,188,60,222,125,247,3,170,170,106,152,154,154,228,227,143,255,202,196,196,184,235,
  250,117,106,185,101,6,11,155,89,222,103,42,171,250,111,112,255,50,78,122,29,159,108,219,203,212,22,241,103,84,152,108,108,108,48,48,208,75,44,22,
  163,174,174,129,250,250,6,38,39,39,88,93,93,117,103,30,233,177,53,193,155,253,238,218,209,191,156,232,169,132,91,237,88,92,92,196,123,239,253,129,
  178,178,114,134,135,7,249,228,147,15,153,157,157,65,211,244,128,224,95,122,59,102,59,103,127,93,35,255,210,195,111,219,142,125,20,20,20,80,93,93,
  75,67,67,35,163,163,195,108,108,108,4,58,78,175,77,239,228,213,241,102,184,185,185,185,116,118,238,37,39,39,135,203,151,47,186,75,17,193,114,35,
  102,134,140,130,183,148,123,111,118,29,139,197,120,255,253,63,82,93,93,195,204,204,20,31,127,252,33,227,227,99,1,187,222,233,245,248,173,30,217,175,
  7,108,110,110,50,52,52,64,36,18,161,182,182,158,166,166,22,38,38,198,88,93,93,9,216,245,131,90,74,78,95,14,201,150,76,48,62,62,66,78,
  78,46,213,213,117,84,86,86,19,137,68,25,26,234,231,243,207,63,162,175,175,135,120,60,158,81,147,43,187,194,50,1,49,205,68,34,206,248,248,24,
  67,67,253,20,20,20,82,89,89,77,91,219,46,52,77,99,122,122,50,195,35,26,60,231,204,242,40,91,221,215,233,247,241,86,158,226,108,75,184,14,
  73,42,43,43,227,221,119,127,79,99,99,51,235,235,107,124,255,253,87,252,248,227,119,246,242,173,30,240,236,166,103,131,25,70,186,144,168,245,72,38,
  147,140,141,141,50,50,50,68,113,113,41,149,149,85,180,183,239,98,125,125,141,217,217,153,52,34,66,86,207,159,163,142,237,28,111,52,26,225,173,183,
  222,165,165,165,141,149,149,101,62,255,252,35,134,135,7,220,88,55,103,91,233,36,41,56,217,194,231,189,201,244,16,249,61,146,59,37,163,217,38,9,
  233,36,85,211,52,70,70,134,208,52,141,186,186,6,154,155,91,89,92,156,103,110,110,46,235,177,58,203,126,18,207,40,81,74,87,129,13,198,197,4,
  211,83,211,189,16,233,29,66,48,40,209,155,121,135,195,97,94,120,225,12,157,157,123,89,92,92,224,163,143,254,194,194,194,66,96,32,241,15,38,94,
  252,128,153,209,1,166,107,160,100,202,231,7,31,126,239,196,78,182,151,205,171,229,4,179,142,143,143,80,80,80,72,109,109,61,165,165,101,244,247,247,
  184,25,113,193,89,92,102,45,170,187,109,199,244,96,73,167,35,125,253,245,183,104,110,110,101,106,106,130,207,63,255,136,149,149,85,223,140,248,94,219,
  209,120,160,237,232,47,21,144,62,235,213,52,141,177,177,97,42,42,42,169,170,170,166,184,184,132,190,190,30,116,93,207,226,165,202,94,108,119,39,68,
  201,10,218,205,92,226,82,20,178,150,117,184,23,187,86,85,149,151,94,122,133,206,206,189,44,47,47,241,241,199,127,101,110,110,238,190,236,250,65,63,
  238,100,215,134,161,51,62,62,74,65,65,17,213,213,53,84,85,85,211,213,117,211,190,30,233,3,151,121,31,74,203,98,155,123,195,175,23,4,229,229,
  149,212,214,54,160,235,58,151,47,255,202,119,223,125,201,218,218,122,128,172,251,137,194,214,228,48,232,13,5,216,216,216,160,191,191,7,85,85,168,169,
  169,163,165,165,149,84,42,201,196,196,68,160,76,210,118,94,142,204,180,125,191,228,134,71,190,211,237,104,39,222,226,226,226,98,222,123,239,31,168,169,
  169,101,110,110,150,207,62,251,187,157,0,17,44,92,236,191,111,131,231,107,164,17,69,255,249,11,54,54,214,233,239,239,37,55,55,151,234,234,90,90,
  91,219,237,76,176,185,52,114,2,254,24,198,116,219,15,133,84,142,30,61,198,225,195,199,73,36,18,124,246,217,223,153,158,158,204,240,252,100,22,204,
  54,179,198,24,249,73,109,112,249,54,115,114,121,39,50,154,110,111,233,4,213,63,73,154,154,154,32,20,10,83,93,93,75,117,117,13,125,125,61,110,
  200,129,119,44,153,194,156,18,207,40,81,242,150,22,252,51,135,204,34,133,65,173,30,145,53,166,198,127,243,171,170,149,5,244,210,75,175,33,4,124,
  254,249,199,76,77,77,166,5,16,26,25,46,254,59,205,186,179,147,166,108,143,76,55,122,246,7,129,65,197,63,40,56,231,106,24,6,51,51,83,52,
  53,181,80,85,85,205,218,218,42,83,83,147,1,143,82,48,51,68,164,165,241,239,188,29,179,197,193,52,55,183,240,202,43,175,219,29,211,71,238,160,
  236,15,194,245,7,164,62,170,118,12,150,197,48,3,131,141,21,131,164,51,61,61,73,71,71,39,165,165,101,172,173,173,50,61,61,157,225,157,203,214,
  57,221,157,71,137,45,202,214,220,187,93,251,99,72,106,107,107,121,229,149,215,81,85,149,207,63,255,152,241,241,177,251,182,235,7,255,216,153,93,207,
  206,78,209,216,216,76,69,69,37,154,166,49,62,62,26,40,197,98,93,215,7,209,215,248,197,20,157,129,215,155,152,117,118,238,231,212,169,51,8,33,
  56,127,254,71,126,253,245,156,91,10,197,122,4,73,123,186,174,78,54,239,138,127,192,115,6,224,209,209,97,192,202,146,107,104,104,98,105,105,129,249,
  249,249,172,222,36,199,252,210,239,231,173,239,107,255,242,151,200,186,228,150,205,91,28,139,197,120,239,189,223,83,95,223,192,220,220,44,159,124,242,33,
  83,83,83,1,130,228,47,21,147,233,153,241,191,54,178,220,131,142,12,132,193,240,240,32,209,104,140,154,154,58,26,27,155,153,152,24,103,109,109,197,
  245,222,248,201,77,250,189,22,10,169,148,148,148,240,250,235,111,18,139,229,240,227,143,223,48,60,60,224,214,105,203,203,43,160,170,170,134,213,213,101,
  119,255,225,112,132,170,170,26,18,137,132,59,185,52,77,40,40,40,160,180,180,140,245,245,181,192,242,150,105,235,79,170,170,74,77,77,173,157,69,87,
  141,162,40,108,108,172,187,237,88,88,104,17,252,245,245,181,128,109,71,163,81,218,219,59,104,104,104,34,55,55,143,213,213,21,192,36,26,141,210,214,
  214,65,125,125,3,57,57,185,172,174,174,48,59,59,69,121,121,21,149,149,85,132,66,42,131,131,253,105,237,105,202,37,55,73,148,130,18,249,14,73,
  242,215,233,10,138,156,101,243,138,136,172,117,137,20,69,33,26,13,115,244,232,73,26,27,155,184,117,235,6,87,174,92,76,91,230,241,58,63,191,78,
  207,195,154,113,111,87,16,49,93,68,17,172,204,45,195,208,105,106,106,161,188,188,130,91,183,174,187,222,16,175,99,244,183,71,102,221,181,244,118,12,
  14,34,65,151,188,227,185,80,85,133,55,222,120,147,146,146,82,110,222,188,198,245,235,87,50,72,210,227,213,142,153,177,3,126,155,177,60,62,58,173,
  173,237,68,34,17,250,251,251,208,180,212,29,179,141,238,134,40,249,139,18,248,127,247,32,236,58,20,82,120,254,249,23,169,171,107,160,167,167,139,95,
  126,57,231,14,206,254,76,170,244,202,228,193,229,183,135,231,81,10,46,103,102,183,107,77,75,209,222,222,65,97,97,33,93,93,183,73,165,146,15,52,
  182,106,43,157,32,231,26,84,85,213,240,198,27,239,16,137,68,248,245,215,243,92,188,120,193,141,21,241,215,141,203,70,216,253,158,195,173,200,162,55,
  0,91,54,49,57,57,78,56,28,162,190,190,145,250,250,70,134,134,6,216,216,216,200,88,118,244,103,237,5,151,202,20,95,29,67,127,209,103,37,45,
  70,6,252,66,175,233,247,183,227,45,126,225,133,151,216,183,239,57,86,87,87,249,232,163,255,96,118,118,214,45,50,156,190,4,110,24,153,247,137,63,
  43,204,48,200,122,254,254,73,192,216,216,8,197,197,197,84,87,215,80,89,89,201,208,208,128,157,8,131,123,239,164,175,50,56,253,217,161,67,71,233,
  232,232,100,124,124,132,243,231,127,12,120,41,15,31,62,198,153,51,175,50,62,62,206,202,138,165,97,85,82,82,202,219,111,191,199,216,216,40,107,107,
  171,238,49,236,219,247,28,199,143,159,160,187,251,118,32,219,217,177,203,188,188,60,254,225,31,254,76,52,26,69,81,20,14,28,56,130,162,40,204,206,
  78,33,132,224,196,137,231,121,241,197,87,25,30,30,100,125,125,13,33,172,248,163,87,94,121,131,170,42,139,88,149,148,148,50,60,60,68,110,110,46,
  47,189,244,58,85,85,213,168,170,74,97,97,49,83,83,147,36,18,9,54,54,214,105,109,109,167,162,162,138,158,158,219,108,110,110,102,216,205,227,42,
  93,242,56,34,244,180,145,164,244,14,204,185,249,243,242,242,49,140,84,96,70,31,156,217,56,193,110,94,160,155,55,168,120,219,202,207,207,167,179,115,
  15,169,84,210,118,233,251,99,98,140,64,138,248,78,178,34,238,20,99,149,30,208,152,190,169,59,214,50,178,59,56,93,55,125,10,174,193,250,105,61,
  61,221,28,56,112,152,202,202,106,26,26,154,232,233,233,118,131,4,45,229,99,211,157,137,229,229,229,161,235,247,210,142,65,133,231,202,202,74,170,171,
  107,73,38,147,92,189,122,41,45,54,43,24,120,123,175,237,184,93,91,222,77,27,250,219,209,48,60,37,244,224,223,76,122,122,186,56,122,244,4,181,
  181,245,84,87,215,48,48,208,159,177,84,33,196,189,117,78,222,82,9,129,253,231,228,228,0,86,155,109,117,61,252,30,161,173,236,186,184,184,132,134,
  6,75,167,235,202,149,95,51,150,45,131,203,110,119,10,96,101,139,37,159,173,201,226,206,131,138,113,237,210,90,66,118,84,171,61,165,106,39,155,167,
  187,251,22,199,142,157,164,180,180,140,214,214,54,174,95,191,134,162,152,40,138,165,185,227,120,12,238,237,122,136,140,101,44,127,221,145,88,44,198,137,
  19,207,19,139,197,232,233,185,205,47,191,156,119,201,145,174,27,110,128,109,48,155,51,40,23,18,60,231,96,155,59,182,168,170,142,66,184,138,174,27,
  156,63,255,19,165,165,229,180,182,182,243,242,203,175,241,31,255,241,63,92,175,159,163,255,228,111,47,85,21,132,66,33,242,243,243,73,36,226,129,140,
  44,63,81,246,39,120,24,129,122,134,100,120,139,85,85,161,185,185,133,67,135,142,96,154,38,223,127,255,21,51,51,51,1,146,228,245,145,233,253,155,
  119,93,178,157,191,39,240,106,157,127,208,166,52,190,253,246,43,74,74,202,168,172,172,230,240,225,99,124,251,237,215,168,170,115,188,193,115,119,238,131,
  220,220,92,90,90,218,0,236,9,155,23,147,100,73,80,84,176,176,48,207,174,93,187,25,27,27,177,251,68,97,147,29,255,114,153,73,40,20,34,18,
  137,100,244,17,78,127,25,10,133,8,135,35,92,191,126,149,201,201,49,246,236,217,207,201,147,167,233,237,189,77,78,78,46,197,197,165,108,108,172,211,
  212,212,204,236,236,52,134,97,208,214,182,139,242,242,114,254,246,183,255,96,117,117,133,88,44,7,93,215,104,109,221,69,105,105,25,159,125,246,55,86,
  87,87,200,201,201,33,149,74,0,38,19,19,163,76,78,78,208,216,216,76,71,199,30,22,22,126,10,156,179,213,118,130,187,85,3,127,86,161,60,141,
  129,220,126,229,103,39,19,101,255,254,253,252,243,63,255,47,116,116,116,144,151,151,67,78,78,148,156,156,136,251,136,70,173,71,36,18,114,51,87,60,
  141,31,213,157,53,150,149,85,144,155,155,199,212,212,164,111,134,100,100,153,37,109,189,172,179,243,216,7,210,178,36,252,30,130,173,7,253,173,130,95,
  211,3,154,157,207,45,209,178,81,0,154,155,91,3,241,93,158,71,206,106,151,99,199,142,241,207,255,252,95,104,107,107,35,55,55,150,165,29,195,68,
  34,225,140,118,116,178,75,156,89,106,125,125,3,177,88,140,169,169,73,230,231,231,3,3,115,186,6,204,118,203,99,59,37,73,193,246,220,186,253,182,
  106,67,191,199,100,171,192,240,245,245,117,6,6,250,80,85,149,234,234,154,172,153,89,247,125,195,250,68,33,67,33,149,206,206,78,254,243,127,254,127,
  178,123,247,238,52,187,142,186,118,157,237,122,120,54,109,93,143,242,242,10,74,74,74,153,158,158,98,97,97,62,235,50,102,112,153,203,31,44,156,253,
  145,205,86,183,138,155,202,118,141,118,64,155,178,44,177,26,129,152,170,254,254,94,132,176,236,45,24,223,225,63,246,251,239,107,130,181,30,161,185,185,
  141,230,230,54,150,151,151,56,127,254,71,146,201,164,75,148,82,41,45,64,154,252,37,137,44,175,135,165,41,86,81,81,65,65,65,129,155,193,233,197,
  201,24,62,157,156,224,196,34,153,76,241,195,15,223,176,177,177,65,115,115,43,251,246,61,183,133,234,180,151,237,21,137,132,121,251,237,119,120,231,157,
  247,169,170,170,180,109,199,122,196,98,214,61,237,60,7,109,72,113,237,40,232,157,10,113,244,232,113,34,145,8,55,111,94,163,167,167,59,64,222,211,
  151,28,253,247,178,16,16,139,197,104,110,110,161,179,115,15,197,197,197,129,243,183,38,2,94,252,98,42,165,103,220,131,63,253,244,45,0,207,61,119,
  144,218,218,90,159,157,7,99,176,156,137,91,81,145,229,133,90,89,89,102,110,110,54,16,144,93,91,91,143,174,235,156,63,255,35,213,213,181,68,34,
  17,215,102,53,45,149,81,227,211,234,103,181,172,171,28,142,253,105,90,202,245,54,79,77,141,187,199,86,89,89,77,50,153,228,194,133,159,104,108,108,
  38,22,139,217,114,7,41,162,209,152,61,41,130,120,124,211,213,113,139,70,163,68,163,49,183,114,128,67,62,77,211,100,120,120,0,128,134,134,198,44,
  201,30,146,252,60,51,30,165,236,89,24,164,105,121,88,55,117,36,18,162,174,174,129,154,154,58,38,38,70,185,121,243,10,179,179,211,36,18,9,119,
  134,103,117,98,186,27,27,19,244,78,9,219,152,45,101,226,185,185,89,219,157,153,30,68,188,189,184,219,214,153,35,164,197,146,100,6,240,121,179,43,
  145,213,61,237,108,43,51,128,48,88,134,193,169,13,101,213,204,50,81,20,75,32,83,215,117,42,43,43,3,174,121,255,141,30,10,41,68,34,17,234,
  234,234,169,169,249,71,70,70,6,232,234,186,206,204,204,12,169,84,210,109,199,80,72,243,13,2,102,214,101,138,178,178,114,132,16,140,143,143,100,16,
  145,244,184,151,187,241,68,248,59,164,173,218,51,243,154,108,223,134,59,111,71,147,169,169,9,14,30,60,66,105,105,41,161,80,8,93,55,80,20,199,
  211,129,59,139,203,12,228,36,107,128,167,63,0,215,155,164,122,203,103,145,72,152,202,202,42,222,123,239,143,140,143,143,112,251,246,53,102,102,38,73,
  36,146,129,65,249,78,118,93,86,86,102,95,143,81,18,137,164,111,201,195,72,91,218,98,219,99,12,198,206,100,243,214,152,89,179,198,210,175,169,95,
  204,111,219,154,194,62,143,80,122,118,145,149,176,48,10,60,79,73,73,41,170,26,114,21,156,189,153,245,221,123,181,252,251,246,151,192,112,238,147,156,
  156,92,14,30,180,84,151,47,95,254,213,46,175,226,17,26,63,65,74,247,132,91,30,30,149,183,222,122,135,221,187,247,112,227,198,117,254,254,247,191,
  218,19,177,244,64,116,211,30,120,253,222,52,152,159,159,231,242,229,95,120,225,133,151,217,191,255,0,93,93,183,48,140,132,93,55,78,177,203,21,249,
  39,146,42,57,57,185,236,218,213,72,123,251,46,122,123,111,211,211,115,147,149,149,149,172,228,206,89,50,76,175,55,233,144,165,182,182,118,26,27,155,
  72,36,226,252,252,243,185,12,114,231,247,20,103,171,249,88,90,90,202,7,31,252,129,162,162,98,62,253,244,99,46,93,250,213,46,17,20,140,95,114,
  108,217,89,225,114,238,161,209,209,81,110,222,188,198,190,125,7,216,187,119,63,147,147,147,40,138,229,81,11,46,155,91,143,226,226,18,84,85,101,122,
  122,146,141,141,245,192,189,95,83,83,207,220,220,44,35,35,67,28,59,118,138,198,198,22,250,251,187,211,38,174,254,160,118,124,75,220,74,22,178,100,
  218,251,44,38,149,74,112,242,228,139,204,204,76,97,24,6,117,117,13,44,47,47,210,219,219,197,225,195,199,168,168,168,100,116,212,82,221,46,47,175,
  228,189,247,254,129,161,161,1,46,95,254,133,229,229,37,122,123,111,83,94,94,206,219,111,127,192,240,240,32,87,174,252,234,46,13,130,112,99,103,139,
  138,138,201,201,201,69,211,86,211,200,189,36,64,79,60,81,218,106,16,204,28,92,210,7,19,145,166,107,98,145,36,103,208,87,20,133,250,250,38,234,
  234,26,25,27,27,162,175,239,22,51,51,83,108,110,198,73,38,189,65,62,232,94,22,110,32,158,83,171,199,10,166,99,139,180,229,187,91,22,243,214,
  206,201,226,206,103,75,2,228,79,21,79,143,157,217,138,44,165,103,216,57,5,39,151,151,151,209,52,141,188,188,124,159,56,99,80,43,198,82,216,245,
  180,92,154,155,219,105,106,106,99,120,184,143,254,254,110,102,102,38,137,199,147,164,82,154,219,177,102,107,71,69,81,92,165,231,165,165,197,172,237,232,
  95,222,217,233,210,88,250,210,69,58,1,245,183,175,127,219,254,24,134,237,246,229,111,199,244,34,195,78,59,58,118,145,159,95,64,52,26,37,149,74,
  97,154,138,189,68,146,185,100,229,87,97,79,47,253,224,28,175,227,5,112,206,195,179,113,53,64,102,235,235,155,168,175,111,98,108,108,136,254,254,46,
  166,166,198,183,181,107,103,91,150,46,77,190,107,215,193,204,34,182,244,0,109,71,48,50,51,203,204,0,201,207,118,189,178,217,122,186,16,104,230,49,
  145,225,21,242,159,159,115,62,209,104,148,188,188,60,52,77,179,151,223,76,91,136,82,185,131,104,235,150,22,151,22,155,163,4,74,190,84,87,215,48,
  59,59,99,103,64,102,198,227,56,246,157,30,12,237,220,103,206,242,77,52,26,33,28,118,186,105,175,160,170,63,209,192,41,128,234,20,137,21,194,228,
  198,141,171,28,60,120,148,138,138,42,90,91,219,233,234,186,77,40,244,255,103,239,191,130,227,200,210,52,65,244,115,17,2,90,107,173,65,16,4,9,
  106,149,146,169,179,42,179,170,186,170,186,107,186,103,122,102,215,230,225,94,91,179,107,182,47,247,245,62,92,179,49,219,53,187,119,198,108,118,238,206,
  206,246,244,118,247,212,180,40,157,217,41,200,204,100,102,82,107,18,36,72,104,173,129,128,138,64,104,23,247,225,248,113,63,199,221,3,130,154,172,56,
  101,40,32,193,64,132,251,241,35,190,243,253,255,255,125,86,201,58,5,38,100,125,244,152,253,239,243,249,209,217,217,141,230,230,118,140,142,246,99,124,
  124,24,203,203,203,230,156,78,36,172,67,144,85,106,110,129,69,143,71,70,99,99,51,36,73,198,205,155,215,16,12,6,29,108,31,29,95,118,144,196,
  142,105,54,143,136,132,21,233,248,215,64,4,52,249,92,37,250,25,162,168,67,81,20,60,120,112,15,109,109,187,208,218,218,142,219,183,111,96,105,41,
  96,24,229,218,173,86,96,142,253,141,141,144,169,233,68,194,110,89,40,45,45,195,202,138,140,150,150,54,19,4,142,140,12,130,23,162,21,76,117,127,
  118,142,186,53,85,37,190,147,123,246,116,35,26,13,35,28,14,227,214,173,171,240,122,125,168,170,170,198,220,220,44,234,235,155,224,241,120,80,95,223,
  136,169,169,73,168,170,138,139,23,191,199,216,216,48,14,28,56,130,15,63,252,49,62,251,236,119,88,91,91,195,133,11,223,99,108,108,4,7,14,28,
  198,251,239,127,132,47,191,252,20,107,107,171,198,90,180,142,68,34,1,89,246,32,51,51,19,27,27,27,174,85,139,233,182,13,160,244,188,117,150,147,
  154,231,55,58,39,18,22,28,161,9,74,13,211,144,131,207,231,117,24,211,10,130,128,154,154,6,84,85,213,97,118,118,28,227,227,35,152,156,28,51,
  23,4,183,220,24,73,146,76,250,51,153,76,112,167,146,84,229,158,169,238,143,119,162,118,215,194,73,53,217,216,242,104,186,72,146,92,13,125,27,42,
  177,130,35,79,9,16,140,202,13,50,137,189,94,15,52,77,1,91,237,70,100,17,36,206,217,155,190,103,125,125,11,106,106,26,48,61,61,134,241,241,
  97,204,204,76,25,11,170,123,63,82,197,113,0,136,199,227,41,61,152,88,48,177,221,113,66,251,211,14,54,121,250,222,98,133,156,32,119,107,19,91,
  183,74,50,250,61,153,84,140,156,17,153,235,47,107,236,170,134,27,60,31,74,227,147,105,41,0,178,64,169,245,62,108,82,189,196,108,162,86,171,174,
  174,71,85,85,29,102,102,198,49,62,62,108,136,160,38,82,142,107,178,33,147,231,145,72,196,93,196,241,68,227,160,160,167,4,242,118,54,136,7,191,
  112,101,6,237,33,9,103,24,204,189,196,58,213,124,178,3,78,234,247,70,88,63,34,235,97,1,75,58,239,52,35,103,105,103,64,201,94,5,202,62,
  183,142,142,78,0,192,216,216,48,54,54,54,56,0,206,86,232,177,137,245,172,117,140,199,35,115,99,138,254,55,177,52,161,249,66,130,237,154,5,238,
  231,104,52,134,251,247,123,112,228,200,9,212,215,55,96,100,100,200,44,210,96,199,157,199,35,195,231,243,56,114,106,252,254,12,116,116,116,163,161,161,
  21,147,147,195,24,31,31,193,210,210,18,188,94,247,67,16,109,133,133,69,168,175,111,68,50,153,196,200,200,144,203,28,18,108,99,74,100,170,47,173,
  195,173,185,65,201,132,53,165,33,59,81,84,161,40,212,12,88,119,140,67,58,222,22,23,151,48,49,49,142,166,166,22,84,87,215,26,242,45,110,54,
  38,132,37,39,99,159,87,68,47,41,41,133,44,123,16,137,68,144,159,95,136,229,229,0,202,203,43,225,241,120,204,60,38,154,27,200,134,117,41,131,
  198,206,115,107,220,72,80,20,5,55,110,92,49,61,0,5,65,64,93,93,3,100,217,131,141,141,16,74,74,202,176,176,48,207,89,78,1,192,236,236,
  12,102,103,127,139,159,255,252,207,177,107,87,39,46,93,186,0,0,152,153,153,198,236,236,12,126,250,211,95,96,215,174,78,92,190,124,222,212,86,34,
  7,3,1,30,143,199,161,105,71,128,165,248,80,54,72,105,70,233,57,203,53,226,43,21,156,202,184,110,140,18,91,5,228,241,200,240,122,101,199,6,
  111,13,92,17,213,213,141,168,168,168,69,125,125,19,198,199,71,48,53,53,206,37,36,211,141,148,110,80,44,187,96,167,193,55,15,19,216,43,194,96,
  211,92,113,210,248,246,123,164,27,7,81,242,213,204,251,182,196,205,182,206,235,96,95,195,107,38,9,6,61,207,111,58,116,19,247,122,61,76,66,56,
  223,36,73,70,93,93,11,42,42,106,177,176,48,131,209,209,1,204,206,78,187,246,35,93,48,88,208,146,138,17,216,201,56,97,109,9,236,27,49,189,
  23,182,15,73,98,53,189,38,13,100,61,218,217,134,233,198,186,88,63,139,102,127,209,19,177,221,239,201,206,128,178,33,12,150,81,98,129,18,125,61,
  205,23,73,213,47,213,213,13,168,168,168,65,67,195,52,198,199,135,49,57,57,198,61,15,250,76,4,65,228,158,135,61,172,64,147,87,237,21,119,169,
  67,157,110,218,83,206,42,165,84,115,155,61,16,216,171,116,88,54,208,206,66,57,15,27,236,245,233,230,198,108,7,23,170,10,179,96,97,39,227,142,
  175,114,147,204,98,143,186,186,70,196,98,81,76,77,77,184,130,69,210,127,252,179,101,5,26,101,89,226,0,32,249,111,75,88,144,233,33,110,115,182,
  214,41,242,243,232,232,48,14,31,62,142,202,202,106,228,231,231,155,236,45,237,119,2,228,73,238,90,170,67,89,70,70,22,218,218,246,162,170,170,1,
  51,51,227,24,29,29,194,202,74,192,145,220,77,231,109,105,105,41,178,178,178,49,49,49,102,134,129,88,6,204,13,104,178,146,2,244,139,222,63,253,
  189,61,223,141,116,131,150,50,183,48,145,136,99,106,106,2,77,77,45,104,104,104,50,20,218,21,142,81,178,198,136,104,92,167,202,172,79,34,26,26,
  154,49,63,63,139,115,231,190,5,160,195,227,241,225,23,191,248,151,104,104,104,198,204,204,36,36,73,130,207,231,135,36,121,224,245,138,136,68,162,140,
  63,93,134,89,57,156,72,36,184,180,12,81,148,16,141,70,145,72,40,102,31,180,183,239,198,232,232,8,46,95,190,8,146,20,238,193,95,254,229,191,
  69,69,69,37,150,150,22,225,241,120,17,12,174,35,39,39,23,30,143,23,107,107,107,240,251,253,240,120,60,8,6,131,200,205,205,133,215,235,67,40,
  20,226,0,228,102,197,10,105,66,233,5,5,74,78,16,68,232,109,55,0,145,42,20,199,158,240,60,30,250,37,167,220,224,217,141,190,186,186,17,165,
  165,85,104,108,108,197,232,40,9,93,176,54,20,252,228,114,3,49,219,175,158,178,54,117,50,81,90,90,154,145,72,36,48,55,55,107,222,103,99,99,
  35,226,241,4,230,231,231,80,94,94,129,186,186,58,72,146,140,64,32,128,129,129,126,68,163,49,99,177,16,77,157,145,237,123,88,241,9,148,246,138,
  65,26,238,161,253,233,241,16,134,100,171,126,244,122,125,168,169,105,68,105,105,5,150,150,230,49,50,210,135,197,197,121,46,252,192,230,159,208,210,227,
  84,192,99,59,121,34,108,232,82,20,69,180,182,182,34,28,14,99,97,97,158,163,246,155,154,154,205,36,235,185,185,89,12,13,13,154,128,147,132,25,
  245,29,92,131,123,245,26,203,210,209,208,12,175,6,44,64,20,249,252,14,251,169,154,126,209,100,99,22,56,89,225,8,121,219,227,186,170,170,30,165,
  165,149,104,108,108,195,232,104,31,230,230,166,185,113,173,235,112,0,31,187,77,136,36,9,168,172,172,66,117,117,13,50,51,51,144,76,38,49,63,63,
  143,169,169,9,243,89,150,149,85,160,182,182,222,124,38,27,27,33,244,244,220,70,77,77,29,20,69,193,212,212,148,13,44,144,247,110,111,223,133,68,
  34,142,137,137,113,100,101,101,163,161,161,17,163,163,35,216,216,32,90,50,25,25,25,104,104,104,194,216,216,152,25,218,164,225,205,242,242,114,172,175,
  175,33,16,8,56,180,163,248,205,223,234,71,30,4,178,64,99,251,99,142,205,241,97,165,1,202,202,202,224,245,122,49,63,63,139,133,133,121,102,93,
  208,93,199,172,211,66,70,228,128,11,13,103,241,33,70,133,3,95,252,58,100,205,235,80,40,132,229,229,37,195,76,55,223,236,59,22,160,208,131,228,
  86,145,133,236,236,92,180,181,117,161,186,154,0,166,145,145,62,172,175,7,185,220,35,58,6,0,34,124,72,109,63,220,216,56,150,41,181,131,36,137,
  49,230,163,107,143,40,170,92,206,90,170,170,75,54,28,183,184,184,128,104,52,138,202,202,42,120,189,62,40,138,202,244,149,14,103,117,161,85,53,154,
  153,153,129,188,188,2,244,244,220,50,153,56,69,137,98,108,108,20,21,21,85,24,29,29,70,44,22,195,107,175,157,66,52,26,129,44,123,112,253,250,
  85,172,174,174,34,51,51,11,31,126,248,177,145,116,174,224,226,197,243,152,154,154,52,67,111,241,120,12,170,170,27,108,167,128,204,76,31,242,242,242,
  113,231,206,45,195,167,14,80,148,56,38,38,198,80,89,89,139,68,66,197,235,175,191,137,104,52,2,159,207,143,233,233,73,60,120,112,31,117,117,245,
  56,122,244,56,98,177,56,124,62,31,102,103,137,73,50,205,59,229,5,106,245,52,56,122,25,25,37,246,180,201,178,27,155,177,45,110,140,146,117,66,
  217,222,167,122,189,62,84,86,214,161,164,164,2,75,75,115,24,30,190,143,229,229,37,51,108,65,79,31,246,112,217,195,223,27,89,44,15,29,58,130,
  213,213,21,44,45,45,152,44,211,254,253,135,12,225,184,69,28,57,114,12,94,175,23,195,195,131,216,189,187,19,29,29,187,241,251,223,255,22,225,112,
  196,60,45,63,174,92,48,10,142,216,28,1,250,181,221,230,243,101,160,186,186,1,165,165,149,88,88,152,198,200,72,31,150,151,151,204,147,168,117,98,
  118,243,62,218,30,179,195,110,136,236,248,56,118,236,56,166,167,167,16,8,44,154,255,230,247,251,241,218,107,111,96,126,126,14,43,43,203,56,121,242,
  85,52,53,53,227,244,233,47,144,76,90,121,22,59,3,155,128,83,195,7,92,104,135,108,160,58,52,77,96,146,136,69,7,80,98,181,120,236,140,146,
  245,60,192,0,215,157,141,107,143,199,139,138,138,26,20,23,151,97,121,121,1,67,67,189,88,90,90,48,159,7,15,148,248,195,9,221,160,79,158,124,
  21,62,159,15,125,125,247,145,157,157,141,147,39,95,69,52,26,193,55,223,156,70,44,22,195,238,221,157,104,106,106,193,216,216,136,153,52,236,241,200,
  232,238,222,143,141,141,13,204,205,205,112,108,16,169,238,202,195,169,83,111,35,30,143,227,151,191,252,191,160,170,73,236,217,179,23,133,133,133,184,112,
  225,28,52,77,195,190,125,221,104,106,106,193,200,200,144,9,12,203,202,202,241,230,155,167,144,72,36,81,88,88,136,7,15,238,227,242,229,139,92,85,
  147,29,44,209,254,37,9,181,60,232,161,225,236,157,204,95,167,80,163,128,146,18,82,236,177,186,186,194,8,17,234,182,252,43,75,81,157,7,9,162,
  131,81,17,4,202,38,178,90,104,18,151,31,199,106,41,177,64,33,153,76,96,97,97,30,197,197,165,40,40,40,196,236,236,180,11,216,222,217,188,206,
  202,202,65,107,235,30,212,212,52,98,114,114,4,163,163,125,8,133,54,204,181,177,176,176,200,188,255,84,32,83,215,157,185,164,124,20,64,226,124,237,
  236,227,220,169,7,228,4,138,186,14,172,175,175,33,20,10,162,180,180,12,217,217,217,136,70,35,204,250,34,184,204,97,43,151,42,28,14,227,203,47,
  63,67,40,20,52,89,88,65,0,206,159,255,14,94,175,15,27,27,97,252,238,119,191,130,223,159,97,94,235,250,122,16,241,120,20,127,255,247,255,13,
  30,143,199,100,211,214,214,214,76,32,185,190,190,142,95,255,250,159,16,137,132,13,224,38,32,18,137,224,119,191,251,13,214,215,215,24,80,45,224,171,
  175,78,195,231,243,33,26,141,224,139,47,254,25,217,217,217,136,68,34,8,4,136,63,222,232,232,40,86,87,215,144,147,147,141,104,52,138,149,149,101,
  168,170,98,92,143,184,105,234,64,26,48,189,4,64,201,253,68,43,186,0,37,193,101,115,114,23,80,219,105,28,214,227,241,162,178,178,14,101,101,213,
  152,157,157,192,240,240,125,172,172,172,112,116,239,195,12,56,139,62,118,50,101,186,174,155,39,84,234,21,69,239,221,231,243,98,114,114,2,247,238,245,
  96,104,104,16,63,253,233,159,161,189,189,3,183,110,221,116,228,244,164,14,19,81,71,251,212,149,120,180,188,149,181,36,96,21,156,119,218,143,132,97,
  106,66,69,69,29,166,167,71,49,50,210,103,218,189,88,140,210,195,87,29,217,101,20,236,204,11,171,225,228,245,122,49,48,208,135,169,169,9,204,207,
  207,227,221,119,223,71,117,117,13,198,199,199,31,219,226,97,157,122,201,130,43,203,18,83,33,39,152,57,1,108,174,138,157,193,35,207,94,99,18,206,
  85,115,51,163,207,132,50,2,172,15,213,118,199,117,121,121,13,74,74,42,49,55,55,129,225,225,62,44,47,7,28,26,75,110,204,173,44,75,152,154,
  154,64,111,111,15,36,73,194,189,123,119,240,131,31,252,24,175,188,242,58,206,158,61,3,191,223,143,201,201,113,156,63,255,45,19,6,34,27,50,29,
  71,246,74,169,93,187,118,97,121,57,128,204,204,44,52,52,52,98,98,98,12,119,239,222,194,137,19,175,225,238,221,219,208,117,96,215,174,221,184,124,
  249,146,153,63,69,54,156,53,124,254,249,63,99,99,35,132,214,214,118,188,251,238,7,184,119,175,199,44,157,118,74,50,88,21,138,20,40,209,74,49,
  2,124,244,29,142,59,119,91,36,90,236,17,12,174,167,28,211,238,134,196,34,35,219,192,111,226,36,225,92,226,216,148,237,20,59,168,170,106,134,191,
  178,178,178,185,130,22,22,152,144,92,154,157,205,107,18,146,235,66,109,109,19,198,198,6,48,58,58,128,100,82,65,78,78,46,146,201,36,226,241,104,
  202,126,99,67,228,118,175,65,89,150,108,215,162,155,99,159,222,183,36,233,208,52,145,73,65,112,91,63,116,132,195,97,19,28,229,230,230,34,16,88,
  218,244,112,195,2,176,68,34,137,88,108,153,19,87,21,4,32,28,142,24,135,83,96,121,121,197,53,162,16,139,45,185,218,154,144,239,138,177,143,176,
  207,73,67,60,30,224,222,71,16,4,132,195,36,217,155,228,33,205,56,174,87,81,162,136,70,103,28,246,57,228,37,90,90,125,251,101,6,74,118,189,
  32,81,20,144,159,159,135,220,220,92,215,82,92,231,73,15,230,228,35,137,138,36,68,145,149,149,243,80,215,35,73,18,106,106,26,81,89,89,139,201,
  201,17,100,103,231,114,39,239,135,213,97,177,3,65,93,215,224,247,251,145,151,151,103,46,12,178,236,49,233,103,42,227,79,117,53,232,38,252,40,116,
  170,211,44,149,8,166,85,84,84,33,153,76,184,134,131,50,51,179,31,110,160,201,50,234,235,91,81,85,85,143,241,241,65,100,100,100,58,22,207,71,
  97,31,237,97,20,150,173,161,99,70,215,53,168,170,2,93,39,201,248,84,179,230,113,131,124,194,168,249,80,86,86,137,88,44,106,38,117,178,121,36,
  110,27,39,155,95,149,157,157,99,38,106,151,150,150,35,18,9,155,204,42,251,60,114,114,242,30,122,92,147,220,188,58,76,77,141,32,35,35,219,193,
  40,185,129,104,246,223,18,137,56,110,220,184,140,55,222,120,199,144,66,80,224,245,250,144,155,155,107,108,54,9,68,34,81,142,25,179,54,41,114,191,
  181,181,13,232,239,239,69,105,105,57,154,155,91,49,53,53,129,209,209,97,236,222,189,7,157,157,93,136,197,226,8,6,215,49,48,240,192,120,206,186,
  249,217,154,166,33,59,59,7,153,153,153,152,159,159,51,18,107,225,146,92,11,120,60,68,70,33,28,182,146,171,217,77,112,103,155,138,96,50,66,60,
  32,211,81,80,80,8,77,211,16,139,69,82,158,228,61,30,15,178,179,179,141,80,182,51,236,196,38,233,123,60,30,228,229,229,113,106,245,108,153,62,
  1,67,33,40,138,226,58,143,40,80,40,44,44,66,89,89,185,41,17,96,63,72,122,60,190,135,26,71,25,25,89,232,232,216,143,218,218,102,76,79,
  143,195,239,207,64,34,17,227,170,199,236,247,159,151,151,107,232,67,89,172,22,189,22,65,128,89,174,15,0,121,121,249,40,46,46,129,174,131,185,127,
  197,12,29,199,227,113,44,45,5,92,129,158,174,235,136,70,163,198,117,102,218,88,61,125,147,195,164,83,143,203,58,32,10,41,215,92,187,7,156,253,
  247,150,12,137,219,235,172,3,172,229,138,192,143,57,254,32,198,175,57,108,8,158,48,214,188,117,74,186,189,164,161,55,86,237,246,228,201,87,113,224,
  192,193,103,122,61,146,36,163,161,161,205,156,76,143,178,193,58,13,53,201,34,80,95,223,136,140,140,12,179,170,162,184,184,4,139,139,115,198,194,174,
  153,14,225,85,85,53,80,20,5,67,67,67,59,10,85,109,117,77,36,31,36,19,31,126,248,147,39,214,143,30,143,23,45,45,157,28,72,216,44,12,
  184,115,192,100,45,42,246,124,20,93,7,106,107,235,145,157,157,131,93,187,118,99,113,113,1,139,139,11,155,158,202,119,218,135,180,210,170,168,168,4,
  31,124,240,163,148,175,165,73,185,246,106,76,183,118,234,212,135,79,112,92,75,168,175,111,117,220,195,102,0,156,5,165,52,92,144,149,149,137,120,60,
  142,230,230,54,147,41,156,159,159,197,157,59,55,193,38,218,179,76,90,105,105,41,252,126,31,102,102,166,16,10,5,241,198,27,111,195,239,247,33,22,
  139,227,202,149,139,120,247,221,31,32,22,139,225,204,153,47,92,54,55,146,51,117,228,200,81,84,84,84,226,211,79,127,111,86,162,218,193,179,162,40,
  200,201,201,195,187,239,126,244,84,214,10,69,73,34,145,72,186,28,168,72,82,124,83,83,51,222,124,243,20,83,116,96,141,93,122,253,25,25,89,0,
  128,242,242,42,188,245,214,15,28,114,9,108,114,248,231,159,255,51,38,38,198,93,159,19,21,63,108,104,104,70,67,67,243,19,187,231,236,236,92,180,
  183,119,25,140,74,212,148,43,96,199,18,205,133,123,245,213,215,177,119,111,119,74,160,194,174,3,187,119,119,99,247,238,238,148,159,187,178,178,140,255,
  254,223,255,91,74,6,143,86,167,185,169,233,59,231,188,211,138,199,174,106,255,104,235,20,237,11,94,239,139,5,76,238,162,174,169,63,55,149,177,173,
  32,72,105,123,146,63,6,160,196,182,245,245,53,204,205,205,186,24,138,166,70,221,118,250,221,239,207,120,104,54,196,62,241,68,241,241,12,66,190,68,
  155,8,158,221,191,127,207,12,195,100,101,101,27,159,165,67,81,200,230,27,137,68,16,143,199,241,249,231,159,34,24,92,127,108,6,135,150,144,162,134,
  149,149,128,35,209,155,46,44,153,153,217,220,233,236,81,251,241,201,2,109,231,105,175,170,170,26,89,89,89,152,157,157,65,79,207,29,195,11,203,190,
  9,61,58,232,76,38,147,8,6,215,56,107,17,22,116,23,22,22,65,211,52,44,47,47,57,94,67,43,103,114,114,242,32,73,18,214,214,86,12,198,
  192,185,161,250,124,254,135,102,75,237,97,26,54,137,118,235,177,66,62,159,42,21,199,98,49,120,60,50,150,151,3,24,30,30,0,0,163,52,158,110,
  2,78,51,213,170,170,90,228,231,23,224,208,161,99,166,87,85,69,69,21,198,198,70,177,180,180,136,96,112,29,241,120,28,139,139,11,156,182,19,173,
  98,91,95,95,67,111,239,61,44,47,47,99,223,190,253,88,90,90,66,60,30,115,92,47,17,252,84,176,182,182,106,50,47,206,176,200,206,24,37,118,
  253,97,223,175,176,176,200,81,160,96,223,156,125,62,159,153,203,179,53,19,235,217,146,53,244,122,125,14,118,213,222,98,177,168,169,41,101,87,20,39,
  58,90,164,154,234,81,215,16,90,114,159,170,138,142,176,92,81,172,175,175,113,160,155,142,107,93,183,100,88,4,65,68,34,17,71,34,17,231,230,48,
  235,113,23,10,133,54,13,27,178,185,91,155,173,195,214,122,108,215,102,218,28,40,109,142,153,248,253,40,21,168,33,235,161,14,55,57,12,59,227,159,
  122,173,211,185,175,199,177,142,165,219,11,0,148,216,1,123,227,198,117,220,191,127,207,156,128,155,149,20,211,146,108,143,71,132,199,227,129,207,71,190,
  106,106,26,209,217,249,240,172,212,234,106,0,163,163,253,168,168,168,69,101,101,173,81,101,166,63,244,130,194,150,169,210,73,31,8,44,97,124,124,212,
  204,119,104,111,223,109,36,154,106,144,101,25,131,131,253,184,121,243,58,98,177,24,167,99,98,119,83,127,248,13,94,52,146,115,63,55,243,57,88,171,
  3,89,22,209,216,216,142,182,182,174,135,254,140,165,165,57,140,140,244,163,169,105,23,74,74,202,161,170,110,167,171,71,111,150,160,31,95,117,116,235,
  214,77,195,44,51,145,194,177,253,209,54,10,202,54,46,46,206,227,226,197,111,141,16,31,191,153,250,253,25,248,232,163,159,33,28,222,192,183,223,158,
  54,173,16,216,5,49,43,43,27,39,78,188,137,156,156,60,156,63,255,13,98,177,136,185,9,73,146,96,72,3,136,168,168,168,197,222,189,71,31,105,
  92,143,141,13,160,184,184,2,181,181,141,204,184,118,58,141,219,245,144,116,93,71,83,83,155,41,212,39,203,94,44,47,7,48,62,62,202,73,3,16,
  70,84,117,44,252,36,63,108,12,179,179,211,144,101,25,121,121,249,168,171,107,196,200,200,48,116,93,199,198,198,134,33,124,106,55,67,37,127,191,182,
  182,134,245,245,53,244,245,61,192,255,244,63,253,63,80,81,81,129,241,241,81,88,142,241,186,185,241,6,131,235,248,254,251,175,17,137,108,152,247,194,
  90,229,236,148,145,229,213,168,173,144,215,43,175,156,66,109,109,131,169,75,101,103,43,116,93,199,208,208,0,254,246,111,255,43,23,118,99,195,96,146,
  36,163,187,251,48,74,74,202,177,176,48,131,158,158,27,140,247,161,165,236,77,191,47,45,45,50,155,48,175,102,79,65,212,216,216,48,110,223,190,110,
  0,117,107,94,19,233,20,25,251,247,159,64,73,73,197,67,143,251,169,169,81,204,204,76,160,171,139,88,151,72,146,236,48,247,165,243,242,202,149,75,
  184,123,183,135,203,77,162,97,72,146,106,81,128,195,135,95,65,86,86,54,30,60,232,193,196,196,136,195,174,133,218,160,36,147,9,132,66,27,92,184,
  157,109,62,159,31,0,76,176,101,103,143,83,177,74,78,125,53,221,230,51,200,131,33,183,70,10,85,192,29,18,248,255,230,69,110,105,14,22,91,81,
  203,230,60,242,159,167,219,198,162,192,72,67,96,199,186,96,233,246,130,1,37,90,238,202,162,251,104,52,138,120,60,206,41,182,166,10,217,216,203,94,
  19,9,47,20,197,103,110,88,59,109,161,208,58,134,135,239,99,110,110,18,27,27,97,20,21,149,217,6,238,206,117,119,232,137,152,61,181,144,235,246,
  152,154,62,186,46,152,139,141,162,144,127,87,85,21,177,88,204,20,209,227,55,250,157,151,214,187,45,24,116,131,82,85,213,38,181,64,242,189,20,37,
  249,8,64,179,15,179,179,83,136,70,163,168,171,107,230,22,166,135,5,211,180,47,173,205,145,140,135,204,204,12,100,102,102,66,146,100,36,147,9,51,
  177,56,145,136,155,32,201,233,37,135,71,102,10,173,13,65,51,42,102,146,112,42,71,91,155,70,36,18,70,34,145,176,201,78,8,102,69,19,0,132,
  195,27,102,174,9,91,245,230,245,122,30,97,92,175,97,100,164,15,51,51,19,8,135,195,200,206,206,231,22,107,150,101,177,52,159,44,189,37,89,246,
  96,247,238,61,216,181,171,19,95,125,245,185,153,84,155,149,149,13,159,47,195,12,65,210,100,84,191,63,211,100,9,226,241,4,74,75,137,183,220,87,
  95,125,137,249,249,89,35,100,165,226,232,209,19,200,204,204,70,48,184,14,89,150,161,40,138,139,17,43,73,154,214,117,82,157,212,208,64,252,9,131,
  193,32,103,156,106,103,74,195,225,13,132,195,97,70,29,91,115,104,1,109,183,177,85,138,108,66,55,13,1,209,208,153,29,128,235,58,73,8,142,70,
  167,184,202,51,170,137,101,121,248,145,240,116,36,18,193,220,220,12,103,166,75,45,105,146,73,133,187,118,73,114,110,218,148,253,13,133,130,8,133,130,
  174,130,165,126,191,135,51,130,221,201,250,49,63,63,133,225,225,7,8,4,22,145,76,42,230,1,136,2,69,55,55,129,80,136,60,7,34,98,107,85,
  222,209,235,33,235,162,106,142,253,213,213,85,35,47,137,220,55,107,169,194,38,115,219,133,136,189,94,31,252,126,191,201,110,58,109,137,54,207,53,98,
  215,68,190,212,158,7,86,110,32,154,152,246,130,3,209,178,44,161,162,162,18,197,197,197,88,92,92,192,252,252,60,83,181,104,103,182,82,73,149,184,
  203,45,176,32,41,109,116,251,71,194,40,177,3,198,26,160,154,13,61,187,235,40,89,139,169,106,128,15,205,220,244,119,178,0,108,108,4,49,54,214,
  143,169,169,49,68,163,17,35,121,208,10,33,216,43,79,118,206,150,177,246,3,100,33,11,135,195,134,146,42,89,120,67,161,16,194,225,8,20,69,65,
  48,24,52,152,36,197,97,24,187,93,54,201,205,83,206,173,124,89,20,69,67,97,154,237,115,21,138,34,238,168,31,9,88,88,195,232,104,63,166,166,
  8,139,67,205,131,89,147,207,71,97,115,120,186,156,156,206,86,86,150,209,216,216,108,154,88,78,79,79,225,226,197,243,88,94,94,54,62,95,229,18,
  121,237,198,175,238,39,205,237,50,74,108,95,139,174,101,231,84,98,130,48,162,98,138,211,170,200,177,76,180,148,152,247,106,219,249,184,142,68,54,48,
  58,218,135,137,137,17,68,163,81,179,47,44,209,61,247,211,52,0,172,172,172,160,190,190,17,5,5,5,240,120,60,72,38,21,124,241,197,39,166,161,
  242,202,202,10,186,187,15,224,253,247,63,130,40,138,8,135,55,112,250,244,231,88,92,156,71,87,215,126,252,240,135,63,129,36,73,88,90,90,196,210,
  210,34,230,230,102,17,8,44,50,202,195,179,8,6,131,200,207,47,192,202,202,10,66,161,16,18,137,4,231,25,72,91,109,109,29,246,239,63,128,120,
  60,142,140,140,12,124,247,221,183,88,88,88,48,193,11,207,104,145,49,79,43,14,89,112,68,153,153,157,137,156,82,219,26,137,121,238,164,255,168,168,
  99,126,126,190,35,249,157,29,239,4,3,107,16,4,213,84,108,103,65,24,251,236,45,3,92,154,204,172,153,126,107,244,30,221,204,134,101,89,70,126,
  126,129,1,148,66,38,168,160,201,190,86,88,82,54,159,255,246,194,180,10,2,129,5,12,13,221,199,226,226,44,18,9,197,212,26,90,91,91,69,105,
  105,5,178,179,243,54,29,135,100,253,166,107,51,91,64,160,216,152,19,193,76,222,166,70,184,118,191,56,158,221,177,250,59,47,47,15,217,217,57,80,
  148,164,89,253,183,147,105,205,50,152,219,93,19,88,77,46,187,52,197,174,93,29,232,234,218,11,85,85,240,234,171,175,225,234,213,203,184,113,227,6,
  0,213,232,15,61,229,65,118,171,207,227,25,224,173,255,54,29,149,123,41,128,146,206,32,114,152,126,102,186,174,25,191,23,82,230,0,208,50,95,82,
  29,166,66,81,20,36,147,219,207,41,10,6,215,12,125,144,126,99,35,177,152,7,222,131,74,231,104,218,135,191,55,114,115,95,126,249,133,9,46,168,
  206,208,215,95,159,49,69,203,190,252,242,115,168,170,106,184,101,235,220,169,216,206,132,60,236,38,207,2,64,214,22,197,218,212,133,109,245,163,174,235,
  8,6,87,13,15,184,62,196,227,73,110,131,98,175,87,211,156,97,55,214,60,118,167,236,163,166,105,248,234,171,175,12,91,6,201,76,148,143,197,98,
  248,253,239,127,131,88,44,1,69,209,108,213,45,143,2,214,132,77,23,89,242,124,237,201,209,186,109,211,208,92,22,63,75,30,128,205,145,16,69,209,
  52,1,117,179,72,216,140,25,157,156,28,198,232,232,128,233,50,110,189,175,192,49,164,188,151,157,5,154,206,156,249,18,217,217,89,240,120,60,102,174,
  28,27,254,190,118,237,10,238,220,185,205,176,57,196,209,252,234,213,43,184,125,251,150,41,77,65,250,92,53,194,40,10,19,150,93,196,111,126,243,143,
  208,52,146,147,119,246,236,215,208,52,64,81,52,71,56,226,214,173,155,24,26,26,70,118,118,150,97,58,155,52,54,38,209,204,209,32,146,12,58,179,
  193,107,156,240,41,13,101,185,1,177,173,154,5,198,104,137,59,249,253,194,194,156,1,148,10,145,153,153,137,80,104,195,33,223,65,43,88,137,200,169,
  96,128,37,158,173,162,215,67,251,130,173,114,163,140,178,5,20,220,109,96,124,62,31,202,203,43,205,252,44,10,148,216,241,78,0,149,178,173,123,86,
  148,36,2,129,5,140,140,244,99,106,106,220,88,155,248,245,40,16,88,68,107,107,7,74,74,74,77,192,199,178,61,246,131,145,29,36,82,6,157,53,
  159,166,0,145,222,59,111,184,205,223,63,171,194,95,88,88,132,156,156,92,204,207,207,25,6,230,78,182,133,77,129,224,215,31,60,148,108,137,91,30,
  45,253,239,145,145,17,12,12,244,65,16,4,28,61,122,12,251,247,31,68,111,111,47,34,145,8,147,167,165,239,16,204,233,219,242,69,181,175,75,233,
  246,2,3,37,222,22,4,38,219,66,99,185,169,204,98,249,147,30,255,111,201,164,8,73,82,76,197,211,84,109,109,109,25,51,51,227,24,30,238,55,
  4,212,220,44,76,68,23,51,213,173,7,167,123,200,136,53,180,213,16,141,198,28,30,101,170,26,53,7,117,34,17,113,77,52,116,230,145,108,95,52,
  207,153,232,72,22,30,86,105,151,55,13,21,204,223,111,22,98,155,156,28,193,200,200,0,194,225,8,119,114,103,23,103,107,211,208,92,64,210,195,132,
  186,116,80,149,242,120,60,102,110,226,108,75,36,20,88,162,114,246,220,174,157,2,77,33,5,109,175,153,125,201,130,94,158,105,179,131,82,103,158,132,
  149,55,195,63,15,74,177,83,145,209,173,198,245,250,250,138,105,146,27,12,134,28,108,26,205,169,98,55,102,254,20,109,1,231,120,60,129,120,60,225,
  162,226,45,152,161,179,88,44,238,178,193,146,223,167,82,191,102,251,59,153,140,154,215,145,76,42,142,241,205,206,181,245,117,146,163,100,133,226,69,211,
  150,134,62,103,59,67,109,7,72,238,204,196,118,216,82,106,105,68,63,143,20,93,4,2,1,108,108,132,144,155,155,135,242,242,74,132,66,131,142,194,
  2,203,167,17,142,241,65,67,199,236,124,166,33,39,54,63,135,2,5,242,252,0,187,73,43,0,20,20,20,33,55,55,15,115,115,51,88,89,89,49,
  239,147,0,108,107,46,18,157,57,125,19,6,73,197,210,210,28,198,199,135,48,54,54,108,50,72,148,137,99,159,235,194,194,60,20,69,65,77,77,29,
  178,179,115,176,178,178,226,144,81,177,54,107,13,138,226,28,11,172,54,152,166,105,198,189,107,230,65,209,2,183,154,121,96,112,90,216,144,170,72,65,
  16,48,49,49,150,82,252,211,173,32,104,107,243,245,135,56,78,9,2,18,137,56,114,114,178,145,151,151,143,188,188,60,140,142,142,152,2,145,118,17,
  212,237,171,196,59,65,162,219,161,54,109,128,251,18,50,74,44,16,161,207,151,250,112,217,43,77,236,3,154,167,149,173,60,130,84,27,202,250,250,10,
  166,166,198,48,54,54,132,181,181,53,211,20,215,158,196,71,180,42,36,243,4,76,67,38,44,213,189,213,164,226,205,86,5,14,44,145,159,157,126,118,
  60,219,195,123,104,217,171,36,118,178,193,219,251,145,93,40,53,13,182,141,89,131,174,75,166,150,73,42,128,52,49,49,140,137,137,81,4,131,65,36,
  18,73,155,203,56,63,153,233,243,112,171,124,219,142,174,18,219,151,84,237,151,205,239,218,204,213,222,10,203,224,33,251,48,245,34,202,254,185,170,234,
  174,161,55,22,108,178,246,56,246,215,176,32,144,176,42,116,140,91,227,50,153,84,83,48,163,171,152,154,26,197,216,216,48,86,87,87,185,156,14,123,
  206,5,9,191,88,207,131,173,254,179,179,74,244,20,158,202,200,214,173,210,144,215,126,209,183,12,3,216,199,187,29,200,90,33,118,43,41,214,10,97,
  234,41,194,37,58,195,36,89,201,208,132,21,209,118,28,122,163,58,80,44,19,43,203,64,52,26,195,208,208,0,186,187,15,162,182,182,30,195,195,67,
  96,37,42,236,161,105,55,70,151,13,177,82,160,64,13,166,173,164,110,205,44,193,183,135,220,232,231,181,183,119,24,33,205,105,108,108,108,152,107,26,
  241,182,211,160,105,34,195,96,185,31,128,230,230,166,48,62,62,132,201,201,49,196,98,9,36,147,73,51,79,136,207,27,35,159,191,184,184,136,185,185,
  25,212,212,212,161,162,162,18,171,171,171,156,32,172,61,252,198,130,37,242,59,137,11,133,58,217,52,205,212,80,98,15,214,188,72,49,9,125,54,53,
  181,64,215,117,140,141,141,154,57,157,169,194,168,228,80,40,185,244,37,255,243,206,217,24,126,111,104,105,105,69,119,247,1,248,253,25,248,199,127,252,
  165,99,61,221,76,227,105,59,204,54,91,65,104,159,147,206,124,177,52,0,122,97,129,146,189,122,129,55,196,220,154,193,17,69,162,220,106,71,215,118,
  160,20,10,173,99,104,232,1,166,167,39,176,190,78,0,82,34,97,57,99,179,3,137,158,208,36,73,103,146,83,253,46,136,94,216,246,164,98,1,24,
  205,125,162,49,106,59,232,114,178,88,91,187,170,167,234,43,123,89,48,253,157,223,159,1,81,148,144,76,70,145,72,196,25,70,195,170,168,160,84,59,
  207,196,173,96,120,184,15,211,211,19,8,6,215,13,160,169,114,14,227,236,134,76,63,55,22,35,101,220,153,153,153,174,125,183,147,197,137,158,206,169,
  57,48,45,151,181,22,16,190,106,235,97,251,144,191,78,184,170,195,211,42,155,120,60,97,110,110,206,133,89,231,64,144,27,80,226,129,43,191,49,176,
  166,182,138,237,72,30,10,173,99,100,100,0,147,147,163,230,184,166,99,155,13,125,242,97,10,17,177,88,212,28,215,110,253,96,239,59,55,28,186,157,
  83,171,91,66,173,157,77,115,31,243,110,96,69,231,128,190,155,87,159,199,227,53,14,56,170,105,78,202,3,14,21,170,170,63,20,64,166,121,73,4,
  36,89,128,97,96,160,15,221,221,7,209,212,212,138,158,158,91,8,4,2,142,67,16,27,250,117,103,27,117,142,89,163,99,201,202,77,212,184,252,43,
  75,160,87,52,195,78,45,45,237,136,70,163,24,29,29,49,231,46,187,190,106,154,228,0,102,180,205,206,78,98,100,164,31,115,115,51,136,70,163,230,
  24,98,217,45,251,65,145,20,156,68,49,50,50,132,154,154,58,236,219,119,16,67,67,131,156,166,146,85,153,103,133,31,9,32,178,14,128,44,248,39,
  64,81,225,216,63,54,129,221,238,226,96,249,100,182,32,39,39,23,19,19,99,8,4,150,152,191,97,115,214,8,232,167,146,18,25,25,25,198,90,199,
  27,105,147,168,198,195,231,245,176,160,126,98,98,28,107,107,107,104,107,107,199,137,19,175,226,171,175,190,52,149,190,31,134,165,226,15,191,188,237,144,
  223,239,135,215,235,49,152,224,120,10,176,148,70,75,47,60,163,180,211,215,144,9,79,54,73,66,105,171,230,224,161,19,107,99,35,132,7,15,238,98,
  124,124,4,27,27,65,142,210,101,79,46,52,39,201,90,124,200,137,149,38,107,230,229,229,59,156,233,105,238,20,171,153,225,220,24,116,151,228,78,107,
  34,242,62,99,206,82,80,183,24,243,118,65,146,155,87,30,205,35,40,44,44,132,199,227,65,40,20,52,171,105,232,251,170,42,12,122,94,52,23,182,
  181,181,21,244,247,247,98,98,98,196,60,173,178,253,200,86,228,216,251,81,146,68,211,160,179,176,176,152,3,81,244,26,233,98,101,15,69,108,103,44,
  108,94,149,162,63,84,31,218,193,166,93,166,130,246,163,149,60,75,52,128,220,189,190,116,219,66,165,167,0,12,214,102,105,157,222,105,200,134,38,249,
  146,215,132,195,27,120,240,224,46,198,198,134,153,113,109,85,73,185,141,107,171,196,93,71,48,24,52,194,53,133,144,101,25,137,68,210,1,124,216,126,
  99,147,202,83,149,102,239,252,89,185,63,55,103,136,219,242,235,210,117,221,81,237,197,50,166,133,133,133,0,136,66,53,13,5,243,26,57,118,208,177,
  147,77,74,99,24,64,203,18,101,113,113,1,35,35,67,104,106,106,193,174,93,157,184,112,225,123,115,253,32,108,160,104,130,101,218,167,86,88,76,55,
  198,146,200,133,11,41,139,194,135,70,5,198,162,71,224,114,130,14,28,56,12,143,199,131,241,241,81,204,204,76,219,0,150,5,216,217,220,55,194,32,
  77,163,183,247,14,22,23,231,17,141,70,77,128,101,85,156,41,92,40,152,246,5,101,238,101,89,194,208,208,32,58,59,247,162,180,180,12,109,109,187,
  112,239,94,143,153,211,69,191,19,6,216,30,54,183,66,141,244,186,88,80,107,7,136,172,21,10,107,237,147,151,151,135,253,251,15,1,0,238,223,191,
  103,168,184,219,115,163,96,238,21,116,236,231,229,229,195,231,243,65,211,162,134,221,16,121,158,108,142,217,206,25,31,42,1,64,158,229,234,234,170,41,
  210,250,131,31,124,140,194,194,34,14,40,109,247,189,237,38,230,206,208,35,140,162,11,47,66,161,16,163,105,150,206,81,122,169,128,210,163,52,171,154,
  140,130,32,32,24,12,225,218,181,203,232,237,237,65,52,26,230,54,32,118,49,176,88,16,139,77,145,36,178,128,201,178,132,197,197,69,0,196,86,34,
  39,39,27,235,235,65,102,177,162,241,127,129,97,55,182,199,138,184,151,131,238,28,44,166,58,225,179,167,62,203,56,212,90,88,43,42,42,33,8,2,
  230,231,231,192,74,23,176,229,213,162,40,98,109,109,21,87,174,92,68,111,239,29,243,36,102,169,216,242,26,47,110,253,72,217,190,165,165,69,104,154,
  134,234,234,90,19,24,145,190,163,253,72,43,137,180,148,253,184,25,232,220,73,127,109,23,104,242,128,78,228,108,71,104,149,81,117,117,13,0,152,254,
  105,219,41,239,117,207,219,129,11,192,227,65,151,170,146,178,231,91,183,174,163,167,231,38,34,145,212,227,218,218,220,216,231,33,154,64,148,84,4,42,
  168,174,174,133,215,235,69,50,153,180,29,0,44,128,98,177,15,250,54,25,57,193,245,222,83,189,124,235,133,156,13,185,185,155,249,82,16,81,91,219,
  0,0,8,4,2,102,130,188,91,94,218,195,174,51,100,243,180,18,199,53,77,67,34,145,196,237,219,55,209,216,216,140,238,238,131,24,31,31,53,220,
  227,69,14,36,91,249,84,108,232,137,128,219,203,151,47,162,191,159,248,240,217,15,46,246,131,5,171,139,36,138,2,26,27,155,209,222,222,129,120,60,
  142,158,158,91,76,133,156,85,229,105,49,155,4,152,205,204,76,227,222,189,30,140,141,13,155,54,63,244,122,88,253,34,54,167,139,173,234,99,45,52,
  130,193,32,238,223,191,135,215,94,123,19,71,143,158,192,212,212,4,214,214,214,32,73,84,80,82,55,159,31,123,79,116,92,173,172,172,224,55,191,249,
  21,60,30,143,97,244,170,49,135,4,193,1,146,100,89,52,53,152,68,81,196,225,195,199,145,157,157,141,177,177,17,140,142,14,51,225,102,246,224,166,
  155,224,117,125,125,13,177,88,20,21,21,149,200,206,206,65,44,22,99,128,45,89,147,200,53,239,108,77,183,207,223,170,170,106,196,227,81,36,18,73,
  180,182,182,33,20,10,98,125,125,221,150,10,176,57,115,229,190,166,243,154,94,116,108,212,212,212,65,16,4,172,172,4,76,137,20,183,240,116,186,253,
  145,1,37,246,180,64,7,31,201,63,0,122,123,239,218,76,3,193,37,181,82,102,137,45,183,39,167,7,145,153,200,2,86,87,151,17,8,44,162,180,
  180,12,165,165,21,8,6,67,230,34,97,63,177,80,86,96,59,172,200,78,146,237,236,12,197,86,236,7,221,24,249,9,101,233,191,228,229,229,163,178,
  178,26,0,48,54,54,194,109,180,124,8,82,199,205,155,55,204,247,96,23,56,182,146,136,173,200,179,174,69,52,127,86,85,13,147,147,19,8,135,55,
  80,84,84,140,154,154,122,76,76,140,153,27,9,61,117,218,217,149,205,250,113,187,253,231,182,200,109,246,183,155,245,163,37,16,104,25,162,214,215,55,
  34,26,141,98,110,110,46,197,243,22,30,73,168,212,158,155,214,215,247,0,3,3,253,143,60,174,3,129,69,44,46,46,160,162,162,10,85,85,53,24,
  26,26,48,55,11,178,193,89,27,44,169,214,212,177,157,202,196,157,128,195,237,62,75,54,15,131,5,170,244,57,208,239,25,25,153,104,104,104,132,170,
  42,152,156,28,119,17,15,212,241,176,154,51,118,129,71,210,55,26,52,141,8,196,78,78,78,160,167,231,22,246,237,59,128,147,39,95,199,167,159,254,
  22,27,27,27,96,29,107,248,106,59,43,60,163,105,26,198,198,198,48,62,62,102,99,63,5,6,28,242,32,137,152,219,138,200,205,205,193,137,19,175,
  154,166,197,19,19,19,54,25,12,235,51,217,28,171,111,190,249,218,236,55,250,124,236,122,83,252,220,214,57,192,198,62,59,85,85,209,211,115,27,141,
  141,77,168,169,169,195,235,175,191,133,47,190,248,4,241,120,194,246,58,205,213,178,35,153,76,98,97,97,222,101,156,240,12,62,175,99,69,190,168,71,
  96,60,30,199,141,27,215,16,139,197,185,123,160,133,52,20,44,145,10,221,32,166,167,167,208,220,220,138,218,218,58,44,47,7,152,177,175,219,246,22,
  157,19,61,77,181,22,177,230,231,244,218,155,155,155,81,91,75,192,75,60,30,199,153,51,95,154,218,95,86,66,183,176,169,117,144,115,252,243,32,153,
  178,106,30,143,140,134,134,38,0,192,232,232,136,57,222,45,38,243,225,199,126,26,40,189,52,128,9,230,194,197,158,124,69,81,115,228,62,177,39,13,
  139,218,166,192,0,32,90,39,86,28,63,28,14,163,191,191,15,39,79,150,98,239,222,110,76,76,140,153,6,144,108,8,66,85,5,166,60,121,103,90,
  28,219,216,238,183,29,34,226,23,23,235,244,197,158,62,58,58,118,35,63,191,0,51,51,211,230,6,207,87,214,89,33,56,178,32,105,80,20,190,31,
  221,66,26,22,123,1,179,100,155,230,43,132,66,65,140,143,143,98,207,158,125,232,238,62,128,153,153,73,40,10,96,119,209,160,202,228,246,164,222,71,
  239,199,173,129,85,170,126,180,22,38,210,151,116,225,222,183,111,63,188,94,47,6,7,251,177,184,184,224,200,251,120,148,202,19,62,103,199,202,107,162,
  137,185,143,99,92,79,76,140,163,162,162,10,135,14,29,193,248,248,136,161,165,101,129,87,186,185,83,54,132,60,15,225,161,40,253,135,233,138,173,198,
  181,117,16,34,207,99,239,222,110,100,101,101,99,97,97,14,83,83,19,174,201,233,143,67,96,148,50,216,162,72,158,11,77,142,191,121,243,26,42,42,
  170,80,94,94,129,55,223,124,7,95,125,245,57,162,209,24,100,153,183,5,178,155,242,218,43,153,156,137,186,22,107,192,142,197,140,140,12,188,249,230,
  187,40,42,42,198,194,194,60,46,95,190,96,50,73,110,32,199,74,15,32,99,200,10,51,129,187,46,187,230,24,59,15,249,80,46,204,247,72,36,18,
  248,254,251,179,248,241,143,127,142,134,134,38,156,60,249,58,206,157,59,11,170,147,68,67,149,110,38,209,124,226,183,14,123,37,26,31,54,182,238,191,
  177,177,25,175,190,250,6,0,224,250,245,43,152,156,156,176,221,191,110,179,112,33,207,44,30,143,99,116,116,24,141,141,205,216,183,239,0,6,7,251,
  141,92,84,209,198,32,10,174,125,224,150,2,226,204,155,2,190,255,254,59,100,101,101,194,227,241,152,225,62,242,239,86,63,82,53,111,86,154,197,201,
  36,241,17,2,2,148,45,86,77,16,4,236,218,213,137,162,162,98,108,108,132,48,50,50,228,226,66,144,214,82,74,3,37,157,230,113,144,132,65,18,
  214,177,38,136,125,211,225,197,245,192,36,205,90,177,124,2,122,200,98,210,223,255,0,93,93,251,80,91,91,143,206,206,46,244,244,220,2,32,114,155,
  60,221,224,45,181,212,167,31,27,102,5,38,217,211,23,187,184,148,151,87,162,179,115,159,185,184,88,74,200,154,67,171,137,246,5,219,143,124,206,138,
  110,2,36,171,31,97,138,203,209,77,153,2,200,155,55,175,163,173,173,3,181,181,245,104,111,223,141,7,15,238,57,22,39,186,241,236,84,248,237,113,
  246,161,125,113,162,125,71,78,241,100,177,170,169,169,67,103,231,94,36,147,73,220,187,215,227,40,71,222,78,85,228,118,198,53,5,73,118,198,238,113,
  140,235,123,247,122,208,222,222,129,178,178,10,236,219,119,0,55,111,94,3,32,114,44,200,139,50,174,75,74,202,176,119,239,126,0,192,141,27,215,144,
  72,36,185,205,130,63,89,63,250,90,195,178,125,180,228,127,109,109,13,103,207,158,193,199,31,255,9,154,154,90,160,105,239,225,236,217,211,136,68,162,
  96,5,87,237,122,79,169,24,52,183,77,152,110,142,153,153,153,120,243,205,119,208,208,208,132,96,112,29,95,125,245,133,77,47,203,201,40,177,172,146,
  149,95,233,156,215,236,189,217,175,147,10,43,82,176,104,177,106,2,22,22,22,240,205,55,167,241,254,251,63,68,87,87,55,84,85,197,197,139,231,0,
  36,77,105,4,43,111,203,157,229,117,203,129,99,195,77,244,123,115,115,43,78,157,122,7,94,175,15,247,239,223,197,141,27,87,109,76,24,187,142,233,
  102,165,31,101,0,135,135,135,208,217,217,133,202,202,106,28,58,116,212,0,117,162,141,1,19,32,8,219,27,251,238,96,137,20,177,196,227,49,83,3,
  141,36,148,3,172,180,197,102,227,210,125,236,91,76,18,97,21,73,142,150,36,73,184,113,227,170,145,107,166,187,88,94,165,219,118,155,212,214,214,246,
  255,122,153,110,200,77,27,136,61,253,240,238,208,154,99,35,217,206,160,79,38,19,80,85,5,245,245,141,168,172,172,66,32,16,192,250,250,170,35,153,
  206,158,195,194,82,196,79,231,139,101,145,68,206,79,138,76,168,92,188,251,238,7,40,40,40,68,95,223,125,220,184,113,141,211,151,177,235,31,89,236,
  145,110,59,113,110,183,31,249,190,140,199,99,16,4,1,245,245,13,168,168,168,196,220,220,28,194,225,16,215,143,124,146,247,179,232,71,103,30,8,233,
  67,11,36,21,23,151,224,221,119,63,68,102,102,38,122,123,239,226,214,173,27,142,114,100,10,52,105,72,104,239,222,253,72,36,226,120,240,224,158,201,
  72,178,205,239,247,163,173,109,23,252,254,12,220,185,115,203,92,236,172,254,230,25,130,199,49,174,21,133,140,235,166,166,22,84,84,84,98,121,57,96,
  20,47,188,88,227,58,39,199,26,215,67,67,3,184,116,233,130,99,92,111,167,127,182,191,222,56,65,48,237,175,112,56,140,213,213,21,212,215,55,160,
  172,172,28,197,197,37,88,90,90,64,44,22,229,242,73,220,18,114,237,191,179,192,1,31,110,41,41,41,197,219,111,191,135,134,134,38,132,195,97,156,
  62,253,153,153,192,205,135,204,52,7,123,101,87,115,102,199,16,173,6,116,106,107,185,39,239,219,171,86,1,146,188,28,137,132,81,87,87,143,170,170,
  26,228,231,231,99,105,105,1,241,120,220,81,92,194,178,129,206,251,118,38,109,211,62,56,112,224,16,94,125,245,13,248,253,25,24,28,236,195,217,179,
  95,113,46,0,169,238,157,29,211,170,170,34,26,141,162,177,177,25,149,149,85,8,135,55,16,8,44,34,149,152,229,214,99,223,158,55,36,186,36,221,
  195,37,9,123,123,239,203,142,125,26,33,144,36,17,62,159,15,111,189,245,46,42,43,171,48,51,51,141,239,191,63,139,68,34,185,201,122,158,6,76,
  127,148,64,201,62,129,249,201,237,164,221,237,73,109,110,167,23,59,213,79,147,95,179,179,115,80,89,89,141,186,186,6,4,131,65,172,174,46,115,155,
  138,125,147,231,19,128,159,220,23,123,194,102,23,23,246,196,93,84,84,140,119,223,253,0,229,229,149,152,159,159,195,215,95,127,201,157,64,233,134,236,
  92,40,221,195,23,169,24,31,103,63,242,191,91,90,90,64,126,126,1,42,42,42,141,252,128,101,4,131,107,207,101,63,186,45,76,21,21,149,120,231,
  157,15,80,88,88,132,233,233,41,124,245,213,231,198,34,173,113,20,55,59,118,50,51,119,14,148,98,177,152,193,142,186,129,161,199,57,174,3,200,204,
  204,68,101,101,53,106,107,235,177,177,17,194,202,74,224,133,25,215,133,133,69,120,251,237,247,81,89,89,141,197,197,5,156,62,253,153,67,97,223,2,
  74,143,119,173,177,242,104,120,176,176,182,182,134,64,96,17,213,213,181,40,43,43,71,93,93,3,146,201,36,2,129,37,110,131,228,55,84,231,151,221,
  87,78,150,37,116,118,118,225,212,169,119,81,82,82,134,245,245,53,124,249,229,103,152,156,156,112,169,22,115,130,67,62,121,56,213,24,218,60,89,223,
  206,148,178,73,255,244,223,150,150,150,176,182,182,138,154,154,90,148,151,87,162,190,190,17,137,68,194,28,87,244,158,72,31,136,219,236,3,17,101,101,
  229,120,253,245,183,176,111,223,1,200,178,7,61,61,183,240,237,183,223,24,126,152,154,235,51,103,15,45,118,1,227,245,245,53,72,146,136,218,218,122,
  212,214,214,33,145,72,96,113,113,30,108,117,161,27,104,181,143,125,107,156,10,166,73,187,19,44,89,160,200,109,12,164,122,79,250,190,244,103,154,155,
  38,138,34,178,179,179,113,234,212,187,104,106,106,65,48,184,142,47,191,252,103,172,175,175,155,18,24,86,149,103,58,145,59,13,148,54,201,7,177,202,
  113,157,244,249,214,239,33,216,232,96,13,211,211,19,200,203,203,71,69,69,37,26,27,155,225,241,120,177,178,18,48,252,218,40,53,238,126,34,124,26,
  44,136,181,185,91,236,135,215,235,69,123,123,7,222,121,231,125,20,21,21,35,16,88,194,153,51,159,27,85,79,26,55,153,220,61,208,30,174,31,121,
  173,41,214,56,82,199,204,204,52,10,11,11,81,94,94,137,230,230,86,115,195,214,52,245,185,234,71,246,187,207,231,199,158,61,123,241,230,155,111,35,
  47,47,31,115,115,179,248,252,243,79,176,177,17,102,76,139,117,51,52,195,202,8,60,60,80,162,207,195,153,123,244,184,198,53,0,76,79,79,26,135,
  128,42,52,54,182,192,231,243,34,16,8,64,85,159,231,113,237,65,123,123,7,222,122,235,61,148,148,148,98,121,57,128,211,167,63,115,140,107,55,15,
  187,199,25,158,77,5,22,214,215,215,49,57,57,142,162,162,98,148,150,150,155,172,93,44,22,51,12,143,181,109,2,5,1,62,159,15,117,117,13,56,
  117,234,29,236,219,119,0,62,159,15,147,147,227,56,125,250,51,204,205,205,56,252,208,44,253,172,212,33,151,84,99,104,59,161,85,154,231,228,150,79,
  69,219,202,202,10,166,167,167,80,82,82,138,146,146,82,52,55,183,162,170,170,26,138,146,68,36,18,1,177,156,225,153,91,183,47,175,215,139,226,226,
  18,28,57,114,28,175,189,246,38,74,74,202,16,143,199,112,238,220,119,184,122,245,146,121,72,217,140,77,98,159,141,125,236,207,205,205,194,227,145,81,
  93,93,139,250,250,70,228,228,228,34,16,88,132,162,36,153,28,69,59,3,150,122,156,210,220,57,150,1,228,1,150,179,122,118,179,247,180,14,106,22,
  131,234,245,122,209,208,208,136,183,223,126,31,213,213,181,8,133,66,248,234,171,207,13,86,81,53,139,154,220,214,243,116,219,230,28,255,232,163,143,210,
  61,182,197,2,200,211,190,162,225,188,77,190,251,124,94,156,56,241,26,186,186,246,25,73,122,235,184,127,255,46,198,199,199,16,14,111,24,34,119,124,
  229,196,147,28,163,110,244,191,199,227,129,223,239,67,69,69,37,118,239,238,50,43,220,38,39,39,240,237,183,95,97,105,105,137,83,43,182,111,44,143,
  98,20,203,247,35,239,184,206,38,150,103,101,101,226,181,215,78,161,189,125,23,36,73,198,202,202,50,30,60,184,135,137,137,9,68,34,97,174,188,213,
  206,22,62,141,126,20,69,210,143,153,153,89,168,172,172,194,238,221,123,80,90,90,14,93,215,49,50,50,132,179,103,191,66,48,24,228,20,132,237,229,
  211,244,4,88,84,84,132,127,245,175,254,71,132,66,65,252,234,87,255,221,20,49,101,91,94,94,62,62,250,232,199,200,207,47,192,95,255,245,127,193,
  234,234,170,81,162,173,153,159,193,122,214,61,238,113,237,245,122,112,242,36,29,215,94,99,92,223,195,228,228,56,54,54,130,136,199,159,135,113,45,195,
  239,207,64,69,69,5,58,58,246,160,170,138,72,51,76,77,77,224,155,111,206,24,224,110,243,113,253,56,193,146,93,244,145,5,215,52,52,232,243,249,
  112,224,192,97,236,217,179,15,217,217,217,0,128,197,197,5,12,15,15,98,126,126,6,27,27,33,67,14,64,53,188,45,69,136,162,4,143,71,70,118,
  118,46,42,43,171,208,212,212,138,146,146,82,0,68,30,162,167,231,22,110,221,186,206,133,155,88,79,59,187,221,209,227,206,43,179,75,52,216,89,16,
  182,58,52,35,35,3,221,221,7,185,251,95,95,95,195,232,232,48,22,23,231,16,12,174,35,145,72,64,215,85,3,204,144,191,203,206,206,69,81,81,
  9,234,234,26,204,231,156,72,196,49,53,53,137,75,151,206,99,113,113,145,243,192,227,251,192,249,172,221,146,225,173,235,148,112,248,240,81,28,58,116,
  20,126,191,31,209,104,4,15,30,220,195,232,232,136,121,125,180,152,194,109,236,187,9,16,187,185,33,88,121,96,0,111,185,163,187,190,31,123,208,148,
  101,178,166,151,148,148,97,215,174,221,102,133,219,226,226,2,206,158,253,10,51,51,83,156,84,203,147,28,251,105,160,148,110,142,88,57,191,248,89,149,
  79,237,237,187,176,111,223,126,84,86,86,51,241,249,21,67,173,58,241,76,168,78,98,187,34,35,43,43,27,69,69,197,166,68,63,1,33,189,184,125,
  251,134,161,32,173,154,154,59,143,27,36,185,45,164,60,117,204,178,3,34,58,59,247,96,239,222,253,40,47,175,48,255,126,121,57,96,108,32,137,103,
  70,25,203,178,140,156,156,92,20,21,21,155,247,19,8,44,162,183,247,30,110,223,190,193,245,29,253,217,9,70,164,71,2,74,188,38,146,106,138,16,
  62,201,113,221,218,218,134,238,238,3,168,172,172,54,69,9,215,214,86,13,213,239,103,55,174,69,81,66,118,118,54,138,138,74,184,113,61,48,208,135,
  155,55,73,89,184,115,92,243,57,26,143,19,48,164,2,11,246,36,115,250,189,184,184,216,44,8,41,42,42,54,223,39,26,141,34,20,90,71,52,26,
  129,162,40,198,33,39,19,185,185,121,166,27,0,25,123,75,152,152,24,195,189,123,61,88,94,94,230,42,219,220,74,249,159,196,61,239,4,44,217,191,
  23,21,21,161,179,115,15,106,107,27,184,181,9,0,98,177,168,97,98,171,193,235,245,33,51,51,147,179,58,10,6,215,49,55,55,139,190,190,251,24,
  29,29,118,72,24,176,76,18,45,46,113,171,64,181,95,39,27,198,149,36,9,117,117,245,56,112,224,16,106,106,106,33,203,30,243,179,215,215,215,16,
  143,199,182,144,184,176,179,107,78,128,228,116,159,160,33,208,205,223,87,16,68,100,101,101,161,176,176,24,94,175,215,4,155,195,195,131,184,113,227,170,
  233,85,186,21,171,152,6,74,59,88,255,211,93,176,117,99,145,191,37,57,96,196,46,141,249,219,215,247,0,163,163,35,104,105,105,69,125,125,3,138,
  139,75,145,159,95,128,130,130,194,231,226,250,67,161,32,150,150,22,49,51,51,133,145,145,97,83,200,205,185,192,176,229,175,143,103,66,217,85,202,137,
  189,17,233,71,98,93,96,9,209,245,246,222,197,232,232,48,154,154,90,81,87,87,103,246,35,187,153,60,171,166,105,26,214,214,214,176,188,188,132,169,
  169,73,12,15,15,34,24,92,79,217,143,76,15,24,250,52,207,215,41,110,59,227,122,96,160,31,227,227,163,104,106,106,65,67,67,19,74,74,74,144,
  151,87,96,42,144,63,235,22,12,174,35,16,88,194,244,244,20,70,71,135,177,188,28,216,100,92,107,143,117,92,111,54,198,41,126,165,38,201,84,147,
  135,202,148,44,45,45,225,236,217,175,81,88,88,136,154,154,26,84,84,84,161,176,176,8,121,121,249,40,41,41,227,194,163,154,166,25,218,92,51,88,
  93,93,198,236,236,12,38,39,39,176,186,186,230,168,78,101,1,225,163,153,102,63,158,251,39,222,139,252,253,75,146,142,229,229,0,206,159,255,30,185,
  185,183,80,93,93,131,178,178,10,20,22,22,33,43,43,27,126,127,6,242,242,242,77,107,157,80,40,136,72,36,140,245,245,53,44,46,46,96,118,118,
  198,48,223,213,184,48,183,93,195,205,14,146,88,128,98,191,78,34,119,161,114,253,52,62,62,134,201,201,9,52,55,183,160,177,177,9,37,37,101,40,
  40,40,68,110,110,222,115,49,246,55,54,54,48,55,55,131,153,153,105,140,142,14,99,97,97,129,51,189,78,131,164,52,80,122,170,155,9,107,190,202,
  110,242,108,2,36,49,153,212,208,219,123,15,247,239,247,162,160,32,31,217,217,57,240,122,125,144,101,249,153,13,76,93,39,174,239,209,104,4,107,107,
  107,166,223,149,93,148,144,151,2,120,252,229,163,219,237,71,77,211,17,10,109,224,238,221,59,232,237,189,139,130,130,124,100,101,229,192,231,243,186,26,
  232,62,45,86,81,85,85,36,18,113,132,195,97,172,175,175,35,153,76,186,110,80,236,198,197,86,68,61,111,235,210,102,224,213,109,92,63,120,112,31,
  125,125,15,80,80,80,96,108,102,62,72,146,236,8,13,60,249,235,182,192,67,34,17,71,52,26,53,198,117,156,211,202,113,19,63,125,210,101,209,78,
  176,192,170,56,11,102,41,60,177,3,34,223,87,86,150,177,190,190,134,190,190,251,200,201,201,65,118,118,54,252,126,63,58,58,246,160,177,177,21,131,
  131,15,48,48,240,0,241,120,220,180,162,160,76,25,91,120,97,103,144,88,177,216,39,13,146,54,187,127,114,223,48,191,91,134,206,58,100,153,108,246,
  35,35,67,152,152,24,69,70,70,6,178,179,179,145,149,149,137,125,251,142,160,164,164,28,215,175,95,192,218,218,10,194,225,48,54,54,194,80,20,197,
  240,154,211,28,97,70,187,222,83,42,128,232,14,234,120,223,57,42,122,43,138,34,6,7,7,48,52,52,136,220,220,92,228,228,228,194,231,243,193,227,
  241,108,49,246,89,107,35,152,226,164,69,69,165,56,112,224,8,150,151,151,208,211,115,195,208,42,19,92,20,179,55,203,37,211,205,177,31,12,174,51,
  229,255,252,122,110,183,125,73,99,163,52,80,122,42,11,0,85,38,166,250,76,214,34,160,51,22,38,68,67,37,16,88,198,242,242,10,71,177,62,75,
  214,192,77,91,135,95,88,52,155,102,10,158,216,169,155,232,247,164,234,71,221,160,195,53,163,42,46,128,64,96,249,185,237,71,251,38,204,158,100,45,
  205,22,253,185,30,215,44,88,178,63,15,106,115,66,95,187,186,186,194,56,194,11,207,252,250,221,100,42,220,54,206,39,57,174,83,245,41,213,201,33,
  97,88,193,236,87,50,254,5,71,158,72,40,20,66,36,18,54,170,41,73,30,225,242,114,0,51,51,211,142,144,26,107,39,98,191,127,55,205,163,167,
  117,80,227,199,20,1,7,214,253,235,156,34,60,77,90,182,214,5,21,177,88,24,249,249,185,40,40,40,130,32,16,233,131,217,217,9,36,147,138,169,
  26,110,89,38,105,140,61,139,238,200,197,218,12,32,110,135,1,35,161,66,235,121,173,175,175,35,24,12,110,57,246,157,33,109,43,252,30,139,69,113,
  224,192,17,68,34,97,35,108,168,57,180,209,182,3,232,157,194,178,236,216,215,29,7,132,167,9,152,211,64,41,13,150,184,137,101,13,64,94,180,205,
  110,60,203,158,50,158,242,85,115,167,16,235,148,231,220,64,158,246,226,106,249,77,89,204,215,246,250,241,89,108,206,91,247,163,93,91,234,105,51,45,
  143,115,92,211,13,67,16,68,198,176,84,52,21,132,73,170,146,224,186,65,60,105,160,234,220,40,240,220,128,36,123,159,178,227,156,40,81,91,222,106,
  172,242,178,229,77,103,89,89,80,37,116,73,146,56,198,136,77,206,165,126,138,252,124,222,94,89,255,211,26,83,180,26,206,186,127,235,117,52,231,141,
  5,21,126,191,31,187,118,237,133,199,67,114,111,90,90,58,49,51,51,129,197,197,5,19,112,81,133,127,118,237,178,194,221,219,127,214,155,141,125,34,
  54,169,153,172,16,43,160,171,235,78,155,145,205,199,190,149,191,69,115,138,72,5,159,199,96,248,121,11,19,55,15,203,205,65,146,158,6,73,105,160,
  244,252,110,42,228,20,174,25,78,216,214,198,110,153,136,242,165,167,207,34,68,193,78,14,118,33,73,165,220,252,52,78,220,180,79,248,235,18,158,235,
  126,228,25,37,108,171,31,45,191,191,23,105,124,195,4,255,172,55,220,179,110,169,124,181,220,198,44,47,162,232,62,23,158,22,3,105,1,6,221,176,
  54,129,41,123,193,178,43,214,120,177,171,172,195,225,219,103,175,100,178,116,113,158,13,139,180,221,121,78,230,186,102,10,175,178,128,142,22,118,16,203,
  151,26,243,245,164,80,166,11,43,43,103,25,86,137,101,150,52,135,166,216,195,220,63,63,246,233,248,17,108,63,235,59,58,168,185,169,250,179,222,111,
  214,119,125,211,53,109,43,159,80,126,141,113,218,148,164,65,82,26,40,61,83,176,196,218,122,208,141,158,159,20,216,212,228,240,105,109,244,246,77,158,
  159,72,207,230,244,249,114,245,163,187,194,243,139,182,46,185,1,162,237,246,249,179,2,83,91,111,34,207,199,122,193,30,84,168,238,16,77,244,182,171,
  51,107,154,192,141,53,183,228,92,123,216,205,141,117,120,254,14,151,2,51,151,192,177,73,84,15,168,173,173,203,241,30,229,229,36,217,125,114,114,130,
  91,7,236,236,238,227,3,4,194,35,207,155,135,159,127,105,48,147,6,74,47,17,88,226,79,139,252,70,79,79,8,207,91,200,200,94,130,250,44,233,
  249,63,134,126,20,94,128,216,155,221,108,147,122,72,57,237,53,220,78,197,207,19,40,18,92,108,33,104,53,19,64,115,100,158,5,176,178,143,9,107,
  83,167,162,171,4,28,17,35,109,242,157,50,67,206,228,92,205,97,97,243,188,129,163,205,192,146,157,101,97,133,25,235,235,155,81,80,224,172,110,37,
  50,21,123,48,55,55,195,41,81,187,139,10,235,15,53,246,41,83,101,87,131,167,236,23,157,11,59,57,60,208,103,156,234,64,74,221,34,182,115,237,
  118,207,59,202,128,241,172,148,96,211,243,98,83,27,244,231,254,80,145,6,74,127,68,128,137,29,248,207,34,84,228,198,132,108,54,17,159,135,137,179,
  89,63,178,167,172,116,63,62,57,144,228,180,37,113,122,85,177,194,121,60,192,18,158,209,181,235,160,22,24,108,94,137,101,102,74,126,230,61,247,158,
  237,134,193,142,117,106,36,75,115,76,4,65,51,140,87,121,70,201,77,23,137,205,139,123,145,198,155,5,60,68,70,228,84,66,102,102,38,90,90,58,
  83,254,109,113,113,57,106,107,27,48,48,208,207,249,156,81,112,105,29,180,118,254,108,41,72,178,244,221,248,177,239,230,189,183,217,216,119,254,74,55,
  129,122,60,30,3,0,67,7,77,113,248,232,109,247,160,64,115,165,82,141,125,65,208,13,67,241,52,88,74,3,165,231,12,48,165,62,73,61,191,215,
  248,188,247,35,15,162,210,253,248,36,154,155,119,155,155,229,130,207,231,67,89,89,57,74,74,74,144,147,147,107,38,227,62,171,231,16,141,70,177,178,
  178,140,249,249,57,4,131,65,136,162,110,186,193,179,9,196,244,245,207,211,134,97,95,27,236,85,149,246,141,214,25,102,122,241,0,185,29,36,201,178,
  4,143,135,168,142,183,182,118,32,39,39,111,147,49,42,162,177,177,29,51,51,147,72,36,20,200,178,202,248,159,233,80,85,154,99,164,239,232,154,156,
  32,201,26,247,172,183,158,199,227,49,198,126,25,114,115,115,32,73,30,108,22,38,115,207,79,130,169,70,94,80,80,132,35,71,94,133,166,169,204,65,
  108,251,38,205,177,88,20,171,171,171,88,88,152,195,234,234,170,1,156,5,3,76,11,166,224,45,61,36,164,193,82,26,40,189,144,32,42,221,210,253,
  248,60,108,94,118,144,196,250,106,81,117,238,226,226,98,236,217,179,23,77,77,45,200,202,202,134,36,73,207,77,88,145,234,91,205,204,76,227,222,189,
  30,140,143,143,65,81,20,115,115,98,55,140,167,97,181,242,176,99,123,171,107,218,238,6,250,124,179,73,118,144,36,195,235,245,160,176,176,0,181,181,
  205,91,190,71,113,113,57,234,234,154,16,14,247,66,81,20,40,138,108,50,109,146,36,24,110,2,91,179,74,124,113,136,187,83,0,29,251,121,121,121,
  216,179,167,11,45,45,237,200,205,205,133,36,201,143,101,236,231,228,228,162,179,115,239,35,189,135,166,105,72,38,19,152,159,159,195,253,251,119,49,52,
  52,8,69,81,160,170,154,45,154,225,4,75,233,150,6,74,233,150,110,233,182,173,19,62,155,151,97,109,96,212,247,202,239,247,99,255,254,3,56,116,
  232,24,60,30,15,84,85,65,56,28,198,242,114,192,176,48,73,62,67,22,76,52,173,121,114,115,243,208,220,220,138,230,230,86,12,13,245,227,242,229,
  139,88,90,90,116,108,104,180,130,140,202,28,164,79,214,79,119,172,177,96,132,128,36,25,30,143,140,166,166,246,77,217,36,246,189,26,26,90,49,53,
  53,129,68,34,137,100,146,103,149,168,20,195,118,158,105,106,59,37,18,10,244,249,124,232,236,220,131,35,71,78,32,51,51,19,154,166,33,18,177,198,
  126,34,145,176,67,89,251,39,112,140,146,5,90,248,156,64,150,73,218,14,104,166,239,65,44,76,136,146,123,93,93,3,234,234,26,48,57,57,142,43,
  87,46,96,122,122,218,252,76,85,165,159,199,31,20,210,99,63,13,148,210,45,221,94,210,77,135,79,108,103,23,225,135,89,244,220,194,109,20,36,21,
  22,22,226,245,215,79,161,177,177,25,170,170,96,116,116,24,3,3,15,48,57,57,142,104,52,230,34,151,240,244,250,128,221,128,137,56,99,21,90,91,
  219,209,212,212,130,150,150,118,148,151,87,225,155,111,78,99,100,100,200,193,220,0,130,17,146,75,159,172,159,46,155,4,51,156,37,203,178,201,40,21,
  23,23,163,166,166,113,219,239,85,88,88,138,186,186,6,132,66,33,36,147,42,39,147,176,157,92,37,183,48,32,11,146,60,30,9,217,217,57,120,237,
  181,55,208,222,190,27,0,49,89,30,26,234,55,141,206,249,113,159,58,92,102,47,46,32,0,95,112,1,240,78,89,7,251,219,217,199,61,125,175,146,
  146,82,52,55,183,161,169,169,21,181,181,245,40,47,175,192,119,223,125,131,251,247,239,57,222,143,30,20,232,216,79,131,165,52,80,74,183,116,123,217,
  183,31,208,106,23,251,2,234,4,7,110,140,12,95,209,67,195,109,178,44,161,160,160,0,31,124,240,17,202,203,43,176,182,182,138,107,215,46,161,175,
  239,62,87,134,77,23,221,237,126,222,227,96,37,220,64,158,166,233,152,158,158,194,236,236,52,250,251,31,224,240,225,99,104,104,104,194,123,239,253,0,
  103,206,124,142,161,161,1,71,217,184,174,91,218,67,233,205,226,233,128,91,231,56,35,172,82,99,99,43,178,178,114,118,244,126,45,45,157,24,31,31,
  67,60,158,224,194,196,196,175,112,235,18,123,187,33,180,5,224,136,209,242,123,239,125,136,186,186,6,68,34,97,220,188,121,13,247,239,247,152,30,106,
  110,114,32,169,15,11,186,57,215,232,231,82,157,184,84,243,134,45,100,113,190,167,85,237,70,243,219,2,129,37,4,2,75,24,28,236,67,119,247,33,
  180,181,117,224,212,169,119,33,203,50,238,220,185,101,147,48,17,192,86,225,165,219,14,128,82,186,195,210,45,221,158,76,227,23,41,125,139,50,97,183,
  191,181,191,143,117,130,165,98,150,60,136,224,23,88,55,217,2,171,178,80,0,205,197,102,153,37,159,207,135,183,223,126,31,229,229,21,8,4,150,112,
  250,244,63,99,105,105,49,165,143,214,179,88,63,236,39,116,154,112,59,59,59,131,211,167,255,25,39,78,188,134,61,123,246,225,173,183,222,67,56,188,
  129,233,233,105,51,212,70,171,246,182,234,255,103,63,70,82,253,187,243,235,121,102,146,172,13,218,206,46,137,40,44,44,66,125,125,235,142,223,55,59,
  59,23,141,141,45,134,149,142,93,185,63,117,82,60,43,81,192,142,3,58,134,60,30,25,175,189,118,202,96,172,130,248,230,155,47,49,59,59,205,149,
  242,219,213,247,45,3,241,212,135,5,42,156,75,195,220,169,192,146,147,1,210,109,253,198,75,93,208,247,18,4,1,43,43,203,248,238,187,175,17,12,
  174,227,208,161,99,120,229,149,55,176,177,17,194,224,224,160,105,173,101,157,107,244,231,98,236,167,25,165,116,75,183,116,123,34,172,10,5,53,86,105,
  176,93,45,88,119,168,9,219,45,17,216,144,0,91,14,45,138,2,142,28,57,142,218,218,58,172,175,175,225,139,47,62,65,32,16,112,216,131,88,161,
  2,253,169,159,78,237,213,68,100,211,177,60,233,34,145,40,206,157,59,11,175,215,135,182,182,93,120,243,205,119,240,79,255,244,75,68,163,49,227,218,
  69,155,215,90,122,108,61,121,80,11,142,189,33,97,94,25,173,173,187,225,245,250,30,234,125,219,218,246,96,96,160,31,209,104,220,165,124,63,245,115,
  117,94,143,104,126,117,117,237,67,123,123,7,98,177,24,190,254,250,11,204,207,207,58,172,68,156,194,150,86,216,140,87,32,215,29,236,39,181,108,162,
  37,251,169,192,18,43,125,193,31,118,192,133,205,156,32,80,197,205,155,87,225,241,120,177,111,223,1,188,241,198,219,88,88,88,192,218,218,26,52,77,
  52,12,137,121,32,150,102,84,183,215,196,116,23,164,91,186,189,152,96,201,210,161,33,97,3,251,23,31,150,16,56,54,198,174,151,36,138,2,42,43,
  171,176,123,55,209,178,57,127,254,91,7,72,34,246,25,170,225,222,174,66,81,120,115,214,167,241,165,40,170,241,101,253,76,109,61,232,181,198,227,9,
  156,63,255,45,150,151,3,40,43,43,71,119,247,65,78,11,202,178,193,224,173,113,210,237,137,143,96,243,167,226,226,18,52,54,182,61,244,59,249,253,
  25,216,189,123,47,7,154,233,207,155,207,33,190,2,143,206,163,194,194,66,116,117,117,3,0,174,93,187,136,133,133,89,147,117,242,251,51,80,92,92,
  202,49,153,178,44,163,168,168,24,30,143,135,99,129,50,50,50,145,159,95,96,128,37,205,97,113,4,0,249,249,5,102,2,118,118,118,54,23,10,164,
  5,10,164,178,212,186,110,73,146,80,93,93,141,182,182,93,40,41,41,51,67,111,146,36,163,170,170,6,205,205,109,40,44,44,2,160,227,198,141,203,
  152,158,158,68,78,78,46,14,30,60,204,205,113,94,140,53,61,238,211,64,41,221,210,237,165,3,74,224,192,13,5,72,180,130,136,124,121,204,106,34,
  143,135,130,38,30,48,217,65,3,253,190,107,87,39,178,178,178,49,56,216,135,225,225,65,115,145,103,61,198,88,149,104,119,165,104,45,165,221,134,5,
  114,220,191,182,243,122,235,223,9,88,98,95,199,50,94,161,80,8,183,111,95,7,0,236,222,189,7,25,25,25,14,181,113,183,196,218,116,123,114,0,
  159,109,123,247,30,124,100,29,174,230,230,54,20,23,151,48,236,17,15,128,83,129,53,126,220,147,249,212,210,210,138,162,162,98,204,204,76,97,112,176,
  143,123,253,222,189,251,241,243,159,255,5,42,42,170,205,251,201,203,203,199,71,31,253,4,133,133,188,146,120,123,123,7,62,248,224,135,240,249,124,38,
  128,98,89,216,204,204,76,252,248,199,63,195,254,253,7,209,222,222,129,143,63,254,41,58,58,58,205,107,57,122,244,56,254,252,207,255,13,10,10,10,
  205,235,204,204,204,192,219,111,191,135,19,39,94,67,99,99,11,14,30,60,2,143,199,131,204,204,12,188,245,214,123,56,116,136,228,229,237,219,119,8,
  30,143,15,138,162,224,238,221,91,72,38,147,216,189,123,15,242,243,11,82,130,165,116,219,94,75,135,222,210,45,221,94,16,144,196,230,119,208,61,38,
  35,35,3,146,36,33,145,136,113,202,230,172,229,5,117,153,167,201,215,22,35,101,201,2,228,229,229,163,166,166,22,186,174,163,167,231,150,205,145,156,
  183,204,176,231,101,164,22,92,117,252,102,83,166,193,253,125,82,121,152,209,156,13,1,172,211,187,197,154,233,24,24,232,195,254,253,135,145,151,151,143,
  198,198,102,220,187,119,151,73,2,183,242,84,210,145,135,167,213,8,240,110,108,108,65,125,125,227,35,191,155,207,231,199,161,67,199,48,59,251,59,232,
  122,194,24,43,250,54,230,17,25,63,20,52,100,100,100,162,161,161,9,0,112,255,254,93,40,74,146,249,12,31,10,11,139,177,188,28,64,115,115,43,
  102,102,166,32,8,58,3,56,88,32,168,59,148,187,217,113,75,228,8,36,0,192,149,43,23,177,176,48,135,246,246,14,28,63,254,42,134,135,7,144,
  147,147,139,236,236,28,108,108,132,80,85,85,131,149,149,101,0,64,91,91,7,74,74,202,240,171,95,253,18,137,68,194,156,231,93,93,251,145,155,155,
  143,223,253,238,31,13,163,101,209,188,255,201,201,49,44,44,204,161,186,186,22,173,173,109,184,122,245,50,199,38,107,26,95,12,146,110,105,160,148,110,
  233,246,210,52,214,61,93,16,4,180,181,181,97,215,174,221,120,240,224,46,38,39,39,160,105,170,121,138,37,108,139,6,73,178,194,83,86,50,43,31,
  126,43,41,41,69,97,97,17,2,129,37,44,45,45,153,239,225,150,196,205,3,149,205,42,238,236,213,60,246,133,217,61,92,194,91,120,8,41,128,19,
  201,193,178,140,102,249,4,103,77,211,145,72,36,49,50,50,132,67,135,142,162,186,186,6,247,239,223,179,157,166,211,155,197,83,129,71,58,239,91,39,
  73,50,122,122,110,66,16,4,51,68,76,25,78,93,215,32,203,30,148,151,87,35,35,35,11,83,83,163,136,197,34,16,4,209,120,15,205,96,18,137,
  162,245,198,70,24,162,40,153,85,140,169,0,185,221,120,152,230,7,137,34,97,135,202,203,43,176,177,17,194,226,226,60,7,156,171,171,107,33,8,192,
  213,171,23,112,228,200,9,200,178,140,100,50,201,20,19,136,70,190,27,201,59,162,249,69,244,115,216,249,102,85,171,89,243,102,110,110,6,146,36,27,
  202,223,21,136,197,98,184,113,227,42,154,154,154,49,48,240,0,137,68,2,138,146,132,207,231,71,105,105,57,102,102,166,204,107,75,38,19,240,251,253,
  40,42,42,198,226,226,60,119,16,2,128,241,241,17,84,87,215,162,182,182,30,215,174,93,225,198,125,154,77,74,3,165,116,75,183,151,130,65,178,179,
  46,86,164,194,10,29,249,124,94,212,215,55,162,166,166,14,211,211,19,24,29,29,192,248,248,168,9,148,146,73,5,138,98,229,21,209,36,83,123,114,
  109,81,81,17,68,81,52,196,252,18,14,1,60,90,74,207,95,155,176,165,44,129,189,90,143,53,167,181,222,139,95,184,121,160,4,27,123,229,220,0,
  237,192,140,173,80,154,158,158,196,161,67,71,81,80,80,8,73,162,74,206,236,201,26,155,130,189,167,201,20,190,188,64,9,166,175,157,170,106,184,115,
  231,54,238,223,191,7,159,143,134,138,61,70,152,216,210,87,202,202,202,65,70,70,22,250,251,123,16,12,174,113,99,57,145,80,144,72,36,17,143,39,
  141,177,173,112,57,106,91,177,73,118,38,40,63,63,31,178,236,193,220,220,8,34,145,48,199,108,86,84,84,99,118,118,6,227,227,163,216,183,239,32,
  234,234,26,48,58,58,228,200,249,163,160,139,204,81,167,47,156,245,51,25,251,36,143,73,195,225,195,39,176,184,56,15,0,168,172,172,198,226,226,60,
  30,60,184,139,206,206,46,20,22,22,97,97,97,30,163,163,67,40,45,45,199,219,111,191,143,241,241,81,220,185,115,3,235,235,107,24,28,236,67,113,
  113,49,222,121,231,67,140,143,143,224,206,157,155,216,216,8,153,215,62,55,55,11,77,211,144,151,151,15,191,223,15,69,9,219,180,157,210,235,108,26,
  40,165,91,186,189,224,32,137,47,95,182,152,36,214,135,74,146,8,122,146,36,9,117,117,141,168,172,172,70,123,123,39,134,134,30,96,106,106,28,201,
  164,98,120,99,41,102,8,142,221,8,232,130,78,61,168,66,161,160,67,119,136,45,165,166,106,222,91,45,182,116,115,180,87,198,81,209,71,22,156,88,
  213,63,78,246,200,237,111,89,176,196,111,124,206,235,8,133,130,0,0,191,223,143,172,172,44,40,138,98,148,75,235,70,137,181,232,0,99,79,239,57,
  219,157,234,5,46,164,227,198,136,188,136,108,18,173,212,34,224,221,46,207,64,153,38,201,240,111,147,32,8,146,57,22,52,13,136,197,18,102,30,90,
  50,73,128,17,253,162,191,183,18,167,183,14,165,90,224,156,124,229,228,228,24,236,84,136,83,155,207,200,200,64,105,105,25,188,94,47,118,239,238,130,
  207,231,67,99,99,51,70,71,135,57,54,212,2,220,252,216,118,171,40,35,70,181,34,58,59,187,16,14,111,32,24,92,67,79,207,45,120,60,94,131,
  189,34,185,130,25,25,153,168,173,173,199,194,194,60,18,137,4,46,92,248,14,19,19,99,216,179,103,31,222,123,239,135,248,242,203,79,177,182,182,134,
  115,231,206,162,182,182,1,93,93,221,120,231,157,15,113,230,204,103,38,88,10,6,215,145,72,36,224,241,120,144,157,157,131,112,56,226,0,111,233,150,
  6,74,233,150,110,47,48,72,226,203,249,41,32,162,155,41,173,108,243,120,36,238,239,61,30,47,42,42,106,80,82,82,142,64,96,1,3,3,247,48,
  61,61,105,108,44,86,248,141,21,139,20,4,193,44,211,142,199,227,14,118,135,149,36,32,255,38,114,26,76,169,114,50,216,106,31,139,233,17,108,44,
  89,234,42,156,84,127,75,247,30,246,229,86,114,186,200,128,57,193,172,212,163,58,57,20,100,178,141,10,87,62,237,16,28,125,174,180,122,209,178,227,
  16,29,192,244,229,96,148,200,119,69,81,109,97,82,205,0,74,50,100,89,133,44,123,205,241,153,76,42,136,70,19,70,242,190,5,150,200,207,10,83,
  84,192,230,206,109,246,28,157,125,233,241,120,1,0,137,68,156,251,219,226,226,82,120,60,30,108,108,132,224,245,250,176,176,48,143,138,138,42,200,178,
  4,77,83,108,135,9,94,162,131,101,110,89,160,47,138,18,84,85,197,237,219,55,177,176,48,135,88,44,10,93,215,209,210,210,10,65,16,176,186,186,
  2,191,63,3,243,243,179,168,173,173,199,245,235,151,161,235,196,211,112,124,124,20,179,179,83,248,201,79,254,12,45,45,237,184,118,237,50,146,73,5,
  35,35,67,152,155,155,193,79,126,242,103,104,108,108,193,157,59,55,33,138,130,225,135,151,52,229,24,236,135,11,251,245,165,91,26,40,165,91,186,189,
  48,155,138,29,119,176,96,130,154,117,82,101,99,154,28,234,152,216,70,158,71,73,73,5,230,231,167,48,52,116,223,60,157,90,73,218,58,147,100,42,
  154,139,60,15,92,116,19,128,148,150,150,162,172,172,2,25,25,25,72,38,147,88,94,14,96,121,121,201,76,126,45,46,46,65,69,69,149,121,178,142,
  68,194,232,239,239,71,69,69,37,20,69,193,220,220,28,3,70,44,144,212,212,212,140,68,34,129,217,217,105,100,102,102,162,166,166,14,83,83,19,216,
  216,32,97,16,191,223,111,254,46,28,14,59,128,146,223,159,1,89,150,16,141,70,109,249,32,176,129,62,56,64,146,32,208,83,254,179,216,44,72,66,
  61,5,73,212,181,126,51,49,194,23,143,77,178,212,164,85,149,168,162,211,220,50,22,8,91,197,3,34,60,158,164,185,121,39,147,42,18,137,36,39,
  71,193,130,36,54,135,110,107,93,44,193,149,181,181,124,209,52,70,25,91,64,99,99,11,230,231,231,112,241,226,57,131,97,202,196,159,252,201,159,161,
  190,190,25,115,115,211,134,229,137,7,126,127,38,0,29,241,120,220,156,123,89,89,153,16,69,209,200,149,139,67,85,85,147,29,149,36,9,225,112,24,
  235,235,235,16,4,1,30,143,140,214,214,93,24,26,26,192,141,27,87,77,153,128,63,253,211,191,64,73,73,25,150,151,3,240,249,252,72,36,226,144,
  36,15,68,81,68,52,26,133,44,123,225,241,144,156,41,10,132,98,177,152,1,212,192,41,120,187,91,172,164,215,219,52,80,74,183,116,123,65,128,17,
  187,169,164,90,224,41,168,96,173,70,232,215,102,77,146,36,84,85,213,163,178,178,14,83,83,163,24,31,31,196,210,210,34,146,201,132,121,26,167,140,
  12,15,202,248,60,30,65,16,240,230,155,111,35,43,43,11,35,35,67,200,200,40,194,222,189,251,16,14,135,240,237,183,95,33,22,139,161,187,251,0,
  218,219,59,48,54,54,2,65,16,177,182,182,138,225,225,33,28,63,126,2,161,80,8,75,75,95,114,39,112,81,20,144,155,155,139,31,254,240,71,136,
  197,162,248,219,191,253,43,136,162,128,99,199,78,160,188,188,28,23,46,124,15,77,211,177,127,255,65,236,218,213,129,191,255,251,255,198,149,148,11,130,
  0,159,207,135,159,255,252,23,136,68,194,248,195,31,126,203,177,92,108,152,144,244,161,200,49,115,236,169,122,179,220,150,39,201,30,210,107,178,216,36,
  97,83,150,238,89,132,8,31,71,99,25,37,194,76,186,251,157,201,178,136,100,82,53,239,81,81,20,51,7,137,213,205,74,38,85,6,32,1,238,97,
  218,84,140,237,86,170,216,164,140,63,55,55,31,189,189,61,72,38,21,131,9,11,97,98,98,28,229,229,149,24,27,27,65,36,18,193,241,227,175,26,
  160,197,131,91,183,72,238,144,223,79,202,246,9,155,169,224,202,149,203,152,158,158,50,24,78,13,209,104,204,44,174,160,0,44,43,43,7,87,174,92,
  66,34,161,24,236,214,42,230,230,230,80,94,94,5,81,244,224,228,201,87,176,186,186,130,220,220,124,4,2,1,220,185,115,11,109,109,29,232,238,222,
  143,149,149,21,228,231,23,96,126,126,14,3,3,15,184,190,118,99,49,211,45,13,148,210,45,221,94,200,147,55,191,184,11,54,251,2,222,125,157,134,
  106,60,30,25,219,13,203,8,130,128,218,218,38,84,87,215,99,106,106,20,83,83,163,152,159,159,53,23,108,54,79,200,158,240,202,230,74,13,14,246,
  227,250,245,43,16,69,17,153,153,25,248,209,143,126,134,147,39,223,192,217,179,167,225,241,120,208,223,255,0,231,206,157,53,67,42,20,152,144,147,180,
  61,228,37,98,215,174,14,204,205,205,34,59,59,11,245,245,13,152,152,24,195,149,43,23,240,250,235,111,161,167,231,22,84,85,199,238,221,157,56,127,
  254,123,68,163,81,14,40,73,146,132,163,71,143,163,160,160,0,177,88,20,146,36,153,214,14,108,206,15,251,121,244,239,45,59,8,226,176,254,108,24,
  37,112,0,137,213,183,178,141,16,23,182,68,127,97,198,54,107,220,76,25,83,90,165,232,214,8,251,162,155,44,143,29,36,81,253,44,146,151,244,120,
  149,225,169,110,88,44,22,195,185,115,223,34,16,88,50,115,251,84,85,197,165,75,231,145,145,145,137,72,36,138,51,103,190,64,118,118,142,113,63,34,
  2,129,69,108,108,132,241,217,103,159,152,58,74,170,170,97,101,101,213,8,37,10,88,95,15,226,179,207,62,193,234,234,10,146,73,5,130,32,32,26,
  141,225,139,47,62,195,194,2,95,185,246,245,215,167,33,203,50,214,214,214,112,225,2,80,80,80,128,254,254,62,140,140,12,67,215,117,12,14,14,32,
  18,137,32,63,63,31,253,253,15,48,62,62,102,206,89,162,84,159,90,142,35,141,151,210,64,41,221,210,237,133,5,76,172,22,18,101,156,104,57,50,
  175,200,77,190,84,53,185,163,207,16,69,9,117,117,45,168,172,172,195,220,220,20,38,38,72,126,131,162,40,54,70,137,158,188,117,115,81,213,117,18,
  86,160,215,145,72,196,113,249,242,121,188,245,214,251,240,120,60,208,52,21,126,191,31,62,159,215,200,171,32,108,0,203,158,208,176,6,93,208,27,26,
  154,240,224,193,61,148,149,149,163,181,181,29,83,83,19,152,156,28,195,234,234,10,58,59,187,16,143,39,176,190,190,134,225,225,65,14,244,8,130,128,
  142,142,14,20,23,23,227,206,157,219,40,41,41,49,66,29,42,195,24,217,55,64,5,146,68,129,8,159,163,245,44,124,175,236,18,13,108,200,144,62,
  15,158,65,210,57,211,212,23,237,32,192,247,53,12,214,67,51,25,61,194,132,104,14,47,53,86,19,76,85,117,155,158,23,54,101,144,118,122,157,244,
  179,99,177,56,102,103,103,56,17,83,0,88,93,93,195,234,234,26,0,96,122,122,218,245,243,199,198,70,83,152,230,234,208,180,4,102,102,102,184,67,
  80,56,28,65,36,18,225,24,102,65,0,2,129,128,249,186,161,161,65,199,245,42,74,4,3,3,253,230,33,138,142,109,22,136,186,133,242,211,237,97,
  128,82,58,143,43,221,210,237,9,237,16,41,190,92,94,99,86,236,208,23,232,228,148,42,73,18,74,75,202,16,143,199,56,203,18,143,71,134,236,145,
  145,157,157,247,80,151,230,241,120,81,91,219,132,210,210,74,44,46,204,194,159,145,9,10,141,4,65,0,249,159,113,29,180,146,199,252,18,140,235,22,
  176,190,190,6,77,85,145,153,145,137,120,44,134,214,214,93,240,122,125,16,69,17,51,51,83,184,113,253,42,7,10,116,170,53,35,0,101,165,101,240,
  120,60,152,157,153,198,218,234,10,222,122,235,125,100,102,100,32,18,137,226,242,165,243,120,255,131,143,16,139,197,240,197,231,159,114,125,39,8,2,242,
  114,243,176,123,119,23,46,93,60,143,130,130,2,148,151,149,3,186,14,243,202,141,235,19,5,82,213,230,241,120,80,86,90,142,141,141,13,115,99,214,
  117,70,68,147,116,254,211,109,2,32,218,116,157,232,166,150,147,157,107,177,139,154,14,77,213,160,107,128,174,233,208,181,77,198,211,11,4,154,32,128,
  220,147,160,27,247,165,3,162,205,204,89,211,161,169,228,217,144,251,214,205,239,120,24,54,73,119,249,206,252,76,175,67,99,242,159,84,85,35,227,131,
  225,111,55,147,197,176,112,45,111,82,109,222,51,116,94,233,212,94,6,200,162,40,254,79,93,111,69,20,4,232,34,233,15,10,188,105,159,110,107,61,
  74,99,128,173,129,146,158,238,165,116,75,183,39,132,147,220,255,199,174,76,58,52,104,186,230,40,81,79,42,73,172,173,173,162,176,176,8,239,190,247,
  3,51,239,0,182,133,58,85,50,247,118,155,223,159,129,218,186,38,235,125,141,208,20,119,197,148,233,226,126,79,126,246,249,188,208,1,196,226,49,120,
  188,94,204,207,207,226,250,245,75,0,128,104,52,10,8,182,235,22,1,154,32,94,89,85,141,252,252,2,28,60,116,20,146,36,34,39,55,23,101,229,
  21,24,29,29,193,226,210,2,214,214,86,17,143,199,177,20,88,98,62,149,248,108,29,63,241,10,146,201,4,150,87,150,209,220,210,10,127,70,6,242,
  242,243,177,186,186,66,246,32,0,16,116,64,32,76,82,78,78,46,222,120,243,93,147,193,216,76,156,240,217,55,1,94,175,7,170,170,34,28,14,67,
  213,200,24,225,191,116,174,79,94,166,57,194,207,143,212,243,104,231,247,207,142,92,251,104,38,255,165,233,154,217,223,170,205,170,199,13,32,217,77,115,
  183,3,212,4,6,36,217,255,38,21,11,180,89,110,145,38,8,16,117,193,0,97,34,65,84,26,76,112,183,157,94,76,183,173,24,165,116,75,183,116,
  123,198,167,108,167,51,249,234,234,42,254,240,135,223,224,245,215,79,161,165,165,205,118,200,20,92,101,4,30,229,148,79,53,133,184,77,74,119,50,1,
  100,195,160,165,250,58,154,155,219,177,177,17,66,34,17,135,44,203,88,95,95,199,210,210,130,163,10,137,42,41,235,204,97,186,166,166,22,147,147,19,
  152,155,155,129,44,203,200,203,35,102,161,35,35,195,208,52,13,145,72,24,241,120,194,12,137,208,107,163,76,27,73,48,127,11,165,165,101,200,206,206,
  65,71,71,39,46,94,60,199,125,6,123,253,138,162,184,110,102,207,54,156,229,124,126,130,0,68,34,73,220,185,115,11,61,61,183,77,161,80,54,36,
  149,46,233,126,124,115,143,159,135,224,170,240,88,239,66,251,124,99,133,87,157,60,143,245,123,123,57,62,253,157,221,2,200,210,129,210,93,64,146,158,
  146,124,34,255,46,218,44,138,88,186,40,29,123,75,3,165,116,75,183,151,129,123,114,209,28,210,245,56,190,249,230,43,156,59,247,157,89,233,70,12,
  112,61,240,249,100,248,124,50,106,107,155,209,221,125,252,161,63,57,24,92,197,196,196,48,242,243,139,80,83,211,200,93,131,29,72,200,178,140,204,204,
  44,100,102,102,35,35,195,143,234,234,26,180,182,238,194,217,179,167,13,166,70,68,78,78,54,242,243,139,140,60,27,213,96,120,4,228,230,230,161,168,
  136,24,136,38,147,73,243,191,79,159,254,28,179,179,51,16,4,162,153,115,226,196,171,200,204,204,194,250,250,58,100,217,195,41,46,211,107,138,70,163,
  248,253,239,127,107,110,28,71,143,158,64,83,83,19,46,92,56,103,0,62,48,57,33,58,36,73,70,48,184,134,51,103,254,25,193,224,186,185,169,232,
  186,182,45,69,231,39,6,145,92,28,239,169,230,16,201,77,75,32,145,72,114,94,123,219,169,236,74,183,71,157,139,112,200,22,88,76,143,238,194,242,
  164,174,176,99,171,236,220,188,225,44,97,205,212,85,123,108,165,168,219,35,167,5,12,84,253,220,186,135,244,211,76,3,165,116,75,183,151,234,100,171,
  115,254,106,116,147,39,214,13,73,35,55,73,130,215,43,67,85,21,232,186,7,130,224,131,170,42,15,245,121,235,235,171,152,158,30,195,248,248,32,66,
  161,16,186,187,143,153,215,97,215,183,161,63,47,44,44,160,188,188,28,71,143,158,128,207,231,131,40,74,248,234,171,47,49,49,49,10,0,88,92,92,
  192,158,61,251,112,252,248,107,144,36,17,225,112,24,223,124,115,6,51,51,51,232,236,236,194,171,175,190,1,81,148,16,8,44,97,117,117,25,147,147,
  147,88,92,92,52,216,38,1,51,51,211,8,4,150,144,151,87,128,149,149,85,172,172,172,154,50,6,188,32,36,207,168,133,66,33,44,47,47,27,167,
  126,29,128,200,148,162,235,102,210,112,60,30,71,34,145,100,0,146,110,134,86,158,23,208,193,178,10,44,155,193,234,94,165,55,192,39,7,146,236,207,
  130,157,19,118,6,202,98,121,220,89,65,30,12,243,226,170,101,101,229,136,199,99,70,69,156,5,192,88,70,107,171,235,163,77,146,136,94,19,81,155,
  199,99,175,4,76,183,52,80,74,183,116,123,198,27,163,245,197,154,208,90,139,180,200,109,142,180,234,77,81,36,40,138,106,154,133,238,132,65,154,154,
  26,195,244,244,40,214,215,215,13,121,0,221,44,155,119,3,72,244,186,190,251,238,44,242,243,137,111,84,52,26,193,242,242,50,119,66,190,125,251,54,
  134,134,134,204,18,119,77,83,17,137,68,112,251,246,77,12,13,13,66,16,68,131,105,74,34,22,139,67,85,239,26,2,121,164,45,45,45,225,211,79,
  255,96,106,231,92,188,120,206,48,65,85,83,132,35,72,123,240,224,62,134,134,6,145,76,42,144,36,186,49,137,16,4,118,195,17,152,62,182,192,199,
  179,100,148,54,3,204,246,240,15,207,52,166,55,193,167,5,152,220,152,157,237,53,30,204,83,171,19,65,16,80,81,81,137,159,255,252,23,184,125,251,
  38,190,255,254,91,227,189,5,23,95,195,173,63,143,22,38,80,144,180,181,50,57,92,0,95,186,165,129,82,186,165,219,11,180,56,211,141,144,230,68,
  144,69,80,48,243,110,146,73,90,10,172,154,250,46,219,105,225,112,16,99,99,131,152,154,26,195,250,250,154,169,159,68,255,158,253,238,100,53,200,247,
  112,56,140,112,56,236,112,67,167,45,153,220,64,56,28,118,44,242,137,68,18,27,27,97,151,141,199,141,233,90,55,63,59,20,218,224,64,27,15,146,
  172,255,142,68,162,16,197,152,1,208,68,174,52,154,45,169,119,3,72,228,103,245,153,131,15,251,230,72,239,217,30,142,125,94,0,221,203,217,156,158,
  131,219,201,1,220,12,196,83,43,29,170,225,229,243,249,112,242,228,43,16,4,24,150,58,34,116,93,51,230,184,149,135,180,221,225,184,185,80,109,170,
  107,77,143,161,52,80,74,183,116,123,225,88,165,212,121,17,212,99,141,93,132,21,69,68,50,169,64,150,69,51,73,58,85,139,70,195,24,29,237,199,
  232,232,32,214,214,214,56,6,133,181,48,177,24,44,141,81,128,214,29,44,19,123,66,78,181,64,111,199,63,202,89,237,227,252,55,39,216,177,64,37,
  255,209,86,31,209,13,199,121,58,7,23,106,163,76,149,5,148,158,237,243,223,108,60,176,192,233,225,216,141,116,99,199,153,19,216,184,129,165,135,132,
  90,142,170,56,214,184,89,192,209,163,199,17,141,70,112,255,126,47,60,30,47,167,34,159,202,72,119,171,207,227,77,180,5,238,32,147,110,105,160,148,
  110,233,246,82,46,226,44,139,160,170,58,151,196,169,40,188,240,164,162,184,3,165,88,44,138,177,177,1,12,14,62,48,92,196,45,43,8,139,161,176,
  132,32,45,193,73,137,75,4,117,134,226,120,160,242,176,85,119,155,51,75,238,225,135,84,73,173,244,4,78,175,83,20,221,43,136,40,83,103,129,36,
  213,80,125,126,62,114,148,54,3,76,155,189,46,221,30,110,204,209,255,164,213,98,84,106,131,36,70,243,22,55,172,186,248,246,0,12,108,0,8,168,
  175,111,64,117,117,13,126,245,171,191,199,171,175,190,225,0,58,143,131,13,99,61,14,173,75,21,82,30,206,210,45,13,148,210,45,221,94,176,69,156,
  207,51,176,194,44,212,206,1,16,4,149,81,116,22,144,76,242,64,41,18,217,192,196,196,48,6,6,30,32,24,92,51,76,69,9,72,34,161,58,141,
  203,119,162,64,137,26,122,250,124,126,164,162,230,237,161,31,123,8,204,109,179,112,3,131,219,217,112,220,128,148,206,232,194,176,0,141,0,35,247,221,
  198,227,241,66,146,136,99,59,49,4,214,108,86,24,170,67,229,249,121,0,73,15,251,154,116,219,178,23,29,125,26,139,145,177,239,247,251,57,111,67,
  187,193,242,163,116,191,223,159,129,119,222,121,15,131,131,3,168,168,168,68,81,81,17,0,226,117,72,149,190,31,150,189,226,237,134,44,143,67,175,215,
  7,143,199,107,84,79,38,28,0,41,157,239,182,19,160,148,238,167,116,75,183,39,183,38,111,165,132,155,226,223,169,10,51,93,8,53,85,135,0,29,
  42,52,136,130,10,69,160,218,217,100,177,92,95,95,195,196,196,40,134,6,239,99,117,109,21,154,74,188,169,146,138,10,37,169,32,153,84,185,234,46,
  222,62,67,71,112,61,8,0,40,40,40,132,40,72,16,160,1,212,188,148,185,38,86,13,89,223,20,36,9,174,255,186,85,142,210,78,65,3,123,125,
  244,139,85,15,47,44,40,4,64,36,5,194,225,8,52,85,55,213,158,53,85,51,250,73,75,111,24,207,219,28,217,238,60,218,193,99,211,53,16,229,
  111,13,128,104,41,129,135,130,100,236,231,231,21,192,231,245,66,83,99,208,32,64,20,200,151,102,170,189,19,229,247,237,10,75,210,247,135,14,228,100,
  231,96,118,118,22,89,153,89,216,183,119,63,10,11,139,32,203,30,84,86,84,97,101,121,109,199,247,70,199,189,165,162,79,174,213,250,31,80,144,95,
  0,175,215,139,112,120,3,27,27,27,228,122,232,117,165,149,185,119,6,148,210,170,156,233,150,110,79,106,15,216,90,9,119,187,170,195,186,174,67,208,
  1,104,128,160,18,186,70,7,176,182,182,134,187,119,239,224,222,189,219,88,89,9,48,204,15,9,41,209,60,28,139,77,210,57,144,36,73,34,36,72,
  88,94,9,64,81,146,168,171,107,128,215,231,69,82,73,18,21,109,13,230,119,208,36,105,232,79,189,4,153,3,70,54,214,74,135,113,141,2,32,74,
  68,161,152,254,119,109,93,61,0,32,16,88,130,166,169,208,161,49,74,215,105,117,226,231,117,142,96,219,51,100,103,207,77,211,117,8,186,6,85,3,
  68,73,132,166,107,88,93,91,65,60,30,67,101,85,21,178,178,179,17,141,197,32,136,128,160,11,228,187,40,80,145,119,194,60,10,91,3,122,77,7,
  68,1,80,53,13,16,128,185,249,57,252,225,15,191,51,43,223,126,248,195,143,161,170,26,238,63,232,181,84,192,141,113,185,213,189,145,67,8,153,199,
  228,250,200,184,23,68,250,69,126,87,83,91,11,65,16,176,188,28,64,60,30,131,166,235,230,152,223,206,231,164,27,203,40,165,91,186,165,219,243,189,
  153,112,102,185,186,233,100,78,28,196,7,49,48,208,111,46,192,148,90,215,52,103,117,151,149,140,13,3,36,81,186,94,195,252,252,60,150,150,22,81,
  81,81,133,170,170,106,12,15,15,66,215,73,30,20,64,147,137,9,98,34,41,29,150,88,222,211,0,76,188,34,50,31,106,160,97,72,73,146,76,0,
  40,8,2,50,50,50,208,216,216,12,69,73,98,114,114,140,9,177,237,172,148,58,221,94,142,57,68,191,179,9,242,162,72,216,216,169,169,73,52,55,
  183,162,166,166,14,43,43,203,0,136,44,135,36,73,220,161,128,55,155,117,103,151,220,12,153,73,193,129,102,204,33,96,113,113,209,81,209,201,43,120,
  111,62,15,168,65,52,61,236,208,170,58,58,254,61,30,15,26,27,91,0,0,163,163,195,76,113,128,230,42,42,155,110,105,160,148,110,233,246,146,44,
  246,22,91,68,23,224,104,52,102,46,144,22,155,164,59,74,251,157,185,69,26,4,1,80,85,2,50,34,145,48,198,199,199,80,81,81,133,67,135,142,
  98,108,108,132,171,24,147,101,98,25,162,170,4,88,177,85,101,79,115,193,101,55,18,10,148,168,89,176,181,97,144,223,239,221,219,141,236,236,28,204,
  207,207,26,142,238,186,173,127,210,9,173,127,172,243,136,128,36,2,92,226,241,4,70,70,6,209,220,220,138,3,7,14,99,112,176,31,209,104,196,60,
  36,208,70,230,10,15,182,55,3,74,44,144,167,121,67,180,221,186,117,211,246,122,202,244,34,37,8,227,65,20,127,64,144,101,201,152,7,228,144,208,
  222,222,129,178,178,114,132,66,65,12,14,246,187,22,101,164,199,126,26,40,165,91,186,189,148,167,98,178,200,195,56,25,146,133,91,16,52,243,4,106,
  247,141,179,235,34,177,139,49,21,170,163,76,75,111,111,15,218,219,119,161,178,178,26,251,247,31,196,141,27,215,0,136,144,101,118,65,39,21,65,169,
  42,178,158,22,88,114,2,37,145,59,93,87,84,84,160,171,171,219,216,148,110,32,153,84,56,73,4,75,105,57,189,91,252,241,205,33,221,113,160,24,
  26,26,66,103,231,52,170,170,170,113,248,240,81,156,59,247,173,41,194,202,142,59,77,211,24,5,108,125,219,99,148,7,76,196,158,134,149,255,32,175,
  213,183,156,87,78,16,38,154,227,159,140,125,1,5,5,5,232,238,62,8,65,16,112,235,214,13,68,163,81,147,85,78,139,150,166,129,82,186,165,219,
  75,223,136,159,19,152,18,120,193,225,21,182,153,201,38,217,24,232,162,75,217,38,226,125,182,186,186,138,155,55,175,227,173,183,222,195,145,35,39,177,
  178,178,130,209,209,97,19,44,145,170,59,1,186,46,26,155,140,102,126,230,211,5,74,214,137,154,221,44,104,216,33,39,39,27,175,190,250,38,114,114,
  114,49,52,52,128,7,15,122,205,252,44,214,55,46,237,153,246,199,201,38,89,50,18,36,145,95,16,4,132,195,97,220,184,113,5,101,101,63,194,193,
  131,71,177,178,178,140,222,222,187,32,217,216,96,52,142,68,38,92,150,154,145,228,53,141,96,211,54,178,170,213,172,113,40,153,64,105,59,239,203,142,
  125,42,21,34,138,2,252,126,63,94,121,229,117,148,148,148,97,118,118,26,61,61,183,28,7,132,180,202,123,26,40,165,91,186,189,180,39,97,86,203,
  133,234,43,177,38,157,44,104,73,165,189,99,81,250,2,151,239,36,138,2,84,85,195,237,219,55,81,86,86,129,61,123,246,226,237,183,223,199,183,223,
  158,193,224,224,128,161,120,45,152,167,82,114,170,22,159,217,66,107,207,79,162,121,73,133,133,133,120,243,205,119,80,93,93,139,213,213,21,124,247,221,
  215,166,130,57,205,235,74,123,166,253,177,51,74,22,88,18,4,194,204,106,154,128,193,193,1,148,149,93,197,209,163,39,240,198,27,239,64,20,37,220,
  189,123,219,84,123,223,9,163,100,31,167,124,254,17,155,219,103,177,179,219,125,95,251,216,167,128,41,55,55,23,175,188,242,6,90,90,218,77,159,197,
  104,52,198,229,42,166,67,206,105,160,148,110,233,246,71,3,150,216,69,127,179,215,167,122,15,86,163,137,156,172,85,115,17,62,123,246,43,120,60,30,
  180,183,119,224,189,247,62,66,81,209,101,60,120,208,139,181,181,53,19,104,61,11,165,232,84,9,221,162,40,64,150,101,52,55,183,226,240,225,227,40,
  41,41,197,234,234,50,62,251,236,19,195,116,212,74,102,183,159,168,211,237,143,111,254,240,64,137,90,219,16,192,113,241,226,57,120,189,62,236,223,127,
  16,111,189,245,30,10,11,11,113,239,94,15,2,129,0,4,65,55,149,223,217,156,191,84,121,74,246,113,202,255,126,115,246,119,51,91,20,251,216,23,
  4,1,13,13,77,56,114,228,56,42,43,171,177,177,177,129,211,167,63,195,220,220,44,163,196,239,28,255,233,182,19,160,148,238,175,116,75,183,39,180,
  50,195,93,175,68,223,230,107,245,237,129,159,135,62,89,131,20,95,171,138,102,124,30,81,96,137,70,163,56,253,229,231,216,216,8,225,224,193,35,56,
  118,236,21,52,53,181,98,100,100,16,83,147,19,88,92,90,68,60,22,123,230,57,74,146,44,35,63,47,31,213,53,181,104,110,110,69,109,109,61,68,
  81,196,228,228,4,206,158,253,10,11,243,243,150,183,155,162,146,242,107,85,131,174,233,14,77,168,116,123,134,115,100,59,115,229,49,232,40,217,199,190,
  166,107,16,0,168,176,198,62,116,5,223,125,251,53,34,145,48,142,30,61,129,253,251,15,163,190,190,9,35,35,67,152,156,24,199,194,226,60,162,145,
  8,68,97,187,204,143,51,236,198,51,74,6,120,33,191,216,126,238,147,40,34,47,55,15,85,213,53,104,110,106,65,109,93,3,188,94,47,22,23,230,
  113,246,236,87,152,156,156,128,162,24,21,175,138,113,80,72,143,253,52,163,148,110,233,150,110,15,119,178,166,137,221,130,64,78,214,0,17,104,252,254,
  187,179,152,158,154,196,177,227,39,81,86,86,129,210,210,50,236,221,123,0,209,104,24,145,112,24,137,100,226,217,136,213,9,128,36,201,200,200,200,68,
  70,70,38,114,114,114,32,8,2,98,177,24,110,220,184,138,158,59,183,176,177,17,54,125,220,20,19,36,165,79,212,233,177,111,141,125,65,128,201,42,
  25,112,201,28,23,151,47,93,192,204,204,20,142,29,59,137,218,218,122,20,22,22,97,207,158,189,136,68,34,8,135,55,136,210,245,102,99,136,201,69,
  226,152,36,23,140,8,221,166,100,180,197,251,138,162,4,191,63,3,89,153,89,200,202,206,134,36,73,80,20,5,215,175,95,193,205,27,215,177,190,190,
  238,240,51,76,143,253,71,4,74,105,177,169,116,75,183,39,117,88,118,19,199,179,207,56,157,121,37,30,73,76,239,161,78,214,154,0,162,249,109,253,
  79,211,69,227,75,67,255,64,63,38,38,39,208,216,216,132,246,246,221,168,174,174,70,81,81,9,138,138,74,158,139,62,86,20,5,243,243,179,24,26,
  26,192,192,64,63,86,86,150,57,243,91,98,81,162,65,85,117,46,71,131,221,20,211,237,217,206,17,108,34,35,153,250,223,31,109,142,208,177,47,138,
  2,116,85,135,4,209,120,55,137,176,172,154,136,177,177,49,204,205,205,162,182,182,30,187,118,237,70,77,77,29,10,11,139,80,88,88,244,92,244,157,
  166,105,88,94,94,194,240,240,32,250,250,30,96,121,57,96,10,203,90,227,223,153,159,148,30,251,105,70,41,221,210,45,221,118,116,178,182,36,7,136,
  252,54,159,39,33,138,58,194,225,48,238,223,239,69,95,223,125,100,100,100,162,176,176,16,217,217,57,240,122,189,91,250,189,61,41,74,73,81,20,68,
  34,97,172,172,172,32,20,10,58,52,163,216,220,12,55,144,148,110,233,177,79,43,64,69,17,70,145,130,21,18,147,36,194,188,68,34,81,12,14,14,
  96,120,120,16,62,159,31,69,69,69,200,201,201,129,215,235,131,32,136,41,199,190,155,60,128,32,0,5,5,69,232,238,62,132,149,149,0,238,222,189,
  5,69,81,28,121,83,155,131,24,50,182,163,209,40,214,214,86,176,182,182,102,22,41,208,113,158,106,236,167,177,81,26,40,165,91,186,165,219,67,110,
  24,84,161,152,234,51,233,186,96,84,225,16,157,38,73,18,141,16,133,0,69,9,33,28,222,0,31,86,120,154,215,108,227,35,168,173,4,44,61,40,
  62,121,85,115,173,114,75,159,168,211,99,223,14,150,232,97,129,130,22,73,130,49,23,72,133,103,52,26,197,236,236,12,3,132,156,118,58,44,80,98,
  149,178,105,9,127,89,89,5,186,187,15,97,99,35,132,129,129,7,38,136,177,192,141,37,104,233,28,239,78,133,113,10,174,216,177,158,10,36,165,229,
  48,254,200,128,146,115,128,110,46,251,190,93,199,242,157,78,180,199,117,157,110,215,247,60,14,102,97,147,29,241,97,250,227,73,247,127,186,237,12,44,
  81,125,38,85,165,11,171,96,106,15,209,242,123,34,35,176,245,156,123,26,128,137,93,248,217,252,11,183,10,159,173,170,148,210,45,13,150,88,176,68,
  244,147,84,0,146,9,202,9,112,226,85,179,249,42,84,119,192,196,50,75,94,175,23,0,12,155,17,25,201,100,210,144,250,0,199,206,110,167,154,149,
  170,107,243,128,41,13,146,210,64,9,124,153,37,45,141,228,7,35,152,211,174,110,83,42,78,93,50,157,10,76,177,190,61,188,231,143,176,101,117,194,
  214,215,9,215,235,123,222,22,117,55,192,71,203,91,217,5,103,179,235,117,43,111,221,9,120,178,63,163,173,62,47,221,30,126,195,32,73,174,132,89,
  210,140,60,14,194,58,17,144,196,42,22,11,207,0,41,217,15,20,238,106,203,150,159,215,243,126,0,73,183,231,99,14,176,150,61,212,203,144,106,29,
  145,189,228,225,15,7,169,14,205,214,94,181,117,8,219,57,142,249,125,195,62,7,220,14,20,233,246,71,198,40,177,66,115,212,182,192,174,89,97,151,
  169,231,147,217,192,129,21,74,221,243,117,9,252,160,102,61,114,216,205,37,213,235,83,41,7,187,1,37,150,54,181,20,147,245,45,1,217,211,108,172,
  4,63,123,93,108,159,108,245,220,236,113,123,231,107,54,95,200,216,42,173,116,123,50,96,137,206,13,42,178,71,213,191,53,77,48,69,247,72,153,243,
  147,97,11,183,191,89,88,243,150,221,44,210,0,41,221,30,113,135,217,241,188,225,127,231,22,53,208,83,188,175,155,102,72,122,172,62,127,64,233,5,
  122,38,130,32,0,116,17,135,0,81,16,33,137,34,100,137,55,4,20,12,131,80,77,213,185,248,175,170,106,16,5,61,5,0,210,161,9,124,124,152,
  128,3,209,100,79,136,222,5,173,12,50,6,191,190,9,40,128,0,201,0,73,178,44,51,96,73,224,88,47,26,38,160,85,58,42,12,93,27,193,0,
  75,120,246,96,73,20,5,163,207,133,148,125,162,166,184,86,250,220,44,224,200,179,108,110,125,231,166,44,109,198,226,117,13,162,81,218,251,92,143,95,
  55,77,24,125,147,215,60,6,141,152,199,1,66,76,192,4,30,48,209,11,114,50,183,79,39,4,231,150,159,196,94,119,42,214,56,13,146,94,144,57,
  178,213,252,192,227,157,35,116,93,98,215,55,67,77,9,220,127,233,48,126,99,140,120,125,107,2,72,39,103,9,162,91,196,21,77,88,5,19,172,166,
  17,116,242,218,84,247,101,142,113,58,7,97,72,28,16,222,139,92,169,96,173,209,186,241,55,162,32,152,123,213,243,176,143,188,176,64,233,133,148,7,
  32,72,9,130,8,72,178,8,217,67,220,147,101,89,50,89,27,203,158,65,131,168,170,68,121,85,228,99,203,172,227,58,45,163,20,52,141,89,144,5,
  206,153,157,148,26,211,13,218,160,58,83,244,159,40,146,207,19,37,1,178,71,130,199,67,175,79,228,152,25,150,77,82,85,21,201,164,49,193,44,68,
  248,76,217,19,59,123,71,221,170,173,80,13,5,161,128,8,98,113,1,193,185,65,137,34,185,23,81,178,28,175,217,126,176,179,78,246,249,76,129,46,
  84,18,27,82,213,167,83,66,255,104,123,128,238,90,248,207,174,130,250,19,40,125,126,220,128,137,189,12,62,132,234,206,164,62,105,38,201,14,156,82,
  133,211,211,237,69,192,73,169,133,1,54,155,67,143,115,142,208,181,73,16,97,29,226,204,47,178,102,81,76,37,48,103,92,29,122,74,86,201,56,2,
  67,135,200,129,36,69,81,204,245,44,169,36,161,233,228,224,167,106,154,249,179,253,174,92,199,55,116,46,106,39,136,100,253,37,176,73,132,8,141,200,
  126,232,58,4,145,0,55,221,64,86,233,185,241,48,140,210,11,135,145,44,86,66,146,36,124,252,241,79,208,218,218,182,173,191,253,252,243,79,49,52,
  212,15,128,84,32,116,118,238,197,235,175,159,194,200,200,32,62,251,236,83,36,147,10,231,176,44,138,2,42,42,42,241,47,255,229,191,198,224,96,63,
  126,255,251,223,153,86,15,118,230,201,173,73,146,0,89,150,224,241,200,248,243,63,255,75,148,148,148,166,124,109,32,176,136,7,15,238,161,167,231,142,
  121,143,138,162,66,16,136,183,144,219,38,145,42,180,160,235,112,108,102,169,114,132,82,189,31,187,136,144,208,161,197,218,181,183,239,194,135,31,126,132,
  139,23,207,225,234,213,203,38,200,36,215,43,24,97,78,119,192,69,222,67,194,27,111,188,129,67,135,142,226,210,165,243,184,117,235,58,0,192,231,243,
  227,223,254,219,255,59,226,241,24,254,203,127,249,79,230,223,254,252,231,127,142,210,210,50,252,221,223,253,95,152,157,157,133,162,88,134,176,154,150,158,
  248,79,23,156,176,249,97,110,244,192,179,191,198,116,75,183,109,177,73,176,179,220,214,65,144,77,149,16,69,17,153,153,153,200,203,203,67,65,65,33,
  178,178,178,33,203,210,38,99,207,93,26,0,0,10,10,10,1,0,121,121,249,216,183,239,144,99,63,177,214,179,205,243,95,99,177,56,214,215,215,16,
  12,174,99,109,109,21,201,164,2,65,208,140,207,34,7,87,10,200,216,68,245,205,82,70,210,237,37,2,74,143,122,114,32,76,134,8,89,22,209,218,
  218,10,0,104,106,106,69,73,73,17,150,150,150,33,138,68,201,148,14,72,73,18,109,63,235,102,238,208,86,159,69,1,134,44,75,91,134,38,138,139,
  75,241,234,171,167,0,8,232,233,185,109,78,52,114,45,186,117,116,160,231,9,93,119,228,237,184,185,78,91,159,43,24,85,77,108,200,43,245,251,177,
  249,68,178,44,25,247,97,221,15,0,200,178,4,159,207,107,168,191,18,99,213,100,146,176,119,20,44,81,54,140,5,74,178,44,154,9,193,146,36,194,
  227,33,239,231,241,136,230,181,122,60,178,99,81,35,159,47,154,215,74,159,83,186,165,193,73,186,165,219,78,65,18,111,46,43,154,7,65,235,103,9,
  133,133,133,104,107,107,71,125,125,35,170,170,106,32,203,143,103,203,204,203,43,192,209,163,175,60,242,251,132,66,65,76,78,78,96,112,176,31,99,99,
  35,136,199,19,102,46,33,217,63,0,192,9,150,210,237,37,7,74,108,102,191,170,170,248,228,147,223,193,239,247,194,235,149,225,245,122,144,149,149,137,
  159,253,236,47,49,53,53,134,239,190,59,131,100,82,133,162,40,166,239,13,221,156,203,203,43,80,89,89,131,229,229,37,20,21,149,96,247,238,46,92,
  188,120,206,156,64,148,81,162,160,128,2,37,82,42,45,108,10,124,40,8,97,39,33,157,156,191,249,205,223,33,153,76,112,159,163,105,26,14,31,62,
  137,230,230,93,232,238,62,136,251,247,239,26,9,129,170,9,12,40,248,98,195,13,124,133,143,206,1,38,251,201,198,13,40,217,13,86,217,114,106,158,
  185,35,165,172,116,33,241,122,101,179,31,125,62,15,100,89,132,162,72,16,69,197,100,194,84,149,130,60,205,4,108,244,61,233,105,205,2,74,178,1,
  148,100,243,181,110,64,137,158,244,72,194,59,158,145,216,97,186,61,207,155,223,31,43,136,252,99,191,255,135,233,47,123,65,16,93,95,232,129,240,192,
  129,67,232,234,234,54,35,1,193,224,58,22,22,230,17,12,174,155,76,208,246,0,25,24,96,198,30,120,249,2,132,237,200,216,208,247,244,122,189,40,
  42,42,70,101,101,53,118,239,222,131,142,142,78,140,142,14,227,242,229,139,152,153,153,114,25,15,154,41,67,96,151,56,72,183,151,148,81,178,235,70,
  168,170,10,85,21,205,16,16,125,141,101,97,96,81,153,116,82,52,53,181,0,0,250,251,239,97,239,222,67,104,108,108,198,181,107,151,12,22,68,112,
  61,125,80,26,150,108,254,169,19,145,173,16,147,117,58,177,152,38,209,76,230,182,114,113,68,220,184,113,9,217,217,185,40,47,175,66,65,65,33,86,
  87,87,76,48,193,158,126,40,3,100,37,128,171,174,37,162,118,160,100,63,57,165,122,63,90,29,104,15,113,202,178,104,230,88,209,251,145,36,9,62,
  159,7,170,106,129,36,11,192,168,182,73,47,64,16,104,117,162,232,232,87,242,179,213,79,178,44,58,88,49,86,106,97,39,149,41,233,246,114,3,162,
  205,242,166,158,238,186,148,122,3,122,92,155,210,102,247,255,172,239,157,61,252,61,175,0,138,237,43,251,97,144,2,164,172,172,76,188,249,230,59,216,
  189,123,15,4,65,192,212,212,4,238,221,187,131,133,133,121,108,108,132,16,143,199,55,77,125,112,178,86,100,253,179,14,216,188,227,27,43,154,186,89,
  245,176,253,189,178,178,50,145,147,147,135,198,198,102,236,221,187,31,77,77,45,40,43,43,199,185,115,223,162,183,247,174,227,249,208,60,41,65,72,135,
  224,94,106,160,100,55,243,164,9,190,138,162,65,146,44,3,64,58,48,120,225,57,152,172,140,44,75,168,171,107,66,56,28,194,252,252,20,138,139,75,
  209,214,214,137,150,150,54,12,14,246,67,211,4,51,100,36,73,214,196,162,3,157,126,167,64,135,141,115,91,44,137,100,228,39,209,208,27,141,137,91,
  149,116,150,78,7,1,55,94,175,207,4,9,52,57,93,16,4,236,218,213,137,246,246,221,166,191,214,220,220,52,250,251,239,99,120,120,16,154,166,33,
  43,43,7,127,241,23,255,3,166,166,38,240,219,223,254,138,73,198,5,202,202,42,240,103,127,246,23,184,127,255,30,206,157,251,6,178,44,33,51,51,
  3,135,14,29,71,107,107,135,217,183,247,238,221,198,213,171,23,205,28,45,0,248,233,79,127,129,96,48,136,185,185,105,156,56,241,58,0,224,218,181,
  243,72,38,147,230,125,118,117,237,199,222,189,135,0,0,179,179,83,232,237,189,139,190,190,62,14,128,145,231,6,243,139,173,250,99,195,155,118,64,105,
  95,116,72,63,139,174,57,86,233,246,199,3,142,236,50,32,60,80,120,86,3,195,93,183,134,13,101,63,44,104,176,231,23,178,27,61,191,129,62,219,
  251,223,172,10,241,81,238,255,73,140,39,11,36,81,38,137,176,73,153,153,25,248,240,195,143,209,212,212,130,120,60,142,75,151,206,161,175,175,23,241,
  120,204,33,98,202,63,239,205,242,149,44,121,13,186,142,57,65,140,19,116,185,9,88,178,102,190,27,27,97,68,34,17,44,46,206,99,96,224,1,142,
  30,61,137,182,182,14,188,251,238,135,240,120,60,184,125,251,166,197,39,153,100,1,45,76,122,252,2,193,47,51,232,146,95,196,200,133,174,147,242,124,
  232,128,42,104,80,69,13,138,168,66,20,4,147,18,213,53,29,74,146,58,39,91,57,71,162,32,160,177,161,25,217,217,57,24,28,236,133,44,137,152,
  155,157,68,91,91,39,218,218,58,48,54,58,76,254,70,48,152,21,136,220,90,72,203,69,105,233,168,40,0,162,32,114,37,239,146,36,65,150,36,120,
  100,25,146,40,66,100,6,156,72,39,41,0,65,36,201,200,126,159,23,251,247,31,71,97,97,49,54,54,66,136,132,55,224,145,101,64,0,186,186,246,
  99,255,254,35,220,253,87,84,84,163,162,162,26,25,25,25,232,123,208,139,104,52,140,153,153,41,148,150,150,35,55,39,27,209,88,204,92,50,219,219,
  218,1,0,19,19,35,240,121,189,200,202,202,196,219,239,252,0,133,133,197,220,123,238,217,211,141,170,202,26,124,250,233,175,205,16,165,32,8,200,203,
  203,67,117,117,141,249,186,112,120,3,126,159,31,0,80,94,94,137,210,210,74,243,223,42,43,107,80,89,89,131,88,44,142,209,145,17,226,88,45,18,
  102,207,94,110,107,21,108,208,130,92,152,223,83,45,247,2,144,186,76,248,185,28,168,216,186,236,255,9,149,62,191,84,0,73,119,130,35,10,182,217,
  83,186,219,186,254,52,43,241,216,60,65,77,211,1,42,55,194,148,135,211,50,237,237,108,44,169,238,223,237,103,183,251,127,54,85,136,140,174,21,173,
  223,50,148,223,205,3,28,59,223,183,146,208,208,241,216,36,52,88,153,18,81,16,32,137,150,196,140,36,138,240,200,30,156,122,243,29,52,53,181,32,
  188,177,129,51,103,62,195,196,196,152,165,19,167,145,146,126,42,137,130,45,68,140,93,153,37,65,128,174,9,46,218,75,182,240,27,248,226,28,1,2,
  64,165,109,140,110,19,37,193,148,33,88,91,93,197,87,95,125,129,80,40,136,131,7,143,226,205,55,223,65,48,24,196,240,208,32,116,145,172,195,186,
  166,25,125,206,72,5,232,155,247,149,27,56,183,23,14,217,159,235,203,8,152,94,200,28,37,250,176,72,204,149,128,35,69,33,229,156,201,164,81,69,
  160,235,80,84,13,170,162,154,172,15,12,112,82,87,223,68,152,153,217,73,8,162,136,96,112,5,225,112,8,229,229,85,40,45,45,195,226,210,2,4,
  213,162,210,183,115,58,33,9,226,146,201,144,200,178,12,81,18,201,23,195,142,252,240,163,127,177,233,251,93,191,118,193,120,15,1,62,159,31,157,157,
  251,144,72,36,112,255,254,45,76,78,12,65,215,129,218,186,102,180,181,117,162,179,179,27,67,131,125,16,4,1,227,227,35,168,170,170,193,158,61,123,
  113,231,206,77,115,51,105,106,110,69,40,20,196,226,226,28,188,94,25,93,93,251,81,88,88,140,197,197,57,92,187,246,61,148,100,18,178,236,193,190,
  238,163,168,174,174,199,190,238,3,232,189,119,219,188,247,194,194,98,76,76,140,226,194,249,175,205,251,172,169,109,0,0,148,150,86,226,238,221,27,232,
  123,112,23,154,166,161,171,235,32,118,119,238,195,235,175,159,194,228,196,184,17,250,20,77,70,41,221,210,237,97,89,20,58,127,89,246,150,204,61,209,
  161,118,255,244,89,21,158,93,32,235,141,102,124,231,89,8,123,129,195,86,74,246,110,34,173,214,189,227,185,188,127,81,116,87,137,6,132,109,152,190,
  62,157,113,101,229,75,90,41,9,93,123,247,97,87,71,39,18,137,4,206,156,249,12,147,147,227,220,190,99,62,95,27,72,50,255,141,69,29,230,45,
  91,32,130,140,97,29,16,173,113,187,197,102,103,253,40,144,255,99,195,102,118,101,75,85,85,112,229,242,5,120,60,94,236,221,187,31,111,188,241,22,
  22,230,231,177,190,190,70,100,89,52,209,172,22,166,172,146,219,24,116,27,123,60,19,38,186,140,121,39,123,248,50,1,166,23,82,71,137,125,0,154,
  46,64,213,84,136,154,104,232,37,177,57,74,42,52,99,147,214,141,7,157,151,151,135,186,186,70,172,174,46,99,117,45,64,64,140,0,140,143,15,97,
  247,238,253,168,111,104,194,242,202,146,249,30,162,100,45,62,162,36,64,80,97,106,107,136,58,13,181,9,70,124,155,73,12,148,173,73,8,97,235,62,
  94,88,152,69,79,207,117,44,45,45,152,235,92,91,123,7,188,94,31,238,223,191,133,169,169,97,162,235,1,96,122,122,4,130,0,116,119,31,67,91,
  123,135,17,134,235,195,161,67,199,208,208,216,140,251,15,238,64,215,129,170,170,26,228,228,228,226,254,253,59,198,53,137,104,106,110,71,56,28,194,149,
  43,223,144,33,47,9,208,116,5,55,110,156,67,65,65,17,154,155,219,209,215,119,151,235,227,91,183,174,146,126,132,33,146,102,252,219,200,72,63,250,
  250,122,168,30,27,110,223,185,138,130,66,146,96,88,90,86,134,233,233,105,8,70,74,151,83,5,197,58,57,233,208,157,167,40,23,165,218,205,244,84,
  158,79,66,105,107,117,164,39,173,17,243,162,130,36,29,108,229,37,153,115,172,209,40,11,24,172,5,252,89,133,100,5,102,79,211,205,60,71,154,30,
  0,86,156,85,35,247,98,158,196,5,247,252,22,243,254,141,51,150,32,178,66,173,246,62,176,192,209,179,189,127,157,3,67,154,166,67,16,173,62,32,
  255,77,94,70,132,170,9,235,166,155,243,221,57,238,245,109,252,111,103,99,139,252,159,32,130,209,74,18,144,155,151,131,131,7,143,64,146,36,156,63,
  127,22,147,83,99,230,186,70,193,17,17,26,214,13,227,104,187,125,14,28,160,138,246,135,201,196,104,148,169,17,33,10,124,158,171,57,6,4,102,201,
  19,88,182,198,73,254,144,168,138,6,81,16,205,160,154,170,169,184,118,237,34,138,139,75,80,85,85,131,67,135,143,224,155,111,206,88,218,80,186,96,
  172,199,58,119,111,174,128,82,208,173,190,98,132,134,121,87,6,42,211,98,137,35,155,96,88,192,75,3,152,94,106,121,0,147,69,98,18,246,104,18,
  119,65,65,17,126,244,163,127,229,248,155,230,230,118,244,244,92,55,227,192,146,36,153,239,69,153,34,93,231,147,171,37,73,68,101,101,37,254,226,47,
  254,181,227,253,126,249,203,191,130,170,38,205,255,254,221,239,254,27,18,137,56,4,65,64,77,77,3,186,186,14,66,215,129,225,225,126,44,47,47,113,
  20,167,207,71,114,150,118,239,222,143,221,187,247,187,222,163,207,231,55,170,225,4,204,205,205,160,174,174,17,249,249,5,8,133,214,209,208,64,152,179,
  201,201,81,136,162,0,191,223,15,175,215,11,175,215,139,143,62,250,11,215,247,75,38,19,102,101,25,109,36,62,175,49,98,144,100,224,175,172,44,155,
  215,75,251,100,126,126,6,149,149,213,200,203,203,199,204,204,12,151,203,149,110,233,182,179,19,63,152,60,18,171,152,130,50,1,118,157,26,39,251,251,
  164,81,131,83,1,220,206,170,208,124,71,154,68,203,74,113,184,233,218,184,121,68,178,122,62,244,190,233,239,220,78,254,207,250,254,89,70,137,178,22,
  68,227,71,55,117,125,0,221,53,247,234,105,140,47,59,51,39,138,34,118,237,234,68,97,97,17,230,230,102,140,68,104,1,130,64,238,199,231,243,35,
  59,59,27,139,139,139,230,90,38,203,50,242,242,242,177,186,186,138,100,50,97,222,123,102,102,38,50,50,252,88,89,89,53,165,93,232,243,161,225,226,
  130,130,2,20,23,151,64,146,36,204,205,205,32,18,137,152,215,151,153,153,133,204,204,76,44,47,7,72,202,2,195,18,214,214,214,33,47,47,15,129,
  192,18,150,150,22,205,49,83,91,91,135,236,236,28,172,172,4,176,180,180,136,104,52,138,155,55,175,161,180,180,28,93,93,251,112,227,198,53,172,174,
  174,64,146,88,239,70,235,254,82,1,74,251,220,179,172,194,44,194,139,181,8,163,102,188,128,230,176,250,122,209,1,211,75,4,148,82,83,216,34,115,
  26,107,109,221,189,233,187,120,189,62,116,116,236,69,111,239,109,136,34,31,159,37,34,99,252,64,162,19,129,13,175,217,175,139,85,214,86,20,213,16,
  7,19,48,58,58,132,181,181,85,156,58,245,33,78,156,120,19,177,88,20,243,243,51,59,186,107,143,199,107,190,255,228,228,8,234,234,26,81,87,215,
  136,7,15,122,80,91,219,136,149,149,0,86,86,2,228,4,178,13,133,111,143,199,11,143,199,139,152,145,231,68,254,206,80,45,23,4,211,105,158,61,
  61,111,205,0,186,177,129,110,201,139,122,138,191,113,255,158,110,47,47,72,98,101,44,120,109,27,150,85,34,223,139,139,75,144,151,151,111,58,179,63,
  171,166,170,10,214,214,214,176,178,178,140,68,34,9,93,215,152,234,84,122,176,210,82,138,0,218,215,44,246,128,199,110,84,244,119,130,64,14,63,5,
  5,133,200,205,205,51,15,117,207,170,197,227,49,172,174,174,98,101,101,217,44,166,33,161,71,193,144,11,209,77,176,200,230,65,62,109,22,204,94,241,
  150,153,153,137,166,166,102,0,192,237,219,55,160,105,42,247,218,189,123,187,209,213,213,141,63,252,225,55,152,157,157,129,32,136,40,40,40,192,135,31,
  126,140,207,63,255,20,115,115,115,16,13,150,112,215,174,14,180,183,239,194,175,127,253,79,136,70,163,142,207,205,206,206,196,143,126,244,83,108,108,132,
  16,141,70,113,236,216,9,220,184,113,13,131,131,164,0,230,240,225,163,104,107,219,133,95,255,250,239,177,188,76,250,209,239,247,227,181,215,222,68,65,
  65,17,194,225,16,234,235,27,112,230,204,23,240,120,60,120,253,245,83,200,205,205,67,52,26,65,117,117,45,206,159,255,22,201,100,2,83,83,99,152,
  159,159,69,77,77,29,218,219,59,112,229,202,37,0,26,3,146,4,87,121,21,251,28,36,243,141,20,37,101,100,100,64,85,147,230,223,178,138,227,180,
  226,92,20,85,227,121,235,166,150,222,203,0,152,94,10,160,196,62,140,205,54,241,202,74,18,138,154,155,155,198,215,95,127,230,48,208,173,171,107,196,
  43,175,188,141,154,154,6,244,246,222,182,81,217,52,15,137,110,242,50,0,197,252,188,249,249,57,252,251,127,255,191,24,127,35,154,226,136,116,129,163,
  131,35,145,80,16,143,43,230,103,207,207,207,227,222,189,91,56,112,224,24,14,31,126,5,159,125,246,27,131,113,2,226,241,184,49,121,175,96,106,106,
  152,89,144,53,36,147,10,146,73,5,138,162,66,81,200,201,99,100,100,24,221,221,71,208,220,220,142,80,40,8,175,215,135,241,241,59,80,140,60,45,
  77,139,32,145,136,35,153,76,224,235,175,127,207,45,206,84,70,129,42,108,83,153,0,10,238,136,234,54,169,70,163,139,92,86,86,54,231,93,164,105,
  164,15,1,96,113,113,222,161,235,68,147,60,217,42,68,178,153,104,38,32,99,193,153,251,115,214,28,84,119,186,189,92,96,201,58,205,242,66,128,172,
  228,70,89,89,57,58,59,187,80,95,223,128,236,236,28,200,178,231,153,107,9,17,6,69,65,36,18,193,244,244,36,122,123,239,98,106,106,210,212,94,
  163,96,105,51,17,64,22,36,81,189,49,34,248,106,173,37,30,143,7,245,245,13,232,232,216,131,170,170,106,248,124,126,211,86,232,217,222,63,177,232,
  88,95,95,195,232,232,48,122,123,239,97,101,101,217,188,111,75,49,90,115,217,152,159,62,147,68,191,242,242,242,80,89,89,109,228,114,206,59,24,251,
  138,138,74,104,154,134,246,246,14,204,207,207,146,141,83,150,145,147,147,11,143,199,195,137,224,250,253,132,125,98,181,226,0,203,223,211,235,245,33,43,
  43,27,151,47,159,199,252,252,28,118,239,238,196,43,175,188,142,241,241,17,228,228,228,160,180,180,12,62,159,15,117,117,245,88,91,91,133,174,235,104,
  107,219,133,234,234,90,124,246,217,31,176,177,17,52,14,199,42,118,239,222,143,210,210,50,124,241,197,39,136,70,163,240,122,61,38,187,165,170,26,198,
  198,70,80,83,83,135,250,250,6,92,189,122,137,27,131,91,235,0,90,140,146,199,67,220,37,142,31,63,129,242,242,10,92,185,114,222,0,195,186,173,
  250,92,69,50,41,190,148,128,73,126,209,211,32,216,74,4,98,130,171,49,155,170,177,65,171,100,175,110,111,239,4,0,60,120,208,139,120,44,233,56,
  201,13,14,14,98,223,190,35,40,44,44,70,126,126,17,150,151,151,184,13,89,36,72,1,144,100,64,87,160,235,18,4,104,22,253,168,82,213,110,13,
  186,38,66,147,36,136,162,6,141,3,74,73,36,226,73,78,253,250,198,245,107,40,47,175,70,85,85,13,186,187,143,224,210,197,239,32,136,2,102,166,
  167,176,119,239,33,194,130,233,58,198,39,134,0,93,135,207,151,133,99,199,142,161,188,188,10,223,124,243,37,134,135,6,140,133,85,199,216,216,8,246,
  236,233,198,43,175,188,133,68,34,142,190,190,94,40,73,3,4,73,34,230,230,166,81,87,215,132,3,251,95,193,157,59,151,137,223,144,166,161,165,165,
  19,93,93,7,177,178,18,192,63,27,149,111,244,222,149,164,10,37,169,26,57,19,186,89,69,216,209,177,15,241,120,2,15,30,220,129,174,233,120,245,
  149,183,80,84,84,130,165,165,69,172,174,172,66,51,42,14,117,186,46,10,228,111,117,73,227,92,222,85,197,0,74,146,102,146,77,138,162,57,24,37,
  77,211,200,51,214,232,172,195,139,95,245,182,147,215,189,236,108,18,99,120,77,77,175,89,195,107,73,146,144,95,144,143,163,71,143,99,207,158,125,144,
  101,25,154,166,33,145,136,99,109,109,21,241,120,236,153,130,36,175,215,135,220,220,60,228,231,23,160,176,176,8,93,93,221,24,28,232,195,249,11,223,
  99,105,113,209,170,239,52,159,175,6,141,228,246,90,185,191,140,153,182,40,8,228,254,69,17,146,72,250,161,186,166,22,39,79,190,138,250,250,70,243,
  128,19,139,69,177,182,182,130,100,50,249,108,192,146,145,32,147,145,145,137,236,236,28,148,149,85,160,188,188,18,7,14,28,198,237,91,55,112,237,218,
  21,132,195,97,222,100,150,169,0,20,4,241,225,231,199,142,170,222,104,229,44,107,126,11,228,231,21,64,150,101,204,207,207,34,18,14,147,245,197,104,
  85,85,181,0,128,75,151,206,97,255,254,195,240,200,30,40,138,2,81,16,160,40,73,163,122,206,2,74,208,117,40,138,2,1,34,68,65,100,64,160,
  97,40,110,252,93,44,26,69,60,22,197,216,232,8,14,31,58,14,159,215,139,178,210,10,196,99,49,220,184,126,5,141,141,205,232,239,239,67,50,17,
  39,21,212,30,15,68,1,72,196,227,72,196,227,70,114,56,249,189,174,3,177,104,20,9,115,252,147,49,48,59,51,13,77,211,144,159,95,128,140,140,
  12,108,132,194,164,50,156,171,62,118,130,22,65,128,113,173,198,184,51,140,231,51,51,50,208,220,220,138,154,154,90,12,15,247,163,239,193,61,172,175,
  175,34,169,40,80,21,114,120,151,37,133,1,76,42,68,65,227,1,19,179,55,191,72,85,114,47,15,163,164,105,208,52,209,220,196,9,112,177,232,93,
  175,55,3,213,213,53,8,6,215,49,208,223,103,149,120,154,67,139,160,254,190,7,247,112,248,200,9,52,55,181,99,113,97,193,124,143,186,186,70,252,
  229,191,254,191,185,126,254,197,11,196,243,204,138,209,90,12,11,65,241,162,57,249,226,49,5,177,88,194,102,19,34,226,236,55,103,240,243,63,253,11,
  180,181,117,96,122,106,2,163,163,195,88,88,152,199,173,91,87,177,127,255,17,116,239,63,134,238,253,199,184,207,157,153,153,194,224,64,191,153,192,46,
  138,2,134,134,6,176,103,79,55,0,96,122,122,10,161,208,134,213,15,138,128,115,223,159,197,143,126,92,130,234,154,122,84,215,212,59,238,229,198,141,
  203,156,231,29,97,148,12,31,55,81,128,168,9,38,219,51,59,59,141,238,238,195,232,238,62,204,208,239,113,124,247,237,215,38,21,75,99,215,86,204,
  95,55,126,79,65,44,153,96,228,250,37,115,93,164,191,99,39,146,245,126,155,11,179,165,219,139,27,114,179,242,34,248,47,89,150,208,220,210,138,83,
  167,222,65,65,65,33,98,177,40,198,199,71,49,56,216,135,153,233,73,4,131,33,135,126,207,86,155,102,170,235,176,3,160,173,174,219,60,72,137,2,
  178,178,179,81,94,94,137,230,230,86,212,215,55,162,181,109,23,170,107,106,241,221,183,223,224,222,189,30,199,186,69,153,37,123,216,141,101,146,36,73,
  130,215,235,193,161,195,71,112,242,228,235,144,101,25,193,224,58,70,70,134,48,60,52,136,133,133,121,68,163,17,171,242,74,223,222,245,63,234,243,98,
  22,79,2,238,36,17,121,121,249,168,173,173,71,91,251,46,84,86,86,227,232,177,147,168,173,107,192,55,223,156,198,244,212,52,119,77,102,222,163,185,
  185,63,173,60,37,48,146,10,2,114,114,115,0,0,161,224,186,233,154,64,91,117,85,13,2,75,139,24,28,232,195,190,125,7,80,91,87,143,241,241,
  81,146,24,205,228,241,152,201,235,2,56,7,4,118,108,176,227,60,55,47,31,138,170,96,127,55,57,156,234,186,134,154,154,90,44,46,206,225,206,157,
  91,104,111,223,141,130,252,124,44,45,45,98,104,120,16,53,181,117,248,193,15,127,130,251,247,239,226,193,131,94,132,195,27,232,239,235,69,85,85,53,
  126,104,252,126,160,255,62,162,209,136,25,77,9,133,130,72,36,18,240,120,60,200,201,206,65,36,28,33,225,52,81,132,46,106,134,110,29,101,123,88,
  189,64,107,206,201,178,8,143,87,134,207,39,67,52,52,5,125,62,63,118,239,222,135,150,150,93,24,25,233,199,232,200,0,150,150,22,145,76,42,72,
  36,72,164,67,150,41,96,162,246,86,20,48,177,235,247,139,195,48,201,47,122,101,141,166,235,16,116,141,152,202,107,42,146,138,101,50,152,84,20,243,
  33,116,119,116,194,231,243,227,250,245,171,136,198,18,102,8,135,157,237,162,40,226,202,213,203,216,187,239,0,234,27,154,112,233,242,121,243,253,182,186,
  6,197,168,184,163,0,67,84,173,92,10,81,20,161,25,159,149,84,18,136,39,18,76,101,39,137,3,47,46,5,112,227,198,85,156,56,241,42,14,29,
  62,142,254,129,62,40,170,130,27,55,174,34,26,141,160,189,189,211,20,156,140,199,227,184,115,135,156,212,84,179,210,143,44,174,211,211,211,88,90,90,
  64,73,73,25,6,6,250,12,239,31,11,180,41,170,130,95,255,250,151,56,118,236,85,180,183,91,249,90,179,179,83,184,114,229,34,22,23,231,77,64,
  162,155,215,172,32,169,40,230,181,42,6,240,154,158,158,192,204,204,36,14,29,58,110,0,183,73,92,188,120,30,179,179,51,72,42,10,20,85,133,98,
  188,151,32,8,128,166,19,217,5,38,180,166,168,26,18,6,40,18,68,217,60,66,82,81,75,118,2,41,138,130,164,162,146,190,214,212,23,168,234,205,
  249,51,255,42,60,150,138,158,103,13,116,182,203,188,164,106,162,233,214,110,84,149,74,2,68,163,90,115,119,103,39,222,125,247,67,120,189,62,76,79,
  79,226,234,213,75,152,152,24,117,148,222,187,9,31,166,98,175,28,128,73,223,241,3,182,142,89,2,160,107,2,66,161,16,54,54,6,48,50,50,136,
  138,138,42,28,56,112,4,173,173,237,248,224,195,143,144,149,157,133,75,151,46,16,119,119,93,135,168,11,208,169,174,141,192,108,226,70,133,18,189,127,
  175,143,228,162,28,58,116,20,186,174,227,222,189,30,92,187,118,9,203,203,1,166,4,223,186,119,187,38,207,147,2,27,246,112,169,170,9,8,44,7,
  176,188,178,140,158,187,183,208,222,222,129,195,135,143,163,178,178,10,31,127,252,39,248,236,179,63,96,108,108,12,146,169,175,36,50,247,12,163,202,10,
  142,113,175,167,156,75,59,155,35,60,17,101,253,143,10,253,198,226,113,131,241,32,239,233,247,103,160,188,162,10,203,203,75,104,110,105,133,36,73,104,
  105,105,195,248,248,40,15,18,69,18,217,32,247,192,144,58,130,238,24,42,138,170,64,146,36,116,118,118,33,26,141,64,20,69,156,253,246,12,60,94,
  31,106,106,235,160,168,10,26,26,155,225,243,251,81,91,215,128,197,165,69,196,98,17,124,241,197,167,232,232,216,131,189,123,247,163,190,190,9,159,127,
  254,59,108,108,108,224,179,207,126,135,206,206,189,232,234,218,143,186,186,6,124,249,229,39,70,126,169,142,164,146,128,162,36,33,8,100,12,145,74,109,
  50,246,36,72,38,54,229,211,28,140,188,36,99,222,201,70,216,205,227,149,29,57,112,94,175,15,187,118,237,69,99,99,27,198,198,6,48,54,54,132,
  133,133,57,36,18,146,241,165,64,146,21,40,138,104,2,38,8,2,4,205,58,240,130,145,203,112,171,0,77,51,74,143,145,242,166,113,82,69,1,54,
  54,194,248,119,255,238,255,237,152,208,23,47,94,192,197,139,23,96,149,239,106,156,181,9,77,12,85,85,9,255,241,63,254,7,243,4,176,177,177,129,
  255,237,127,251,255,184,250,162,81,144,66,7,1,203,120,176,21,21,146,36,226,175,255,250,255,180,66,77,54,163,91,85,21,32,73,26,46,93,186,128,
  203,151,47,58,98,196,183,110,221,196,237,219,183,204,223,211,152,112,210,16,212,100,243,27,36,73,192,223,254,237,95,155,159,69,251,134,2,51,58,104,
  207,156,249,2,223,124,115,154,235,67,74,143,210,215,255,205,223,252,21,179,17,81,147,94,1,247,239,247,98,96,160,207,76,40,189,118,237,138,201,20,
  17,246,73,49,243,154,88,192,165,105,48,174,23,248,230,155,175,241,237,183,223,112,42,181,241,120,2,255,235,255,250,239,28,85,114,127,243,55,255,149,
  75,24,228,21,215,211,172,210,243,2,144,220,242,77,236,160,37,213,233,145,103,149,68,206,6,168,161,161,9,111,191,253,62,188,94,31,238,222,189,141,
  11,23,190,71,44,22,113,41,207,182,114,20,157,195,194,154,111,172,174,215,118,180,210,82,129,60,214,2,136,190,23,77,234,21,69,96,110,110,6,95,
  124,241,9,86,86,2,56,122,244,36,94,125,245,77,68,34,17,220,185,115,203,150,79,9,219,28,166,185,88,132,81,58,118,236,36,14,29,58,138,100,
  50,137,239,191,255,198,72,56,182,236,134,84,213,42,89,7,220,11,34,158,14,88,18,152,194,22,1,125,125,247,49,59,59,131,83,167,222,69,115,115,
  43,62,252,240,99,252,250,215,127,143,249,249,121,227,222,249,132,238,39,29,54,228,223,159,181,3,17,205,48,38,251,76,74,74,74,145,145,145,1,77,
  211,80,90,90,142,181,181,85,148,151,87,114,85,193,172,70,20,59,14,232,231,217,11,84,68,145,132,164,250,251,251,48,59,59,133,245,245,85,0,2,
  154,154,90,224,241,120,145,72,36,81,90,90,142,213,213,21,52,52,52,225,198,141,43,198,248,86,209,219,123,7,131,131,125,248,197,47,254,18,29,29,
  93,184,114,229,34,52,77,193,157,59,183,48,56,216,143,95,252,226,95,161,173,109,55,238,220,185,97,235,83,221,188,79,138,117,44,167,4,1,154,198,
  219,69,81,211,120,154,155,228,241,16,70,147,117,78,96,155,207,231,71,123,251,94,212,214,54,97,122,122,12,195,195,3,88,92,156,67,60,174,32,145,
  72,34,153,84,32,73,10,146,73,17,162,168,26,14,26,116,190,106,70,158,222,243,189,142,191,20,161,55,171,147,105,162,47,143,144,237,11,38,111,38,
  75,7,8,77,184,212,32,73,170,89,6,105,149,36,243,19,141,44,84,22,139,68,64,129,206,108,226,130,81,182,175,155,201,156,44,120,225,175,95,48,
  128,128,232,98,203,192,11,204,209,201,73,65,141,29,40,41,138,192,129,15,242,121,60,80,162,118,47,86,185,191,179,95,120,137,126,141,97,221,104,66,
  170,198,249,214,81,134,142,130,37,10,28,89,74,90,20,97,254,78,215,53,87,209,181,84,139,37,223,183,90,26,32,61,103,128,41,149,165,134,197,112,
  8,41,45,45,40,155,100,149,193,11,86,78,82,126,46,222,125,247,3,248,253,25,184,123,247,54,190,253,246,43,75,125,159,171,130,212,93,170,34,245,
  20,188,130,96,215,234,123,232,67,26,191,241,234,140,223,27,121,127,85,85,113,233,210,121,104,154,134,227,199,95,197,235,175,159,194,210,210,34,166,167,
  167,152,131,154,104,174,69,118,41,128,150,150,54,28,61,122,2,0,240,205,55,95,226,222,189,30,198,227,146,63,52,108,229,21,246,100,195,166,214,90,
  69,158,51,1,123,235,235,107,248,226,139,79,241,254,251,63,68,115,115,43,222,122,235,61,252,246,183,255,132,141,141,48,247,60,54,83,87,127,50,97,
  55,39,136,102,229,27,0,160,190,190,17,243,243,179,248,230,155,211,80,85,21,57,57,185,248,249,207,255,28,53,53,245,88,90,90,52,14,185,10,103,
  67,69,251,67,81,146,92,8,152,252,155,245,124,22,23,231,177,180,180,100,62,239,214,214,93,24,28,236,199,249,243,223,2,0,10,10,10,241,167,127,
  250,23,200,207,39,190,159,244,128,28,139,197,160,170,138,201,184,211,245,57,30,143,27,64,79,117,25,219,86,245,164,29,36,17,160,228,100,148,100,89,
  54,109,180,8,88,146,83,2,37,218,50,51,179,209,218,186,7,85,85,245,152,157,157,68,127,255,93,44,45,45,25,5,76,73,136,98,210,102,40,175,
  130,232,110,225,185,247,159,123,73,114,148,40,75,68,7,186,176,201,107,193,37,18,91,19,136,252,29,15,108,172,197,219,205,204,144,53,146,101,153,23,
  139,117,98,95,187,89,28,150,138,118,233,41,124,173,4,71,133,30,11,24,216,183,228,95,199,211,241,180,242,68,211,68,168,170,232,240,28,226,195,24,
  58,247,30,230,208,86,5,19,80,178,213,35,108,30,81,42,71,108,50,33,232,189,146,141,101,179,83,170,219,51,102,21,127,211,237,217,179,73,169,84,
  179,105,254,7,221,68,216,188,50,59,24,183,235,157,81,160,32,138,2,14,28,56,130,130,130,66,76,79,79,226,194,133,239,13,144,164,187,74,74,88,
  215,69,231,132,128,212,178,33,169,89,176,77,238,122,203,156,10,190,202,147,28,14,0,1,215,175,95,65,97,97,17,218,219,119,227,240,225,163,88,92,
  92,128,170,198,141,247,208,184,235,160,18,8,57,57,57,56,114,228,24,100,89,198,181,107,151,13,144,68,242,32,9,72,82,205,156,15,139,73,123,22,
  85,0,252,26,37,73,68,71,138,92,143,128,104,52,130,239,191,255,6,249,249,36,127,169,163,163,19,55,110,92,183,173,45,2,23,126,124,54,251,136,
  181,182,102,103,231,160,178,178,6,61,61,183,145,72,144,244,133,213,213,85,204,204,76,163,169,169,5,83,83,83,0,128,247,222,251,33,194,225,13,120,
  60,30,220,184,113,13,193,96,16,197,197,197,248,211,63,253,115,195,49,34,137,75,151,46,96,114,114,210,4,59,100,156,203,80,20,21,162,40,32,39,
  39,11,229,229,21,248,252,243,79,140,10,101,29,75,75,75,88,89,89,70,99,99,51,230,231,231,241,218,107,111,96,117,117,21,185,185,185,8,6,131,
  184,115,231,22,218,218,58,112,224,192,97,172,175,175,33,47,47,15,203,203,1,244,246,222,53,14,173,118,150,200,98,250,44,83,113,1,118,115,119,50,
  254,36,206,111,148,122,150,110,183,101,101,229,160,165,101,55,170,171,235,49,49,49,130,222,222,59,88,94,94,182,29,158,216,42,117,253,185,215,218,123,
  105,116,148,232,164,163,58,63,110,200,212,105,100,200,255,61,139,106,73,56,149,119,107,102,39,177,219,162,239,102,110,104,9,174,185,135,4,44,250,211,
  210,24,113,11,85,240,64,205,41,159,239,182,1,56,147,90,117,83,72,147,8,192,185,51,110,110,33,19,246,122,52,205,189,111,120,64,147,26,236,144,
  123,214,55,101,144,82,253,109,106,176,153,110,207,6,48,217,237,32,68,51,36,235,220,128,44,81,58,154,216,79,231,155,155,152,100,65,65,1,246,239,
  63,132,68,34,129,27,55,174,34,22,139,114,128,72,146,36,243,100,77,194,22,9,88,198,159,58,60,30,15,20,69,133,44,203,72,38,147,156,89,52,
  53,66,85,85,5,94,175,223,56,45,147,191,139,70,163,16,4,146,131,33,203,18,84,85,55,62,91,51,55,116,159,207,7,143,199,7,85,85,16,139,
  17,65,86,143,199,3,89,246,26,247,99,228,212,37,147,6,147,170,224,202,149,11,168,175,111,68,91,91,7,238,220,185,133,145,145,17,227,80,39,114,
  160,145,246,65,125,125,35,106,106,234,176,180,180,128,155,55,175,25,97,123,221,96,177,85,7,139,109,151,68,217,201,60,217,73,34,187,243,48,103,1,
  29,202,40,73,18,253,189,4,65,208,177,188,28,192,173,91,55,240,206,59,31,224,240,225,99,120,240,160,23,145,72,132,59,212,62,91,83,99,254,240,
  27,139,69,241,245,215,95,26,73,202,170,201,138,158,61,251,53,50,51,51,17,137,68,241,135,63,252,22,153,153,89,230,24,94,92,92,64,36,18,193,
  175,126,245,143,166,166,151,170,106,8,4,150,77,214,102,125,61,136,223,254,246,87,8,4,2,6,235,46,32,18,137,224,247,191,255,45,102,103,167,205,
  195,175,32,8,248,244,211,79,32,203,18,86,86,86,160,105,58,10,10,10,17,12,174,99,124,124,20,137,68,18,253,253,253,136,68,34,200,205,205,71,
  40,180,110,248,210,233,54,214,198,154,123,100,94,242,6,229,100,207,19,28,140,18,157,31,44,88,114,147,108,217,172,101,100,100,161,189,189,11,117,117,
  77,24,26,234,195,189,123,61,88,90,90,98,198,172,200,68,49,128,231,185,204,247,133,150,7,176,74,138,5,211,120,86,100,184,85,1,78,211,62,106,
  4,40,184,77,72,179,234,220,41,138,104,143,199,187,46,40,186,245,25,220,107,244,212,0,201,188,102,250,75,10,8,116,22,32,184,153,111,166,0,83,
  246,127,115,189,71,221,180,72,176,104,103,102,114,9,2,4,240,167,84,214,172,145,30,72,136,45,12,57,57,11,96,158,131,49,249,76,219,19,221,254,
  94,188,73,193,163,135,61,158,87,244,142,205,75,154,31,163,225,231,51,11,185,25,39,82,106,44,234,245,122,32,123,100,152,110,157,198,141,170,26,169,
  66,85,53,13,146,104,132,102,169,204,131,97,80,42,81,176,100,72,3,116,236,234,132,199,227,193,200,240,16,70,71,134,153,67,130,142,252,252,66,156,
  124,229,53,92,185,114,9,225,141,13,124,248,131,143,113,231,246,77,12,14,246,67,135,142,140,204,108,188,254,198,41,92,191,118,25,199,142,191,130,251,
  189,119,49,50,50,100,110,6,71,143,28,7,4,1,51,211,147,56,245,214,187,136,199,227,208,117,32,30,139,226,203,211,159,161,162,162,18,199,143,191,
  98,138,6,6,2,75,184,122,229,18,98,177,40,234,235,27,177,255,192,33,120,189,4,156,93,186,120,14,11,11,11,120,255,253,143,144,159,151,143,68,
  34,1,217,227,193,189,123,119,112,239,222,29,104,36,135,21,171,43,43,184,123,247,14,14,31,62,134,230,230,86,140,141,142,65,19,116,136,130,70,146,
  186,141,30,19,5,17,94,143,23,187,140,98,139,158,59,183,17,10,6,77,185,13,77,85,161,42,26,249,210,52,211,86,232,81,146,184,133,77,108,150,
  220,242,189,156,235,161,113,56,213,116,232,34,89,7,85,128,51,195,30,24,232,195,158,61,123,81,81,81,133,134,134,38,60,184,223,107,91,152,4,56,
  52,16,183,51,119,30,102,62,186,252,55,107,124,27,141,196,49,185,49,105,133,54,117,242,132,86,150,87,176,178,76,66,97,19,227,227,142,195,128,14,
  96,99,99,4,72,193,122,39,212,4,38,198,39,185,125,32,188,17,193,88,120,140,239,108,65,192,226,194,130,121,169,3,253,253,220,90,175,235,58,148,
  164,130,7,15,250,72,41,63,163,57,38,64,4,68,98,151,67,64,171,140,130,252,66,114,125,84,203,78,215,204,249,168,107,206,125,201,170,120,243,192,
  235,149,225,243,122,225,51,204,208,119,218,50,50,178,208,213,117,16,205,205,187,112,247,238,45,244,222,187,139,165,165,101,34,57,67,153,173,109,24,245,
  166,25,165,135,164,253,233,9,76,20,69,82,37,98,179,52,96,79,89,154,78,141,12,133,148,97,30,123,88,206,141,77,177,22,10,129,11,25,108,135,
  25,97,223,135,141,233,243,158,63,90,74,182,200,206,90,109,150,27,178,213,162,199,189,135,217,119,96,66,38,26,241,97,210,116,6,195,233,188,99,185,
  96,245,185,221,119,139,205,119,34,116,179,110,158,202,220,242,180,210,44,209,139,203,38,73,70,254,131,199,35,161,160,160,0,135,14,31,198,220,236,52,
  38,39,199,25,177,57,141,79,246,87,69,72,70,216,200,156,11,146,104,130,37,106,232,172,105,26,70,199,134,57,166,85,215,1,143,199,131,242,242,74,
  248,124,62,40,201,36,138,139,75,240,202,171,111,24,150,16,97,248,253,30,84,84,84,152,39,236,221,157,93,24,27,31,129,0,178,9,236,233,218,135,
  239,190,253,26,5,133,133,144,101,15,206,159,255,14,137,120,156,84,203,38,18,168,174,174,133,40,138,248,246,236,87,200,202,206,198,91,111,189,7,77,
  83,113,243,198,53,156,122,235,29,220,187,219,131,129,129,7,200,202,206,70,56,28,134,63,195,135,138,138,74,220,186,121,29,51,51,211,16,69,17,193,
  96,144,204,31,42,29,164,106,24,31,27,193,254,253,135,208,212,212,130,203,217,23,16,10,133,160,65,4,155,18,44,138,2,114,114,115,81,91,87,143,
  112,120,3,83,211,19,188,184,159,198,20,93,104,169,253,198,118,178,150,110,150,179,229,150,156,159,122,125,35,7,36,82,208,98,125,215,117,29,177,104,
  20,19,227,99,168,168,168,66,75,75,27,250,251,238,51,96,130,205,205,124,186,172,146,14,59,251,194,231,128,217,243,64,183,187,206,187,245,149,176,69,
  101,151,91,116,96,171,3,186,102,20,0,88,214,90,58,39,193,144,157,157,129,183,223,249,208,124,62,219,71,34,124,20,69,20,31,77,249,61,51,51,
  11,71,143,190,130,189,123,15,226,250,245,43,184,121,243,6,150,3,43,16,212,231,127,173,123,225,228,1,88,195,72,154,0,202,250,64,177,30,72,150,
  226,179,155,147,181,251,192,163,139,17,141,130,105,26,99,238,103,243,237,225,181,49,82,13,52,62,225,147,14,102,54,247,137,7,24,34,151,43,68,54,
  26,129,115,30,103,217,48,103,178,183,144,130,26,183,219,130,104,230,181,208,80,9,11,186,52,77,0,84,13,154,160,113,2,97,214,231,137,92,2,46,
  123,79,108,72,146,44,52,2,159,132,45,240,62,65,180,111,95,182,182,149,145,231,227,54,252,124,22,72,137,154,139,74,178,8,73,150,224,245,201,232,
  234,218,135,67,135,142,98,116,116,16,131,131,125,152,154,26,119,168,247,210,74,81,150,206,103,231,112,78,110,14,138,138,138,16,143,199,72,226,179,141,
  89,213,116,13,137,4,209,35,147,100,9,129,192,18,116,93,195,171,175,189,129,51,103,62,3,4,1,138,162,64,213,20,12,12,244,225,228,201,215,144,
  153,153,137,120,60,134,154,218,58,8,130,128,241,137,17,116,119,31,196,234,234,50,166,166,198,204,141,65,52,194,70,161,80,16,27,97,242,53,50,50,
  72,84,147,253,62,100,100,100,98,118,110,10,145,104,24,225,200,134,233,241,165,105,42,70,199,134,205,178,125,107,32,88,115,107,101,117,5,129,192,34,
  202,203,43,145,157,147,141,96,40,68,250,208,182,108,20,21,21,195,235,245,98,114,114,28,171,171,43,208,116,29,170,166,153,166,172,154,110,27,39,194,
  214,82,7,110,172,246,78,60,215,216,106,193,148,175,55,152,41,77,215,160,233,2,68,144,239,134,255,45,166,103,166,144,76,38,80,94,94,1,81,146,
  160,67,225,215,165,109,152,226,226,161,229,1,156,34,29,58,67,41,233,58,97,142,52,93,51,250,90,131,162,170,220,218,229,92,239,119,162,47,33,112,
  107,183,61,159,108,179,68,124,251,231,242,128,138,236,137,18,116,8,134,233,178,166,139,208,161,153,192,44,26,141,18,33,76,1,14,3,103,119,96,44,
  112,97,117,18,110,206,48,165,20,30,165,173,175,175,153,238,19,47,202,186,247,194,50,74,22,72,18,141,12,125,82,37,35,138,2,14,31,62,134,19,
  39,94,197,191,255,247,255,11,179,97,187,87,75,217,19,51,105,249,62,173,218,178,151,46,242,246,2,162,75,69,26,127,218,182,135,225,248,156,12,171,
  156,246,167,63,253,83,0,192,175,127,253,15,92,162,155,198,104,78,16,191,53,205,1,246,232,125,179,160,197,154,184,130,235,164,112,230,135,88,108,220,
  255,252,63,255,63,113,238,220,183,184,116,233,162,9,114,232,103,82,105,126,250,119,178,204,127,246,143,126,244,51,84,87,19,53,219,127,248,135,191,197,
  220,220,60,147,79,33,154,247,96,245,237,31,23,155,228,12,89,88,162,163,162,168,155,94,76,4,56,139,207,101,223,240,135,5,145,75,248,148,101,9,
  137,68,2,153,153,89,104,105,217,133,218,218,6,204,205,77,97,112,240,1,166,167,39,160,40,170,41,74,71,37,36,120,134,148,220,123,126,126,1,100,
  217,131,96,112,29,193,224,90,74,166,132,205,147,187,124,249,50,62,252,240,71,168,169,169,199,194,194,44,0,82,189,19,8,44,0,32,230,161,35,35,
  67,104,107,235,192,204,204,164,193,58,107,40,46,46,193,161,67,199,161,170,10,226,241,24,250,250,122,161,105,42,50,50,50,80,84,84,4,191,63,3,
  149,149,213,232,235,35,66,127,67,67,3,248,224,131,31,225,238,221,91,184,127,255,46,226,241,152,185,33,29,58,116,212,4,74,99,99,163,88,90,90,
  52,11,24,0,29,225,240,6,214,215,215,81,94,94,137,188,188,124,44,44,204,59,138,79,4,1,40,40,40,0,0,4,131,235,92,126,21,91,161,69,
  15,87,246,33,98,85,222,233,41,199,158,253,89,166,170,88,220,138,245,181,127,78,170,82,127,154,183,72,124,240,18,200,202,202,70,118,118,54,130,193,
  117,23,240,101,3,23,79,148,96,210,93,15,203,212,90,137,202,192,208,53,208,237,96,108,223,67,220,11,114,82,64,55,29,14,161,84,247,195,188,158,
  146,221,179,214,110,54,247,135,178,181,18,98,177,40,190,255,254,43,211,18,133,122,13,82,150,151,47,106,162,130,147,146,33,11,32,25,70,234,50,90,
  91,247,160,185,185,227,161,123,122,105,105,1,189,189,119,48,48,48,128,229,229,101,196,227,201,23,166,40,231,133,4,74,236,198,76,30,168,108,170,216,
  210,69,27,0,124,62,47,163,17,196,139,40,90,73,197,2,87,30,79,168,98,139,11,164,14,206,86,146,156,96,126,142,5,26,68,14,24,165,154,44,
  116,50,176,166,150,22,11,70,254,221,239,247,114,201,175,138,98,209,191,68,131,66,224,202,253,37,201,186,111,103,34,45,175,29,195,38,83,91,202,225,
  150,35,57,123,157,116,3,164,229,163,116,82,178,128,148,150,112,211,207,204,203,203,71,117,117,45,46,94,252,22,247,239,223,133,170,234,240,249,60,80,
  20,209,208,15,161,247,192,246,237,31,159,111,27,59,52,44,15,47,154,79,160,51,96,91,127,206,231,32,15,146,124,62,15,124,62,15,119,127,62,159,
  31,245,245,45,168,168,168,193,194,194,12,250,251,239,97,118,118,138,3,75,188,86,23,249,187,236,236,76,8,130,128,120,60,6,69,81,82,84,99,90,
  69,28,178,44,99,110,110,6,247,239,247,224,216,177,87,112,250,244,167,80,13,113,191,181,181,21,44,45,45,162,178,178,26,163,163,67,168,168,168,196,
  119,223,125,3,42,107,33,203,30,228,228,228,64,85,85,83,84,47,145,72,160,184,184,20,39,78,188,129,210,210,50,76,76,140,98,96,224,62,0,224,
  226,197,111,17,8,44,98,247,238,189,104,105,105,199,153,51,159,97,99,35,8,64,64,118,118,142,201,116,17,123,9,90,254,78,171,61,53,83,253,217,
  231,243,153,236,183,125,189,200,204,204,4,0,196,98,81,78,163,136,172,53,58,99,194,173,187,108,246,96,194,44,224,54,97,251,51,100,1,42,203,108,
  167,98,148,82,133,134,220,42,111,105,90,129,245,59,32,145,136,27,85,197,34,124,62,31,39,98,104,37,180,139,142,207,120,146,26,75,118,3,110,150,
  189,183,140,141,233,154,155,26,196,176,0,200,13,48,218,1,147,174,11,156,100,0,95,185,232,4,167,214,94,197,55,82,92,163,25,137,244,176,205,21,
  114,237,107,107,196,172,152,77,254,103,37,92,216,3,62,157,215,30,143,12,159,207,3,191,223,139,140,12,31,18,137,248,67,245,239,218,218,10,250,250,
  122,48,56,56,128,213,213,85,196,98,73,70,161,91,75,89,193,154,6,74,143,116,146,229,217,12,143,71,70,115,115,19,78,157,122,23,185,185,121,8,
  133,130,8,133,130,198,66,228,65,118,118,14,78,158,124,3,149,149,53,6,170,93,196,149,43,23,48,53,53,129,253,251,15,225,232,209,147,152,158,158,
  52,89,144,161,161,1,124,242,201,239,108,73,212,124,152,138,110,14,69,69,69,248,225,15,127,132,178,178,114,243,111,191,248,226,19,0,64,93,93,3,
  142,29,123,5,165,165,101,0,128,233,233,73,156,62,253,25,98,177,168,81,246,124,24,135,15,19,125,148,64,96,9,62,159,15,27,27,65,248,124,94,
  8,2,208,217,217,109,42,94,199,227,49,92,184,112,14,183,110,221,4,49,226,181,152,42,89,150,112,226,196,73,28,58,116,196,76,182,11,133,130,184,
  123,247,38,238,223,191,135,125,251,14,224,240,225,19,24,29,29,66,99,99,11,0,162,160,125,250,244,103,136,70,163,16,69,17,239,190,251,161,249,111,
  163,163,67,230,6,78,141,54,85,149,38,107,138,104,108,108,196,91,111,189,135,188,188,60,196,227,113,44,47,47,162,178,178,6,103,206,124,130,119,222,
  249,8,0,112,226,196,27,104,104,104,129,207,231,131,215,235,195,111,127,251,247,8,135,35,104,111,223,141,55,223,124,7,191,255,253,111,112,239,94,175,
  209,199,59,137,153,191,20,48,201,33,124,71,22,56,170,115,194,178,151,218,115,60,23,69,110,46,80,173,21,175,87,118,221,108,125,62,63,106,107,155,
  80,89,89,139,153,153,9,244,245,221,193,226,226,34,18,137,36,199,154,90,57,72,180,106,72,113,217,144,237,249,131,186,217,175,61,61,183,80,83,83,
  143,61,123,186,205,42,56,77,211,49,61,61,137,186,186,70,52,53,181,33,22,139,97,113,113,22,186,174,67,150,189,88,94,94,194,165,75,231,76,230,
  70,211,136,90,243,194,194,60,206,158,61,141,226,226,98,188,242,202,155,40,40,40,68,32,16,64,34,145,192,189,123,183,49,52,212,143,183,222,122,31,
  135,14,29,197,233,211,159,65,215,117,220,184,113,5,83,83,147,230,1,199,42,129,103,229,48,4,134,13,22,225,86,121,42,73,178,113,255,170,75,78,
  163,100,86,173,18,129,75,221,21,208,176,161,29,26,190,119,2,37,222,132,55,21,251,161,187,20,151,164,44,106,1,56,177,93,150,129,22,4,145,3,
  19,252,88,17,28,2,160,110,149,112,79,99,67,181,87,72,179,223,237,108,212,102,169,14,146,36,112,204,88,118,118,14,124,62,47,0,1,137,68,28,
  161,208,6,116,93,101,24,166,205,180,176,220,239,155,154,242,242,41,27,186,45,18,34,153,21,167,172,120,175,93,100,147,60,19,171,2,83,81,68,67,
  32,82,229,0,236,118,250,111,99,35,136,254,254,187,24,28,236,67,40,20,66,44,150,68,34,193,139,17,191,40,194,193,47,92,213,155,40,10,140,97,
  159,136,162,194,34,124,240,193,71,24,29,29,198,175,254,233,239,208,209,177,7,199,142,191,102,176,31,18,246,237,59,136,156,156,60,252,242,151,127,133,
  120,44,134,143,62,254,25,94,127,253,45,252,211,63,254,173,41,160,229,247,251,241,55,127,243,159,81,84,88,130,31,252,240,39,56,122,228,56,46,94,
  186,96,26,176,210,184,50,251,185,178,36,225,163,143,126,12,191,207,143,255,243,191,252,39,212,214,214,225,157,119,63,68,32,112,12,35,195,131,120,239,
  189,31,34,176,180,128,255,252,191,255,7,148,148,150,225,237,183,63,192,199,31,255,9,126,247,219,127,64,83,83,43,14,31,62,129,75,151,190,195,224,
  192,3,28,63,241,58,90,91,59,16,14,135,224,245,200,216,221,185,15,187,119,239,195,153,51,159,98,106,114,2,157,157,251,112,234,212,59,88,91,93,
  197,200,200,176,89,73,38,8,2,26,27,26,113,242,228,107,184,122,229,2,238,221,187,141,130,130,2,188,251,222,71,56,116,232,56,134,135,250,33,27,
  39,54,191,63,3,127,245,127,254,71,84,86,213,224,253,247,63,198,193,3,71,112,245,234,5,28,57,122,18,213,213,181,248,228,147,127,66,40,24,196,
  187,239,125,100,91,196,173,74,190,130,130,2,252,224,7,31,99,100,100,8,255,248,143,223,98,247,238,46,156,56,241,58,0,96,113,97,14,127,248,253,
  63,224,227,31,253,25,174,93,61,143,193,193,7,200,202,206,193,143,127,252,47,176,127,255,97,220,185,125,29,39,78,188,134,161,161,1,12,15,13,18,
  115,198,151,221,0,150,154,127,106,164,2,5,18,44,51,95,118,97,18,68,232,34,219,15,36,151,65,211,158,223,208,155,40,146,228,107,143,44,193,35,
  203,230,23,97,115,83,159,254,101,217,131,186,186,102,84,87,55,96,98,98,8,67,131,247,177,178,18,48,253,5,41,243,75,54,109,3,72,26,99,133,
  86,36,89,34,166,164,202,74,83,13,32,2,17,193,96,8,87,175,94,194,107,175,157,130,32,8,4,136,169,26,102,103,166,141,121,119,28,3,3,15,
  16,141,196,152,10,88,17,74,146,168,8,51,112,22,154,166,33,28,14,35,20,12,162,187,251,48,26,27,91,177,184,176,8,143,215,139,100,50,137,112,
  56,130,64,96,9,37,37,101,102,254,158,174,1,201,132,66,114,136,24,107,6,24,214,36,208,89,151,11,82,233,231,86,85,34,112,227,67,128,46,136,
  208,5,34,204,36,64,71,110,78,46,10,10,11,49,61,61,5,197,144,72,240,249,124,40,42,42,198,210,210,146,33,89,64,62,63,55,55,23,153,153,
  89,152,159,159,135,170,42,176,235,30,201,178,132,170,170,42,228,23,20,66,20,128,149,213,21,44,45,46,152,151,147,147,147,139,188,188,124,204,204,204,
  64,85,21,243,125,51,51,51,80,91,91,143,204,204,76,44,47,47,99,114,106,2,154,166,33,35,35,3,117,181,245,200,202,202,194,218,250,42,102,103,
  166,9,48,130,85,157,204,178,146,28,47,163,235,16,5,209,202,229,97,139,79,140,49,64,190,131,51,175,221,201,124,76,89,89,186,73,53,170,190,217,
  102,46,56,63,134,60,86,222,232,57,51,35,19,255,230,223,252,15,134,100,133,132,169,169,73,156,57,125,6,27,27,27,102,14,168,206,140,155,237,204,
  125,65,16,160,169,58,68,65,7,68,102,173,177,221,155,166,18,243,113,150,73,162,194,192,14,214,74,100,171,201,69,40,18,241,108,219,110,11,133,214,
  48,58,50,128,254,254,123,8,133,194,136,27,70,240,68,165,91,37,115,93,161,134,233,214,243,116,41,54,79,51,74,143,101,161,22,5,180,183,239,130,
  207,231,199,205,27,151,33,74,34,70,70,250,81,85,93,139,218,218,6,200,146,132,220,220,60,8,130,0,143,44,65,245,72,248,252,243,223,154,32,74,
  50,16,242,249,115,95,65,83,21,204,207,207,160,175,175,23,7,15,29,198,149,43,151,160,73,162,195,194,131,126,110,69,101,37,74,75,203,112,229,242,
  5,40,74,2,227,227,195,248,63,254,243,127,128,174,3,251,186,15,194,231,243,225,236,217,47,33,203,18,214,215,150,209,123,239,54,142,29,127,13,229,
  229,21,104,106,110,193,202,202,18,70,71,6,32,203,18,110,92,191,136,250,250,38,114,157,30,25,187,119,239,195,232,200,0,2,75,243,144,101,9,189,
  189,119,208,216,212,138,174,189,251,48,54,54,106,48,92,164,90,109,122,122,18,255,191,255,244,255,133,215,235,129,199,35,35,30,143,98,113,113,14,141,
  141,109,240,122,45,53,213,59,183,175,66,146,68,44,46,204,98,99,35,136,188,252,60,200,30,25,237,237,187,49,52,212,135,245,53,162,213,113,249,210,
  247,248,225,71,63,179,74,253,69,1,154,177,204,181,25,125,125,227,250,37,72,162,136,145,225,62,212,212,212,162,182,182,145,132,16,141,211,147,96,156,
  32,35,225,13,220,185,125,13,251,186,15,155,62,117,231,190,63,203,157,50,95,246,198,137,234,185,80,204,130,121,178,23,237,188,211,11,193,40,73,212,
  184,213,212,91,217,222,146,34,73,18,26,27,219,81,91,219,132,241,241,33,140,143,15,34,176,180,64,170,226,152,16,24,173,174,52,247,47,26,86,208,
  52,72,146,108,134,213,37,73,34,222,130,73,5,131,3,253,104,108,108,70,87,87,55,52,85,71,82,81,49,55,55,135,104,36,130,250,250,70,124,241,
  249,39,166,233,114,50,153,68,73,105,25,142,29,127,149,48,74,154,134,219,183,111,152,215,160,107,58,146,73,5,115,115,179,40,45,45,71,70,70,38,
  14,31,57,142,72,36,12,175,215,135,166,166,22,92,188,240,189,121,13,7,14,30,70,117,77,29,188,30,15,102,102,166,113,255,126,175,179,162,140,134,
  147,68,216,114,10,45,80,32,136,118,197,107,227,208,162,105,208,5,1,71,143,29,195,177,99,39,241,119,127,247,215,152,156,24,135,174,3,229,229,229,
  248,232,227,159,224,147,63,252,14,19,147,19,144,140,117,107,239,222,189,216,213,177,27,255,253,151,191,196,198,70,136,3,42,196,160,53,23,127,246,139,
  191,192,204,204,52,98,177,40,94,169,172,194,157,59,55,113,183,231,54,0,224,228,201,87,177,255,192,33,252,221,223,253,53,166,167,38,161,235,58,178,
  178,179,241,206,59,239,195,239,247,99,125,125,29,181,181,181,88,92,154,135,215,227,193,59,239,188,143,140,204,44,172,175,173,162,174,174,14,203,129,69,
  36,147,73,195,195,142,25,3,134,164,4,31,70,210,156,12,226,99,246,174,75,245,62,58,244,135,170,183,75,109,109,195,179,198,69,69,133,136,68,34,
  248,135,191,255,111,144,36,9,154,166,27,249,109,2,39,177,240,56,144,130,14,55,77,63,205,246,165,50,33,48,157,219,91,1,64,84,4,40,34,97,
  148,146,137,164,105,190,158,58,196,182,140,201,201,17,12,12,220,71,48,24,66,34,158,228,140,114,73,33,135,226,250,217,207,123,248,237,133,21,156,164,
  241,250,188,252,60,4,131,235,136,199,227,38,40,88,93,13,160,182,182,1,130,0,244,245,221,197,27,111,188,143,159,255,233,191,198,202,74,0,243,243,
  51,232,185,115,141,76,72,81,68,34,17,71,56,188,97,46,68,36,252,229,71,102,102,6,66,161,13,51,185,150,93,88,4,65,52,195,109,161,208,58,
  191,233,11,64,158,17,2,36,98,115,100,145,35,201,165,64,73,105,25,138,138,74,48,63,63,107,229,5,137,2,86,87,151,33,24,167,55,175,215,135,
  246,93,93,104,223,213,197,221,51,205,87,128,113,26,148,12,106,59,51,43,19,199,142,190,66,38,99,113,9,10,10,138,77,74,150,126,198,202,74,128,
  76,0,227,191,125,94,63,242,243,242,224,245,250,16,9,135,76,161,184,149,149,0,63,225,233,201,19,64,94,94,190,153,92,42,26,185,53,107,171,203,
  168,173,109,76,153,112,217,215,215,131,230,22,226,36,126,241,226,119,72,36,227,198,123,10,248,35,192,73,142,112,136,166,49,42,93,116,129,48,4,10,
  1,209,212,163,34,180,244,243,219,65,52,71,205,67,195,110,166,122,239,206,104,116,89,246,160,185,185,3,53,53,141,152,154,26,197,228,228,48,230,231,
  102,161,169,73,83,93,219,100,28,152,162,140,181,245,117,124,247,221,89,44,46,5,160,169,10,46,94,60,143,104,36,78,194,24,130,128,115,231,190,199,
  204,204,12,54,54,194,68,175,69,0,174,94,187,138,177,241,49,44,44,46,66,49,204,174,71,70,134,225,241,120,225,241,122,76,107,30,64,68,127,127,
  31,22,22,230,17,141,37,160,105,42,122,238,220,70,105,89,57,18,73,5,203,129,0,202,202,43,160,170,42,190,61,251,21,70,71,135,33,138,18,206,
  157,251,14,197,197,197,16,0,163,58,205,102,215,98,147,17,33,249,138,68,218,132,101,85,116,110,227,34,135,10,98,116,66,230,115,110,110,30,10,11,
  139,176,188,28,64,91,107,59,230,102,167,161,233,58,68,35,84,46,138,228,185,152,9,200,140,144,39,107,67,65,215,3,143,44,3,208,113,254,220,89,
  172,175,175,161,171,171,27,199,142,159,196,200,200,32,36,73,70,97,81,17,86,86,150,209,80,223,128,133,249,89,104,154,134,93,237,187,80,82,82,138,
  255,250,87,255,187,197,108,104,26,118,117,236,67,73,105,25,254,250,191,254,103,104,186,14,89,146,204,188,76,81,224,25,37,93,215,108,98,181,48,13,
  196,173,106,88,11,44,89,86,33,143,22,78,51,231,30,172,18,122,0,144,141,112,39,5,11,134,135,43,39,255,242,48,27,57,253,91,234,29,87,85,
  93,131,185,185,57,36,18,196,188,150,230,22,209,138,180,157,218,120,184,37,227,11,46,247,110,218,109,169,42,84,69,77,205,40,233,130,43,41,145,10,
  40,173,173,45,99,98,98,24,35,35,131,88,93,93,69,50,145,52,193,17,249,82,141,130,30,62,220,231,86,137,254,220,134,222,94,180,178,236,84,101,
  132,244,191,53,38,41,78,81,85,204,204,76,226,151,191,252,63,208,209,177,23,69,69,37,232,232,216,139,230,230,118,124,249,229,239,32,138,36,105,83,
  16,5,42,186,203,159,250,152,242,103,162,135,34,16,109,33,182,164,212,129,255,249,226,111,246,218,236,252,174,78,55,21,205,250,23,205,80,255,186,123,
  247,58,122,123,111,155,49,221,100,146,248,229,104,12,71,169,67,71,105,89,57,254,236,207,254,37,150,151,151,176,186,26,192,212,212,56,214,214,86,208,
  208,208,106,150,16,195,120,95,85,179,172,18,216,43,210,12,157,41,205,134,236,53,35,209,206,62,70,4,26,45,96,36,79,216,126,39,90,47,68,110,
  206,227,245,155,42,181,228,187,206,245,200,11,83,10,255,16,77,211,53,8,58,160,106,100,12,105,186,0,29,162,105,99,0,0,146,108,20,2,136,34,
  4,81,135,168,139,70,110,139,8,237,249,37,148,32,26,108,136,236,145,33,123,36,200,30,137,232,32,25,155,245,78,155,207,231,71,115,115,7,170,170,
  234,48,57,57,130,72,36,98,205,23,163,140,152,148,156,147,113,28,12,6,113,235,214,13,115,49,191,115,231,182,197,64,65,64,32,16,64,32,16,48,
  55,86,65,0,198,199,199,48,62,62,198,205,195,249,133,121,44,48,97,38,182,205,206,206,152,115,97,97,113,1,243,11,243,0,128,171,215,174,56,54,
  65,93,39,230,164,116,252,91,44,146,145,3,68,25,67,193,42,177,151,101,153,200,42,112,121,74,68,165,152,243,213,162,202,254,70,120,182,170,186,26,
  128,142,239,190,251,26,39,78,188,134,140,204,12,35,223,208,96,100,68,170,141,198,250,56,90,191,103,223,151,128,37,144,112,144,71,6,4,96,126,97,
  214,96,183,61,40,47,47,71,60,30,199,165,75,231,176,119,239,126,244,222,239,193,198,70,8,177,120,28,126,191,31,85,213,53,152,157,157,54,193,80,
  60,30,131,223,159,129,170,234,106,204,204,76,19,150,70,164,161,71,118,141,21,32,123,60,16,141,195,28,61,52,40,138,70,138,72,68,166,63,140,103,
  110,174,239,186,246,80,107,6,93,95,69,136,230,234,163,233,58,226,113,146,164,236,207,200,224,246,2,65,4,116,149,60,51,80,37,249,109,126,166,14,
  171,108,95,55,226,111,145,104,4,75,75,139,56,122,244,24,50,51,179,112,238,220,183,24,24,24,176,238,68,192,142,202,228,233,251,131,74,230,136,48,
  165,34,116,232,240,120,61,240,120,60,72,36,18,136,197,99,80,53,149,176,174,12,72,74,85,245,102,238,29,204,58,79,15,23,180,173,175,175,98,100,
  164,31,99,99,195,88,93,93,70,50,169,154,38,184,102,136,205,144,4,177,124,9,53,87,39,139,231,63,71,233,133,62,161,235,88,91,91,67,103,103,
  30,60,30,175,89,77,82,88,88,68,128,146,98,153,219,222,189,123,11,130,32,160,184,184,20,31,124,240,19,148,151,87,1,32,201,117,30,143,23,138,
  18,133,174,235,200,201,201,197,210,210,2,98,177,24,115,154,176,211,169,58,22,140,69,51,39,39,215,172,226,160,34,151,193,224,58,114,114,58,225,245,
  122,141,73,168,161,172,172,18,0,176,184,184,128,64,96,9,5,5,197,38,66,215,117,13,5,5,69,88,89,9,96,109,141,104,76,228,231,23,153,73,
  111,236,64,99,243,51,0,160,165,165,21,241,120,28,159,124,242,79,102,82,237,27,111,124,96,222,191,170,234,230,207,172,179,186,174,91,122,22,217,217,
  217,230,73,189,176,176,196,2,74,198,239,232,164,12,6,215,145,155,155,7,159,207,15,85,37,108,25,101,175,104,149,30,217,40,116,243,222,246,237,59,
  12,0,24,27,27,70,87,215,1,244,247,247,33,18,89,114,85,28,127,217,67,112,236,9,74,81,20,228,229,229,227,79,254,228,95,64,51,64,165,251,
  114,251,220,114,186,220,9,150,38,6,75,146,132,172,172,236,135,126,215,140,140,44,180,181,117,33,153,76,154,160,203,206,194,216,253,254,182,10,227,186,
  37,31,179,76,143,91,149,149,93,86,131,173,206,100,147,178,157,214,68,58,39,162,200,151,145,243,130,138,199,142,189,138,253,251,15,59,174,11,128,217,
  135,108,223,170,42,97,160,170,170,106,176,188,28,192,216,216,48,142,29,123,5,53,53,117,24,30,30,96,140,188,97,234,153,81,80,206,51,35,58,236,
  222,106,162,40,162,160,160,16,162,40,224,245,215,223,194,236,236,52,20,37,129,250,250,102,44,46,206,99,100,100,0,7,14,28,70,73,73,41,194,225,
  13,140,141,13,161,183,183,20,63,250,209,79,49,57,57,142,75,151,206,97,101,101,5,195,195,131,40,46,46,197,199,31,147,223,95,190,124,30,107,107,
  171,182,170,58,82,165,248,193,7,63,130,66,75,96,205,208,27,185,199,188,188,124,135,69,148,155,8,240,195,132,220,236,54,75,84,126,34,47,47,31,
  62,159,23,154,22,51,188,40,5,51,162,192,169,143,111,243,179,237,149,135,99,99,163,152,157,157,70,70,70,38,14,30,60,140,183,222,122,23,11,11,
  11,88,54,148,190,31,134,165,98,139,155,184,16,173,64,36,54,188,94,31,66,161,16,214,215,215,25,153,28,62,161,219,57,71,52,195,208,89,96,204,
  211,45,243,220,80,40,136,222,222,219,24,25,25,68,40,20,52,170,179,85,38,188,230,220,183,94,84,128,244,194,2,37,187,152,225,157,59,183,112,232,
  208,17,156,58,245,62,62,251,236,119,168,170,170,65,93,93,19,0,82,126,254,195,31,254,12,0,240,197,23,191,135,162,36,77,192,50,51,51,133,250,
  122,242,186,87,94,121,11,95,126,249,7,84,84,84,163,189,189,19,151,46,157,119,209,178,224,117,150,102,103,103,177,184,184,128,142,142,46,244,246,222,
  69,70,134,31,63,251,217,159,99,122,122,18,151,47,159,199,190,125,7,240,198,27,239,225,159,255,249,183,200,207,207,199,158,61,221,8,4,22,49,63,
  63,15,143,231,30,222,127,255,71,232,232,216,139,158,158,155,216,187,247,0,188,94,31,81,5,78,42,184,119,239,22,14,28,56,134,198,198,118,244,246,
  246,160,162,162,26,239,189,247,3,92,189,122,25,23,47,94,96,22,122,32,26,141,193,231,243,33,63,191,8,107,107,203,232,238,62,98,222,35,65,244,
  228,20,64,53,43,168,52,128,174,235,72,36,20,244,244,220,196,161,67,199,49,57,57,129,169,169,73,28,58,116,140,185,79,149,209,234,0,134,135,251,
  113,224,192,97,28,60,120,12,87,174,156,67,89,89,37,106,106,26,204,190,166,159,165,170,26,18,9,5,77,77,45,104,109,237,192,133,11,223,98,120,
  120,0,63,249,201,47,112,244,232,73,252,254,247,191,121,97,40,215,199,53,94,217,49,187,178,178,138,79,63,253,29,62,250,232,39,40,42,42,70,186,
  57,155,199,227,217,50,140,105,7,161,118,176,227,172,30,114,47,227,214,117,213,177,241,240,66,128,214,134,199,111,126,91,91,11,241,162,125,124,200,57,
  59,59,7,217,217,57,59,218,28,137,89,107,53,86,87,151,177,123,119,23,36,73,66,115,243,255,159,189,247,14,174,227,74,175,125,127,221,125,2,114,
  206,32,64,130,4,1,18,96,206,96,166,50,149,211,140,164,177,103,156,237,113,182,111,189,119,235,189,170,87,247,85,93,215,125,215,174,235,153,235,241,
  88,51,206,99,143,38,73,163,156,41,82,20,197,40,230,8,6,16,32,136,64,228,28,79,234,238,247,199,238,120,206,1,131,70,137,82,127,42,148,64,
  224,160,195,238,221,123,175,189,246,247,173,85,67,75,75,83,194,118,137,48,229,149,93,66,180,166,183,151,19,192,105,154,46,242,171,86,174,33,28,14,
  211,221,125,141,147,39,143,1,18,149,149,179,209,117,141,218,218,122,82,82,82,168,168,152,205,213,171,87,136,197,98,28,60,184,151,203,151,47,177,122,
  117,3,143,61,246,117,94,126,249,5,134,134,6,57,112,224,67,46,95,190,196,154,53,226,231,175,190,250,75,70,71,71,92,237,36,228,68,114,103,188,
  215,104,52,202,206,157,239,112,237,90,87,220,36,171,255,138,239,163,251,157,212,117,97,118,27,14,135,40,43,155,69,102,102,22,161,80,216,144,109,209,
  173,47,1,224,220,198,229,55,82,216,182,251,161,189,56,210,52,141,80,40,204,133,11,141,44,88,80,71,102,102,214,45,3,165,120,67,234,248,47,51,
  7,84,164,159,72,12,14,14,88,146,21,241,50,27,201,238,195,18,54,150,132,88,176,36,137,124,189,177,177,49,142,28,57,196,137,19,71,152,154,154,
  76,220,210,179,192,145,106,85,125,222,238,0,201,6,74,183,217,92,165,107,34,131,95,66,67,149,36,38,39,166,120,233,197,231,121,224,193,71,248,131,
  63,248,115,67,201,87,148,251,71,194,81,118,188,251,38,247,220,251,0,223,252,230,239,27,128,33,204,129,3,31,48,50,60,140,86,105,63,180,223,250,
  173,63,6,224,228,137,163,28,59,250,17,154,166,179,100,201,50,238,189,247,126,126,250,147,231,104,110,105,54,146,72,65,67,39,134,202,107,175,189,204,
  195,15,63,198,239,253,158,248,219,142,246,54,118,190,183,131,72,36,196,59,111,191,193,250,13,155,249,246,183,255,2,128,230,230,38,246,124,176,139,72,
  56,202,149,150,43,124,184,103,23,91,182,222,197,234,213,235,233,239,239,101,108,108,20,93,215,8,135,162,28,59,122,4,69,246,179,113,227,54,54,110,
  220,102,108,197,157,226,224,129,3,162,18,207,92,13,235,26,167,78,158,160,166,102,1,143,61,246,52,0,3,3,125,92,186,216,72,237,130,122,178,50,
  115,45,186,52,26,137,25,186,69,146,241,34,104,68,194,81,142,30,249,136,244,244,76,182,111,127,84,220,67,71,155,189,125,22,211,172,137,65,151,37,
  134,135,70,216,249,222,91,108,217,122,23,191,241,27,127,72,36,34,6,213,210,210,114,177,154,136,106,22,123,37,161,176,122,245,6,58,58,218,57,117,
  234,4,154,170,115,244,200,71,220,121,215,189,84,87,215,114,246,204,217,219,202,215,236,87,234,175,136,254,106,239,213,70,56,127,190,145,241,177,49,145,
  180,44,73,51,89,19,127,129,249,36,9,217,48,207,244,27,138,220,126,159,143,244,140,12,86,44,95,79,218,199,100,149,52,77,99,96,160,151,241,241,
  81,170,170,106,236,42,160,143,225,133,55,83,245,144,243,159,178,28,87,165,228,2,91,14,246,197,201,164,233,201,39,68,116,187,109,156,62,120,174,220,
  28,7,147,113,228,200,1,58,218,175,162,248,220,214,16,154,170,81,191,104,25,181,181,117,214,125,152,190,93,121,185,249,4,131,65,134,135,135,200,204,
  204,162,187,171,147,202,217,85,162,114,210,240,241,138,197,212,184,54,18,108,64,44,26,51,238,68,28,207,190,110,137,88,52,202,238,247,119,210,215,215,
  3,146,168,240,157,63,191,22,73,18,86,44,25,25,153,244,244,116,83,81,49,27,191,207,79,56,28,65,146,116,122,186,187,121,243,141,151,249,245,111,
  254,14,53,53,11,248,232,208,1,52,77,163,183,167,155,55,222,120,153,111,126,243,183,169,173,169,227,240,225,3,232,72,198,121,37,84,53,198,251,239,
  191,203,216,216,136,168,48,52,23,18,214,164,27,163,167,183,71,36,17,199,52,52,85,55,124,238,116,219,15,236,22,94,15,203,17,64,210,197,151,102,
  127,141,142,140,208,209,222,70,245,252,90,42,42,102,51,56,48,128,110,248,23,234,178,142,46,235,160,96,120,215,233,73,53,169,18,222,13,73,114,249,
  220,161,67,126,94,129,5,206,23,213,47,97,116,100,68,248,198,233,206,93,102,177,101,58,83,65,159,205,130,26,91,247,178,140,34,43,86,98,188,89,
  49,168,200,10,213,213,134,228,75,75,179,117,175,154,106,48,115,90,220,123,149,208,86,118,69,169,138,70,132,40,251,247,237,119,9,157,154,32,219,169,
  203,244,101,3,72,183,37,163,228,202,159,209,52,75,184,176,163,163,147,127,254,167,31,88,226,141,206,207,135,66,125,252,228,185,255,16,157,74,145,45,
  63,41,73,146,80,13,16,241,250,107,47,185,124,125,98,134,233,228,137,227,199,89,187,182,65,24,99,154,91,39,18,104,186,72,128,27,232,31,224,63,
  126,244,111,174,149,163,121,141,205,45,205,52,91,70,158,246,80,107,190,176,199,143,31,227,248,241,99,46,250,223,137,244,63,248,224,125,118,239,222,229,
  162,73,99,49,213,240,172,19,227,181,172,73,140,143,79,240,227,255,252,247,4,165,240,29,59,222,65,85,85,218,218,218,57,120,224,128,203,175,232,159,
  255,233,89,235,90,101,89,102,199,187,111,179,227,221,183,45,9,127,116,44,157,11,251,229,148,141,227,93,229,231,63,251,145,213,142,235,55,108,37,35,
  35,131,104,36,198,192,244,0,127,255,189,255,101,173,48,254,229,159,127,96,231,48,169,58,199,143,31,227,200,145,35,214,170,227,171,192,42,57,253,237,
  64,67,66,178,192,235,213,171,109,46,101,247,235,233,210,124,225,128,146,161,8,239,247,43,248,3,62,82,130,1,82,83,3,100,133,179,137,169,209,143,
  117,204,161,161,126,218,174,94,230,210,165,115,228,229,21,50,103,206,124,76,177,84,103,136,237,5,205,177,21,38,205,112,141,51,131,36,29,61,169,129,
  181,107,139,72,18,32,34,249,24,68,194,22,86,60,200,146,21,201,26,107,36,135,47,162,25,131,131,3,174,162,14,243,186,208,97,86,197,108,215,54,
  185,166,139,190,83,87,183,152,142,142,54,246,237,221,99,189,191,191,255,7,127,194,188,234,26,6,7,6,8,4,2,228,231,231,19,154,158,70,146,101,
  38,198,69,149,91,90,90,26,133,69,69,76,77,78,162,105,66,227,70,176,12,226,158,252,1,33,206,27,141,198,68,142,138,36,177,100,201,10,46,92,
  104,228,224,129,125,0,4,83,82,248,221,223,253,67,202,202,42,232,237,237,38,61,35,147,145,145,97,114,178,115,8,6,131,140,12,15,147,158,158,33,
  100,1,70,134,201,202,206,33,37,37,149,209,177,17,87,70,162,9,136,123,186,175,49,50,58,98,37,233,155,19,109,44,22,75,90,161,229,76,29,248,
  85,67,115,108,229,133,66,33,154,91,46,51,175,186,134,149,43,87,115,233,226,121,166,166,166,19,223,65,9,36,213,78,47,152,233,29,117,123,95,202,
  22,203,179,114,213,42,138,138,138,173,177,126,231,206,119,153,154,154,140,171,122,213,174,155,208,29,207,82,10,61,65,67,112,88,177,45,188,234,234,22,
  81,84,84,194,196,196,4,151,46,93,116,221,111,50,147,246,248,45,105,49,199,2,168,22,27,46,4,131,221,137,237,241,206,23,95,54,128,116,219,110,
  189,57,59,41,104,134,24,98,204,101,193,97,251,149,185,179,97,21,69,177,6,47,69,145,137,25,157,222,76,58,179,30,186,33,202,53,167,170,138,208,
  116,136,238,238,46,87,199,48,87,121,130,142,213,110,168,104,155,172,35,38,83,108,141,223,42,136,223,159,55,59,162,249,55,186,75,237,219,109,67,98,
  82,162,201,76,126,227,77,133,227,169,228,120,193,50,243,222,214,53,172,103,243,230,109,188,183,227,45,90,91,155,201,43,42,100,206,156,106,206,159,63,
  67,36,18,179,146,3,237,124,42,123,213,106,39,122,235,9,47,211,151,61,68,63,193,30,120,144,33,134,163,239,200,73,133,245,190,200,33,59,202,215,
  101,89,104,173,196,98,154,1,12,228,91,58,214,224,96,47,109,109,205,180,183,181,48,57,37,170,212,20,37,96,121,145,185,230,42,89,248,134,153,147,
  77,97,97,33,178,44,49,48,48,104,41,214,103,103,103,19,8,4,232,239,239,183,64,169,36,73,20,22,22,17,10,133,24,25,25,177,22,46,206,73,
  45,61,61,157,146,210,82,114,115,114,8,135,67,92,189,122,213,202,123,52,255,62,26,141,50,52,52,104,200,97,137,11,203,206,202,161,170,170,10,93,
  215,233,232,236,96,104,112,16,36,200,205,201,165,170,106,174,72,144,238,233,98,116,100,216,85,145,164,200,138,40,176,136,23,84,68,71,49,13,72,77,
  191,71,77,39,152,154,66,70,102,38,71,143,124,100,89,191,72,146,198,165,139,23,40,45,157,69,219,213,171,140,143,143,179,98,249,106,106,107,235,80,
  100,133,227,199,143,90,0,106,235,214,59,80,85,149,104,52,202,193,131,251,105,111,107,3,116,98,49,213,200,87,20,37,224,170,166,145,158,150,70,48,
  37,133,243,141,231,44,41,133,88,108,146,214,43,45,20,21,149,16,137,68,217,188,101,155,200,199,204,200,164,165,165,153,243,231,207,51,119,238,60,214,
  111,216,196,248,216,40,233,233,25,92,109,189,194,197,11,231,197,120,137,238,210,62,146,100,197,114,176,87,85,161,171,227,204,109,49,19,141,205,202,172,
  100,219,70,31,103,225,34,182,149,4,83,100,142,115,151,46,94,100,233,210,21,148,150,150,177,122,205,58,246,237,221,3,200,248,124,14,22,71,147,208,
  100,33,25,35,43,186,91,227,41,158,245,49,88,57,167,144,231,129,253,123,41,46,41,69,145,101,67,148,52,230,0,73,162,218,82,66,178,192,242,140,
  178,3,113,199,118,18,0,138,34,147,153,153,201,154,181,13,248,124,62,142,29,61,204,228,228,132,197,196,57,173,188,174,215,140,238,49,75,115,89,94,
  197,131,122,103,238,108,50,193,204,47,195,98,216,119,187,86,27,137,10,6,64,83,65,210,81,53,247,170,60,169,100,191,170,90,204,139,100,152,102,2,
  132,35,81,151,197,135,89,221,208,218,122,133,43,87,90,220,213,8,142,227,202,186,134,100,250,114,169,137,172,192,76,74,174,78,89,255,153,92,187,109,
  176,103,202,208,219,0,195,5,174,140,202,13,115,210,117,50,91,206,85,143,179,35,39,91,65,187,134,105,221,13,50,101,93,84,137,28,60,120,128,156,
  156,92,238,185,247,1,235,119,231,206,157,97,207,158,221,46,59,10,19,40,89,128,46,190,154,78,75,4,111,95,74,83,92,199,234,12,93,80,240,186,
  38,89,253,200,238,59,234,109,197,42,73,146,132,164,73,134,241,166,110,152,226,74,68,99,50,209,88,236,166,247,68,134,134,250,185,114,229,34,157,157,
  87,25,31,31,115,232,171,232,196,212,152,197,30,104,186,234,50,126,149,21,49,128,167,166,164,113,223,246,7,200,200,200,224,185,231,126,68,40,20,2,
  96,241,146,165,148,151,151,243,234,171,47,163,135,66,86,177,197,157,119,221,69,123,123,59,251,247,239,117,76,214,88,98,144,11,22,46,100,211,38,33,
  138,154,151,151,199,202,85,107,216,181,235,93,134,135,135,72,79,79,231,177,199,191,198,192,64,63,111,189,245,42,161,80,24,93,135,138,138,74,182,110,
  189,147,209,209,81,20,69,193,231,247,49,50,50,68,69,197,108,182,110,189,147,137,137,49,36,73,34,53,53,133,211,167,79,184,118,238,52,93,75,88,
  204,153,143,91,51,23,56,14,147,214,169,169,105,118,236,120,91,0,64,135,240,227,222,125,31,144,154,154,198,200,232,24,175,189,246,50,153,153,153,6,
  235,166,51,56,56,196,248,248,24,191,252,229,243,4,131,65,68,82,120,140,190,190,62,98,70,78,225,232,216,40,47,189,244,75,134,135,135,132,222,145,
  36,49,29,10,243,234,171,47,209,215,215,231,88,96,193,142,247,222,193,239,15,48,49,49,78,244,253,24,25,25,25,76,77,77,208,218,122,5,93,135,
  43,173,45,76,77,79,145,153,153,197,212,212,36,237,237,87,109,179,108,116,84,7,80,82,85,213,48,157,181,197,15,237,2,22,205,225,75,150,152,192,
  255,113,223,11,49,156,154,213,168,146,48,145,213,96,108,124,148,67,135,246,243,240,195,143,179,102,205,122,250,251,251,184,112,161,209,144,49,149,80,144,
  141,207,234,200,9,101,237,210,245,153,31,3,136,135,194,33,58,58,174,218,227,191,44,182,230,37,25,100,73,6,73,19,63,179,84,212,175,127,92,43,
  129,219,100,146,20,137,64,48,192,157,119,221,75,97,97,49,221,221,93,28,63,113,4,85,83,173,138,55,179,114,84,51,141,149,175,211,142,194,15,94,
  71,55,100,74,164,36,26,87,215,243,169,251,50,237,22,248,190,28,19,18,46,207,28,103,194,158,27,37,75,86,21,136,44,203,236,223,191,143,131,7,
  15,36,48,45,51,153,60,202,178,108,12,108,122,28,3,99,123,57,185,19,248,220,224,195,105,108,41,170,8,146,249,6,145,224,240,156,108,139,202,202,
  29,178,6,16,209,6,206,68,84,251,26,244,164,131,141,121,174,100,91,23,206,54,177,255,78,227,157,119,222,226,189,247,222,113,37,204,138,85,106,98,
  133,195,76,224,225,122,46,217,95,102,86,201,172,72,82,29,121,102,102,69,205,237,200,40,57,189,185,162,81,25,229,38,21,124,71,71,135,104,106,58,
  71,91,91,11,163,163,99,214,214,178,19,204,139,127,187,251,137,211,40,88,146,116,102,207,174,36,53,53,133,140,140,12,202,203,203,105,107,187,106,108,
  51,165,144,150,150,110,108,73,200,86,27,103,100,164,147,150,150,106,89,62,56,183,189,101,89,36,74,71,163,81,246,239,255,144,140,140,116,238,191,255,
  17,86,175,94,199,238,221,59,40,43,43,39,16,8,144,147,147,67,105,105,25,29,29,237,248,124,62,214,173,219,192,216,216,40,187,119,239,176,152,180,
  148,148,20,26,26,54,48,62,62,202,158,61,59,141,227,43,73,22,82,36,209,166,137,247,178,179,109,30,84,53,66,119,119,183,195,250,65,252,237,200,
  200,24,163,163,98,139,173,167,167,135,158,158,158,4,134,184,189,189,45,201,226,201,4,44,162,56,197,57,22,197,98,83,214,246,147,115,172,24,30,30,
  177,254,125,249,114,83,194,184,25,141,198,104,110,110,118,25,135,155,186,60,146,36,163,235,154,227,254,53,87,34,176,26,87,186,30,127,189,159,132,129,
  170,83,207,76,146,132,86,153,216,82,146,185,112,161,145,178,178,114,214,173,219,192,125,247,61,136,207,231,227,236,217,211,214,115,53,237,61,4,219,146,
  220,151,45,222,51,51,190,42,212,189,152,183,223,35,241,183,242,13,173,98,226,237,108,156,213,110,169,169,169,220,117,215,189,44,88,80,199,196,196,4,
  59,118,188,197,212,212,148,99,92,118,2,207,155,91,224,57,23,242,241,222,147,250,12,140,218,151,113,108,191,77,149,185,237,129,51,49,47,201,93,202,
  155,32,50,166,75,14,198,197,246,181,49,183,16,236,173,40,103,231,212,173,210,127,39,112,176,4,227,18,170,91,220,222,74,78,195,65,247,117,74,51,
  186,161,59,13,123,109,10,212,30,72,157,213,51,166,225,100,252,11,101,131,52,41,105,190,70,98,110,133,148,240,194,199,255,220,189,2,182,247,174,157,
  219,110,183,50,168,125,21,114,148,220,237,135,209,151,102,122,150,183,7,88,50,253,6,237,247,80,188,139,145,72,44,233,179,215,117,157,241,241,81,46,
  95,110,164,165,229,34,163,163,99,86,25,113,252,86,128,36,97,1,46,49,209,58,53,137,176,38,222,170,170,121,180,181,181,18,8,4,88,176,160,142,
  107,215,218,45,54,84,85,99,200,178,98,49,43,66,48,79,181,216,37,119,101,154,233,183,167,25,166,173,49,198,199,199,232,239,239,35,43,43,155,148,
  148,20,170,171,107,56,125,250,4,89,89,89,204,153,83,197,181,107,29,214,223,249,253,126,171,162,201,244,105,147,36,172,159,155,21,170,137,166,168,122,
  156,31,28,14,182,37,230,0,23,186,107,177,226,180,161,72,182,208,186,30,64,112,190,183,241,11,162,248,212,128,196,67,74,215,101,191,103,78,45,144,
  173,237,46,231,248,225,30,59,18,245,125,62,105,166,194,6,0,184,218,211,20,26,149,36,248,224,131,93,4,131,65,150,47,95,197,246,237,15,83,84,
  84,194,161,67,251,153,156,156,52,64,146,134,44,235,14,207,193,228,139,203,100,99,249,76,139,98,123,193,235,62,102,114,160,100,190,7,118,191,85,20,
  133,89,179,42,217,178,229,14,202,203,43,152,158,158,230,221,119,223,164,179,179,195,181,133,233,92,192,222,168,13,227,199,45,231,60,246,85,28,199,111,
  171,170,55,201,81,73,96,102,248,43,138,91,6,223,78,72,214,93,219,101,206,5,155,174,11,10,88,81,100,145,36,41,75,86,181,128,44,37,230,139,
  200,146,142,38,73,168,104,226,123,135,31,143,36,219,73,154,38,104,106,104,88,207,161,67,7,19,246,213,157,19,129,166,187,247,161,19,148,85,37,201,
  181,133,38,97,116,114,217,61,24,153,19,148,18,231,66,238,102,130,116,151,151,144,115,91,44,254,188,201,94,8,235,122,204,125,3,227,3,206,28,2,
  231,11,249,85,211,72,186,89,48,104,61,147,56,192,107,127,127,123,189,139,102,53,150,249,78,202,146,68,36,28,77,152,60,198,199,71,105,110,62,207,
  197,139,103,153,156,152,114,9,211,57,243,232,204,234,63,89,150,152,154,154,68,215,117,130,193,32,178,36,4,240,44,1,69,9,50,51,50,41,42,42,
  230,216,177,195,76,77,78,176,253,254,71,8,6,130,66,64,214,186,30,99,75,67,214,92,126,141,18,238,137,76,150,36,145,180,173,235,248,253,1,114,
  178,243,200,204,204,100,206,156,42,78,158,56,138,207,231,163,168,168,132,143,14,237,167,178,178,138,133,117,139,72,79,75,99,106,106,154,67,7,247,113,
  223,246,135,121,230,153,111,113,226,248,17,46,95,190,132,170,170,124,116,104,63,247,109,127,136,167,190,254,235,156,62,125,156,150,150,38,33,159,161,99,
  217,179,196,12,207,171,248,177,1,32,52,45,182,17,131,129,160,163,66,203,200,141,84,53,203,183,43,81,198,132,36,38,194,137,73,181,51,167,5,48,
  99,127,76,52,127,117,79,254,241,54,24,146,209,222,86,165,153,14,138,236,183,192,74,56,28,117,40,68,107,196,162,201,109,53,62,233,201,216,124,7,
  85,93,51,198,51,201,214,206,213,99,188,253,246,27,76,79,79,179,106,213,90,86,175,94,71,77,205,2,78,159,58,78,115,75,51,227,227,99,196,162,
  81,59,15,243,122,239,8,82,226,115,145,18,155,220,181,133,117,35,145,73,132,193,173,207,175,16,12,4,40,44,44,166,126,209,18,234,235,23,3,162,
  64,96,199,142,183,184,210,210,34,198,100,83,13,219,225,173,166,105,201,115,160,190,106,224,231,75,205,40,153,52,166,44,203,40,62,153,249,213,243,121,
  250,153,95,227,151,47,252,156,182,182,86,107,165,98,78,224,102,71,140,95,185,89,246,8,178,157,220,189,126,253,6,54,109,222,150,112,206,190,190,94,
  118,237,124,151,174,174,46,215,113,76,234,212,201,68,173,89,219,192,134,13,155,57,121,242,40,170,102,151,132,155,127,55,123,118,21,95,127,234,25,158,
  127,254,103,92,109,189,146,192,56,153,131,204,156,57,85,60,249,181,103,120,241,151,63,231,202,149,43,104,154,110,208,211,146,107,133,170,40,50,171,86,
  173,230,222,251,238,231,223,254,237,31,25,27,29,181,174,123,213,234,181,108,218,180,149,83,167,78,242,206,219,111,58,64,147,236,26,208,226,191,79,54,
  72,217,191,151,220,82,251,104,73,181,73,188,184,185,129,199,13,162,110,55,86,215,148,131,176,139,8,34,145,152,229,247,54,52,52,200,149,150,139,52,
  158,63,195,244,212,148,181,69,107,42,247,154,73,187,206,247,218,92,112,8,129,60,141,148,148,84,210,211,51,24,27,27,197,212,2,2,157,252,252,124,
  138,10,139,41,200,47,96,42,53,149,172,172,44,202,202,102,137,49,192,52,161,53,243,67,84,227,255,113,0,41,62,98,170,74,118,118,14,27,54,110,
  33,45,53,149,198,115,103,56,119,238,12,85,85,243,200,204,204,162,180,180,156,204,172,44,138,138,138,201,204,204,97,106,106,154,190,190,94,158,255,197,
  115,172,88,185,154,173,219,238,166,168,168,132,125,251,62,160,175,175,151,23,127,249,51,150,47,95,197,134,141,219,40,42,42,225,192,129,61,4,141,251,
  1,24,27,31,19,99,149,57,97,59,102,195,177,241,49,0,50,50,179,92,236,176,123,251,40,49,41,215,45,112,41,197,49,49,55,238,135,215,3,70,
  206,139,188,30,192,55,109,158,204,138,61,231,161,179,178,179,240,249,252,132,66,211,76,140,79,184,170,122,103,18,65,252,52,24,94,147,165,52,183,224,
  220,186,151,58,187,119,239,164,187,187,139,181,107,27,40,43,155,197,230,45,119,178,121,203,157,244,245,245,50,50,50,196,244,116,200,16,138,189,225,155,
  146,228,187,228,240,244,102,115,251,252,62,63,233,233,233,20,22,21,91,58,92,147,147,19,52,55,55,113,96,255,62,134,134,6,173,119,43,166,170,66,
  86,65,243,22,177,95,49,160,228,160,28,101,217,146,250,247,249,21,2,1,191,213,221,132,30,135,130,51,57,217,92,189,154,255,182,52,78,100,51,193,
  91,28,235,7,207,126,215,245,242,63,248,208,19,60,241,228,83,252,232,71,255,66,104,218,189,111,111,150,126,154,215,229,51,52,81,132,194,171,102,189,
  140,230,185,123,122,58,249,251,239,253,47,52,77,39,16,240,91,149,3,230,155,36,27,222,78,126,191,120,52,126,191,159,96,74,0,213,82,72,53,182,
  54,44,175,55,197,210,97,9,6,252,164,164,136,54,88,190,124,53,235,26,54,177,127,255,135,28,57,124,200,240,225,194,202,253,112,110,53,234,58,86,
  53,160,89,138,172,199,229,50,36,75,84,7,99,18,146,188,23,233,211,0,81,95,252,107,118,74,117,136,254,169,106,42,221,221,93,12,14,246,211,120,
  238,180,152,240,117,167,196,133,40,255,54,183,88,204,190,44,97,111,35,40,10,140,142,140,50,62,54,74,118,78,46,37,37,37,140,143,143,185,84,179,
  43,42,103,19,137,70,72,77,77,35,35,35,147,177,209,81,106,106,22,112,245,170,177,248,136,171,16,181,183,123,180,36,0,66,44,60,20,69,97,100,
  100,152,15,63,220,77,104,122,10,85,141,33,73,18,243,107,22,48,50,60,76,73,105,153,177,69,20,99,86,69,37,61,61,93,232,58,76,79,79,115,
  240,192,94,218,174,182,242,216,227,79,113,226,196,81,166,167,167,136,68,194,28,57,122,144,142,206,171,60,244,208,215,56,122,244,16,153,25,153,20,20,
  20,17,10,77,51,62,54,42,146,106,85,167,152,141,104,135,254,126,161,94,95,84,88,68,86,86,150,145,27,36,37,217,2,157,89,35,42,217,196,155,
  12,56,205,220,255,174,151,67,153,28,224,39,219,178,115,254,187,172,180,156,96,48,72,87,87,39,145,104,248,166,69,16,63,141,247,77,8,114,218,133,
  43,38,88,210,117,80,116,56,223,120,142,43,45,205,44,92,88,207,188,234,249,20,21,21,83,80,80,72,81,81,241,23,226,253,11,133,66,116,118,118,
  112,237,90,39,151,155,46,114,181,173,85,232,30,153,219,153,154,102,233,82,197,167,68,120,96,233,22,129,210,237,232,245,102,86,169,200,70,89,36,8,
  157,144,199,159,120,138,89,179,42,233,239,239,101,199,142,55,201,207,47,98,251,246,135,120,229,149,23,184,122,181,213,26,152,191,245,173,223,230,252,249,
  70,14,30,220,47,188,125,12,95,31,179,170,89,146,109,118,69,215,117,78,157,58,198,67,15,61,65,89,89,41,29,29,109,108,219,118,55,117,117,182,
  97,237,192,64,31,7,15,238,161,160,160,144,181,107,55,0,240,237,63,252,11,222,121,231,85,182,111,127,148,129,129,62,10,10,138,0,216,185,243,45,
  238,190,251,1,222,120,227,37,58,58,218,200,202,202,230,158,123,30,180,94,190,230,230,38,118,238,124,203,2,63,254,128,66,48,232,163,118,201,82,182,
  109,187,135,61,123,222,231,192,129,3,2,164,41,162,218,72,81,36,227,179,62,252,1,63,11,22,44,98,93,195,38,14,31,222,207,201,147,71,81,124,
  50,115,42,170,120,252,241,167,120,241,197,95,208,214,214,138,44,203,220,125,247,125,44,90,180,148,239,127,255,111,89,181,106,45,13,13,155,104,105,105,
  98,222,188,26,64,136,79,190,253,246,235,132,66,211,252,249,159,255,215,164,207,227,217,103,191,47,6,245,91,240,40,242,226,203,17,162,234,84,3,213,
  212,53,212,25,30,30,225,141,55,94,99,106,106,34,65,111,197,157,184,107,91,242,88,11,31,100,235,56,209,88,148,214,171,87,88,181,106,45,115,231,
  205,231,114,179,173,60,157,146,146,74,117,117,45,135,14,237,163,177,241,44,0,243,230,85,115,231,157,247,129,36,114,145,36,195,108,214,172,87,149,12,
  246,88,86,100,203,187,203,149,27,98,48,204,177,88,140,161,161,65,139,221,18,137,226,21,188,252,242,11,244,245,117,91,19,212,130,5,117,156,60,121,
  148,148,148,20,198,199,199,209,117,137,148,212,84,33,39,32,65,70,102,6,211,211,147,70,197,91,42,177,88,20,77,87,41,45,43,39,37,37,133,139,
  23,27,153,152,156,176,60,21,237,106,86,208,145,25,25,25,182,196,92,75,74,203,24,30,25,70,86,132,95,160,240,108,3,73,23,82,9,183,50,249,
  37,251,200,205,253,157,238,74,66,182,33,107,18,123,24,227,203,242,118,115,20,195,204,158,51,23,73,146,104,110,110,18,44,135,179,2,235,51,26,67,
  92,201,202,154,36,132,48,205,255,98,34,181,66,213,100,20,69,66,157,154,226,196,201,99,156,58,125,130,252,252,2,178,178,178,73,75,75,37,16,8,
  186,114,221,110,101,161,255,171,2,21,85,85,9,135,67,76,78,78,49,60,60,200,248,248,184,43,7,41,25,67,103,229,122,241,229,173,50,246,24,165,
  36,157,205,78,32,21,63,107,104,216,196,219,111,191,130,36,201,220,127,255,35,108,219,118,15,111,190,249,18,225,112,152,165,75,151,211,221,221,137,170,
  202,228,231,23,81,82,82,202,171,175,190,100,120,73,129,162,216,249,69,38,75,20,207,96,153,223,175,90,181,150,186,186,37,188,250,234,47,24,24,232,
  167,178,114,54,247,220,243,48,75,151,174,100,223,190,157,164,165,165,179,108,217,26,158,123,238,135,22,133,156,149,149,205,11,47,252,152,204,204,76,139,
  181,242,249,20,2,1,31,247,221,247,16,129,64,144,31,253,232,135,148,151,87,114,207,61,15,48,60,220,207,208,208,128,245,185,197,139,151,177,97,195,
  54,118,239,126,143,227,199,143,89,192,72,150,77,19,82,217,96,159,20,234,235,23,211,208,176,149,179,103,79,112,246,236,73,252,126,5,85,181,61,128,
  204,243,138,149,187,108,176,95,126,139,9,75,77,77,229,159,255,249,123,148,149,85,240,224,131,143,179,102,205,58,14,28,216,203,179,207,126,199,26,92,
  50,51,179,120,234,169,111,114,234,212,73,70,70,134,189,183,232,171,14,152,52,29,161,91,12,147,147,83,76,79,135,13,160,65,66,226,172,170,234,73,
  43,35,109,99,88,187,18,169,177,241,12,171,86,173,101,206,156,121,20,23,151,208,103,152,215,22,21,21,19,8,4,104,109,109,177,206,209,209,209,142,
  170,170,204,154,85,201,248,248,24,179,102,85,242,181,175,125,3,77,211,153,154,154,100,239,222,221,140,141,141,177,118,237,122,42,43,103,227,247,7,232,
  239,239,227,157,119,222,36,22,139,25,199,113,39,26,203,50,204,155,87,195,228,228,4,189,189,221,214,228,214,212,116,129,21,43,86,147,159,95,200,194,
  133,139,40,41,41,101,122,122,154,252,252,124,14,30,220,75,36,18,102,243,230,59,40,42,42,102,122,122,154,220,220,60,62,250,104,31,126,191,159,197,
  139,151,27,199,184,72,36,18,77,104,11,211,107,43,20,154,54,170,176,102,177,100,201,50,90,91,91,152,158,14,89,12,182,155,197,209,172,66,149,27,
  77,192,201,217,95,233,19,27,147,77,64,100,86,188,153,99,171,153,78,80,89,57,135,112,56,204,165,75,23,227,180,119,62,91,39,249,68,97,69,12,
  29,46,67,24,211,104,79,97,97,34,33,73,26,189,189,189,244,245,245,93,167,29,63,91,54,215,222,45,73,212,218,115,2,166,47,155,248,163,7,148,
  110,250,165,20,47,183,19,196,156,61,123,194,90,197,158,59,119,146,134,134,173,148,150,150,209,212,212,200,226,197,43,200,204,204,96,114,114,146,57,115,
  170,232,237,237,97,108,108,212,85,150,237,172,116,115,162,254,96,48,200,218,181,27,25,27,27,229,218,181,118,122,122,58,57,127,254,148,225,174,173,48,
  48,208,195,228,228,56,41,41,41,66,29,213,148,150,87,108,91,130,238,238,78,38,39,199,153,156,28,163,188,188,210,154,24,138,138,74,40,40,40,226,
  232,209,131,196,98,17,90,91,47,243,175,255,250,247,200,178,204,172,89,226,115,133,133,197,44,95,190,150,253,251,63,224,194,133,179,34,1,221,177,165,
  32,59,216,175,170,170,249,44,95,190,150,72,36,76,115,243,5,99,64,149,141,45,65,39,80,242,91,66,101,38,192,50,143,113,226,196,71,40,138,76,
  79,79,39,227,227,163,228,228,228,224,247,43,86,91,249,253,1,182,111,127,152,209,209,81,246,239,223,99,77,136,146,183,255,246,149,11,247,246,5,8,
  21,95,217,170,18,117,14,234,118,133,151,158,32,54,234,204,21,17,223,139,149,250,181,107,215,104,106,186,64,77,205,66,150,47,95,205,123,239,189,37,
  20,241,7,250,121,245,213,23,8,133,166,173,99,132,195,97,94,122,233,23,68,34,33,122,123,187,25,26,26,180,76,117,163,209,24,227,227,227,236,221,
  187,155,211,167,79,90,224,42,28,14,163,105,170,53,6,156,61,123,154,230,230,38,135,116,131,206,149,43,45,180,182,182,184,4,247,134,134,134,120,254,
  249,159,50,57,57,193,145,35,135,40,41,41,53,4,46,251,24,26,26,64,146,36,142,29,251,136,226,226,18,130,193,32,3,3,125,12,15,15,177,114,
  101,3,249,249,133,116,118,118,112,245,106,171,5,28,133,92,68,162,10,126,83,211,37,150,47,95,197,188,121,53,212,214,214,113,230,204,73,43,191,208,
  9,80,204,10,196,155,173,14,187,213,119,245,70,249,75,206,113,212,246,27,83,240,249,196,66,78,150,37,82,83,211,88,189,122,29,126,191,159,195,135,
  15,50,58,58,146,48,193,127,214,2,180,241,96,201,148,238,176,129,178,240,123,115,130,64,247,214,226,231,53,230,217,237,100,235,236,105,113,210,26,250,
  151,82,33,251,243,3,74,183,91,187,57,183,243,29,223,95,235,236,176,42,23,134,6,5,27,147,149,153,205,229,203,231,89,188,120,5,117,11,23,115,
  242,228,81,230,207,175,165,233,210,69,163,46,198,246,186,113,42,198,254,254,239,255,185,235,148,157,157,237,236,249,96,167,168,240,81,36,10,11,138,169,
  173,173,71,146,36,74,75,103,145,158,158,201,196,196,184,187,162,6,251,218,6,7,251,237,235,117,68,97,161,216,142,19,162,116,50,178,164,217,0,208,
  56,206,242,229,107,197,103,198,199,92,85,127,241,231,48,63,219,221,125,141,252,252,2,182,108,190,135,215,95,127,1,73,146,145,208,144,12,137,124,197,
  82,112,85,92,58,50,230,247,131,131,253,14,165,96,137,96,48,69,124,214,24,16,183,108,190,139,172,172,108,158,255,197,115,142,42,56,18,253,184,188,
  248,210,131,36,107,146,49,36,44,84,77,71,147,244,4,225,87,123,64,159,65,71,75,50,182,62,140,234,40,77,213,208,100,137,88,52,198,129,253,251,
  168,170,170,102,209,162,165,244,244,116,115,250,212,113,166,167,166,152,158,154,178,251,158,209,225,134,13,197,108,73,130,174,107,215,18,38,134,72,56,204,
  248,216,184,187,82,12,123,177,21,154,158,22,214,31,14,85,200,209,145,145,248,41,10,89,150,24,28,232,183,254,102,124,108,204,176,60,49,42,232,36,
  152,24,31,103,106,114,66,108,1,2,115,170,230,177,118,237,6,116,93,231,196,241,35,140,143,141,91,21,73,154,195,158,67,150,197,185,85,73,98,160,
  191,159,19,199,143,114,215,221,247,177,109,219,93,244,247,247,209,221,117,205,48,167,3,20,208,36,161,22,173,107,218,77,149,113,223,152,93,138,103,45,
  110,101,225,138,203,178,67,145,21,195,231,78,97,237,218,6,230,206,173,102,100,100,152,99,71,143,184,252,219,156,190,99,214,56,172,127,62,253,88,146,
  68,53,156,36,73,66,185,219,52,24,78,98,106,252,121,44,14,227,45,116,116,125,102,81,226,120,134,206,3,73,95,49,70,201,238,41,78,101,103,155,
  46,119,118,136,241,241,49,186,187,59,169,169,173,167,179,179,157,194,194,98,94,122,241,121,251,93,180,180,52,68,2,40,192,63,124,255,59,150,208,155,
  51,76,135,237,53,107,55,8,150,104,98,156,11,23,79,179,112,225,82,187,99,26,111,184,170,197,137,165,169,154,179,178,62,209,111,199,76,154,54,63,
  107,252,242,228,201,195,84,86,206,101,195,198,173,92,105,109,73,178,162,183,207,211,116,169,145,67,135,62,100,206,156,121,108,217,122,47,43,87,54,112,
  248,240,126,215,160,232,52,234,116,173,136,204,241,73,179,229,21,156,235,38,89,150,88,188,100,5,85,115,231,243,222,142,183,24,25,25,73,144,61,240,
  222,195,175,54,179,36,250,89,162,232,169,147,85,154,137,169,176,69,0,13,215,114,77,70,214,116,174,93,235,100,207,7,187,184,243,174,123,217,186,245,
  46,98,177,40,141,231,206,184,38,105,93,159,121,130,143,47,91,119,234,214,152,189,59,153,58,126,124,185,126,124,226,115,178,73,50,193,138,196,200,225,
  170,154,59,159,187,238,186,31,89,150,57,126,236,48,141,141,103,237,50,120,85,179,4,72,237,243,104,72,170,132,38,75,156,56,113,140,89,179,42,88,
  176,176,158,135,30,122,140,55,223,120,133,174,174,107,214,164,109,49,9,178,20,87,102,158,4,208,196,177,65,146,52,3,98,210,19,245,129,102,122,181,
  157,199,52,165,82,156,149,192,126,191,159,117,13,27,104,104,216,68,44,22,227,253,93,59,24,26,26,20,137,198,22,163,166,185,198,177,207,122,66,79,
  38,176,104,11,8,75,134,246,89,60,80,146,110,26,112,126,98,211,93,18,86,201,173,177,245,213,80,201,246,128,210,77,134,109,194,103,191,208,249,121,
  5,140,141,14,163,235,18,185,121,5,128,144,165,215,53,157,171,87,155,105,104,216,202,146,165,43,233,236,108,39,28,14,89,72,69,213,132,62,11,104,
  150,109,128,16,143,179,245,134,204,151,71,215,117,170,230,86,211,218,218,204,158,61,59,80,100,153,180,180,52,150,44,89,141,174,235,196,28,165,173,49,
  67,35,198,188,222,152,161,161,162,25,146,5,170,166,209,111,228,92,100,100,100,217,180,169,166,35,75,146,37,45,208,223,215,75,71,123,27,15,63,242,
  117,150,45,91,205,71,135,14,88,3,151,100,168,98,107,198,121,78,156,60,74,36,18,165,169,233,34,57,185,5,44,93,186,146,158,222,110,174,180,92,
  182,218,41,16,8,88,47,142,249,189,102,130,35,16,247,16,83,29,104,206,112,45,207,43,98,213,170,245,156,62,117,140,230,230,75,214,94,120,60,88,
  242,226,171,199,42,37,91,233,222,204,170,56,126,34,112,10,0,106,170,138,106,76,76,199,143,31,37,35,51,147,134,134,141,220,123,239,131,228,230,230,
  115,234,212,49,198,199,198,44,128,162,95,87,139,42,217,164,38,89,255,72,4,56,90,18,209,191,228,2,128,214,191,165,36,219,246,41,41,44,94,188,
  140,181,107,55,18,8,4,184,116,241,60,31,126,184,219,40,219,118,106,6,105,113,199,22,222,95,178,44,49,61,29,98,231,206,119,73,73,73,101,78,
  213,92,30,127,226,41,246,239,219,195,217,179,167,141,220,42,201,16,64,76,124,15,147,107,44,185,117,218,220,0,210,13,104,227,165,80,102,62,166,155,
  81,50,183,170,242,11,10,88,183,118,3,75,150,46,71,211,52,246,124,176,139,11,23,206,59,0,162,150,96,111,244,121,246,101,39,176,118,2,38,183,
  20,194,231,159,106,144,108,241,49,147,227,129,7,144,190,194,64,201,237,90,44,64,194,210,101,43,233,237,237,65,146,96,241,226,21,92,185,114,153,158,
  238,110,36,9,90,154,155,88,177,162,129,170,170,106,62,248,96,167,97,252,104,175,100,132,214,156,157,251,19,141,38,87,11,214,117,24,25,25,161,184,
  184,68,208,202,178,204,134,13,119,18,48,68,225,162,145,168,165,221,36,203,10,211,211,17,3,120,169,68,34,49,208,117,162,6,8,137,69,69,25,117,
  127,127,31,181,181,245,156,57,115,138,128,63,192,215,159,250,117,58,59,219,185,112,225,156,117,45,221,221,93,52,54,158,97,229,202,213,52,158,59,195,
  192,192,144,99,146,178,203,160,35,145,40,225,176,72,76,61,176,255,67,170,170,170,217,188,249,78,122,123,122,232,237,21,182,6,115,170,170,105,105,105,
  162,172,188,130,217,179,231,137,115,24,134,182,98,123,34,106,129,52,243,197,243,249,252,108,187,227,62,174,93,107,231,224,193,125,182,105,165,166,123,218,
  28,30,88,74,132,37,31,163,178,199,150,237,16,201,201,170,42,33,73,34,127,40,28,142,240,225,158,221,68,163,81,54,111,222,102,37,100,95,188,208,
  200,149,43,205,12,13,13,10,193,72,73,74,90,205,227,102,83,221,10,250,166,129,169,9,150,116,116,116,77,182,68,101,205,74,33,93,187,142,176,163,
  185,37,110,144,180,217,89,57,204,153,83,69,109,109,29,149,179,171,12,102,248,56,123,62,216,197,212,212,180,37,180,153,204,242,199,105,54,172,170,226,
  158,134,135,135,121,253,245,151,185,231,222,251,89,176,160,142,237,247,63,204,188,121,243,185,116,233,60,87,90,175,48,53,57,121,83,106,248,46,251,11,
  69,182,36,25,156,64,201,20,194,21,239,181,230,202,35,186,217,99,151,148,148,82,61,191,134,133,11,23,81,80,80,72,40,20,98,207,158,93,98,203,
  45,206,56,219,78,60,214,191,80,253,217,9,152,226,197,97,111,36,196,249,89,188,107,183,170,137,229,197,175,8,148,110,183,50,65,205,50,52,212,144,
  12,173,8,16,90,38,191,246,235,191,13,136,178,246,93,187,118,16,142,68,145,101,137,152,58,193,249,243,103,88,190,124,53,23,47,158,39,166,26,78,
  247,154,106,37,239,41,138,110,49,64,145,104,212,26,40,108,160,36,225,211,117,14,29,58,192,35,143,60,206,183,126,227,219,0,92,185,114,153,107,215,
  218,41,40,40,102,58,20,225,210,165,11,44,94,188,130,111,253,198,183,121,235,173,215,4,123,164,106,68,12,227,221,72,68,136,117,68,99,49,66,225,
  8,239,188,243,26,247,221,247,48,191,243,59,127,4,64,123,251,85,222,123,239,29,74,74,74,172,107,153,14,133,249,240,195,221,204,155,87,195,198,77,
  219,120,241,197,23,236,178,106,93,50,140,72,133,210,109,40,28,182,38,130,55,223,124,133,111,124,227,55,185,231,222,7,120,229,149,95,178,103,207,78,
  182,110,189,155,185,191,255,231,244,247,247,90,82,0,225,72,132,88,76,220,123,40,28,113,27,216,234,58,179,231,204,35,43,43,155,172,172,108,254,240,
  143,254,210,245,60,222,124,243,117,14,31,62,236,201,3,120,241,177,7,106,179,47,171,154,200,255,193,20,76,54,122,146,142,142,30,214,249,240,195,221,
  244,246,118,179,113,227,86,74,75,203,41,45,45,103,217,242,85,140,140,12,49,60,60,76,40,52,109,177,44,201,39,115,1,151,156,5,8,137,166,208,
  36,84,99,217,171,117,39,91,230,150,88,86,20,133,148,148,84,178,179,179,201,203,203,39,207,96,181,71,70,134,217,191,127,15,231,206,157,33,18,137,
  218,66,128,49,219,16,214,101,14,173,73,73,222,37,133,33,3,44,117,118,182,179,97,195,102,106,106,23,50,119,222,124,134,135,135,24,26,26,100,116,
  116,196,74,78,23,215,230,86,176,140,183,211,112,90,96,0,4,131,41,134,32,104,20,28,230,223,238,118,72,198,22,26,210,36,126,31,169,169,105,228,
  229,229,147,155,155,71,86,86,54,0,173,173,45,236,219,183,199,144,103,113,203,67,152,122,63,150,155,194,23,104,146,79,122,29,134,200,238,23,105,120,
  243,64,209,103,19,210,29,119,220,113,219,180,180,75,108,210,40,111,247,251,21,71,181,89,162,202,180,57,32,172,94,221,64,81,81,17,175,188,242,146,
  225,41,100,174,234,226,253,170,36,43,103,192,201,40,153,9,138,62,159,98,84,137,201,46,159,185,248,85,151,123,235,206,30,120,109,47,57,226,6,105,
  221,177,197,231,84,15,214,93,171,58,211,93,221,25,138,34,148,112,157,148,112,188,234,182,169,226,29,191,2,52,141,57,205,170,27,167,32,165,16,210,
  148,93,147,138,45,28,168,90,3,157,240,235,210,63,85,69,93,47,190,228,3,81,92,191,117,154,170,250,124,138,85,165,169,40,50,25,25,25,212,214,
  46,100,233,210,229,86,117,232,39,57,217,124,18,12,65,127,127,31,103,206,156,228,194,133,70,134,135,135,93,126,136,246,251,162,38,176,41,78,32,99,
  86,140,57,239,95,150,101,10,10,10,89,188,120,41,139,22,45,33,55,55,239,19,185,255,153,124,30,63,238,241,90,91,91,56,117,234,4,205,205,151,
  153,158,158,118,49,73,78,70,41,30,148,122,225,197,23,19,40,109,187,227,182,234,157,146,43,89,80,118,12,32,238,149,161,115,197,148,154,154,198,183,
  190,245,219,124,116,248,32,39,142,31,51,156,238,221,158,66,78,51,220,100,137,220,166,62,136,179,228,213,73,55,39,163,167,221,201,117,186,195,163,205,
  94,141,57,53,155,226,105,85,123,16,79,212,203,136,95,49,58,41,123,39,56,114,110,51,200,178,28,183,186,142,183,119,177,109,17,156,206,212,102,123,
  218,0,208,214,235,240,84,95,189,248,52,193,146,243,29,119,190,235,226,93,244,145,151,159,79,89,105,25,249,5,133,134,68,71,178,108,2,201,97,28,
  45,139,234,84,217,54,194,205,200,200,162,188,188,18,73,146,233,237,189,198,240,240,16,178,36,163,106,154,85,220,96,178,203,51,229,96,153,34,128,195,
  195,195,244,116,119,209,215,223,71,36,28,65,51,138,51,226,1,130,147,73,138,31,55,156,99,151,125,255,138,181,88,147,21,225,115,153,146,154,66,113,
  113,49,37,197,165,100,231,228,16,8,4,92,178,36,142,187,183,69,31,173,123,23,237,168,105,42,21,21,85,164,167,103,162,235,58,221,221,29,140,141,
  141,32,75,202,45,221,127,52,26,101,122,122,154,129,129,126,186,174,93,99,100,100,196,50,10,118,27,249,234,73,65,146,55,110,120,241,69,142,219,52,
  71,201,4,31,154,229,198,46,42,20,204,65,209,102,105,214,174,219,200,182,109,119,114,245,106,43,39,142,31,35,22,75,238,123,35,42,110,76,102,71,
  79,106,84,41,128,1,150,199,155,147,17,114,2,26,243,120,137,6,148,110,218,62,153,163,120,50,160,228,252,219,228,70,152,122,18,35,91,9,92,57,
  25,246,132,145,108,5,24,191,231,173,105,146,75,15,39,94,101,217,57,200,39,250,77,73,31,227,153,254,106,131,228,231,173,227,116,179,172,196,205,124,
  238,171,58,97,36,211,181,177,221,230,237,254,38,0,132,0,29,61,221,221,244,246,244,206,92,178,109,218,163,40,130,161,241,251,20,252,126,31,129,128,
  207,248,191,159,165,203,86,51,107,214,28,0,66,161,105,142,31,219,71,52,26,35,18,137,25,230,189,170,149,87,36,212,164,245,164,219,47,78,245,99,
  61,78,35,200,54,144,214,110,152,251,99,86,239,138,91,183,243,5,21,69,180,141,172,233,200,178,198,228,196,36,173,83,173,92,109,189,58,115,130,177,
  36,140,105,109,6,94,220,187,248,242,147,153,153,65,121,249,108,171,237,70,134,7,56,117,234,176,145,239,24,37,18,137,9,111,190,152,106,20,142,232,
  110,219,165,184,62,43,198,102,45,78,35,73,183,182,216,156,139,83,15,36,121,225,1,165,207,12,48,225,42,245,149,12,151,106,85,21,131,238,193,3,
  251,57,120,96,191,99,32,155,89,216,108,166,10,2,221,208,137,49,193,146,177,238,67,146,116,199,214,155,91,207,194,28,48,244,155,112,105,190,217,201,
  50,89,197,73,60,85,110,159,47,113,66,190,89,44,97,151,84,75,73,156,200,221,34,102,51,85,91,36,250,194,221,28,208,249,56,3,102,178,138,158,
  235,93,195,76,63,191,149,234,61,167,234,244,173,0,181,155,249,220,173,182,221,151,29,44,137,66,3,217,0,16,186,131,81,54,25,90,45,65,0,209,
  249,156,132,114,189,140,44,233,150,175,155,44,139,109,251,194,194,34,170,171,235,173,207,87,84,204,165,178,178,138,171,109,87,144,13,255,58,243,186,156,
  64,201,153,220,157,172,15,217,64,192,46,255,143,103,145,146,129,4,243,124,78,176,100,143,35,166,82,180,102,177,189,201,170,176,220,110,245,18,200,50,
  146,164,217,172,179,41,8,233,87,88,180,120,53,153,153,217,214,223,206,171,94,72,87,87,27,221,61,93,40,49,213,102,158,53,221,242,17,115,230,110,
  222,232,254,157,42,209,238,5,170,39,130,232,133,7,148,62,245,193,212,116,175,143,159,188,156,121,74,102,73,175,251,115,38,173,45,20,128,101,89,75,
  2,50,146,131,50,247,233,116,203,58,196,4,104,110,77,151,27,15,6,51,129,131,100,194,113,201,88,36,231,22,133,125,223,78,17,178,196,243,39,213,
  127,113,13,182,201,25,46,51,135,105,166,106,11,231,224,236,4,42,38,224,186,17,155,98,95,231,199,3,76,206,237,138,248,124,173,248,73,216,249,156,
  220,224,248,86,147,144,147,131,197,153,74,136,111,198,161,61,217,177,190,234,96,73,84,128,105,142,74,36,217,98,128,157,207,59,89,123,154,54,71,146,
  81,217,42,62,35,182,175,252,1,63,115,231,45,32,53,53,205,241,121,153,154,218,37,244,244,92,35,22,85,145,101,213,245,172,237,109,163,228,239,173,
  115,129,228,6,69,55,7,18,156,227,149,121,143,146,100,91,106,136,49,75,178,22,105,51,129,37,179,45,132,128,165,209,158,38,110,50,148,250,11,242,
  139,168,172,156,231,250,155,96,48,149,121,243,22,50,52,52,64,52,18,179,198,56,167,13,77,178,251,119,11,139,218,139,197,100,121,72,154,230,233,251,
  120,113,155,1,165,219,177,66,201,44,233,69,210,45,227,69,203,240,114,70,182,1,208,37,100,9,161,150,171,105,72,154,24,60,93,114,240,186,142,192,
  4,137,19,167,195,59,211,250,178,207,41,129,100,88,17,72,70,153,179,49,194,38,223,22,99,6,64,162,59,10,106,102,24,68,37,129,218,132,145,175,
  228,74,42,183,86,170,146,118,83,57,67,110,128,225,4,94,14,192,165,235,96,249,72,197,29,211,33,15,238,30,180,101,107,75,212,242,208,50,142,17,
  15,204,156,45,108,179,58,55,6,8,241,224,79,146,197,243,176,109,93,36,23,48,138,207,17,115,178,111,182,38,151,59,247,235,122,37,46,246,100,96,
  30,12,215,115,149,100,18,64,122,114,102,203,157,208,175,105,118,137,182,241,211,175,220,192,228,4,75,102,181,145,36,33,76,97,53,205,6,9,154,144,
  17,112,2,124,87,174,147,46,33,235,98,156,176,13,181,69,161,66,126,126,49,85,85,181,9,231,46,44,44,101,86,197,92,46,95,62,111,141,51,58,
  58,170,166,25,149,178,154,5,8,226,153,231,120,33,64,39,115,116,179,74,201,9,32,217,184,127,221,240,194,147,117,231,189,75,51,44,82,68,191,82,
  20,25,5,25,89,215,48,14,132,172,200,40,62,133,234,249,245,4,131,41,9,231,175,156,93,205,149,214,38,166,67,109,200,138,232,158,230,253,39,173,
  212,187,129,90,244,76,204,179,7,146,188,240,24,165,79,11,32,185,38,121,217,170,68,51,43,66,156,147,188,115,117,22,63,57,38,163,129,205,28,36,
  73,210,141,21,108,226,203,28,127,78,81,25,38,37,172,158,236,149,167,236,26,56,227,149,93,147,9,189,57,87,166,206,73,217,9,42,220,9,174,138,
  11,40,137,115,75,86,5,155,19,244,197,151,67,155,198,149,146,100,91,15,56,89,12,231,253,196,15,120,241,43,68,19,156,196,87,16,130,228,0,109,
  184,172,14,156,131,186,123,181,153,248,12,227,25,32,243,249,57,147,213,205,103,19,15,254,226,129,82,178,123,72,198,120,37,203,85,179,159,137,89,53,
  169,57,182,100,109,150,67,180,107,18,181,102,244,132,231,239,236,3,146,5,114,181,47,140,190,204,231,5,150,226,159,131,83,53,57,25,27,234,4,12,
  38,139,100,190,159,54,195,36,19,8,248,169,169,89,132,207,231,79,178,120,144,153,55,111,1,93,93,109,132,66,17,87,206,163,157,115,148,76,228,49,
  57,99,244,113,1,66,124,53,154,153,78,224,30,215,174,119,255,137,38,223,230,123,89,92,92,70,69,197,188,164,231,85,20,31,181,181,139,233,235,235,
  54,238,31,23,171,20,207,170,125,90,247,239,133,23,95,28,160,116,27,246,89,147,62,150,37,153,63,255,243,255,66,110,110,238,140,159,221,249,222,59,
  156,61,43,76,108,255,232,143,254,130,96,74,10,63,251,233,127,208,221,221,109,77,254,75,151,46,231,190,237,15,240,211,159,252,152,203,151,47,139,159,
  27,171,72,231,36,121,179,231,124,239,189,119,56,113,252,152,75,72,206,61,208,39,26,74,58,171,207,236,201,91,182,24,13,77,210,29,102,184,134,95,
  155,44,227,247,41,204,154,85,193,183,126,227,183,9,133,66,252,195,247,191,107,200,31,168,200,146,22,87,37,39,37,200,19,152,85,68,38,208,172,170,
  154,203,19,79,62,205,203,47,61,143,174,235,60,241,228,211,60,255,252,207,104,189,210,226,186,62,33,5,96,12,222,70,194,169,179,82,105,245,234,213,
  220,255,192,67,252,228,39,255,73,75,115,139,149,31,98,38,194,255,198,111,254,22,0,63,255,217,115,214,160,62,83,238,70,60,80,178,183,52,220,174,
  235,146,36,161,200,50,27,55,108,98,235,182,59,173,191,57,119,238,52,239,239,218,129,174,235,164,164,164,242,192,3,143,80,81,41,18,88,67,161,16,
  71,143,28,226,208,161,131,46,48,11,240,27,191,249,59,244,244,116,243,238,59,111,185,158,111,78,78,46,191,249,91,191,203,7,187,119,113,244,232,81,
  12,162,194,117,189,38,96,123,242,201,175,211,221,221,205,193,131,251,93,160,215,217,15,82,82,82,249,147,63,253,51,126,242,220,143,185,118,173,19,213,
  100,34,13,14,225,171,62,185,36,0,38,99,82,142,23,0,140,127,175,100,89,50,252,208,100,116,197,96,102,16,125,164,164,184,60,97,219,201,25,69,
  69,101,84,204,170,98,100,120,212,240,75,196,242,34,83,99,122,130,162,118,34,179,242,201,168,36,199,51,144,55,123,255,86,213,42,160,43,10,232,32,
  27,21,127,129,64,128,234,121,117,248,253,254,25,207,59,107,86,21,101,165,21,76,78,92,178,253,37,117,44,127,54,103,82,118,178,251,247,132,16,189,
  240,24,165,207,125,224,180,87,230,207,254,195,247,44,109,163,181,235,54,176,113,227,22,254,238,127,255,141,251,38,125,10,53,53,11,9,166,164,16,14,
  135,88,184,176,158,161,161,1,171,26,197,231,19,219,103,178,193,208,196,231,65,196,171,249,62,251,15,223,195,239,247,225,243,41,52,52,108,96,253,134,
  205,252,195,247,255,214,197,78,248,253,62,87,217,189,83,39,41,126,64,179,217,20,219,242,36,158,222,54,89,34,123,34,22,247,28,8,248,168,175,175,
  39,28,14,145,146,146,194,162,69,139,185,120,241,60,209,104,140,88,76,77,200,9,48,7,80,243,56,62,159,205,196,41,138,140,63,32,186,132,223,175,
  208,209,209,206,179,255,240,29,84,85,37,24,244,91,247,99,182,155,174,235,40,138,78,204,80,27,119,185,108,27,171,88,179,218,198,121,29,138,34,243,
  252,47,126,138,162,200,4,131,126,235,252,241,106,232,54,8,114,178,10,54,195,21,139,169,72,146,221,198,138,34,177,114,229,42,182,110,187,147,157,239,
  189,67,99,227,25,170,170,230,242,200,163,95,99,124,124,140,19,199,143,176,108,217,114,138,138,139,121,253,181,23,105,107,107,101,197,202,53,108,218,188,
  141,142,142,54,186,187,187,1,72,77,77,225,137,39,159,161,184,184,132,190,190,94,252,126,179,228,90,162,184,184,132,135,30,126,140,148,148,20,87,127,
  113,78,4,194,227,202,199,202,85,171,40,45,43,227,141,55,94,33,16,240,59,128,158,230,154,248,162,209,48,135,14,29,228,193,135,30,230,95,255,229,
  159,12,102,64,36,48,131,150,160,155,245,85,7,76,201,198,3,243,29,115,191,87,178,85,132,97,25,181,42,162,234,173,174,126,249,13,19,235,171,231,
  215,211,122,245,10,211,161,136,241,206,56,153,165,153,115,149,62,45,96,112,179,247,111,47,132,244,4,224,168,40,10,165,37,229,84,206,158,119,195,243,
  213,213,175,224,234,213,43,248,124,17,75,164,211,60,95,188,1,171,7,138,188,240,128,210,23,104,160,76,244,150,114,84,188,185,6,73,155,70,151,101,
  153,197,139,151,114,237,90,7,227,227,163,212,213,47,225,216,177,131,214,100,107,106,143,40,198,214,141,83,91,201,57,240,186,128,128,105,239,36,97,77,
  254,230,165,204,170,168,100,221,186,77,20,21,21,3,66,41,252,237,183,94,35,20,10,33,73,176,106,245,58,214,175,223,12,8,47,183,96,74,10,163,
  163,35,188,242,242,243,60,246,216,83,232,232,252,242,133,159,163,170,26,217,217,57,252,193,183,255,152,221,187,119,177,127,223,62,235,186,204,65,63,35,
  35,131,69,139,151,114,250,212,49,22,214,45,166,166,118,1,173,173,151,45,64,244,228,215,158,6,224,249,95,252,20,144,200,203,203,227,119,127,239,15,
  217,191,255,67,142,31,59,76,94,94,30,247,109,127,136,194,66,113,173,173,173,205,2,40,5,252,204,157,59,151,251,182,63,202,91,111,190,76,123,251,
  85,202,203,43,89,215,96,223,87,91,91,43,175,190,242,18,227,227,19,9,64,204,4,99,230,181,138,103,103,3,165,103,190,241,77,100,73,226,213,87,
  95,224,177,199,159,34,43,43,155,112,56,100,93,199,137,19,71,56,176,255,67,3,184,164,177,117,219,221,84,87,215,0,208,219,219,195,142,29,111,211,
  217,209,65,44,38,185,182,189,22,214,213,211,211,211,205,217,179,167,240,251,125,116,117,117,112,237,90,59,243,171,107,57,115,250,24,213,243,107,105,189,
  210,76,87,87,7,138,34,115,225,252,57,54,108,216,194,236,57,85,244,247,247,81,84,84,194,227,79,124,157,183,223,126,141,59,238,184,215,16,222,20,
  253,99,241,226,101,172,90,189,150,247,119,237,224,241,39,158,114,24,128,138,142,96,94,134,9,14,215,55,108,228,204,217,147,248,253,62,36,73,34,39,
  39,151,237,247,63,76,113,177,80,93,191,116,233,2,175,191,246,50,186,14,103,207,156,162,161,97,3,171,86,173,230,216,177,163,55,101,27,225,1,167,
  235,39,199,187,197,27,77,177,88,31,115,231,205,167,184,184,252,134,199,207,203,43,164,186,186,134,209,145,17,162,126,5,159,79,38,22,147,137,197,12,
  71,121,237,243,173,220,186,113,113,64,226,189,167,164,6,168,95,180,194,81,136,50,115,20,22,150,80,83,179,144,211,167,79,25,169,6,10,138,162,18,
  139,37,43,248,240,250,170,23,95,222,144,111,199,139,118,83,188,142,114,248,56,177,69,115,194,206,205,205,165,124,86,37,109,87,155,105,111,187,66,48,
  24,100,254,252,5,214,224,97,42,86,203,138,45,240,38,142,33,199,129,35,103,85,152,241,51,36,11,144,41,138,68,110,110,46,247,222,251,16,145,72,
  152,31,60,251,29,94,252,229,79,201,206,206,225,241,39,158,38,16,240,177,176,174,158,245,235,55,179,119,239,251,252,211,63,254,29,131,131,253,100,101,
  101,91,121,19,118,14,133,169,245,162,88,224,194,231,179,149,200,197,151,76,237,130,133,4,131,41,180,181,93,161,233,210,121,230,206,157,79,70,70,186,
  165,151,98,182,65,32,224,39,24,244,19,8,10,108,236,83,196,57,182,223,255,48,146,36,241,147,231,254,153,215,95,127,129,242,242,10,1,148,140,182,
  17,236,146,143,252,252,2,238,189,239,33,162,145,48,63,120,246,187,60,255,139,31,147,147,147,203,211,207,252,186,149,175,101,14,166,178,44,40,126,48,
  232,254,56,113,80,159,79,177,132,239,2,1,1,34,178,178,178,105,109,109,230,159,255,233,239,56,114,228,0,43,86,172,161,174,174,158,96,208,207,157,
  119,222,67,78,78,14,191,248,249,143,248,193,179,223,37,18,9,243,212,215,191,65,122,122,122,156,87,149,196,207,126,250,28,63,254,207,127,119,1,183,
  96,48,69,36,222,203,50,153,153,217,140,79,140,89,207,51,18,9,51,54,54,74,78,118,174,85,33,245,206,219,175,211,217,209,238,234,75,230,57,94,
  125,229,5,198,198,70,197,239,28,162,156,146,227,62,101,89,162,174,126,17,217,57,57,92,105,185,108,61,175,7,30,124,132,148,148,20,126,248,195,239,
  241,230,27,175,80,91,187,144,117,235,214,35,203,18,209,104,152,246,182,171,44,172,171,179,182,65,205,99,125,222,26,81,183,91,56,223,127,83,59,200,
  252,74,79,79,165,190,126,197,77,183,233,194,133,203,200,206,201,193,31,240,27,44,178,207,234,227,201,236,79,62,239,251,54,1,162,155,65,19,239,178,
  63,224,163,170,170,154,210,210,138,155,62,230,130,133,75,201,204,202,32,16,240,91,99,143,51,31,212,235,155,94,120,64,233,11,14,150,220,206,211,56,
  170,142,36,151,221,201,188,234,26,194,225,48,109,109,45,116,247,116,50,49,49,70,213,220,106,107,2,51,217,15,159,162,56,146,180,149,184,9,75,118,
  217,130,56,22,109,174,213,235,188,234,26,130,193,32,239,239,122,11,69,145,25,30,25,228,236,217,19,20,22,22,81,90,90,70,117,117,45,131,131,253,
  52,55,95,192,239,87,56,116,104,15,145,72,24,73,18,128,196,84,12,14,4,252,4,2,62,124,6,80,50,173,12,236,149,157,184,198,249,213,181,116,
  117,117,48,57,57,78,123,251,21,131,50,95,98,3,37,67,141,215,60,158,185,5,228,243,41,148,149,149,83,80,80,68,99,227,73,84,53,202,200,240,
  32,231,27,79,217,160,209,0,144,138,79,102,94,245,124,130,193,32,123,246,188,139,207,39,51,56,56,192,137,19,71,40,46,46,161,172,188,220,209,86,
  18,138,34,37,48,74,230,245,154,247,96,219,170,40,72,18,68,34,97,26,207,157,68,81,20,218,218,90,0,145,11,84,90,90,70,213,220,249,156,111,
  60,69,40,52,133,63,224,99,215,206,183,9,166,164,176,124,249,10,23,163,227,76,34,55,39,201,138,138,57,20,20,20,113,233,82,35,146,36,17,12,
  6,197,68,226,170,142,179,159,99,95,95,47,157,157,237,113,28,165,136,115,231,78,51,58,58,146,212,100,213,121,110,69,145,41,45,45,99,116,116,132,
  137,137,113,235,223,133,133,197,156,63,127,150,72,56,76,115,115,19,127,247,191,255,134,163,71,63,178,64,94,95,95,15,179,103,87,145,150,150,22,231,
  71,230,13,84,183,6,24,112,1,37,159,79,168,120,251,3,62,106,106,235,201,203,43,188,233,99,165,165,101,80,95,183,148,128,1,180,132,50,191,221,
  119,102,20,186,252,220,129,162,88,184,57,193,82,106,106,10,139,23,175,186,165,99,229,228,228,83,91,91,143,207,175,24,32,81,142,3,74,94,127,243,
  226,203,31,183,157,60,128,185,253,166,233,194,32,87,211,53,84,77,67,214,84,171,140,90,51,170,197,100,73,70,71,103,225,194,69,180,182,94,38,20,
  14,33,203,18,109,109,45,212,215,47,39,43,59,155,145,145,97,43,159,70,146,29,37,247,138,40,45,214,29,147,165,89,98,172,163,27,231,181,171,195,
  196,57,37,50,51,179,24,31,31,37,28,9,91,186,37,221,221,29,130,202,46,42,166,160,160,136,238,238,78,100,217,20,144,147,25,28,236,55,28,189,
  237,129,71,241,137,107,183,43,195,100,100,69,178,243,156,100,137,156,220,28,202,103,85,114,224,192,110,116,116,198,198,71,25,28,236,167,166,166,158,211,
  167,143,161,35,91,101,85,138,207,158,56,204,123,41,44,42,2,96,104,104,192,56,168,206,208,112,191,133,0,205,107,145,101,200,202,202,102,124,124,140,
  112,36,108,181,147,9,40,74,74,74,232,233,233,194,44,63,54,141,76,77,0,34,201,128,46,97,40,185,136,246,118,149,208,195,224,96,191,81,190,173,
  187,202,235,11,141,109,190,205,91,238,97,243,150,123,18,65,106,220,151,89,2,142,4,153,89,217,220,121,215,253,92,188,120,142,11,23,206,90,215,100,
  9,143,58,10,239,133,49,167,102,1,32,93,55,147,249,237,159,155,31,54,141,152,117,12,241,65,243,72,102,14,7,58,37,37,37,130,121,50,174,197,
  108,235,177,177,17,67,70,194,110,31,241,76,117,122,251,122,196,228,148,155,195,248,196,132,209,223,236,123,242,182,55,110,12,16,116,156,210,25,54,224,
  151,21,241,110,206,159,95,119,203,160,102,126,77,61,23,46,158,97,58,20,182,142,37,100,73,36,187,207,241,197,144,113,48,219,192,188,62,113,255,226,
  171,166,166,238,150,64,162,121,188,121,243,22,208,212,116,129,233,233,176,117,44,179,125,209,189,254,233,197,87,0,40,221,174,108,146,53,185,25,137,197,
  34,103,192,54,118,53,255,93,89,57,135,204,204,108,22,44,200,102,193,130,69,174,227,204,158,61,143,145,145,99,174,65,65,118,168,216,42,138,204,220,
  185,243,248,214,183,126,211,250,204,240,240,48,207,62,251,61,227,188,238,115,154,73,217,201,4,31,205,149,174,189,226,149,93,162,113,118,78,148,93,210,
  235,100,60,146,9,202,85,87,11,13,152,13,27,238,96,195,134,59,92,191,43,43,171,160,179,179,221,1,118,228,4,159,57,71,139,138,97,94,151,28,
  21,59,154,245,25,145,139,113,163,103,98,31,203,121,153,206,109,43,83,60,80,114,232,68,57,245,139,236,138,54,231,49,197,63,222,126,251,37,250,250,
  186,137,70,69,69,95,36,18,37,20,138,206,96,233,2,217,217,57,60,242,200,215,232,239,239,101,239,222,93,22,251,24,14,135,227,228,33,220,215,226,
  66,68,56,43,252,236,48,255,109,38,180,126,210,166,170,41,41,169,9,236,8,120,147,208,205,78,236,206,126,103,179,156,50,243,231,47,184,101,160,0,
  16,12,166,176,100,201,42,122,122,222,114,229,224,9,27,149,47,150,40,168,147,133,116,94,107,106,106,26,117,117,203,62,214,49,115,115,11,168,169,169,
  167,175,175,63,110,203,205,163,147,188,248,170,0,165,219,117,252,149,64,215,204,47,13,77,181,147,169,133,145,163,24,52,230,206,157,207,216,232,40,207,
  61,247,47,174,202,143,251,31,120,156,154,154,58,78,156,56,226,178,35,64,119,191,254,87,174,180,240,255,254,183,255,199,174,104,82,140,252,27,93,70,
  66,179,254,86,141,9,229,224,209,209,81,234,234,178,241,251,2,132,195,33,100,73,162,180,100,22,0,3,253,125,12,14,246,145,159,87,104,42,158,32,
  1,121,121,5,12,14,246,91,115,97,32,16,20,94,81,154,78,192,159,226,196,50,22,34,209,52,88,180,104,41,231,206,157,230,195,15,119,33,25,215,
  150,150,154,202,83,79,255,22,115,102,87,211,222,118,21,93,135,96,48,104,77,250,129,128,253,253,64,127,159,24,8,115,10,24,30,26,6,73,80,237,
  32,202,128,205,228,107,77,213,24,27,27,161,166,166,14,191,47,64,36,50,133,166,233,204,42,23,206,237,61,221,221,118,27,74,160,25,101,212,22,80,
  212,157,4,144,228,184,15,9,77,213,44,245,238,88,212,240,131,50,170,232,116,77,167,175,175,23,128,226,226,50,186,187,174,161,198,84,98,81,213,42,
  83,22,237,228,236,22,18,75,151,44,231,238,123,182,211,209,209,198,27,175,191,104,48,144,162,61,199,198,70,200,207,47,18,58,56,6,200,202,202,202,
  22,14,239,49,213,85,218,109,2,94,243,122,112,40,38,91,253,76,21,95,206,73,202,186,103,161,25,128,142,68,95,175,184,143,172,204,108,76,237,63,
  23,38,115,108,29,135,66,33,235,223,186,217,150,186,135,149,110,109,120,48,12,96,141,239,179,179,114,168,169,169,251,216,199,155,61,187,154,146,226,50,
  174,92,105,181,142,105,247,231,47,200,243,145,156,227,151,228,184,70,168,175,95,70,118,118,238,199,62,244,252,249,11,57,223,120,142,233,169,107,214,145,
  37,71,91,235,51,248,223,121,225,197,151,33,228,219,249,226,117,244,4,79,33,115,197,175,170,42,126,191,159,69,139,150,114,230,204,73,34,145,24,97,
  195,232,49,20,138,112,249,242,69,50,51,179,41,45,41,183,182,86,156,128,201,26,6,204,92,40,147,193,50,244,83,44,39,112,99,102,53,203,241,47,
  93,60,79,56,28,226,158,123,31,36,166,170,164,166,101,176,120,201,74,6,6,250,232,233,233,166,245,202,101,242,242,11,153,63,191,14,93,215,169,171,
  91,102,129,23,85,213,232,239,239,37,63,191,144,220,188,2,84,85,99,237,186,141,14,246,76,71,53,190,230,207,175,37,59,59,135,11,23,206,19,9,
  71,197,189,133,34,140,142,142,211,220,124,137,170,185,213,40,138,159,190,190,30,10,10,138,200,203,43,68,85,53,86,174,108,176,38,251,238,238,46,174,
  93,107,103,229,202,6,82,83,211,73,75,205,176,38,147,104,44,70,36,26,51,190,87,185,220,116,145,112,56,196,182,59,182,163,198,84,50,210,51,89,
  177,114,13,189,189,61,92,187,118,205,112,6,87,173,118,153,201,26,196,98,141,0,77,215,137,68,98,214,243,11,71,162,134,17,105,204,104,15,149,174,
  46,113,141,139,22,173,32,39,39,159,104,84,101,249,138,53,252,229,127,249,191,152,61,123,142,113,44,221,122,102,235,214,173,231,238,123,182,115,246,236,
  41,94,121,229,121,171,178,81,53,76,61,155,154,46,50,107,86,37,249,249,133,104,154,198,242,21,107,0,104,109,109,37,22,19,159,141,198,98,196,84,
  213,98,185,76,93,170,88,52,38,142,99,0,39,213,42,17,55,140,78,205,123,213,161,167,183,155,162,162,98,241,25,85,163,167,167,155,254,190,94,234,
  234,151,16,8,4,200,204,204,226,15,255,232,47,120,240,193,71,45,156,100,86,195,13,15,13,217,85,111,158,31,214,199,2,73,182,232,185,248,166,166,
  118,33,185,185,5,191,18,203,183,100,233,74,155,77,49,124,212,220,12,233,23,131,81,147,36,119,118,93,102,102,22,11,23,46,250,149,152,206,236,236,
  92,106,23,212,89,66,159,159,4,115,234,133,23,183,15,163,116,251,162,36,195,230,65,48,57,166,0,28,96,233,250,172,94,189,20,128,99,199,142,18,
  10,9,45,16,36,9,69,150,56,113,252,24,13,13,155,152,83,53,159,126,131,181,120,228,209,175,37,156,230,123,127,247,93,6,6,7,45,246,193,246,
  143,51,108,13,140,109,152,104,84,69,146,53,250,251,7,120,227,141,87,216,180,105,27,127,246,103,255,21,128,230,203,151,216,179,103,39,209,104,140,166,
  166,139,100,100,102,209,176,126,43,13,235,183,50,56,216,199,248,248,152,53,33,159,56,126,132,210,210,89,60,254,248,55,196,181,31,253,136,138,138,217,
  182,3,185,174,129,14,53,53,11,233,233,238,230,242,229,38,100,211,235,202,184,183,83,39,143,83,95,191,132,57,115,170,57,114,228,16,179,202,43,121,
  226,9,113,188,19,39,142,80,94,94,73,44,166,18,142,68,121,251,173,215,121,228,209,175,241,204,55,126,27,128,206,142,54,50,51,179,137,70,85,171,
  13,162,145,24,253,3,3,188,253,246,107,108,216,176,133,63,254,147,255,3,16,229,237,239,188,253,166,176,53,112,176,42,178,44,91,160,245,201,39,159,
  73,104,211,127,249,151,31,88,19,127,52,26,179,190,143,68,98,232,154,78,56,18,53,0,154,70,56,20,225,173,55,95,99,251,253,15,243,196,147,191,
  102,49,46,31,124,240,62,45,45,45,182,65,167,44,158,203,186,134,13,128,40,229,95,188,120,153,235,188,223,249,219,255,143,211,167,78,80,93,93,203,
  211,79,127,203,250,249,135,31,238,166,179,179,67,56,163,199,43,10,107,58,177,152,230,156,115,13,147,86,135,10,186,41,136,40,84,253,80,53,137,214,
  214,43,172,89,211,64,65,126,145,149,123,244,230,155,175,114,255,3,143,240,71,127,252,151,128,144,87,120,239,189,183,209,13,189,174,194,162,98,218,218,
  90,153,154,154,178,192,228,173,24,245,122,145,4,180,232,58,5,5,133,44,91,182,250,87,62,118,85,213,124,170,230,86,211,120,238,92,34,123,242,57,
  239,142,38,40,192,27,11,73,89,86,88,189,102,61,57,57,121,191,242,57,86,172,88,203,197,11,23,105,110,110,142,91,80,122,29,212,139,47,249,184,
  178,117,203,214,219,178,151,91,57,8,138,80,218,21,226,137,102,217,174,123,240,136,183,161,48,243,22,100,89,22,137,219,86,14,141,195,33,92,211,45,
  134,196,233,87,228,58,183,67,209,218,252,185,44,219,98,139,96,104,234,56,20,171,77,113,76,107,229,39,75,60,253,244,111,210,209,209,206,251,187,222,
  117,8,42,170,22,51,227,118,225,214,19,146,70,157,213,87,72,88,237,33,73,238,60,39,203,216,87,146,28,192,207,86,6,55,127,103,13,125,78,143,
  42,231,32,172,137,4,103,221,104,47,39,88,0,80,140,237,77,69,145,19,211,24,244,184,103,131,46,168,123,204,173,48,251,25,57,219,212,124,142,130,
  161,209,45,214,208,18,165,148,37,251,188,62,161,90,238,84,200,115,219,157,204,100,209,162,185,238,245,122,83,128,249,59,77,213,172,207,152,146,7,62,
  163,202,232,143,254,248,207,56,115,250,36,31,125,116,208,250,59,243,89,58,251,17,8,145,203,63,248,246,159,242,193,7,187,56,118,236,40,49,35,23,
  43,102,246,3,79,83,233,166,198,4,243,189,246,249,20,124,126,133,148,148,0,247,222,115,31,27,54,110,33,18,137,184,236,131,236,14,41,225,243,9,
  153,138,88,44,234,50,114,182,62,163,131,63,16,160,241,220,25,94,121,229,37,38,38,38,45,22,217,236,255,159,231,51,50,223,125,89,114,235,38,229,
  231,231,113,207,189,219,201,201,201,17,57,145,134,33,176,211,150,36,37,37,141,188,188,66,34,145,48,3,3,189,46,192,105,142,57,170,170,161,40,10,
  39,79,28,231,200,145,195,68,34,81,145,47,104,44,146,62,142,169,180,23,94,120,140,210,103,66,42,137,252,20,21,13,89,146,136,197,64,150,181,132,
  165,165,19,20,152,147,170,236,72,244,52,7,24,115,176,211,140,45,54,219,215,40,110,16,144,196,228,174,105,98,32,113,158,83,118,168,82,75,72,168,
  178,102,39,85,42,50,15,60,248,40,179,103,207,225,133,231,159,99,108,124,140,250,250,37,100,102,102,211,118,245,10,145,72,212,2,71,170,42,170,249,
  76,240,160,235,26,170,105,218,235,76,169,74,48,134,181,175,65,138,179,44,137,111,59,51,143,198,4,42,206,45,11,93,119,84,241,232,137,98,159,201,
  218,202,108,3,77,5,85,211,81,52,217,177,234,116,156,215,241,51,251,28,110,111,55,39,216,148,37,27,232,57,93,216,85,77,71,55,65,135,46,163,
  107,146,157,87,36,107,174,54,50,239,199,188,71,43,23,201,0,104,86,114,183,3,32,186,26,35,25,117,97,42,133,235,226,184,200,18,146,46,161,26,
  191,62,116,240,0,219,238,184,139,253,251,247,57,170,35,117,35,255,11,235,25,73,18,44,89,178,156,80,104,154,227,199,142,185,237,49,188,185,231,214,
  198,4,227,57,155,253,50,22,83,57,113,242,56,231,47,156,195,239,83,132,30,80,192,208,85,242,41,232,232,164,166,166,177,118,237,54,210,210,50,216,
  183,111,7,83,83,19,136,156,185,152,81,56,16,19,192,32,166,17,14,133,173,247,212,85,12,160,127,222,247,237,124,63,237,69,223,216,216,56,239,190,
  251,182,80,241,247,251,8,6,125,134,38,146,144,30,145,101,153,194,194,18,182,108,185,159,225,225,1,14,30,220,5,186,96,81,163,209,24,209,168,216,
  10,15,135,163,68,162,49,194,225,136,49,54,233,94,255,244,226,171,3,148,110,87,103,114,93,215,45,112,131,42,161,203,226,123,73,75,254,89,231,220,
  23,95,17,147,200,62,57,253,196,244,56,191,52,99,154,55,142,99,158,211,60,166,74,114,207,37,89,150,144,85,153,221,187,119,178,125,251,131,124,243,
  91,191,7,64,56,28,98,255,254,15,56,215,120,142,104,52,102,177,36,206,138,170,120,3,215,120,91,20,231,220,109,94,155,83,199,104,166,92,130,120,
  195,217,248,118,136,7,99,9,19,146,197,56,105,110,42,222,178,35,209,18,124,218,226,15,53,211,121,146,249,225,153,43,92,39,19,100,215,236,235,70,
  31,208,4,147,149,164,26,41,209,123,203,125,15,78,247,243,153,112,82,162,34,177,163,10,211,16,33,53,229,2,14,31,57,204,236,57,115,88,179,182,
  129,253,251,247,90,246,47,241,12,72,106,106,26,13,235,55,240,227,31,255,7,209,152,221,7,132,91,189,102,73,21,120,113,243,96,73,211,84,84,73,
  108,237,118,117,117,27,226,173,62,33,186,26,240,89,226,145,126,191,66,90,90,26,177,88,12,208,25,28,236,103,124,124,204,202,75,51,65,146,5,150,
  76,22,73,83,45,121,18,151,68,196,231,187,114,20,174,139,186,134,164,131,170,65,56,18,38,54,18,51,236,142,236,123,23,95,66,211,44,51,51,199,
  122,7,198,198,198,172,188,62,39,80,138,68,98,214,189,139,113,74,244,77,205,33,179,225,245,81,47,60,70,233,11,58,32,138,9,199,156,52,147,11,
  160,57,29,219,109,239,54,221,242,116,75,14,172,244,235,248,56,153,165,244,186,229,37,101,158,35,209,97,94,114,200,13,40,244,247,247,243,211,159,254,
  216,210,51,50,39,255,88,76,117,109,245,197,251,158,153,19,184,123,226,119,140,144,14,31,42,115,27,81,146,64,85,175,215,38,137,224,241,122,109,17,
  15,20,226,75,177,147,79,88,218,140,199,138,7,122,51,131,55,136,223,10,73,124,54,186,161,75,149,60,217,52,89,114,185,19,24,125,252,92,32,221,
  241,28,205,107,213,172,115,253,252,231,63,75,10,118,157,162,168,177,216,56,255,227,127,252,149,213,23,236,109,86,13,51,161,220,139,91,224,153,117,28,
  158,132,24,130,166,118,251,155,64,192,231,83,137,70,21,252,254,160,43,79,110,122,58,108,1,165,104,212,13,24,220,239,168,110,187,2,124,193,238,95,
  188,255,186,225,247,232,254,185,185,93,30,139,9,160,100,230,35,106,154,206,244,116,196,26,139,108,176,164,90,32,41,22,83,103,24,159,188,240,226,203,
  12,148,110,211,142,238,170,236,208,109,173,33,244,153,230,50,3,36,57,74,91,101,67,209,47,158,69,145,28,160,195,156,160,21,67,32,50,97,171,203,
  56,191,228,248,19,23,16,48,143,161,200,160,171,150,11,185,166,106,46,70,194,204,185,49,183,222,174,15,212,192,13,29,226,128,133,81,130,111,13,144,
  215,97,134,36,75,52,79,128,199,4,208,147,68,150,192,98,174,100,187,29,228,36,96,201,109,112,171,185,116,145,102,186,45,75,62,32,41,208,101,70,
  118,200,236,15,34,215,93,79,170,109,227,100,161,146,182,173,126,171,125,48,254,41,216,46,235,166,68,133,9,116,226,159,169,27,100,218,237,104,94,159,
  166,217,19,190,183,80,191,245,5,148,89,85,105,116,8,145,71,227,144,115,80,85,13,159,34,12,177,131,129,136,245,108,194,161,40,33,3,44,56,193,
  82,44,38,0,131,157,191,104,188,167,90,220,123,242,133,185,127,29,85,215,208,101,9,93,19,227,141,221,6,26,106,76,19,42,255,62,153,112,56,102,
  245,189,208,116,216,2,146,162,10,52,102,105,151,89,5,37,38,72,138,79,5,240,250,169,23,30,163,244,197,1,72,241,44,136,115,203,41,145,105,177,
  39,51,115,75,202,57,57,57,147,123,221,172,67,242,137,214,201,122,36,155,236,156,137,223,130,76,113,31,211,92,233,57,153,40,39,139,16,191,74,139,
  23,137,188,17,59,20,207,250,152,32,195,45,174,232,22,166,51,183,33,33,113,171,206,185,253,104,131,68,92,150,46,201,236,93,236,228,105,205,200,227,
  146,103,100,166,146,61,223,153,64,145,174,39,126,78,138,243,248,155,249,62,76,246,49,113,155,47,185,16,231,205,0,37,174,203,170,57,89,172,120,113,
  77,231,51,77,44,60,240,102,157,143,11,18,204,103,44,203,56,12,174,227,183,210,53,84,69,67,85,5,80,48,159,71,56,34,228,67,98,49,205,98,
  85,204,47,23,72,112,120,76,126,17,88,21,119,223,115,223,191,120,255,36,23,203,107,130,30,159,42,19,51,100,64,116,77,39,28,142,218,146,26,198,
  226,45,26,85,93,69,46,137,204,176,39,95,225,133,7,148,190,128,96,9,87,142,145,89,233,22,63,217,196,179,63,178,236,206,77,50,39,247,245,235,
  55,114,240,224,126,23,219,16,159,147,20,239,45,231,60,38,216,190,79,86,162,113,92,181,92,188,31,152,9,28,196,49,192,148,180,138,175,200,154,137,
  121,152,137,185,153,41,135,70,146,226,21,164,237,68,111,211,4,216,108,71,91,45,220,86,63,183,7,88,221,213,142,194,79,74,153,17,96,57,87,224,
  206,109,184,248,1,254,70,249,82,241,101,251,51,1,21,103,181,159,43,49,221,144,95,48,96,156,75,15,198,121,237,201,183,253,220,231,137,239,3,201,
  128,146,187,191,200,168,170,102,73,75,184,1,95,114,118,203,41,124,249,85,136,95,69,147,39,89,219,153,96,201,220,106,215,117,213,245,57,77,211,241,
  249,116,52,77,38,26,141,90,237,29,139,170,198,22,155,27,44,152,255,191,222,54,109,178,252,190,207,170,189,146,53,159,185,253,238,76,21,16,95,138,
  227,253,148,173,247,66,211,117,3,20,169,142,237,57,53,161,194,212,9,230,61,124,228,133,7,148,190,224,32,201,105,182,26,111,86,59,83,242,181,249,
  55,230,49,214,173,91,207,166,205,219,56,126,252,136,53,145,59,87,158,85,85,115,121,230,27,223,228,231,63,123,142,150,150,230,184,73,206,102,86,230,
  206,157,203,83,79,255,58,207,255,226,39,180,180,180,184,6,88,19,92,152,118,2,27,55,110,102,219,29,119,241,215,255,243,191,59,182,222,76,192,229,
  78,144,118,2,24,49,169,187,75,123,237,173,45,243,126,227,153,53,221,152,164,165,4,160,231,54,178,181,191,175,170,18,247,242,194,243,63,163,181,181,
  197,33,83,160,25,131,175,173,112,158,158,158,198,214,109,119,82,91,179,144,96,74,138,213,254,237,237,87,105,106,186,192,169,147,39,29,237,160,185,38,
  18,39,184,72,86,186,111,111,71,105,9,121,22,206,190,96,183,147,13,146,26,26,54,176,101,235,157,252,205,95,255,21,235,214,173,103,203,214,59,249,
  171,255,254,223,44,166,64,115,148,114,219,38,159,78,70,41,49,231,205,41,165,144,200,68,217,9,244,78,214,193,92,185,155,125,202,85,113,151,80,134,
  254,213,92,157,59,89,98,39,144,159,1,22,37,253,251,228,11,5,221,0,196,134,10,188,150,104,71,163,27,21,94,102,155,199,98,49,7,56,208,226,
  68,84,147,131,164,100,96,250,122,140,239,175,250,172,19,219,203,205,174,186,153,30,103,91,152,224,73,228,110,153,231,87,29,239,165,153,159,228,220,94,
  180,251,115,226,182,249,76,76,172,199,48,121,225,1,165,207,117,64,149,92,142,244,166,94,72,60,163,225,44,233,55,127,103,106,45,153,64,73,146,36,
  124,126,5,128,148,20,191,152,60,37,44,125,32,161,94,221,201,119,191,243,63,209,52,141,96,48,144,0,54,44,221,22,191,104,74,179,178,70,208,245,
  18,160,34,73,90,156,238,146,100,125,214,26,172,84,45,14,76,200,14,64,163,184,128,158,169,226,227,28,244,157,223,199,51,49,194,147,202,222,54,112,
  2,77,243,250,157,128,211,188,46,127,64,33,16,244,27,229,243,186,69,217,155,215,49,167,106,46,79,62,249,12,29,29,109,236,120,239,13,174,117,182,
  35,73,18,133,69,37,44,91,182,138,187,238,218,78,109,109,29,175,188,252,188,149,227,96,131,186,120,22,70,143,219,6,197,177,157,169,37,169,118,195,
  85,177,232,124,198,146,44,113,234,212,49,78,159,57,78,48,232,183,18,231,21,69,177,6,121,115,43,198,188,119,231,49,102,154,244,226,193,184,253,44,
  220,32,222,206,229,48,19,178,85,199,36,163,37,48,100,95,229,112,79,250,51,251,26,38,3,80,55,211,118,51,19,85,238,220,49,115,17,163,233,36,
  0,131,248,235,117,46,84,110,102,171,54,30,188,56,1,159,19,228,221,204,61,197,183,151,123,177,112,243,224,204,76,1,112,178,66,138,162,88,247,30,
  207,134,59,23,13,241,41,14,150,6,155,35,255,240,179,100,215,188,240,226,83,7,74,183,91,73,167,120,103,133,131,183,226,147,153,95,51,159,111,124,
  227,91,180,183,95,165,178,114,14,0,125,125,61,188,249,230,171,12,15,15,33,73,112,247,221,247,187,84,154,251,251,123,217,183,239,125,10,11,139,105,
  104,216,12,192,159,252,233,255,201,155,111,190,200,131,15,62,73,127,127,47,133,133,194,181,126,199,142,55,184,247,222,135,120,245,213,231,105,111,191,74,
  118,118,14,219,183,63,66,81,145,176,155,184,124,249,34,239,190,251,58,126,3,112,249,3,62,130,41,126,150,46,88,193,93,119,221,199,238,221,59,217,
  183,111,175,197,56,40,62,241,5,16,8,250,144,101,153,45,91,238,102,254,252,90,94,127,253,101,138,138,74,216,178,229,14,254,250,175,255,10,208,81,
  20,153,223,255,253,63,97,116,116,132,151,95,254,5,79,60,241,12,217,217,57,132,66,211,214,53,28,61,250,17,123,247,238,166,178,114,14,95,251,218,
  55,18,218,108,100,100,152,239,125,239,187,182,221,55,2,232,20,20,20,240,248,227,95,163,164,164,212,186,151,119,222,121,221,2,143,75,150,44,231,177,
  199,158,50,218,180,151,221,187,223,165,187,187,11,144,40,45,45,229,201,39,159,97,207,158,247,184,112,225,172,0,79,62,25,73,146,25,30,238,231,131,
  15,222,225,210,165,70,238,191,255,49,214,174,107,224,216,177,195,104,154,198,202,149,107,89,191,126,139,245,156,82,82,82,25,29,29,225,181,215,94,224,
  209,71,191,78,86,86,14,225,112,200,106,255,61,123,118,50,107,86,165,101,0,124,238,220,41,118,238,124,23,208,89,179,102,61,171,86,173,37,24,20,
  76,214,216,216,40,39,79,30,225,194,133,179,44,95,190,134,181,107,55,242,131,31,124,7,89,17,247,163,248,100,163,156,217,105,62,44,83,93,93,205,
  246,237,15,146,147,35,188,176,246,238,221,205,225,195,7,193,176,68,89,189,186,129,20,131,45,59,126,252,35,14,28,248,16,128,199,30,123,154,236,236,
  235,95,239,233,211,39,120,243,205,215,209,17,101,228,146,46,146,107,61,183,117,135,211,189,100,74,43,232,46,32,114,163,237,56,203,99,12,67,21,61,
  142,197,113,231,246,153,98,176,88,197,11,24,234,250,154,174,51,61,61,77,40,52,141,166,169,118,169,187,241,123,36,241,234,232,218,245,153,163,235,64,
  19,155,167,212,237,107,178,216,41,201,102,124,144,102,6,76,241,237,229,212,84,141,103,55,227,207,167,235,186,208,38,51,187,190,132,37,105,48,61,61,
  69,79,79,55,61,61,221,110,169,3,199,103,101,69,178,139,11,92,88,79,55,45,13,237,223,199,221,143,7,150,188,240,24,165,207,97,112,181,89,16,
  197,98,11,178,179,115,120,225,133,31,19,14,135,121,228,145,175,115,247,221,219,121,229,149,231,89,181,106,29,139,23,47,227,165,151,126,74,127,127,47,
  179,103,87,177,125,251,99,172,88,177,134,61,123,222,37,61,61,141,21,43,26,248,209,143,190,111,177,50,89,89,57,252,252,231,255,78,102,102,150,53,
  24,250,12,177,186,251,239,127,148,96,48,133,127,249,151,239,51,107,86,37,219,183,63,194,208,80,191,48,181,53,62,183,116,233,114,54,109,186,139,93,
  187,222,225,232,209,35,214,150,140,201,218,72,146,1,148,2,126,182,108,185,139,170,170,249,188,254,250,139,116,119,95,163,180,180,204,248,157,98,29,207,
  188,231,64,192,143,44,75,100,101,101,115,225,194,89,94,124,241,167,44,95,190,154,134,134,205,12,12,244,210,212,116,145,191,255,251,191,177,88,155,234,
  234,90,30,124,240,49,62,248,96,151,113,12,123,210,240,249,20,158,120,226,107,164,164,164,242,195,31,254,29,115,230,84,177,125,251,35,12,12,8,227,
  94,128,162,162,98,126,241,139,127,39,20,10,243,248,227,223,224,206,59,239,227,133,23,158,67,146,36,238,188,243,62,78,158,60,74,83,83,35,37,37,
  101,108,218,116,39,5,5,69,68,34,97,206,157,59,193,138,21,13,188,249,230,11,52,53,53,178,124,249,26,46,93,106,164,172,172,130,245,235,183,176,
  119,239,46,26,27,79,179,117,235,61,44,92,184,152,241,241,81,2,1,1,26,179,178,178,217,183,239,24,47,191,252,51,30,124,240,9,182,110,189,155,
  107,215,218,249,225,15,191,195,138,21,107,88,187,118,19,87,174,92,6,96,195,134,45,28,62,188,143,83,167,142,145,157,157,203,3,15,60,198,186,117,
  155,105,105,185,136,207,0,163,126,191,15,69,145,172,246,215,12,17,76,115,34,205,207,207,231,177,199,190,70,123,123,43,63,250,209,63,178,113,227,54,
  54,111,190,131,254,254,62,82,82,82,216,180,105,27,7,14,124,200,137,19,135,89,188,120,25,91,183,222,131,44,43,28,62,188,207,122,22,241,215,219,
  217,217,206,179,207,254,45,203,151,175,102,253,250,45,92,184,112,158,166,166,75,198,86,134,120,150,170,234,77,30,201,152,145,120,102,47,30,0,196,179,
  37,51,21,92,36,50,48,82,66,110,157,36,9,38,113,122,122,138,29,59,222,198,231,243,49,49,49,225,144,16,49,243,247,116,99,155,253,230,239,43,
  185,246,152,205,184,216,118,72,118,161,135,105,66,125,61,92,97,182,151,153,131,39,73,178,131,77,157,73,18,195,205,120,202,14,229,126,128,246,246,54,
  254,225,31,254,110,134,243,200,8,185,11,41,161,168,34,193,238,199,85,221,41,89,249,119,201,42,80,189,240,226,246,2,74,183,91,255,213,237,50,108,
  89,146,68,137,63,112,234,228,81,134,6,7,144,36,137,211,167,143,177,105,211,157,148,151,149,115,246,204,113,206,55,158,180,128,70,127,95,55,19,19,
  99,164,4,83,196,54,156,9,98,100,197,74,127,232,238,234,96,98,98,156,137,241,49,102,85,204,182,126,95,92,92,74,97,97,49,135,15,239,39,22,
  141,112,229,202,101,254,241,135,223,69,150,37,139,205,42,42,42,97,229,202,6,246,238,221,197,249,198,51,40,178,130,38,235,22,123,225,115,88,158,172,
  92,177,134,185,115,107,120,247,221,87,25,25,25,196,239,243,89,192,47,224,247,59,128,146,24,108,252,126,97,179,16,137,132,57,123,230,56,138,44,211,
  218,122,153,134,134,205,228,230,228,137,207,26,147,78,126,65,17,119,223,189,157,35,71,14,113,233,210,69,20,135,255,154,172,72,204,42,159,69,113,113,
  41,7,246,127,72,52,18,225,74,203,101,126,248,131,239,0,80,97,220,203,165,139,231,24,31,31,71,2,186,186,58,88,176,96,17,126,159,194,172,138,
  57,20,22,22,243,222,142,87,201,205,201,227,129,7,30,103,112,160,143,255,252,207,103,41,47,171,224,174,187,31,6,96,122,106,146,222,222,46,106,106,
  234,41,41,46,101,222,188,90,6,6,250,104,186,212,136,207,231,227,208,193,15,152,55,175,198,0,110,62,67,239,38,204,229,203,231,145,101,153,174,174,
  14,202,203,43,57,114,228,128,113,175,45,172,93,187,137,220,220,60,46,156,63,195,191,253,235,223,35,201,18,1,191,143,112,120,146,158,158,107,204,159,
  95,103,128,46,131,225,243,217,192,212,122,22,142,28,163,250,186,69,164,164,164,112,236,216,97,20,89,230,208,193,189,124,116,104,47,186,174,115,255,253,
  143,210,223,223,203,233,83,199,240,41,10,151,46,10,80,88,87,183,132,99,199,14,90,207,226,114,83,35,178,44,211,109,92,239,209,163,226,122,175,180,
  92,102,253,250,45,228,229,229,33,33,35,75,50,170,174,89,18,17,94,97,155,99,2,55,254,83,226,42,40,221,149,173,36,213,221,186,30,40,153,41,
  135,77,60,15,241,59,53,22,163,237,106,171,181,192,48,70,23,100,73,55,84,248,13,113,91,205,125,204,248,202,73,115,155,234,122,219,170,9,160,2,
  208,17,54,56,86,149,172,238,102,203,146,183,23,214,61,152,210,28,138,162,0,51,139,170,38,84,251,198,201,89,88,44,145,46,33,75,118,23,149,21,
  159,237,169,232,136,132,156,78,64,115,229,68,25,154,255,94,95,247,194,99,148,62,135,65,213,177,226,51,173,46,6,7,251,13,106,93,102,120,72,176,
  59,57,185,185,140,140,12,82,92,92,66,77,237,34,36,160,180,108,22,25,25,89,76,76,140,11,144,149,100,127,127,192,96,84,156,20,182,36,9,134,
  5,96,98,124,84,88,133,24,91,40,146,131,3,95,185,178,193,248,204,152,117,125,38,24,147,205,28,43,227,160,171,86,111,32,18,9,51,57,49,142,
  34,75,104,14,165,112,51,111,70,241,41,22,11,164,200,66,43,104,112,160,207,30,244,176,171,239,20,69,12,156,129,64,10,247,221,247,48,163,163,35,
  236,223,247,1,102,250,135,121,108,89,146,40,41,45,53,182,171,70,18,115,115,140,17,177,127,160,47,97,85,175,40,10,69,69,197,116,119,117,16,141,
  70,89,189,122,35,225,112,152,247,222,123,29,36,232,238,238,100,200,104,255,104,52,66,44,22,177,142,81,80,80,68,119,119,7,178,16,165,66,150,101,
  6,7,251,141,123,19,123,34,131,131,253,86,155,71,34,97,241,255,112,24,73,182,39,54,89,22,140,88,106,90,26,171,86,174,7,9,242,243,11,201,
  203,43,180,174,209,61,209,198,247,29,201,242,180,203,201,21,219,109,227,99,99,46,153,4,77,211,40,42,46,161,179,179,205,182,185,81,196,245,46,88,
  176,136,156,236,28,241,44,6,251,140,99,233,132,141,235,13,135,195,224,112,149,151,92,12,128,231,184,238,126,46,82,2,147,148,157,157,197,220,185,115,
  9,4,131,12,14,12,208,217,217,78,188,224,104,122,122,58,179,42,42,105,187,122,149,169,169,169,4,73,16,119,114,183,68,94,94,46,193,96,144,254,
  126,49,78,228,230,230,18,8,4,24,24,232,119,189,252,162,164,30,100,5,210,82,211,169,168,168,164,185,185,153,72,36,108,49,74,146,4,181,11,22,
  210,211,221,195,200,200,112,194,248,161,235,144,145,145,65,122,70,6,61,221,221,174,107,179,115,39,37,135,84,129,96,109,4,195,100,51,49,51,180,156,
  43,87,83,150,37,178,179,115,88,191,126,61,146,36,49,52,52,200,217,179,103,12,165,113,123,11,204,233,235,104,31,3,151,71,36,186,221,94,186,14,
  89,89,153,44,91,190,2,73,146,56,123,230,52,185,185,185,132,66,33,186,186,174,161,235,226,61,116,230,59,9,32,169,97,23,183,122,85,113,94,124,
  57,66,190,157,7,90,89,150,44,160,16,137,58,38,83,199,168,181,116,217,42,30,124,232,235,100,102,102,129,4,231,27,79,49,49,49,46,38,67,221,
  54,27,53,245,70,172,21,146,37,209,239,88,153,57,77,47,29,98,128,154,195,180,244,248,241,67,12,12,244,177,113,211,157,46,160,229,252,67,243,48,
  199,142,30,16,224,106,85,131,107,32,52,217,39,201,88,45,154,63,53,133,244,76,49,61,205,85,126,98,154,227,74,108,222,114,23,193,96,144,119,222,
  126,221,49,117,56,62,55,131,247,155,165,233,100,138,75,170,154,248,210,52,7,11,3,89,153,217,140,79,140,35,73,18,115,170,170,105,187,106,84,3,
  58,180,154,122,186,59,209,117,221,2,47,209,104,216,186,0,217,145,116,45,205,176,109,161,187,18,184,53,203,211,205,60,70,65,97,49,207,60,243,187,
  228,23,20,34,1,29,237,87,104,105,185,232,92,70,91,128,199,153,116,154,108,91,102,166,254,117,253,173,21,43,69,195,229,21,39,218,65,179,124,240,
  236,182,213,147,108,109,120,17,175,129,37,203,18,203,87,172,160,166,182,22,77,211,88,190,124,57,91,182,108,179,114,251,20,159,216,190,206,204,202,98,
  233,210,229,100,102,102,10,3,92,163,72,195,220,142,247,249,196,151,89,244,81,95,191,136,71,30,125,156,140,140,12,100,89,226,142,59,239,102,93,195,
  6,235,29,147,13,224,44,140,182,133,169,114,106,106,42,139,151,44,37,37,37,136,98,156,67,156,71,97,201,146,101,20,21,23,185,126,166,40,10,245,
  245,139,73,75,75,35,35,35,131,226,162,34,235,122,220,95,138,67,218,68,178,36,70,146,109,51,206,212,103,156,76,89,65,65,62,165,101,101,92,184,
  112,158,121,213,213,204,157,55,15,197,39,83,90,86,74,94,94,190,117,206,236,236,44,202,202,202,173,34,152,64,208,79,97,81,17,169,105,105,4,3,
  65,10,139,138,172,107,144,101,137,13,27,55,161,105,42,77,151,46,18,10,77,91,21,112,249,249,249,148,148,148,144,151,151,71,32,224,39,53,45,149,
  242,242,114,203,148,220,41,205,113,43,186,100,94,120,225,49,74,159,220,208,154,192,18,0,20,22,20,49,62,54,138,142,96,23,0,58,59,218,88,186,
  116,21,87,175,54,243,225,158,29,0,164,164,164,176,108,249,90,81,105,22,211,172,210,88,211,191,200,4,77,49,85,53,64,144,168,163,85,53,141,190,
  62,225,172,157,145,153,229,50,102,149,37,201,58,78,95,111,15,237,109,173,60,246,248,55,88,177,114,45,7,15,236,115,1,0,205,97,228,122,226,196,
  17,116,29,86,175,217,64,99,227,25,122,123,186,173,217,60,16,12,18,14,135,208,116,157,96,32,200,56,118,53,142,216,46,176,203,150,237,137,91,103,
  233,210,213,204,157,59,159,29,59,222,96,100,100,88,76,222,14,103,115,147,210,239,233,233,6,68,62,150,166,105,72,186,100,9,102,154,199,84,53,161,
  163,34,27,62,122,226,103,186,97,46,43,34,16,8,10,38,197,0,96,101,101,179,200,203,43,164,177,241,20,0,21,21,85,244,244,92,99,96,160,159,
  193,193,62,242,243,11,237,73,81,146,200,203,47,100,112,176,223,48,216,20,244,125,44,166,18,141,169,150,122,185,80,8,22,165,218,38,16,153,61,123,
  30,145,72,152,55,223,248,165,197,50,221,117,215,67,226,26,85,251,185,138,188,32,205,2,77,150,16,38,34,243,194,100,4,178,178,178,233,239,159,118,
  232,29,137,103,89,88,88,108,149,245,107,104,228,231,23,18,14,135,25,30,30,178,182,78,132,63,155,102,95,175,33,84,168,169,186,11,84,39,47,219,
  246,194,197,244,1,169,41,41,116,119,119,115,190,241,44,253,253,189,108,217,188,149,242,242,89,4,131,65,102,207,174,98,104,112,128,158,222,110,98,177,
  40,138,34,179,100,201,82,202,203,43,56,127,225,28,41,193,20,162,209,40,173,173,87,88,179,102,29,103,206,156,34,20,10,161,233,26,153,153,153,228,
  229,231,49,58,34,145,149,149,69,111,111,15,138,34,83,87,183,136,242,242,10,134,134,7,57,118,244,8,133,133,69,44,94,188,20,159,223,47,242,230,
  178,179,169,93,176,144,211,167,78,80,95,191,152,222,190,30,34,225,16,186,166,50,119,238,60,234,234,23,209,218,218,194,64,127,63,247,222,187,157,139,
  23,207,211,116,249,18,160,19,12,6,88,181,106,13,89,217,217,116,118,180,115,254,252,121,22,47,94,66,78,78,14,193,148,20,14,236,223,79,127,255,
  128,120,199,12,27,38,219,14,73,154,33,169,59,14,204,75,130,197,28,27,27,37,22,141,18,9,135,88,183,182,129,236,156,28,252,254,0,23,47,158,
  103,98,124,130,213,171,215,16,142,132,137,70,162,156,57,115,138,77,155,183,50,61,61,77,86,86,22,131,131,3,100,164,103,210,213,125,141,99,71,143,
  24,172,248,56,101,101,179,104,186,116,137,104,52,202,236,217,179,25,28,28,0,160,178,114,14,249,5,5,124,244,209,33,242,243,10,240,251,253,44,92,
  88,199,251,239,239,178,24,51,83,55,204,102,169,188,78,239,133,199,40,125,70,225,86,133,53,255,191,100,233,42,210,51,50,73,79,207,100,241,226,149,
  180,180,52,49,49,57,201,232,232,8,249,121,133,72,178,130,36,43,108,218,116,15,129,64,16,93,211,13,23,112,213,152,104,125,68,34,81,99,98,86,
  137,132,77,199,112,241,251,88,52,70,87,215,53,250,251,123,89,184,96,49,138,226,39,61,61,147,223,253,221,63,225,238,187,31,180,38,201,104,76,165,
  171,171,139,115,231,78,179,114,229,90,50,179,114,92,38,183,102,217,184,216,90,138,113,244,232,71,140,141,141,178,113,227,29,196,98,42,195,195,98,226,
  94,176,96,17,154,170,179,100,201,10,2,65,227,122,163,49,171,84,223,52,234,180,196,226,84,141,188,188,66,214,172,221,200,161,67,123,185,116,241,130,
  173,1,163,105,142,107,16,95,93,93,93,244,246,118,179,104,241,82,252,254,32,25,25,89,252,222,239,255,41,247,109,127,216,158,240,163,170,101,136,105,
  94,115,44,26,99,116,116,148,188,188,66,84,77,227,234,213,102,106,106,234,201,204,204,162,32,191,136,117,13,91,193,216,146,186,227,206,7,201,200,200,
  226,208,161,15,80,85,149,214,214,203,228,231,23,50,191,166,14,93,215,169,171,95,78,32,16,4,93,23,192,200,0,116,225,112,148,72,216,190,55,167,
  33,169,9,126,66,211,211,4,2,65,178,179,243,136,70,99,44,89,186,134,146,210,89,198,231,163,22,216,139,70,99,168,49,27,52,105,154,168,62,51,
  1,235,133,243,231,9,133,66,172,92,181,6,85,213,152,55,175,134,63,253,179,255,202,138,21,107,184,212,116,129,194,194,98,150,46,91,69,52,170,82,
  90,58,139,5,11,22,209,216,120,154,72,36,106,181,101,36,28,37,18,142,90,215,27,117,152,136,154,128,83,104,211,120,254,88,55,195,20,171,14,246,
  112,100,120,136,222,190,94,22,47,94,194,220,170,121,92,235,108,103,94,245,124,10,11,139,136,70,132,245,200,181,107,157,12,12,246,179,106,213,26,2,
  129,0,179,231,204,161,162,162,146,188,252,124,52,93,195,231,243,161,198,98,116,119,119,145,155,147,203,130,5,117,180,182,182,48,61,61,205,220,185,213,
  204,170,152,205,145,35,135,200,206,202,102,213,170,53,44,91,182,130,246,246,54,90,154,47,35,73,18,153,25,25,204,158,61,7,73,146,40,159,85,65,
  78,78,174,24,55,36,137,190,190,30,58,218,219,88,180,104,9,62,159,143,107,93,157,52,54,158,37,24,12,82,90,86,206,154,53,107,81,20,153,147,
  39,142,50,119,94,53,243,231,207,103,94,245,124,6,6,7,232,235,237,99,253,250,13,14,144,40,221,52,227,232,4,75,106,44,70,126,94,62,15,61,
  252,8,145,72,132,80,40,68,69,229,108,222,223,245,30,23,47,52,82,83,179,128,13,27,55,113,250,244,73,62,220,179,27,127,192,79,77,205,2,34,
  145,8,39,142,31,101,124,108,156,254,190,126,78,159,57,73,89,89,57,62,159,88,63,31,58,116,128,150,230,38,238,187,239,126,230,206,157,139,226,243,
  17,8,166,112,169,233,18,67,195,67,92,184,112,158,220,220,92,114,243,114,233,233,237,166,180,172,140,188,188,188,155,98,100,189,240,194,99,148,62,11,
  184,100,37,16,26,214,3,161,16,223,248,198,239,0,208,209,209,198,174,157,239,18,9,199,56,122,228,16,247,223,255,8,223,250,214,183,1,104,105,110,
  162,179,179,141,194,194,98,66,161,8,23,47,94,96,233,210,85,252,246,239,252,49,111,188,241,138,49,161,170,68,34,81,193,22,68,98,198,132,171,18,
  14,71,121,227,245,87,120,224,193,71,249,131,111,255,25,0,109,109,173,236,216,241,22,197,70,137,125,212,176,64,216,243,193,46,170,171,107,217,178,101,
  27,47,189,248,2,127,252,39,127,193,200,200,48,63,255,217,115,214,68,16,14,71,145,36,216,249,222,59,60,241,228,211,44,93,182,138,99,71,63,98,
  86,69,37,13,13,155,105,104,216,76,71,71,27,253,125,189,198,132,28,51,252,171,4,152,208,52,157,104,204,6,15,117,11,23,3,88,127,235,140,239,
  255,253,255,230,215,191,249,155,12,15,15,241,147,231,254,19,36,120,229,149,151,120,244,209,39,248,195,63,250,11,251,94,222,125,147,162,226,82,11,244,
  69,34,81,163,74,75,181,128,199,165,75,231,89,179,102,3,37,197,229,28,57,124,128,59,238,220,206,227,143,127,147,241,241,49,46,93,106,100,229,202,
  6,30,125,244,215,232,234,234,224,245,215,158,103,108,108,4,85,211,105,110,110,34,51,35,139,245,235,183,177,126,253,54,6,7,250,24,31,31,67,211,
  117,98,209,152,197,126,69,34,49,162,134,109,132,121,206,72,36,70,216,1,100,79,157,58,193,220,121,53,60,242,232,211,128,144,124,56,127,254,12,117,
  117,75,200,204,204,179,254,214,180,100,48,255,78,85,53,254,228,79,255,130,145,225,97,126,250,211,255,100,96,112,128,87,95,125,145,251,238,123,128,191,
  252,47,255,55,0,135,15,31,226,208,161,253,32,73,100,101,102,179,122,77,3,27,54,8,73,131,253,251,246,112,244,232,33,193,84,105,34,9,55,20,
  138,138,123,48,129,93,84,77,0,118,78,35,85,47,110,176,122,147,101,84,163,95,167,165,103,144,158,158,129,174,235,100,101,103,147,149,37,152,153,88,
  76,216,142,228,231,23,80,90,86,70,104,122,26,69,81,232,236,108,167,164,164,148,53,107,215,113,242,196,113,162,145,40,193,96,10,105,233,233,92,190,
  220,68,105,105,25,57,185,185,28,58,184,159,69,139,150,80,88,88,196,200,240,16,195,195,195,116,117,117,81,91,187,144,80,104,154,214,214,22,82,82,
  82,88,176,96,161,152,244,157,137,219,6,171,29,8,4,88,182,124,165,197,58,11,166,55,198,232,232,8,41,41,41,200,178,66,126,65,33,103,207,156,
  98,96,96,128,161,161,33,138,139,75,24,30,30,98,104,112,16,144,168,168,168,252,24,99,31,174,132,112,191,63,64,111,95,15,111,190,241,26,15,62,
  248,48,101,229,229,130,61,79,77,19,139,44,227,15,82,82,82,241,251,3,200,178,34,100,17,52,141,104,44,74,76,141,17,141,70,80,85,205,216,242,
  23,133,31,233,233,105,156,107,60,139,226,243,49,119,110,181,240,143,83,99,204,159,95,195,172,89,21,188,250,202,139,52,172,223,68,48,40,22,59,7,
  246,239,101,116,116,204,235,192,94,124,249,22,112,155,55,109,190,109,70,110,179,92,87,148,234,251,240,251,125,212,212,212,240,245,167,126,141,87,94,126,
  158,171,87,91,49,19,38,77,246,198,170,160,113,228,250,72,146,100,177,23,102,104,154,189,93,131,110,10,56,234,86,249,173,249,199,178,36,185,182,186,
  140,186,14,171,218,204,252,57,113,30,110,206,149,160,41,114,104,30,211,249,183,146,145,28,46,57,238,217,252,157,101,161,128,48,246,20,66,121,182,5,
  9,70,194,183,174,139,28,38,85,85,13,3,80,213,225,189,38,37,88,192,88,201,156,216,121,70,206,210,101,145,183,97,88,149,72,18,126,191,194,230,
  205,119,48,191,102,1,111,188,254,75,38,38,198,93,73,210,102,3,233,186,206,194,133,139,217,180,249,46,118,236,120,131,43,45,77,86,178,180,185,93,
  246,244,211,191,69,71,71,27,59,119,190,99,49,96,17,7,11,228,220,126,48,115,60,156,125,33,222,243,205,188,119,211,172,83,85,5,83,101,182,131,
  83,204,210,62,142,226,222,206,176,197,156,145,146,10,74,26,57,71,198,231,196,191,53,107,91,216,89,33,228,116,171,183,60,198,84,125,198,106,173,175,
  18,115,228,150,4,176,21,226,239,186,251,30,114,114,178,105,190,124,153,138,202,74,38,39,39,185,210,210,196,170,85,107,105,239,104,99,112,112,16,85,
  141,177,98,197,106,134,134,68,177,70,107,107,11,245,245,139,121,225,249,159,209,208,176,145,69,139,151,240,147,159,252,7,147,19,147,164,103,164,179,97,
  195,102,78,159,58,201,202,85,171,81,20,153,125,123,63,100,243,150,109,156,63,127,142,133,11,235,233,232,104,103,246,236,57,52,54,158,101,246,236,57,
  2,68,68,163,204,174,156,195,251,187,119,114,199,29,119,209,216,120,142,101,203,86,112,224,192,94,106,230,215,114,173,235,26,181,181,11,184,112,161,145,
  5,11,234,56,120,112,63,203,151,175,164,175,183,135,137,201,9,74,75,203,232,237,233,161,162,114,54,189,61,61,148,150,149,113,242,196,113,150,45,95,
  193,137,19,199,81,100,133,229,43,86,242,252,47,126,225,218,22,158,201,184,57,62,233,221,20,140,173,172,172,100,201,146,165,236,220,185,131,141,155,54,
  211,223,223,75,126,126,1,25,25,25,248,20,31,151,46,93,36,18,137,176,116,217,114,166,167,167,137,70,35,52,53,93,98,209,162,37,28,61,114,152,
  69,139,151,208,211,221,205,212,212,20,245,245,139,248,240,195,15,136,70,35,212,47,90,76,81,81,17,105,169,105,52,54,158,163,164,180,148,137,137,9,
  234,235,234,153,154,158,166,173,237,42,83,147,147,148,207,170,96,114,98,130,222,222,62,154,155,47,199,169,122,95,199,132,218,11,47,110,151,241,106,211,
  166,77,183,37,80,82,20,1,150,106,106,230,243,204,51,223,226,249,231,127,66,107,235,21,235,229,116,170,92,39,122,114,145,240,226,58,21,159,157,38,
  178,241,202,183,201,189,214,220,91,131,246,170,111,166,201,208,89,254,108,172,81,29,64,73,142,19,108,49,193,130,211,202,195,89,170,236,82,165,118,148,
  46,155,91,61,98,98,118,151,247,58,75,165,45,196,102,109,109,226,240,173,115,131,59,201,0,74,62,159,194,189,247,62,192,146,37,203,57,119,238,52,
  231,207,159,102,96,192,174,22,172,171,91,194,188,121,181,84,84,204,102,247,238,29,156,61,123,146,251,239,127,148,202,202,57,60,255,252,115,140,141,141,
  176,104,209,82,182,110,189,135,183,222,122,133,198,198,115,14,199,114,213,98,129,220,237,34,197,25,223,202,51,154,4,235,174,100,110,221,229,85,23,15,
  124,146,183,195,204,91,32,241,58,56,241,246,49,241,166,182,182,239,159,219,248,216,3,74,246,187,224,156,248,75,74,74,169,170,170,2,96,98,98,156,
  214,214,43,168,106,140,89,179,42,40,43,43,99,100,100,148,206,206,54,242,243,11,25,26,26,164,162,98,54,129,128,159,241,241,49,174,94,189,202,198,
  141,155,81,20,133,15,63,220,131,170,170,248,253,126,138,139,139,25,28,28,36,35,35,3,85,85,153,154,154,164,180,180,140,206,206,118,202,202,202,41,
  43,155,197,192,64,63,173,173,45,100,102,102,81,83,83,203,244,244,52,227,227,227,92,187,214,65,101,229,28,10,10,10,24,29,29,161,163,163,157,252,
  252,66,70,70,134,200,203,203,167,160,160,144,241,241,113,58,58,218,200,204,204,162,188,124,22,29,29,29,248,253,126,186,187,187,168,169,169,37,43,43,
  155,246,246,118,122,122,186,169,168,168,164,175,79,188,43,185,185,121,180,182,182,38,237,31,230,123,156,12,92,138,54,19,237,21,12,6,73,79,79,99,
  98,98,130,140,140,116,116,93,39,28,14,83,85,85,197,196,196,36,253,253,226,92,5,5,5,164,167,103,208,209,209,142,174,235,164,165,165,51,57,57,
  65,106,106,42,145,72,196,112,31,72,97,114,114,18,93,215,201,201,201,161,184,184,152,177,177,49,250,250,122,73,75,75,3,32,43,43,11,159,207,143,
  207,231,163,163,163,157,148,148,20,10,11,139,25,29,29,165,187,187,219,82,240,55,199,143,175,250,162,192,139,47,3,80,218,120,123,1,37,211,99,204,
  4,76,53,181,53,124,243,155,191,201,207,126,250,159,52,55,183,88,222,69,218,12,64,201,237,229,197,12,64,201,61,72,37,3,90,51,1,165,27,9,
  224,57,143,51,211,191,221,86,25,146,11,188,37,59,166,91,108,46,222,69,60,185,219,119,188,79,90,188,129,172,243,250,205,207,153,64,201,89,101,52,
  119,222,60,22,46,172,103,201,146,229,174,107,10,133,66,52,53,93,96,239,135,187,153,14,77,163,107,58,185,121,121,220,123,207,253,84,206,174,178,62,
  115,244,200,33,14,28,216,103,37,211,59,93,218,227,25,37,39,72,50,197,246,226,65,141,19,164,196,3,146,153,124,186,102,50,196,157,9,40,37,235,
  39,78,80,157,204,89,221,91,93,207,4,150,36,7,8,182,43,193,44,33,69,217,174,70,75,120,62,73,158,205,220,185,243,88,178,100,41,239,239,218,
  201,216,216,216,140,10,215,51,61,239,27,29,159,196,53,209,76,67,138,235,93,141,239,131,102,95,112,178,206,55,234,35,137,224,210,174,142,53,237,116,
  172,138,51,105,134,138,210,184,129,43,30,144,205,60,70,36,30,83,55,254,192,121,237,206,123,113,142,59,94,127,247,194,3,74,159,241,192,170,40,182,
  193,172,162,200,150,46,143,85,137,230,162,125,153,113,80,76,124,121,245,25,88,162,248,137,115,230,33,232,227,142,7,241,118,11,201,175,53,249,57,102,
  2,60,241,127,155,220,241,60,217,253,204,52,104,218,19,153,9,86,77,149,99,115,192,78,4,158,54,59,231,84,16,182,143,175,59,88,36,183,25,231,
  199,105,167,196,103,235,190,151,248,21,122,252,49,111,46,244,164,194,135,201,84,145,157,247,57,211,117,120,172,82,34,88,138,7,255,201,108,77,36,41,
  17,205,150,20,151,16,12,6,105,107,187,234,88,88,36,7,190,215,151,135,184,137,94,16,103,196,171,235,92,119,140,113,246,3,39,152,72,92,204,204,
  12,148,156,239,108,188,58,183,147,41,190,113,127,214,103,184,190,68,129,74,119,91,73,73,222,5,55,88,186,153,123,241,194,139,219,41,110,171,100,110,
  115,187,67,144,13,182,155,55,70,18,173,189,122,209,92,166,184,183,226,68,126,11,195,252,39,112,140,91,93,202,234,55,245,55,51,51,94,191,234,181,
  234,214,64,44,0,141,59,207,41,30,40,232,113,171,86,115,85,26,191,162,53,243,201,226,243,26,62,126,59,221,220,189,126,210,131,119,114,37,230,79,
  247,156,183,123,8,38,78,168,56,155,206,246,186,158,220,146,35,25,72,138,7,59,215,174,93,75,96,15,111,196,234,126,26,5,90,55,6,77,201,108,
  63,224,122,34,141,182,242,55,134,192,163,185,128,177,219,44,25,11,126,179,125,54,153,113,239,205,248,238,125,156,123,241,194,11,15,40,125,202,3,43,
  232,136,234,92,21,77,147,226,60,147,146,173,98,62,141,183,245,211,28,1,244,47,216,245,216,147,147,173,245,98,36,121,203,90,130,205,132,249,156,174,
  55,65,57,127,231,78,122,191,149,21,168,55,10,223,222,32,201,61,241,235,186,228,98,58,226,217,206,196,237,234,153,217,161,155,1,74,183,2,40,110,
  196,28,221,26,136,142,207,99,116,178,51,55,7,168,157,109,38,218,74,159,1,248,57,23,116,18,201,44,78,110,230,158,147,47,78,18,89,82,143,61,
  245,194,3,74,95,184,85,232,204,137,212,94,124,58,19,27,134,203,187,88,253,75,198,74,22,226,183,194,146,109,21,38,110,113,233,55,0,184,94,124,
  245,152,165,248,254,162,223,16,212,204,132,117,110,118,162,254,180,53,127,174,207,52,38,142,89,215,187,94,39,99,100,11,83,58,23,38,241,172,155,254,
  137,140,143,118,19,93,15,104,221,218,189,120,225,133,7,148,62,245,65,103,230,173,47,239,229,252,116,219,223,233,8,110,58,158,115,227,212,209,27,78,
  148,222,179,251,170,130,111,231,196,239,158,252,111,252,94,223,108,198,245,173,1,153,207,2,24,254,42,215,225,254,172,148,244,126,63,139,219,186,222,54,
  161,23,94,120,64,233,11,3,152,188,248,60,219,255,147,203,127,242,194,235,79,51,79,192,51,179,62,250,175,116,158,155,59,199,23,123,140,250,44,206,
  227,13,183,94,120,64,201,11,47,188,240,226,75,176,48,114,234,136,121,139,47,47,188,240,226,19,1,74,222,224,224,133,23,94,124,213,0,149,23,94,
  120,225,197,205,134,236,53,129,23,94,120,225,133,23,94,120,225,133,7,148,188,240,194,11,47,188,240,194,11,47,60,160,228,133,23,94,120,225,133,23,
  94,120,225,1,37,47,188,240,194,11,47,188,240,194,11,15,40,121,225,133,23,94,120,225,133,23,94,120,64,201,11,47,188,240,194,11,47,188,240,194,
  3,74,94,120,225,133,23,94,120,225,133,23,30,80,242,194,11,47,188,240,194,11,47,188,240,128,146,23,94,120,225,133,23,94,120,225,133,7,148,188,
  240,194,11,47,188,240,194,11,47,190,18,225,89,152,120,225,133,23,94,120,225,133,23,94,204,16,30,163,228,133,23,94,120,225,133,23,94,120,225,1,
  37,47,188,240,194,11,47,188,240,194,11,15,40,121,225,133,23,94,120,225,133,23,94,120,64,201,11,47,188,240,194,11,47,188,240,194,3,74,94,120,
  225,133,23,94,120,225,133,23,30,80,242,194,11,47,188,240,194,11,47,188,248,226,196,255,63,0,223,128,44,122,27,129,123,10,0,0,0,0,73,69,
  78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00133_png = (const char*) temp2;
const int TalComponent::bmp00133_pngSize = 108326;
