#include "includes.h"
#include "TalComponent.h"

TalComponent::TalComponent (TalCore* const ownerFilter) : AudioProcessorEditor (ownerFilter)
{
    Image backgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
	Image knobImage = ImageCache::getFromMemory(bmp00129_png, bmp00129_pngSize);
	Image toggleButtonLedImage = ImageCache::getFromMemory(bmp00130_png, bmp00130_pngSize);
	Image sliderImage = ImageCache::getFromMemory(bmp00131_png, bmp00131_pngSize);
	Image clipImage = ImageCache::getFromMemory(bmp00132_png, bmp00132_pngSize);
	Image transposeKnobImage = ImageCache::getFromMemory(bmp00133_png, bmp00133_pngSize);
	Image toggleButtonImage = ImageCache::getFromMemory(bmp00134_png, bmp00134_pngSize);
	Image splashImage = ImageCache::getFromMemory(bmp00135_png, bmp00135_pngSize);

    MemoryInputStream mis(electronichighwaysign, electronichighwaysignSize, false);
    this->typeface = new CustomTypeface(mis);
    Font fontDigital(typeface);
    fontDigital.setSizeAndStyle(13.0f, 0, 0.7f, 0.0f);

    // components
    this->oscSyncButton = this->addNormalButton(this, 211, 50, ownerFilter, toggleButtonImage, false, OSCSYNC);
    this->polyModeButton = this->addNormalButton(this, 211, 89, ownerFilter, toggleButtonImage, false, POLYMODE);

    this->panicButton = this->addNormalButton(this, 628, 185, ownerFilter, toggleButtonImage, true, PANIC);
    this->inputModeButton = this->addNormalButton(this, 628, 15, ownerFilter, toggleButtonLedImage, false, INPUTMODE);
    this->chorusButton = this->addNormalButton(this, 628, 91, ownerFilter, toggleButtonImage, false, CHORUS);

    this->volumeKnob = this->addNormalKnob(this, 551, 183, ownerFilter, knobImage, 61, TAL_VOLUME);
    this->harmonicsKnob = this->addNormalKnob(this, 509, 183, ownerFilter, knobImage, 61, HARMONICS);
    this->envelopeReleaseKnob = this->addNormalKnob(this, 462, 183, ownerFilter, knobImage, 61, ENVELOPERELEASE);

    this->portamentoKnob = this->addNormalKnob(this, 173, 183, ownerFilter, knobImage, 61, PORTAMENTO);
    this->tuneKnob = this->addNormalKnob(this, 210, 183, ownerFilter, knobImage, 61, TAL_TUNE);

    this->oscTransposeKnob = this->addNormalKnob(this, 210, 147, ownerFilter, transposeKnobImage, 4, OSCTRANSPOSE);

    this->pulseVolumeSlider = this->addSlider(this, 102, 57, ownerFilter, sliderImage, 63, PULSEVOLUME);
    this->sawVolumeSlider = this->addSlider(this, 126, 57, ownerFilter, sliderImage, 63, SAWVOLUME);
    this->subOscVolumeSlider = this->addSlider(this, 151, 57, ownerFilter, sliderImage, 63, SUBOSCVOLUME);
    this->noiseVolumeSlider = this->addSlider(this, 174, 57, ownerFilter, sliderImage, 63, NOISEVOLUME);

    this->esserIntensityKnob = this->addNormalKnob(this, 412, 183, ownerFilter, knobImage, 61, ESSERINTENSITY);

    this->pulseTuneKnob = this->addNormalKnob(this, 100, 183, ownerFilter, knobImage, 61, PULSETUNE);
	this->sawTuneKnob = this->addNormalKnob(this, 137, 183, ownerFilter, knobImage, 61, SAWTUNE);
	this->pulseFineTuneKnob = this->addNormalKnob(this, 100, 147, ownerFilter, knobImage, 61, PULSEFINETUNE);
	this->sawFineTuneKnob = this->addNormalKnob(this, 137, 147, ownerFilter, knobImage, 61, SAWFINETUNE);

    this->band00Slider = this->addSlider(this, 327, 57, ownerFilter, sliderImage, 63, VOCODERBAND00);
    this->band01Slider = this->addSlider(this, 352, 57, ownerFilter, sliderImage, 63, VOCODERBAND01);
    this->band02Slider = this->addSlider(this, 377, 57, ownerFilter, sliderImage, 63, VOCODERBAND02);
    this->band03Slider = this->addSlider(this, 401, 57, ownerFilter, sliderImage, 63, VOCODERBAND03);
    this->band04Slider = this->addSlider(this, 424, 57, ownerFilter, sliderImage, 63, VOCODERBAND04);
    this->band05Slider = this->addSlider(this, 445, 57, ownerFilter, sliderImage, 63, VOCODERBAND05);
    this->band06Slider = this->addSlider(this, 468, 57, ownerFilter, sliderImage, 63, VOCODERBAND06);
    this->band07Slider = this->addSlider(this, 491, 57, ownerFilter, sliderImage, 63, VOCODERBAND07);
    this->band08Slider = this->addSlider(this, 513, 57, ownerFilter, sliderImage, 63, VOCODERBAND08);
    this->band09Slider = this->addSlider(this, 534, 57, ownerFilter, sliderImage, 63, VOCODERBAND09);
    this->band10Slider = this->addSlider(this, 555, 57, ownerFilter, sliderImage, 63, VOCODERBAND10);

    // peak led
    this->peakLed = new PeakLed("PeakLed", clipImage, ownerFilter);
    this->peakLed->setBounds(634, 155, clipImage.getWidth(), clipImage.getHeight() / 2);
    addAndMakeVisible(this->peakLed);

	// Version info
	this->versionLabel = new Label(T("Version Info"), T("V 1.02"));
    this->versionLabel->setFont(fontDigital);
	this->versionLabel->setBounds(5, 190, 100, 20);
	this->versionLabel->setColour(Label::textColourId, Colour((juce::uint8)100, (juce::uint8)100, (juce::uint8)100, 1.0f));
	addAndMakeVisible(this->versionLabel);

	this->infoLabel = new Label(T("Info text"), T(""));
    this->infoLabel->setFont(fontDigital);
	this->infoLabel->setBounds(100, 3, 100, 20);
	this->infoLabel->setColour(Label::textColourId, Colour((juce::uint8)200, (juce::uint8)200, (juce::uint8)200, 1.0f));
	addAndMakeVisible(this->infoLabel);

    this->splashScreen = new TalSplashScreen(splashImage, 0, 0, 80, backgroundImage.getHeight());
    addAndMakeVisible(this->splashScreen);

        panicButton->setToggleState(false, dontSendNotification);
	this->updateParametersFromFilter();

    setSize(backgroundImage.getWidth(), backgroundImage.getHeight());

    // register ourselves with the filter - it will use its ChangeBroadcaster base
    // class to tell us when something has changed, and this will call our changeListenerCallback()
    // method.
    ownerFilter->addChangeListener(this);
}

TalComponent::~TalComponent()
{
    getProcessor()->removeChangeListener(this);
    deleteAllChildren();
}

FilmStripKnob* TalComponent::addNormalKnob(Component *component, int x, int y, TalCore* const ownerFilter, const Image knobImage, int numOfFrames, int parameter)
{
	FilmStripKnob *filmStripKnob;
	component->addAndMakeVisible(filmStripKnob = new FilmStripKnob(knobImage,
									 numOfFrames,
									 false,
                                     parameter));
    filmStripKnob->setBounds(x, y, knobImage.getWidth(), knobImage.getHeight() / numOfFrames);
	filmStripKnob->addListener(this);
	return filmStripKnob;
}

ImageToggleButton* TalComponent::addNormalButton(Component *component, int x, int y, TalCore* const ownerFilter, const Image buttonImage, bool isKickButton, int parameter)
{
	ImageToggleButton *imageToggleButton;
	component->addAndMakeVisible(imageToggleButton = new ImageToggleButton("Toggle Button", buttonImage, false, isKickButton, parameter));
    imageToggleButton->setBounds(x, y, buttonImage.getWidth(), buttonImage.getHeight() / 2);
	imageToggleButton->addListener(this);
	return imageToggleButton;
}

ImageSlider* TalComponent::addSlider(Component *component, int x, int y, TalCore* const ownerFilter, const Image sliderImage, int height, int parameter)
{
	ImageSlider *imageSlider;
	component->addAndMakeVisible(imageSlider = new ImageSlider(sliderImage, height, parameter));
	imageSlider->setBounds(x, y, sliderImage.getWidth(), height + sliderImage.getHeight());
	imageSlider->addListener(this);
	return imageSlider;
}

//==============================================================================
void TalComponent::paint (Graphics& g)
{
    // just clear the window
    g.fillAll (Colour::greyLevel (0.9f));

    Image backgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
    g.drawImage (backgroundImage,
                 0, 0, backgroundImage.getWidth(), backgroundImage.getHeight(),
                 0, 0, backgroundImage.getWidth(), backgroundImage.getHeight());
}

void TalComponent::sliderDragStarted (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->beginParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderDragEnded (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->endParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderValueChanged (Slider* caller)
{
    TalCore* const filter = getProcessor();

    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        filter->setParameterNotifyingHost(values["index"], (float)caller->getValue());
    }
}

void TalComponent::changeListenerCallback(ChangeBroadcaster* processor)
{
    // this is the filter telling us that it's changed, so we'll update our
    // display of the time, midi message, etc.
    updateParametersFromFilter();
}

void TalComponent::buttonClicked (Button* caller)
{
    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->setParameterNotifyingHost(values["index"], (float)caller->getToggleState());
    }
}

//==============================================================================
void TalComponent::updateParametersFromFilter()
{
    TalCore* const filter = getProcessor();

    // we use this lock to make sure the processBlock() method isn't writing to the
    // lastMidiMessage variable while we're trying to read it, but be extra-careful to
    // only hold the lock for a minimum amount of time..
    filter->getCallbackLock().enter();

    // take a local copy of the info we need while we've got the lock..
	float volume = filter->getParameter(TAL_VOLUME);
    float harmonics = filter->getParameter(HARMONICS);
	float oscTranspose = filter->getParameter(OSCTRANSPOSE);
	float portamento = filter->getParameter(PORTAMENTO);
	float tune = filter->getParameter(TAL_TUNE);
    float envelopeRelease = filter->getParameter(ENVELOPERELEASE);

    float noiseVolume = filter->getParameter(NOISEVOLUME);
    float pulseVolume = filter->getParameter(PULSEVOLUME);
    float sawVolume = filter->getParameter(SAWVOLUME);
    float subOscVolume = filter->getParameter(SUBOSCVOLUME);

    float esserIntensity = filter->getParameter(ESSERINTENSITY);
    float pulseTune = filter->getParameter(PULSETUNE);
    float sawTune = filter->getParameter(SAWTUNE);
    float pulseFineTune = filter->getParameter(PULSEFINETUNE);
    float sawFineTune = filter->getParameter(SAWFINETUNE);

    float band00 = filter->getParameter(VOCODERBAND00);
    float band01 = filter->getParameter(VOCODERBAND01);
    float band02 = filter->getParameter(VOCODERBAND02);
    float band03 = filter->getParameter(VOCODERBAND03);
    float band04 = filter->getParameter(VOCODERBAND04);
    float band05 = filter->getParameter(VOCODERBAND05);
    float band06 = filter->getParameter(VOCODERBAND06);
    float band07 = filter->getParameter(VOCODERBAND07);
    float band08 = filter->getParameter(VOCODERBAND08);
    float band09 = filter->getParameter(VOCODERBAND09);
    float band10 = filter->getParameter(VOCODERBAND10);

    float panic = filter->getParameter(PANIC);
    float inputMode = filter->getParameter(INPUTMODE);
    float chorus = filter->getParameter(CHORUS);
    float polyMode = filter->getParameter(POLYMODE);
    float oscSync = filter->getParameter(OSCSYNC);

    // ..release the lock ASAP
    filter->getCallbackLock().exit();

	this->volumeKnob->setValue(volume, dontSendNotification);
	this->harmonicsKnob->setValue(harmonics, dontSendNotification);
	this->oscTransposeKnob->setValue(oscTranspose, dontSendNotification);
	this->portamentoKnob->setValue(portamento, dontSendNotification);
	this->tuneKnob->setValue(tune, dontSendNotification);
	this->envelopeReleaseKnob->setValue(envelopeRelease, dontSendNotification);

	this->noiseVolumeSlider->setValue(noiseVolume, dontSendNotification);
	this->pulseVolumeSlider->setValue(pulseVolume, dontSendNotification);
	this->sawVolumeSlider->setValue(sawVolume, dontSendNotification);
	this->subOscVolumeSlider->setValue(subOscVolume, dontSendNotification);

    this->esserIntensityKnob->setValue(esserIntensity, dontSendNotification);
	this->pulseTuneKnob->setValue(pulseTune, dontSendNotification);
	this->sawTuneKnob->setValue(sawTune, dontSendNotification);
	this->pulseFineTuneKnob->setValue(pulseFineTune, dontSendNotification);
	this->sawFineTuneKnob->setValue(sawFineTune, dontSendNotification);

    this->band00Slider->setValue(band00, dontSendNotification);
    this->band01Slider->setValue(band01, dontSendNotification);
    this->band02Slider->setValue(band02, dontSendNotification);
    this->band03Slider->setValue(band03, dontSendNotification);
    this->band04Slider->setValue(band04, dontSendNotification);
    this->band05Slider->setValue(band05, dontSendNotification);
    this->band06Slider->setValue(band06, dontSendNotification);
    this->band07Slider->setValue(band07, dontSendNotification);
    this->band08Slider->setValue(band08, dontSendNotification);
    this->band09Slider->setValue(band09, dontSendNotification);
    this->band10Slider->setValue(band10, dontSendNotification);

    this->inputModeButton->setToggleState(inputMode > 0.0f, dontSendNotification);
    this->chorusButton->setToggleState(chorus > 0.0f, dontSendNotification);
    this->polyModeButton->setToggleState(polyMode > 0.0f, dontSendNotification);
    this->oscSyncButton->setToggleState(oscSync > 0.0f, dontSendNotification);

    if (panic == 0.0f)
    this->panicButton->setToggleState(false, dontSendNotification);
}

//==============================================================================
// Binary resources - be careful not to edit any of these sections!


static const unsigned char temp1[] = {120,156,237,93,119,124,21,197,22,190,116,130,138,128,210,164,163,128,60,68,16,164,223,157,43,130,5,20,41,162,136,136,130,235,21,145,77,112,4,41,
  82,68,68,121,74,111,34,177,208,213,80,21,81,90,118,22,84,202,163,136,72,81,164,248,20,4,169,74,83,4,203,59,223,153,220,13,252,146,188,108,
  96,137,55,154,63,252,157,143,59,179,103,206,204,156,57,109,118,227,109,86,216,236,249,116,247,184,174,102,249,59,187,118,121,162,119,231,190,229,239,237,
  218,37,46,16,8,212,29,223,46,84,62,240,39,161,28,129,218,197,219,134,226,98,107,23,223,98,208,63,7,89,0,143,23,46,231,88,45,198,55,8,
  185,0,77,102,44,253,83,80,119,219,218,222,230,134,144,11,78,108,44,236,88,104,2,48,195,57,137,227,22,17,23,75,143,4,86,86,25,173,172,103,
  219,55,21,46,184,243,214,159,12,235,240,22,51,25,12,248,126,178,178,208,25,192,12,231,74,122,28,157,15,22,172,235,36,63,142,223,93,192,79,185,
  143,3,160,179,25,206,157,218,227,231,13,97,185,130,225,41,23,232,199,243,164,246,120,74,153,117,231,188,231,204,244,124,206,41,68,53,195,249,146,58,
  143,105,126,143,120,255,155,131,74,238,218,157,143,65,221,233,243,141,230,135,182,40,57,247,169,18,198,138,197,182,2,157,154,247,53,253,195,53,3,194,
  220,99,123,247,58,250,145,121,117,243,42,231,120,75,166,178,71,227,17,12,170,149,126,91,112,143,31,59,57,2,143,128,50,15,128,245,21,149,42,191,
  96,166,224,81,240,8,134,141,200,97,134,243,39,73,70,123,25,164,33,29,108,106,144,126,106,104,141,105,254,153,112,1,154,204,100,241,35,124,92,198,
  145,145,206,27,26,178,184,194,69,164,117,197,143,204,199,157,96,100,198,231,45,1,214,196,93,164,200,170,37,139,31,227,187,6,23,8,148,234,244,144,
  136,11,95,150,180,48,43,11,228,19,3,215,188,170,100,255,114,181,24,96,240,97,167,135,40,57,114,106,87,209,172,82,23,5,74,141,250,7,210,18,
  187,111,175,22,98,64,206,4,155,31,49,95,108,98,131,7,168,12,223,245,190,1,208,235,160,52,238,121,240,29,91,94,43,114,24,135,142,29,182,65,
  235,78,175,161,248,135,92,9,143,41,187,89,55,163,231,117,52,74,254,179,115,13,12,59,241,209,60,60,188,25,123,213,224,69,162,76,76,14,199,106,
  87,127,189,40,83,121,173,178,182,173,250,92,92,63,154,64,232,177,117,188,85,86,179,252,15,139,233,235,243,56,172,176,93,151,230,114,172,239,19,231,
  136,151,62,201,237,88,203,183,222,46,118,148,136,87,242,167,38,117,89,61,231,140,168,32,62,233,60,77,201,86,229,139,138,99,49,51,20,196,253,189,
  199,12,37,235,119,125,211,0,40,191,160,146,49,172,232,72,37,91,140,95,210,232,145,98,82,129,134,239,186,73,255,80,170,211,78,123,64,206,170,70,
  235,125,131,109,126,132,180,34,17,60,64,229,216,66,65,6,163,134,116,18,7,170,61,103,203,142,173,94,16,234,202,237,54,40,243,0,232,59,238,126,
  85,115,88,31,81,114,81,111,37,115,174,138,21,155,122,14,85,185,135,119,16,21,250,140,163,109,190,60,105,51,182,173,106,37,238,172,84,197,145,175,
  62,87,144,193,187,101,55,26,165,58,149,118,72,53,134,26,5,102,20,113,6,141,28,98,60,219,62,183,35,105,44,227,155,161,135,213,87,195,149,49,
  186,203,151,74,54,44,146,83,228,173,179,92,229,173,115,139,248,240,30,91,89,0,151,63,57,72,89,152,132,11,236,27,30,85,86,153,152,198,194,5,
  224,100,117,222,249,112,50,224,166,35,241,155,146,1,63,222,231,189,71,146,193,91,207,38,42,185,63,60,70,208,73,80,180,136,52,197,109,74,110,239,
  254,161,88,243,193,33,181,226,245,133,162,231,117,36,230,172,213,31,8,200,61,97,201,20,193,19,185,109,210,48,158,89,100,170,102,108,233,38,247,138,
  81,67,202,59,146,180,142,65,171,242,19,196,43,181,138,59,146,54,75,76,59,80,208,1,189,251,253,156,250,7,82,39,133,30,180,45,138,31,201,211,
  230,63,10,60,64,101,75,163,44,131,101,47,252,110,112,143,229,91,87,65,3,21,40,243,0,0,83,244,224,81,240,8,134,141,200,97,134,175,72,58,
  35,5,147,182,101,228,212,3,198,232,230,5,29,11,160,193,221,87,227,40,28,48,106,14,59,109,31,89,244,181,209,233,215,253,182,36,21,197,120,54,
  78,0,61,100,75,154,176,65,63,218,115,159,122,205,232,219,107,131,45,47,27,250,178,113,116,192,26,155,212,192,160,125,178,161,142,70,204,138,215,237,
  113,39,22,4,105,53,108,57,104,100,238,32,49,75,124,188,240,228,70,244,223,50,9,243,84,50,62,46,72,39,55,88,50,254,104,144,122,12,9,146,
  90,27,35,118,156,101,42,73,209,97,14,140,183,31,248,150,134,109,155,40,105,173,197,181,226,121,27,116,115,195,83,182,5,64,6,70,89,103,215,58,
  2,138,194,96,230,51,79,42,9,64,19,85,99,11,37,178,130,202,7,254,88,34,242,116,111,163,38,119,252,64,144,25,80,242,88,229,185,226,230,51,
  205,213,248,109,9,226,203,132,59,20,111,64,157,93,141,213,211,253,38,139,178,135,235,41,217,224,238,241,162,126,215,50,106,137,61,74,180,222,151,83,
  201,214,185,134,99,24,27,246,144,206,182,77,170,60,82,208,220,109,210,116,209,233,215,46,182,92,252,245,107,34,97,111,208,166,179,201,84,86,107,48,
  159,52,247,22,27,54,21,43,36,223,88,248,153,232,114,178,168,2,253,106,184,80,22,64,204,238,235,72,89,48,157,211,21,118,40,158,215,8,117,92,
  79,167,71,227,43,28,6,55,239,202,65,251,119,101,210,190,193,56,124,246,67,53,199,34,155,148,12,42,118,40,224,72,178,140,162,248,177,24,103,71,
  137,107,196,186,80,62,71,210,175,100,246,114,59,180,104,70,241,45,1,199,26,119,162,150,209,225,149,162,240,24,115,130,43,171,20,114,44,28,198,213,
  159,159,36,45,111,51,199,48,150,237,131,26,98,111,97,213,158,55,126,106,242,145,194,9,98,103,73,123,209,200,5,180,54,202,66,19,128,36,51,98,
  144,5,80,80,12,154,181,146,179,103,205,50,200,168,169,49,213,87,26,164,148,54,142,41,92,84,34,76,59,148,105,25,235,29,233,45,171,8,76,152,
  61,112,77,49,209,160,198,48,91,146,81,165,229,110,103,99,98,160,60,67,82,30,237,202,93,128,38,121,176,96,47,6,100,75,224,192,109,82,150,177,
  130,230,98,147,55,18,96,111,65,19,72,217,18,45,50,187,130,52,46,209,218,184,110,154,128,80,144,146,204,234,53,112,86,98,76,245,219,148,220,144,
  231,67,65,118,94,129,242,12,97,39,92,192,147,255,232,166,133,12,36,0,153,4,133,157,134,69,103,110,163,134,252,87,45,250,101,170,222,197,184,203,
  55,136,185,247,94,233,88,117,118,173,32,151,123,181,99,65,168,3,213,254,84,242,243,49,163,217,150,224,172,243,86,205,28,219,131,55,17,19,3,53,
  99,143,196,183,134,246,57,114,250,250,23,25,116,57,57,65,28,222,178,71,73,88,38,90,88,182,80,61,30,126,85,255,64,155,194,38,100,194,27,197,
  21,206,20,123,51,240,96,175,246,105,211,146,12,102,142,61,97,112,143,132,189,137,6,25,7,5,202,60,0,18,114,172,82,232,241,139,216,203,158,92,
  116,108,149,203,137,200,97,134,11,101,32,84,75,39,134,244,30,31,22,78,26,116,221,67,159,11,88,21,11,160,207,123,229,216,255,55,250,180,233,36,
  114,53,69,146,250,208,57,23,31,15,164,35,65,251,98,96,89,173,126,193,101,198,194,110,121,177,5,141,69,149,185,87,105,64,203,147,72,174,100,185,
  177,185,225,51,182,85,246,240,34,184,111,219,66,152,67,198,203,176,96,216,249,23,210,53,49,98,71,63,219,130,39,228,167,0,200,90,58,22,57,77,
  177,116,10,113,198,145,167,32,135,68,189,42,73,12,56,231,50,149,155,43,11,138,220,123,195,15,26,24,203,42,107,99,235,2,110,162,165,229,206,102,
  44,58,145,186,40,139,76,187,225,2,150,3,77,0,102,44,108,19,243,62,126,56,94,243,6,96,150,207,228,235,151,12,184,9,106,160,121,163,19,179,
  156,49,191,127,50,96,222,104,98,222,225,171,147,228,223,187,253,67,129,19,109,145,29,72,6,245,94,34,214,205,15,181,101,32,201,166,27,0,136,138,
  72,185,28,73,187,17,132,201,2,189,162,197,47,138,127,8,61,191,69,145,177,55,72,185,148,204,149,176,215,64,204,2,133,2,181,180,102,17,123,138,
  244,146,65,204,138,82,142,5,17,0,244,49,224,159,0,234,44,95,172,228,201,5,249,25,96,111,31,157,182,94,73,10,156,140,251,158,222,173,64,75,
  46,250,69,255,112,102,127,126,7,61,16,33,242,35,96,18,97,102,134,139,102,64,151,61,233,105,177,52,115,139,11,58,10,102,184,248,57,6,223,149,
  240,194,143,157,25,46,145,196,16,139,192,142,131,142,147,241,69,187,63,148,133,179,255,188,60,171,172,187,223,191,85,188,177,144,130,2,0,178,197,134,
  214,247,8,224,38,184,73,242,213,202,250,186,206,116,177,250,200,239,36,106,73,111,73,152,247,197,188,38,137,33,41,112,35,50,167,172,201,141,254,108,
  48,86,89,55,252,249,134,96,128,200,147,130,118,173,200,240,227,214,175,185,191,16,176,7,172,69,165,155,108,214,77,223,213,220,164,159,2,31,51,92,
  202,53,21,95,240,47,240,10,6,3,10,152,5,121,67,101,125,210,185,162,32,94,218,207,33,234,230,95,216,103,223,243,96,21,36,5,10,7,194,96,
  49,192,7,192,12,151,78,226,140,128,148,67,125,0,178,45,54,11,230,130,28,123,126,182,45,82,74,195,5,232,108,134,203,36,61,78,198,88,80,74,
  160,215,103,194,27,19,149,5,171,126,116,192,157,8,15,198,234,41,35,116,217,121,150,214,144,76,186,193,224,242,39,3,186,9,230,155,59,67,120,114,
  199,202,130,243,135,67,231,62,156,113,80,22,99,188,220,114,138,178,38,223,50,75,48,40,86,112,28,71,152,102,184,108,146,24,244,56,135,169,86,161,
  237,95,144,251,28,99,51,67,0,51,92,46,169,15,166,51,172,40,245,193,238,48,192,83,72,41,204,112,249,192,132,83,37,67,113,225,10,154,198,78,
  75,124,73,12,221,210,218,182,0,150,239,47,70,187,221,122,113,50,64,147,201,157,222,44,49,84,113,167,253,115,23,235,54,23,160,73,119,122,187,189,
  173,59,253,116,244,168,110,115,1,154,116,167,132,159,79,232,78,103,86,22,113,184,205,5,104,210,157,182,151,162,104,8,96,224,190,234,186,205,5,104,
  210,157,58,60,89,83,119,10,212,105,166,219,92,128,38,51,92,49,105,150,181,127,41,175,135,5,224,209,126,88,248,124,50,208,195,146,132,186,19,0,
  183,145,240,201,64,119,2,3,150,13,128,69,2,3,23,104,217,102,255,158,168,59,1,112,27,24,184,64,119,2,3,158,0,0,203,13,6,46,224,9,
  48,39,238,4,192,109,96,224,2,61,203,107,147,102,105,132,202,235,189,4,224,45,4,35,23,232,189,4,35,238,4,192,109,167,198,30,77,6,186,19,
  24,144,88,138,57,237,41,52,88,51,112,1,154,52,39,238,4,192,109,96,224,2,221,105,254,201,123,12,218,33,101,1,208,198,40,107,109,141,98,194,
  5,104,210,195,113,39,0,110,195,40,46,208,157,176,207,220,9,128,219,142,157,90,148,12,116,39,136,194,154,10,192,10,10,81,92,160,53,117,94,252,
  9,221,9,128,219,126,155,88,36,228,2,221,9,162,176,58,3,176,22,67,20,23,104,117,198,40,220,9,128,219,48,138,11,116,39,204,156,70,210,75,
  64,3,56,60,115,23,160,73,15,199,202,7,192,109,24,197,5,201,7,195,61,61,220,134,153,187,64,119,130,40,220,9,128,219,32,138,11,116,39,204,
  156,59,1,176,138,99,230,46,208,157,32,10,171,49,128,119,93,199,40,46,208,157,192,160,99,176,166,230,148,82,215,209,148,154,174,131,129,11,180,174,
  95,151,166,221,194,10,184,64,171,49,22,156,53,20,128,6,210,58,231,2,173,161,158,116,29,163,112,39,0,110,195,40,46,200,192,129,72,105,75,193,
  192,5,126,107,104,74,171,140,81,92,144,65,53,102,149,73,93,141,51,160,161,158,244,42,165,19,0,3,23,100,64,249,176,134,220,9,128,219,176,134,
  46,72,203,157,96,20,23,104,229,171,148,164,124,158,108,40,70,227,78,0,220,134,209,92,144,138,134,178,166,96,45,93,144,1,189,194,40,220,9,128,
  219,48,138,11,146,117,157,245,10,128,213,9,11,238,130,180,28,121,26,42,147,174,149,193,164,120,229,1,120,193,49,41,23,36,219,134,116,181,193,211,
  30,98,20,222,67,0,222,58,140,226,2,237,81,49,10,119,74,221,20,233,141,174,156,17,143,234,41,132,202,100,109,240,228,118,51,217,20,249,103,64,
  60,233,149,103,199,148,158,54,120,179,13,85,146,84,230,200,3,218,91,91,0,236,80,131,99,23,36,3,61,1,180,177,108,0,44,18,218,92,160,101,
  67,27,15,11,192,163,161,205,5,122,216,235,147,134,69,204,202,250,5,144,150,73,74,233,197,160,160,46,208,170,131,112,148,55,28,32,45,27,129,54,
  222,112,128,180,54,220,83,212,237,41,216,247,20,26,84,61,39,216,231,165,0,248,127,135,54,221,165,192,90,242,82,0,164,21,77,122,114,158,231,153,
  75,94,1,180,185,32,3,75,225,37,109,8,255,235,156,40,41,221,89,226,16,241,44,83,183,21,201,78,33,221,20,240,239,29,182,120,138,153,207,219,
  195,126,87,223,168,25,184,128,247,48,92,45,35,219,227,105,229,49,57,94,10,0,94,1,210,119,225,2,189,20,23,184,242,169,164,241,158,86,30,203,
  196,157,0,184,13,203,228,130,84,10,2,105,173,188,167,69,189,33,35,199,31,109,85,231,87,16,220,169,218,241,254,134,69,171,158,12,208,164,59,241,
  246,0,120,207,12,210,216,67,79,85,150,125,247,118,52,184,211,216,94,55,216,220,230,2,52,153,225,234,73,179,204,170,27,30,190,49,163,86,58,221,
  117,243,109,155,194,53,206,89,220,243,227,48,172,169,11,46,42,150,78,99,113,211,117,14,88,211,109,205,146,34,96,86,125,172,169,11,208,100,134,107,
  94,92,220,154,74,10,237,41,213,241,148,103,123,74,80,252,43,23,121,242,69,41,79,17,70,113,65,6,124,145,39,99,231,169,18,228,95,253,198,83,
  152,236,41,179,242,45,76,246,22,201,221,228,187,141,240,47,60,190,68,181,240,180,92,159,167,122,179,167,90,137,167,44,166,214,57,22,144,215,11,128,
  151,9,26,227,130,228,42,113,38,90,25,79,41,180,231,106,94,186,137,175,103,151,247,247,61,135,181,163,192,157,164,85,6,241,228,78,60,57,1,255,
  114,141,191,183,54,220,156,145,234,167,231,42,85,186,153,231,121,113,86,74,111,159,74,122,154,242,114,40,249,68,251,115,57,228,41,110,240,164,50,158,
  238,125,60,133,4,254,21,204,61,105,67,157,127,132,109,240,28,214,165,91,98,200,228,11,62,79,86,6,109,124,45,151,122,193,92,95,203,101,174,149,
  225,21,231,78,0,220,134,133,118,129,86,190,186,89,93,249,60,229,57,254,153,162,104,172,185,121,190,5,244,199,123,213,203,136,202,120,138,34,61,59,
  166,75,80,0,72,25,18,112,1,192,191,43,176,204,62,209,245,179,79,116,52,23,200,255,222,145,109,3,223,175,123,81,102,96,149,1,96,77,193,235,
  90,46,208,42,67,171,166,181,1,192,103,109,72,197,202,64,20,238,4,192,109,16,197,5,186,19,68,225,78,0,222,3,30,148,69,93,112,81,42,3,
  6,46,184,244,42,147,242,165,65,94,39,238,4,144,214,59,119,13,207,41,81,249,115,71,224,233,222,213,83,245,233,188,123,87,79,161,102,90,111,98,
  158,87,125,226,90,51,24,184,32,149,234,19,183,129,129,11,244,237,76,163,236,245,202,200,122,113,167,71,22,182,213,86,230,163,201,197,181,188,46,64,
  147,25,169,143,26,60,220,177,83,245,108,190,188,114,1,154,204,112,208,247,55,4,60,221,124,120,190,160,114,147,162,75,255,134,128,113,78,117,243,252,
  186,3,68,114,65,42,117,135,139,210,47,127,235,14,174,225,229,101,130,188,46,72,37,211,252,127,139,122,190,58,195,103,186,32,149,76,243,255,197,165,
  233,217,249,176,72,179,174,12,185,93,112,81,53,220,84,86,62,165,203,75,37,0,242,116,178,61,101,43,41,163,111,76,202,5,90,9,67,23,247,230,
  125,38,87,170,51,217,29,95,104,4,119,75,20,164,15,151,254,226,19,242,78,243,229,21,248,104,188,248,244,175,248,245,23,100,43,233,190,221,238,159,
  174,55,206,200,29,157,127,239,157,102,242,253,125,52,86,244,253,171,6,95,224,247,23,238,235,162,145,23,72,245,18,108,233,157,84,209,239,123,235,141,
  90,83,92,128,38,143,119,191,183,70,241,221,111,38,27,218,44,93,121,77,215,208,102,178,121,140,194,91,134,112,19,223,43,62,23,88,110,76,229,197,
  50,207,106,156,238,213,169,39,13,245,100,175,178,108,105,175,233,95,99,212,50,249,75,142,44,235,226,60,233,149,167,75,235,204,181,50,225,219,162,216,
  89,250,23,243,121,82,190,104,245,131,254,24,181,40,244,94,222,110,212,110,207,72,201,197,147,99,242,207,231,100,242,139,233,254,106,195,165,186,95,77,
  229,42,194,63,109,184,227,175,41,144,68,99,250,153,201,111,134,251,247,173,204,223,59,82,187,51,171,151,53,178,108,210,24,141,49,159,127,153,165,39,
  243,216,236,239,247,197,65,42,218,224,169,194,154,242,186,45,149,247,72,189,124,133,234,163,109,104,238,187,247,242,47,65,243,84,131,244,239,35,97,79,
  201,80,38,135,154,119,101,36,212,188,64,211,157,74,60,234,223,59,230,158,142,152,127,166,219,147,45,245,116,88,253,205,159,211,141,108,61,105,195,221,
  25,189,235,205,196,2,156,127,202,119,158,94,177,166,160,205,5,25,120,9,34,165,54,164,242,54,149,111,217,73,184,69,118,156,117,105,46,56,51,241,
  175,12,120,210,6,79,175,44,120,178,13,153,28,140,221,147,213,53,52,147,175,37,51,89,141,61,125,201,146,101,51,129,150,81,240,38,73,86,178,161,
  89,54,121,204,228,242,70,171,139,43,199,70,99,113,254,146,150,99,255,2,3,18,101,31,206,132,91,251,110,138,252,115,150,254,221,93,122,122,209,39,
  219,94,101,182,242,181,249,107,130,176,75,164,87,105,189,58,27,141,229,216,104,84,62,79,241,85,38,107,232,189,89,61,82,243,100,212,162,81,249,178,
  172,81,107,155,209,119,239,179,100,113,62,10,99,25,111,185,215,125,89,253,68,103,231,94,62,70,219,158,84,230,254,140,254,61,248,191,237,82,120,123,
  173,222,83,105,202,147,207,241,228,78,188,216,208,112,59,223,255,255,21,158,78,116,74,217,160,41,46,184,20,31,239,100,98,158,147,201,69,193,104,140,
  71,61,25,144,7,162,184,216,236,233,19,84,79,39,218,147,54,120,42,236,122,250,0,43,10,205,99,184,253,197,197,126,169,216,6,79,25,166,127,182,
  212,211,61,97,86,245,94,225,7,255,17,47,84,123,254,244,205,83,54,231,233,219,236,172,88,250,244,166,50,29,254,154,151,182,60,253,9,11,79,127,
  157,162,118,134,255,47,10,105,89,229,243,254,240,68,90,119,189,23,248,210,214,5,190,151,28,126,40,105,123,60,185,186,11,244,98,169,92,65,120,250,
  243,7,231,89,64,254,211,27,48,161,46,208,127,179,164,227,197,233,87,42,214,198,147,126,101,217,183,206,50,249,165,192,135,207,241,24,233,254,81,12,
  79,126,209,191,55,38,188,156,162,240,35,89,252,69,169,112,167,192,243,53,191,16,58,164,154,23,127,110,233,10,163,185,96,94,188,231,197,77,25,215,
  47,63,247,47,104,112,92,31,238,28,197,33,187,167,84,48,251,3,248,168,252,0,254,209,40,14,65,207,211,208,236,15,224,179,63,128,191,72,231,105,
  102,127,0,255,207,248,0,254,177,236,15,224,163,217,197,121,210,171,104,252,0,62,28,197,206,210,191,152,207,147,242,69,171,31,252,135,127,0,255,120,
  246,7,240,217,31,192,187,218,208,37,251,3,248,236,15,224,163,58,82,123,34,171,151,53,178,108,210,24,141,49,95,38,95,131,119,205,254,0,62,154,
  63,128,127,50,251,3,248,104,254,0,190,91,246,7,240,217,31,192,187,218,96,101,241,123,157,11,72,134,178,210,7,240,177,217,113,86,246,7,240,81,
  253,1,124,92,86,215,208,236,15,224,179,110,38,208,61,251,35,140,44,127,97,16,141,229,141,167,178,63,128,143,246,47,60,163,236,45,208,176,204,254,
  0,62,251,3,248,191,202,94,61,157,253,1,124,52,223,93,70,99,124,149,201,26,218,35,171,71,106,158,140,90,52,42,95,150,53,106,61,179,63,128,
  143,230,47,90,158,201,234,39,58,59,247,202,236,239,230,122,101,127,0,159,229,63,128,239,157,253,1,124,246,7,240,127,85,189,175,79,20,23,155,179,
  63,128,247,209,83,244,205,254,0,62,138,189,87,248,217,127,196,11,213,217,31,192,251,152,46,244,203,200,137,246,244,62,22,38,199,123,8,144,214,183,
  237,158,172,114,74,31,157,202,205,191,39,131,235,233,26,55,229,202,95,232,103,235,253,163,37,69,78,229,118,225,188,20,57,173,255,85,189,167,20,217,
  139,166,134,7,248,254,1,188,127,47,148,121,138,220,162,240,219,246,240,192,115,236,124,186,239,207,120,82,29,255,82,26,79,74,232,101,229,195,207,5,
  106,23,223,34,226,98,7,228,76,48,238,250,236,50,199,26,191,45,65,184,192,124,177,137,109,161,9,192,140,253,106,184,18,177,85,139,57,214,246,54,
  55,24,46,40,25,127,116,153,133,38,0,51,60,136,56,182,133,50,22,223,98,4,2,129,65,22,192,227,133,203,57,86,139,241,13,66,46,64,147,25,
  75,255,20,212,221,6,163,144,11,78,108,44,236,88,104,2,48,195,207,251,206,113,112,210,172,119,237,206,39,6,228,172,106,72,13,18,140,229,91,87,
  25,15,215,62,99,200,19,27,199,24,59,74,92,35,48,65,234,33,172,65,35,115,27,45,141,178,66,222,245,217,58,99,219,170,207,13,208,17,59,250,
  25,252,195,184,19,181,140,21,175,47,100,42,243,159,157,203,96,194,146,41,70,222,58,183,24,178,85,249,9,196,249,118,163,65,141,97,76,229,184,19,
  11,130,0,160,36,73,35,254,225,167,38,117,237,65,35,135,24,160,248,129,193,232,46,95,26,155,27,158,74,100,9,177,192,17,145,205,240,11,73,147,
  40,83,121,173,40,19,147,195,145,0,149,114,255,168,30,123,98,41,83,249,209,77,11,25,180,232,253,158,152,57,246,132,146,199,42,207,21,37,23,253,
  162,64,95,187,237,55,253,3,173,155,115,235,21,239,137,118,165,114,57,114,232,203,11,196,244,245,121,28,208,165,83,242,234,31,66,143,93,230,52,249,
  233,109,166,178,250,119,227,24,220,184,242,21,177,250,243,147,74,118,93,250,130,56,93,97,135,250,162,221,64,49,229,218,169,202,122,234,195,247,53,168,
  214,96,190,152,154,247,53,101,209,58,49,144,35,167,118,21,13,106,92,166,182,238,105,47,104,105,108,73,171,32,86,86,25,109,220,243,96,21,209,235,
  224,49,67,158,92,144,31,243,19,143,23,158,204,11,47,183,173,106,197,128,118,206,24,184,166,152,144,229,23,204,12,78,124,52,143,0,85,87,110,55,
  248,135,132,189,137,80,196,224,61,15,190,99,200,152,21,165,12,108,224,232,46,237,152,202,29,37,226,13,189,163,241,188,111,252,67,159,247,190,49,94,
  125,110,164,81,115,216,105,67,182,222,55,216,192,210,130,146,128,130,127,168,48,161,52,107,5,168,44,48,163,136,0,128,18,144,86,4,173,217,179,254,
  37,72,14,37,251,151,171,37,182,119,175,163,190,26,46,196,180,3,35,149,69,51,211,243,238,219,107,131,193,160,179,179,213,224,37,25,83,253,54,94,
  27,185,53,96,138,185,247,94,233,44,186,122,54,83,89,103,249,98,6,183,220,241,177,168,216,161,128,222,206,121,117,243,58,145,253,53,195,67,146,118,
  124,250,250,23,105,245,150,7,37,0,84,102,230,51,79,138,82,157,118,26,210,57,222,82,236,60,91,72,144,234,8,168,187,69,122,40,90,149,47,42,
  100,226,227,61,233,56,127,98,128,214,157,94,195,224,31,136,93,240,240,22,147,169,124,251,129,206,12,176,55,111,63,240,109,80,150,110,114,175,232,244,
  235,254,224,187,101,91,50,149,115,70,84,96,0,74,27,151,200,63,92,43,158,183,31,157,118,55,83,230,1,48,66,61,205,198,131,37,36,166,137,17,
  145,205,216,35,139,190,54,88,120,0,8,63,247,169,215,12,22,190,76,229,230,198,202,42,133,4,173,90,144,133,223,222,102,78,144,133,39,96,64,120,
  80,22,30,0,178,62,241,229,112,166,116,60,134,50,216,220,240,25,131,133,111,150,255,97,3,178,226,152,177,240,164,80,141,0,64,89,120,0,200,74,
  182,195,96,225,193,3,32,98,240,88,66,8,31,17,217,12,191,24,104,59,170,10,153,162,41,215,86,8,241,193,5,128,188,77,54,151,9,65,237,228,
  254,193,215,132,160,178,93,78,22,101,42,191,171,89,152,193,139,255,189,34,52,247,169,18,134,188,113,101,1,126,4,148,121,0,144,16,137,195,138,22,
  12,145,1,74,228,71,168,171,13,30,160,204,20,160,89,254,178,33,18,36,145,135,197,35,17,57,204,216,245,21,149,96,110,0,96,159,175,196,34,193,
  18,181,255,246,61,1,1,232,104,48,149,45,141,183,24,76,120,99,162,96,137,118,158,29,205,143,128,50,15,0,176,39,221,21,44,17,30,129,0,224,
  193,18,129,41,192,85,131,23,9,150,8,195,226,145,136,28,102,236,77,173,247,25,204,13,0,236,113,242,88,34,24,55,8,128,221,99,137,250,246,106,
  193,0,210,176,68,232,30,249,143,121,0,128,61,158,103,137,240,8,4,0,15,150,8,76,1,112,230,88,34,12,139,71,34,114,152,225,151,146,206,15,
  172,93,213,30,43,148,4,168,59,125,190,154,51,98,42,83,216,5,6,100,169,233,12,125,164,36,233,147,160,41,169,155,207,52,23,63,118,114,148,4,
  32,75,167,114,174,138,21,205,15,181,85,114,76,243,123,196,101,67,99,20,206,64,249,5,149,12,235,174,207,238,194,210,8,62,235,12,158,109,223,148,
  155,100,157,93,141,69,142,61,63,219,56,55,252,48,90,192,13,148,217,127,218,180,36,143,71,230,218,56,59,151,4,32,41,12,72,132,83,194,34,66,
  93,1,64,121,18,0,165,155,108,230,30,160,18,142,8,0,246,244,230,51,107,149,132,16,237,74,125,194,163,144,121,33,153,73,178,92,9,123,21,89,
  45,113,199,35,191,42,89,239,165,202,180,209,121,29,208,210,149,201,248,0,144,198,58,45,122,215,36,93,41,228,200,199,158,104,40,42,244,185,202,57,
  58,224,78,166,146,228,103,208,121,231,195,226,243,49,212,3,182,29,143,128,50,15,128,149,5,242,57,48,40,60,10,214,14,195,98,169,88,142,28,123,
  122,177,96,88,110,150,116,255,224,120,1,209,177,33,60,23,236,16,64,100,203,204,240,80,159,54,241,147,206,211,20,41,145,160,83,160,216,212,145,48,
  202,110,214,77,144,215,208,63,208,194,178,168,173,247,229,84,48,78,212,242,165,13,74,255,217,73,246,179,173,13,63,120,217,208,151,109,30,246,88,204,
  12,155,204,59,83,73,50,48,0,37,94,250,7,114,29,137,232,1,202,143,0,84,236,240,10,76,110,34,75,74,6,206,134,14,241,35,24,174,197,248,
  37,228,232,218,11,142,54,122,111,120,66,224,32,129,66,179,249,135,134,69,114,10,50,120,226,185,217,87,8,73,66,9,132,46,95,38,220,193,20,10,
  194,128,24,235,30,164,139,252,8,40,243,0,0,211,21,175,223,44,106,14,235,99,176,94,98,88,50,131,90,14,168,16,4,131,78,145,57,76,100,37,
  131,232,3,215,188,202,148,20,181,46,3,80,126,4,0,211,71,15,94,15,60,2,0,30,100,107,53,83,44,33,70,225,53,5,32,25,212,177,152,235,
  244,54,64,14,242,17,81,113,62,160,202,160,236,156,1,200,94,243,249,32,135,171,207,71,245,239,244,249,0,229,243,1,128,243,193,39,8,199,1,143,
  128,50,15,128,117,33,191,207,199,191,3,251,168,83,92,236,115,179,71,136,35,241,20,246,34,56,96,0,29,168,223,181,140,99,33,30,229,95,96,85,
  25,96,41,114,37,92,231,88,253,130,141,4,128,25,126,57,80,105,127,37,114,117,100,171,66,52,57,5,103,15,191,101,47,254,186,68,136,66,173,68,
  73,209,10,123,182,26,239,230,208,174,142,98,77,118,44,191,136,189,20,171,198,5,37,133,133,20,238,13,75,4,229,51,2,64,143,169,110,131,142,137,
  206,206,125,138,118,60,87,104,88,209,145,138,44,8,83,30,5,32,50,172,25,91,170,83,233,16,148,66,2,212,120,247,121,85,104,123,17,166,136,137,
  25,236,94,241,135,8,223,120,189,146,143,20,59,38,202,30,94,100,131,61,197,140,182,164,121,9,196,214,207,182,207,29,2,149,175,62,87,144,1,92,
  43,251,120,48,165,72,208,142,140,98,178,97,153,118,160,160,99,189,91,118,163,129,211,96,33,90,100,64,170,204,225,178,213,244,163,241,226,230,93,20,
  115,227,244,143,216,113,86,221,20,215,157,169,36,181,101,128,61,60,80,237,79,37,243,116,111,35,10,214,60,170,64,43,239,255,94,255,240,214,179,137,
  10,170,67,58,169,36,121,15,156,94,133,144,23,20,60,24,32,231,184,227,145,190,74,194,125,209,126,42,80,10,125,244,15,71,226,55,177,146,83,122,
  160,216,251,209,217,118,104,113,113,198,29,89,110,227,141,12,16,139,126,153,144,207,145,71,226,91,179,200,144,148,69,167,128,88,208,214,57,49,187,103,
  240,124,205,216,31,59,133,68,219,81,251,149,4,56,116,236,176,170,179,188,190,158,196,128,239,203,113,95,168,208,152,234,121,28,137,72,156,150,223,65,
  84,12,202,222,22,128,204,170,65,24,59,219,196,160,172,80,33,3,234,240,206,91,74,38,236,13,26,240,117,8,213,65,41,196,206,201,206,15,222,240,
  233,126,147,245,176,103,215,58,42,34,135,25,235,28,223,40,86,44,54,148,4,160,73,219,200,51,105,89,108,249,241,192,217,136,48,141,37,246,40,166,
  108,177,1,64,73,193,109,169,65,81,181,236,133,1,226,200,162,7,21,103,46,212,93,81,126,198,148,153,2,68,70,49,99,199,84,95,73,10,84,15,
  22,96,165,232,216,234,5,213,191,220,2,166,242,134,63,223,96,208,58,215,112,49,126,91,85,37,145,211,144,137,180,65,57,101,3,160,253,164,136,117,
  2,83,137,12,26,224,95,247,47,198,177,176,153,41,173,141,29,25,197,12,191,18,136,189,124,3,29,214,122,47,189,205,199,206,66,88,193,39,242,227,
  129,213,52,128,249,228,38,228,165,12,160,90,234,202,146,142,69,231,68,31,214,88,90,53,12,101,75,0,68,183,159,253,48,27,251,98,240,164,145,144,
  66,47,65,229,172,213,181,25,64,83,16,60,73,164,152,136,150,143,14,88,195,84,34,31,3,160,19,203,17,183,133,24,154,65,82,72,102,35,173,64,
  72,103,147,113,93,98,32,230,70,226,1,42,47,127,114,16,3,104,23,71,230,139,126,153,10,176,12,148,83,62,0,178,218,54,214,9,193,144,92,89,
  96,40,146,21,182,182,185,135,119,80,18,89,221,188,186,47,41,50,227,224,163,117,137,150,72,129,190,244,201,167,250,135,23,234,29,80,253,130,203,72,
  151,254,80,18,9,225,153,253,249,29,168,14,168,36,70,12,200,62,136,97,167,115,59,178,195,43,19,56,141,90,252,245,107,58,93,166,244,156,1,105,
  172,232,121,29,245,248,186,206,116,177,176,91,94,135,124,20,63,106,253,54,122,174,6,36,174,152,220,241,3,101,125,159,56,135,1,114,78,113,255,125,
  91,21,12,129,221,236,144,146,129,192,32,206,160,225,67,56,147,198,145,1,232,186,52,151,120,242,205,51,144,249,19,227,155,161,135,21,40,229,131,250,
  7,210,13,69,234,68,153,229,27,74,30,58,102,145,36,67,224,169,180,251,91,177,216,166,248,164,184,138,108,175,25,30,150,20,253,132,239,186,137,34,
  134,206,65,9,64,107,20,204,127,182,58,83,137,180,20,0,89,44,108,175,196,121,131,241,197,182,98,221,37,50,40,50,224,148,248,44,105,132,117,183,
  144,22,145,10,43,9,125,32,127,165,98,118,95,39,200,197,160,62,80,93,80,204,175,192,29,84,194,104,1,128,238,155,60,93,255,112,252,112,188,194,
  163,52,101,101,181,206,117,57,36,179,89,50,18,118,105,68,84,51,60,60,80,101,238,189,228,82,122,54,110,17,34,99,103,91,131,70,54,13,65,203,
  44,148,44,0,36,202,12,48,33,5,102,140,101,42,225,144,0,160,253,36,116,35,174,7,224,124,131,110,41,99,41,254,1,174,143,236,32,253,112,24,
  171,102,176,131,133,130,177,163,197,65,4,40,19,179,130,189,184,213,102,223,173,33,23,172,123,232,115,101,29,168,22,98,32,223,122,54,24,10,126,245,
  163,186,162,69,189,208,117,21,78,43,114,33,55,161,180,228,188,220,178,60,83,11,78,198,5,155,122,14,85,86,209,106,165,137,221,80,37,255,109,85,
  101,144,115,85,245,16,45,183,194,26,132,16,96,78,186,190,118,136,76,137,178,174,26,92,47,25,144,19,82,22,154,0,184,47,37,19,234,231,14,213,
  67,181,110,238,168,228,224,122,85,67,152,33,216,131,242,128,164,35,26,192,10,90,7,11,254,12,89,6,201,213,71,126,167,101,170,136,82,69,8,84,
  6,191,170,193,128,76,120,136,34,168,160,172,241,110,99,2,185,19,207,236,191,147,23,223,140,221,186,103,187,230,2,64,65,157,146,0,214,237,223,168,
  181,125,183,35,209,87,146,214,85,196,253,152,139,15,13,40,157,179,23,25,32,209,207,91,39,224,72,132,140,88,167,227,135,175,17,241,151,29,84,114,
  217,11,191,27,196,135,66,233,67,6,102,42,183,6,246,144,102,76,86,20,224,162,80,161,36,10,47,176,156,112,81,172,161,55,181,126,156,77,55,178,
  76,182,229,67,223,94,198,160,239,184,109,58,144,208,162,14,89,22,17,217,12,143,240,189,40,56,210,119,142,163,124,231,56,218,119,142,99,2,227,200,
  252,196,197,194,128,146,7,208,150,180,90,131,249,10,81,50,168,68,213,18,0,81,9,135,220,48,23,119,191,255,177,2,69,225,143,127,184,243,214,159,
  20,236,126,199,86,164,37,240,47,100,7,28,108,57,106,90,214,200,169,7,140,30,141,41,44,129,43,129,222,128,94,51,128,162,33,128,43,127,222,161,
  80,124,1,229,225,0,16,174,215,186,121,23,217,160,152,25,228,142,118,195,109,51,101,9,1,34,34,155,225,177,231,76,98,132,58,174,187,224,97,248,
  180,117,161,213,138,75,176,200,5,161,85,152,12,123,176,89,171,63,64,84,185,19,174,76,129,118,51,127,208,63,208,1,227,56,5,84,82,68,201,0,
  117,33,76,83,214,239,250,166,1,27,81,161,207,56,166,188,0,0,160,183,247,207,233,240,15,152,56,87,39,80,221,3,15,0,184,182,219,38,93,230,
  176,132,40,180,70,68,54,195,227,2,253,40,216,136,139,133,63,226,157,0,128,172,196,132,169,188,189,255,96,6,200,18,121,39,74,117,122,136,19,5,
  80,222,9,0,136,56,99,126,127,193,59,129,204,1,59,129,74,14,239,196,198,121,147,4,239,68,175,131,146,79,48,40,239,4,0,22,190,248,150,65,
  76,121,56,0,114,197,188,152,20,168,190,204,11,127,219,164,97,76,89,66,128,136,200,102,214,86,163,241,231,236,0,171,17,0,30,70,48,195,106,52,
  236,244,16,177,119,251,135,234,206,91,159,226,217,89,112,217,172,70,175,212,154,136,26,154,2,101,53,2,128,214,76,59,48,146,169,68,66,6,80,252,
  216,203,188,71,114,233,148,151,4,180,134,92,62,83,73,81,42,3,80,86,35,0,76,188,245,190,193,186,72,12,30,0,93,78,78,16,172,70,144,16,
  106,20,17,217,204,218,103,96,130,239,246,237,213,0,105,15,237,41,22,153,52,92,17,199,182,13,93,64,26,173,44,52,1,152,225,137,129,66,219,219,
  208,240,235,43,10,74,113,227,208,103,75,67,23,176,175,71,19,128,25,126,45,48,247,222,81,196,25,199,100,238,83,37,28,153,43,225,49,113,227,202,
  2,14,10,47,160,28,27,1,32,73,90,251,80,33,135,93,97,159,113,197,28,220,82,128,74,187,89,55,6,36,47,47,9,71,216,16,156,171,22,79,
  247,43,231,64,93,65,145,242,49,64,133,156,210,48,71,142,45,20,164,233,95,205,195,129,242,225,7,40,185,168,183,160,80,140,38,63,41,48,241,209,
  219,104,62,136,110,56,42,238,95,174,86,8,217,32,40,197,222,202,2,160,9,240,37,70,104,107,192,84,239,253,86,59,180,117,79,123,37,15,22,172,
  27,162,156,81,81,52,69,105,127,11,101,1,148,137,105,172,172,150,70,217,100,128,38,114,224,149,185,111,55,243,250,16,73,161,40,226,170,26,162,229,
  64,104,203,236,45,0,220,56,89,148,167,135,40,150,83,214,186,80,62,148,176,249,146,132,203,31,86,225,196,255,136,154,195,78,107,80,177,195,43,74,
  2,208,54,33,116,16,44,3,0,15,29,190,235,253,100,192,50,80,74,141,44,139,146,71,71,96,34,18,209,30,162,37,80,112,179,0,184,206,2,176,
  169,103,62,103,246,172,89,58,190,7,128,161,177,74,55,217,172,65,225,196,152,16,29,118,101,33,8,227,95,176,134,0,102,44,98,211,247,126,59,161,
  172,30,141,71,48,224,12,134,43,57,1,147,169,133,157,119,1,248,176,46,244,222,240,132,202,87,162,158,64,136,205,201,46,214,108,212,144,78,122,110,
  0,60,165,58,187,86,24,46,224,185,81,14,142,250,184,194,13,21,175,47,146,183,102,149,186,40,80,22,19,128,7,164,4,217,0,128,177,102,64,145,
  154,193,233,13,174,50,139,111,9,56,184,72,99,241,161,107,46,224,57,98,102,60,199,112,252,57,85,36,10,187,21,23,134,96,37,185,84,196,0,79,
  113,19,236,10,3,84,7,40,155,84,92,114,194,47,102,236,201,5,255,22,92,44,2,192,19,168,176,66,235,101,158,54,205,56,15,173,223,181,140,78,
  76,113,243,7,128,178,11,172,182,196,229,28,46,136,80,122,1,229,154,63,0,74,10,148,128,25,52,114,209,100,64,198,210,230,84,21,0,217,184,129,
  27,30,8,199,215,77,148,116,27,184,212,67,177,15,148,236,83,23,6,160,116,254,150,241,15,168,113,162,148,71,89,183,109,33,133,34,99,101,75,228,
  224,244,156,194,154,146,210,43,137,53,68,197,0,116,228,212,174,250,135,27,87,190,162,48,236,254,193,241,138,179,46,202,22,201,62,94,37,64,37,238,
  48,1,176,239,148,229,40,217,217,185,79,80,46,204,214,30,148,175,4,1,96,71,228,107,239,98,169,165,6,168,3,226,80,90,40,25,242,233,164,4,
  65,208,250,40,28,126,30,15,201,51,18,50,216,26,78,204,224,52,1,80,32,229,187,87,228,127,168,172,129,114,249,12,0,133,17,196,227,36,149,178,
  176,37,92,236,70,141,40,79,247,47,184,182,65,14,220,230,221,163,181,178,35,219,105,134,95,135,165,38,5,193,165,39,146,200,164,187,192,28,10,137,
  26,167,153,184,136,65,222,73,203,106,112,152,143,114,0,50,83,20,150,57,85,165,253,97,0,202,201,44,0,82,70,90,1,131,115,72,56,109,20,244,
  144,84,32,187,228,187,94,164,155,160,60,77,0,78,72,137,41,103,168,216,108,0,146,197,224,28,54,102,197,235,40,55,43,178,60,40,212,144,229,126,
  35,176,230,131,67,36,59,185,58,232,180,45,223,105,244,21,23,46,145,240,115,37,19,174,25,0,69,126,210,56,27,73,38,74,212,164,126,185,117,138,
  2,189,66,69,11,150,155,60,132,190,78,220,81,34,222,6,253,120,96,53,21,185,95,84,232,49,97,201,20,190,151,48,232,72,83,60,149,87,128,114,
  141,2,224,142,71,250,146,31,124,81,201,135,98,135,115,153,27,85,39,154,135,146,11,187,173,18,69,38,47,71,225,79,144,112,156,76,241,213,50,5,
  61,108,9,44,154,153,54,9,216,26,98,102,35,85,163,196,207,230,18,5,201,222,8,23,203,172,223,52,85,36,79,137,48,114,92,29,37,221,70,173,
  198,94,240,241,183,172,231,102,44,165,192,184,102,80,178,93,253,245,226,170,240,60,148,96,68,185,141,243,116,56,131,160,1,34,162,7,234,178,15,233,
  170,43,0,57,100,46,159,255,113,255,235,74,162,118,133,64,131,82,108,125,1,83,166,242,90,142,60,112,7,255,105,211,73,138,239,6,30,174,253,172,
  2,157,61,235,95,250,7,100,165,56,105,180,149,137,252,86,5,182,18,177,1,236,134,68,77,130,146,212,32,20,110,123,155,57,137,60,44,69,33,118,
  68,14,51,252,166,239,209,195,91,190,115,156,28,168,127,247,213,196,81,188,181,75,116,218,89,214,145,0,251,7,95,227,144,225,20,175,111,42,238,200,
  95,115,127,129,186,165,147,167,205,127,152,178,187,2,232,23,92,166,123,108,239,254,33,63,2,202,60,0,182,4,174,117,144,55,87,43,93,89,63,210,
  123,195,245,204,3,148,153,2,188,190,233,75,81,121,127,37,61,44,30,137,200,97,198,146,63,209,220,0,192,30,134,137,199,67,113,15,2,96,233,89,
  34,148,131,0,72,207,116,15,148,131,241,8,40,243,0,0,251,175,235,92,171,37,194,35,16,0,60,88,34,48,5,192,33,96,137,48,44,30,137,200,
  97,198,226,202,28,101,26,4,138,33,188,141,82,181,71,142,80,215,165,148,25,52,58,117,90,156,154,240,167,106,86,105,191,176,111,248,78,89,27,215,
  77,35,240,168,6,100,113,108,9,128,186,35,74,118,180,25,65,220,20,193,64,25,88,10,148,70,45,0,156,105,11,182,211,5,92,53,133,1,220,220,
  240,84,16,213,76,58,50,65,62,93,184,221,2,37,99,102,91,0,100,162,148,5,255,85,101,238,102,37,201,46,178,95,37,173,53,138,76,14,56,18,
  78,236,221,28,121,29,184,59,92,174,194,5,48,128,62,115,249,73,223,138,83,216,55,115,108,15,225,2,158,50,202,245,40,70,226,80,113,197,29,111,
  153,152,47,230,66,232,38,86,31,249,93,191,118,242,115,159,159,81,96,23,51,159,57,168,172,38,155,223,228,203,50,139,162,119,118,191,146,150,134,139,
  51,160,187,87,252,161,127,0,251,61,87,173,209,149,48,0,30,24,107,13,96,134,167,4,136,137,136,11,79,77,122,1,110,226,234,73,90,247,1,234,
  142,111,135,215,230,242,134,92,160,117,127,226,234,146,130,218,109,235,225,124,251,147,65,147,14,77,29,11,77,0,102,120,90,70,94,217,244,244,138,171,
  167,247,58,51,249,83,237,76,254,52,198,211,87,47,158,190,217,244,252,2,111,186,111,173,250,246,183,255,194,211,125,87,194,25,190,115,156,233,59,199,
  183,125,231,248,142,239,28,223,245,157,99,130,239,28,103,249,206,113,182,239,28,231,248,206,113,110,0,239,149,196,197,110,234,57,84,116,51,159,80,156,
  29,185,96,227,188,73,202,66,19,128,25,158,231,251,240,243,125,231,248,158,239,28,223,247,157,227,2,223,57,126,224,59,199,133,190,115,252,208,119,142,
  31,249,206,113,145,239,28,23,251,206,113,137,239,28,151,250,206,113,153,239,28,19,125,231,104,251,206,81,249,206,209,241,157,227,114,223,57,174,240,157,
  227,199,190,115,252,196,119,142,159,250,206,113,165,239,28,87,249,206,113,181,239,28,215,248,206,241,63,190,115,92,235,59,199,117,190,115,92,239,59,199,
  13,190,115,252,204,119,142,27,125,231,248,185,239,28,55,249,206,241,11,223,57,110,246,157,227,22,223,57,110,245,157,227,54,223,57,126,233,59,199,175,
  124,231,184,221,119,142,95,251,206,113,135,239,28,119,250,206,113,151,239,28,119,251,206,241,27,223,57,254,215,119,142,223,250,206,241,59,223,57,238,241,
  157,227,94,223,57,126,239,59,199,125,190,115,220,239,59,199,31,124,231,120,192,119,142,7,125,231,120,200,119,142,135,125,231,120,196,119,142,71,125,231,
  248,163,239,28,127,242,157,227,49,223,57,30,247,157,227,9,223,57,158,244,157,227,41,223,57,254,236,59,199,95,124,231,120,218,119,142,191,250,206,241,
  140,239,28,207,250,206,241,55,223,57,254,238,59,199,63,124,231,248,167,239,28,169,107,224,127,51,0,85,123,0,0};
const char* TalComponent::electronichighwaysign = (const char*) temp1;






















// JUCER_RESOURCE: bmp00128_png, 48600, "E:/Code/tal/tal-reverb-3-juce/resources/bmp00128.png"
static const unsigned char resource_TalComponent_bmp00128_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,208,0,0,0,240,8,6,0,0,0,41,100,83,131,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,185,241,73,68,65,84,120,218,236,
  93,121,124,19,101,250,255,206,145,164,73,11,20,154,130,66,15,60,160,45,136,208,2,138,156,162,156,222,7,94,235,234,174,186,235,125,224,122,187,234,
  254,60,118,189,87,93,215,117,213,85,215,131,245,4,197,131,251,144,83,20,104,185,219,130,66,155,20,5,154,182,208,35,73,147,204,204,239,143,201,76,
  103,38,147,179,73,154,166,239,247,243,201,39,201,28,207,251,204,59,239,243,206,247,125,230,121,159,151,26,92,88,40,128,128,128,128,128,128,128,128,128,
  128,128,32,34,208,164,10,8,8,8,8,8,8,8,8,8,8,34,7,11,138,34,181,64,64,64,64,64,64,64,64,64,64,16,33,136,7,154,128,128,
  128,128,128,128,128,128,128,32,10,176,164,10,226,143,177,99,199,146,74,32,232,86,216,188,121,51,177,35,2,2,2,130,52,238,83,9,8,129,78,121,
  242,124,218,105,167,145,138,32,32,29,62,177,35,2,2,2,210,167,18,16,2,77,16,233,67,255,181,215,94,35,149,65,208,173,112,219,109,183,165,76,
  135,79,236,136,128,128,128,244,169,4,41,79,160,201,20,194,248,129,210,124,19,16,116,183,246,75,17,59,34,32,32,32,72,171,62,149,32,65,4,154,
  84,1,1,1,1,1,1,1,1,65,207,129,32,168,151,0,161,72,70,54,66,160,9,8,8,8,8,8,8,8,8,130,19,230,80,251,9,153,38,4,
  154,128,128,128,128,128,128,128,128,16,231,24,206,33,68,58,52,72,30,104,2,2,2,2,2,2,2,2,66,158,3,206,239,172,140,116,70,167,61,208,
  7,106,106,226,166,204,9,131,7,247,152,138,207,202,202,130,217,108,6,77,211,240,122,189,112,185,92,112,185,92,160,40,10,189,122,245,130,217,108,134,
  203,229,66,115,115,51,44,22,11,44,22,11,24,134,145,207,151,206,49,24,12,48,155,205,42,217,46,151,11,237,237,237,176,88,44,112,187,221,65,143,
  1,160,187,189,185,185,153,88,6,65,183,177,33,151,203,133,214,214,214,128,109,60,207,203,118,195,243,188,238,113,90,251,51,26,141,48,155,205,200,200,
  200,0,0,120,60,30,56,157,78,152,76,166,160,182,210,187,119,111,98,95,4,4,4,105,69,158,181,178,136,55,90,143,64,199,88,41,7,14,28,136,
  187,50,18,25,63,225,132,19,186,103,109,74,117,25,166,78,123,247,238,141,145,167,142,194,181,215,94,7,179,217,130,186,58,59,254,253,198,107,168,171,
  179,195,100,50,225,220,115,47,192,185,231,94,128,247,63,120,23,107,215,126,7,75,102,38,30,125,228,9,228,228,228,200,50,164,115,70,142,28,133,203,
  230,92,169,146,255,217,231,31,99,237,218,239,112,199,29,119,99,251,246,10,0,8,56,230,219,111,191,130,211,229,212,61,119,229,202,229,50,209,32,232,
  65,160,40,32,21,58,201,8,237,200,108,177,224,111,127,125,14,255,126,227,53,236,216,177,29,12,195,96,252,132,73,184,108,206,21,120,248,207,247,163,
  184,168,4,115,230,92,137,156,156,28,184,92,78,188,255,254,187,216,190,99,27,0,4,181,63,179,197,130,27,255,120,11,70,142,44,5,0,236,221,87,
  141,87,95,125,9,70,163,17,47,190,240,143,0,91,89,189,122,37,166,78,61,155,216,23,1,1,65,202,244,169,130,32,196,191,92,66,160,117,8,116,
  138,144,103,173,252,110,75,162,195,128,97,24,228,229,229,225,166,155,110,195,27,111,188,134,189,251,170,113,217,156,43,113,158,159,48,83,20,133,188,188,
  124,184,92,78,228,229,229,131,101,59,110,209,223,158,126,28,13,13,13,0,128,187,231,222,135,51,198,141,135,211,229,196,166,77,27,241,217,231,31,139,
  196,96,100,41,46,155,115,5,54,110,92,175,42,87,121,12,32,122,194,166,78,61,59,232,185,228,1,79,144,234,240,120,60,216,187,183,26,35,71,150,
  162,186,186,10,0,144,151,151,143,237,219,183,33,55,55,23,215,94,123,29,190,249,246,43,108,218,180,17,227,198,141,199,77,55,221,134,191,61,253,56,
  0,4,181,191,75,47,189,28,57,57,86,60,242,232,131,0,128,155,111,186,13,87,255,230,90,217,70,36,27,28,58,164,8,55,221,116,27,246,237,171,
  38,246,69,64,64,144,50,72,84,200,69,87,122,161,227,201,57,227,201,45,233,174,188,144,84,40,39,217,48,155,205,24,55,110,2,54,109,218,136,237,
  59,182,161,161,161,1,223,124,251,21,198,141,27,143,126,253,114,208,183,111,63,140,28,89,138,85,171,86,224,140,113,227,3,94,1,3,0,207,243,0,
  68,47,180,30,236,117,118,248,124,190,152,244,235,204,185,4,4,201,68,123,123,59,182,239,216,134,161,67,138,96,52,26,97,52,26,49,116,72,17,246,
  238,171,198,200,83,75,177,125,251,54,172,94,189,18,135,15,31,198,234,213,43,177,106,245,10,140,27,55,33,164,253,157,49,110,60,222,255,224,29,28,
  62,124,8,135,15,31,194,191,223,120,13,227,198,141,135,217,108,145,203,117,56,28,216,190,99,27,54,109,218,136,145,167,150,18,251,34,32,32,232,182,
  100,59,26,194,157,236,120,232,3,7,14,196,157,11,198,83,102,84,11,169,236,79,50,169,61,112,224,0,78,236,70,158,104,10,225,19,167,51,52,13,
  139,217,130,186,58,27,124,94,47,56,159,15,77,141,13,184,247,222,59,225,114,185,48,110,220,120,52,52,52,96,229,202,101,56,235,172,105,40,44,40,
  68,109,109,13,0,224,225,135,254,2,0,242,235,232,173,91,55,99,198,140,89,24,55,110,60,198,141,27,15,0,216,231,127,229,172,213,69,121,12,0,
  188,241,198,107,160,52,219,165,115,93,78,39,73,254,222,195,16,73,219,77,53,93,188,30,15,246,237,173,198,181,215,92,135,62,189,251,32,39,39,7,
  57,57,57,216,183,183,26,67,207,189,0,13,13,14,240,28,7,10,128,207,235,69,99,131,3,67,134,20,193,229,114,233,218,159,20,34,229,168,175,135,
  219,31,195,220,214,218,130,134,134,6,88,253,251,36,189,120,142,67,67,131,3,57,57,86,184,234,108,196,190,8,8,8,186,188,79,21,4,33,162,242,
  228,44,27,29,27,196,255,17,120,152,147,117,61,251,147,16,233,208,89,126,73,210,216,165,0,120,158,151,189,202,67,134,20,97,199,142,10,208,52,45,
  191,158,150,8,244,211,79,63,14,179,217,130,185,115,239,83,121,159,55,109,218,136,207,63,255,24,215,92,115,29,26,27,27,192,113,156,106,194,161,242,
  24,9,202,87,204,218,115,219,219,219,201,77,33,232,22,118,211,212,212,136,237,219,43,100,146,186,119,111,53,154,154,26,99,182,191,104,225,114,57,137,
  125,17,16,16,164,60,180,222,99,189,133,84,34,73,95,151,140,80,142,100,57,107,247,119,146,68,211,169,118,65,178,98,52,141,222,189,123,163,181,173,
  45,173,26,49,231,247,92,229,229,21,192,96,48,128,101,89,152,205,102,252,245,175,207,97,200,144,161,24,57,114,20,166,78,157,134,23,94,248,7,70,
  142,44,197,144,33,69,170,243,247,237,219,139,77,155,54,226,220,115,47,128,193,96,80,61,176,23,45,250,10,83,167,78,67,94,94,94,144,135,189,11,
  135,15,31,198,225,195,135,225,245,122,131,158,107,52,26,73,111,67,208,45,224,243,249,80,87,103,71,94,94,62,242,242,242,81,231,15,145,144,188,195,
  210,64,210,96,48,32,39,199,138,198,198,134,160,246,39,133,105,228,230,230,194,108,54,195,108,54,35,43,171,23,114,114,114,228,185,7,178,231,129,101,
  145,151,151,15,167,211,73,236,139,128,128,32,229,8,178,114,187,118,223,177,230,102,180,180,180,160,213,255,105,110,105,81,29,19,46,180,35,145,161,28,
  201,230,154,157,41,47,101,243,64,103,101,101,225,164,147,78,194,136,17,35,210,170,145,187,221,110,236,216,33,122,204,78,61,117,20,114,114,114,112,249,
  229,191,81,121,148,31,125,244,65,220,123,239,157,120,244,209,7,145,151,151,143,254,253,251,203,251,156,78,39,86,175,94,142,145,35,71,97,200,144,161,
  42,217,117,117,117,88,189,122,5,206,57,231,2,221,178,205,102,51,6,12,24,128,1,3,6,168,200,183,246,92,189,184,107,2,130,84,132,203,229,194,
  166,77,27,49,116,104,17,134,14,45,194,166,77,27,229,109,226,96,244,108,12,24,48,0,83,167,158,141,169,83,167,97,211,166,13,65,237,111,255,254,
  159,177,106,213,10,92,115,205,245,24,48,224,56,12,24,112,28,110,186,233,54,191,204,14,162,108,181,90,49,98,196,72,140,28,89,138,29,59,42,136,
  125,17,16,16,164,28,121,214,146,96,193,191,141,231,121,48,2,80,146,147,131,97,131,6,98,248,224,2,48,20,37,147,104,33,12,249,78,6,137,238,
  46,136,56,141,221,137,39,158,136,253,251,247,3,80,187,250,19,162,20,203,194,106,181,226,184,227,142,67,117,117,117,247,73,159,34,165,172,9,161,47,
  199,243,168,59,120,16,159,207,255,68,149,70,235,131,15,223,197,184,211,199,99,239,190,189,104,58,218,132,99,199,142,161,95,191,126,168,171,179,227,212,
  145,101,178,124,1,128,205,110,199,170,213,43,113,238,185,23,200,105,185,64,81,104,107,107,195,170,213,43,241,240,67,143,98,248,41,167,118,232,132,192,
  24,232,77,155,54,162,238,160,93,247,220,17,167,142,194,143,63,110,130,199,227,33,61,83,79,65,4,109,55,21,117,225,5,1,205,45,205,178,135,184,
  185,165,25,188,32,192,209,224,192,251,31,252,23,115,46,189,2,115,230,92,9,151,203,137,55,222,252,23,234,14,30,4,0,93,251,115,186,92,88,188,
  228,91,228,229,23,224,201,39,159,1,32,198,45,255,239,163,15,100,175,241,67,138,121,8,159,207,255,4,117,7,15,98,200,208,98,98,95,4,4,4,
  93,210,167,234,165,172,83,110,83,113,53,138,66,75,75,11,88,8,104,111,112,0,52,13,138,102,112,188,151,195,65,150,134,0,49,198,89,144,15,167,
  100,242,173,252,175,186,198,56,66,226,152,201,2,77,211,200,202,202,66,107,91,27,178,178,178,162,191,197,39,158,120,162,16,205,197,49,12,3,150,101,
  225,241,120,18,66,162,89,150,197,113,199,29,135,162,162,34,244,239,223,31,63,252,240,67,183,177,151,177,99,199,98,236,216,177,248,215,191,254,21,246,
  88,189,133,28,24,134,1,195,48,112,185,92,240,120,60,242,107,228,246,246,118,152,205,102,56,157,78,56,157,78,80,20,5,139,197,2,163,209,40,47,
  148,226,116,58,225,243,249,84,11,73,24,12,134,144,11,169,120,189,222,160,231,146,52,91,61,11,183,222,122,43,54,111,222,140,205,155,55,119,43,59,
  2,32,219,137,212,174,165,69,76,148,11,16,197,123,33,21,165,188,96,11,169,16,251,34,32,32,125,106,34,251,212,80,113,205,122,251,4,65,64,91,
  107,43,78,112,59,193,130,6,75,211,224,188,2,124,20,133,26,179,17,230,94,89,160,40,42,128,40,43,255,135,218,215,221,8,116,239,222,189,113,210,
  73,39,161,95,191,126,49,101,230,136,106,18,225,137,39,158,136,250,250,122,208,52,45,63,92,226,73,162,37,242,92,92,92,140,252,252,124,172,89,179,
  38,109,141,171,181,181,53,236,67,84,73,6,218,20,177,224,130,32,160,173,173,77,222,38,29,19,76,110,168,213,207,194,157,75,64,144,234,80,218,137,
  18,210,128,51,26,251,243,120,60,240,120,60,56,118,236,152,106,123,123,123,123,80,27,106,110,110,38,246,69,64,64,144,52,4,11,217,136,136,72,3,
  176,153,204,56,193,235,131,192,50,0,120,240,188,24,186,193,243,60,24,134,9,152,76,168,156,56,168,157,68,40,68,145,193,35,18,142,217,101,145,14,
  177,200,136,246,132,220,220,92,180,182,182,34,51,51,19,110,183,27,109,109,109,157,38,210,44,203,34,43,43,11,253,251,247,199,73,39,157,132,65,131,
  6,97,213,170,85,49,201,162,105,90,53,163,94,249,159,162,40,249,191,212,8,164,193,128,116,12,77,211,242,228,34,151,203,5,142,227,136,181,18,16,
  16,16,16,16,16,116,27,242,28,140,68,91,44,22,184,219,218,176,223,192,34,223,43,166,249,180,153,12,224,41,209,129,96,50,153,100,47,180,146,28,
  135,34,209,193,182,197,138,100,69,58,156,112,194,9,200,206,206,142,93,78,44,151,219,43,43,11,189,122,247,6,229,143,237,235,236,194,0,22,139,5,
  199,31,127,60,6,12,24,128,222,189,123,99,233,146,37,49,229,26,164,105,26,70,163,17,30,143,7,60,207,195,104,52,130,53,24,228,155,79,251,99,
  136,125,94,47,60,30,15,12,6,131,60,217,199,231,243,1,20,133,140,140,12,100,103,103,227,196,19,78,194,230,205,63,68,229,49,162,52,223,4,4,
  221,9,169,148,7,154,216,17,1,1,1,233,83,3,137,50,21,98,155,244,91,249,173,56,80,214,41,195,98,129,211,229,66,173,129,5,40,209,43,205,
  50,12,32,8,240,180,183,195,104,50,117,244,195,20,5,72,228,88,73,146,117,8,115,60,174,245,164,19,79,196,145,36,70,58,172,93,179,38,102,189,
  99,206,3,221,226,127,101,57,248,132,19,208,167,79,159,78,165,102,50,153,76,200,204,204,196,210,37,75,226,86,73,70,163,17,22,139,5,253,114,114,
  112,234,169,163,96,50,101,192,102,171,69,117,117,37,104,154,150,39,240,48,44,11,154,166,65,51,12,122,247,238,3,147,201,132,198,198,6,100,103,247,
  5,171,153,73,79,64,64,64,64,64,64,64,144,10,208,102,204,144,190,5,63,193,213,126,43,207,177,152,205,16,4,65,230,66,130,146,104,75,231,248,
  73,51,20,228,89,229,137,214,144,230,120,121,161,251,231,230,162,37,9,145,14,171,99,140,116,232,52,129,150,80,147,130,75,110,51,44,139,227,7,14,
  194,232,209,99,144,105,201,196,105,167,157,129,62,125,250,96,219,182,114,28,58,244,43,172,214,92,84,85,237,65,187,219,45,223,236,145,35,75,49,118,
  204,88,172,88,185,28,141,141,13,196,50,9,8,8,8,8,8,8,82,131,44,235,196,50,67,19,174,161,76,59,23,138,80,43,97,48,24,100,18,45,
  101,224,80,174,84,40,103,230,208,35,209,130,0,129,162,18,66,162,147,17,233,208,89,176,221,38,69,92,36,240,167,140,201,200,200,192,185,231,156,15,
  163,209,136,134,6,135,234,16,154,162,80,86,58,26,94,175,7,63,255,252,19,12,6,3,124,62,31,70,158,58,10,70,163,73,95,102,52,229,71,123,
  14,1,65,138,217,79,74,232,65,236,136,128,128,128,244,169,161,24,181,152,218,86,241,45,121,141,149,249,156,5,132,143,139,54,248,67,95,65,81,48,
  26,141,208,243,243,106,211,219,9,80,76,28,76,80,122,187,150,150,22,0,192,224,193,131,227,23,233,176,116,105,220,116,100,211,175,189,82,24,56,112,
  32,198,141,27,143,242,242,45,65,143,155,49,125,22,22,186,22,192,227,241,70,189,244,47,1,1,1,1,1,1,1,65,50,16,44,93,93,48,207,179,
  242,163,221,175,39,15,128,60,31,76,123,156,54,133,157,222,196,194,68,47,239,93,83,83,147,146,247,133,78,183,134,70,211,52,178,179,251,134,61,238,
  164,147,78,70,89,233,24,208,196,203,69,64,64,64,64,64,64,208,141,201,180,30,121,150,86,30,212,251,207,243,188,252,9,69,192,245,8,121,36,122,
  244,4,176,233,68,31,41,0,12,77,99,224,241,3,3,246,237,219,87,13,143,167,93,181,237,120,157,227,36,252,240,195,247,98,140,116,12,58,16,74,
  78,208,29,109,7,41,212,118,137,29,17,16,16,144,62,53,12,145,86,102,219,80,124,4,158,135,215,231,235,32,178,202,37,186,53,147,9,85,58,250,
  29,138,148,255,55,5,128,162,105,176,12,3,138,86,251,91,245,178,113,80,122,50,211,153,64,247,132,134,188,106,213,50,124,247,221,42,88,44,153,48,
  132,136,161,113,56,28,112,56,234,1,0,71,143,54,161,189,189,157,244,2,4,4,4,4,4,4,4,93,142,96,177,204,90,15,179,203,237,198,69,23,
  157,143,227,142,27,128,67,135,14,201,33,22,210,135,166,105,29,186,47,128,231,149,242,5,184,221,110,252,250,235,175,216,177,99,15,76,70,99,192,121,
  18,209,22,122,168,195,163,71,16,232,13,27,214,225,216,177,99,224,121,30,189,122,245,214,61,102,199,142,109,216,189,103,23,56,142,3,69,81,224,20,
  139,177,16,16,16,16,16,16,16,16,116,21,105,214,219,174,154,44,232,255,239,110,111,199,133,23,158,135,185,115,239,192,207,63,255,132,163,71,27,65,
  81,0,203,50,160,105,6,52,77,251,73,52,165,34,207,146,3,91,10,235,224,56,14,237,237,237,248,245,215,58,212,213,213,193,225,56,10,139,217,44,
  123,166,149,30,109,153,72,39,56,22,58,245,8,116,26,102,225,208,219,46,0,224,5,1,28,175,94,89,208,227,21,211,183,108,223,177,13,110,183,27,
  38,147,9,12,195,116,200,139,151,14,4,4,169,110,59,177,182,249,100,218,50,1,1,1,65,15,238,83,165,172,27,138,13,114,246,13,142,227,208,171,
  87,22,174,191,254,26,252,248,227,247,216,191,127,47,220,110,151,188,186,50,203,178,96,24,6,12,195,248,9,116,199,138,131,162,247,90,0,207,251,224,
  243,113,240,122,57,120,189,94,180,182,182,98,228,200,34,172,94,189,25,188,32,128,129,38,255,179,191,175,150,179,114,40,115,68,167,121,31,206,246,228,
  246,237,241,180,163,173,173,13,0,208,222,222,222,99,2,223,9,8,8,8,8,8,8,186,15,194,77,22,148,246,123,60,30,236,223,255,19,234,234,106,
  209,212,212,8,131,129,5,195,80,0,104,80,148,224,255,192,31,206,33,173,145,66,129,231,1,138,18,252,33,210,2,4,129,243,127,4,184,92,30,8,
  2,31,144,133,67,169,15,213,3,29,30,61,134,64,247,239,223,31,188,38,44,163,161,161,81,110,12,189,123,139,161,29,28,199,129,231,121,113,189,120,
  183,27,110,183,155,88,46,1,1,1,1,1,1,65,202,19,108,65,16,224,118,187,192,243,28,76,38,35,50,50,76,48,153,50,96,52,26,97,52,26,
  100,47,52,77,51,50,145,22,195,55,196,248,105,175,215,7,159,207,7,131,193,3,143,199,3,159,143,3,69,209,16,4,192,231,243,129,101,89,21,145,
  238,201,100,58,237,178,112,232,225,209,71,158,144,87,34,92,184,112,129,188,253,248,227,143,71,191,190,253,208,208,216,128,203,47,187,10,89,89,89,88,
  180,248,27,216,237,54,156,57,229,44,172,89,179,10,237,81,16,232,84,203,100,64,64,16,173,253,80,41,100,199,196,142,8,8,8,72,159,26,134,56,
  139,172,85,98,175,0,5,48,12,13,131,193,224,39,205,70,152,76,38,152,76,226,111,131,193,160,32,208,234,16,14,142,227,97,48,248,224,241,248,64,
  81,98,172,180,209,232,129,193,32,70,251,114,62,31,104,154,22,211,255,74,89,58,52,159,158,212,135,179,164,137,19,16,16,16,16,16,16,16,164,54,
  84,97,28,232,72,95,39,47,213,45,8,160,40,26,12,195,200,49,207,74,18,109,52,154,252,241,208,129,4,26,0,124,62,14,62,159,23,20,213,145,
  109,131,101,221,242,188,48,1,128,207,235,5,67,211,160,116,150,241,238,105,32,4,154,128,128,128,128,128,128,128,160,155,144,104,57,239,115,199,70,117,
  94,103,74,156,40,104,48,176,42,79,180,228,129,102,89,198,79,138,105,208,52,228,16,14,154,230,252,169,234,40,255,132,66,14,70,163,72,196,1,74,
  44,71,57,193,219,95,174,20,178,65,178,112,116,103,196,123,214,107,180,153,0,82,45,147,1,1,65,34,219,123,119,177,99,2,2,2,130,52,237,83,
  229,172,28,18,153,133,184,34,51,77,211,96,24,22,12,195,250,189,209,6,24,12,172,28,7,221,225,129,22,189,205,60,207,131,166,57,217,35,205,113,
  60,56,142,247,31,75,7,246,201,210,181,233,101,225,232,33,125,56,241,64,19,16,16,16,16,16,16,16,164,17,111,167,105,198,159,178,142,246,127,24,
  153,80,139,159,14,2,45,133,126,72,68,90,36,221,28,88,214,7,138,98,116,189,202,74,111,51,201,194,145,38,240,249,124,216,179,103,55,28,142,122,
  28,59,118,12,30,79,59,246,253,180,23,6,131,1,199,142,29,67,75,107,11,214,111,88,139,157,187,182,163,181,181,21,45,173,45,0,128,53,107,87,
  131,101,89,121,37,194,221,187,119,146,12,28,4,4,4,4,4,4,4,221,138,60,75,19,0,41,138,150,189,209,162,71,90,253,191,35,6,154,2,69,
  241,16,4,218,63,193,144,211,28,71,147,138,213,35,208,233,150,133,67,16,4,212,28,216,15,91,109,141,60,194,178,213,214,66,16,4,113,4,198,178,
  168,173,173,1,207,137,13,132,241,167,100,169,57,176,31,2,32,175,249,126,240,151,131,240,180,183,71,21,32,175,157,137,74,64,208,157,108,39,149,178,
  112,116,181,46,23,94,120,9,0,200,89,123,10,10,10,113,199,29,119,203,219,170,170,42,241,192,3,127,198,125,247,205,197,85,87,93,3,167,179,13,
  11,23,46,192,3,15,60,130,103,159,125,138,52,40,2,130,78,226,221,119,231,225,186,235,174,70,113,113,9,46,188,240,82,44,92,56,31,23,94,120,
  41,158,125,246,41,76,156,56,25,211,167,207,2,0,44,95,190,4,235,215,175,13,56,190,171,237,48,25,253,152,20,178,33,232,148,37,122,161,41,191,
  103,152,86,45,227,45,17,108,105,187,4,154,22,37,137,251,105,5,17,15,126,109,193,62,0,201,194,209,45,137,128,217,108,134,217,108,6,199,243,16,
  4,1,94,143,7,173,173,173,34,137,102,59,46,153,231,121,208,254,87,15,125,178,179,209,63,183,63,14,29,62,4,151,203,133,65,3,7,161,238,96,
  93,84,105,236,8,8,8,210,19,55,220,112,19,94,125,245,37,216,108,181,120,237,181,183,112,223,125,119,193,98,201,132,197,98,129,197,98,65,65,65,
  33,150,47,95,2,167,179,141,84,86,18,112,206,57,231,227,178,203,174,148,255,215,215,31,65,110,110,127,0,192,119,223,173,194,162,69,95,227,185,231,
  94,146,247,127,247,221,42,172,89,179,10,247,222,251,16,50,51,51,177,103,207,46,60,255,252,211,120,238,185,151,176,104,209,215,248,238,187,85,120,247,
  221,121,120,254,249,167,241,251,223,223,32,203,170,175,63,2,0,104,107,107,195,227,143,63,130,223,253,238,6,212,214,30,192,230,205,63,224,222,123,31,
  194,224,193,39,160,190,254,8,190,251,110,21,206,60,243,44,220,127,255,221,184,239,190,135,112,228,200,17,188,247,222,219,228,70,37,0,22,139,5,23,
  94,120,9,254,242,151,135,97,177,100,226,142,59,238,198,250,245,107,123,172,227,67,136,224,40,41,223,51,64,203,255,165,85,8,197,111,1,130,32,254,
  150,227,157,33,40,206,211,95,225,185,167,35,237,8,52,107,48,96,234,89,211,112,241,69,115,80,94,190,5,13,13,14,180,182,181,98,197,242,165,96,
  252,41,93,250,231,246,71,159,62,125,240,235,175,191,162,161,177,1,0,208,175,111,63,204,152,49,91,206,3,61,124,248,8,52,52,56,18,70,160,127,
  247,187,27,112,230,153,103,1,0,246,236,217,133,255,254,247,109,60,251,236,223,241,192,3,127,66,110,110,127,220,123,239,131,120,255,253,119,112,237,181,
  215,227,250,235,127,11,0,24,54,236,20,220,114,203,29,200,204,204,196,119,223,173,194,251,239,191,131,103,159,253,187,170,179,127,224,129,63,5,45,243,
  222,123,31,196,176,97,167,160,173,173,13,47,190,248,12,106,106,14,96,236,216,211,49,101,202,84,188,240,194,51,184,236,178,43,97,181,230,226,245,215,
  95,197,99,143,61,137,247,223,127,7,195,134,157,130,49,99,78,195,19,79,60,26,241,131,109,206,156,43,0,0,239,191,255,14,118,239,222,21,244,186,
  36,188,254,250,171,216,188,249,135,160,50,175,189,246,122,156,121,230,89,184,227,142,155,48,118,236,233,40,40,24,140,249,243,63,193,61,247,60,136,193,
  131,79,64,77,205,1,188,254,250,171,24,59,246,116,12,27,54,28,239,189,247,14,110,185,229,14,121,223,139,47,62,131,191,252,229,41,85,61,221,127,
  255,221,228,73,68,16,49,242,243,11,96,179,213,2,0,106,107,107,80,80,80,136,202,202,221,40,40,40,148,7,236,5,5,133,242,49,4,137,199,119,
  223,173,146,73,234,115,207,189,132,207,62,251,24,53,53,7,112,223,125,15,97,209,162,175,1,0,207,63,255,52,246,236,217,5,0,120,236,177,39,177,
  102,141,72,174,159,121,230,239,24,59,246,244,128,213,221,164,239,207,62,251,88,150,241,236,179,127,71,97,225,96,12,27,118,138,124,204,148,41,103,193,
  98,177,224,186,235,174,198,189,247,62,8,171,53,23,102,179,5,231,156,115,62,74,74,134,227,191,255,37,228,185,179,16,4,1,23,94,120,9,114,114,
  172,170,123,83,80,80,136,250,250,122,56,157,78,56,157,78,252,229,47,15,7,61,190,199,17,105,13,169,85,38,235,232,88,97,80,80,36,240,80,45,
  202,45,103,248,160,40,65,113,126,96,93,10,126,42,174,34,211,61,50,6,58,29,179,112,4,217,103,52,26,209,43,171,23,38,76,152,140,178,178,209,
  248,232,227,121,104,108,106,12,222,34,99,201,194,17,233,57,20,240,221,154,85,248,252,243,79,240,204,211,47,98,236,105,227,116,26,162,122,38,235,148,
  41,83,177,103,207,46,44,94,242,45,30,123,244,9,204,95,240,41,0,224,243,249,159,202,157,125,176,178,7,15,62,1,195,134,157,130,39,158,124,12,
  179,103,157,139,177,99,79,71,77,109,13,74,134,13,199,176,97,167,32,51,43,11,53,181,53,152,60,121,42,6,159,112,34,6,15,62,1,99,199,158,
  14,179,197,130,202,202,221,17,215,195,236,217,231,225,243,249,162,94,147,39,79,197,238,61,187,117,175,171,190,254,8,30,120,240,30,220,123,207,3,40,
  25,54,28,155,183,252,24,186,119,0,48,229,204,179,225,108,107,3,40,145,168,3,192,245,55,92,131,123,239,121,0,103,158,121,22,218,156,78,0,192,
  216,211,198,33,55,183,63,238,184,243,102,60,246,232,19,24,54,124,132,248,80,252,252,147,176,245,212,99,17,77,219,237,9,186,232,205,34,215,252,182,
  215,217,49,126,194,36,216,235,236,0,40,140,159,48,9,27,55,174,15,170,247,253,247,223,31,39,213,168,176,196,35,81,229,116,21,49,121,238,185,231,
  2,238,143,0,65,174,107,57,19,129,63,131,64,102,86,22,4,65,192,189,247,62,8,0,120,224,193,123,48,120,240,9,248,124,254,167,104,115,58,81,
  91,123,0,185,253,7,116,116,48,154,251,61,103,206,21,152,51,231,10,188,255,254,187,0,128,205,155,127,192,57,231,156,135,35,245,71,0,80,24,54,
  108,56,182,108,249,17,160,40,188,240,226,179,0,0,71,131,3,115,230,92,129,239,214,172,66,189,163,62,160,29,196,235,254,167,43,2,238,49,128,234,
  189,85,40,200,47,132,213,154,27,104,147,20,133,251,239,123,24,197,197,37,184,254,134,107,130,31,159,230,253,24,69,211,16,196,117,184,85,164,151,231,
  197,183,236,60,207,131,227,4,127,90,58,49,53,29,199,117,132,116,72,222,103,233,88,241,120,241,195,243,60,4,129,87,101,204,147,206,83,94,159,188,
  77,251,172,39,89,56,8,18,53,186,14,183,95,185,190,61,0,212,212,214,96,206,165,151,163,205,217,134,199,159,120,20,109,109,226,235,226,57,151,94,
  142,57,151,94,142,247,223,127,23,223,173,89,165,43,175,190,254,8,234,235,143,224,218,107,174,195,150,173,155,177,104,209,215,200,204,204,196,216,49,167,
  163,190,254,8,38,79,158,138,181,107,87,195,98,177,96,204,232,177,16,4,1,37,37,195,1,0,91,182,110,142,248,186,246,236,217,133,217,179,206,197,
  230,45,63,224,197,191,63,11,139,37,83,229,61,144,62,86,107,46,222,254,207,251,0,128,69,139,191,13,43,119,243,230,31,48,123,214,185,88,188,68,
  60,182,176,112,176,76,186,119,239,217,141,225,195,134,203,100,125,247,238,157,184,244,146,203,112,203,205,183,99,241,226,111,177,121,243,15,114,29,133,171,
  39,2,2,61,216,108,181,200,207,47,128,221,110,67,65,129,232,141,22,4,1,55,92,127,35,222,121,247,45,184,156,78,76,155,54,3,31,127,60,47,
  42,130,64,16,59,206,156,114,22,206,156,114,150,108,203,115,46,189,28,128,232,152,144,250,198,23,94,124,86,246,64,7,235,131,181,253,19,0,124,246,
  249,39,88,188,248,27,191,83,224,92,108,217,186,25,215,94,115,29,114,115,251,195,86,43,190,101,144,6,236,18,214,172,89,133,75,47,185,12,123,36,
  167,1,185,255,157,70,85,85,37,4,65,192,168,81,101,242,189,177,217,106,81,80,80,0,139,197,130,119,222,125,11,207,62,243,162,124,255,170,170,42,
  1,0,163,70,149,165,177,175,131,210,229,16,74,18,43,45,203,221,65,136,57,112,156,15,28,39,165,164,227,0,8,114,206,103,137,116,139,199,113,240,
  249,36,18,205,201,36,58,216,160,154,234,193,206,40,66,160,83,224,1,176,103,207,46,108,222,252,3,46,189,228,178,144,228,122,241,226,111,176,103,207,
  46,76,153,50,21,143,61,250,132,236,249,80,118,246,193,208,214,214,134,7,30,188,7,179,103,159,135,177,99,78,195,176,146,97,248,110,205,106,212,215,
  31,193,230,45,63,98,248,176,225,88,188,248,27,212,214,214,96,202,148,179,224,112,212,163,176,112,48,156,78,39,118,239,222,25,241,53,189,254,239,127,
  98,202,228,169,24,59,246,52,60,243,244,139,120,225,197,103,116,143,115,56,234,241,192,131,247,224,218,107,175,195,148,201,103,6,125,200,41,7,15,153,
  153,153,152,50,249,76,236,14,115,108,109,109,13,238,188,235,22,92,122,233,229,184,244,210,203,97,182,88,0,104,60,245,4,4,97,6,176,19,39,76,
  66,209,208,98,216,236,181,120,251,157,55,113,195,245,55,194,98,177,224,163,143,230,193,233,116,162,186,186,10,57,57,86,153,76,3,128,83,67,170,8,
  18,119,127,190,91,179,74,246,16,63,251,204,139,248,236,243,79,144,155,155,27,224,132,144,251,144,154,3,24,86,50,12,181,181,7,80,88,120,2,190,
  91,179,26,109,109,109,200,205,205,149,195,187,130,161,181,181,21,139,22,127,131,57,151,94,14,65,16,176,123,207,110,140,29,115,26,22,45,250,26,247,
  222,243,0,106,106,14,200,111,222,36,242,78,144,24,39,147,211,233,196,71,31,205,195,253,247,61,12,167,211,9,187,221,22,149,115,42,29,137,180,148,
  187,89,185,44,55,207,115,240,249,56,120,189,62,24,141,62,248,124,62,120,60,30,8,2,192,178,2,56,174,35,19,135,116,142,184,18,161,207,255,241,
  194,231,243,193,235,21,73,181,94,249,90,242,220,211,200,52,155,110,151,75,133,216,158,232,140,26,209,204,60,165,252,30,11,233,1,144,153,153,9,0,
  24,92,56,24,22,75,166,174,220,71,30,121,28,149,149,187,241,193,251,239,98,204,232,211,112,194,224,19,68,163,137,160,204,49,99,78,195,181,215,94,
  143,39,158,120,20,20,128,89,179,206,197,152,209,99,81,88,56,24,133,133,131,69,29,44,22,209,131,60,251,60,188,254,250,171,184,249,230,219,145,153,
  153,137,6,71,125,68,215,148,153,153,137,87,94,121,29,255,254,247,63,241,249,231,159,224,209,71,159,144,207,211,94,151,164,55,252,222,104,42,130,123,
  177,104,209,55,184,231,158,7,0,65,64,77,205,1,140,25,61,22,75,22,127,131,97,37,195,80,83,115,64,206,194,50,231,210,203,81,82,50,28,79,
  61,245,23,100,90,50,49,216,127,125,144,202,36,136,139,141,36,218,134,187,82,151,175,191,250,2,95,127,245,133,106,219,227,143,63,18,160,227,31,254,
  112,45,0,96,111,117,21,30,120,224,79,164,125,37,185,173,82,154,109,107,215,172,198,163,143,62,129,37,254,183,90,247,222,243,0,0,96,237,218,213,
  248,252,243,79,112,243,205,183,99,246,236,243,176,103,207,46,108,221,242,35,50,45,153,184,246,218,235,228,109,85,149,187,229,62,100,206,165,151,163,190,
  254,136,220,87,173,91,187,26,231,204,62,15,240,255,30,59,230,52,188,253,159,247,225,112,212,227,189,247,222,81,181,91,210,14,58,143,63,252,225,90,
  80,126,219,122,254,249,191,1,0,158,127,254,111,160,0,108,220,184,14,27,55,174,83,221,123,237,241,169,148,81,40,158,164,89,59,64,160,41,10,188,
  226,27,0,120,94,240,147,103,15,12,6,22,30,79,199,242,220,130,32,45,154,162,76,99,7,8,130,232,117,246,122,189,240,122,125,240,122,189,240,120,
  68,18,205,243,29,30,104,218,255,9,154,125,67,177,188,55,201,194,65,144,20,180,181,181,97,205,154,85,184,249,230,219,1,0,159,127,254,137,60,42,
  124,235,173,247,0,0,127,255,251,115,248,211,159,238,199,172,89,231,162,166,230,0,214,172,89,133,201,147,207,148,61,173,245,245,71,240,208,67,247,234,
  202,223,178,229,71,76,158,124,38,158,121,230,69,8,130,128,249,243,63,197,165,151,94,142,39,159,124,12,53,53,7,240,200,35,143,99,242,228,169,168,
  173,173,193,145,35,135,177,103,207,46,172,93,187,26,37,37,195,35,246,168,180,181,181,97,254,252,79,229,107,88,179,102,149,172,231,77,55,221,166,186,
  46,171,53,87,117,93,145,120,34,42,43,119,203,158,234,197,139,191,65,113,241,48,188,245,214,123,168,175,63,130,247,223,127,23,99,198,156,230,47,119,
  53,70,143,30,139,255,252,231,125,212,215,31,193,191,255,253,79,220,124,243,237,114,61,57,28,245,120,240,193,123,72,163,35,136,217,11,214,147,189,46,
  169,0,237,27,55,165,61,75,131,26,233,91,137,187,238,186,5,55,221,116,27,198,140,57,13,37,37,195,177,102,141,152,157,35,152,44,237,189,189,243,
  206,155,229,223,79,62,249,88,128,252,27,252,177,184,209,122,17,59,219,6,147,37,55,152,252,120,120,125,227,33,51,173,109,209,191,218,96,80,167,131,
  159,184,10,0,56,206,39,123,146,189,94,47,24,198,35,215,13,207,139,161,26,226,74,132,8,32,208,226,57,162,183,218,235,245,248,191,125,106,66,172,
  137,127,86,198,58,247,164,222,144,42,42,42,18,210,167,125,81,176,88,44,56,235,172,233,184,248,162,75,21,89,56,218,176,114,229,50,152,205,102,100,
  101,102,97,218,180,153,242,36,194,109,219,202,1,136,179,236,207,153,125,158,156,133,227,172,169,103,99,237,186,53,56,122,244,104,196,229,143,25,51,6,
  163,71,143,198,27,111,188,145,176,135,117,172,157,71,34,101,199,187,35,77,84,219,32,8,142,155,110,186,9,91,183,110,197,150,45,91,186,92,151,206,
  218,81,50,72,115,50,109,165,179,109,55,145,229,68,83,87,209,202,239,44,129,234,105,153,24,8,82,171,207,143,87,159,170,205,20,163,12,77,82,254,
  151,98,157,189,94,47,238,190,251,15,112,56,26,224,245,122,96,54,103,32,35,195,12,163,209,8,131,193,8,131,129,213,44,229,221,17,194,33,198,63,
  251,224,243,113,240,120,218,225,118,187,113,228,72,3,190,249,102,5,92,46,55,12,6,3,40,138,66,70,70,134,202,131,173,12,231,208,126,167,51,136,
  7,58,69,16,143,206,62,145,15,140,238,254,48,234,169,75,141,18,116,127,59,84,202,75,228,64,54,210,114,98,189,190,100,244,33,132,52,19,164,123,
  159,31,106,18,33,77,211,240,249,188,120,231,157,143,49,118,236,8,248,124,28,12,6,22,6,131,17,44,203,194,96,48,248,87,26,164,229,9,132,162,
  60,65,53,145,80,242,94,187,92,46,172,94,253,61,218,219,61,50,121,6,160,34,207,61,25,61,134,64,103,101,101,5,13,132,239,234,78,151,116,250,
  164,67,37,72,15,242,156,46,58,146,62,137,128,244,249,169,67,152,245,236,81,239,186,104,154,70,102,102,22,28,142,38,172,90,181,9,60,47,104,50,
  104,232,172,42,168,67,200,165,156,209,98,218,59,30,172,127,197,102,189,114,149,255,201,36,194,238,220,208,160,31,180,127,214,212,105,184,248,162,75,177,
  109,91,185,188,52,47,0,148,149,142,134,221,86,139,134,198,6,76,158,116,102,80,121,177,232,16,149,129,147,126,46,153,61,42,33,209,65,218,45,144,
  58,241,107,201,158,136,69,236,144,128,128,244,249,169,222,167,202,100,90,25,15,173,136,125,22,32,230,133,238,221,171,151,152,130,142,22,87,98,22,0,
  8,60,47,46,182,18,106,112,44,77,0,84,144,101,3,203,202,219,229,229,192,131,133,109,232,92,127,90,19,104,98,93,93,59,58,142,117,127,60,58,
  132,68,203,143,228,26,99,45,167,51,114,137,39,154,32,149,236,144,32,245,238,121,34,251,166,88,101,119,165,220,104,100,39,34,46,190,179,215,158,110,
  125,190,68,152,41,63,145,22,116,174,159,97,24,208,130,0,159,130,96,83,0,4,233,56,5,1,87,17,94,5,137,166,160,142,105,166,40,10,38,147,
  73,93,86,15,206,13,157,126,43,17,82,84,252,134,62,177,172,34,20,225,57,242,40,50,84,103,17,170,67,136,177,177,10,234,37,133,194,202,239,244,
  67,36,220,170,105,177,118,164,157,148,43,16,242,19,216,110,35,168,215,164,219,114,50,136,84,23,216,97,42,13,100,19,61,24,79,21,50,26,105,223,
  151,232,190,73,219,110,130,149,17,112,237,93,40,87,79,118,216,190,36,201,50,83,174,207,143,99,159,170,12,227,208,230,124,150,73,180,226,183,244,161,
  0,152,50,50,224,241,120,100,194,76,69,88,30,130,144,103,154,166,59,178,111,32,200,164,193,30,67,160,9,82,206,51,98,50,24,96,96,152,176,199,
  122,57,14,237,94,175,202,176,34,121,120,68,43,63,150,135,72,172,215,160,53,96,189,7,104,180,178,9,81,38,72,4,113,11,118,78,60,83,136,69,
  35,51,214,114,98,153,164,24,109,29,69,123,29,177,200,78,86,91,8,213,55,37,179,189,165,66,221,164,170,237,246,148,62,95,69,162,53,215,46,237,
  99,24,70,246,26,235,181,135,96,246,31,44,195,70,79,205,184,65,8,116,138,63,156,165,237,6,134,193,182,93,187,66,142,21,25,134,193,136,146,98,
  153,224,70,211,89,198,34,63,220,3,80,91,118,172,215,16,201,181,68,43,59,88,135,74,66,57,136,29,134,218,238,106,107,67,110,78,78,88,57,245,
  13,13,48,103,102,134,37,139,218,242,82,121,32,155,168,193,120,34,7,202,145,34,214,251,26,73,223,212,89,217,233,32,55,21,100,166,123,159,31,204,
  35,29,236,216,72,211,59,234,77,8,212,146,231,96,251,99,25,132,119,123,2,157,110,147,8,57,159,15,213,85,149,120,167,225,13,28,61,122,20,237,
  237,237,216,191,255,39,24,12,6,28,61,122,20,45,173,45,216,184,113,29,246,236,217,137,150,150,22,180,180,182,0,0,214,173,251,14,44,203,162,193,
  225,0,0,236,222,179,11,110,151,43,97,43,17,234,26,182,166,17,50,12,139,156,190,217,232,149,149,37,111,111,105,109,69,67,211,81,156,115,206,57,
  176,237,255,57,230,104,149,152,229,135,74,228,30,175,50,18,161,127,144,14,134,208,103,117,93,244,244,149,8,149,118,152,155,147,19,241,64,173,205,237,
  214,19,22,178,173,69,51,16,244,72,4,49,194,129,44,213,153,50,66,216,185,246,58,98,150,175,41,135,234,228,64,217,19,5,129,238,244,125,237,2,
  217,169,32,183,213,229,10,75,142,4,65,232,114,153,242,245,167,64,159,31,207,62,85,235,65,14,26,214,1,168,98,158,1,127,220,179,34,196,40,96,
  50,161,98,245,192,112,68,58,20,121,238,105,207,212,180,243,64,243,60,143,3,7,246,163,182,182,70,142,213,169,171,179,67,16,4,57,247,97,109,109,
  13,14,28,224,59,114,33,82,20,106,106,14,136,55,223,159,223,240,224,193,131,240,122,60,73,241,122,133,26,17,246,202,202,194,222,159,127,150,255,15,
  61,233,36,52,30,61,22,183,81,94,162,229,71,82,70,52,94,1,109,93,69,163,191,94,57,196,11,221,51,17,169,29,70,58,80,235,204,67,49,22,
  249,209,78,212,74,197,107,232,10,217,74,194,17,137,220,88,250,166,104,100,135,243,216,105,9,83,87,202,141,166,221,117,181,204,238,214,231,207,158,125,
  30,74,74,134,99,203,150,31,113,237,181,215,225,223,255,254,39,106,107,107,240,167,63,221,143,197,139,191,197,111,127,251,59,0,64,125,253,17,188,244,
  210,243,152,59,247,30,44,94,188,8,235,215,175,193,107,175,189,5,64,0,199,241,16,4,30,60,47,224,137,39,30,5,0,60,244,208,163,176,217,109,
  120,249,165,231,113,234,200,82,92,125,245,181,88,188,248,27,108,250,225,123,252,241,15,55,99,208,160,60,28,60,88,135,121,243,222,199,169,167,142,196,
  236,217,231,129,166,105,236,219,183,15,31,126,248,95,140,26,85,134,43,174,184,10,20,37,114,165,79,62,249,31,54,110,92,7,139,37,19,119,221,117,
  15,6,15,30,12,155,205,134,183,222,122,29,179,102,157,131,201,147,167,98,238,92,113,165,207,194,194,66,124,240,193,127,211,156,64,167,225,36,66,142,
  227,224,227,56,57,119,161,207,231,131,193,96,0,205,136,43,239,24,13,6,208,52,13,158,23,151,173,4,0,31,199,65,16,4,57,89,120,168,201,69,
  33,203,87,126,199,203,21,231,71,73,73,9,56,143,199,255,42,197,223,25,160,51,147,8,227,47,63,150,50,164,17,113,36,158,8,85,221,70,40,91,
  247,30,133,219,214,19,145,164,137,123,73,183,163,100,12,2,209,185,87,151,97,7,130,113,168,139,68,151,17,207,58,138,118,160,28,118,2,89,144,9,
  126,145,212,73,76,125,83,20,245,45,132,240,50,234,221,147,174,150,219,213,109,46,106,153,93,217,231,71,211,167,42,220,213,147,167,76,197,7,31,188,
  43,110,160,0,135,195,129,135,255,124,31,230,206,189,23,147,167,76,5,64,131,162,69,217,52,77,227,149,87,94,68,65,65,1,198,143,159,132,191,252,
  223,159,33,8,2,172,86,43,88,214,128,19,6,159,128,204,172,94,40,28,124,130,127,33,21,22,103,159,61,3,44,107,192,67,15,223,135,27,111,188,
  21,103,140,159,8,183,203,133,173,91,183,224,155,111,191,194,189,247,60,128,9,19,38,193,229,118,129,97,88,220,53,247,86,213,53,205,156,117,14,40,
  138,194,205,183,252,1,115,231,222,139,73,147,207,148,149,159,60,249,76,56,93,78,80,20,45,14,12,134,13,199,223,255,254,92,90,62,50,233,116,231,
  4,6,131,1,125,251,246,197,152,209,99,81,86,58,26,89,89,89,152,60,121,10,174,187,238,143,152,49,99,22,50,253,49,88,57,57,57,48,155,205,
  93,238,9,139,100,52,78,51,52,40,154,142,218,27,21,233,113,177,202,239,76,25,210,185,193,60,131,209,232,160,148,29,75,253,18,244,60,132,108,39,
  58,3,53,137,148,40,159,141,49,219,73,4,242,59,187,178,96,34,202,72,84,29,5,102,137,136,76,182,178,175,208,126,34,117,80,132,147,27,113,223,
  20,133,236,168,250,167,20,147,155,138,50,187,123,159,191,167,114,55,6,23,14,70,73,201,112,29,62,174,31,78,33,45,146,66,211,140,106,27,195,48,
  248,245,215,95,80,90,90,134,73,19,39,227,208,161,67,96,24,26,131,7,159,128,93,187,118,128,162,40,28,216,255,19,6,15,62,1,12,67,131,97,
  25,184,221,46,252,240,195,247,40,25,54,92,150,241,175,215,222,196,19,143,63,45,203,45,44,28,140,138,138,173,0,128,170,202,61,40,44,28,12,138,
  162,176,117,235,102,204,158,125,46,50,45,34,175,90,188,228,219,180,37,207,64,26,79,34,100,89,22,25,25,25,200,27,148,143,203,46,187,2,133,133,
  39,96,219,182,114,212,214,214,32,215,58,0,39,158,112,34,188,30,15,50,76,25,104,107,107,195,5,231,95,140,189,123,171,176,181,124,11,120,158,79,
  168,110,151,95,126,121,72,227,254,118,225,66,127,95,161,223,203,48,52,43,27,202,21,87,92,17,117,249,137,150,159,232,50,58,43,91,234,124,62,253,
  244,83,194,28,9,2,108,82,219,206,34,25,168,117,198,14,19,37,191,187,95,67,162,100,19,157,19,43,55,21,101,126,246,217,103,221,170,63,90,188,
  248,91,204,158,117,158,252,63,55,55,23,111,189,249,95,212,215,215,227,195,15,255,139,49,163,79,83,45,171,173,36,205,74,146,205,178,44,246,253,180,
  15,179,102,158,131,163,71,155,208,238,105,7,77,51,96,24,241,35,46,255,205,248,151,247,102,192,208,140,188,36,184,244,97,24,6,183,222,246,71,117,
  125,7,41,211,102,171,133,37,51,19,147,39,79,69,101,229,238,180,127,110,164,221,36,66,233,219,98,54,99,252,248,137,232,211,39,27,217,217,253,34,
  58,191,172,108,12,236,117,54,28,58,116,72,108,20,49,148,31,201,164,129,207,20,196,77,189,124,166,248,59,51,35,67,119,180,45,19,68,150,1,195,
  136,13,248,211,79,62,9,74,16,131,17,244,88,229,71,244,26,179,147,101,68,130,88,100,235,78,128,64,72,49,61,10,145,182,221,116,213,69,207,38,
  165,118,22,201,64,77,219,134,35,201,191,27,171,252,88,236,48,213,174,33,26,59,143,183,108,162,115,108,114,63,249,248,227,136,218,94,150,255,77,110,
  87,201,12,213,231,35,137,125,126,180,253,152,242,248,181,107,191,195,236,217,231,162,173,205,9,10,254,16,142,135,239,149,143,117,58,219,208,63,183,63,
  6,244,239,175,34,177,52,77,203,229,209,20,5,134,161,81,103,183,97,214,204,115,176,108,217,98,228,23,20,128,166,105,252,250,203,65,140,25,115,26,
  86,174,92,134,97,195,135,195,110,179,201,132,57,211,146,137,73,147,207,196,134,245,107,65,107,100,74,176,219,106,49,102,204,105,88,182,108,49,134,13,
  27,14,155,173,22,102,179,5,0,176,116,201,34,252,233,79,247,163,178,50,253,159,173,105,25,194,97,54,155,49,118,236,233,56,241,196,147,163,62,215,
  96,48,2,0,76,70,99,220,245,138,246,213,82,208,206,130,97,64,83,116,200,114,34,9,123,136,86,126,40,121,66,208,236,28,177,93,67,100,29,78,
  236,245,19,205,125,32,72,47,68,125,255,35,24,168,5,179,193,136,202,136,65,126,204,222,133,40,175,33,226,122,74,228,53,196,81,118,168,176,147,100,
  235,28,83,232,93,23,202,213,107,215,33,183,119,161,204,238,222,231,59,157,109,88,188,248,219,160,251,215,173,251,14,179,102,157,139,167,158,122,14,85,
  85,123,80,85,181,39,192,27,12,0,52,205,160,188,124,11,220,110,23,42,42,182,202,30,230,101,203,150,128,166,105,188,254,250,219,232,159,123,28,54,
  108,88,11,154,166,49,101,202,84,188,244,210,63,209,212,216,128,101,203,22,203,164,252,205,55,255,139,55,223,252,47,38,79,62,19,20,69,97,201,146,
  69,160,40,10,111,190,249,95,88,173,253,177,118,237,106,185,204,202,202,221,178,247,121,214,172,115,241,167,63,221,159,182,207,146,244,155,68,8,32,195,
  108,198,204,153,231,162,166,102,127,204,162,178,251,246,133,203,237,6,199,113,209,149,31,227,68,172,192,73,40,193,101,208,52,3,202,63,218,142,91,48,
  89,132,242,245,38,212,4,159,112,25,89,25,177,77,46,138,78,255,128,50,34,153,104,210,147,208,137,182,155,78,186,232,181,229,136,6,106,33,86,63,
  11,215,190,162,149,31,202,94,130,217,98,172,215,32,68,104,35,157,169,163,120,12,148,99,153,232,23,47,185,122,164,44,210,250,8,39,63,22,157,19,
  41,87,183,93,4,185,183,93,45,179,203,251,252,40,250,177,37,75,23,97,201,210,69,0,128,181,235,214,0,20,165,187,77,194,218,117,107,196,109,138,
  178,150,46,91,140,165,203,22,203,171,4,58,26,26,112,243,45,55,128,162,105,220,115,239,93,0,128,87,254,241,119,249,148,103,158,125,74,165,195,178,
  229,75,176,108,249,146,14,145,52,141,245,27,214,97,253,134,117,1,215,229,116,57,241,215,191,61,174,218,252,225,188,247,228,253,127,127,233,121,213,181,
  165,235,243,53,237,98,160,25,134,193,160,129,121,232,215,175,95,0,129,110,109,109,69,187,167,29,123,42,119,161,233,104,35,28,142,122,120,125,250,249,
  67,173,214,92,52,52,52,192,229,114,165,220,53,210,180,122,194,68,188,87,15,83,202,79,198,53,116,39,217,4,61,110,100,17,217,32,48,230,212,88,
  17,14,4,117,236,56,220,66,45,157,45,163,43,235,40,234,129,114,212,247,32,145,117,18,157,236,80,139,126,164,162,220,174,171,223,68,183,99,2,2,
  66,160,81,56,120,112,192,246,165,203,22,97,197,138,101,240,120,60,216,181,107,39,118,237,218,169,218,239,245,170,115,62,103,102,102,129,97,186,216,32,
  169,16,62,153,120,48,196,8,228,135,203,163,25,54,159,110,132,215,16,201,195,47,210,215,121,113,171,31,2,130,4,13,212,82,113,32,219,221,6,227,
  122,178,35,39,141,145,203,141,169,111,138,178,62,180,101,68,27,98,144,108,185,169,230,224,32,78,19,130,46,33,208,233,58,137,80,137,195,135,127,193,
  234,85,43,208,214,218,10,134,101,193,249,124,224,56,14,52,77,131,245,231,125,174,222,91,133,113,167,143,151,207,201,206,238,3,3,203,198,109,37,66,
  65,179,93,181,74,16,128,194,130,2,249,119,195,225,195,17,151,89,88,88,24,102,108,174,46,19,49,200,143,182,141,196,122,13,161,174,67,217,237,71,
  35,187,160,160,64,150,105,179,219,17,42,72,166,167,247,191,61,97,37,194,152,236,48,130,129,90,167,236,48,2,249,202,118,172,62,68,102,191,161,237,
  48,5,174,33,102,59,143,82,118,56,244,36,157,187,66,110,87,203,76,165,62,63,85,250,84,130,4,17,232,158,112,145,53,53,53,240,120,60,33,71,
  223,71,155,154,186,84,71,155,205,38,123,8,228,44,19,145,156,87,91,235,127,142,134,55,211,128,44,31,145,202,143,34,35,137,180,68,104,44,215,32,
  17,2,42,136,76,9,209,202,166,82,112,97,14,130,212,68,82,236,48,70,27,137,101,65,163,68,217,122,167,236,60,68,25,74,91,143,185,15,9,131,
  84,234,155,18,174,115,23,200,77,5,153,164,207,39,72,14,129,78,195,73,132,218,49,159,219,237,6,175,156,52,161,23,220,31,108,181,162,104,234,39,
  148,209,106,39,247,40,255,199,178,234,161,166,204,104,39,141,196,229,154,244,11,138,125,5,199,96,255,59,91,63,193,238,51,153,72,216,185,246,158,104,
  59,78,132,46,93,108,135,157,177,145,168,39,180,37,114,37,213,78,218,121,208,201,200,177,222,247,68,181,219,68,247,77,201,208,57,149,229,38,74,102,
  170,244,249,17,246,169,191,253,237,239,144,107,237,143,151,94,126,62,228,113,151,92,114,25,156,109,109,88,178,116,17,114,173,185,184,241,198,91,81,88,
  56,24,181,181,53,120,233,229,23,224,116,182,1,0,38,79,58,83,94,254,123,193,130,207,176,100,233,34,88,44,153,248,219,95,159,197,195,127,126,0,
  78,103,27,254,250,215,231,144,107,205,5,0,172,93,247,29,150,46,89,132,191,254,181,99,209,147,15,63,124,15,245,142,35,184,123,238,125,248,240,195,
  247,100,121,210,62,237,255,181,235,190,235,113,143,76,186,167,92,104,86,86,22,250,244,233,3,134,97,208,171,87,47,88,173,86,249,63,65,106,33,226,
  244,95,4,4,221,184,109,119,87,29,123,178,77,146,190,137,160,43,49,122,244,88,140,30,61,86,254,157,155,155,139,185,119,223,14,75,102,38,74,74,
  134,1,0,44,150,76,92,114,201,28,124,248,225,123,120,233,229,231,113,201,37,151,193,98,201,196,152,209,99,97,177,116,28,39,145,235,151,94,126,30,
  147,39,157,41,111,123,233,229,231,177,96,193,103,152,57,235,156,128,242,111,188,233,58,220,120,211,117,114,154,186,80,199,18,2,157,38,152,50,229,44,
  60,245,228,179,184,242,138,171,209,55,187,47,46,56,255,98,60,242,231,255,147,255,3,192,196,9,147,137,117,18,16,16,16,16,50,74,64,144,114,144,
  136,111,97,225,96,20,22,14,70,101,213,30,88,44,153,184,233,198,91,177,116,201,34,108,221,186,217,191,191,16,22,75,38,182,108,221,140,202,202,61,
  184,241,166,235,224,116,182,97,244,232,177,168,119,212,99,140,159,128,107,97,201,204,84,253,119,182,181,5,28,243,230,27,239,170,188,212,161,142,237,9,
  96,73,179,36,32,32,32,32,32,32,32,72,101,2,61,28,91,183,110,70,174,53,23,37,197,195,176,100,233,34,204,189,251,118,92,114,201,28,92,114,
  201,28,88,44,22,57,111,52,0,57,156,3,128,236,121,126,233,229,231,113,211,141,183,194,98,17,201,242,37,151,92,6,64,12,225,144,72,240,221,115,
  239,3,32,134,101,104,113,227,77,215,1,128,28,250,17,234,216,30,65,160,211,45,11,7,149,0,153,201,208,161,64,154,253,47,8,112,68,147,193,194,
  127,30,16,65,12,180,31,177,200,143,54,127,107,172,215,160,189,22,61,207,84,180,178,37,89,54,155,141,100,225,8,211,206,169,52,182,101,98,135,169,
  113,13,137,178,243,72,209,19,117,78,166,220,84,144,153,10,125,126,52,125,106,36,199,78,158,52,69,38,190,86,107,46,50,51,51,81,92,60,12,127,
  251,219,227,200,180,100,162,176,112,48,40,136,147,40,157,206,54,140,25,61,22,78,103,27,230,206,189,15,11,22,124,166,34,188,195,74,134,129,130,24,
  194,145,155,155,171,42,247,229,151,159,199,111,127,251,251,136,234,43,212,177,61,130,64,167,221,36,66,138,138,211,157,140,113,34,83,140,19,177,108,118,
  187,220,41,91,162,88,70,92,58,47,154,135,94,44,242,163,125,112,199,122,13,145,60,164,162,149,77,105,39,143,42,239,149,246,222,245,88,6,157,98,
  179,214,187,104,66,35,177,195,196,95,67,162,236,60,82,244,68,157,147,41,55,21,100,166,68,159,31,85,159,74,161,164,100,24,222,120,227,93,84,86,
  238,193,203,175,188,160,218,43,197,61,223,116,243,245,176,88,50,241,210,223,95,197,188,121,239,99,244,232,177,120,227,141,119,225,112,212,227,205,55,95,
  151,87,9,92,240,197,231,184,241,198,91,1,0,11,190,248,28,163,199,140,197,130,47,62,199,210,165,139,112,201,197,115,100,121,160,40,172,91,183,22,
  15,63,252,24,150,44,93,44,111,91,187,110,13,102,205,58,71,177,186,160,248,245,198,27,239,2,80,175,58,40,29,187,110,253,154,30,72,160,9,8,
  8,8,8,8,8,8,8,186,4,31,206,123,175,131,148,234,96,235,214,205,114,140,179,211,217,134,155,110,190,30,0,240,231,71,30,208,61,126,221,186,
  53,88,167,88,234,123,169,34,180,99,193,23,159,7,28,47,201,147,190,43,43,247,200,231,72,219,148,242,148,255,149,199,18,2,221,205,225,243,249,80,
  93,93,133,255,54,254,7,77,77,77,104,111,111,71,77,237,1,24,12,6,52,53,53,161,165,181,21,223,111,218,128,170,234,61,104,110,110,70,75,107,
  43,0,96,253,134,181,96,89,22,13,13,14,0,192,158,61,187,225,118,187,137,101,19,16,16,16,16,16,16,16,16,164,55,129,22,4,1,53,53,7,
  96,179,117,36,83,63,120,176,14,0,20,113,81,181,168,173,173,81,189,166,172,173,173,81,201,249,229,151,131,240,249,124,164,133,16,16,16,16,16,16,
  16,16,16,168,9,116,58,46,229,205,115,28,56,159,15,52,195,200,36,217,100,52,130,102,24,248,124,62,152,76,38,24,12,6,248,124,62,184,92,46,
  8,130,0,129,231,193,113,28,24,150,5,69,81,224,121,94,37,51,154,242,99,173,83,1,177,77,90,12,245,63,86,185,122,242,168,4,232,31,78,46,
  21,167,54,65,69,112,61,209,150,101,177,100,98,238,220,123,80,80,48,24,54,91,13,222,124,243,117,228,230,230,226,143,127,188,5,22,75,38,214,173,
  91,131,121,243,222,195,164,73,83,112,245,213,98,210,249,47,252,113,104,128,152,150,72,121,172,213,154,11,135,163,30,95,124,241,57,158,122,234,25,124,
  241,197,231,184,250,234,223,225,173,183,94,71,109,109,13,230,206,189,23,143,60,242,0,174,190,250,119,152,52,105,10,0,224,149,87,94,64,101,229,158,
  184,217,79,186,47,229,77,236,48,53,174,33,81,118,158,168,182,152,46,58,39,82,190,144,34,50,19,217,231,119,215,62,149,32,65,4,58,45,87,34,
  244,127,83,20,5,163,209,136,129,3,7,97,228,200,81,224,121,1,223,127,191,1,51,166,207,66,89,217,104,84,87,87,225,243,249,159,162,185,249,24,
  6,14,202,131,195,81,15,175,215,171,47,51,134,242,163,65,65,65,129,60,41,165,254,215,95,35,63,175,176,80,163,66,96,217,218,201,46,177,202,15,
  55,129,73,89,78,103,174,33,28,98,145,77,81,148,184,76,115,28,39,148,136,201,227,41,220,124,203,13,184,235,174,123,48,105,242,153,176,90,115,81,
  89,85,137,165,75,23,225,225,135,30,195,23,95,126,142,139,47,190,12,243,230,189,15,135,163,30,119,221,117,15,214,173,95,3,167,211,137,137,147,206,
  84,29,251,214,91,175,227,143,127,188,5,14,71,61,156,78,167,220,245,78,156,52,5,181,182,90,63,233,30,142,73,147,166,224,145,71,31,196,232,178,
  49,184,248,226,203,80,89,245,68,252,236,39,221,87,34,36,118,152,18,215,144,40,59,239,74,185,221,81,231,168,228,134,201,4,35,103,126,57,116,168,
  107,101,38,176,207,239,182,125,42,65,130,8,116,186,94,24,203,194,100,50,225,164,147,78,198,165,151,92,129,1,3,6,96,219,182,114,208,154,198,108,
  96,197,42,56,107,234,52,252,122,232,23,172,91,183,38,144,68,39,1,54,155,77,238,44,204,6,67,84,231,69,251,208,139,85,126,52,15,238,206,92,
  67,56,196,34,155,74,64,39,86,80,80,40,79,236,168,170,220,131,226,146,97,168,170,220,131,139,47,158,3,167,179,13,127,123,250,9,20,22,12,134,
  197,98,193,214,242,205,112,58,157,184,249,150,27,58,116,171,173,81,29,107,179,213,98,98,149,120,254,188,121,239,3,0,42,171,246,160,176,96,48,74,
  138,135,201,101,218,108,181,112,56,234,177,116,217,98,44,93,182,152,244,98,196,14,19,98,135,137,188,134,68,217,121,87,202,237,142,58,199,179,47,141,
  41,243,75,23,200,76,69,92,125,245,181,152,52,113,138,95,255,90,188,252,202,11,126,39,74,7,70,151,141,193,196,73,83,240,202,43,47,2,0,238,
  188,243,79,40,41,30,6,167,211,137,151,95,121,1,118,187,120,221,79,62,241,52,114,114,172,184,231,222,59,1,0,47,190,240,15,84,86,237,198,63,
  254,241,18,126,243,155,107,48,113,194,100,56,157,78,188,253,246,27,0,128,59,238,184,91,46,247,127,31,125,0,167,211,137,39,30,255,27,30,251,203,
  195,200,181,230,226,206,59,255,132,255,253,239,3,252,230,55,215,224,214,219,254,168,210,103,194,196,201,248,199,63,254,142,215,255,245,31,185,190,29,142,
  122,44,93,186,24,87,95,125,45,0,255,91,215,101,139,241,212,147,207,192,234,207,31,189,110,253,26,249,25,215,157,145,182,43,17,154,205,102,156,113,
  198,4,12,57,121,40,50,50,50,34,58,103,200,201,67,145,147,99,5,0,178,196,55,65,76,88,186,108,49,254,246,180,232,17,126,248,161,199,228,237,
  218,206,80,239,216,146,226,97,88,239,159,217,188,181,124,115,199,113,75,23,97,230,204,217,242,255,54,103,27,169,104,2,2,2,130,52,66,101,213,30,
  252,233,158,59,96,181,230,98,116,89,224,106,129,197,37,195,80,92,84,2,139,197,130,130,130,66,20,23,149,224,111,79,63,129,202,170,61,24,61,122,
  140,24,138,42,8,224,121,30,130,32,160,184,168,4,197,69,37,224,56,14,60,47,160,116,84,25,70,141,44,195,163,143,61,132,165,75,23,225,252,243,
  47,130,32,8,56,114,228,48,110,189,237,143,56,82,127,4,23,92,112,177,74,134,246,183,242,83,84,92,34,235,115,203,173,127,192,158,202,221,88,187,
  238,59,252,237,233,39,112,245,213,215,226,173,183,94,199,43,175,188,136,139,47,22,23,121,145,200,244,43,175,188,136,73,19,167,200,100,154,16,232,20,
  36,207,99,70,143,197,137,39,156,20,245,185,6,255,136,220,16,197,200,156,160,103,194,102,171,149,243,105,22,151,12,131,205,86,139,135,31,122,12,163,
  71,143,197,188,121,239,195,233,116,162,160,160,16,78,167,19,163,203,198,160,164,120,24,254,253,250,219,114,103,18,236,88,45,225,94,183,126,141,156,64,
  223,102,171,69,97,193,96,88,173,185,152,57,99,182,138,164,19,16,16,16,16,116,127,56,53,78,18,139,197,130,178,210,49,112,56,234,49,113,194,100,
  56,28,245,104,104,112,224,234,223,92,11,91,109,13,190,248,98,190,124,172,148,72,97,104,81,49,70,142,42,67,109,109,13,120,158,71,191,126,57,56,
  80,179,31,14,71,61,106,107,107,48,104,208,32,240,254,185,95,130,32,96,253,186,53,24,114,242,80,153,52,75,178,4,129,87,109,211,211,71,137,194,
  130,193,126,39,208,22,84,86,237,81,109,11,117,141,221,17,105,185,18,97,134,201,132,25,51,102,227,192,129,253,49,203,202,181,230,70,157,137,131,66,
  242,39,13,36,122,82,4,149,64,217,137,150,171,148,157,136,9,37,203,150,46,146,73,177,195,81,143,255,205,123,31,85,149,123,112,215,93,247,96,230,
  140,217,176,217,106,177,222,31,239,252,199,63,222,34,143,192,93,78,39,40,255,111,237,177,5,254,142,70,169,175,203,233,196,210,165,139,48,105,210,20,
  84,85,237,65,121,249,102,60,245,228,51,0,128,87,94,121,49,46,245,215,21,109,183,59,232,210,147,237,144,234,70,118,30,169,124,161,7,232,156,172,
  186,160,52,114,226,57,113,53,30,50,187,122,18,97,52,253,24,5,160,164,120,24,254,254,226,171,176,217,106,81,85,181,71,117,94,73,241,48,56,28,
  245,216,186,117,51,74,74,134,97,217,178,197,120,244,209,7,49,125,250,44,148,149,141,193,208,161,69,248,231,63,95,6,0,240,60,143,170,170,61,56,
  227,140,137,16,4,1,223,127,191,14,131,6,229,131,231,121,240,28,7,248,61,200,94,175,15,60,207,139,28,199,191,77,250,173,220,38,17,108,65,16,
  100,157,138,139,74,80,95,127,4,229,229,91,100,125,148,215,170,60,86,137,139,47,158,3,0,88,191,126,141,252,28,236,214,4,58,221,38,17,50,44,
  139,146,97,167,160,111,223,126,1,4,250,231,159,127,66,107,91,27,54,109,218,128,234,234,74,184,220,110,184,219,245,115,61,31,63,112,16,26,26,27,
  224,107,107,139,170,252,88,39,13,40,39,47,29,249,229,151,200,207,139,97,226,79,172,242,163,137,189,236,204,53,132,67,44,178,19,49,161,196,233,114,
  225,233,103,158,84,109,115,52,56,112,203,173,127,80,109,91,191,97,45,214,111,88,27,80,78,85,117,101,192,177,242,54,138,234,56,143,162,176,108,249,
  18,44,91,190,4,160,40,204,251,223,7,152,247,191,15,98,214,59,222,109,55,17,118,220,85,186,16,59,76,206,53,196,221,206,21,203,55,135,171,155,
  68,78,34,76,9,157,147,84,23,193,86,100,76,21,153,93,62,137,48,170,126,140,66,85,213,30,108,45,223,130,153,51,102,195,233,114,169,206,43,43,
  27,131,252,252,2,228,231,139,147,46,75,203,198,224,55,87,93,131,103,159,251,43,4,0,211,206,158,33,15,58,120,158,67,173,173,22,83,166,156,5,
  187,221,134,150,214,54,112,28,135,122,71,61,166,76,153,138,190,253,114,112,252,192,65,168,173,61,0,1,2,56,206,7,1,192,105,167,141,67,101,229,
  110,56,93,78,248,124,62,12,28,148,7,139,197,2,159,143,131,0,30,60,207,201,101,148,150,142,70,94,94,62,242,243,197,182,102,201,204,244,223,51,
  10,54,187,56,225,189,108,244,88,249,77,170,180,237,139,47,231,35,215,154,43,82,237,52,224,158,105,55,137,144,166,105,100,247,201,86,109,115,187,221,
  248,232,227,15,240,211,79,251,224,241,120,96,179,219,96,179,219,64,81,148,108,168,77,71,155,144,149,149,37,159,147,157,157,157,212,56,104,229,228,165,
  12,150,141,234,188,104,31,122,177,202,143,230,193,221,153,107,8,135,88,100,83,100,38,52,1,177,195,148,185,134,68,216,121,164,164,49,145,147,8,83,
  69,231,100,212,69,48,178,155,42,50,187,99,159,191,126,253,90,92,124,209,28,76,156,56,25,235,215,139,78,23,139,197,130,178,178,49,120,250,153,39,
  97,179,213,226,129,251,255,12,171,53,23,181,181,7,240,151,199,158,128,32,0,31,125,252,161,92,111,28,199,163,173,173,13,229,21,91,96,183,219,1,
  63,73,222,190,189,2,39,158,120,18,238,249,211,3,0,4,188,253,206,91,0,128,94,189,122,227,175,79,61,139,95,126,57,136,255,125,244,33,218,218,
  218,176,113,227,58,252,230,170,107,64,211,20,190,250,250,75,240,188,0,159,143,195,223,95,124,21,12,67,3,160,240,220,243,127,131,221,110,195,253,247,
  61,140,9,19,38,3,16,189,212,78,167,19,255,251,232,3,252,225,134,155,64,81,20,190,248,114,126,64,72,226,67,15,62,138,165,203,22,195,225,168,
  39,4,58,149,192,48,12,10,7,15,86,109,219,190,163,2,63,253,180,15,46,151,11,52,77,251,131,234,121,208,52,13,154,166,65,81,20,246,237,171,
  70,126,94,190,124,78,174,181,63,140,81,204,252,37,32,32,32,32,32,32,32,136,22,243,254,215,145,145,66,202,158,33,193,233,116,226,214,219,254,40,
  15,42,158,125,238,175,0,128,229,203,151,168,6,26,210,239,167,254,250,127,0,128,234,234,74,121,223,170,85,43,0,0,11,22,124,134,5,11,62,83,
  157,247,224,67,247,6,12,56,62,159,255,41,230,47,248,76,53,176,217,180,105,163,238,224,250,185,231,255,38,235,35,237,91,191,126,173,60,0,144,240,
  232,99,15,201,191,181,111,94,9,129,78,101,175,82,109,77,216,212,116,210,106,133,4,4,4,4,4,4,4,4,169,12,137,4,107,73,180,118,187,30,
  201,14,37,79,235,185,87,190,169,87,254,87,190,45,80,158,163,253,159,206,96,73,51,36,32,32,32,32,32,32,32,72,109,178,28,13,121,14,70,164,
  67,145,104,37,57,214,126,7,59,174,39,35,45,151,242,214,94,83,97,225,96,84,87,87,130,243,249,84,51,99,149,179,70,243,6,229,169,206,241,122,
  189,64,144,153,164,161,202,143,53,123,128,114,242,210,225,40,38,162,20,250,87,114,82,180,108,85,249,130,104,49,170,67,98,150,175,145,141,16,229,116,
  234,26,194,32,22,217,210,132,146,174,92,214,53,213,109,135,100,225,32,118,152,180,107,72,128,157,71,26,247,123,36,138,9,105,221,85,231,100,212,69,
  176,120,229,84,145,217,213,125,126,178,250,49,217,46,165,143,226,191,32,8,242,254,176,4,154,162,196,243,52,223,148,118,18,166,116,190,114,191,246,184,
  30,66,174,211,46,11,7,199,113,168,169,173,197,136,17,163,228,205,167,158,90,138,253,7,126,198,190,125,123,225,245,122,193,42,38,7,74,35,181,147,
  135,20,169,68,29,59,214,4,175,207,23,253,82,222,49,102,15,176,217,237,114,195,54,69,49,121,209,102,183,235,142,34,131,141,92,59,43,63,220,138,
  81,241,40,35,28,98,145,77,233,221,155,100,47,235,154,226,182,67,178,112,16,59,76,230,53,196,219,206,35,37,141,81,77,244,235,166,58,39,163,46,
  130,145,221,84,145,217,229,125,126,130,250,49,193,79,110,101,114,236,39,209,242,71,179,224,137,146,231,132,36,208,26,50,76,105,234,71,16,4,208,20,
  165,78,49,232,255,79,41,190,123,210,243,52,237,22,82,225,121,30,135,15,31,82,27,95,70,6,110,186,241,54,156,113,198,4,100,100,100,32,63,63,
  31,167,158,58,10,67,135,22,201,139,90,244,205,238,171,58,231,232,209,163,224,56,14,4,4,4,4,4,4,4,4,93,1,61,194,171,23,186,17,236,
  195,243,188,238,135,227,56,249,19,236,24,229,39,112,129,21,33,162,176,144,116,70,218,197,64,115,28,135,3,7,126,70,83,83,99,192,190,147,78,60,
  25,187,118,238,192,184,211,39,160,172,108,52,170,171,171,176,240,200,2,180,233,228,122,182,217,107,225,118,187,137,245,18,16,16,16,16,16,16,164,20,
  153,14,69,154,189,94,47,218,219,219,193,243,188,106,18,96,36,68,87,242,34,83,20,5,138,162,64,211,52,76,38,147,156,214,87,202,96,166,157,88,
  40,201,15,54,185,144,16,232,110,2,183,219,141,229,43,150,226,228,147,134,196,44,131,120,160,9,8,8,8,8,8,8,82,141,68,235,77,42,84,146,
  231,99,199,142,225,162,139,206,199,113,199,13,192,161,67,135,100,50,171,36,197,26,218,12,64,0,207,43,137,182,0,183,219,141,95,127,253,21,59,118,
  236,65,159,62,125,100,18,173,71,142,123,226,164,194,180,37,208,91,182,252,8,206,231,131,213,154,27,213,185,82,186,187,112,105,239,8,8,8,8,8,
  8,8,8,146,69,156,131,145,102,233,63,207,243,104,110,110,198,133,23,158,135,185,115,239,192,207,63,255,132,163,71,27,65,81,0,203,50,160,105,70,
  94,251,130,166,41,21,121,150,230,32,42,67,60,218,219,219,241,235,175,117,168,171,171,131,195,209,132,156,156,28,93,93,128,158,185,88,25,155,174,151,
  236,118,185,176,181,124,11,10,10,10,113,234,169,165,232,211,167,79,216,115,246,236,217,133,250,250,35,0,196,70,20,109,221,4,203,2,162,197,165,115,
  230,168,102,179,203,6,224,255,255,213,130,5,17,151,121,217,101,151,5,213,67,41,83,137,206,200,15,57,251,63,78,101,132,67,44,178,41,29,35,159,
  63,127,126,68,215,214,83,144,74,89,56,146,125,63,230,204,153,67,236,48,201,215,16,79,59,191,252,178,203,34,155,180,36,8,248,234,139,47,210,90,
  231,132,202,85,51,38,89,142,170,14,186,88,166,94,159,255,249,231,159,235,218,77,162,251,152,120,247,169,130,38,51,152,160,200,182,1,65,128,207,235,
  69,175,94,89,184,254,250,107,240,227,143,223,99,255,254,189,112,187,93,48,24,12,96,89,22,44,203,130,97,24,48,12,227,39,208,148,28,226,33,18,
  112,1,60,239,131,207,199,193,235,229,224,245,122,209,218,218,138,145,35,139,176,122,245,102,112,28,39,38,97,80,100,226,8,200,106,166,240,68,167,251,
  51,149,77,215,153,146,28,207,195,237,118,163,182,182,6,111,191,253,111,156,118,218,56,112,28,15,94,99,152,94,159,15,0,176,124,197,82,28,61,218,
  164,14,219,136,182,110,164,227,195,156,55,127,254,124,221,228,230,210,183,145,142,124,110,167,212,49,4,27,9,234,141,20,59,35,63,210,217,255,209,148,
  241,217,103,159,69,60,122,21,4,33,170,153,238,114,199,169,136,235,10,122,45,61,125,185,239,84,202,194,145,228,251,241,185,194,38,123,170,29,38,250,
  26,18,105,231,159,105,236,60,84,221,68,37,183,59,234,156,96,185,225,208,213,50,117,251,252,96,125,74,162,251,152,4,101,225,144,50,110,64,25,223,
  236,255,239,241,180,99,255,254,159,80,87,87,139,166,166,70,24,12,44,24,134,2,64,131,162,4,255,71,138,113,150,178,209,81,224,121,128,162,4,63,
  55,22,32,8,156,255,35,192,229,242,64,16,120,185,28,101,198,13,65,175,207,232,130,62,188,107,8,116,154,163,189,189,29,191,252,250,11,150,46,91,
  12,138,162,224,241,120,240,205,183,11,177,108,249,98,120,189,94,121,141,246,35,71,14,203,1,247,61,61,57,56,1,1,1,1,1,1,65,215,34,220,
  228,63,109,154,58,233,227,118,187,192,243,28,76,38,35,50,50,76,48,153,50,96,52,26,97,52,26,100,47,52,77,51,50,145,22,29,216,98,8,136,
  215,235,131,207,231,131,193,224,129,199,227,129,207,199,129,162,104,8,2,224,243,249,228,9,132,193,86,46,236,73,232,17,43,17,26,12,6,0,128,203,
  229,2,69,81,104,105,105,145,3,222,165,160,120,163,209,8,143,199,211,99,211,177,16,16,16,16,16,16,16,116,63,114,173,157,88,200,48,52,12,6,
  131,159,52,27,97,50,153,96,50,137,191,13,6,131,130,64,171,67,56,56,142,135,193,224,131,199,227,3,69,137,177,210,70,163,7,6,131,24,172,224,
  245,122,193,48,76,216,149,10,133,30,179,144,74,154,19,103,134,97,80,82,50,12,133,5,131,177,100,233,34,120,60,30,121,191,52,27,149,227,56,140,
  26,85,10,155,205,38,123,162,9,8,8,8,8,8,8,8,82,141,56,235,45,144,210,65,170,69,199,160,20,243,172,36,209,70,163,201,31,15,29,72,
  160,1,192,231,227,224,243,121,65,81,29,161,89,44,235,150,29,141,128,248,86,159,209,132,212,4,35,204,233,78,164,211,150,64,103,102,102,162,79,159,
  108,156,124,210,16,204,154,117,46,140,38,35,86,173,94,161,34,208,44,203,194,96,48,128,227,56,12,57,185,8,195,135,143,192,252,249,159,162,185,185,
  153,88,41,1,1,1,1,1,1,65,202,17,231,14,178,172,191,143,162,196,137,130,6,3,171,242,68,75,30,104,150,101,252,36,152,6,77,67,14,225,
  160,105,206,159,226,142,242,79,40,228,96,52,138,68,28,160,2,242,60,247,180,188,207,1,4,58,221,46,151,2,96,50,153,144,55,40,15,167,159,126,
  134,76,134,205,25,102,244,237,219,15,173,173,173,242,204,80,3,43,53,12,17,70,131,17,125,250,100,163,185,185,25,25,38,83,76,11,169,196,58,235,
  182,160,160,64,54,128,95,235,234,34,62,175,176,160,32,80,135,48,19,127,58,35,63,210,201,75,209,150,17,205,68,157,67,7,15,70,173,63,69,81,
  176,217,108,170,123,147,236,25,217,169,110,55,169,84,7,93,149,17,164,167,219,97,162,175,33,25,118,30,74,38,128,168,229,118,71,157,19,41,55,28,
  82,69,102,87,247,249,137,238,83,197,228,115,154,190,210,127,95,105,154,6,77,211,96,24,22,12,195,250,189,209,6,24,12,172,28,7,221,225,129,22,
  189,205,226,2,41,156,236,145,230,56,30,28,199,251,143,165,213,215,226,207,6,162,204,186,65,105,116,73,181,103,74,66,8,116,218,205,146,164,40,152,
  50,50,112,222,121,23,225,232,209,38,213,174,1,3,142,67,93,157,29,130,255,154,25,127,163,210,67,118,223,190,168,175,175,143,126,49,149,24,103,221,
  218,236,118,185,83,51,68,113,190,205,110,143,250,161,215,25,249,145,62,184,163,45,35,154,135,84,52,51,255,37,253,229,153,193,202,114,210,48,11,199,
  221,119,223,29,116,223,75,47,189,20,186,221,166,82,29,116,81,70,144,158,110,135,137,190,134,164,216,121,24,210,24,173,220,238,168,115,34,229,134,67,
  170,200,236,242,62,63,209,125,170,152,62,163,35,35,135,162,44,138,2,104,154,241,167,172,163,253,31,70,38,212,226,167,131,64,139,222,227,14,34,45,
  146,110,14,44,235,3,69,49,170,140,27,148,178,127,214,124,148,217,57,122,66,38,142,180,11,225,160,105,26,131,6,229,225,196,19,79,66,121,249,22,
  213,190,172,172,44,249,230,50,12,131,94,189,122,163,189,189,93,223,19,149,95,136,230,230,102,57,75,7,1,65,119,64,48,146,28,138,88,19,16,16,
  16,16,164,7,68,46,75,203,228,88,242,70,139,30,105,245,255,142,24,104,10,20,197,67,16,104,255,4,67,78,115,92,228,3,152,158,20,202,145,118,
  4,154,101,89,20,22,158,16,176,125,107,249,102,236,216,177,93,158,32,200,113,28,26,27,27,224,243,231,129,246,250,60,170,227,251,244,201,86,133,119,
  16,16,16,16,16,16,16,16,116,23,34,77,211,82,90,94,90,181,140,183,68,176,165,237,18,104,90,244,49,139,251,105,5,17,143,166,220,158,19,12,
  73,167,95,163,161,48,160,255,0,213,182,189,123,171,48,127,254,167,1,33,29,237,237,237,50,161,222,189,123,183,106,159,213,154,11,163,209,72,172,144,
  128,128,128,128,128,128,160,187,178,34,57,223,179,72,249,36,66,44,145,235,142,255,0,37,199,59,3,130,226,60,42,34,114,220,211,38,17,166,37,129,
  206,200,200,80,109,219,185,107,59,220,110,119,200,120,230,218,218,3,196,206,8,8,8,8,8,8,8,186,21,231,9,62,39,162,99,213,237,142,21,6,
  5,197,74,233,130,230,120,41,179,134,160,56,95,8,73,146,201,66,42,4,93,14,213,236,255,8,39,86,72,231,133,26,5,234,206,156,239,132,252,136,
  103,255,71,89,70,84,51,221,163,152,249,95,160,153,145,77,64,64,236,176,235,174,33,25,118,30,74,38,128,168,229,118,71,157,19,41,55,28,82,69,
  102,186,247,249,218,28,206,29,255,165,85,5,197,9,129,60,207,131,227,4,127,90,58,49,53,29,199,117,132,116,136,203,119,83,242,177,226,241,226,135,
  231,121,8,2,15,173,233,235,181,173,158,72,164,211,50,141,93,128,65,229,23,98,223,222,106,248,188,94,213,49,202,99,251,231,246,87,157,227,245,121,
  229,84,45,209,150,29,205,57,210,172,86,155,205,38,15,21,217,104,102,182,107,210,244,248,91,178,122,8,170,41,207,208,25,249,254,116,53,90,153,202,
  114,162,45,195,110,179,233,202,69,144,114,12,81,204,200,182,75,29,168,66,126,176,251,148,206,230,31,73,90,56,42,69,234,160,43,210,31,17,59,76,
  252,53,36,203,206,181,109,71,208,232,31,181,220,238,168,115,34,229,134,65,202,200,76,129,62,63,174,125,170,142,45,202,169,227,52,109,84,90,150,187,
  131,16,115,224,56,31,56,78,74,73,199,1,16,228,156,207,18,233,22,143,227,224,243,73,36,154,147,73,180,146,36,83,0,104,137,128,43,245,208,212,
  117,186,147,234,180,75,99,199,241,60,108,182,90,140,24,49,82,222,54,118,236,56,184,219,221,216,184,113,61,218,218,218,192,250,87,40,228,121,30,62,
  159,15,60,207,163,164,228,20,149,156,99,199,142,194,235,243,33,202,232,249,192,134,30,222,109,33,167,163,137,249,94,132,58,79,187,47,150,165,202,195,
  165,254,209,234,30,109,25,145,214,91,44,117,164,151,194,40,88,121,233,108,236,145,164,133,75,112,234,184,19,79,60,9,7,15,214,5,205,124,211,41,
  59,234,52,131,38,118,152,240,107,72,150,157,7,35,254,177,220,215,238,172,115,162,229,198,155,20,38,66,102,87,247,249,113,236,83,85,4,89,34,178,
  52,45,18,85,158,23,127,43,22,53,225,121,14,62,31,7,175,215,7,163,209,7,159,207,7,143,199,3,65,0,88,86,0,199,117,100,226,144,66,55,
  196,149,8,125,254,143,23,62,159,15,94,47,167,10,127,165,40,10,20,77,139,3,20,255,245,73,255,165,15,213,3,82,216,137,4,58,205,192,243,60,
  142,53,31,11,216,126,222,185,23,34,59,187,31,190,253,246,43,156,124,242,16,12,26,56,8,245,245,71,80,189,183,26,205,205,199,144,157,157,173,33,
  208,199,228,12,29,4,4,4,157,135,53,39,23,167,141,61,3,95,124,249,89,120,18,77,64,64,64,64,16,132,151,119,16,101,153,212,42,136,43,207,
  11,126,242,236,129,193,192,194,227,233,88,158,91,16,164,69,83,104,77,216,135,232,117,246,122,189,240,122,125,240,122,189,240,120,188,178,147,81,89,142,
  82,135,158,28,3,157,118,147,8,57,142,67,101,229,110,52,53,53,5,236,203,202,204,68,134,201,132,226,162,18,76,153,50,21,35,70,140,132,33,72,
  170,58,155,173,134,60,228,9,8,226,77,162,173,185,184,248,162,203,96,50,153,72,101,16,16,16,16,68,64,148,131,145,84,37,113,86,146,91,142,243,
  201,158,100,145,8,123,224,241,120,208,222,238,65,123,123,187,255,163,252,221,241,145,142,245,120,60,240,122,165,111,95,128,62,161,38,47,18,2,221,141,
  209,222,222,142,149,171,150,117,74,198,177,99,199,162,95,133,144,128,128,128,144,104,2,2,2,130,36,147,107,137,208,242,60,15,135,227,48,154,155,143,
  169,72,178,219,237,134,203,229,134,203,229,82,124,148,219,58,246,185,221,110,255,241,46,52,55,183,224,135,31,42,228,5,82,180,94,104,61,2,223,83,
  144,150,89,56,92,46,23,182,110,221,140,163,71,143,162,176,160,48,170,115,165,149,7,147,237,125,86,206,70,254,37,138,153,195,209,204,18,7,196,87,
  62,137,154,57,223,153,50,66,141,178,149,114,129,216,51,11,216,163,56,143,32,57,36,58,213,194,57,122,186,29,38,250,26,146,97,231,218,50,226,145,
  153,164,59,234,156,104,185,193,144,74,50,211,173,207,215,222,75,45,137,102,24,6,62,159,23,239,188,243,49,198,142,29,1,159,143,131,193,192,194,96,
  48,130,101,89,24,252,243,191,164,21,6,59,114,65,11,170,137,132,146,247,218,229,114,97,245,234,239,209,222,238,129,197,98,233,209,100,185,199,16,104,
  0,104,109,109,197,222,189,85,104,106,108,64,81,81,9,250,244,233,19,242,248,54,167,19,219,119,84,160,169,169,177,75,244,85,166,219,97,163,60,47,
  218,135,94,212,51,243,195,228,125,212,118,202,177,148,17,205,67,42,22,217,145,200,39,8,196,197,23,93,22,55,89,189,122,247,78,121,18,221,211,237,
  48,209,215,144,12,59,15,71,26,123,138,206,137,150,27,12,169,36,51,157,251,124,229,181,41,9,116,78,142,21,14,135,3,171,86,109,2,207,11,154,
  12,26,148,206,60,74,42,160,109,72,57,163,197,180,119,60,204,102,179,42,102,58,148,231,185,39,61,103,211,58,15,180,219,237,198,161,195,135,240,214,
  127,94,71,65,65,33,50,76,25,240,120,189,112,52,212,163,166,182,6,135,143,28,134,203,237,2,0,44,95,190,24,94,175,87,14,150,39,32,32,0,
  6,13,202,75,168,124,171,53,23,19,39,78,193,202,149,203,72,101,19,16,16,16,132,32,204,202,73,131,74,194,171,36,173,12,195,32,55,55,87,158,
  252,215,222,222,46,103,217,208,27,72,41,101,74,223,210,10,132,44,171,92,254,155,10,25,3,221,19,23,88,97,211,241,18,149,215,228,245,122,113,236,
  232,81,84,182,182,130,97,24,120,125,62,108,220,184,30,155,127,220,4,31,199,193,229,18,9,116,91,91,91,72,57,145,150,155,236,92,186,137,46,79,
  79,62,149,160,251,69,37,184,45,116,54,39,232,149,87,94,141,241,227,39,1,0,170,171,43,241,218,107,175,96,252,248,73,184,242,202,171,1,0,95,
  125,245,5,42,42,182,226,47,127,121,10,175,189,246,10,0,224,182,219,238,194,227,143,63,130,210,210,209,152,48,97,18,30,127,252,17,249,156,21,43,
  150,226,171,175,190,192,5,23,92,140,105,211,102,226,181,215,94,129,211,217,134,219,110,155,11,139,197,34,151,113,219,109,119,161,168,168,4,0,228,115,
  254,242,151,167,144,147,99,5,0,52,52,56,240,248,227,143,68,85,23,145,182,221,131,7,235,226,118,47,122,247,238,141,94,189,212,94,104,135,163,30,
  27,214,175,9,184,79,169,146,147,186,167,217,97,162,175,33,25,118,30,207,50,186,179,206,137,150,219,29,100,118,101,30,232,120,245,99,122,100,89,10,
  223,16,0,221,15,69,211,160,13,6,49,28,195,235,149,243,148,43,115,197,11,122,117,32,229,117,150,210,209,233,144,229,204,204,76,145,80,67,157,235,
  89,155,247,185,39,248,161,217,180,204,211,167,185,38,134,101,97,244,79,88,242,113,28,88,233,191,199,35,55,70,134,97,2,39,13,198,146,215,51,30,
  121,31,163,201,175,154,224,220,189,186,242,227,93,94,34,115,70,198,53,39,40,133,141,27,215,227,171,175,191,192,99,143,62,137,210,210,209,184,242,202,
  171,241,238,187,111,193,233,114,226,182,91,239,66,245,222,42,185,247,168,174,174,66,117,117,21,166,79,159,133,162,162,98,44,95,177,212,95,159,226,33,
  69,69,37,0,245,165,76,142,65,1,87,94,249,91,108,220,184,30,43,86,46,149,203,0,40,124,252,201,60,216,237,54,220,119,239,67,114,25,95,125,
  253,5,86,172,88,22,121,221,133,203,241,171,115,175,191,92,248,121,220,110,197,216,177,227,112,218,216,113,42,242,252,229,194,207,209,238,241,4,222,167,
  68,183,107,98,135,93,115,13,201,176,243,120,150,209,157,117,78,180,220,238,32,179,43,243,64,39,160,31,83,145,103,165,231,88,90,179,91,243,97,40,
  10,25,102,179,236,133,166,228,174,77,189,72,156,158,247,88,235,109,86,145,103,157,201,132,65,235,62,173,9,116,26,131,162,40,24,12,6,228,229,229,
  97,196,136,81,224,124,62,108,216,184,30,51,166,207,66,89,217,104,84,87,87,225,243,249,159,162,185,249,24,6,14,28,8,135,195,1,183,219,173,187,
  100,109,162,145,159,159,47,255,142,118,242,82,180,232,172,252,112,177,151,137,190,134,104,100,43,235,181,174,174,46,33,247,174,98,91,185,162,60,245,181,
  172,88,185,20,183,221,122,23,26,26,28,216,184,113,189,188,221,110,183,33,63,191,0,165,163,202,52,250,22,224,171,175,191,128,211,233,132,221,110,67,
  142,213,170,58,167,186,186,74,46,227,130,243,47,198,5,231,95,140,143,63,153,167,146,221,29,32,147,231,20,75,21,73,236,48,185,215,208,21,178,137,
  206,137,149,219,213,50,147,209,231,167,18,199,81,78,38,212,110,23,4,65,158,56,24,44,132,67,219,159,232,17,103,109,24,71,184,124,208,61,37,14,
  58,109,9,180,52,227,116,240,224,19,113,233,37,151,161,127,255,1,216,182,173,28,180,230,198,74,121,160,39,79,154,138,195,135,15,97,237,186,239,224,
  241,120,146,174,175,114,182,48,19,197,121,177,76,252,233,236,228,168,72,38,47,69,91,70,36,70,39,149,195,198,88,175,241,48,234,241,227,39,98,252,
  248,137,168,174,174,130,211,229,12,123,124,117,117,149,76,124,149,144,8,242,5,23,92,140,138,138,242,0,226,29,9,84,30,104,66,158,137,29,198,193,
  14,19,125,13,137,178,243,104,38,206,245,20,157,19,45,55,30,164,34,17,50,227,221,231,167,18,81,214,146,102,233,123,236,216,211,113,209,133,151,128,
  227,121,44,89,242,45,156,78,39,206,61,231,124,252,223,227,143,224,220,115,206,71,239,62,217,248,232,163,15,48,115,230,57,56,117,196,72,184,220,46,
  124,245,213,23,232,211,39,27,51,103,204,6,40,160,221,221,142,79,63,251,8,165,165,163,65,83,52,22,47,249,6,191,189,250,119,168,181,29,0,69,
  209,152,53,243,28,48,12,139,133,95,45,192,230,205,63,4,37,207,61,9,116,186,94,152,217,108,198,233,167,157,129,147,79,58,25,38,83,70,68,231,
  156,116,210,201,232,215,47,71,124,120,50,12,8,8,180,216,184,113,61,238,188,235,22,188,246,175,87,96,183,139,157,127,233,168,50,20,21,21,251,59,
  239,192,7,130,211,233,132,211,213,166,75,174,115,114,172,176,219,107,21,157,191,13,69,67,75,96,177,88,144,159,95,128,6,135,67,225,89,41,64,81,
  81,177,110,25,132,60,19,16,16,16,244,12,40,137,171,197,98,193,185,231,92,128,133,95,125,129,247,222,123,27,231,204,62,31,153,153,153,96,89,3,
  104,154,6,203,26,96,96,13,24,62,236,20,140,24,49,18,255,121,251,13,148,151,111,193,180,179,103,130,101,89,152,76,38,188,248,226,179,104,108,108,
  196,216,49,167,193,192,178,96,13,172,120,174,193,0,154,102,112,230,148,179,177,98,229,50,44,95,177,24,227,198,141,239,209,164,57,214,129,93,183,34,
  207,101,165,163,49,120,240,9,104,104,112,68,117,174,193,96,144,191,201,66,42,4,161,224,116,58,241,241,39,243,112,221,117,127,4,0,57,244,2,0,
  110,187,245,46,0,192,107,255,122,37,232,249,21,219,182,98,252,248,137,168,216,86,142,235,252,219,62,254,100,30,110,187,245,46,76,155,54,3,213,213,
  85,168,216,86,46,78,58,188,226,106,185,12,201,155,45,133,112,52,52,56,240,248,19,143,166,124,125,53,56,234,177,99,71,5,33,207,4,4,4,4,
  49,144,102,189,201,132,121,121,249,232,213,43,11,187,118,237,128,211,233,196,159,31,185,31,99,199,158,14,131,193,128,191,62,245,44,120,94,192,79,63,
  237,131,53,55,23,13,142,122,52,55,31,195,145,35,135,49,107,230,57,216,185,107,59,76,38,147,28,238,234,245,250,252,249,158,253,33,176,44,11,134,
  97,80,83,123,0,103,77,157,134,93,187,118,226,237,183,223,8,233,121,38,105,236,186,57,76,38,19,206,58,107,6,14,28,248,57,102,25,86,171,21,
  135,14,29,130,207,231,35,150,27,198,56,180,134,157,174,248,248,147,121,1,219,54,110,92,31,16,127,124,231,93,183,168,254,107,195,55,148,231,60,248,
  208,61,1,231,72,219,36,232,145,240,238,64,152,181,216,223,9,123,36,32,32,32,32,8,124,246,138,31,26,46,151,75,126,62,211,52,13,134,97,240,
  231,71,238,199,197,23,93,138,236,236,126,96,24,6,44,203,130,162,40,48,12,35,199,70,75,68,251,232,209,38,172,90,189,28,167,157,54,14,180,95,
  46,195,136,158,232,143,62,250,0,167,159,126,6,70,141,44,195,195,15,61,134,167,159,121,82,206,96,214,83,201,115,90,18,104,150,101,81,82,60,12,
  125,251,246,197,129,3,129,15,240,54,103,27,126,248,241,123,252,244,243,94,184,156,46,184,131,120,195,142,59,110,32,26,27,27,147,70,160,11,10,10,
  100,18,154,202,19,127,130,25,136,150,64,167,218,36,66,138,162,200,74,132,4,196,14,83,232,26,186,74,54,209,57,177,114,83,101,18,97,58,246,249,
  122,206,170,131,7,235,224,118,187,49,106,100,41,156,46,39,110,252,227,173,248,230,219,133,50,89,166,105,6,12,203,224,216,209,163,40,44,28,140,156,
  28,43,242,243,11,112,228,200,17,208,52,13,131,193,136,231,95,120,26,247,221,251,16,218,219,219,225,105,247,96,200,208,161,160,40,10,185,185,185,216,
  127,224,39,188,248,194,43,248,224,131,255,98,209,226,175,113,207,159,238,71,78,142,21,117,117,246,30,77,158,211,146,64,211,52,141,222,125,178,85,219,
  218,219,221,248,207,219,31,97,223,190,189,104,111,111,71,93,157,29,191,252,114,80,94,182,18,0,142,30,61,138,172,172,44,249,156,62,125,250,128,101,
  147,87,61,118,187,93,54,140,84,154,188,164,237,128,34,125,112,39,234,26,98,213,159,196,107,17,16,59,236,218,107,72,214,132,60,61,249,202,122,73,
  212,196,202,88,235,35,17,58,39,90,110,188,72,69,34,39,17,166,107,159,175,93,57,208,233,116,226,219,69,95,225,119,191,187,1,128,128,111,190,253,
  10,237,237,237,96,24,70,246,54,179,12,131,157,187,118,160,168,168,4,15,61,40,190,189,124,235,63,175,163,95,191,28,176,44,139,166,166,70,236,169,
  220,141,179,207,154,142,77,63,108,196,132,9,147,240,226,11,175,160,177,177,17,59,119,110,7,77,211,248,253,239,111,0,64,97,211,166,141,132,60,167,
  51,129,46,212,140,106,119,238,220,142,159,127,254,73,126,229,160,231,85,254,233,231,189,200,203,235,88,117,205,154,147,11,163,209,216,109,140,41,216,54,
  237,3,85,206,25,73,64,64,144,20,59,20,136,189,69,245,160,237,76,72,88,48,249,169,28,102,150,40,157,187,99,93,16,196,70,164,191,255,126,3,
  190,255,126,131,106,255,166,77,27,65,81,20,62,159,255,137,124,252,252,5,159,98,254,130,79,21,71,237,195,143,63,110,2,69,81,152,55,239,61,121,
  235,99,127,121,72,213,126,86,175,94,137,213,171,87,198,100,207,132,64,119,115,216,236,181,240,122,189,33,143,249,229,151,131,93,102,4,241,236,204,130,
  229,131,76,244,195,47,17,15,189,160,199,38,224,161,66,64,30,70,137,34,21,122,178,83,113,32,155,238,131,241,88,238,113,162,251,166,112,242,99,109,
  151,137,146,155,232,1,98,34,101,246,132,65,105,168,252,206,164,62,9,129,38,32,32,32,232,86,15,181,238,50,144,77,244,53,116,53,25,37,72,92,
  219,238,44,233,77,132,76,114,111,208,169,58,36,246,20,1,129,78,183,74,18,175,71,125,77,5,5,131,177,111,223,94,120,189,222,128,85,123,164,198,
  53,112,224,32,213,57,94,159,55,182,78,191,19,13,79,126,64,69,209,224,35,89,253,39,224,193,23,131,252,168,95,189,198,120,13,17,117,160,81,202,
  14,86,71,225,22,166,232,9,157,107,168,186,234,106,61,187,74,151,68,217,97,103,219,113,76,237,180,19,118,24,81,57,157,180,243,120,201,166,105,58,
  114,29,120,62,37,250,166,68,233,156,200,186,136,104,240,214,137,254,57,17,50,187,178,207,79,133,62,149,16,225,4,18,232,116,187,32,142,227,96,179,
  213,226,148,83,78,149,183,141,56,229,84,212,212,236,199,190,125,123,225,114,185,192,48,140,188,180,165,207,231,3,207,243,56,249,164,161,42,57,199,142,
  29,13,27,246,17,43,46,185,228,146,144,203,106,126,241,217,103,17,203,186,244,210,75,163,46,63,209,242,19,93,70,103,100,43,59,147,47,190,248,130,
  244,0,4,186,54,73,236,176,123,219,57,209,57,254,114,231,204,153,211,45,100,234,245,249,11,22,44,32,157,28,1,33,208,225,192,243,60,142,212,31,
  82,109,51,153,50,112,195,245,55,97,193,23,159,225,135,31,190,199,113,199,29,135,156,156,28,56,157,46,212,214,214,162,173,173,21,217,217,217,26,2,
  125,44,174,41,236,148,222,89,201,152,131,145,232,104,198,139,243,231,207,143,122,230,60,157,64,249,177,150,17,201,72,89,170,163,104,100,43,59,206,158,
  236,125,38,8,180,67,229,127,61,155,76,148,29,198,210,142,35,181,145,100,149,209,25,249,193,108,78,59,128,233,140,236,80,72,149,190,73,79,231,96,
  97,50,169,42,55,94,245,27,79,153,164,207,39,32,4,58,70,112,28,135,3,7,14,160,233,104,83,192,190,19,79,56,9,59,119,108,199,105,99,207,
  64,89,217,104,84,87,87,225,240,161,67,104,107,107,13,56,214,110,175,77,233,21,211,4,65,128,160,71,190,35,120,77,31,45,225,8,183,77,16,132,
  128,206,57,218,168,171,68,207,206,79,149,123,22,143,123,66,144,154,196,60,86,18,223,29,203,72,84,166,140,80,118,210,153,50,18,29,187,29,207,250,
  8,55,41,172,171,229,18,16,16,68,63,176,235,54,104,111,111,199,170,85,203,59,37,227,216,177,99,73,93,202,59,92,39,125,242,201,39,163,119,239,
  222,106,98,170,211,9,74,100,86,239,19,78,126,159,62,125,34,34,128,17,201,85,122,215,53,215,160,149,23,75,156,121,180,250,39,139,164,198,82,255,
  177,222,51,130,174,37,194,201,44,39,216,64,86,249,73,68,25,241,174,171,72,201,104,103,234,53,84,93,37,122,208,18,111,249,157,153,79,147,232,58,
  14,215,254,82,65,38,1,1,33,208,81,194,229,114,161,188,124,11,246,238,171,142,250,92,41,238,57,81,241,207,225,12,92,233,89,6,128,86,167,11,
  67,78,60,17,20,207,163,177,190,30,237,30,111,72,114,21,170,19,146,137,153,70,254,208,147,78,2,45,8,162,124,175,47,40,89,14,55,217,35,24,
  180,215,224,86,212,173,208,73,111,72,48,253,147,217,177,70,75,120,5,68,238,161,39,100,58,181,200,108,36,3,77,61,91,140,183,252,206,16,230,104,
  174,33,22,4,147,159,200,129,114,164,245,28,235,0,188,43,235,35,150,186,232,138,58,14,181,63,153,50,9,153,38,72,22,210,54,141,157,203,229,66,
  69,197,86,228,231,21,96,228,200,82,0,225,13,173,178,106,15,28,142,122,0,98,44,117,50,161,205,146,193,178,44,46,190,248,98,208,20,45,46,199,
  201,48,242,111,80,192,160,65,131,176,123,91,69,128,23,87,47,156,35,96,159,95,254,37,151,92,2,154,162,65,211,180,44,159,166,105,128,2,242,242,
  242,176,123,251,182,176,100,63,232,54,127,25,23,93,116,145,110,25,20,77,137,101,108,171,80,157,167,215,97,234,149,195,178,44,46,189,244,210,208,250,
  251,101,39,170,243,12,233,85,86,220,203,176,199,168,123,122,241,43,68,121,228,97,144,120,66,173,189,183,210,64,13,97,6,106,177,46,94,17,169,252,
  112,247,63,84,57,209,148,17,141,205,199,42,63,150,129,114,188,101,119,182,78,186,162,62,186,147,220,174,146,73,250,72,2,66,160,99,189,40,150,133,
  32,8,112,187,221,168,169,61,128,119,255,251,31,140,29,115,154,152,113,67,211,225,121,253,19,5,87,174,90,150,240,176,141,96,11,42,40,201,58,69,
  81,168,111,104,192,176,161,67,80,181,115,167,62,73,5,80,14,160,161,177,17,57,185,185,234,142,156,162,100,242,165,37,211,82,167,226,104,108,196,176,
  161,67,80,185,99,135,76,216,180,216,186,73,128,163,177,17,214,254,253,195,147,102,29,162,232,104,108,68,241,144,147,81,185,115,167,46,25,164,40,10,
  229,63,0,245,13,13,176,246,239,31,177,119,91,16,4,212,55,52,96,120,209,80,81,255,32,216,10,81,118,238,128,1,113,243,220,69,242,224,212,214,
  197,192,129,3,209,212,212,4,147,201,132,166,166,166,176,215,216,55,59,91,142,189,119,186,92,65,137,52,121,64,196,223,14,245,6,159,241,28,168,233,
  13,114,163,145,31,105,91,213,94,75,60,203,136,199,53,68,67,70,99,149,29,76,215,120,15,192,245,226,138,227,89,223,137,186,143,241,184,127,225,158,
  193,93,37,51,146,201,130,164,255,36,136,203,179,100,204,152,49,105,245,110,152,166,105,20,21,21,131,243,249,112,240,151,131,104,111,111,7,199,113,48,
  155,205,0,0,183,219,13,139,197,130,140,140,12,120,60,30,180,180,180,128,231,121,48,12,3,158,231,59,229,229,43,45,45,197,168,81,163,240,238,187,
  239,70,212,113,133,251,173,220,118,253,245,215,227,237,183,223,142,232,225,169,245,104,82,17,60,88,110,190,249,102,252,251,223,255,14,249,96,8,69,152,
  35,13,49,208,235,220,194,77,42,138,134,192,198,210,105,70,155,197,36,146,237,130,32,160,111,223,190,24,126,202,41,48,103,100,160,177,177,17,91,182,
  108,9,169,111,223,190,125,113,209,69,23,225,203,47,191,68,78,78,14,126,254,249,231,184,233,126,215,93,119,225,149,87,94,9,186,255,186,235,174,195,
  182,109,219,80,81,81,209,229,54,28,137,29,117,22,17,221,71,158,143,44,79,46,77,139,159,40,81,127,248,48,114,115,114,194,31,167,51,16,76,149,
  50,162,145,31,110,160,172,189,15,142,35,71,162,146,29,177,61,112,92,100,121,133,99,185,175,137,106,51,93,45,151,162,0,134,137,76,102,164,245,155,
  8,153,154,235,143,71,159,31,43,82,169,79,37,72,12,210,206,3,205,48,12,10,242,11,113,206,57,231,99,213,170,229,216,179,103,55,218,156,109,104,
  105,105,134,219,237,6,199,113,104,109,109,69,107,107,171,234,129,153,172,144,13,201,59,20,204,3,22,236,193,174,213,79,143,196,66,177,122,151,36,143,
  66,71,168,128,150,56,107,73,114,56,143,144,116,140,82,102,56,125,168,8,61,114,193,94,23,199,171,3,76,20,121,214,94,183,242,152,140,140,12,52,
  54,52,32,59,59,27,7,15,30,12,78,250,253,247,173,172,172,12,123,247,238,133,32,8,242,241,170,123,166,83,159,196,27,221,57,59,12,185,61,70,
  98,28,41,36,194,26,106,48,72,81,84,204,228,57,25,101,68,34,95,121,92,52,247,39,119,192,128,136,6,230,81,235,30,41,105,139,205,131,147,152,
  54,211,157,228,38,162,126,99,144,217,149,228,153,128,16,232,110,143,236,236,190,40,42,42,198,105,167,157,129,159,247,239,195,143,63,110,66,117,117,21,
  88,150,5,207,243,113,205,243,28,13,66,198,14,7,249,173,244,142,235,197,215,74,75,223,10,193,200,129,98,91,40,34,29,138,56,43,101,68,226,73,
  151,116,165,194,16,24,237,239,120,119,112,241,144,23,233,61,83,110,227,121,30,253,7,12,16,99,245,26,27,67,102,44,25,124,194,9,248,233,167,159,
  112,188,63,228,67,247,26,20,247,151,144,232,228,147,235,206,182,183,120,166,49,76,70,25,209,212,71,162,218,95,170,181,235,120,166,199,35,72,222,125,
  34,32,32,4,186,19,40,43,29,3,154,162,241,235,175,191,226,164,147,78,134,219,237,66,109,109,13,218,218,218,146,58,97,80,34,59,161,72,167,50,
  46,90,218,199,113,28,126,251,219,223,138,49,220,82,38,13,5,113,154,55,111,94,72,210,74,69,64,164,111,188,241,70,149,126,50,121,23,4,8,60,
  47,151,205,251,127,11,126,82,47,248,183,41,241,225,135,31,118,16,252,8,98,8,163,73,187,20,107,184,72,172,157,108,184,16,22,85,198,18,197,190,
  194,194,66,172,92,177,2,133,133,133,200,205,205,69,109,109,109,128,231,94,210,160,119,239,222,24,54,108,24,250,245,235,135,253,63,255,140,67,135,14,
  201,158,105,249,109,66,8,111,52,33,209,241,37,134,209,230,36,142,38,3,68,34,115,18,39,170,140,68,16,199,68,147,209,120,200,143,182,15,137,182,
  140,104,22,173,73,5,185,241,110,107,241,150,153,200,244,125,4,4,61,150,64,75,160,41,10,39,157,120,50,202,202,70,99,107,249,86,124,243,205,151,
  112,58,157,200,200,200,0,199,113,240,122,189,73,241,42,104,73,170,158,7,83,217,209,189,247,222,123,65,73,220,149,87,94,41,18,88,137,84,9,2,
  4,69,168,5,165,67,216,84,132,20,192,27,111,188,17,178,147,213,35,137,74,29,180,250,40,73,158,222,0,33,88,71,174,37,139,209,62,180,58,155,
  194,41,236,67,71,121,205,33,190,5,65,192,207,251,247,99,210,164,73,16,4,1,63,254,248,163,238,64,77,242,40,87,84,84,160,162,162,2,227,198,
  141,195,47,191,252,18,52,84,70,214,93,113,143,9,18,67,162,67,13,220,58,235,209,141,150,36,196,26,178,148,168,50,18,33,63,209,100,52,18,175,
  125,188,115,15,199,67,118,184,108,40,137,146,27,237,192,48,82,146,30,111,153,137,238,243,9,8,8,129,214,65,86,102,38,204,25,102,56,157,78,156,
  49,110,60,142,29,59,134,157,187,118,192,227,241,128,97,152,184,103,228,8,55,113,73,75,192,116,127,107,73,43,196,156,213,188,32,128,146,246,83,20,
  40,63,129,133,244,29,162,83,161,34,212,169,83,250,32,124,220,169,222,183,82,159,68,199,67,71,116,207,194,212,139,60,184,16,4,252,250,235,175,248,
  245,151,95,212,161,55,122,113,226,138,55,2,223,111,218,212,145,69,69,51,224,9,168,27,205,189,37,94,232,196,144,232,68,61,128,19,65,182,146,93,
  70,34,229,39,163,126,18,77,172,186,91,104,75,42,134,207,197,67,38,233,23,9,18,129,180,91,72,197,235,245,98,231,174,29,88,180,232,107,212,212,
  30,128,215,23,217,130,40,199,31,63,8,99,198,156,134,220,92,113,54,119,126,126,62,76,38,83,194,200,115,176,255,202,133,51,120,77,136,132,50,148,
  130,231,121,49,132,130,231,225,241,120,196,237,122,199,232,200,18,16,124,85,65,61,130,156,112,125,194,120,116,99,245,72,197,155,60,43,245,50,155,205,
  16,4,1,217,217,217,232,211,167,15,122,247,233,35,94,139,116,205,254,112,23,179,217,44,238,239,221,187,163,30,20,31,169,158,0,32,51,43,11,25,
  25,25,98,253,104,66,100,244,238,91,162,234,168,167,146,232,72,112,199,29,119,36,84,135,68,204,1,72,102,25,137,148,159,140,250,33,232,94,136,196,
  30,73,123,33,72,20,210,210,3,253,235,175,191,160,177,177,65,238,108,109,182,90,12,29,82,4,138,10,63,94,160,253,51,146,79,60,241,100,52,54,
  54,202,57,121,227,9,33,4,97,211,253,72,196,72,135,108,2,192,130,5,11,84,157,133,60,145,80,242,252,106,60,147,122,161,1,122,175,100,245,72,
  182,234,191,248,71,212,79,17,158,16,173,62,170,99,19,232,69,237,140,92,101,125,93,112,193,5,216,185,115,39,26,27,27,81,92,92,12,94,16,224,
  114,58,81,225,159,40,40,175,220,72,211,24,58,116,40,70,140,24,129,159,247,239,199,218,53,107,84,178,228,251,224,247,66,159,53,117,42,242,11,10,
  176,108,233,82,236,219,183,47,224,205,1,5,64,160,40,156,120,194,9,56,116,232,16,204,102,51,154,154,154,130,78,212,36,136,190,125,144,1,8,1,
  65,207,27,24,19,16,16,2,173,32,126,110,183,91,254,191,111,223,94,212,214,214,128,162,40,184,221,110,236,218,181,3,12,203,200,100,57,25,250,104,
  54,168,136,104,196,68,90,16,208,214,214,6,183,219,13,183,219,13,175,215,43,103,18,97,89,22,70,163,17,38,147,9,102,179,25,153,153,153,202,94,
  68,85,118,64,92,116,144,73,141,202,101,191,165,229,167,85,100,218,175,143,203,229,66,123,123,59,60,30,143,74,31,131,193,128,140,140,12,100,100,100,
  32,51,51,83,206,228,129,16,161,24,42,162,175,179,146,98,180,4,59,230,88,231,32,247,44,59,59,27,13,13,13,16,4,1,37,37,37,104,108,108,
  4,199,243,104,242,147,103,94,121,191,124,62,48,12,3,155,205,134,189,213,213,56,116,232,16,92,46,151,238,61,59,238,184,227,144,147,147,3,163,193,
  0,179,217,44,134,192,32,240,21,209,132,241,227,113,172,185,25,211,166,77,195,175,135,14,97,235,150,45,36,148,35,65,15,220,104,227,55,195,213,121,
  34,38,91,37,178,12,130,228,222,219,88,38,74,119,165,220,84,189,47,164,239,35,32,4,58,142,240,249,124,50,105,17,4,1,123,247,85,195,102,175,
  149,9,117,178,201,189,182,35,11,230,113,86,126,90,90,90,208,210,210,34,231,175,214,194,235,245,194,235,245,162,173,173,13,0,144,149,149,133,94,189,
  122,161,87,175,94,29,164,213,255,161,180,164,58,180,194,186,68,62,82,125,156,78,103,160,62,234,222,46,96,229,68,109,126,104,57,255,116,132,25,59,
  18,121,207,154,154,154,228,28,205,28,207,163,177,177,17,249,249,249,56,88,87,23,16,150,210,210,210,130,69,139,22,161,161,161,33,236,61,107,108,108,
  196,7,31,124,128,19,79,60,17,7,14,28,16,61,250,20,5,158,162,64,209,180,76,164,119,237,222,141,230,99,199,208,55,59,27,191,40,114,69,19,
  210,156,188,135,115,60,38,109,37,42,179,66,50,202,232,44,25,75,36,25,237,42,27,232,14,177,195,233,72,46,73,72,15,1,33,208,73,0,195,48,
  96,89,22,30,143,7,189,122,245,70,73,201,48,24,12,6,108,223,190,13,30,79,99,151,232,20,212,211,236,39,98,188,32,192,211,222,142,163,71,143,
  226,216,177,99,81,61,88,90,91,91,101,239,112,118,118,54,140,38,83,196,1,239,193,226,163,19,162,15,207,67,8,242,38,32,21,59,70,237,125,106,
  106,106,2,195,178,232,213,171,23,154,154,154,58,85,71,59,118,236,192,174,93,187,208,187,119,239,160,117,116,180,169,9,22,139,5,7,15,30,196,193,
  131,7,201,195,35,141,200,121,119,43,131,144,60,2,2,2,66,160,211,152,52,211,52,13,139,197,130,1,253,7,160,119,159,108,84,85,85,98,250,180,
  153,40,43,27,141,234,234,42,236,220,177,61,225,132,75,254,173,37,97,138,237,74,146,42,125,187,93,46,52,54,54,6,245,242,70,82,246,209,163,71,
  225,243,249,208,175,95,63,100,152,205,17,145,104,221,9,141,9,212,135,23,132,14,189,148,233,219,148,11,173,0,42,79,117,34,31,178,225,238,153,52,
  9,178,98,219,54,140,25,51,6,12,195,96,229,202,149,113,169,35,158,231,67,214,81,134,217,140,162,162,34,108,223,190,61,112,229,201,36,214,17,1,
  129,18,22,139,5,127,125,234,57,188,248,247,103,209,216,216,32,255,190,226,242,223,32,47,47,31,117,117,118,252,247,189,183,97,205,177,226,150,91,238,
  192,220,187,111,195,229,151,93,133,241,227,39,2,0,94,127,253,85,140,28,89,42,255,7,128,39,158,124,12,141,141,13,200,203,203,199,173,183,220,1,
  179,217,130,189,251,170,241,201,39,255,195,163,143,60,142,215,95,127,21,57,86,43,46,191,236,42,220,253,167,219,1,0,247,252,233,1,124,243,205,66,
  84,239,173,82,201,183,219,109,120,253,223,175,194,233,116,226,188,243,46,132,219,229,194,138,149,203,96,177,88,112,203,205,119,32,63,191,0,13,13,14,
  124,255,253,6,156,119,222,133,178,14,159,126,246,17,46,191,236,42,221,255,78,167,19,175,255,251,85,216,237,54,0,64,126,126,1,110,185,249,14,88,
  44,22,84,239,173,194,235,175,191,138,241,227,39,202,199,127,243,205,66,84,108,43,199,163,143,60,142,21,43,151,225,155,111,22,226,209,71,30,135,163,
  193,129,162,161,197,114,25,213,123,171,228,255,146,222,57,57,214,0,217,143,62,242,56,190,255,126,3,86,172,92,70,26,32,1,65,23,128,78,199,139,
  50,155,205,232,215,175,31,6,13,202,195,201,39,13,193,53,191,253,61,110,187,109,46,70,156,114,42,140,6,67,68,50,26,28,14,120,189,222,248,42,
  166,19,87,43,40,22,70,81,18,86,79,123,123,167,136,152,18,173,173,173,104,108,108,132,167,189,93,29,163,27,225,135,79,160,62,218,208,21,101,157,
  132,172,187,100,65,71,143,3,7,14,160,166,166,6,46,151,11,107,215,174,197,202,85,171,18,122,207,148,117,52,113,194,4,12,29,58,20,87,93,117,
  85,234,212,17,65,143,135,211,233,68,93,157,29,197,69,37,24,58,180,24,46,151,11,163,70,149,1,0,238,254,211,237,112,186,156,56,227,140,9,242,
  241,69,67,139,49,126,252,68,60,249,212,95,240,205,55,11,101,210,90,189,183,10,115,239,190,13,115,239,190,13,141,141,98,232,211,249,231,95,132,109,
  219,42,240,231,71,238,71,126,94,1,202,74,71,171,50,114,72,3,197,156,28,43,242,243,11,48,100,104,145,138,140,62,252,231,251,144,147,99,149,245,
  41,29,85,134,145,35,75,1,0,227,207,152,8,139,197,130,185,119,223,6,71,131,3,25,102,51,94,127,253,85,0,192,147,79,253,69,150,35,233,164,
  252,111,175,179,225,140,113,29,215,116,222,121,23,98,219,182,114,60,252,231,251,144,159,87,128,81,163,202,112,249,101,87,225,191,239,189,141,215,95,127,
  21,231,157,119,33,44,22,139,170,92,0,248,244,211,143,240,205,55,11,209,208,224,192,220,187,111,67,131,195,17,160,247,229,151,93,133,141,223,175,87,
  201,38,32,32,232,90,164,157,7,154,101,89,140,28,57,10,179,102,157,135,3,251,127,70,67,131,3,253,251,31,23,209,185,135,14,31,194,177,99,199,
  0,0,149,85,123,226,74,160,149,158,76,53,231,17,84,25,54,164,207,209,163,71,227,66,196,148,132,140,101,89,228,230,230,130,87,196,212,234,115,198,
  14,15,180,148,78,45,81,250,244,239,223,191,99,98,163,194,43,175,151,3,58,217,217,38,82,229,158,41,235,104,249,242,229,50,105,80,78,130,237,170,
  58,234,105,32,94,253,224,216,190,189,2,121,121,249,0,128,189,123,171,144,159,95,128,237,219,43,0,0,251,246,86,99,200,208,34,236,219,91,13,64,
  244,214,218,237,54,52,52,56,176,98,229,50,172,88,185,12,151,95,118,21,138,134,22,227,229,151,94,195,198,141,235,241,233,103,31,201,100,251,191,223,
  191,13,167,211,41,147,202,243,206,187,16,183,220,162,78,97,86,58,170,12,13,13,14,76,24,63,9,223,124,179,48,96,222,132,211,233,68,209,208,98,
  80,20,133,130,130,66,228,231,23,96,104,81,49,182,249,117,148,136,179,116,140,242,158,191,242,242,191,100,93,149,50,93,110,151,124,92,113,81,9,222,
  253,254,63,178,158,146,156,109,219,202,229,227,243,243,11,64,81,20,246,237,171,198,248,51,38,6,157,52,173,213,187,160,160,16,223,124,179,80,28,168,
  28,180,195,154,99,37,109,145,216,35,1,33,208,241,55,168,62,189,179,209,55,187,47,14,68,112,60,231,79,191,182,114,213,50,180,182,182,202,147,222,
  60,30,79,226,136,153,54,45,156,102,91,75,75,139,76,228,227,137,99,199,142,33,35,35,3,189,122,247,238,72,203,166,8,3,80,17,199,36,233,99,
  54,155,101,125,130,173,94,152,10,157,100,176,123,118,220,113,199,65,16,4,236,219,183,47,161,117,148,147,147,3,150,101,193,178,172,156,90,49,213,234,
  168,167,224,174,187,238,34,149,224,199,43,175,188,34,255,174,222,91,133,179,207,158,1,138,162,96,183,219,48,180,168,88,151,236,72,223,74,242,41,109,
  223,187,175,26,255,250,215,63,2,182,187,252,253,178,114,219,191,254,245,15,228,88,173,184,226,242,223,0,0,70,141,42,195,215,223,44,196,21,151,255,
  70,14,129,40,26,90,140,167,255,246,2,236,118,27,246,250,245,171,216,86,14,107,142,85,62,70,43,91,247,158,207,189,21,0,228,144,16,137,80,127,
  253,245,151,170,227,34,145,5,0,27,55,174,199,239,127,255,135,160,251,181,122,147,182,73,64,64,8,116,74,192,235,243,98,251,142,109,216,188,121,19,
  154,155,69,210,83,95,95,47,103,81,72,16,3,11,74,204,160,9,97,104,105,105,73,136,30,18,25,206,234,213,75,55,23,177,82,79,45,161,79,164,
  62,189,122,245,130,32,101,146,144,188,209,122,132,48,204,170,138,93,113,207,78,63,253,116,12,26,52,8,47,189,244,82,194,234,136,231,121,76,152,48,
  1,253,250,245,195,209,163,71,81,81,81,129,214,214,214,212,168,163,30,76,24,9,212,144,98,129,71,141,42,195,138,149,203,96,182,88,228,223,67,139,
  138,229,253,210,177,103,159,61,3,57,57,86,148,142,42,195,168,81,101,170,253,74,84,239,173,18,207,175,179,225,209,71,158,192,74,157,152,223,252,252,
  2,228,231,23,224,58,63,41,149,200,123,245,222,42,108,219,86,142,105,103,207,128,211,233,84,133,78,228,228,88,177,109,91,185,172,227,173,183,222,41,
  18,214,234,240,132,245,137,39,31,195,99,143,62,33,59,92,244,244,252,228,211,255,201,245,33,17,107,233,26,171,247,86,193,94,103,83,197,62,107,175,
  89,169,183,52,32,177,215,217,144,159,87,128,141,27,215,147,246,73,64,208,197,160,123,210,197,30,57,114,24,78,151,19,223,125,183,10,95,127,253,37,
  106,106,106,228,85,224,120,197,66,32,201,227,103,154,148,118,254,188,202,241,12,3,208,162,173,173,13,109,109,109,186,177,199,122,177,216,78,255,241,137,
  66,107,107,171,120,189,97,86,216,75,21,104,245,99,89,22,213,213,213,216,191,127,127,194,202,60,124,248,48,56,142,195,160,65,131,208,187,119,239,0,
  93,8,8,82,5,123,247,86,201,132,79,34,186,175,188,252,47,88,115,172,42,210,39,17,196,199,30,125,2,231,159,127,145,236,201,45,26,90,140,87,
  94,254,23,94,121,249,95,200,201,177,2,0,190,254,250,75,140,63,99,162,232,145,173,179,161,194,31,18,161,196,168,81,101,216,182,173,28,119,205,189,
  21,239,254,247,63,24,127,70,199,100,196,141,27,215,195,108,182,96,218,217,51,0,136,222,228,135,30,190,23,249,249,5,168,246,123,119,245,116,84,17,
  83,191,78,18,26,26,28,50,193,149,160,213,115,219,182,114,124,242,233,255,112,221,239,255,128,91,111,189,19,95,127,253,165,138,112,175,92,17,122,242,
  159,164,247,248,241,19,241,201,167,255,11,144,13,136,241,225,175,188,252,47,149,30,4,4,4,201,1,53,122,244,232,180,122,10,27,141,70,76,157,58,
  13,231,158,115,62,202,203,183,192,94,87,7,150,101,112,224,192,79,168,171,171,131,203,229,74,24,249,40,45,45,69,105,105,41,222,121,231,29,93,194,
  165,36,171,193,150,197,118,56,28,104,108,76,108,90,189,94,189,122,33,55,55,23,12,195,116,120,49,131,120,160,27,26,26,208,212,212,148,88,125,122,
  247,70,255,254,253,193,178,44,104,127,108,175,42,206,87,90,193,48,73,185,160,163,185,103,153,153,153,104,105,110,22,87,14,76,32,134,13,27,134,147,
  135,12,193,193,131,7,225,108,107,139,186,142,230,206,157,139,151,95,126,57,168,252,235,175,191,30,21,21,21,168,168,168,232,114,27,214,179,35,2,2,
  2,130,238,132,84,234,83,9,18,131,180,13,225,40,47,223,130,245,27,214,226,151,95,196,197,38,60,30,15,56,142,83,17,109,138,162,224,241,120,32,
  8,2,104,154,142,187,23,90,151,164,107,86,31,20,52,219,147,177,176,139,199,227,65,187,199,131,12,147,73,38,98,208,46,236,226,255,78,196,82,230,
  1,250,180,183,195,211,222,14,134,166,1,134,9,76,199,166,36,248,202,237,9,32,209,209,222,179,150,230,102,28,60,120,48,225,117,116,224,192,1,180,
  180,180,192,108,54,131,233,226,58,34,32,32,32,32,32,32,4,58,205,224,245,122,177,118,237,106,176,44,171,90,54,89,117,209,44,139,17,35,78,5,
  77,51,216,189,123,39,156,78,39,250,247,239,143,35,71,142,36,63,148,67,145,121,66,210,63,209,240,249,124,224,124,62,120,104,26,25,38,83,80,189,
  146,169,143,207,231,67,187,199,3,179,217,12,42,213,227,120,187,224,158,73,203,164,119,155,58,34,32,32,32,32,32,72,99,164,93,12,180,32,8,112,
  185,92,104,105,105,209,37,207,0,64,211,52,6,12,56,30,35,78,57,21,230,12,51,0,192,98,201,84,165,5,235,42,4,211,57,158,144,6,9,94,
  175,23,94,159,79,69,6,133,174,212,199,227,129,47,9,100,180,91,223,179,110,90,71,4,4,4,4,4,4,132,64,19,164,13,146,65,254,162,129,151,
  144,67,82,71,4,4,4,4,4,4,132,64,19,68,3,150,77,124,84,141,210,211,206,249,195,57,36,80,93,172,143,143,227,82,142,212,167,218,61,235,
  142,117,68,64,64,64,64,64,64,8,52,65,252,224,207,158,32,193,16,225,82,227,241,32,124,82,185,156,94,220,183,63,190,54,153,250,72,101,42,39,
  123,146,123,214,77,235,136,128,128,128,128,128,32,141,193,166,91,46,89,202,191,32,135,244,27,128,254,127,237,117,235,38,95,136,174,110,180,171,213,41,
  101,8,154,137,103,2,212,11,114,72,48,153,76,170,92,161,137,34,99,202,133,84,120,5,25,83,102,152,72,154,62,18,1,245,151,201,113,156,106,85,
  66,89,47,205,202,123,137,106,187,41,121,207,226,80,71,161,234,75,175,237,118,21,82,73,23,2,2,2,2,210,143,17,232,129,120,160,147,73,238,53,
  191,3,242,245,82,20,204,102,115,194,245,48,26,141,98,249,254,255,60,207,139,158,77,137,140,249,115,9,83,20,133,140,36,232,99,50,26,85,255,121,
  158,23,203,15,82,119,61,241,158,165,114,29,17,16,16,16,16,16,244,52,244,200,165,188,5,65,192,225,195,135,80,95,127,4,46,183,184,176,74,155,
  179,85,38,70,82,94,232,100,188,38,151,202,163,40,10,148,32,192,98,177,32,51,51,51,97,171,255,153,76,38,152,76,166,142,20,104,18,33,236,80,
  72,244,176,250,183,91,204,230,132,235,99,244,235,67,249,73,124,170,143,216,187,226,158,117,183,58,234,137,152,62,109,38,242,243,11,224,116,57,177,98,
  249,82,56,93,78,92,121,197,213,162,29,89,44,200,207,47,68,117,117,37,0,113,73,231,252,252,2,249,220,143,63,153,135,105,211,102,194,154,99,69,
  117,117,21,138,138,58,150,120,222,176,113,61,10,252,75,85,111,216,184,94,150,113,193,5,23,195,234,95,173,207,110,183,193,102,183,97,194,248,137,176,
  219,109,88,190,98,41,138,138,74,84,255,173,57,86,76,155,62,19,78,167,19,43,86,44,149,117,211,234,227,104,112,224,171,175,190,144,203,176,88,44,
  88,177,124,41,28,13,14,249,26,37,153,23,92,112,49,170,171,171,80,160,184,150,252,252,2,249,122,148,229,72,231,232,201,237,169,144,238,225,87,95,
  125,17,180,126,1,96,133,255,183,197,108,209,189,103,21,219,202,81,81,177,53,232,253,145,218,140,180,95,106,71,210,127,9,78,151,19,21,21,229,1,
  247,83,217,46,164,118,37,149,91,93,93,169,106,75,27,54,174,71,81,81,49,190,250,234,11,88,44,150,128,125,165,165,101,98,155,255,120,94,64,91,
  200,177,230,202,178,37,187,208,182,83,179,197,130,234,234,42,76,24,63,17,95,125,245,133,108,103,210,50,233,122,237,152,128,32,17,232,145,30,104,159,
  207,135,29,59,183,99,251,142,109,242,202,132,0,112,226,9,39,193,98,17,59,168,227,143,31,40,18,205,56,146,174,112,251,36,175,111,86,175,94,9,
  89,0,67,233,81,150,87,173,83,236,147,200,153,244,91,58,38,43,43,43,97,250,152,205,102,149,55,92,111,65,144,72,234,48,81,68,57,21,238,89,
  42,215,17,1,84,68,195,106,205,5,0,220,118,251,92,228,228,88,81,90,58,26,118,187,13,213,213,85,88,225,39,181,78,151,24,238,83,84,84,2,
  187,221,6,187,221,134,105,211,102,162,216,191,239,202,43,69,50,32,157,91,144,95,128,105,211,102,194,110,183,225,202,43,59,200,196,132,241,147,224,116,
  137,203,102,59,26,28,184,254,186,63,194,110,183,97,252,132,73,40,45,29,173,250,95,84,84,130,235,175,191,17,22,179,5,5,249,5,152,48,126,18,
  236,118,27,156,46,39,138,138,74,100,114,82,84,84,34,147,242,11,46,184,24,197,254,125,23,92,112,49,74,75,71,203,122,76,155,54,19,165,165,163,
  49,97,252,36,153,220,75,159,210,210,209,40,29,37,238,83,214,193,248,9,147,48,125,218,204,0,185,61,153,60,75,247,252,182,219,231,6,173,223,9,
  227,197,251,87,58,106,180,124,143,180,109,232,202,43,174,198,244,105,51,131,222,31,0,42,249,82,59,178,41,218,128,221,110,67,233,168,209,1,247,211,
  106,205,133,221,110,67,131,67,28,232,20,228,23,168,202,85,222,99,169,189,78,24,63,9,0,2,246,89,115,172,242,53,229,231,23,4,180,5,165,108,
  187,221,38,47,11,175,188,102,233,154,138,138,74,228,182,86,84,84,18,84,95,2,2,66,160,227,12,143,199,35,175,66,8,0,3,7,230,97,234,212,
  179,209,175,95,14,0,160,184,184,36,174,4,90,193,112,58,8,170,30,105,165,40,244,234,213,11,125,250,244,137,123,209,102,179,25,230,140,12,185,76,
  153,0,250,151,130,86,45,7,173,32,135,189,122,245,66,239,68,232,99,177,136,132,94,210,199,175,19,173,83,63,93,138,174,188,103,221,165,142,8,68,
  175,151,163,30,31,127,60,15,22,179,69,38,6,0,224,116,58,101,239,171,246,193,46,122,142,39,161,162,98,43,190,250,234,11,84,84,108,69,149,223,
  99,88,85,93,169,242,208,62,255,252,223,116,203,149,60,140,57,86,43,222,121,231,205,160,255,45,22,11,54,108,92,143,229,43,150,98,249,138,165,40,
  240,123,197,151,175,88,42,199,241,75,122,74,237,207,110,179,225,157,119,223,66,233,168,50,249,216,234,234,74,148,142,42,211,175,131,6,135,236,101,148,
  80,85,93,137,141,27,214,97,252,132,73,1,114,123,42,148,247,220,110,171,197,132,241,19,117,235,215,233,114,202,191,245,218,144,212,62,242,243,11,34,
  186,63,202,118,84,93,93,41,203,146,100,231,88,173,242,64,80,175,141,105,203,85,34,212,92,16,167,211,9,139,197,2,167,211,9,167,211,41,219,71,
  176,182,96,243,123,148,131,217,141,72,252,139,145,95,80,32,147,235,80,250,18,16,196,27,108,58,190,10,214,94,147,160,51,177,74,59,177,48,80,8,
  162,158,0,160,156,124,166,87,182,106,114,154,98,185,108,237,132,70,10,64,239,222,189,225,245,122,227,22,22,96,202,200,128,37,51,83,87,103,150,166,
  59,38,162,73,203,122,43,235,69,16,208,167,119,111,248,226,173,143,223,219,175,186,126,138,2,77,211,170,9,113,20,116,231,120,38,165,253,116,249,61,
  139,83,29,69,210,150,83,105,18,161,158,29,119,183,254,71,154,130,154,159,95,0,135,195,17,176,93,218,39,253,23,32,160,173,173,13,203,151,47,85,
  245,67,229,229,91,144,147,147,131,9,227,39,99,252,248,73,248,191,199,255,172,58,223,98,182,64,16,4,124,244,241,135,152,62,125,38,110,191,109,46,
  254,249,207,151,3,254,191,253,206,27,184,234,202,223,250,189,134,57,178,135,240,237,119,222,128,32,8,152,54,109,38,42,42,182,194,102,171,5,0,44,
  92,184,0,22,179,5,23,92,112,177,234,53,127,64,255,170,105,121,118,91,173,236,213,150,119,9,226,113,22,179,37,64,238,71,31,127,216,99,31,192,
  202,123,62,125,250,76,221,250,181,219,106,145,159,95,168,219,182,0,224,202,43,174,86,145,69,189,251,35,8,66,208,118,36,183,63,161,163,189,154,21,
  161,34,102,179,5,249,249,5,168,181,213,162,222,81,47,31,47,151,43,64,213,206,157,174,182,14,121,154,125,86,171,21,118,123,173,188,237,237,119,222,
  84,181,5,71,67,189,202,46,170,170,246,232,182,51,1,2,156,206,54,228,231,23,194,108,177,192,233,108,11,170,111,87,247,5,36,220,46,125,65,38,
  17,38,213,145,73,169,126,43,189,188,90,175,166,209,100,66,223,190,125,97,182,88,226,66,196,178,50,51,193,50,76,71,57,138,178,89,150,13,233,129,
  166,104,26,70,163,49,49,250,0,1,94,84,150,101,85,19,246,66,14,116,122,194,61,75,241,58,34,80,163,192,79,118,36,44,95,190,20,11,191,90,
  160,123,236,219,239,188,137,229,203,151,194,209,80,15,139,57,19,165,165,163,113,251,237,115,85,199,76,159,62,19,214,156,92,188,243,206,155,176,90,173,
  42,249,21,21,91,241,246,59,111,34,63,191,16,19,39,76,194,115,207,137,158,197,162,226,226,128,255,215,95,127,35,22,46,252,2,213,213,149,40,200,
  47,196,132,241,147,177,97,195,58,56,28,14,76,159,62,19,22,139,69,165,231,85,87,254,22,142,134,122,44,95,177,4,165,165,163,225,112,56,144,99,
  181,202,94,74,71,144,87,228,54,123,173,174,39,178,32,191,16,118,123,109,128,92,139,197,210,49,80,236,65,208,187,231,122,245,91,94,177,53,100,253,
  44,95,177,68,62,62,212,253,9,213,142,148,168,168,216,42,147,92,0,176,219,107,241,182,226,45,134,182,220,72,218,185,180,47,63,191,80,245,209,182,
  5,173,93,132,66,131,195,33,183,51,165,119,58,152,190,4,4,241,6,75,170,32,241,4,76,59,2,85,109,243,147,50,1,0,165,240,252,210,0,50,
  204,102,228,228,228,160,17,128,203,229,138,122,36,43,197,207,90,130,144,103,169,124,137,140,73,255,245,135,90,52,50,50,50,226,174,143,76,12,209,17,
  151,173,92,152,36,85,72,97,87,222,179,88,234,136,144,233,174,65,81,81,9,138,138,74,176,97,195,58,84,87,85,1,23,64,38,71,122,228,226,185,
  103,95,146,247,93,120,193,37,152,128,73,216,176,97,157,234,152,234,170,42,76,191,125,22,74,75,71,139,30,98,5,185,153,62,109,22,166,79,155,133,
  13,27,214,33,39,39,23,207,61,251,18,156,78,39,54,108,88,135,9,227,39,171,254,91,115,114,101,93,36,194,50,97,194,36,76,240,135,85,0,192,
  255,253,229,175,112,52,212,227,185,231,254,6,71,67,61,166,79,155,37,31,191,97,195,58,148,150,142,150,101,46,95,177,4,78,87,155,124,204,219,239,
  188,137,137,126,89,21,21,91,101,143,42,32,214,129,211,233,196,63,95,123,9,165,165,163,85,114,165,73,98,61,45,156,99,225,194,47,112,195,245,55,
  202,247,124,249,138,37,184,255,190,63,171,234,119,194,132,73,104,112,56,224,104,168,135,53,39,55,40,225,157,62,109,22,150,219,151,6,220,159,9,19,
  38,201,109,228,159,255,124,57,104,59,10,215,166,159,123,246,37,185,93,104,203,45,42,46,150,239,177,116,79,165,182,45,17,241,219,111,159,43,135,111,
  72,111,28,36,242,172,108,11,90,187,120,238,249,191,6,29,168,1,80,133,38,133,211,151,128,32,238,188,160,180,180,52,173,222,47,132,203,3,173,12,
  225,80,18,147,210,210,209,56,109,236,233,88,180,248,27,216,237,54,156,125,214,116,252,240,227,247,104,110,110,142,184,236,210,210,82,148,149,149,225,237,
  183,223,86,109,15,22,18,32,125,135,250,240,130,0,183,203,133,150,150,22,184,221,110,180,183,183,71,230,193,52,153,144,145,145,129,12,105,2,154,146,
  132,41,190,13,6,67,128,119,67,10,13,144,191,147,160,143,68,238,141,6,3,50,51,51,117,117,141,150,32,6,35,175,225,206,13,22,2,212,213,247,
  44,154,58,210,94,227,221,119,223,141,151,94,122,41,104,217,55,220,112,3,202,203,203,81,81,81,209,229,54,28,204,142,82,29,22,139,5,214,156,92,
  56,93,109,242,67,95,233,229,179,217,107,97,85,120,6,181,251,164,243,37,98,83,144,95,40,255,214,238,3,0,171,213,10,139,57,83,246,102,58,157,
  78,213,57,90,25,210,127,229,177,18,156,174,54,89,150,82,127,171,223,195,168,36,49,161,100,74,36,73,121,156,84,142,86,119,73,174,212,255,40,189,
  214,177,218,110,119,131,222,125,85,214,175,212,94,36,47,189,20,6,161,188,63,218,109,218,243,165,251,170,215,198,148,247,67,41,71,249,76,144,136,187,
  182,93,40,143,215,182,101,233,191,30,241,215,182,111,109,27,211,202,82,150,167,173,19,169,205,132,211,183,43,144,74,125,42,65,130,60,208,36,6,154,
  146,14,210,149,19,235,98,42,209,16,178,144,3,2,136,139,104,152,205,102,100,100,100,200,147,31,125,62,31,124,62,159,152,195,25,226,82,207,44,203,
  130,101,89,24,141,70,49,215,179,142,199,89,185,13,136,126,21,189,68,232,163,244,174,74,250,72,3,28,229,119,36,228,56,150,246,17,205,57,93,117,
  207,98,169,163,104,174,59,213,226,142,187,99,236,96,91,91,91,64,252,123,173,173,70,245,191,190,190,62,232,62,237,249,202,253,122,178,69,89,245,33,
  203,11,245,95,187,79,43,75,171,111,36,50,245,244,15,44,71,148,43,221,223,214,214,214,184,216,110,170,144,235,104,200,127,184,54,35,213,191,242,56,
  229,61,145,126,7,107,87,218,54,162,87,158,86,142,32,8,170,123,34,253,86,234,175,45,55,210,54,161,119,140,182,141,5,107,47,122,117,34,213,119,
  56,125,187,178,29,144,24,232,52,38,208,164,10,18,15,45,185,209,35,61,161,140,221,192,178,224,141,70,120,60,158,142,156,192,193,140,82,147,154,14,
  33,200,179,201,100,130,193,96,8,44,91,10,75,240,231,131,166,18,172,143,244,219,232,215,167,187,120,154,146,121,207,66,213,81,44,222,121,2,130,84,
  34,151,241,150,157,104,27,232,204,32,60,21,200,127,119,211,191,187,215,55,1,33,208,221,134,172,74,175,187,60,30,15,120,158,135,209,104,4,77,211,
  240,120,60,112,58,157,200,204,204,132,209,104,132,215,235,133,211,233,148,61,130,41,65,176,253,100,234,145,71,30,135,207,231,3,231,243,97,245,234,21,
  168,171,171,67,105,233,104,52,52,54,64,16,120,212,59,28,200,233,215,15,235,55,172,131,57,195,140,209,163,199,160,178,114,15,134,15,63,5,59,118,
  108,71,115,75,51,198,157,54,14,251,247,239,199,168,210,50,48,52,13,154,97,176,173,162,28,7,106,246,195,96,48,192,100,50,5,141,159,149,8,190,
  146,68,63,244,208,99,224,252,94,212,21,43,150,226,231,253,63,227,148,83,78,133,32,240,224,121,1,13,13,13,232,215,175,31,54,108,92,7,75,134,
  5,101,163,199,160,170,114,15,134,159,50,2,59,118,84,160,185,185,25,103,140,155,128,159,247,255,132,178,210,209,160,25,6,12,77,163,162,162,28,246,
  58,27,76,146,7,182,19,29,93,44,49,199,177,220,43,189,123,6,0,179,102,159,139,83,79,25,9,159,207,135,3,53,7,240,201,167,255,131,211,233,
  196,208,147,135,194,235,245,160,166,230,0,6,13,204,67,113,201,48,236,218,41,222,167,49,163,199,98,221,134,117,56,101,248,41,40,44,60,1,12,77,
  99,231,174,29,160,41,26,101,163,199,136,247,142,102,80,189,183,10,7,14,252,76,58,127,130,180,36,206,130,32,224,206,59,239,12,56,231,213,87,95,
  141,218,118,35,113,76,36,131,196,221,121,231,157,1,58,188,242,202,43,93,66,254,187,219,224,37,93,6,91,4,233,76,160,211,233,245,2,69,193,192,
  178,152,60,233,76,156,119,222,133,40,47,223,130,134,6,7,78,59,237,12,244,233,211,7,219,182,149,99,225,194,5,152,54,109,38,202,202,70,163,186,
  186,10,11,23,46,64,67,99,3,134,13,59,69,107,98,162,199,48,154,250,145,142,213,57,71,78,51,166,76,59,38,8,242,183,180,77,138,57,166,105,
  26,255,254,247,171,104,106,108,148,211,167,217,236,181,184,240,194,75,177,110,221,119,104,112,56,48,113,210,20,80,160,64,83,20,104,154,1,77,137,68,
  171,160,160,16,187,119,239,4,77,139,147,208,104,154,198,151,11,23,200,11,112,24,88,22,25,38,19,104,63,65,214,118,32,178,94,74,157,0,48,52,
  141,215,223,252,23,154,154,26,33,0,224,56,14,171,86,46,195,217,211,102,162,114,207,78,56,26,28,152,52,105,50,40,80,160,40,128,97,24,80,52,
  5,134,166,81,80,48,24,187,119,237,4,77,83,178,78,95,44,248,172,67,159,140,12,208,52,221,161,79,12,233,3,17,120,7,245,155,73,12,29,41,
  165,45,71,231,158,1,0,67,209,176,217,109,248,98,193,167,184,225,15,55,99,248,176,225,216,86,81,142,129,199,15,132,199,35,18,104,138,162,192,48,
  52,242,242,11,80,185,103,55,104,154,129,53,39,7,3,7,230,97,205,119,171,48,104,208,32,156,114,202,8,84,238,217,3,154,162,240,249,252,79,229,
  58,98,88,86,85,71,170,186,146,136,188,94,125,68,210,150,163,109,239,137,123,186,5,181,163,100,193,98,177,224,170,171,174,193,71,31,125,128,233,211,
  103,163,186,122,143,63,91,197,44,56,157,78,44,92,56,31,23,94,120,169,28,191,105,183,119,164,26,147,254,3,128,205,86,139,162,162,97,168,174,222,
  131,162,162,97,88,184,112,190,124,204,140,25,179,228,115,236,246,90,216,108,181,152,48,97,50,28,14,7,22,46,156,175,218,191,112,225,2,56,28,245,
  40,46,46,145,229,204,152,49,75,150,149,239,207,112,177,108,217,18,92,120,225,165,42,125,1,224,163,143,62,80,201,147,202,0,128,11,47,188,20,22,
  139,5,203,151,47,193,244,233,179,80,81,177,5,0,80,90,58,6,22,139,69,214,173,160,160,48,236,53,78,152,48,89,182,133,143,63,254,16,211,166,
  205,130,217,108,198,138,21,75,229,220,208,203,151,47,197,249,231,95,44,110,95,185,12,13,138,84,99,255,240,147,75,233,206,223,126,219,109,106,27,141,
  210,118,5,116,126,137,123,65,219,46,131,29,163,179,159,231,184,192,1,128,98,192,125,254,249,23,195,106,181,98,227,198,117,254,5,78,126,43,31,246,
  201,39,243,48,109,218,44,185,141,185,92,46,88,44,22,85,155,44,42,26,166,106,131,203,150,45,193,140,25,179,144,151,39,166,29,116,58,157,48,155,
  205,168,171,179,193,102,171,69,126,126,1,242,242,10,240,201,39,243,144,159,95,128,33,67,196,9,128,57,57,57,16,4,1,7,15,218,49,104,80,62,
  62,251,236,35,56,157,78,228,228,88,113,214,89,211,65,81,20,86,174,90,14,107,78,14,206,56,99,34,40,10,216,184,113,61,40,138,194,120,255,162,
  41,20,69,225,227,143,63,68,65,65,33,198,143,159,4,138,162,224,112,56,176,113,227,58,76,155,54,19,20,69,97,249,242,37,176,90,173,114,27,182,
  88,44,184,242,202,223,226,227,143,63,20,203,179,230,98,218,217,51,0,136,171,47,142,31,63,9,213,213,123,252,109,188,0,43,86,44,195,229,151,255,
  6,130,32,192,236,95,20,76,210,25,0,234,234,108,88,185,114,121,208,250,14,117,127,169,68,245,101,93,220,167,90,44,22,220,126,251,221,200,207,47,
  64,67,131,3,175,190,250,18,238,184,227,110,108,216,176,86,206,116,242,234,171,111,224,227,143,63,84,181,191,127,254,243,37,84,87,87,17,150,28,2,
  180,224,111,64,233,240,129,230,55,226,208,113,118,70,135,152,206,243,191,178,103,104,26,0,37,255,7,196,152,217,130,252,124,244,234,213,11,160,69,18,
  234,15,150,245,255,166,208,216,216,136,190,125,251,194,106,237,47,191,254,167,41,10,151,94,114,25,206,57,231,2,24,141,70,152,204,102,208,12,35,150,
  75,81,129,186,248,183,65,241,95,42,31,26,125,140,38,19,142,63,238,120,80,52,5,80,0,77,51,1,58,53,52,249,117,202,237,15,138,162,229,212,
  111,151,93,254,27,220,112,253,141,200,240,235,19,77,221,241,0,120,255,132,61,121,155,255,191,188,61,196,36,63,213,113,218,243,131,148,167,189,87,218,
  123,38,61,68,104,169,222,105,26,130,0,156,116,242,16,244,239,223,31,125,251,245,19,23,164,161,41,52,53,53,33,187,111,95,228,88,115,65,211,20,
  178,179,251,162,185,165,25,109,206,54,236,251,105,31,190,253,246,107,8,20,64,51,44,238,158,123,31,30,120,224,225,168,235,40,25,237,86,72,83,125,
  114,172,185,40,43,27,131,28,107,46,38,78,156,140,252,130,66,220,112,195,77,176,88,44,40,40,40,196,244,25,179,48,113,226,100,56,157,78,216,236,
  181,16,0,148,149,141,129,205,94,11,155,159,76,231,231,23,34,191,160,80,62,127,226,196,201,170,50,242,243,11,97,181,138,19,186,234,29,14,220,112,
  195,205,176,217,107,81,92,92,130,11,47,188,84,222,15,0,119,220,113,183,124,222,140,25,179,144,95,80,136,233,211,103,67,0,48,125,250,108,216,236,
  181,152,62,125,54,74,203,198,4,232,107,181,230,226,134,27,110,82,149,231,104,168,135,224,39,207,197,197,37,126,34,125,9,44,22,11,166,79,159,141,
  233,211,103,195,234,175,131,9,19,68,121,249,254,73,95,210,117,234,93,99,81,81,9,236,117,54,216,236,181,152,54,109,150,188,36,249,249,231,95,132,
  188,188,2,12,26,148,143,243,206,187,8,67,134,12,1,207,243,56,247,156,243,67,218,174,143,227,58,103,187,254,109,124,140,237,128,135,38,7,188,114,
  159,86,95,101,223,228,255,112,60,31,88,190,255,156,115,207,189,16,67,134,12,65,173,173,6,191,255,253,31,209,47,199,138,81,163,202,228,54,212,234,
  116,138,147,240,172,185,176,90,115,225,104,168,15,104,147,202,54,88,107,19,219,97,94,94,129,234,62,151,150,142,198,25,103,76,68,158,159,60,15,29,
  90,140,113,103,76,192,233,167,143,71,94,94,62,78,63,253,12,180,182,182,194,102,175,5,199,243,24,49,226,84,244,237,151,3,30,192,111,175,249,61,
  50,50,76,48,153,140,184,230,183,191,199,192,65,121,56,249,228,33,168,181,213,226,119,191,251,3,6,229,229,99,200,144,34,127,249,53,104,115,58,145,
  231,95,37,80,42,255,250,235,111,132,197,98,129,217,108,198,245,215,223,168,178,5,201,206,250,229,88,33,0,184,254,186,63,202,111,144,175,187,238,143,
  200,201,201,65,94,126,1,70,142,44,195,212,169,211,49,40,47,31,35,70,140,196,169,167,142,68,91,91,27,108,246,90,248,56,14,35,70,156,138,90,
  91,13,166,76,57,27,167,143,27,175,91,223,218,54,163,220,47,4,233,247,187,107,31,166,188,222,241,19,38,33,35,35,3,183,221,126,35,234,235,143,
  96,252,248,73,224,121,62,160,253,74,223,127,249,191,63,227,203,133,243,113,193,5,151,132,174,55,242,33,121,160,187,218,99,174,250,86,140,130,41,154,
  198,157,119,254,9,143,254,249,255,144,157,221,87,38,103,25,102,51,178,179,251,34,211,146,137,126,253,250,129,130,159,184,82,226,55,69,1,63,255,252,
  19,134,12,25,226,159,124,38,202,95,248,213,23,88,253,221,10,209,139,73,211,1,229,6,211,77,149,217,129,166,113,251,109,119,225,225,135,30,67,118,
  118,95,217,147,156,95,80,128,126,253,114,96,52,24,253,185,146,37,93,196,223,20,40,252,252,243,79,24,58,100,168,156,75,57,199,106,197,55,223,124,
  137,207,62,255,88,36,218,225,188,63,202,14,73,49,170,87,118,128,64,248,12,25,218,15,52,15,72,40,60,202,202,135,98,40,47,2,165,121,173,124,
  226,137,39,225,79,119,223,143,230,99,199,80,107,171,193,73,39,158,140,86,255,132,151,130,252,2,127,253,80,56,112,224,103,156,124,210,73,138,9,130,
  20,114,115,251,227,162,139,46,197,37,151,92,134,76,75,38,138,134,22,225,237,119,222,192,171,255,124,37,100,187,33,72,150,55,39,19,235,55,172,197,
  178,101,75,84,219,171,170,58,114,206,74,11,145,68,3,201,139,184,108,217,18,148,87,108,193,196,137,147,253,158,226,122,85,198,4,155,77,204,177,44,
  145,222,139,46,188,20,85,85,123,176,108,217,18,84,85,237,65,153,34,151,110,65,65,33,150,45,95,130,101,203,23,7,100,35,80,234,107,177,88,96,
  179,215,226,63,111,191,129,245,27,214,162,160,160,16,5,5,133,168,170,222,227,207,172,145,41,103,70,144,206,147,190,173,214,220,128,44,11,130,32,200,
  117,96,54,155,97,179,215,226,157,119,223,242,135,122,241,16,4,1,38,83,6,108,246,90,252,247,189,183,67,218,174,207,231,139,201,118,245,222,104,68,
  227,88,9,102,247,218,254,70,79,111,40,62,28,199,201,191,181,199,157,126,250,25,40,175,216,138,21,43,150,129,231,121,12,29,90,36,239,179,217,106,
  1,65,192,250,13,235,80,95,127,4,14,71,61,170,170,42,33,8,130,124,239,181,80,222,83,73,191,170,170,74,180,181,181,193,108,182,32,111,80,62,
  120,158,71,125,253,97,244,235,155,131,65,131,242,80,91,91,3,158,231,68,66,197,243,168,174,174,130,207,39,254,31,52,112,16,142,63,110,32,86,172,
  92,142,21,43,151,227,248,227,143,7,207,243,224,56,30,245,245,245,224,184,142,243,148,247,92,121,141,146,215,123,217,242,37,226,202,151,5,133,1,111,
  15,148,11,14,229,231,23,96,249,138,165,88,182,124,9,242,242,242,81,107,171,193,160,129,121,24,50,100,8,90,91,91,48,100,200,80,212,218,106,192,
  113,188,172,183,164,179,195,225,0,199,249,39,105,7,105,47,186,109,70,217,118,144,220,69,187,226,25,146,162,119,173,0,48,228,228,34,148,151,111,21,
  61,205,255,124,25,11,191,90,32,223,51,249,77,9,47,14,84,165,111,241,55,31,113,25,61,55,132,131,160,235,121,180,210,211,171,104,152,175,190,250,
  18,26,27,27,196,198,170,124,101,79,1,130,192,227,196,19,78,194,9,39,156,0,80,64,159,62,217,48,176,44,104,154,134,205,86,139,83,78,25,9,
  3,203,160,79,118,31,148,148,12,199,202,149,203,66,18,191,160,58,105,200,225,63,255,245,138,24,86,162,58,150,130,197,98,6,107,96,49,100,200,80,
  252,244,211,94,128,162,144,97,202,0,237,247,148,219,237,54,140,61,109,28,140,6,3,142,53,31,67,81,81,137,42,151,113,168,87,96,66,152,87,166,
  170,223,122,217,84,34,121,69,71,81,160,164,178,148,175,252,162,238,40,40,28,168,57,128,221,187,119,226,180,211,198,161,221,237,198,169,35,71,193,104,
  48,130,23,120,12,26,148,135,45,91,127,68,110,142,21,187,118,110,199,176,146,17,112,185,218,208,220,220,140,83,78,25,137,234,234,74,28,168,249,25,
  231,157,123,17,142,30,109,82,45,84,0,69,38,14,130,228,227,63,111,191,129,223,92,117,13,126,115,213,53,50,225,45,40,40,132,197,98,145,201,75,
  176,197,41,66,189,94,213,166,133,83,162,184,120,24,138,139,135,97,217,242,197,242,43,121,167,179,13,197,69,195,0,136,171,20,134,67,85,85,37,28,
  14,7,46,242,123,152,11,242,11,97,179,213,194,225,168,199,151,254,215,232,23,93,120,41,10,242,11,241,191,143,62,128,195,31,82,81,85,85,137,25,
  211,103,203,225,27,122,3,3,201,35,175,180,187,124,127,248,192,151,11,231,35,35,35,3,231,159,119,17,6,13,204,3,207,139,15,222,175,190,249,18,
  70,163,17,179,102,157,139,227,143,27,136,249,11,62,13,106,187,62,159,15,188,63,164,44,26,219,149,250,76,149,29,251,223,172,133,179,159,96,33,27,
  106,210,167,222,31,140,80,72,3,0,232,244,165,28,199,129,23,120,209,99,232,39,46,28,207,33,63,79,12,71,168,174,174,82,245,107,242,74,129,121,
  5,42,89,5,5,133,98,238,120,127,90,55,65,16,96,177,88,144,159,87,160,34,178,131,6,229,193,110,183,161,182,182,22,253,250,229,192,100,202,128,
  163,65,36,194,199,31,63,16,70,163,209,79,70,59,116,246,250,188,242,111,41,131,144,207,231,195,37,23,207,193,134,13,235,192,11,60,56,142,195,160,
  129,121,16,4,1,213,213,85,114,153,90,61,69,210,170,83,167,66,32,73,3,0,206,95,39,131,6,229,129,227,120,56,28,14,156,116,226,16,84,87,
  87,226,184,1,199,227,184,227,58,116,246,250,188,184,224,130,139,209,220,210,140,77,155,54,234,214,183,160,19,234,38,104,156,68,209,180,147,84,33,206,
  225,192,243,60,90,219,90,84,199,74,131,31,185,174,253,255,57,142,199,35,127,254,63,80,20,133,127,189,254,143,136,202,238,201,49,228,105,153,198,46,
  30,49,71,122,158,134,104,70,129,145,54,120,65,167,131,212,245,200,40,61,163,188,0,151,219,141,157,187,118,224,162,139,46,5,0,172,223,176,14,253,
  7,12,64,209,208,34,236,220,181,3,45,173,205,24,93,58,26,71,234,143,160,79,239,222,120,236,177,39,193,178,44,190,254,250,75,108,217,242,99,84,
  198,169,92,194,90,224,53,250,248,61,8,188,32,192,237,118,163,182,230,0,46,191,252,42,80,160,176,126,195,90,28,127,252,64,20,23,21,97,231,206,
  109,168,63,114,4,163,71,143,65,77,237,1,8,130,128,39,158,120,26,12,195,224,171,175,22,224,199,31,127,136,169,211,8,168,51,237,131,45,212,160,
  193,255,224,149,59,88,169,51,208,235,88,67,212,141,178,142,196,216,121,30,229,229,91,48,121,210,153,24,123,218,56,64,0,94,252,251,179,48,153,76,
  184,229,230,59,208,187,87,111,228,88,173,160,25,6,173,173,205,24,121,234,40,252,240,227,38,176,12,141,123,239,121,0,205,45,205,48,26,69,194,13,
  0,79,61,245,44,24,134,193,151,95,206,199,143,63,110,10,58,200,137,100,57,239,206,180,221,174,240,166,116,245,131,169,227,27,248,195,13,55,97,222,
  255,222,199,204,25,179,101,162,188,181,124,11,150,45,91,44,123,214,150,46,91,12,155,173,22,127,252,195,205,170,174,168,227,91,125,77,54,91,45,222,
  250,207,191,81,86,54,70,246,216,90,204,22,5,137,221,35,190,210,246,47,215,45,157,83,92,60,76,246,80,23,23,15,131,32,8,176,90,115,229,165,
  143,5,1,114,12,107,89,233,104,49,236,194,86,43,151,39,225,234,223,92,139,122,71,61,150,46,91,140,153,51,102,99,222,255,222,151,203,150,202,219,
  90,190,25,23,95,52,71,183,94,108,54,49,116,96,198,244,89,50,65,126,251,157,55,33,8,2,174,186,242,183,56,226,56,130,101,203,151,224,236,179,
  166,163,186,186,18,60,207,99,206,37,151,161,222,81,143,21,43,151,97,202,228,169,248,230,219,133,16,32,198,248,106,109,215,171,72,251,24,181,237,42,
  22,57,146,42,133,82,132,167,69,67,76,132,32,100,57,92,170,50,101,218,74,45,161,59,114,228,48,50,140,25,200,48,153,192,113,34,121,225,57,30,
  203,150,47,69,93,157,93,85,134,156,126,21,2,150,45,95,226,191,231,226,62,169,13,42,81,91,91,227,95,161,178,0,103,77,157,134,242,138,45,56,
  239,220,11,192,243,28,234,29,71,48,252,148,17,48,26,141,176,217,106,225,245,249,80,177,109,43,214,172,89,141,129,3,7,193,231,243,130,227,56,52,
  54,54,160,185,185,25,189,251,244,1,5,10,205,205,205,240,249,56,248,124,94,60,249,212,95,0,0,83,167,78,3,199,115,120,247,191,255,81,172,187,
  32,58,120,222,126,231,77,88,44,22,76,59,123,38,114,114,252,113,220,78,23,120,94,255,141,131,195,81,143,182,182,54,244,235,39,198,99,183,181,182,
  162,170,170,18,83,38,159,5,187,189,6,118,187,29,99,79,27,135,13,27,215,193,231,243,162,98,91,57,214,174,245,235,236,245,98,213,170,229,152,52,
  233,76,24,77,38,152,205,102,52,53,54,170,234,27,65,250,119,65,59,15,40,138,118,210,85,253,88,52,28,163,178,106,15,70,141,44,197,242,229,75,
  113,243,77,183,161,174,206,14,142,227,228,55,13,82,59,21,183,249,240,215,191,61,142,134,6,71,196,19,250,123,50,145,78,59,15,180,207,231,67,245,
  222,42,52,190,247,54,154,154,154,208,222,222,142,218,218,26,24,12,6,52,53,53,161,181,181,21,155,54,109,64,117,117,37,154,155,155,229,156,145,27,
  54,172,3,203,178,104,104,104,16,27,93,229,158,136,23,192,136,215,67,90,137,71,31,125,48,96,210,154,0,224,133,23,158,145,73,226,138,21,75,177,
  98,133,122,185,211,157,59,119,128,2,240,195,15,223,227,199,31,190,7,69,81,216,182,173,92,110,224,202,142,68,208,153,64,24,84,55,65,192,255,253,
  223,159,3,61,195,130,128,23,255,254,156,252,224,91,177,114,25,86,40,189,221,20,133,157,59,183,3,20,133,77,155,54,224,135,31,54,130,162,40,84,
  148,111,13,88,101,79,251,192,136,138,56,235,212,101,48,111,180,42,247,119,144,235,215,75,61,24,84,166,162,142,190,250,250,75,121,251,75,47,63,15,
  1,192,15,155,54,66,0,224,118,187,241,210,203,207,3,128,60,136,217,188,249,7,108,222,242,35,40,0,75,150,45,198,210,101,139,85,117,82,190,117,
  75,208,251,22,234,254,17,116,14,54,91,45,170,170,246,224,174,59,239,129,211,217,134,245,235,215,32,215,154,139,187,238,188,7,0,240,214,127,254,141,
  226,226,97,152,57,99,54,102,206,152,13,155,38,119,237,186,245,107,241,199,63,220,140,226,226,97,88,170,32,55,47,190,240,15,172,95,191,6,95,124,
  57,95,117,124,121,249,22,140,46,27,131,39,159,120,70,150,63,201,31,198,177,181,124,11,46,190,104,14,150,46,91,12,167,211,137,90,155,24,135,92,
  94,190,5,75,151,45,70,89,217,24,188,248,194,63,224,116,182,97,233,178,197,152,56,113,138,76,230,175,254,205,181,0,128,121,255,123,31,163,203,198,
  160,184,120,24,94,124,225,31,112,56,234,241,244,51,79,162,222,81,143,153,51,102,203,199,235,161,220,95,126,164,120,238,217,151,32,8,2,150,46,95,
  140,233,103,207,0,199,241,88,190,98,41,6,13,26,4,129,23,112,164,190,30,147,39,159,9,158,227,177,106,213,10,156,127,222,69,224,121,30,159,126,
  246,81,128,77,121,189,94,8,254,135,124,180,182,171,103,35,145,216,76,36,121,251,35,29,184,123,245,60,232,126,18,247,237,162,175,113,245,213,215,98,
  236,216,211,81,85,181,7,123,247,86,201,94,65,61,189,229,241,185,70,191,25,211,103,97,198,244,89,88,191,126,13,190,92,184,0,130,32,160,168,168,
  4,207,60,253,34,204,102,51,218,218,218,80,81,177,21,231,204,62,15,60,47,122,137,103,76,159,133,150,214,22,216,237,54,248,188,62,76,24,63,9,
  103,140,155,136,131,7,237,104,247,120,112,245,213,191,67,83,83,35,86,127,183,18,231,204,62,31,20,128,85,171,87,130,162,40,149,135,154,231,57,112,
  62,31,30,255,191,191,129,166,105,188,252,202,11,254,176,14,65,126,99,178,108,185,184,218,36,69,81,114,27,19,4,1,207,63,247,50,108,182,26,8,
  130,128,219,111,155,139,250,122,113,176,117,249,101,87,65,16,120,44,95,177,12,118,187,13,173,173,45,168,181,213,250,51,81,141,193,222,125,213,240,249,
  124,24,127,198,68,156,49,110,60,126,249,165,14,30,143,7,251,246,237,197,168,145,165,152,112,198,68,140,30,61,22,235,55,172,195,134,13,107,3,222,
  236,106,251,247,88,219,73,170,121,157,131,61,251,54,110,92,135,17,167,140,192,243,207,189,4,71,131,3,159,124,250,17,134,15,63,5,179,102,158,131,
  89,51,207,193,162,197,223,192,235,149,222,46,120,117,219,123,176,231,97,36,207,237,116,6,117,234,169,167,166,141,11,90,138,201,5,196,12,16,193,26,
  153,94,199,36,173,202,199,113,28,56,142,195,169,35,70,162,178,106,143,106,117,172,112,40,43,43,67,89,89,25,254,243,159,255,196,230,61,213,243,54,
  107,227,253,52,228,21,90,175,140,226,91,187,16,135,238,82,222,58,198,16,16,47,150,68,125,34,93,97,48,168,7,56,152,46,129,141,37,168,110,136,
  80,31,221,178,19,92,71,209,172,62,168,196,61,247,220,131,23,95,124,49,232,254,63,252,225,15,40,47,47,71,121,121,121,151,219,113,36,118,148,44,
  104,67,23,10,10,10,225,112,136,171,238,89,173,185,242,10,104,14,135,56,217,75,121,172,52,1,80,242,232,74,94,106,233,124,189,21,248,148,242,149,
  251,67,233,161,213,211,106,205,149,203,84,234,32,77,40,148,100,234,29,3,64,85,174,36,203,106,205,133,211,217,166,210,69,239,184,252,252,2,185,221,
  219,237,54,244,235,151,35,203,54,155,205,16,4,1,46,151,11,217,125,251,66,16,4,52,53,54,194,148,145,1,248,183,107,109,228,170,171,174,194,71,
  31,125,20,149,205,68,98,199,225,28,7,145,14,212,195,13,220,175,185,246,90,124,240,254,251,65,109,213,108,177,32,167,95,14,126,249,229,32,0,32,
  47,47,31,7,15,214,169,244,54,155,205,160,40,74,174,123,187,221,166,186,47,82,54,10,101,187,201,201,177,66,16,4,185,125,56,28,245,232,215,47,
  7,78,103,27,92,46,23,6,14,28,4,167,211,137,198,198,6,244,237,219,15,166,12,19,32,0,77,77,141,200,206,238,11,1,226,27,197,163,254,201,
  206,20,40,28,61,218,4,0,232,219,183,159,252,155,162,40,12,26,148,39,215,253,193,131,117,114,155,82,122,49,67,217,130,228,157,150,218,164,164,123,
  67,131,67,206,182,33,93,71,118,118,95,89,199,140,140,12,177,13,29,109,66,118,118,54,14,253,250,43,50,50,50,252,111,114,44,178,142,136,160,127,
  79,68,78,253,68,244,169,225,200,115,184,55,37,209,242,169,80,117,66,133,24,192,246,24,15,116,186,133,112,72,193,239,62,159,15,52,77,131,162,40,
  121,84,79,211,52,24,134,145,95,171,177,44,11,134,97,32,8,2,142,59,238,120,148,148,12,67,117,117,21,14,30,172,131,53,55,23,204,62,38,174,
  33,28,145,122,80,131,18,105,29,34,166,138,239,149,114,54,43,190,101,207,41,208,241,95,51,226,22,148,222,156,32,58,38,75,159,104,188,226,145,16,
  85,65,199,243,16,201,67,88,149,235,89,147,6,73,79,126,50,234,72,57,0,212,171,175,112,117,71,66,56,162,71,109,109,77,208,255,245,245,71,84,
  251,2,87,10,60,18,82,150,222,10,109,202,99,84,171,250,133,208,35,148,94,202,223,193,87,161,11,126,29,210,62,229,49,82,89,218,227,4,65,80,
  233,33,189,150,151,238,99,91,91,155,108,7,13,14,135,188,221,169,216,174,181,173,246,246,118,113,34,94,20,182,11,133,29,69,227,137,142,134,60,43,
  237,60,148,39,218,211,222,222,17,194,161,179,96,146,179,77,28,148,72,215,97,183,219,2,244,86,14,176,106,107,107,64,81,148,238,125,81,214,115,91,
  91,91,128,110,202,208,28,123,157,93,214,95,34,170,146,254,218,54,210,232,127,43,43,233,175,36,198,20,69,193,94,103,15,184,15,245,245,71,84,215,
  17,202,22,148,43,10,42,175,73,183,221,52,136,147,97,165,111,105,187,211,47,67,170,43,167,211,25,124,129,170,16,237,35,158,158,232,120,247,99,33,
  223,126,70,73,162,131,197,229,7,243,208,107,223,194,70,186,8,92,218,19,232,158,118,193,20,69,193,100,50,193,100,50,33,51,51,11,78,167,216,217,
  156,53,117,26,178,178,178,176,111,223,222,174,39,14,138,142,33,24,161,86,18,49,153,108,73,157,183,222,226,40,240,167,91,211,188,182,210,203,29,172,
  125,136,5,243,176,38,66,31,165,94,138,138,9,78,234,117,116,11,184,6,101,153,210,53,43,227,224,244,38,141,40,117,131,126,54,144,164,213,145,162,
  44,18,178,65,208,93,28,25,33,7,187,17,218,238,39,159,126,26,181,237,42,237,53,82,114,20,9,121,214,213,53,204,192,125,222,188,121,29,118,171,
  88,148,74,165,179,210,254,21,147,197,227,77,254,99,209,95,69,174,226,164,127,196,237,37,69,244,77,213,208,132,104,28,114,145,120,162,35,117,198,132,
  115,220,244,164,80,142,30,65,160,37,210,204,178,44,120,158,71,73,201,112,156,125,214,116,52,55,31,195,194,133,11,116,189,50,201,110,236,1,139,116,
  4,73,221,228,114,185,224,118,187,225,241,120,224,245,122,101,239,12,195,48,48,24,12,48,26,141,200,200,200,16,95,235,233,53,98,13,105,13,150,229,
  34,128,208,7,209,169,51,250,132,34,209,136,228,129,16,132,172,58,157,78,184,219,219,69,157,60,30,181,78,70,163,168,147,201,4,139,197,162,235,229,
  213,235,44,208,133,117,36,223,55,64,55,110,47,210,142,143,128,160,203,250,55,29,175,92,34,109,87,158,4,22,1,57,74,20,249,143,101,224,158,48,
  242,159,98,250,119,103,125,187,218,190,66,217,91,184,164,4,225,60,208,225,158,47,193,72,116,143,245,64,167,99,22,14,169,241,72,33,26,57,57,57,
  24,49,98,36,40,138,70,69,249,22,12,43,25,142,227,143,63,30,205,205,199,162,110,184,209,140,162,99,105,240,193,62,210,107,185,96,49,217,62,255,
  18,219,46,151,11,199,142,29,131,197,98,65,102,102,38,50,51,51,3,226,105,181,161,2,193,188,25,65,189,170,113,212,71,213,177,233,44,145,29,214,
  251,28,139,78,78,39,142,1,33,117,82,205,236,79,129,58,210,78,184,140,165,243,10,215,150,83,41,132,35,90,219,35,72,77,2,29,247,254,45,66,
  219,85,245,45,81,216,138,150,152,133,27,176,43,7,205,209,12,148,229,80,2,205,252,6,61,27,23,20,164,48,98,242,159,162,250,43,251,82,189,180,
  128,169,90,223,177,146,197,120,245,169,177,14,146,130,145,233,112,14,71,189,216,112,221,186,234,102,94,123,226,129,142,0,210,74,70,199,31,63,16,22,
  75,38,206,153,125,62,6,12,24,128,109,219,202,81,17,65,222,84,64,156,253,157,44,114,16,234,193,226,245,122,209,220,220,140,150,150,150,168,12,209,
  233,116,194,229,114,161,189,189,29,189,123,247,134,193,96,208,245,252,6,243,176,6,35,134,137,208,39,152,55,26,65,226,11,133,36,233,132,20,171,35,
  109,39,167,215,145,17,16,164,218,224,39,217,253,91,92,108,37,73,206,13,4,121,243,165,242,242,73,125,81,116,55,34,165,244,23,20,241,200,130,206,
  226,52,221,190,190,187,208,198,34,25,168,70,234,152,208,218,135,94,150,172,96,30,234,158,248,12,74,59,2,205,48,12,74,71,149,97,206,156,43,81,
  89,185,27,13,13,14,121,102,110,36,144,210,218,109,218,180,17,30,143,39,238,222,25,93,2,134,192,101,69,5,65,128,187,189,29,199,142,30,141,42,
  19,136,86,190,152,187,211,135,62,217,217,200,48,153,162,27,57,107,200,97,162,245,145,99,213,116,8,107,176,14,162,71,213,145,98,33,21,229,235,105,
  61,25,132,76,19,116,133,247,57,101,250,183,48,182,18,42,142,184,187,146,255,84,214,63,84,218,65,116,247,193,86,23,216,87,36,228,89,154,188,26,
  9,145,14,230,113,14,231,169,238,233,207,154,180,11,225,160,40,10,125,250,100,203,169,125,194,129,215,92,191,20,214,225,114,185,192,113,92,212,175,95,
  244,94,217,68,245,90,205,191,238,188,215,227,233,212,195,69,219,129,0,0,221,183,47,12,6,67,216,215,155,122,250,121,189,222,132,235,163,204,128,161,
  244,0,7,235,92,147,161,83,74,212,145,209,8,26,0,120,30,160,40,240,10,221,84,157,61,130,175,158,21,233,2,63,169,210,31,144,229,98,187,161,
  215,57,21,250,183,8,109,37,224,26,20,68,46,24,41,225,5,1,130,63,163,83,123,123,59,142,29,59,22,31,242,223,167,15,76,38,147,216,231,208,
  180,168,127,12,3,98,33,197,245,143,100,208,146,202,245,29,45,89,140,87,159,26,75,120,148,207,231,67,123,123,123,76,169,236,180,217,56,24,134,129,
  209,104,148,179,154,133,12,127,233,97,243,113,122,92,22,14,142,231,81,107,171,193,129,154,253,216,181,107,71,100,203,73,39,226,161,163,243,173,52,128,
  230,230,230,184,60,92,148,15,25,134,97,208,175,95,63,173,181,4,78,216,211,120,85,19,174,143,206,82,170,202,204,32,234,85,15,59,234,170,199,212,
  81,78,78,199,68,79,205,172,242,238,178,228,44,65,15,35,213,93,213,191,69,97,43,130,142,23,52,148,247,86,234,131,188,94,111,167,200,156,30,249,
  239,235,39,255,148,206,164,110,42,130,129,178,16,132,108,165,138,254,193,226,138,181,186,119,151,250,78,133,1,107,40,242,220,210,210,130,139,46,58,31,
  199,29,55,0,135,14,29,146,175,69,250,72,235,101,40,174,26,128,160,90,37,18,254,92,224,191,254,250,43,118,236,216,131,94,189,122,233,156,23,72,
  186,123,18,88,164,165,151,39,240,154,54,111,254,1,171,86,45,71,243,177,163,216,178,249,7,120,189,94,57,51,71,144,86,218,241,137,188,101,171,191,
  131,109,87,76,132,208,126,75,75,152,74,161,36,241,68,107,107,171,63,125,95,102,71,195,151,58,16,77,186,52,229,3,175,173,173,45,225,250,104,39,
  56,10,26,226,42,104,234,45,25,58,165,82,29,101,101,101,137,186,240,188,88,71,82,93,41,99,250,66,189,38,213,107,151,193,218,124,215,63,33,34,
  211,151,32,85,158,232,41,213,191,69,100,43,74,123,209,89,241,85,79,111,201,27,154,200,129,187,64,81,128,159,164,4,37,255,122,118,174,168,251,84,
  211,95,234,67,17,108,53,215,96,89,140,82,165,190,59,107,27,157,237,199,244,158,129,218,250,242,215,21,207,243,104,109,109,197,133,23,158,135,185,115,
  239,192,207,63,255,132,163,71,27,65,81,0,203,50,160,105,70,246,36,211,52,165,34,207,146,170,226,74,146,60,56,142,67,123,123,59,126,253,181,14,
  117,117,117,112,56,142,34,187,79,31,241,62,210,116,224,32,78,235,113,214,113,60,165,29,129,238,25,244,25,24,59,246,116,8,130,128,47,23,206,199,
  160,65,121,104,108,108,192,209,163,71,67,202,16,130,200,138,212,235,18,48,90,212,122,100,180,31,5,25,75,196,235,107,73,182,180,130,152,164,7,21,
  38,219,69,50,244,225,253,102,172,154,112,18,98,242,96,79,171,35,137,208,11,193,226,31,181,222,53,189,215,165,17,180,95,33,133,236,151,208,231,238,
  192,157,83,175,127,11,107,43,74,219,208,121,179,20,52,244,4,72,252,192,61,43,11,180,180,76,182,223,166,165,239,80,131,99,33,133,245,151,200,84,
  56,207,121,202,214,119,56,199,68,130,251,84,237,219,5,221,123,12,49,28,213,199,113,232,213,43,11,215,95,127,13,126,252,241,123,236,223,191,23,110,
  183,11,6,131,1,44,203,202,153,201,24,134,241,19,232,142,176,73,145,128,11,224,121,31,124,62,14,94,47,7,175,215,139,214,214,86,140,28,89,132,
  213,171,55,131,227,121,208,52,13,218,127,127,40,229,71,111,80,151,230,158,105,186,39,117,246,70,163,17,38,163,9,163,70,150,225,222,123,30,196,244,
  233,51,65,211,76,215,120,107,52,29,186,100,28,46,151,11,46,151,43,97,197,75,242,245,146,211,11,58,196,48,89,250,168,58,11,189,180,69,202,115,
  220,238,30,87,71,146,7,38,104,108,48,241,214,18,164,6,163,238,242,254,45,172,173,68,48,24,208,221,151,4,242,143,112,115,21,244,6,46,58,139,
  52,165,164,254,81,244,91,169,86,223,145,180,145,174,28,196,42,219,186,199,227,193,254,253,63,161,174,174,22,77,77,141,240,249,188,224,121,31,0,30,
  20,37,248,63,240,123,161,225,255,72,225,29,210,45,17,32,8,156,255,35,192,229,242,64,16,248,136,226,186,123,210,220,149,244,11,225,16,132,136,135,
  124,199,13,56,30,140,38,166,199,100,52,193,233,114,194,100,50,193,237,118,131,243,249,162,44,91,8,120,37,168,221,175,29,65,42,95,121,186,221,238,
  132,87,145,211,233,132,209,104,4,203,48,234,213,240,116,72,108,123,123,123,82,244,49,153,76,96,24,166,99,73,107,133,78,170,24,104,65,64,123,15,
  173,163,140,140,12,176,12,163,219,73,81,145,182,205,104,218,110,151,218,176,64,6,5,221,141,48,167,80,255,22,212,86,66,77,108,210,132,160,64,115,
  13,201,34,255,153,22,75,71,206,100,61,29,67,132,111,164,154,254,218,62,74,233,133,214,246,235,66,119,169,239,100,247,99,202,240,28,233,153,200,243,
  98,189,241,188,58,92,199,31,202,225,118,187,192,243,28,76,38,35,50,50,76,48,153,50,96,52,26,97,52,26,100,47,52,77,51,50,145,22,79,23,
  67,64,188,94,49,13,160,193,224,129,199,227,129,207,199,129,162,104,8,2,192,249,124,160,0,208,12,35,150,237,159,136,169,36,207,61,41,30,58,237,
  66,56,60,94,47,190,91,179,10,117,7,237,200,206,238,139,172,204,172,168,206,207,177,90,225,180,219,48,106,84,41,182,109,171,136,234,245,145,94,216,
  135,160,183,95,103,9,104,105,123,162,242,79,43,225,245,122,225,243,249,58,38,19,232,204,140,150,244,142,87,42,191,112,250,120,189,222,32,147,27,2,
  95,243,37,75,167,84,173,35,134,97,116,151,100,143,164,109,70,211,118,187,140,147,165,144,46,4,225,239,85,42,246,111,65,109,37,92,187,83,188,206,
  87,18,59,30,72,26,249,55,101,100,128,245,219,51,239,119,38,208,17,120,250,148,97,28,169,162,191,42,14,86,250,173,157,68,168,121,227,152,170,245,
  221,85,253,152,110,42,72,197,55,175,200,90,34,101,21,99,24,90,94,76,198,104,52,194,100,50,193,100,18,127,27,12,6,5,129,86,135,112,112,28,
  15,131,193,7,143,199,7,138,18,99,165,141,70,15,12,6,22,20,5,120,125,62,80,52,13,138,166,229,16,41,222,159,59,91,55,140,35,221,9,116,
  58,94,84,107,107,43,118,238,220,129,140,140,12,48,12,131,163,71,155,80,50,108,120,84,45,57,59,59,59,174,25,58,66,45,197,169,52,146,100,60,
  96,56,142,147,201,161,209,104,12,244,10,40,126,251,162,241,192,119,70,31,142,3,229,243,193,100,52,6,142,250,53,117,149,52,157,82,173,142,252,157,
  23,195,48,65,71,252,36,7,52,65,151,16,233,84,236,223,194,216,74,88,175,159,198,115,157,52,231,134,215,11,90,67,254,35,93,81,81,208,122,161,
  83,65,255,32,169,235,116,235,188,155,213,119,170,216,157,186,78,197,107,145,98,158,149,36,218,104,52,249,227,161,3,9,180,248,108,229,224,243,121,65,
  81,29,67,8,150,117,203,118,36,13,130,25,134,33,207,26,164,109,8,135,0,159,215,11,183,255,6,255,184,249,7,108,219,86,46,38,80,167,40,28,
  60,88,135,178,178,209,17,12,31,227,144,133,35,196,235,181,128,255,130,32,47,71,154,72,72,9,214,125,62,31,104,154,6,203,178,186,41,157,146,170,
  143,255,158,49,20,5,214,96,232,88,193,74,217,81,244,244,58,2,224,243,122,225,163,105,185,45,199,212,62,35,37,13,93,105,195,145,232,75,144,122,
  247,42,149,250,183,16,182,162,155,210,78,59,223,66,147,69,36,217,206,13,57,188,80,185,66,94,4,246,155,114,250,71,58,88,73,225,250,142,153,44,
  198,57,132,35,212,71,240,183,123,241,18,24,255,146,230,172,202,19,45,121,160,89,150,241,147,98,26,52,13,57,132,131,166,57,255,91,96,202,63,161,
  144,131,209,200,250,157,137,148,238,189,234,201,20,58,237,66,56,180,51,191,1,160,189,189,29,110,183,27,44,203,194,100,50,225,251,77,27,80,89,181,
  27,199,29,119,60,124,33,58,244,184,100,225,208,249,22,130,232,41,36,185,158,164,7,154,214,211,222,85,175,208,5,0,62,158,7,3,117,10,36,33,
  198,251,145,110,117,36,233,227,227,56,176,33,86,211,10,215,62,227,213,222,19,122,157,32,33,28,221,165,191,77,229,254,45,82,91,9,200,98,161,163,
  123,178,200,191,244,86,203,203,48,29,171,230,69,17,166,149,146,250,71,249,204,238,14,245,29,13,31,137,151,141,133,253,248,201,54,77,211,160,105,26,
  12,195,130,97,88,191,55,218,0,131,129,149,227,160,59,60,208,180,92,23,52,205,201,30,105,142,227,193,113,188,255,88,90,255,94,249,231,44,245,88,
  2,157,142,23,149,145,145,1,147,201,164,90,33,78,154,140,32,101,77,104,109,109,133,195,225,144,27,198,206,93,219,113,202,240,83,147,167,100,16,239,
  33,195,48,9,15,9,80,198,25,251,56,14,44,199,169,94,117,66,225,157,73,182,62,156,207,7,142,101,229,9,64,122,232,233,117,228,243,249,68,82,
  175,19,155,77,64,144,18,72,149,254,173,155,218,138,224,31,184,27,98,24,40,19,253,123,70,125,135,202,136,33,166,106,102,252,41,235,104,255,135,145,
  9,181,248,233,32,208,162,135,189,131,72,139,164,155,3,203,250,64,81,76,64,152,32,129,159,64,167,91,101,176,44,139,137,19,39,97,204,232,211,176,
  111,223,94,28,59,118,12,160,4,172,91,183,22,94,175,87,181,78,124,123,123,187,188,68,229,246,237,219,80,89,185,71,38,73,109,109,78,248,124,190,
  78,47,229,173,183,106,144,60,194,213,228,117,148,244,79,198,3,70,153,66,157,227,56,208,18,57,212,164,109,75,138,62,26,98,202,115,28,4,233,129,
  167,147,162,169,39,214,145,76,222,253,29,153,202,43,174,136,97,83,45,133,30,164,195,141,182,51,238,170,135,3,233,168,187,207,67,60,149,250,183,80,
  182,34,248,251,20,10,250,41,202,148,225,15,170,236,16,93,68,254,197,108,8,6,57,143,50,194,216,112,42,234,207,26,12,106,189,181,11,82,105,38,
  154,166,100,125,199,176,40,72,188,151,242,14,200,84,18,100,159,152,146,142,150,201,177,228,141,22,61,210,234,255,29,49,208,20,40,138,135,32,208,254,
  9,134,156,230,56,90,46,71,57,233,48,212,243,166,39,32,237,242,64,83,20,5,147,201,140,147,79,30,138,1,3,142,67,78,78,14,76,38,115,216,
  248,37,143,199,163,90,241,232,199,205,155,146,50,11,88,34,104,82,167,98,48,26,147,50,200,80,118,100,130,98,22,45,148,179,167,129,164,140,200,37,
  111,179,116,135,120,229,114,222,126,61,148,119,47,41,58,165,88,29,49,12,163,42,159,39,228,146,160,187,244,201,73,238,223,66,217,10,21,163,222,148,
  178,79,72,34,249,231,149,113,173,129,15,187,0,175,58,149,130,250,83,26,157,3,250,81,32,96,91,170,213,119,119,124,207,39,122,161,253,43,112,130,
  86,45,227,45,17,108,113,123,199,71,138,127,86,110,147,242,67,19,232,240,132,180,154,168,35,143,18,3,124,11,193,151,215,14,242,223,237,118,195,231,
  245,198,111,18,161,255,55,5,245,132,56,233,35,45,23,157,97,50,161,37,73,228,80,90,94,85,53,249,192,175,7,229,215,219,100,50,161,165,37,177,
  26,25,180,241,197,60,47,151,175,151,249,34,25,58,165,92,29,41,98,243,40,0,2,199,169,218,22,21,206,67,18,201,100,22,50,137,144,32,214,123,
  149,66,253,91,56,91,9,234,77,212,209,159,82,120,117,13,6,67,194,157,42,82,46,124,153,252,251,39,88,135,212,93,39,23,119,202,233,31,162,206,
  149,122,119,139,250,142,214,62,18,57,137,48,162,50,40,57,223,179,232,51,149,8,177,68,174,41,80,148,0,65,16,127,119,120,229,5,197,121,84,112,
  125,122,50,129,78,167,203,215,123,45,7,29,2,22,201,127,105,91,188,38,17,106,247,11,232,88,106,86,25,144,111,50,153,144,97,54,195,157,160,4,
  242,210,44,92,160,99,201,108,94,175,195,240,127,155,76,38,100,100,100,36,172,35,51,26,141,96,53,94,41,222,159,160,189,195,94,213,250,153,50,50,
  18,174,83,170,213,145,193,96,16,219,136,34,247,102,52,147,55,34,105,203,100,18,33,65,172,247,42,213,250,183,80,182,18,236,57,33,133,119,232,77,
  106,3,0,83,70,70,226,7,238,6,131,172,43,133,142,73,110,122,186,7,75,171,153,138,250,83,138,190,84,8,83,231,169,90,223,177,218,71,34,39,
  17,202,251,131,16,125,45,207,22,87,24,20,20,135,170,243,209,116,132,102,8,138,243,245,151,187,7,77,247,248,62,186,71,45,229,205,178,172,156,92,
  223,104,52,202,255,165,125,202,255,52,29,159,170,9,22,58,66,73,33,10,208,172,37,79,81,176,152,205,9,203,175,104,52,26,101,217,218,87,107,242,
  235,51,255,111,233,149,143,197,98,73,172,62,138,135,157,84,110,64,29,74,250,248,245,78,184,78,169,84,71,38,147,170,157,40,59,247,80,237,139,128,
  32,225,78,139,84,235,223,98,176,21,104,8,169,170,31,82,216,124,134,159,252,39,10,70,163,17,70,201,123,174,92,108,68,71,111,229,181,232,94,87,
  42,233,175,87,191,58,219,82,185,190,187,147,13,118,172,42,40,14,8,120,158,7,199,9,254,180,116,98,106,58,142,227,229,125,226,127,65,181,77,202,
  192,193,243,188,127,9,111,141,109,235,60,163,123,226,51,39,189,66,56,130,92,75,223,236,190,40,29,85,138,99,199,142,225,215,67,191,162,127,110,127,
  244,233,211,7,173,173,173,248,245,208,175,240,120,60,56,97,240,9,96,89,22,135,14,31,130,203,229,194,160,129,131,80,119,176,46,186,101,163,131,45,
  221,169,8,71,16,244,94,115,106,58,20,139,217,12,143,199,19,213,42,136,145,32,35,67,92,206,83,219,113,209,52,45,135,35,200,99,82,197,171,52,
  115,2,245,49,73,250,72,113,207,130,32,174,116,164,25,241,202,97,19,254,109,61,170,142,148,49,214,254,137,27,180,194,227,32,133,111,68,20,198,17,
  109,219,237,42,59,38,175,7,187,93,191,155,10,253,91,56,91,129,214,110,20,158,55,249,28,255,183,224,15,41,80,78,134,180,152,205,104,119,187,19,
  50,113,202,168,232,11,85,241,192,209,216,118,42,234,31,65,223,35,183,157,20,175,111,42,154,144,142,4,44,229,13,69,222,101,173,141,41,7,141,82,
  162,132,14,66,204,129,227,124,224,56,41,37,29,7,64,144,99,158,37,210,45,30,199,193,231,147,72,52,39,147,104,149,205,72,229,74,3,229,78,76,
  182,236,214,4,58,157,46,53,216,181,148,150,142,193,153,103,158,141,109,219,202,177,112,225,2,76,152,48,25,101,101,163,81,93,93,133,133,11,23,160,
  161,177,1,147,39,79,69,86,86,22,22,45,254,6,118,187,13,195,135,143,128,163,193,17,213,107,249,96,185,129,131,230,133,214,204,70,86,122,15,50,
  51,51,193,113,156,156,126,47,30,157,69,70,70,70,199,72,91,233,33,210,166,120,82,198,172,249,143,75,148,62,242,107,61,197,235,61,70,122,53,164,
  243,224,83,190,138,237,41,117,164,10,103,145,58,121,191,62,66,28,236,35,84,219,237,42,59,38,75,121,119,207,126,55,85,250,183,96,182,34,245,49,
  218,101,189,5,133,110,202,172,16,80,132,161,36,122,160,44,17,58,101,72,129,82,79,233,183,238,146,228,82,54,132,20,213,95,249,173,230,134,106,157,
  187,77,125,39,177,31,211,218,84,64,93,41,159,145,138,109,60,207,193,231,227,224,245,250,96,52,138,25,70,60,30,15,4,1,96,89,1,28,215,145,
  137,67,10,221,16,87,34,244,249,63,94,49,63,182,151,11,204,201,173,44,75,51,64,238,73,253,54,11,130,164,130,82,44,17,170,108,248,218,151,31,
  6,131,1,189,122,245,146,211,237,117,246,225,98,54,155,65,251,103,167,107,203,99,24,70,245,176,163,52,68,81,160,168,164,233,35,133,216,80,154,76,
  23,130,194,147,33,17,219,158,82,71,202,145,191,50,247,180,106,59,9,217,32,32,253,91,68,182,18,74,111,149,167,81,65,78,168,36,144,127,149,222,
  138,80,194,88,108,187,187,233,223,221,235,187,171,109,78,170,59,169,29,243,188,224,39,207,30,24,12,44,60,30,175,34,21,157,180,104,138,50,141,29,
  32,8,162,215,217,235,245,194,235,245,193,235,245,194,227,17,73,180,148,157,36,146,80,141,158,244,44,74,207,16,142,120,93,83,60,150,242,70,199,171,
  77,189,227,100,82,168,56,86,54,116,131,1,125,122,247,70,115,75,75,204,19,212,50,252,19,238,130,145,85,64,244,248,82,58,19,5,180,97,20,73,
  213,71,171,75,16,253,122,66,29,105,73,178,188,236,108,144,182,73,5,155,169,79,178,112,16,36,226,1,158,66,253,91,36,182,18,242,53,188,54,4,
  5,129,177,187,6,150,69,175,172,172,228,144,127,191,238,148,102,226,114,40,219,73,69,253,131,213,187,170,237,164,96,125,83,65,194,127,18,198,33,66,
  244,135,1,25,98,16,232,253,149,192,113,62,217,147,236,245,122,193,48,30,249,90,121,94,12,213,16,87,34,68,0,129,22,207,17,189,213,94,175,199,
  255,237,11,184,111,218,79,79,236,175,123,68,8,71,103,228,117,54,11,135,188,79,99,136,202,87,87,170,5,67,52,163,55,131,209,136,94,189,122,129,
  101,89,180,123,60,240,122,60,17,233,98,48,26,97,50,26,85,19,226,244,136,33,203,178,1,43,236,201,9,228,117,102,174,39,68,31,133,94,44,203,
  130,102,24,253,87,148,202,14,69,161,91,58,215,81,128,135,153,162,192,248,235,8,122,175,24,131,140,254,73,22,14,130,132,222,179,20,232,223,162,182,
  149,112,109,79,39,108,76,89,126,239,222,189,209,146,96,242,175,237,11,195,189,34,79,85,253,67,214,187,166,157,164,82,125,35,138,186,79,84,159,170,
  151,101,5,122,11,152,248,189,195,28,199,193,225,56,140,230,230,99,240,122,61,114,222,103,105,121,110,159,207,167,89,202,187,35,132,67,140,127,246,193,
  231,227,224,241,180,195,237,118,163,185,185,5,63,252,80,17,144,13,75,208,113,220,8,33,254,167,37,129,78,59,15,116,60,175,39,22,121,161,206,9,
  145,31,26,138,81,156,202,179,73,81,96,253,89,67,140,70,35,188,30,15,188,254,229,105,57,142,83,173,64,39,46,213,201,192,192,178,242,130,5,202,
  153,240,1,65,255,126,217,1,29,133,98,18,135,94,242,254,184,235,163,120,240,178,146,23,88,167,19,209,78,36,68,15,169,35,165,247,137,18,132,14,
  125,98,105,155,221,197,235,75,38,17,118,207,254,183,139,251,183,206,218,138,118,34,100,72,15,186,209,136,94,89,89,96,25,70,36,255,94,111,100,228,
  223,96,16,201,191,201,20,160,183,82,127,134,97,58,222,198,69,50,145,48,197,245,87,13,110,228,166,17,56,1,53,37,235,59,86,239,115,60,250,212,
  16,185,214,41,29,18,235,243,121,241,206,59,31,99,236,216,17,240,249,56,24,12,44,12,6,49,235,152,193,96,240,175,52,168,92,52,69,28,26,40,
  39,18,74,222,107,151,203,133,213,171,191,71,123,187,71,174,27,226,125,78,99,15,180,222,136,111,229,170,101,216,184,113,189,60,35,117,254,130,79,241,
  197,151,159,203,255,105,154,198,39,159,254,15,64,71,108,208,234,239,86,194,237,118,199,119,196,169,137,11,20,66,76,144,83,118,136,12,203,130,229,197,
  145,163,193,104,132,193,104,84,123,99,21,158,199,128,180,107,10,15,17,165,249,205,178,44,24,189,213,158,148,203,152,42,95,29,38,90,31,131,1,12,
  203,170,189,169,33,242,47,39,69,167,20,168,35,229,228,27,214,96,232,88,150,24,129,147,94,66,121,110,35,246,92,165,144,45,19,116,35,164,64,255,
  22,169,173,168,188,100,58,161,89,122,158,81,109,153,240,235,110,52,153,96,48,153,224,245,120,224,11,67,254,89,63,249,167,130,232,173,210,159,101,131,
  78,20,254,127,246,206,60,46,234,106,109,224,223,223,50,51,50,131,96,200,140,2,2,131,44,137,32,42,155,118,5,209,22,115,203,50,181,50,189,105,
  203,251,118,187,213,125,203,222,180,212,238,91,125,178,155,237,183,123,179,91,105,165,101,123,154,150,107,106,41,106,130,129,154,48,44,46,108,154,11,152,
  150,194,144,9,243,254,49,195,56,32,40,200,210,140,158,239,231,51,31,152,245,247,156,231,119,206,115,158,115,206,115,158,211,148,35,234,206,242,159,119,
  243,163,187,235,251,34,103,159,219,196,166,54,165,35,56,103,67,161,4,120,121,233,169,168,56,206,134,13,219,168,173,181,53,200,160,113,238,169,130,82,
  35,41,7,235,114,70,219,211,222,213,214,79,49,232,242,104,171,13,151,158,59,3,125,153,112,234,212,41,172,86,43,178,44,59,210,178,216,156,199,90,
  214,57,206,64,189,231,237,125,198,251,57,27,109,26,113,198,234,102,107,234,14,246,168,169,169,169,31,19,124,110,107,104,116,84,218,240,253,186,209,104,
  195,239,216,92,70,185,184,108,8,170,55,19,220,134,242,184,26,47,141,195,128,53,214,73,218,92,102,5,26,139,61,190,108,116,228,146,166,75,108,28,
  20,184,183,63,221,241,246,173,185,109,165,225,119,26,110,32,116,117,198,108,46,171,82,182,6,179,222,146,170,82,235,112,254,235,102,209,155,235,252,75,
  141,252,95,39,151,198,197,249,111,105,91,247,52,249,221,93,94,119,182,179,13,55,16,214,161,215,235,29,142,111,45,103,206,212,54,226,32,219,206,59,
  40,171,23,214,226,240,137,92,55,16,186,254,189,156,251,161,203,34,15,180,243,237,218,90,106,92,142,100,182,217,108,245,158,3,231,60,191,168,235,159,
  71,142,134,27,110,234,45,93,113,238,242,219,220,185,47,59,242,56,158,161,172,172,140,67,135,14,145,177,109,43,251,247,239,101,220,184,91,200,222,145,
  69,124,255,4,10,10,10,200,201,217,197,240,17,163,217,188,233,91,70,141,26,131,228,88,166,217,183,111,47,153,153,219,156,75,45,206,165,156,186,195,
  98,154,72,118,223,152,92,237,37,143,86,163,177,55,212,166,54,198,185,58,167,142,207,140,159,112,27,3,146,7,114,166,166,134,194,194,124,230,207,127,
  147,1,201,3,233,211,167,47,199,126,62,198,238,31,119,113,224,96,25,195,175,31,233,252,157,236,29,89,244,8,14,166,123,183,0,100,73,66,146,101,
  86,172,88,78,106,74,26,59,119,237,224,240,161,159,90,173,35,128,136,136,40,166,222,121,55,167,78,157,226,64,89,25,150,252,92,76,198,110,172,94,
  189,2,189,193,192,224,148,52,187,174,226,19,201,200,248,158,19,199,127,38,109,200,53,20,228,231,145,60,96,32,178,195,112,101,100,110,99,79,97,193,
  89,29,53,145,243,89,28,229,45,248,163,25,50,228,106,166,76,185,219,25,79,185,112,225,2,226,227,19,185,178,87,52,167,78,158,226,213,87,95,196,
  75,239,197,212,41,119,163,211,233,216,188,37,157,207,63,255,4,128,7,31,120,136,172,172,237,108,220,244,29,67,135,92,67,89,89,41,167,127,63,77,
  124,255,4,0,114,114,118,115,240,96,25,195,71,140,102,83,250,119,28,61,114,132,136,240,72,122,199,196,242,213,242,165,196,198,198,17,16,24,72,229,
  169,74,82,83,7,179,105,211,119,228,228,252,200,172,153,127,199,108,238,201,172,89,143,50,119,238,43,206,14,255,187,141,27,88,248,222,130,11,218,229,
  58,38,222,58,137,129,3,175,226,204,153,26,74,75,139,120,243,205,121,220,247,215,191,225,235,235,75,85,101,37,185,185,185,20,20,230,163,215,235,217,
  178,37,29,131,193,192,181,215,12,115,126,191,164,164,132,157,187,178,137,141,141,227,204,239,191,83,80,144,135,119,231,206,12,250,83,42,93,253,253,249,
  229,196,9,214,111,248,134,209,163,111,36,188,103,4,225,225,225,88,173,86,158,120,226,177,102,181,117,87,121,211,210,134,50,121,242,84,238,185,231,14,
  110,159,116,7,93,253,252,120,237,181,87,240,210,235,121,252,241,191,243,236,156,39,169,170,170,98,218,35,51,156,242,127,179,110,45,217,89,219,185,238,
  186,225,28,63,113,156,239,190,219,64,80,80,15,18,19,147,177,88,114,41,47,47,39,37,37,21,173,78,71,121,197,81,54,111,218,72,98,226,0,34,
  34,35,89,186,244,115,66,67,204,116,245,247,167,178,242,20,49,189,99,249,234,171,47,185,253,246,63,147,150,54,20,69,86,120,249,229,231,201,203,203,
  109,118,191,24,223,47,158,9,19,38,242,248,227,143,16,23,215,143,27,111,188,153,249,243,223,228,142,41,119,97,171,173,97,91,198,247,172,92,241,21,
  215,13,27,65,81,209,62,246,236,41,196,224,237,205,181,215,12,99,221,250,181,232,245,6,82,6,165,178,115,215,14,18,226,19,41,44,44,100,119,206,
  46,250,246,237,79,76,76,44,223,126,187,158,171,174,26,132,175,143,47,38,83,55,186,116,233,194,63,254,241,116,243,108,105,75,109,89,91,252,86,19,
  62,67,83,253,15,224,236,223,206,184,78,70,185,14,84,26,233,207,234,13,48,26,57,12,169,238,52,209,11,245,65,210,101,146,15,90,110,236,120,72,
  79,126,92,104,73,165,35,174,127,193,207,56,42,111,99,207,157,203,34,142,37,146,25,51,166,81,92,180,159,236,236,44,254,243,159,127,99,14,53,227,
  227,235,3,178,140,234,112,58,85,141,134,168,168,40,180,58,157,125,20,45,203,168,170,202,198,141,223,242,225,71,31,144,153,185,205,217,48,234,150,72,
  101,71,174,101,215,99,119,109,141,200,213,81,242,72,174,242,52,92,38,114,213,143,139,142,20,89,33,115,123,6,79,61,53,155,208,208,48,226,226,250,
  49,113,226,100,178,178,183,147,157,253,3,241,9,137,104,181,58,84,141,134,205,91,210,201,181,228,18,19,211,135,244,77,27,201,205,221,141,181,250,55,
  62,254,120,49,0,70,163,137,192,192,160,54,209,17,192,222,125,123,152,53,251,49,186,248,118,33,215,146,131,37,55,199,185,225,73,150,206,234,74,163,
  170,68,70,70,129,44,163,40,42,146,44,163,200,10,139,63,124,159,247,63,88,200,158,61,133,231,232,232,124,245,167,61,235,109,71,183,99,155,120,120,
  220,67,146,36,238,188,107,50,119,222,53,153,226,226,98,98,98,250,240,236,156,167,41,40,200,35,33,33,137,212,148,52,246,236,45,228,149,87,94,36,
  53,37,13,189,94,15,146,68,81,81,17,17,17,145,248,248,250,210,39,174,47,229,21,229,36,39,15,36,115,123,6,155,183,164,211,175,127,60,90,109,
  39,52,170,138,236,72,199,101,31,144,43,206,118,167,42,42,254,254,70,194,195,195,137,136,136,66,111,48,96,54,247,116,182,99,36,120,225,197,231,152,
  122,231,36,222,123,111,65,179,237,114,221,202,228,222,125,123,120,226,239,143,97,50,117,167,111,191,120,52,170,134,13,27,190,225,185,185,115,88,182,124,
  9,190,62,190,40,234,217,236,14,26,141,134,239,51,182,178,108,249,82,118,238,202,6,32,36,212,76,72,168,25,36,137,144,16,51,190,190,190,124,177,
  228,51,58,121,121,17,16,16,136,127,87,35,185,150,221,60,242,191,255,195,236,39,30,107,81,91,119,125,175,110,201,93,150,206,234,168,127,124,2,190,
  62,62,68,93,25,109,183,115,14,249,63,251,252,83,134,13,27,142,70,171,69,213,104,240,247,55,162,213,106,241,243,243,71,163,209,32,43,10,9,137,
  73,148,150,150,178,98,197,114,140,254,38,130,67,66,145,21,25,85,81,136,140,188,18,73,182,95,171,115,103,31,2,2,2,137,238,29,75,90,218,80,
  102,207,158,193,23,75,62,101,252,132,91,155,221,47,2,20,20,230,227,227,211,153,224,144,80,66,66,204,20,23,23,49,126,194,45,236,220,145,197,63,
  95,123,153,126,253,18,136,136,188,210,190,73,84,174,175,243,186,89,83,103,127,164,170,4,4,6,2,16,24,24,100,223,103,35,73,200,178,66,225,158,
  2,254,253,250,171,60,251,143,167,219,220,182,182,165,13,107,178,110,54,232,123,234,245,67,14,7,90,171,211,157,123,50,165,203,41,186,52,242,122,189,
  207,187,92,167,238,183,26,187,254,229,104,239,68,30,104,119,90,238,116,9,83,112,93,150,81,85,21,85,177,31,51,110,48,24,240,243,235,138,78,171,
  197,100,50,33,57,12,196,79,63,29,36,50,50,202,158,67,25,80,20,149,97,215,13,71,150,37,150,44,253,156,170,170,42,231,113,229,231,91,14,109,
  24,50,209,216,50,81,123,201,211,228,177,192,174,114,185,200,36,43,50,82,173,228,52,156,178,34,163,209,106,201,203,179,0,16,221,43,154,220,220,28,
  199,142,99,251,236,119,93,66,120,73,150,157,249,166,205,230,158,84,89,171,24,244,167,84,114,115,119,183,137,142,234,102,13,52,90,45,58,157,22,189,
  94,79,116,175,104,214,172,89,233,52,234,146,44,83,94,81,142,209,104,34,32,32,208,41,143,170,42,220,57,245,30,180,90,13,11,23,189,123,86,158,
  198,100,17,8,220,140,133,239,125,200,145,35,71,120,242,169,89,84,84,84,112,199,148,59,217,190,61,131,85,171,87,114,253,176,225,220,116,211,56,172,
  86,43,47,191,252,60,86,171,21,73,146,216,183,127,47,131,6,77,197,228,111,34,40,48,8,163,209,132,201,100,162,172,172,212,25,146,209,213,223,31,
  69,85,145,228,179,157,127,93,118,156,206,157,189,49,135,133,81,91,83,75,197,177,99,36,37,13,160,250,55,43,165,101,37,132,153,123,58,142,22,151,
  152,49,125,38,0,143,78,127,136,163,71,143,158,63,236,196,229,121,157,131,8,246,56,220,234,223,170,209,104,52,220,48,250,38,70,142,28,195,23,95,
  124,138,209,104,194,86,91,99,63,73,85,146,80,53,26,174,30,122,45,0,43,86,44,199,219,187,51,170,162,226,215,165,11,93,253,252,57,114,228,48,
  73,137,201,12,73,27,138,197,146,195,79,63,29,36,52,212,76,106,234,96,38,77,186,131,197,139,23,178,105,211,198,22,133,18,72,46,142,209,130,249,
  139,168,173,173,197,98,201,1,32,49,62,137,227,199,143,147,16,159,200,174,93,59,236,54,83,182,111,40,235,226,107,151,41,56,56,4,139,37,135,144,
  144,80,162,162,162,248,245,212,73,100,89,198,100,52,98,177,228,112,250,244,105,126,249,245,4,62,62,62,200,178,204,129,131,7,136,139,235,75,110,110,
  14,254,254,70,108,212,98,48,24,48,155,205,148,150,149,80,94,94,206,170,85,43,88,181,106,69,163,101,104,24,74,80,167,239,170,170,42,10,11,11,
  232,21,29,141,201,100,162,164,180,152,254,253,227,89,181,106,37,214,234,106,42,79,157,36,44,44,140,160,128,64,74,138,139,156,101,86,29,14,180,36,
  75,246,193,150,172,240,203,175,191,98,50,26,9,53,155,157,177,209,146,36,19,20,24,200,245,195,134,161,191,239,65,62,252,232,125,210,211,55,54,25,
  210,224,246,190,67,131,16,142,241,227,110,33,49,113,0,53,53,103,40,44,204,231,157,119,231,211,169,83,39,254,122,223,223,152,247,198,107,88,173,86,
  254,114,239,253,88,171,173,44,92,248,14,195,175,31,201,225,35,135,233,221,59,134,146,146,34,138,246,23,49,101,202,93,24,12,6,74,74,246,243,206,
  187,11,234,213,173,134,97,28,34,132,227,18,9,225,168,169,169,225,208,79,7,89,177,98,57,135,15,31,162,170,170,10,89,150,236,233,145,218,187,172,
  45,56,186,83,106,204,120,52,116,202,28,175,43,170,130,162,200,142,88,65,213,233,128,245,137,141,99,71,118,54,102,115,24,91,183,164,51,100,232,53,
  156,62,125,26,89,146,49,24,188,89,179,118,5,7,15,28,64,81,20,58,233,116,103,101,108,120,252,230,5,66,38,220,65,158,198,116,36,203,18,41,
  169,105,164,166,166,145,151,159,203,111,214,106,52,170,234,252,76,64,64,32,38,163,137,184,184,126,152,205,97,84,85,86,178,118,237,106,100,73,66,149,
  21,12,122,47,84,69,33,101,80,42,217,217,63,112,237,181,195,136,141,233,67,126,190,165,213,58,170,91,78,179,207,152,41,200,146,76,64,64,32,222,
  222,157,241,246,246,198,108,14,35,39,103,55,122,47,61,249,249,22,250,245,237,143,78,171,197,96,48,16,22,214,147,231,158,123,198,190,9,70,81,206,
  205,86,210,208,200,183,228,104,217,54,168,187,29,18,194,33,178,112,120,222,36,0,18,72,48,229,142,137,206,54,48,99,198,195,140,28,121,3,73,137,
  3,136,233,29,203,139,47,205,37,63,207,66,74,234,96,102,206,124,130,87,95,125,137,252,124,11,123,10,243,81,85,13,131,82,82,40,42,218,143,222,
  203,139,62,177,113,120,117,234,68,77,77,13,38,163,9,173,70,99,15,171,146,100,100,199,224,61,36,56,24,175,78,157,8,232,30,72,87,191,174,84,
  28,43,167,32,63,143,145,163,110,96,72,218,213,100,101,109,167,103,207,112,176,217,103,145,231,206,157,227,116,40,155,219,78,36,64,150,37,98,98,250,
  240,242,203,255,162,184,168,136,61,5,249,168,99,198,178,122,245,74,214,173,91,99,119,90,198,223,138,159,159,31,58,141,6,173,35,235,193,134,245,223,
  112,232,240,33,0,162,34,175,228,224,193,82,126,237,236,67,64,80,32,185,57,187,249,228,211,15,25,58,244,26,38,79,186,131,29,59,178,208,235,189,
  88,182,108,9,171,87,175,104,69,251,1,73,150,184,107,234,100,238,152,114,39,70,127,35,6,189,158,152,152,62,188,242,202,11,220,125,247,189,232,189,
  188,80,85,133,155,110,28,71,109,109,13,91,183,110,230,247,223,79,19,17,17,73,94,94,46,41,41,131,57,122,228,8,181,53,53,103,7,252,142,65,
  188,34,43,142,149,50,153,159,127,254,153,238,221,3,184,97,244,24,42,42,202,169,168,168,112,30,208,241,155,181,186,69,161,39,13,109,124,65,65,62,
  97,161,61,137,138,234,197,186,117,107,80,111,188,217,30,150,224,8,177,211,105,117,152,205,102,246,237,223,203,158,194,124,36,36,135,156,18,50,146,115,
  176,85,83,115,134,19,39,126,33,49,33,153,210,146,98,122,247,142,225,138,46,93,8,11,235,201,154,53,171,89,179,118,85,251,76,76,180,181,29,59,
  143,238,26,134,117,40,138,66,86,118,38,95,127,181,140,233,211,103,209,55,174,31,157,58,121,97,52,26,137,140,136,34,39,119,55,6,131,129,224,224,
  16,34,35,34,241,242,242,66,167,213,97,208,235,209,105,117,140,25,115,35,251,247,239,101,229,170,175,153,61,235,73,226,251,39,176,107,215,142,250,125,
  95,19,153,97,164,203,232,56,239,86,103,225,40,42,42,106,51,97,194,194,194,90,245,253,154,218,90,170,170,170,200,202,206,226,199,221,63,58,111,230,
  233,211,246,100,224,237,222,239,211,194,236,1,141,236,126,109,236,117,89,81,236,249,41,29,101,49,153,186,225,239,111,68,213,104,208,27,244,132,135,71,
  112,180,252,40,38,83,55,162,123,69,179,121,243,38,6,94,117,21,219,183,111,115,206,178,52,60,70,181,69,247,221,221,228,113,200,36,203,50,91,182,
  166,243,254,251,11,1,251,198,9,85,163,161,127,124,2,85,149,85,168,170,134,131,63,29,192,104,52,242,233,39,31,114,219,109,183,211,163,71,48,197,
  37,69,244,12,15,167,127,255,4,178,119,100,49,40,37,149,148,148,193,200,178,68,239,152,88,242,11,242,90,173,35,92,50,1,200,138,76,149,213,46,
  79,72,72,8,122,189,129,240,240,8,186,117,235,70,239,152,88,182,101,108,229,190,191,60,64,85,85,21,199,142,85,208,51,60,162,94,254,212,198,116,
  116,161,140,27,77,213,207,54,171,187,238,212,142,4,110,229,68,215,221,183,228,228,129,76,157,122,15,255,247,228,44,0,70,140,24,205,223,159,120,154,
  92,75,14,139,23,191,79,98,194,0,204,230,48,103,123,219,187,111,15,105,131,135,242,229,151,75,56,112,176,12,85,163,33,33,49,9,107,149,149,62,
  113,125,89,189,102,37,105,131,135,240,227,143,59,57,254,203,9,58,119,246,38,33,49,9,239,15,59,99,234,102,114,238,177,176,90,173,148,150,148,16,
  22,214,147,226,226,34,100,73,110,252,216,238,243,180,229,115,103,70,101,242,11,242,248,225,135,76,174,31,54,156,42,171,21,69,81,145,21,217,217,214,
  21,85,65,81,53,40,26,13,222,157,59,19,23,215,143,204,140,109,40,202,81,108,54,27,209,189,99,208,233,116,232,180,157,168,172,60,73,80,80,15,
  250,246,233,199,188,55,94,35,58,58,134,80,71,184,137,212,140,28,187,23,156,133,198,37,195,132,36,147,150,118,53,178,44,51,109,218,163,212,214,218,
  136,142,142,65,81,84,86,172,92,142,159,95,87,103,174,108,173,70,195,201,95,127,229,230,177,227,89,254,245,50,2,252,253,57,117,242,36,214,234,106,
  130,122,4,115,252,196,113,130,131,67,57,120,224,0,70,147,9,31,31,95,246,237,221,195,13,163,111,100,243,150,141,118,249,101,153,146,226,98,70,143,
  186,17,163,169,27,201,73,3,72,74,30,192,147,79,206,110,182,190,1,242,11,242,24,55,110,2,167,78,85,82,90,86,202,79,7,15,18,221,187,55,
  101,7,74,49,155,123,178,45,99,43,170,170,193,167,179,15,90,157,14,69,85,48,155,195,8,10,234,97,79,119,168,209,160,200,10,94,94,94,252,242,
  203,175,36,39,39,115,160,172,148,184,62,113,124,187,97,157,75,42,55,90,165,239,14,179,99,77,228,126,110,44,67,135,172,200,72,54,71,8,135,86,
  139,162,40,244,143,79,224,228,201,147,244,143,79,32,215,146,131,86,171,163,160,48,159,171,175,185,142,159,127,62,134,162,42,246,76,88,138,74,239,222,
  177,44,90,244,46,86,171,213,30,74,212,72,191,243,71,100,220,112,39,127,211,62,3,237,6,5,105,248,155,173,41,88,109,109,109,135,57,204,109,189,
  4,211,232,236,34,246,88,95,217,145,131,113,235,214,45,76,154,244,103,70,12,31,69,94,190,133,194,194,2,52,170,125,217,106,253,186,181,244,137,237,
  99,95,202,82,84,166,77,155,142,36,75,164,167,127,231,116,50,207,39,67,163,206,252,31,32,79,99,75,66,141,235,71,70,146,108,245,150,253,62,249,
  248,67,238,189,247,126,176,217,88,186,244,11,172,86,171,83,158,109,219,190,103,248,240,17,108,217,154,142,34,203,168,170,66,98,98,50,59,118,100,243,
  214,91,243,72,76,72,98,202,148,187,89,178,228,179,86,235,168,238,51,170,98,143,217,180,90,173,124,191,109,11,247,254,247,253,0,124,249,229,23,40,
  142,48,18,171,213,202,250,245,107,73,77,77,67,146,236,175,205,127,123,33,178,44,179,104,209,187,108,220,244,237,5,245,35,16,184,11,239,47,250,24,
  27,54,222,125,103,62,197,37,69,188,248,194,171,216,108,54,222,123,111,1,71,143,30,225,145,71,102,48,114,228,104,138,139,138,72,223,124,118,217,252,
  224,129,50,146,146,146,41,40,204,199,106,181,242,217,167,31,213,107,47,86,171,21,85,85,121,248,225,71,145,144,120,251,237,55,56,80,86,198,107,255,
  124,3,89,150,121,233,165,185,36,37,39,35,73,18,89,89,153,84,28,171,160,170,170,202,229,96,21,120,236,49,187,19,247,237,119,235,121,231,157,183,
  155,221,142,235,226,106,211,211,55,50,126,220,173,12,30,60,4,69,145,25,55,238,22,110,30,59,129,163,71,143,144,95,96,33,54,38,150,215,255,245,
  38,249,5,121,24,141,70,30,159,249,4,0,199,42,42,48,24,12,60,58,253,97,12,6,3,47,189,248,26,207,205,125,134,145,195,71,242,246,91,239,
  82,81,81,193,155,111,205,227,190,191,60,192,132,9,183,113,203,45,19,169,168,40,231,209,233,15,95,212,61,112,61,166,89,146,36,146,146,6,176,116,
  233,231,172,92,245,53,55,223,60,129,196,164,1,40,138,125,22,249,251,239,183,48,99,198,44,214,175,255,6,141,70,195,206,93,59,152,248,251,239,236,
  219,187,135,49,99,198,114,234,228,73,44,150,28,30,124,224,33,12,6,3,249,5,249,148,150,149,144,58,56,141,99,21,21,164,111,222,72,65,97,126,
  189,107,90,242,114,249,33,43,131,23,95,120,21,73,146,152,59,119,78,139,251,159,146,146,98,170,171,127,163,160,48,31,128,143,62,254,128,255,249,219,
  35,12,191,126,20,150,188,92,118,238,220,193,216,155,198,51,118,236,120,110,26,59,142,229,203,150,114,248,240,33,166,207,152,137,132,196,242,229,75,9,
  13,13,37,38,166,15,203,150,45,161,123,247,64,114,45,57,206,80,57,69,145,25,63,225,86,110,185,101,34,229,229,71,153,62,99,154,219,219,86,169,
  209,193,221,185,225,131,138,162,48,232,79,131,73,77,73,35,47,223,194,158,189,133,220,117,215,127,49,111,222,191,152,50,229,46,244,122,61,90,173,22,
  75,238,110,38,76,152,72,247,110,1,28,57,124,24,85,213,216,29,105,85,67,117,117,117,147,97,26,77,134,91,182,147,254,220,213,223,148,194,204,230,
  22,15,34,138,138,139,219,189,162,132,153,205,30,215,113,36,37,37,145,148,156,204,188,215,95,191,248,81,107,51,150,62,92,67,6,26,27,165,74,45,
  104,140,173,149,165,35,229,113,71,153,58,82,158,214,24,167,153,51,103,242,236,179,207,54,249,254,95,239,191,159,237,153,153,108,223,190,253,146,104,71,
  2,247,164,163,236,91,91,118,228,231,147,249,188,178,54,99,70,81,234,128,50,120,154,252,158,174,239,142,176,169,77,233,168,238,245,201,147,167,98,179,
  217,88,188,120,33,54,155,141,235,175,31,206,152,49,227,156,57,161,23,44,120,139,49,99,198,242,241,199,31,16,18,18,234,248,127,49,241,241,137,20,
  20,228,209,171,87,12,101,101,37,172,93,187,138,57,115,158,103,241,226,69,252,240,67,230,5,211,17,182,139,243,236,198,254,102,139,67,56,138,59,160,
  48,117,74,51,123,152,19,221,38,137,211,155,49,203,233,250,185,243,201,113,161,74,110,107,166,44,110,35,143,59,202,212,129,242,216,90,89,55,197,81,
  222,130,63,156,14,178,111,182,142,148,185,161,189,104,129,243,111,235,136,50,120,154,252,158,174,239,142,176,169,77,172,90,215,233,70,150,229,122,135,172,
  36,38,14,100,249,87,75,89,179,102,21,99,111,26,79,98,98,178,115,35,251,150,173,155,25,49,226,6,100,69,113,132,37,41,44,91,190,132,105,15,
  79,103,212,168,49,228,229,89,248,33,107,123,147,135,204,212,233,176,61,202,233,238,254,166,234,142,133,113,189,158,217,3,103,162,219,166,125,52,179,163,
  105,225,239,93,42,242,180,68,166,233,211,167,243,252,243,207,95,150,58,18,8,132,125,235,56,153,165,11,56,255,127,100,25,60,77,126,79,215,119,71,
  212,199,166,116,179,232,253,119,235,149,113,206,179,79,57,223,91,250,229,231,231,124,126,218,35,15,2,56,51,145,72,146,196,67,15,223,255,135,234,208,
  19,252,77,247,207,194,225,105,187,57,219,56,123,128,116,222,75,217,154,95,153,219,170,163,114,51,121,154,35,147,235,1,45,151,171,142,46,248,123,238,
  118,120,137,200,194,113,121,56,210,238,218,86,154,33,115,155,59,255,29,84,6,79,147,223,99,245,221,1,54,181,225,137,188,77,93,79,186,88,217,207,
  85,94,253,223,187,212,108,116,11,203,211,236,25,232,226,146,146,14,45,135,44,203,120,123,123,83,101,181,162,247,242,18,61,141,7,140,160,221,113,68,
  223,48,119,165,208,145,64,32,236,91,123,203,215,34,231,95,200,127,89,232,187,195,6,163,13,195,41,90,17,51,126,209,14,120,43,241,20,127,83,117,
  215,241,131,183,183,55,225,225,225,248,249,249,177,119,239,94,207,25,192,32,98,55,221,177,163,16,247,228,194,105,236,112,163,186,43,218,145,192,195,141,
  78,179,219,158,144,255,210,148,215,109,108,106,27,12,38,46,117,91,124,177,254,102,179,67,56,66,67,66,40,41,45,61,235,144,180,227,212,189,170,170,
  248,251,251,211,189,123,119,44,185,185,30,228,161,184,217,50,184,224,236,108,132,184,39,23,214,129,59,29,164,34,218,145,64,32,184,20,108,174,176,99,
  45,198,83,252,205,22,231,129,86,20,5,85,85,57,125,250,116,187,20,74,85,85,186,119,239,78,88,88,24,93,186,116,17,53,73,208,38,204,154,61,
  91,40,65,32,16,8,4,2,15,193,221,253,205,22,133,112,132,132,132,112,252,248,113,231,233,61,109,93,168,186,194,244,234,213,139,224,224,96,214,175,
  95,239,89,131,205,6,127,5,238,193,51,207,60,35,148,208,204,250,43,150,62,5,2,129,224,210,178,169,158,136,39,248,155,45,158,129,190,226,138,43,
  168,174,174,198,96,48,80,93,93,77,101,101,101,171,11,166,170,42,222,222,222,152,76,38,194,195,195,9,10,10,98,237,218,181,162,6,9,4,2,129,
  64,32,16,92,134,184,187,191,121,81,105,236,58,233,116,248,117,237,138,36,73,84,86,86,114,230,204,153,86,41,73,175,215,19,16,16,64,183,110,221,
  240,241,241,225,235,175,190,242,208,225,166,77,196,60,9,68,221,21,237,72,32,16,8,155,42,236,88,27,224,206,254,166,122,177,95,252,249,216,49,0,
  34,163,162,240,245,245,69,171,213,94,180,16,58,157,14,131,193,224,185,142,179,64,32,16,8,4,2,129,160,205,113,87,127,179,213,105,236,10,11,11,
  197,221,173,27,112,34,98,158,4,162,238,138,118,36,16,8,132,77,21,118,172,173,113,55,127,83,22,183,68,32,16,8,4,2,129,64,32,104,62,170,
  136,207,105,203,33,167,200,95,43,240,240,250,43,242,64,11,4,2,193,165,101,83,5,237,130,152,129,22,8,4,2,129,64,32,16,8,90,128,42,198,
  70,109,56,216,108,240,87,32,240,180,250,43,242,64,11,4,2,193,165,101,83,5,237,131,212,163,71,15,113,127,219,144,228,228,100,6,12,24,32,20,
  33,240,40,50,50,50,200,204,204,20,237,72,32,16,8,46,65,155,42,104,123,68,12,116,27,147,153,145,33,148,32,16,245,86,180,35,129,64,32,108,
  170,224,18,70,234,17,20,36,60,104,129,64,32,16,8,4,2,129,160,153,136,77,132,2,129,64,32,16,8,4,2,65,11,16,155,8,5,2,129,64,
  32,16,8,4,130,22,32,102,160,5,2,129,64,32,16,8,4,2,225,64,11,4,2,129,64,32,16,8,4,237,195,255,15,0,212,120,191,180,65,39,
  200,186,0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00128_png = (const char*) resource_TalComponent_bmp00128_png;
const int TalComponent::bmp00128_pngSize = 50374;

static const unsigned char resource_TalComponent_bmp00129_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,25,0,0,5,246,8,6,0,0,0,113,195,3,147,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,189,93,73,68,65,84,120,218,236,
  253,103,144,92,231,153,38,136,62,199,166,207,50,40,3,128,68,1,85,240,32,0,2,5,162,232,189,21,72,81,18,41,18,52,106,53,213,234,158,158,
  221,189,187,29,123,39,98,110,68,239,252,184,55,226,222,141,187,187,63,102,98,231,207,222,118,114,45,75,74,162,36,146,18,69,111,0,18,32,225,13,
  81,40,95,64,217,172,202,202,202,170,116,199,222,31,159,57,223,119,242,100,21,200,30,245,40,186,167,34,16,34,41,50,191,58,38,207,247,156,231,125,
  140,2,224,57,252,129,127,116,0,254,74,255,66,42,149,250,81,95,223,205,216,180,105,19,210,153,12,0,160,92,169,96,33,159,199,224,224,32,46,156,
  63,7,215,117,87,252,69,21,0,207,70,253,31,217,108,246,71,135,159,121,6,219,183,111,71,46,55,135,171,87,175,98,161,176,8,215,245,96,24,58,
  154,178,25,52,53,53,193,113,28,124,118,241,34,62,252,240,3,248,190,255,220,53,47,114,219,237,183,255,232,240,225,195,248,236,179,75,56,114,228,8,
  150,150,75,48,77,19,177,88,28,186,174,65,213,200,31,93,213,144,74,37,177,118,109,39,202,165,18,222,125,231,109,228,243,249,231,86,93,228,193,7,
  30,252,209,195,95,122,4,47,190,248,18,38,38,38,144,78,103,160,25,6,12,93,135,174,235,208,13,3,154,170,65,81,20,168,154,10,69,81,201,105,
  77,38,160,170,42,142,125,116,4,185,92,78,90,72,21,255,166,183,183,247,71,15,60,244,32,190,243,157,239,98,122,102,6,153,108,19,12,51,6,211,
  48,160,235,58,52,93,135,170,168,80,84,149,28,141,170,66,85,21,168,170,130,74,181,138,154,101,97,207,190,94,36,18,201,31,133,23,241,1,248,186,
  174,255,232,241,175,124,5,47,253,252,151,40,149,74,72,38,83,208,116,13,186,174,65,211,52,104,186,14,77,213,248,135,43,138,66,79,4,57,33,190,
  15,212,106,22,124,223,199,158,27,247,129,125,46,0,159,31,201,35,135,30,195,224,208,48,166,167,167,17,79,36,201,2,154,14,85,163,139,168,193,111,
  174,40,10,20,5,80,232,26,190,239,195,243,60,248,190,7,203,178,16,79,36,176,113,83,247,143,165,211,165,40,202,143,187,55,109,196,217,51,103,144,
  74,38,97,24,38,12,157,156,34,93,211,161,209,197,84,77,131,162,4,11,145,5,192,23,240,92,15,158,71,22,106,111,239,144,175,201,182,237,59,176,
  80,40,160,92,169,192,48,99,193,41,162,127,194,231,159,125,56,249,227,3,62,249,112,207,247,224,121,46,28,203,130,174,27,72,165,82,63,230,139,172,
  89,211,134,124,126,1,134,97,10,11,4,71,160,105,154,244,219,147,107,224,211,211,228,194,245,60,248,174,11,215,113,224,56,228,239,93,207,69,27,61,
  26,157,92,116,149,28,133,97,72,23,89,211,201,233,97,215,128,29,1,123,72,120,158,11,223,243,249,105,114,93,15,174,235,192,245,92,248,190,15,211,
  52,248,34,176,44,7,49,85,135,102,24,228,8,84,118,7,169,80,213,224,46,247,253,224,66,251,158,203,63,220,243,92,184,174,75,255,215,131,227,186,
  128,234,177,35,39,119,87,185,92,130,162,170,48,248,169,81,233,69,22,239,34,5,128,31,124,40,61,37,174,235,192,117,93,254,199,113,29,184,142,3,
  207,243,224,216,78,112,225,11,133,2,253,6,203,11,120,190,199,143,130,220,65,8,238,34,215,131,231,186,240,248,2,14,92,199,134,231,58,112,28,27,
  228,12,89,193,34,83,147,19,112,93,23,186,110,64,211,117,126,129,187,215,180,194,115,221,224,66,123,46,191,139,92,151,252,182,174,231,145,139,237,186,
  112,92,23,142,67,142,68,85,85,204,205,229,130,69,108,219,126,166,82,46,195,208,117,248,190,15,203,182,241,200,253,247,224,208,189,247,160,69,85,80,
  171,86,201,17,120,244,116,73,71,192,78,145,11,215,33,139,40,138,2,215,113,80,173,86,159,145,30,43,51,211,83,208,52,21,142,235,226,225,7,238,
  197,173,219,182,194,207,207,35,166,107,193,111,45,126,56,253,123,215,177,225,179,35,112,29,56,182,141,100,34,129,66,97,1,117,143,149,241,177,209,195,
  197,98,17,95,126,248,1,220,213,189,9,83,151,46,225,149,99,199,49,92,40,194,48,99,252,238,241,60,15,14,251,62,56,54,57,69,244,58,88,150,
  5,195,48,224,251,62,38,39,39,15,71,62,133,225,57,184,117,219,86,204,140,141,225,149,99,199,209,63,59,135,88,60,1,207,117,225,211,223,220,97,
  119,142,235,192,166,167,200,117,28,216,182,13,77,85,145,205,100,112,249,114,191,244,177,26,128,27,216,223,76,76,76,188,84,41,22,159,42,85,170,56,
  61,49,133,88,44,30,220,57,158,199,111,77,215,177,225,56,228,212,217,142,13,219,182,97,154,38,154,178,89,92,186,244,25,28,199,57,28,222,180,158,
  14,239,100,241,120,252,167,55,238,63,128,84,58,141,106,165,10,171,86,133,227,186,176,29,7,240,92,88,54,57,61,190,71,190,112,201,100,18,158,235,
  226,242,229,126,184,174,123,56,106,103,124,170,17,0,216,208,181,241,103,235,214,173,131,25,139,195,177,29,88,142,5,215,113,225,211,199,134,162,40,240,
  108,27,249,194,2,38,39,39,159,94,9,72,60,181,26,164,49,77,243,103,29,29,157,136,197,99,80,85,21,182,237,192,170,85,145,203,229,96,219,246,
  211,171,253,247,215,180,200,63,245,71,197,63,195,143,190,234,161,42,202,207,246,237,219,135,238,158,205,104,109,105,134,162,170,40,151,43,88,200,207,99,
  104,120,4,151,251,47,193,247,253,167,87,59,93,95,111,240,225,47,62,254,149,175,225,182,219,110,65,165,82,193,228,228,20,242,11,11,176,45,27,134,
  105,32,153,76,161,41,155,65,185,82,193,249,115,231,112,252,216,199,240,125,255,169,107,94,100,219,246,237,47,126,251,219,223,70,62,191,128,119,223,123,
  31,185,220,44,84,85,229,240,72,213,117,104,138,2,195,48,145,205,102,209,217,217,129,229,165,34,222,126,235,77,20,10,133,167,86,93,100,255,254,253,
  47,190,240,173,111,225,149,215,126,135,207,46,156,71,34,153,130,25,139,65,215,52,24,20,123,177,109,153,236,53,228,178,102,51,105,0,96,224,238,169,
  134,23,190,167,103,243,139,207,127,227,27,248,225,143,126,130,207,46,156,71,58,157,129,25,139,193,208,117,178,53,235,134,0,42,84,178,152,170,66,211,
  84,44,151,74,168,89,22,246,238,59,128,68,34,249,98,195,69,158,122,250,105,252,238,245,55,48,57,57,137,116,38,128,167,4,84,104,80,53,149,2,
  60,149,239,57,12,189,40,80,96,89,54,60,223,199,222,253,189,136,68,144,119,222,121,215,139,75,75,75,232,239,239,71,58,149,130,166,235,208,53,141,
  252,209,53,168,170,6,77,85,161,168,10,5,23,100,91,38,168,5,240,40,114,177,45,11,241,88,12,221,61,155,95,172,59,146,27,118,239,198,39,159,
  158,64,60,30,167,71,96,240,107,160,170,12,127,145,191,38,123,191,136,191,132,93,211,243,80,179,44,180,119,116,66,81,148,23,249,126,210,213,181,241,
  165,154,101,161,80,40,32,30,51,9,130,215,52,40,20,251,138,184,43,12,143,200,145,200,187,166,99,219,80,21,21,77,77,77,193,145,180,181,173,65,
  126,62,79,206,57,219,231,117,29,186,33,220,73,170,38,44,160,192,243,252,96,1,151,32,23,143,35,24,23,174,231,162,101,77,91,240,141,55,99,9,
  148,171,85,152,70,12,154,70,142,130,92,131,224,8,216,5,102,184,11,0,61,69,62,89,64,216,243,61,207,131,109,219,136,137,224,206,113,29,56,182,
  67,0,54,125,147,210,84,13,138,170,72,224,14,62,189,75,194,167,136,254,225,251,190,235,64,85,20,40,244,213,66,5,224,151,75,37,64,129,180,128,
  170,169,194,69,86,193,81,65,232,20,177,163,112,24,200,112,108,130,90,232,255,207,47,252,226,226,34,224,67,122,209,241,92,15,226,33,176,139,236,137,
  240,148,2,60,215,117,200,66,142,13,215,245,225,184,78,61,184,155,158,154,132,237,56,208,117,3,138,170,160,102,89,216,185,99,43,110,218,220,13,171,
  102,209,11,237,145,29,145,94,3,2,147,24,160,163,71,224,186,112,92,27,142,109,67,83,21,20,22,22,130,69,92,215,125,178,90,46,65,211,52,88,
  182,131,190,3,251,241,252,131,15,224,134,206,54,24,78,141,194,78,95,184,77,61,142,24,93,134,183,92,1,42,121,228,198,88,92,92,148,191,140,83,
  83,147,80,20,5,15,222,123,55,190,118,83,47,242,195,195,184,56,58,14,199,247,229,215,3,207,13,0,55,93,192,101,167,202,113,224,216,22,146,137,
  56,138,139,139,240,125,255,73,233,177,50,113,245,202,19,15,223,119,55,30,184,113,15,102,198,198,240,218,71,31,227,253,129,65,168,9,242,116,101,23,
  216,99,216,203,117,225,185,14,125,39,33,120,216,161,24,216,208,117,92,185,50,206,239,21,105,103,60,246,246,59,232,137,199,240,202,177,227,184,156,155,
  71,34,153,14,110,81,254,130,19,220,166,158,227,192,245,124,216,182,13,199,38,23,185,185,185,25,131,3,3,240,125,255,9,113,63,249,154,180,159,236,
  218,245,139,244,154,54,152,241,36,170,181,42,121,77,99,167,198,247,233,99,195,162,104,222,225,232,81,85,85,52,55,53,97,106,106,18,185,92,238,137,
  240,166,245,181,8,112,247,139,222,3,7,145,72,166,80,169,84,96,89,53,14,79,109,250,193,236,150,5,128,68,34,14,77,213,48,52,52,132,74,165,
  252,68,212,206,248,181,70,0,96,83,119,247,47,214,174,93,15,221,48,224,216,54,106,182,77,23,160,111,80,170,10,248,62,10,133,2,174,92,25,151,
  78,81,120,145,175,174,134,88,178,217,236,47,91,215,172,65,60,30,135,2,5,174,231,162,86,179,176,144,159,199,210,210,18,124,223,255,218,138,128,231,
  90,22,249,23,3,238,126,185,255,192,77,232,233,233,65,75,115,19,20,133,188,243,231,231,114,24,30,25,65,255,165,75,171,157,174,198,215,68,81,148,
  95,62,249,212,211,56,120,211,1,84,42,21,92,185,58,129,133,124,158,48,119,17,224,238,216,199,31,125,109,165,69,190,18,254,135,187,118,221,240,242,
  159,125,251,207,48,53,49,137,247,62,60,130,124,126,158,108,102,134,129,152,105,82,40,164,64,215,13,14,238,22,11,5,188,243,246,155,40,22,139,95,
  91,117,145,190,190,190,151,159,255,198,55,240,235,95,191,130,254,254,75,72,36,83,136,197,99,48,116,3,170,202,248,47,157,227,47,182,21,100,41,9,
  74,193,221,87,27,94,248,173,219,182,189,252,244,225,195,248,193,63,254,16,253,151,47,35,157,201,240,5,52,77,88,128,130,187,0,228,105,88,46,149,
  57,115,23,143,199,95,142,92,68,81,148,151,15,63,125,24,175,190,246,59,204,204,204,34,157,78,67,211,13,153,88,99,11,80,244,34,130,59,192,135,
  101,217,240,125,31,123,247,245,70,223,194,247,222,123,31,102,115,57,12,14,14,34,149,74,65,55,116,142,30,117,145,243,210,2,106,80,81,64,81,11,
  35,215,92,202,220,37,177,169,187,251,229,48,115,247,242,142,157,59,241,233,137,19,72,36,18,132,247,162,31,206,78,81,0,178,101,106,42,160,7,3,
  112,103,89,53,180,119,172,133,162,40,47,243,253,100,227,166,77,168,148,203,88,90,90,134,97,154,208,52,21,58,227,189,216,69,86,192,161,81,24,220,
  113,222,139,110,9,142,109,195,208,117,100,9,184,35,64,162,163,189,29,249,133,2,52,250,122,160,235,236,58,4,119,17,3,119,236,166,20,193,157,207,
  177,87,192,125,89,182,141,150,150,214,224,27,175,27,49,148,202,101,152,166,73,127,123,153,119,140,2,119,193,2,4,111,73,176,200,37,175,224,177,88,
  76,0,119,142,13,219,113,41,77,43,96,47,14,238,20,137,189,35,8,146,238,253,158,207,63,152,227,46,207,131,239,41,80,149,0,220,161,84,42,145,
  111,48,189,192,154,170,129,92,4,53,26,96,211,83,227,122,1,44,101,23,222,118,61,216,54,69,47,110,192,220,249,197,98,145,16,45,244,90,184,190,
  143,184,105,34,157,48,235,142,128,33,21,134,224,93,215,225,80,200,161,180,20,33,226,0,203,178,131,11,63,61,53,9,215,113,160,107,58,44,219,70,
  215,245,235,241,231,95,249,50,122,50,105,88,53,75,58,2,6,143,24,7,198,79,17,37,214,8,123,68,192,221,92,110,86,98,238,30,103,71,115,231,
  45,125,248,198,99,143,162,89,215,144,77,36,97,215,202,194,247,192,151,216,83,199,161,0,207,117,249,2,142,109,115,102,186,92,46,203,251,137,99,213,
  240,167,79,191,128,205,217,12,10,87,175,224,189,115,231,113,98,116,12,102,60,37,125,15,194,236,169,237,56,240,41,122,116,93,130,92,210,233,52,138,
  197,34,124,223,127,92,90,196,115,108,108,206,102,48,122,185,31,111,157,62,139,225,185,121,232,137,36,84,40,4,200,177,23,28,199,225,215,196,117,29,
  248,30,61,2,250,134,197,168,119,10,238,56,169,182,13,0,114,115,115,63,158,205,47,60,151,171,212,48,56,51,135,88,50,197,57,248,128,88,118,41,
  147,234,74,215,128,192,83,242,112,108,109,105,193,224,224,0,28,199,121,92,220,79,30,19,159,152,237,237,237,191,217,187,239,0,60,223,167,44,170,207,
  23,96,32,143,156,34,198,61,186,176,173,26,116,93,71,83,54,139,209,209,17,20,139,197,47,135,55,173,199,34,192,221,111,68,112,87,171,213,232,181,
  240,232,59,72,112,235,42,2,184,27,28,28,64,181,90,253,114,212,206,248,216,10,224,238,55,34,184,179,108,139,131,109,0,244,101,201,69,177,176,128,
  43,87,175,194,247,253,47,55,218,227,31,91,13,177,164,82,233,223,116,116,118,194,52,77,40,10,200,3,208,178,49,151,155,69,185,92,110,248,225,159,
  107,145,63,248,196,244,159,11,220,189,34,50,119,190,162,162,86,171,33,55,51,131,145,209,17,12,92,190,12,223,247,31,91,237,116,61,218,232,195,191,
  246,196,147,184,249,230,62,137,185,243,92,23,170,166,33,147,201,32,157,74,161,92,169,224,220,153,211,56,126,252,248,99,159,107,145,109,219,183,191,34,
  50,119,115,115,57,130,185,12,131,51,119,186,170,214,129,187,55,223,248,61,74,165,229,199,86,93,164,183,183,247,149,63,125,225,5,188,246,234,107,184,
  112,241,34,226,137,4,98,241,4,249,112,74,77,213,131,187,128,185,59,250,225,251,200,231,243,143,69,242,93,0,252,158,158,205,175,60,247,252,243,248,
  225,143,126,130,11,159,93,66,58,147,69,60,145,160,208,40,12,238,20,206,218,137,204,221,190,3,7,17,143,199,95,105,136,32,159,123,238,57,252,238,
  245,55,48,53,53,133,116,58,13,221,160,195,76,186,136,166,178,161,166,202,137,53,241,164,16,112,7,236,185,113,127,244,88,246,238,123,238,121,117,110,
  126,30,151,46,93,66,138,51,119,186,196,26,169,156,247,162,11,248,132,209,17,167,166,182,101,33,145,76,97,227,166,238,87,235,142,100,231,206,93,28,
  220,153,20,61,114,106,80,156,218,169,106,192,220,41,10,39,117,0,159,51,21,4,220,117,66,81,148,87,249,34,27,55,117,191,90,179,44,44,46,22,
  97,198,98,20,3,7,224,142,92,104,53,130,185,243,37,82,135,177,21,182,101,67,55,12,6,238,232,88,182,173,13,249,249,60,7,119,154,136,30,69,
  4,175,200,163,113,134,127,125,129,88,227,219,179,227,160,181,117,77,240,88,137,199,76,148,202,101,24,134,89,255,225,154,76,108,194,103,212,148,47,177,
  70,236,84,121,2,76,50,77,19,156,246,176,108,7,49,215,133,78,103,240,225,169,169,42,48,167,64,52,185,201,182,101,145,185,147,192,93,165,92,6,
  20,242,122,166,235,228,40,92,58,233,81,85,5,74,29,68,13,232,40,151,239,255,193,72,144,128,10,87,2,119,2,115,167,193,166,207,166,189,61,155,
  16,55,52,225,131,35,152,59,62,251,117,248,17,216,148,45,242,124,14,238,2,230,206,178,44,120,62,208,117,253,122,252,233,161,135,241,181,219,110,65,
  155,161,195,178,172,208,43,130,0,36,220,0,127,49,96,225,121,100,17,93,85,176,144,159,15,46,188,227,56,95,82,225,253,246,208,195,15,224,150,181,
  157,112,171,85,76,94,25,39,8,196,117,225,235,226,69,166,80,200,247,5,252,21,32,23,199,113,249,197,91,90,90,130,196,119,245,29,60,136,123,118,
  237,196,212,165,75,56,209,127,25,39,70,199,96,41,26,140,88,60,242,54,229,11,56,54,157,193,7,200,69,0,119,95,146,54,173,223,255,250,87,232,
  105,105,198,251,31,30,193,248,98,17,70,34,5,67,85,67,111,81,242,59,8,71,48,2,246,98,204,221,101,1,220,41,0,30,225,232,228,250,235,126,
  219,181,101,59,12,195,36,216,138,130,59,70,172,177,185,175,75,63,52,76,172,249,190,143,214,214,86,12,92,238,71,181,90,253,82,228,83,120,244,234,
  196,151,62,187,112,14,138,2,104,154,70,47,36,145,35,176,121,111,64,205,186,252,149,193,178,44,168,170,138,150,150,22,76,92,189,34,45,80,119,36,
  2,184,251,237,129,155,14,34,158,72,161,92,41,195,174,213,248,112,223,115,109,122,155,146,197,224,251,72,38,18,208,52,206,220,125,41,106,103,124,184,
  209,222,220,221,179,249,119,157,157,107,41,184,115,96,89,53,216,142,11,223,115,224,3,208,84,149,128,187,98,145,49,119,143,52,218,227,31,94,13,177,
  52,53,55,255,110,77,235,26,24,166,9,85,33,204,157,101,217,200,207,207,49,230,238,145,213,208,202,195,127,104,220,245,199,193,220,165,82,169,223,133,
  53,119,203,165,18,242,243,121,12,13,14,224,194,133,243,215,116,186,30,106,192,160,190,222,72,115,23,143,153,72,167,83,104,106,106,66,205,178,112,225,
  252,5,28,251,248,40,124,223,127,248,154,23,185,253,142,59,95,127,250,233,167,112,225,194,69,28,61,122,20,203,37,194,86,152,49,51,144,98,169,42,
  52,157,32,201,206,142,118,44,21,151,24,115,247,240,170,139,28,58,116,232,245,251,238,191,31,63,249,233,207,48,61,53,133,100,42,13,195,52,57,231,
  194,40,17,182,144,8,238,60,207,195,199,71,63,68,62,159,127,184,225,133,239,235,235,123,253,238,123,238,193,63,252,253,63,96,46,151,67,58,147,133,
  25,51,3,205,157,38,0,139,16,184,91,90,46,193,118,28,220,216,123,19,76,211,124,61,114,17,211,52,95,63,244,232,163,120,241,197,159,163,84,169,
  32,145,100,196,154,17,64,34,77,173,199,94,116,91,86,20,182,73,41,184,113,255,129,232,91,248,75,135,30,195,192,192,32,166,103,102,144,72,36,41,
  238,165,148,148,170,134,70,130,106,72,239,21,32,23,2,238,146,184,126,67,215,235,18,22,214,52,237,245,174,174,13,56,125,250,52,146,201,36,167,6,
  21,6,176,245,96,192,92,63,150,173,71,46,182,101,163,163,179,83,150,249,236,220,185,11,249,124,30,53,203,130,25,163,162,62,77,131,41,129,59,74,
  13,2,245,71,17,26,106,218,182,5,211,48,17,143,199,127,207,79,87,235,154,86,204,205,47,16,180,162,5,224,78,213,2,252,203,53,119,74,0,238,
  2,10,74,198,94,236,157,191,163,163,51,248,198,107,170,134,74,69,4,119,34,177,70,225,41,20,38,86,131,15,153,26,244,4,26,132,129,59,5,128,
  25,51,5,205,157,109,67,81,201,64,89,215,3,205,29,191,147,216,2,226,212,52,180,239,75,219,178,99,3,190,23,8,3,0,248,203,165,50,191,155,
  124,40,176,93,135,200,21,24,53,24,197,220,73,228,166,204,123,177,157,212,117,132,177,108,113,177,0,192,135,227,185,200,166,83,184,235,214,91,240,68,
  239,94,160,86,129,162,40,146,176,210,171,155,92,59,1,231,69,63,216,113,8,114,172,86,43,193,233,154,184,122,5,219,182,110,197,193,59,239,192,193,
  205,61,104,214,53,44,204,206,34,174,169,88,118,108,104,154,46,169,7,252,48,44,18,177,23,5,22,138,162,32,151,203,5,139,52,53,53,225,91,79,
  63,133,102,199,70,185,80,192,201,177,49,156,25,26,198,178,231,67,209,85,120,158,39,141,101,25,37,27,13,238,108,174,185,179,109,251,65,190,72,46,
  151,123,240,242,197,139,111,92,159,136,225,245,79,78,96,124,177,8,45,150,128,17,75,208,15,244,234,193,157,231,74,112,149,129,59,199,113,144,205,102,
  145,155,157,145,72,181,30,0,24,26,25,249,230,116,165,134,5,215,135,25,79,64,211,84,120,30,17,183,114,106,80,100,237,28,249,20,57,46,65,143,
  100,136,163,97,108,108,236,193,186,87,236,241,233,233,7,222,251,232,8,98,134,65,192,130,27,40,55,29,138,220,125,58,49,149,6,203,46,83,13,90,
  208,52,13,201,84,138,105,238,234,223,126,1,160,86,173,61,112,230,212,9,40,84,101,192,148,155,126,29,122,180,131,35,176,137,106,208,52,12,52,53,
  53,161,255,210,103,112,93,247,129,240,166,117,127,4,184,123,51,106,44,43,226,94,6,85,21,69,65,42,153,132,27,104,238,30,136,218,25,239,111,4,
  0,186,186,54,190,185,110,253,122,24,102,172,78,115,199,89,88,219,198,76,110,22,185,92,238,129,149,128,196,253,171,33,150,68,34,249,102,123,123,59,
  204,152,73,196,124,158,135,90,181,134,153,153,105,216,182,253,192,181,104,238,238,251,3,193,45,229,154,113,215,63,225,199,255,60,204,221,91,251,247,239,
  199,166,238,30,73,115,55,151,203,97,116,108,140,105,238,238,255,66,167,75,81,148,183,68,230,110,98,98,2,11,133,69,174,185,19,153,187,51,103,206,
  224,211,227,199,238,255,92,139,236,218,117,195,91,127,246,237,63,67,46,55,135,183,223,121,23,243,243,115,208,52,29,166,105,64,215,13,190,45,232,186,
  30,98,238,94,71,169,84,186,127,213,69,250,250,250,222,122,254,27,223,192,111,94,121,13,151,62,187,200,53,119,1,184,211,185,98,77,163,34,230,8,
  230,238,254,134,204,221,214,109,219,222,34,99,217,31,17,197,26,31,203,234,2,184,211,184,20,136,189,246,49,112,87,179,44,220,216,123,19,226,241,248,
  91,145,143,21,69,81,222,62,252,244,97,188,246,234,107,152,153,153,161,204,29,81,172,113,230,78,211,4,59,130,82,55,212,172,81,73,208,170,99,217,
  129,161,97,50,150,101,200,69,85,9,184,139,34,213,66,148,8,40,115,23,79,36,208,181,113,211,219,117,139,236,216,185,19,39,79,158,66,60,30,39,
  168,69,35,18,68,77,39,23,91,147,232,193,122,98,205,71,128,90,28,203,226,112,136,47,210,211,179,249,237,74,181,138,197,98,17,177,120,140,47,192,
  124,15,210,41,66,120,1,74,172,185,30,167,68,44,219,102,156,241,219,252,194,7,154,59,141,179,167,220,251,35,208,179,97,112,71,206,144,12,141,68,
  25,124,235,154,128,185,131,97,198,200,139,142,97,242,119,16,113,246,206,206,127,20,247,232,135,228,88,174,235,9,99,217,120,64,224,176,237,83,55,200,
  81,248,80,224,193,39,50,69,69,246,156,160,1,184,147,8,54,215,129,239,41,242,53,41,149,74,252,183,180,108,27,109,173,45,216,182,182,19,240,61,
  225,109,74,9,81,131,94,61,254,165,219,48,211,29,121,116,223,161,104,101,22,173,109,29,232,222,216,133,238,141,27,113,112,115,15,114,163,35,24,184,
  122,21,90,218,224,158,19,241,20,177,33,154,12,141,2,206,69,83,85,206,220,233,0,112,221,250,245,248,147,39,191,138,13,134,14,183,90,69,49,63,
  143,169,220,28,98,10,65,238,4,69,202,234,129,192,216,18,28,133,109,59,92,100,150,136,199,249,88,86,7,224,87,203,101,62,200,188,56,58,142,193,
  169,105,204,86,107,208,227,73,2,228,5,244,232,82,183,64,120,1,215,9,96,43,232,141,65,199,178,190,6,96,83,110,110,238,187,190,102,188,112,98,
  112,24,253,51,115,176,52,13,134,73,228,237,34,192,38,10,194,6,240,84,152,154,38,147,73,148,150,151,177,184,88,184,71,250,198,127,255,165,23,239,
  153,43,87,144,105,202,82,236,235,69,188,228,184,245,228,166,43,78,77,45,58,2,209,165,177,172,132,187,206,156,252,20,190,231,65,165,96,65,226,224,
  93,249,8,124,79,6,216,140,185,107,202,102,49,52,56,0,223,247,239,137,92,164,90,173,222,115,230,212,9,168,154,10,93,215,235,69,125,226,173,26,
  2,119,154,166,161,165,165,5,227,227,99,168,86,171,247,132,55,173,187,35,32,208,187,251,123,15,32,150,72,160,90,173,194,170,89,68,53,224,123,212,
  14,226,240,23,80,0,72,38,18,80,53,13,67,100,44,123,79,212,206,120,215,10,204,221,123,235,214,173,133,170,25,176,109,27,182,109,193,177,109,250,
  42,1,114,90,125,31,197,197,69,198,220,221,221,104,143,191,235,26,152,187,247,90,91,215,16,201,137,170,194,113,29,212,106,150,56,150,189,123,53,180,
  114,23,254,192,63,127,52,154,187,247,14,30,60,136,174,77,221,146,230,46,52,150,93,245,116,221,217,224,195,223,127,250,153,103,113,240,166,3,40,44,
  20,48,53,51,211,80,115,71,199,178,119,127,174,69,246,238,189,241,253,111,254,233,55,137,230,238,131,15,145,95,200,115,112,103,24,38,20,77,171,51,
  84,8,224,238,174,85,23,185,237,246,219,223,63,124,248,48,126,245,242,175,112,121,96,64,2,119,68,220,39,50,22,100,168,233,121,46,154,178,89,145,
  185,187,171,225,133,223,181,235,134,247,159,120,226,9,124,231,187,223,199,192,208,48,210,153,44,98,241,192,45,171,27,130,28,142,91,65,85,24,134,129,
  229,82,192,220,197,226,241,247,27,105,238,222,127,226,137,39,240,235,95,255,26,243,243,243,124,44,203,68,125,12,57,138,164,14,217,150,125,14,85,107,
  181,26,57,221,100,44,91,191,200,67,15,61,140,169,153,25,12,143,140,33,149,74,113,150,72,19,70,129,162,230,78,150,195,129,139,202,108,203,70,34,
  153,66,215,198,77,239,75,88,88,81,148,247,55,111,217,130,147,39,79,113,230,206,100,182,16,9,220,169,146,176,76,180,227,50,67,165,75,41,194,182,
  142,144,91,182,103,243,102,170,185,43,114,238,87,85,53,89,181,38,136,250,100,96,23,140,4,125,207,135,239,122,132,185,211,77,164,82,169,247,57,224,
  110,107,107,71,110,62,207,117,143,228,195,85,105,114,45,218,113,101,199,44,165,68,56,99,68,68,51,97,183,44,76,195,32,154,59,1,220,69,50,119,
  136,86,13,186,2,192,96,163,89,219,6,98,34,115,103,59,46,20,221,131,166,169,112,61,143,124,241,12,3,62,85,108,138,115,95,63,132,189,2,197,
  154,252,7,190,207,61,18,42,0,44,20,22,224,184,46,178,217,44,250,110,216,133,111,30,122,24,7,175,91,7,199,178,248,191,8,40,18,53,72,60,
  142,126,196,2,212,10,202,144,11,131,68,93,215,93,135,135,110,189,5,93,205,77,104,74,167,225,86,171,152,208,117,56,142,141,152,96,40,150,78,145,
  27,136,249,24,154,119,41,53,98,59,54,20,53,70,125,18,20,11,247,92,119,29,14,238,221,131,225,51,167,49,52,52,132,225,201,41,76,44,44,32,
  102,154,242,96,57,116,13,194,167,200,19,252,15,122,34,33,51,119,239,126,240,62,54,166,18,248,224,194,69,44,58,14,160,232,48,76,19,186,161,144,
  187,199,15,8,102,199,117,40,146,175,63,69,54,61,18,230,19,174,84,202,119,176,7,228,237,0,208,181,113,211,135,155,183,110,135,239,251,252,86,229,
  210,208,58,89,98,52,184,99,208,40,147,78,163,180,188,132,241,43,87,238,144,30,43,227,99,163,119,212,42,101,24,6,131,66,190,4,238,26,46,224,
  202,11,104,154,6,69,85,249,2,117,79,225,147,39,62,129,239,121,208,84,45,226,54,245,66,231,159,30,129,29,44,0,0,77,217,44,6,66,110,89,
  233,61,190,90,173,222,206,152,59,77,83,5,228,238,52,6,119,174,203,193,93,107,75,11,70,71,71,96,219,246,237,226,231,42,0,110,139,0,119,71,
  246,247,30,8,152,59,234,150,245,92,7,182,75,120,97,203,182,224,82,146,153,129,187,203,253,151,216,2,117,59,227,109,141,246,230,158,158,205,71,58,
  248,88,54,208,220,249,158,71,111,16,194,25,23,22,22,112,245,234,149,219,87,218,227,111,187,6,67,197,145,214,53,107,16,139,197,161,170,10,28,199,
  65,173,90,195,252,252,28,74,165,210,237,215,194,174,221,246,47,6,220,249,171,128,187,163,76,115,215,212,220,12,77,1,42,213,26,230,114,57,12,13,
  15,97,112,96,224,182,107,57,93,183,54,250,240,70,154,59,195,52,145,76,38,3,230,238,244,25,124,250,201,177,219,62,215,34,59,119,238,58,250,237,
  63,255,54,114,185,57,188,243,238,123,152,159,159,131,162,106,136,9,204,157,166,170,48,76,19,217,76,0,238,126,255,250,239,80,169,148,111,91,117,145,
  190,190,190,163,156,185,187,116,137,232,189,24,184,211,195,227,40,38,34,39,134,10,207,243,112,228,195,247,177,88,40,220,214,240,194,111,217,186,245,40,
  51,84,92,186,116,9,233,116,26,113,42,234,211,245,250,121,151,162,168,252,69,148,141,101,123,111,186,25,166,105,30,109,4,238,142,62,115,248,25,188,
  250,219,215,49,61,61,195,193,157,70,71,177,82,82,137,0,46,152,67,80,81,20,202,220,249,184,113,255,1,40,138,114,180,238,217,117,239,189,247,97,
  110,126,30,131,3,3,146,168,143,57,102,195,224,206,247,195,94,135,32,10,133,130,59,212,113,144,59,118,238,196,167,212,45,43,46,192,37,113,106,136,
  255,10,13,212,216,48,199,247,124,216,81,81,40,155,186,123,62,170,84,171,40,44,22,168,37,90,231,19,211,122,222,75,145,56,242,48,161,64,56,22,
  139,49,119,31,241,69,58,218,137,230,78,215,13,201,115,162,233,122,8,255,214,91,66,164,121,151,200,185,120,110,52,115,199,94,112,160,0,80,131,163,
  224,146,56,69,38,214,188,6,196,26,27,77,81,205,29,89,164,82,173,194,87,117,64,1,210,241,24,90,215,180,193,182,170,88,44,85,232,43,130,26,
  32,187,8,83,139,87,199,26,145,69,25,102,211,1,248,235,154,155,177,163,187,27,77,217,12,182,54,55,33,155,205,226,157,79,62,197,7,11,35,48,
  83,38,191,2,126,232,20,173,4,238,52,77,133,79,82,78,8,238,186,227,222,123,112,243,134,235,81,89,88,64,177,88,196,76,110,22,133,229,101,168,
  44,10,69,128,167,141,4,101,76,204,196,192,157,170,154,168,86,107,193,233,250,236,212,105,116,86,74,120,235,244,89,20,170,53,148,74,203,112,117,3,
  102,50,77,190,108,144,45,33,124,20,40,17,156,116,6,236,4,204,93,126,78,0,119,47,254,230,215,184,116,101,63,116,35,6,85,215,96,164,178,48,
  105,168,139,56,185,102,105,49,16,198,179,18,254,117,2,252,11,223,71,137,26,42,136,14,178,86,187,101,106,122,6,169,116,10,186,166,243,183,166,240,
  2,140,185,115,66,178,68,206,123,81,67,69,60,22,99,154,187,91,164,103,215,149,177,145,91,74,75,75,48,12,157,42,116,101,112,231,187,245,204,157,
  39,162,71,250,242,163,170,106,36,115,199,241,81,0,238,84,254,154,198,29,203,158,87,71,48,59,2,60,37,122,73,31,217,128,185,187,57,114,44,91,
  173,86,111,150,192,93,56,45,134,31,133,45,233,30,69,112,55,57,113,21,213,106,245,230,240,166,213,23,1,238,142,245,30,56,64,52,119,229,50,44,
  203,170,151,35,82,199,114,4,115,119,115,212,206,216,183,2,184,59,182,118,221,122,168,154,70,152,59,203,162,231,159,168,57,212,122,205,221,205,141,246,
  248,190,213,208,70,115,115,243,177,214,214,53,136,197,137,185,203,117,235,220,178,55,175,134,86,250,254,171,227,174,127,22,230,206,52,205,227,7,251,110,
  70,119,79,15,183,120,86,202,101,228,230,243,24,30,28,192,103,159,93,132,239,251,125,171,157,174,131,141,62,252,249,231,191,129,27,118,223,128,92,110,
  14,19,147,147,88,42,46,193,118,28,196,99,38,146,169,20,178,153,12,106,150,197,220,178,125,159,107,145,190,155,111,57,254,252,243,207,161,191,255,50,
  62,60,114,20,75,75,69,18,129,70,101,113,44,37,78,211,53,52,101,155,194,224,174,111,213,69,30,124,224,193,227,15,127,233,17,188,244,210,47,112,
  245,234,21,164,210,105,232,70,240,225,186,180,239,7,14,193,144,230,174,175,33,184,235,237,237,61,206,114,238,38,38,39,137,230,206,140,7,224,174,206,
  204,170,210,103,149,22,104,238,246,223,4,211,52,143,71,190,51,234,186,126,252,241,175,124,5,63,255,197,203,88,94,94,230,126,7,77,35,19,83,45,
  116,4,226,80,147,129,59,203,178,1,133,107,238,26,231,220,77,78,78,34,145,36,11,48,191,131,90,7,238,194,70,202,128,18,177,106,53,36,146,73,
  108,232,218,248,73,216,45,251,73,247,166,141,56,19,210,220,105,28,100,27,146,36,142,155,89,253,136,40,20,159,228,122,180,119,134,198,178,59,118,238,
  196,66,161,128,74,181,42,169,54,53,33,130,70,28,44,115,246,142,146,58,228,241,31,0,11,219,34,204,93,34,145,252,132,47,210,182,166,13,185,220,
  60,5,119,70,240,138,160,203,232,145,44,36,78,174,9,231,2,223,15,137,152,92,202,220,181,7,223,120,207,247,81,44,45,195,167,176,147,217,17,88,
  152,91,240,219,251,141,153,59,17,185,208,125,94,100,238,252,182,182,118,52,233,26,146,93,215,163,43,155,129,177,166,13,167,79,157,194,240,226,50,116,
  149,165,148,248,28,61,242,172,193,16,184,35,243,45,155,108,114,212,43,1,192,215,21,69,249,244,217,7,239,71,134,198,53,176,159,126,93,135,239,185,
  80,148,24,61,2,129,104,142,2,119,194,246,236,80,3,31,213,220,65,247,125,255,166,43,211,211,159,198,171,21,76,229,230,96,57,14,10,203,203,152,
  88,88,128,174,105,194,41,242,248,236,189,17,169,99,83,182,194,113,28,24,186,206,83,50,116,0,248,209,207,127,142,244,154,54,212,108,7,10,21,91,
  24,186,9,67,85,234,180,167,117,228,38,61,2,155,230,75,136,49,116,211,51,51,193,45,124,234,226,197,155,170,150,131,84,58,141,68,60,78,213,102,
  170,4,176,249,247,32,60,243,165,120,203,247,152,99,214,165,34,52,23,150,101,221,36,61,187,230,114,51,208,117,157,63,13,124,207,151,240,175,68,178,
  69,17,107,52,103,205,177,45,36,18,9,20,22,242,245,207,174,145,225,161,3,229,229,37,24,134,1,8,137,73,1,123,42,223,162,81,232,209,117,93,
  154,115,231,97,114,114,242,64,36,238,58,121,226,4,60,215,163,131,101,143,42,104,253,192,80,68,233,113,62,65,21,176,151,109,19,105,98,42,149,194,
  229,254,254,198,4,78,165,82,62,112,246,244,9,114,71,232,58,167,1,37,230,212,9,129,59,219,134,109,213,96,24,6,154,155,154,48,48,112,25,174,
  235,30,8,111,90,189,17,224,238,4,99,238,202,149,50,193,91,148,142,13,27,42,84,69,65,34,30,7,0,92,190,220,15,199,113,14,68,237,140,189,
  141,221,178,61,39,214,174,93,23,24,42,28,11,142,237,208,47,169,194,49,216,66,126,158,93,131,134,123,124,239,53,184,4,78,180,119,116,240,23,77,
  215,117,97,215,106,152,158,153,129,101,89,7,174,133,138,218,255,135,210,218,253,33,53,119,254,231,94,36,145,72,158,60,216,119,16,93,27,55,213,129,
  187,145,225,33,92,36,134,138,222,47,116,186,18,137,228,201,231,159,127,14,59,118,238,140,52,84,100,178,25,14,238,206,157,61,139,79,142,31,107,184,
  88,228,34,183,221,126,251,73,49,196,120,185,84,106,8,238,50,153,12,214,173,93,43,130,187,222,85,23,57,116,232,208,201,251,238,191,31,63,253,217,
  75,152,156,184,138,84,58,195,247,125,73,204,20,145,115,39,48,119,189,13,185,250,190,190,190,147,119,223,115,15,254,238,239,191,131,217,153,105,100,178,
  77,136,197,226,36,196,141,70,208,138,90,123,6,54,68,112,215,123,160,15,134,97,156,108,100,168,56,117,232,209,71,241,226,75,63,71,165,82,38,131,
  101,67,24,5,82,228,34,70,208,50,189,151,239,249,1,115,167,160,49,184,227,134,138,105,98,168,32,124,151,46,40,214,116,41,254,151,81,131,97,206,
  197,178,44,36,83,41,108,232,218,120,74,58,18,77,211,78,117,117,109,192,153,51,103,36,112,39,70,161,240,169,169,170,9,19,187,80,12,157,24,98,
  28,30,203,238,220,185,11,11,133,2,170,213,26,137,193,214,2,244,40,155,89,3,104,20,124,184,47,133,38,48,249,181,105,196,144,72,36,79,241,69,
  178,77,205,152,158,158,1,84,149,242,41,190,28,133,18,102,238,26,238,251,65,182,170,235,121,88,35,50,119,205,77,25,24,102,28,61,235,215,161,61,
  149,132,22,79,96,190,80,64,190,102,215,77,173,197,107,16,182,33,138,94,83,5,224,111,203,186,166,105,167,190,249,232,163,48,151,151,248,135,196,226,
  113,252,232,141,55,145,175,212,234,23,136,200,26,244,4,104,202,70,129,128,0,238,60,207,195,146,239,67,91,42,162,84,179,176,84,42,1,0,242,75,
  75,82,36,99,56,231,185,254,8,130,5,60,215,129,167,128,199,216,233,190,239,239,123,245,151,47,159,46,123,30,150,42,149,224,142,208,117,232,102,92,
  56,2,79,214,61,54,240,59,176,132,94,67,215,229,156,187,87,222,123,23,101,31,80,205,24,140,68,10,241,116,22,102,60,73,191,11,178,37,218,21,
  18,226,228,189,223,150,134,154,170,162,96,118,86,0,119,142,227,236,179,107,53,66,217,242,219,199,231,151,57,42,169,143,223,69,162,215,148,234,189,20,
  133,204,189,44,203,218,39,61,86,102,103,166,97,24,122,40,41,156,56,53,92,54,22,247,229,180,234,40,236,229,56,14,18,137,4,230,169,231,55,12,
  238,110,44,45,151,2,230,78,136,99,244,125,33,239,209,113,228,35,16,152,59,219,182,9,184,243,60,204,76,79,239,139,196,93,167,4,230,142,115,143,
  146,29,71,54,117,135,7,203,170,170,18,112,183,146,161,162,82,41,223,40,26,42,28,71,120,84,240,239,129,35,163,71,170,185,51,76,162,43,162,134,
  138,27,195,155,214,222,136,173,247,140,56,150,101,81,40,44,37,153,129,59,66,149,147,16,99,223,243,208,223,127,169,110,129,134,139,4,224,174,251,12,
  139,66,241,92,23,85,171,70,82,194,233,243,77,81,85,56,182,141,220,236,12,114,185,220,141,43,1,137,189,215,0,238,206,172,105,107,135,105,26,196,
  59,239,186,162,161,226,198,107,1,98,123,241,7,254,249,227,96,238,52,77,59,219,119,176,15,93,221,221,104,202,102,9,127,92,179,48,55,55,135,161,
  193,1,102,168,216,187,218,233,218,211,232,195,159,121,246,57,236,223,191,15,133,133,2,38,38,38,176,88,44,194,178,29,196,227,49,89,115,119,246,44,
  142,31,251,120,239,231,90,164,183,183,247,236,159,190,240,2,6,7,135,240,225,145,163,88,92,44,16,74,132,129,59,186,37,55,48,84,236,93,117,145,
  187,238,190,251,236,147,79,62,137,23,95,250,5,70,134,135,144,74,19,191,3,247,57,134,34,32,194,110,89,10,238,246,54,124,157,219,187,247,198,179,
  143,63,254,56,254,254,31,190,139,177,177,49,14,238,56,122,100,138,41,65,115,47,70,161,8,99,217,179,145,8,82,81,148,179,79,60,249,4,126,249,
  203,151,177,88,88,64,58,157,230,237,8,252,3,217,64,83,81,101,183,38,8,255,66,198,178,88,193,45,251,165,47,97,108,116,12,99,227,87,40,115,
  167,113,106,80,116,45,7,180,160,26,98,237,60,192,39,241,115,137,100,10,93,93,27,207,74,15,72,69,81,206,109,222,178,21,39,79,159,65,58,149,
  226,57,119,204,80,65,60,167,236,136,180,8,222,75,142,225,176,172,26,58,58,215,202,71,178,101,235,86,44,22,151,80,90,94,134,110,26,228,8,88,
  20,157,174,75,154,199,112,12,115,56,99,141,133,24,155,166,137,68,34,121,142,47,210,222,222,142,185,185,57,162,132,162,23,217,224,174,153,40,197,96,
  125,74,114,216,10,234,184,46,243,254,146,111,252,243,143,60,2,175,184,136,165,174,235,144,73,165,0,0,31,127,118,9,3,249,69,196,13,35,116,4,
  98,4,93,216,107,26,236,251,80,0,195,32,15,20,93,81,148,115,107,214,175,71,38,155,1,203,85,99,226,49,77,81,8,91,4,69,98,238,100,112,
  39,6,185,5,98,38,155,114,197,129,136,236,157,119,176,84,40,144,208,53,0,186,97,96,178,176,8,221,48,34,36,137,226,41,18,181,247,129,214,158,
  224,46,48,173,183,175,251,190,191,251,55,239,190,119,190,123,203,86,84,171,85,250,141,182,160,27,49,146,187,173,212,11,202,196,24,102,166,76,15,116,
  119,52,165,68,211,228,40,148,185,185,28,84,69,69,60,70,8,181,152,105,240,88,108,81,119,26,30,238,179,15,119,220,128,107,97,110,14,77,211,184,
  207,84,37,142,204,234,238,106,165,204,105,90,22,40,25,126,7,169,99,79,221,192,68,17,56,53,45,78,141,84,171,213,221,117,224,46,102,154,124,246,
  238,121,190,140,30,5,4,47,218,162,61,113,52,232,18,222,43,153,72,240,64,23,105,145,209,145,225,221,75,197,69,232,186,198,167,8,158,23,32,120,
  30,7,232,201,11,216,20,154,186,142,3,219,178,8,197,168,40,152,156,156,220,189,138,230,78,165,208,84,12,146,118,121,144,52,75,230,181,41,154,116,
  220,128,185,203,100,50,184,220,127,105,69,205,221,13,231,206,158,14,162,80,132,111,176,39,94,108,234,154,113,108,114,20,142,77,142,128,165,36,59,142,
  115,67,88,115,183,43,2,220,93,96,99,217,74,165,130,154,85,147,108,233,65,214,10,201,185,75,38,147,128,239,51,230,238,134,168,157,113,215,10,99,
  217,11,100,44,171,211,16,99,171,46,196,216,115,93,20,10,5,92,189,122,229,134,149,246,248,93,215,160,185,187,208,186,102,13,205,185,83,225,114,67,
  69,14,149,74,249,134,107,1,119,187,254,208,184,235,143,67,115,23,143,199,47,222,124,203,45,232,233,238,65,34,153,148,152,187,209,225,33,22,133,114,
  195,106,167,107,103,131,235,112,113,165,40,20,198,220,217,182,133,115,231,46,224,227,143,142,236,250,92,139,220,126,199,157,23,197,40,20,150,237,21,5,
  238,210,153,116,56,10,101,215,170,139,28,58,116,232,34,99,238,166,38,39,144,76,165,131,137,93,196,212,148,189,33,135,52,119,187,26,94,248,190,190,
  190,139,156,185,155,157,225,81,40,129,145,82,238,214,10,192,157,192,220,221,212,7,211,52,47,54,98,238,46,178,40,148,74,153,212,143,241,156,59,74,
  3,170,17,160,66,108,170,96,224,174,161,91,86,138,66,73,6,81,40,140,88,227,126,7,30,133,66,16,100,176,161,145,129,154,85,35,204,221,245,27,
  186,46,74,15,72,77,211,46,118,117,109,192,233,51,12,220,5,166,22,45,116,138,88,4,176,170,42,240,61,97,238,200,182,101,159,186,101,195,99,217,
  173,219,182,97,54,151,67,113,185,4,151,10,149,20,170,111,20,195,12,89,12,29,7,119,33,41,16,19,210,212,108,155,49,119,23,185,190,235,185,67,
  135,160,87,202,200,220,184,7,21,42,120,201,149,202,40,212,236,72,175,175,8,240,120,191,153,48,80,243,92,23,158,170,162,173,189,29,87,198,199,8,
  36,218,223,219,11,117,46,135,29,27,55,242,15,249,249,7,31,98,161,186,32,12,151,209,128,88,243,165,18,53,238,117,0,130,250,49,93,215,63,59,
  117,242,36,138,116,38,216,154,37,164,127,169,82,133,169,27,13,142,64,144,99,137,98,114,129,18,17,159,86,186,227,56,59,95,122,227,205,207,210,217,
  102,84,106,85,152,186,14,83,215,40,200,139,69,50,119,225,10,12,81,169,206,146,249,21,138,213,248,133,95,90,94,134,25,51,17,51,77,194,121,233,
  38,84,186,64,0,178,61,94,32,193,120,22,207,117,235,164,240,30,101,140,20,69,65,141,70,161,168,0,252,233,169,41,120,158,79,35,130,212,200,40,
  108,110,203,97,29,15,43,89,66,216,205,147,19,234,199,42,149,242,142,90,181,2,221,48,36,223,15,175,191,8,187,5,188,250,118,48,130,90,130,148,
  12,138,193,118,72,84,212,204,244,20,103,238,68,207,53,193,193,1,246,146,212,3,124,82,234,5,229,93,22,25,203,138,81,40,34,184,219,193,152,59,
  49,18,211,11,151,120,184,81,150,16,155,51,119,166,105,194,247,60,228,114,185,29,43,50,119,42,11,209,139,16,183,70,81,131,172,185,69,211,52,164,
  210,233,186,250,177,48,115,183,35,208,220,233,130,169,219,21,114,86,194,0,155,40,216,12,131,100,75,80,230,110,71,120,211,218,30,177,175,95,234,61,
  112,16,241,100,146,212,143,9,81,40,158,107,115,88,234,58,14,84,77,67,50,145,16,235,199,118,68,237,140,219,86,96,238,250,197,16,99,135,70,49,
  123,244,123,192,94,17,102,103,166,165,107,240,185,22,17,142,172,159,213,143,41,138,66,131,245,44,76,79,79,193,113,156,237,215,2,238,182,253,139,1,
  119,171,105,238,250,111,190,249,22,108,236,238,174,215,220,13,13,226,179,139,23,224,251,254,246,47,116,186,226,241,120,255,179,207,62,39,105,238,10,133,
  69,120,158,7,67,215,165,177,172,80,63,182,253,154,23,185,245,150,91,251,159,121,238,89,162,185,251,240,67,20,151,150,97,154,245,99,217,48,184,163,
  204,221,246,168,69,182,134,52,119,151,69,205,29,3,119,146,215,84,211,37,65,153,239,123,28,220,81,205,221,246,149,52,119,151,153,230,110,106,106,18,
  153,108,19,226,92,239,21,202,183,149,106,200,4,205,93,239,65,152,166,217,31,57,16,208,117,253,50,211,220,21,151,150,145,72,82,51,171,16,133,162,
  42,106,67,49,127,96,168,88,129,185,19,53,119,41,161,126,76,85,131,156,103,77,147,193,157,136,90,56,115,87,171,33,73,52,119,151,195,154,187,203,
  215,95,183,30,39,78,156,68,60,145,8,68,125,134,33,141,101,217,60,88,44,239,170,219,247,125,86,63,22,26,203,30,186,255,1,172,79,38,176,38,
  147,66,107,83,6,49,209,212,194,193,93,208,105,26,110,199,173,203,86,165,224,46,30,143,95,230,95,198,187,30,122,16,55,54,101,113,125,34,134,18,
  61,175,103,6,6,113,122,98,10,70,210,20,68,125,170,36,137,99,138,53,49,99,216,225,22,4,5,29,29,157,24,103,224,78,93,90,194,0,141,200,
  72,197,76,52,103,178,232,31,31,231,175,209,226,96,89,116,11,128,53,84,8,86,80,182,115,134,115,238,252,95,191,251,30,116,85,131,93,171,160,41,
  157,65,115,60,134,252,210,18,98,241,68,144,115,215,96,238,235,54,0,22,66,206,29,193,194,139,75,75,104,110,105,69,197,3,172,114,21,249,154,13,
  77,213,160,27,90,29,114,225,211,211,168,40,64,97,230,72,194,219,220,224,194,23,23,23,185,75,201,212,117,30,46,45,15,150,253,80,159,128,91,199,
  61,50,206,197,162,156,151,220,45,59,53,9,207,243,105,185,141,178,66,207,137,39,221,69,65,97,87,192,61,178,107,162,170,170,60,150,173,85,171,91,
  203,165,18,1,119,170,38,1,108,145,26,132,47,187,102,216,17,16,252,27,28,141,162,168,108,52,184,85,250,198,231,102,131,177,44,139,164,105,212,212,
  18,78,42,241,152,201,155,14,53,147,137,56,235,150,173,3,119,91,249,88,214,243,235,191,7,174,28,34,237,9,236,41,83,13,138,99,217,201,201,201,
  173,145,164,154,12,238,188,160,161,66,12,148,230,37,182,118,144,212,71,219,194,84,85,69,230,26,192,221,150,0,220,105,130,161,88,104,105,19,53,119,
  118,16,230,194,52,119,151,8,184,219,18,102,238,54,71,108,189,131,215,146,115,231,216,228,157,50,17,143,195,247,61,92,238,239,103,11,212,237,140,155,
  27,1,128,238,158,205,131,157,157,107,97,232,26,108,199,133,229,144,118,60,159,126,41,217,43,194,220,92,14,51,211,211,91,86,2,18,155,175,65,72,
  54,216,209,217,9,195,52,161,208,215,106,97,44,187,229,90,192,221,230,127,49,224,110,53,183,236,208,254,253,251,209,179,101,43,154,179,25,248,138,138,
  106,133,228,220,9,81,40,91,86,59,93,61,141,62,92,172,31,99,57,119,81,13,21,52,231,110,243,231,90,100,215,174,27,134,88,206,221,59,239,188,
  131,185,121,82,63,198,12,173,138,166,65,87,85,168,186,129,230,108,157,91,118,243,170,139,244,245,245,13,173,156,115,39,119,106,134,53,119,20,220,109,
  110,184,200,214,109,219,134,254,242,47,255,146,164,36,11,102,214,232,126,51,133,203,226,216,40,196,52,13,184,174,139,35,31,188,135,90,181,186,185,238,
  238,82,20,101,168,190,126,76,231,57,31,172,250,53,32,116,84,186,45,251,92,41,90,163,237,72,55,54,98,238,238,186,251,110,146,115,55,48,192,193,
  93,80,94,171,114,130,77,68,143,8,101,172,145,158,179,26,18,137,4,54,110,234,30,146,22,217,116,253,117,67,247,31,56,128,211,167,79,35,30,139,
  209,184,13,146,8,163,235,66,12,176,202,148,132,34,184,11,105,238,220,122,112,167,3,240,15,125,229,107,184,227,198,189,208,43,101,228,139,75,152,42,
  20,144,43,149,177,80,174,132,152,187,168,110,51,47,50,2,88,215,116,100,179,217,161,98,177,184,89,7,128,238,214,22,20,166,166,176,182,181,133,19,
  107,71,207,157,199,219,159,245,83,88,36,71,1,70,129,59,182,239,176,167,52,0,180,174,89,131,98,177,72,32,209,27,31,30,193,135,77,205,136,87,
  74,216,208,222,142,141,235,214,226,74,46,7,157,142,100,27,74,226,132,93,51,152,150,18,253,145,239,219,44,231,142,32,200,66,185,140,138,15,20,10,
  5,92,206,23,144,24,189,2,77,241,17,139,199,67,71,160,72,65,210,1,247,40,131,59,207,115,232,86,5,145,138,170,64,215,117,196,99,113,36,98,
  113,40,154,10,208,91,87,46,80,243,100,238,81,76,42,17,139,83,67,57,119,65,67,5,192,131,116,88,238,118,24,220,17,183,140,144,243,17,46,146,
  96,251,62,205,191,171,81,183,44,1,119,147,19,68,133,105,154,252,81,17,232,79,149,72,230,206,141,2,119,20,177,48,27,232,252,252,92,176,136,109,
  219,61,229,114,153,190,15,6,60,112,112,139,214,95,100,9,154,58,162,91,214,171,171,31,227,144,72,212,220,41,138,66,177,151,32,137,19,123,101,69,
  81,31,5,117,142,75,44,10,196,80,17,103,110,217,110,233,177,50,62,54,218,205,220,178,158,231,81,127,144,56,185,246,234,138,84,24,23,204,184,48,
  155,246,59,232,154,228,150,245,235,198,178,158,43,71,161,176,47,154,47,116,203,138,45,33,182,29,240,95,68,231,218,132,203,151,251,249,81,212,237,241,
  213,106,181,251,236,105,2,238,24,138,100,196,38,59,29,226,109,234,80,78,210,182,106,220,45,59,62,62,6,203,178,186,195,251,201,166,8,8,52,194,
  221,178,180,161,194,161,223,100,209,80,225,209,4,75,230,150,29,184,220,95,183,64,195,69,132,177,236,72,135,208,80,81,179,106,188,80,16,190,15,141,
  214,143,45,22,139,184,74,220,178,221,141,246,248,77,171,65,154,230,230,230,145,86,214,80,161,50,73,168,141,133,252,60,138,197,98,247,181,128,187,77,
  255,213,113,215,63,11,115,167,235,250,232,193,155,14,162,103,203,22,100,40,115,87,171,86,49,147,155,195,232,240,16,115,203,110,90,237,116,109,108,244,
  225,135,159,121,150,107,238,198,175,78,96,177,80,168,115,203,178,177,44,109,168,216,116,205,139,236,223,191,127,244,133,111,125,139,107,238,10,5,18,253,
  47,70,209,173,48,150,221,180,234,34,119,222,121,215,232,215,159,250,58,126,254,243,159,99,120,120,68,26,203,138,25,195,81,110,89,128,135,24,111,106,
  184,200,238,221,123,70,95,248,214,11,248,193,63,254,8,249,124,30,169,84,50,200,24,142,160,6,89,105,39,203,230,214,53,13,62,124,28,253,224,61,
  84,171,213,77,81,224,110,244,171,95,253,42,126,245,242,175,144,207,231,229,110,89,53,154,123,148,153,59,192,118,72,78,103,67,230,238,79,158,252,58,
  70,70,71,49,48,60,140,4,125,76,136,224,78,109,144,18,7,95,216,53,125,159,116,203,198,227,232,218,184,105,84,122,197,238,235,235,27,253,179,195,
  79,97,29,124,220,178,99,7,154,179,105,58,243,213,233,31,35,8,116,81,85,153,185,131,28,133,226,121,4,220,9,13,21,228,123,114,231,141,251,224,
  85,42,56,176,125,27,14,108,223,134,185,98,17,253,227,227,56,62,50,42,93,100,105,230,40,36,52,4,147,59,186,53,59,36,196,56,149,74,143,149,
  74,203,27,117,0,56,55,56,136,82,165,12,221,182,176,189,171,11,109,217,44,166,18,73,56,174,15,83,68,143,144,35,128,195,11,176,145,148,227,186,
  80,84,21,237,29,237,40,141,44,147,35,89,182,108,92,152,152,194,92,46,135,147,87,39,208,221,210,130,252,210,18,204,120,60,114,44,11,225,20,69,
  199,208,121,0,108,57,10,197,243,60,24,138,138,120,60,14,219,3,6,242,132,255,34,98,24,101,245,185,111,136,53,114,92,2,238,164,156,187,114,105,
  25,170,162,64,215,52,24,52,72,79,167,183,105,189,52,209,167,58,251,168,248,89,47,48,84,80,224,205,23,41,44,46,2,10,233,150,85,104,167,153,
  200,220,137,210,80,177,141,205,171,107,8,179,185,161,2,0,203,185,147,193,29,139,104,82,235,218,113,125,41,134,35,60,123,95,173,161,66,5,224,219,
  182,221,85,46,151,105,130,82,240,93,8,152,59,212,131,59,33,10,155,57,54,108,199,230,157,242,190,239,163,82,41,111,12,49,119,51,84,129,169,32,
  168,188,16,140,197,161,185,175,35,182,179,185,242,80,51,17,143,99,177,80,64,221,212,97,108,116,164,107,105,177,200,13,21,1,7,201,204,220,126,112,
  254,5,129,159,207,125,167,46,31,106,42,170,138,171,87,175,116,53,208,220,29,167,224,78,229,179,119,95,176,135,136,58,71,82,169,236,4,150,16,219,
  226,154,187,129,149,152,59,203,178,186,78,157,56,206,13,21,46,175,64,22,146,98,28,57,243,142,161,71,93,215,209,66,53,119,182,109,119,133,247,147,
  13,17,224,110,188,247,192,1,196,18,73,84,42,21,14,238,60,122,13,130,187,41,136,66,129,162,96,224,114,127,221,2,13,23,97,63,27,55,117,143,
  175,91,23,221,45,203,208,191,227,56,40,44,228,49,57,57,217,181,18,144,216,112,13,134,138,241,182,246,14,238,220,247,60,34,231,153,153,157,65,173,
  90,237,186,22,112,119,253,53,194,39,229,15,134,187,196,7,239,31,108,17,69,81,174,28,60,120,16,221,155,183,112,67,69,185,82,65,126,46,135,161,
  225,17,102,168,216,240,133,78,151,162,40,87,158,124,234,105,220,114,115,31,150,151,151,49,54,126,5,139,133,66,100,136,177,48,150,221,112,205,139,236,
  222,189,231,202,11,223,122,161,62,196,56,2,220,137,134,10,10,238,54,172,186,200,173,183,220,122,229,153,231,158,197,203,191,250,13,46,247,95,66,42,
  157,9,213,143,173,12,238,142,125,116,4,185,92,110,67,195,111,252,182,237,219,175,60,249,212,215,241,189,239,255,0,3,3,3,200,100,155,234,250,205,
  100,199,108,160,187,99,221,178,187,111,236,133,105,154,87,34,23,209,52,237,202,215,190,246,4,94,254,213,43,200,229,230,144,201,100,228,32,17,173,222,
  204,74,222,45,131,59,187,86,173,2,240,177,255,192,193,232,103,215,255,247,175,255,26,41,219,66,126,118,138,124,233,20,37,192,191,82,136,177,38,69,
  161,4,251,13,104,25,167,133,120,34,137,141,155,186,249,209,104,0,50,251,247,239,191,250,87,223,252,38,176,144,71,87,123,27,58,155,178,240,61,23,
  85,215,11,69,161,132,153,59,113,231,12,130,68,136,73,47,142,201,137,137,255,200,143,36,166,170,120,251,227,143,49,50,53,133,238,117,235,112,95,111,
  47,14,245,29,68,90,87,1,33,68,79,22,51,69,143,163,124,151,8,251,116,18,159,125,133,207,25,45,69,197,59,167,206,32,119,245,10,182,93,127,
  29,110,232,218,128,229,114,5,69,203,65,50,147,224,35,114,58,155,228,83,211,186,32,17,161,37,9,182,205,152,59,242,141,79,196,227,168,214,106,40,
  123,62,46,76,78,99,104,190,128,132,166,144,153,60,203,89,145,158,47,193,190,47,73,225,133,86,36,161,161,2,188,161,66,119,61,18,38,29,35,108,
  93,77,213,160,243,10,184,122,213,160,68,110,214,41,165,28,120,138,203,255,59,222,80,161,170,10,69,239,10,37,157,163,174,65,48,123,23,67,243,164,
  132,50,42,129,15,82,173,57,184,43,178,151,81,126,238,69,97,101,176,0,100,112,231,133,36,113,212,134,232,58,46,60,31,60,231,78,5,128,153,169,
  9,82,93,193,101,10,122,96,75,247,3,120,20,153,84,18,34,215,136,69,196,134,166,42,152,159,203,73,204,221,245,149,114,153,211,26,225,107,64,153,
  181,128,220,20,221,154,142,156,115,231,208,234,5,63,146,185,155,158,130,174,107,17,23,56,100,203,241,93,126,222,121,152,139,8,141,28,7,201,68,2,
  75,132,185,187,94,102,238,198,199,174,35,224,206,224,55,106,240,61,16,6,203,142,144,88,226,58,1,130,241,92,57,196,120,124,236,58,68,25,89,79,
  157,34,204,157,170,4,253,102,65,129,176,199,93,227,34,115,199,175,7,157,189,55,101,179,24,28,184,220,24,220,213,170,181,235,24,115,199,192,29,83,
  104,178,202,17,169,223,140,105,238,108,27,154,174,163,133,134,24,91,150,117,93,120,211,90,31,49,150,157,96,154,59,17,220,137,204,29,59,255,10,5,
  119,138,170,50,230,238,186,168,157,113,125,99,205,93,207,4,139,66,97,13,21,182,77,202,60,248,11,146,231,49,67,197,117,43,1,137,245,215,0,238,
  38,88,20,10,83,65,85,43,85,228,136,161,226,186,107,1,108,235,241,7,254,249,227,24,203,182,183,183,79,238,187,113,31,54,245,116,35,155,206,192,
  114,28,212,106,53,204,205,205,97,100,120,8,103,207,158,133,235,186,235,87,130,179,10,128,117,81,31,222,218,218,58,249,204,179,207,161,167,167,27,83,
  19,147,184,58,53,133,66,161,8,207,247,16,51,77,164,82,73,172,105,109,133,227,56,184,112,225,2,142,30,249,16,190,239,175,111,116,77,234,22,185,
  251,158,123,38,159,120,226,9,156,61,119,30,31,127,244,17,202,149,10,12,195,36,126,83,22,230,70,191,217,169,100,10,235,214,117,162,92,42,225,237,
  183,222,68,161,80,88,191,234,34,143,61,254,213,201,123,238,190,19,63,250,241,79,48,59,51,131,84,58,29,228,219,50,73,156,208,45,192,206,78,58,
  149,132,170,170,12,220,173,111,120,225,111,185,245,246,201,219,111,62,136,191,249,219,191,71,126,126,30,153,108,19,76,51,46,219,113,165,116,94,141,187,
  101,203,149,10,106,150,133,61,251,122,17,139,199,39,35,23,57,184,111,255,228,159,28,122,4,63,126,241,37,84,132,110,89,118,106,36,244,168,32,100,
  168,32,127,106,53,11,158,231,55,110,168,248,119,255,227,255,13,155,178,25,220,181,125,43,54,175,39,38,84,85,81,229,48,23,170,20,84,105,75,110,
  224,53,13,104,17,219,38,221,178,93,27,55,77,74,139,92,127,253,134,73,84,171,152,201,205,226,190,222,94,124,243,254,123,241,181,131,7,208,18,167,
  14,77,169,250,53,212,142,32,98,47,159,134,179,71,213,143,101,179,89,252,242,195,35,248,254,91,239,224,211,207,46,33,17,139,99,207,230,30,36,76,
  83,40,144,16,249,22,63,50,223,214,163,21,100,182,69,94,185,19,137,228,36,255,50,182,181,183,161,80,40,226,234,194,34,94,61,115,14,99,115,115,
  72,232,58,114,229,42,226,169,20,119,44,243,121,23,117,108,212,69,161,8,18,32,85,85,208,222,222,30,104,238,52,85,67,165,90,65,50,30,135,110,
  24,184,56,147,131,166,105,136,39,146,2,99,164,132,160,169,156,22,35,34,23,207,243,137,120,137,24,42,130,250,49,104,81,97,222,193,119,33,104,102,
  245,235,90,65,197,5,28,58,37,82,160,73,221,178,88,92,90,130,170,106,48,249,64,89,165,145,128,66,28,163,231,195,19,134,251,174,48,208,116,164,
  132,100,91,248,107,79,214,220,169,10,160,112,231,184,42,37,85,3,36,99,13,33,75,52,203,150,240,4,237,23,219,53,73,62,164,192,220,77,79,77,
  194,245,252,128,185,83,53,65,185,169,8,172,29,163,161,156,186,118,48,142,28,221,32,231,110,102,102,58,88,164,90,173,174,171,86,202,48,12,35,212,
  113,18,76,174,137,222,206,227,183,169,228,22,112,108,10,141,108,122,103,169,132,170,178,237,117,210,55,126,118,134,129,187,0,65,114,120,202,77,45,242,
  88,214,163,212,160,227,202,229,93,164,68,45,23,165,185,27,89,39,133,24,75,240,212,19,32,169,144,53,232,122,117,167,201,140,197,224,121,46,114,185,
  220,186,200,167,240,167,159,28,147,152,187,112,82,95,184,126,151,97,50,70,172,105,154,134,84,50,137,254,75,43,184,101,45,203,90,123,234,196,113,250,
  26,161,81,213,172,195,205,196,162,99,195,182,5,89,162,85,131,97,154,220,80,225,251,254,186,240,166,213,25,222,201,12,195,152,62,112,83,31,82,169,
  20,42,213,26,157,148,58,117,204,29,123,221,72,198,227,112,92,7,253,151,46,193,247,253,181,81,59,99,103,35,16,209,213,181,113,122,237,186,117,48,
  12,3,174,235,195,178,107,84,201,225,1,158,7,40,10,108,155,24,90,115,185,220,218,149,112,87,231,53,184,4,166,59,58,58,97,154,38,84,77,133,
  227,56,168,84,42,152,159,155,131,227,56,107,175,5,220,117,254,139,1,119,171,49,119,211,162,230,78,81,85,84,170,181,48,115,183,246,11,157,46,69,
  81,166,163,26,42,88,183,108,56,10,133,230,220,173,189,230,69,182,239,216,57,253,23,127,241,231,188,91,54,159,159,135,170,105,52,2,216,32,97,85,
  170,6,221,208,165,134,10,202,220,173,126,11,247,245,245,77,63,255,141,111,224,181,87,95,195,133,139,68,115,199,162,80,62,7,115,183,54,188,8,87,
  123,125,233,238,187,103,14,220,121,23,94,127,227,77,76,77,77,33,157,78,195,52,136,43,160,126,1,77,26,168,249,190,15,195,48,224,185,30,142,126,
  248,62,42,149,242,218,186,187,235,182,219,111,159,249,127,255,251,127,143,194,64,63,150,11,121,196,98,180,75,158,114,93,26,205,86,35,62,211,122,167,
  38,7,119,190,135,61,55,238,67,164,107,227,145,91,110,193,226,252,28,110,223,189,27,127,114,255,125,216,211,213,5,83,83,120,34,159,88,125,165,105,
  90,68,245,130,207,233,245,68,34,129,77,221,61,51,117,71,50,190,184,136,191,255,217,139,152,43,22,177,99,227,70,28,190,231,46,28,216,184,17,182,
  227,68,128,59,85,2,22,140,53,98,204,29,209,220,117,66,81,148,25,254,61,217,184,169,123,198,245,129,35,23,47,225,194,228,52,30,216,181,3,187,
  54,117,97,98,97,1,113,42,135,11,164,240,225,250,5,200,114,44,223,131,75,187,101,51,153,76,64,170,173,105,35,13,21,241,120,12,174,231,225,205,
  139,151,112,226,202,4,42,150,195,11,58,197,10,184,48,122,172,35,214,92,23,80,20,172,105,107,15,22,137,199,226,40,149,203,208,116,131,183,166,151,
  108,7,166,97,214,31,1,67,46,64,40,134,206,19,36,113,68,115,39,141,101,45,219,66,204,77,144,146,199,8,245,64,152,220,100,0,163,110,44,43,
  96,47,207,83,100,230,174,82,46,3,10,164,158,19,77,215,35,42,224,132,234,43,199,173,27,203,58,148,53,178,157,96,219,14,192,93,177,8,64,145,
  124,215,76,146,32,207,124,131,24,58,198,121,133,67,220,124,161,219,76,4,119,62,25,203,146,59,66,54,84,40,92,158,192,103,142,81,236,41,181,127,
  138,99,89,77,85,144,159,159,15,142,196,113,156,142,114,185,76,229,112,132,123,228,186,71,17,228,133,84,131,1,176,112,3,183,44,165,107,225,251,88,
  90,90,146,149,106,51,211,83,148,232,172,87,207,138,98,74,217,6,202,100,137,30,167,6,109,219,70,60,206,53,119,29,210,55,254,202,248,88,199,210,
  98,145,166,37,215,75,67,153,161,88,84,175,217,162,107,64,64,47,154,166,53,238,150,61,125,138,117,203,170,245,174,25,41,107,91,176,132,8,97,46,
  172,161,98,100,120,152,31,69,148,230,174,227,228,167,199,185,171,137,240,141,158,80,123,225,72,97,134,108,76,107,213,136,230,110,77,107,43,38,174,94,
  65,165,82,238,8,239,39,237,17,204,221,44,55,84,148,75,176,44,75,98,238,194,81,40,137,120,28,154,166,97,144,52,84,116,68,237,140,237,43,24,
  42,102,59,169,230,142,119,203,178,4,37,90,63,230,203,221,178,29,141,246,248,246,213,16,75,54,155,157,229,245,99,138,2,199,37,154,187,185,220,44,
  74,165,82,199,181,128,187,246,127,49,224,110,53,67,69,78,28,203,170,170,138,114,165,138,185,57,73,115,215,177,218,233,106,107,244,225,79,63,243,44,
  122,247,239,195,82,113,137,119,203,90,182,3,211,208,145,206,100,209,148,205,160,90,173,136,154,187,246,107,94,100,247,238,61,57,54,150,125,231,131,15,
  176,144,207,67,55,12,50,41,162,213,75,186,170,65,51,116,52,101,178,104,111,95,131,114,169,132,119,223,121,27,249,124,190,125,213,69,110,187,253,246,
  220,19,79,60,129,87,127,251,58,6,250,47,201,221,178,186,206,9,182,85,192,93,123,195,69,190,249,245,167,114,135,238,189,7,255,219,223,253,29,102,
  115,115,124,44,203,90,17,36,104,20,2,119,204,80,33,128,187,246,186,187,171,171,107,99,238,175,190,245,2,204,229,37,220,127,195,13,88,223,214,198,
  197,122,186,30,94,160,94,204,207,12,21,158,231,97,239,62,153,185,211,0,36,1,160,239,96,223,191,87,21,64,173,85,177,117,195,6,244,172,91,139,
  229,82,9,197,106,141,138,248,5,122,80,213,66,90,163,224,117,220,113,28,34,144,81,148,127,95,40,44,252,31,210,145,108,221,190,29,223,255,205,171,
  248,135,55,223,198,196,220,28,122,174,191,30,125,219,183,66,115,29,94,1,43,50,119,170,26,236,154,226,180,200,247,92,210,65,223,222,1,69,81,114,
  252,72,54,117,119,231,174,191,238,122,12,143,140,160,98,59,56,63,49,137,152,231,98,96,106,26,115,22,145,194,139,216,139,11,201,33,103,173,240,108,
  85,154,194,80,90,94,66,165,82,249,63,116,0,104,107,235,64,126,161,0,77,37,106,53,40,10,94,191,112,9,134,105,32,78,193,93,24,193,243,169,
  105,168,180,147,77,139,44,219,70,182,169,25,249,124,158,118,203,154,58,137,5,50,99,193,93,36,93,131,168,110,51,1,88,72,35,65,50,142,242,60,
  15,177,88,44,120,172,88,182,3,211,117,57,69,30,180,180,41,33,199,134,34,1,108,95,106,8,245,2,49,147,235,64,243,80,15,238,20,69,9,9,
  43,69,207,181,80,187,43,36,244,138,224,142,83,32,140,61,138,2,119,108,145,32,72,90,149,98,80,196,140,97,89,10,23,66,46,148,142,82,224,51,
  151,185,207,53,119,14,157,176,177,188,115,249,58,40,18,184,19,99,128,195,126,95,6,62,84,85,229,9,113,12,220,181,149,75,165,192,236,181,130,107,
  38,200,119,142,142,0,118,28,151,143,91,41,184,11,190,140,51,212,80,1,48,112,39,0,237,48,52,106,148,84,66,173,160,130,161,162,45,12,238,218,
  150,22,139,60,65,81,52,181,240,42,24,158,245,232,72,150,16,102,13,101,25,195,170,162,54,6,119,39,62,61,70,12,42,138,90,95,36,200,78,145,
  235,4,238,13,87,182,132,0,64,83,83,19,6,73,253,88,91,228,34,182,109,183,125,250,201,199,76,44,19,188,131,120,66,74,178,67,158,77,34,184,
  35,73,125,204,80,49,10,203,178,218,194,251,201,154,8,126,107,238,166,131,55,35,150,72,160,90,173,194,170,5,245,99,34,184,99,154,187,68,34,14,
  77,213,112,153,140,101,219,162,118,198,53,43,104,238,230,214,173,91,47,187,101,45,43,248,146,177,250,177,197,2,198,175,92,105,91,9,72,172,89,13,
  210,36,18,201,185,206,181,157,82,206,157,85,179,48,59,59,131,106,181,218,118,45,224,174,245,95,130,230,78,185,166,69,118,239,217,59,191,121,243,22,
  180,182,52,67,211,84,222,45,59,60,50,140,193,129,1,248,190,191,210,41,247,87,60,93,143,61,254,213,249,59,239,184,13,149,74,5,211,211,51,88,
  40,20,136,200,53,212,45,123,238,204,105,28,63,126,124,205,231,186,38,219,182,111,159,255,246,183,191,141,165,226,18,222,126,231,93,76,207,76,19,19,
  165,25,35,131,28,10,135,76,162,124,10,135,24,175,89,117,145,111,126,253,169,249,174,29,59,112,242,244,25,156,61,115,26,241,68,2,177,120,130,183,
  35,240,234,81,10,141,196,110,89,1,220,173,105,136,234,255,205,95,254,219,249,191,250,214,11,40,12,244,227,210,185,51,136,39,83,72,36,82,145,11,
  40,170,34,104,238,72,20,10,27,203,198,227,241,249,134,139,28,220,179,27,211,131,3,184,125,247,110,60,115,239,189,104,73,196,185,128,156,204,184,232,
  2,84,45,69,190,144,138,32,5,142,54,84,104,0,18,212,119,50,95,243,92,124,240,241,49,236,222,180,17,93,107,59,209,158,205,96,100,114,18,190,
  170,17,128,199,230,192,13,192,157,239,177,25,74,28,10,240,255,40,20,22,254,119,233,72,110,216,189,27,167,207,156,195,137,225,17,252,237,59,239,161,
  102,89,60,161,87,165,251,62,17,50,169,146,106,48,74,204,95,171,85,209,46,140,101,117,138,131,231,109,199,65,161,80,64,54,147,193,204,220,60,254,
  246,157,247,160,251,62,92,61,134,152,97,72,19,83,37,84,94,27,69,75,153,134,129,214,214,214,249,124,62,191,134,212,143,181,173,193,92,110,142,84,
  30,235,58,81,147,47,149,176,80,173,33,22,139,9,150,232,250,201,181,52,53,13,213,94,181,180,182,6,71,98,198,18,40,87,171,48,204,24,153,53,
  106,81,224,174,177,106,80,178,31,82,112,225,251,100,164,200,193,29,75,208,215,117,226,158,17,191,7,81,99,217,122,167,166,48,96,166,254,19,143,149,
  222,6,134,138,18,81,112,234,6,7,119,74,131,180,68,241,20,5,78,77,135,51,22,140,53,34,204,170,31,224,46,209,45,171,112,104,170,10,2,126,
  79,192,192,130,67,211,13,149,119,57,54,17,241,59,54,224,251,168,213,170,130,230,142,228,112,17,31,60,141,99,212,40,201,73,126,25,69,82,13,178,
  102,214,112,206,179,35,68,160,169,154,134,194,194,130,4,238,90,43,165,18,25,203,74,178,68,191,238,8,124,17,189,8,130,50,135,54,84,184,212,80,
  44,157,33,17,220,233,70,56,115,40,58,204,91,12,49,14,4,101,54,145,196,81,11,21,109,5,109,13,131,187,86,209,80,209,48,204,80,200,24,230,
  65,34,124,238,75,76,197,134,174,99,124,108,20,215,96,168,112,249,17,136,81,116,225,35,96,167,204,113,72,21,67,54,155,197,200,200,48,63,138,40,
  67,69,43,51,84,104,154,206,111,69,166,55,149,142,128,19,107,54,119,108,180,182,180,96,102,122,10,165,82,169,53,188,105,181,68,64,160,124,164,161,
  34,148,181,237,80,175,92,50,145,128,110,24,24,28,24,64,165,82,110,141,218,25,91,26,50,119,155,183,228,59,59,58,3,112,71,53,119,158,107,243,
  33,129,239,251,40,22,139,24,31,27,149,78,209,53,47,34,104,189,242,45,173,173,244,105,160,194,165,143,243,252,252,188,116,23,173,4,36,90,240,7,
  254,185,86,183,172,242,79,93,100,53,121,226,66,95,223,205,216,180,105,19,210,20,145,148,43,21,44,228,243,24,28,24,192,133,11,231,225,186,110,203,
  74,191,152,2,160,185,1,131,186,176,82,136,113,58,157,66,83,83,19,28,199,193,103,23,47,226,195,15,63,128,239,251,45,141,78,67,221,34,247,222,
  113,231,194,35,143,127,25,35,35,163,117,245,99,166,97,210,167,181,10,157,230,174,116,118,180,139,204,93,203,170,139,252,187,255,251,191,91,120,112,251,
  86,252,63,255,230,111,113,101,102,22,153,76,22,134,24,67,167,233,146,168,140,61,76,83,201,132,168,185,107,105,136,187,238,190,231,158,133,63,123,252,
  49,148,11,5,220,177,115,39,58,90,90,8,223,168,25,66,206,157,38,7,232,210,241,96,169,76,52,119,123,247,29,64,34,145,92,136,92,68,215,245,
  133,59,239,184,19,127,253,159,254,51,106,150,133,219,246,236,198,151,14,244,162,41,102,0,84,161,94,231,51,69,32,5,98,204,157,235,241,177,108,253,
  34,143,28,122,12,163,227,227,56,122,228,3,124,231,237,119,97,235,58,110,218,185,3,187,55,108,0,28,139,90,115,85,41,76,4,138,24,131,66,232,
  17,219,178,16,39,81,40,11,210,34,138,162,44,176,110,217,53,109,29,24,153,153,197,247,222,121,31,19,115,115,24,157,157,133,153,72,65,9,77,174,
  197,169,93,96,129,35,3,53,203,178,208,222,46,71,161,96,219,246,29,88,44,46,161,82,173,34,158,72,32,161,155,152,156,156,196,15,231,231,185,238,
  139,53,84,132,133,228,129,214,40,40,46,176,29,7,49,195,64,34,145,92,168,84,202,45,42,0,172,89,211,74,26,42,120,199,134,74,8,49,154,216,
  35,246,202,138,73,125,245,134,10,234,12,164,163,89,169,161,66,215,117,148,202,36,203,150,77,172,121,106,143,166,173,232,247,245,249,72,214,17,234,199,
  60,0,142,212,45,11,199,241,96,168,30,33,213,12,61,68,46,135,168,65,17,88,48,180,40,176,118,228,136,108,73,232,174,2,192,242,242,18,77,75,
  12,22,96,23,87,129,8,42,234,117,143,190,80,189,32,214,46,177,95,128,47,178,80,88,148,12,21,170,16,133,66,88,83,79,178,229,144,113,184,144,
  111,199,166,166,66,175,19,251,222,240,177,236,204,212,4,9,199,49,116,161,64,88,145,120,72,145,208,12,192,93,224,247,101,186,59,86,249,174,40,10,
  230,104,158,161,74,1,68,51,75,143,9,134,251,44,169,196,171,187,77,195,229,181,140,212,97,34,127,69,81,0,210,80,209,44,141,101,103,103,166,249,
  88,54,232,88,70,61,255,27,194,191,108,52,238,186,1,101,152,76,36,144,143,202,185,27,27,29,105,94,90,92,132,174,235,129,36,145,162,71,95,200,
  187,17,211,98,56,61,43,224,96,210,45,235,99,114,114,178,185,1,184,251,4,174,235,82,112,231,215,85,192,185,66,5,137,231,218,188,223,151,73,226,
  66,245,99,104,4,238,154,79,126,114,140,51,119,158,75,244,191,226,107,154,99,147,211,195,84,131,84,239,136,152,105,146,172,21,210,14,214,28,222,180,
  154,34,52,119,133,253,7,14,34,149,206,160,86,173,194,170,85,249,75,78,128,30,105,136,177,170,34,153,76,194,243,92,150,115,215,28,181,51,54,53,
  2,17,215,111,232,42,172,95,191,30,102,44,78,50,133,237,26,229,125,105,253,152,162,192,177,109,204,231,231,49,51,61,221,188,18,212,105,90,13,177,
  196,227,241,66,91,91,59,140,88,12,154,170,192,117,61,212,170,21,228,114,57,216,182,221,124,45,120,170,233,15,13,238,254,104,52,119,133,125,251,246,
  161,187,103,51,90,91,154,27,105,238,154,191,208,233,210,52,173,240,216,151,191,130,219,110,187,101,85,205,221,197,115,231,240,49,113,203,54,95,243,34,
  223,252,250,83,133,45,27,187,176,232,3,31,126,244,49,114,185,89,168,154,14,131,142,102,85,93,135,166,40,208,13,3,77,217,38,116,118,118,96,121,
  169,136,183,222,122,19,139,133,194,234,183,240,255,248,63,253,85,225,223,62,241,85,252,240,187,223,195,79,222,255,0,10,85,165,51,43,110,120,168,217,
  96,44,219,220,240,194,111,234,238,46,236,221,182,21,231,79,156,192,158,205,61,56,116,240,0,76,94,154,42,204,187,56,37,162,73,13,21,12,220,165,
  82,169,66,67,41,239,225,195,207,224,7,47,253,2,223,255,237,235,200,102,179,184,125,247,110,244,110,238,225,143,25,86,90,47,38,245,5,198,10,58,
  150,245,125,236,217,219,0,220,221,121,231,93,139,75,75,75,24,30,30,198,197,145,17,252,250,228,105,100,179,89,220,208,181,1,166,239,113,106,80,11,
  197,208,49,73,15,155,216,217,150,5,51,158,192,166,238,158,2,251,229,117,145,185,59,254,201,167,136,199,99,136,197,19,56,114,250,52,20,215,193,224,
  240,48,212,68,50,80,15,10,105,49,225,250,177,96,144,227,160,163,115,45,198,70,71,22,125,223,111,82,41,115,183,88,179,44,44,46,46,34,22,143,
  195,208,117,24,134,137,99,253,3,88,86,52,34,137,147,22,136,174,31,99,239,251,22,229,86,90,90,90,130,211,213,214,182,6,249,249,60,111,102,211,
  105,106,76,220,36,74,78,230,212,20,149,234,209,245,99,194,64,205,113,208,212,220,34,51,119,108,44,203,72,53,162,245,170,87,110,138,99,217,240,96,
  217,243,130,225,26,137,169,9,198,178,196,159,224,121,60,103,176,62,177,71,158,61,212,45,224,122,117,241,179,148,239,19,192,93,169,44,224,46,45,226,
  54,13,62,220,99,215,128,21,25,8,177,103,174,160,90,103,216,44,112,109,44,46,2,62,248,155,84,64,145,11,229,93,8,89,65,69,33,153,84,222,
  69,174,137,15,5,182,237,132,152,59,154,124,168,10,11,136,207,30,177,126,44,170,99,67,220,150,201,24,17,200,135,198,178,77,101,234,150,85,84,225,
  133,51,116,138,248,88,150,65,84,41,177,207,35,213,87,116,44,171,40,10,138,117,204,221,212,36,29,246,215,183,68,139,71,224,10,81,40,158,84,94,
  107,243,8,96,129,185,107,146,22,185,122,101,188,169,72,193,29,57,247,126,228,96,217,115,69,180,104,75,167,202,117,136,223,97,69,230,238,148,144,115,
  231,138,100,114,136,123,100,236,169,72,13,58,130,230,110,104,104,136,31,69,20,184,107,58,117,226,56,224,123,220,80,193,209,163,39,187,101,29,129,26,
  100,131,229,230,230,102,166,185,107,10,111,90,217,8,8,180,216,123,224,32,31,203,218,150,69,95,15,28,30,65,199,26,42,80,175,185,107,138,218,25,
  179,141,0,192,166,238,238,69,177,161,130,69,161,248,52,89,145,49,119,139,133,2,211,220,53,53,218,227,51,171,241,92,169,84,106,177,189,163,3,177,
  88,28,138,2,58,136,177,48,151,155,69,185,92,110,248,225,215,186,200,231,158,142,254,83,199,178,95,120,116,123,45,224,174,184,255,192,77,232,233,233,
  65,75,115,19,20,69,229,81,40,195,35,35,204,61,147,253,66,167,107,211,245,215,21,119,247,30,4,27,203,94,185,58,129,197,197,197,134,224,238,163,
  143,63,202,126,174,69,254,252,207,255,188,248,220,93,119,226,103,191,127,3,39,134,70,48,159,155,37,81,40,166,193,7,57,44,72,33,34,10,37,187,
  42,224,190,243,206,187,138,255,243,55,158,71,101,97,1,186,109,33,159,155,65,34,153,66,58,157,70,34,145,68,60,30,71,44,22,67,44,22,135,97,
  26,168,84,171,24,29,27,135,235,121,184,235,158,251,208,222,222,94,92,113,145,173,219,182,21,15,61,242,48,254,250,63,253,103,44,59,14,110,223,189,
  27,55,109,222,76,159,0,161,218,43,193,115,170,235,154,20,133,18,143,199,139,13,23,57,252,244,97,252,254,237,119,241,193,251,239,226,237,179,231,145,
  205,102,241,240,193,3,88,151,73,195,163,33,147,245,46,193,32,165,68,108,168,80,20,165,88,183,200,125,247,221,95,156,155,159,199,192,192,0,58,58,
  215,225,253,19,39,113,113,110,30,165,154,5,199,247,131,56,148,80,12,176,47,9,201,61,62,164,233,218,184,169,254,22,222,177,115,39,142,31,255,4,
  137,68,2,134,25,3,20,224,181,247,63,132,161,42,40,65,165,67,77,81,177,22,236,59,44,118,158,37,243,59,1,184,11,142,100,83,119,119,177,82,
  46,99,177,88,132,25,139,65,211,200,64,191,88,169,160,80,179,17,163,118,132,48,114,17,51,134,197,109,217,177,44,104,170,134,166,230,230,34,95,164,
  163,189,29,249,133,2,109,104,211,248,43,66,44,164,36,88,137,185,147,165,64,228,104,90,90,132,177,172,110,196,80,42,151,97,26,38,87,10,6,121,
  55,81,9,113,114,191,67,125,214,29,97,238,120,253,24,1,18,100,127,150,4,125,28,220,213,43,55,217,236,189,110,238,235,202,57,119,26,19,6,0,
  64,153,107,238,12,142,26,153,36,78,124,176,70,198,112,120,245,229,93,60,231,46,42,196,56,240,252,42,117,252,35,191,200,108,136,89,87,123,21,48,
  72,76,99,47,50,119,196,45,235,186,65,222,182,214,96,168,41,220,69,34,115,199,99,56,4,181,84,157,230,206,182,237,108,153,141,101,121,153,74,200,
  119,18,42,144,16,69,125,158,39,36,244,218,65,206,29,123,88,134,12,21,154,164,181,131,31,1,238,120,136,155,19,138,100,166,9,189,212,80,177,84,
  44,214,63,187,174,140,143,101,229,177,44,224,134,195,188,93,183,254,20,177,153,35,93,136,165,97,142,143,143,101,35,223,126,235,114,238,132,209,184,27,
  225,154,33,253,90,54,167,108,125,223,71,83,54,139,161,193,129,21,153,187,12,115,203,50,230,78,76,170,102,233,48,220,115,194,200,78,1,220,141,143,
  143,161,90,173,102,195,59,99,58,2,220,45,237,219,127,19,18,169,20,97,238,172,26,63,69,98,20,138,75,243,182,83,201,36,160,40,24,28,184,12,
  203,178,50,81,219,111,122,5,67,197,18,107,168,176,184,106,208,150,192,157,231,186,40,44,46,98,226,234,21,248,190,159,105,180,199,167,175,97,214,184,
  212,222,209,9,211,52,120,22,116,181,82,197,252,252,28,3,119,153,213,208,74,26,127,224,159,63,26,230,110,73,100,238,0,112,205,157,208,80,241,197,
  78,215,246,158,158,165,155,123,15,160,107,199,14,232,134,206,153,59,207,117,235,53,119,103,207,226,248,177,143,51,159,107,145,7,31,120,112,233,63,253,
  135,191,198,249,19,39,240,159,95,124,9,147,115,243,18,184,227,110,217,232,156,187,204,170,215,164,183,183,119,233,177,199,191,140,115,23,46,194,43,151,
  177,99,237,90,232,186,33,129,187,120,44,134,88,44,38,128,187,49,17,220,45,173,184,72,79,207,230,165,231,158,127,30,223,249,238,247,241,247,191,124,
  25,45,29,29,184,109,247,46,116,181,146,107,33,139,250,2,112,167,105,50,184,139,197,227,75,13,23,121,238,185,231,240,219,223,190,142,185,185,57,12,
  12,143,224,248,229,1,172,223,208,133,190,109,219,0,207,129,170,233,130,82,93,17,96,17,139,66,169,1,240,177,247,198,6,13,21,119,223,115,207,210,
  220,252,60,250,47,95,70,58,157,66,34,153,196,59,167,207,98,104,54,135,161,169,41,18,137,162,40,116,76,174,9,130,50,89,84,102,89,22,146,201,
  4,54,117,247,44,213,45,178,115,231,46,124,122,226,36,18,137,4,52,221,64,60,22,67,161,80,192,75,175,255,30,35,139,75,48,19,105,33,10,80,
  137,136,65,147,67,140,219,218,67,245,99,155,186,187,151,106,150,133,194,226,34,237,152,35,231,222,48,12,148,60,192,52,99,17,224,206,151,42,47,69,
  73,156,109,217,208,53,29,77,205,205,75,124,145,214,53,196,45,75,24,59,131,130,59,226,22,144,192,29,16,153,208,43,210,33,108,81,199,117,208,218,
  186,38,248,198,199,99,38,239,89,214,69,82,77,2,119,42,39,189,60,223,143,32,55,233,135,59,14,55,84,16,9,61,7,119,228,183,144,226,24,67,
  37,118,162,223,55,64,143,94,96,164,12,133,24,107,30,248,233,229,57,119,16,189,14,66,90,98,228,220,151,163,71,89,150,232,56,65,206,7,217,190,
  133,11,207,236,2,154,174,69,12,151,21,41,80,71,74,73,142,200,185,115,169,177,146,140,101,5,205,29,11,49,54,26,33,120,177,161,56,66,88,201,
  126,123,151,74,226,28,186,231,75,110,89,219,182,51,229,82,137,27,91,56,184,243,3,91,8,164,134,182,16,52,21,132,100,140,212,137,52,84,204,138,
  134,10,238,251,137,162,103,229,161,166,100,201,117,108,30,91,78,13,21,25,105,145,241,241,177,12,3,119,141,212,179,242,41,114,120,60,138,99,59,92,
  65,200,244,173,97,67,133,12,238,34,12,21,1,184,243,234,192,29,147,37,218,54,105,205,107,110,110,102,204,93,166,17,184,75,159,61,115,138,130,59,
  53,8,212,241,152,129,197,145,67,13,217,88,214,178,160,209,1,192,85,194,220,101,194,59,99,42,2,220,45,31,184,233,32,111,168,176,44,139,235,31,
  196,56,26,199,38,178,184,68,60,14,69,85,49,60,52,136,106,181,26,9,238,82,43,128,187,101,17,220,217,182,5,199,38,223,5,5,65,31,135,224,
  150,77,55,218,227,83,171,33,150,108,54,187,220,186,166,141,188,203,11,110,217,133,60,215,220,165,87,67,43,169,127,29,224,174,181,181,117,121,207,222,
  189,232,233,238,225,154,187,229,82,9,249,249,60,134,134,6,113,225,252,185,47,126,186,118,239,186,97,249,191,123,246,25,204,23,10,200,217,14,242,249,
  188,164,185,99,221,178,161,134,138,107,191,240,55,223,118,251,242,255,246,63,255,21,50,201,20,94,127,245,21,124,239,119,175,195,53,40,214,162,195,77,
  133,130,187,136,134,138,186,133,52,0,166,248,15,14,29,58,180,252,216,163,143,226,63,127,255,135,104,77,38,176,62,147,198,204,98,17,139,213,42,97,
  144,12,147,254,49,160,27,68,36,187,88,44,194,52,77,108,232,218,136,249,185,220,95,87,42,149,255,181,225,133,239,235,235,91,190,251,158,123,240,119,
  127,255,29,12,126,118,1,103,7,135,208,210,209,129,190,237,91,145,49,141,186,172,21,113,44,187,92,98,221,178,55,193,52,205,229,200,69,76,211,92,
  62,244,232,163,120,241,165,159,163,82,46,161,101,77,59,62,185,120,17,5,199,69,247,186,117,104,73,167,225,251,158,84,32,33,159,119,5,150,69,222,
  225,247,134,234,199,248,221,245,165,67,143,97,96,96,144,23,168,25,134,137,90,205,194,59,199,63,129,187,88,192,124,165,2,35,158,148,118,76,62,150,
  229,212,148,11,215,241,145,76,165,176,161,107,227,242,149,241,177,52,63,18,77,211,150,187,186,54,224,244,233,211,72,38,147,116,128,166,34,17,139,225,
  196,103,151,112,110,118,14,102,34,205,147,97,69,246,174,174,91,139,62,145,59,59,215,202,167,107,231,206,93,88,40,20,80,171,17,191,15,51,180,104,
  116,60,107,232,6,84,37,104,103,21,48,11,167,165,196,97,14,177,185,25,136,199,227,203,124,145,150,150,22,228,114,243,220,212,173,27,6,213,222,233,
  2,169,163,73,38,74,38,234,147,165,64,194,126,227,185,60,181,143,107,238,42,149,50,101,237,52,26,36,18,180,18,7,115,119,70,209,122,114,59,130,
  36,199,114,185,66,77,236,150,133,101,219,80,84,29,154,216,239,203,93,227,162,37,39,224,128,195,192,194,163,130,62,198,119,129,82,136,1,184,43,87,
  8,201,169,106,82,199,114,29,184,139,96,238,216,94,207,40,17,219,113,132,114,9,193,45,187,88,88,128,166,169,129,230,78,11,103,222,200,162,62,87,
  172,124,23,175,3,109,168,136,138,66,193,228,228,4,60,145,185,171,123,201,137,106,102,13,211,130,65,206,135,99,219,80,21,165,174,91,54,77,52,119,
  154,68,11,50,246,58,252,61,8,215,239,50,57,28,95,64,85,105,191,147,149,150,193,221,236,44,245,181,67,66,241,245,223,3,79,26,141,179,180,24,
  199,14,18,122,19,137,4,230,243,243,245,207,174,209,225,193,116,105,185,68,106,120,152,161,59,76,207,138,179,119,49,2,152,185,103,169,101,221,247,60,
  204,76,79,167,163,53,119,159,18,41,92,29,184,115,29,158,55,20,46,177,99,47,61,172,64,34,189,90,183,108,181,90,77,179,40,20,206,220,137,186,
  211,136,26,18,135,114,96,134,73,28,129,253,164,91,54,125,77,224,142,133,24,87,43,213,32,231,78,36,252,89,207,137,170,34,153,72,192,117,29,12,
  92,190,92,183,192,170,144,168,107,227,166,229,117,235,214,193,52,12,184,30,121,71,183,29,7,190,96,201,113,28,7,185,217,25,228,114,185,244,63,9,
  119,197,227,241,229,142,142,78,196,168,68,196,117,72,108,41,237,150,93,149,148,251,87,4,238,54,93,127,221,114,123,91,7,54,108,14,52,119,229,50,
  233,150,29,29,27,99,154,187,47,6,238,20,69,89,254,223,255,195,127,192,129,173,91,240,234,7,71,80,160,35,16,54,150,141,232,150,77,127,174,35,
  217,185,107,215,242,55,159,127,30,59,54,109,66,101,46,135,197,220,44,222,58,123,30,169,84,82,136,66,81,161,25,6,154,50,89,236,218,117,3,174,
  91,127,221,50,117,203,174,126,11,247,245,245,45,63,255,141,111,224,165,95,254,10,112,108,252,47,223,120,14,3,131,131,248,225,219,239,160,166,104,48,
  13,83,138,161,99,155,25,115,203,210,250,177,244,74,99,217,229,167,15,31,198,15,254,241,135,24,25,26,194,84,110,14,67,179,57,244,92,127,61,177,
  35,208,169,41,211,26,177,118,10,81,115,119,99,239,65,196,40,128,168,91,68,81,148,229,195,79,31,198,107,175,190,134,233,233,25,222,49,247,233,240,
  8,180,120,28,241,152,9,197,119,161,233,129,168,79,130,118,74,0,238,110,108,4,238,238,189,247,62,210,45,59,52,140,116,58,5,85,211,144,50,77,
  12,12,141,224,101,219,194,201,145,81,36,146,41,161,219,44,28,253,239,243,212,253,120,34,129,174,141,155,150,199,199,70,249,126,226,43,138,178,188,99,
  231,78,124,122,226,4,226,137,56,87,209,232,186,129,90,173,134,19,67,35,4,220,25,49,169,103,35,10,220,177,137,105,71,103,39,20,69,9,112,87,
  119,119,15,42,213,42,150,150,150,17,143,37,56,239,165,235,132,189,11,198,178,225,153,163,95,167,185,243,92,50,157,48,13,147,159,114,73,115,167,210,
  36,234,168,217,59,57,77,106,29,184,243,5,96,225,10,172,145,69,27,42,248,53,49,204,24,150,75,101,152,70,76,122,61,16,243,182,101,98,205,111,
  12,44,4,246,66,202,185,35,40,195,133,110,232,50,184,147,248,95,212,201,177,124,97,106,42,110,110,182,227,64,87,67,245,99,165,210,50,17,86,104,
  178,168,79,81,53,161,64,66,142,225,240,67,229,5,108,4,200,146,74,24,93,200,23,41,144,238,43,1,220,5,45,121,138,42,218,114,228,107,16,30,
  203,178,65,53,3,119,82,20,202,244,212,36,201,166,19,128,182,28,230,141,16,176,240,234,231,190,212,10,234,121,62,137,85,209,52,20,22,242,18,115,
  151,170,84,136,147,38,192,191,242,88,86,2,216,190,23,137,30,73,6,119,141,98,53,15,197,98,49,21,201,220,137,118,92,177,134,68,36,214,234,143,
  130,33,23,75,202,185,139,50,84,164,88,136,49,194,21,24,82,121,23,19,84,202,115,95,102,71,208,168,128,185,97,20,10,235,150,85,84,69,208,154,
  70,92,100,135,24,42,36,106,144,14,54,133,40,148,84,35,112,151,58,123,250,4,84,133,148,219,249,44,204,37,92,164,226,10,216,139,194,83,150,115,
  119,101,124,12,181,106,53,21,222,180,146,225,157,44,145,72,150,246,247,30,32,154,187,74,21,150,109,133,220,202,206,74,57,119,169,168,157,49,185,2,
  115,87,234,228,221,178,84,115,103,215,119,203,22,23,11,184,114,245,170,116,138,174,121,17,129,185,43,181,181,181,147,18,22,69,129,235,185,168,86,107,
  152,159,203,177,177,108,106,53,180,146,252,215,1,238,246,108,217,82,218,181,121,51,150,52,13,215,111,32,197,108,17,245,99,95,236,116,41,138,82,250,
  139,127,243,151,248,171,103,15,99,244,204,25,252,244,131,35,88,246,253,200,250,49,58,150,77,125,174,35,217,189,123,79,233,133,111,189,128,177,177,113,
  124,252,201,167,72,86,43,200,229,102,49,52,147,67,50,153,228,57,119,108,44,187,99,231,46,172,91,183,190,244,230,27,191,71,169,180,156,90,245,154,
  220,118,251,237,165,191,248,55,127,129,223,188,242,26,190,247,221,239,224,147,225,17,180,116,116,224,134,13,27,16,143,145,42,140,68,44,78,199,178,166,
  164,185,187,251,222,251,208,218,218,90,90,113,145,157,59,119,149,158,120,226,9,124,239,123,63,192,208,208,16,58,214,174,195,224,208,48,236,116,6,187,
  54,117,161,53,153,32,70,74,46,36,87,120,67,197,210,114,192,220,197,226,241,82,35,112,87,122,242,201,39,241,235,223,188,138,220,220,28,210,233,20,
  98,241,4,230,230,230,48,59,57,137,233,60,177,169,145,169,105,120,168,73,254,151,105,135,194,13,21,252,154,60,244,208,195,152,154,153,193,240,48,1,
  119,154,78,250,205,20,69,193,175,143,124,132,249,233,73,120,154,65,247,28,1,216,241,148,56,178,231,216,150,203,26,42,74,227,99,163,193,126,162,40,
  74,105,243,150,45,56,121,242,20,101,238,104,253,174,174,195,52,77,76,78,77,3,102,2,186,25,147,211,97,105,120,43,139,153,103,239,251,150,101,161,
  173,189,93,62,93,61,155,55,163,82,46,99,105,169,200,43,224,52,22,73,163,233,136,197,227,156,212,169,31,205,210,2,41,150,239,229,146,144,86,211,
  136,33,155,205,150,2,205,93,91,59,114,243,121,232,186,33,45,32,102,220,4,190,107,95,208,123,5,5,106,117,156,139,231,162,69,28,203,234,134,65,
  13,21,166,180,128,166,201,126,83,113,106,45,50,119,97,90,202,113,92,90,63,38,130,59,207,133,234,249,4,255,178,140,59,233,195,195,146,56,212,45,
  32,110,106,204,45,43,129,187,229,37,210,45,107,72,24,88,161,167,73,0,119,18,115,231,213,141,4,89,153,71,224,150,13,119,203,2,28,61,106,82,
  5,114,144,117,231,173,50,18,12,98,56,108,120,62,34,52,119,30,99,238,196,164,106,72,23,153,211,82,17,176,136,117,154,58,84,45,165,107,42,239,
  118,96,110,217,84,133,118,203,202,174,113,69,176,166,251,13,100,137,14,39,214,152,101,132,129,187,106,181,114,13,99,217,168,56,154,58,114,51,8,210,
  179,109,50,150,77,38,18,40,20,10,124,159,169,3,119,186,166,202,10,14,225,254,151,91,242,68,199,6,225,189,28,219,34,222,97,85,197,196,196,213,
  232,167,240,137,19,159,192,247,193,221,178,126,4,69,238,132,132,149,162,168,15,0,239,150,109,8,238,106,213,106,234,252,185,211,188,126,76,190,139,156,
  6,212,32,57,2,54,150,29,31,27,133,109,219,215,6,238,122,15,28,128,25,39,224,206,182,173,144,44,212,169,235,150,21,234,199,254,11,128,59,177,
  91,22,68,43,185,68,114,238,82,255,36,220,149,205,102,75,98,136,177,237,216,176,106,22,114,185,28,170,213,202,127,3,119,193,79,123,123,123,233,75,
  119,223,131,172,166,96,210,241,176,102,77,43,103,238,132,156,187,47,14,238,30,255,202,215,240,236,151,31,197,158,205,61,56,246,230,27,120,233,228,105,
  210,130,68,211,73,56,115,119,250,12,62,253,228,216,231,187,240,59,119,238,42,125,251,207,191,141,169,137,73,28,63,117,26,255,195,215,159,64,121,106,
  2,223,251,221,239,73,236,108,44,22,169,185,251,253,235,191,67,165,82,94,29,220,245,245,245,149,254,237,127,247,111,241,187,215,223,192,79,126,246,51,
  92,29,31,195,88,169,132,206,246,14,172,93,179,6,166,105,34,25,79,16,115,101,200,80,113,239,253,15,160,169,185,121,101,112,183,101,235,214,210,211,
  135,15,227,135,63,252,17,46,93,186,132,76,182,9,154,110,96,118,122,6,137,150,22,108,104,111,135,2,159,70,208,17,90,35,12,238,122,111,234,91,
  25,220,61,115,248,25,188,246,234,107,152,18,166,166,49,211,192,213,169,41,140,204,204,226,74,46,135,24,21,147,203,141,236,50,184,107,200,220,221,115,
  207,189,152,155,159,199,192,208,48,82,169,20,17,47,105,42,116,35,142,217,201,41,252,116,122,26,11,229,10,18,137,4,253,165,16,248,28,132,124,47,
  219,114,89,20,74,105,108,116,68,126,212,239,220,181,11,199,63,249,36,24,135,235,6,116,157,116,201,87,28,7,75,142,135,100,42,29,17,228,166,72,
  109,224,12,220,181,135,53,119,61,61,155,75,140,185,51,99,166,64,172,49,223,131,1,67,23,135,154,92,180,38,13,55,69,112,71,239,188,82,29,115,
  167,105,52,223,67,96,139,216,111,206,122,77,249,2,116,247,36,1,34,50,52,114,104,252,105,235,154,182,224,154,196,98,113,44,151,136,230,142,131,59,
  166,220,20,236,184,160,134,98,145,247,10,183,35,48,135,160,13,200,134,10,203,182,161,155,2,115,167,234,117,126,135,96,246,30,61,115,116,88,205,59,
  45,142,210,125,59,66,115,7,202,220,169,186,112,138,180,144,115,38,130,26,20,140,172,142,43,118,11,132,162,80,138,197,34,205,125,20,34,129,84,45,
  66,84,233,73,11,132,29,179,92,107,79,173,8,140,241,14,52,119,174,75,220,178,154,232,218,168,183,198,137,41,201,225,185,47,111,74,160,13,21,82,
  20,138,109,219,169,114,169,36,188,69,41,146,169,175,46,45,49,194,115,226,48,112,231,6,154,187,82,169,36,127,25,115,179,51,17,204,157,31,212,142,
  185,33,181,26,35,217,28,91,200,253,176,36,205,93,36,115,183,196,235,199,234,125,239,193,107,65,64,9,122,44,239,195,14,250,205,86,101,238,78,159,
  248,4,190,71,220,178,82,67,177,152,150,40,160,71,199,145,169,65,198,220,13,173,198,220,157,57,117,130,75,22,234,126,123,207,11,58,54,28,153,185,
  83,85,21,173,148,185,171,94,43,115,199,193,93,181,42,105,238,108,177,196,214,113,224,3,72,254,83,152,187,158,158,205,165,181,235,214,67,213,180,160,
  161,194,182,201,83,23,128,170,40,140,255,101,154,187,47,206,220,53,53,55,151,90,91,215,32,78,3,213,93,207,13,187,101,255,27,184,3,155,68,148,
  158,124,234,105,220,181,107,39,38,134,134,48,230,184,48,99,49,204,204,204,138,13,21,95,236,116,153,166,89,122,254,249,111,96,199,206,29,104,77,38,
  209,219,221,141,215,95,125,5,111,94,186,140,230,150,22,36,83,41,180,52,55,163,92,169,224,236,217,115,248,244,248,199,13,23,139,60,146,190,155,111,
  41,61,255,252,115,232,239,191,140,239,124,247,251,184,238,186,235,177,225,235,95,5,0,44,228,114,88,88,90,134,174,170,196,3,145,73,99,231,206,157,
  88,191,110,29,222,124,227,245,18,123,94,173,120,77,30,124,224,193,210,243,207,63,135,159,254,244,167,248,213,175,126,5,199,182,73,38,100,58,131,181,
  173,45,124,254,27,139,39,96,152,6,106,53,75,96,238,238,95,29,220,245,246,246,150,30,120,232,65,252,95,127,243,119,152,152,152,68,38,219,132,88,
  34,129,74,181,130,82,161,128,230,76,22,25,41,219,54,26,220,153,166,25,13,238,116,93,47,61,254,149,175,224,231,191,120,25,203,75,75,72,80,197,
  154,161,147,81,199,149,233,105,28,61,119,30,21,203,130,174,10,49,116,28,28,130,24,42,124,212,133,24,243,11,255,216,227,95,45,53,55,55,225,221,
  119,223,67,38,147,129,170,169,4,123,81,205,99,58,30,67,217,178,96,152,49,185,145,50,148,49,225,122,30,52,77,199,200,208,0,174,140,143,165,68,
  205,93,169,123,211,70,156,58,125,6,201,100,146,226,44,2,238,52,157,232,29,45,207,71,44,158,8,101,124,201,51,71,150,107,111,91,22,218,59,66,
  224,110,251,246,29,68,115,87,173,112,112,167,9,110,1,38,139,19,185,71,194,119,5,249,94,98,198,132,101,91,48,13,19,113,10,188,105,206,221,26,
  228,243,11,196,204,173,233,18,246,18,205,196,98,4,51,159,251,250,161,232,89,70,238,120,94,189,230,174,84,42,145,102,48,145,88,83,131,124,103,145,
  208,9,152,35,55,90,107,79,131,219,36,205,93,205,178,73,120,52,211,220,69,128,59,73,146,24,81,160,38,138,250,108,215,3,96,51,188,160,80,205,
  93,153,236,205,170,22,128,59,77,109,8,238,252,168,5,120,5,169,195,171,0,228,134,138,197,2,161,3,117,45,228,55,17,231,141,98,97,106,148,36,
  78,236,148,167,99,217,106,77,100,238,166,40,219,166,243,185,175,232,26,15,8,206,198,146,56,209,169,25,169,185,171,84,202,169,106,181,202,193,93,240,
  77,86,120,214,138,95,231,216,144,37,113,65,13,22,9,210,115,35,53,119,124,44,43,74,67,253,58,73,156,168,30,112,56,91,36,112,145,182,141,100,
  50,137,66,40,231,78,1,160,140,142,12,167,75,203,37,154,83,39,135,24,75,196,154,0,178,153,223,151,181,131,89,181,26,12,195,0,72,206,93,180,
  230,238,12,173,31,211,84,85,234,57,225,163,113,9,197,7,29,27,142,29,248,29,50,153,12,250,251,47,53,126,212,151,74,165,180,8,238,196,40,148,
  48,130,15,119,108,24,6,137,123,24,184,220,127,109,154,187,68,34,185,188,191,247,0,169,31,171,84,80,179,106,252,98,19,214,46,32,57,21,133,140,
  101,225,251,232,39,65,122,159,79,115,183,169,187,123,57,136,66,9,114,238,152,230,78,81,8,148,93,200,207,75,215,224,11,201,225,18,137,228,114,71,
  39,113,203,178,15,174,85,107,152,157,157,225,183,233,127,211,220,241,83,117,211,77,55,225,208,35,15,99,147,231,226,200,249,243,200,169,58,74,165,50,
  70,135,135,112,225,194,249,47,174,185,75,165,82,203,207,62,251,44,118,236,220,137,169,137,73,236,232,233,198,230,108,6,191,120,245,85,12,20,203,104,
  111,95,131,108,38,131,154,101,137,221,178,233,107,62,146,219,110,191,125,249,240,225,195,248,236,179,75,248,206,247,126,128,185,217,105,60,249,228,215,177,
  97,239,110,16,17,243,36,233,100,162,134,10,58,150,197,155,111,188,190,28,165,185,171,187,38,135,14,29,90,126,226,137,39,240,163,31,253,24,191,249,
  205,111,96,91,22,50,77,45,68,128,148,72,32,110,152,48,12,141,143,101,195,224,174,181,181,117,121,197,69,2,67,197,63,96,122,122,154,128,187,88,
  28,134,97,160,86,37,102,187,214,108,6,186,170,240,232,83,6,238,152,161,98,239,254,3,215,104,168,168,84,144,72,166,248,7,152,134,129,165,82,9,
  39,6,6,113,188,127,128,118,60,104,161,202,113,194,220,41,138,82,103,168,224,23,254,43,95,125,98,57,157,78,225,189,247,63,160,224,78,163,73,125,
  196,174,78,26,112,93,40,170,10,147,66,213,0,185,136,180,148,7,77,215,49,60,120,25,87,175,140,215,27,42,206,156,57,131,116,42,197,11,132,153,
  145,197,160,189,242,70,60,70,166,167,170,34,226,10,206,220,17,107,168,7,199,170,241,216,223,58,67,69,181,90,131,102,144,169,41,159,150,170,66,28,
  141,18,50,82,34,170,161,194,163,154,187,24,18,137,100,32,236,107,93,211,202,13,21,124,44,75,213,131,65,222,77,80,253,26,212,143,201,17,192,174,
  72,40,120,46,214,180,9,204,157,166,106,168,84,202,136,197,98,117,102,86,37,100,137,102,33,110,204,80,25,69,234,120,174,3,87,81,194,33,198,14,
  124,133,82,130,98,20,54,191,131,66,65,198,66,136,69,148,98,205,165,45,185,18,115,183,84,42,211,187,73,92,64,13,85,144,9,161,58,190,216,43,
  224,213,41,214,130,0,93,87,54,84,72,224,142,27,139,20,1,88,71,196,208,9,113,191,174,4,147,136,144,73,50,84,76,79,77,145,222,63,221,168,
  139,4,146,199,227,110,93,51,130,232,16,224,122,47,199,129,170,40,152,153,153,150,192,93,186,26,50,84,200,195,229,104,97,101,24,7,219,142,13,219,
  33,64,155,114,145,233,8,230,206,144,114,86,32,48,119,81,89,43,82,82,9,61,109,14,53,84,228,40,255,40,45,50,50,60,68,193,157,206,181,167,
  46,253,38,7,174,113,71,130,164,158,107,7,167,139,50,119,49,211,4,124,95,18,243,215,229,220,145,110,89,69,184,139,234,17,124,16,238,18,164,196,
  89,150,69,98,128,83,169,149,193,93,165,82,78,159,57,117,130,39,89,186,194,109,202,191,213,142,205,67,12,197,35,48,229,16,227,107,55,84,48,112,
  39,70,161,56,142,205,125,240,174,235,65,129,79,66,140,93,23,151,35,208,227,53,128,187,158,229,206,206,78,24,102,140,128,59,199,130,75,193,157,239,
  251,80,84,21,174,227,96,118,102,250,159,110,168,72,36,146,203,36,231,206,164,233,200,14,170,65,136,241,127,3,119,193,143,166,105,203,125,7,251,208,
  213,221,141,7,110,191,13,218,244,36,222,58,125,22,115,182,131,241,241,113,92,186,244,217,170,224,78,95,233,195,159,121,246,57,236,223,191,15,133,133,
  2,70,199,199,177,182,165,5,42,125,179,109,107,111,199,182,237,219,113,219,29,119,224,220,217,179,203,199,143,125,252,249,12,21,251,247,239,95,126,225,
  91,223,194,248,232,24,126,248,163,159,160,80,88,128,170,170,56,120,211,1,108,168,86,145,95,90,194,244,244,44,226,177,184,168,185,91,166,154,187,240,
  98,245,109,109,119,221,125,247,242,147,79,62,137,151,94,124,9,35,163,196,21,144,74,167,161,40,42,146,116,19,138,233,6,12,211,131,25,139,161,90,
  171,97,116,108,28,217,76,26,119,223,123,31,62,62,250,225,114,62,159,207,52,188,240,123,247,222,184,252,248,227,143,227,239,255,225,187,24,29,191,130,
  116,38,139,88,60,70,121,47,29,153,4,153,249,50,139,21,219,247,73,206,93,153,106,238,14,194,52,205,200,40,20,69,81,148,229,39,158,124,2,191,
  252,229,203,88,44,44,32,157,201,240,86,92,85,35,181,187,87,70,71,241,206,39,159,98,177,86,163,115,199,96,83,19,132,225,145,204,93,154,98,224,
  165,206,206,181,120,227,173,183,145,201,100,72,53,140,48,212,84,52,13,138,170,2,46,137,211,208,116,131,3,139,240,166,38,48,119,25,81,115,183,180,
  121,203,86,156,60,77,192,29,159,251,82,120,164,105,228,136,116,85,69,44,30,135,166,27,17,41,189,129,161,194,182,44,116,174,13,185,101,183,110,219,
  134,197,226,18,74,229,50,116,51,16,245,105,66,20,13,99,240,36,191,175,64,174,5,208,200,131,77,153,187,84,42,181,36,140,101,219,72,67,133,174,
  211,40,20,173,206,84,33,6,137,112,80,33,138,250,92,87,34,20,108,199,193,154,182,118,193,80,97,24,168,84,42,36,130,38,140,189,84,69,216,247,
  197,140,15,200,106,41,223,149,64,30,194,99,89,219,118,97,40,129,161,88,60,69,34,176,136,178,130,70,129,59,219,113,224,251,161,40,148,229,210,50,
  160,168,82,67,69,125,4,93,163,185,111,144,82,198,19,123,93,7,190,231,200,204,221,98,161,64,148,80,70,56,10,80,20,244,5,175,8,4,100,8,
  65,46,212,243,43,22,71,213,231,220,77,79,193,243,252,208,133,22,26,194,148,208,43,130,27,120,77,37,75,8,239,25,32,230,140,92,200,45,155,169,
  148,75,156,189,14,159,38,46,234,115,61,233,8,60,209,18,66,89,35,203,170,17,181,20,105,168,144,83,74,230,114,179,48,140,128,11,110,136,127,189,
  224,8,28,215,229,37,131,12,185,184,180,161,98,33,202,45,59,50,60,148,41,45,151,160,235,90,157,231,36,220,142,27,188,139,56,84,88,233,115,106,
  80,215,117,168,228,93,63,179,10,184,83,235,83,146,5,190,215,117,105,208,161,195,78,149,13,171,86,3,20,21,217,76,6,253,3,151,87,4,119,25,
  6,238,88,20,74,189,254,129,181,82,56,220,177,108,91,22,12,195,64,107,115,19,6,6,6,96,219,118,38,242,41,28,130,64,75,189,7,14,240,40,
  20,18,98,28,164,245,48,106,208,118,108,50,128,73,36,160,2,184,116,185,31,142,227,124,177,156,59,166,185,115,28,7,150,109,7,204,157,170,194,163,
  189,142,87,175,94,249,124,113,103,17,140,17,207,185,83,20,149,166,197,216,172,91,54,115,45,224,238,143,47,231,78,81,148,165,207,253,223,172,118,36,
  137,68,114,169,239,230,62,108,220,212,131,246,214,22,28,188,97,23,134,79,124,138,15,46,92,196,248,124,30,23,9,115,151,249,66,224,46,149,74,45,
  61,247,252,243,188,126,108,116,108,12,51,115,115,88,155,205,160,220,218,2,37,149,194,221,187,247,224,150,91,111,197,133,243,23,150,62,254,232,72,230,
  115,45,114,251,29,119,46,61,253,244,83,248,236,179,75,248,222,247,190,143,165,229,18,84,85,193,230,45,91,177,100,217,152,206,47,32,63,51,131,185,
  194,18,154,178,25,108,217,186,5,157,157,157,75,215,156,115,119,232,208,161,165,175,125,237,171,248,193,63,254,8,175,188,242,10,106,150,141,84,58,141,
  4,181,130,100,232,110,7,0,241,88,12,150,101,99,252,202,85,184,158,135,59,238,186,7,173,173,173,75,171,49,119,75,44,10,37,55,59,131,116,38,
  195,197,252,170,162,162,169,41,136,219,118,84,157,183,81,50,183,172,16,133,178,196,166,25,124,166,69,95,225,150,14,61,250,40,94,124,145,68,161,36,
  83,105,106,71,160,144,200,208,81,171,213,240,254,199,199,112,102,104,24,186,174,67,140,255,149,12,21,17,204,93,134,50,119,197,116,58,133,247,63,248,
  144,106,238,116,106,102,21,170,175,84,5,158,235,146,80,61,90,92,84,7,238,92,15,154,174,97,120,112,0,87,175,140,103,69,230,174,216,213,181,1,
  167,207,156,65,42,153,164,154,187,224,195,53,97,176,105,196,226,196,74,165,42,66,174,170,80,222,69,235,199,234,152,187,29,44,10,165,90,131,110,138,
  240,84,141,76,138,19,147,249,153,230,222,117,130,168,13,102,168,96,1,211,188,126,44,96,238,12,142,127,195,204,157,232,119,240,34,146,249,165,242,2,
  207,229,77,185,100,44,171,233,40,211,40,20,177,130,68,140,161,11,251,29,8,185,38,71,0,139,156,75,4,184,179,225,43,154,140,127,67,71,160,8,
  162,62,222,169,229,123,145,67,77,199,117,160,192,227,71,78,38,166,203,37,104,154,42,27,42,66,71,224,123,190,84,1,199,249,199,8,230,46,232,58,
  19,193,221,226,34,161,104,53,177,134,47,212,239,139,16,176,112,153,177,181,94,177,22,25,133,50,61,121,149,212,143,105,186,96,203,9,26,194,68,237,
  169,40,240,19,123,53,100,197,26,33,214,36,230,174,90,173,102,107,181,154,224,150,141,232,247,173,107,168,240,36,112,199,20,107,182,109,67,81,53,198,
  30,101,35,199,178,12,104,179,91,52,156,18,199,174,129,172,88,11,192,158,99,219,72,198,99,13,153,187,108,105,105,9,154,26,146,134,134,177,151,235,
  68,140,198,105,130,156,109,33,102,154,240,9,115,151,141,6,119,39,233,180,148,129,59,81,88,233,58,114,144,49,155,154,82,237,157,85,171,65,211,180,
  213,163,80,42,149,114,86,4,119,46,87,15,200,73,213,54,61,53,54,107,42,182,106,136,209,40,148,75,36,10,37,27,249,20,14,49,119,69,214,80,
  81,171,214,96,89,53,206,212,121,158,195,13,21,14,229,127,67,245,99,217,40,32,209,112,111,222,184,169,187,184,110,221,122,218,45,203,156,75,161,40,
  20,219,70,46,55,43,93,131,207,181,136,128,88,138,237,237,237,48,41,115,23,138,66,201,94,11,36,202,252,209,129,187,63,8,115,103,154,102,241,230,
  155,111,193,198,238,110,100,51,25,36,98,49,244,180,52,227,232,177,99,248,244,226,69,92,30,25,17,27,42,62,95,215,70,60,30,47,62,251,236,115,
  184,97,247,13,200,229,230,48,49,57,137,161,225,81,116,111,234,194,45,247,223,7,175,184,136,90,50,133,187,31,124,8,103,206,156,41,30,39,245,99,
  77,141,22,169,91,253,214,91,110,93,124,230,185,103,209,223,127,25,223,255,193,63,162,88,92,130,97,18,129,70,215,134,245,88,156,159,195,84,110,14,
  163,35,163,88,90,42,97,235,182,109,88,187,118,29,222,124,227,245,197,82,169,212,116,45,154,187,197,103,158,123,22,63,254,49,209,220,85,171,53,164,
  210,105,164,146,41,196,98,49,92,119,221,117,228,49,238,56,80,53,13,149,106,149,131,59,58,150,93,92,77,115,183,248,192,67,15,226,111,254,230,111,
  49,53,53,137,108,83,179,228,212,52,77,19,215,173,89,3,171,82,65,213,182,160,8,104,38,4,238,22,27,105,238,22,185,230,110,121,153,140,101,117,
  141,99,47,85,81,145,136,39,80,152,153,193,192,224,32,174,206,229,185,152,198,247,3,112,167,40,74,36,115,151,165,224,110,49,147,205,224,189,247,222,
  231,134,10,198,123,177,205,76,83,53,24,154,194,187,1,52,35,70,55,52,185,242,82,211,52,140,14,13,98,124,124,172,73,4,119,139,93,93,27,112,
  230,244,105,36,147,9,50,138,165,162,62,157,17,106,116,123,118,161,66,51,98,208,140,184,0,42,130,8,8,214,165,213,214,72,115,87,169,86,105,206,
  157,30,208,82,82,198,154,224,150,13,233,188,197,97,142,29,140,101,23,37,205,93,46,55,47,169,6,69,73,156,184,239,51,230,40,156,49,204,246,28,
  190,45,123,46,218,219,219,101,205,93,181,90,33,134,138,134,21,200,34,184,243,56,184,107,52,177,139,214,220,209,215,3,41,142,49,212,239,16,16,58,
  81,219,178,80,125,229,68,230,220,149,249,171,66,128,224,41,246,18,132,252,8,79,174,133,109,89,180,33,218,14,117,207,134,115,238,0,132,166,165,244,
  52,241,126,179,168,153,163,27,41,137,115,40,29,34,129,187,169,169,73,248,190,39,128,59,77,18,87,66,105,172,123,100,64,219,230,102,10,242,207,149,
  136,156,187,166,42,141,66,33,65,122,162,152,15,156,122,242,35,236,184,108,98,231,179,156,59,215,229,204,157,101,89,77,43,107,238,234,222,162,130,87,
  4,39,4,176,25,115,71,104,169,26,209,220,209,64,23,105,145,209,145,225,38,2,238,200,136,156,1,236,168,225,190,23,6,216,78,208,239,64,52,123,
  30,38,39,39,155,86,1,119,74,125,90,12,195,94,98,67,155,128,30,153,99,35,155,201,224,114,127,63,194,175,216,252,111,42,149,114,179,52,150,141,
  210,61,242,150,226,32,231,206,170,213,96,26,6,154,155,8,115,119,77,221,178,241,120,188,32,142,101,197,156,59,219,33,46,38,155,94,11,69,81,145,
  76,72,154,187,230,40,72,212,180,194,88,182,176,118,237,186,192,45,75,143,128,143,101,21,5,158,109,35,95,88,144,250,125,63,215,34,2,184,43,180,
  119,116,240,134,10,215,117,97,215,106,152,158,153,129,101,89,205,215,2,238,254,21,117,203,238,223,191,31,155,186,123,208,218,210,12,215,39,226,202,181,
  241,24,222,122,239,61,188,127,226,196,170,221,178,250,74,31,254,181,39,158,196,205,55,247,161,82,169,96,98,98,2,11,133,69,44,45,149,112,207,29,
  183,226,241,219,111,131,94,41,163,249,250,46,76,79,79,21,142,175,208,45,27,185,200,174,93,55,20,254,236,219,127,134,92,110,14,47,253,252,151,200,
  231,231,161,168,36,149,204,243,60,224,142,91,49,51,62,142,225,201,41,20,138,37,108,223,177,147,129,187,66,169,84,106,94,117,145,190,190,190,194,243,
  223,248,6,126,243,202,107,184,244,217,69,196,19,9,196,19,137,96,98,167,27,232,94,187,22,133,169,41,204,45,45,195,178,29,97,44,123,63,62,62,
  250,97,33,159,207,55,55,188,240,91,183,109,43,176,156,187,75,151,46,33,157,201,242,5,8,137,6,172,235,104,199,218,108,6,133,165,34,22,233,55,
  93,52,84,236,59,112,16,137,68,178,16,185,136,162,40,5,150,115,55,51,51,139,116,58,205,49,23,99,141,0,32,157,78,97,104,116,12,23,71,199,
  81,243,61,104,90,224,119,172,213,44,248,62,176,231,198,125,209,223,147,251,238,187,191,176,109,251,118,188,250,218,111,37,230,142,213,47,136,1,186,26,
  200,183,221,245,1,85,211,121,112,190,56,150,29,27,25,194,216,232,72,179,56,150,45,236,216,185,19,159,126,122,2,137,4,3,119,76,177,22,128,59,
  50,49,85,225,169,58,124,149,116,60,177,66,2,182,169,249,174,11,203,170,161,189,163,3,138,162,20,248,34,221,221,61,168,148,203,40,44,22,104,67,
  133,81,103,71,224,251,191,162,64,83,229,24,8,79,138,161,243,225,88,22,116,205,136,200,185,91,40,240,107,160,9,53,100,162,36,78,81,2,51,171,
  24,98,225,135,3,20,232,130,212,45,171,232,0,20,195,140,161,184,188,12,211,136,73,126,223,168,185,111,112,4,88,17,185,144,40,148,152,172,185,211,
  61,159,130,59,77,34,55,3,252,235,11,148,84,3,199,134,152,111,235,41,33,112,87,42,65,85,16,120,76,5,230,142,179,167,62,3,217,43,204,125,
  197,140,9,74,186,5,244,96,177,8,40,74,208,80,17,158,152,210,227,112,67,158,235,198,224,206,129,2,95,54,84,76,77,78,240,22,123,153,34,132,
  164,26,172,43,16,142,112,108,176,125,95,211,52,110,69,96,110,217,150,74,185,204,175,5,179,68,73,131,229,21,117,143,162,29,193,227,90,201,134,13,
  21,124,184,239,5,61,39,128,47,69,161,52,90,128,209,82,201,68,130,53,84,180,72,139,140,143,141,182,44,45,22,249,52,193,231,101,130,238,138,11,
  72,25,195,86,141,191,13,175,144,115,71,234,199,84,78,114,70,68,2,133,192,29,187,70,142,77,222,124,155,154,154,112,153,68,161,180,68,46,98,89,
  86,203,217,211,34,184,115,165,5,88,166,176,152,119,99,91,53,78,13,182,182,180,96,108,116,4,150,101,181,132,159,194,205,17,224,110,225,192,77,7,
  185,91,150,143,101,217,11,14,79,238,161,81,40,201,4,52,149,187,101,91,162,32,81,67,16,208,211,179,121,161,67,24,203,214,108,155,127,225,88,211,
  78,200,45,219,210,8,119,173,10,206,178,217,236,130,216,80,33,186,101,75,165,82,203,181,128,187,230,63,16,220,82,174,25,119,173,68,150,125,30,112,
  167,172,162,185,203,247,29,236,67,207,150,45,124,19,90,46,149,96,104,26,22,198,199,241,218,7,239,195,178,172,214,47,170,185,203,139,154,187,241,171,
  19,152,154,154,134,77,239,168,175,61,242,16,54,223,123,55,174,111,107,197,209,129,193,252,169,83,167,224,251,126,235,231,209,220,229,95,248,214,183,48,
  56,56,132,31,252,227,143,80,44,46,146,30,39,90,63,102,198,227,200,36,18,152,25,27,195,213,185,60,182,110,223,129,13,93,155,240,206,219,111,230,
  139,197,98,235,170,128,251,174,187,239,206,191,240,173,111,225,231,63,255,57,126,249,203,95,162,82,169,32,153,74,35,153,76,34,65,3,93,246,238,190,
  1,107,179,25,140,76,77,97,166,84,194,244,76,78,172,31,203,175,184,200,238,221,123,242,143,63,254,56,254,225,59,223,195,200,232,56,209,220,197,226,
  60,161,87,163,2,254,189,123,118,195,171,84,48,57,63,79,102,97,20,220,213,44,11,123,246,245,34,30,143,231,27,129,187,252,215,159,250,58,126,245,
  242,175,144,207,231,57,184,99,51,71,93,39,114,184,84,58,141,116,173,134,209,203,253,132,189,139,153,2,115,87,131,239,251,145,99,217,22,0,120,248,
  225,71,242,235,175,191,30,111,190,249,150,60,150,101,35,65,161,188,214,208,52,196,76,29,149,82,5,48,204,149,192,93,171,8,238,242,44,231,46,136,
  66,9,122,29,52,77,23,114,214,84,56,158,143,82,205,1,12,35,18,185,56,1,184,203,243,69,182,108,221,138,229,82,25,165,210,50,97,238,116,77,
  64,143,245,205,172,170,170,64,85,80,39,5,98,79,108,203,178,160,107,58,146,201,148,0,238,90,215,96,110,110,142,215,141,53,4,119,144,29,179,34,
  184,19,153,10,86,49,222,38,49,119,6,73,229,49,77,51,210,239,16,160,199,128,146,226,187,166,199,200,53,87,138,227,176,109,91,54,84,88,182,13,
  69,55,3,152,170,202,245,99,162,83,51,24,205,186,60,188,213,141,136,128,168,11,49,230,221,178,82,191,89,200,212,237,203,0,59,172,181,23,193,29,
  175,96,34,147,83,194,173,144,177,172,66,29,52,161,83,196,142,32,196,26,133,227,103,101,236,69,210,49,44,81,115,199,192,157,70,193,157,196,255,2,
  145,41,113,242,188,81,6,119,174,235,66,211,52,153,185,179,109,123,13,3,119,178,114,211,151,34,128,163,236,184,124,238,43,228,124,40,96,57,119,213,
  53,13,198,178,138,132,30,163,225,169,35,229,60,187,158,47,21,72,36,226,113,158,35,29,6,119,107,150,150,150,248,91,174,207,101,10,114,206,157,19,
  82,110,18,207,9,49,179,90,84,177,166,168,42,174,94,189,178,38,26,220,125,114,172,78,115,231,133,114,238,68,106,208,247,2,131,139,67,77,197,89,
  154,115,215,240,81,111,89,214,154,160,161,66,21,20,28,242,41,18,163,80,28,58,115,212,117,29,173,45,45,24,26,26,132,109,219,107,194,79,225,214,
  8,112,55,31,30,203,202,11,4,224,78,81,20,162,185,83,85,244,247,95,170,91,160,225,34,194,88,118,190,115,237,58,196,12,29,182,227,194,114,108,
  56,182,195,153,59,214,45,155,39,110,217,53,43,65,157,214,107,96,238,230,59,58,59,169,161,66,133,235,58,168,213,44,228,136,230,110,205,181,224,169,
  86,252,129,127,254,57,152,59,229,90,152,187,249,131,7,15,162,107,83,55,90,91,8,236,45,87,42,200,205,204,160,180,88,192,248,232,40,198,167,167,
  219,86,59,93,107,26,124,248,220,147,79,61,141,91,110,238,195,242,242,50,174,92,157,192,66,62,207,67,140,227,241,56,254,252,203,143,98,104,116,12,
  223,255,233,79,241,222,167,159,182,125,46,4,185,123,247,158,185,23,190,245,2,166,38,38,241,147,159,190,136,249,249,57,9,220,121,190,143,13,215,95,
  79,126,153,197,5,92,191,118,45,30,123,252,171,115,239,191,251,54,138,197,98,219,170,215,228,214,91,110,157,251,139,127,243,23,120,245,181,223,225,39,
  63,251,25,138,197,34,25,203,166,82,136,39,146,136,39,72,244,198,158,221,55,192,171,84,112,113,116,28,3,115,4,102,81,112,55,183,226,34,219,182,
  111,159,123,242,169,175,227,123,223,255,1,6,7,7,145,201,54,193,140,153,82,74,134,162,40,104,110,110,193,254,173,91,80,152,154,194,224,212,52,204,
  120,60,12,238,230,26,129,187,185,167,159,122,26,191,121,229,53,228,114,115,117,204,29,43,237,116,61,15,219,183,110,198,218,108,6,151,198,198,48,83,
  42,193,208,13,62,150,245,125,162,185,83,20,101,174,238,154,60,112,255,3,152,153,37,33,198,41,106,102,229,224,78,144,99,105,186,142,145,225,97,252,
  67,46,135,218,82,17,154,25,147,54,53,171,230,32,158,72,98,227,166,110,140,142,12,67,4,119,115,91,183,111,199,137,19,50,184,19,13,21,140,216,
  209,53,29,85,219,197,72,190,128,73,219,131,65,23,225,192,194,247,225,88,181,250,110,89,198,220,21,151,138,228,26,232,154,32,234,171,215,218,171,170,
  2,93,85,160,41,138,0,238,130,109,185,102,243,110,217,57,137,185,99,33,198,156,185,211,3,197,160,200,220,5,173,160,62,23,243,179,194,46,79,12,
  207,117,29,180,180,180,10,134,10,51,134,74,165,202,115,238,152,230,78,83,53,105,106,202,245,142,92,208,68,170,95,69,112,71,134,152,14,108,223,175,
  215,220,241,156,59,73,115,23,193,123,249,144,192,29,147,187,59,34,122,116,61,232,116,43,224,178,209,74,165,18,209,80,161,201,81,128,136,118,108,184,
  130,153,149,133,24,251,130,122,157,93,19,133,32,11,69,170,31,99,0,27,161,34,149,250,172,21,33,87,130,206,185,108,10,238,234,66,140,29,199,230,
  33,198,236,110,66,104,114,42,166,197,176,217,187,91,55,61,13,218,193,230,231,114,18,184,107,151,67,140,69,91,78,216,142,235,201,105,137,108,223,167,
  19,59,206,220,145,156,187,246,6,204,93,120,192,191,122,129,26,51,85,144,138,190,26,18,137,56,150,162,114,238,198,199,70,219,3,230,78,224,128,35,
  249,95,49,76,186,190,29,76,211,244,107,97,238,84,142,224,197,172,149,192,204,26,4,136,16,252,229,242,34,15,150,84,226,251,126,123,35,112,215,126,
  234,196,113,254,232,112,93,193,41,203,202,106,29,91,38,55,105,28,160,174,235,104,9,152,187,246,240,246,219,22,1,238,114,98,253,152,8,238,2,215,
  120,208,45,203,66,140,47,19,112,215,30,181,199,183,53,30,203,118,231,130,40,20,202,220,217,86,224,90,162,224,174,176,184,136,137,171,87,164,83,116,
  205,139,8,66,254,28,113,203,198,249,35,165,86,173,137,33,198,237,171,161,149,182,47,138,167,254,139,145,106,255,44,99,217,246,246,246,217,125,55,238,
  67,87,119,55,154,51,25,88,14,49,22,207,207,231,49,56,48,128,169,169,73,228,114,185,142,213,14,185,189,209,135,63,245,244,97,244,244,116,99,106,
  98,18,87,167,166,80,40,20,225,249,30,98,166,137,84,42,137,142,246,118,60,116,115,31,126,242,227,31,227,39,191,253,45,92,215,237,184,230,35,185,
  247,222,251,102,191,250,181,175,226,220,217,179,248,206,119,190,139,114,133,232,140,98,177,56,223,49,45,203,194,245,235,214,33,99,26,184,101,231,14,228,
  74,101,156,58,127,110,54,234,168,234,22,121,236,241,175,206,222,115,247,157,248,222,247,127,128,217,153,25,194,111,101,50,100,74,164,106,196,95,167,168,
  200,102,155,112,207,129,94,228,135,135,241,241,103,151,224,25,38,110,190,245,118,28,251,232,72,221,66,106,8,61,206,222,121,199,109,248,155,191,253,123,
  228,231,231,145,201,54,113,189,151,202,140,148,170,6,199,115,209,187,119,55,50,166,129,75,99,99,24,155,155,131,237,122,168,89,22,118,223,184,31,241,
  120,124,54,114,145,68,34,57,251,240,151,30,193,79,127,250,34,172,90,149,107,238,68,112,199,252,141,241,68,18,7,55,247,160,52,57,137,51,67,195,
  240,244,24,20,4,110,217,61,55,54,236,150,61,132,207,46,245,99,54,151,67,38,147,145,22,16,131,68,20,10,133,126,243,209,199,88,7,31,179,165,
  50,140,88,66,2,119,137,100,2,93,27,55,205,142,143,141,118,240,35,209,117,125,246,186,235,174,195,249,243,231,145,78,165,66,57,119,140,193,211,248,
  95,3,192,248,213,9,28,155,152,130,103,198,105,74,131,203,33,82,164,230,110,199,142,157,200,231,243,168,85,107,60,145,143,43,214,66,211,58,54,142,
  98,148,149,200,123,121,172,191,201,182,97,234,38,18,137,228,44,95,164,165,165,5,115,243,11,1,53,168,9,150,4,61,152,57,74,94,95,65,204,47,
  90,64,57,37,21,104,238,200,88,150,117,203,214,185,101,181,112,74,117,224,216,128,64,77,121,190,80,28,69,209,36,96,67,55,163,186,101,67,110,89,
  62,247,69,3,230,142,82,83,62,43,142,16,176,23,124,21,140,143,35,81,40,203,37,162,61,53,12,105,1,174,86,147,252,190,161,32,61,207,227,141,
  236,158,192,226,49,87,115,48,150,93,44,64,229,13,45,106,100,137,90,157,45,199,149,193,157,71,71,228,44,122,182,222,80,65,27,42,116,93,15,105,
  31,1,209,22,205,2,116,69,167,178,227,136,28,164,92,249,30,54,84,116,134,27,42,68,228,40,57,54,92,143,215,140,242,164,106,199,149,250,29,84,
  85,101,21,176,157,117,224,142,88,114,163,179,86,60,207,163,64,218,165,80,200,145,78,21,175,95,112,28,36,18,9,204,81,136,42,45,50,54,58,210,
  89,162,204,157,47,188,228,72,248,151,53,179,114,89,162,203,177,23,35,214,204,88,12,158,231,34,151,203,117,70,62,133,63,253,228,24,31,203,242,139,
  236,122,220,57,19,150,134,138,71,96,209,156,187,84,50,137,254,75,151,86,100,238,58,79,157,56,14,248,30,105,168,112,61,201,18,34,129,59,222,183,
  76,26,218,204,88,140,4,233,145,240,138,181,162,150,79,1,208,25,222,184,12,195,152,62,112,83,31,18,169,20,172,154,133,90,173,42,9,43,3,247,
  140,11,85,85,145,74,36,96,59,14,107,164,90,27,181,199,119,54,2,0,27,186,54,78,119,174,93,139,68,44,6,219,241,164,40,148,8,67,197,218,
  149,128,68,231,53,184,4,166,215,118,118,194,96,33,198,52,204,112,118,118,22,142,227,172,189,22,128,214,249,135,198,93,127,52,154,187,105,81,115,167,
  168,42,42,213,26,114,51,51,24,29,27,67,255,165,207,190,248,233,82,20,101,58,74,115,71,76,147,36,247,145,149,218,217,249,121,252,255,126,248,195,
  200,59,171,225,145,236,220,185,107,250,219,127,254,109,228,114,115,248,249,47,94,198,220,92,142,90,64,201,204,139,17,57,142,235,224,133,111,126,3,55,
  174,187,29,186,109,225,239,126,245,235,233,106,181,186,118,213,69,122,111,58,56,253,167,223,252,19,188,246,234,107,184,112,241,34,18,201,20,146,169,20,
  145,99,169,42,7,19,174,235,98,215,214,237,184,113,221,90,76,93,186,132,92,97,17,119,222,115,31,78,159,248,100,58,124,59,75,23,126,243,150,45,
  211,207,61,251,12,254,241,135,63,198,133,207,46,241,40,20,70,234,112,180,162,170,48,227,49,124,249,222,123,80,205,231,113,228,252,121,140,44,46,65,
  85,84,236,217,215,139,68,34,57,221,80,202,251,236,51,207,226,183,191,125,157,4,233,9,81,40,97,90,202,113,28,244,237,223,135,181,217,12,6,6,
  7,113,126,252,10,226,137,36,42,213,42,124,223,175,211,220,169,2,200,158,154,155,159,71,255,229,203,130,161,66,231,243,95,145,115,49,13,3,211,179,
  179,24,62,115,26,31,127,118,9,85,85,167,134,62,226,115,160,13,21,83,117,215,100,251,142,29,248,244,211,19,72,82,183,44,9,206,85,37,167,128,
  202,75,84,85,140,79,76,226,251,83,211,80,60,23,177,88,60,16,243,187,30,92,219,65,71,231,90,140,143,141,78,249,190,191,78,165,211,133,169,154,
  101,97,177,72,234,199,52,77,13,134,154,33,214,136,13,53,73,167,57,224,41,154,28,36,226,121,176,93,7,154,74,122,131,2,67,69,91,27,111,168,
  96,220,163,24,98,204,128,54,87,75,133,204,172,174,36,32,119,184,161,162,169,185,37,56,93,241,88,156,4,186,24,58,71,142,124,52,30,197,220,121,
  126,136,88,11,149,23,56,118,125,183,172,77,243,33,12,195,228,183,107,125,67,69,160,183,19,23,112,61,23,126,184,180,222,245,160,209,176,177,96,44,
  91,34,241,141,6,227,187,248,197,214,26,54,179,138,134,98,167,206,72,233,240,129,91,96,168,160,245,99,170,38,106,238,228,172,71,102,7,229,163,113,
  79,72,69,22,227,0,27,186,101,167,38,5,205,93,125,199,50,4,65,89,32,199,138,10,49,38,116,148,227,56,80,85,21,249,249,249,96,192,236,56,
  206,122,2,238,140,250,32,61,161,223,172,94,185,41,76,177,5,205,29,59,234,122,205,221,244,148,16,166,39,164,197,136,212,96,200,177,225,185,65,222,
  48,35,214,28,219,66,50,17,103,154,187,245,50,115,55,62,182,190,184,184,72,193,93,189,99,217,21,201,77,79,156,251,6,32,155,149,119,25,186,142,
  171,141,152,187,83,180,91,54,28,133,34,50,119,14,143,0,12,106,72,40,238,133,239,251,200,102,179,24,26,26,226,71,17,5,238,214,159,58,113,156,
  115,89,82,98,143,35,171,7,88,222,138,216,177,177,166,181,21,19,87,175,160,82,41,175,15,111,191,235,34,152,187,73,198,220,85,42,21,216,150,197,
  221,178,204,204,18,174,31,211,52,13,67,67,67,117,11,52,92,68,96,238,38,25,115,39,54,84,184,148,185,99,217,208,11,11,11,184,122,101,124,253,
  74,64,98,221,106,104,163,169,185,121,178,181,149,148,67,133,53,119,180,161,98,253,106,104,101,221,127,65,136,165,252,241,50,119,138,162,76,28,60,120,
  16,221,155,183,160,41,75,76,177,213,154,133,185,185,57,177,161,226,186,213,14,111,125,163,15,127,250,153,103,113,240,166,3,210,88,214,178,29,196,227,
  49,169,91,246,179,139,23,112,254,220,57,20,139,197,235,174,121,145,221,187,247,76,176,177,236,123,31,124,136,185,249,121,152,166,73,185,47,77,10,211,
  53,76,19,127,242,228,87,97,45,44,224,127,253,63,255,79,76,228,114,215,173,122,186,110,189,229,214,137,103,158,123,22,47,255,250,21,12,244,95,226,
  57,119,12,57,18,112,65,118,80,203,182,177,117,99,23,182,118,118,32,95,90,198,29,189,189,120,251,228,201,137,92,104,33,185,91,118,251,246,9,62,
  150,29,24,16,114,238,116,222,80,197,180,119,240,125,180,52,53,225,169,7,239,71,105,114,18,111,159,60,137,201,114,5,123,247,29,64,34,145,156,104,
  52,150,157,120,242,169,167,241,234,43,175,242,177,44,25,164,233,82,253,2,153,16,145,162,244,175,63,250,8,82,174,131,51,253,253,56,61,54,14,69,
  55,225,249,30,246,238,219,15,69,81,38,162,192,29,102,102,103,164,250,49,190,128,56,72,211,8,153,144,73,147,124,158,153,177,49,124,112,225,34,124,
  106,108,181,106,22,98,241,56,54,110,234,174,191,38,59,118,238,196,199,199,142,145,134,10,234,150,85,67,138,53,17,123,149,202,21,188,244,155,87,208,
  102,232,88,116,60,232,102,44,80,71,91,46,218,218,59,48,54,58,50,225,251,254,245,42,0,101,83,119,247,4,233,150,93,230,163,241,186,5,84,113,
  75,246,161,170,10,42,150,131,177,165,50,116,211,20,252,141,30,28,203,130,105,24,50,184,107,107,235,64,126,161,192,223,59,194,224,78,165,165,30,129,
  98,74,225,218,71,85,65,93,43,146,75,77,148,18,184,51,77,19,165,114,25,166,97,202,134,10,73,49,24,164,36,215,129,59,87,6,119,172,91,214,
  148,153,59,11,166,27,167,169,49,154,84,253,202,251,205,132,118,92,9,220,185,225,210,122,154,107,235,133,110,225,74,185,12,5,10,45,79,9,167,149,
  212,203,18,3,206,139,82,131,117,138,53,135,167,41,203,224,78,1,63,85,245,172,157,16,71,195,184,71,94,241,30,140,0,235,26,42,194,154,59,215,
  113,8,217,47,148,215,50,75,180,72,110,186,94,116,90,140,88,191,192,114,238,88,66,156,74,105,140,235,203,156,185,83,66,236,157,236,57,241,220,168,
  211,35,59,53,65,185,178,58,112,55,51,61,37,132,233,137,67,229,144,29,215,11,135,25,186,1,192,160,32,47,30,39,99,89,223,247,55,72,47,166,
  87,198,199,54,44,9,224,142,53,179,33,100,199,21,39,215,30,109,199,101,142,13,14,238,84,21,227,227,99,27,26,140,101,69,112,7,64,28,203,186,
  98,13,134,35,213,143,177,18,53,0,104,202,102,49,56,60,180,34,115,183,225,196,39,31,115,205,93,93,74,50,125,89,242,67,49,116,86,173,198,53,
  119,163,163,35,168,86,171,27,194,59,227,245,17,252,214,149,222,3,7,145,72,165,81,171,86,185,91,150,157,123,209,96,4,4,245,99,151,73,183,236,
  134,168,237,247,250,21,52,119,87,214,174,91,15,131,185,101,45,139,167,84,179,231,152,231,186,44,231,110,195,74,64,226,250,107,24,203,94,105,107,239,
  144,52,119,172,91,182,82,41,111,184,22,48,118,253,31,26,119,253,113,48,119,0,176,119,239,141,87,186,123,54,163,181,165,25,80,85,148,203,101,20,
  242,121,12,143,12,99,224,242,229,174,107,57,93,13,207,233,99,143,127,117,252,206,59,110,67,165,82,193,244,244,12,22,10,5,18,203,104,154,72,38,
  147,225,110,217,174,207,181,72,79,207,230,241,191,252,183,127,137,165,226,18,222,126,231,93,76,207,76,147,162,2,51,38,227,47,195,68,115,54,139,182,
  53,45,216,189,115,7,94,254,201,143,113,236,220,249,174,85,79,215,254,253,251,199,95,248,214,183,240,250,239,223,196,217,51,167,17,79,36,144,72,166,
  160,107,26,111,69,96,219,180,170,169,40,20,10,216,189,115,59,110,221,182,21,233,71,30,193,84,110,110,124,124,122,186,171,225,133,223,184,169,123,252,
  249,111,124,3,63,253,233,139,56,123,246,44,5,119,9,154,113,27,176,119,108,1,203,182,113,227,222,221,248,242,173,55,35,63,60,140,51,3,131,184,
  126,243,86,164,82,169,241,134,139,60,243,204,51,120,227,247,111,96,124,124,140,107,238,68,59,130,170,178,189,159,148,226,108,233,233,198,115,15,220,135,
  153,241,113,188,125,242,36,206,78,77,35,145,76,225,134,61,55,66,81,148,241,186,69,238,188,243,174,241,165,165,37,194,61,166,51,146,161,66,215,233,
  182,172,211,78,121,85,131,15,31,91,54,247,192,171,84,112,230,220,121,28,31,25,133,25,79,162,86,171,49,230,174,254,154,220,176,123,55,62,61,113,
  18,137,68,130,23,72,72,156,151,42,236,251,80,96,24,6,222,251,224,40,150,7,6,112,225,202,21,152,113,226,107,112,169,26,186,189,163,19,99,163,
  35,227,190,239,119,169,0,208,213,181,113,188,102,89,88,88,88,160,110,89,90,55,198,204,45,42,145,196,137,204,157,66,7,57,71,199,39,176,164,104,
  0,125,142,49,11,130,192,220,145,77,171,173,109,13,22,11,5,104,170,42,217,113,117,169,126,65,67,84,12,157,174,169,64,40,200,152,49,119,205,34,
  184,139,153,49,20,151,150,97,152,177,250,110,89,33,231,35,0,22,222,202,245,99,116,152,105,72,204,29,221,82,117,35,64,240,236,46,98,224,78,68,
  46,162,99,217,149,166,167,129,152,223,23,124,18,148,185,99,221,178,90,224,154,225,71,160,242,180,68,121,44,75,39,168,194,228,148,141,163,28,14,48,
  220,122,112,167,73,224,78,173,11,146,22,53,119,174,64,176,73,224,142,34,22,69,81,96,219,78,104,44,235,56,82,67,69,20,184,227,209,64,2,200,
  14,234,23,216,196,142,134,24,171,74,29,184,219,88,46,149,132,177,172,26,186,6,225,16,99,26,255,43,198,208,209,56,102,199,182,0,26,100,193,185,
  77,118,75,78,79,79,9,154,59,63,66,150,40,35,23,71,232,247,229,121,207,172,64,34,102,98,113,113,17,190,239,111,148,192,221,213,43,227,155,8,
  184,211,100,128,205,192,157,84,28,33,187,55,88,28,51,107,102,53,12,19,87,198,199,56,215,18,9,238,24,197,36,53,180,249,114,168,36,143,2,20,
  56,72,0,104,106,106,194,48,97,238,54,53,2,119,155,206,158,62,193,137,104,47,148,146,204,216,82,158,146,108,179,156,59,11,170,2,180,182,180,96,
  122,106,18,149,74,121,83,120,103,220,24,193,220,141,74,81,40,181,26,127,31,12,71,161,192,247,9,115,167,235,24,26,28,172,91,160,225,34,194,54,
  60,42,117,203,90,22,13,146,22,220,178,190,79,220,178,227,99,210,41,186,230,69,216,79,115,115,243,104,107,235,26,196,98,49,146,186,224,121,168,213,
  106,200,207,207,161,88,44,110,186,22,180,178,241,95,12,184,83,86,193,193,35,125,125,55,99,211,166,77,72,83,183,108,185,82,193,92,110,14,35,35,
  195,56,119,246,12,124,223,239,94,237,116,109,106,244,225,98,136,241,213,171,87,177,80,88,36,134,10,67,71,83,54,131,166,166,38,56,142,131,11,23,
  46,224,232,145,15,27,46,22,185,200,109,183,223,62,114,248,240,97,244,247,95,198,7,31,30,197,242,114,145,104,237,77,147,131,59,149,238,156,169,84,
  18,107,215,118,162,80,88,196,80,255,103,248,172,191,191,123,213,69,30,126,248,145,145,7,31,122,16,63,123,241,231,152,156,156,68,42,149,132,105,198,
  37,181,20,23,149,209,240,144,106,173,134,135,238,187,7,187,215,173,197,255,231,63,254,71,156,27,28,236,110,120,225,123,123,123,71,238,187,255,62,124,
  231,59,223,197,204,244,20,50,153,76,221,2,154,170,9,122,35,13,182,227,224,206,91,250,240,192,141,123,96,46,47,97,239,142,29,136,199,227,35,145,
  139,232,186,62,242,248,87,190,130,95,252,226,151,88,46,145,16,99,93,55,36,65,153,88,92,175,40,10,108,199,198,109,7,15,112,4,249,246,201,147,
  24,89,88,168,211,220,241,69,30,122,228,16,134,71,198,48,57,53,141,36,19,245,233,66,51,149,212,109,74,234,42,187,55,118,73,11,28,29,28,1,
  52,19,137,100,2,27,55,117,143,72,139,40,138,50,178,185,167,27,167,79,157,36,174,125,22,164,39,200,18,195,25,195,186,161,99,122,122,6,159,156,
  61,135,163,231,206,227,232,208,8,226,169,52,137,66,169,214,208,30,24,42,136,82,109,199,142,157,88,44,46,161,92,169,72,113,52,170,112,254,89,142,
  1,215,26,249,68,124,246,202,135,71,161,122,30,226,201,52,192,228,64,142,13,93,55,144,74,165,70,74,165,82,15,209,220,181,182,144,134,10,195,228,
  71,160,170,90,160,185,83,25,184,19,182,100,128,4,188,248,62,106,100,54,197,73,4,155,106,238,152,61,36,232,150,45,87,164,16,227,70,154,59,49,
  107,133,141,163,32,213,141,146,173,186,222,45,107,57,48,99,26,84,161,33,68,17,174,129,252,181,146,173,160,124,199,100,209,255,142,13,199,245,0,191,
  46,10,101,153,16,47,108,98,45,176,118,129,222,11,60,10,133,207,222,125,185,145,221,161,249,118,129,91,54,50,10,37,76,207,202,147,107,47,162,29,
  204,147,164,88,54,151,98,249,126,68,20,138,231,186,210,27,149,28,98,12,73,185,233,11,67,77,155,237,243,20,216,177,137,93,157,91,214,178,172,158,
  82,169,196,113,87,61,107,231,201,92,151,39,102,12,123,240,92,33,231,142,214,143,209,156,187,205,18,238,202,205,206,112,205,29,155,249,74,126,95,65,
  150,40,138,250,68,198,206,166,67,205,100,34,17,157,115,55,54,58,178,121,105,177,8,211,48,121,75,30,123,69,8,186,205,92,9,69,218,46,57,117,
  188,68,130,231,220,249,152,156,156,220,220,208,80,65,192,178,42,191,34,132,24,108,238,150,117,3,67,49,155,154,166,83,169,149,67,140,45,203,218,252,
  201,177,163,240,104,208,148,39,102,171,132,82,146,109,155,84,83,51,89,162,25,139,145,250,49,210,14,182,57,188,105,245,68,104,238,134,14,220,212,135,
  84,58,67,26,42,106,53,250,98,227,0,158,11,203,14,220,227,170,166,33,153,72,136,33,198,155,163,118,198,158,70,0,160,171,107,227,208,218,117,235,
  96,24,6,92,215,139,204,185,19,52,119,155,87,2,18,61,215,160,185,27,234,92,187,142,119,203,122,46,145,243,76,79,79,193,113,156,205,215,2,238,
  122,174,225,223,249,195,226,174,127,174,177,236,224,190,125,251,192,152,59,174,185,203,205,99,116,100,136,9,44,183,172,118,42,54,55,250,240,199,191,242,
  53,220,118,219,45,168,84,42,152,156,156,66,126,97,129,107,238,196,177,236,249,115,231,64,115,238,182,92,243,34,219,182,111,31,252,246,183,191,141,124,
  126,1,239,189,247,62,102,115,179,80,53,82,53,106,26,6,84,93,135,170,40,48,77,19,217,76,22,157,157,29,88,94,42,226,213,87,95,65,173,90,
  221,178,234,34,189,189,189,131,127,250,194,11,120,229,181,223,225,179,11,231,145,72,166,96,198,200,212,84,209,52,152,188,87,128,49,70,228,17,20,143,
  199,177,117,115,15,46,159,58,137,55,143,30,221,210,240,154,244,244,108,30,124,238,249,231,241,195,31,253,4,147,19,87,145,78,103,160,155,196,72,25,
  4,137,168,156,53,98,110,129,154,101,97,199,182,173,120,242,206,219,113,46,25,199,103,3,3,131,19,185,220,150,200,199,202,83,79,63,141,223,189,254,
  6,38,39,39,145,206,100,73,213,187,144,208,171,134,208,163,239,251,176,29,27,247,223,125,39,190,118,19,177,137,140,77,77,99,67,87,23,20,69,25,
  100,79,120,126,11,223,121,231,93,3,139,139,139,36,169,47,205,198,178,164,118,140,147,58,170,216,45,64,12,99,143,63,242,37,28,92,219,137,153,177,
  49,188,117,226,36,206,78,78,34,150,109,174,119,203,6,204,29,201,185,211,184,161,194,128,41,52,179,18,98,45,208,220,123,62,144,174,213,80,152,154,
  194,203,31,30,193,233,137,41,104,102,28,142,101,163,189,163,19,138,162,12,240,107,210,213,181,113,160,102,89,88,92,44,6,204,157,38,171,6,217,2,
  146,74,202,247,241,210,123,31,98,67,115,22,195,133,34,140,120,2,62,125,21,103,99,217,124,62,47,228,220,205,147,186,99,206,220,233,114,136,177,216,
  111,198,152,59,149,250,83,7,167,103,97,154,49,248,142,195,217,35,219,113,234,153,187,18,149,94,233,90,24,197,203,132,142,148,148,65,235,119,125,248,
  129,215,148,190,239,215,229,220,217,84,16,198,18,99,196,217,123,16,160,171,68,90,65,165,32,17,207,227,169,176,190,23,106,168,40,151,75,164,126,76,
  55,248,109,170,8,121,41,114,214,135,95,151,181,34,226,48,94,228,69,117,248,220,45,91,92,92,4,224,211,82,59,141,95,100,177,161,34,168,96,144,
  21,107,162,28,203,17,134,154,158,15,212,106,181,192,247,59,53,53,9,219,22,153,59,77,146,196,69,201,18,153,28,78,28,164,185,174,48,150,85,21,
  20,23,11,18,115,183,173,90,46,145,231,147,34,187,102,16,54,20,11,19,108,150,177,230,10,212,160,75,73,29,0,88,88,88,168,103,238,116,49,10,
  5,178,230,78,162,103,165,180,146,128,255,98,224,46,30,231,154,187,109,210,34,87,175,140,111,99,154,59,25,65,6,118,28,159,53,180,113,38,219,9,
  136,53,135,104,238,84,149,84,156,80,230,46,2,220,125,122,156,118,53,144,55,168,128,185,243,37,43,174,199,33,170,64,13,210,246,22,65,115,183,45,
  114,17,219,182,183,157,162,81,40,186,166,133,184,71,87,50,22,217,182,35,201,18,53,85,65,75,75,11,38,39,174,162,82,41,111,11,111,90,91,195,
  59,89,44,30,191,124,32,212,45,27,24,139,200,98,182,109,195,163,156,124,72,115,183,61,106,103,220,182,130,230,174,95,116,203,18,88,74,244,15,158,
  239,115,204,188,88,40,176,156,187,237,141,246,248,109,171,33,150,108,54,219,191,166,173,157,107,238,60,223,71,181,90,21,53,119,219,87,67,43,219,254,
  208,184,235,143,198,80,209,191,255,192,77,232,233,233,65,75,115,19,20,69,69,185,82,65,126,46,135,161,225,17,6,238,190,216,233,82,20,165,255,201,
  167,158,198,193,155,14,160,82,169,224,202,213,9,44,46,46,194,182,108,24,166,33,129,187,11,231,47,224,227,143,142,108,255,92,139,236,218,117,67,255,
  159,125,251,207,48,53,49,137,247,143,28,197,252,252,28,52,77,151,240,151,174,146,173,33,155,37,224,110,177,80,192,187,239,190,141,197,66,97,71,212,
  34,210,111,208,215,215,119,233,249,111,124,3,191,254,245,43,232,239,191,132,100,42,205,11,36,8,246,50,164,100,126,246,160,203,102,50,176,109,7,23,
  206,157,193,213,171,87,118,52,188,240,91,182,110,189,244,244,225,195,248,193,15,126,136,254,203,151,229,156,59,129,247,226,132,130,26,196,115,204,229,243,
  88,191,126,45,254,95,127,245,63,97,207,150,45,151,26,143,101,15,63,67,66,140,103,103,3,67,5,243,150,138,153,170,26,137,253,37,239,235,132,21,
  186,251,142,219,241,39,119,223,137,181,166,129,155,119,239,142,190,187,238,187,239,254,75,179,185,28,46,15,14,33,157,78,243,40,20,166,88,19,201,77,
  182,223,120,158,139,182,214,22,220,118,219,173,232,109,111,195,204,216,24,78,244,95,198,208,220,60,54,110,234,190,52,54,58,178,67,58,146,29,59,119,
  242,156,59,22,98,172,11,205,172,129,91,32,216,53,93,199,197,174,157,59,113,83,215,6,92,56,119,14,47,127,120,4,111,127,214,143,146,7,6,238,
  46,241,35,217,184,169,251,82,165,92,198,98,177,200,251,125,181,80,253,69,96,5,13,166,13,186,97,224,163,143,143,193,172,148,241,209,167,159,98,186,
  84,129,105,198,224,56,54,12,77,71,83,83,19,10,133,2,57,146,206,142,118,228,23,10,84,107,23,202,23,214,52,161,157,66,180,130,146,51,80,179,
  29,188,113,252,19,44,88,46,76,211,228,3,53,199,117,208,76,162,80,168,91,214,48,176,92,42,83,181,166,30,234,150,85,235,246,125,54,231,130,207,
  98,32,104,106,47,159,214,121,0,28,57,10,197,113,200,166,164,27,129,45,90,122,209,145,192,157,47,213,144,136,221,90,158,160,185,131,175,112,243,49,
  207,185,131,162,4,167,74,213,66,240,52,154,123,140,234,216,224,62,71,55,100,168,96,33,107,204,200,202,102,239,97,83,113,125,74,178,224,216,164,76,
  42,171,38,81,149,144,230,110,102,122,138,79,177,3,122,182,158,61,173,31,141,179,68,12,22,63,235,242,24,90,69,9,141,101,109,219,222,89,46,149,
  130,241,184,162,132,66,140,27,251,29,8,169,233,72,204,157,71,201,178,72,205,157,97,232,80,64,100,186,242,145,200,23,57,92,32,236,138,146,56,199,
  65,42,201,193,221,174,176,230,110,87,113,113,17,186,161,7,213,199,66,20,74,221,43,130,24,67,71,85,131,204,119,162,107,58,198,199,199,118,53,54,
  84,208,34,46,246,14,226,134,59,254,196,185,175,67,174,7,59,2,120,46,154,154,154,48,56,56,176,34,115,183,235,212,201,79,3,205,157,43,214,31,
  123,2,139,42,246,155,217,60,134,185,185,165,5,87,198,199,80,173,86,119,133,55,173,157,17,99,217,139,36,231,46,137,106,165,2,219,170,81,128,29,
  104,238,120,136,177,162,32,153,136,67,83,185,230,110,87,212,206,184,179,49,184,235,185,40,53,84,80,112,231,49,122,92,37,60,101,177,176,128,43,87,
  175,178,11,141,207,181,136,0,238,46,182,174,89,131,88,44,14,85,81,224,208,156,187,249,185,28,74,165,210,174,107,97,238,118,126,1,40,165,124,94,
  220,245,71,193,220,93,16,153,59,95,81,81,173,84,48,151,203,97,116,108,140,129,187,27,86,59,236,93,141,62,92,116,203,134,153,187,76,38,19,104,
  238,206,156,193,167,199,143,221,240,185,22,217,190,99,231,133,191,248,139,63,71,46,55,135,119,223,123,31,115,115,57,168,170,22,128,59,250,173,214,116,
  13,77,217,38,14,238,222,124,227,247,40,149,150,111,88,117,145,222,155,14,94,96,110,217,243,23,46,32,153,74,115,51,107,56,250,159,76,137,84,248,
  240,209,68,39,220,31,188,255,46,22,11,133,27,26,46,210,211,179,249,194,127,255,63,252,247,124,238,155,78,167,185,21,84,141,200,183,101,200,133,192,
  35,31,30,128,76,50,129,161,139,231,209,63,60,124,67,221,221,165,40,202,249,231,158,123,14,191,253,237,235,152,156,156,68,38,147,161,234,40,85,110,
  166,18,182,101,230,115,100,77,135,55,108,223,130,7,110,189,21,103,219,90,241,191,14,15,215,231,112,223,117,247,221,152,155,159,199,229,129,1,14,238,
  184,230,145,118,10,212,27,41,201,64,115,77,115,51,238,216,115,3,246,92,127,29,220,229,37,164,147,9,244,108,220,120,126,120,108,108,183,180,200,206,
  157,187,240,233,137,147,136,199,227,220,18,34,157,34,105,106,74,126,73,219,178,177,117,115,55,158,120,224,126,24,203,75,152,188,50,142,163,231,47,226,
  236,228,36,154,219,59,160,140,143,159,247,125,127,55,3,119,231,137,91,118,177,225,69,86,57,107,20,228,124,40,170,130,153,220,28,74,133,2,134,47,
  156,199,219,167,207,32,87,169,81,207,182,142,108,83,19,22,11,5,114,36,220,45,171,106,194,2,194,53,8,29,1,219,49,85,85,195,194,98,17,63,
  126,227,45,20,231,114,40,58,196,188,74,128,132,130,214,214,53,193,34,241,24,49,84,24,166,41,184,101,213,186,134,10,209,204,74,66,169,124,104,138,
  130,233,153,25,120,0,125,120,6,10,66,201,80,225,80,109,60,183,132,48,254,151,230,120,201,115,95,121,223,247,169,131,195,15,53,83,121,138,60,150,
  85,74,165,18,160,64,42,18,14,60,39,170,68,11,134,221,2,110,56,99,152,229,123,81,148,201,73,181,98,177,8,64,9,145,155,161,10,140,40,112,
  199,246,126,175,190,91,86,129,31,170,31,155,156,144,114,238,228,40,64,133,206,28,163,193,29,39,55,233,190,207,36,89,130,91,150,44,226,56,206,158,
  74,185,68,98,123,21,225,40,26,228,60,75,167,137,181,35,8,224,206,167,231,54,194,45,59,77,213,180,76,247,40,155,89,163,163,0,201,17,248,226,
  130,182,141,68,208,80,177,39,236,150,221,83,92,92,132,41,68,145,139,237,96,34,123,42,135,233,137,8,198,225,110,217,43,43,185,101,25,115,23,216,
  114,66,237,184,17,25,195,236,8,60,207,67,51,5,119,190,239,239,69,84,102,129,101,89,123,131,134,10,69,136,196,12,5,27,177,10,56,97,238,171,
  170,42,90,91,91,113,245,234,21,84,171,213,189,225,253,100,79,120,39,75,36,146,103,247,247,30,32,224,174,90,225,99,89,54,123,15,231,220,37,226,
  113,168,154,134,33,210,80,177,55,106,103,220,211,104,111,238,238,217,124,182,179,115,45,237,150,117,9,184,11,185,101,61,207,67,113,113,145,49,119,123,
  27,237,241,123,174,193,45,123,182,181,117,13,226,177,24,159,48,216,182,141,252,252,60,150,150,150,26,126,248,231,90,228,95,140,230,238,76,88,115,183,
  92,42,33,63,159,199,208,208,32,46,156,63,7,223,247,111,92,237,116,237,109,0,178,207,28,126,230,153,72,205,93,60,102,34,147,205,32,155,201,192,
  113,28,156,61,123,14,199,62,62,218,112,177,200,69,110,191,227,206,51,79,63,253,20,46,92,184,136,35,71,143,162,84,42,115,183,44,73,198,82,9,
  215,165,19,36,217,217,209,142,165,226,18,222,121,251,77,20,139,197,27,87,93,228,208,161,67,103,238,187,255,126,252,228,39,63,195,212,212,36,82,233,
  12,111,166,226,180,148,128,191,248,145,103,210,240,60,15,31,31,253,16,249,124,254,198,134,23,190,175,175,239,204,221,247,220,131,191,251,251,127,64,46,
  55,139,108,83,115,80,125,37,46,32,36,52,144,34,175,160,126,108,207,190,3,208,52,237,76,228,34,134,97,156,57,244,232,163,248,233,139,47,161,82,
  46,35,153,74,7,102,86,77,14,114,83,132,234,5,81,134,189,88,92,66,60,22,195,95,60,251,44,12,195,56,83,7,238,14,61,250,101,12,12,12,
  34,55,155,227,33,198,98,214,138,26,226,30,3,27,40,73,234,75,37,226,216,189,107,7,238,238,217,132,180,174,227,212,153,211,56,118,238,188,194,111,
  97,77,211,78,119,117,109,192,219,239,188,131,84,42,41,249,125,69,112,167,82,199,70,96,162,244,160,169,26,110,59,184,23,7,55,247,160,89,215,80,
  46,20,112,102,104,8,153,76,22,138,162,156,246,125,127,159,78,156,203,59,72,206,29,181,117,68,98,47,73,107,15,14,178,21,197,195,193,27,118,33,
  179,188,132,147,23,206,227,204,208,48,6,114,57,104,70,12,241,120,2,149,74,153,156,174,150,53,107,48,55,191,192,67,140,117,61,68,172,105,106,8,
  220,41,92,183,82,174,84,241,206,251,31,96,141,2,188,113,250,12,44,234,42,208,21,5,109,237,237,184,50,62,70,67,140,13,3,213,42,213,220,73,
  220,111,80,191,32,39,149,4,82,120,93,83,113,110,112,8,182,227,192,51,99,80,189,160,71,62,38,119,203,90,60,231,78,44,16,142,178,227,134,171,
  175,184,161,213,23,237,33,54,92,132,198,178,203,165,50,231,129,37,230,142,158,174,112,66,111,96,100,117,235,226,127,25,184,243,168,6,79,24,203,22,
  160,169,74,221,228,90,21,220,2,129,177,88,182,130,136,81,40,162,91,128,188,90,88,194,88,118,114,2,174,231,213,131,59,137,127,20,250,125,93,175,
  174,250,74,4,22,30,77,40,147,52,119,213,106,117,127,181,82,161,229,17,98,145,135,92,1,231,242,196,30,33,72,143,213,189,11,224,14,138,202,252,
  143,251,101,112,199,115,238,20,106,98,13,124,215,245,57,207,33,107,142,27,12,154,109,219,70,50,30,67,46,55,91,255,236,26,29,25,222,79,114,238,
  2,146,147,189,131,120,94,132,130,80,156,251,10,11,152,166,9,223,247,145,203,229,246,71,131,187,147,39,104,67,133,156,115,23,182,132,68,85,192,113,
  205,93,58,189,162,230,78,169,84,202,189,172,91,150,229,220,145,166,66,81,185,25,196,209,216,20,216,57,142,3,195,52,145,205,102,209,127,233,51,184,
  174,219,139,80,206,221,254,8,230,238,228,190,253,7,144,72,165,80,173,86,97,89,86,224,150,21,114,238,60,215,133,162,170,72,38,18,112,93,7,3,
  151,47,179,5,234,118,198,253,43,184,101,79,174,93,187,22,134,25,163,132,166,13,203,150,155,162,29,199,65,110,118,6,185,92,174,119,37,32,177,127,
  53,72,19,143,199,79,118,116,116,242,158,107,166,185,155,153,153,134,109,219,189,215,194,192,237,255,23,3,238,86,99,238,78,178,156,187,150,214,86,168,
  10,80,46,19,230,110,100,116,4,3,151,47,195,247,253,222,149,184,73,5,64,111,131,15,63,33,142,101,89,206,29,27,203,138,204,221,185,51,167,113,
  252,248,241,3,43,93,147,222,136,177,236,9,214,45,251,214,219,111,35,159,207,243,200,83,195,48,27,142,101,41,115,119,96,213,69,250,250,250,78,136,
  221,178,162,230,78,15,5,136,105,186,78,9,1,5,45,77,89,184,158,135,163,31,190,143,124,62,127,160,225,133,223,186,109,219,9,54,150,101,221,178,
  177,56,241,250,138,172,17,27,11,18,150,155,16,111,133,226,18,106,150,133,27,123,15,34,30,143,159,136,92,68,81,148,19,172,91,118,122,102,70,142,
  66,9,201,177,20,97,91,102,216,130,213,143,57,142,141,29,187,118,71,223,194,247,222,123,31,102,115,57,12,12,13,55,200,90,145,185,71,238,177,163,
  147,162,26,181,87,181,175,105,197,19,247,222,141,123,110,189,245,68,221,251,9,139,66,97,99,217,48,177,166,241,216,45,89,208,239,249,30,76,211,192,
  142,13,215,97,239,150,205,216,220,65,18,183,103,175,94,193,187,31,125,196,17,36,186,123,54,127,26,68,161,196,34,193,93,176,0,219,247,21,238,12,
  120,248,209,71,208,219,222,134,194,212,20,38,175,140,227,226,232,56,174,206,229,209,210,212,244,233,194,226,226,77,58,0,180,179,110,89,77,140,2,212,
  4,12,44,30,129,108,67,180,28,7,99,99,227,216,0,31,111,124,244,49,6,103,102,48,183,180,12,51,30,71,83,107,43,22,22,23,131,134,138,229,
  82,153,114,192,161,94,7,97,191,23,67,44,24,37,98,104,26,46,156,63,143,225,139,23,49,54,51,3,77,55,160,232,6,60,207,15,51,119,132,179,
  34,81,40,186,116,4,170,26,244,59,68,130,59,223,195,114,217,194,130,235,194,136,197,137,44,203,113,128,70,245,99,134,200,255,170,106,112,23,69,52,
  179,250,180,222,138,1,12,48,199,6,29,67,49,76,192,193,29,27,203,138,95,52,41,130,46,162,223,33,42,107,133,77,77,67,134,10,69,114,203,6,
  242,196,80,144,11,234,147,250,88,136,155,19,226,189,24,114,137,234,150,61,88,166,134,10,145,131,23,225,169,212,173,37,34,23,30,162,27,68,161,1,
  141,186,101,185,161,66,188,139,252,122,191,195,10,212,32,195,94,130,230,238,160,204,220,141,141,30,36,13,21,26,7,118,1,122,20,227,24,157,58,191,
  3,95,144,102,12,175,216,45,123,250,212,167,17,245,99,97,106,208,163,178,80,87,106,7,179,109,27,80,20,52,55,53,97,120,104,144,31,5,191,187,
  216,159,106,181,218,199,192,29,171,31,99,224,78,78,170,118,96,219,14,53,179,6,99,217,214,150,22,140,147,177,108,95,24,220,29,140,96,238,142,247,
  30,56,128,120,34,133,114,185,76,192,29,43,81,241,252,134,81,40,180,91,182,47,106,103,60,184,130,91,246,56,235,150,101,110,89,135,70,99,250,140,
  185,115,93,20,23,11,108,44,219,215,104,143,63,120,13,110,217,227,45,43,231,220,245,173,6,238,14,254,139,1,119,202,42,224,238,24,171,31,91,65,
  115,119,243,106,167,171,175,209,135,179,156,187,194,66,1,19,83,83,88,44,20,120,253,152,168,185,59,119,246,44,142,31,251,248,230,207,181,200,238,221,
  123,142,177,156,187,119,222,127,31,11,11,132,18,97,248,139,129,10,93,215,67,224,238,117,148,74,165,155,87,93,228,182,219,111,63,118,248,240,97,252,
  226,151,191,194,208,224,0,146,169,52,143,64,171,11,16,211,2,157,125,54,147,17,153,187,155,27,94,248,157,59,119,29,123,226,137,39,240,221,239,253,
  0,195,195,195,72,103,178,161,5,116,137,80,96,155,155,166,105,156,185,219,119,224,32,76,211,60,214,8,220,29,123,242,201,39,241,155,223,252,6,115,
  115,115,66,245,171,33,192,83,74,232,104,114,5,3,97,140,64,187,101,129,27,247,31,168,187,133,21,0,202,67,15,61,140,169,153,25,12,143,140,33,
  149,74,113,73,92,152,26,84,21,64,21,50,38,8,120,33,58,23,219,117,177,180,188,132,120,34,129,157,219,183,31,19,93,27,80,20,229,227,205,91,
  182,224,232,71,31,83,67,133,46,75,226,52,61,114,230,8,90,54,110,91,164,221,123,77,115,51,182,119,118,160,167,107,3,22,167,38,241,191,92,190,
  252,177,239,251,183,232,0,208,179,121,51,42,229,50,74,165,146,28,36,29,98,238,234,165,88,132,198,218,179,107,7,110,222,178,25,105,85,65,154,106,
  192,207,76,1,109,109,109,200,229,114,4,18,117,180,181,35,199,67,140,141,192,41,27,2,119,4,226,200,224,206,115,28,196,98,49,116,119,118,96,244,
  114,63,206,92,157,196,208,212,20,230,151,150,144,73,38,145,131,208,45,91,42,151,3,183,172,166,133,192,93,208,41,31,30,106,170,186,142,75,151,7,
  240,106,165,130,79,79,157,196,124,205,130,227,122,136,197,98,208,36,67,133,227,66,53,124,104,70,20,184,83,193,36,119,81,224,14,62,25,210,188,157,
  203,17,187,136,166,67,241,137,222,91,21,52,119,74,169,180,76,168,14,85,30,44,175,6,238,60,150,239,76,101,63,98,230,68,184,91,22,139,139,139,
  80,20,133,24,42,212,250,242,8,113,1,9,123,69,80,131,108,152,6,160,190,91,214,229,33,198,162,254,81,148,196,121,117,237,184,65,13,131,205,177,
  23,67,46,170,170,34,151,147,235,199,110,45,243,16,99,33,10,219,39,223,132,58,73,92,40,2,144,9,44,121,206,29,173,124,175,86,43,242,99,133,
  184,101,53,174,125,244,60,191,222,204,26,194,190,14,5,119,98,206,29,97,238,226,40,46,22,224,251,254,173,97,183,236,173,140,185,171,143,163,241,27,
  83,131,130,7,133,65,35,69,85,49,126,229,202,173,145,184,235,212,201,19,82,20,10,115,203,214,89,162,67,154,59,6,79,21,69,33,154,187,129,203,
  64,40,74,158,255,84,42,229,219,88,20,138,166,105,28,220,241,185,175,80,1,23,20,168,217,188,64,173,181,165,5,35,67,67,176,44,235,182,240,166,
  117,107,4,184,59,218,123,224,0,204,120,162,158,185,11,229,220,41,0,18,137,56,84,85,195,0,209,220,221,22,181,51,222,186,194,88,246,104,167,8,
  238,232,88,214,163,246,40,230,185,94,36,57,119,183,173,180,199,223,122,13,154,187,163,76,115,199,180,219,181,106,13,243,243,115,40,149,74,183,93,11,
  115,119,235,127,117,220,245,207,197,220,29,97,154,187,166,230,102,104,10,80,169,214,48,151,203,97,120,100,24,131,3,3,240,125,255,246,213,78,215,109,
  141,62,252,154,53,119,36,231,238,246,207,181,200,246,29,59,143,48,205,221,59,239,190,71,52,119,154,142,152,100,168,208,234,192,221,239,95,255,29,42,
  149,242,237,171,46,210,215,215,119,36,204,220,201,90,123,97,28,165,5,193,173,12,220,29,249,240,125,44,22,10,183,175,100,168,56,242,244,225,195,248,
  225,15,127,196,153,187,120,60,33,45,192,198,81,12,123,177,237,154,129,187,3,7,111,134,105,154,71,34,113,151,162,40,71,158,57,252,12,94,125,237,
  119,152,154,158,145,13,21,17,146,56,85,160,165,72,120,46,3,119,62,3,119,245,207,174,123,238,185,23,179,185,28,6,7,7,165,110,89,85,0,22,
  225,140,53,241,135,61,185,45,203,66,44,158,192,245,27,186,62,148,78,151,162,40,31,238,220,181,139,27,42,120,82,159,30,104,239,56,231,162,106,117,
  94,83,215,117,96,217,54,106,22,25,24,104,170,130,27,183,111,135,162,40,31,138,134,10,84,170,85,110,168,144,17,188,86,215,41,31,94,32,30,143,
  99,93,58,133,246,150,22,52,101,51,216,190,126,29,102,167,167,113,244,147,227,1,169,214,214,214,206,221,178,117,147,107,77,158,189,135,129,69,181,86,
  195,190,189,123,240,181,155,122,81,89,88,32,233,164,229,50,150,74,37,116,180,52,7,139,36,226,196,45,107,112,254,87,156,154,170,18,53,27,70,46,
  170,162,96,106,122,26,179,75,203,248,244,147,79,49,85,40,96,38,159,199,114,173,6,159,62,80,117,0,138,101,219,80,13,183,142,123,212,52,13,242,
  245,173,7,119,170,162,98,106,106,6,63,248,229,203,24,155,158,134,67,183,106,18,225,33,116,203,150,203,101,222,45,27,14,48,150,83,226,228,5,216,
  88,182,90,173,226,234,124,30,154,65,26,71,24,21,229,73,13,21,180,91,150,131,59,45,234,22,245,35,202,11,132,194,84,223,135,101,219,82,136,177,
  101,217,2,184,163,221,178,172,161,66,229,40,82,142,0,142,50,179,114,89,150,40,199,162,82,172,57,58,208,100,224,238,78,177,161,130,101,109,179,235,
  16,156,162,96,44,235,73,130,50,1,220,81,92,12,223,39,190,150,40,112,23,160,235,224,58,4,232,209,151,145,11,5,119,226,220,215,177,45,145,185,
  187,75,82,170,141,141,142,220,85,90,90,130,193,12,21,33,114,147,156,38,39,146,185,19,169,65,238,150,109,220,45,251,9,124,207,227,226,48,241,34,
  187,94,189,99,131,151,120,208,5,66,110,217,187,34,23,169,86,171,119,69,49,119,158,16,96,28,30,203,138,73,125,107,90,91,153,91,246,174,240,166,
  117,103,4,184,123,191,247,192,1,196,227,9,148,171,85,174,185,243,92,135,216,220,69,230,78,112,203,82,230,238,174,168,157,241,206,70,123,243,230,45,
  91,222,239,236,92,7,77,85,96,209,100,62,145,185,83,233,43,66,177,88,100,154,187,187,26,237,241,119,174,230,105,200,102,179,239,181,181,181,195,140,
  153,28,53,18,230,46,135,114,185,4,223,247,239,94,13,173,220,245,175,99,44,107,154,230,123,55,223,124,11,54,118,119,35,75,53,119,149,114,25,185,
  249,60,134,7,7,88,67,197,23,59,93,166,105,190,247,252,243,223,192,13,187,111,64,46,55,135,137,201,73,20,10,139,240,60,162,185,75,166,82,104,
  105,110,22,153,187,187,63,215,34,125,55,223,242,222,243,207,63,135,254,254,203,248,240,200,17,44,45,45,17,25,187,16,98,172,42,132,185,75,103,210,
  88,183,118,173,200,220,221,29,181,136,244,15,31,124,224,193,119,31,123,252,203,248,241,143,127,138,43,87,198,185,230,174,110,98,167,105,210,118,204,192,
  221,135,239,191,139,98,177,120,79,195,11,223,219,219,251,238,3,15,61,136,191,249,155,191,197,212,212,36,50,217,38,190,128,38,76,237,196,77,173,30,
  220,221,2,195,48,222,141,92,68,211,180,119,31,255,202,87,240,243,95,188,204,115,238,24,36,98,82,248,64,239,165,133,242,189,168,75,179,102,1,138,
  130,125,189,55,69,223,194,95,122,244,203,24,28,26,198,228,228,36,18,137,100,160,146,210,67,154,187,6,181,75,65,183,108,13,137,100,10,27,186,54,
  190,27,6,119,239,118,111,218,136,51,167,79,147,110,89,230,150,149,32,146,28,253,47,206,188,184,21,212,243,97,187,46,202,149,50,154,91,215,200,71,
  178,99,199,78,44,20,10,168,84,171,114,28,77,29,177,166,72,228,38,135,70,244,49,227,121,30,82,134,142,206,116,10,251,183,108,198,166,235,175,123,
  151,27,89,219,218,218,144,207,47,240,156,59,217,169,172,69,86,47,48,112,97,91,22,218,219,214,224,142,61,55,32,94,173,160,45,155,69,140,78,74,
  251,251,63,195,232,213,9,33,231,174,84,10,186,101,181,144,238,78,13,130,68,130,240,92,159,186,4,201,158,126,112,239,30,76,93,186,132,74,173,138,
  185,98,17,75,165,146,252,88,177,108,155,184,197,35,240,111,120,129,48,239,165,40,10,10,139,139,120,255,227,99,56,121,250,12,10,229,18,22,151,75,
  112,21,5,21,202,119,233,0,176,180,188,140,150,214,182,16,184,83,228,94,173,8,98,141,16,56,62,150,75,37,188,248,198,155,112,92,34,240,115,92,
  151,196,113,57,98,206,221,226,34,161,7,13,93,122,69,144,131,92,34,204,172,2,60,10,148,158,132,54,84,235,12,21,83,147,240,60,175,174,200,67,
  129,216,206,230,73,41,201,129,152,63,72,73,102,164,142,235,186,208,2,205,29,129,68,181,106,245,190,64,115,167,240,151,77,40,129,98,173,126,230,24,
  196,209,56,188,99,131,56,54,20,58,126,183,44,235,190,8,112,167,35,72,170,174,103,79,165,185,47,203,90,225,142,229,192,107,154,76,38,249,112,89,
  90,100,100,120,232,190,210,50,25,50,195,143,102,79,235,156,51,2,185,73,34,128,107,52,231,206,195,228,228,228,125,141,221,178,174,71,109,157,46,165,
  7,125,201,41,35,145,156,158,39,81,131,170,170,34,147,78,227,114,255,10,57,119,149,74,249,62,102,168,208,52,157,51,165,158,144,57,228,184,174,220,
  18,226,144,30,108,211,52,209,220,220,204,114,238,238,11,111,90,247,69,128,187,183,246,247,30,8,162,80,106,85,201,45,75,136,53,151,218,72,20,36,
  18,9,192,247,209,223,127,9,174,235,222,31,181,51,222,183,2,115,247,86,71,231,218,160,161,194,182,72,169,13,125,148,16,115,158,131,194,66,30,147,
  147,147,247,175,180,199,223,183,26,98,73,36,146,111,117,116,118,114,253,175,235,18,230,142,106,238,238,191,22,230,238,190,127,29,224,46,145,72,190,117,
  176,239,32,54,247,108,70,34,153,148,192,221,232,240,16,46,92,56,15,223,247,239,143,194,208,226,223,220,223,192,173,241,230,179,207,62,139,29,59,119,
  174,104,168,168,89,150,152,115,247,64,163,107,82,183,200,109,183,223,254,230,225,195,135,241,217,103,151,112,228,200,17,44,151,74,60,196,152,237,253,74,
  99,112,247,192,170,139,28,58,116,232,205,251,238,191,31,63,123,241,231,152,184,122,69,2,119,154,16,102,33,142,101,153,91,86,96,238,30,88,201,80,
  241,38,49,84,124,7,179,51,211,28,220,177,169,105,125,105,167,194,95,68,151,75,4,220,237,63,112,16,134,97,188,25,185,136,105,154,111,30,122,244,
  81,188,248,210,207,81,169,148,17,79,36,35,134,154,34,184,83,235,144,75,173,70,178,162,162,192,157,2,64,249,210,161,199,48,48,48,136,153,153,89,
  36,18,201,160,91,54,60,18,172,227,189,148,160,238,210,247,80,171,214,144,76,165,113,253,134,174,55,37,122,80,211,180,55,186,186,54,224,244,153,51,
  72,9,245,99,1,254,21,139,235,89,190,109,0,139,56,6,160,91,130,93,171,162,163,179,83,62,93,59,119,238,194,66,161,128,90,181,6,131,33,120,
  102,199,101,31,30,50,20,7,108,5,209,177,84,45,27,53,219,66,213,170,161,84,46,195,87,84,152,166,249,6,255,50,182,180,180,32,151,155,167,13,
  121,122,228,212,84,28,42,139,172,145,235,18,158,108,93,107,11,210,142,5,45,158,0,0,116,196,99,64,105,9,167,46,94,12,12,21,149,10,97,238,
  88,253,174,220,208,70,36,113,81,99,89,203,182,177,174,165,25,127,254,212,147,176,175,94,145,46,248,39,180,24,93,167,44,17,124,69,11,210,18,35,
  192,29,20,165,110,1,207,247,161,248,30,74,229,50,49,24,143,145,244,177,229,114,5,150,227,160,90,171,6,244,96,113,185,132,230,230,102,98,143,10,
  1,60,25,220,213,251,29,0,96,121,105,25,63,248,229,203,184,50,59,11,187,86,133,227,251,208,52,3,213,106,89,24,203,22,22,104,58,178,124,13,
  4,234,81,138,0,14,251,29,106,86,13,163,179,57,216,158,15,79,213,224,122,36,51,207,170,9,33,198,51,211,211,240,60,143,215,143,133,35,105,60,
  63,66,18,39,100,12,59,142,75,222,79,4,36,163,42,10,230,230,230,130,69,42,149,242,67,181,106,149,71,161,16,137,174,124,155,134,161,145,231,137,
  65,110,1,176,96,62,108,151,128,187,135,234,12,21,134,161,11,202,77,63,154,220,20,242,5,195,162,62,230,214,76,36,18,117,134,10,5,128,50,50,
  60,244,48,3,119,190,31,152,139,86,42,82,145,250,205,168,223,33,102,154,240,61,15,185,92,238,225,200,137,233,169,8,230,46,156,84,237,121,66,191,
  153,23,0,110,102,168,72,173,98,168,64,165,82,126,152,49,119,170,74,152,59,159,71,208,120,82,207,9,31,205,218,68,115,23,147,13,21,15,135,55,
  173,135,34,246,245,215,123,15,16,183,44,201,185,35,99,89,143,54,84,72,134,10,69,65,50,153,20,67,140,31,142,218,25,31,90,193,80,241,186,104,
  168,176,28,171,46,196,216,117,93,204,205,229,48,51,61,253,240,74,184,235,161,107,64,44,175,183,119,116,192,52,9,57,237,216,14,106,150,133,89,2,
  238,30,190,22,112,247,240,191,14,112,167,105,218,239,250,14,246,161,171,187,27,45,205,205,129,91,118,46,135,225,145,17,244,95,186,4,223,247,31,249,
  66,167,75,215,245,223,29,126,230,89,236,223,191,143,104,238,38,38,176,88,44,194,178,29,152,134,142,116,38,27,214,220,61,242,185,22,233,237,237,253,
  221,159,190,240,2,6,7,135,240,193,135,71,176,184,88,144,192,157,166,235,132,185,51,116,30,98,44,140,101,31,137,90,68,250,135,119,223,115,207,111,
  159,120,226,9,188,248,210,47,48,58,58,138,100,50,41,51,119,186,94,23,129,230,251,126,216,45,251,165,134,23,126,247,238,61,191,253,242,151,191,140,
  191,255,135,239,224,202,248,24,50,153,12,205,185,51,200,214,204,26,42,116,153,181,16,221,178,55,246,222,4,211,52,127,219,72,115,247,219,175,63,245,
  117,252,242,151,47,99,177,80,64,50,69,146,74,216,88,150,7,88,168,42,23,242,139,78,77,85,85,105,94,151,18,169,185,35,204,221,151,190,132,177,
  241,43,24,191,114,149,219,113,77,35,132,30,133,10,140,240,188,139,212,91,121,176,45,11,201,84,26,93,93,27,127,27,102,238,126,187,121,203,86,156,
  60,121,138,128,59,234,53,13,42,48,162,166,118,144,54,53,8,19,59,2,238,214,202,71,178,117,219,54,210,80,81,46,19,67,133,174,5,118,220,186,
  94,83,68,246,58,185,130,145,178,102,17,163,94,42,149,250,173,196,220,205,205,205,73,243,94,209,45,171,133,240,111,120,215,116,93,15,150,85,227,105,
  177,142,77,90,90,215,172,105,67,169,84,18,192,93,181,202,245,93,50,246,21,251,29,196,32,61,159,6,132,185,72,36,18,248,234,61,119,194,158,39,
  192,97,222,118,177,198,208,240,250,241,79,48,206,44,185,181,154,141,152,162,113,220,21,166,6,235,9,29,153,88,83,53,21,7,247,238,65,126,120,88,
  186,33,222,62,125,38,120,118,149,203,37,196,147,73,62,239,213,116,77,48,180,4,158,159,200,140,97,207,67,165,82,197,251,31,31,67,127,136,83,153,
  103,209,118,0,176,88,44,98,77,123,7,111,168,8,148,3,74,195,102,86,182,239,3,62,74,165,101,252,248,245,55,164,240,176,88,60,129,106,41,176,
  33,40,211,83,147,112,93,79,32,152,149,186,2,97,22,5,88,159,243,65,187,4,40,46,96,216,57,102,26,152,37,154,59,130,86,106,213,234,163,213,
  74,89,206,29,84,149,186,193,114,216,169,233,80,158,43,160,6,109,30,164,71,167,219,143,54,0,119,136,200,27,20,2,220,92,22,197,232,70,196,162,
  56,176,44,11,201,68,130,78,198,67,143,149,209,145,225,71,25,184,35,167,6,117,71,32,201,225,248,111,238,242,24,102,155,54,84,0,192,213,171,87,
  30,109,200,220,129,42,255,217,179,40,112,203,122,1,69,75,153,85,206,166,218,22,159,154,102,51,25,12,16,205,93,67,112,247,232,233,19,159,64,213,
  84,232,186,22,120,122,233,135,243,208,98,26,98,236,186,30,111,106,213,117,157,100,173,12,12,192,182,237,199,194,134,138,71,35,120,149,87,246,237,239,
  37,134,138,74,25,182,101,113,222,145,165,36,178,187,10,0,146,148,216,25,184,220,207,22,168,219,25,31,93,193,80,241,10,51,84,136,110,89,145,185,
  19,186,101,31,91,105,143,127,116,53,196,146,74,253,255,217,187,210,224,186,202,243,252,156,237,238,146,181,221,43,201,139,108,73,118,136,89,108,144,23,
  112,83,3,133,36,16,7,176,1,203,96,103,106,72,77,82,146,41,233,132,116,66,58,228,103,102,72,102,72,103,72,167,100,38,36,5,18,160,53,123,
  128,144,16,214,36,52,193,196,11,54,120,183,228,93,187,174,164,43,221,237,172,253,241,45,231,251,206,61,87,146,105,66,59,161,158,201,100,242,71,111,
  238,119,116,245,61,231,121,159,37,249,18,1,119,161,221,178,215,205,6,220,125,254,47,6,220,41,51,224,224,23,87,95,186,26,29,237,29,82,20,202,
  240,240,8,142,247,246,224,0,97,238,174,159,233,184,174,171,162,125,124,113,186,40,148,84,42,201,235,199,132,40,148,235,103,61,228,83,127,189,246,197,
  77,155,186,113,240,224,33,188,253,246,219,152,18,162,80,130,134,138,84,77,42,24,133,114,253,140,67,214,173,91,247,226,85,87,95,141,237,79,62,141,
  254,254,126,36,18,9,137,22,212,66,152,59,178,150,77,137,204,221,245,213,153,187,75,47,123,145,69,161,12,13,14,160,166,166,6,177,168,48,64,215,
  43,160,17,137,168,83,131,154,187,23,67,135,24,134,241,226,186,117,159,195,83,79,63,131,82,177,136,120,34,73,33,145,32,199,82,212,64,167,188,144,
  54,166,40,40,151,77,120,158,135,101,23,95,82,161,185,83,0,40,44,10,101,96,96,80,160,6,25,246,210,104,237,130,38,81,131,254,189,15,14,137,
  76,211,68,34,89,131,249,11,218,94,144,62,137,166,105,47,180,181,45,192,222,189,123,145,74,38,233,82,147,17,107,58,52,85,231,106,41,25,220,137,
  247,142,159,113,96,153,38,90,90,90,43,153,187,108,54,139,82,185,76,45,160,170,64,75,169,18,115,23,212,220,137,172,145,71,147,98,45,179,12,93,
  55,16,139,197,94,224,67,26,26,27,48,50,74,214,178,254,106,92,140,97,22,1,182,87,177,239,114,29,34,36,179,44,242,7,212,162,33,198,233,76,
  179,143,86,52,85,243,153,187,16,6,85,6,21,242,90,214,117,92,64,85,112,65,219,2,140,142,143,163,96,17,50,205,241,60,196,98,81,112,152,106,
  90,22,20,205,160,125,39,2,53,88,145,146,44,15,240,92,15,182,109,35,149,74,161,251,198,13,152,234,237,65,177,92,66,158,174,99,127,254,95,191,
  199,209,35,71,200,39,153,202,23,48,167,174,158,26,42,228,213,56,251,111,95,231,37,67,35,219,182,17,141,70,144,31,31,199,72,46,71,174,134,104,
  4,241,104,140,165,148,144,33,185,137,113,52,52,52,86,24,42,252,188,109,86,90,207,138,235,61,46,131,83,224,97,124,108,12,143,62,249,36,198,166,
  166,16,85,20,24,145,8,82,241,4,6,70,134,253,33,3,253,253,88,176,112,17,109,105,19,109,57,254,235,1,57,38,151,23,215,251,230,10,23,147,
  83,121,140,90,38,60,250,61,97,71,216,123,182,79,98,238,214,151,75,37,178,96,22,50,111,104,117,87,0,158,250,8,134,1,11,215,181,1,143,74,
  18,61,146,97,168,194,131,101,89,235,37,82,141,128,59,131,223,221,236,152,60,15,244,7,184,21,49,204,34,53,72,232,65,106,168,136,199,48,74,162,
  152,101,82,237,120,111,207,134,252,84,30,186,166,209,227,241,66,205,172,28,220,89,38,161,6,89,231,134,99,115,222,203,115,93,12,14,12,108,152,22,
  220,169,138,74,234,93,69,112,39,228,61,251,46,26,151,163,24,230,119,72,205,130,185,219,224,107,238,52,190,254,246,35,80,28,94,125,193,86,228,44,
  14,197,136,68,48,167,182,22,135,8,115,183,33,120,105,173,15,222,100,177,88,236,249,174,21,171,16,75,36,80,42,150,80,54,203,220,80,33,229,220,
  81,91,78,60,22,131,231,186,108,239,187,33,236,102,92,63,13,115,247,124,107,235,92,9,220,89,148,38,84,232,75,164,109,89,44,10,229,198,233,128,
  196,134,89,48,119,207,49,230,14,32,81,40,2,115,119,227,108,192,221,134,191,24,112,55,147,230,238,185,106,154,187,19,199,142,98,63,209,220,221,248,
  161,142,43,22,139,61,183,121,243,150,80,205,157,161,235,210,90,118,255,7,251,25,184,187,49,108,7,172,0,168,248,127,177,230,178,53,207,222,186,101,
  51,215,220,229,114,147,48,34,190,230,14,170,26,106,168,160,81,40,55,133,125,146,27,3,154,187,103,175,249,220,181,120,234,233,103,113,246,204,105,185,
  29,172,34,223,182,106,206,221,77,211,105,238,158,253,244,103,63,131,135,31,126,4,3,84,115,167,25,134,111,102,21,32,42,235,52,215,232,171,31,91,
  203,46,239,90,133,72,36,242,108,232,16,93,215,159,189,97,253,122,60,253,204,115,152,154,154,226,204,157,17,244,250,10,239,248,242,165,166,240,164,253,
  170,204,221,231,175,187,1,199,122,122,9,254,77,166,200,203,169,64,170,137,9,101,34,123,39,10,202,192,53,119,9,204,95,208,246,172,180,164,81,20,
  229,217,182,182,5,1,205,29,233,151,211,37,106,80,149,138,163,130,156,11,67,145,150,101,33,77,152,59,255,62,97,107,89,166,185,211,232,111,78,68,
  15,210,130,65,90,10,252,90,22,235,222,173,114,25,241,104,20,241,120,226,25,126,92,245,13,13,92,115,23,76,139,169,132,167,138,116,68,226,102,136,
  23,71,81,127,93,58,157,246,191,241,186,174,33,159,207,35,18,137,72,137,61,114,206,29,120,252,44,63,38,129,88,179,76,203,215,23,209,205,145,30,
  12,49,86,117,213,39,213,132,189,111,85,89,34,29,96,217,132,212,188,249,170,43,113,242,200,97,140,229,11,200,78,78,98,178,92,134,93,210,252,23,
  211,66,126,10,181,245,77,210,171,154,156,49,44,62,100,200,102,86,203,66,77,67,3,86,45,187,8,243,233,85,80,164,59,223,71,127,245,107,14,83,
  49,62,145,195,156,134,166,128,52,84,13,52,131,121,18,122,116,36,43,130,139,223,190,179,3,165,209,81,164,18,113,212,36,147,0,128,28,181,33,232,
  0,208,215,119,150,128,59,106,100,245,27,218,100,223,73,88,199,134,162,170,232,235,235,195,99,199,123,73,39,10,92,196,52,13,141,117,117,56,57,48,
  224,127,25,203,165,210,198,98,161,200,57,121,113,0,231,190,68,59,174,80,32,97,91,164,161,130,212,52,184,40,148,77,140,228,139,56,157,29,67,54,
  155,221,24,208,220,13,240,235,181,226,219,44,50,216,140,177,115,93,33,126,206,215,224,217,182,141,154,100,2,147,185,137,80,230,110,99,53,230,206,31,
  80,197,177,65,193,29,83,172,121,240,208,215,215,215,141,176,124,225,221,59,119,72,107,89,246,146,227,199,97,219,82,106,53,183,132,56,194,90,54,153,
  156,94,115,87,42,149,186,223,223,247,30,7,119,156,61,21,9,205,64,137,29,91,203,26,145,8,101,238,142,192,113,156,238,224,165,181,49,228,94,127,
  106,229,170,75,43,234,199,24,184,179,5,91,142,162,106,72,196,99,162,230,174,59,236,102,220,88,13,0,44,106,111,127,202,175,31,35,107,89,219,178,
  120,225,115,64,115,215,61,29,144,216,56,11,112,247,84,166,185,153,167,253,144,69,76,25,3,131,131,48,77,179,123,54,224,174,251,227,1,238,20,69,
  121,146,229,220,177,110,217,144,156,187,77,31,234,184,20,69,121,82,116,203,206,34,231,110,211,57,13,57,255,252,11,158,100,57,119,111,188,249,22,70,
  71,71,136,91,54,66,114,238,120,11,143,110,160,174,18,220,109,10,27,178,41,32,137,219,46,186,101,197,250,49,41,64,183,10,115,71,193,221,45,85,
  31,252,226,37,75,182,111,186,229,22,252,236,177,199,113,248,240,97,30,133,194,221,2,210,165,166,241,0,99,121,45,187,10,209,88,108,123,181,181,236,
  118,230,150,29,28,28,170,140,66,145,138,163,180,10,206,139,229,220,1,192,242,139,187,194,127,133,89,206,221,177,99,199,144,74,165,248,209,168,66,121,
  151,104,67,12,50,119,60,200,173,92,70,44,158,192,194,69,237,219,43,134,124,114,233,82,236,220,189,155,186,101,9,60,213,69,214,72,243,155,169,228,
  202,75,79,64,46,140,34,44,35,157,201,200,50,159,246,142,206,237,197,66,1,147,185,73,95,18,23,76,73,86,85,121,45,232,201,151,154,203,27,145,
  200,31,78,93,51,80,91,91,187,157,35,72,150,115,167,211,56,70,77,171,28,32,230,173,248,133,233,126,4,132,35,220,158,44,109,174,161,177,209,255,
  198,251,57,119,134,188,247,213,196,156,143,74,244,200,6,216,14,209,177,176,80,87,215,177,97,1,60,126,128,181,33,192,117,5,183,172,42,118,108,40,
  252,63,149,165,245,212,114,160,168,104,77,55,66,241,60,152,229,18,74,101,19,165,82,73,254,219,85,160,185,219,146,161,34,208,239,224,175,3,101,104,
  100,154,38,90,90,91,241,15,159,191,22,187,247,238,69,223,232,40,206,140,100,49,94,200,99,48,155,21,214,178,227,227,104,74,55,251,218,7,77,19,
  236,8,34,122,71,192,204,234,160,100,150,209,146,105,34,194,242,214,86,180,183,18,62,120,36,151,195,143,158,123,222,39,58,251,251,251,176,176,163,147,
  27,89,229,18,15,84,144,155,108,128,235,186,208,53,29,135,15,31,193,3,71,143,34,170,27,152,223,212,128,185,244,129,159,25,30,150,116,144,155,75,
  197,226,127,36,83,53,244,90,165,38,175,192,67,246,92,159,251,242,251,77,93,140,100,199,208,103,150,97,57,14,118,31,59,134,88,44,138,57,241,4,
  198,41,241,89,89,63,38,246,155,73,182,28,150,12,103,243,255,109,211,142,13,215,37,58,47,21,68,138,229,65,69,223,200,8,60,207,219,44,13,57,
  117,242,196,102,230,150,245,179,86,252,119,16,81,10,199,192,157,235,80,67,5,165,6,109,203,164,223,53,29,125,103,207,132,255,21,102,33,198,26,53,
  84,200,81,40,110,165,91,86,160,6,109,203,100,107,42,244,144,134,138,205,236,111,167,132,32,139,197,194,150,189,123,252,40,20,185,2,142,246,156,8,
  110,89,49,10,37,144,115,183,69,248,185,80,0,108,14,225,85,158,88,177,114,21,95,203,154,166,73,195,36,9,44,117,29,95,212,7,0,201,68,2,
  170,170,178,110,217,45,97,55,227,230,105,214,178,79,136,57,119,60,84,207,113,160,128,214,143,57,142,232,150,221,82,237,142,223,60,19,98,153,83,87,
  247,68,125,125,61,162,209,24,13,225,113,131,57,119,91,102,66,43,155,63,10,220,245,191,31,133,162,105,218,227,171,87,173,70,199,226,197,168,161,204,
  93,185,84,194,224,240,8,78,245,246,48,230,238,11,51,29,215,150,106,63,252,214,205,91,184,230,238,212,153,179,152,24,31,135,101,219,21,110,89,129,
  185,251,194,172,135,92,114,201,37,143,223,254,197,47,114,205,93,46,55,33,133,24,51,205,157,216,80,81,200,231,241,250,235,175,97,98,124,252,11,51,
  14,89,187,246,242,199,55,118,111,196,51,207,60,131,222,222,227,164,126,44,26,19,218,193,130,126,7,149,107,238,0,96,199,31,254,11,195,195,195,95,
  8,181,33,0,80,46,188,240,162,199,215,111,88,143,159,252,251,35,56,121,242,20,106,106,231,144,210,40,186,200,12,58,54,84,213,7,120,83,249,60,
  202,166,137,11,151,119,33,30,79,60,142,176,148,18,69,81,30,219,216,189,17,63,127,254,231,152,24,31,147,152,187,202,197,178,184,111,244,248,161,48,
  230,238,162,229,23,135,131,187,207,126,246,26,156,60,117,26,39,78,157,22,42,48,52,217,107,42,57,54,196,141,169,111,226,51,77,14,238,30,11,106,
  238,30,235,92,188,24,123,246,188,135,120,60,206,207,158,237,29,213,192,230,154,35,24,1,57,66,186,247,203,72,167,101,112,135,206,197,139,49,149,39,
  57,119,226,98,95,11,108,77,197,102,42,82,187,36,82,34,126,198,154,109,89,208,117,3,201,100,242,49,254,224,211,141,77,200,102,73,20,138,143,224,
  133,16,55,246,9,20,49,99,194,7,22,34,184,227,75,29,215,97,80,85,145,114,238,12,35,172,91,86,240,251,170,12,61,186,21,200,133,196,155,122,
  220,118,232,121,129,16,99,203,118,160,232,174,31,172,163,138,208,84,173,232,55,243,7,248,109,172,158,235,160,108,89,92,77,8,47,16,98,156,207,79,
  201,81,40,108,45,171,106,21,175,8,236,182,100,59,199,82,185,140,76,38,131,91,174,253,44,58,27,27,16,87,20,88,229,50,202,166,197,163,80,8,
  184,155,152,224,224,78,11,233,117,80,136,165,149,179,70,240,252,123,223,50,203,184,224,252,165,88,177,100,49,26,61,7,3,217,49,244,246,245,227,236,
  216,24,246,31,63,65,254,14,2,88,86,40,228,111,154,183,160,13,10,148,10,237,163,92,193,91,233,22,240,232,74,60,123,242,36,162,209,40,230,53,
  53,97,233,162,133,72,70,34,248,229,239,126,7,219,113,158,215,1,40,150,101,109,45,22,10,63,77,166,106,0,120,62,123,234,122,60,142,78,54,84,
  8,233,112,158,135,157,187,118,97,135,227,160,38,26,69,75,67,61,46,88,176,0,167,135,135,81,44,151,183,74,247,201,224,64,63,206,91,90,15,203,
  178,125,206,145,59,150,4,120,42,172,104,249,222,151,254,27,201,77,226,204,64,63,122,6,6,80,162,36,103,16,220,109,21,163,80,152,176,210,11,86,
  192,49,3,49,29,192,23,158,54,249,13,99,111,7,167,78,159,222,26,10,238,118,254,113,135,148,115,199,184,71,191,64,56,0,238,4,52,105,210,8,
  205,57,181,181,211,107,238,76,211,220,186,103,215,187,96,212,186,227,200,13,109,220,115,34,150,215,82,106,208,48,12,212,213,213,161,167,231,24,44,203,
  218,26,212,220,109,13,1,119,143,118,173,88,21,202,220,5,187,101,217,90,86,81,85,28,57,124,8,150,101,221,22,118,51,110,157,102,45,251,104,107,
  235,92,24,186,6,211,118,104,36,144,205,13,21,140,96,163,57,119,183,77,7,36,182,206,130,185,123,52,157,201,32,18,141,82,141,36,233,150,29,30,
  26,68,169,84,186,109,54,204,221,214,63,55,238,250,63,19,98,252,200,37,43,86,162,163,163,3,13,245,245,0,192,67,140,5,67,197,237,51,29,215,
  109,213,126,248,205,221,155,112,217,165,171,49,62,54,142,254,193,65,140,101,179,48,45,27,177,88,148,135,24,151,74,69,236,219,199,221,178,183,207,122,
  200,133,23,94,244,8,11,49,254,205,239,222,70,118,44,11,93,55,96,24,58,221,123,145,244,74,198,220,165,211,141,162,230,238,246,25,135,172,185,108,
  205,35,183,110,217,140,231,95,120,9,71,143,28,65,60,30,175,236,148,175,194,220,9,224,238,246,170,15,254,19,231,157,247,200,205,221,27,241,232,79,
  127,134,99,71,143,18,67,5,205,88,171,112,11,104,190,230,94,215,73,20,74,217,52,113,209,197,93,136,197,98,143,132,34,72,69,81,30,217,212,189,
  9,191,120,233,23,24,29,25,65,109,77,138,67,35,174,247,170,184,247,253,247,124,133,105,238,92,15,203,8,115,87,137,32,63,125,245,167,9,115,215,
  123,28,241,68,18,80,53,46,137,171,102,199,101,208,133,255,197,246,8,215,18,79,36,177,112,81,251,195,21,199,181,228,188,243,176,115,215,46,41,231,
  142,112,143,6,63,30,141,134,230,135,9,250,249,165,230,17,183,108,58,211,12,69,81,30,230,67,58,58,58,31,46,22,10,152,154,202,83,83,139,94,
  185,88,166,187,119,9,220,133,186,4,93,148,77,11,186,166,243,151,38,21,0,154,132,134,10,178,95,15,46,150,53,73,80,230,71,64,64,98,44,216,
  133,198,188,142,245,13,62,115,167,48,230,78,23,114,238,180,208,148,56,133,195,83,178,185,147,129,5,163,70,24,184,139,197,98,62,130,100,89,104,149,
  81,128,50,130,244,2,249,182,92,18,71,189,140,10,221,133,217,182,5,120,170,252,61,201,231,243,149,224,78,211,164,126,95,207,243,184,215,84,196,191,
  68,207,21,197,188,134,122,148,138,69,148,202,101,33,61,78,204,185,155,152,96,18,6,169,64,141,199,96,11,244,172,92,156,234,162,88,42,96,101,215,
  10,108,253,220,53,232,106,95,136,116,109,138,179,71,101,49,212,165,191,239,44,218,59,58,5,170,92,9,116,204,85,182,227,178,126,51,67,55,112,126,
  107,11,12,219,198,117,151,174,198,72,46,135,147,253,3,216,127,250,52,222,220,209,195,17,100,151,235,186,47,164,51,205,235,99,241,120,64,247,232,71,
  1,74,205,172,130,37,193,117,93,188,119,248,48,122,79,159,65,50,26,65,67,77,10,75,22,44,192,137,129,65,236,57,112,96,27,127,240,204,80,81,
  223,208,64,193,157,32,75,16,216,83,71,56,42,155,39,36,219,40,20,10,232,31,26,198,158,158,30,44,106,106,196,69,157,29,216,71,125,216,65,112,
  183,109,114,34,71,193,157,39,125,15,68,81,31,209,121,201,105,49,158,231,66,163,235,192,99,3,131,120,117,239,251,248,160,167,39,252,175,240,174,157,
  12,220,105,18,193,236,63,7,193,45,43,160,71,70,13,42,138,130,250,250,122,28,239,237,129,231,121,219,66,135,88,150,181,109,207,174,119,229,181,172,
  227,39,37,138,71,100,7,220,178,156,185,59,121,2,166,105,110,11,94,90,127,23,2,238,126,18,6,238,92,26,135,109,241,253,47,145,197,37,226,113,
  64,81,112,236,232,17,152,166,121,71,216,205,184,173,26,0,104,239,232,252,177,216,45,91,182,88,222,182,204,220,141,79,76,224,236,153,211,240,60,239,
  142,106,119,252,182,153,13,21,169,31,167,51,105,206,220,177,156,187,225,161,33,20,139,133,59,102,3,238,182,253,197,128,187,105,255,53,52,52,252,120,
  69,215,10,180,181,183,163,174,166,134,100,17,148,138,24,29,205,226,120,111,15,246,237,219,7,199,113,238,8,74,224,130,199,21,122,166,233,116,250,161,
  238,77,183,160,163,163,29,253,103,251,112,166,191,31,227,227,57,184,158,139,104,36,130,100,50,129,250,186,58,184,174,139,131,7,14,224,237,183,127,7,
  207,243,190,84,237,153,84,12,185,226,202,43,31,186,233,166,155,176,239,253,15,240,251,63,188,131,82,177,128,72,52,134,136,17,225,118,80,133,94,5,
  181,169,36,90,90,154,69,230,238,75,51,14,185,238,134,13,15,93,121,197,90,60,241,31,255,137,161,161,97,164,82,73,24,70,212,143,0,150,90,65,
  253,40,148,154,100,18,170,170,226,143,239,190,131,193,129,129,47,85,125,240,107,46,91,243,208,167,46,93,133,135,126,252,19,100,179,99,168,169,169,161,
  87,178,86,185,80,19,53,119,154,134,66,177,132,178,105,226,130,139,150,35,30,79,60,20,58,36,30,79,60,116,205,231,174,197,83,207,61,143,114,217,
  68,92,240,154,134,109,236,252,102,42,31,127,49,112,23,198,220,209,40,148,117,56,120,232,48,134,135,71,16,139,197,248,226,88,151,26,65,197,202,37,
  217,10,202,98,232,76,211,68,44,30,71,219,194,69,15,33,16,133,242,163,121,243,230,97,255,7,31,8,21,24,4,185,168,186,81,177,53,13,203,185,
  99,236,144,227,146,236,168,76,166,57,220,80,81,54,45,41,134,78,211,52,90,127,85,153,148,65,168,16,79,168,27,117,225,210,148,88,219,52,161,235,
  6,226,241,196,143,42,12,21,236,33,139,134,10,6,75,131,97,46,190,37,65,72,105,224,61,78,100,207,210,36,105,238,52,157,24,42,4,82,51,24,
  102,40,254,214,243,86,208,160,222,139,231,170,186,80,20,135,171,172,116,0,74,217,52,37,67,133,206,62,129,166,9,134,98,69,238,22,16,238,125,194,
  76,144,12,85,230,22,84,224,50,252,172,248,221,178,170,10,131,190,69,41,170,202,17,60,63,28,225,252,121,167,44,219,113,185,54,177,32,88,150,207,
  183,208,80,55,31,220,141,143,65,147,178,6,253,110,7,145,64,101,231,207,83,122,105,98,226,202,174,46,92,189,162,11,17,154,230,99,209,132,56,214,
  235,64,192,93,127,31,218,168,230,142,229,60,7,11,132,89,221,46,231,189,28,146,101,91,155,170,193,245,151,175,133,117,230,52,26,13,13,123,123,122,
  177,239,4,161,5,89,67,133,6,96,133,99,219,47,53,183,180,94,31,141,197,248,0,177,200,153,61,100,169,64,141,30,135,170,170,152,202,142,34,166,
  169,152,215,212,132,79,182,181,161,45,157,70,223,224,32,122,207,156,249,123,137,44,24,30,26,196,226,79,212,17,112,231,9,232,93,12,49,118,200,119,
  129,157,187,231,145,30,179,95,253,254,29,188,29,139,98,121,71,59,46,91,250,73,180,52,212,35,75,220,178,10,251,36,43,1,96,124,108,236,165,150,
  230,150,235,98,241,56,28,55,224,251,225,15,92,140,160,243,155,218,20,5,200,23,75,120,255,232,17,156,24,30,65,79,95,31,118,236,219,7,0,47,
  77,203,220,137,152,87,116,44,219,60,165,143,36,245,57,52,10,197,117,108,212,213,213,163,236,56,120,253,15,127,128,231,121,119,86,99,238,238,20,153,
  59,215,113,121,20,138,184,26,183,28,87,146,196,89,102,25,145,104,20,53,212,142,43,14,144,142,203,119,110,20,95,58,115,250,228,117,233,166,52,18,
  137,36,133,160,150,159,115,71,139,84,44,203,146,250,4,20,69,193,161,131,7,224,56,206,157,97,55,227,157,213,64,68,91,219,194,31,102,90,90,16,
  141,198,57,37,200,13,21,228,85,24,182,105,98,120,120,8,195,195,195,95,153,14,119,221,57,11,151,192,15,91,154,155,97,208,182,28,215,245,80,46,
  21,49,52,52,4,219,182,191,50,27,112,119,231,199,3,220,41,138,242,195,160,230,174,88,42,99,120,112,16,39,78,158,100,245,99,95,153,233,184,190,
  82,229,135,63,24,166,185,99,191,81,98,253,152,144,115,247,213,89,15,89,186,244,252,7,183,221,177,13,195,195,35,120,235,55,191,37,13,21,170,70,
  155,65,53,162,183,15,49,84,80,230,238,171,51,14,233,90,185,234,193,219,182,254,45,94,254,197,203,216,127,240,16,103,238,216,117,172,235,97,110,89,
  143,123,83,40,115,247,213,170,15,190,115,241,226,7,183,108,190,21,143,63,254,4,14,28,58,44,49,119,122,160,192,86,214,220,105,124,45,187,236,226,
  21,136,199,19,15,86,29,178,113,99,55,126,245,202,171,232,31,24,228,5,18,211,39,244,66,240,217,17,230,206,245,220,234,224,238,138,43,175,124,112,
  98,98,2,71,143,28,70,50,153,144,10,36,164,173,169,26,174,150,242,215,178,36,175,126,81,123,251,131,21,244,224,5,23,92,136,157,187,118,35,26,
  139,67,55,12,190,53,213,121,21,134,30,82,63,22,72,104,160,119,142,105,89,72,103,90,160,40,202,191,241,227,90,184,168,253,223,138,165,18,114,185,
  28,215,220,233,194,206,209,247,158,200,110,1,166,245,118,5,81,159,235,17,215,134,161,235,152,51,103,142,255,76,26,155,154,144,29,205,242,176,194,10,
  191,131,192,26,137,171,65,38,234,19,125,142,14,237,143,55,45,75,98,238,16,139,70,144,47,20,104,5,111,160,37,36,64,13,242,164,152,10,230,206,
  21,8,5,242,169,164,181,172,105,217,136,56,14,213,63,24,85,114,158,229,16,99,175,202,74,208,161,1,174,186,199,121,99,66,205,21,105,183,172,94,
  193,61,170,33,207,64,102,238,28,199,70,169,68,94,128,248,64,150,28,75,115,240,42,152,59,18,96,172,72,105,49,65,91,142,120,68,165,82,9,157,
  29,237,104,173,175,67,190,80,144,26,42,202,98,253,152,212,45,75,13,21,108,131,237,51,119,240,137,53,199,166,165,232,101,212,213,55,224,139,55,221,
  136,235,87,118,97,73,75,6,182,109,242,240,201,49,226,51,133,6,96,181,235,186,47,167,51,205,235,226,137,56,72,144,145,194,147,98,66,247,190,20,
  15,155,150,137,230,76,6,109,13,245,200,196,227,232,108,105,70,99,170,6,163,147,147,48,29,7,103,137,90,237,101,13,180,159,81,211,180,117,153,230,
  22,222,48,229,187,54,228,126,7,79,200,24,86,160,96,120,104,8,123,222,219,3,199,117,208,218,216,128,37,11,22,32,93,91,131,119,247,239,199,216,
  248,248,93,0,20,13,192,165,0,148,220,196,196,47,211,233,204,58,18,194,230,86,166,36,179,8,230,64,136,177,7,15,5,211,194,222,222,227,232,25,
  24,64,99,42,133,222,190,126,188,190,99,199,93,161,55,227,158,93,127,196,218,43,174,130,170,168,176,28,91,40,241,240,225,169,88,129,97,243,158,51,
  23,186,170,162,231,212,41,252,167,101,227,76,32,23,50,8,238,238,146,192,157,224,55,113,3,162,62,182,100,102,81,40,170,170,162,181,117,46,78,157,
  58,137,108,54,123,87,16,220,173,14,128,187,95,14,244,247,173,75,55,165,17,141,197,120,88,52,223,92,139,110,89,170,185,75,37,147,136,199,98,56,
  122,244,8,166,166,166,238,10,187,25,255,97,26,67,197,191,182,180,204,229,204,157,105,154,176,44,147,36,90,82,230,206,243,60,18,133,66,52,119,119,
  85,187,227,239,154,9,177,212,214,214,254,160,169,41,13,163,162,91,150,55,84,124,109,38,180,114,215,199,6,220,253,96,213,170,85,104,239,92,140,57,
  181,181,80,85,21,133,98,9,35,35,35,56,222,115,12,135,14,29,252,240,199,165,40,202,15,194,186,101,197,156,187,84,50,1,203,50,241,254,251,92,
  115,247,181,106,67,190,22,178,150,125,64,92,203,142,141,143,113,112,231,215,143,169,208,117,131,71,161,20,242,121,188,245,230,27,200,102,179,255,56,227,
  144,53,151,173,121,128,175,101,15,31,146,12,21,186,166,145,94,205,0,37,226,121,30,82,73,162,79,165,224,238,31,167,91,203,62,112,115,247,70,60,
  250,179,199,200,90,150,135,24,83,104,196,6,4,238,127,77,83,145,47,20,69,112,247,64,53,67,197,3,155,186,55,225,197,151,94,198,240,208,112,5,
  184,211,3,196,90,16,220,169,138,2,211,180,224,122,46,150,95,220,5,69,81,30,168,106,168,56,113,188,87,112,108,248,97,46,106,200,181,236,27,248,
  60,30,218,70,234,199,162,88,184,168,29,65,183,236,3,159,92,186,20,187,119,239,161,224,142,218,17,244,128,36,46,144,148,193,98,32,130,17,104,101,
  211,68,83,58,195,62,141,194,192,29,138,133,2,38,114,57,63,169,90,80,14,202,9,189,149,220,35,7,119,76,115,103,154,136,24,134,12,238,50,233,
  38,100,199,198,165,213,120,120,206,179,15,44,24,53,232,219,113,29,56,92,204,228,194,178,109,25,220,233,134,65,98,129,168,238,84,171,80,110,42,146,
  29,215,243,61,33,254,181,76,111,79,118,99,178,118,48,201,45,203,214,178,186,30,22,164,167,72,70,214,202,86,80,81,14,79,46,56,149,155,0,120,
  183,44,93,203,234,149,73,49,193,126,95,47,32,133,103,168,197,162,172,157,235,146,140,47,118,148,16,187,101,1,8,134,138,64,145,138,39,62,3,79,
  218,92,155,166,137,24,85,80,249,82,120,171,18,220,13,14,244,195,182,109,98,168,224,245,99,178,29,129,209,179,142,176,115,52,77,19,53,53,41,220,
  113,243,77,152,95,87,131,98,177,8,75,88,168,5,235,199,190,94,204,231,253,242,8,217,140,91,181,223,193,52,203,184,234,111,174,64,123,115,6,235,
  86,175,194,170,206,14,26,42,226,73,245,99,26,128,203,0,64,213,180,107,155,91,90,168,164,170,146,34,231,145,64,236,88,28,7,158,167,96,248,204,
  25,68,45,19,29,115,231,98,126,83,35,106,226,113,76,149,75,24,26,29,69,46,55,241,117,54,100,13,0,37,151,155,120,37,147,206,92,19,79,36,
  8,207,5,79,198,191,33,221,178,174,107,35,59,145,195,206,99,199,224,152,38,62,177,96,1,218,50,25,156,28,24,194,187,123,223,187,155,189,51,178,
  33,68,65,56,56,112,77,219,194,118,154,182,235,134,124,2,25,61,90,182,67,242,61,108,27,187,15,29,196,224,196,4,12,85,197,246,215,95,67,185,
  92,126,69,196,93,107,4,220,251,202,88,118,228,154,185,243,230,67,85,21,216,150,45,24,139,125,230,142,28,23,37,60,105,75,107,34,145,68,209,113,
  241,214,187,239,98,34,151,187,59,8,238,214,4,108,185,175,156,61,115,234,154,166,166,52,98,241,184,196,214,57,142,235,39,36,83,112,167,40,10,106,
  82,41,68,163,81,244,28,61,130,124,161,112,119,216,205,248,245,105,52,119,255,226,71,161,208,134,10,203,226,127,86,2,57,119,119,79,7,36,190,62,
  243,90,54,249,47,77,233,12,162,81,57,231,142,118,203,126,99,54,204,221,221,31,15,112,7,0,203,150,45,255,126,123,71,39,26,234,235,0,85,133,
  89,246,187,101,143,30,57,242,63,59,174,117,215,221,240,253,43,46,255,107,20,139,69,12,12,12,98,108,124,28,150,105,194,136,68,144,72,36,252,110,
  217,189,123,177,243,221,29,223,56,167,33,157,139,23,127,255,203,95,254,50,38,115,147,120,227,205,183,48,56,52,8,85,213,72,5,153,65,146,50,52,
  69,129,102,68,36,183,44,101,238,254,41,108,200,55,2,71,115,255,182,59,182,225,149,95,191,134,125,251,246,33,30,143,115,228,40,102,125,176,43,129,
  129,137,250,57,115,224,184,46,3,119,255,84,245,193,47,92,212,126,255,214,219,182,98,251,246,167,176,111,239,123,72,165,88,121,109,248,0,85,200,186,
  29,207,77,138,154,187,251,171,14,185,245,214,91,241,234,107,111,226,212,105,18,230,194,235,199,2,148,160,232,119,96,55,167,66,193,157,231,121,88,222,
  181,18,138,162,220,95,49,100,237,218,203,239,159,156,156,196,129,3,251,137,91,214,48,56,185,169,6,114,62,164,107,57,112,239,19,230,46,142,69,237,
  29,149,159,228,130,11,9,115,23,139,197,56,122,52,40,177,166,115,230,200,55,179,114,222,11,149,192,194,42,151,24,184,187,159,3,137,182,182,133,247,
  151,77,19,99,99,99,136,68,12,14,79,195,143,40,72,11,202,102,86,70,15,26,186,142,122,162,101,85,124,205,221,104,150,6,73,27,116,199,168,10,
  32,91,13,201,183,149,189,166,34,177,198,170,198,235,234,234,253,111,124,36,26,167,134,10,223,80,204,237,56,146,161,88,220,154,82,4,80,81,94,224,
  114,189,125,148,174,66,8,184,163,211,13,97,201,95,221,204,90,249,12,24,239,34,22,166,11,225,38,228,193,23,242,121,64,81,4,72,36,15,8,207,
  26,244,132,142,7,121,128,231,248,71,200,31,124,46,151,3,235,150,13,27,32,153,186,69,221,35,253,149,45,9,197,157,30,221,164,82,174,198,127,240,
  3,253,125,36,19,141,243,191,90,64,119,7,41,196,141,37,149,148,75,37,52,212,213,225,175,150,47,67,185,152,135,45,212,47,232,154,138,241,177,49,
  201,193,252,205,82,33,15,221,48,72,20,138,8,237,2,105,137,124,177,105,145,88,154,91,55,220,128,171,150,93,136,75,22,45,66,185,92,226,165,169,
  158,235,250,240,23,192,167,8,14,214,63,211,220,210,2,151,118,201,137,207,192,11,60,3,166,32,80,20,5,243,106,107,208,26,143,99,201,188,121,128,
  227,224,100,127,31,140,88,28,197,98,1,227,227,99,223,148,134,228,38,38,94,205,164,51,159,137,115,230,78,12,213,9,248,29,232,175,104,169,84,194,
  158,3,7,161,193,195,146,249,243,209,214,156,65,44,18,193,224,240,48,14,31,59,6,0,175,86,216,218,118,179,250,49,69,245,95,19,152,58,211,113,
  165,223,34,219,182,184,78,245,233,183,127,143,159,254,154,84,3,197,140,8,14,147,168,154,123,68,4,201,123,197,29,199,121,109,106,114,226,51,45,173,
  115,1,16,100,200,94,213,120,245,24,171,31,99,249,67,148,30,60,61,50,130,145,124,1,239,188,183,7,3,35,163,247,4,47,173,111,134,104,238,190,
  215,181,98,21,162,241,4,202,165,146,164,185,99,223,3,139,2,62,120,30,226,241,24,245,100,31,130,101,89,247,132,221,140,223,156,198,45,251,189,150,
  214,185,80,53,141,187,101,73,253,152,205,187,101,1,32,151,203,225,212,201,19,236,136,112,78,67,216,191,186,186,186,239,53,52,52,34,26,141,18,48,
  78,125,241,163,35,195,152,156,156,172,250,195,197,33,247,124,172,192,221,119,167,1,119,223,154,13,184,187,103,26,112,247,221,115,0,119,223,58,167,33,
  157,139,23,127,119,54,224,206,48,34,97,107,217,111,133,13,249,86,224,104,238,251,19,128,187,127,158,14,220,221,247,39,2,119,247,125,84,224,238,190,
  48,112,119,223,228,228,36,14,30,248,224,207,15,238,162,177,248,159,18,220,221,199,135,180,181,45,188,239,207,13,238,148,255,7,119,255,231,192,29,62,
  10,112,167,124,84,224,238,222,15,3,238,12,131,128,187,27,186,46,158,17,220,173,165,224,238,234,115,5,119,170,74,192,93,99,52,90,13,220,221,43,
  13,201,77,76,188,145,73,103,174,62,87,112,183,251,192,1,232,213,193,221,27,21,127,32,63,12,184,83,160,84,3,119,247,138,164,218,229,12,233,57,
  142,243,198,212,228,196,85,231,2,238,136,163,188,2,220,125,59,232,150,189,55,4,220,125,231,127,0,238,190,29,118,51,222,59,13,184,251,206,57,130,
  187,111,87,187,227,239,157,5,184,251,206,12,224,238,219,51,161,149,123,63,10,220,245,103,143,66,249,239,1,0,255,88,46,93,209,176,110,151,0,0,
  0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00129_png = (const char*) resource_TalComponent_bmp00129_png;
const int TalComponent::bmp00129_pngSize = 51250;

static const unsigned char resource_TalComponent_bmp00130_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,27,0,0,0,80,8,6,0,0,0,232,75,14,250,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,5,194,73,68,65,84,120,218,236,
  152,203,110,100,87,21,134,191,181,47,231,84,217,105,59,137,221,221,233,86,66,210,81,132,80,43,180,210,60,0,98,198,148,39,64,72,76,121,12,6,
  153,51,8,111,0,83,102,188,65,80,102,40,16,161,128,154,92,90,178,227,118,183,83,190,116,157,218,183,197,224,92,170,202,101,192,65,149,68,10,117,
  164,45,159,227,90,187,254,189,254,253,175,255,236,90,162,170,124,83,151,225,27,188,220,117,3,197,136,154,17,96,65,11,104,4,13,42,107,7,19,39,
  234,247,28,91,47,111,35,86,176,106,8,23,17,51,50,90,154,34,107,5,51,59,66,117,167,230,230,206,43,212,166,198,137,231,232,228,144,38,53,72,
  101,84,195,245,0,255,43,152,179,70,171,187,35,196,123,142,103,207,48,24,172,24,26,158,227,95,240,232,89,89,35,141,2,24,33,74,33,219,25,198,
  25,98,110,144,26,156,58,196,203,250,212,152,82,145,18,50,70,193,168,224,212,225,169,48,197,82,98,65,99,89,175,64,210,52,193,212,32,181,37,197,
  2,2,146,32,156,71,114,115,125,48,185,78,81,139,21,117,59,14,115,195,98,43,139,150,66,153,22,226,179,72,153,93,95,141,114,93,7,17,35,138,
  23,196,42,20,208,4,154,190,90,157,201,119,214,174,190,187,96,75,210,223,222,222,214,241,120,76,206,249,127,250,50,17,80,85,114,206,156,158,158,201,
  191,5,187,115,231,21,125,239,189,223,176,191,191,199,241,241,17,170,138,136,65,4,68,100,184,7,104,53,37,128,118,247,74,43,52,165,105,26,14,14,
  30,115,255,254,247,245,163,143,62,150,21,26,239,221,123,67,223,125,247,215,188,243,206,3,102,179,41,57,71,84,19,198,100,172,5,231,4,231,192,123,
  193,123,161,170,12,85,53,127,182,22,140,81,84,51,80,128,194,195,135,63,224,246,237,125,93,201,236,193,131,183,121,243,205,215,249,224,131,247,121,244,
  232,99,154,102,138,247,30,231,28,206,57,172,181,88,107,49,70,0,65,68,80,109,179,41,69,41,37,145,82,38,198,76,140,145,243,243,115,66,136,236,
  237,189,184,74,227,100,50,225,209,163,127,240,248,241,167,156,156,60,195,123,135,181,2,24,68,180,27,61,157,253,222,8,165,128,200,34,149,185,27,202,
  116,26,120,254,188,89,5,203,57,209,52,83,74,201,212,117,197,104,84,83,215,35,170,170,162,170,252,144,157,49,118,0,85,165,203,172,16,99,34,165,
  132,247,129,16,2,41,101,68,12,165,232,213,2,177,214,224,189,239,0,42,234,186,166,174,219,123,239,253,2,216,50,141,57,23,188,79,132,144,58,17,
  9,85,21,240,222,13,130,90,2,19,17,172,181,195,30,45,2,86,85,221,237,223,42,88,251,10,202,164,20,17,153,151,172,115,13,214,182,177,87,100,
  38,136,180,34,240,222,45,101,216,103,230,92,251,57,24,140,97,160,209,152,140,49,6,144,78,44,153,170,106,23,125,37,152,8,24,99,48,198,96,173,
  195,90,215,101,233,241,222,13,251,54,207,172,205,162,148,130,49,121,200,52,231,66,206,165,139,53,93,45,94,153,25,24,99,59,153,155,110,216,1,188,
  29,115,176,190,232,123,208,118,129,25,231,18,34,115,186,175,204,172,223,92,17,51,100,217,102,186,252,60,223,51,65,164,160,106,58,241,228,75,113,230,
  63,123,99,155,93,191,162,249,23,247,147,23,255,63,88,144,209,142,25,179,48,230,245,120,13,215,151,161,158,218,16,233,38,246,11,153,63,131,116,251,
  211,22,246,124,158,172,236,217,10,88,171,178,69,195,213,174,166,134,136,75,241,218,237,159,46,204,215,225,179,21,26,231,62,215,110,118,41,133,156,123,
  223,107,229,156,243,156,214,214,162,100,136,109,227,219,81,74,65,181,112,213,105,195,45,174,112,121,114,38,231,68,206,189,140,51,160,131,164,251,12,218,
  184,76,74,61,96,30,0,47,179,112,9,44,119,238,157,168,170,214,239,66,8,168,130,115,74,206,115,69,246,108,180,14,146,186,17,73,41,17,99,187,
  128,203,217,13,96,165,104,7,212,250,90,8,115,11,82,237,11,120,81,250,160,218,102,19,99,36,198,68,140,145,16,90,192,82,202,213,52,182,116,164,
  97,133,49,70,172,13,131,196,75,105,233,90,117,253,22,172,157,211,178,16,99,232,254,166,171,105,12,161,225,248,248,11,78,79,39,196,24,134,186,234,
  45,40,165,116,201,174,22,93,191,167,49,19,194,140,166,105,56,61,61,99,50,57,35,132,184,10,118,120,120,196,135,31,254,5,16,82,202,157,25,87,
  56,231,22,94,47,102,16,71,155,177,46,137,164,103,101,58,157,114,120,248,132,131,131,67,206,207,47,174,62,17,223,189,123,83,223,122,235,30,33,180,
  52,130,12,86,213,91,216,220,105,100,169,142,90,245,181,106,14,33,112,113,241,156,103,207,190,228,228,100,66,41,243,223,2,75,199,239,221,221,27,106,
  140,233,248,158,91,216,101,39,88,62,101,45,95,189,48,166,211,25,165,100,217,156,245,191,197,62,136,21,53,175,9,229,150,194,177,32,7,74,153,126,
  13,125,16,51,50,186,251,179,23,121,251,167,247,169,198,53,163,56,226,201,95,143,168,246,157,134,227,180,222,62,136,255,137,227,213,95,190,206,207,127,
  248,11,222,224,53,246,252,14,191,189,243,59,62,57,249,20,51,54,90,166,107,234,131,140,171,74,111,255,234,46,102,239,6,191,255,226,143,212,50,102,
  123,92,243,73,250,27,251,63,218,231,226,253,243,245,209,232,42,48,163,17,23,38,49,27,127,78,253,178,231,203,139,3,242,171,153,221,180,197,248,123,
  91,235,83,227,108,154,56,59,158,80,66,193,73,197,13,251,18,187,213,45,198,241,5,102,79,19,23,255,188,88,95,102,33,23,241,63,222,214,209,103,
  167,108,165,29,206,78,158,146,205,12,123,42,28,253,249,41,179,207,102,107,110,186,252,105,202,196,127,14,15,95,162,186,229,40,177,208,252,125,202,217,
  31,38,104,163,95,79,31,68,118,13,122,179,192,185,192,19,69,227,166,15,242,127,219,7,153,31,92,203,112,223,191,165,23,199,210,202,141,12,199,132,
  179,179,243,175,222,7,89,60,126,207,127,100,204,207,251,107,237,131,56,39,212,181,99,60,246,108,109,121,70,35,75,93,91,234,90,54,125,144,77,31,
  100,211,7,217,244,65,54,125,144,77,31,100,211,7,217,244,65,54,125,144,111,181,15,242,175,1,0,14,236,9,156,226,88,36,247,0,0,0,0,73,
  69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00130_png = (const char*) resource_TalComponent_bmp00130_png;
const int TalComponent::bmp00130_pngSize = 4247;

static const unsigned char resource_TalComponent_bmp00131_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,9,0,0,0,16,8,2,0,0,0,75,42,194,20,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,0,232,73,68,65,84,120,218,124,
  144,205,74,195,80,20,132,103,238,61,151,91,107,172,89,8,74,2,46,186,113,231,43,248,52,62,133,155,250,10,190,71,222,194,174,132,64,151,34,114,
  81,72,65,116,33,42,77,211,252,28,55,118,211,196,206,114,62,56,103,102,56,155,221,174,202,18,61,101,89,38,247,243,121,158,231,125,86,85,149,193,
  255,18,16,73,154,40,20,224,214,84,130,33,4,233,92,87,252,20,116,128,209,63,212,81,107,192,67,154,163,186,155,182,140,21,78,65,64,129,154,250,
  73,188,42,227,171,168,185,108,153,116,24,147,6,22,134,165,177,111,226,31,71,226,38,110,125,183,234,7,57,187,62,21,175,222,223,156,216,115,177,17,
  173,19,49,214,110,196,127,140,198,7,135,50,141,46,22,15,11,62,85,59,255,154,151,70,228,219,153,103,59,144,243,171,21,83,155,52,74,119,251,121,
  134,247,32,80,44,139,229,192,42,138,125,155,237,99,156,28,199,155,106,61,116,18,191,3,0,61,200,90,141,18,252,223,119,0,0,0,0,73,69,78,
  68,174,66,96,130,0,0};
const char* TalComponent::bmp00131_png = (const char*) resource_TalComponent_bmp00131_png;
const int TalComponent::bmp00131_pngSize = 3005;

static const unsigned char resource_TalComponent_bmp00132_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,11,0,0,0,22,8,6,0,0,0,22,228,102,99,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,1,219,73,68,65,84,120,218,156,
  145,187,110,19,81,20,69,215,185,247,206,220,113,28,199,142,99,66,4,17,18,164,131,6,144,248,134,20,136,154,15,224,203,40,233,160,165,227,19,16,
  66,66,162,8,73,147,200,138,227,36,126,140,51,247,113,40,64,56,15,10,194,46,143,182,182,246,217,75,84,149,127,149,187,126,104,139,85,11,100,148,
  6,104,52,203,13,179,19,209,190,43,217,238,246,176,34,168,49,204,66,67,101,172,46,114,146,43,230,174,88,182,124,139,181,59,125,76,229,145,194,49,
  62,26,18,23,11,74,17,109,84,197,1,88,227,244,158,47,41,36,113,49,58,6,99,16,99,97,209,176,90,148,76,115,90,214,16,192,160,72,10,216,
  148,112,198,144,130,226,69,80,103,41,196,0,96,0,98,142,18,82,66,49,168,177,168,115,80,88,178,17,66,78,132,156,175,62,88,199,72,45,6,47,
  150,16,51,2,68,17,166,77,67,125,185,6,192,185,42,146,18,171,8,165,181,100,133,58,39,78,82,36,252,158,79,174,67,241,34,234,16,50,16,80,
  162,234,159,157,229,54,4,13,183,208,21,220,165,136,238,136,103,77,149,25,153,67,129,81,14,55,113,119,68,244,85,119,147,221,39,79,105,249,146,80,
  121,190,14,143,120,80,182,245,160,153,45,113,139,181,186,91,116,120,179,253,144,23,175,95,226,30,221,199,247,122,124,122,251,142,241,254,1,27,198,234,
  40,167,95,184,125,81,241,120,115,139,13,115,193,254,135,247,200,138,167,229,219,196,31,135,60,31,220,229,203,124,178,172,97,179,161,50,17,191,24,177,
  178,119,198,122,213,97,62,174,217,54,37,113,173,195,160,106,45,215,88,196,134,227,201,148,121,54,196,86,27,93,239,34,131,30,161,229,24,53,115,142,
  234,233,50,57,107,228,219,249,41,251,149,39,120,203,120,50,199,36,225,220,90,62,15,15,217,187,108,86,77,82,137,104,113,54,227,25,37,155,229,10,
  33,103,190,215,51,62,78,78,56,253,27,110,39,162,125,96,29,71,131,50,36,49,251,95,220,63,7,0,237,202,224,103,251,133,99,87,0,0,0,0,
  73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00132_png = (const char*) resource_TalComponent_bmp00132_png;
const int TalComponent::bmp00132_pngSize = 3248;

static const unsigned char resource_TalComponent_bmp00133_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,25,0,0,0,100,8,6,0,0,0,115,85,155,124,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,14,190,73,68,65,84,120,218,228,
  154,217,111,221,199,117,199,191,179,253,238,202,85,92,68,33,218,21,145,178,84,217,138,104,185,42,144,165,176,30,12,39,65,27,5,129,152,20,105,244,
  208,60,196,125,111,255,132,6,125,106,95,139,54,47,129,3,165,81,27,32,6,4,187,240,38,201,142,21,237,178,108,107,161,72,201,162,184,92,110,247,
  94,46,119,249,253,102,233,195,204,111,126,191,75,82,150,101,183,69,129,92,96,192,5,196,156,153,115,206,204,249,204,247,144,24,99,240,191,253,225,79,
  252,3,206,205,208,208,16,118,237,222,141,174,174,46,16,66,80,175,215,49,63,55,143,241,251,227,184,63,62,78,158,52,7,121,220,78,132,16,230,229,
  151,95,198,209,163,127,134,40,138,48,93,42,161,82,169,32,138,36,184,16,40,228,243,200,229,115,88,174,46,225,198,141,235,184,118,245,10,121,42,35,
  123,247,238,53,127,243,211,159,162,90,173,226,221,179,231,48,61,53,5,38,4,2,17,128,115,1,206,57,40,165,16,66,160,171,171,11,189,125,189,152,
  155,157,197,59,111,189,137,90,109,149,60,209,200,225,195,135,205,201,147,39,113,230,245,55,240,209,205,143,144,205,231,144,9,50,224,66,128,51,6,198,
  24,56,23,32,148,130,82,10,66,0,128,160,163,163,29,50,146,120,255,189,179,168,86,42,228,177,70,246,236,217,99,126,246,202,43,248,205,111,78,99,
  114,114,10,197,182,162,155,156,131,113,110,13,48,187,11,202,24,8,33,160,148,128,16,2,165,52,50,65,128,48,12,241,254,123,103,17,54,155,222,16,
  77,7,120,228,196,8,94,127,253,13,103,160,205,186,134,217,201,227,97,13,80,107,128,184,121,12,64,8,65,163,217,4,101,20,207,62,247,181,22,239,
  120,35,95,255,198,55,48,191,88,198,157,187,119,81,44,22,253,202,233,26,3,36,30,132,0,206,85,218,24,24,163,97,140,70,179,217,68,161,80,196,
  182,237,59,76,139,17,206,185,121,118,255,1,252,225,226,69,80,46,96,40,117,190,231,118,8,225,12,57,87,17,2,66,0,99,0,99,140,29,218,64,
  107,13,99,12,194,40,68,95,127,127,235,57,25,249,193,15,240,226,129,103,64,230,102,177,88,175,99,97,101,21,203,205,16,13,165,18,23,81,27,131,
  120,216,88,90,3,241,228,70,27,40,173,160,149,66,32,2,228,243,5,83,171,173,18,14,0,251,7,7,145,35,4,71,247,63,3,202,24,148,214,184,
  62,122,15,111,126,124,11,65,38,235,131,75,40,1,65,188,122,56,3,206,144,82,208,90,67,43,5,165,20,0,160,175,175,47,217,201,181,107,215,48,
  118,235,22,2,99,48,208,221,141,45,155,186,81,89,89,1,113,49,33,176,129,141,211,165,213,64,50,177,82,26,90,75,72,41,97,140,70,144,201,36,
  70,238,207,148,144,205,23,80,173,84,145,203,78,162,61,151,131,140,66,100,114,57,239,255,56,139,12,96,131,28,199,64,107,40,173,237,46,180,130,148,
  118,24,109,64,41,73,2,31,54,155,16,140,35,147,9,64,25,71,77,42,72,30,128,49,142,181,199,55,118,149,54,206,128,82,80,82,66,41,5,169,
  164,115,155,114,177,209,137,145,74,165,2,16,184,108,114,41,75,221,14,72,226,34,109,146,21,107,229,38,210,214,85,82,74,40,233,190,143,36,0,32,
  138,162,196,200,108,169,4,165,148,191,147,24,99,32,132,166,86,143,150,52,85,62,6,118,216,152,216,88,72,25,65,202,8,140,82,76,77,77,38,70,
  234,245,26,169,215,106,222,8,33,240,25,149,62,11,241,202,181,139,193,90,99,74,73,72,151,89,238,96,146,150,19,63,63,55,11,206,197,58,223,195,
  196,171,151,126,114,237,221,227,86,159,250,57,10,67,100,179,89,44,46,44,172,191,86,198,199,238,145,122,109,21,66,8,231,127,59,185,118,215,69,203,
  234,101,250,76,72,40,25,121,195,214,221,20,147,147,147,235,47,72,0,184,118,229,18,96,12,40,165,48,218,94,61,241,196,254,171,11,184,221,65,178,
  155,48,138,96,140,65,71,71,59,110,223,186,181,241,5,9,0,171,171,171,228,198,245,171,160,148,128,49,106,87,25,79,44,83,126,247,174,82,222,0,
  37,4,221,93,93,24,189,123,23,82,74,242,196,202,152,207,231,205,240,145,23,32,130,12,234,181,26,194,40,130,86,210,158,104,165,16,201,8,50,178,
  198,0,131,108,54,139,64,112,124,242,201,39,235,12,124,102,141,7,128,189,131,131,166,183,175,31,148,50,68,81,132,48,12,33,163,8,90,43,192,215,
  19,131,74,185,140,135,15,31,146,167,6,137,244,167,173,173,205,12,108,217,2,33,2,27,39,165,208,104,54,48,61,53,133,70,163,241,197,105,229,127,
  242,67,241,127,240,249,124,112,183,111,31,246,236,222,141,206,206,78,16,66,81,171,213,48,55,63,135,251,227,247,113,255,254,151,128,187,66,161,96,254,
  241,239,255,14,147,179,115,160,221,155,80,94,44,163,92,41,91,184,227,28,197,98,17,185,92,14,75,213,42,174,95,187,138,235,215,175,61,37,220,13,
  14,154,127,249,231,127,66,119,179,137,83,191,253,45,126,125,254,125,80,17,32,19,4,96,156,91,44,98,20,65,16,160,171,179,11,61,61,61,152,157,
  157,193,187,111,191,133,90,173,246,228,20,62,114,228,136,249,225,143,254,10,23,47,92,192,183,143,12,67,87,42,248,213,219,239,96,114,165,134,76,38,
  227,46,81,6,198,44,220,193,85,205,142,142,14,132,205,16,191,127,239,28,170,213,86,184,107,9,252,224,224,160,57,49,242,67,156,58,245,107,188,249,
  206,187,184,62,118,31,253,3,3,248,211,193,65,4,148,56,90,73,12,80,202,220,160,88,170,46,65,27,131,67,195,207,35,8,2,179,161,17,74,169,
  57,113,98,4,103,206,156,193,204,204,12,186,186,186,112,225,195,155,104,112,142,237,91,6,208,25,4,0,140,55,64,92,77,142,75,44,8,208,104,52,
  192,24,195,193,231,14,109,156,93,199,142,29,67,169,52,139,209,209,123,22,238,4,71,121,105,9,239,92,186,140,210,228,20,86,140,129,16,194,175,220,
  186,9,208,238,34,141,235,78,179,217,68,177,216,134,175,108,221,102,30,77,60,76,234,9,165,212,12,14,13,225,242,149,43,200,229,243,96,194,6,55,
  151,201,226,253,27,55,49,94,169,34,200,23,108,60,28,88,248,29,192,180,148,2,173,20,154,97,136,254,205,155,91,221,181,103,207,30,172,174,214,177,
  180,188,140,32,19,64,112,71,140,156,65,4,1,2,33,64,9,181,62,33,233,138,153,148,101,163,53,140,171,154,50,138,16,4,25,20,138,69,227,221,
  213,211,211,139,249,249,121,159,158,49,3,83,74,193,124,112,45,85,88,60,53,206,77,169,186,31,195,133,27,148,16,244,246,244,36,49,17,129,192,106,
  173,134,64,4,118,242,22,200,102,190,222,195,229,76,108,64,107,227,200,69,123,106,137,139,27,140,129,8,130,196,136,148,10,32,18,92,88,192,142,223,
  33,132,18,15,22,132,16,7,118,9,3,155,24,143,82,165,217,2,94,4,147,74,16,10,0,181,90,13,52,69,241,140,113,231,251,196,64,204,188,158,
  30,85,92,142,181,7,59,207,95,49,147,57,114,113,112,87,5,33,196,250,159,80,80,26,191,160,224,235,60,214,224,105,188,106,143,67,50,41,205,81,
  20,1,198,32,12,195,196,200,204,244,36,180,210,224,130,187,87,84,235,129,3,136,75,211,20,123,153,245,220,37,165,51,38,21,24,165,152,157,157,77,
  195,93,157,212,235,117,112,46,90,39,55,173,15,29,173,85,11,255,234,52,11,59,122,84,74,250,12,172,215,235,173,112,183,48,63,11,33,132,75,211,
  228,170,136,15,90,146,57,150,226,19,255,91,52,146,74,194,104,141,40,138,144,205,102,45,95,175,189,187,238,141,222,117,112,199,93,246,160,37,216,246,
  52,187,84,117,43,143,177,200,179,87,24,186,180,39,152,152,120,184,49,220,93,189,124,17,48,6,4,196,187,70,59,12,74,67,117,156,61,54,163,92,
  86,57,130,111,111,107,195,189,209,209,207,134,187,143,110,126,8,230,158,15,201,100,41,122,143,249,87,38,89,21,69,17,40,165,232,238,234,194,248,248,
  152,7,237,207,172,140,197,98,155,57,60,60,12,38,2,212,235,117,68,97,216,146,178,210,27,146,128,49,200,229,114,224,156,227,206,237,91,8,195,240,
  233,224,110,112,104,200,244,244,246,129,82,234,225,46,138,36,180,86,32,0,24,101,48,208,88,170,86,241,224,193,131,47,7,119,29,29,29,166,191,191,
  31,92,8,16,16,40,173,16,54,155,152,153,153,217,176,166,255,17,195,93,46,151,51,195,195,195,216,177,99,39,218,218,219,96,12,80,175,215,80,42,
  149,176,178,184,136,15,46,93,130,84,138,124,97,184,27,25,25,193,51,251,247,99,113,177,140,71,147,147,168,86,151,160,148,130,16,2,197,246,118,252,
  228,248,95,98,236,15,23,240,139,223,189,134,11,55,110,144,167,218,201,209,163,71,205,63,252,252,231,184,123,119,20,191,252,229,171,88,90,94,134,16,
  2,65,16,128,115,14,165,13,190,186,247,171,32,141,58,184,49,248,202,214,109,248,206,95,124,207,156,125,251,45,44,47,47,145,39,26,121,233,165,151,
  204,137,19,35,56,117,234,223,49,49,49,129,124,33,143,98,177,232,139,25,227,28,205,102,136,131,251,134,64,150,151,241,225,216,24,30,45,150,209,183,
  121,51,190,254,173,63,71,103,87,183,169,148,23,31,15,119,71,142,28,49,47,30,59,134,127,253,183,95,96,114,202,10,107,129,163,70,230,106,63,0,
  12,108,25,192,243,207,236,67,105,114,18,247,102,102,145,201,229,176,188,180,12,37,53,190,118,120,24,65,38,179,49,220,101,179,89,243,157,239,126,23,
  167,79,255,39,86,86,86,80,112,171,79,192,130,130,50,235,170,3,123,247,162,77,43,140,62,124,136,69,7,116,0,208,108,54,0,66,113,240,217,67,
  27,7,254,248,241,227,166,80,108,195,217,179,231,172,246,232,176,136,115,238,193,130,50,91,57,219,242,57,12,180,21,81,169,86,81,170,55,192,69,224,
  234,189,189,165,57,227,24,27,189,227,111,98,175,220,109,221,182,13,215,111,220,64,190,80,0,227,2,140,185,122,159,170,251,140,50,112,198,176,218,108,
  226,238,220,60,230,67,9,206,133,19,213,180,55,20,70,27,192,221,190,125,251,176,184,88,65,163,209,132,8,2,8,198,193,120,130,70,22,178,29,125,
  16,2,70,41,184,147,9,227,242,105,107,141,29,145,131,187,92,46,151,192,93,119,247,38,204,47,44,64,8,1,145,202,162,52,181,199,178,160,45,104,
  100,141,44,98,111,103,99,180,7,10,10,160,183,183,55,73,97,46,56,106,245,26,132,176,26,23,227,204,146,11,163,137,52,104,9,213,215,124,187,129,
  84,105,54,137,114,167,148,189,165,69,144,82,238,162,72,2,132,186,24,164,93,100,201,5,49,220,121,176,208,78,181,51,235,240,52,214,88,0,120,221,
  152,2,192,202,202,170,229,94,47,168,197,46,114,112,231,201,49,165,61,166,16,53,46,104,90,107,95,146,141,195,166,4,238,170,21,16,74,19,137,156,
  210,4,238,28,3,91,176,176,25,100,140,129,242,238,73,248,55,174,255,86,232,180,111,21,111,100,122,106,18,198,24,31,108,226,119,225,32,155,192,83,
  139,210,26,90,73,104,169,91,193,66,198,178,148,229,0,74,41,102,75,165,20,220,213,106,164,81,111,128,115,230,39,247,124,231,221,147,28,54,191,131,
  180,44,229,37,169,208,54,8,44,34,173,85,238,74,16,34,240,110,106,21,152,117,74,228,76,92,35,125,160,163,22,237,43,151,203,97,110,110,110,99,
  184,171,213,86,193,57,75,100,89,231,239,152,191,18,60,149,45,252,27,27,14,163,8,153,32,0,140,65,169,52,243,24,229,238,242,69,247,224,164,174,
  139,96,82,103,65,166,248,55,33,73,171,158,74,132,205,38,24,163,40,22,10,184,125,251,73,202,221,181,43,96,148,56,184,75,208,52,253,76,176,207,
  3,199,192,14,149,50,153,12,58,219,59,112,251,246,45,168,53,229,248,177,202,221,225,231,143,32,147,201,161,86,175,121,184,83,82,66,105,237,228,193,
  8,145,148,160,132,32,159,203,193,24,179,161,129,39,34,209,238,221,123,76,255,230,1,112,206,17,201,200,174,94,70,246,81,228,4,81,37,35,204,205,
  205,99,102,102,250,203,193,93,161,80,48,125,253,253,8,68,0,16,64,73,133,48,12,81,42,205,172,227,222,63,114,184,203,100,50,230,133,23,94,192,
  174,93,187,208,214,222,14,2,130,122,163,129,133,249,121,140,143,143,227,230,205,15,55,12,246,231,114,87,54,155,53,35,35,35,56,120,240,89,204,47,
  44,96,226,209,35,148,203,21,104,173,33,132,64,161,88,68,103,71,7,180,82,184,245,201,199,56,127,254,28,180,214,228,115,27,25,30,30,54,63,254,
  241,95,99,108,124,28,231,206,159,199,82,117,201,150,229,32,176,149,211,9,57,140,11,20,10,5,244,247,247,163,81,175,225,205,255,122,99,157,160,182,
  161,145,111,126,243,91,230,248,247,191,143,211,167,255,3,247,31,60,64,161,80,64,144,9,28,26,113,112,97,129,194,170,21,204,151,228,98,177,8,74,
  25,46,252,254,61,204,205,150,30,127,24,15,28,248,19,243,147,147,39,241,234,171,191,66,185,92,70,190,80,128,72,245,24,109,151,136,251,174,105,210,
  22,132,211,1,4,140,209,120,239,220,187,104,184,231,245,186,182,236,241,227,199,241,187,215,94,67,185,92,70,161,88,132,16,162,5,139,226,90,147,238,
  20,17,167,179,128,16,132,97,8,173,53,14,29,26,222,56,133,95,124,241,24,38,167,166,240,233,167,159,90,246,90,139,67,140,186,223,165,251,190,38,
  41,9,174,244,46,175,172,130,114,142,125,67,251,76,75,10,83,74,205,207,94,249,91,124,240,193,5,251,200,20,28,130,11,43,234,56,60,34,36,237,
  34,128,128,248,150,96,24,134,48,198,160,171,173,136,173,59,122,177,127,199,118,76,79,62,106,61,39,59,119,238,68,173,86,199,202,234,42,50,153,172,
  239,187,83,23,11,70,153,87,141,90,218,81,0,194,40,194,129,161,65,60,183,125,43,122,242,57,20,69,0,40,137,70,121,1,109,133,130,89,94,117,
  109,217,77,155,122,176,176,184,224,154,201,220,175,222,183,197,93,38,197,226,166,147,93,96,140,165,197,142,246,118,28,216,181,19,99,31,127,132,59,243,
  11,24,159,158,193,100,185,140,254,190,254,180,114,23,96,181,86,179,153,68,173,129,4,238,104,139,238,101,180,129,142,209,84,27,112,70,113,235,206,29,
  100,26,117,156,191,124,9,149,122,19,13,41,237,131,41,155,77,195,93,132,140,54,190,153,153,78,209,4,81,93,128,145,116,236,180,214,32,0,230,230,
  23,112,102,102,26,82,105,219,102,167,20,70,43,191,107,175,220,17,66,172,139,92,70,173,167,150,180,36,213,42,174,41,173,160,29,94,106,167,28,121,
  45,50,54,82,173,86,97,128,228,223,20,54,128,108,223,18,143,137,197,11,156,110,200,164,153,41,157,114,23,165,149,187,217,210,12,100,36,253,233,38,
  41,200,246,244,14,147,244,125,83,61,70,41,109,157,151,190,209,28,249,126,227,244,244,76,98,164,217,108,58,28,74,68,181,132,224,211,218,163,91,189,
  218,160,255,235,212,36,41,149,247,111,163,177,70,185,155,153,158,242,130,152,23,238,210,202,93,204,188,90,121,141,43,129,139,84,155,54,138,144,207,231,
  81,169,148,215,95,43,143,38,30,146,165,106,25,92,8,31,104,165,116,75,28,210,255,158,16,55,54,61,104,71,18,97,24,121,61,127,98,98,98,99,
  184,187,113,237,42,162,176,9,198,24,140,177,68,226,251,188,90,251,230,190,215,190,164,67,212,72,34,148,17,40,37,104,43,22,63,27,238,26,141,6,
  185,126,229,10,148,180,125,171,181,61,223,52,7,71,81,228,4,78,219,65,13,132,64,103,103,7,238,221,187,247,249,218,178,153,76,198,28,58,60,140,
  66,177,13,141,122,3,205,176,9,41,101,42,163,108,154,42,109,133,255,92,46,7,2,131,219,183,111,63,125,91,118,219,246,237,102,243,192,22,100,130,
  140,3,234,208,26,115,112,71,8,129,86,10,139,139,11,45,173,241,47,196,93,185,92,206,244,245,245,65,4,1,24,101,80,74,162,209,108,98,110,118,
  246,255,15,220,253,247,0,62,59,165,241,170,144,131,246,0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00133_png = (const char*) resource_TalComponent_bmp00133_png;
const int TalComponent::bmp00133_pngSize = 6547;

static const unsigned char resource_TalComponent_bmp00134_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,27,0,0,0,52,8,6,0,0,0,123,46,67,182,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,6,33,73,68,65,84,120,218,228,
  88,219,138,21,71,20,93,187,170,186,143,102,20,5,29,38,38,138,81,36,132,16,68,127,37,151,255,72,94,242,5,9,228,61,4,63,41,36,144,39,
  131,15,34,74,96,136,19,199,56,158,51,103,166,207,169,170,93,59,15,117,233,234,203,40,248,224,75,26,138,190,85,245,174,189,246,174,181,87,53,137,
  8,222,215,97,234,155,157,157,29,57,127,254,60,152,249,157,62,70,4,136,8,152,25,171,213,49,157,105,236,218,181,15,229,193,131,159,113,245,234,21,
  188,124,249,2,34,2,34,5,34,128,136,202,53,0,68,48,8,128,164,107,65,68,72,176,217,108,240,252,249,62,62,255,252,83,121,244,232,241,192,160,
  2,128,91,183,62,145,159,126,250,1,247,238,221,197,118,219,129,217,65,196,67,41,134,214,128,49,4,99,128,166,33,52,13,161,109,21,218,182,191,215,
  26,80,74,32,194,0,2,128,128,251,247,63,195,222,222,85,153,120,118,247,238,23,184,125,251,38,126,255,253,87,60,125,250,24,155,77,135,166,105,96,
  140,129,49,6,90,107,104,173,161,20,1,32,16,17,68,162,55,33,8,66,240,240,158,225,28,195,57,135,245,122,13,107,29,174,92,185,60,133,113,185,
  92,226,233,211,39,216,223,255,11,71,71,175,208,52,6,90,19,0,5,34,73,45,195,153,99,67,8,1,32,170,161,228,212,4,93,103,113,122,186,153,
  26,99,246,216,108,58,132,192,88,44,90,156,59,183,192,98,113,14,109,219,162,109,155,226,157,82,186,24,21,65,242,44,192,57,15,239,61,154,198,194,
  90,11,239,25,68,10,33,200,124,130,104,173,208,52,77,50,208,98,177,88,96,177,136,215,77,211,84,198,134,48,50,7,52,141,135,181,62,37,17,161,
  109,45,154,198,148,132,26,24,35,34,104,173,75,140,106,131,109,187,72,241,155,26,3,0,239,25,222,59,16,169,254,163,102,3,173,99,223,25,207,8,
  68,49,9,154,198,12,60,204,158,25,19,223,3,10,74,161,192,168,20,67,41,5,128,82,178,48,218,54,78,122,214,24,17,160,148,130,82,10,90,27,
  104,109,146,151,13,154,198,148,184,245,158,69,47,66,8,80,138,139,167,204,1,204,33,245,85,105,45,206,122,6,40,165,83,154,171,212,116,49,30,91,
  111,44,47,250,108,52,78,144,97,140,7,81,15,247,172,103,57,184,68,170,120,25,61,29,222,247,49,35,16,5,136,168,148,60,60,234,167,222,204,141,
  209,187,60,163,254,195,121,112,253,188,80,144,146,132,140,170,90,191,30,39,116,53,67,169,101,61,197,46,148,6,230,137,244,247,0,165,248,196,133,221,
  143,163,73,204,38,198,98,150,213,132,43,105,77,149,30,163,254,146,226,39,213,120,41,239,38,48,246,60,23,131,29,66,0,115,230,189,152,206,204,61,
  172,145,162,168,244,141,253,99,11,33,64,36,96,174,76,154,122,134,195,193,12,102,15,230,156,198,12,64,74,74,103,15,98,63,134,247,217,32,23,131,
  99,20,70,198,56,177,183,71,219,70,190,179,214,66,4,48,70,192,220,103,100,70,35,50,136,79,205,193,123,15,231,226,4,198,222,21,99,33,72,50,
  20,121,205,218,158,130,68,242,2,174,83,31,16,137,222,56,231,224,156,135,115,14,214,70,131,33,132,121,24,35,28,190,204,208,57,7,173,109,73,241,
  16,34,92,83,214,143,198,226,152,136,130,115,54,157,253,60,140,214,110,240,242,229,63,88,173,150,112,206,150,117,149,41,200,123,63,162,171,154,245,51,
  140,12,107,183,216,108,54,88,173,142,177,92,30,195,90,55,53,118,112,240,2,15,31,254,9,128,224,61,39,50,110,97,140,169,202,139,42,201,17,61,
  150,65,146,100,84,186,174,195,193,193,33,158,63,63,192,122,125,50,92,189,121,45,124,244,209,174,220,185,115,11,214,70,24,1,42,84,149,41,172,103,
  26,26,172,163,152,125,49,155,173,181,56,57,57,197,171,87,175,113,116,180,68,8,129,38,198,0,224,210,165,139,162,148,74,120,247,20,54,102,130,161,
  202,26,30,57,49,186,110,139,16,152,102,61,123,31,135,194,123,60,222,171,34,46,48,126,119,249,146,124,243,205,87,184,112,225,2,214,235,21,68,250,
  68,152,43,132,83,69,28,207,206,57,44,151,71,248,237,183,63,240,227,223,7,83,69,252,253,238,85,249,250,235,47,113,227,198,13,120,111,17,2,3,
  96,16,5,40,133,210,180,166,170,245,247,241,189,36,53,28,207,55,111,94,195,183,151,46,78,21,241,245,235,31,99,119,247,10,158,61,123,130,195,195,
  127,224,156,173,84,240,176,74,103,79,115,217,137,231,158,245,153,25,219,237,6,222,51,118,118,62,152,38,72,215,117,56,60,124,129,163,163,127,113,114,
  114,130,16,184,176,118,93,113,243,245,84,29,163,120,4,196,242,98,173,159,103,144,168,106,45,66,8,104,26,157,164,91,148,111,89,41,245,210,96,92,
  104,115,153,9,133,95,153,195,64,91,78,178,81,41,149,180,97,47,229,162,230,55,48,70,39,22,25,110,44,128,88,92,181,214,73,114,83,18,169,89,
  137,225,44,221,72,69,202,101,143,178,118,204,59,152,154,27,107,170,98,230,145,8,242,101,98,103,232,198,94,186,213,82,60,43,225,33,17,247,48,102,
  200,178,34,22,9,48,70,157,237,89,244,78,37,217,166,83,83,149,104,213,105,112,142,89,46,170,1,68,10,204,125,129,141,176,170,74,97,205,120,150,
  13,70,184,168,192,22,159,77,75,76,76,121,13,128,33,162,160,84,140,93,36,98,53,43,229,42,69,60,22,167,217,83,26,220,15,181,99,40,134,136,
  66,165,190,212,12,227,140,136,56,38,10,6,31,173,13,230,217,214,19,235,51,180,150,122,120,23,69,140,17,28,99,69,60,44,166,195,123,122,123,137,
  153,42,226,177,178,149,153,254,189,34,174,251,207,74,185,90,17,103,174,139,105,140,193,95,1,165,66,18,168,97,32,189,251,191,6,161,122,246,150,122,
  22,101,179,84,234,152,17,130,154,196,43,43,226,200,32,99,185,30,202,132,223,168,136,107,246,214,58,164,5,235,33,2,104,13,132,64,147,61,117,212,
  247,92,90,30,63,39,55,6,198,152,37,233,123,5,230,225,6,60,198,133,38,207,178,128,141,114,142,147,214,231,179,21,113,15,7,151,154,228,189,31,
  124,84,107,41,105,157,51,173,143,85,222,88,248,34,88,153,207,128,209,123,135,245,122,133,174,235,192,236,211,186,82,131,248,69,178,237,119,149,245,79,
  151,220,188,119,112,206,161,235,54,232,186,88,64,103,126,39,29,99,127,127,31,0,165,120,13,57,177,222,80,140,137,56,163,146,99,103,173,195,106,117,
  140,215,175,151,216,110,183,103,9,158,139,178,183,183,91,112,143,219,215,121,250,25,139,212,122,9,120,239,177,221,70,85,252,203,105,247,63,81,196,255,
  13,0,25,133,238,183,168,224,187,173,0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00134_png = (const char*) resource_TalComponent_bmp00134_png;
const int TalComponent::bmp00134_pngSize = 4342;

static const unsigned char resource_TalComponent_bmp00135_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,208,0,0,0,240,8,6,0,0,0,41,100,83,131,0,0,0,9,112,72,89,
  115,0,0,11,19,0,0,11,19,1,0,154,156,24,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,2,3,4,73,68,65,84,120,218,236,
  253,233,119,29,215,121,38,142,62,53,156,9,192,1,72,2,224,4,146,34,193,153,34,9,74,34,41,201,34,61,72,178,157,72,138,231,142,146,184,59,
  221,107,181,59,253,251,154,181,250,15,184,255,64,127,234,79,247,254,250,118,210,234,36,182,220,113,91,241,32,79,146,60,104,36,41,138,164,196,81,20,
  1,78,0,7,204,211,25,234,212,112,63,212,217,85,111,237,122,247,174,58,144,156,88,249,93,120,105,201,2,206,169,218,181,107,15,207,251,236,231,125,
  94,227,191,252,245,95,7,104,255,120,158,7,203,178,196,127,194,247,125,24,134,17,254,255,32,128,101,154,160,63,126,16,192,0,16,4,225,37,130,32,
  128,101,89,225,127,27,6,16,4,48,12,3,1,249,55,200,231,13,195,136,174,239,249,62,12,0,166,124,15,223,7,152,223,139,107,186,174,11,219,182,
  225,251,62,76,211,140,254,77,127,60,207,3,218,247,50,196,247,195,139,192,48,12,248,190,159,120,110,207,247,19,207,74,219,43,183,75,126,14,249,135,
  246,41,125,126,195,52,129,32,72,92,67,119,47,209,127,166,105,70,215,164,207,76,255,78,191,39,247,5,237,59,213,15,247,189,0,136,250,206,247,253,
  176,63,165,118,202,223,229,158,69,252,184,174,11,195,48,96,154,38,2,0,38,249,76,32,141,155,188,63,159,214,118,115,239,135,142,139,128,25,39,170,
  57,75,159,197,247,253,104,156,209,241,37,174,199,141,141,68,27,218,223,49,20,127,87,206,7,49,182,13,3,65,70,159,139,223,201,235,132,24,203,157,
  180,87,60,55,215,46,250,110,117,239,70,247,238,229,241,196,61,139,88,39,233,125,68,187,84,223,133,52,62,163,53,37,71,187,60,207,3,128,212,24,
  200,154,23,226,187,134,97,192,104,183,215,148,250,77,244,191,106,236,113,215,165,107,145,238,157,137,123,139,61,67,181,214,117,50,247,233,30,164,123,191,
  137,241,33,61,183,235,121,225,62,212,190,134,239,121,236,92,215,93,155,238,45,186,231,226,230,110,158,49,154,90,175,0,4,190,207,246,165,106,143,146,
  223,157,47,222,151,98,78,139,107,136,189,50,26,171,210,123,226,198,56,237,23,67,51,135,84,123,167,235,186,176,108,59,194,26,129,180,158,201,239,93,
  92,159,27,159,97,147,227,125,95,238,231,32,8,162,247,167,27,71,170,31,215,117,201,26,36,158,54,190,185,161,193,7,185,48,151,31,68,207,176,82,
  204,37,122,127,37,152,235,19,123,142,127,37,216,209,150,39,8,253,161,23,227,22,15,67,234,88,209,129,220,3,115,192,128,222,79,116,38,183,168,112,
  160,72,252,216,182,157,248,12,183,232,70,19,190,61,57,162,251,146,54,208,73,105,100,45,144,18,0,54,20,27,143,0,90,137,201,217,254,189,209,190,
  191,248,157,88,152,84,11,167,216,156,117,27,7,55,216,185,197,120,37,224,206,72,119,70,180,217,171,22,77,221,143,120,111,1,0,35,227,179,31,231,
  231,211,218,110,85,123,184,247,169,2,72,170,113,148,7,44,210,57,236,51,96,134,91,43,18,129,39,153,99,170,205,85,53,46,13,105,94,202,115,94,
  213,71,50,120,206,122,126,213,223,41,104,79,181,81,241,61,25,108,210,231,21,0,76,172,63,170,246,104,3,4,213,26,204,172,49,114,64,194,61,115,
  2,60,72,32,50,15,152,203,51,46,85,65,75,180,73,50,27,241,74,215,40,177,1,202,155,184,138,64,160,99,147,130,104,203,178,248,49,26,4,240,
  219,227,130,5,189,204,222,196,221,87,16,32,31,103,140,202,99,197,16,65,107,250,66,209,30,21,144,61,22,2,228,211,235,41,238,41,131,227,196,56,
  39,107,169,106,126,112,115,34,255,226,29,7,28,134,180,255,202,227,60,241,78,130,0,134,105,136,33,150,184,55,221,255,197,152,137,1,155,9,211,52,
  216,245,78,135,5,232,127,139,247,32,143,61,213,60,234,24,115,25,159,28,230,50,13,65,60,120,109,240,154,141,185,62,177,231,248,87,130,29,89,0,
  173,2,95,129,2,44,202,139,184,220,17,114,36,193,69,195,242,38,68,55,31,102,244,118,196,242,113,108,8,141,13,163,200,167,221,22,57,202,101,95,
  106,6,83,165,27,100,242,181,56,182,69,176,10,20,228,27,164,189,116,1,160,128,93,244,155,232,99,186,216,172,100,113,230,22,0,83,98,216,140,28,
  44,158,8,18,100,182,204,96,22,85,202,220,117,194,2,124,90,219,173,3,113,50,152,204,51,238,69,63,24,100,3,229,130,49,35,131,237,74,140,255,
  28,172,145,197,244,63,11,190,232,2,42,205,103,121,46,232,54,33,57,120,228,192,98,39,44,102,39,167,8,226,222,242,251,49,53,223,229,230,127,30,
  118,29,57,216,79,213,218,43,111,66,130,121,203,58,61,147,55,21,63,8,226,177,36,24,25,50,222,85,239,129,3,218,65,251,90,74,112,206,48,184,
  186,247,225,249,62,76,197,158,34,3,153,196,137,14,121,95,242,102,76,199,190,174,159,56,66,132,142,113,193,114,25,109,16,72,55,102,45,216,150,218,
  146,55,112,166,115,203,36,8,210,151,246,18,43,199,188,144,223,157,65,16,156,161,9,184,114,147,25,25,239,214,106,239,111,129,52,127,76,2,82,227,
  49,199,172,91,132,161,23,188,107,196,40,182,175,19,237,9,25,128,79,53,199,162,181,84,180,141,130,254,232,58,6,130,192,71,224,7,169,64,231,147,
  198,92,244,36,161,83,204,5,35,236,35,203,50,19,124,19,200,124,12,130,0,129,56,217,252,255,99,199,16,64,139,27,254,207,191,251,59,45,83,37,
  255,222,19,199,91,154,136,45,5,240,52,71,145,114,52,249,113,164,19,89,160,40,107,163,231,58,57,15,16,205,195,6,82,246,71,142,166,229,141,73,
  62,54,227,216,38,238,89,100,233,8,189,143,12,238,84,139,180,0,143,180,93,180,255,59,5,39,121,143,39,57,96,33,88,155,32,8,96,219,54,60,
  207,75,44,84,255,26,218,173,26,187,226,243,178,228,65,215,6,218,14,110,126,202,71,202,116,44,233,2,0,121,62,231,253,81,201,53,86,250,67,1,
  44,39,221,200,146,98,200,99,69,124,95,180,75,117,188,174,122,158,188,107,142,252,187,60,82,7,121,173,203,51,102,184,77,141,155,239,121,153,54,249,
  184,155,94,83,117,252,41,31,151,38,78,112,152,254,160,204,44,199,232,171,54,81,177,15,169,250,54,239,94,209,233,248,227,246,53,21,168,160,237,206,
  35,139,225,174,155,245,217,78,214,50,185,255,233,247,93,215,141,246,27,213,28,224,230,9,247,190,242,236,135,42,146,66,183,246,210,123,103,205,117,250,
  57,186,23,172,36,184,230,222,43,55,102,243,226,146,172,211,193,223,55,230,130,68,202,113,192,149,5,219,127,96,207,241,207,137,29,255,253,191,253,183,
  17,80,183,229,35,29,89,210,160,155,192,170,133,67,201,134,145,73,37,51,175,220,196,73,28,35,183,59,83,214,169,230,1,195,242,66,110,219,54,11,
  22,243,108,132,242,34,148,56,74,96,162,41,14,84,203,19,141,2,42,110,66,210,137,74,175,197,69,103,242,132,144,251,92,5,174,85,64,60,117,236,
  35,181,73,119,188,43,47,202,42,16,160,219,124,197,98,46,247,45,125,62,110,225,254,180,180,155,211,59,115,115,5,210,169,194,199,57,2,166,243,142,
  155,211,170,103,146,219,75,165,71,220,194,76,127,47,207,219,78,2,19,14,56,171,250,180,147,147,22,249,121,105,219,178,78,16,116,107,71,158,13,68,
  5,94,84,32,93,222,72,100,173,120,214,201,151,188,174,112,0,155,91,71,228,251,171,78,92,84,122,73,113,61,1,200,104,254,134,104,59,199,64,113,
  68,130,42,48,208,173,131,193,10,181,172,121,78,187,120,125,105,58,104,206,43,163,83,141,149,64,33,29,225,214,137,60,160,48,171,45,34,200,209,245,
  155,65,36,47,186,113,191,146,246,168,214,7,249,115,244,216,159,211,51,203,239,74,198,54,89,65,114,167,253,150,7,60,235,246,90,93,48,246,73,98,
  46,49,183,116,152,139,238,147,220,250,194,225,134,255,167,96,199,196,186,8,114,188,33,111,216,242,166,45,22,64,213,241,106,222,65,67,193,92,222,65,
  171,155,136,92,187,117,108,14,253,44,61,6,82,69,233,220,113,45,7,210,116,207,170,3,167,170,8,158,3,52,220,70,165,122,6,121,177,17,131,72,
  30,232,89,236,188,106,226,168,158,205,243,188,84,155,228,126,203,3,44,76,211,12,143,63,53,160,238,211,222,238,172,241,203,1,87,249,232,92,197,10,
  174,148,113,227,164,63,220,51,115,236,190,42,186,23,253,42,207,253,149,104,93,243,108,202,226,93,10,198,63,11,148,235,244,194,89,235,155,216,140,229,
  107,102,205,75,25,88,185,174,11,207,243,82,107,44,183,206,6,29,234,239,243,244,179,235,186,137,241,166,74,220,137,146,104,53,193,183,46,144,16,96,
  80,204,17,10,16,197,58,165,154,3,186,96,129,99,156,117,207,45,250,123,165,253,170,59,249,18,207,71,231,162,110,60,200,253,35,247,185,153,83,30,
  197,93,83,30,155,52,239,70,110,123,214,62,42,143,15,213,24,224,100,138,186,241,34,222,189,106,126,172,148,52,224,250,92,140,49,25,144,233,218,39,
  63,175,220,87,170,241,198,229,40,209,181,134,3,146,178,252,242,147,198,92,121,242,64,242,238,93,20,83,252,63,14,59,6,1,108,63,8,96,181,55,
  0,110,98,101,53,66,23,73,208,99,226,78,221,3,58,57,94,81,209,245,242,166,200,49,137,170,251,100,29,87,112,26,85,221,230,152,197,180,233,88,
  43,21,91,216,233,49,148,78,235,42,31,135,69,25,250,154,9,167,99,81,228,13,152,123,6,110,81,233,244,185,228,104,247,211,212,110,213,36,165,12,
  157,60,199,184,123,233,54,176,78,152,216,78,65,35,55,255,105,255,209,57,174,74,212,232,164,157,58,166,83,5,168,242,48,76,89,238,43,89,78,23,
  220,233,144,236,142,147,231,249,4,224,146,65,180,138,121,237,148,5,203,114,31,80,5,211,70,70,130,97,150,124,129,2,23,17,216,200,12,147,238,100,
  72,183,254,169,216,224,60,125,78,147,182,243,178,160,89,167,40,58,208,145,117,100,173,250,126,158,253,69,247,61,142,185,230,78,187,242,56,51,113,73,
  247,186,246,200,253,155,39,143,131,187,174,204,238,203,4,129,138,220,208,201,3,58,125,223,220,169,128,105,202,73,141,226,126,65,34,199,50,252,119,64,
  254,1,124,223,107,183,47,64,16,208,246,252,243,99,174,78,216,120,57,216,16,114,12,46,248,251,87,141,29,13,3,54,20,52,188,234,129,228,72,134,
  91,116,179,38,60,103,21,66,39,68,39,26,98,221,34,33,3,33,174,93,170,9,41,128,24,253,189,124,204,146,181,216,101,137,222,179,128,85,94,80,
  33,191,224,172,141,143,11,14,232,113,24,5,113,220,198,203,29,237,208,54,11,32,208,41,147,215,233,224,87,109,22,159,214,118,203,108,17,167,255,162,
  11,133,74,55,152,117,212,216,73,244,174,2,239,89,57,2,186,163,181,78,219,66,245,236,121,64,146,172,231,214,233,68,105,192,37,179,113,170,147,135,
  60,63,28,27,163,235,11,122,60,152,87,62,162,146,117,233,130,13,213,26,193,253,119,30,0,154,245,35,7,80,116,126,118,114,18,161,179,118,212,1,
  39,170,195,149,55,222,60,39,111,170,62,146,115,44,84,155,62,167,223,148,199,179,14,184,169,52,223,114,255,115,58,208,44,107,63,93,112,39,179,211,
  121,64,140,234,26,121,131,73,221,201,151,138,53,84,189,207,188,164,21,71,82,100,7,48,65,4,124,41,232,229,215,182,132,167,139,4,148,179,192,157,
  175,60,1,74,222,139,79,241,214,97,21,10,122,243,204,101,121,141,209,5,32,50,187,254,175,25,59,218,20,56,201,71,195,156,116,128,78,78,213,32,
  86,101,49,203,172,158,60,201,85,139,63,167,203,227,6,188,88,68,228,65,161,139,122,179,244,85,42,214,81,94,200,196,6,72,219,208,169,215,50,101,
  131,85,9,9,28,32,208,177,30,157,176,176,89,145,184,142,173,144,25,96,78,150,146,151,141,203,203,216,169,180,129,159,150,118,115,186,86,238,89,228,
  35,50,89,211,197,93,59,79,59,228,228,26,21,179,166,99,99,84,122,217,149,232,154,85,199,123,116,190,203,115,92,5,202,85,73,101,114,159,169,22,
  92,209,47,170,239,201,129,25,119,93,122,138,192,57,136,80,198,36,75,58,210,201,9,129,232,79,234,89,158,245,94,56,70,175,147,113,76,3,15,58,
  135,84,243,77,199,196,114,0,148,254,55,237,87,238,153,184,99,230,60,1,163,234,36,71,214,96,102,205,49,21,192,215,5,55,220,218,164,218,71,140,
  28,150,115,242,201,138,138,152,201,2,207,170,107,211,100,80,149,254,156,202,124,196,53,101,146,130,211,254,178,214,178,4,236,232,78,132,242,250,129,115,
  125,30,255,255,32,5,152,1,163,205,24,91,4,172,10,171,60,209,118,186,174,203,14,127,105,176,38,198,46,13,42,147,237,73,186,97,4,129,79,176,
  153,197,182,81,24,175,200,224,253,147,192,92,42,208,73,37,31,220,233,146,76,206,253,107,194,142,182,176,61,49,200,3,171,88,186,172,69,73,94,236,
  84,201,42,170,12,110,221,81,116,22,88,145,163,122,213,34,33,71,214,121,35,106,149,214,77,102,22,184,35,201,78,179,193,179,38,190,78,47,167,98,
  61,84,71,48,186,123,137,191,81,118,44,107,177,231,22,72,26,16,232,216,76,25,108,172,212,169,225,211,210,110,157,147,3,151,32,145,215,87,52,47,
  91,42,64,34,199,136,113,146,24,29,176,88,201,143,202,43,61,207,252,207,163,87,86,37,121,233,158,35,175,133,153,172,99,148,37,23,178,150,87,150,
  77,112,27,129,188,6,229,213,125,170,198,142,96,93,185,227,200,188,96,182,19,0,34,51,190,170,62,206,227,96,196,201,164,100,38,46,175,39,173,252,
  57,174,232,131,42,184,144,251,72,39,219,203,83,232,71,149,0,153,231,148,66,38,183,56,91,194,60,65,152,42,40,236,228,121,84,227,83,151,132,202,
  157,60,168,246,52,154,95,145,69,110,113,129,171,44,75,76,7,100,218,145,77,218,25,254,119,236,134,147,111,46,230,73,12,230,128,157,122,174,37,1,
  181,220,222,216,97,82,102,189,3,34,31,249,120,152,75,5,56,229,185,173,147,108,208,125,231,211,138,29,163,0,222,48,96,154,134,17,122,104,50,47,
  90,252,35,44,136,116,11,67,30,150,36,101,58,206,52,88,149,204,145,71,218,192,121,230,234,62,39,54,57,145,192,160,99,116,228,231,148,63,79,55,
  204,78,101,24,170,62,19,122,65,93,66,146,204,204,232,172,90,100,32,152,37,228,167,155,175,46,67,92,6,15,52,49,135,142,157,60,128,139,70,178,
  170,119,72,175,207,45,166,159,166,118,115,44,77,94,240,146,247,111,98,92,80,253,169,188,145,102,177,95,242,145,91,214,253,117,253,160,98,186,104,63,
  200,109,163,115,54,235,186,92,64,35,95,151,99,201,178,214,44,46,185,139,59,182,228,54,16,206,70,80,165,177,167,243,61,207,17,187,238,111,186,226,
  76,220,201,99,158,68,50,29,27,205,249,207,235,236,229,56,22,51,207,154,174,107,163,60,175,85,99,91,87,208,68,149,196,39,214,249,60,32,67,5,
  170,242,122,76,235,214,25,14,92,100,1,97,213,218,207,125,94,149,200,201,237,47,121,79,156,68,210,104,30,233,12,189,87,158,4,71,154,112,71,199,
  83,8,142,40,128,84,75,54,98,219,236,128,0,79,40,130,38,3,190,31,234,156,61,207,39,32,214,128,176,76,23,255,77,255,49,77,11,134,97,182,
  193,112,242,111,226,59,190,31,68,215,228,255,129,178,237,34,56,16,255,36,215,173,248,251,166,105,144,128,160,115,204,69,19,49,85,251,141,14,59,230,
  145,155,254,161,99,71,203,52,97,203,232,155,59,78,205,202,98,84,33,123,46,147,91,206,44,165,64,84,78,182,160,3,93,150,46,228,177,19,203,211,
  94,221,113,146,42,26,210,69,194,157,216,95,101,217,63,229,57,82,81,37,146,233,146,18,233,38,196,49,94,242,181,117,73,60,220,98,76,143,108,132,
  141,19,151,197,172,58,130,203,122,151,226,123,42,207,199,79,91,187,85,44,160,74,159,175,138,156,179,254,174,98,12,57,139,51,202,194,211,182,100,109,
  190,66,54,192,205,41,209,71,242,103,228,197,41,75,127,169,11,118,229,177,44,175,103,220,251,211,1,34,206,2,139,147,224,116,114,194,196,245,115,94,
  118,57,15,59,168,98,103,57,41,129,138,49,239,148,161,150,219,171,98,213,117,140,81,158,77,46,239,41,41,157,171,178,60,42,79,34,57,119,178,180,
  146,32,86,183,46,228,209,124,234,24,53,250,57,157,108,102,165,1,145,78,126,163,234,15,241,190,169,93,33,5,91,42,176,163,178,105,236,148,80,136,
  199,154,92,56,60,32,64,81,176,192,89,197,146,130,4,163,171,59,117,15,159,211,204,69,154,241,9,152,80,156,52,234,24,114,238,62,20,44,203,235,
  150,32,0,205,232,158,73,187,193,124,152,75,149,140,188,82,236,168,42,217,253,105,192,142,54,167,155,146,35,125,223,247,163,154,225,178,30,76,181,89,
  170,162,7,49,185,84,108,131,110,113,162,215,225,54,194,143,123,36,153,247,248,66,6,12,89,44,79,222,5,82,181,120,103,149,81,214,105,72,229,119,
  69,163,243,44,87,1,238,232,75,190,167,252,206,85,218,38,149,167,43,93,120,185,77,81,103,230,175,58,242,254,180,181,155,123,103,178,68,32,239,17,
  122,94,150,48,239,53,228,254,205,114,100,160,78,18,178,54,150,50,81,121,216,83,110,67,229,181,130,124,112,33,214,44,234,106,146,229,172,163,58,166,
  148,153,55,90,13,77,44,228,157,172,57,92,112,164,3,24,121,11,159,112,107,179,74,38,64,237,232,184,77,139,62,163,78,119,159,181,126,82,80,149,
  167,237,170,44,123,90,8,138,27,203,116,204,113,128,185,147,32,36,11,8,117,82,240,67,7,174,59,241,48,215,157,18,81,144,160,146,33,229,61,210,
  150,199,30,39,113,81,1,15,57,167,42,203,162,147,158,248,209,117,65,118,36,234,28,240,24,202,245,152,159,23,148,136,50,8,171,155,100,108,229,241,
  35,191,170,60,50,62,213,123,140,251,59,4,187,97,123,3,133,191,182,161,0,217,114,210,98,236,20,66,217,106,145,244,24,223,27,26,54,158,151,236,
  173,20,59,114,193,60,71,94,202,235,209,31,20,118,108,87,249,181,243,108,204,50,163,39,27,109,115,110,7,244,187,114,231,169,54,101,221,128,163,11,
  189,46,17,40,143,238,145,91,224,233,61,58,1,37,148,169,212,129,91,238,184,86,151,184,34,111,88,186,62,202,243,194,197,59,200,83,1,47,207,61,
  84,12,90,86,66,163,124,52,167,202,132,213,89,180,169,54,177,79,91,187,101,125,158,106,113,81,141,217,172,241,42,207,189,60,109,203,2,223,242,28,
  84,201,12,184,228,16,193,30,168,22,47,157,142,144,147,78,232,158,155,75,160,202,186,47,119,162,144,181,160,202,137,147,121,251,86,85,152,71,231,179,
  157,55,195,92,6,62,170,192,128,243,2,150,193,15,23,240,228,209,223,11,253,53,29,231,178,67,14,199,22,113,0,74,190,94,214,233,95,214,188,248,
  184,85,48,243,146,46,92,137,115,217,25,97,165,109,225,72,2,46,248,212,5,164,89,128,78,199,190,101,89,147,201,123,178,234,212,72,124,158,203,85,
  161,184,163,115,223,243,36,160,76,130,103,190,40,79,60,39,213,207,72,231,162,238,181,173,228,157,210,251,3,22,219,239,97,59,77,45,80,143,231,82,
  204,84,203,73,135,114,243,4,168,142,193,52,197,67,58,253,245,202,177,163,172,227,151,215,0,177,94,168,92,87,254,197,177,163,97,164,1,52,167,35,
  201,74,154,200,178,34,145,171,219,168,64,3,213,78,114,21,112,58,173,194,198,37,75,168,138,3,200,199,195,121,39,64,86,121,229,60,14,16,220,134,
  250,73,84,207,162,22,47,2,232,171,250,133,107,127,86,52,157,199,212,92,149,60,149,135,61,149,173,118,84,99,236,211,218,238,78,142,140,100,96,151,
  87,119,72,117,217,116,19,209,181,37,203,37,128,91,27,242,204,203,172,228,18,213,230,202,157,200,228,145,112,100,21,41,146,203,187,235,142,172,179,42,
  171,169,142,31,229,132,55,186,182,209,226,2,178,39,49,87,142,157,99,100,133,3,16,247,156,42,9,147,106,44,171,130,36,238,249,242,156,160,233,250,
  74,183,86,170,222,111,222,228,106,221,9,157,28,92,229,153,203,42,153,27,221,220,85,243,93,46,53,221,169,141,163,220,118,249,93,229,217,119,232,252,
  83,185,237,200,107,30,13,58,229,126,23,18,44,25,244,170,222,5,231,243,173,75,204,149,221,134,146,107,129,220,199,126,27,220,101,5,76,52,88,75,
  252,53,193,232,102,1,50,89,142,166,123,166,188,129,47,199,184,170,214,213,248,90,249,48,151,232,151,248,30,73,118,58,41,115,48,164,4,203,206,44,
  60,85,26,100,93,78,72,94,236,168,218,151,255,37,176,163,223,22,171,27,0,108,213,17,180,106,195,148,23,120,249,51,178,150,73,117,116,168,250,157,
  106,211,232,196,187,86,229,157,43,191,76,85,86,176,110,81,164,139,136,109,219,185,128,146,104,135,235,186,40,20,10,153,160,85,12,4,106,21,180,18,
  87,10,213,96,204,91,252,69,103,109,198,189,107,157,149,145,60,40,243,36,151,233,88,139,127,13,237,150,65,73,158,177,190,18,105,146,156,1,255,73,
  220,35,171,124,181,92,106,58,143,46,144,3,111,242,156,94,137,94,77,245,188,242,241,112,30,246,94,197,48,202,235,165,252,125,21,1,209,137,191,170,
  238,40,85,183,105,169,138,94,232,138,113,232,42,168,234,18,100,229,96,64,183,230,139,251,200,26,105,218,54,157,59,146,44,189,82,73,181,100,31,98,
  142,253,82,245,157,202,34,51,79,66,158,74,151,172,2,176,42,82,131,171,152,151,53,38,242,238,147,42,210,70,183,231,136,100,64,42,187,200,195,106,
  103,21,221,201,46,188,33,3,71,63,1,130,165,79,43,238,19,176,246,114,121,215,20,213,158,156,150,72,80,160,30,176,133,100,226,62,50,82,214,115,
  73,87,13,61,177,208,137,85,45,253,149,0,212,113,123,204,84,96,145,30,47,70,212,247,252,247,242,189,235,143,131,29,229,235,255,115,99,71,139,72,
  85,108,206,61,64,165,251,227,162,95,58,241,57,6,133,59,50,87,213,173,111,181,90,153,150,36,158,231,37,34,95,193,190,112,128,87,222,16,132,30,
  135,3,73,157,68,88,212,186,74,149,28,193,189,28,25,60,83,141,33,237,59,149,52,65,87,5,74,102,231,116,30,162,157,128,82,113,77,206,18,40,
  47,32,211,49,123,242,223,233,70,156,87,239,166,2,7,127,232,237,150,153,44,249,154,42,246,41,139,45,214,21,150,200,115,205,143,115,226,65,19,135,
  84,5,96,116,5,136,100,198,150,234,47,5,96,146,61,104,59,241,249,229,216,88,185,234,99,30,86,51,47,27,45,63,47,87,241,74,14,238,56,217,
  87,214,73,137,202,183,156,155,235,89,149,253,84,204,75,22,163,196,49,153,186,83,14,29,59,46,23,3,81,173,197,116,253,229,18,93,179,74,103,103,
  145,18,114,82,92,39,39,40,114,223,100,141,87,206,22,85,236,119,43,209,113,203,193,8,71,136,229,41,92,162,10,180,196,126,198,73,58,85,251,141,
  60,55,116,185,42,212,99,153,130,52,202,144,38,231,167,14,48,6,202,247,203,1,119,53,40,77,254,127,161,19,142,255,191,250,164,69,207,46,235,117,
  189,210,27,137,238,197,205,211,44,107,196,228,243,187,9,169,6,101,168,233,103,104,128,154,196,23,38,155,148,249,251,192,142,186,254,113,28,231,159,21,
  59,70,242,182,172,13,76,181,16,231,25,108,157,50,92,116,66,114,145,5,199,8,136,23,171,107,143,170,98,27,183,152,200,209,6,87,229,75,55,40,
  116,90,90,213,181,56,43,179,44,41,9,189,22,151,228,145,149,229,158,165,17,148,39,138,92,81,76,5,8,84,5,96,178,0,124,22,88,209,181,81,
  222,44,63,45,237,214,185,9,168,130,76,213,184,146,129,17,183,40,172,36,27,95,53,126,184,163,109,85,185,110,213,9,80,158,160,72,214,213,102,201,
  142,178,250,106,37,64,97,37,239,154,59,206,85,149,80,86,189,35,217,204,159,178,181,52,24,145,11,139,200,235,18,119,143,188,206,74,92,0,207,37,
  4,113,21,248,56,118,94,254,94,30,118,57,47,91,75,79,20,228,53,49,203,135,88,181,87,200,255,94,201,73,13,215,15,180,58,34,199,54,139,177,
  206,141,119,174,160,132,74,163,156,101,247,168,219,211,228,127,103,213,73,80,73,85,100,86,81,85,216,133,74,14,210,73,123,129,182,210,159,239,7,146,
  53,91,190,253,78,224,135,180,173,164,190,207,210,196,68,144,98,119,117,201,130,58,25,147,42,176,200,179,230,168,214,39,213,254,149,174,127,64,159,85,
  244,145,173,124,15,66,31,29,183,51,4,210,113,226,162,222,154,178,211,19,117,213,231,139,197,34,27,152,255,190,177,163,45,199,102,170,218,245,170,137,
  153,151,197,200,218,212,84,37,80,85,157,209,201,230,152,85,244,37,175,159,166,238,154,170,99,77,89,111,152,165,131,238,36,48,161,199,218,121,203,146,
  118,202,94,114,118,96,42,73,128,204,22,230,153,204,242,2,46,31,231,234,74,49,235,116,185,159,150,118,103,45,208,42,230,118,37,11,144,188,233,234,
  198,75,30,246,61,79,112,201,49,121,170,49,46,111,178,220,216,148,23,52,206,153,67,151,133,157,21,28,113,137,124,121,1,187,234,100,130,62,151,44,
  53,200,43,227,201,210,12,171,2,72,153,149,81,173,203,249,217,47,181,28,143,22,230,225,238,43,3,192,172,114,192,58,194,99,37,18,36,153,9,230,
  138,232,8,16,174,178,136,212,245,181,106,109,213,37,141,115,0,153,178,239,156,75,129,46,136,204,114,216,225,114,10,184,192,88,37,175,228,192,159,206,
  153,75,21,216,203,239,64,78,250,147,199,106,18,23,36,217,232,56,209,13,29,141,217,100,32,103,166,52,214,170,211,75,238,251,66,63,156,3,6,146,
  235,229,155,115,121,171,144,210,32,34,253,249,206,156,88,146,125,107,176,204,52,39,243,8,175,69,43,58,6,236,123,203,34,240,242,98,71,85,129,35,
  85,64,255,137,96,199,118,47,152,166,25,218,216,101,233,69,197,100,238,52,177,77,78,94,234,148,233,200,243,146,243,44,174,42,157,183,106,35,149,153,
  158,149,254,168,64,93,214,75,235,164,223,184,69,77,182,108,81,153,226,103,69,177,244,218,156,52,68,165,57,202,195,158,234,52,216,186,114,158,220,115,
  200,1,196,167,165,221,89,129,88,30,43,46,213,152,163,215,149,45,162,84,71,127,89,139,184,206,86,46,171,48,131,46,194,87,105,230,184,98,35,42,
  240,160,75,114,203,42,192,208,233,9,154,188,206,80,22,145,59,10,148,25,123,206,87,90,53,206,84,46,42,121,18,199,56,16,216,201,38,157,103,77,
  83,129,19,78,198,33,7,59,89,71,205,58,41,146,170,250,167,204,214,115,146,68,157,61,36,103,157,149,53,31,116,71,192,242,24,17,243,144,171,112,
  154,5,214,117,123,5,87,96,37,79,82,187,220,118,174,236,114,39,164,142,234,4,91,29,48,80,109,115,236,97,76,1,105,82,43,156,4,160,58,185,
  148,62,248,204,198,29,114,98,111,122,205,50,83,192,144,239,19,153,245,54,25,25,38,125,239,22,121,30,174,100,183,74,82,166,51,8,144,131,161,252,
  39,172,42,102,58,201,60,7,204,103,197,248,229,158,227,147,193,142,186,253,150,11,252,62,17,236,216,174,66,24,51,208,10,173,114,158,5,54,143,136,
  93,254,140,168,78,163,243,131,213,177,128,170,255,150,75,120,170,168,123,29,27,155,229,84,209,9,176,18,207,199,233,123,116,21,153,242,190,108,213,123,
  200,195,64,203,207,202,45,166,42,134,151,211,209,210,4,11,206,186,112,37,63,121,77,216,63,173,237,166,85,179,58,149,56,168,128,171,110,28,170,153,
  134,108,221,175,138,173,238,116,156,114,39,88,170,207,113,210,164,236,133,94,173,85,230,42,31,230,1,40,121,2,111,110,237,144,1,132,204,238,101,201,
  69,184,64,39,171,112,21,247,60,157,234,222,243,4,21,89,236,81,39,30,184,170,13,46,207,201,160,12,66,117,196,7,151,235,163,26,47,121,92,71,
  242,216,32,202,99,68,222,103,84,1,65,94,191,243,78,8,30,46,168,225,230,163,206,114,86,182,41,204,146,69,234,61,236,243,124,150,123,230,100,33,
  17,85,65,35,29,136,213,157,250,81,240,36,42,7,202,237,78,131,240,60,129,167,161,41,5,14,246,57,211,191,55,72,129,151,52,55,202,237,97,114,
  241,17,238,126,130,161,206,195,204,114,96,154,99,166,101,32,13,37,151,251,251,197,142,186,124,176,143,139,29,109,142,217,80,177,150,186,10,87,92,244,
  44,235,187,228,98,11,121,216,224,78,22,45,149,207,171,174,214,57,199,102,138,168,35,203,98,229,227,128,95,110,194,139,151,167,178,43,226,152,13,238,
  57,117,71,160,170,13,150,110,122,220,56,144,153,91,157,117,83,167,82,146,44,86,92,247,35,23,236,248,52,181,155,155,71,157,36,197,233,54,3,154,
  64,212,105,190,130,202,118,74,246,194,206,227,145,156,149,160,197,49,212,242,251,202,2,252,194,221,134,155,123,212,47,91,165,71,213,5,13,58,175,85,
  213,252,225,192,223,39,97,75,153,189,177,165,217,223,172,147,142,188,100,9,247,140,121,199,166,92,226,155,251,156,60,31,178,250,75,94,235,84,58,111,
  58,15,68,240,162,115,249,144,229,37,170,10,108,178,139,7,253,142,170,224,140,174,64,150,92,92,165,211,147,207,172,60,14,238,56,155,99,163,85,129,
  151,206,169,72,199,250,233,214,8,223,143,143,248,229,98,31,20,88,197,229,184,209,145,53,98,182,141,47,7,184,233,24,180,51,175,147,100,116,131,76,
  139,70,21,64,204,114,106,225,3,136,212,95,218,239,204,32,128,56,141,185,248,19,47,53,152,214,147,69,73,135,142,228,115,196,229,211,85,204,55,7,
  162,127,31,216,81,5,200,87,132,29,201,247,237,44,246,33,43,178,211,1,199,188,0,78,183,56,202,114,138,60,236,136,252,153,60,85,163,104,39,233,
  62,159,119,83,237,36,210,165,11,42,7,198,85,199,222,242,166,64,25,89,85,82,145,124,204,70,219,222,169,254,83,6,163,42,7,151,188,12,126,39,
  94,179,89,193,200,167,161,221,121,108,195,242,130,92,21,40,236,228,104,62,75,231,167,139,230,87,114,146,36,42,84,233,28,31,242,216,45,202,39,61,
  28,67,199,5,208,201,162,8,70,238,62,226,108,189,184,185,73,129,148,240,206,85,37,17,119,2,154,179,172,245,184,191,211,103,237,148,152,232,132,233,
  204,35,89,226,130,51,185,44,47,117,56,210,149,125,87,1,53,57,179,95,87,11,160,147,231,224,222,161,184,23,39,203,208,49,185,89,62,245,121,130,
  80,174,189,121,223,165,74,110,35,203,223,228,251,208,68,77,213,126,153,93,224,2,81,233,235,248,121,141,84,130,90,120,125,147,20,98,138,75,102,119,
  202,52,243,237,49,18,160,49,207,178,166,150,132,100,123,122,175,36,137,58,255,186,128,68,223,132,159,201,78,198,20,196,136,76,94,112,39,0,42,89,
  40,223,247,50,144,142,63,147,108,122,26,68,255,190,176,99,30,119,158,76,236,232,251,48,68,125,141,44,22,149,91,140,85,209,82,94,0,41,11,191,
  229,135,145,19,178,184,8,90,183,192,169,6,159,106,225,12,219,99,131,138,222,169,37,151,74,15,198,149,60,85,189,120,174,110,124,86,153,87,78,55,
  45,15,92,89,91,41,79,6,122,212,38,179,45,170,246,208,163,78,149,46,47,75,87,45,222,111,167,218,121,142,193,144,35,83,149,108,225,211,214,238,
  60,11,66,231,12,138,58,32,202,2,204,121,170,91,169,146,171,242,0,253,60,193,141,238,221,168,44,189,116,1,140,74,6,144,37,213,201,195,228,113,
  159,83,89,196,201,12,80,94,219,60,185,244,124,94,112,199,57,78,200,9,126,244,30,186,108,117,149,4,132,179,241,212,1,54,185,204,175,106,45,202,
  58,213,161,44,177,188,33,202,108,48,125,110,174,228,186,172,75,207,74,98,229,158,139,38,155,169,130,13,249,116,81,213,111,92,65,28,90,176,165,147,
  18,226,242,30,166,58,117,225,100,55,92,178,161,44,143,225,156,170,152,183,149,0,83,188,84,193,82,50,150,50,11,104,154,134,84,57,56,212,254,230,
  15,244,117,128,59,9,228,86,154,27,161,147,99,169,8,14,149,107,135,106,93,203,90,63,146,115,133,174,237,60,160,150,49,132,142,153,206,42,200,35,
  131,228,116,242,96,28,112,228,209,39,127,18,216,49,235,52,172,35,236,40,214,83,0,182,236,70,65,47,34,251,106,210,35,43,185,230,125,158,73,157,
  167,18,153,106,145,202,19,145,113,25,196,121,168,253,32,176,0,120,185,143,5,84,37,119,63,14,35,167,90,176,101,38,75,222,144,116,32,81,12,160,
  106,119,15,106,141,122,234,154,186,104,87,246,168,166,27,180,109,90,104,56,205,68,27,84,182,123,186,72,82,117,164,40,51,29,170,235,113,11,191,170,
  221,170,73,245,47,217,110,93,2,81,39,139,182,12,40,56,135,153,60,44,74,30,86,142,11,164,179,18,9,59,241,218,94,105,2,113,158,138,86,89,
  39,62,89,63,50,152,229,28,2,116,247,227,52,168,173,86,43,225,15,207,249,233,170,138,202,112,172,16,183,94,200,1,82,167,12,184,238,189,136,160,
  38,171,240,137,232,59,207,243,82,126,248,114,27,179,170,148,113,9,238,242,90,192,49,190,121,78,121,84,239,84,7,18,185,114,197,89,123,31,183,231,
  169,28,106,196,60,206,147,84,78,199,13,253,188,74,146,201,173,129,220,216,208,149,225,142,215,91,174,228,179,138,213,77,3,89,113,61,121,31,151,251,
  36,12,162,179,231,106,252,252,73,86,91,181,94,117,50,15,210,129,122,12,8,129,64,203,206,235,164,30,225,88,70,194,202,47,156,87,65,174,92,144,
  244,62,38,127,46,80,56,102,164,219,197,49,220,170,49,160,178,127,139,229,65,129,228,233,157,222,251,226,219,241,178,142,172,247,145,53,151,117,223,237,
  8,59,182,65,180,129,182,186,155,99,205,178,192,31,157,244,157,28,63,102,61,88,39,94,186,242,239,84,29,192,29,137,137,54,12,14,12,226,192,131,
  251,177,107,231,78,244,245,246,177,139,15,87,37,77,244,129,77,44,135,178,22,104,157,247,168,0,34,242,117,56,143,104,10,102,85,218,29,250,221,63,
  254,163,63,198,250,117,131,81,229,68,89,246,161,26,156,92,132,252,240,67,71,240,244,83,95,136,54,105,90,234,53,175,118,56,43,41,135,27,39,156,
  227,65,30,176,37,47,84,127,72,237,254,56,236,179,138,229,205,178,167,227,230,185,60,230,196,127,123,158,167,28,211,92,25,108,122,127,153,57,118,93,
  55,74,0,145,127,84,191,151,217,23,21,48,19,201,152,43,97,181,59,73,14,19,115,78,158,119,162,191,116,235,168,104,63,45,195,110,24,70,194,251,
  94,117,122,146,229,95,172,146,167,200,239,47,15,91,236,186,110,106,77,87,129,40,46,95,67,181,39,136,231,86,49,241,89,71,215,89,182,117,2,4,
  210,132,41,85,223,231,89,155,233,152,162,172,91,167,39,67,242,251,211,125,71,172,207,156,116,133,150,125,207,19,76,115,123,29,23,80,112,215,12,130,
  32,26,7,242,216,230,131,106,90,184,198,143,254,225,173,229,98,75,180,24,116,167,223,21,151,0,38,36,22,233,4,184,32,53,206,66,86,219,64,92,
  229,15,9,29,53,237,227,60,235,177,120,78,113,157,248,255,27,196,255,56,132,84,134,97,178,164,100,220,126,209,46,249,217,232,51,27,237,190,10,255,
  29,183,23,81,223,134,253,110,100,230,119,209,241,171,114,65,137,37,40,6,57,45,48,148,14,73,180,63,249,53,20,138,251,196,90,243,80,3,207,95,
  51,121,34,145,141,29,243,146,80,170,119,188,18,236,8,0,182,64,210,242,23,56,112,198,29,131,115,94,159,201,104,42,200,149,16,34,31,19,229,93,
  160,242,108,156,220,162,105,91,37,60,249,133,207,97,96,96,16,211,211,51,240,124,3,15,61,244,40,174,93,187,132,119,79,159,78,124,87,206,96,165,
  127,251,194,23,158,194,194,194,28,78,158,58,149,122,38,142,53,86,73,97,116,199,241,242,177,121,39,1,136,97,24,48,237,18,170,213,62,220,189,55,
  153,26,76,5,187,128,67,35,7,80,169,84,48,113,123,28,215,110,220,194,208,134,245,152,184,51,17,31,190,181,175,121,232,224,8,70,14,29,68,177,
  104,96,223,238,157,24,187,126,3,245,166,163,108,231,74,1,169,238,221,113,229,102,117,108,48,119,79,85,249,224,127,137,118,231,1,110,28,219,203,177,
  206,242,41,145,206,204,95,6,48,220,177,118,30,91,40,122,74,165,2,64,153,153,204,154,196,144,44,86,136,178,200,42,153,140,142,169,235,52,136,23,
  253,74,131,6,89,190,163,75,86,229,60,146,179,88,194,172,188,20,238,119,52,240,214,85,209,227,222,133,231,121,41,251,38,157,99,5,87,184,136,75,
  206,211,29,103,103,173,239,121,243,10,58,33,97,100,121,145,60,7,184,28,3,238,189,102,5,60,43,41,128,145,199,14,83,5,160,243,36,90,233,114,
  30,116,201,90,252,156,17,201,106,102,78,182,217,96,245,186,170,62,85,37,205,165,79,254,232,126,26,104,25,83,145,244,199,1,88,142,109,13,75,94,
  211,83,79,85,187,197,248,225,228,26,104,75,78,56,25,77,248,123,219,182,18,0,212,178,210,86,126,28,171,78,229,47,226,93,168,230,30,221,31,146,
  9,247,80,50,211,201,32,90,149,244,72,115,26,84,50,75,131,244,103,128,216,161,35,157,143,21,246,245,63,15,118,204,187,38,203,115,192,86,29,9,
  230,253,209,85,204,235,52,147,56,143,29,16,183,249,119,122,4,94,44,148,241,212,147,95,64,173,9,188,244,163,159,163,86,155,6,0,20,10,221,120,
  242,243,159,195,154,85,107,48,51,55,147,0,8,17,51,109,24,240,72,255,188,255,193,121,108,219,182,29,123,118,239,130,109,218,56,127,233,34,142,61,
  241,25,44,46,46,226,220,185,247,145,39,43,65,102,120,228,146,147,121,6,6,87,57,74,92,119,169,182,128,173,91,247,97,199,246,29,248,237,239,126,
  135,173,91,182,160,111,213,42,220,187,59,137,39,142,127,6,11,11,11,56,125,250,61,108,221,182,19,71,30,251,12,6,6,86,227,31,255,241,7,152,
  184,115,7,159,61,118,12,229,82,1,191,249,221,219,112,3,11,211,51,115,88,191,110,0,235,54,108,193,212,244,44,234,205,41,118,211,150,245,149,42,
  13,222,74,228,11,156,107,8,119,4,206,31,143,5,74,16,240,207,221,110,21,168,211,1,108,85,225,29,25,200,232,230,30,103,183,166,234,39,174,24,
  144,204,22,137,207,113,224,252,227,248,168,115,21,13,169,132,140,3,141,234,227,71,35,19,56,171,216,245,60,235,153,204,250,171,214,80,206,29,70,23,
  12,203,73,59,42,63,123,213,26,33,8,128,78,10,165,136,124,137,188,251,128,172,215,86,109,102,242,51,103,125,62,175,14,93,183,158,102,37,248,170,
  28,40,84,109,206,51,126,116,114,9,142,168,202,115,10,38,127,158,3,220,66,239,172,211,154,231,101,212,85,53,12,196,87,124,223,35,146,27,75,10,
  34,5,80,90,89,130,95,252,111,113,175,80,199,235,251,1,3,186,141,4,104,22,101,165,179,24,217,240,115,22,107,73,151,6,167,136,174,43,131,205,
  100,241,18,190,216,75,114,108,208,164,69,16,166,57,96,239,203,49,177,182,109,49,110,22,33,91,157,100,113,233,154,164,126,183,105,185,82,32,129,124,
  67,186,94,236,1,45,191,223,216,174,79,207,234,27,134,252,28,201,123,196,227,252,159,15,59,118,234,190,101,24,6,108,131,188,173,149,250,224,114,71,
  198,186,98,17,170,7,147,19,84,84,139,39,221,76,59,109,167,97,24,120,226,241,199,48,189,176,136,83,239,188,149,24,156,173,214,50,102,231,166,241,
  181,175,125,5,23,47,93,194,59,39,78,224,192,254,253,216,187,103,55,28,199,131,1,31,3,3,253,104,121,1,106,203,75,232,234,170,224,202,135,215,
  208,221,85,194,227,143,63,3,0,24,191,59,139,222,190,53,56,124,228,40,14,141,28,196,204,204,28,166,38,239,227,252,197,75,88,92,90,210,178,75,
  58,47,224,188,25,188,220,103,22,22,22,81,41,87,177,110,227,102,252,219,127,251,109,212,106,45,76,207,204,225,209,199,31,3,0,156,126,239,12,110,
  221,190,137,158,106,31,14,31,62,8,207,247,177,121,211,22,28,26,121,24,171,251,215,195,0,48,114,112,17,167,207,188,135,153,153,25,28,126,228,16,
  94,125,237,87,137,247,163,74,66,85,37,208,137,133,158,30,33,102,5,10,220,216,82,177,138,42,22,95,21,112,252,75,181,91,197,38,170,146,29,104,
  185,226,60,26,56,213,102,75,35,125,142,81,211,249,148,115,247,161,128,86,7,32,233,105,78,167,69,76,40,32,167,86,147,121,252,170,243,46,186,116,
  14,137,35,108,29,192,147,43,235,229,101,66,178,64,44,13,64,86,210,87,92,208,160,42,154,197,205,153,60,206,32,58,203,63,213,184,166,255,94,105,
  112,165,98,108,229,19,154,60,21,54,179,170,33,230,61,217,82,157,50,232,198,152,156,232,72,63,47,251,221,102,105,218,185,182,115,193,150,42,185,148,
  206,91,30,92,7,140,31,178,145,240,115,86,5,239,157,217,201,113,123,158,96,183,3,37,192,165,96,43,143,76,39,4,226,226,221,154,202,182,82,70,
  153,99,211,5,67,44,131,89,154,184,199,125,47,30,7,86,196,78,115,204,180,252,221,36,19,46,23,48,73,178,217,84,46,19,22,51,49,72,251,210,
  167,216,156,247,120,50,136,48,18,22,117,116,252,36,115,111,64,130,132,44,121,140,172,119,14,131,155,244,24,237,28,59,170,246,65,29,118,212,217,39,
  115,65,153,173,154,120,121,192,178,142,18,87,69,128,20,128,136,205,70,181,24,235,218,194,77,242,172,68,22,0,216,48,184,1,61,189,171,240,155,151,
  95,6,167,177,57,117,234,36,110,143,143,227,185,231,158,193,204,236,34,122,170,125,216,252,192,86,156,125,255,67,44,45,204,97,211,3,91,48,121,123,
  18,23,206,95,193,35,135,31,194,103,158,120,60,90,120,174,124,120,11,75,75,117,116,119,117,193,113,90,120,227,173,247,0,35,64,95,111,31,138,197,
  46,0,75,185,142,99,185,141,141,91,236,228,119,166,210,119,78,78,77,99,235,230,46,92,184,120,25,143,63,250,16,126,245,202,235,184,115,247,30,182,
  109,27,130,105,89,24,222,182,29,189,189,171,49,114,232,65,188,254,250,41,24,150,133,207,60,241,24,110,143,79,224,251,255,251,123,232,95,179,22,79,
  126,254,40,78,157,126,23,181,218,34,108,203,100,19,102,84,128,140,130,62,250,60,114,22,61,151,240,151,53,89,116,160,141,43,138,163,42,21,255,47,
  213,110,213,252,227,64,183,152,47,58,39,145,172,34,40,121,88,114,249,186,162,31,58,57,9,81,149,106,22,236,230,199,97,164,169,196,68,183,73,202,
  39,57,121,1,16,199,100,230,45,46,242,113,24,247,172,35,234,60,204,164,60,182,242,4,17,134,97,68,213,42,85,1,168,110,189,145,45,227,116,99,
  187,179,82,194,217,73,163,50,96,239,84,35,169,219,99,228,247,174,242,25,167,227,154,115,22,201,35,45,148,219,33,164,52,186,254,201,42,22,195,141,
  71,157,76,72,149,48,233,251,162,29,126,36,211,160,12,164,240,30,166,204,170,234,213,113,235,110,120,63,35,181,167,164,181,224,177,166,153,123,143,220,
  218,34,131,115,193,14,83,219,58,25,56,114,160,55,13,172,13,194,34,115,160,240,227,157,182,169,214,239,152,249,166,137,139,201,54,202,195,66,36,54,
  134,93,35,192,174,159,112,50,81,17,39,130,145,150,223,41,101,165,147,253,197,91,214,113,150,137,233,231,132,20,208,36,175,21,190,167,0,190,175,199,
  142,42,124,216,9,118,212,85,106,78,156,6,172,100,97,215,233,167,84,229,148,179,38,189,142,177,146,237,228,116,44,174,110,193,50,12,3,155,30,24,
  66,189,222,68,224,59,169,207,237,223,183,23,107,7,215,195,178,109,44,47,53,112,244,232,97,252,239,239,255,19,14,236,223,135,177,107,215,113,253,250,
  5,108,216,48,128,185,185,58,70,71,47,162,88,48,49,180,241,105,24,134,129,243,31,124,132,223,254,238,13,108,123,96,35,6,6,250,81,40,216,152,
  155,157,198,244,204,125,54,122,81,89,114,169,22,247,60,21,188,84,140,201,237,155,55,112,96,223,94,188,253,246,73,220,187,51,142,27,55,174,97,245,
  154,245,184,121,227,46,124,24,88,59,56,136,235,55,38,240,127,126,240,99,220,185,115,3,150,101,225,198,245,43,152,154,156,66,16,120,184,119,239,58,
  222,63,223,7,211,52,209,108,52,49,61,53,163,220,200,84,99,132,211,205,235,54,125,29,251,169,234,79,85,217,107,157,142,247,95,186,221,92,80,164,
  122,183,121,51,145,85,207,147,23,72,203,76,126,167,71,249,89,62,187,58,47,226,188,78,27,28,120,150,229,38,121,181,160,28,195,205,201,12,242,4,
  186,186,207,202,9,158,42,159,93,250,249,172,160,69,21,168,168,64,171,170,127,69,95,233,170,244,209,182,187,174,155,89,90,188,147,160,133,179,212,203,
  202,27,160,224,85,55,78,117,62,240,121,215,95,121,92,115,160,93,182,121,203,195,68,231,13,104,57,11,52,110,252,203,222,212,121,247,242,228,248,13,
  129,138,0,155,42,96,21,39,187,209,49,109,100,158,168,80,137,134,96,152,105,127,37,231,160,17,233,96,77,19,169,234,133,28,233,33,75,64,40,24,
  166,146,13,170,29,150,157,198,146,76,174,122,205,80,159,248,196,223,245,253,88,90,34,216,233,78,3,122,1,98,227,231,78,187,103,132,247,9,18,76,
  113,156,244,152,148,125,24,134,248,111,35,98,174,195,132,68,147,241,132,54,218,172,183,159,178,68,76,2,105,128,202,88,116,64,154,239,63,144,254,10,
  82,190,220,156,172,67,120,132,115,253,249,73,99,71,78,10,102,61,246,232,163,255,47,241,161,179,239,191,159,235,40,88,119,35,57,161,164,147,204,101,
  46,43,53,11,68,114,255,214,233,89,235,245,101,60,113,236,51,232,234,234,194,194,220,28,60,223,131,1,3,123,119,239,195,23,191,252,37,172,93,191,
  14,61,61,93,176,109,3,125,171,250,16,248,30,26,13,7,78,211,193,253,251,19,24,232,223,136,161,161,181,184,112,225,60,220,150,139,125,15,238,197,
  157,187,83,24,24,232,197,214,7,54,96,231,238,157,120,245,215,111,163,92,44,34,128,143,187,119,239,104,153,120,206,218,74,215,199,89,246,121,28,72,
  107,181,28,220,189,59,131,153,217,187,152,155,155,65,16,4,112,154,53,92,189,54,134,130,93,193,240,182,33,188,252,242,79,176,188,60,31,13,210,229,
  229,165,104,17,5,128,201,201,123,48,12,3,142,211,192,141,155,55,83,145,55,183,89,168,50,219,185,197,93,215,7,220,241,178,12,88,84,213,236,40,
  211,161,219,76,255,37,218,173,122,207,242,60,200,83,213,75,30,15,220,198,200,45,20,28,112,225,230,85,39,204,106,94,125,175,142,165,204,195,128,170,
  164,20,121,55,38,149,14,78,6,212,58,0,209,73,54,184,106,227,231,147,152,242,247,185,235,186,172,206,85,126,142,60,222,215,58,205,191,188,121,234,
  216,222,172,49,154,167,20,184,236,107,159,101,107,151,117,31,157,255,182,10,188,210,189,68,181,86,228,117,3,82,173,41,186,224,72,37,139,144,223,137,
  28,68,229,29,59,234,146,225,70,202,29,67,102,130,227,82,218,70,102,17,146,164,243,69,82,91,44,52,179,226,58,50,219,44,59,62,196,255,152,137,
  223,199,0,71,4,72,212,237,130,2,101,48,206,24,113,242,27,117,214,224,250,64,254,71,102,61,229,127,132,156,195,178,204,72,118,33,238,43,164,22,
  170,107,103,221,59,89,117,22,209,61,146,0,154,222,3,9,247,144,228,223,205,84,155,40,184,149,215,216,100,128,97,32,219,67,91,159,179,150,196,66,
  166,228,110,2,237,105,0,231,28,242,113,176,163,108,145,73,175,53,114,224,64,172,129,142,58,67,241,224,121,51,19,179,54,145,44,189,86,22,51,164,
  3,241,50,115,39,179,56,52,114,152,157,157,199,63,254,227,15,48,50,50,130,47,60,249,69,216,5,19,129,23,32,48,128,55,223,122,31,205,230,34,
  170,61,221,48,12,27,174,231,97,102,102,17,39,78,157,129,231,133,62,202,239,127,112,6,45,111,63,0,3,51,115,83,120,233,165,159,96,106,102,14,
  3,107,250,49,48,208,143,59,119,79,98,242,254,56,110,92,191,12,219,46,42,159,79,48,62,116,145,166,140,84,39,90,202,60,253,57,51,115,39,113,
  244,237,251,62,140,160,142,235,215,47,97,228,224,46,84,202,37,212,27,181,68,130,150,122,224,249,224,52,93,130,1,209,89,197,100,201,23,116,44,177,
  186,90,18,114,182,27,202,83,142,127,169,118,231,9,152,184,226,10,157,130,216,172,239,203,99,175,83,185,67,39,199,245,242,156,237,164,76,180,14,108,
  203,9,89,89,96,41,107,140,232,42,203,233,214,54,170,95,229,116,119,242,251,84,93,95,165,243,150,93,129,40,123,172,75,230,206,147,228,151,197,172,
  169,60,154,179,228,27,42,70,82,158,15,156,101,94,86,145,9,58,126,116,99,142,158,80,100,121,151,211,247,79,109,30,229,36,189,172,96,73,37,231,
  202,90,219,229,61,76,72,108,116,243,43,43,232,82,251,221,11,134,84,214,2,203,39,42,31,167,234,159,186,132,114,220,151,166,22,36,233,231,107,144,
  2,126,233,4,65,238,111,6,124,63,125,26,196,5,4,241,30,157,188,118,150,83,132,104,159,74,86,195,57,126,168,130,23,46,25,80,128,77,161,199,
  230,117,251,212,65,131,222,87,158,47,84,154,18,51,219,220,137,91,172,243,214,75,59,68,98,105,150,62,154,39,49,160,148,135,8,6,253,247,133,29,
  101,105,36,235,75,254,95,254,250,175,163,171,254,207,191,251,59,237,70,167,91,28,243,38,185,169,178,123,229,123,208,138,82,42,175,198,172,68,44,174,
  154,151,252,247,98,169,11,5,187,140,90,125,9,129,239,40,89,162,74,185,130,98,161,136,217,249,217,142,172,133,116,153,160,157,28,59,114,81,26,215,
  7,170,194,33,170,13,194,243,60,116,119,175,194,242,242,92,116,220,145,199,109,64,181,208,211,202,135,170,236,243,172,164,82,21,112,224,158,39,95,53,
  172,63,220,118,119,122,156,43,91,213,125,156,235,169,54,124,58,166,86,26,196,201,243,174,147,18,227,186,249,156,197,78,103,93,75,238,183,143,83,196,
  70,119,31,213,241,253,199,57,206,207,218,36,116,249,40,191,143,103,204,51,55,178,170,198,230,41,103,173,2,213,92,126,64,158,53,83,197,252,102,121,
  112,171,130,100,93,73,245,78,157,104,116,5,107,116,214,118,121,172,56,249,83,172,16,84,9,87,141,116,91,185,156,17,125,159,233,236,224,210,243,144,
  99,75,213,96,50,25,40,83,77,115,210,57,130,202,1,40,112,211,145,23,242,56,206,243,28,73,160,101,176,214,124,170,74,198,89,115,58,15,9,161,
  115,216,136,199,70,144,74,244,164,109,150,1,126,92,244,36,201,28,39,253,187,179,42,32,202,223,85,229,62,32,71,253,17,158,201,78,222,219,32,99,
  249,147,195,142,156,153,197,95,126,251,219,97,41,111,196,38,124,209,23,57,227,116,106,242,159,199,20,190,19,118,74,183,248,171,142,151,184,78,209,1,
  36,89,87,69,23,160,150,83,71,173,54,11,207,109,36,238,39,15,244,93,187,246,224,79,255,244,79,97,89,197,142,192,51,77,148,224,10,45,112,71,
  8,157,176,141,170,231,166,78,17,148,233,161,125,33,218,210,104,44,166,244,123,121,142,96,85,90,99,157,255,107,94,48,169,26,107,170,106,107,159,214,
  118,119,242,126,185,170,156,121,192,17,29,127,89,32,151,22,217,249,36,10,189,100,233,207,185,2,68,42,64,69,159,195,117,221,212,51,113,186,113,174,
  159,168,4,97,37,207,153,167,205,89,158,222,121,217,217,60,107,169,238,217,242,156,6,172,100,157,81,245,3,247,29,241,60,162,152,142,170,112,142,60,
  110,100,9,19,5,34,114,245,65,122,186,166,179,48,204,147,75,32,183,67,252,67,215,111,234,7,174,3,236,43,145,48,234,138,166,232,242,10,242,37,
  185,26,145,198,57,102,101,211,137,185,50,187,71,139,136,164,25,75,78,10,197,127,150,106,103,99,125,117,146,177,76,158,120,24,82,1,35,51,33,59,
  160,14,22,201,98,29,6,41,242,18,255,183,220,86,218,102,177,246,197,5,74,192,22,122,17,207,66,139,182,216,182,157,144,127,168,36,124,170,253,130,
  27,51,49,107,107,40,139,191,208,190,86,21,87,73,74,99,226,164,207,240,154,6,145,103,36,147,64,211,253,25,235,214,117,167,174,180,77,226,154,234,
  83,191,236,32,159,99,232,213,107,108,16,21,155,81,97,71,110,125,82,173,3,114,210,190,209,126,48,241,9,51,235,72,147,190,60,221,162,153,23,20,
  36,45,185,108,172,234,91,13,203,176,82,29,162,3,202,157,106,217,84,172,35,183,80,170,126,22,151,150,208,93,237,70,127,255,218,142,192,3,103,91,
  166,11,30,58,201,220,206,187,33,211,133,158,190,63,106,186,223,221,213,131,125,123,118,102,178,147,157,108,250,42,176,202,129,30,249,94,186,119,145,87,
  210,240,105,108,55,7,204,228,49,163,114,104,201,179,57,171,250,64,104,50,233,117,229,36,60,185,218,161,202,43,88,174,128,39,143,125,57,145,46,207,
  218,33,87,146,203,42,240,161,210,142,210,64,86,174,252,153,23,100,202,65,168,238,93,175,148,96,80,173,21,42,173,186,28,56,211,207,229,33,61,84,
  85,20,185,107,228,169,208,167,146,173,89,150,149,72,158,150,43,176,114,30,173,114,128,160,10,116,233,123,236,244,148,33,207,187,23,0,139,86,49,229,
  170,244,233,252,200,185,121,153,167,200,12,23,60,211,231,214,85,13,229,24,101,14,228,166,117,195,122,71,13,225,138,64,181,205,113,50,25,23,192,0,
  177,239,177,7,81,244,131,123,79,234,10,132,52,113,49,96,65,115,12,168,147,178,135,248,249,146,140,189,12,152,211,140,187,12,96,77,9,248,6,16,
  18,146,180,198,88,255,79,120,205,184,122,97,28,16,154,176,172,216,37,132,155,19,92,101,83,249,221,82,59,189,36,73,194,149,138,151,245,223,70,4,
  130,121,45,117,118,14,72,252,30,12,101,165,85,209,111,58,220,67,147,71,101,175,233,36,200,206,198,142,89,167,61,153,216,145,4,183,54,37,197,101,
  16,160,114,36,208,29,101,171,142,228,100,123,16,211,52,177,97,253,58,124,229,171,95,65,179,209,68,203,105,193,113,93,180,156,38,26,245,6,150,235,
  203,88,90,152,131,15,3,129,31,192,50,45,248,1,224,56,46,60,215,133,227,120,112,93,7,174,215,108,63,148,31,234,150,125,23,190,215,66,173,214,
  68,173,190,164,92,36,229,146,178,58,15,97,0,184,51,49,129,70,205,65,119,119,21,147,147,157,251,50,203,137,96,186,99,93,93,38,60,23,237,169,
  130,31,46,202,82,29,43,22,138,21,60,246,216,103,113,245,234,117,180,188,86,234,253,234,0,65,94,155,47,213,113,153,188,137,228,245,137,205,195,188,
  126,26,219,205,205,27,10,244,242,84,184,203,35,39,224,54,42,89,175,173,139,226,85,207,41,3,28,93,69,61,221,49,56,7,216,184,19,155,60,9,
  93,170,64,72,165,21,207,115,20,254,73,73,35,178,64,52,29,123,58,89,0,101,210,100,153,143,92,124,70,247,140,42,16,45,131,94,157,118,81,231,
  167,74,223,185,92,66,158,174,205,170,35,215,44,43,188,60,210,35,149,229,28,109,135,156,99,193,245,141,248,140,236,219,172,155,235,242,241,112,150,12,
  36,107,124,203,128,129,223,155,232,187,146,175,99,176,96,38,107,124,198,174,71,200,36,128,132,190,57,246,85,182,148,251,84,186,50,158,161,188,79,187,
  152,178,230,4,193,138,108,220,196,243,133,236,107,160,180,166,147,65,103,26,164,5,17,8,141,223,85,160,100,87,179,2,55,195,72,87,250,75,86,245,
  51,82,44,107,188,166,171,79,178,61,207,79,128,122,33,47,73,151,217,54,36,215,11,170,16,48,218,182,120,242,154,96,16,13,117,236,210,17,143,59,
  154,184,41,218,101,69,9,157,156,173,93,12,214,211,5,151,98,63,112,35,113,15,170,193,78,202,78,120,80,174,203,165,200,218,247,57,236,104,211,33,
  202,101,126,170,54,160,60,139,112,86,181,175,102,195,129,101,89,56,243,254,21,52,155,77,20,108,27,133,130,141,74,87,5,165,98,15,54,110,30,68,
  193,182,225,186,205,112,208,4,64,193,42,160,88,44,193,48,45,0,30,108,203,106,103,158,154,176,76,11,65,251,127,205,166,131,31,254,240,159,176,188,
  60,207,2,120,14,104,105,171,115,153,22,252,192,71,171,213,232,104,227,228,152,110,157,53,82,167,0,75,215,223,242,38,162,99,153,102,231,166,209,104,
  54,81,237,237,197,204,236,116,234,58,178,70,182,211,18,200,42,22,157,3,66,42,59,58,213,70,172,234,183,79,67,187,197,134,65,53,142,156,117,149,
  202,162,80,197,80,113,127,83,129,84,121,92,228,73,196,84,21,160,81,185,157,200,193,228,199,97,0,84,1,128,174,175,179,250,40,43,55,35,139,209,
  207,115,108,191,82,25,133,10,180,233,214,89,213,51,168,44,19,117,193,74,86,31,168,42,104,114,37,229,85,69,123,56,208,171,146,197,232,60,198,243,
  50,238,52,8,200,114,48,201,83,221,147,219,63,184,57,75,79,63,196,247,56,73,11,13,244,116,115,42,189,158,7,140,245,90,160,88,63,86,158,20,
  168,211,54,171,216,110,177,15,115,167,78,178,43,70,184,30,10,105,157,33,5,53,233,68,64,85,240,41,192,27,103,71,26,203,18,56,13,108,12,14,
  249,177,47,191,127,67,171,245,87,21,82,225,170,6,234,246,245,88,90,35,187,132,4,9,77,116,204,172,167,115,102,146,129,176,222,202,85,126,23,113,
  49,155,36,144,14,75,167,91,82,255,25,210,188,53,72,251,85,123,76,158,2,44,1,107,137,23,63,191,62,79,175,83,135,57,49,102,185,182,216,170,
  151,69,25,134,78,172,161,84,153,149,220,132,41,22,11,240,125,96,236,250,24,102,167,38,216,69,216,52,203,8,2,135,68,64,22,12,163,24,205,79,
  219,178,96,219,37,248,65,19,166,105,193,52,76,24,166,129,145,131,7,208,183,170,47,2,208,92,185,102,213,143,156,241,236,251,62,10,197,2,76,211,
  68,173,214,64,171,213,74,44,186,98,65,207,42,158,161,98,119,116,62,216,43,9,78,178,64,96,108,94,111,146,137,20,0,134,133,158,158,30,76,77,
  79,102,250,31,235,158,83,197,52,202,218,247,78,45,199,242,120,143,127,154,218,45,111,192,116,147,160,155,44,245,197,229,126,47,183,49,11,136,232,64,
  232,74,198,85,158,10,118,121,222,93,86,130,50,247,126,117,39,57,233,18,181,62,251,125,213,137,142,46,1,46,171,205,186,220,142,143,147,84,38,187,
  113,228,1,237,242,90,37,191,107,10,254,117,18,49,122,130,70,217,112,89,138,32,255,62,79,98,30,151,16,167,98,181,179,200,134,78,19,106,59,89,
  79,85,235,188,106,188,200,78,46,156,27,75,94,191,114,149,187,15,255,206,100,54,16,136,29,22,100,171,203,236,123,199,64,44,143,195,145,161,4,217,
  42,134,94,102,155,169,75,149,40,220,146,244,112,206,182,2,77,87,3,76,130,105,46,49,50,214,103,243,129,69,26,180,25,164,191,211,9,144,89,140,
  190,144,71,4,129,17,249,69,199,107,61,144,46,28,19,131,83,217,253,34,185,230,196,129,148,78,251,79,129,174,106,207,163,210,67,241,188,60,144,142,
  147,13,101,18,72,46,5,47,187,117,36,131,248,56,185,84,149,216,106,154,178,175,116,192,18,4,244,217,116,213,100,117,167,210,186,132,67,155,219,72,
  116,108,10,103,66,175,203,104,214,185,80,116,85,186,208,114,61,44,45,46,96,245,170,53,56,116,104,4,171,86,175,198,59,239,156,196,196,196,173,246,
  253,26,8,130,0,221,93,21,212,235,45,120,190,3,203,242,162,62,107,249,64,171,181,148,104,95,16,4,152,159,155,197,200,193,67,152,24,191,149,136,
  206,184,99,60,185,99,185,5,173,84,172,32,240,125,24,166,137,66,161,160,237,112,21,11,173,58,134,228,244,162,157,178,82,31,167,152,129,216,136,91,
  45,7,48,192,122,158,234,152,220,188,150,113,66,159,41,203,103,116,62,181,121,221,13,228,227,203,79,67,187,185,164,6,89,134,145,199,70,82,78,4,
  204,147,237,45,111,248,42,251,30,206,2,141,243,169,214,177,151,58,198,57,143,67,133,12,114,213,229,134,147,250,87,217,43,53,43,176,144,215,191,44,
  118,55,111,49,41,25,184,230,113,76,224,216,81,213,9,139,74,170,36,235,133,243,0,62,85,130,179,60,198,69,255,203,146,31,110,47,225,250,61,207,
  250,165,146,28,169,78,63,58,13,250,228,113,155,148,15,24,29,157,226,232,78,67,56,153,130,220,39,98,13,214,73,115,210,255,166,71,235,73,87,16,
  14,212,202,160,80,30,239,92,176,18,254,219,87,86,255,75,50,163,60,59,203,173,131,42,224,44,91,207,137,126,162,221,165,247,222,167,115,40,80,106,
  117,5,35,26,203,21,232,216,230,78,16,76,137,37,13,164,207,154,137,107,132,214,111,130,41,85,227,35,234,135,29,2,126,147,200,108,226,132,184,24,
  244,11,204,101,165,64,117,12,198,125,194,24,39,181,204,49,104,143,215,110,209,214,248,153,211,249,23,84,231,78,129,52,149,105,8,16,31,23,212,49,
  52,39,98,32,125,19,40,78,204,226,19,0,249,148,37,105,111,199,141,111,17,108,251,100,45,214,147,75,170,34,78,42,204,100,89,86,146,129,166,140,
  70,222,35,200,44,127,103,149,172,192,52,77,244,84,123,208,114,90,216,189,107,23,30,63,250,16,186,171,85,0,192,250,245,235,113,249,242,21,156,56,
  241,46,186,42,5,60,249,228,147,88,179,122,21,28,167,133,95,252,234,215,24,31,191,14,0,88,59,176,14,15,63,116,16,45,215,195,111,126,247,6,
  60,207,137,6,218,194,194,60,74,165,18,0,11,128,155,90,212,59,102,110,109,27,48,2,120,210,113,155,12,212,104,73,72,1,184,84,165,167,57,118,
  154,147,154,100,149,75,87,57,44,100,29,95,114,131,173,84,44,42,55,152,149,176,59,244,126,89,236,164,12,200,84,54,104,89,71,213,159,150,118,203,
  243,39,79,226,25,167,15,83,37,22,201,71,221,252,49,166,122,222,114,201,81,92,96,32,219,79,234,170,102,202,39,54,157,248,6,115,14,51,89,128,
  67,215,183,42,77,121,158,35,123,14,24,233,2,3,157,244,64,21,208,230,101,67,197,179,209,114,220,121,2,114,85,128,150,39,208,225,254,63,61,221,
  202,202,225,144,45,213,100,64,159,85,1,82,119,173,60,196,3,221,40,85,65,128,92,232,138,155,63,89,185,51,220,189,229,177,72,239,65,143,139,57,
  134,95,6,22,50,248,164,14,5,28,112,163,77,204,178,198,227,170,240,165,159,83,4,167,102,174,224,56,153,112,104,72,128,84,232,99,205,20,43,204,
  5,103,60,131,77,115,1,12,45,203,29,191,147,128,48,155,50,240,75,130,106,10,216,40,208,227,189,167,117,69,118,16,225,132,216,109,196,76,104,122,
  99,64,106,166,254,70,25,224,152,21,54,165,202,142,1,169,246,72,223,103,18,128,82,137,76,82,126,97,164,44,11,105,160,66,3,143,176,220,59,125,
  175,102,98,60,82,215,28,202,70,83,73,71,90,34,148,116,35,81,5,183,178,87,52,71,64,81,233,143,42,112,213,237,243,148,136,161,107,146,45,119,
  76,167,64,35,175,222,143,251,156,235,121,168,246,116,225,201,47,28,199,249,139,163,184,117,227,36,234,141,6,0,31,219,183,111,193,55,190,241,85,44,
  206,47,98,253,134,245,120,245,213,83,240,131,6,74,229,10,124,223,199,195,7,15,227,51,199,142,98,122,190,129,251,247,238,161,171,171,15,115,115,119,
  163,5,106,126,97,62,33,126,87,21,137,200,42,117,28,189,116,223,69,161,88,68,95,111,47,22,22,38,217,14,214,109,142,2,24,103,45,178,42,29,
  173,10,120,119,154,8,199,49,83,98,227,107,57,45,216,18,227,69,129,17,199,26,101,129,62,186,232,113,160,93,21,24,168,250,82,46,248,161,43,178,
  240,135,222,238,78,228,21,170,82,229,226,119,178,95,165,10,60,171,202,17,171,230,46,199,138,115,204,28,119,202,162,3,106,242,98,72,25,87,1,4,
  197,191,85,32,89,87,82,61,143,206,154,147,51,173,164,108,57,87,184,69,37,243,80,233,133,185,241,220,169,147,7,101,164,85,39,11,84,123,75,251,
  49,171,74,33,5,117,220,248,163,73,174,220,61,85,82,18,29,49,32,143,45,153,13,207,83,142,59,15,83,172,74,10,214,89,95,202,243,35,23,9,
  147,241,57,57,128,16,224,94,39,241,224,219,101,176,69,52,116,57,50,58,125,179,105,90,18,1,21,36,152,77,154,156,166,2,221,73,0,39,3,90,
  26,36,168,37,26,60,3,15,45,243,78,253,160,227,202,137,244,239,38,217,87,197,191,227,126,164,122,90,190,68,56,36,96,150,55,143,41,72,244,45,
  215,247,161,3,71,204,48,39,231,149,33,1,94,180,101,27,105,150,218,48,100,199,40,10,140,147,114,137,112,174,134,215,178,44,35,98,110,185,181,93,
  4,15,49,200,231,0,173,250,116,35,45,233,80,213,196,224,147,11,229,191,83,38,58,185,198,38,173,27,69,128,70,247,54,185,56,85,214,105,63,12,
  3,1,0,155,179,39,82,105,3,85,21,6,117,236,151,42,106,7,0,207,15,96,90,38,238,79,205,227,205,55,223,65,179,30,3,211,219,183,174,225,
  240,35,143,96,223,254,67,184,119,111,18,195,195,235,240,227,31,255,8,128,143,93,219,119,227,115,79,30,199,187,103,63,196,91,175,255,14,158,183,152,
  122,120,199,105,161,80,40,193,46,148,225,182,150,115,235,60,185,12,251,129,213,3,120,242,115,199,97,219,22,190,244,165,207,227,71,63,90,194,221,123,
  19,74,48,193,201,64,100,103,130,78,22,125,93,214,127,167,69,26,84,12,162,97,24,152,95,88,68,119,119,143,146,177,161,122,190,149,4,90,50,232,
  225,88,45,26,104,116,194,76,170,156,3,254,208,219,221,9,56,202,251,119,21,136,214,1,241,172,77,159,158,142,112,44,168,216,188,40,144,231,124,87,
  57,61,178,12,222,108,219,78,49,232,42,207,111,46,49,68,21,44,168,158,143,122,238,234,88,101,110,61,147,215,186,172,4,235,172,181,39,15,67,173,
  43,238,33,128,42,167,185,85,173,219,58,169,135,234,30,220,218,71,89,44,209,14,10,170,233,90,147,39,55,128,123,183,220,251,161,247,85,149,188,166,
  223,167,5,143,228,126,231,130,102,78,107,156,119,140,169,2,57,206,138,146,75,88,12,193,105,58,201,55,60,105,72,3,57,122,180,207,177,178,114,80,
  173,6,133,156,44,34,201,236,234,228,19,28,224,148,1,118,252,121,147,61,126,231,203,89,115,247,48,18,140,44,199,52,135,96,144,234,107,169,166,59,
  72,48,207,97,59,147,73,115,170,228,72,158,225,79,235,111,61,207,133,101,217,109,121,133,145,0,118,252,181,3,226,128,193,5,155,201,254,241,60,17,
  112,5,137,182,199,222,206,38,1,238,1,97,118,77,137,105,231,240,90,252,254,92,215,131,105,26,17,187,29,207,109,175,237,120,34,250,140,58,37,139,
  224,68,157,188,76,37,32,188,68,179,115,16,157,100,251,205,68,224,34,51,201,186,245,79,231,35,111,155,166,9,241,184,148,77,208,149,122,149,25,52,
  85,185,67,185,144,71,156,141,26,254,183,211,8,45,232,110,222,186,135,102,125,154,93,124,156,134,135,151,254,233,101,60,243,199,79,226,249,231,159,199,
  15,127,248,35,28,62,122,24,87,71,239,224,141,223,190,2,192,97,53,151,45,167,5,19,30,182,62,48,140,27,55,174,192,182,76,24,8,55,122,219,
  178,81,40,22,194,133,32,48,96,218,38,108,211,132,23,132,199,161,134,233,163,104,151,0,4,40,20,203,120,252,209,71,81,93,21,202,75,170,189,85,
  124,229,43,95,193,223,255,195,139,104,52,22,114,109,58,157,86,201,83,1,65,149,54,147,10,245,179,10,56,112,199,233,226,103,118,118,22,171,87,247,
  40,217,25,249,24,67,197,46,173,196,81,68,62,254,207,91,157,140,2,87,85,89,231,63,212,118,211,137,171,59,118,207,195,166,170,34,104,157,131,67,
  158,177,153,245,108,50,11,152,76,56,9,50,217,69,249,186,28,72,215,37,14,202,253,73,189,205,243,6,151,157,178,206,121,2,28,157,158,55,203,227,
  61,11,132,231,209,225,170,88,238,60,172,173,174,58,160,234,20,45,89,226,215,204,148,137,232,78,14,184,121,41,207,37,29,155,45,127,79,176,227,52,
  48,227,78,228,132,175,121,167,197,86,56,2,41,239,88,80,125,46,121,255,36,208,74,254,91,118,156,72,191,19,249,154,201,96,59,79,144,100,72,108,
  110,218,37,72,205,210,38,217,103,78,243,172,43,148,166,147,104,196,207,22,40,174,149,126,158,152,77,23,76,172,145,10,48,40,8,230,192,109,200,12,
  123,16,197,72,98,217,137,161,100,155,133,67,133,240,125,142,101,56,102,36,129,224,116,205,234,4,63,128,86,223,19,126,203,194,186,46,124,69,66,71,
  109,16,208,76,45,242,130,132,93,29,13,0,100,157,116,82,243,109,164,172,247,226,164,79,175,205,70,39,61,155,227,96,196,80,204,251,164,46,58,8,
  60,141,110,58,31,136,78,234,179,213,228,65,150,33,129,110,239,176,3,82,24,157,70,230,92,89,203,172,40,91,246,162,149,125,71,233,96,23,81,89,
  8,118,61,248,126,82,167,188,97,237,6,28,28,25,193,165,43,183,80,237,233,198,15,127,248,18,190,245,111,190,133,207,126,246,115,176,237,34,76,44,
  192,48,90,81,157,246,20,128,118,155,48,45,224,139,79,31,195,226,210,33,180,28,23,65,96,194,247,1,211,48,224,121,14,252,32,128,231,251,66,129,
  131,192,15,224,7,1,76,195,135,109,151,96,23,138,240,188,6,2,233,88,170,218,219,141,141,27,215,227,163,143,230,58,46,95,158,181,233,173,100,243,
  86,129,103,221,241,60,215,238,185,185,121,172,27,92,149,153,0,150,215,202,112,37,63,42,89,141,170,63,85,199,144,159,134,118,171,10,145,116,242,67,
  35,105,157,59,128,12,50,117,58,95,85,50,81,222,224,48,207,56,164,27,177,142,49,207,114,93,144,143,247,179,124,191,85,5,62,116,115,74,0,58,
  46,201,80,197,144,118,194,34,119,10,214,116,126,224,157,72,36,56,134,154,59,65,113,93,55,151,212,72,5,122,57,151,33,142,9,85,39,123,153,108,
  174,137,60,198,56,198,154,174,119,89,65,12,5,2,157,172,189,89,121,18,226,251,162,31,213,243,34,72,129,48,33,45,72,187,208,164,165,10,186,87,
  36,23,209,200,30,111,6,1,81,233,96,81,117,130,144,149,20,24,50,164,73,251,56,249,29,233,216,102,249,36,67,88,163,201,50,2,225,44,69,253,
  131,133,155,71,242,250,73,150,53,126,255,65,10,132,199,122,223,32,2,187,226,250,177,244,36,96,222,143,25,5,16,2,160,11,240,30,22,148,17,37,
  192,125,194,148,26,9,0,154,214,180,251,109,139,185,80,182,225,251,116,14,6,140,126,220,72,48,238,161,83,6,101,107,105,185,110,67,9,164,13,195,
  34,73,132,113,96,231,121,62,226,100,192,216,163,90,184,101,196,128,63,150,140,208,181,93,0,239,56,32,176,192,39,23,166,65,180,236,166,162,7,209,
  241,216,227,10,187,112,39,31,220,250,19,237,29,42,61,155,216,112,243,104,3,105,134,178,188,144,234,89,155,54,123,228,214,162,251,150,75,125,216,57,
  188,9,159,251,194,49,140,221,90,192,229,75,151,240,149,231,158,198,169,119,223,195,219,111,190,131,47,126,233,143,112,254,252,135,56,118,236,33,60,241,
  196,49,92,251,232,67,236,222,190,7,87,175,223,192,173,91,163,81,91,108,171,136,150,3,188,242,234,219,152,157,187,143,122,221,133,231,185,112,91,45,
  248,190,7,223,119,96,24,118,123,0,139,104,178,4,223,175,183,219,93,132,101,149,224,186,203,40,22,75,248,234,87,255,4,67,67,27,16,4,1,154,
  13,23,75,139,205,142,18,125,242,48,80,178,115,9,39,221,80,233,53,179,228,29,89,236,148,239,251,152,158,157,70,161,176,43,147,61,201,218,92,178,
  172,191,84,155,77,22,179,246,113,124,131,255,208,219,189,18,118,148,6,188,186,18,191,170,128,35,207,15,149,55,228,177,105,92,9,227,170,90,75,116,
  239,66,150,133,228,77,150,83,185,241,100,245,141,234,116,227,227,48,218,42,0,182,146,177,171,2,113,156,107,82,26,160,153,112,93,87,121,26,35,39,
  38,174,100,254,113,250,91,177,129,201,210,142,172,160,78,21,232,113,193,140,78,195,47,143,5,185,128,75,39,167,80,52,25,73,229,217,78,75,130,167,
  143,227,205,20,136,78,3,62,93,82,160,145,121,50,194,149,68,78,235,141,13,73,186,160,158,127,121,10,158,200,73,129,161,252,195,72,232,178,229,68,
  208,172,194,44,33,24,141,129,107,168,215,53,19,140,185,184,151,0,171,73,198,145,190,127,147,237,79,10,230,100,73,128,109,211,164,59,63,170,226,71,
  229,23,148,29,166,125,35,48,79,236,50,98,19,204,101,72,204,39,103,255,41,222,65,236,145,13,24,8,135,110,144,114,20,137,191,27,164,152,107,81,
  65,49,102,194,3,162,31,6,146,122,98,85,114,185,217,118,183,48,19,123,82,140,97,4,104,230,44,2,141,84,80,70,159,157,38,23,234,64,116,126,
  121,102,32,157,138,152,137,61,64,149,11,161,91,135,108,238,184,86,69,107,171,54,215,172,35,96,213,205,187,202,101,32,8,112,240,192,78,148,139,6,
  74,69,11,67,67,91,208,213,221,131,139,87,175,227,173,55,222,64,203,153,199,43,175,190,138,63,250,163,47,195,50,45,212,27,1,206,159,191,132,233,
  233,113,124,254,243,159,197,200,200,8,22,23,107,184,50,58,145,88,168,74,165,18,44,171,128,59,119,238,98,113,241,142,130,33,107,73,32,178,70,62,
  211,68,171,213,0,0,52,155,46,126,246,179,95,226,47,255,242,47,48,62,49,133,19,239,156,194,189,251,183,181,27,87,222,164,65,221,241,170,206,197,
  164,147,5,62,107,32,136,77,108,121,113,1,150,89,64,193,46,161,229,54,51,153,214,78,171,248,117,146,41,159,247,216,51,143,12,225,15,181,221,92,
  34,157,124,148,175,3,84,178,199,44,39,127,224,54,63,149,86,148,27,27,212,225,161,147,62,203,27,92,102,173,31,156,102,155,203,205,200,3,240,84,
  57,30,121,130,10,153,221,212,85,5,204,42,122,179,82,48,170,98,65,40,225,161,146,191,8,144,71,181,234,156,12,44,47,0,238,244,71,119,34,164,
  27,55,89,193,69,222,32,148,243,81,230,222,49,39,31,200,19,32,210,113,160,178,41,76,87,110,148,129,111,160,76,10,228,65,179,190,31,116,250,102,
  78,219,44,131,225,100,1,14,35,53,78,232,103,195,241,4,2,148,45,196,21,234,146,197,93,66,134,210,146,78,159,140,40,152,16,199,248,188,94,58,
  153,104,40,131,102,25,96,133,65,77,186,47,67,9,6,96,24,62,9,78,210,46,16,233,224,32,78,68,244,125,161,243,247,34,89,71,204,188,38,221,
  60,130,32,78,206,163,82,13,161,201,14,193,190,153,8,170,104,155,69,123,228,90,1,49,200,5,226,50,233,126,130,9,143,255,134,132,87,115,50,40,
  11,218,44,181,248,125,64,44,238,226,106,132,158,23,182,45,124,246,88,71,46,148,0,114,0,41,174,43,44,249,196,137,129,24,47,66,139,158,46,240,
  20,95,83,7,162,57,50,134,99,161,105,145,25,221,90,172,3,209,84,254,41,198,150,45,47,194,186,13,65,117,132,150,135,157,19,58,52,170,175,174,
  246,174,193,253,169,5,124,244,225,53,236,221,183,3,78,203,195,245,91,247,241,209,213,211,184,123,119,20,174,27,250,59,223,186,117,19,47,188,240,191,
  48,188,125,15,150,23,61,52,26,247,49,58,122,15,163,99,55,176,106,213,26,212,150,235,112,156,165,196,228,238,234,234,129,15,31,203,181,89,165,255,
  114,39,236,213,210,210,44,230,231,23,113,239,238,36,238,220,185,150,131,93,183,82,71,193,89,218,193,60,192,75,165,95,206,250,187,78,183,22,31,227,
  26,128,17,86,118,108,185,250,205,80,215,62,85,191,170,146,23,179,28,9,178,244,171,121,19,3,255,208,218,205,157,82,200,50,134,60,165,169,59,9,
  208,84,224,86,85,208,103,165,37,171,117,165,197,117,73,201,170,121,164,242,39,150,153,92,93,16,65,3,19,78,75,171,10,82,228,68,107,186,158,228,
  13,68,184,103,150,175,171,42,4,148,231,218,89,158,210,50,49,34,159,54,174,196,230,81,184,164,100,249,90,203,44,17,87,100,133,243,157,205,19,24,
  231,157,247,220,88,212,37,129,231,29,39,148,189,166,114,31,17,160,208,231,78,95,195,72,37,149,37,231,143,213,49,112,142,239,103,228,98,155,133,235,
  4,39,201,72,159,184,80,176,69,143,200,3,216,182,149,178,166,75,147,1,70,91,159,235,39,156,47,194,255,54,217,147,18,57,200,72,2,103,202,212,
  83,119,16,83,146,180,24,9,86,56,8,124,194,4,155,41,240,173,115,210,136,159,203,130,97,120,109,121,141,77,88,94,149,63,189,40,110,98,51,251,
  46,34,141,116,114,124,250,41,38,59,201,204,154,82,208,32,88,101,179,13,82,99,150,92,128,221,216,161,67,176,196,34,33,80,204,15,175,45,239,8,
  218,53,50,2,137,145,246,225,251,128,109,91,18,123,107,164,2,47,234,117,29,59,172,4,18,155,30,40,29,58,4,216,22,5,102,210,69,204,128,88,
  135,46,247,119,146,209,151,251,41,94,179,76,86,138,167,42,176,34,183,211,206,179,17,115,199,159,178,204,64,103,4,159,62,154,13,59,207,46,2,45,
  215,199,169,119,79,224,212,187,111,180,9,241,0,128,155,186,159,227,52,113,241,194,153,196,66,21,248,13,204,207,221,75,208,243,226,59,182,13,184,110,
  11,129,223,82,30,249,201,236,159,44,26,151,51,236,103,167,167,208,219,219,149,250,140,170,20,109,94,160,147,54,151,55,58,102,247,116,101,123,233,103,
  168,207,168,172,171,11,130,22,28,167,129,82,169,136,70,179,174,125,22,249,218,212,191,154,75,4,210,129,50,26,84,233,180,195,28,200,235,4,72,252,
  161,181,155,3,167,121,60,124,85,126,203,178,149,29,125,230,44,32,162,98,189,243,22,68,145,77,232,229,205,48,11,188,116,42,9,208,245,181,96,203,
  5,144,145,147,76,101,151,8,46,104,161,125,38,254,158,183,172,115,158,246,210,246,169,174,151,215,173,66,172,187,89,186,123,250,158,196,122,243,113,0,
  106,167,146,42,157,172,71,149,28,157,55,64,208,237,59,84,42,162,74,142,87,157,204,232,10,209,232,78,77,242,174,231,92,149,53,1,208,58,101,156,
  99,121,67,122,205,74,254,94,157,192,167,102,179,233,119,141,8,116,39,247,244,152,45,84,237,181,73,29,118,92,160,138,218,117,10,208,21,179,212,72,
  233,191,101,128,28,126,199,143,128,97,252,222,228,192,213,132,231,81,50,208,111,187,93,152,26,57,135,28,244,8,224,102,165,88,106,62,145,211,64,173,
  230,224,254,253,41,108,220,184,22,229,114,17,134,1,216,182,73,152,91,110,92,199,114,18,225,32,34,156,60,194,235,122,18,96,55,18,239,33,4,185,
  1,130,192,76,104,180,101,130,77,220,219,178,12,184,174,96,237,61,169,128,152,79,158,49,72,176,252,182,109,71,249,60,180,56,77,60,167,41,120,13,
  37,29,150,101,182,129,113,16,233,192,57,99,10,234,99,45,91,18,115,160,91,126,127,226,30,190,159,220,231,147,1,33,127,170,164,158,99,241,222,102,
  103,69,224,89,44,81,150,79,166,98,187,107,95,223,133,1,179,253,114,2,248,126,51,181,41,200,27,159,0,14,150,105,133,131,199,52,96,25,38,10,
  197,34,138,197,2,138,133,18,130,192,199,3,155,183,192,247,2,132,133,84,210,155,35,167,77,150,35,167,184,10,82,184,209,222,191,63,141,93,187,119,
  3,40,192,52,93,37,40,203,2,116,50,27,44,107,239,58,213,224,113,131,136,219,140,40,163,162,178,221,50,45,187,93,128,38,153,28,42,235,35,229,
  235,231,201,138,87,1,77,14,232,112,44,109,214,6,198,149,110,94,105,187,247,238,217,131,227,199,142,229,102,90,19,35,155,113,59,89,94,94,198,63,
  124,239,123,202,13,235,217,103,158,193,208,198,141,74,176,250,183,47,188,0,199,113,180,125,108,89,86,212,238,48,25,54,217,71,175,191,241,6,46,93,
  190,172,5,30,92,146,147,202,90,136,254,252,201,115,207,165,218,47,62,251,127,94,122,9,83,83,83,218,185,113,244,200,17,140,28,60,200,130,152,127,
  248,222,247,176,184,184,200,190,247,106,181,138,237,195,195,232,239,239,199,246,225,225,212,117,63,186,118,13,83,83,83,184,54,58,138,229,229,229,168,239,
  228,192,34,43,215,99,112,112,16,27,55,108,192,142,237,219,209,223,223,159,106,227,217,115,231,224,56,14,206,158,59,167,204,49,224,2,26,177,168,87,
  171,85,12,111,219,134,193,193,65,12,111,219,150,24,79,6,128,209,177,49,76,78,78,98,116,108,44,234,11,221,188,26,218,184,17,207,62,243,76,234,
  111,186,96,106,124,98,2,227,227,227,184,120,233,18,154,205,166,118,190,237,222,181,11,199,143,29,83,142,11,238,249,169,229,161,248,57,121,234,20,206,
  158,59,151,248,238,115,207,62,155,24,75,121,231,160,232,255,133,133,5,124,116,237,26,195,88,169,55,69,29,49,209,169,35,143,122,253,11,144,46,205,
  12,194,44,82,192,151,79,174,34,88,68,126,173,5,203,110,235,42,5,234,128,179,236,81,28,246,175,197,38,218,241,160,35,153,220,70,129,179,235,186,
  40,20,108,169,212,182,159,98,191,69,146,90,88,250,58,212,222,138,231,15,231,18,117,170,162,114,9,35,2,144,225,181,76,226,95,237,167,18,16,147,
  193,70,128,216,42,143,130,228,100,159,210,160,199,247,1,199,113,177,176,176,140,233,233,89,0,6,74,165,18,198,199,239,195,48,12,172,94,221,135,106,
  181,7,134,225,195,182,45,54,240,161,254,218,73,215,16,43,225,107,29,7,46,72,176,253,180,253,113,82,96,210,133,67,182,240,20,90,114,223,23,26,
  112,63,98,115,227,211,24,202,218,199,21,254,226,117,47,201,68,203,115,64,232,160,195,96,199,72,36,98,114,178,60,89,174,193,253,77,38,158,100,199,
  153,152,97,143,19,79,85,36,156,46,15,35,8,2,4,8,96,64,98,160,59,137,178,85,199,152,92,145,6,61,243,10,12,109,92,133,111,126,253,171,
  112,93,7,115,243,11,152,152,184,3,167,213,68,177,80,64,87,185,7,93,93,61,232,234,42,160,210,213,133,174,74,23,2,24,40,151,139,176,44,27,
  150,109,193,48,13,152,1,96,217,54,96,0,126,251,101,22,10,54,90,45,15,93,93,221,168,213,230,52,11,132,33,105,138,212,207,125,243,246,56,70,
  30,58,132,98,177,136,86,203,101,65,111,30,192,200,45,214,178,136,157,147,201,232,250,83,118,52,80,45,138,180,18,36,199,78,45,46,45,162,88,44,
  37,18,171,56,22,102,37,9,117,180,29,220,81,59,237,3,249,89,116,30,185,92,177,148,223,71,187,85,160,51,145,168,69,118,29,163,189,210,6,25,
  222,190,197,98,49,5,24,2,32,1,128,247,237,221,155,2,26,186,113,103,106,142,159,84,71,252,170,170,154,116,124,209,207,208,254,29,29,29,197,208,
  198,141,108,208,189,105,104,40,2,208,170,118,136,231,151,199,237,212,212,20,150,150,150,82,159,175,86,171,120,244,232,209,8,108,6,10,176,179,125,120,
  24,219,135,135,241,232,209,163,184,61,62,142,83,239,190,155,104,139,170,84,122,228,1,63,48,128,71,143,30,205,4,116,135,70,70,162,64,64,4,42,
  116,92,168,42,222,245,246,246,226,232,145,35,216,62,60,156,124,167,134,145,224,147,134,183,109,195,240,182,109,120,244,232,81,140,79,76,224,196,201,147,
  209,115,232,170,4,114,126,254,170,117,110,104,227,70,12,109,220,136,163,71,142,224,196,169,83,56,115,230,76,71,85,102,179,78,56,116,146,179,78,116,
  243,42,153,75,177,88,196,209,35,71,0,0,123,246,236,193,79,126,250,83,101,153,110,202,196,231,217,3,243,202,73,212,129,102,150,190,57,144,42,188,
  101,239,201,2,200,137,35,246,216,42,206,96,228,8,80,158,102,196,247,75,51,187,2,160,167,37,17,136,8,143,228,218,192,1,209,100,169,104,238,57,
  10,133,2,1,60,66,82,16,187,82,8,251,52,113,15,161,153,14,89,70,63,229,138,33,146,12,133,158,55,126,190,144,176,11,101,11,73,38,150,38,
  242,197,192,11,68,62,97,180,181,207,105,121,141,248,105,181,2,120,158,143,153,153,5,204,207,47,192,48,12,20,139,165,196,190,20,4,1,102,103,231,
  49,59,59,15,211,52,209,219,219,131,238,238,18,108,219,70,185,92,128,172,249,77,150,233,6,132,175,50,5,203,114,193,19,61,152,14,224,186,94,123,
  143,180,65,125,198,69,16,100,154,226,186,22,113,130,177,162,251,39,101,29,65,34,225,49,118,251,48,18,149,49,147,193,87,252,28,177,75,7,47,15,
  78,178,215,65,10,51,208,19,207,52,59,77,147,49,3,228,93,178,180,206,58,100,254,216,89,155,170,10,140,233,192,166,12,4,233,49,37,253,252,135,
  31,126,132,190,85,253,176,237,34,214,12,172,69,255,224,90,60,176,117,27,252,0,104,57,30,90,173,38,106,203,75,240,124,96,110,177,129,123,147,243,
  112,221,38,60,207,128,211,114,208,108,54,224,121,77,152,102,25,190,235,163,217,106,98,105,41,140,246,6,6,250,208,191,102,61,90,173,22,219,110,89,
  54,145,199,53,96,106,234,62,12,0,27,55,108,196,216,245,15,19,155,66,94,189,98,86,159,169,192,166,236,202,145,247,232,33,143,20,132,254,255,90,
  189,134,158,106,119,162,66,143,204,226,114,99,35,239,125,229,164,55,10,118,233,152,162,76,185,46,217,75,214,13,139,73,247,251,104,55,199,38,170,78,
  109,84,37,128,185,235,237,219,187,87,11,128,5,128,122,247,244,105,214,170,45,143,92,128,43,116,36,39,111,168,250,70,220,51,153,84,148,76,90,187,
  54,58,138,71,143,30,69,177,88,76,141,253,161,161,161,8,252,115,109,173,86,171,24,24,24,96,255,126,109,108,44,5,252,54,13,13,165,216,85,67,
  225,227,69,175,183,121,211,38,108,222,180,9,175,188,250,42,70,199,198,50,1,145,204,178,114,167,84,220,243,28,63,118,12,3,3,3,248,237,239,126,
  151,154,227,212,74,144,62,71,138,120,208,188,211,161,141,27,241,141,175,125,45,122,14,85,146,97,244,123,177,117,50,100,135,106,29,121,244,200,17,108,
  220,176,1,63,125,249,101,54,103,130,43,136,211,201,152,204,114,7,233,132,212,225,126,55,180,113,35,190,248,244,211,248,213,43,175,176,123,143,42,97,
  49,175,124,36,139,169,226,54,113,126,27,142,165,23,50,3,167,58,93,163,160,128,58,97,240,37,182,13,37,75,78,237,47,67,107,182,180,196,67,20,
  114,145,147,234,228,123,197,218,96,131,177,160,51,24,203,58,222,99,58,233,212,16,219,158,249,190,208,16,39,217,110,49,237,105,210,30,13,44,146,192,
  57,150,4,196,254,200,73,199,19,90,146,60,93,220,133,126,63,217,175,205,166,143,122,189,137,217,217,133,232,244,70,172,133,84,50,197,233,109,23,22,
  150,176,184,184,12,219,182,177,106,85,21,182,109,162,88,44,160,84,178,219,236,111,204,220,138,64,34,249,158,131,40,121,81,48,214,177,156,47,57,150,
  99,153,157,73,146,5,13,2,132,147,250,118,225,120,18,254,222,79,140,101,90,69,80,14,208,146,69,88,228,0,223,136,180,218,178,51,6,45,168,194,
  5,216,20,68,171,237,235,84,235,134,218,27,90,69,172,114,123,54,157,163,182,206,172,95,87,142,86,85,38,89,29,65,164,35,249,251,147,147,237,234,
  130,70,164,163,50,140,98,20,125,6,65,179,157,225,202,3,0,221,207,242,210,28,110,92,191,161,180,125,163,81,103,126,246,209,193,253,251,147,216,177,
  99,59,198,174,127,200,118,60,103,135,146,197,108,231,145,128,100,233,21,179,18,120,100,32,167,98,173,156,70,11,213,238,94,86,15,44,146,134,100,240,
  221,105,57,113,157,230,215,117,221,84,59,117,149,6,211,182,58,248,103,111,55,125,239,92,117,49,93,224,25,4,65,226,200,158,61,89,48,12,172,89,
  179,6,235,215,175,199,228,228,36,62,238,15,237,3,57,1,76,215,31,186,49,214,108,54,113,109,116,20,123,247,236,73,125,110,227,134,13,40,20,10,
  169,96,150,178,196,170,107,95,186,116,41,209,78,25,116,118,162,129,23,127,127,250,169,167,34,240,169,250,25,222,182,45,33,225,81,73,111,84,247,220,
  187,103,15,154,205,38,222,57,113,34,49,247,4,19,189,113,195,6,60,67,158,131,91,188,35,160,3,222,164,140,123,14,86,198,213,222,249,163,53,47,
  166,124,180,26,216,77,67,67,120,236,209,71,113,226,228,73,9,252,152,9,116,241,113,60,213,233,17,114,167,115,80,214,26,203,159,219,62,60,140,51,
  3,3,152,156,156,204,237,181,47,251,195,234,164,113,89,137,194,252,175,213,142,26,116,83,231,246,42,10,50,211,62,191,129,116,204,110,72,96,208,208,
  20,153,138,237,208,146,50,11,209,94,67,233,228,33,216,97,154,20,168,7,35,33,112,138,19,254,146,96,58,246,82,14,239,75,157,30,184,234,116,113,
  133,187,208,229,34,212,253,26,236,231,146,210,150,116,174,131,109,27,153,243,61,102,190,67,9,201,242,114,19,139,139,203,168,213,26,104,181,90,237,211,
  239,2,235,63,174,58,141,166,18,206,153,153,144,149,46,20,108,116,119,119,161,82,41,162,171,171,132,32,112,163,132,58,121,236,136,96,128,178,206,49,
  40,22,99,195,140,220,81,68,17,24,10,42,41,144,22,182,190,177,100,36,72,85,103,116,93,42,123,9,18,44,175,184,118,76,106,197,167,7,177,148,
  148,126,47,86,36,80,121,12,95,197,86,61,30,101,204,73,147,24,233,188,202,67,56,201,216,48,149,191,36,158,39,234,240,14,24,56,85,20,79,129,
  3,199,94,169,193,106,64,244,54,13,248,126,13,65,208,104,71,64,166,210,69,163,147,31,215,117,149,237,150,47,201,5,6,98,48,92,191,113,29,155,
  55,15,1,176,217,197,95,118,58,208,105,196,101,217,133,106,225,228,142,99,59,97,123,58,1,222,174,235,161,90,237,101,179,198,105,133,55,106,233,146,
  119,115,164,71,125,170,177,37,142,99,184,36,39,161,163,228,23,54,51,250,238,39,221,238,136,109,211,140,63,113,156,201,177,106,92,6,175,248,25,24,
  24,136,216,87,229,209,119,251,186,195,219,182,49,118,62,134,182,111,85,115,89,204,7,209,103,157,48,241,226,57,229,141,120,116,116,84,249,94,133,62,
  153,101,84,135,134,82,125,229,251,62,70,199,198,34,54,199,48,66,13,225,211,79,61,149,248,28,151,152,201,6,245,82,155,62,123,252,56,138,197,162,
  82,82,243,217,227,199,211,174,24,12,41,160,235,183,67,35,35,232,235,235,75,205,191,82,169,132,167,159,126,26,80,181,85,62,134,212,188,15,250,28,
  234,170,112,134,86,166,162,91,27,14,141,140,160,167,167,39,81,197,83,48,228,170,235,167,230,55,212,94,214,114,229,74,122,159,172,159,60,54,161,15,
  108,217,194,22,71,81,145,12,42,182,90,125,138,20,68,150,104,20,216,164,127,151,182,73,83,129,82,186,246,133,76,33,82,128,89,5,42,4,107,172,
  34,110,104,1,146,24,92,37,191,31,179,217,70,10,124,202,126,219,201,4,197,64,241,153,228,53,69,252,21,234,151,205,132,221,28,101,133,197,231,100,
  111,230,248,189,199,172,176,101,217,40,20,10,81,82,157,0,139,130,17,167,39,144,162,48,77,252,94,147,237,78,182,201,140,190,35,130,12,223,55,49,
  51,179,140,235,215,239,226,206,157,73,44,45,213,16,4,65,251,254,134,178,176,149,28,124,113,120,41,150,35,24,168,213,26,152,155,91,198,236,108,13,
  173,150,137,66,161,130,66,161,208,30,23,134,148,148,153,188,86,236,138,18,180,159,137,246,173,153,24,3,177,115,73,248,14,204,118,126,153,248,123,60,
  134,226,192,33,108,167,92,120,5,144,61,190,101,219,65,26,0,210,66,56,242,41,130,60,239,98,162,39,190,15,39,215,200,62,93,54,50,2,220,156,
  216,177,141,153,109,221,34,157,149,152,38,24,67,238,136,93,118,178,160,11,165,186,100,105,54,251,144,117,116,168,3,34,220,243,132,255,230,189,70,41,
  67,25,46,194,6,110,222,188,137,207,124,230,40,54,111,26,198,205,91,87,18,118,69,89,137,86,121,217,103,85,169,238,149,252,100,149,166,149,7,218,
  82,109,30,182,61,144,219,214,173,147,242,198,84,54,64,143,179,87,122,108,203,177,68,159,84,187,47,93,190,156,72,184,163,237,254,204,227,143,179,178,
  11,0,248,238,139,47,38,146,188,84,25,195,226,254,84,251,74,127,22,22,22,208,219,219,155,98,106,79,158,58,197,178,197,157,28,135,211,69,151,27,
  47,170,185,149,85,208,99,124,98,2,139,139,139,168,86,171,44,72,166,253,41,126,10,133,66,66,95,108,16,250,139,2,114,195,48,176,111,239,222,40,
  193,85,190,183,72,32,123,239,204,153,232,239,15,29,58,132,163,71,142,176,1,91,177,88,196,161,145,17,156,60,117,42,213,166,125,123,247,178,224,218,
  32,247,17,146,148,98,177,136,71,143,30,197,222,61,123,216,0,249,224,129,3,120,243,173,183,18,193,217,193,3,7,80,214,60,199,153,179,103,113,230,
  236,217,212,115,168,192,62,247,28,58,0,250,219,223,253,14,87,175,94,141,54,165,67,35,35,120,232,208,161,212,51,139,241,245,208,161,67,120,253,141,
  55,148,85,186,228,159,215,223,120,3,23,46,94,212,174,93,170,2,83,58,194,97,113,113,17,223,125,241,197,68,48,109,24,6,30,121,248,97,28,57,
  124,152,149,78,244,244,244,40,129,228,199,61,189,17,9,109,156,223,114,188,246,25,138,80,142,95,211,40,107,167,99,125,169,52,130,178,169,170,192,78,
  62,234,142,139,139,200,140,179,42,153,142,3,226,113,59,146,9,97,105,6,91,150,153,208,146,218,212,54,46,121,228,31,183,49,118,250,48,37,208,110,
  72,64,61,217,63,161,110,215,79,176,248,84,47,222,30,197,169,239,203,210,3,223,15,191,119,239,222,44,150,150,150,217,147,70,174,18,166,138,249,87,
  145,145,98,109,22,152,73,248,105,215,235,45,52,155,30,122,123,123,80,44,26,240,125,7,174,219,138,64,108,24,180,37,181,238,182,109,49,36,67,236,
  234,145,150,187,168,24,105,159,120,119,7,176,44,35,114,8,137,181,210,113,193,148,88,43,46,39,20,38,153,104,129,29,195,57,100,17,143,234,80,226,
  65,189,165,101,16,77,25,241,60,193,54,245,149,238,20,163,113,216,49,57,114,114,44,186,116,115,165,236,157,109,219,108,194,22,167,255,140,109,75,130,
  4,107,230,103,36,89,169,218,152,167,227,104,39,123,158,151,96,151,117,229,143,229,129,30,182,49,192,220,220,44,90,142,139,7,182,62,144,96,225,58,
  177,133,18,237,80,77,166,60,149,199,84,44,138,202,24,92,119,100,74,223,105,224,219,168,116,245,101,178,64,116,209,112,93,55,23,107,36,107,39,197,
  70,163,106,187,142,37,82,37,111,254,115,181,91,101,27,214,73,187,125,223,199,182,173,91,83,223,153,154,154,98,229,5,213,106,53,5,54,169,191,122,
  39,243,136,43,215,204,121,2,203,254,188,89,63,20,36,211,121,180,137,97,153,131,32,192,230,77,155,34,11,36,122,47,167,217,196,237,241,241,68,176,
  32,18,245,228,31,199,113,240,147,151,95,198,217,115,231,18,27,216,217,115,231,240,127,94,122,73,217,246,221,187,118,165,214,4,0,56,120,224,128,242,
  249,196,125,232,189,95,127,227,141,72,230,32,191,131,157,59,118,164,88,242,67,35,35,73,217,76,251,111,205,102,51,245,28,134,97,36,158,131,251,161,
  193,92,30,87,36,75,114,30,57,115,246,44,126,240,195,31,42,157,55,132,196,38,239,152,151,129,5,247,126,85,160,154,59,253,83,173,107,2,100,136,
  96,67,197,70,229,41,10,70,231,164,238,132,44,110,171,7,223,247,18,199,196,233,112,75,205,56,171,139,250,152,17,107,202,17,58,244,218,242,53,101,
  162,74,150,105,196,190,185,130,105,52,82,101,212,169,207,114,178,100,182,73,254,73,50,147,130,12,73,3,89,90,38,90,56,32,196,218,91,145,180,151,
  180,59,51,18,207,23,251,44,203,237,51,19,109,142,251,204,32,178,13,202,116,27,17,195,169,98,152,147,255,63,188,78,171,229,98,105,201,193,245,235,
  119,240,225,135,55,176,184,184,164,96,225,147,39,160,50,232,50,77,19,182,109,195,182,237,196,216,229,254,73,58,145,196,22,193,182,29,26,35,248,190,
  137,114,121,21,86,173,90,139,158,158,213,40,151,43,161,169,130,101,36,156,52,146,160,47,102,156,195,126,9,18,172,52,207,72,203,94,203,126,226,179,
  66,62,35,222,49,101,255,105,241,23,154,28,75,193,171,232,39,249,132,136,142,83,206,51,159,156,7,178,178,158,108,204,148,15,60,235,177,99,152,167,
  100,103,153,224,235,220,54,242,210,223,194,39,208,48,236,182,190,38,217,64,93,37,63,142,141,229,106,152,103,49,176,156,157,153,142,157,229,106,165,135,
  207,224,227,222,189,251,216,178,121,3,222,49,139,0,124,20,139,69,244,244,84,81,46,151,177,106,85,31,6,214,172,66,79,181,47,178,133,115,93,7,
  191,254,245,175,177,92,91,102,23,116,149,39,106,214,64,80,201,106,242,2,39,186,232,137,62,104,52,155,176,201,145,144,106,35,147,143,240,101,240,213,
  73,162,163,120,167,186,113,152,55,91,254,159,163,221,114,145,156,206,124,94,227,159,77,67,67,232,235,235,75,221,239,218,232,40,166,166,166,34,192,72,
  219,56,60,60,140,241,137,137,142,3,172,172,207,171,142,185,85,158,211,170,192,239,246,248,120,138,45,13,130,0,165,82,9,219,182,110,197,245,27,55,
  18,127,27,30,30,78,109,250,1,66,219,54,97,219,23,4,1,6,7,7,149,146,139,139,151,46,165,108,242,68,251,166,166,166,112,238,253,247,35,80,
  76,219,93,42,149,48,56,56,136,153,153,153,132,164,166,84,42,177,60,225,217,115,231,162,251,200,9,103,239,157,57,131,135,14,29,194,205,91,183,48,
  53,53,133,133,133,5,92,27,29,77,73,144,6,6,6,80,46,151,217,229,252,210,229,203,152,154,154,98,217,142,169,169,41,156,61,119,46,241,28,226,
  89,138,197,34,6,6,6,82,125,160,90,51,60,207,75,121,134,47,46,46,226,210,229,203,169,32,69,188,59,113,253,78,138,22,169,146,127,85,201,188,
  218,163,87,133,231,121,87,87,23,30,121,248,97,118,19,13,130,0,243,243,243,236,62,182,146,181,51,57,175,45,197,49,176,190,106,32,199,78,82,237,
  48,173,152,38,126,39,10,141,36,65,105,246,9,157,58,81,79,244,79,242,56,157,190,187,180,68,139,15,146,68,63,22,10,118,42,96,136,93,57,146,
  218,106,223,71,196,102,134,18,24,159,41,104,34,7,9,102,74,147,29,251,248,38,165,34,158,39,170,228,165,157,73,194,119,103,36,250,32,233,50,18,
  254,161,94,111,97,121,185,137,153,153,249,4,128,165,39,167,114,240,193,177,210,114,229,96,218,103,22,179,207,210,62,167,164,28,149,29,138,119,83,40,
  148,96,219,61,48,77,3,142,211,128,235,54,81,175,47,163,209,168,193,52,13,184,174,31,37,23,38,147,52,67,22,57,89,245,207,32,149,20,5,80,
  22,146,25,193,38,135,142,29,161,143,118,88,168,69,36,25,134,69,95,144,0,212,225,239,227,202,146,201,64,56,157,108,40,152,103,209,94,215,141,253,
  162,121,140,68,75,174,211,181,199,143,238,39,191,119,65,226,230,73,62,204,131,29,109,174,52,116,214,145,87,94,128,71,55,230,88,188,174,7,131,121,
  202,235,114,54,98,242,98,172,43,221,106,24,6,10,118,41,210,63,217,133,2,108,203,70,185,84,68,193,46,162,80,44,160,88,40,160,80,42,192,180,
  12,84,74,125,40,216,30,10,197,10,202,149,50,214,175,31,68,79,181,138,127,247,239,190,29,229,117,218,118,17,45,215,67,163,225,96,121,185,137,122,
  163,134,102,115,25,125,125,61,216,177,115,7,206,159,191,130,37,146,120,152,85,80,160,147,5,93,150,156,216,118,25,143,61,126,24,173,70,19,147,247,
  167,177,184,92,195,252,252,44,90,174,19,78,74,152,128,1,20,236,66,172,177,51,13,120,174,135,114,201,198,234,213,125,216,60,244,0,230,23,231,209,
  211,211,131,66,193,66,87,87,47,202,133,18,236,130,137,98,169,128,82,161,212,78,116,232,70,165,171,130,159,255,226,231,152,38,64,68,102,221,100,231,
  7,121,113,225,180,172,244,191,91,173,86,100,117,36,51,233,89,140,155,46,224,203,170,36,199,181,91,55,110,101,150,77,215,238,7,30,120,32,229,22,
  98,24,6,110,143,143,99,122,122,26,142,227,164,244,173,219,135,135,113,226,228,73,52,155,77,182,66,226,39,249,35,123,134,231,241,132,159,154,154,194,
  212,212,84,66,215,45,190,51,56,56,136,177,235,215,19,243,81,48,211,137,108,107,32,165,167,214,121,100,203,108,189,188,142,141,142,141,97,228,224,65,
  182,205,67,27,55,70,192,211,52,77,12,244,247,107,92,120,166,82,114,30,10,164,255,246,133,23,88,9,19,29,19,155,55,109,82,246,31,125,14,174,
  13,226,57,84,174,19,121,0,180,216,16,184,245,149,251,126,100,231,215,223,143,169,169,169,92,210,185,227,199,142,69,9,152,220,252,16,254,236,151,46,
  95,198,239,94,127,61,95,224,223,62,129,249,171,239,124,39,243,168,150,222,119,124,98,34,209,102,157,191,171,142,208,224,24,232,244,62,101,48,46,19,
  60,203,29,3,164,164,77,25,101,85,41,219,44,64,118,158,98,15,194,218,141,74,19,146,96,222,144,116,201,252,179,210,10,110,170,61,43,89,196,3,
  210,179,27,9,237,108,220,255,113,5,60,209,22,193,154,202,96,88,78,8,4,124,2,158,141,68,21,66,177,52,11,231,145,116,46,82,250,249,197,119,
  66,16,234,99,121,185,129,122,189,137,185,185,5,77,5,201,228,137,179,42,15,140,2,94,74,186,112,133,154,56,0,77,231,43,157,63,52,0,22,115,
  185,88,44,193,247,125,148,74,85,212,106,139,112,156,6,10,133,22,28,167,153,112,190,144,183,194,216,207,59,172,134,72,189,195,169,172,35,28,123,161,
  255,182,208,216,135,22,120,33,88,14,139,181,132,121,108,161,148,68,36,31,166,109,238,132,180,67,252,46,30,211,201,207,11,75,195,56,128,8,72,162,
  34,36,71,16,67,74,220,228,79,137,147,123,88,192,24,9,100,203,150,19,248,54,26,150,138,36,167,44,155,162,60,149,211,40,112,208,101,77,203,15,
  163,91,212,232,239,57,16,164,115,88,120,236,240,163,88,183,113,3,214,174,93,23,154,98,27,38,16,0,6,124,0,22,60,223,133,231,135,186,31,63,
  8,224,185,65,100,157,215,106,181,240,225,71,19,152,157,153,135,235,58,88,90,94,192,242,114,13,205,122,29,77,167,5,215,93,142,88,246,112,96,118,
  225,47,254,226,207,225,56,173,212,2,164,234,59,58,24,116,165,137,229,164,56,241,185,117,235,214,227,240,195,135,49,51,187,128,145,67,37,88,166,137,
  70,163,134,70,163,14,4,6,2,211,134,129,0,5,203,134,209,78,228,128,225,195,243,124,148,138,21,116,247,148,240,220,87,158,67,189,222,64,16,132,
  199,88,45,167,5,207,11,208,108,54,224,184,117,184,45,7,203,13,23,213,213,61,48,12,19,75,75,141,212,113,11,125,78,186,128,100,29,251,202,207,
  198,85,7,20,110,6,114,95,116,194,124,171,108,172,86,210,110,85,48,163,106,119,169,84,194,174,157,59,83,127,111,58,14,38,39,39,97,154,38,46,
  94,186,132,67,35,35,9,0,47,60,163,57,208,152,85,237,82,213,47,170,57,166,242,187,150,79,49,228,77,229,218,232,40,6,6,6,82,215,220,179,
  123,119,36,117,16,224,89,14,16,12,195,192,226,226,34,198,39,38,18,243,95,104,159,185,119,177,176,176,160,124,135,226,239,170,68,186,74,165,146,120,
  78,202,14,203,238,23,147,4,96,202,27,157,46,224,167,99,83,4,83,220,186,74,159,131,91,63,23,22,22,148,107,90,169,84,74,156,40,229,245,18,
  134,226,249,228,31,209,238,188,69,146,100,71,29,153,204,224,198,163,206,151,89,20,40,138,2,219,118,34,163,204,38,209,251,95,188,116,57,229,90,163,
  90,95,242,4,226,20,116,200,192,57,57,174,12,229,28,164,58,87,174,140,49,119,77,234,166,193,249,73,115,69,79,146,5,78,56,22,90,239,42,19,
  179,197,126,187,208,149,15,219,54,163,196,201,152,189,53,89,217,70,236,5,140,132,133,92,108,141,103,72,65,185,145,114,96,144,191,39,180,219,97,41,
  233,100,249,109,209,182,56,16,49,73,64,2,112,30,215,162,16,76,179,233,98,118,118,1,11,11,75,104,54,157,132,14,89,6,102,116,174,115,178,77,
  161,237,21,73,133,116,222,200,53,39,196,254,192,201,93,229,61,136,22,158,17,32,58,44,62,83,136,100,180,226,126,133,66,1,245,122,29,158,215,66,
  185,236,1,240,225,56,203,104,54,27,74,214,149,250,137,135,174,38,113,223,197,192,213,100,180,209,201,132,193,176,64,77,208,214,140,251,196,150,48,32,
  214,139,96,157,73,100,16,45,24,237,32,240,36,221,186,124,66,23,7,105,170,226,43,201,231,148,231,78,26,80,235,214,73,42,231,140,75,121,107,142,
  189,185,77,148,254,183,110,243,80,109,56,186,197,157,202,62,242,92,155,99,166,116,192,105,231,174,237,184,126,251,62,78,156,60,131,66,177,140,150,227,
  160,213,114,225,180,26,112,91,30,28,167,30,49,12,65,208,2,53,102,7,169,156,40,238,17,110,92,38,122,122,122,81,237,89,141,174,174,34,214,172,
  26,196,114,109,9,19,119,103,225,123,62,154,205,90,138,209,204,218,140,56,207,95,29,123,34,250,171,80,48,49,53,57,139,127,248,222,247,96,219,54,
  170,213,62,244,116,247,194,180,10,8,224,33,240,125,184,110,29,158,23,214,212,105,57,245,246,224,182,80,40,116,161,187,167,23,139,11,247,177,180,52,
  143,86,203,107,107,24,29,246,184,164,127,96,3,246,237,222,1,167,85,79,188,235,188,129,149,28,40,169,0,117,214,134,189,18,251,53,110,140,118,210,
  110,93,148,154,213,238,161,141,27,89,73,130,176,109,11,130,0,227,227,227,209,145,58,29,219,219,183,111,199,216,245,235,236,253,84,65,39,7,134,229,
  177,197,157,218,112,44,52,215,183,244,250,23,47,93,194,163,71,143,166,62,91,169,84,176,118,237,218,136,233,164,238,27,180,237,114,242,38,5,136,188,
  7,107,83,123,162,64,43,56,114,192,83,199,122,210,159,197,197,197,21,187,0,137,254,84,37,65,210,231,80,129,125,199,113,148,227,170,88,44,38,152,
  163,32,8,34,187,186,149,150,79,239,244,71,30,131,89,238,23,242,26,166,243,194,151,17,178,105,24,10,240,24,254,220,158,152,192,27,111,190,201,6,
  177,249,78,248,100,17,15,183,190,164,229,26,186,53,72,128,218,52,136,51,19,71,219,33,208,70,196,190,201,39,98,161,239,175,145,171,90,160,170,108,
  119,58,168,49,73,177,18,106,243,102,166,24,225,228,245,131,132,142,150,79,10,164,71,237,212,207,217,146,174,13,233,232,61,89,190,91,252,223,112,45,
  74,91,248,133,251,37,237,191,216,130,77,56,104,136,223,123,158,143,70,195,197,236,236,44,150,150,106,145,92,147,22,229,224,246,105,250,55,26,0,136,
  207,57,142,3,219,182,81,169,84,208,104,52,208,108,54,81,44,22,83,26,105,217,27,154,186,149,201,235,176,104,151,92,184,134,130,104,1,228,139,197,
  98,4,168,155,205,38,121,166,53,112,156,70,155,72,91,66,171,213,36,5,76,66,169,109,24,40,185,48,140,216,97,35,4,174,32,193,137,192,63,32,
  88,40,182,183,139,75,157,251,137,19,8,1,132,133,164,135,75,86,231,152,104,33,151,18,178,146,52,198,75,2,98,129,129,194,189,194,82,38,255,113,
  193,67,30,236,40,255,206,206,3,204,84,101,105,185,232,53,207,77,179,22,96,57,129,65,142,10,242,120,79,235,218,50,59,55,135,174,114,15,238,220,
  185,158,201,86,90,102,17,235,215,15,162,127,245,0,204,130,137,153,233,25,220,189,123,15,107,86,15,98,112,112,16,221,93,69,4,48,177,118,221,16,
  54,109,26,128,105,217,240,125,15,245,154,139,107,163,99,152,154,190,128,32,240,208,114,221,84,32,145,229,24,145,87,202,33,152,39,177,81,56,78,11,
  141,102,29,65,224,160,213,114,48,51,83,195,204,204,29,237,53,44,179,128,32,48,224,249,119,96,220,47,2,72,123,246,218,86,1,149,74,55,202,37,
  11,51,179,211,240,252,0,11,11,11,176,11,165,204,247,157,245,44,186,100,31,85,144,38,126,215,105,18,170,170,18,226,74,219,173,139,80,85,237,222,
  177,99,7,203,78,77,78,78,70,109,188,61,62,30,201,56,12,211,140,42,26,14,111,219,134,98,177,136,70,163,193,178,141,129,134,133,206,210,253,211,
  255,159,167,156,249,239,94,127,61,229,174,209,106,181,48,58,54,150,44,73,221,110,227,208,198,141,17,43,184,97,253,250,212,154,34,116,212,220,145,49,
  45,182,163,155,251,121,3,153,149,202,90,228,4,162,188,193,153,28,164,249,190,15,195,52,211,62,13,140,236,33,207,58,42,91,12,154,134,145,176,222,
  147,19,97,59,9,10,117,82,167,44,198,91,238,7,85,17,21,29,97,162,154,135,201,228,179,80,134,114,233,242,149,104,76,138,19,31,149,7,190,250,
  196,74,229,190,173,6,207,201,235,36,65,43,173,206,166,58,229,73,218,121,161,237,251,235,51,167,88,166,18,56,203,213,2,233,61,93,215,107,219,155,
  113,146,131,176,255,109,91,20,239,48,19,250,236,240,120,63,46,174,17,238,209,1,105,115,218,166,140,178,200,98,85,138,53,179,114,251,141,246,125,229,
  126,50,36,128,157,118,251,138,129,44,16,39,188,5,109,224,103,70,224,57,180,239,4,150,151,27,184,125,251,110,212,183,2,212,210,188,0,149,13,173,
  108,83,23,4,1,90,173,22,138,197,98,232,176,115,240,32,190,240,133,47,160,167,167,7,247,239,223,199,149,43,87,48,58,58,138,249,249,121,184,174,
  139,98,177,216,150,92,20,83,58,115,14,231,8,205,181,156,208,41,48,146,235,186,104,54,155,209,53,251,250,250,80,40,20,80,171,213,48,55,55,151,
  8,202,203,229,110,20,139,21,244,247,175,67,189,190,128,197,197,69,56,78,45,74,32,23,239,139,234,163,147,96,54,144,198,91,172,115,142,229,56,84,
  246,33,107,162,99,191,110,57,40,211,159,152,129,16,152,134,162,218,32,181,217,243,83,254,207,60,11,45,87,54,52,34,98,52,239,218,110,103,45,86,
  170,9,207,45,194,58,175,99,229,209,28,211,88,177,232,201,201,90,186,227,122,202,142,170,18,82,68,91,230,230,230,177,166,127,35,128,2,1,138,6,
  186,42,221,216,189,107,23,54,109,218,8,24,192,7,231,47,97,237,218,117,248,204,103,66,54,205,109,249,81,209,136,238,238,10,12,195,192,114,173,133,
  70,195,193,245,27,55,209,211,83,70,203,243,240,234,171,191,193,210,82,29,45,103,22,165,98,23,60,191,5,215,245,149,122,26,21,27,152,55,97,39,
  92,244,226,254,9,203,116,22,19,3,164,88,172,96,253,218,181,48,45,11,48,44,172,95,183,22,158,23,78,62,215,5,30,127,236,17,4,134,133,250,
  242,34,2,216,152,155,155,198,47,126,249,115,84,123,86,227,179,199,142,163,167,167,132,98,185,132,66,177,4,215,105,225,215,175,189,130,155,227,183,209,
  114,28,152,166,141,98,161,0,135,169,250,200,201,86,178,130,46,57,201,40,139,133,215,109,174,170,49,170,187,102,158,118,171,190,71,219,162,186,71,177,
  88,76,21,79,17,139,241,248,196,68,194,222,75,20,39,9,36,16,64,75,123,179,190,193,25,50,41,149,108,168,147,178,202,170,159,86,171,133,209,209,
  209,228,51,182,181,205,219,135,135,113,246,220,57,12,14,14,98,221,186,117,41,96,53,53,53,21,1,108,218,6,17,44,112,239,184,84,42,193,113,28,
  101,162,51,181,213,147,223,35,181,28,204,250,169,86,171,145,140,66,181,238,113,230,251,244,239,130,21,226,96,88,169,84,74,105,219,233,61,56,123,64,
  241,179,180,180,164,124,111,178,165,104,188,78,184,185,89,98,225,164,209,169,44,68,23,76,100,57,6,229,185,62,149,75,56,142,131,209,177,235,184,216,
  62,197,201,154,167,234,251,9,96,24,164,74,12,199,118,106,60,112,230,216,232,44,173,177,204,114,83,251,46,121,142,134,199,248,106,224,28,151,124,14,
  82,133,84,194,68,191,52,107,47,128,51,45,167,45,254,70,75,90,203,218,99,219,54,18,69,58,228,68,200,164,158,59,41,11,224,139,157,152,160,150,
  103,170,42,139,20,92,155,102,32,57,70,132,96,43,14,2,66,57,65,173,230,96,121,185,142,187,119,39,73,127,20,82,184,131,35,243,232,248,164,228,
  67,173,86,139,0,120,189,94,135,105,154,232,234,234,10,143,246,109,27,187,118,237,194,206,157,59,177,188,188,140,209,209,81,140,142,142,226,254,253,251,
  88,94,94,134,231,121,40,20,10,40,149,74,41,135,27,57,105,80,244,169,236,245,47,0,116,87,87,23,186,186,186,176,106,213,170,72,146,86,46,151,
  209,221,221,29,177,224,226,31,207,243,208,213,213,141,193,193,181,104,181,90,152,153,153,70,173,182,132,122,125,25,245,250,18,60,175,69,238,27,107,212,
  125,223,32,69,83,232,60,8,18,182,117,201,121,25,254,93,124,87,36,118,134,238,53,188,101,32,13,96,69,178,162,42,144,229,200,84,129,29,85,146,
  216,236,128,88,143,29,115,49,208,121,22,67,17,25,201,20,60,55,232,58,93,120,85,12,73,44,151,48,24,27,174,86,226,216,80,94,28,133,231,160,
  97,24,152,157,155,195,186,13,27,35,61,151,97,24,40,88,69,124,229,185,103,97,23,187,241,209,181,107,168,148,187,113,248,145,195,120,237,215,111,194,
  178,108,28,57,252,16,126,248,210,171,152,154,186,141,190,190,85,248,202,87,254,8,19,19,247,241,202,43,191,129,235,54,17,4,117,28,26,57,130,135,
  30,58,132,185,217,25,4,65,35,28,12,166,1,219,46,74,181,235,63,94,241,147,172,68,23,207,247,194,133,5,38,2,120,48,12,11,95,121,238,57,
  184,190,129,70,179,133,74,165,140,254,254,85,232,238,42,226,246,173,73,216,182,137,143,174,93,199,221,251,243,120,234,201,199,112,237,234,24,46,93,185,
  138,32,240,81,40,20,177,88,107,160,103,245,42,216,150,133,159,253,236,21,76,77,221,131,227,212,218,237,104,193,247,3,116,119,247,160,57,59,147,121,
  42,145,165,187,85,1,85,170,239,148,251,71,85,226,59,79,133,71,213,59,209,181,187,147,31,174,221,42,15,105,195,48,240,31,254,242,47,19,109,84,
  141,151,225,109,219,112,246,220,185,136,57,144,25,61,207,247,35,187,50,46,160,208,57,136,228,13,22,84,63,133,66,1,31,94,189,138,227,199,142,197,
  146,133,246,119,250,251,251,81,173,86,19,182,118,180,111,174,223,184,193,50,175,66,134,193,37,75,246,244,244,96,122,122,58,85,178,54,74,94,100,18,
  26,85,207,81,175,215,213,0,186,167,39,161,167,150,23,230,227,199,142,161,80,40,96,122,122,26,151,46,95,142,64,61,109,11,149,155,200,239,160,183,
  183,151,173,52,201,61,71,222,53,67,37,7,225,254,214,91,173,42,181,241,174,52,142,229,96,129,254,80,31,104,157,60,35,15,128,167,253,187,184,180,
  132,239,189,248,34,170,213,42,190,248,212,83,169,2,68,197,98,17,71,143,28,198,192,192,0,126,249,171,95,105,171,191,113,213,198,4,251,153,4,128,
  177,172,34,60,74,142,247,12,93,53,52,21,192,230,128,51,213,15,171,128,51,5,198,190,111,144,68,57,164,88,65,93,242,191,144,55,208,228,43,203,
  178,37,55,16,35,21,8,132,123,114,120,148,30,87,45,132,84,1,143,254,183,0,95,102,84,2,58,6,187,212,1,65,248,11,27,9,246,57,173,93,
  78,75,118,98,192,38,124,132,99,121,70,168,69,246,209,104,120,152,155,91,192,252,252,98,34,41,80,4,132,114,210,159,92,171,128,222,83,16,85,190,
  239,163,80,40,224,192,129,3,56,112,224,0,138,197,34,198,199,199,241,209,71,31,225,205,55,223,196,169,83,167,176,99,199,14,236,217,179,7,59,118,
  236,192,186,117,235,176,127,255,126,140,140,140,96,114,114,18,119,238,220,193,248,248,56,38,39,39,35,137,7,205,197,144,79,255,41,30,18,213,92,69,
  114,250,170,85,171,80,173,86,35,9,151,104,119,161,80,64,177,88,68,185,92,142,174,53,55,55,135,90,173,134,74,165,2,219,182,177,180,180,212,174,
  222,88,66,87,151,141,114,185,167,237,230,225,192,113,26,112,156,122,194,70,47,214,194,91,210,94,18,180,61,192,147,213,4,133,4,36,214,77,199,146,
  12,234,178,166,218,151,98,7,14,49,190,12,34,19,225,89,104,234,203,158,100,155,161,116,250,138,243,243,172,76,236,152,60,45,109,3,104,110,179,205,
  115,196,167,59,2,231,142,33,169,134,49,203,214,76,5,134,116,166,214,220,113,29,101,105,197,195,55,155,30,186,187,186,80,42,85,209,106,205,1,0,
  186,171,61,88,183,97,3,254,225,187,47,97,106,234,70,24,221,26,5,248,65,11,247,239,85,96,154,15,99,247,174,205,88,85,45,194,245,91,168,215,
  151,209,191,166,15,135,14,238,195,253,169,251,184,113,227,163,80,71,102,153,100,193,51,80,40,84,16,248,1,92,215,97,13,215,59,213,121,231,2,211,
  1,224,121,110,24,133,27,161,67,136,239,7,248,209,143,126,132,86,171,6,211,44,96,221,250,97,124,227,107,95,194,205,27,227,168,148,138,56,125,246,
  61,216,118,5,190,23,46,118,27,214,15,96,215,206,7,112,226,196,25,252,230,55,63,195,161,135,30,197,150,161,13,152,152,72,59,5,212,235,77,172,
  234,237,197,76,27,64,235,88,76,29,176,229,220,84,100,144,156,247,168,56,143,134,137,59,134,94,73,187,117,96,155,107,183,44,109,200,146,60,113,247,
  30,24,24,64,181,90,77,232,114,19,21,29,153,82,245,226,200,80,44,36,20,140,202,126,207,89,69,110,116,182,129,226,158,178,45,154,248,220,142,237,
  219,149,229,203,47,92,188,24,181,139,110,98,130,41,230,116,234,59,182,111,199,236,236,172,242,244,70,87,233,145,38,43,26,134,145,112,146,145,127,6,
  7,7,19,22,130,180,29,197,98,17,219,135,135,81,40,20,176,125,120,24,143,30,61,26,89,195,125,116,237,26,150,150,150,226,231,104,179,241,242,251,
  29,222,182,45,210,135,115,54,158,186,231,184,61,62,206,230,167,168,198,180,44,31,19,86,129,170,249,51,63,55,151,216,128,228,42,97,242,154,155,199,
  51,154,130,103,213,230,150,216,12,201,169,193,175,94,125,21,223,250,198,215,217,60,130,225,109,91,241,208,161,67,209,9,141,202,215,60,41,29,73,219,
  125,37,215,4,83,2,149,89,243,34,115,133,202,180,163,139,219,224,183,231,109,32,89,174,201,44,116,218,70,142,250,255,10,102,48,246,78,54,153,99,
  114,176,86,114,130,149,246,125,51,33,193,144,129,84,252,14,169,172,194,146,202,60,211,126,178,37,199,13,164,18,198,146,64,93,174,152,104,166,190,91,
  175,187,168,215,67,11,58,113,114,101,89,86,162,100,182,44,7,147,1,179,108,75,39,64,183,200,205,234,233,233,193,23,190,240,5,108,219,182,13,166,
  105,226,161,135,30,194,236,236,44,110,221,186,133,139,23,47,226,163,143,62,194,173,91,183,240,238,187,239,98,120,120,24,219,183,111,199,182,109,219,48,
  56,24,74,63,119,239,222,141,133,133,5,220,191,127,31,247,239,223,71,173,86,75,157,160,209,53,90,200,56,68,210,96,169,84,66,169,84,138,116,207,
  113,57,114,59,81,155,67,172,247,2,252,247,245,245,161,84,42,37,152,105,1,16,109,187,208,14,18,187,80,46,187,112,156,38,60,175,25,26,7,180,
  243,164,68,128,37,130,36,97,65,104,154,20,68,203,114,142,88,238,33,18,15,227,249,102,176,250,227,164,148,74,128,92,35,114,162,225,138,168,169,246,
  118,78,182,152,148,113,80,230,90,143,29,147,186,247,54,128,54,208,153,142,52,11,56,228,173,55,206,109,188,242,198,169,98,224,116,154,225,44,150,102,
  110,110,10,133,66,17,197,162,137,102,51,236,148,249,249,89,220,186,121,19,199,158,56,140,31,253,248,14,130,160,133,0,45,24,6,176,121,203,70,152,
  150,137,131,35,123,208,216,181,29,203,245,101,172,234,173,194,178,45,28,24,217,143,247,222,123,31,55,110,140,194,52,44,56,205,6,124,191,145,92,108,
  219,255,227,54,138,149,128,103,21,139,38,174,109,181,253,42,69,230,122,171,85,199,71,163,163,120,250,139,95,192,47,126,254,51,24,134,143,133,249,41,
  120,129,143,163,143,29,196,189,187,211,192,57,31,27,135,214,163,92,41,98,120,199,3,152,154,236,195,212,244,28,92,183,213,46,221,91,64,0,151,109,
  207,226,226,60,86,247,173,134,97,220,200,197,36,169,64,42,53,228,151,199,144,204,228,102,141,47,93,161,154,78,78,90,242,180,91,23,8,201,237,238,
  237,237,77,1,33,221,169,4,213,248,165,129,194,182,68,81,143,44,160,32,127,78,6,61,121,124,173,243,72,102,196,117,71,199,198,88,47,235,161,161,
  33,22,12,142,142,141,161,213,106,177,1,176,240,133,46,22,139,41,128,252,224,190,125,24,187,126,157,181,97,171,86,171,74,198,223,113,156,212,119,38,
  39,39,217,106,138,190,239,227,208,200,8,46,94,186,20,177,64,84,79,127,104,100,36,5,230,170,213,42,142,30,57,130,139,151,46,37,158,227,184,227,
  68,37,121,3,34,8,220,183,119,47,70,199,198,48,53,53,149,114,127,169,86,171,169,106,135,66,67,221,106,63,135,172,47,166,0,55,0,180,73,225,
  165,82,41,186,62,55,6,166,103,102,148,167,98,170,223,119,98,173,40,54,48,186,15,201,213,10,169,5,220,194,194,2,206,158,123,31,71,143,28,38,
  135,205,241,207,209,35,135,113,109,116,148,117,54,145,131,140,216,43,56,253,119,217,123,153,218,228,101,187,97,240,172,179,176,13,203,2,206,66,151,28,
  23,175,0,1,16,188,199,113,218,103,56,189,231,38,191,155,100,194,67,80,20,246,166,109,155,41,59,61,203,74,235,86,211,137,131,105,153,70,8,174,
  205,84,0,65,251,149,182,45,41,219,8,18,197,96,60,207,111,219,222,197,69,103,60,47,192,220,92,173,13,154,157,104,221,21,115,82,30,91,52,87,
  128,75,220,75,229,42,144,191,149,74,37,44,46,46,226,251,223,255,62,134,134,134,240,224,131,15,98,247,238,221,168,86,171,216,191,127,63,246,237,219,
  135,217,217,89,140,141,141,225,202,149,43,184,122,245,42,70,71,71,177,118,237,90,60,240,192,3,17,144,238,239,239,199,224,224,32,202,229,50,174,93,
  187,22,1,116,185,144,10,93,151,187,186,186,82,9,133,162,24,137,92,200,69,128,109,215,117,225,121,30,138,197,34,74,165,82,4,178,109,219,70,185,
  92,70,173,86,195,226,226,34,26,141,6,90,45,225,22,102,163,92,182,96,24,221,40,149,66,247,49,207,107,194,113,26,104,54,235,145,44,131,6,70,
  122,16,29,36,198,127,92,185,208,72,0,101,153,93,166,165,190,227,113,19,68,201,166,105,153,26,216,202,156,89,120,170,211,162,124,2,91,217,84,127,
  152,135,97,83,249,245,210,13,146,59,30,139,203,97,119,6,82,232,181,116,155,123,254,114,215,6,102,102,230,224,183,28,12,174,29,192,242,242,76,52,
  73,255,233,71,255,132,103,158,253,19,124,251,47,254,12,39,78,190,139,171,87,63,4,224,225,131,247,47,99,255,254,17,188,249,214,69,124,244,209,5,
  52,155,117,124,237,43,207,160,222,116,241,139,95,188,130,150,51,27,78,98,191,1,215,245,18,47,188,213,114,17,180,11,170,36,142,216,21,128,139,78,
  102,185,60,120,116,156,218,62,250,20,153,179,114,52,102,91,5,32,64,162,29,239,191,255,30,42,93,159,195,151,190,244,69,252,226,23,191,68,163,49,
  135,153,233,89,140,141,221,196,233,211,167,96,24,46,46,92,56,141,157,59,119,96,122,242,62,222,124,251,117,8,59,62,211,52,49,55,63,135,157,59,
  134,144,22,222,3,115,139,179,216,178,161,63,193,96,102,233,183,179,228,3,114,64,167,75,46,226,238,215,73,153,247,78,219,221,201,223,229,118,203,165,
  187,229,106,116,50,16,208,181,103,223,222,189,56,247,254,251,218,99,106,93,21,65,57,215,129,122,27,135,54,96,151,50,245,164,186,31,225,9,189,122,
  245,234,196,53,168,167,115,130,69,189,125,91,89,13,81,84,228,59,122,228,72,170,79,138,197,34,158,123,230,153,168,148,183,24,19,15,29,58,20,1,
  91,238,116,67,0,91,121,93,187,116,249,114,170,24,140,0,153,223,250,198,55,112,226,228,73,140,142,141,69,155,233,225,71,30,193,254,7,31,100,211,
  206,68,34,40,253,57,247,254,251,97,137,113,193,203,180,233,16,250,28,52,48,26,57,120,144,45,181,45,60,179,47,92,188,168,244,40,231,124,103,233,
  51,249,190,143,77,67,67,56,116,232,80,20,52,200,215,249,232,218,181,132,180,69,181,110,201,227,132,59,162,205,51,111,228,64,140,218,205,209,128,50,
  180,121,60,24,30,95,51,227,255,208,200,72,228,51,205,237,89,233,100,90,189,211,134,74,143,153,252,78,144,144,71,164,75,111,235,147,24,211,146,8,
  51,193,30,11,22,142,3,206,201,189,80,71,86,249,41,207,229,248,26,34,1,143,151,105,240,253,33,88,106,131,176,216,50,248,229,60,158,193,200,51,
  2,194,152,27,41,43,190,80,123,109,71,64,218,117,77,44,44,212,113,239,222,125,120,94,210,38,80,101,47,42,151,223,150,215,27,207,243,162,64,94,
  128,112,78,182,181,188,188,140,203,151,47,227,234,213,171,88,179,102,13,118,239,222,141,7,31,124,16,27,54,108,136,152,230,195,135,15,227,222,189,123,
  248,232,163,143,48,62,62,142,171,87,175,226,230,205,155,24,24,24,192,154,53,107,80,171,213,176,180,180,4,203,178,82,14,61,180,58,97,185,92,142,
  28,54,68,91,105,129,23,78,47,237,186,46,202,229,114,100,107,39,172,245,196,223,12,195,8,235,56,84,42,88,181,106,21,90,173,22,22,22,22,176,
  188,188,140,102,179,217,6,205,30,138,197,18,10,133,34,128,110,52,155,13,84,42,30,130,192,133,235,54,177,188,188,20,201,103,99,187,58,217,115,90,
  172,140,62,1,213,104,59,188,4,18,163,204,217,204,137,19,3,234,29,173,158,211,233,52,250,128,45,198,194,73,218,242,28,112,71,99,197,48,136,141,
  157,6,128,118,194,246,82,59,24,181,222,213,32,29,154,237,72,145,7,16,169,60,125,105,52,46,58,211,243,91,184,63,121,31,155,134,182,226,250,216,
  135,116,233,195,79,127,250,35,236,223,63,130,47,124,225,179,216,250,192,86,188,246,235,223,162,86,115,209,106,250,88,90,188,139,229,165,123,48,205,112,
  32,181,28,31,110,107,57,186,79,163,209,68,177,84,134,105,22,17,4,205,182,230,113,25,110,171,22,29,1,113,21,138,178,158,41,246,226,244,18,65,
  8,157,56,244,51,166,101,181,7,113,28,57,175,29,220,136,43,151,46,225,201,39,159,198,231,62,247,89,188,254,250,27,97,196,233,212,34,139,58,211,
  52,48,113,231,62,186,43,0,173,24,9,0,211,211,147,168,86,87,161,167,171,23,75,181,249,196,223,154,13,7,149,238,213,108,64,197,45,90,186,231,
  23,207,167,2,215,156,11,194,199,117,90,248,36,218,173,243,139,166,237,150,89,68,242,161,196,209,126,30,38,88,148,246,166,37,175,117,243,130,22,184,
  208,5,53,180,192,133,124,194,161,11,102,84,197,63,142,102,104,119,5,27,60,58,54,150,96,126,228,132,71,225,139,45,39,255,0,128,93,40,224,200,
  225,195,41,224,203,177,235,194,218,142,130,84,122,189,139,151,46,97,239,158,61,17,160,164,68,64,181,90,197,211,79,61,165,125,135,180,223,206,158,61,
  155,146,56,36,158,67,186,68,168,227,61,146,120,14,218,255,50,59,218,108,54,113,238,253,247,19,207,152,149,139,146,53,14,228,190,189,114,229,74,46,
  59,77,209,70,122,125,93,133,81,223,247,49,62,49,129,159,253,252,231,9,64,47,143,127,202,168,211,118,56,142,131,19,39,79,225,216,19,159,97,219,
  190,111,239,94,156,57,123,22,139,139,139,10,23,142,64,2,132,89,224,57,143,100,195,72,237,131,50,80,149,223,85,50,233,157,178,185,65,228,40,161,
  78,72,52,164,178,229,208,144,81,226,218,86,130,5,151,125,152,67,118,209,100,159,75,214,107,135,69,81,204,20,147,72,245,216,161,244,196,100,65,51,
  39,7,147,75,143,135,127,139,251,161,213,242,225,56,46,166,166,22,176,176,176,152,0,155,186,121,207,157,150,203,126,224,34,40,222,185,115,39,134,135,
  135,113,238,220,57,140,141,141,161,187,187,59,53,7,133,47,191,109,219,88,92,92,196,219,111,191,141,247,222,123,15,155,55,111,198,131,15,62,136,173,
  91,183,98,245,234,213,216,176,97,3,54,108,216,128,32,8,112,230,204,25,140,141,141,97,126,126,30,139,139,139,9,41,6,29,163,182,109,71,26,230,
  82,169,20,73,80,168,7,181,140,115,168,86,155,238,145,34,81,81,184,133,52,26,141,132,164,150,222,179,84,42,97,213,170,85,168,215,235,88,88,88,
  72,200,60,194,207,20,96,89,54,92,215,2,96,99,213,170,110,24,70,128,90,109,33,196,68,45,87,146,107,24,146,134,25,9,41,7,61,169,8,159,
  37,205,46,83,230,153,2,117,217,214,46,150,102,36,65,52,173,202,153,36,107,121,55,14,225,20,146,198,142,72,5,93,185,82,176,57,118,32,207,98,
  74,143,36,146,11,167,167,5,35,89,224,129,179,213,227,142,177,184,44,74,241,187,241,59,227,216,185,115,4,161,138,37,180,113,121,244,200,227,152,95,
  92,196,249,15,46,224,238,157,73,124,237,107,207,226,171,95,125,14,191,252,229,239,16,192,69,255,234,245,184,134,75,240,253,101,220,189,123,15,197,82,
  55,124,63,206,154,191,63,53,131,122,125,9,129,223,130,231,135,12,115,171,213,128,231,250,168,148,43,88,32,186,103,217,105,130,11,32,232,203,162,186,
  85,10,162,41,67,29,203,32,124,152,150,149,40,157,62,48,184,30,135,70,14,160,84,42,98,112,112,63,222,126,251,61,92,188,120,5,245,218,82,114,
  115,191,112,14,229,114,33,213,127,11,243,247,113,239,222,44,74,229,2,22,150,146,27,185,227,52,81,41,151,216,119,195,29,165,232,244,196,186,128,73,
  174,252,164,215,78,241,146,14,157,27,76,222,118,119,18,200,209,118,15,14,14,38,164,1,244,20,226,213,215,94,195,232,216,88,20,100,201,237,252,203,
  127,247,239,80,102,60,139,135,134,134,82,165,189,59,145,253,232,88,26,29,155,174,123,127,148,41,188,54,58,170,4,181,244,231,218,232,104,228,64,161,
  122,15,142,227,224,119,175,191,142,167,158,124,50,237,89,141,108,227,123,218,167,191,123,253,117,214,31,218,48,12,56,142,131,95,189,250,42,190,241,181,
  175,41,175,199,57,231,200,247,63,123,238,92,100,201,71,129,164,234,57,82,99,86,42,22,162,74,216,19,254,208,242,58,168,211,63,231,149,48,157,61,
  119,14,183,110,223,142,190,167,92,179,69,0,129,180,238,94,53,103,228,245,153,22,175,208,141,49,57,216,57,52,114,144,113,40,9,63,115,104,100,4,
  191,253,221,239,20,243,94,7,144,243,129,103,33,1,145,25,174,248,57,120,240,156,6,142,136,28,45,124,159,234,223,205,76,230,90,102,238,212,192,57,
  57,14,125,63,136,10,186,8,224,18,178,183,217,192,57,6,184,22,169,54,24,72,229,181,69,50,88,144,40,25,30,4,73,108,64,101,25,50,8,162,
  30,211,245,186,139,102,179,133,201,201,89,56,78,210,52,64,87,183,66,118,160,225,246,89,1,40,171,213,42,158,127,254,121,60,244,208,67,97,16,182,
  111,31,254,219,127,251,111,152,159,159,71,111,111,111,116,77,225,88,34,214,2,234,98,114,235,214,45,220,186,117,11,253,253,253,216,182,109,27,134,135,
  135,209,213,213,133,185,185,57,140,183,215,3,202,38,83,210,166,80,40,68,108,115,185,92,142,216,116,113,125,65,76,114,222,212,130,80,163,159,161,237,
  19,115,183,80,40,160,82,169,160,80,40,68,88,68,46,90,38,24,113,215,117,177,188,188,140,70,163,17,253,67,231,170,235,186,112,93,31,229,114,31,
  42,149,62,4,129,139,165,165,121,184,110,171,253,142,124,226,100,147,36,77,77,83,56,120,4,145,244,66,128,106,154,172,24,123,67,171,177,67,146,60,
  48,152,61,156,183,180,211,25,25,208,228,67,58,231,18,65,135,138,69,214,49,211,42,191,94,238,133,170,192,81,30,205,181,24,244,156,220,33,212,216,
  4,9,29,147,60,57,228,197,155,78,178,187,119,239,227,161,135,74,40,20,187,209,114,230,97,154,38,26,142,139,207,127,238,56,142,31,251,12,130,32,
  172,74,54,208,95,64,177,96,227,189,51,103,49,59,179,20,69,42,167,222,125,27,134,81,72,116,230,244,212,4,94,122,233,37,192,240,19,50,132,122,
  163,129,117,235,55,226,254,228,120,226,104,83,167,165,83,37,67,170,236,3,19,204,147,105,160,167,90,193,150,77,91,80,175,53,209,104,45,99,98,226,
  6,108,179,140,225,225,173,40,20,128,125,123,247,99,118,118,6,182,109,99,211,208,214,72,91,87,42,117,161,84,42,99,203,230,33,216,118,9,190,239,
  192,178,138,104,181,234,88,90,154,197,166,161,45,176,219,22,121,165,146,133,235,55,175,3,129,1,167,229,177,192,144,147,242,228,213,125,171,250,64,6,
  6,114,36,205,125,94,214,61,114,96,250,247,221,110,149,117,157,240,77,230,216,221,200,19,250,246,109,236,216,190,61,213,198,189,123,246,224,157,19,39,
  58,150,173,112,32,69,37,5,145,101,32,170,68,97,238,250,162,170,224,208,198,141,90,176,56,58,58,170,149,143,81,70,59,120,245,213,4,11,220,169,
  87,183,8,86,228,32,86,150,159,252,228,167,63,197,115,207,62,155,56,16,12,218,43,124,214,88,24,159,152,192,169,119,223,85,94,127,116,108,12,254,
  43,175,224,139,79,63,205,126,63,74,226,36,133,29,228,113,251,202,171,175,166,202,127,115,235,138,28,92,26,109,127,104,157,164,71,200,79,78,156,60,
  153,107,92,9,235,68,58,215,84,214,79,220,239,184,96,132,147,164,112,196,200,123,103,206,226,248,177,39,216,57,240,224,190,125,120,255,131,15,18,118,
  133,162,72,86,248,121,139,5,207,114,69,62,53,243,108,40,217,77,202,14,115,18,154,216,81,10,176,44,59,98,201,108,91,248,219,35,23,112,142,159,
  69,246,210,23,71,230,49,48,13,175,77,25,55,234,96,17,75,80,146,172,116,186,82,160,12,208,57,223,102,145,244,40,116,205,212,165,35,100,17,99,
  182,185,80,176,64,11,157,136,162,46,134,97,97,121,185,137,165,165,58,102,103,23,162,96,145,147,120,169,106,65,200,90,102,129,63,132,172,129,226,132,
  197,197,197,104,238,185,174,139,161,161,33,124,253,235,95,199,11,47,188,16,49,198,226,243,113,0,144,124,39,194,81,99,97,97,1,231,206,157,195,135,
  31,126,136,114,185,28,85,160,237,233,233,137,246,44,122,111,225,198,81,46,151,163,223,11,48,43,192,51,119,242,41,216,106,153,76,243,125,63,81,246,
  155,22,108,169,84,42,209,103,132,92,67,236,69,162,93,165,82,41,170,108,232,121,30,26,141,6,28,199,65,179,217,68,189,94,71,163,209,128,101,89,
  81,229,69,207,243,224,56,62,42,149,85,8,75,122,123,104,52,106,112,156,176,30,133,105,250,8,130,216,90,48,102,122,141,246,56,52,163,162,45,133,
  130,205,178,208,166,9,120,94,82,95,205,203,19,227,146,246,42,15,123,254,100,44,32,73,210,6,139,29,19,39,159,244,130,186,210,216,170,6,200,201,
  126,242,192,214,85,42,148,143,86,84,27,33,127,52,19,27,205,235,54,93,153,153,137,179,250,151,96,154,6,6,250,251,113,247,206,18,10,133,2,110,
  223,26,199,203,51,179,168,246,174,66,169,88,198,252,194,2,238,221,187,143,90,109,18,51,179,247,1,56,137,118,123,94,51,181,176,57,78,3,229,210,
  42,24,134,143,98,209,130,105,25,232,237,171,162,86,111,106,251,84,21,148,168,124,146,117,44,109,87,165,130,158,158,110,60,251,39,207,192,107,5,112,
  253,118,98,86,96,192,15,60,32,8,112,244,209,67,8,124,15,166,25,26,231,183,92,15,149,74,25,165,82,1,115,115,139,112,91,30,90,174,211,30,
  224,126,187,48,67,1,155,54,15,225,145,195,135,97,89,38,166,167,238,227,230,237,155,88,88,152,198,155,111,189,165,124,95,43,213,67,211,64,139,43,
  180,194,49,218,121,138,248,168,218,211,105,187,117,236,57,215,238,189,82,50,27,5,42,89,76,248,212,212,20,182,15,15,167,158,69,184,63,124,212,78,
  66,233,180,78,158,234,40,53,107,108,170,244,226,220,34,51,58,58,170,212,61,83,144,157,87,182,245,209,181,107,88,92,90,194,163,71,143,98,104,227,
  70,109,144,37,131,218,19,39,79,98,106,106,138,149,231,112,159,255,63,47,189,132,207,30,59,22,37,61,230,97,109,79,158,58,21,201,67,228,228,55,
  250,253,107,163,163,88,248,225,15,241,248,99,143,165,250,135,91,243,76,211,132,31,4,184,121,251,54,78,158,60,137,233,233,105,182,40,70,150,12,142,
  38,46,234,158,225,204,217,179,185,79,94,184,177,146,103,254,116,226,253,172,2,78,151,46,95,198,67,135,70,208,211,211,19,101,210,211,175,30,26,25,
  193,235,111,188,33,109,144,166,150,121,206,2,207,113,242,97,82,243,76,51,251,147,224,54,157,176,43,124,153,5,67,39,88,183,88,235,107,177,236,152,
  240,120,230,220,50,40,227,28,2,147,24,72,8,86,215,178,204,40,225,42,4,221,129,228,173,204,3,231,240,153,109,9,52,211,226,26,114,121,237,164,
  220,128,74,51,12,163,208,150,168,32,1,156,195,247,23,192,243,12,204,205,45,161,94,111,97,113,113,137,173,110,201,145,102,116,221,85,73,240,132,148,
  225,225,135,31,198,254,253,251,209,211,211,131,70,163,129,197,197,69,220,188,121,19,99,99,99,120,240,193,7,163,226,40,71,142,28,65,179,217,196,235,
  175,191,158,144,63,8,95,101,1,98,203,229,50,42,149,74,4,218,186,187,187,19,121,0,66,174,33,3,123,241,61,145,88,44,18,30,169,227,131,40,
  150,66,157,53,228,49,69,19,11,69,63,9,0,44,116,207,130,109,22,56,34,100,144,195,223,183,90,173,72,170,65,101,72,52,96,16,224,187,171,171,
  43,2,212,203,203,203,81,5,70,145,111,210,106,181,224,251,64,165,210,139,114,217,135,227,52,208,106,53,81,175,47,71,129,161,184,174,231,197,154,123,
  90,192,37,237,68,20,207,181,184,128,18,5,204,70,226,191,101,178,146,59,77,166,58,237,120,156,208,50,230,62,203,60,107,25,232,60,199,191,121,172,
  231,242,128,36,85,85,170,172,227,118,142,25,167,142,7,89,94,212,182,93,132,93,40,96,227,134,117,120,234,243,143,161,218,183,10,134,105,192,115,3,
  52,155,13,212,235,13,44,47,55,176,99,120,51,156,86,163,109,183,99,32,240,91,176,108,27,182,97,193,176,45,88,102,88,162,213,52,45,148,43,93,
  40,22,45,20,11,69,192,52,97,91,161,19,134,101,154,184,113,227,46,171,235,83,1,66,174,79,84,73,15,242,231,70,71,175,226,7,63,104,70,22,
  103,134,97,182,203,126,58,240,189,48,50,68,187,242,147,105,152,240,131,112,194,238,216,57,130,39,62,51,130,239,125,247,127,163,233,44,74,3,207,132,
  97,132,73,76,149,74,55,108,219,196,242,210,98,212,158,90,109,129,237,235,60,114,13,142,109,146,251,73,197,126,170,2,55,29,243,197,5,33,43,105,
  119,30,96,32,174,185,125,120,24,37,198,114,75,176,157,170,211,7,241,253,137,59,119,120,32,11,96,219,240,48,174,141,142,134,115,162,125,244,111,96,
  229,222,213,157,0,38,14,240,203,44,249,71,215,174,225,200,225,195,169,178,217,226,57,175,141,142,230,102,246,197,162,56,53,53,133,159,190,252,50,6,
  6,6,176,113,195,6,172,93,187,150,181,198,155,154,154,194,232,216,24,110,143,143,167,250,89,126,78,238,89,166,166,166,240,131,31,254,16,67,27,55,
  98,96,96,0,195,237,12,122,142,81,158,156,156,76,184,116,100,73,123,128,208,245,67,60,199,166,182,59,137,234,57,174,141,141,97,92,122,142,44,79,
  120,110,238,168,218,114,238,253,247,209,104,52,240,254,7,31,164,2,37,110,222,177,243,227,19,168,252,24,109,242,140,221,159,188,95,136,255,126,253,141,
  55,241,236,51,127,140,0,233,179,218,189,123,246,224,236,185,115,88,92,92,36,27,98,176,34,205,51,5,176,220,105,18,7,158,105,49,150,248,115,73,
  111,92,193,120,133,82,144,32,242,164,165,99,149,22,22,225,165,32,201,103,16,64,89,176,117,102,123,175,162,71,232,225,156,50,72,49,18,89,166,33,
  10,165,24,145,111,115,60,183,147,242,142,116,245,197,100,81,23,234,176,17,179,216,73,176,238,186,192,244,244,60,150,151,235,104,52,154,81,159,113,236,
  42,183,222,115,210,134,152,29,117,34,96,251,204,51,207,224,177,199,30,67,119,119,119,98,108,135,192,47,100,138,99,127,96,19,199,143,31,199,225,195,
  135,163,207,212,235,117,204,205,205,181,75,129,135,69,140,238,223,191,143,91,183,110,37,112,136,0,179,2,208,186,174,27,73,31,104,249,110,250,30,101,
  39,26,97,22,32,227,31,42,187,16,146,24,225,82,68,221,140,162,202,164,228,59,28,225,35,218,35,147,158,162,13,242,154,32,146,43,41,48,167,174,
  32,226,186,173,150,143,98,177,2,199,241,80,173,174,198,210,210,92,20,208,137,42,150,244,212,71,39,89,163,167,31,84,67,45,39,14,242,201,132,233,
  156,140,152,136,165,30,228,249,213,18,198,127,249,235,191,14,68,167,188,240,247,127,175,221,40,169,173,144,14,228,210,1,193,249,153,114,90,201,143,3,
  86,56,112,153,101,250,95,46,117,225,201,39,191,140,223,252,246,183,88,191,110,13,186,186,87,225,222,189,59,176,76,27,229,114,9,125,125,189,232,235,
  91,141,222,190,42,76,3,176,11,69,4,1,240,192,3,235,177,184,88,199,196,157,123,104,57,173,118,231,3,174,219,196,252,194,28,156,166,131,229,229,
  37,52,155,13,184,46,208,168,47,34,8,76,52,154,13,184,110,77,155,193,174,2,126,42,77,170,14,120,234,22,127,149,150,177,218,59,136,175,127,245,
  89,124,247,31,254,1,142,219,236,168,42,29,87,215,94,110,39,183,64,200,172,153,42,233,40,143,142,50,15,27,253,135,212,110,221,194,153,39,32,80,
  5,83,242,117,84,115,47,111,48,67,131,148,60,197,47,242,22,200,208,157,78,117,58,247,85,243,93,39,3,234,132,52,200,51,87,117,253,204,5,122,
  157,202,150,242,50,224,28,169,144,37,113,234,244,164,65,119,29,213,56,229,8,130,78,219,164,27,235,244,189,232,175,223,25,120,78,39,46,243,132,142,
  46,89,144,178,187,177,222,56,201,24,203,140,151,12,154,40,32,160,185,69,20,56,211,90,11,161,237,156,65,10,140,120,164,154,158,129,108,125,179,145,
  106,91,216,46,91,105,65,71,181,206,242,179,81,176,227,121,33,168,14,131,6,192,113,124,76,78,206,97,97,97,49,33,151,227,152,101,213,184,18,191,
  163,245,32,68,193,148,53,107,214,96,245,234,213,152,159,159,199,220,220,28,138,197,34,122,122,122,162,42,126,229,114,57,250,255,107,215,174,197,222,189,
  123,209,215,215,23,205,31,207,243,176,176,176,128,86,171,21,89,201,9,0,217,104,52,112,239,222,61,188,241,198,27,24,27,27,139,228,22,212,74,79,
  84,29,20,44,174,176,152,19,12,49,45,124,34,202,125,83,91,58,106,83,71,165,35,242,239,197,179,211,130,47,178,113,129,96,184,197,94,233,56,14,
  92,215,141,130,7,241,93,209,191,194,149,164,217,108,70,197,161,124,223,199,242,242,114,196,62,211,235,209,235,11,9,137,40,57,30,218,228,249,104,52,
  106,81,112,21,51,193,65,84,28,39,198,115,134,36,195,9,162,207,197,1,90,122,157,149,43,136,210,96,54,189,191,33,74,96,12,247,75,253,154,252,
  239,255,237,183,211,12,180,76,109,115,12,39,117,26,160,201,109,50,88,206,167,111,14,216,8,65,142,20,56,109,159,238,8,62,43,185,73,252,173,209,
  172,225,229,159,253,19,0,31,163,99,51,169,197,56,254,124,104,28,111,89,33,123,118,252,216,103,80,237,237,199,203,63,253,9,12,195,205,181,161,203,
  128,46,171,141,58,118,89,119,188,201,129,24,57,168,224,218,37,6,72,193,46,163,92,174,160,167,218,139,217,185,41,229,166,154,39,105,79,182,162,211,
  233,248,84,192,141,3,118,242,231,84,229,209,63,13,237,150,159,65,39,69,200,115,234,195,49,10,42,253,114,22,232,227,164,1,242,17,42,189,174,96,
  107,40,219,162,179,106,84,1,197,78,193,153,170,205,170,83,45,110,189,203,10,32,84,192,54,207,154,196,245,183,74,15,43,59,23,201,235,160,110,157,
  204,19,56,171,156,138,116,39,90,121,146,185,185,192,82,53,78,117,123,131,202,187,93,182,246,164,253,167,91,43,243,147,11,122,230,89,7,158,147,22,
  109,113,162,147,108,193,39,220,52,104,177,146,24,64,24,172,79,114,82,74,194,187,14,201,62,206,161,44,36,201,108,39,247,239,32,193,74,199,86,121,
  70,166,190,89,140,79,181,5,29,34,191,106,185,80,76,186,36,185,133,86,203,195,242,114,19,179,179,11,81,41,108,81,240,67,222,51,57,143,123,238,
  100,153,142,71,199,113,208,215,215,135,47,127,249,203,120,228,145,71,208,211,211,131,165,165,37,156,61,123,22,167,79,159,198,216,216,24,110,221,186,21,
  201,19,40,0,220,187,119,47,254,243,127,254,207,232,238,238,142,216,221,139,23,47,226,7,63,248,1,122,123,123,209,221,221,141,254,254,126,244,246,246,
  226,222,189,123,184,209,174,160,90,169,84,34,208,222,223,223,31,37,15,10,160,42,218,40,164,18,2,100,10,0,219,108,54,35,176,45,52,200,50,118,
  160,108,177,240,109,150,215,97,1,174,105,31,137,0,74,252,183,72,8,20,12,179,96,173,197,61,196,191,133,11,135,0,195,245,122,29,245,122,61,2,
  206,50,1,68,115,32,196,251,16,18,22,195,48,176,188,188,76,222,35,34,135,16,203,178,225,251,94,226,59,233,194,89,66,3,157,244,100,151,223,127,
  216,39,178,187,6,103,52,74,215,168,160,35,146,38,37,225,208,177,28,177,141,137,173,5,11,170,100,171,180,245,142,48,131,87,103,105,115,108,169,142,
  73,211,1,25,126,131,243,89,86,137,10,248,219,208,28,190,95,15,51,129,107,14,182,110,173,106,59,91,87,16,35,11,252,102,105,2,85,207,43,179,
  126,92,73,115,221,105,128,88,32,77,203,128,105,4,172,92,132,250,80,115,229,102,185,62,207,235,168,194,89,212,169,222,115,214,231,62,45,237,86,217,
  181,169,0,68,30,6,82,220,131,171,216,167,2,82,178,165,17,167,51,149,79,159,228,141,147,91,23,116,185,15,157,50,207,43,41,136,163,10,30,178,
  214,26,93,176,210,169,125,34,13,34,178,10,230,168,18,158,179,130,27,221,179,171,198,37,87,180,170,83,198,159,43,133,172,235,19,186,185,171,214,71,
  46,240,227,108,206,184,126,82,7,37,186,160,22,164,124,119,90,75,153,150,72,168,246,51,122,148,156,238,155,48,137,206,32,213,250,40,35,205,207,35,
  234,174,65,63,231,186,94,148,116,39,24,245,176,47,132,198,217,104,39,107,65,2,188,161,195,135,105,198,14,26,97,121,237,216,233,34,236,51,131,148,
  220,78,123,55,211,228,42,21,112,150,25,108,81,38,220,243,124,212,106,45,52,26,30,166,167,103,163,53,89,232,116,85,39,15,92,192,79,255,237,251,
  126,59,89,173,5,219,182,35,25,149,24,3,98,236,85,171,85,28,63,126,28,79,60,241,4,174,92,185,130,15,62,248,0,23,46,92,192,196,196,4,
  76,211,68,87,87,23,76,211,196,149,43,87,240,227,31,255,24,127,246,103,127,22,141,177,71,31,125,20,87,174,92,193,169,83,167,80,44,22,113,227,
  70,88,56,172,80,40,68,223,115,93,23,142,227,160,86,171,97,231,206,157,216,179,103,15,26,141,6,38,38,38,112,245,234,213,8,216,11,205,180,208,
  61,211,68,63,33,193,16,73,139,221,221,221,17,0,165,140,178,124,202,68,231,151,248,156,0,231,50,70,19,9,134,2,64,139,127,168,14,154,234,166,
  93,215,69,189,94,143,60,162,101,73,135,88,239,41,240,22,160,155,206,209,56,72,70,162,152,138,176,57,12,157,55,144,208,35,243,39,182,98,158,161,
  157,120,8,38,105,52,136,18,107,147,243,93,38,150,244,235,184,110,125,180,243,30,33,202,27,164,188,216,201,85,227,100,167,137,112,115,42,3,104,181,
  179,47,131,76,134,149,99,223,228,242,180,42,141,112,158,13,90,174,48,20,3,39,51,242,28,108,195,211,120,129,43,88,176,11,22,44,187,8,207,173,
  117,84,125,145,130,17,14,68,112,27,131,204,222,168,152,51,186,73,231,217,84,184,182,45,46,78,133,229,41,237,82,226,40,76,158,124,34,90,231,128,
  84,156,89,110,104,65,70,94,73,129,234,89,57,45,166,232,219,79,75,187,229,177,190,210,164,197,172,160,130,243,60,229,0,176,78,79,171,99,116,131,
  32,136,116,122,169,68,53,224,99,251,116,171,174,151,71,91,159,245,142,56,41,141,46,161,89,197,228,118,82,249,138,123,22,90,46,86,14,228,84,1,
  65,214,250,41,123,196,102,185,39,117,26,204,112,197,41,116,63,244,104,61,139,201,151,53,234,130,5,148,247,2,221,124,141,247,172,128,97,152,211,137,
  119,28,112,215,201,54,228,196,57,89,218,24,239,95,62,68,129,146,116,73,106,57,127,192,111,39,15,210,189,54,217,14,145,124,72,253,117,197,115,10,
  223,105,90,190,155,130,224,176,79,12,194,128,39,19,5,99,247,3,35,23,219,76,89,240,248,181,26,41,0,237,121,192,242,178,131,185,185,69,44,45,
  213,162,253,143,22,57,226,220,180,100,86,145,243,117,118,93,23,141,70,3,253,253,253,24,25,25,193,174,93,187,80,171,213,240,193,7,31,224,236,217,
  179,120,225,133,23,176,121,243,102,236,223,191,31,35,35,35,81,249,237,61,123,246,96,207,158,61,56,118,236,24,174,94,189,138,247,222,123,15,87,175,
  94,133,235,186,40,22,139,120,251,237,183,241,224,131,15,226,193,7,31,140,92,53,128,208,87,89,184,94,136,241,90,171,213,80,40,20,240,216,99,143,
  97,211,166,77,152,152,152,192,216,216,24,110,223,190,141,129,129,129,168,16,145,227,56,168,86,171,48,77,19,181,90,45,122,126,161,93,166,251,190,105,
  154,81,80,176,188,188,140,74,165,130,74,165,130,238,238,238,68,178,158,92,35,66,46,102,71,25,101,106,21,41,112,0,181,178,19,224,88,4,32,190,
  239,99,105,105,41,74,24,148,223,131,44,249,16,123,1,197,105,34,64,16,133,89,74,165,34,154,77,7,52,1,149,226,147,176,156,120,64,78,4,228,
  117,193,87,158,22,165,177,105,252,185,88,239,47,39,233,203,39,96,217,251,176,200,51,178,229,133,81,116,46,7,138,85,192,84,109,227,3,137,57,48,
  19,17,190,186,208,10,82,47,41,185,129,27,108,73,85,250,255,57,160,40,3,108,174,92,166,158,202,55,225,56,181,80,210,97,22,164,82,35,249,153,
  155,60,155,172,138,133,84,73,60,184,106,142,42,25,137,10,4,56,142,135,102,179,133,66,177,160,220,36,117,242,9,57,185,65,165,75,229,52,191,58,
  86,143,107,179,10,12,124,154,218,173,147,251,208,123,234,18,26,229,141,158,234,138,243,72,127,68,224,160,210,35,83,45,161,14,160,11,15,212,60,64,
  255,227,0,55,153,121,201,210,90,103,73,44,242,206,63,157,37,27,95,164,35,121,122,167,122,39,156,244,100,37,178,27,238,20,81,247,204,121,8,6,
  85,176,205,129,69,221,154,38,78,199,184,247,160,146,40,201,253,205,141,47,249,243,252,243,170,214,242,152,221,162,122,96,46,235,94,199,96,171,10,151,
  36,11,130,88,18,27,108,176,128,52,4,253,86,226,52,36,201,44,134,250,99,10,104,67,7,3,225,106,96,179,85,3,121,43,58,68,68,81,82,38,
  18,39,28,170,128,179,58,41,80,150,110,24,168,213,28,44,44,212,80,171,53,208,108,58,145,166,151,130,46,185,48,22,37,127,196,251,144,171,237,81,
  240,220,221,221,141,231,158,123,14,35,35,35,24,28,28,140,174,255,240,195,15,227,139,95,252,34,222,122,235,45,188,251,238,187,248,201,79,126,130,223,
  254,246,183,216,179,103,15,30,123,236,49,28,56,112,0,182,109,99,221,186,117,216,184,113,35,30,121,228,17,220,190,125,27,167,78,157,194,197,139,23,
  49,59,59,139,31,255,248,199,168,215,235,184,119,239,30,78,159,62,141,185,185,57,148,203,229,68,181,66,207,243,240,216,99,143,97,253,250,245,168,213,
  194,224,96,243,230,205,184,123,247,46,110,222,188,137,233,233,233,168,232,74,169,84,194,225,118,66,245,252,252,60,238,221,187,135,251,247,239,39,72,12,
  145,120,24,73,43,11,133,72,222,49,59,59,139,82,169,132,114,185,140,158,158,158,72,63,77,247,124,234,230,33,146,21,155,205,166,182,159,5,235,45,
  220,69,132,102,89,120,64,203,100,41,197,35,226,191,57,182,91,92,135,178,210,149,74,9,205,102,144,56,225,136,199,155,169,192,146,1,89,187,76,201,
  65,67,5,162,131,200,202,49,121,34,45,207,103,36,124,220,195,83,26,53,118,4,218,213,99,161,112,225,224,216,137,180,15,179,207,178,34,170,13,50,
  124,224,38,97,120,245,18,15,122,44,69,45,118,66,246,218,79,21,213,160,19,78,7,244,69,180,198,37,182,100,51,101,62,108,59,156,48,158,219,210,
  178,93,156,21,153,204,40,173,36,193,73,126,102,149,219,137,106,243,85,49,101,226,152,177,88,180,163,8,153,123,46,121,19,84,121,7,103,29,133,168,
  202,149,203,127,151,51,144,179,164,49,220,98,191,146,118,63,251,204,51,74,235,181,75,151,47,71,197,25,62,78,187,179,0,56,13,152,42,149,10,254,
  226,207,254,44,42,231,252,183,47,188,16,105,234,232,181,178,128,81,181,90,197,159,63,255,60,0,224,199,63,249,9,110,221,190,141,191,250,206,119,96,
  89,22,190,251,226,139,9,191,220,60,239,244,208,200,8,14,63,242,8,130,32,192,127,255,31,255,35,213,207,255,249,63,253,39,0,192,223,127,247,187,
  88,94,94,238,56,33,77,197,34,7,65,128,213,171,87,227,248,177,99,216,52,52,20,177,59,191,123,253,245,200,27,121,120,219,54,60,122,244,40,170,
  213,42,130,32,192,201,83,167,162,210,231,226,186,197,98,17,223,252,250,215,209,83,173,226,187,223,251,94,196,50,137,251,141,28,60,136,199,30,125,20,
  151,46,95,198,235,111,188,145,169,159,151,219,46,203,55,242,104,185,243,48,237,220,184,211,157,66,228,9,234,117,107,190,138,93,207,163,101,87,57,10,
  113,12,180,44,13,201,59,62,248,207,6,140,204,2,25,44,83,50,129,79,229,17,45,95,143,95,143,146,123,89,188,159,241,228,19,77,212,75,230,29,
  88,74,48,108,89,65,219,203,58,47,112,54,152,170,134,134,196,74,155,74,55,13,234,0,34,91,208,81,251,188,197,197,58,102,102,230,209,34,245,1,
  228,96,87,247,238,104,16,35,187,77,80,198,218,117,93,244,244,244,224,161,135,30,194,224,224,32,0,68,242,135,66,161,128,237,219,183,99,251,246,237,
  120,230,153,103,240,246,219,111,227,244,233,211,120,239,189,247,112,230,204,25,108,222,188,25,143,61,246,24,30,126,248,97,172,89,179,6,93,93,93,216,
  189,123,55,118,237,218,133,169,169,41,252,237,223,254,45,198,198,198,240,247,127,255,247,137,164,188,70,163,129,114,185,140,161,161,33,212,235,117,76,78,
  78,226,214,173,91,81,130,225,219,111,191,141,123,247,238,69,201,126,93,93,93,216,177,99,7,74,165,18,174,93,187,134,183,223,126,27,235,215,175,71,
  95,95,31,106,181,90,36,167,224,136,16,193,50,211,253,64,36,240,205,204,204,68,190,210,98,207,22,32,92,200,39,196,53,28,199,73,20,88,161,118,
  121,194,202,78,0,104,81,94,188,94,175,39,238,45,215,214,160,239,69,220,143,130,105,250,174,40,211,237,121,173,104,92,5,65,204,40,83,102,153,46,
  247,2,239,37,11,189,210,202,132,42,18,36,36,61,5,64,22,101,195,57,201,70,50,160,200,143,29,109,21,131,75,23,72,174,164,33,39,13,144,23,
  222,236,99,77,67,193,18,208,172,229,56,241,34,239,194,42,51,203,50,48,207,167,191,52,224,121,105,16,212,108,54,81,176,45,216,133,34,188,102,45,
  55,208,144,153,137,172,141,71,231,129,154,39,81,82,231,8,193,89,230,9,255,84,211,180,80,169,148,83,73,156,170,126,231,178,252,105,2,0,13,106,
  56,192,201,129,106,250,252,50,0,145,251,134,203,202,255,36,218,173,3,13,159,84,187,185,49,32,111,28,98,14,13,111,219,22,129,103,32,44,83,124,
  246,220,185,76,38,94,5,88,168,172,229,255,251,55,127,147,57,166,228,10,152,89,12,172,232,199,255,247,255,253,127,43,221,118,242,0,61,238,29,136,
  181,230,248,177,99,137,32,167,88,44,226,233,167,158,194,119,95,124,17,0,18,197,86,12,195,192,209,163,71,225,56,14,46,94,186,148,2,216,116,142,
  249,190,143,114,185,140,71,143,30,197,222,61,123,216,66,12,31,71,126,146,165,21,238,244,116,138,123,87,157,104,204,179,170,116,170,36,21,170,231,225,
  198,36,119,76,175,2,225,121,229,63,217,201,164,114,105,105,164,142,140,229,75,8,235,55,161,217,229,239,103,66,46,21,156,148,27,132,58,100,202,52,
  199,39,175,233,211,5,10,126,105,34,169,156,32,152,254,188,209,174,228,102,72,192,89,174,96,168,2,206,96,88,105,164,116,217,241,158,44,252,169,205,
  20,112,246,188,240,158,243,243,117,76,78,78,163,213,138,181,205,180,127,232,209,190,202,112,64,62,69,22,0,208,243,60,172,94,189,26,171,87,175,142,
  92,49,150,151,151,113,227,198,13,252,215,255,250,95,241,205,111,126,19,135,14,29,138,24,219,51,103,206,68,82,141,117,235,214,225,217,103,159,197,151,
  190,244,37,156,57,115,6,39,78,156,192,216,216,24,190,255,253,239,227,23,191,248,5,142,30,61,138,35,71,142,160,175,175,15,51,51,51,56,113,226,
  4,38,39,39,81,169,84,18,227,177,82,169,96,221,186,117,24,30,30,134,231,121,216,177,99,7,110,221,186,133,83,167,78,225,212,169,83,9,54,216,
  178,44,60,248,224,131,24,24,24,192,220,220,28,214,173,91,135,237,219,183,227,212,169,83,248,240,195,15,97,89,22,202,229,50,202,229,114,202,249,130,
  218,193,81,112,45,179,243,194,139,90,20,56,161,21,57,125,223,71,165,82,137,254,70,19,37,69,130,161,0,205,226,247,243,243,243,152,153,153,137,152,
  116,85,193,55,234,178,65,231,175,144,111,80,141,53,5,217,98,44,133,32,53,44,168,226,121,32,201,183,113,162,107,252,172,180,24,79,192,98,53,145,
  83,199,159,252,202,249,13,250,226,43,113,85,80,69,158,152,170,18,161,170,212,114,158,197,88,245,25,26,73,197,15,22,100,28,175,241,199,111,201,35,
  92,81,166,218,32,145,133,248,140,13,195,240,152,5,154,59,198,143,191,159,100,137,121,160,237,187,1,74,165,2,10,5,11,205,38,47,35,209,153,111,
  119,226,177,171,219,48,116,197,109,178,64,31,39,111,8,163,249,22,252,192,71,119,87,37,145,141,171,74,156,18,160,138,99,83,35,45,158,97,192,178,
  45,88,150,40,67,234,195,182,67,187,158,74,185,130,238,174,30,52,26,75,24,189,113,93,203,140,233,228,62,242,51,126,252,118,3,63,125,249,101,248,
  190,143,205,155,54,225,217,103,158,1,0,124,247,197,23,177,176,176,16,89,19,9,128,5,132,62,189,39,78,158,196,237,241,113,24,134,129,167,159,122,
  10,219,135,135,67,255,222,209,81,60,122,244,40,46,94,186,132,55,222,124,19,199,143,29,195,222,61,123,34,86,83,176,221,175,191,241,6,46,94,186,
  196,50,153,219,183,111,7,128,168,170,223,240,182,109,56,123,238,28,60,207,195,254,7,31,196,241,99,199,176,184,184,136,239,190,248,34,14,141,140,224,
  232,145,35,209,245,7,6,6,240,197,167,158,66,181,90,197,201,83,167,82,167,17,130,37,22,12,116,127,127,63,30,61,122,52,98,118,71,199,198,112,
  226,228,73,150,157,22,139,40,87,132,38,8,2,252,167,239,124,7,8,2,124,247,197,23,97,154,38,254,252,249,231,177,184,184,136,95,189,250,106,4,
  126,199,39,38,34,111,227,145,131,7,241,232,209,163,209,59,62,115,246,44,78,158,58,197,142,227,77,67,67,8,130,0,175,191,241,6,174,141,142,70,
  12,125,111,181,26,177,81,162,79,142,30,57,130,145,131,7,177,113,227,70,92,188,116,9,123,247,236,193,177,39,158,72,121,54,139,13,236,235,95,253,
  42,250,250,250,224,56,78,20,184,200,62,170,98,222,232,164,14,89,146,143,60,160,48,111,41,238,78,146,51,57,189,174,170,50,35,119,26,192,5,83,
  170,83,29,89,250,148,229,152,162,115,72,201,98,207,99,160,239,39,172,229,212,12,149,138,232,224,11,125,133,137,120,1,145,62,36,215,152,120,127,51,
  35,223,100,241,255,147,197,34,232,51,134,155,191,14,60,171,152,100,81,172,36,249,217,100,249,239,88,55,74,139,200,24,25,86,116,201,123,211,100,198,
  216,191,215,128,235,6,112,93,31,51,51,139,88,88,88,138,176,64,169,100,105,61,135,105,192,42,75,1,40,193,36,198,228,250,245,235,177,107,215,46,
  28,62,124,24,91,183,110,69,171,213,138,220,47,174,95,191,142,177,177,49,252,237,223,254,45,30,121,228,17,124,249,203,95,198,208,208,16,30,126,248,
  97,188,246,218,107,248,31,255,227,127,96,247,238,221,120,236,177,199,48,52,52,132,199,30,123,12,143,63,254,56,46,93,186,132,139,23,47,226,220,185,
  115,248,213,175,126,133,19,39,78,96,205,154,53,152,156,156,140,88,102,65,154,217,182,141,122,189,142,109,219,182,97,255,254,253,216,178,101,11,222,122,
  235,45,92,186,116,41,2,178,194,154,78,176,224,245,122,29,189,189,189,145,94,90,36,31,206,207,207,71,22,118,66,162,35,203,35,132,12,69,36,89,
  202,36,154,232,23,97,121,7,132,174,26,64,168,199,166,182,118,194,166,142,94,223,117,221,72,22,37,216,103,161,115,166,73,137,226,31,218,6,170,103,
  22,128,82,78,72,148,3,38,193,114,135,160,217,148,152,93,51,98,139,67,166,152,119,207,224,244,241,58,137,159,250,132,80,247,247,32,243,132,141,174,
  11,182,106,17,92,169,47,179,106,113,12,255,109,69,199,62,105,240,76,197,225,52,162,182,0,184,210,67,26,100,97,52,17,4,158,36,113,176,0,88,
  48,96,2,6,96,91,6,138,197,50,2,180,128,192,79,28,51,132,155,129,133,82,185,130,74,185,140,98,177,130,74,165,140,158,158,110,20,10,69,248,
  190,131,192,7,6,7,215,162,191,127,53,122,251,250,80,40,218,168,86,87,97,105,105,90,33,133,48,177,127,223,67,216,176,126,29,138,165,18,234,245,
  101,44,215,150,49,62,62,129,91,183,175,167,216,188,244,209,165,90,135,45,91,71,201,206,9,162,127,10,118,23,74,229,34,44,211,66,179,89,71,171,
  229,192,178,12,20,139,93,176,108,27,150,9,88,86,9,229,74,21,5,59,64,161,96,162,183,186,26,165,162,141,221,123,246,193,64,25,126,224,192,119,
  3,120,65,40,21,240,61,31,197,66,23,188,160,5,207,117,162,100,195,176,143,77,152,86,17,129,239,160,88,42,163,88,46,133,182,52,110,11,166,225,
  3,102,17,240,131,232,181,6,129,137,229,229,5,148,138,221,216,178,117,19,230,127,240,61,76,206,204,100,110,160,121,108,189,100,38,87,119,26,192,233,
  155,116,64,71,124,231,139,79,63,141,161,141,27,163,246,244,247,247,71,236,231,246,225,97,108,31,30,6,0,12,12,12,160,183,183,55,87,208,36,198,
  164,0,242,226,254,133,66,1,27,55,108,0,0,252,238,245,215,241,231,207,63,143,129,129,1,12,12,12,36,138,106,136,231,151,159,229,139,79,61,133,
  238,238,110,248,190,143,35,135,15,107,65,73,161,80,192,179,207,60,131,50,41,122,50,188,109,27,74,165,18,126,250,242,203,202,163,45,213,130,100,182,
  119,105,186,174,20,10,5,60,253,228,147,232,237,237,141,130,143,31,252,240,135,120,232,208,33,60,242,240,195,56,123,238,28,222,126,231,29,124,254,115,
  159,195,161,145,17,56,142,131,115,239,191,159,90,163,254,63,255,253,191,71,191,219,187,103,15,138,197,34,28,199,193,253,201,201,8,64,139,42,143,83,
  83,83,48,12,3,107,7,7,163,239,95,190,114,5,103,207,157,139,36,45,180,63,150,150,151,241,254,7,31,160,84,42,225,232,145,35,202,96,76,183,
  144,235,60,201,243,176,190,121,130,73,249,61,100,121,202,115,155,141,172,107,204,178,200,228,116,244,156,5,159,106,126,102,157,36,170,72,0,238,244,66,
  213,222,112,15,128,150,125,206,27,100,196,107,171,217,6,136,241,30,148,182,91,51,34,192,44,235,159,195,231,15,72,176,158,78,134,148,165,31,114,110,
  14,181,156,83,21,65,137,11,160,196,215,16,199,210,241,49,120,192,202,222,40,251,44,187,105,136,223,153,166,1,199,241,81,175,59,152,158,158,131,227,
  180,82,54,183,89,18,67,89,2,160,178,172,117,28,7,135,14,29,194,159,254,233,159,70,235,133,56,57,91,191,126,61,214,173,91,135,207,126,246,179,
  104,52,26,184,122,245,42,206,159,63,143,247,222,123,15,229,114,25,253,253,253,248,198,55,190,129,179,103,207,226,165,151,94,194,155,111,190,137,163,71,
  143,226,240,225,195,216,185,115,39,246,238,221,139,157,59,119,98,113,113,17,83,83,83,104,181,90,184,115,231,78,196,12,215,235,117,148,203,101,28,61,
  122,20,221,221,221,88,88,88,192,173,91,183,240,242,203,47,227,224,193,131,0,128,19,39,78,96,97,97,1,229,114,25,173,86,11,219,183,111,199,241,
  227,199,209,108,54,113,251,246,109,188,250,234,171,232,234,234,194,234,213,171,35,61,177,208,46,203,122,97,154,111,34,136,71,122,106,39,39,202,11,12,
  33,250,94,204,255,70,163,1,195,48,80,42,149,176,176,176,192,214,240,16,125,43,52,207,194,93,67,56,145,208,83,90,202,30,183,90,173,168,32,141,
  124,178,32,128,182,0,253,148,85,23,82,148,114,185,152,192,107,50,131,76,229,27,244,148,150,86,32,228,100,30,121,200,199,78,188,255,195,251,90,153,
  36,129,157,165,229,211,105,254,58,105,104,184,200,38,153,229,82,177,11,3,131,253,237,196,8,3,93,149,18,170,189,189,40,23,203,128,97,160,213,10,
  171,230,249,129,3,167,222,132,143,0,133,98,1,69,187,12,203,14,75,80,135,108,102,1,93,221,21,20,11,37,4,190,7,47,240,209,211,93,133,239,
  249,176,236,208,199,217,182,44,4,8,218,165,172,205,118,212,239,3,1,96,218,54,130,192,128,239,57,88,211,191,26,243,243,117,44,204,207,194,135,1,
  183,93,35,190,92,46,97,177,86,199,221,251,83,24,24,88,11,207,227,55,135,112,48,87,112,248,200,97,76,77,223,195,212,228,60,96,4,232,238,233,
  199,150,7,202,152,153,153,197,114,109,94,201,34,109,123,96,15,238,221,187,131,106,111,55,26,13,7,253,253,3,168,215,22,49,126,231,22,186,187,170,
  248,242,151,255,24,197,66,1,139,75,75,161,238,200,117,17,248,62,138,229,18,236,66,25,65,224,160,84,232,130,93,16,90,40,15,129,239,195,245,27,
  48,97,193,48,172,118,6,169,1,191,205,22,120,174,3,215,245,224,182,60,76,77,205,163,218,83,194,236,194,52,154,141,22,60,175,133,0,1,2,63,
  64,248,173,5,184,94,29,161,73,127,1,65,0,120,158,11,4,1,188,118,117,161,70,99,41,138,92,155,205,38,124,63,44,235,25,34,103,0,134,159,
  208,50,125,238,243,127,140,53,171,215,96,122,110,46,197,120,113,128,128,99,48,116,128,57,15,147,175,26,183,50,227,40,164,2,67,27,55,34,8,2,
  188,252,179,159,97,226,206,29,252,249,243,207,163,90,173,98,251,240,112,84,246,121,116,108,12,191,122,229,21,124,246,248,113,236,217,189,59,17,180,201,
  27,8,93,8,101,0,252,224,190,125,81,85,60,177,216,139,138,117,20,64,115,160,76,232,227,124,223,199,75,63,250,17,74,197,34,158,125,230,153,196,
  49,27,125,238,205,155,54,161,92,42,193,113,28,252,195,247,190,135,82,169,132,63,127,254,121,12,109,220,136,106,181,154,98,161,57,208,69,25,8,78,
  214,85,42,151,241,202,171,175,162,92,46,227,233,167,158,66,111,111,47,12,195,136,42,240,29,60,112,0,7,15,28,136,174,57,52,52,148,40,141,77,
  239,85,44,22,241,217,227,199,49,188,109,91,196,108,71,186,61,141,38,247,226,165,75,108,224,36,218,253,147,159,254,20,0,240,208,161,67,74,93,58,
  237,243,44,253,111,167,127,227,174,77,199,13,151,147,194,149,47,214,37,230,169,0,171,236,76,147,5,106,41,136,22,140,150,138,161,182,44,43,1,28,
  178,130,5,217,97,39,175,227,71,168,155,52,59,102,159,133,116,67,88,172,38,201,156,164,247,172,96,148,117,69,67,210,78,27,34,25,208,72,128,65,
  142,117,22,101,193,133,246,184,213,242,80,44,90,196,245,194,82,178,211,201,96,34,121,239,116,177,19,35,165,197,230,19,3,67,224,60,55,183,132,165,
  165,101,184,174,151,104,191,42,153,59,109,97,102,36,198,128,44,77,164,255,216,182,141,59,119,238,224,237,183,223,198,35,143,60,18,5,198,190,239,163,
  94,175,227,250,245,235,152,158,158,70,169,84,194,190,125,251,112,240,224,193,72,255,44,238,123,232,208,33,108,219,182,13,63,251,217,207,240,235,95,255,
  26,167,79,159,198,158,61,123,48,48,48,128,209,209,81,140,143,143,39,78,153,0,160,171,171,11,71,143,30,197,234,213,171,209,215,215,7,215,117,113,
  224,192,1,156,56,113,2,239,188,243,14,222,121,231,157,136,8,43,151,203,216,178,101,11,30,127,252,113,204,207,207,71,137,124,189,189,189,40,149,74,
  104,181,90,81,27,133,196,130,130,101,153,0,163,9,147,2,44,83,73,32,45,178,34,146,50,197,143,208,34,11,43,63,106,77,23,158,12,148,162,103,
  93,94,94,142,18,33,233,218,32,188,252,105,226,111,163,209,136,228,34,50,241,36,159,222,139,119,75,231,172,144,225,116,117,149,8,128,78,74,47,40,
  219,28,7,123,130,161,151,165,154,241,201,77,92,52,229,227,21,226,227,170,232,102,229,77,217,42,91,43,42,187,232,68,75,39,91,114,37,245,211,70,
  91,63,230,161,171,82,197,183,190,249,53,172,238,95,19,101,84,250,126,128,86,203,69,173,182,12,32,192,154,53,171,97,24,6,230,230,22,195,235,5,
  1,60,63,64,171,229,0,1,80,42,151,176,106,85,21,83,211,11,184,126,253,38,22,23,110,193,52,195,129,83,171,215,225,182,28,108,219,186,13,15,
  61,188,31,175,253,250,109,220,153,184,13,207,11,224,186,78,123,210,7,176,44,27,158,23,192,113,154,112,154,53,60,255,252,55,241,254,7,215,113,238,
  220,27,140,94,59,244,45,52,140,114,226,8,78,6,111,133,130,9,199,105,226,215,175,189,142,229,218,108,34,250,227,146,124,104,223,223,185,119,59,172,
  102,120,111,46,172,114,228,212,129,32,4,138,181,122,13,239,156,120,27,253,107,214,226,201,39,143,227,195,171,31,97,110,190,6,3,5,220,190,115,3,
  78,211,129,235,5,8,2,15,139,139,243,168,213,234,112,221,38,44,203,110,79,202,176,143,61,207,133,97,184,81,245,29,122,28,189,113,104,24,95,122,
  234,11,120,255,220,57,24,166,135,79,234,71,156,46,180,167,13,177,166,9,48,51,51,133,222,85,125,108,166,175,60,192,179,128,75,167,5,58,242,120,
  234,234,64,141,124,52,45,142,252,56,128,46,191,251,82,169,20,26,238,175,89,147,120,70,25,28,9,96,57,48,48,128,191,250,206,119,18,172,43,149,
  55,84,42,161,244,70,0,120,185,45,148,157,22,237,161,204,33,93,12,229,227,87,1,182,57,189,173,78,54,196,6,223,65,128,133,197,197,232,247,98,
  179,19,155,195,217,115,231,240,206,137,19,153,215,171,86,171,120,238,153,103,80,173,86,35,185,138,124,31,17,232,136,141,107,161,29,0,136,241,197,1,
  189,60,50,40,213,59,205,58,250,83,45,196,121,127,116,197,118,228,0,70,119,50,160,210,78,211,235,211,202,101,121,18,250,104,27,233,70,68,55,93,
  177,65,231,1,207,244,200,56,45,165,200,210,137,155,74,246,89,247,85,185,76,182,252,55,174,40,86,204,60,235,193,51,5,168,242,94,192,73,54,66,
  125,51,34,150,185,88,44,68,9,81,194,253,34,222,123,192,128,249,56,9,95,101,199,39,183,139,3,206,65,96,98,102,102,9,203,203,161,147,6,109,
  187,96,61,85,146,66,149,181,31,5,184,84,55,75,129,164,0,137,83,83,83,248,209,143,126,132,215,95,127,29,251,247,239,199,19,79,60,129,205,155,
  55,99,245,234,213,232,238,238,198,228,228,36,206,159,63,143,191,251,187,191,131,239,251,145,255,242,208,208,16,74,237,147,180,190,190,62,60,255,252,243,
  40,22,139,248,249,207,127,142,211,167,79,71,247,23,82,10,0,232,237,237,141,74,190,47,47,47,99,203,150,45,88,179,102,13,206,158,61,139,83,167,
  78,225,222,189,123,209,186,221,211,211,131,158,158,30,220,191,127,31,211,211,211,248,224,131,15,80,173,86,113,227,198,13,140,142,142,70,73,135,66,111,
  44,10,134,136,103,164,14,72,98,190,80,199,26,121,14,211,146,220,220,251,147,147,246,4,48,22,146,139,86,171,133,90,173,150,210,83,211,128,89,144,
  15,162,44,184,168,62,40,18,214,5,195,44,216,106,193,58,203,32,92,176,208,50,81,19,143,171,52,112,166,255,45,51,209,156,115,134,240,61,167,210,
  140,180,151,120,103,5,179,68,127,115,236,51,247,99,251,49,132,103,55,67,174,210,149,74,243,151,229,1,107,24,69,4,65,248,130,246,237,217,1,152,
  38,126,245,202,235,152,153,153,129,231,154,112,90,45,44,45,78,163,213,170,33,8,128,103,190,252,101,172,219,184,9,127,247,119,223,131,101,133,236,106,
  248,34,194,107,12,244,175,197,95,124,251,223,224,204,153,243,184,112,254,52,124,191,213,6,183,1,12,195,69,16,248,184,123,231,14,118,237,26,198,221,
  187,119,49,57,57,174,212,16,138,137,93,171,55,225,251,53,242,172,126,251,216,138,70,34,141,68,130,131,188,72,54,26,117,52,157,6,42,93,21,212,
  234,115,236,209,150,138,5,109,52,150,18,209,91,189,182,72,254,219,195,248,248,77,76,76,140,99,235,182,29,152,154,156,197,233,247,222,214,110,70,134,
  1,184,110,3,158,103,36,68,245,180,212,37,221,148,26,117,39,60,13,40,151,80,119,106,29,5,79,121,202,5,115,25,249,139,11,75,24,222,186,150,
  253,27,61,70,210,1,153,188,172,222,74,218,77,55,12,195,48,224,56,78,164,69,126,238,217,103,163,207,58,142,131,107,163,163,216,52,52,4,223,247,
  177,109,235,86,252,213,119,190,147,98,91,196,98,52,188,109,27,254,175,191,250,43,204,207,207,163,82,169,176,155,182,144,106,112,63,165,82,9,67,27,
  55,70,32,185,88,44,226,175,190,243,157,4,75,220,104,52,48,57,57,137,254,254,126,124,235,27,223,64,189,94,71,179,217,140,54,22,121,174,142,79,
  76,132,186,223,82,9,255,225,47,255,50,106,211,237,241,113,44,45,45,177,115,92,244,35,5,247,183,198,199,241,147,159,252,36,31,128,107,247,235,216,
  245,235,24,28,28,196,161,145,17,28,60,112,32,218,112,223,120,243,77,92,186,124,57,117,207,207,30,63,30,1,227,189,123,246,68,122,116,161,137,62,
  122,228,8,170,189,189,145,198,27,0,198,199,199,83,22,129,98,38,232,216,77,185,216,140,78,147,47,3,198,60,32,152,22,124,160,235,133,174,76,60,
  55,126,117,118,141,157,254,200,108,147,202,66,147,2,98,46,209,50,43,105,148,43,40,36,223,135,238,55,148,137,86,239,73,73,143,217,36,123,159,45,
  235,74,75,9,211,73,74,241,186,144,135,121,78,38,204,115,224,57,105,47,7,132,221,106,192,182,141,72,35,42,36,36,73,198,56,32,201,97,86,202,
  30,76,214,57,115,140,115,204,8,198,100,145,231,5,152,157,93,194,226,98,45,98,48,197,24,149,131,26,217,119,60,45,75,73,19,35,66,151,59,48,
  48,128,253,251,247,99,221,186,117,88,88,88,192,205,155,55,113,231,206,29,44,45,45,37,188,135,103,102,102,240,155,223,252,6,39,79,158,196,254,253,
  251,113,252,248,113,108,221,186,21,67,67,67,216,184,113,35,30,125,244,81,156,61,123,22,239,188,243,14,94,125,245,85,116,119,119,99,207,158,61,216,
  190,125,59,122,122,122,34,175,103,81,244,132,142,109,225,192,177,113,227,70,148,74,37,156,60,121,18,23,47,94,196,245,235,215,209,223,223,143,241,241,
  241,168,72,75,87,87,23,26,141,6,134,135,135,241,220,115,207,97,122,122,26,63,249,201,79,112,225,194,133,200,121,131,146,19,212,61,67,36,5,82,
  38,94,48,206,226,123,66,159,44,107,193,147,26,121,35,17,192,83,237,49,181,9,20,235,22,157,95,180,34,98,82,38,17,36,202,159,139,192,70,4,
  25,130,201,22,215,19,215,225,230,60,181,180,139,219,226,195,178,192,158,222,139,146,238,42,16,77,147,0,61,207,79,124,79,64,88,206,193,70,252,45,
  143,132,67,228,35,36,181,208,134,22,116,219,150,105,70,166,208,121,139,71,100,37,170,168,0,73,16,56,48,140,34,60,175,142,82,165,136,185,249,26,
  46,156,255,0,134,209,74,232,160,197,66,116,238,131,115,216,23,20,224,251,73,80,27,31,165,135,237,88,90,170,1,16,3,205,109,39,16,134,108,119,
  211,9,245,191,171,87,13,224,254,189,27,137,197,150,43,29,27,30,51,244,194,247,67,7,144,216,115,48,72,68,59,130,81,167,73,84,49,131,88,70,
  169,208,141,98,209,102,143,36,243,202,94,116,155,200,228,253,9,172,90,189,42,115,67,202,242,130,78,31,193,251,104,181,154,104,182,156,142,116,130,121,
  138,40,200,140,166,120,158,233,233,73,236,223,183,131,13,188,184,163,234,44,192,187,82,119,7,213,88,166,160,81,180,251,87,175,188,194,38,17,58,142,
  131,209,177,49,12,12,12,224,208,200,8,38,238,220,193,248,248,120,66,123,124,233,242,101,108,31,30,142,146,250,122,122,122,34,157,180,220,126,106,209,
  246,183,47,188,16,253,94,72,70,134,135,135,241,219,223,253,14,187,119,239,198,230,77,155,48,62,49,129,209,209,81,28,63,118,44,234,191,215,223,124,
  51,74,34,124,247,244,105,28,26,25,137,0,116,58,248,107,224,199,63,253,41,30,61,122,20,155,55,109,138,164,40,111,191,243,78,66,163,45,191,31,
  249,157,152,18,147,174,213,241,182,159,251,189,51,103,208,211,211,131,189,123,246,68,215,19,224,89,158,43,131,131,131,74,155,65,32,76,30,252,213,43,
  175,224,248,177,99,209,179,158,123,255,125,188,123,250,52,91,81,49,235,40,79,48,98,89,137,108,98,211,160,71,174,157,184,142,112,235,110,39,50,16,
  202,110,169,214,25,149,63,185,252,59,238,253,101,205,47,206,91,63,239,252,148,193,58,221,228,117,207,204,95,211,72,20,80,16,140,146,236,172,145,188,
  175,31,185,90,200,46,21,180,148,118,90,171,75,53,199,252,201,22,77,196,19,251,70,184,150,152,17,211,76,165,24,225,62,104,165,100,25,105,9,134,
  145,72,20,148,229,28,42,191,105,161,167,166,86,116,65,16,130,19,223,15,79,125,23,22,150,162,245,143,22,60,145,131,38,153,205,20,39,59,20,68,
  83,118,82,172,35,235,214,173,195,227,143,63,142,195,135,15,39,220,35,196,58,116,231,206,29,220,186,117,11,183,110,221,194,253,251,247,49,55,55,135,
  133,133,5,52,26,13,188,243,206,59,56,113,226,4,30,124,240,65,60,254,248,227,216,179,103,15,86,173,90,133,207,127,254,243,248,252,231,63,143,209,
  209,81,188,244,210,75,248,205,111,126,131,183,222,122,43,106,95,169,84,130,105,154,145,55,113,185,92,198,99,143,61,134,93,187,118,161,94,175,227,157,
  119,222,193,51,207,60,131,67,135,14,69,247,155,158,158,78,48,186,207,60,243,12,108,219,198,189,123,247,112,226,196,9,204,206,206,98,121,121,25,229,
  114,57,113,2,47,75,29,100,176,74,79,92,197,190,34,203,34,100,54,158,218,208,201,50,9,217,215,89,172,1,194,170,78,220,159,226,3,25,3,9,
  150,92,128,99,193,182,55,26,141,8,80,115,5,203,232,181,40,235,76,253,168,85,39,79,105,119,145,64,137,141,194,251,199,174,29,201,121,106,176,249,
  83,121,150,93,213,218,166,170,135,18,245,219,127,249,235,191,14,194,68,48,3,255,243,127,253,175,84,121,76,78,119,248,113,126,196,6,252,240,67,15,
  97,199,174,3,248,193,63,126,31,65,224,68,9,17,98,17,136,239,105,35,76,34,12,163,130,176,169,225,98,181,245,129,205,120,230,217,63,193,15,126,
  248,10,238,223,253,176,221,214,114,84,118,187,29,35,224,219,127,241,109,156,121,255,2,46,158,127,151,104,244,10,224,92,57,158,125,230,57,204,47,44,
  227,183,191,125,5,134,81,4,208,210,110,184,180,192,76,220,217,22,14,29,58,138,177,177,143,48,63,63,169,4,121,58,11,65,177,17,115,229,149,1,
  224,232,209,99,232,173,246,226,149,87,95,206,4,146,42,47,100,89,135,107,154,38,86,175,94,143,103,159,249,18,190,247,189,239,194,243,93,173,255,171,
  206,111,90,247,254,105,68,29,254,187,140,175,126,245,27,248,245,171,47,99,126,113,33,151,124,130,99,221,116,206,1,191,143,118,231,101,187,255,248,143,
  254,8,155,134,134,112,226,228,73,188,255,193,7,236,51,165,89,173,124,85,45,179,64,137,106,81,232,68,182,162,235,95,213,9,131,238,136,95,117,239,
  188,114,16,186,9,233,74,160,115,215,85,45,150,121,238,173,2,220,89,236,106,158,239,112,76,242,74,214,92,85,114,162,220,119,121,223,121,214,137,140,
  106,60,114,85,84,101,144,172,26,11,242,81,126,214,251,80,251,114,231,147,111,200,5,81,100,63,88,78,214,33,0,107,124,212,75,29,20,210,5,43,
  4,235,205,37,11,82,48,76,165,24,73,201,8,117,215,160,201,230,241,126,40,0,121,188,103,164,65,127,178,106,96,44,209,112,93,15,181,90,11,243,
  243,139,81,193,19,10,130,105,112,47,207,61,110,108,112,251,28,157,99,245,122,29,159,251,220,231,240,173,111,125,43,129,53,230,231,231,49,63,63,143,
  181,107,215,70,201,108,2,156,77,78,78,98,98,98,2,147,147,147,184,125,251,54,38,39,39,113,239,222,61,44,46,46,98,211,166,77,56,126,252,56,
  246,237,219,135,74,165,130,233,233,105,188,246,218,107,56,123,246,108,20,64,11,240,88,44,22,163,164,63,199,113,176,126,253,122,12,14,14,226,232,209,
  163,56,119,238,28,174,94,189,138,153,153,153,200,213,98,219,182,109,232,239,239,199,244,244,52,230,231,231,1,0,107,215,174,69,181,90,197,213,171,87,
  177,184,184,136,114,185,156,40,126,66,79,184,232,30,46,152,105,170,109,46,20,10,209,191,197,223,133,236,67,72,64,196,239,196,119,168,14,90,174,250,
  71,175,229,121,30,234,245,122,226,244,140,86,10,148,73,53,81,80,133,38,254,213,235,245,72,3,45,220,52,100,32,73,219,32,179,222,66,175,190,97,
  195,0,138,69,59,210,47,39,117,204,201,202,131,116,30,138,123,8,205,125,60,246,140,232,58,97,209,20,222,154,78,204,207,24,236,199,146,151,116,113,
  21,189,183,180,235,186,248,143,255,225,223,199,12,180,252,130,117,172,64,150,142,142,243,214,229,153,72,11,203,203,117,84,42,37,24,166,141,192,115,200,
  36,55,19,147,205,247,157,132,30,37,166,215,131,80,150,0,3,174,211,32,29,214,36,218,175,0,128,7,35,8,80,46,85,152,107,36,65,176,97,24,
  152,157,155,67,111,239,64,196,152,211,227,212,24,44,23,209,91,13,245,217,150,109,195,52,3,24,176,80,176,109,216,5,27,174,231,227,214,173,91,40,
  22,138,24,24,216,8,219,42,195,128,135,98,209,128,93,40,226,238,189,73,44,47,207,38,34,79,185,63,105,217,90,62,41,211,71,169,92,84,30,123,
  114,0,90,197,70,83,157,97,179,233,192,182,45,148,203,5,44,46,57,153,90,80,26,65,235,10,163,200,108,76,210,231,216,65,179,217,192,154,213,171,
  180,0,90,181,97,211,5,56,11,8,125,146,237,86,253,12,12,12,224,185,103,158,137,146,53,130,32,192,165,203,151,35,39,9,14,204,113,94,235,121,
  64,14,199,218,169,164,50,157,128,227,44,231,5,85,255,116,242,163,2,225,121,217,82,121,126,168,78,56,228,236,115,174,173,156,182,62,143,62,62,11,
  220,101,201,22,210,71,142,105,75,77,149,215,57,215,55,121,188,226,179,130,140,60,86,122,58,118,157,110,124,220,222,162,251,158,92,124,134,62,35,61,
  233,227,152,106,205,232,100,193,115,108,75,231,71,39,141,105,249,137,106,188,243,58,235,100,217,109,35,178,188,83,105,158,117,224,153,254,127,97,49,23,
  118,141,41,185,126,132,191,179,237,184,64,153,248,189,24,102,73,95,234,88,99,189,188,236,160,209,112,177,176,176,20,205,25,186,110,209,241,41,247,59,
  199,60,211,61,69,148,216,22,12,54,253,124,177,88,196,137,19,39,176,97,195,6,28,59,118,44,81,216,232,173,183,222,194,232,232,40,118,237,218,133,
  205,155,55,227,193,7,31,68,111,111,47,214,173,91,135,117,235,214,69,235,189,0,219,179,179,179,184,121,243,38,94,123,237,53,252,230,55,191,65,79,
  79,15,38,39,39,81,175,215,83,82,53,199,113,176,101,203,22,124,253,235,95,199,210,210,18,102,102,102,240,227,31,255,24,99,99,99,184,113,227,6,
  26,141,6,150,150,150,194,164,230,225,97,216,182,29,181,209,247,125,92,184,112,1,63,254,241,143,49,55,55,23,49,216,221,221,221,9,124,160,147,86,
  81,54,90,214,51,39,65,164,159,58,245,228,48,89,171,109,114,64,215,56,33,3,17,114,27,110,142,211,185,35,0,181,0,148,226,247,66,178,33,240,
  8,213,169,211,154,9,114,144,69,19,139,91,173,208,161,101,199,142,7,224,186,205,72,7,45,67,154,80,150,17,72,39,53,105,236,40,64,55,183,175,
  135,39,72,244,20,209,143,36,30,42,242,227,227,228,76,1,10,23,14,110,113,234,196,141,67,237,139,42,34,104,15,142,227,194,182,76,20,11,37,52,
  188,26,12,163,8,223,111,74,131,198,198,230,77,155,113,247,222,20,90,173,229,182,53,81,236,3,109,154,5,152,166,129,228,173,100,13,75,128,150,231,
  192,98,146,99,146,47,38,28,4,115,179,51,216,180,105,24,229,242,32,154,205,41,150,133,95,183,118,29,190,245,173,111,162,213,114,163,133,207,243,129,
  192,15,72,27,66,199,15,223,15,193,181,1,3,45,175,133,74,185,140,91,183,199,241,242,203,63,6,224,51,109,41,227,137,207,124,6,61,61,221,176,
  236,2,90,45,23,65,208,130,129,182,144,62,240,96,152,38,134,54,109,134,105,4,56,252,240,99,104,185,46,28,167,134,229,165,26,234,141,38,92,215,
  195,204,236,189,148,142,177,187,171,23,245,122,13,1,188,84,194,158,152,96,245,122,29,166,89,64,165,220,133,229,90,93,201,86,115,85,202,232,196,81,
  13,74,238,8,20,0,102,231,22,208,63,176,22,99,55,111,102,2,46,202,104,136,119,39,38,51,55,238,126,159,237,230,126,166,166,166,18,114,11,217,
  255,86,46,183,77,23,162,60,236,176,204,192,114,96,68,62,170,83,29,185,114,247,145,65,141,14,80,103,149,63,231,250,148,43,11,78,29,72,84,149,
  233,84,122,85,93,160,212,41,243,42,131,5,29,195,75,55,19,93,208,194,249,196,171,188,143,41,56,86,21,67,226,174,37,159,70,100,157,78,136,239,
  201,62,179,28,179,200,201,17,242,156,102,168,165,124,124,160,164,75,174,166,235,182,124,90,195,203,226,144,243,93,39,229,25,241,187,23,9,227,186,64,
  148,234,158,77,86,46,97,89,233,13,60,109,13,151,6,204,180,200,88,108,83,23,235,102,13,3,40,20,138,209,119,104,201,98,145,144,31,50,148,6,
  145,161,136,2,43,22,106,181,22,102,102,230,209,104,52,163,185,39,128,51,183,231,229,9,160,233,187,16,172,231,250,245,235,97,154,38,38,38,38,34,
  107,53,49,94,107,181,26,126,240,131,31,160,80,40,224,209,71,31,133,239,251,232,238,238,198,147,79,62,137,102,179,137,215,94,123,13,7,14,28,192,
  206,157,59,83,65,153,97,24,88,181,106,21,86,175,94,141,173,91,183,134,18,176,247,222,195,196,196,68,228,177,44,146,224,0,68,242,10,0,184,123,
  247,46,94,127,253,117,172,93,187,182,189,223,133,108,251,196,196,68,194,30,110,235,214,173,216,181,107,23,22,22,22,240,198,27,111,96,110,110,46,42,
  229,45,75,40,56,7,27,58,47,101,22,154,11,154,69,18,32,103,228,32,131,107,74,0,9,176,43,107,174,5,83,44,87,3,164,50,26,161,49,23,
  253,43,52,218,50,136,167,86,121,66,203,45,52,236,114,208,44,36,27,150,101,97,213,170,85,81,181,196,185,57,39,193,242,234,78,122,213,227,44,150,
  64,209,177,231,121,49,171,45,246,249,88,194,21,64,190,100,167,6,25,42,236,104,67,241,72,186,12,115,213,131,115,44,80,178,51,2,250,4,225,49,
  131,93,136,180,183,178,238,116,85,111,15,190,249,205,175,227,31,127,240,83,76,76,92,3,21,118,27,134,133,85,171,87,193,71,0,207,55,52,27,93,
  1,165,114,185,173,237,242,165,136,37,128,105,150,224,186,177,228,99,122,102,22,221,221,101,148,74,37,52,26,98,17,77,106,116,102,231,102,208,108,54,
  241,222,123,23,240,209,71,87,1,24,112,61,15,174,235,193,178,138,48,208,130,105,81,19,113,7,134,105,195,105,214,49,208,191,22,127,252,71,95,68,
  87,215,106,44,46,222,79,179,151,253,107,49,50,178,31,163,99,227,184,119,119,2,61,213,42,74,229,30,52,27,77,184,173,16,28,183,220,38,102,102,
  231,209,213,85,197,192,224,58,216,118,184,80,151,203,61,40,149,202,232,237,237,194,143,126,244,99,220,186,125,61,209,255,95,249,202,215,80,40,88,184,
  112,225,60,22,23,107,128,225,162,92,236,66,161,104,161,217,116,208,108,58,176,172,2,122,170,21,236,220,185,7,222,149,107,112,61,23,129,31,70,168,
  110,187,100,80,16,184,109,103,143,34,108,187,128,185,249,25,0,94,174,65,169,98,178,230,231,151,176,109,243,32,107,43,199,85,231,83,149,112,207,195,
  142,210,107,208,8,122,37,237,230,88,254,188,237,214,233,89,179,202,185,103,177,200,89,5,62,104,226,137,172,77,83,249,178,230,233,23,149,15,49,109,
  35,231,129,78,219,77,109,144,100,118,88,7,82,117,204,177,42,249,81,21,108,113,39,67,89,137,172,121,43,132,102,173,171,89,58,97,174,12,118,158,
  147,2,217,119,151,178,162,116,195,204,115,202,194,177,191,92,53,90,221,248,85,5,95,121,192,185,234,100,52,222,103,140,92,210,141,240,26,62,0,75,
  242,225,87,39,22,102,37,13,114,9,122,180,0,75,16,24,81,34,160,14,60,39,61,165,141,232,185,226,138,164,134,100,77,23,182,61,76,214,178,18,
  122,233,208,129,201,196,220,220,50,234,245,70,180,31,10,89,0,213,173,210,64,66,37,5,162,192,145,43,181,189,99,199,14,28,59,118,12,59,118,236,
  128,97,24,248,240,195,15,241,155,223,252,6,215,175,95,143,100,20,162,74,222,247,191,255,125,20,139,69,60,244,208,67,240,125,31,131,131,131,248,246,
  183,191,141,231,158,123,46,98,120,133,132,227,196,137,19,248,240,195,15,209,219,219,139,45,91,182,192,117,93,92,186,116,9,227,237,2,86,213,106,53,
  98,102,139,197,34,118,239,222,141,86,171,133,137,137,137,8,212,57,142,131,83,167,78,69,227,92,172,71,59,119,238,196,150,45,91,48,51,51,19,233,
  155,207,159,63,143,222,222,222,8,60,11,214,153,3,192,114,145,19,85,16,194,21,224,209,17,36,98,110,210,19,126,57,145,83,62,29,144,165,21,52,
  209,79,0,92,225,3,77,93,52,28,199,97,188,205,205,104,78,11,247,20,170,169,150,251,194,243,60,20,10,5,244,244,244,96,221,186,117,88,191,126,
  61,230,230,102,81,169,212,81,171,213,82,65,168,239,123,76,249,110,253,58,157,214,74,11,105,18,183,246,167,235,142,100,201,245,196,233,20,39,31,165,
  243,192,206,123,68,156,247,119,42,119,4,177,80,137,227,170,150,235,180,35,30,1,160,189,20,224,158,157,95,192,187,167,223,71,179,233,48,128,197,69,
  193,6,90,45,31,181,229,165,200,85,34,252,123,152,28,225,251,62,76,163,0,211,48,209,108,54,162,191,9,31,66,33,31,160,250,229,165,165,58,10,
  69,27,221,61,69,44,46,2,180,223,68,219,28,167,129,59,19,119,224,122,225,145,196,177,39,30,135,105,121,184,124,229,42,150,22,107,184,119,255,62,
  226,210,167,133,182,206,167,5,192,197,220,220,36,90,142,3,211,224,153,49,195,52,176,48,223,192,47,127,249,26,28,103,134,20,4,240,153,128,165,0,
  160,69,54,43,11,229,114,47,190,242,213,103,177,99,231,118,220,186,125,157,48,158,97,97,21,219,178,177,99,199,46,152,166,5,215,107,65,232,206,77,
  195,66,0,15,150,81,192,194,194,50,118,239,217,141,173,91,119,192,117,29,192,48,97,90,38,130,192,5,2,3,6,2,152,86,1,197,162,141,158,158,
  46,156,191,240,1,126,251,219,223,230,218,116,121,237,107,72,155,12,174,29,68,95,111,63,108,219,70,79,181,7,190,231,226,214,237,27,74,95,104,21,
  115,43,31,209,168,22,172,188,137,81,89,122,106,21,32,202,106,55,173,228,196,77,212,60,160,94,183,208,208,164,20,174,141,42,75,36,249,232,48,143,
  1,189,220,207,156,243,12,5,89,50,11,195,181,91,181,113,231,1,243,124,50,73,146,37,19,126,169,220,145,30,101,132,117,37,200,57,235,206,44,45,
  177,238,25,58,145,240,232,88,94,85,255,200,37,230,229,128,36,203,215,90,23,156,234,2,3,85,78,134,174,79,168,188,140,235,151,244,239,130,220,251,
  22,101,131,229,177,42,60,158,229,228,112,25,60,211,125,45,214,75,243,238,22,201,42,128,73,7,15,207,243,81,40,216,18,40,142,237,233,232,187,74,
  6,155,177,92,67,188,54,42,45,17,68,83,179,233,97,106,106,46,161,109,46,20,146,9,107,116,124,208,192,138,186,32,201,114,2,78,62,213,106,181,
  208,213,213,133,99,199,142,97,100,100,36,26,95,15,63,252,48,246,239,223,143,115,231,206,225,173,183,222,194,245,235,215,35,137,69,171,213,194,139,47,
  190,8,219,182,113,224,192,129,8,128,13,12,12,180,109,108,231,240,203,95,254,18,239,189,247,94,228,4,20,4,1,206,159,63,159,8,202,69,123,75,
  165,18,186,187,187,177,110,221,58,124,227,27,223,128,109,219,24,29,29,197,79,127,250,83,44,44,44,68,250,98,10,120,125,223,71,95,95,95,196,132,
  207,205,205,225,197,23,95,140,100,34,166,105,162,92,46,167,230,7,111,83,104,176,110,53,42,16,45,159,108,201,128,89,118,227,224,0,186,236,183,77,
  19,9,41,147,44,100,21,130,164,160,246,164,92,158,130,188,238,137,49,146,212,15,199,140,183,97,24,40,151,5,1,217,192,237,219,183,241,192,3,15,
  180,137,17,79,146,85,37,139,162,208,96,94,4,140,66,175,44,114,228,244,18,52,32,175,23,180,174,96,32,183,175,170,228,176,118,180,56,172,128,17,
  147,179,56,85,71,107,241,127,199,139,74,171,229,195,243,154,176,12,126,65,20,237,122,235,173,223,40,64,137,143,98,193,134,231,161,13,198,105,251,136,
  31,161,87,131,231,180,96,219,133,118,91,253,20,128,74,28,113,184,30,2,47,64,181,167,31,19,254,168,50,74,156,157,95,194,142,237,59,177,111,239,
  62,172,91,31,234,161,119,238,218,1,207,243,113,225,194,53,216,22,176,126,253,58,124,240,193,21,172,95,63,136,201,201,105,4,104,225,202,229,49,4,
  134,5,215,119,88,166,222,109,53,17,192,69,185,88,196,227,143,125,9,155,183,108,192,196,248,29,44,46,214,113,225,226,89,212,235,75,0,12,84,42,
  171,209,104,204,193,52,45,12,244,111,132,239,135,178,141,70,99,22,215,175,223,198,3,155,215,165,222,163,105,88,120,235,205,183,112,254,226,89,152,102,
  161,205,134,248,68,131,87,129,97,88,109,70,198,67,16,120,176,76,19,133,98,23,130,192,131,109,183,19,32,60,31,126,16,160,167,167,31,95,255,250,
  31,97,161,157,88,33,7,79,134,81,192,145,35,71,225,123,46,220,86,11,190,239,193,178,10,232,238,233,65,165,92,65,177,84,129,101,249,176,237,2,
  122,122,186,209,213,221,133,175,126,245,43,237,210,169,69,244,245,117,225,239,255,215,255,194,194,242,18,171,43,205,114,41,224,0,3,7,198,116,65,159,
  138,93,85,205,7,149,30,84,197,68,200,243,167,83,27,62,142,245,229,42,99,230,61,73,82,121,194,103,5,28,92,80,66,55,93,174,140,188,12,96,
  213,12,97,186,152,13,231,15,172,91,16,117,12,46,215,95,170,160,140,3,102,92,219,116,129,87,39,63,92,82,96,30,153,15,39,23,202,11,226,85,
  218,100,46,208,224,18,59,117,193,36,55,254,63,190,52,48,125,134,42,51,72,249,159,55,222,163,178,170,43,202,172,87,40,155,48,18,224,57,78,60,
  54,145,78,146,15,191,207,131,231,100,162,95,60,135,121,135,13,145,184,24,2,114,192,117,3,52,155,14,230,231,151,35,64,42,152,235,44,141,59,39,
  15,163,127,19,78,13,34,169,76,14,196,230,230,230,240,194,11,47,224,145,71,30,193,163,143,62,138,173,91,183,70,190,194,143,60,242,8,14,29,58,
  132,243,231,207,71,90,231,70,163,129,197,197,69,124,239,123,223,139,88,99,1,236,10,133,2,154,205,38,78,158,60,137,102,179,25,149,215,166,107,120,
  171,213,194,210,210,82,100,253,118,240,224,65,28,63,126,28,87,174,92,193,47,126,241,139,168,148,54,71,36,136,107,20,139,69,124,240,193,7,184,113,
  227,6,6,7,7,49,55,55,7,215,117,19,122,112,221,201,143,152,171,220,248,20,76,63,55,7,100,12,69,73,31,154,232,71,61,252,5,158,162,36,
  132,188,207,8,237,49,5,199,244,255,211,107,10,201,134,156,119,33,175,227,212,142,149,62,143,104,215,234,213,171,209,213,213,5,219,182,113,255,254,125,
  212,106,181,200,87,59,12,242,92,233,154,94,219,203,153,62,123,236,183,30,155,70,160,35,247,159,188,129,127,214,231,69,49,38,206,209,72,156,113,217,
  186,205,88,206,84,228,64,129,46,89,41,29,97,21,224,251,6,0,7,75,75,179,8,188,0,165,114,69,121,212,11,0,93,93,225,241,77,163,81,79,
  93,211,50,44,24,8,16,40,88,19,49,105,150,150,151,209,221,93,110,47,52,101,0,173,196,2,78,23,254,90,125,30,174,219,64,79,119,89,153,144,
  19,218,186,56,216,242,64,12,82,29,167,133,209,143,110,161,92,41,225,224,193,157,88,92,172,97,110,118,6,7,30,220,131,82,185,136,93,187,31,192,
  201,147,239,181,35,124,31,158,27,107,68,19,215,6,208,106,57,24,30,222,129,189,123,119,98,98,124,26,35,135,30,4,0,172,234,237,197,196,221,187,
  216,178,229,1,108,222,178,30,191,252,197,207,112,243,214,61,28,63,254,121,92,184,112,46,210,61,207,207,205,161,123,247,54,105,243,17,85,137,154,145,
  119,232,198,141,155,176,110,112,45,12,203,198,253,187,19,152,153,171,227,145,71,14,161,88,0,94,125,237,215,24,28,88,135,201,169,9,52,22,151,177,
  118,237,102,52,27,53,204,45,78,197,229,142,151,102,177,176,176,140,122,189,158,96,244,226,68,145,46,60,252,200,67,176,140,0,119,238,76,193,245,92,
  248,94,104,202,190,180,92,199,237,137,113,212,107,53,4,65,1,11,11,211,237,227,35,15,158,7,56,45,15,223,252,214,215,176,105,211,16,46,94,185,
  146,98,200,228,178,197,28,3,72,23,119,46,8,18,237,149,219,173,98,131,101,240,37,47,150,186,57,192,181,91,150,39,80,157,118,94,80,193,185,185,
  100,49,202,58,176,37,179,170,89,139,15,87,182,154,99,167,84,1,12,5,69,42,150,82,126,198,78,146,221,116,101,172,185,10,106,178,45,159,204,248,
  168,180,159,58,125,111,214,187,84,1,195,60,199,152,89,215,86,157,4,112,247,86,201,111,184,62,213,157,32,232,198,170,236,159,175,210,48,115,207,160,
  2,207,113,129,134,24,112,198,201,74,65,123,243,246,153,62,81,3,227,112,223,179,18,14,28,194,230,78,184,96,80,251,185,240,121,76,73,182,97,72,
  219,109,16,37,66,137,130,40,182,109,18,0,21,219,220,217,118,168,85,150,61,105,57,240,28,255,222,68,171,21,202,225,106,181,70,180,38,210,138,114,
  170,226,71,52,129,146,38,139,113,39,49,33,32,50,177,125,251,118,108,222,188,25,150,101,161,94,15,143,231,151,150,150,176,176,176,128,165,165,37,188,
  241,198,27,56,117,234,20,246,237,219,135,39,158,120,2,187,118,237,138,230,220,193,131,7,49,50,50,130,243,231,207,227,212,169,83,184,112,225,2,238,
  223,191,143,191,249,155,191,193,127,252,143,255,17,59,119,238,36,251,204,18,170,213,106,148,48,39,250,219,117,93,180,90,45,172,94,189,26,135,14,29,
  194,196,196,4,198,199,199,113,249,242,101,76,77,77,97,211,166,77,152,157,157,197,213,171,87,163,103,147,61,203,233,79,161,80,136,146,18,69,213,62,
  46,80,86,97,37,25,204,137,189,157,86,13,84,233,159,229,210,215,148,65,150,157,45,100,118,152,130,94,217,6,79,72,61,226,66,38,70,180,55,210,
  228,65,186,103,202,186,110,78,146,40,218,36,188,177,203,229,50,86,173,90,133,245,235,215,99,205,154,53,120,243,205,55,209,104,52,48,56,56,24,22,
  129,171,45,193,247,189,104,62,196,237,22,50,14,113,143,32,193,82,27,70,208,158,47,70,234,196,36,237,209,236,119,180,22,233,214,102,33,133,162,123,
  97,34,145,19,128,31,4,113,18,33,151,188,211,137,14,148,219,76,211,86,97,30,130,192,134,105,22,225,121,225,6,102,217,233,141,149,102,126,30,220,
  191,31,253,131,67,248,233,79,127,24,29,125,69,199,54,133,240,58,166,105,181,129,57,191,201,52,157,58,202,149,82,187,3,90,90,93,160,105,20,17,
  4,38,236,130,17,201,66,184,12,126,3,30,238,223,159,133,101,1,253,253,171,97,26,38,86,175,94,133,238,158,46,220,186,53,137,95,253,234,21,248,
  126,19,95,255,218,55,49,62,49,137,225,225,13,184,120,241,10,10,197,66,155,33,240,216,77,65,12,168,82,169,132,114,165,128,13,27,214,224,252,249,
  143,48,63,183,136,39,142,61,132,45,219,54,161,92,46,160,229,120,120,234,169,167,112,225,226,117,84,186,10,184,250,209,149,104,226,205,204,220,135,101,
  23,80,42,86,208,116,234,209,113,95,232,130,100,162,175,119,61,158,254,226,147,64,224,98,97,161,134,7,30,216,140,222,207,62,142,229,229,38,106,203,
  117,148,202,5,28,251,140,141,109,219,183,224,244,233,119,112,241,226,251,248,204,227,199,112,125,244,50,222,191,48,75,198,129,9,215,3,74,197,82,244,
  190,104,159,58,78,3,45,199,197,196,221,187,120,235,157,211,152,157,155,130,235,214,16,38,147,22,176,105,104,11,198,39,110,133,229,202,235,53,52,154,
  75,48,140,18,254,232,203,95,194,207,127,254,99,140,142,221,196,230,205,15,224,226,149,43,41,6,132,3,82,178,28,134,150,68,213,1,13,185,221,50,
  32,146,143,170,242,130,23,142,185,209,107,43,13,37,240,166,247,212,45,4,220,162,167,242,212,230,54,134,60,192,153,11,170,229,19,39,213,73,22,103,
  63,39,3,72,21,88,206,203,50,231,97,115,59,89,223,100,231,135,44,95,119,85,0,160,170,202,166,11,2,185,163,222,188,238,32,220,166,47,23,59,
  160,155,82,214,24,204,115,82,163,99,160,101,93,38,37,95,184,194,84,42,167,39,126,172,4,160,158,205,233,121,98,106,171,240,137,196,65,97,9,151,
  46,90,129,72,143,76,175,151,150,44,129,153,119,54,121,158,176,190,64,8,206,141,84,210,88,218,227,89,108,230,49,120,166,246,116,226,111,245,122,11,
  203,203,181,132,76,131,18,96,242,241,188,236,231,78,79,93,68,31,115,218,86,223,247,241,200,35,143,96,100,100,4,219,182,109,195,170,85,171,18,160,
  172,94,175,99,105,105,41,170,124,55,63,63,143,123,247,238,225,238,221,187,24,24,24,192,186,117,235,18,50,145,131,7,15,98,223,190,125,248,240,195,
  15,241,238,187,239,226,157,119,222,193,223,252,205,223,224,27,223,248,6,138,197,34,206,156,57,131,11,23,46,160,217,108,162,80,40,36,222,249,230,205,
  155,163,123,239,217,179,7,135,14,29,194,123,239,189,135,211,167,79,99,126,126,30,55,110,220,128,109,219,81,226,98,95,95,31,230,230,230,80,171,213,
  80,42,149,216,211,36,241,251,56,127,201,77,173,225,42,11,69,1,144,105,210,29,39,31,164,249,39,226,51,242,28,147,79,121,68,223,211,160,72,214,
  64,83,89,134,96,161,41,115,77,75,109,11,240,44,18,1,197,103,227,178,242,188,237,167,104,147,208,74,211,42,142,183,111,223,198,218,181,107,35,139,
  64,145,52,26,182,201,75,229,37,136,57,39,19,45,177,149,93,144,58,93,162,251,12,213,64,139,118,202,62,252,220,122,76,129,183,62,97,209,140,64,
  57,187,119,250,62,175,129,214,101,233,11,33,190,156,32,165,58,158,75,38,105,5,109,173,179,1,195,44,194,243,93,148,138,165,20,11,76,163,196,147,
  239,158,65,169,244,33,146,149,97,218,26,70,191,133,0,1,76,211,142,52,192,64,154,110,119,189,22,138,133,174,196,119,149,213,189,12,11,158,231,34,
  8,188,40,131,153,211,6,153,150,129,153,153,5,188,242,202,207,49,50,242,8,250,251,215,160,213,114,49,126,230,54,174,93,27,133,231,53,224,121,77,
  252,236,23,47,99,105,113,25,103,207,173,199,220,220,125,172,93,187,177,221,191,124,153,108,199,9,25,226,235,55,63,194,129,145,237,232,233,233,66,163,
  214,68,169,98,227,206,157,73,252,228,39,63,195,67,15,61,132,114,169,2,63,104,225,177,199,246,227,204,123,23,224,186,205,232,185,106,181,26,2,63,
  64,177,88,138,0,180,168,98,5,4,120,112,255,1,12,172,233,197,27,111,190,135,254,254,85,97,63,6,101,148,138,6,190,247,15,63,198,182,29,59,
  176,107,120,11,46,92,184,132,227,199,31,199,204,204,28,10,133,0,83,51,211,210,196,10,80,171,47,163,171,171,162,0,23,77,140,94,191,133,3,15,
  110,71,169,82,65,163,217,196,244,244,44,250,251,87,97,113,161,129,189,123,135,241,209,213,235,232,233,237,195,204,244,4,126,243,219,95,99,199,206,157,
  88,59,176,26,1,128,107,31,125,132,131,95,123,22,229,82,25,141,102,131,5,44,242,81,62,5,209,28,176,228,0,172,110,211,151,19,102,100,64,214,
  169,93,156,14,48,171,142,76,57,32,68,217,100,174,34,38,221,4,244,139,4,239,137,173,10,74,100,141,178,124,140,71,35,117,213,58,162,98,46,229,
  205,156,174,33,178,78,92,110,159,14,120,117,234,123,221,9,131,43,3,59,238,29,112,76,117,39,199,140,121,36,55,42,219,44,149,149,150,170,90,160,
  238,244,37,143,22,91,53,71,104,89,111,90,237,140,43,82,64,219,40,87,32,84,157,80,164,181,143,113,2,145,188,225,134,207,110,48,12,150,126,188,
  132,207,98,164,142,148,179,146,196,194,202,114,118,91,235,108,182,11,164,24,146,94,89,246,123,14,34,231,140,32,48,19,122,231,240,186,22,234,117,7,
  205,166,139,122,189,25,189,47,1,156,229,245,139,219,167,229,0,78,188,71,122,164,47,88,95,234,206,176,101,203,22,60,252,240,195,169,119,101,219,54,
  170,213,42,122,122,122,82,228,3,77,12,150,223,161,101,89,216,179,103,15,118,237,218,133,221,187,119,227,133,23,94,192,223,255,253,223,195,178,44,44,
  45,45,69,242,15,1,242,28,199,193,142,29,59,112,224,192,1,108,218,180,9,211,211,211,81,177,20,145,228,39,60,140,197,254,208,223,223,143,127,243,
  111,254,13,90,173,22,46,92,184,128,19,39,78,164,214,126,110,173,149,65,179,108,165,40,191,115,21,91,203,17,34,114,240,36,175,167,226,122,142,227,
  68,126,208,98,142,8,166,152,22,87,161,247,23,133,80,40,110,227,74,107,139,235,208,211,7,21,185,67,223,91,181,90,69,177,88,132,109,219,88,92,
  92,196,228,228,100,244,142,23,23,23,35,251,66,81,97,55,12,2,92,201,146,216,100,79,187,12,195,39,123,123,64,36,29,72,236,11,2,155,9,43,
  202,208,145,141,35,36,41,110,236,220,106,85,151,251,100,235,244,105,244,203,98,224,200,154,69,89,191,195,233,228,146,154,170,240,129,187,43,93,40,22,
  139,8,171,50,234,108,163,28,212,235,14,123,143,150,27,122,22,39,193,168,40,188,18,77,1,4,129,137,66,177,220,6,145,113,36,164,58,158,116,29,
  23,189,213,85,81,167,203,133,84,194,118,152,176,76,11,149,114,21,183,110,221,199,232,232,77,24,0,92,183,142,238,238,50,124,63,12,12,230,102,22,
  225,122,13,220,185,115,3,197,98,55,86,247,173,1,162,210,171,105,112,98,180,23,201,153,233,41,252,240,135,63,193,193,3,251,176,106,117,15,26,77,
  15,63,251,217,171,88,94,158,193,91,111,189,17,177,171,149,114,15,46,92,188,148,40,238,225,186,33,21,98,219,116,48,121,240,92,31,189,189,85,220,
  186,117,3,135,70,246,224,203,127,20,86,171,187,123,119,30,11,11,77,184,173,101,44,44,77,161,92,218,7,215,111,226,189,247,222,193,166,77,27,241,
  252,243,95,71,171,209,196,194,252,92,106,64,213,151,235,232,237,91,165,212,36,143,142,142,99,228,192,78,92,190,120,13,93,125,107,112,244,240,1,152,
  166,1,167,209,192,143,126,244,11,124,233,75,79,99,234,254,28,54,111,217,130,7,182,236,194,177,39,30,197,175,95,123,165,45,69,153,132,235,6,24,
  26,90,135,107,163,55,148,146,8,157,240,95,231,139,43,111,250,42,160,168,74,40,200,2,68,28,192,150,165,28,89,146,132,44,0,168,178,103,83,177,
  193,220,245,228,190,227,252,135,85,165,147,185,96,68,231,131,172,210,1,170,2,7,174,96,7,253,12,167,125,83,49,68,186,254,85,121,45,115,178,23,
  202,22,231,9,160,242,72,108,116,100,132,188,214,170,250,151,99,177,116,129,155,108,97,73,129,22,151,59,144,71,118,162,170,218,69,51,249,185,83,128,
  78,116,218,121,2,215,228,251,76,90,198,241,207,100,164,18,246,146,236,179,209,222,107,252,72,247,204,185,115,208,100,118,241,231,66,193,110,7,3,102,
  244,247,36,128,247,35,185,8,117,0,17,238,25,113,41,111,3,166,105,99,121,217,193,194,194,98,98,28,114,229,212,117,1,56,29,67,242,184,41,20,
  10,145,67,195,224,224,32,14,31,62,140,189,123,247,226,151,191,252,37,78,159,62,141,159,253,236,103,56,127,254,60,62,247,185,207,225,193,7,31,140,
  152,97,90,79,65,128,218,90,173,134,161,161,33,172,94,189,26,149,74,5,149,74,5,221,221,221,145,147,133,104,95,171,213,194,197,139,23,241,246,219,
  111,163,80,40,68,64,175,82,169,68,101,161,55,111,222,140,174,174,46,220,185,115,7,119,239,222,197,196,196,4,30,126,248,97,140,140,140,192,117,93,
  220,188,121,51,2,251,174,235,98,195,134,13,120,252,241,199,209,219,219,139,115,231,206,161,213,106,69,37,169,101,27,74,57,9,79,14,230,85,39,99,
  28,169,33,231,157,200,167,76,220,56,151,79,199,68,159,80,123,79,1,130,57,162,81,92,39,204,31,178,19,167,6,116,62,10,240,76,75,127,211,211,
  33,213,115,38,64,99,91,178,209,223,223,143,205,155,55,227,242,229,203,184,125,251,54,108,219,70,165,82,65,189,94,135,227,56,145,157,96,120,26,209,
  36,68,103,16,105,140,105,208,42,116,207,116,125,9,147,121,125,50,71,213,129,160,154,176,10,18,115,179,19,16,205,202,198,200,26,26,9,40,178,54,
  59,149,254,140,122,189,202,131,131,255,142,5,207,107,161,84,233,70,0,11,139,243,11,16,217,150,233,114,192,5,108,218,252,0,110,223,190,9,223,167,
  32,58,108,103,173,230,160,84,52,219,209,41,34,246,57,174,125,238,195,48,124,212,107,117,172,25,24,130,109,119,195,117,23,64,179,181,229,1,238,251,
  78,216,22,43,72,200,55,228,73,115,225,252,251,184,82,184,130,221,187,247,252,255,104,251,207,46,59,174,43,75,20,157,97,143,207,76,164,5,18,222,
  144,240,142,132,33,69,128,146,8,146,162,145,170,164,50,34,85,78,125,187,171,250,126,236,113,71,191,31,210,163,63,244,123,37,169,91,170,186,85,146,
  88,242,162,72,138,162,17,73,128,6,160,1,18,36,60,144,9,147,153,48,233,143,15,255,62,68,236,125,118,236,179,119,68,36,37,157,49,56,72,102,
  230,57,103,71,196,54,107,205,53,215,156,216,179,103,71,56,1,16,154,185,231,242,5,40,74,0,199,118,16,248,1,92,207,67,224,135,10,27,165,114,
  17,243,115,139,52,145,224,95,165,114,57,154,136,69,212,170,75,56,125,250,60,204,156,9,203,106,34,8,2,244,244,12,194,48,76,120,174,13,221,200,
  227,228,201,83,80,149,0,189,189,3,112,29,23,170,166,97,116,229,6,32,0,154,140,142,115,16,120,168,213,170,232,237,235,197,71,31,159,194,79,126,
  218,196,161,67,15,162,191,191,31,158,231,224,202,149,27,184,20,89,38,59,182,135,243,231,46,193,247,45,188,248,226,47,240,232,23,191,140,129,190,34,
  154,237,102,215,51,186,123,247,14,118,110,223,36,157,128,213,165,37,180,44,7,143,61,241,112,184,177,206,46,161,92,46,226,195,15,63,194,212,244,77,
  44,85,171,248,232,163,147,216,185,111,15,190,241,141,103,112,242,228,41,92,191,113,61,90,20,1,230,23,22,176,106,229,26,92,185,58,33,12,96,249,
  18,187,172,161,41,107,201,126,57,213,24,254,255,69,136,183,72,85,35,110,34,147,222,244,32,10,238,120,131,10,89,233,63,201,58,62,141,66,34,50,
  42,145,53,22,46,231,222,198,141,146,124,169,211,167,40,112,206,194,203,230,209,183,44,175,46,53,28,174,169,147,255,61,79,133,144,209,44,120,69,17,
  209,124,73,67,130,217,103,44,162,83,36,233,246,38,169,204,168,9,218,248,89,20,50,100,180,147,164,74,197,231,181,53,151,205,159,44,159,197,255,90,
  36,117,149,117,79,232,230,126,43,2,234,70,183,188,29,225,100,139,199,162,211,134,65,242,89,132,50,162,235,136,130,107,13,245,122,27,213,234,18,189,
  118,150,34,144,180,215,137,18,37,209,250,37,191,35,136,231,147,79,62,137,35,71,142,160,88,44,34,8,2,236,223,191,31,231,207,159,71,16,4,24,
  31,31,199,181,107,215,176,125,251,118,124,249,203,95,198,134,13,27,98,13,126,67,67,67,216,187,119,47,222,126,251,109,106,60,162,170,42,74,165,18,
  138,197,34,250,250,250,80,169,84,80,42,149,0,0,55,111,222,196,244,244,52,53,93,97,171,89,61,61,61,104,183,219,56,122,244,40,138,197,34,60,
  207,195,133,11,23,240,254,251,239,227,212,169,83,56,117,234,20,13,252,9,5,99,105,105,9,173,86,11,167,78,157,194,23,191,248,69,60,246,216,99,
  244,111,137,250,7,175,56,194,83,153,100,244,54,145,233,21,203,25,79,82,200,16,237,169,188,4,30,11,134,241,60,104,145,113,26,75,9,97,247,3,
  150,146,67,144,102,0,180,170,192,39,14,178,61,150,173,110,144,251,54,51,51,131,92,46,135,66,161,16,163,220,234,186,78,27,48,13,195,64,169,84,
  130,170,2,174,235,112,159,29,128,40,162,133,215,209,233,19,8,13,131,68,177,35,225,136,147,30,2,159,161,110,101,59,75,217,74,83,10,246,76,17,
  238,36,128,74,79,66,130,200,134,205,150,219,100,114,66,105,122,164,29,142,89,148,165,69,89,120,16,219,200,227,122,125,1,124,220,119,223,125,176,29,
  31,119,110,143,35,212,234,116,233,197,45,69,100,255,98,169,130,122,253,94,87,233,139,108,72,141,102,19,249,156,14,207,115,132,124,206,56,239,199,67,
  179,89,71,185,220,139,120,67,74,252,122,90,237,26,208,174,225,163,143,63,192,217,179,159,32,159,47,35,80,84,120,174,131,85,43,71,241,244,211,95,
  194,39,159,94,192,245,27,151,35,173,77,3,237,118,11,134,161,33,159,171,196,168,36,241,195,216,69,223,138,10,254,238,239,158,135,231,249,8,252,8,
  56,15,124,248,129,31,114,199,53,3,249,188,6,63,8,208,106,88,80,53,13,154,198,242,8,129,223,253,238,13,88,118,51,118,189,181,90,11,197,98,
  25,170,170,98,102,102,18,175,188,50,29,187,239,228,58,63,57,253,46,83,118,105,227,205,55,94,142,233,139,134,159,167,67,85,20,20,138,21,228,162,
  141,83,116,80,14,14,12,2,190,143,43,151,175,35,95,40,226,250,196,53,76,79,223,197,244,237,235,80,20,224,87,191,250,5,60,175,133,249,183,231,
  49,126,101,29,198,39,62,141,189,127,113,105,1,125,229,178,84,22,72,116,48,179,217,181,168,1,32,9,229,77,10,6,179,4,1,162,102,146,180,207,
  146,33,148,73,106,6,73,252,107,22,29,204,234,160,151,166,61,156,132,242,102,9,64,210,58,216,179,142,37,13,121,231,127,158,197,64,133,215,17,151,
  149,49,69,247,94,84,141,224,155,136,100,40,118,210,184,69,52,139,180,170,135,140,159,189,92,173,101,17,29,69,22,124,203,80,181,44,40,58,75,235,
  144,81,92,68,201,162,56,120,22,241,226,179,38,117,114,3,23,145,230,115,247,65,171,118,5,231,33,200,211,161,151,16,67,149,184,51,26,225,128,122,
  0,244,24,255,153,28,244,142,19,150,227,107,181,208,29,143,232,240,202,148,100,248,0,80,246,115,62,104,100,207,113,199,113,176,97,195,6,124,233,75,
  95,130,105,154,244,25,172,94,189,26,189,189,189,152,157,157,69,62,159,71,16,4,56,119,238,28,46,94,188,136,61,123,246,224,161,135,30,162,141,130,
  132,234,241,237,111,127,27,119,238,220,193,7,31,124,128,51,103,206,96,113,113,17,181,90,13,213,106,53,134,128,26,134,129,124,62,15,207,243,208,104,
  52,40,93,97,203,150,45,120,246,217,103,49,59,59,139,207,62,251,12,213,106,21,163,163,163,152,157,157,165,212,32,98,55,109,24,6,254,242,47,255,
  18,134,97,96,106,106,10,215,174,93,195,165,75,151,208,106,181,48,48,48,128,235,215,175,195,243,60,154,16,200,180,251,211,248,253,34,133,27,17,160,
  193,107,57,243,84,43,153,193,19,171,146,65,2,83,190,225,145,165,115,176,235,132,253,78,86,43,154,165,34,138,152,2,34,20,154,124,46,73,122,136,
  155,228,220,220,28,29,131,227,56,104,52,26,84,190,206,48,12,212,235,117,74,165,169,84,42,152,157,189,11,199,177,99,235,164,179,94,52,174,249,79,
  97,214,69,192,0,162,108,2,219,65,171,73,35,98,70,241,141,204,103,80,210,158,161,178,177,163,146,246,199,156,197,109,90,233,46,217,69,44,128,231,
  133,155,95,173,190,0,223,119,81,169,244,96,97,62,142,80,119,144,100,15,111,188,241,59,168,106,62,202,6,156,88,230,111,217,1,20,0,249,28,177,
  9,213,0,184,12,26,16,5,187,173,22,116,195,128,110,152,176,173,166,144,67,203,150,92,234,205,22,70,86,246,65,81,114,8,130,54,93,40,158,23,
  183,124,141,216,69,176,29,23,182,211,249,220,201,73,23,109,203,197,181,137,113,204,206,76,118,73,214,240,77,7,236,107,102,230,54,126,246,179,95,163,
  80,40,162,221,182,177,180,180,4,215,117,162,239,13,179,47,4,42,30,121,228,11,40,228,13,188,246,198,239,17,4,128,174,41,80,84,192,247,93,232,
  122,17,11,11,183,187,184,178,119,103,22,112,248,208,78,220,183,121,27,28,199,133,235,185,161,38,181,22,26,193,40,208,160,106,6,60,207,129,166,234,
  200,229,115,208,53,192,52,75,232,237,237,193,138,254,62,228,115,121,152,166,6,77,213,17,4,46,86,172,232,199,210,210,66,215,179,39,255,189,125,199,
  38,104,42,240,202,43,175,32,128,135,32,112,161,40,57,40,138,9,69,113,225,121,33,74,110,89,75,52,120,6,66,147,150,66,49,135,117,107,214,32,
  112,219,93,205,45,178,3,143,69,6,100,65,73,82,80,42,226,142,202,190,39,235,226,148,141,251,243,200,156,45,71,183,58,107,64,154,164,97,44,67,
  132,121,132,63,237,90,210,44,180,101,26,206,73,141,162,34,249,62,130,222,240,207,149,215,180,78,10,52,147,154,53,179,62,47,118,157,139,20,94,210,
  158,227,231,65,104,211,130,109,254,224,236,214,93,69,38,122,205,114,230,42,31,56,139,206,21,162,43,203,163,113,60,178,47,186,119,188,86,179,40,32,
  78,123,133,32,138,150,58,127,67,100,76,132,242,7,180,42,74,14,251,14,66,166,83,190,38,9,138,59,42,26,132,191,170,115,252,103,5,174,11,52,
  26,45,234,129,64,154,177,136,227,28,47,181,40,74,158,146,56,207,164,87,134,239,31,33,193,218,141,27,55,48,57,57,137,141,27,55,210,207,26,30,
  30,198,170,85,171,104,240,26,4,1,202,229,50,92,215,197,135,31,126,136,179,103,207,98,215,174,93,120,248,225,135,177,115,231,78,250,61,35,35,35,
  248,250,215,191,142,195,135,15,227,251,223,255,62,110,223,190,29,83,184,32,230,73,150,101,161,88,44,98,195,134,13,88,92,92,196,237,219,183,113,247,
  238,93,188,246,218,107,88,181,106,21,242,249,60,206,156,57,131,203,151,47,67,81,20,20,139,69,184,174,139,225,225,97,124,241,139,95,196,205,155,55,
  49,49,49,129,222,222,94,172,89,179,6,158,231,225,218,181,107,88,88,88,160,136,41,219,136,40,218,147,178,244,195,240,156,96,50,15,121,153,64,126,
  31,226,169,173,162,103,198,83,172,8,117,67,180,158,88,190,58,111,136,68,222,199,43,43,145,247,177,13,134,162,32,159,124,159,97,24,40,22,139,168,
  84,42,216,188,121,51,22,22,22,48,53,53,69,233,53,36,121,1,128,114,84,61,175,213,106,0,128,145,145,17,148,74,37,76,78,182,153,94,37,85,
  136,244,178,60,231,14,173,3,76,31,0,233,97,80,24,117,29,21,162,70,195,180,253,56,61,184,70,140,230,43,173,130,138,140,84,210,38,80,86,84,
  135,127,79,248,128,115,17,21,195,136,202,8,30,10,133,34,119,51,61,110,3,244,224,251,141,88,134,66,52,62,61,223,134,162,134,146,63,225,205,118,
  99,157,157,49,18,190,174,160,144,47,195,182,22,99,155,138,72,138,172,109,89,208,180,0,190,111,209,32,92,164,5,200,163,24,134,94,70,111,79,9,
  195,43,71,97,26,26,182,108,218,140,129,21,101,44,46,45,225,222,189,73,169,13,46,127,191,111,221,26,79,69,208,110,77,78,97,227,134,81,52,155,
  11,2,36,169,42,84,94,184,118,245,34,238,191,127,45,246,63,240,0,150,150,150,194,242,137,162,35,151,43,194,245,45,232,170,9,69,1,86,244,175,
  64,185,156,199,221,187,179,104,183,45,56,118,27,181,122,29,183,110,77,192,113,124,216,150,135,102,99,9,45,171,133,124,174,8,207,115,132,25,221,154,
  53,247,97,227,134,149,0,128,103,159,125,26,1,20,40,240,81,40,244,132,207,57,8,39,161,231,251,8,60,23,182,235,64,129,2,77,5,202,149,62,
  104,186,138,234,210,2,206,142,157,142,149,44,69,168,97,150,32,68,180,89,137,198,45,226,213,102,81,29,144,5,216,178,113,39,209,31,210,212,29,228,
  73,42,132,229,62,62,232,21,161,39,50,180,133,108,178,132,91,200,7,162,252,88,249,70,227,229,38,13,73,122,211,172,221,55,143,224,164,53,28,38,
  205,11,153,62,116,150,0,82,4,40,240,207,57,109,126,6,203,133,80,50,160,210,172,105,141,136,243,42,75,128,146,180,185,211,18,47,89,149,66,38,
  111,152,52,239,147,92,52,59,7,242,242,2,254,80,139,62,155,252,100,220,52,165,91,179,186,163,53,29,8,13,86,58,235,11,17,10,205,82,117,136,
  138,7,177,234,14,123,117,154,205,54,28,199,165,188,79,195,48,98,168,34,203,111,101,127,46,106,176,148,37,243,205,102,72,7,92,191,126,61,118,237,
  218,133,158,158,30,52,26,13,204,207,207,99,102,102,6,139,139,139,152,157,157,197,204,204,12,54,110,220,72,223,167,170,42,214,173,91,135,79,63,253,
  180,203,64,168,167,167,7,142,227,224,147,79,62,193,133,11,23,176,107,215,46,28,61,122,148,218,112,95,189,122,21,31,127,252,49,170,213,106,140,130,
  163,170,42,213,119,46,151,203,56,112,224,0,118,236,216,129,201,201,73,184,174,139,215,94,123,13,99,99,99,248,236,179,207,80,42,149,104,16,172,40,
  10,218,237,54,77,42,154,205,38,238,187,239,62,92,191,126,29,199,143,31,135,105,154,104,181,90,200,231,243,212,65,143,221,15,100,198,35,252,189,205,
  10,10,144,49,177,188,234,180,132,146,7,39,69,9,45,89,195,172,212,42,171,52,193,246,233,136,36,55,249,245,222,45,5,23,79,110,201,103,147,166,
  69,93,215,49,63,63,15,215,117,209,223,223,143,233,233,105,218,160,104,24,6,229,59,147,0,90,81,20,90,13,88,179,102,13,154,205,102,212,59,64,
  238,153,159,146,224,178,149,1,126,127,11,24,30,243,242,246,204,248,218,80,18,104,27,241,96,188,67,7,238,222,171,52,81,0,157,116,112,37,161,115,
  217,54,92,55,44,253,171,122,104,121,29,185,223,197,59,169,227,23,241,244,83,207,224,230,205,105,156,59,127,134,147,48,113,160,4,64,224,135,114,105,
  252,70,26,110,108,42,20,197,195,210,226,34,84,69,65,177,88,70,181,154,3,96,39,115,253,130,0,154,102,32,8,194,247,103,209,43,221,179,235,32,
  30,124,112,31,114,121,29,142,235,194,243,28,60,120,96,39,234,181,141,168,244,20,241,241,199,167,241,238,187,111,39,30,12,188,229,38,123,16,139,238,
  121,136,204,139,15,32,209,123,154,205,57,252,228,63,126,72,55,125,217,1,188,126,253,78,60,249,248,81,252,230,197,23,209,108,215,18,13,12,170,88,
  16,6,146,138,162,0,138,143,177,177,79,225,182,61,212,91,117,180,45,27,158,237,161,101,53,17,32,64,62,87,192,151,190,244,37,120,110,11,99,99,
  151,160,168,26,28,199,69,189,182,132,90,189,142,122,163,133,118,187,46,220,232,100,220,95,62,235,76,58,132,165,227,94,6,149,32,11,18,44,27,119,
  82,201,61,237,251,146,154,82,248,224,133,223,208,211,54,30,126,19,102,171,51,188,204,145,140,187,44,115,63,92,14,117,67,246,108,69,202,18,105,165,
  215,207,139,158,242,252,208,229,6,182,178,96,38,13,164,72,26,127,150,70,82,66,145,96,3,29,222,52,138,127,70,89,52,171,211,232,33,50,147,170,
  180,96,36,41,153,203,150,184,40,153,40,75,221,251,129,2,98,188,149,124,221,1,52,77,21,52,30,131,162,203,113,4,171,35,139,71,244,162,59,9,
  30,251,61,6,28,199,69,163,209,146,86,112,89,41,53,18,72,241,207,85,70,217,224,63,83,211,52,28,58,116,8,15,62,248,32,214,173,91,135,98,
  177,24,155,227,237,118,59,134,6,179,223,227,251,62,30,120,224,1,156,60,121,18,119,239,222,141,248,173,106,204,20,137,232,53,127,252,241,199,184,116,
  233,18,54,108,216,128,32,8,112,227,198,13,52,26,13,106,232,66,198,212,110,183,113,255,253,247,99,255,254,253,232,235,235,195,185,115,231,112,250,244,
  105,84,42,149,152,146,3,208,81,150,8,130,0,207,61,247,28,138,197,34,126,241,139,95,96,97,97,1,47,189,244,18,138,197,34,10,133,2,44,203,
  10,171,206,209,94,197,7,179,50,69,37,25,120,34,234,149,18,37,130,108,163,159,168,137,80,132,60,243,193,58,187,14,89,245,26,30,25,22,5,224,
  108,66,35,10,150,217,113,202,184,221,249,124,30,133,66,161,227,161,81,175,195,178,44,122,255,9,245,134,240,161,27,141,6,108,59,116,115,238,235,235,
  163,65,125,185,92,70,169,84,194,236,236,108,180,110,72,239,154,210,37,63,199,7,206,44,50,221,189,119,7,2,148,63,123,47,83,82,124,36,162,116,
  177,211,68,212,48,175,123,158,7,40,10,52,166,25,48,201,189,42,141,119,153,188,9,17,233,145,22,20,24,80,85,13,154,162,71,220,152,128,227,200,
  132,153,87,163,217,132,109,59,204,228,210,169,253,182,174,155,80,84,5,166,89,228,143,174,136,87,22,242,100,90,86,232,84,88,169,20,113,231,142,205,
  220,36,49,138,214,110,215,97,24,122,228,94,232,73,17,70,130,64,28,60,112,4,7,14,236,193,217,79,175,226,210,165,115,88,90,90,128,105,104,40,
  150,202,152,157,157,199,234,213,171,241,204,211,79,64,85,20,188,247,254,241,76,60,196,100,132,73,193,208,208,16,26,141,69,233,65,193,46,162,238,146,
  168,220,81,47,8,2,220,188,57,142,165,218,131,168,244,20,208,108,215,62,183,252,214,244,212,4,166,167,38,18,131,128,254,161,81,172,40,171,248,228,
  204,39,137,104,16,79,125,225,147,61,209,225,241,135,202,134,137,180,141,101,168,225,231,25,247,114,203,246,73,136,123,22,154,6,123,159,100,168,115,22,
  20,150,229,172,102,9,4,217,50,97,82,80,45,106,188,36,229,125,209,126,228,186,110,42,90,154,5,109,228,15,23,54,249,74,11,94,151,51,167,216,
  160,82,20,0,101,67,79,178,35,208,34,151,68,217,253,102,15,222,180,113,241,1,8,175,233,205,174,155,172,50,143,203,73,126,197,247,168,179,31,119,
  156,255,228,207,57,169,129,82,204,125,86,187,202,196,221,42,11,36,97,21,73,218,177,115,95,139,184,162,109,88,86,93,120,182,240,110,128,236,184,121,
  109,120,126,205,176,207,141,109,24,243,60,15,43,87,174,196,182,109,219,208,211,211,67,41,28,36,80,35,60,100,81,50,70,62,111,104,104,8,255,244,
  79,255,132,87,94,121,5,231,206,157,139,53,228,145,121,100,154,38,85,95,56,127,254,124,12,209,100,65,34,82,237,88,88,88,192,171,175,190,138,175,
  126,245,171,216,187,119,47,222,123,239,61,156,62,125,26,139,139,139,20,57,46,151,203,232,233,233,129,239,251,184,125,251,54,78,156,56,129,199,30,123,
  12,143,61,246,24,94,122,233,37,120,158,135,118,187,77,3,103,254,124,75,106,220,91,78,194,205,38,19,172,132,28,11,86,176,251,190,136,94,193,206,
  57,89,133,144,13,228,9,197,133,252,156,196,89,236,179,19,85,13,121,208,132,111,46,36,63,39,251,185,97,24,232,237,237,69,169,84,162,116,141,15,
  62,248,0,182,109,163,84,42,133,6,104,245,58,189,118,211,52,177,176,176,0,199,113,80,42,149,208,215,215,71,85,64,250,251,251,233,222,109,89,237,
  104,109,116,116,149,217,235,14,175,67,148,4,242,180,10,254,57,249,153,246,17,34,101,153,157,130,23,167,146,36,237,207,58,187,121,200,80,128,44,200,
  137,204,88,34,222,4,146,135,239,135,77,128,150,213,70,224,185,48,243,121,168,106,14,190,239,32,180,217,86,41,34,16,4,192,91,111,189,25,27,147,
  170,250,212,52,133,168,155,228,76,147,67,175,125,122,35,20,69,129,21,101,213,149,74,143,96,65,168,145,156,74,103,236,118,36,115,163,105,6,92,183,
  153,184,208,54,172,95,143,131,7,247,226,87,191,126,9,147,147,19,24,28,24,65,16,0,141,102,13,141,102,29,154,90,192,228,228,53,252,228,167,75,
  120,254,249,231,112,241,210,101,204,207,223,77,188,135,73,38,29,107,86,111,198,145,35,143,160,175,175,140,147,31,140,225,129,7,30,134,239,57,184,124,
  229,10,154,205,165,174,160,133,125,6,162,140,92,236,3,111,195,113,28,228,243,133,174,160,45,9,121,76,218,148,100,1,123,173,90,197,232,200,26,41,
  95,146,119,107,100,249,172,50,126,46,143,180,253,41,198,45,43,203,45,103,220,252,38,194,163,203,178,198,201,172,212,2,126,92,105,182,222,34,105,64,
  30,193,202,130,196,202,228,1,69,182,205,60,42,176,28,123,245,180,224,153,253,92,62,72,226,251,30,100,138,36,105,218,213,73,96,130,232,254,139,92,
  40,101,212,156,52,234,78,82,146,36,218,211,101,148,13,81,192,156,166,222,33,163,64,177,135,185,236,251,178,104,99,39,113,206,147,14,194,100,91,123,
  112,104,213,242,147,161,184,1,12,255,251,128,26,158,132,151,24,58,13,18,195,147,16,144,81,34,195,147,144,39,202,242,102,69,201,58,111,245,204,151,
  250,217,247,176,37,126,203,10,181,161,121,11,239,185,185,57,252,224,7,63,128,235,186,200,229,114,84,9,163,82,169,96,96,96,128,254,63,65,16,7,
  7,7,49,60,60,28,147,82,91,181,106,21,254,243,127,254,207,248,236,179,207,112,226,196,9,92,191,126,157,6,78,108,67,26,9,178,72,0,216,110,
  183,145,203,229,40,98,153,203,229,16,4,1,230,231,231,97,219,54,94,124,241,69,228,243,121,220,187,119,15,170,170,34,159,207,67,215,117,180,90,45,
  60,242,200,35,216,176,97,3,250,250,250,112,227,198,13,252,244,167,63,197,247,191,255,125,244,245,245,197,230,177,200,24,78,54,111,68,84,141,36,83,
  41,222,53,144,159,219,73,218,233,50,67,35,94,134,85,212,168,200,38,28,124,163,175,8,85,229,207,29,158,190,193,175,67,34,55,103,154,38,102,103,
  103,41,111,125,102,102,134,94,47,81,219,104,54,155,240,125,159,162,253,132,75,61,58,58,138,124,62,143,106,181,74,121,233,150,101,81,5,16,153,124,
  92,124,189,42,169,149,2,241,153,158,13,112,146,1,87,178,247,103,169,62,235,97,32,234,67,75,56,176,100,138,1,89,202,145,113,130,122,199,2,213,
  241,44,44,44,46,98,203,166,117,152,159,157,193,252,252,60,234,245,187,212,127,60,124,143,14,85,205,193,117,91,84,235,89,211,244,72,79,208,135,235,
  133,165,5,221,48,163,247,177,52,144,142,36,138,231,219,176,29,11,165,114,62,150,189,116,164,238,226,227,119,29,15,134,161,195,204,229,224,56,106,98,
  121,124,219,182,45,184,122,229,58,166,167,239,226,224,129,47,224,225,47,28,192,111,126,243,6,166,167,38,241,244,83,79,33,151,215,240,139,95,254,18,
  115,115,247,112,225,220,101,236,216,190,27,39,222,189,251,185,40,1,59,118,60,128,47,127,233,17,76,92,191,142,227,199,79,161,221,182,177,115,215,22,
  172,90,57,130,131,7,15,224,173,183,222,193,149,171,231,133,229,99,126,241,138,148,6,86,173,90,143,117,235,54,162,144,211,177,122,245,16,206,124,156,
  108,246,145,84,110,151,185,151,241,129,97,179,217,134,145,51,19,3,215,44,92,230,164,251,248,167,24,119,82,64,157,117,220,236,166,199,163,157,252,184,
  88,141,85,89,208,70,54,100,118,99,229,147,137,44,202,10,105,244,162,52,148,134,252,29,223,224,149,134,86,203,244,150,63,143,73,13,123,64,202,132,
  240,69,207,123,57,213,1,153,11,43,31,68,203,108,205,101,244,19,81,175,9,127,112,138,74,144,73,7,126,90,80,44,10,14,248,70,189,172,149,198,
  180,251,183,60,100,89,137,57,95,166,31,168,202,178,36,34,201,199,137,40,71,188,61,184,56,97,87,104,131,97,248,126,8,236,188,3,20,10,101,220,
  185,51,75,27,0,121,27,121,246,222,103,105,212,231,247,35,77,211,40,2,107,154,38,70,71,71,177,113,227,70,172,91,183,14,165,82,137,162,178,174,
  235,162,90,173,98,114,114,18,19,19,19,184,113,227,6,238,222,189,75,185,214,188,117,113,127,127,63,246,236,217,131,195,135,15,99,205,154,53,52,24,
  214,52,13,187,118,237,194,206,157,59,241,179,159,253,12,191,255,253,239,99,247,143,69,77,93,215,69,161,80,192,83,79,61,5,203,178,112,235,214,45,
  220,189,123,23,213,106,149,122,75,152,166,137,106,181,138,165,165,37,170,238,80,169,84,240,245,175,127,29,0,48,62,62,142,147,39,79,98,116,116,148,
  162,220,33,186,105,117,41,138,240,238,126,203,9,144,69,160,137,172,186,201,255,158,223,75,248,196,153,221,15,68,213,51,54,161,100,247,123,22,121,22,
  129,155,44,53,68,36,79,39,170,156,123,158,7,195,48,98,231,78,171,213,66,187,221,166,252,242,133,133,5,42,105,104,24,6,26,141,6,154,205,38,
  28,199,137,153,229,24,134,129,161,161,33,106,114,227,251,62,202,229,50,237,115,235,168,159,5,93,193,106,92,63,189,19,208,178,203,92,20,92,139,17,
  233,160,107,143,39,178,119,172,110,191,72,50,121,57,85,93,246,231,58,128,88,240,156,20,197,139,50,98,89,150,197,119,173,134,159,101,3,240,176,105,
  227,125,216,190,99,43,86,244,15,32,151,51,49,56,248,101,216,182,131,143,62,62,139,179,99,39,65,244,254,20,197,193,67,135,30,68,219,86,112,230,
  204,41,74,201,8,127,15,228,243,5,4,65,128,182,101,71,223,97,80,4,181,67,9,9,141,85,90,205,22,6,7,7,160,40,5,4,129,136,51,173,
  80,149,141,118,219,134,170,42,40,228,139,104,212,121,110,12,199,249,83,77,20,10,121,124,253,107,95,197,234,117,43,161,170,10,12,93,195,190,125,251,
  177,102,221,8,84,85,193,253,247,111,195,167,159,126,140,98,161,132,106,189,150,136,182,200,30,214,170,149,27,241,229,47,126,1,111,191,245,30,62,59,
  127,26,135,14,30,195,131,7,182,162,81,111,224,238,221,57,92,188,120,25,79,60,249,24,10,133,28,206,126,122,58,117,98,176,11,218,52,203,248,202,
  147,95,193,224,80,31,102,103,102,224,122,62,22,22,150,176,97,211,86,216,30,48,53,117,61,51,202,195,207,153,36,19,9,77,47,96,207,158,29,88,
  57,50,140,7,246,31,198,205,27,227,152,157,159,137,29,38,162,64,95,118,224,45,39,224,249,67,198,157,148,104,46,119,220,162,234,14,159,121,179,58,
  155,73,193,131,72,162,136,215,51,78,10,254,146,168,40,124,0,198,55,185,176,7,190,232,96,72,163,79,164,41,171,44,7,33,20,125,30,219,4,156,
  164,127,45,122,14,105,115,44,141,235,40,58,208,146,172,201,69,78,139,108,224,42,226,50,202,170,6,162,189,69,214,56,203,82,109,68,115,75,134,176,
  103,73,200,100,14,110,178,106,37,63,191,228,138,27,217,131,248,164,189,33,126,255,136,70,45,209,155,13,40,37,164,243,172,193,104,59,43,140,67,32,
  98,65,124,88,253,208,113,231,206,12,150,150,170,40,149,74,40,20,10,49,126,58,111,29,45,146,97,20,5,123,228,126,180,90,45,180,90,45,148,74,
  37,28,62,124,24,123,246,236,193,250,245,235,169,205,182,204,220,104,113,113,17,19,19,19,248,232,163,143,112,246,236,89,26,40,177,110,145,11,11,11,
  120,237,181,215,240,209,71,31,225,225,135,31,198,23,190,240,5,12,14,14,2,0,110,223,190,77,85,49,8,202,77,190,163,213,106,81,238,108,16,4,
  40,22,139,24,25,25,137,128,160,29,56,117,234,20,78,156,56,129,66,161,64,175,157,85,200,32,207,249,220,185,115,24,29,29,69,127,127,63,222,126,
  251,109,124,244,209,71,244,111,89,176,64,100,224,211,145,98,245,40,93,36,173,140,159,165,1,144,61,67,217,245,71,158,35,31,252,138,2,113,145,190,
  60,121,254,108,224,79,26,249,100,213,8,126,172,188,246,186,136,83,79,18,144,21,43,86,192,52,77,172,89,179,6,183,111,223,198,249,243,231,225,121,
  30,53,185,105,54,155,116,31,43,20,10,104,52,26,104,183,219,208,52,13,195,195,195,180,178,151,207,231,145,207,231,209,110,183,97,219,22,70,70,86,
  192,178,234,112,156,28,141,229,130,64,97,246,23,159,202,60,178,1,49,73,68,217,70,195,206,58,10,36,148,211,238,128,58,188,255,74,108,221,178,10,
  31,89,228,61,179,208,242,130,0,98,21,14,209,164,137,59,51,165,35,209,252,68,9,255,222,195,64,255,0,158,125,246,73,180,90,30,16,248,184,119,
  111,30,239,188,243,9,84,181,133,111,124,227,89,180,154,117,92,189,122,1,154,86,68,16,180,113,251,206,93,168,90,49,70,22,87,85,5,170,26,160,
  183,103,69,72,3,168,45,146,91,199,241,98,52,38,144,86,160,105,30,130,192,162,25,14,159,49,82,222,156,175,118,161,132,44,119,152,125,125,118,246,
  28,254,226,175,254,28,147,147,119,241,111,255,254,83,28,124,112,55,70,86,174,194,185,243,159,97,124,226,26,14,30,120,0,235,215,175,135,174,233,184,
  111,219,122,188,240,194,47,165,11,55,137,42,176,97,195,6,156,63,127,5,99,159,134,162,241,87,175,141,97,112,40,135,251,239,223,2,211,52,241,250,
  27,175,99,102,118,6,79,63,117,12,55,110,220,194,82,117,54,227,225,169,227,107,95,251,6,218,173,5,252,240,223,127,4,203,110,66,81,114,80,85,
  3,149,74,9,95,249,202,19,168,215,118,227,165,151,95,204,140,0,243,101,90,81,176,98,154,21,252,213,95,253,37,20,184,184,125,59,236,246,126,240,
  193,253,56,253,241,135,248,240,147,143,98,25,185,168,252,155,182,0,62,15,114,205,143,219,52,77,236,219,187,23,251,246,238,165,127,115,102,108,12,167,
  62,252,80,26,60,203,198,93,46,151,241,173,231,158,3,0,188,252,202,43,152,154,158,150,110,228,188,76,27,217,156,77,211,196,163,71,143,98,83,36,
  45,53,59,59,139,215,223,124,147,118,182,139,16,201,71,143,30,197,246,109,219,98,63,159,154,158,198,201,83,167,48,59,59,43,84,71,72,11,160,201,
  127,175,30,29,197,51,79,63,141,106,181,138,23,126,242,147,174,247,177,226,253,162,100,135,175,10,36,241,141,87,143,142,226,217,103,158,233,250,142,83,
  31,126,136,211,103,206,8,3,105,246,187,61,207,195,142,237,219,49,56,56,136,119,223,123,79,120,16,42,138,130,189,123,246,224,240,161,67,184,112,241,
  34,222,57,126,60,113,131,173,84,42,120,254,155,223,4,0,252,240,199,63,70,189,94,151,74,245,145,207,62,116,240,32,0,224,59,223,251,30,76,211,
  196,87,159,121,134,6,36,175,190,246,26,110,220,184,33,108,42,202,234,192,149,21,13,78,146,46,20,37,149,34,131,32,54,216,96,165,35,69,47,145,
  46,185,232,122,216,132,140,111,128,229,209,90,150,7,201,202,99,197,155,244,178,191,196,230,66,221,168,24,107,198,208,109,168,66,170,41,74,36,217,10,
  44,46,214,208,106,89,20,197,107,183,219,168,84,42,200,229,114,49,179,11,18,16,202,80,103,126,255,179,44,11,142,227,160,175,175,15,143,60,242,8,
  246,237,219,135,181,107,215,198,104,52,179,179,179,88,90,90,66,111,111,47,157,103,100,190,247,246,246,98,255,254,253,216,182,109,27,206,156,57,131,151,
  94,122,9,141,70,131,54,249,133,254,5,26,45,205,191,250,234,171,24,27,27,195,238,221,187,97,89,22,206,159,63,143,185,185,185,152,249,9,145,53,
  219,176,97,3,14,28,56,0,69,81,112,254,252,121,92,186,116,9,255,246,111,255,134,53,107,214,160,84,42,225,230,205,155,244,123,72,179,32,9,114,
  9,66,111,89,22,78,157,58,5,77,211,208,219,219,11,219,182,169,22,49,153,23,172,42,144,200,0,133,85,149,145,201,198,241,136,165,104,63,78,211,
  87,103,165,230,216,70,79,54,160,101,255,159,69,142,217,222,179,142,186,76,184,223,179,215,38,2,95,216,53,201,87,143,248,138,33,63,199,102,103,103,
  177,101,203,22,108,223,190,29,247,238,221,163,215,80,42,149,96,89,22,234,245,58,173,30,84,42,21,220,187,119,15,237,118,27,35,35,35,232,239,239,
  135,109,219,176,109,155,234,63,47,46,46,34,8,92,212,235,139,208,117,51,146,134,12,186,214,10,145,124,236,4,190,65,108,157,177,84,214,14,151,187,
  251,156,232,152,170,40,136,55,254,41,148,233,208,205,185,86,4,116,169,229,53,186,119,192,11,65,0,205,6,18,50,174,158,204,157,141,148,37,120,126,
  33,251,25,131,253,131,8,160,225,247,111,127,128,135,31,218,142,185,133,6,250,250,138,48,115,21,76,79,223,197,238,189,187,49,49,113,43,202,198,90,
  184,126,227,122,44,195,7,84,172,90,185,30,171,70,87,98,211,250,117,80,85,21,229,82,15,238,66,137,2,102,226,60,226,69,255,237,161,88,28,66,
  185,92,194,220,124,19,128,6,69,113,18,14,34,5,197,82,17,170,170,160,92,174,32,212,12,87,33,59,167,110,220,26,199,133,11,151,176,114,229,106,
  172,93,179,10,183,239,220,131,109,7,152,159,187,3,64,67,173,214,196,131,7,118,98,211,198,53,120,239,196,39,184,125,251,186,244,64,74,66,195,76,
  83,199,226,82,149,78,254,249,249,123,120,235,173,19,88,181,114,4,141,70,216,24,57,53,117,13,183,111,239,193,218,181,171,177,116,110,54,118,8,242,
  229,100,242,140,247,237,59,136,156,225,227,103,63,123,9,64,128,222,158,17,124,245,171,79,163,213,108,224,141,55,95,199,79,127,250,31,248,235,191,250,
  22,54,111,220,130,107,19,87,187,144,161,229,82,81,130,32,0,20,3,95,253,218,159,163,89,159,195,47,127,253,50,134,6,215,98,113,233,30,6,6,
  70,240,196,19,199,48,191,184,128,107,227,215,132,85,141,172,89,163,236,62,46,103,220,108,112,67,94,36,152,62,245,225,135,194,13,85,86,141,9,157,
  42,147,147,212,180,133,187,99,251,118,108,138,100,165,0,96,112,112,16,123,247,236,193,241,19,39,132,129,128,162,132,70,69,1,57,253,163,77,117,213,
  202,149,120,234,201,39,241,31,63,253,41,44,203,146,38,60,178,32,135,124,254,228,212,20,254,249,187,223,149,162,197,44,18,157,20,200,137,42,93,210,
  138,88,16,80,33,123,0,56,116,240,32,102,102,102,112,107,114,82,170,254,65,238,221,23,31,125,148,38,46,178,196,103,236,236,89,140,157,61,155,90,
  145,75,210,170,78,66,172,217,223,29,61,114,132,206,175,183,222,126,27,227,227,227,194,131,48,205,33,50,41,33,204,42,221,40,66,223,68,251,62,139,
  174,241,200,186,12,193,99,229,244,68,77,88,178,64,133,124,15,217,187,200,124,98,131,231,172,123,65,247,159,41,137,13,147,113,250,70,32,224,59,147,
  115,161,251,61,29,235,111,29,138,2,44,45,85,161,105,97,48,74,168,7,139,139,139,200,231,243,40,22,139,52,80,21,169,52,240,8,52,121,6,142,
  227,96,104,104,8,15,62,248,32,30,120,224,1,244,246,246,210,107,32,186,204,31,125,244,17,230,230,230,40,173,99,213,170,85,56,112,224,0,30,120,
  224,1,138,52,43,138,130,92,46,135,195,135,15,99,120,120,24,63,248,193,15,80,173,86,41,63,153,4,94,165,82,9,190,239,99,102,102,6,111,188,
  241,6,125,14,100,236,100,92,35,35,35,180,236,191,99,199,14,184,174,139,141,27,55,66,215,117,124,252,241,199,24,31,31,143,85,83,10,133,80,137,
  201,247,125,92,190,124,25,119,239,222,141,201,81,146,239,173,215,235,93,244,52,214,232,141,95,127,34,254,44,79,125,73,162,204,137,184,200,124,213,72,
  86,181,228,185,203,188,101,56,251,140,201,216,121,52,155,200,226,241,137,19,79,215,32,21,11,54,56,230,27,78,201,119,152,166,9,219,182,41,194,223,
  108,54,41,117,35,148,181,13,239,13,145,167,107,181,90,8,130,0,125,125,125,212,94,157,252,191,170,170,148,50,68,84,82,66,238,125,43,82,243,40,
  194,117,253,136,198,65,238,145,31,5,181,172,194,6,137,29,189,200,182,219,235,122,62,29,74,47,219,248,72,26,159,181,174,102,194,184,1,75,55,245,
  163,155,11,191,124,244,57,100,31,168,114,25,59,254,112,97,165,116,72,73,65,118,96,202,203,123,58,180,232,225,111,223,182,6,131,131,3,24,26,26,
  194,250,213,67,40,150,195,172,199,243,21,228,242,5,52,27,97,35,193,254,189,7,208,182,93,156,63,127,6,64,128,135,30,122,24,7,30,124,0,174,
  235,34,0,176,176,176,136,71,143,62,136,251,239,95,139,147,39,63,193,252,252,20,194,142,231,34,130,160,133,189,123,30,196,225,135,14,163,88,204,161,
  84,42,225,207,254,236,207,113,225,194,5,92,190,124,145,186,217,40,138,6,67,47,96,215,206,61,184,127,219,125,232,235,171,0,0,30,63,246,8,198,
  39,86,227,227,143,199,176,180,116,87,120,112,106,154,134,215,94,251,29,14,60,120,0,187,119,238,68,161,152,71,16,248,56,122,244,48,28,219,65,177,
  148,199,252,124,21,199,143,191,135,241,241,139,169,1,158,12,25,186,53,121,3,123,247,236,197,199,159,132,147,165,88,28,196,214,173,219,113,250,204,37,
  60,250,232,1,140,12,15,97,250,118,51,18,218,87,133,7,80,55,215,203,192,253,247,109,193,199,31,190,79,203,41,249,124,14,115,115,139,184,127,235,
  122,236,221,187,31,111,189,253,58,62,250,228,99,220,183,117,43,174,77,92,141,33,74,73,82,48,73,137,214,225,195,71,208,83,208,241,239,191,122,19,
  59,119,31,196,163,71,30,192,216,199,167,241,222,169,247,241,202,111,95,195,151,190,120,16,183,110,220,128,237,185,93,239,21,117,47,39,241,112,101,58,
  194,105,227,222,180,113,35,6,7,7,17,4,1,78,188,251,46,206,95,184,128,29,219,183,227,232,145,35,93,1,53,65,21,45,203,194,169,15,63,196,
  249,11,23,104,128,251,228,227,143,163,82,169,224,212,135,31,210,192,143,140,105,211,198,141,120,244,232,81,138,222,140,79,76,224,131,147,39,169,16,61,
  31,204,144,255,63,119,254,60,38,38,38,40,250,43,163,34,16,92,76,81,20,76,77,79,227,215,47,190,136,213,163,163,248,179,175,125,13,197,98,17,
  59,182,111,199,199,159,124,130,66,161,128,35,143,60,130,45,155,55,3,8,229,138,78,158,58,133,11,23,47,34,151,203,81,212,155,92,227,239,94,123,
  13,83,211,211,88,179,122,53,158,125,230,25,212,106,53,252,232,133,23,162,53,115,12,155,54,110,196,189,123,247,112,253,198,13,28,58,120,144,162,185,
  207,63,247,28,122,42,21,252,252,151,191,196,154,213,171,1,0,75,75,75,152,184,126,29,61,61,61,20,161,127,233,229,151,187,2,93,250,204,131,0,
  191,254,205,111,48,191,176,128,191,121,254,121,152,166,137,161,161,33,76,77,79,227,208,193,131,177,106,193,173,201,73,188,241,230,155,24,26,28,196,23,
  31,125,148,34,217,255,245,31,255,17,223,249,222,247,112,244,200,17,108,223,182,13,31,156,60,137,92,46,135,181,107,214,224,234,181,107,20,129,38,137,
  9,251,140,1,224,228,169,83,248,248,147,79,186,158,11,249,254,90,173,134,215,222,120,3,179,179,179,82,78,229,161,131,7,177,121,211,38,0,192,241,
  19,39,112,241,210,37,40,138,130,237,219,183,227,139,71,143,162,90,173,226,199,255,241,31,216,191,111,95,236,30,254,205,243,207,163,84,42,225,151,191,
  254,53,189,135,213,106,21,215,198,199,177,125,219,182,174,249,73,208,110,17,213,36,43,15,158,77,26,68,235,47,139,77,56,111,147,156,196,241,205,74,
  213,233,118,121,204,230,32,199,35,213,162,36,128,4,194,68,33,32,206,207,12,75,195,188,97,23,251,123,226,58,8,132,72,162,101,217,212,222,152,112,
  89,219,237,118,212,172,29,74,135,177,77,222,34,170,1,249,121,163,209,192,174,93,187,240,237,111,127,155,218,103,123,158,71,155,6,93,215,197,165,75,
  151,112,233,210,37,90,126,111,183,219,184,114,229,10,46,93,186,132,15,63,252,16,95,251,218,215,176,118,237,90,250,62,211,52,177,121,243,102,124,233,
  75,95,194,79,126,242,19,58,78,62,121,35,168,49,185,103,173,86,139,6,110,166,105,226,217,103,159,197,208,208,16,62,250,232,35,252,240,135,63,196,
  234,213,171,81,175,215,113,245,234,85,148,74,37,184,174,75,221,13,45,203,194,202,149,43,177,107,215,46,204,207,207,99,231,206,157,248,244,211,79,241,
  251,223,255,62,38,117,39,74,188,146,42,139,108,18,42,170,130,138,144,89,94,158,83,166,240,36,74,26,69,213,122,94,178,148,95,55,172,74,6,251,
  29,44,23,157,149,196,35,200,59,31,144,179,231,25,207,251,38,223,145,203,229,168,70,115,111,111,47,38,39,39,233,25,67,232,54,108,21,32,151,203,
  161,94,175,83,73,195,209,209,81,26,128,23,139,69,244,246,246,82,254,121,62,175,195,117,109,212,106,53,42,131,168,170,90,148,92,6,240,60,135,198,
  142,97,194,171,199,20,107,136,164,93,24,248,147,181,28,72,130,90,133,50,7,66,99,185,78,144,236,121,129,64,55,154,240,174,179,68,200,172,242,134,
  18,91,215,162,230,83,18,59,234,73,72,9,143,248,137,54,189,229,241,72,60,76,223,190,13,93,5,6,6,135,49,61,117,23,43,87,13,227,202,181,
  155,88,183,118,4,43,250,251,97,217,46,76,51,135,102,228,157,98,59,13,12,14,173,134,162,104,88,187,106,61,30,122,232,0,102,103,22,97,91,14,
  2,40,56,127,241,50,166,38,39,240,197,71,31,198,95,252,197,215,240,194,11,63,71,173,118,15,65,224,226,209,163,143,99,223,254,157,184,124,121,28,
  142,109,193,135,130,201,91,147,56,242,200,33,12,15,175,196,59,239,188,9,85,13,80,44,12,224,217,103,158,196,240,202,21,248,236,179,171,56,119,246,
  50,142,61,241,16,62,253,236,10,70,87,173,192,55,190,241,53,252,232,71,63,129,101,45,9,17,40,32,192,71,31,127,136,83,31,158,68,46,87,66,
  111,111,31,30,61,250,5,172,91,191,22,239,188,253,33,198,198,78,195,113,235,66,196,43,45,19,38,175,137,137,113,28,123,236,24,182,111,219,131,139,
  151,62,69,171,181,136,77,155,214,98,116,116,56,42,163,20,112,232,192,49,108,220,176,30,167,79,127,152,233,0,2,66,11,240,123,51,243,116,17,222,
  185,123,3,119,103,54,99,221,186,54,206,158,61,3,69,81,176,184,176,136,156,185,53,211,97,155,37,104,223,122,255,6,188,244,155,151,176,115,247,3,
  120,248,161,221,104,53,219,88,177,162,55,44,43,205,220,64,173,182,15,107,87,175,195,181,155,227,210,114,117,154,21,117,146,177,66,150,113,247,246,246,
  210,96,242,194,197,48,241,185,112,241,34,62,59,119,142,126,222,3,251,247,199,2,171,92,46,135,163,71,142,132,77,58,83,83,52,120,38,193,21,251,
  90,61,58,138,199,143,29,195,236,236,44,126,254,203,95,210,224,199,52,77,188,252,202,43,116,189,177,29,213,167,207,156,193,67,135,15,99,199,246,237,
  216,185,99,7,222,57,126,28,23,46,94,148,162,148,162,50,250,237,59,119,80,171,213,80,169,84,232,166,252,248,177,99,24,93,181,10,191,123,237,53,
  92,27,31,199,223,126,235,91,244,58,134,134,134,176,105,227,70,156,25,27,195,153,177,49,124,235,185,231,240,149,39,159,196,255,254,254,247,99,155,137,
  239,251,216,190,109,27,54,172,95,15,0,24,30,30,166,29,242,116,79,137,254,251,81,6,121,5,128,239,255,203,191,8,199,219,229,36,201,208,99,88,
  39,51,223,247,177,97,253,122,236,219,187,23,83,211,211,120,233,229,151,105,208,187,99,251,118,204,204,204,36,174,135,209,209,81,172,91,187,54,76,114,
  56,20,148,165,93,144,235,61,124,232,16,108,219,198,228,212,20,157,79,59,119,236,160,193,243,111,94,126,153,30,80,178,121,198,6,250,213,106,181,131,
  80,101,216,83,21,69,193,145,47,124,1,195,195,195,177,100,65,212,128,70,180,115,101,165,72,54,120,96,17,99,150,167,157,133,63,207,43,107,44,103,
  221,125,222,215,114,245,189,179,200,5,118,184,205,241,100,35,222,32,23,71,58,89,126,52,65,184,226,215,218,9,190,136,106,1,209,209,245,60,143,106,
  237,150,203,101,170,90,65,2,91,190,177,147,4,56,147,147,147,248,253,239,127,143,195,135,15,83,201,48,50,30,195,48,240,15,255,240,15,216,189,123,
  55,222,124,243,77,220,188,121,19,154,166,161,84,42,133,180,195,207,62,195,204,204,12,254,251,127,255,239,232,233,233,129,109,219,148,10,81,169,84,186,
  172,159,121,94,62,121,230,185,92,14,235,214,173,163,166,43,0,240,163,31,253,8,253,253,253,216,178,101,11,26,141,6,94,127,253,245,24,157,97,227,
  198,141,216,182,109,27,130,32,192,7,31,124,128,233,233,105,124,231,59,223,193,186,117,235,80,46,151,49,57,57,25,11,102,101,198,39,124,5,73,36,
  173,40,226,255,202,170,153,60,71,152,167,209,37,85,26,249,207,38,129,43,207,101,103,19,110,62,16,103,171,16,236,223,177,174,147,236,25,197,38,142,
  172,222,51,249,29,249,61,225,149,55,26,13,236,217,179,7,247,223,127,63,126,253,235,95,99,118,118,150,242,171,9,218,76,190,183,84,42,97,113,113,
  17,173,86,11,229,114,25,35,35,35,176,44,139,206,145,142,3,97,3,181,218,44,12,35,135,86,171,25,5,253,54,53,199,243,60,151,6,207,225,28,
  102,173,205,121,231,193,64,192,107,14,4,85,31,210,147,160,8,127,215,125,126,40,127,240,30,211,165,180,21,81,70,84,69,129,46,67,227,210,228,137,
  178,108,72,221,159,165,99,169,222,198,241,227,39,112,232,161,195,152,155,13,57,92,187,119,111,131,231,121,184,121,107,26,151,175,220,196,210,226,61,186,
  233,124,118,238,60,128,243,0,52,244,15,244,161,221,118,240,202,171,175,225,224,3,187,176,125,231,14,12,13,247,225,103,63,95,192,91,111,189,135,255,
  252,95,254,30,107,215,174,199,249,243,247,112,228,11,95,192,254,7,118,226,246,244,18,166,167,22,112,247,222,20,238,222,157,2,96,163,86,157,199,95,
  253,245,95,227,211,79,207,99,105,105,26,143,62,242,16,74,229,2,254,237,223,95,64,187,233,226,169,175,60,14,69,85,96,89,109,252,242,87,175,224,
  111,191,245,77,220,119,223,86,156,59,247,81,228,211,46,207,60,109,187,137,217,217,22,38,38,110,97,221,250,181,184,126,99,2,150,93,21,90,128,46,
  7,121,241,125,7,175,189,254,6,142,61,246,37,12,13,15,99,113,113,9,141,122,3,154,166,66,211,76,60,249,228,49,204,205,45,224,245,55,222,196,
  236,220,61,105,179,68,156,151,106,195,15,124,20,43,37,204,45,134,242,53,102,174,140,3,7,182,161,94,171,66,211,114,240,125,29,35,43,215,2,129,
  183,172,73,199,110,108,236,252,42,22,75,104,181,91,184,59,119,23,190,170,227,218,149,79,81,174,12,224,203,95,124,136,234,167,212,234,13,148,123,75,
  177,3,157,117,225,146,33,100,50,174,116,214,174,127,126,220,50,30,48,249,239,141,27,54,80,58,199,153,177,49,138,104,110,218,180,9,213,90,13,165,
  82,9,158,231,225,87,47,190,136,156,105,198,56,188,171,35,244,112,112,112,16,255,245,31,255,177,19,204,173,90,21,219,12,201,188,26,28,28,164,104,
  239,205,201,73,236,220,190,29,251,247,237,163,188,222,31,252,235,191,38,6,75,50,250,75,46,151,163,72,230,147,79,60,17,91,247,67,67,67,180,17,
  136,112,193,103,103,103,113,225,226,197,206,61,141,132,114,21,69,193,240,240,48,20,69,193,248,196,4,94,123,253,245,46,254,53,185,167,227,19,19,248,
  205,203,47,227,63,253,195,63,32,0,208,211,211,131,185,185,57,138,148,178,135,18,75,185,33,8,254,87,159,125,54,158,92,94,191,142,106,181,74,145,
  109,22,137,174,84,42,56,51,54,134,227,39,78,224,232,145,35,152,156,154,194,75,47,191,28,187,15,61,149,10,254,191,255,252,207,208,52,45,22,216,
  138,158,49,123,79,123,122,122,186,168,61,147,83,83,137,193,179,232,117,248,208,33,252,236,23,191,160,65,90,146,153,10,123,15,95,122,229,21,252,95,
  223,254,54,189,206,11,23,47,98,124,98,130,82,143,90,173,22,94,121,245,213,204,135,131,136,170,147,228,200,198,151,183,69,159,73,2,122,25,178,45,
  11,170,179,232,64,103,113,236,204,226,80,40,110,104,12,34,219,238,14,82,166,170,98,187,116,98,148,66,130,107,190,116,92,169,148,168,30,51,219,40,
  70,180,145,21,69,129,227,56,88,90,90,66,46,151,163,77,134,34,245,22,69,81,144,207,231,209,106,181,240,226,139,47,226,236,217,179,56,118,236,24,
  246,239,223,79,131,39,146,0,61,248,224,131,216,186,117,43,222,123,239,61,188,243,206,59,152,153,153,161,207,98,211,166,77,180,220,79,120,199,181,90,
  13,31,124,240,1,13,230,200,190,203,114,142,201,255,219,182,141,53,107,214,224,249,231,159,71,187,221,198,27,111,188,129,211,167,79,163,209,104,160,209,
  104,224,234,213,171,148,59,27,4,1,74,165,18,90,173,22,122,122,122,80,171,213,240,212,83,79,97,219,182,109,120,243,205,55,113,250,244,105,90,73,
  35,72,182,40,168,149,210,212,4,212,57,153,66,134,12,100,33,252,109,246,140,73,107,34,151,209,55,68,70,41,108,5,129,167,107,176,10,55,236,217,
  35,210,248,230,105,32,188,241,154,227,56,93,84,19,203,178,96,219,54,138,197,34,90,173,22,26,141,70,140,62,210,108,54,209,104,52,224,186,46,122,
  122,122,80,42,149,176,180,20,130,133,164,209,208,178,44,42,85,152,203,229,208,110,183,209,110,55,225,251,30,114,185,34,237,17,112,93,23,165,82,37,
  26,155,7,199,177,40,186,76,208,230,112,45,17,197,28,159,107,4,12,186,104,24,33,207,153,239,7,33,247,34,252,111,199,113,96,24,70,236,125,241,
  202,81,208,181,46,147,248,207,50,190,188,231,121,209,64,35,103,80,217,70,38,202,186,210,100,174,210,187,213,85,0,26,78,143,157,195,221,123,51,216,
  189,103,47,170,75,85,12,13,15,225,189,119,207,224,179,243,159,193,177,155,8,2,7,196,33,170,243,185,62,218,118,19,80,20,180,91,1,222,124,235,
  20,86,141,142,98,97,201,194,220,236,29,172,26,25,8,75,204,182,135,117,107,238,199,129,67,251,241,209,71,23,49,58,218,135,47,63,246,32,28,119,
  63,126,248,239,63,199,210,210,2,230,23,234,176,109,11,107,214,173,71,237,220,2,86,173,89,133,143,62,62,139,133,249,123,120,234,169,103,49,56,60,
  0,207,117,35,121,60,55,234,34,46,164,112,238,226,77,134,182,109,69,72,230,32,230,231,167,83,109,125,179,36,32,19,19,151,240,202,43,109,236,219,
  183,15,195,67,3,176,29,27,99,99,231,66,26,198,199,159,226,163,143,79,164,126,7,111,117,124,245,234,85,60,249,196,147,248,205,75,47,97,102,230,
  14,2,95,71,109,169,129,145,149,131,120,238,249,191,196,221,187,75,232,233,41,224,213,223,190,20,51,208,88,78,153,149,157,27,150,109,35,151,47,97,
  237,154,13,184,53,121,29,65,16,160,86,111,163,86,183,144,203,153,104,91,54,12,195,164,247,143,47,189,241,229,57,54,91,231,59,147,89,222,243,114,
  199,93,175,215,105,128,185,105,227,70,140,79,76,96,112,112,16,223,248,243,63,199,248,196,4,126,247,218,107,137,137,15,9,60,69,234,24,236,102,56,
  63,63,143,159,255,242,151,137,229,111,18,160,229,76,19,227,227,227,56,126,252,56,26,245,58,14,29,60,136,74,165,66,57,139,108,0,205,151,216,201,
  63,131,131,131,20,21,39,99,36,175,215,223,120,3,215,198,199,187,214,250,133,139,23,177,111,239,94,236,221,179,7,131,131,131,56,122,228,8,102,34,
  122,2,169,163,17,35,131,180,166,78,118,67,39,171,38,207,160,201,124,210,158,212,88,235,251,62,78,188,251,46,150,150,150,176,99,251,118,60,122,244,
  40,106,181,26,165,95,136,232,12,162,251,51,57,53,149,88,22,102,121,132,178,160,139,52,56,109,223,182,13,23,46,94,164,244,13,217,235,212,135,31,
  194,182,109,202,131,222,177,125,59,173,116,144,121,71,18,44,17,18,234,56,78,236,121,231,115,185,88,83,162,109,219,120,229,213,87,49,59,59,155,168,
  242,145,101,239,225,165,181,200,179,73,107,164,77,11,148,147,36,25,121,183,181,44,251,76,150,30,136,14,205,66,118,78,5,12,202,172,80,131,7,242,
  243,78,96,205,170,173,248,145,254,51,219,48,219,65,158,139,197,2,170,213,26,5,47,200,123,9,242,203,186,243,145,198,64,162,108,192,238,31,236,115,
  203,229,114,208,117,29,147,147,147,248,247,127,255,119,156,62,125,26,199,142,29,195,166,77,155,232,65,79,228,196,158,124,242,73,236,220,185,19,167,78,
  157,194,252,252,60,118,239,222,141,189,123,247,82,250,135,162,40,184,126,253,58,126,253,235,95,227,202,149,43,49,101,12,86,177,137,189,135,166,105,98,
  102,102,6,255,250,175,255,138,245,235,215,163,191,191,159,242,166,201,61,32,1,141,162,40,248,235,191,254,107,244,245,245,225,237,183,223,198,91,111,189,
  5,211,52,209,219,219,139,153,153,25,138,186,243,251,188,200,241,47,238,70,44,110,40,79,210,53,23,241,239,73,208,204,251,36,136,140,180,68,170,67,
  105,50,191,44,8,196,114,152,121,195,23,158,158,193,254,155,7,118,248,51,130,140,129,204,25,214,73,144,77,130,22,23,23,81,175,215,105,176,94,169,
  84,80,171,213,232,158,76,64,16,195,48,80,42,149,208,211,211,67,121,245,237,118,19,67,67,61,168,213,230,161,40,38,66,133,141,128,206,53,223,119,
  35,148,58,228,249,147,127,72,160,74,170,59,36,176,149,33,208,221,235,82,214,139,19,242,170,1,133,34,237,178,61,158,152,44,201,42,16,50,208,137,
  239,107,33,220,116,106,4,148,60,116,8,157,113,100,242,117,172,100,140,168,169,70,85,117,168,106,104,78,50,125,123,10,147,83,183,176,106,120,37,190,
  249,173,231,113,231,238,93,216,214,2,53,82,33,196,113,69,233,108,70,75,213,37,20,242,6,250,250,86,0,65,14,134,105,96,221,186,30,108,222,188,
  5,171,70,6,224,251,62,154,141,22,246,238,218,140,70,195,198,71,31,125,130,158,138,134,231,158,255,43,204,220,173,162,217,108,33,8,28,152,70,30,
  154,170,34,79,54,45,187,13,77,15,23,241,167,159,126,138,15,222,63,131,141,27,86,97,113,201,194,154,213,235,48,48,216,135,227,239,126,152,106,245,
  202,222,143,123,119,239,210,192,155,63,120,150,251,98,191,119,250,246,77,76,223,190,25,155,28,133,66,5,134,145,220,101,44,66,109,20,69,193,39,159,
  188,143,190,222,94,252,197,55,254,2,247,238,205,160,209,104,194,208,181,104,178,168,176,90,139,248,229,27,47,97,110,126,86,216,240,200,55,190,136,144,
  2,214,64,196,115,91,184,118,237,58,190,254,141,63,199,199,167,198,48,113,227,38,22,22,103,240,206,241,119,17,192,196,154,53,235,177,106,120,0,31,
  157,252,125,172,100,37,106,254,227,29,186,248,230,137,164,70,205,180,113,95,189,118,13,187,119,237,194,200,200,8,30,63,118,44,62,215,163,245,112,109,
  124,28,131,131,131,56,120,224,0,45,243,7,65,128,241,241,113,216,182,141,249,249,121,244,247,247,227,47,191,241,13,186,65,17,116,101,106,106,10,251,
  246,238,237,66,160,47,92,188,136,183,223,121,167,107,237,16,132,102,251,182,109,93,170,26,149,158,30,108,219,186,21,31,126,244,81,215,198,65,202,121,
  171,86,174,140,125,79,187,221,198,249,11,23,208,110,183,49,53,61,77,41,37,143,61,246,24,16,109,214,63,122,225,5,236,219,187,151,6,133,255,242,
  255,254,191,120,246,153,103,176,58,226,196,177,129,157,166,105,152,11,59,110,177,105,227,70,252,223,255,244,79,49,235,221,180,13,171,82,169,196,56,208,
  147,83,83,82,132,145,253,61,121,222,67,67,67,0,128,107,227,227,56,51,54,22,123,102,236,107,205,234,213,148,3,205,115,44,69,243,107,226,250,117,
  12,15,15,227,225,135,30,194,195,15,61,20,11,128,175,69,77,127,65,16,224,181,215,95,199,166,77,155,176,125,219,54,28,62,116,8,47,189,252,178,
  20,213,85,20,133,162,217,251,246,238,69,165,82,193,225,67,135,112,109,124,28,237,118,155,114,82,255,235,63,254,99,102,52,91,81,20,90,141,32,1,
  206,95,68,26,186,63,122,225,133,216,231,176,72,153,136,110,193,75,108,201,76,32,100,166,48,162,202,37,75,45,225,145,69,30,185,19,41,118,200,169,
  90,242,6,111,25,74,221,205,89,230,65,134,100,144,164,227,56,72,116,159,213,152,115,33,255,93,166,169,195,48,116,180,219,86,215,181,176,28,100,182,
  201,144,208,58,74,165,82,151,84,27,27,8,150,203,101,216,182,141,179,103,207,98,124,124,28,135,15,31,198,177,99,199,40,242,75,2,165,53,107,214,
  96,116,116,148,54,250,147,231,117,239,222,61,156,56,113,2,31,127,252,49,109,108,100,3,62,199,113,96,154,38,90,173,22,109,26,36,251,121,163,209,
  192,194,194,2,198,199,199,169,233,9,187,223,19,126,180,97,24,248,237,111,127,139,161,161,33,172,91,183,14,235,215,175,199,155,111,190,73,199,192,6,
  146,124,146,151,69,139,55,73,235,92,84,45,23,85,121,216,138,7,75,179,224,37,4,69,13,136,34,160,71,68,223,16,75,251,250,93,224,14,111,146,
  194,203,6,243,9,61,225,74,19,7,200,124,62,143,7,30,120,0,179,179,179,56,126,252,56,165,219,16,163,20,34,69,167,235,58,74,165,18,230,230,
  230,168,201,205,134,13,27,168,150,51,177,228,14,155,3,219,176,237,26,108,187,65,105,171,173,86,51,74,230,10,145,97,138,5,223,119,35,116,216,131,
  227,216,148,83,204,35,239,228,246,133,168,116,228,215,65,131,237,56,149,35,254,220,227,188,102,86,17,71,22,55,177,168,53,159,140,121,30,73,84,130,
  46,196,90,52,255,226,49,70,0,93,164,57,41,67,123,68,19,143,207,12,88,173,197,110,36,162,29,57,14,26,0,66,183,63,171,221,130,235,120,220,
  198,163,82,169,147,206,207,3,220,185,115,7,231,207,157,199,211,79,31,197,169,83,167,113,227,250,20,238,187,111,35,158,124,226,8,84,85,197,185,243,
  227,184,115,231,58,118,110,91,13,215,117,225,186,54,70,70,54,135,27,147,161,227,75,95,58,138,179,103,207,97,199,246,45,208,13,3,55,110,92,131,
  235,54,112,229,202,85,236,217,189,19,83,147,215,113,231,206,93,4,129,131,179,159,45,98,203,230,251,240,244,51,95,193,228,228,93,76,78,222,200,140,
  198,3,192,204,220,29,220,185,115,23,125,189,189,159,43,104,22,149,138,100,223,105,91,22,124,38,113,201,202,247,35,255,254,253,91,175,226,204,216,16,
  54,110,220,132,92,206,196,189,123,183,177,106,245,58,148,11,6,126,19,201,215,101,213,199,148,161,222,236,92,121,247,248,235,152,155,185,131,61,187,119,
  99,247,222,93,80,148,32,92,108,62,208,106,53,241,201,39,239,163,222,108,10,15,245,52,23,50,190,105,131,71,42,150,51,238,151,127,251,91,60,116,
  248,48,13,88,125,223,199,133,139,23,241,193,201,147,84,177,65,81,148,78,19,161,109,227,228,201,147,180,1,238,157,19,39,240,196,177,99,40,151,203,
  56,121,234,20,30,216,191,159,6,208,147,83,83,120,231,248,113,60,122,244,104,44,120,38,210,105,252,248,206,140,141,33,151,203,209,177,216,182,141,51,
  99,99,184,54,62,142,195,135,14,225,204,216,88,215,6,45,155,55,83,211,211,120,255,131,15,40,2,253,206,241,227,56,122,228,8,214,172,94,13,85,
  81,80,171,215,241,206,241,227,168,86,171,56,121,234,20,253,222,109,91,67,30,252,219,239,188,131,217,217,217,88,32,77,104,5,103,198,198,176,111,239,
  94,76,78,77,97,106,106,138,222,155,180,10,64,210,129,148,164,25,76,126,119,225,226,69,172,30,29,165,84,19,18,160,18,36,247,86,212,56,67,56,
  158,185,92,46,211,28,24,59,123,22,185,92,174,75,202,240,204,216,24,69,242,21,69,193,82,181,138,51,99,99,216,190,109,27,86,143,142,210,170,69,
  218,250,59,125,230,12,109,36,61,124,232,16,222,126,231,29,108,221,186,21,235,214,174,197,212,244,52,174,93,187,22,155,35,89,81,86,30,253,34,92,
  68,118,93,100,169,40,202,122,96,210,122,99,100,129,180,108,111,200,186,38,227,137,101,199,19,32,137,235,216,205,157,84,36,193,116,16,211,121,238,166,
  106,16,31,2,145,126,124,232,58,24,34,202,113,99,40,199,177,225,186,158,16,84,34,231,170,109,219,20,181,37,247,202,117,93,44,46,46,162,80,40,
  196,208,98,222,176,135,56,201,17,42,197,103,159,125,134,199,31,127,28,15,60,240,0,76,211,140,209,246,136,250,198,157,59,119,112,252,248,113,74,187,
  32,129,23,79,153,123,250,233,167,209,211,211,131,235,215,175,227,220,185,115,212,94,155,149,185,99,131,163,102,179,137,173,91,183,226,111,255,246,111,113,
  249,242,101,188,254,250,235,152,155,155,195,173,91,183,112,235,214,45,156,141,246,76,86,136,128,151,93,19,161,201,124,178,37,66,24,89,253,104,89,226,
  154,164,64,195,127,166,232,103,73,212,32,182,119,128,77,62,73,192,203,163,164,44,199,157,159,103,44,159,93,100,212,66,18,47,98,66,67,206,149,185,
  185,57,172,95,191,30,27,55,110,164,85,48,114,95,52,77,163,136,52,161,107,176,82,130,163,163,163,40,149,74,104,54,155,212,160,134,52,126,186,110,
  24,32,171,170,1,77,51,225,68,49,91,88,209,214,225,56,30,67,95,82,163,247,16,29,112,143,209,125,86,98,52,13,246,113,104,154,10,114,84,251,
  126,248,121,132,246,33,138,53,195,53,159,220,36,152,246,188,121,138,7,255,124,133,102,85,209,85,120,190,15,229,255,243,255,252,63,1,121,48,255,242,
  111,255,214,149,57,177,92,204,229,16,173,121,132,131,4,198,29,14,177,2,64,71,222,52,240,15,223,254,91,188,249,246,71,184,124,241,99,168,170,17,
  221,60,30,118,39,101,0,19,95,120,232,0,118,239,217,11,77,15,181,254,102,238,205,97,236,236,57,92,189,122,13,158,215,68,127,255,16,158,127,238,
  47,49,113,125,10,31,125,244,33,14,31,122,0,3,3,3,112,93,29,129,231,99,100,180,15,39,79,142,225,189,247,222,6,224,65,85,77,60,253,212,
  227,216,176,113,51,106,181,6,150,22,150,80,174,148,209,223,223,131,171,87,174,225,205,223,191,13,219,105,50,154,133,114,243,24,118,1,60,245,228,215,
  208,108,91,120,231,157,223,45,155,246,194,46,26,217,51,32,232,194,227,143,127,13,51,247,166,113,246,211,79,150,197,243,149,25,34,248,190,143,117,235,
  118,224,192,3,59,240,203,95,255,60,182,1,243,154,147,105,92,68,190,217,129,223,160,42,61,67,120,244,209,47,162,175,162,227,215,47,254,54,148,211,
  129,43,213,222,36,7,134,200,84,129,15,24,255,148,227,22,109,132,34,52,51,105,220,252,65,154,180,192,179,247,25,100,75,240,210,94,44,31,143,109,
  110,97,55,228,180,36,155,160,213,68,171,57,139,195,233,114,180,141,101,159,39,226,71,202,208,214,164,117,39,226,240,203,190,43,75,16,152,116,189,50,
  119,66,153,245,124,150,107,16,209,46,68,73,101,154,252,87,218,243,96,203,218,162,61,37,203,51,144,41,38,36,61,219,206,153,192,34,196,232,226,75,
  134,215,20,25,31,232,90,236,61,221,170,28,132,247,12,176,14,132,124,3,161,200,185,82,215,53,138,72,7,129,31,227,180,47,46,54,48,55,183,32,
  52,214,16,41,59,17,132,208,243,60,216,182,13,93,215,169,222,46,223,31,192,223,75,210,240,181,103,207,30,60,241,196,19,216,176,97,3,84,85,69,
  171,213,194,244,244,52,222,125,247,93,156,61,123,22,173,86,11,249,124,158,174,111,18,52,145,210,123,95,95,31,254,238,239,254,14,213,106,21,61,61,
  61,161,222,251,11,47,208,49,138,180,169,125,223,71,111,111,47,214,175,95,143,173,91,183,194,178,44,188,248,226,139,244,94,176,243,81,86,49,229,121,
  223,73,149,212,207,187,46,178,190,151,119,193,76,114,53,20,5,252,124,83,34,1,24,69,141,170,124,21,134,5,130,216,255,39,115,58,151,203,197,154,
  169,137,131,224,198,141,27,241,212,83,79,225,189,247,222,195,153,51,103,224,251,62,250,251,251,241,212,83,79,97,108,108,12,103,206,156,137,122,207,118,
  227,193,7,31,196,141,27,55,240,234,171,175,98,247,238,221,216,181,107,23,90,173,22,102,102,238,97,102,230,14,214,173,219,0,93,55,209,108,54,112,
  243,230,37,20,10,69,244,246,174,68,16,248,88,88,152,70,171,213,194,192,192,154,40,192,110,194,178,218,40,20,250,224,186,14,106,181,121,216,182,21,
  51,72,33,49,20,65,158,61,175,131,64,147,159,119,206,219,110,223,7,214,112,133,188,175,219,221,85,233,226,82,119,216,134,65,87,175,3,75,17,225,
  13,87,68,70,61,255,240,119,127,11,68,99,208,69,89,177,12,237,89,142,6,175,136,15,231,121,46,51,73,84,168,170,11,69,205,67,1,160,194,102,
  46,32,212,114,238,4,49,62,163,29,104,227,196,123,239,226,147,51,103,177,101,211,6,60,252,200,81,188,246,250,113,44,46,78,66,81,242,80,20,19,
  11,11,243,248,217,207,127,133,39,158,120,12,127,254,231,207,194,178,28,20,10,121,40,170,14,85,5,206,157,187,138,247,223,63,142,32,112,161,40,26,
  124,223,198,75,47,191,140,129,254,1,172,93,187,6,235,55,108,198,240,72,63,62,60,249,25,222,125,255,117,102,163,131,208,138,82,140,142,132,22,227,
  78,173,158,41,248,17,45,108,145,235,144,136,31,235,121,14,60,70,242,45,203,179,17,61,111,54,241,153,159,159,129,31,196,77,48,248,142,232,52,125,
  90,62,208,18,33,3,213,165,123,184,125,103,26,8,250,80,171,47,66,81,3,168,138,38,117,68,19,161,96,60,114,193,54,46,253,169,198,45,10,202,
  101,13,47,178,113,139,130,153,229,4,190,178,67,64,100,57,206,6,84,73,14,142,50,228,55,201,254,156,188,134,135,135,241,213,103,158,137,185,146,93,
  184,120,81,24,60,203,203,235,74,166,235,76,91,127,34,203,108,30,57,74,218,243,178,91,65,47,207,41,81,84,89,146,205,39,81,210,43,59,236,69,
  115,152,157,119,36,112,225,169,80,124,80,205,187,148,102,121,241,168,159,140,151,154,117,95,146,57,73,118,143,73,233,66,132,217,183,177,104,114,22,46,
  117,218,207,229,215,16,48,227,244,161,235,106,23,2,94,44,22,48,63,191,152,88,238,103,253,20,8,133,128,4,184,237,118,155,154,156,152,166,73,21,
  20,88,190,56,203,143,54,77,19,99,99,99,184,114,229,10,213,120,30,31,31,199,217,179,103,209,110,183,81,44,22,81,46,151,97,89,22,117,156,219,
  190,125,59,182,108,217,130,165,165,37,106,146,242,189,239,125,15,107,215,174,69,127,127,63,38,35,165,23,69,81,208,104,52,104,63,13,65,162,73,19,
  222,226,226,34,238,222,189,139,83,167,78,161,80,40,196,168,8,98,205,109,69,24,140,167,173,83,217,158,32,2,87,100,14,202,73,168,54,159,200,138,
  172,190,69,116,86,246,187,216,51,136,109,240,76,2,30,217,247,178,206,148,236,250,88,177,98,5,74,165,18,74,165,18,250,250,250,112,241,226,69,42,
  59,71,18,217,90,173,70,233,26,68,22,143,80,131,12,195,192,250,245,235,209,106,181,168,60,33,49,80,9,233,134,13,120,94,29,181,218,34,138,197,
  94,120,158,13,85,5,116,221,140,42,20,30,92,215,131,105,230,225,251,1,12,67,71,173,214,132,166,233,244,25,132,116,14,15,65,160,70,49,28,192,
  74,212,37,169,111,176,82,119,29,73,62,54,241,72,79,234,227,103,174,152,14,194,26,228,117,0,219,20,80,132,65,206,117,153,129,2,187,145,167,29,
  54,162,32,144,229,95,177,124,181,14,74,16,102,30,154,170,67,85,117,216,14,217,216,189,174,12,1,232,52,107,132,23,106,160,217,108,224,214,173,73,
  124,41,167,163,88,44,96,113,209,64,16,184,0,194,64,242,238,221,59,248,183,127,251,49,70,70,134,208,219,219,11,93,55,113,228,11,15,227,220,213,
  9,28,127,251,141,8,121,86,99,15,116,102,118,6,243,11,243,56,125,102,12,255,248,95,254,11,22,170,181,174,12,72,116,31,100,72,174,97,168,104,
  183,235,153,208,52,118,115,145,185,7,202,178,238,32,240,161,32,219,129,199,202,18,241,85,134,142,35,163,134,86,171,6,207,15,160,41,10,220,168,84,
  196,94,35,223,105,47,42,119,137,100,172,120,237,75,0,112,157,0,110,66,3,28,143,68,136,202,94,124,48,192,218,95,255,169,198,157,36,204,159,101,
  220,162,117,147,165,113,42,237,160,151,5,46,105,188,66,209,92,79,42,131,241,175,217,217,89,252,224,95,255,117,89,137,183,8,169,145,241,7,201,239,
  217,132,146,229,199,243,77,63,188,123,106,150,100,133,175,158,201,215,92,208,245,25,236,225,41,42,37,203,64,134,164,70,73,153,51,32,63,79,210,2,
  84,22,201,18,53,66,101,81,173,17,237,115,73,149,19,217,61,19,221,115,94,93,128,239,165,225,19,224,229,200,167,254,161,213,24,81,67,46,255,249,
  172,17,87,136,66,179,60,110,133,202,110,241,73,127,18,101,146,80,34,52,77,67,177,88,140,184,168,157,38,67,146,168,146,32,139,149,49,243,60,143,
  202,141,189,241,198,27,116,175,43,22,139,148,198,68,16,232,141,145,179,233,202,149,43,209,106,181,112,224,192,1,28,57,114,4,63,248,193,15,112,251,
  246,109,140,143,143,227,234,213,171,52,208,82,85,21,123,247,238,69,185,92,198,196,196,4,238,220,185,19,163,115,144,239,33,84,1,18,92,147,107,231,
  213,148,100,77,214,124,5,91,198,109,230,231,26,111,26,198,131,34,105,159,39,67,192,101,224,8,251,76,249,192,89,150,204,242,65,187,168,215,128,124,
  22,161,107,16,142,60,49,68,209,117,29,79,60,241,4,42,149,10,46,95,190,76,165,232,136,106,198,226,226,34,92,215,165,115,65,215,117,56,142,67,
  105,54,61,61,61,168,215,235,112,28,135,62,179,176,250,224,162,221,110,32,108,162,213,105,115,107,248,217,185,232,94,170,112,28,59,10,174,61,184,174,
  13,199,177,144,203,21,97,219,54,52,77,133,235,122,66,171,237,32,8,98,113,87,168,12,23,8,18,29,209,179,79,115,86,142,39,167,34,208,36,142,
  84,243,9,57,24,52,92,188,39,178,115,77,151,41,24,240,65,65,119,68,159,28,112,120,2,78,174,170,230,224,251,118,228,236,100,0,112,209,104,213,
  96,57,45,228,139,101,65,243,161,14,192,227,236,82,85,248,190,5,32,64,62,95,2,130,0,174,215,70,16,216,204,228,87,162,247,249,184,119,239,54,
  238,221,187,13,223,87,177,109,203,246,16,220,134,199,160,23,26,20,133,28,40,122,20,164,171,176,218,54,122,123,251,168,211,77,18,146,34,10,138,0,
  192,119,109,40,129,38,69,217,210,53,73,211,75,169,100,97,146,230,142,52,174,48,219,32,36,106,36,234,252,219,65,224,217,208,13,3,158,68,66,40,
  45,56,200,194,53,35,14,125,237,182,35,124,47,95,106,206,234,196,38,146,188,251,99,143,91,212,229,253,121,198,205,11,239,39,209,110,68,193,158,12,
  41,19,33,46,73,156,66,17,125,43,41,9,19,33,149,73,205,153,162,228,64,68,147,16,37,0,73,135,24,127,255,147,2,38,150,163,152,118,32,167,
  109,160,178,245,41,107,130,19,93,63,175,174,144,244,247,162,231,157,20,80,202,146,49,81,128,156,214,28,149,68,153,145,221,15,177,53,182,248,25,242,
  242,97,124,201,159,159,71,157,185,183,124,157,87,66,233,224,169,24,221,215,43,95,139,97,83,116,60,56,8,145,110,54,105,240,225,251,74,204,150,156,
  240,139,69,137,23,79,29,35,231,46,9,112,116,93,71,177,88,164,118,217,196,148,132,181,81,119,93,55,182,174,9,63,154,167,245,56,142,131,158,158,
  30,60,246,216,99,168,84,42,232,235,235,195,43,175,188,2,203,178,232,60,171,215,235,148,50,210,110,183,105,131,227,200,200,8,190,249,205,111,98,124,
  124,28,219,182,109,195,221,187,119,241,187,223,253,46,182,223,144,235,32,148,5,146,24,203,4,7,100,251,106,18,189,34,41,248,77,74,74,68,62,12,
  124,133,142,111,160,229,233,105,236,127,147,235,200,74,157,227,239,3,31,164,179,193,56,249,60,162,186,162,170,42,28,199,65,181,90,133,231,121,232,237,
  237,197,252,252,124,236,185,229,243,121,52,155,77,52,155,77,186,191,172,90,181,10,0,104,144,60,56,56,72,215,86,163,209,192,192,64,63,114,57,35,
  10,184,67,61,103,77,211,161,170,196,139,32,0,160,66,211,12,248,62,208,140,140,58,194,152,41,136,120,254,46,12,35,135,70,163,5,211,84,17,4,
  94,196,105,238,196,91,196,252,36,14,204,162,203,238,91,182,174,137,108,93,146,217,138,12,108,228,227,219,238,247,199,109,199,217,117,37,235,39,138,169,
  112,240,205,71,236,1,39,227,181,201,6,43,42,35,251,126,27,161,75,160,74,145,230,32,112,81,91,170,97,229,240,48,46,42,196,138,187,211,157,25,
  71,52,0,160,35,92,239,69,29,159,170,106,112,11,39,44,25,116,54,171,48,88,183,220,22,35,148,79,183,40,65,176,163,34,252,122,63,226,237,116,
  27,6,176,25,166,116,113,171,26,84,77,149,162,202,105,50,80,60,37,129,183,255,36,227,48,141,2,28,183,149,153,11,38,162,29,116,7,237,1,160,
  154,40,149,202,104,91,115,82,174,90,82,51,95,26,10,78,190,175,82,233,3,130,22,93,0,185,92,9,35,195,107,48,48,176,2,249,188,137,201,91,
  55,112,115,114,66,120,191,69,27,174,44,129,248,99,143,59,105,83,231,155,132,100,99,97,3,23,209,115,32,115,88,20,244,200,92,25,147,248,218,162,
  64,154,240,179,101,156,190,229,150,189,101,18,64,34,148,133,15,172,68,149,25,30,149,78,66,9,179,104,7,203,130,108,25,133,231,243,54,161,242,60,
  106,89,50,43,170,166,136,14,81,145,97,132,40,160,150,37,224,178,131,68,118,15,68,201,142,40,105,17,61,55,126,127,148,233,170,39,209,202,68,123,
  45,91,13,10,123,101,180,204,215,38,154,115,217,173,192,21,74,227,19,191,135,148,130,227,231,21,219,12,198,154,180,200,44,224,69,247,152,80,56,8,
  2,153,207,231,225,186,46,90,173,22,45,199,23,10,133,152,250,144,136,242,195,239,11,158,231,225,248,241,227,24,30,30,198,183,191,253,109,60,242,200,
  35,56,117,234,20,222,125,247,221,8,69,12,209,236,82,169,132,251,239,191,31,147,147,147,152,159,159,199,189,123,247,240,195,31,254,16,171,86,173,194,
  200,200,8,22,22,22,186,214,41,235,132,39,171,212,36,241,136,249,61,64,182,175,167,85,202,100,247,90,166,19,45,90,135,108,98,195,191,159,85,142,
  16,37,9,73,213,24,17,248,34,162,135,244,70,66,4,196,58,155,229,191,91,150,133,133,133,5,106,203,77,104,63,173,86,11,205,102,19,182,109,163,
  167,167,7,171,86,173,66,179,217,164,244,158,190,190,62,180,219,237,232,57,7,40,149,20,44,46,206,161,92,94,129,118,187,9,199,177,80,169,172,128,
  166,233,112,28,23,174,235,68,243,139,60,15,242,108,85,186,22,1,53,116,147,246,72,224,172,66,81,66,153,57,150,158,65,154,10,59,58,206,124,69,
  87,139,205,227,14,101,35,200,148,48,19,174,181,200,170,187,123,79,8,186,154,135,89,69,16,217,254,24,11,160,85,73,48,145,132,64,45,71,119,179,
  115,33,228,48,208,41,213,2,80,49,55,183,128,21,43,134,153,224,153,32,12,58,120,121,184,78,182,167,162,80,200,3,10,224,185,30,23,56,122,156,
  237,172,31,149,35,2,248,190,34,68,185,226,139,40,180,169,100,237,51,69,40,141,8,225,97,145,93,195,204,81,30,113,218,226,239,46,5,202,17,173,
  110,212,207,129,10,83,26,160,36,5,29,162,82,19,139,246,27,134,46,229,154,45,167,68,218,215,59,140,21,253,125,104,212,27,161,172,147,153,67,206,
  12,5,216,215,175,91,137,156,17,160,220,243,151,168,84,10,80,148,176,179,222,106,55,49,56,60,136,145,193,30,76,78,223,16,82,47,178,54,147,44,
  119,220,196,25,144,181,169,22,81,57,100,220,77,17,26,202,143,59,109,44,89,168,83,73,136,45,255,59,209,119,176,135,94,22,4,145,191,102,178,145,
  175,89,189,26,15,60,240,0,94,252,205,111,82,75,236,124,144,199,74,216,253,232,133,23,80,173,86,133,148,141,180,18,253,114,248,200,73,232,115,146,
  4,213,231,161,213,240,90,224,114,151,78,69,152,32,145,4,58,201,221,108,57,149,139,52,205,254,44,230,35,178,32,69,134,194,137,198,45,218,131,68,
  73,190,40,185,12,191,203,135,231,185,93,213,166,180,253,136,80,247,210,42,84,236,88,211,19,179,36,30,117,120,80,247,246,246,208,70,66,17,50,202,
  114,162,69,215,64,180,214,137,20,29,81,72,112,28,135,242,163,115,185,28,229,206,202,246,61,146,32,146,50,127,173,86,195,247,191,255,125,220,190,125,
  27,75,75,75,148,206,103,219,54,6,6,6,240,149,175,124,5,59,119,238,196,205,155,55,241,131,31,252,0,142,227,224,252,249,243,184,121,243,38,84,
  85,69,163,209,136,73,216,137,0,11,89,160,155,4,112,176,200,181,76,118,142,111,246,78,170,188,136,170,100,124,60,195,211,232,100,32,23,225,169,147,
  202,0,79,91,21,125,182,104,142,137,120,208,228,103,166,105,98,104,40,164,160,14,14,14,98,239,222,189,248,205,111,126,67,239,59,153,247,11,11,11,
  104,183,219,180,90,144,203,229,168,108,106,187,221,198,206,157,59,145,203,229,208,108,54,225,56,14,138,197,34,242,249,60,28,199,65,187,221,70,173,182,
  0,199,177,24,100,56,28,67,168,216,66,184,211,213,232,154,66,100,185,221,110,83,68,56,252,27,7,138,18,26,154,104,154,138,122,125,137,170,211,16,
  5,14,98,126,194,123,102,200,122,203,186,147,44,37,134,20,203,188,21,120,14,115,119,236,24,116,37,222,60,208,37,3,167,248,128,92,151,101,69,89,
  208,22,190,172,145,86,78,36,141,129,138,226,208,172,194,247,93,220,190,51,141,53,235,55,66,81,204,136,138,161,65,81,136,56,125,208,149,141,147,207,
  46,21,11,112,93,31,150,229,74,17,179,206,130,214,145,51,114,240,61,43,49,67,36,8,175,109,59,200,229,212,68,149,135,52,58,71,46,151,67,189,
  86,237,178,237,228,27,156,100,155,115,146,2,128,18,239,150,129,170,155,203,54,105,73,50,207,81,20,192,117,108,42,39,40,10,10,151,19,80,236,218,
  189,27,7,14,236,198,212,173,187,8,224,193,243,236,80,71,81,207,35,111,154,208,84,96,105,241,10,206,95,184,131,217,153,187,168,213,155,0,124,108,
  223,241,32,214,142,14,36,6,76,105,243,239,15,25,247,114,202,216,73,180,166,229,124,63,123,152,138,186,128,69,159,43,226,104,39,89,150,243,116,30,
  17,133,66,132,154,240,1,143,231,121,232,233,233,137,185,3,202,202,158,162,141,49,13,197,95,142,225,135,44,1,16,113,126,249,49,201,16,82,30,245,
  205,146,24,36,149,114,211,158,137,200,201,47,105,47,72,66,137,179,244,172,164,141,57,173,201,83,84,14,207,78,163,72,175,14,201,26,188,194,159,107,
  203,154,15,73,247,128,208,58,52,173,91,165,39,41,120,142,223,71,133,86,79,85,181,83,205,36,183,117,104,168,31,205,102,11,150,101,199,120,176,44,
  48,195,38,3,50,57,55,199,113,40,66,70,100,239,108,219,70,171,213,130,227,56,200,229,114,177,125,131,79,120,217,224,148,80,66,206,157,59,71,37,
  209,218,237,54,54,111,222,140,109,219,182,225,250,245,235,120,241,197,23,113,226,196,9,124,243,155,223,196,230,205,155,113,238,220,57,20,139,69,88,150,
  21,115,50,36,65,187,104,254,100,49,239,73,170,208,240,241,137,136,27,45,74,224,146,122,11,68,123,145,108,125,178,65,178,227,56,116,63,225,169,70,
  124,178,28,175,152,248,194,103,193,175,117,93,215,81,40,20,96,154,38,213,104,254,179,63,251,51,172,88,177,130,26,240,16,69,22,0,88,92,92,164,
  10,42,68,173,133,85,160,90,183,110,29,69,174,155,205,38,10,133,60,124,223,139,130,74,23,182,109,65,215,13,228,114,133,200,201,144,172,81,133,9,
  130,125,228,114,69,166,25,217,129,97,228,41,31,58,148,184,211,224,56,30,74,37,3,75,75,237,132,253,49,108,182,13,239,15,186,104,29,252,35,35,
  205,131,60,181,131,117,40,36,159,75,164,244,68,47,190,169,144,196,57,50,218,136,168,242,200,34,220,52,128,22,149,90,101,221,176,252,132,23,109,216,
  178,224,33,212,235,84,193,202,140,168,170,138,233,233,219,200,155,6,250,86,12,98,97,126,154,82,48,8,34,205,150,201,216,143,214,117,3,190,31,160,
  213,174,50,212,13,178,208,66,254,52,249,62,69,201,65,51,84,248,130,166,47,209,70,110,219,54,116,179,152,89,71,178,107,193,105,57,228,243,57,212,
  27,245,46,244,120,57,92,169,44,27,141,130,0,158,215,202,116,104,200,38,5,95,174,2,128,165,165,121,244,84,42,184,55,115,87,90,134,74,27,43,
  249,27,203,246,112,125,98,28,191,252,213,175,99,191,27,24,218,130,103,158,58,6,69,7,92,87,197,248,248,4,88,222,251,221,187,119,113,223,166,117,
  137,159,45,186,54,150,179,255,135,140,155,188,254,226,235,95,199,224,224,32,166,166,167,241,218,235,175,227,47,191,241,13,84,42,21,188,115,252,56,46,
  92,188,72,77,74,8,90,253,173,231,158,67,173,86,195,107,111,188,65,173,178,111,77,78,98,118,118,22,207,127,243,155,49,27,104,32,110,21,205,142,
  119,112,112,16,71,31,121,132,26,133,204,206,206,226,157,19,39,168,198,231,190,189,123,169,214,178,109,219,56,121,234,20,117,180,251,214,115,207,161,82,
  169,224,165,151,95,198,225,67,135,48,57,53,133,106,181,138,71,143,30,197,212,244,52,46,92,184,128,199,143,29,195,247,255,229,95,240,23,95,255,58,
  42,149,10,222,125,239,189,174,235,241,60,15,127,243,252,243,80,20,5,47,189,252,50,158,125,230,25,216,182,141,215,222,120,3,83,83,83,248,219,111,
  125,139,142,247,255,254,167,127,162,70,39,252,189,37,99,37,247,246,228,169,83,56,251,233,167,177,191,233,169,84,176,121,211,38,122,191,136,57,75,82,
  98,153,70,167,225,101,248,248,195,44,137,34,34,178,6,230,215,111,154,211,158,136,26,39,251,30,25,77,34,109,189,46,167,81,142,7,11,100,165,240,
  44,73,97,210,33,149,150,240,201,104,59,105,72,119,50,178,152,141,154,209,221,159,163,198,164,237,8,42,38,75,96,147,52,163,121,89,189,206,123,92,
  172,93,187,10,55,111,222,166,252,84,30,85,103,41,91,124,114,195,239,99,164,161,144,52,150,17,126,108,171,213,162,74,29,196,69,142,124,30,255,185,
  108,0,76,246,77,223,247,177,102,205,26,108,219,182,13,59,118,236,192,239,127,255,123,188,247,222,123,248,206,119,190,67,105,36,124,191,13,27,64,242,
  129,46,31,40,38,205,179,172,234,64,89,148,105,210,170,50,124,112,47,106,14,102,191,143,189,70,182,193,152,173,82,240,46,134,164,161,51,173,154,70,
  120,238,36,248,5,16,217,101,135,200,178,170,170,184,119,239,30,150,150,150,232,61,37,154,222,164,98,103,219,54,70,70,70,80,40,20,208,106,181,80,
  169,84,80,175,215,81,46,151,105,208,111,24,10,86,174,92,129,102,179,142,124,190,20,141,207,134,97,228,105,140,214,110,183,160,40,42,85,220,232,104,
  78,27,145,57,150,31,89,103,155,112,93,15,154,166,194,243,92,4,129,2,93,7,218,237,58,149,173,99,237,186,121,157,229,120,83,96,16,163,66,137,
  155,7,149,216,231,240,142,134,157,255,102,101,236,210,246,0,57,181,86,6,48,17,19,21,80,2,75,198,224,109,57,188,49,241,103,146,65,106,84,255,
  15,48,176,84,93,66,171,89,197,182,109,91,163,239,241,163,160,87,233,226,171,176,47,51,87,128,239,7,112,29,155,162,217,157,77,203,227,120,46,109,
  184,142,29,222,218,104,49,116,186,230,187,23,111,173,86,71,177,144,207,188,176,249,135,80,169,244,64,83,53,212,107,53,41,250,156,182,200,147,14,92,
  230,200,130,162,106,177,44,87,166,166,192,150,236,69,255,176,174,114,154,166,193,245,116,148,75,229,212,0,51,137,46,65,56,83,65,224,97,113,150,15,
  134,76,60,254,248,17,92,31,255,20,191,126,241,151,24,93,179,10,95,121,226,201,88,48,188,176,48,139,64,213,97,112,232,161,8,165,34,227,206,18,
  16,165,142,155,217,252,30,63,118,140,6,207,47,189,252,114,151,5,54,251,126,246,222,231,114,57,60,113,236,24,14,29,60,136,67,7,15,226,139,71,
  143,74,215,15,251,94,50,142,66,161,128,175,62,243,12,13,158,1,96,96,96,0,143,63,246,24,60,207,195,238,93,187,98,1,169,97,24,56,242,200,
  35,24,93,181,42,54,142,125,251,246,97,112,112,144,58,231,1,64,206,52,241,232,209,163,24,159,152,160,38,14,178,57,206,86,125,158,124,226,9,90,
  38,124,244,200,17,105,224,202,31,58,108,160,79,126,199,154,212,144,57,255,56,119,191,178,160,160,105,47,222,225,138,165,75,101,145,55,227,249,188,73,
  40,108,82,163,156,168,58,198,30,184,228,208,101,101,204,146,170,9,73,28,227,229,4,211,172,190,44,187,15,240,215,148,164,187,205,114,94,147,26,80,
  217,181,205,222,127,62,56,204,146,108,200,40,31,178,100,153,175,164,164,253,92,164,21,157,212,196,202,162,101,226,191,81,209,106,89,212,121,144,92,63,
  47,77,202,155,147,200,18,7,210,52,216,225,178,106,40,20,10,212,5,176,209,104,208,224,153,159,39,73,141,159,249,124,30,39,79,158,196,255,252,159,
  255,19,47,190,248,34,60,207,67,46,151,67,181,90,69,173,86,163,1,157,168,122,157,166,225,156,69,63,157,71,134,101,64,73,26,77,143,183,35,231,
  131,219,184,245,186,210,197,223,103,247,48,18,212,178,218,204,50,213,37,81,82,204,170,203,240,149,47,207,243,80,169,84,48,58,58,138,190,190,62,234,
  24,73,92,73,73,112,92,173,86,41,234,31,4,1,76,211,164,52,140,208,184,196,197,246,237,219,99,38,43,229,114,120,126,135,52,140,26,218,237,26,
  28,167,25,53,184,250,176,237,22,124,223,131,97,228,232,89,237,251,110,196,77,86,35,186,16,209,6,15,162,36,205,134,227,216,80,85,35,218,91,125,
  138,40,231,114,6,85,92,233,220,10,86,121,141,189,79,50,29,103,158,46,209,9,158,121,117,12,222,217,144,80,187,146,230,69,231,127,131,76,251,13,
  155,12,241,251,155,158,196,201,225,55,109,182,44,144,116,224,166,33,34,138,226,82,89,31,162,247,119,235,214,45,108,220,184,25,31,188,159,131,231,181,
  168,108,93,82,89,185,84,172,160,213,118,104,150,67,110,74,199,113,42,30,124,183,91,54,116,189,67,5,73,234,254,175,86,103,177,57,183,5,128,6,
  210,104,152,181,153,41,8,130,48,184,247,2,52,27,205,46,234,71,55,47,86,135,174,151,224,123,22,116,221,68,46,87,132,166,41,48,12,61,42,171,
  229,161,235,26,76,51,7,85,83,161,235,10,76,35,15,93,51,144,207,151,177,122,245,40,134,6,87,96,120,120,20,166,89,132,231,89,240,3,5,129,
  239,195,113,29,120,174,15,215,245,225,6,22,212,64,165,46,141,154,166,65,85,34,156,63,114,16,82,85,29,190,239,66,85,53,108,218,180,22,186,190,
  26,102,161,130,98,161,2,211,212,161,40,62,234,245,22,154,205,58,90,109,27,240,93,24,122,14,165,74,9,170,170,67,211,148,144,61,173,234,8,20,
  31,125,125,189,184,121,243,26,84,213,4,148,120,224,217,219,55,0,67,241,240,222,123,239,195,11,2,188,252,202,27,248,155,231,255,26,253,189,253,152,
  91,12,131,237,192,15,19,159,254,254,21,184,115,111,70,120,152,137,184,227,114,126,84,247,97,203,43,71,184,174,75,211,176,222,190,62,244,245,245,1,
  0,198,153,0,52,64,186,170,135,105,154,120,237,245,215,145,203,229,240,248,177,99,20,117,254,247,31,253,8,219,182,109,195,151,30,125,148,186,27,126,
  114,250,116,215,251,87,143,142,194,48,12,88,150,133,31,254,248,199,176,109,59,38,137,180,121,211,38,4,65,64,145,220,163,71,142,96,251,182,109,216,
  188,121,51,117,67,12,231,114,21,191,121,233,37,120,158,135,161,200,234,121,112,112,48,102,241,156,106,66,128,176,87,226,181,215,95,71,181,86,195,183,
  158,123,14,61,61,61,244,115,8,135,249,59,223,251,94,87,240,21,4,1,54,109,220,72,145,118,94,23,186,84,42,209,255,126,253,141,55,98,247,139,
  109,114,252,188,1,180,8,21,77,75,140,121,85,132,229,200,167,101,225,128,139,44,232,89,57,190,44,101,101,25,21,34,141,146,193,247,117,240,18,154,
  162,230,54,217,119,138,154,176,178,130,44,108,147,165,104,15,78,106,56,149,7,230,73,9,50,24,99,20,49,26,69,80,96,93,215,18,41,70,178,49,
  132,8,56,98,205,132,228,101,24,6,166,167,111,198,74,244,182,109,199,168,15,124,131,184,168,167,65,180,94,201,231,144,38,49,93,215,105,35,25,203,
  141,230,231,159,172,202,64,146,206,207,62,251,140,242,113,9,101,128,151,66,149,5,249,50,10,151,140,70,193,174,115,190,242,155,246,93,236,103,179,252,
  111,158,202,34,11,250,69,201,28,73,100,89,231,196,164,121,201,142,155,72,11,178,213,39,94,1,137,208,53,52,77,67,111,111,47,74,165,18,14,28,
  56,128,173,91,183,226,255,252,159,255,67,207,27,178,7,86,171,85,216,182,77,63,47,159,207,211,255,118,28,7,253,253,253,88,185,114,37,234,245,58,
  229,184,147,6,196,86,171,5,219,182,224,121,54,52,205,136,220,3,157,46,251,242,144,59,221,138,26,249,130,136,99,111,81,4,218,178,28,138,16,219,
  182,139,92,206,64,171,85,165,77,123,186,174,193,117,125,174,241,143,60,19,37,70,155,224,111,35,207,107,78,171,188,137,78,41,209,62,150,117,175,103,
  247,251,164,53,79,16,104,69,81,34,21,14,134,0,47,59,112,72,240,156,164,13,205,203,225,137,55,254,56,108,30,4,161,254,239,189,123,119,113,255,
  246,61,40,20,86,160,94,111,9,121,129,100,163,39,124,164,158,158,18,230,23,170,240,125,175,11,53,225,199,163,40,10,20,85,71,0,75,40,213,199,
  47,128,124,190,136,124,193,196,200,200,90,168,138,6,199,105,67,211,117,40,0,92,207,133,231,134,165,141,0,1,124,207,133,231,7,8,252,0,138,26,
  110,226,131,3,43,160,27,42,86,141,174,129,237,88,200,153,57,232,134,1,67,215,81,42,21,97,26,121,228,11,57,228,114,121,228,76,3,186,97,16,
  65,196,80,146,46,240,225,216,109,184,158,3,64,71,224,123,104,182,235,112,28,23,10,148,72,110,198,70,189,49,11,96,20,139,139,11,184,115,231,102,
  184,48,92,7,74,244,28,115,102,30,138,170,66,215,85,228,13,3,186,166,35,240,21,104,90,62,204,32,125,210,12,1,40,129,130,64,1,12,163,23,
  190,239,192,110,215,49,188,118,20,55,213,155,168,46,205,0,209,134,224,186,22,202,197,18,250,251,122,160,25,58,218,205,6,108,199,193,226,210,28,114,
  102,14,158,23,96,96,96,4,251,246,237,192,220,189,57,76,76,220,194,154,181,26,86,140,196,105,11,235,214,173,199,226,194,2,188,232,254,183,154,179,
  152,95,92,196,232,232,8,102,23,102,59,27,33,2,20,139,197,84,10,17,255,59,118,14,178,210,87,73,58,219,44,183,141,204,117,59,42,147,238,219,
  187,23,215,198,199,97,89,22,148,232,239,115,185,28,6,163,160,148,124,22,209,222,212,52,13,213,90,13,61,76,64,13,0,59,182,111,167,246,204,151,
  46,95,198,241,19,39,186,80,78,118,76,100,211,20,169,85,100,41,81,207,206,206,118,29,2,181,90,13,181,90,173,235,179,76,211,196,192,192,64,119,
  96,176,204,70,99,254,128,77,43,233,83,99,29,193,253,226,155,196,216,125,138,87,170,73,11,50,101,193,173,172,7,132,167,1,201,184,202,164,177,107,
  185,232,175,8,125,75,106,250,36,115,154,167,40,101,105,74,21,149,176,179,160,240,73,115,43,77,35,57,13,88,145,201,61,38,233,179,119,14,73,63,
  150,64,179,52,140,110,174,163,194,28,220,241,207,36,129,91,120,120,166,243,103,211,14,243,142,25,88,60,144,110,183,67,249,85,18,92,17,58,132,174,
  235,104,54,155,29,3,174,200,32,69,180,55,137,28,126,217,243,177,221,110,135,202,76,166,137,66,161,64,209,211,70,163,129,92,46,71,131,96,150,131,
  45,83,212,82,20,133,26,161,100,69,126,63,79,117,154,109,174,101,105,18,162,117,145,100,180,196,126,78,90,69,38,13,253,38,40,46,191,111,178,13,
  189,178,254,47,209,248,52,77,163,207,157,156,69,134,97,208,74,196,237,219,183,177,114,229,74,28,58,116,136,34,207,100,79,33,103,17,81,213,32,235,
  126,112,112,16,190,239,83,160,97,104,104,136,210,120,72,50,55,56,216,7,199,177,163,32,219,134,231,185,40,151,87,208,230,63,98,192,166,105,6,189,
  247,182,109,193,48,242,244,255,107,181,165,232,58,20,198,16,135,172,95,151,158,35,133,66,9,174,235,69,252,106,80,132,217,113,92,90,209,9,27,9,
  187,21,71,88,38,0,107,164,194,74,223,241,210,117,73,230,77,124,128,206,234,243,39,5,199,73,234,89,157,56,185,179,143,233,236,135,200,28,165,200,
  162,38,65,52,31,52,203,154,186,4,71,37,136,203,96,135,223,108,2,176,177,184,184,4,93,87,145,203,171,104,54,69,188,166,238,5,84,46,231,112,
  107,186,134,32,176,184,102,159,32,70,1,33,255,110,183,171,208,244,188,180,28,201,102,186,107,215,172,69,206,212,240,212,83,143,193,117,66,148,219,243,
  92,168,138,14,69,165,223,208,57,236,34,247,60,85,15,103,73,169,84,128,97,106,56,118,236,81,216,182,131,192,11,208,182,155,104,181,219,176,219,62,
  170,213,121,220,190,125,15,142,227,162,109,181,208,106,134,193,177,231,133,19,61,12,156,131,76,141,73,195,35,195,184,62,113,11,23,47,93,16,6,4,
  178,96,33,237,208,203,23,250,240,23,127,254,53,188,251,193,187,32,110,144,34,89,61,145,134,105,46,223,139,29,59,183,96,197,224,10,172,91,59,138,
  122,179,1,51,55,204,226,49,216,181,107,59,62,62,249,78,108,49,216,142,13,51,103,112,90,168,128,146,96,1,44,71,131,212,216,189,227,93,216,146,
  54,127,182,97,228,135,63,254,49,229,60,239,216,190,29,103,198,198,96,89,22,202,229,50,14,29,60,136,189,123,246,192,182,237,152,38,43,95,134,101,
  95,71,163,224,25,0,182,109,219,134,109,91,183,226,226,165,75,56,51,54,70,145,220,223,188,244,18,110,77,78,162,213,106,161,84,46,227,219,127,255,
  247,244,254,90,150,133,31,189,240,2,38,174,95,199,240,240,48,14,29,58,132,135,14,31,166,159,73,144,11,62,128,37,135,130,232,32,177,35,35,155,
  195,135,14,97,191,109,211,235,225,215,249,179,207,60,67,127,102,89,22,230,230,230,80,169,84,232,247,252,211,127,249,47,120,229,183,191,69,165,82,193,
  163,71,143,34,8,2,252,243,119,191,139,107,227,227,24,28,28,164,244,12,242,58,245,225,135,148,90,242,135,184,250,137,246,174,164,64,116,121,200,70,
  119,16,200,255,45,9,158,211,20,46,146,126,207,234,142,202,170,110,89,165,229,120,52,59,41,176,150,81,46,100,9,137,168,81,53,105,93,202,84,78,
  100,73,11,207,111,151,57,181,134,193,105,90,160,36,66,59,227,129,118,231,26,212,101,205,3,210,112,40,30,127,188,1,158,140,165,80,40,160,94,111,
  80,35,12,77,211,96,24,6,202,229,50,236,104,221,137,44,207,101,37,110,81,32,72,180,160,201,231,19,164,155,88,124,147,198,67,94,139,153,157,219,
  162,196,70,150,16,139,116,147,101,250,239,50,83,162,36,84,57,73,224,128,159,95,236,188,36,247,129,159,127,108,240,75,2,78,18,20,147,61,159,236,
  229,188,247,5,143,216,38,205,97,81,51,170,231,121,200,231,243,177,253,152,4,203,132,111,60,55,55,71,81,100,242,188,136,218,9,121,207,224,224,32,
  86,175,94,141,122,189,142,254,254,126,170,216,97,219,97,131,106,179,89,71,127,127,9,142,211,64,163,17,154,162,216,182,69,43,33,190,31,68,127,215,
  128,170,106,200,229,242,52,246,8,175,191,24,141,209,141,2,81,5,182,237,64,85,217,166,108,192,182,219,52,57,13,131,112,183,203,174,93,215,85,216,
  118,200,58,96,251,11,226,149,39,80,38,65,156,10,36,71,168,187,105,30,124,51,179,34,0,71,187,233,27,236,247,38,129,27,34,208,78,151,217,137,
  202,130,232,36,190,46,63,105,68,232,115,184,96,59,124,104,192,6,160,162,209,180,161,194,71,165,210,19,53,18,66,80,74,84,185,146,152,137,118,171,
  29,147,36,9,199,161,71,27,88,188,164,231,122,62,52,93,7,219,220,33,51,80,120,245,119,175,65,83,85,216,174,3,199,105,67,129,30,209,68,52,
  4,112,161,40,65,88,10,81,52,168,154,6,67,15,121,162,158,239,33,240,125,236,216,177,7,123,247,238,198,11,47,252,56,116,8,82,252,174,135,148,
  22,204,101,205,236,115,102,14,86,187,37,68,254,228,170,36,226,146,5,27,28,183,154,117,104,134,129,85,43,87,134,118,219,16,107,14,147,103,31,211,
  206,245,28,220,154,156,70,95,165,0,187,221,66,224,107,200,229,204,8,89,172,224,161,135,191,0,213,183,112,101,124,60,54,233,111,79,223,197,174,157,
  123,209,104,186,112,156,0,215,111,92,69,171,213,64,79,79,143,84,53,132,31,119,22,126,156,108,220,60,218,65,228,128,206,140,141,225,232,145,35,216,
  187,103,15,174,141,143,227,228,169,83,52,152,124,253,141,55,240,232,209,163,180,185,134,223,164,187,155,62,185,255,22,104,223,18,228,249,205,223,255,30,
  135,15,29,194,224,224,32,20,69,193,236,236,44,78,158,58,69,199,4,69,193,97,174,137,144,208,55,248,53,77,3,52,65,25,152,92,79,56,247,127,
  135,47,125,241,139,177,0,154,124,214,241,19,39,112,244,200,17,88,150,133,215,223,120,131,162,217,83,211,211,88,61,58,74,81,121,94,219,118,236,236,
  89,228,114,57,236,222,181,139,142,229,204,216,24,206,140,141,81,158,158,236,57,137,16,83,254,224,146,233,35,119,104,82,106,34,130,156,53,152,102,19,
  49,25,141,77,198,251,204,42,17,39,50,151,201,26,220,102,249,189,40,56,146,89,221,127,222,100,227,243,88,175,203,238,95,82,229,37,139,37,187,200,
  222,91,244,93,68,167,89,68,181,235,124,87,144,113,95,14,184,115,38,212,188,45,151,75,168,215,27,84,77,65,81,20,24,134,65,105,29,186,174,211,
  32,154,223,219,210,158,1,79,11,34,106,29,68,93,131,252,140,104,71,243,200,170,136,186,195,7,211,105,207,82,244,92,101,207,87,116,246,166,249,19,
  136,62,135,109,230,99,3,117,246,158,137,98,19,182,18,205,34,190,132,26,33,162,203,136,104,71,60,143,86,100,92,69,62,123,100,100,132,86,2,52,
  77,195,196,196,68,108,143,115,93,23,75,75,75,81,208,27,174,227,124,62,108,240,171,215,235,52,128,222,186,117,43,116,93,143,89,170,147,6,68,207,
  243,208,106,213,97,89,117,20,10,61,17,39,25,244,252,13,105,154,62,117,16,84,85,13,174,27,208,216,140,200,213,117,26,8,93,0,106,20,204,171,
  88,90,90,2,209,129,102,205,132,88,244,187,131,22,135,255,38,148,40,54,40,38,52,8,207,243,99,107,148,32,209,241,70,192,32,102,132,147,180,23,
  116,159,178,98,77,104,254,239,178,198,193,236,10,215,101,168,68,214,146,36,219,192,149,246,197,138,162,71,174,127,157,136,159,12,218,178,92,216,118,11,
  131,67,43,113,227,250,5,201,102,174,68,232,181,7,77,53,161,106,26,22,151,238,69,220,181,60,84,149,72,167,248,17,119,57,254,170,85,155,88,181,
  122,24,114,23,155,206,98,107,52,171,49,238,146,162,248,84,99,154,60,8,215,237,108,116,109,238,186,231,230,230,224,184,30,154,173,102,248,158,12,226,
  29,105,174,109,241,197,28,34,237,154,90,68,177,80,198,226,210,108,38,84,57,173,212,203,54,72,168,170,139,153,217,123,88,181,106,132,6,208,34,174,
  152,104,115,119,156,38,94,252,245,175,128,32,164,185,236,216,185,15,129,171,97,255,190,47,96,199,142,237,168,85,239,225,215,47,254,170,235,218,63,254,
  232,4,84,216,216,191,127,63,116,93,69,189,62,139,182,213,74,117,172,139,143,91,77,164,123,36,141,155,188,46,92,188,72,213,44,68,255,95,171,213,
  240,157,239,125,143,142,251,223,127,244,163,216,134,254,221,255,253,191,233,88,150,150,150,240,157,239,125,143,190,247,159,191,251,93,105,224,245,207,223,253,
  110,140,70,50,53,61,141,159,253,226,23,82,87,188,51,103,206,224,244,233,211,194,107,38,138,32,236,231,95,188,116,9,151,46,95,238,186,239,147,83,
  83,177,49,254,251,143,126,68,215,32,171,24,66,254,142,63,140,94,122,249,229,174,96,149,189,95,4,109,62,245,225,135,49,41,45,69,81,80,173,86,
  241,255,251,206,119,232,198,88,175,215,233,88,68,135,149,104,222,178,182,222,60,135,152,111,250,73,115,53,147,233,39,103,9,78,179,6,122,105,129,244,
  31,98,63,205,171,3,240,168,9,191,150,178,38,234,73,72,94,82,69,40,233,154,147,244,203,101,135,37,27,80,138,26,132,150,195,34,16,5,114,73,
  220,103,145,156,22,255,221,113,37,0,162,123,171,68,160,75,124,255,98,3,87,34,67,70,100,233,88,4,149,175,70,240,106,29,162,96,149,244,115,176,
  65,116,62,31,150,230,45,203,138,5,210,73,58,221,50,148,55,201,28,76,70,147,148,73,18,202,190,91,132,64,243,85,89,25,157,148,109,254,98,99,
  13,22,33,101,147,97,114,239,101,243,150,183,235,78,26,15,239,120,74,232,26,166,105,162,183,183,23,223,248,198,55,224,186,46,254,199,255,248,31,168,
  215,235,116,204,182,109,83,93,103,242,249,125,125,125,177,177,233,186,142,13,27,54,160,217,108,198,16,244,82,169,68,3,108,114,123,77,51,7,64,129,
  235,58,112,93,59,250,30,50,46,63,250,89,103,143,181,172,142,161,25,145,180,11,155,12,77,248,126,136,108,147,224,89,215,13,120,158,19,93,183,14,
  207,35,104,186,67,29,10,187,229,233,216,164,36,238,222,201,42,110,116,222,203,210,56,226,243,142,24,177,200,130,171,180,230,193,248,218,86,51,239,27,
  1,99,61,170,179,7,142,8,109,150,5,114,44,226,151,52,129,227,95,236,68,38,25,65,244,96,124,26,240,90,86,11,142,237,163,28,53,20,117,155,
  122,16,213,14,143,14,94,81,116,184,14,9,104,121,110,179,215,149,12,84,171,243,216,188,101,43,20,37,143,32,104,103,210,9,77,18,212,78,18,73,
  247,3,23,154,170,70,94,242,233,29,242,188,172,86,18,194,66,38,159,231,121,40,22,115,0,60,180,218,86,234,231,139,74,114,241,242,110,17,35,195,
  171,161,27,26,116,221,64,161,144,195,138,190,65,172,28,25,66,179,233,162,217,108,33,128,134,92,174,128,222,222,50,74,197,10,110,220,184,130,137,235,
  151,133,104,77,248,188,195,207,31,26,90,137,122,115,9,87,174,92,198,228,228,109,220,185,123,85,162,203,27,224,195,143,78,226,212,135,31,208,113,110,
  220,178,19,138,170,73,17,104,54,32,19,5,75,124,39,122,242,28,21,31,166,162,239,72,66,129,216,160,37,9,57,207,26,128,241,229,204,172,188,211,
  52,196,138,183,143,77,107,140,76,43,173,202,130,75,145,113,17,123,40,177,84,2,86,3,59,235,245,145,191,101,21,67,100,234,9,210,53,43,64,144,
  146,214,230,114,209,87,254,30,196,45,169,151,207,39,149,233,201,139,184,205,124,63,128,172,220,156,246,125,50,87,213,44,115,145,253,59,153,9,76,210,
  28,231,19,162,240,185,235,82,203,237,44,146,158,241,249,2,218,12,197,150,118,227,212,4,130,182,37,35,206,172,148,93,16,40,84,159,150,160,128,44,
  109,146,4,61,228,60,213,117,157,54,253,133,65,140,56,193,18,53,125,178,65,28,249,111,18,152,17,235,111,194,145,118,28,135,162,212,132,139,205,238,
  115,105,52,166,52,189,115,209,154,75,179,221,78,82,122,145,253,94,214,0,203,38,120,100,173,241,220,86,81,227,174,104,14,177,6,41,50,157,105,145,
  21,55,203,115,119,28,7,215,175,95,199,151,191,252,101,140,142,142,226,210,165,75,177,96,158,84,48,171,213,106,108,156,43,87,174,132,227,56,232,235,
  235,67,62,159,71,46,151,67,185,92,70,187,221,142,250,193,124,236,220,121,63,52,13,148,166,67,168,77,122,228,15,225,251,126,88,9,71,135,35,30,
  54,13,182,81,46,247,210,224,146,128,130,170,170,193,182,189,200,124,46,148,186,51,205,34,90,45,155,210,160,44,203,133,231,89,52,113,84,213,80,2,
  47,236,39,32,201,161,26,11,158,137,203,95,120,143,2,38,225,100,145,252,64,184,166,121,186,168,136,73,192,234,177,167,237,155,44,163,65,182,247,203,
  128,100,218,175,32,10,42,120,90,135,104,242,178,92,79,217,2,18,129,222,68,191,175,75,246,6,1,26,205,26,204,28,207,187,44,2,104,119,221,40,
  207,117,225,251,14,10,197,10,253,108,89,131,10,25,183,227,42,145,173,119,250,171,131,124,41,49,59,111,158,34,193,103,168,4,25,176,109,27,80,194,
  114,137,235,90,153,203,2,178,195,86,86,166,246,125,23,129,26,151,110,73,11,208,100,220,208,125,123,15,226,240,67,187,112,125,98,50,156,60,142,141,
  233,201,27,80,20,21,195,131,3,48,76,3,249,98,25,142,99,193,182,154,232,95,81,196,208,138,93,152,184,126,57,21,173,242,92,23,87,111,94,199,
  238,61,187,177,162,175,15,174,187,29,65,160,81,151,163,122,163,137,102,211,130,101,55,160,4,58,108,167,133,90,109,17,182,221,132,231,250,232,97,202,
  252,73,150,218,162,13,155,160,185,29,17,120,47,49,208,19,161,15,188,168,127,82,249,52,89,37,0,169,146,96,252,119,240,230,1,73,141,112,73,244,
  150,180,128,79,118,48,176,8,113,150,249,42,66,160,216,231,145,84,9,96,15,42,25,87,114,57,235,136,189,246,52,183,71,25,239,45,45,88,200,66,
  7,145,73,167,165,149,14,179,126,54,159,212,137,184,178,60,5,102,57,47,153,157,185,140,34,38,27,83,154,173,187,232,30,240,129,73,252,123,197,7,
  102,150,57,210,45,197,22,6,188,225,1,30,68,65,65,92,162,144,160,101,157,96,154,24,176,240,215,218,113,32,36,18,92,225,223,197,159,43,27,140,
  178,61,10,36,200,53,12,131,202,148,241,235,129,173,246,136,158,135,40,160,38,146,149,4,13,37,40,55,145,30,19,161,198,34,206,179,8,100,224,247,
  31,17,47,154,15,172,217,32,63,13,212,144,85,13,100,205,184,89,62,147,93,155,236,61,102,233,45,124,194,37,91,19,172,44,157,162,40,40,149,74,
  212,218,154,149,206,37,255,158,159,159,71,187,221,238,50,244,33,63,3,128,158,158,30,108,218,180,9,75,75,75,232,233,233,65,161,80,64,95,95,31,
  5,134,92,215,69,181,58,7,207,107,162,213,106,192,48,138,145,17,28,137,57,52,74,61,178,237,54,116,61,7,69,9,233,24,182,109,69,40,180,15,
  93,247,163,247,181,225,121,128,227,132,74,52,150,85,133,239,135,141,128,166,233,68,84,10,192,52,141,8,196,115,163,207,112,49,63,63,27,221,71,143,
  26,164,132,189,83,113,4,152,221,79,249,42,67,167,138,35,70,159,217,234,25,79,253,96,215,94,119,149,32,254,236,195,239,78,174,104,38,37,116,20,
  104,86,37,205,55,108,70,198,79,182,52,126,97,194,150,5,85,13,192,54,88,116,222,227,161,94,93,68,161,208,207,241,13,155,208,52,157,106,19,178,
  23,97,219,54,42,149,254,132,242,162,142,32,112,25,105,186,37,24,122,200,57,179,237,182,128,11,19,8,14,78,113,153,146,45,23,179,159,65,178,59,
  207,245,161,107,26,76,179,0,199,169,103,22,121,207,226,114,24,11,50,160,65,83,52,161,74,66,82,153,90,180,57,220,184,121,9,155,55,175,196,43,
  191,253,21,247,206,28,214,111,184,31,211,83,19,112,156,58,253,233,218,53,247,99,247,206,141,210,128,137,109,56,60,113,226,245,48,40,247,20,84,151,
  102,67,215,34,183,141,82,177,7,166,161,99,207,238,157,88,53,58,136,234,226,18,170,75,117,232,186,9,69,13,185,88,197,74,25,86,115,73,138,152,
  201,204,48,248,160,69,148,244,165,141,91,20,4,137,28,2,249,166,18,153,107,92,22,69,10,242,140,121,87,178,164,78,98,209,225,34,11,56,121,73,
  70,209,102,177,92,20,154,159,83,50,219,251,172,220,227,36,7,51,150,135,152,5,237,77,186,134,44,206,125,178,107,72,106,142,226,159,91,218,124,224,
  145,49,94,246,74,22,252,203,108,142,101,215,147,133,198,146,132,40,138,80,58,209,90,17,61,43,62,176,201,50,183,100,188,220,248,190,45,127,117,228,
  196,216,57,201,163,196,136,5,185,29,39,54,63,42,237,138,157,78,229,168,40,249,140,56,7,154,80,16,11,133,2,154,205,38,135,186,117,204,56,8,
  181,130,148,254,139,197,34,108,59,108,238,18,233,134,103,121,126,236,92,13,3,37,59,102,7,237,121,30,253,124,182,25,90,180,38,211,170,102,124,66,
  158,196,117,231,239,171,104,61,138,20,47,146,20,21,68,141,137,124,95,16,159,20,179,201,60,191,238,120,238,173,104,143,101,63,191,92,46,163,175,175,
  15,158,231,97,197,138,21,152,158,158,198,252,252,124,172,42,100,89,22,230,231,231,99,159,75,120,234,36,128,14,130,0,219,183,111,71,62,159,71,181,
  90,165,99,33,52,156,78,16,236,69,243,75,165,207,177,213,106,70,102,40,6,147,28,249,80,85,45,82,198,112,152,245,219,105,98,12,43,46,10,218,
  109,11,229,114,62,82,216,240,97,154,69,184,174,31,125,150,15,93,55,35,14,182,75,237,191,59,115,83,161,204,130,80,23,157,117,8,236,156,201,225,
  218,212,184,222,48,57,117,35,190,246,32,49,82,129,112,15,37,13,133,50,155,111,209,124,19,157,33,252,207,116,112,188,91,217,34,32,27,209,114,248,
  128,221,7,139,207,220,36,53,10,162,195,32,9,240,97,91,62,250,251,10,0,140,200,210,91,137,56,99,126,172,196,22,142,35,128,239,1,129,239,112,
  155,135,66,209,232,32,136,59,32,85,171,85,248,129,135,222,222,1,204,204,212,4,37,56,57,61,67,118,232,198,203,176,157,44,169,213,110,2,138,2,
  211,212,17,245,3,164,102,213,105,188,59,30,177,163,200,90,6,212,138,223,108,69,238,58,141,134,5,215,141,7,11,138,146,199,87,158,250,10,86,173,
  28,134,109,237,195,79,126,250,31,52,187,213,13,13,45,203,234,226,93,178,135,58,79,169,184,113,227,138,16,93,28,251,244,34,254,254,239,159,199,91,
  111,190,137,171,215,39,160,169,38,138,197,28,6,6,6,208,191,98,5,122,122,202,194,107,75,114,133,75,10,136,248,77,61,109,220,236,223,243,115,158,
  167,62,100,177,95,151,253,63,239,88,201,34,34,89,145,187,36,158,187,172,36,154,69,223,60,185,81,75,233,74,168,147,208,109,89,50,71,254,78,68,
  185,145,241,52,147,238,65,210,254,148,36,219,153,165,172,156,134,84,36,33,122,50,171,240,172,65,75,86,245,153,207,83,241,74,229,254,9,42,10,89,
  220,228,120,35,15,158,119,158,166,48,34,30,107,64,155,145,68,223,77,36,79,217,82,49,79,213,96,149,155,58,64,138,146,193,218,92,137,73,115,133,
  222,3,221,136,120,248,59,149,158,129,196,230,155,15,50,89,84,153,149,43,35,74,29,138,162,160,221,110,199,42,43,252,90,202,98,22,194,86,74,201,
  217,69,254,33,198,28,132,27,205,211,52,147,18,178,164,224,82,166,208,33,162,21,241,220,102,217,254,37,42,175,179,1,51,59,215,120,196,90,180,255,
  242,77,232,73,84,22,246,231,196,69,144,4,223,229,114,25,249,124,30,219,183,111,199,227,143,63,142,159,253,236,103,120,235,173,183,98,85,80,199,113,
  168,131,32,25,11,65,150,203,229,50,125,22,235,214,173,163,201,22,225,58,175,90,181,146,57,119,66,227,19,77,211,65,184,204,190,79,140,84,204,168,
  49,49,148,171,115,28,11,133,66,15,53,251,113,93,43,106,240,51,162,192,219,162,40,188,174,171,148,242,1,32,146,63,84,233,186,8,231,188,43,160,
  223,241,107,52,84,47,11,145,240,184,234,6,207,223,239,214,133,14,150,77,211,232,84,133,148,88,194,28,183,236,142,35,210,178,88,73,26,63,48,223,
  174,242,37,253,36,47,120,81,86,203,58,179,240,50,97,73,238,123,161,66,6,9,120,253,104,130,184,209,131,234,70,24,248,195,211,243,92,212,234,139,
  40,151,139,93,151,71,186,71,187,14,119,223,65,16,40,81,249,65,17,30,168,89,156,2,121,11,96,209,1,220,108,182,17,4,64,255,138,97,161,13,
  107,86,41,185,180,177,184,30,160,233,6,74,165,222,232,154,181,132,247,41,137,92,178,190,21,195,8,124,47,118,191,31,60,112,20,125,149,60,126,252,
  163,159,32,151,203,99,176,191,163,17,172,233,122,168,43,45,225,151,138,202,239,50,62,156,101,219,112,93,31,141,86,216,24,225,7,14,26,205,6,110,
  220,188,129,211,99,103,240,246,241,19,82,238,111,146,53,119,210,253,251,99,140,155,124,39,161,137,136,154,209,146,56,203,73,186,167,188,195,155,140,182,
  146,116,93,105,1,19,219,253,253,121,95,236,24,69,38,42,236,30,145,100,14,193,7,165,34,37,147,36,26,76,150,113,178,255,144,103,150,180,175,145,
  146,44,123,13,188,115,85,26,114,155,68,77,72,154,179,36,89,75,15,228,150,71,187,144,253,29,239,98,202,222,3,158,158,212,209,131,13,18,203,233,
  178,100,138,77,184,100,232,36,187,143,139,159,165,159,105,253,38,25,105,116,143,221,79,108,4,237,148,163,9,154,133,88,32,47,115,163,99,105,128,68,
  198,140,92,63,155,128,243,116,64,226,50,72,172,155,139,197,34,114,185,28,147,24,40,66,183,72,209,126,194,74,165,177,104,52,225,65,147,64,201,52,
  77,138,82,179,207,135,93,227,50,218,92,18,170,204,107,53,203,232,39,201,38,108,157,196,152,55,55,33,115,134,32,233,228,30,242,129,52,31,172,203,
  20,154,68,213,53,145,211,47,89,167,228,222,221,189,123,23,147,147,147,56,112,224,0,149,255,229,175,193,182,109,52,26,141,88,160,183,118,237,90,88,
  150,133,85,171,86,161,82,169,32,151,203,161,88,44,82,26,136,235,218,88,187,118,24,158,215,68,189,94,139,174,141,24,77,153,52,169,8,199,25,54,
  242,133,193,179,23,129,150,126,244,51,55,170,224,183,35,86,128,22,153,198,216,240,125,162,188,225,33,46,163,27,209,69,3,31,190,31,114,165,235,245,
  26,167,179,30,183,218,38,40,50,15,70,17,30,52,25,127,216,120,232,197,208,231,142,13,120,32,81,206,8,132,232,115,50,175,62,96,158,173,47,5,
  85,100,103,50,253,25,27,23,39,161,44,105,7,84,18,71,90,188,161,123,80,20,141,110,82,60,234,219,178,26,128,2,24,70,17,174,235,48,165,47,
  118,145,133,200,181,231,121,88,90,90,196,208,240,186,232,50,220,24,42,46,30,127,128,86,187,142,114,185,34,149,154,202,122,237,162,140,151,13,170,61,
  47,0,2,32,95,200,71,19,65,139,228,245,60,24,70,62,114,1,84,97,152,38,131,78,132,210,120,128,15,85,81,97,230,202,240,61,23,1,92,104,
  170,129,92,190,8,93,211,160,40,30,204,92,1,186,14,244,245,14,160,88,50,241,216,99,95,70,173,214,134,110,152,8,2,27,174,227,64,83,77,248,
  190,11,221,48,144,203,229,16,0,176,219,77,40,170,14,215,117,96,219,14,28,199,133,213,182,176,176,180,132,149,43,87,97,113,97,142,185,223,38,118,
  110,223,128,247,78,188,13,47,0,2,63,192,226,226,2,189,63,185,92,14,14,131,148,200,80,123,145,189,49,159,84,24,70,14,129,239,193,115,173,212,
  3,95,38,101,148,84,122,22,29,242,127,140,113,103,69,45,69,227,150,5,242,105,218,184,178,0,137,61,168,100,138,37,50,100,133,189,23,203,225,58,
  203,164,23,101,77,96,178,234,75,22,36,157,79,44,178,32,231,73,165,230,36,75,104,17,146,199,30,150,36,217,78,123,206,89,209,94,49,191,23,137,
  247,33,109,207,230,19,123,81,162,146,116,255,120,116,80,20,228,45,23,201,78,107,168,149,53,155,102,73,86,196,136,167,146,16,196,203,230,136,220,125,
  145,52,31,117,151,255,131,24,72,17,121,98,209,102,195,206,218,67,204,229,80,4,174,240,104,46,9,112,201,119,178,74,29,44,119,153,213,54,22,221,
  155,164,102,40,226,186,167,235,58,13,76,137,51,29,145,92,99,209,104,190,17,157,119,41,102,3,18,98,82,19,71,5,197,206,138,50,254,62,225,21,
  147,207,19,37,114,124,37,144,165,170,137,146,113,81,224,159,36,63,201,54,6,146,196,154,40,95,144,160,157,240,213,85,53,68,112,235,245,122,36,251,
  22,95,3,174,235,194,138,170,183,65,16,96,120,120,152,114,157,139,197,34,42,149,10,117,128,36,159,191,176,48,131,80,89,84,161,207,221,113,108,0,
  62,114,185,10,51,198,144,218,209,219,59,192,4,212,46,124,63,8,165,127,219,94,244,204,109,4,129,143,86,203,130,105,26,104,181,90,116,205,104,154,
  138,118,187,17,189,55,252,135,240,159,129,128,86,66,58,28,255,14,135,185,91,223,25,92,19,110,16,83,72,9,215,168,34,160,110,248,177,231,207,58,
  0,202,169,27,60,117,10,210,125,214,247,189,152,4,114,114,236,40,118,1,213,179,4,136,178,114,43,255,59,94,139,145,231,18,5,65,104,5,45,59,
  95,28,219,70,222,52,35,222,78,210,33,27,45,122,199,130,153,203,35,46,166,223,177,100,21,221,8,171,221,166,157,208,124,233,48,252,183,30,90,102,
  43,42,252,32,228,232,152,102,17,133,66,17,190,111,193,48,10,208,117,3,136,184,66,186,161,33,103,22,97,24,38,76,211,64,0,32,159,43,160,82,
  46,35,95,48,240,208,225,125,216,186,117,51,20,40,208,181,80,67,218,247,163,131,25,65,120,37,81,141,33,188,127,46,2,248,8,124,5,166,145,131,
  231,59,208,52,3,190,23,192,15,60,120,158,19,149,82,2,88,118,11,186,170,66,85,128,102,163,134,59,119,110,195,182,108,168,154,26,106,55,6,10,
  204,92,46,68,30,34,231,66,69,213,144,47,20,160,42,161,69,121,190,80,70,177,80,194,252,252,12,6,7,123,112,238,108,199,174,122,237,218,251,16,
  248,14,174,140,95,198,192,224,122,40,170,2,40,122,52,109,194,146,79,160,20,83,15,55,94,78,75,36,21,166,107,6,20,85,133,231,199,55,77,214,
  157,74,38,79,151,149,95,158,85,218,111,57,227,22,125,86,214,113,139,54,108,158,219,39,90,87,252,70,44,163,71,100,9,82,211,16,252,44,180,24,
  153,114,130,44,216,255,67,232,9,89,2,71,89,66,207,186,175,137,238,1,127,144,202,26,74,121,138,15,91,126,79,106,146,99,249,194,89,147,36,217,
  53,138,74,230,236,245,101,65,241,68,129,178,140,171,157,52,103,146,230,49,207,19,231,215,142,200,48,68,150,72,164,93,79,156,198,247,249,92,241,226,
  159,201,22,107,149,88,16,172,40,106,204,185,176,163,85,75,230,7,152,64,187,19,120,168,170,138,98,177,32,213,72,102,131,63,214,117,151,160,209,134,
  97,208,128,54,151,203,209,64,154,15,30,101,205,198,50,254,62,9,8,217,202,12,9,238,9,223,150,4,238,34,5,30,190,103,74,166,163,156,69,121,
  35,173,233,150,111,186,99,147,89,190,82,37,171,132,179,54,219,34,74,86,210,158,77,170,1,67,67,67,52,152,111,183,219,152,155,155,139,253,29,145,
  10,36,154,206,228,123,72,80,204,142,113,199,142,29,148,182,65,154,3,7,7,7,97,24,6,13,206,67,4,56,52,227,9,155,93,125,154,68,153,102,
  158,34,187,164,10,239,56,30,114,57,66,119,105,195,247,131,208,201,88,81,163,239,177,105,192,235,186,54,53,211,82,85,53,106,94,13,40,253,40,76,
  166,26,180,210,210,65,116,227,50,117,97,165,5,84,101,173,243,156,130,88,5,188,3,90,41,17,170,45,118,220,228,41,27,34,224,53,27,112,17,100,
  142,27,178,232,225,147,107,214,121,54,73,18,154,204,151,242,100,90,158,162,3,41,252,189,219,5,179,179,239,177,93,31,138,10,248,28,119,89,116,35,
  52,77,131,213,182,145,203,25,116,50,17,80,61,108,84,20,7,0,154,166,96,227,198,117,176,109,11,133,66,41,250,185,139,74,165,140,222,222,18,42,
  149,62,128,44,180,32,84,7,9,179,30,13,174,219,138,50,55,15,10,52,64,9,213,37,60,63,64,187,93,135,170,104,112,61,15,182,213,194,189,217,
  89,172,219,48,136,187,119,103,113,250,244,71,104,181,154,176,157,54,20,104,212,221,199,245,28,4,126,0,63,8,125,226,21,168,112,92,75,128,20,116,
  244,175,249,135,168,105,101,60,223,247,77,92,184,112,9,87,174,93,68,16,24,81,227,164,47,148,220,225,127,54,208,191,14,107,214,174,195,245,155,55,
  177,249,190,251,160,106,57,220,191,117,63,86,173,28,193,198,141,107,241,201,71,239,3,0,230,231,110,163,213,114,240,55,223,250,22,154,237,22,126,249,
  203,159,195,178,92,228,114,249,196,160,137,71,61,101,243,75,211,1,207,181,169,201,0,255,25,105,92,84,217,247,102,89,16,127,200,184,101,92,220,229,
  142,155,95,43,108,48,150,196,209,229,165,242,88,105,46,190,17,38,201,93,41,41,88,227,181,152,249,0,72,118,63,101,1,168,236,179,248,230,198,36,
  7,178,164,228,68,214,68,201,186,141,145,64,33,201,128,101,57,149,6,217,51,22,25,76,177,125,19,73,202,37,73,213,17,254,126,146,146,246,231,77,
  34,249,231,156,22,192,102,225,60,243,136,47,65,195,210,130,123,217,121,34,3,82,186,21,47,58,62,0,221,214,222,217,221,76,147,185,150,65,236,187,
  85,85,225,58,254,213,88,115,20,225,96,134,252,82,133,34,101,105,74,22,252,53,147,114,62,9,164,137,220,29,161,91,136,130,213,229,88,112,179,40,
  43,203,235,101,221,242,72,80,223,205,97,13,98,191,23,249,67,136,26,180,248,0,150,165,121,240,52,10,150,122,34,162,203,136,192,190,164,228,147,221,
  235,121,58,145,200,100,134,92,15,105,20,236,233,233,193,211,79,63,141,187,119,239,226,127,253,175,255,69,199,76,214,146,101,89,148,75,204,42,107,248,
  190,79,29,92,1,96,245,234,213,104,52,26,49,192,100,112,112,5,44,171,77,171,3,29,199,63,51,38,1,204,238,137,225,251,195,235,208,117,131,83,
  246,64,100,223,237,3,240,224,121,68,163,90,69,179,217,65,155,115,57,3,150,101,71,232,54,168,255,69,216,176,200,246,30,116,104,17,36,120,238,32,
  196,1,7,0,121,17,112,168,8,247,143,120,147,161,104,15,71,12,149,142,207,99,217,126,215,29,60,135,61,116,154,244,12,23,169,113,200,170,102,138,
  162,64,87,18,16,52,118,50,167,233,231,102,45,217,37,109,184,90,228,74,227,71,93,161,157,5,168,64,228,0,85,173,89,40,228,115,200,23,250,208,
  110,205,69,55,194,133,231,197,15,39,54,184,184,113,125,2,187,247,236,193,193,131,123,225,146,204,29,64,179,81,199,244,212,20,238,205,156,70,173,90,
  15,253,227,3,15,158,27,142,39,128,14,215,107,9,178,31,210,60,216,253,96,215,172,94,135,70,163,141,169,233,27,169,58,178,65,224,3,10,207,23,
  50,163,146,157,129,98,177,2,77,211,161,169,33,247,174,84,233,65,95,239,0,122,122,86,160,111,69,25,95,126,236,203,184,127,235,46,228,11,69,248,
  190,143,107,215,198,113,246,236,251,177,241,178,27,130,166,105,216,186,245,1,60,122,244,33,148,202,38,114,70,128,169,233,187,24,28,24,196,237,187,211,
  208,117,7,239,191,251,123,92,188,124,37,26,187,141,95,253,234,151,24,30,90,1,203,110,161,217,108,32,8,20,24,166,177,236,82,191,40,200,49,205,
  60,252,168,84,197,30,230,124,80,201,150,60,147,146,188,172,115,238,15,29,183,44,216,204,58,110,25,250,75,230,129,140,3,42,11,110,217,3,75,214,
  216,152,214,185,47,51,54,72,82,244,16,205,109,209,1,153,69,79,85,22,172,240,247,153,71,241,68,86,209,162,32,148,71,158,210,116,111,151,143,92,
  198,231,146,8,153,78,66,122,101,229,99,209,179,201,130,212,202,236,199,101,123,115,82,208,158,21,205,22,57,54,242,168,125,82,112,247,121,147,128,52,
  37,163,164,107,144,43,183,4,41,38,43,1,130,64,97,202,199,30,0,181,75,66,139,52,205,135,104,174,193,184,182,9,206,194,4,121,88,18,152,121,
  158,71,93,237,116,93,71,187,221,142,81,58,100,235,144,167,201,176,243,147,253,25,27,76,19,26,6,225,70,139,56,204,97,208,38,246,143,96,231,51,
  95,9,226,159,45,91,222,23,245,42,116,248,179,190,176,137,149,252,140,55,108,226,43,30,34,169,94,209,61,98,63,147,252,83,173,86,49,63,63,143,
  255,244,159,254,19,86,175,94,141,249,249,121,142,170,19,73,231,58,14,69,118,131,32,64,62,159,199,240,240,48,108,219,198,186,117,235,240,225,135,31,
  82,174,179,235,186,145,83,100,11,235,214,141,192,243,26,104,52,234,80,20,45,114,196,181,162,251,161,83,90,71,56,252,32,178,134,47,68,247,199,130,
  170,42,48,140,28,108,219,139,158,135,23,113,157,195,0,90,215,59,244,137,80,14,78,141,208,233,208,146,219,247,125,56,142,135,208,163,35,52,222,33,
  180,36,214,101,48,254,223,29,139,123,207,139,247,40,16,229,181,184,42,143,2,25,239,185,123,47,69,36,45,169,8,169,27,236,124,233,60,7,85,2,
  192,42,210,51,148,196,70,162,216,81,180,159,232,73,170,6,172,62,37,139,214,200,100,62,248,77,91,100,145,76,178,160,240,43,20,248,190,10,77,139,
  154,8,221,0,166,161,65,211,21,41,95,139,189,121,150,213,132,174,41,48,12,29,237,86,186,76,152,162,40,248,244,220,57,124,122,238,92,148,73,249,
  148,147,157,158,28,216,204,102,79,196,251,125,116,156,9,187,239,69,189,214,64,57,202,50,211,208,20,62,184,58,122,228,113,12,15,175,132,97,154,232,
  237,41,192,243,220,104,2,135,122,142,150,101,193,113,60,180,91,13,124,242,241,199,240,124,31,8,124,216,86,27,195,67,43,241,229,47,31,196,228,173,
  203,152,95,152,237,186,7,228,185,84,202,5,148,202,38,28,219,194,249,243,103,49,186,246,126,140,12,247,224,210,229,115,184,116,249,60,181,246,12,39,
  162,130,102,107,17,215,111,46,210,107,152,156,188,134,47,60,252,0,202,165,18,234,34,169,17,201,1,37,226,255,22,10,5,56,78,187,43,13,225,203,
  192,89,18,54,22,73,72,67,69,151,115,176,138,198,45,77,6,51,140,91,212,37,46,250,93,90,80,201,203,201,241,65,52,25,39,65,0,201,28,144,
  173,99,62,240,73,74,158,121,105,197,180,178,154,172,81,136,45,201,178,135,80,92,138,168,59,40,76,146,245,19,241,138,69,229,225,52,133,148,172,47,
  145,114,75,146,195,91,214,57,45,83,207,201,162,54,147,229,239,146,144,243,229,218,157,179,168,63,255,126,81,194,179,220,251,156,62,30,37,53,249,138,
  31,202,224,2,41,18,232,42,93,174,134,172,132,22,223,213,207,254,62,62,31,149,174,127,231,114,38,70,71,87,226,198,141,91,169,247,57,105,110,146,
  114,191,105,154,200,229,114,212,10,156,213,30,230,19,34,217,25,41,171,74,17,250,0,113,43,52,12,131,82,58,200,231,19,26,3,27,43,136,104,156,
  162,231,46,107,80,101,169,78,108,220,193,54,50,138,122,88,216,191,231,21,68,248,128,94,36,3,200,175,87,223,247,35,97,131,206,223,144,64,55,12,
  50,67,107,244,153,153,153,46,169,81,82,117,97,19,165,251,239,191,31,149,74,5,173,86,11,197,98,17,133,66,129,90,117,19,132,223,178,90,8,2,
  11,166,89,160,84,33,207,115,225,121,97,19,96,232,250,231,209,166,65,53,50,108,99,121,216,65,192,38,65,30,90,173,102,52,126,11,154,166,96,105,
  105,145,217,183,116,168,170,69,239,151,235,122,33,77,21,1,124,95,161,104,115,199,37,144,60,183,32,138,163,2,74,211,32,72,114,136,24,19,35,34,
  45,106,74,100,207,206,112,157,177,188,231,78,255,64,183,121,17,249,238,238,42,129,76,173,69,132,62,7,137,113,67,86,16,133,221,83,244,36,249,164,
  78,201,73,239,138,212,147,2,8,81,151,106,231,119,10,0,3,170,234,117,149,32,106,245,58,128,144,247,101,209,77,135,45,203,241,72,170,195,216,57,
  118,31,196,241,155,202,115,103,186,75,32,225,223,107,116,146,16,233,163,238,67,159,125,79,71,102,143,167,72,52,26,45,172,232,31,160,40,245,114,14,
  139,98,209,128,166,123,184,49,113,9,179,115,51,184,125,231,118,148,121,251,112,221,0,43,250,86,66,211,115,184,123,247,74,36,90,222,217,160,86,173,
  108,98,203,125,91,208,106,183,164,154,207,170,170,226,163,143,223,197,212,212,77,148,75,57,212,26,85,84,171,243,88,183,238,32,254,234,47,255,30,134,
  161,193,243,29,184,110,180,56,60,15,150,221,66,187,233,96,113,169,134,249,249,57,56,142,143,66,161,132,209,149,163,184,124,237,74,98,57,68,108,195,
  217,217,176,242,5,131,106,147,202,232,9,203,9,100,178,184,249,201,144,214,229,140,59,11,173,34,237,37,147,143,100,187,170,101,232,174,136,235,39,227,
  107,201,198,205,106,208,242,123,64,22,167,56,86,137,134,215,4,22,41,5,164,5,10,50,233,58,182,153,68,102,229,45,43,215,202,230,129,76,1,35,
  77,221,33,173,121,80,164,182,34,227,189,102,9,20,217,247,176,206,154,89,2,241,229,36,141,105,136,203,114,80,238,164,234,70,90,194,146,85,199,155,
  71,149,100,166,65,221,22,223,60,175,82,246,247,1,136,185,10,107,162,210,189,126,20,170,33,77,202,216,138,162,49,1,119,16,227,73,175,88,81,194,
  232,232,195,248,224,131,15,18,27,216,100,213,34,118,30,133,232,160,143,92,46,7,211,52,161,40,10,69,62,211,116,191,121,195,27,254,222,179,251,16,
  203,139,54,12,131,6,152,100,62,138,36,62,179,160,224,162,189,138,196,28,44,229,137,7,13,72,48,37,234,189,74,218,71,68,201,40,15,62,144,247,
  151,203,101,20,10,5,74,197,176,44,43,54,207,73,227,29,219,40,72,52,157,89,75,118,242,243,29,59,118,196,18,144,32,8,80,42,149,168,235,36,
  81,192,208,52,13,249,124,137,170,84,132,61,53,14,149,131,35,115,203,117,109,168,170,14,69,209,24,123,110,39,250,123,63,106,102,108,68,118,220,6,
  74,165,60,26,141,70,68,169,32,72,189,27,67,111,13,67,135,101,53,35,94,114,135,191,204,59,9,178,193,51,27,52,119,158,47,34,157,231,110,179,
  20,18,124,179,180,13,246,57,116,239,143,129,192,185,85,230,172,155,4,66,164,211,210,146,120,240,93,8,116,214,13,92,20,56,164,145,176,69,155,153,
  166,49,118,220,220,133,219,86,19,186,174,163,144,47,160,81,143,223,60,81,25,147,76,132,66,161,132,234,82,119,54,17,127,8,233,27,50,43,57,68,
  208,12,17,250,22,231,87,121,93,129,2,249,219,86,187,1,51,159,135,166,229,16,4,109,41,154,41,66,246,230,231,231,17,248,26,222,63,249,110,87,
  112,177,125,251,1,28,57,114,8,170,166,224,183,175,40,152,152,184,24,251,253,138,254,62,180,173,54,26,141,154,16,137,98,191,231,246,157,91,244,125,
  119,110,79,224,181,223,189,138,74,165,140,185,185,57,56,142,131,114,185,23,143,29,123,12,51,119,111,227,246,228,117,148,202,21,140,140,84,176,105,227,
  40,12,211,64,177,160,9,105,34,178,210,42,159,100,144,177,229,115,57,180,90,14,254,88,175,229,150,181,63,239,184,151,83,42,78,203,122,69,1,189,
  200,17,75,20,240,200,164,235,146,74,207,50,19,152,164,224,43,77,107,154,95,239,89,147,136,36,233,48,222,69,143,253,91,153,33,130,44,144,77,147,
  142,148,105,29,243,193,112,26,223,88,148,212,177,116,28,182,241,47,77,135,90,132,236,38,33,184,162,160,73,102,69,159,54,175,210,16,241,172,21,153,
  164,196,78,70,229,17,149,218,187,199,177,124,43,121,246,149,102,4,70,2,103,81,240,220,73,218,0,77,83,99,65,3,57,172,227,251,189,74,209,237,
  86,171,133,71,30,249,50,214,174,93,139,87,95,125,21,75,75,75,82,222,110,210,185,197,6,209,142,227,160,80,40,80,235,104,93,215,97,219,157,230,
  48,81,21,38,139,246,55,155,84,145,56,128,32,210,236,207,201,253,23,89,211,243,137,52,111,78,34,82,243,225,221,25,121,105,76,209,30,194,210,67,
  178,210,249,100,210,131,229,114,25,235,214,173,67,187,221,198,225,195,135,81,175,215,241,226,139,47,118,85,245,28,199,193,226,226,98,236,179,43,149,10,
  77,50,122,122,122,208,108,54,209,211,211,131,254,254,126,26,116,147,185,55,50,50,0,215,237,40,173,52,155,245,8,68,43,51,18,193,46,124,223,133,
  170,234,81,130,230,195,182,195,103,174,235,6,149,162,179,109,11,174,235,80,171,108,215,181,97,24,29,52,220,182,219,200,229,114,104,52,194,164,207,48,
  204,232,58,28,202,61,174,86,151,104,208,239,121,113,15,142,142,166,58,98,193,51,27,4,119,64,86,145,148,164,202,32,215,242,125,165,83,129,232,160,
  200,113,26,24,186,18,125,17,181,42,105,79,20,81,224,196,177,163,124,13,234,89,203,227,34,199,45,209,123,179,32,98,241,242,86,103,129,213,106,53,
  248,158,131,222,222,33,204,204,220,4,16,74,182,177,6,37,236,171,209,108,193,245,28,148,186,180,160,229,196,114,30,53,206,138,74,177,201,131,200,90,
  91,244,32,148,64,67,33,111,66,85,53,144,179,83,116,240,137,202,212,179,179,75,184,127,203,118,42,219,215,225,17,233,248,194,23,246,227,211,177,79,
  160,25,6,30,216,183,15,19,19,23,99,227,237,239,239,67,171,221,140,109,68,105,135,38,233,178,190,126,99,34,246,243,153,217,25,76,77,237,196,141,
  241,171,184,112,249,66,215,251,6,250,87,80,65,120,217,166,36,10,58,249,13,179,82,46,163,198,148,149,248,242,222,31,243,37,146,83,250,188,227,78,
  107,182,205,18,196,203,198,184,156,96,132,71,127,100,205,113,178,0,153,159,255,252,1,151,165,180,76,14,182,229,6,151,89,26,28,211,84,64,210,2,
  162,164,123,205,114,212,179,4,45,73,215,151,244,204,146,156,211,178,52,109,47,39,65,20,125,206,231,165,166,100,9,70,147,2,215,36,42,143,232,32,
  75,227,106,199,169,9,114,71,70,25,109,132,29,191,168,234,18,223,199,229,77,105,221,70,44,4,125,38,136,115,192,89,142,35,102,85,220,106,53,113,
  248,240,97,172,89,179,6,47,191,252,50,206,159,63,159,89,101,134,167,67,144,117,218,106,181,224,56,14,229,70,179,52,8,150,147,159,212,19,33,66,
  110,89,133,14,118,62,145,32,145,80,231,120,94,178,72,23,63,105,254,179,99,100,81,81,190,231,129,149,205,227,19,40,158,26,151,212,156,73,208,115,
  150,26,64,56,229,182,109,227,214,173,91,120,226,137,39,240,232,163,143,226,234,213,171,20,41,230,141,84,90,173,86,236,251,6,7,7,233,103,110,217,
  178,5,211,211,211,24,24,24,160,20,155,16,213,183,177,109,219,58,0,109,52,155,245,136,26,170,48,46,129,172,124,158,7,207,115,169,2,71,231,126,
  250,20,229,37,129,182,231,57,48,140,28,92,215,131,239,187,104,181,22,152,123,175,82,17,131,78,117,197,143,2,232,32,66,176,59,200,115,39,224,5,
  163,133,31,86,225,9,133,34,41,120,22,1,36,157,223,177,28,232,64,224,44,11,136,244,158,147,43,87,144,84,147,186,227,212,36,79,140,52,192,128,
  174,15,86,10,38,11,90,32,10,148,101,72,84,250,65,26,255,204,102,187,141,118,171,133,21,253,229,104,241,37,141,73,67,40,2,174,66,87,227,92,
  167,14,18,32,11,96,252,204,200,74,86,148,70,132,216,215,234,213,104,19,83,187,184,92,105,223,57,59,55,3,205,212,160,107,57,230,239,3,108,223,
  190,23,10,52,156,60,245,17,110,222,156,70,223,138,62,24,70,41,118,253,189,189,101,44,46,44,117,109,174,137,165,136,132,103,181,180,52,143,21,253,
  3,93,166,24,225,56,231,225,184,158,20,249,19,73,8,137,74,250,165,74,15,234,181,106,44,168,249,67,95,188,241,5,223,220,39,21,74,95,198,184,
  249,251,186,156,113,47,23,229,91,110,128,35,162,77,200,202,236,124,160,67,104,25,105,174,135,178,64,87,52,166,172,136,16,239,110,181,156,4,134,87,
  182,200,182,233,38,155,26,101,49,166,73,66,182,248,125,147,116,213,167,105,82,39,25,91,125,222,132,241,15,157,91,34,67,159,52,94,50,91,57,16,
  253,189,72,41,72,150,232,116,223,31,45,181,130,36,58,96,9,2,24,26,56,136,52,101,89,126,102,231,251,59,229,248,142,13,50,187,183,135,1,119,
  135,30,210,153,3,157,32,59,12,96,2,44,44,204,163,90,173,98,120,120,24,127,247,119,127,135,39,159,124,146,242,109,69,134,68,73,77,175,188,66,
  69,171,213,162,148,184,124,62,15,195,48,164,149,139,164,253,140,119,43,100,207,19,143,105,250,38,231,125,231,190,186,66,99,181,164,245,76,238,99,199,
  86,58,136,209,195,216,159,179,198,85,108,197,123,185,77,169,124,117,221,52,77,154,12,180,219,109,44,46,46,162,88,12,1,186,185,185,57,97,37,158,
  52,86,178,136,253,230,205,155,233,153,51,56,56,8,85,85,209,211,211,19,187,111,243,243,247,34,195,19,19,68,53,198,117,221,24,231,154,52,76,58,
  142,29,161,235,58,35,153,9,56,78,155,210,42,66,234,139,6,214,212,199,117,45,26,240,147,94,38,18,192,3,65,132,64,7,140,130,71,92,86,174,
  219,78,155,245,26,8,213,53,200,26,18,5,207,241,115,180,67,197,32,244,16,246,111,9,112,195,174,107,241,154,206,242,243,116,45,126,94,173,74,4,
  150,166,82,56,40,119,81,114,184,37,33,30,73,232,89,50,226,172,72,157,96,234,245,58,250,7,6,0,24,32,77,122,98,244,40,244,126,119,29,15,
  138,170,196,116,51,59,205,36,26,66,131,21,182,81,80,67,46,87,64,161,80,64,177,88,128,174,231,96,24,58,116,67,135,174,27,208,53,21,182,237,
  193,115,61,184,158,5,67,207,35,64,40,13,147,203,229,161,168,42,52,69,135,231,123,104,52,234,145,44,76,40,93,103,217,109,104,122,30,237,86,3,
  182,211,134,235,251,48,77,13,189,189,253,184,119,175,10,64,139,38,140,23,161,201,38,131,132,168,48,205,2,114,249,60,12,67,67,62,95,140,116,166,
  117,184,94,120,237,133,194,8,14,30,124,0,159,141,141,193,15,156,176,100,151,55,97,26,38,60,175,5,32,7,192,10,203,57,86,54,222,99,26,63,
  210,247,125,204,206,46,96,199,182,251,18,221,215,216,146,148,8,21,76,178,72,85,20,37,188,239,180,108,228,37,206,165,44,101,111,118,35,77,250,188,
  63,116,220,178,3,254,243,142,251,243,34,140,162,0,137,175,108,136,154,233,248,181,252,121,145,78,118,19,146,93,103,150,132,129,221,188,216,114,90,26,
  133,36,75,210,159,214,244,44,67,207,255,88,168,45,27,20,203,120,175,127,140,102,70,190,66,200,219,195,47,103,188,162,4,41,201,68,70,148,172,100,
  213,38,23,85,125,248,64,79,212,248,26,34,112,113,206,178,40,217,34,165,232,78,178,165,66,198,137,12,145,73,131,81,29,224,155,1,69,20,40,114,
  48,43,240,125,66,87,84,99,246,198,44,239,186,209,168,193,117,29,212,235,30,76,211,196,87,190,242,21,172,95,191,14,47,191,252,50,238,220,185,155,
  168,229,44,66,205,249,234,112,171,213,130,231,133,159,29,54,105,59,49,78,174,172,105,79,164,196,34,66,137,217,128,156,236,161,186,174,211,64,150,151,
  154,228,19,0,153,108,39,95,1,20,105,92,139,158,65,18,218,44,171,28,19,73,192,190,190,62,138,222,179,123,140,239,251,168,86,67,96,103,97,97,
  33,22,104,177,200,59,121,95,16,4,88,179,102,13,214,174,93,139,118,187,29,27,79,46,151,139,1,57,228,140,42,20,74,116,44,142,211,145,143,211,
  34,151,95,118,238,16,219,109,130,22,179,58,216,97,83,163,29,25,159,88,200,231,61,250,25,196,106,62,76,222,124,202,213,111,54,235,240,125,47,166,
  132,193,218,97,243,58,207,196,209,16,80,35,96,48,116,149,206,26,60,135,255,86,162,190,173,238,4,138,216,132,199,171,54,98,213,141,36,96,54,158,
  28,169,137,213,192,238,216,49,251,75,167,129,70,134,77,80,70,209,72,58,116,227,136,5,121,159,47,45,227,212,107,53,20,203,131,209,77,232,214,62,
  142,241,99,16,78,220,93,59,183,163,183,167,7,65,224,193,52,12,20,75,5,148,203,161,249,137,237,56,8,252,240,243,115,57,29,134,174,67,129,2,
  219,110,135,27,71,16,192,106,183,67,31,120,47,64,224,7,80,17,32,87,200,195,15,124,98,36,14,143,56,250,208,73,225,193,52,71,96,26,57,104,
  134,25,89,96,251,80,21,3,1,2,64,241,81,42,149,161,170,10,190,246,181,167,81,175,55,105,86,71,80,10,77,211,161,32,128,170,34,162,121,132,
  139,25,10,80,173,86,97,26,26,6,7,215,96,113,113,9,149,114,25,143,60,242,48,148,192,194,201,72,155,121,102,230,14,218,118,11,207,60,251,85,
  56,78,88,222,121,229,149,223,192,243,92,104,186,154,233,144,76,10,40,200,225,91,173,213,80,40,228,186,202,104,60,218,201,35,102,34,133,4,89,89,
  71,85,52,88,86,91,200,185,229,145,203,164,32,148,29,55,203,177,252,83,141,91,182,22,150,59,110,81,112,145,213,21,48,9,209,227,59,240,179,32,
  185,162,239,77,154,43,89,92,26,179,186,154,38,85,17,150,163,38,33,234,248,151,93,3,31,52,19,58,19,31,136,47,23,13,22,59,102,5,66,85,
  132,172,1,230,114,130,94,25,111,56,13,161,103,249,172,203,53,124,73,154,159,124,0,44,162,8,201,154,158,211,19,168,184,60,149,200,77,48,11,104,
  16,119,47,12,98,159,211,173,20,17,64,81,8,71,90,141,104,27,10,227,200,214,81,244,232,140,201,167,186,184,213,234,18,28,39,68,28,9,87,121,
  253,250,245,88,191,126,4,197,162,137,91,183,110,195,182,157,84,35,179,36,147,18,214,6,220,52,77,106,204,193,6,210,236,181,241,73,65,82,85,134,
  79,62,9,74,77,208,97,130,158,178,193,9,187,31,19,180,89,20,152,203,16,235,164,103,202,211,207,88,77,106,25,184,48,60,60,140,21,43,86,192,
  113,28,108,223,190,29,183,111,223,198,245,235,215,99,215,111,89,22,92,215,141,122,174,226,182,235,100,141,148,203,101,250,29,107,215,174,133,97,132,206,
  126,68,254,79,211,52,244,246,246,196,16,122,114,45,134,97,50,129,48,162,38,64,157,82,76,194,103,232,82,20,153,140,193,117,45,166,193,143,216,194,
  119,76,117,154,205,6,12,67,163,104,180,97,228,160,40,42,92,215,166,134,63,225,153,43,10,146,227,14,131,174,235,49,10,27,225,218,32,252,232,78,
  111,88,114,240,28,34,213,10,141,237,120,245,20,17,59,33,126,102,35,129,206,145,13,169,22,129,59,162,61,136,157,143,142,227,196,170,2,177,179,35,
  77,56,154,223,176,120,169,38,158,147,38,115,69,203,138,124,204,47,44,97,203,202,181,208,180,60,92,183,214,5,175,243,175,233,169,73,220,119,255,125,
  48,244,145,112,83,112,92,88,182,133,123,119,167,208,168,55,224,184,46,10,249,60,160,232,176,218,109,84,171,139,168,86,155,168,86,231,224,80,57,61,
  72,169,41,50,254,48,207,245,34,47,67,205,1,170,15,77,85,49,58,186,14,207,126,237,105,220,185,125,11,215,198,175,66,5,160,104,58,148,64,133,
  227,180,225,70,170,26,150,213,134,109,59,104,183,219,56,242,133,47,97,169,214,192,165,75,23,241,220,243,95,199,51,207,60,134,118,219,70,206,52,209,
  108,44,224,165,223,188,65,243,43,215,181,240,206,59,239,97,235,214,77,112,45,7,211,183,174,162,213,12,239,153,182,12,5,8,190,243,153,221,128,194,
  224,18,80,53,93,40,229,38,210,211,229,131,3,25,39,56,174,14,161,192,118,173,216,253,22,105,46,103,185,22,145,82,195,159,106,220,236,156,89,238,
  184,255,208,70,196,172,210,124,124,240,39,115,9,99,15,78,81,195,172,232,239,147,155,56,146,131,151,44,82,102,34,69,143,172,148,23,217,94,38,123,
  201,244,105,211,2,112,94,54,81,212,205,207,95,83,22,53,163,207,67,175,72,211,155,151,113,173,179,42,164,36,37,2,73,243,83,68,73,224,27,195,
  179,244,227,164,161,178,73,129,113,150,134,97,150,151,25,6,191,106,76,218,142,253,44,77,83,99,74,77,36,128,112,93,63,250,157,70,131,234,142,52,
  94,0,162,216,65,104,22,100,223,13,130,0,245,122,3,170,170,163,88,52,176,118,237,74,180,90,46,166,167,111,11,175,67,132,18,147,42,10,187,119,
  177,154,206,172,82,7,225,243,38,85,110,100,247,89,196,195,38,178,110,132,206,193,82,74,8,157,132,45,155,179,84,45,118,14,137,100,72,249,125,75,
  4,132,176,123,93,90,47,3,25,75,62,159,71,169,84,194,166,77,155,240,244,211,79,227,205,55,223,196,213,171,87,99,9,52,65,152,155,205,102,76,
  237,167,84,42,161,183,183,23,174,235,98,235,214,173,24,27,27,131,231,121,168,84,42,148,166,21,106,53,7,216,183,111,59,124,191,69,13,75,58,85,
  244,8,160,163,40,114,216,232,167,170,122,100,250,65,20,71,220,104,111,14,237,225,195,123,74,100,235,108,40,74,11,170,234,163,217,172,211,57,150,203,
  229,209,104,84,41,112,71,146,9,162,0,2,248,84,166,142,77,86,8,53,73,211,16,5,223,10,88,213,12,82,65,103,41,23,29,121,98,69,138,60,
  119,20,206,148,88,128,30,159,79,221,200,51,139,34,243,104,116,167,161,177,27,125,38,58,212,100,141,38,237,135,50,185,205,48,193,49,132,107,79,85,
  213,16,129,14,0,168,9,165,54,209,164,149,77,202,52,158,93,188,68,30,191,225,128,130,197,197,57,20,114,102,24,176,121,74,42,117,228,173,119,222,
  198,241,19,239,193,243,157,212,128,94,112,228,208,210,156,172,25,48,9,49,99,187,143,233,134,228,91,128,15,56,0,174,142,95,65,163,254,101,76,223,
  190,139,75,151,46,37,202,21,1,192,254,125,135,177,99,119,72,149,104,182,44,88,109,7,39,63,56,133,91,83,215,225,58,30,218,86,147,67,40,21,
  92,186,116,30,227,227,55,162,201,106,69,200,184,9,171,85,207,116,32,201,16,76,182,164,102,232,57,206,149,8,49,122,4,171,132,192,162,17,228,111,
  100,242,98,241,105,223,145,125,146,29,138,162,228,140,215,197,101,199,45,179,160,254,163,142,91,18,92,100,25,119,210,124,75,147,123,74,123,134,124,118,
  157,133,154,33,146,242,75,115,133,227,187,150,179,240,143,179,72,164,241,65,93,154,172,87,150,251,146,182,30,210,236,93,211,130,203,52,93,106,62,161,
  225,3,7,54,168,72,226,45,39,105,143,139,26,230,146,230,19,175,28,196,55,163,166,41,192,36,5,180,50,85,23,89,226,42,75,170,68,201,95,252,
  223,254,178,80,246,110,164,75,129,232,207,197,13,74,114,183,187,240,185,42,145,54,109,92,139,150,213,208,85,213,144,226,97,89,109,180,219,77,20,10,
  133,216,61,41,22,75,168,86,23,80,46,231,113,248,240,3,56,123,246,28,62,253,244,211,174,177,201,192,42,89,242,66,100,210,88,52,186,221,14,61,
  5,248,96,148,93,127,105,198,44,252,179,34,146,119,44,247,157,5,55,120,10,3,91,33,100,29,6,211,154,189,69,52,45,62,137,101,215,60,249,111,
  195,48,232,223,221,190,125,27,205,102,19,223,252,230,55,67,37,176,200,36,134,223,139,88,53,19,50,198,117,235,214,161,191,191,31,181,90,13,195,195,
  195,232,239,239,199,236,236,44,138,197,34,67,99,113,81,175,207,67,81,92,24,70,14,142,227,210,42,87,200,77,14,141,75,58,20,12,7,65,224,71,
  63,235,140,217,117,109,132,252,97,143,177,48,183,35,106,136,1,207,115,35,62,116,56,199,194,4,128,208,23,195,185,71,36,235,8,154,77,230,115,120,
  102,170,49,29,103,66,207,232,220,95,133,153,255,106,140,222,72,18,75,81,240,220,249,158,78,32,205,186,42,118,76,134,68,193,179,2,153,252,175,184,
  74,202,82,22,149,72,128,33,185,210,153,212,160,155,164,146,165,40,10,130,36,4,58,77,58,39,107,9,53,142,128,233,8,57,201,136,184,97,126,215,
  102,214,104,52,161,171,10,114,166,1,199,22,35,7,252,98,114,61,59,218,180,124,238,129,138,21,66,100,55,134,55,17,200,18,68,39,109,214,170,106,
  0,65,128,222,222,126,248,190,74,175,61,220,80,194,76,146,253,238,123,247,166,241,193,123,167,96,219,109,92,190,124,13,135,14,237,134,237,248,168,215,
  107,4,223,70,127,255,40,86,142,12,97,100,100,4,61,189,21,148,138,121,232,134,30,77,94,31,173,182,131,149,35,43,112,229,242,213,76,193,12,191,
  17,137,16,187,32,240,35,138,74,247,166,201,150,254,120,174,218,242,2,65,31,166,102,126,46,10,64,82,213,68,100,165,252,199,29,55,150,109,54,33,
  43,61,202,214,144,204,173,80,84,118,202,162,82,33,171,168,240,107,65,36,80,47,75,22,178,172,23,81,128,149,52,230,229,56,4,38,37,58,89,42,
  23,89,130,232,164,138,5,155,76,203,16,93,150,26,34,211,155,102,209,105,54,209,96,209,229,36,84,94,70,117,224,19,9,254,90,210,76,83,210,238,
  135,44,41,146,141,131,191,231,161,28,151,46,157,171,50,254,126,231,250,89,138,32,18,58,236,129,228,203,228,41,23,226,247,241,166,42,228,220,33,1,
  114,252,179,58,40,30,107,0,225,121,30,170,213,42,122,123,87,116,5,30,97,115,34,176,180,52,135,175,126,245,171,88,179,102,13,78,156,56,129,122,
  189,158,74,187,225,215,49,187,63,178,46,126,4,141,214,52,141,154,178,240,73,116,247,25,46,142,17,120,223,8,130,34,19,196,153,111,48,228,147,116,
  130,140,138,80,109,25,77,133,31,159,76,62,143,77,48,121,122,31,145,0,172,215,235,88,177,98,5,181,211,230,247,7,158,118,161,105,26,118,236,216,
  1,93,215,81,175,215,97,154,38,42,149,10,106,181,26,74,165,18,189,207,29,190,57,168,237,58,209,107,14,41,16,196,230,218,143,85,15,194,164,46,
  188,190,240,217,56,49,196,214,243,92,184,110,168,243,109,219,77,152,102,5,142,99,51,77,243,68,106,80,129,231,133,28,232,197,197,5,6,125,6,58,
  118,220,29,90,45,161,123,200,104,21,252,179,102,131,108,246,172,101,131,97,54,120,150,83,219,186,145,231,78,128,45,138,251,20,142,166,17,36,174,229,
  52,38,132,136,254,38,18,205,160,9,29,0,37,8,186,117,160,101,198,2,108,201,49,139,126,174,248,64,114,99,23,39,178,220,109,181,154,80,53,160,
  80,42,161,86,187,23,179,22,247,60,53,34,159,187,93,156,72,94,243,89,196,234,78,178,243,77,187,201,4,217,148,5,229,162,14,243,32,240,225,122,
  14,60,223,141,20,69,216,128,136,112,188,25,58,202,237,73,76,77,223,138,54,14,13,8,20,60,118,236,81,236,216,185,11,138,226,35,159,43,34,159,
  55,209,108,44,194,48,115,24,89,57,136,215,127,247,38,110,223,185,29,114,156,76,3,189,61,101,152,102,14,183,111,223,75,13,218,100,9,15,63,15,
  28,215,133,166,231,132,7,190,168,252,47,51,223,72,10,170,84,85,131,162,42,210,146,171,44,80,224,173,172,147,84,32,254,20,227,78,43,21,39,141,
  59,173,145,140,207,134,101,207,145,119,236,34,159,43,147,77,147,93,43,31,8,137,52,148,69,188,214,52,221,76,54,24,74,106,68,93,78,226,33,106,
  148,75,75,24,101,201,49,223,227,33,227,77,39,5,170,73,205,111,73,65,165,72,181,69,132,236,242,174,106,89,233,37,162,103,198,163,123,159,151,219,
  157,229,103,89,105,22,162,6,158,36,69,21,145,132,92,199,33,16,194,253,127,57,77,154,172,252,92,7,161,238,70,171,89,132,142,184,182,134,72,152,
  26,107,26,36,182,199,172,177,138,231,1,213,234,34,60,111,77,76,21,160,92,46,69,72,118,232,7,80,175,215,176,103,207,30,172,88,209,139,211,167,
  63,193,149,43,227,177,10,154,168,2,33,163,173,145,223,177,220,104,130,72,179,110,120,108,32,206,34,198,50,101,24,25,200,198,75,209,145,231,204,35,
  205,162,164,145,215,221,231,231,143,8,248,224,131,29,178,118,250,251,251,161,170,42,22,23,23,97,89,86,140,110,23,106,31,87,209,110,183,97,219,118,
  236,62,177,13,230,44,173,99,197,138,21,88,185,114,37,85,180,240,125,31,166,105,162,167,167,7,229,114,153,38,35,4,189,14,223,159,139,37,49,97,
  239,147,78,121,233,225,88,92,122,159,109,59,20,4,176,172,118,84,1,214,34,203,109,7,142,211,142,250,184,58,247,44,76,140,194,64,53,151,51,225,
  186,30,130,192,163,20,14,207,115,153,68,207,139,33,196,113,107,238,14,101,36,228,62,179,232,116,32,144,163,67,148,56,170,93,180,141,112,124,94,20,
  168,147,254,2,177,29,183,100,183,145,4,184,113,195,148,238,160,63,222,208,206,174,151,228,216,49,219,158,167,18,235,122,249,166,36,14,58,249,44,79,
  166,55,155,116,88,243,2,248,236,191,219,150,13,4,62,122,123,122,112,103,218,103,54,81,45,234,106,22,117,85,106,96,155,13,147,154,27,227,221,198,
  6,20,197,149,110,50,34,189,80,126,131,74,67,110,84,77,71,224,243,155,26,81,9,201,71,60,40,91,48,54,23,239,191,255,62,16,56,240,188,0,
  142,107,195,177,44,44,44,214,81,111,46,193,52,74,248,246,183,255,30,183,166,166,176,184,56,75,159,221,221,187,202,178,38,67,22,132,213,115,93,228,
  204,28,84,69,67,160,44,79,2,139,181,53,77,250,62,85,1,44,187,37,229,197,178,247,231,243,28,244,203,69,215,178,142,155,159,111,203,25,55,171,
  67,42,71,255,229,165,75,126,237,176,246,220,73,229,255,36,195,149,164,166,210,36,217,50,153,201,12,123,184,167,5,183,105,207,42,237,254,139,146,89,
  158,74,32,67,208,101,215,207,255,62,141,47,156,213,244,134,69,147,69,232,45,171,221,75,208,59,145,211,91,90,192,158,196,233,206,138,46,39,37,135,
  162,64,77,86,149,144,209,113,150,83,237,17,175,131,160,75,63,54,203,199,116,20,36,220,72,201,67,212,60,220,253,89,221,20,146,48,32,14,75,198,
  10,151,48,134,159,173,105,113,45,90,130,66,87,171,75,93,86,213,186,158,163,136,117,88,141,60,143,190,190,1,172,88,177,2,229,178,142,251,238,91,
  135,155,55,239,160,217,108,197,246,169,180,102,84,158,210,198,170,101,176,65,116,24,160,57,93,6,70,162,243,79,70,171,17,1,80,252,247,179,10,30,
  194,32,133,187,38,22,1,231,65,48,190,255,132,220,207,124,62,143,251,239,191,31,65,16,96,120,120,24,235,215,175,199,175,127,253,107,204,206,206,198,
  170,65,36,168,37,1,49,1,85,114,185,28,42,149,10,28,199,193,198,141,27,241,201,39,159,192,178,44,244,244,244,32,159,207,163,221,110,211,224,172,
  92,46,32,151,91,129,122,125,9,166,89,136,172,181,67,45,102,66,235,33,156,115,242,93,225,218,102,215,123,40,79,23,34,209,33,39,154,149,166,107,
  183,45,104,154,142,118,187,30,53,177,2,138,162,71,21,125,133,9,168,195,166,214,48,136,14,24,180,57,46,171,200,86,153,195,42,153,31,169,211,32,
  70,229,8,105,71,65,44,89,229,101,131,137,35,103,92,2,50,160,142,135,124,48,220,157,96,242,205,169,65,130,193,86,28,105,142,239,207,42,124,223,
  139,73,225,241,149,153,172,103,145,60,118,20,4,208,105,252,186,52,158,89,86,169,187,36,84,162,29,73,201,244,244,246,50,55,82,71,16,216,204,88,
  116,14,178,247,186,248,52,217,174,195,205,92,18,78,191,201,10,10,133,1,232,186,129,66,33,7,195,52,80,41,247,160,88,204,99,219,182,251,224,122,
  6,42,149,34,10,249,60,74,165,2,76,83,133,170,24,104,91,109,188,252,242,43,88,88,152,238,250,196,115,231,199,164,247,213,118,154,176,108,27,131,
  131,3,88,92,156,149,110,240,50,39,41,209,129,38,67,162,92,207,70,16,184,81,242,162,164,38,41,236,247,242,243,70,182,201,154,57,29,182,99,75,
  15,84,17,66,33,226,2,202,158,211,159,106,220,178,67,61,203,184,89,84,131,13,2,73,198,156,20,88,167,173,183,164,146,122,26,58,42,235,121,72,
  11,66,147,168,79,89,244,53,147,246,20,217,245,200,29,234,186,131,123,126,124,34,218,138,136,159,46,66,225,151,147,200,37,85,126,120,148,43,233,57,
  37,25,244,100,73,16,216,178,250,231,121,101,173,24,72,247,145,168,156,45,162,101,164,209,122,100,77,173,97,18,234,197,2,84,114,152,119,214,66,64,
  131,96,150,118,17,7,71,20,122,224,138,121,228,100,207,236,70,161,59,250,207,236,252,80,185,164,137,173,140,250,32,10,83,161,13,183,71,131,239,16,
  48,82,105,176,98,89,109,76,79,223,128,109,183,177,98,197,0,76,179,128,66,193,197,230,205,163,152,153,89,194,157,59,179,210,121,205,86,167,146,2,
  107,18,4,18,52,58,159,207,67,211,52,161,150,179,104,62,166,205,39,118,79,17,125,30,235,32,200,7,198,178,134,101,25,128,197,174,17,194,243,94,
  88,88,192,206,157,59,241,245,175,127,29,182,109,227,157,119,222,193,236,236,108,108,95,35,136,56,59,14,32,116,35,28,30,30,198,210,210,18,214,175,
  95,143,213,171,87,99,124,124,28,229,114,153,2,32,97,243,103,21,170,106,193,52,21,180,219,173,72,219,25,20,89,38,146,136,113,25,59,214,89,144,
  84,6,44,178,138,58,21,96,199,137,146,60,29,186,174,82,253,229,48,232,37,156,229,142,218,70,120,239,188,40,200,246,187,212,53,186,101,229,2,38,
  232,6,149,189,83,148,14,114,28,174,61,130,116,7,156,52,163,200,24,165,243,249,172,124,166,184,9,208,23,8,20,32,115,240,156,132,100,147,103,154,
  4,228,136,164,94,179,130,64,122,214,141,95,116,208,136,116,86,179,148,3,147,208,172,32,218,200,216,133,161,235,62,136,208,56,123,211,194,14,82,159,
  110,134,89,26,7,9,97,62,169,105,68,166,55,202,35,235,157,224,175,140,39,159,120,26,35,195,3,244,125,109,171,9,207,245,112,103,250,46,0,31,
  35,131,121,180,173,22,150,22,23,49,121,107,1,173,102,27,67,67,67,216,255,224,3,200,153,218,178,15,173,32,240,209,108,54,80,46,229,187,80,83,
  190,145,137,13,94,120,202,131,232,16,227,75,204,154,170,67,213,180,88,202,146,36,229,182,220,131,184,88,236,135,169,107,88,90,172,10,3,184,44,10,
  1,162,113,39,241,138,255,24,227,78,11,60,179,240,105,249,191,227,203,234,60,135,49,75,64,203,127,102,22,45,228,52,235,210,44,107,155,71,151,210,
  146,208,44,9,184,200,70,91,180,169,241,206,100,162,102,190,52,155,111,17,189,70,54,191,178,74,12,202,130,125,190,244,158,150,144,240,124,60,17,146,
  151,54,103,69,7,64,82,112,149,213,214,157,15,128,101,110,174,34,20,136,239,81,144,161,222,73,247,186,115,86,176,210,91,97,144,26,231,36,39,53,
  13,249,96,53,155,227,191,239,166,132,116,208,83,159,209,195,141,151,180,9,234,167,40,126,228,50,23,196,16,109,223,7,26,141,26,108,219,65,62,159,
  135,239,251,104,52,234,152,154,186,25,161,106,160,159,111,219,14,108,219,66,79,79,47,26,141,26,116,93,197,170,85,43,80,42,21,49,57,121,7,237,
  182,37,92,179,252,61,22,81,146,200,153,97,89,22,60,207,163,220,104,66,233,96,185,209,108,128,193,170,21,201,230,163,40,118,72,106,20,149,41,23,
  137,232,151,236,30,79,254,158,88,152,147,160,205,178,44,76,78,78,98,255,254,253,148,115,206,7,201,44,55,155,53,68,1,128,141,27,55,162,191,191,
  31,115,115,115,232,233,233,193,202,149,43,49,62,62,78,27,5,201,190,83,175,215,16,4,62,116,221,132,162,104,177,248,33,236,119,10,171,206,4,241,
  247,60,39,114,12,204,115,210,116,97,99,97,187,221,70,79,79,41,106,54,108,83,121,59,85,13,168,113,9,65,140,243,121,3,182,109,193,247,93,154,
  48,18,119,224,14,69,195,231,16,95,116,105,51,135,186,210,97,111,22,153,211,154,22,127,102,252,60,239,60,83,8,131,103,217,249,218,249,157,223,5,
  34,36,189,143,77,54,196,193,115,92,175,93,102,16,39,2,73,68,123,81,119,236,168,164,7,208,188,14,170,76,176,125,185,141,62,108,55,40,31,76,
  211,207,80,0,223,243,96,24,57,42,123,34,223,240,61,169,151,121,86,36,61,43,156,207,150,155,8,143,139,140,219,177,219,88,179,122,0,103,207,126,
  134,177,177,51,8,2,3,173,246,146,112,236,236,235,218,196,77,236,220,181,7,1,220,174,44,45,139,14,111,224,43,48,204,92,106,128,71,22,167,200,
  230,52,173,211,95,211,52,152,102,62,42,35,117,63,179,229,52,107,201,14,227,129,193,33,184,142,139,102,219,74,164,47,164,5,174,252,184,101,137,219,
  31,107,220,50,68,120,185,227,230,37,205,248,195,100,57,220,108,17,170,147,69,89,70,118,63,196,182,197,233,201,69,146,245,240,231,161,212,164,34,1,
  28,146,207,222,87,25,50,154,180,222,178,82,91,100,115,68,212,104,149,134,72,39,169,183,164,53,117,243,123,118,86,90,134,200,229,142,7,17,146,18,
  35,145,210,205,114,100,230,68,159,153,118,255,146,117,156,145,201,73,140,85,128,234,128,14,50,9,86,128,40,54,133,220,228,78,9,156,69,158,137,238,
  51,111,5,78,130,123,223,87,160,170,29,89,187,80,202,174,141,169,169,91,88,189,122,45,108,219,198,228,228,117,204,207,223,163,1,54,41,157,55,26,
  85,184,110,104,209,220,249,44,160,183,55,143,114,121,3,102,102,150,48,61,125,55,246,12,88,87,224,52,201,72,122,150,57,14,229,243,234,186,78,29,
  12,73,32,45,146,149,77,147,11,77,74,236,69,78,112,162,102,91,153,54,56,139,74,147,36,132,204,31,34,157,7,132,46,130,213,106,21,205,102,51,
  166,166,193,170,127,176,40,116,120,238,153,184,239,190,251,168,122,137,109,219,52,177,32,238,132,228,251,93,151,156,93,26,109,16,101,247,160,48,102,80,
  41,53,198,182,29,26,19,145,239,13,17,127,55,242,172,200,195,117,93,88,86,11,182,221,164,70,61,182,29,82,58,195,164,38,12,102,13,195,132,227,
  184,17,50,29,208,192,148,55,71,33,241,151,231,5,20,141,238,56,2,134,61,89,196,163,130,183,230,38,193,43,191,62,249,224,57,172,6,169,93,65,
  176,136,182,193,178,6,178,4,207,241,121,16,196,80,111,82,101,226,63,79,6,182,137,0,148,52,199,102,17,0,160,243,155,96,22,5,138,229,152,44,
  112,151,144,82,250,211,232,69,153,134,6,214,185,144,200,173,164,73,44,37,241,233,146,74,160,34,116,92,244,29,162,67,42,8,92,44,44,44,193,113,
  85,52,154,245,216,248,202,149,65,108,223,186,19,133,98,17,231,47,124,134,187,119,174,51,165,249,240,90,138,133,10,128,123,137,154,170,162,3,206,204,
  153,240,28,47,182,9,241,178,110,252,198,73,238,159,40,152,148,171,137,68,139,47,144,151,206,200,247,101,185,199,252,243,43,151,74,168,215,151,150,133,
  248,126,158,191,251,99,143,59,139,145,208,114,62,135,71,98,210,174,37,237,123,120,94,32,153,27,252,26,103,29,195,248,3,46,171,137,65,150,235,206,
  186,79,176,9,95,82,98,196,38,30,73,115,34,109,220,98,132,36,144,162,184,188,13,119,214,121,153,244,188,101,65,56,143,236,203,94,178,224,89,198,
  91,102,181,190,217,53,144,165,9,53,237,154,120,245,144,172,21,156,52,180,92,22,72,145,131,180,51,126,166,158,153,16,100,119,7,251,226,38,65,145,
  202,7,65,147,195,239,37,231,82,7,1,239,80,176,72,224,77,2,134,78,48,29,254,189,143,241,241,43,152,159,159,195,230,205,247,195,113,92,120,94,
  232,26,215,105,222,2,229,37,107,154,65,131,127,223,87,161,170,97,16,63,60,220,139,66,33,135,241,241,91,210,245,32,211,194,231,239,5,225,231,250,
  190,15,195,48,40,96,68,248,209,124,224,204,7,36,172,71,66,146,55,68,154,106,142,168,161,154,221,151,72,149,201,52,77,12,15,15,195,48,12,52,
  26,13,44,44,44,80,41,61,242,183,237,118,27,142,227,208,235,146,33,140,61,61,61,244,255,11,133,2,6,6,6,168,129,138,227,56,40,20,10,24,
  30,30,68,79,79,49,22,136,135,65,105,168,180,193,54,252,134,235,204,134,162,228,57,222,112,39,176,38,99,245,125,15,142,99,33,8,0,203,114,34,
  128,208,141,212,53,64,19,28,194,115,38,115,113,105,105,9,149,74,153,81,208,224,249,206,157,216,139,165,114,144,4,45,252,108,133,171,160,4,140,167,
  130,152,130,17,6,236,62,253,12,62,136,21,239,175,221,156,231,44,70,41,113,148,153,239,75,97,155,2,197,2,18,124,99,108,26,160,145,53,118,84,
  121,52,33,9,141,145,29,56,73,93,240,172,188,147,104,176,113,117,139,232,193,210,102,14,63,134,138,119,38,171,39,44,11,137,28,191,88,75,209,44,
  101,232,238,69,173,36,44,112,114,221,33,63,168,92,234,141,141,33,159,239,197,115,223,252,107,236,216,177,17,155,54,13,227,169,175,60,70,55,216,112,
  241,235,80,84,21,174,103,167,160,37,158,224,190,171,200,231,243,176,156,118,12,113,224,209,125,118,33,139,38,68,150,23,225,105,5,136,187,135,201,184,
  166,73,22,215,162,57,48,60,178,18,11,243,115,248,83,190,254,20,227,94,14,114,157,53,33,229,5,237,249,223,201,214,146,200,34,150,56,125,241,235,
  133,253,76,86,82,234,243,162,194,108,82,38,162,166,176,142,143,105,104,44,207,65,79,91,227,228,251,249,127,100,72,46,63,110,153,170,0,249,185,8,
  149,96,19,46,145,67,107,22,4,84,134,220,241,193,181,40,240,73,42,73,166,85,22,120,201,51,217,158,65,246,217,44,107,128,231,146,203,20,82,210,
  84,68,88,141,224,164,196,74,116,246,16,116,140,157,107,108,192,203,3,57,73,141,111,228,204,16,37,104,113,125,231,142,2,8,171,8,209,161,54,116,
  2,153,56,23,149,13,238,3,84,171,75,184,125,251,22,26,141,26,202,229,82,116,158,116,108,151,201,163,245,125,31,197,98,169,139,34,66,232,30,189,
  189,5,236,218,117,31,250,250,122,132,251,190,236,25,200,232,50,142,227,196,236,173,21,69,161,178,119,50,58,6,191,111,177,192,28,75,179,226,247,51,
  246,185,165,37,110,132,43,110,24,6,114,185,28,118,236,216,129,117,235,214,97,203,150,45,248,111,255,237,191,97,207,158,61,212,252,132,221,231,72,18,
  194,203,232,17,202,71,171,213,194,206,157,59,209,223,223,79,221,5,13,195,160,227,182,109,27,150,181,132,181,107,7,208,104,132,170,29,157,190,162,48,
  144,211,117,35,66,98,221,232,154,109,120,158,31,147,180,11,145,252,112,93,217,182,27,27,35,145,184,211,117,194,79,111,50,72,113,8,50,230,114,121,
  106,106,2,40,48,12,21,141,70,45,102,130,66,212,55,216,224,217,243,124,248,190,71,29,1,67,69,179,128,73,8,187,131,231,80,82,209,235,210,239,
  230,131,242,240,59,59,234,51,252,126,214,121,79,246,224,153,87,252,224,231,42,25,95,92,84,66,17,50,42,254,84,177,163,202,7,207,73,101,104,209,
  65,207,6,201,113,39,27,143,126,105,146,147,27,143,156,40,138,9,215,117,97,24,38,52,173,12,223,87,104,119,116,252,128,84,160,40,121,41,95,42,
  73,45,160,195,67,138,31,16,34,253,71,49,106,174,1,48,162,67,168,2,64,135,109,219,40,20,226,82,83,251,247,239,67,177,96,224,63,254,227,231,
  248,221,111,95,195,64,127,31,122,123,135,58,159,239,217,81,215,173,150,26,160,240,135,169,162,24,161,126,167,29,8,239,43,123,144,203,68,247,179,52,
  96,1,161,125,185,239,187,9,147,91,204,173,227,155,22,121,46,17,121,95,33,175,97,102,102,230,15,10,72,147,14,133,63,213,184,147,16,204,207,19,
  68,179,255,240,207,71,68,71,224,185,214,34,105,53,194,9,148,5,224,196,49,76,198,131,205,34,17,151,180,55,44,135,95,206,238,29,162,96,60,235,
  115,103,19,162,164,234,10,171,116,193,202,106,101,9,230,120,158,244,114,232,64,73,156,117,118,157,139,208,219,63,132,250,194,38,217,228,123,216,253,143,
  189,103,162,100,36,109,143,149,81,80,200,189,149,237,57,188,164,94,146,41,144,172,145,52,252,71,75,164,197,116,127,175,42,72,50,252,232,108,17,73,
  53,146,160,213,231,56,159,29,67,9,242,119,164,233,138,85,33,224,27,183,88,201,45,215,245,208,108,54,81,40,148,163,18,120,16,157,11,33,74,77,
  202,246,249,124,1,154,166,211,160,138,32,137,190,239,69,200,116,128,245,235,71,48,58,58,210,53,71,69,212,203,180,234,153,231,121,104,183,219,104,183,
  219,244,44,34,122,198,162,231,33,147,92,228,3,99,54,57,17,249,48,136,18,83,246,51,201,56,20,69,193,248,248,56,42,149,10,158,122,234,41,12,
  13,13,161,191,191,63,70,93,97,169,41,132,138,194,190,86,174,92,137,190,190,62,44,46,46,162,82,169,96,237,218,181,80,20,133,82,66,200,94,208,
  110,55,209,106,213,34,218,68,46,230,240,24,6,197,62,85,189,32,218,230,228,246,20,10,133,152,6,115,24,84,251,177,253,38,156,7,14,124,31,168,
  215,23,193,54,253,117,170,27,74,52,63,188,72,225,195,67,171,213,132,101,181,40,253,35,188,87,44,213,1,244,187,0,242,126,63,22,96,19,84,155,
  141,229,20,69,229,128,132,120,144,74,2,120,162,103,205,234,84,119,239,7,106,87,240,220,217,131,3,33,24,193,106,77,179,201,111,103,47,209,164,128,
  35,187,183,39,129,43,252,252,90,110,236,168,179,95,194,31,202,203,145,21,226,51,250,36,249,175,100,94,160,7,171,213,70,177,148,131,231,145,9,104,
  64,81,124,152,102,17,249,124,25,57,211,68,0,15,174,235,192,117,43,112,28,15,158,215,134,162,42,48,205,60,84,69,131,235,58,209,98,214,225,186,
  109,4,80,17,248,30,52,205,140,38,134,66,121,111,154,166,67,85,13,248,126,168,15,169,106,70,20,88,248,40,21,75,48,12,3,133,98,15,114,185,
  28,116,77,135,170,41,40,22,202,208,117,21,133,66,1,142,99,163,84,52,96,217,46,76,179,15,174,219,198,250,245,91,112,224,192,94,156,254,120,12,
  205,214,18,138,197,60,20,85,193,170,85,171,17,0,104,54,106,240,131,0,10,84,168,138,218,165,194,144,86,226,46,22,243,208,117,21,173,118,163,43,
  112,144,85,13,88,27,210,229,56,212,57,118,216,160,162,42,10,124,166,124,23,95,108,226,198,173,142,112,124,208,117,88,134,243,206,68,185,210,139,90,
  163,145,41,72,102,231,25,203,149,147,85,63,120,132,228,143,55,110,173,155,191,191,204,113,39,89,133,39,113,184,100,37,88,17,178,206,203,232,101,213,
  240,253,188,1,154,44,224,19,53,3,138,14,212,52,30,26,47,107,149,197,188,35,169,100,45,115,88,228,93,253,210,104,108,34,154,69,86,115,27,114,
  128,18,185,58,158,126,245,121,104,48,105,52,27,30,53,230,63,159,79,30,210,2,233,52,141,231,229,140,51,75,165,80,182,118,178,58,110,138,18,84,
  246,255,179,208,232,8,114,215,153,179,1,60,47,228,73,135,86,222,138,128,227,75,84,66,66,62,116,168,104,16,150,207,45,171,141,85,171,86,83,212,
  147,84,26,201,103,207,206,222,131,174,107,93,170,10,225,58,32,30,9,225,247,143,140,244,162,183,183,132,235,215,167,209,230,122,75,68,247,66,180,86,
  217,113,19,116,148,80,20,72,112,73,42,92,73,103,9,47,45,199,174,91,226,30,43,227,251,243,52,17,226,36,72,238,57,105,12,92,88,88,0,0,
  212,106,53,74,211,16,37,245,29,148,183,51,166,3,7,14,196,100,253,250,250,250,98,1,116,167,193,207,163,65,35,161,94,144,189,44,108,188,211,104,
  176,78,62,207,182,45,218,132,202,34,184,150,213,140,57,225,182,219,45,120,158,69,63,51,164,115,216,204,216,1,93,55,34,42,137,69,165,232,58,6,
  40,36,129,243,35,110,180,18,75,240,194,57,199,211,58,216,132,46,206,79,22,105,44,119,108,184,125,218,204,215,209,135,150,7,207,34,228,153,255,157,
  8,44,224,155,30,89,234,70,210,145,155,100,34,37,219,199,121,31,0,126,15,96,207,79,85,85,169,123,183,78,38,48,153,60,108,9,106,185,27,179,
  204,70,54,75,105,145,53,91,169,213,107,24,28,94,139,29,59,118,96,203,125,91,80,46,150,96,152,74,212,64,1,32,240,105,176,141,64,69,0,13,
  174,103,195,48,244,24,96,28,68,153,79,16,217,95,170,154,138,192,15,98,217,146,70,126,22,177,53,124,215,135,162,170,240,3,7,158,235,193,247,21,
  216,86,3,182,227,194,243,124,184,142,13,223,247,81,171,214,209,108,54,224,57,54,60,95,195,154,181,235,176,229,190,245,248,187,191,123,30,173,150,133,
  225,161,94,76,140,223,192,241,119,223,65,16,0,243,11,11,184,114,229,42,142,29,59,2,199,241,241,206,219,39,112,243,230,68,100,30,162,119,101,67,
  162,131,63,126,104,249,80,21,5,110,196,73,19,105,196,242,247,58,73,210,40,233,229,56,54,116,61,7,195,200,193,178,219,194,195,132,87,247,224,3,
  29,185,51,84,14,134,110,192,182,236,68,83,8,217,129,159,5,113,227,199,245,199,24,183,40,104,250,60,227,78,210,74,23,233,176,38,5,10,178,3,
  140,13,204,146,214,239,231,81,150,200,82,17,16,141,45,201,77,79,244,115,145,170,0,155,24,102,161,160,200,212,75,68,193,31,43,188,159,85,165,34,
  233,250,211,238,169,40,0,207,162,36,242,135,38,60,34,74,132,104,63,207,162,55,45,58,136,216,96,56,77,199,93,68,219,73,106,140,22,207,85,133,
  202,98,49,167,0,163,30,208,141,162,177,146,118,36,216,16,217,123,147,3,61,12,84,89,53,2,133,186,200,37,169,21,196,29,9,59,223,65,190,187,
  213,106,65,211,52,170,13,220,161,154,132,239,153,153,185,141,165,165,121,106,4,194,63,27,223,87,98,28,208,98,209,192,150,45,107,48,63,95,195,244,
  244,61,169,141,187,72,28,64,228,241,16,82,14,194,160,142,80,57,72,48,203,87,115,101,138,32,124,211,168,204,14,156,149,92,36,127,83,46,151,169,
  78,51,49,60,33,113,203,236,236,44,102,102,102,80,169,84,232,223,240,223,73,2,91,214,160,135,40,107,16,135,71,207,243,144,207,135,149,237,145,145,
  97,174,186,238,83,26,133,174,155,116,111,15,41,30,106,23,234,205,86,120,154,205,22,122,122,74,49,106,141,227,120,112,156,22,12,163,133,32,112,81,
  171,45,208,121,166,235,225,253,181,44,135,242,139,73,69,177,213,170,50,129,103,16,125,183,7,214,110,155,252,142,13,192,201,124,20,169,100,240,84,45,
  210,124,24,167,58,128,86,84,58,50,115,30,60,175,123,238,196,3,228,180,192,26,84,45,39,62,71,187,131,231,110,58,71,60,200,230,229,16,147,250,
  120,210,250,51,18,247,240,232,134,234,50,116,129,69,43,179,34,200,188,83,81,247,251,114,8,2,43,245,96,126,239,189,15,176,126,253,109,12,15,148,
  97,106,62,174,94,185,140,106,117,17,245,250,34,234,205,86,216,193,26,17,214,115,57,3,133,98,17,126,100,87,233,56,30,92,215,134,26,233,105,122,
  190,13,5,132,91,165,192,115,3,128,49,3,241,188,0,174,227,68,63,83,224,123,97,118,229,120,86,172,115,149,31,235,224,208,22,12,15,246,195,246,
  21,92,189,116,22,154,161,96,229,170,1,188,119,252,109,148,202,101,188,255,254,2,110,221,186,3,128,52,124,216,120,233,165,223,160,167,28,146,253,235,
  205,6,138,197,10,13,134,73,230,153,21,173,209,52,35,234,235,147,55,127,166,73,7,101,157,44,174,235,64,211,84,152,134,73,3,104,190,228,33,58,
  144,179,28,246,134,89,128,174,169,180,188,37,67,196,68,25,98,90,167,118,90,208,242,135,140,59,45,176,204,50,238,44,193,152,108,44,172,78,116,154,
  118,174,44,136,149,5,79,73,65,138,40,73,206,130,106,203,44,121,147,52,200,147,180,186,249,36,40,45,192,90,110,99,100,86,181,158,164,103,39,226,
  22,243,141,157,105,250,205,89,231,91,82,208,153,148,60,37,37,76,50,57,200,164,103,159,102,138,179,220,170,38,95,61,147,161,218,241,70,54,181,43,
  0,232,252,187,83,78,238,172,205,128,6,23,172,25,74,7,45,230,3,137,142,18,71,40,245,21,80,250,72,124,62,40,49,189,92,182,81,177,115,166,
  116,116,161,155,205,58,92,215,69,161,80,198,210,210,18,66,45,224,206,117,133,232,105,199,209,150,168,48,136,155,6,195,127,231,114,58,86,174,236,71,
  169,84,194,141,27,83,52,240,76,171,216,240,107,149,47,125,19,36,151,80,196,68,148,32,17,120,192,247,155,200,230,3,235,174,90,40,20,176,122,245,
  106,232,186,142,197,197,69,204,206,206,82,69,12,242,249,237,118,59,102,163,205,163,139,68,73,164,80,40,96,205,154,53,20,177,238,235,235,163,252,100,
  34,123,167,170,62,182,109,219,8,207,107,161,94,175,130,104,50,43,138,6,93,55,34,170,68,156,2,214,106,53,163,196,34,31,171,226,176,60,235,78,
  19,119,232,91,65,80,235,208,33,208,231,26,31,67,117,141,14,186,140,72,102,208,133,101,217,12,237,34,128,170,250,12,101,131,160,221,1,88,89,185,
  32,240,99,1,173,231,197,157,5,69,84,137,56,117,169,19,172,18,147,150,80,133,70,235,162,236,117,20,58,178,5,207,29,234,148,214,197,137,78,14,
  158,65,17,112,217,25,150,212,156,42,59,143,210,204,162,216,247,233,73,101,107,182,155,56,139,136,190,104,83,139,71,252,237,76,27,234,252,226,2,230,
  22,230,25,91,84,81,102,18,142,177,209,2,176,184,152,41,24,228,55,254,206,4,143,7,201,220,163,237,250,220,222,222,17,252,213,95,126,5,185,156,
  14,207,11,240,243,122,11,229,66,15,108,7,184,50,62,78,27,4,68,175,106,189,163,210,209,108,214,67,109,207,222,222,88,166,45,158,112,113,218,5,
  233,234,134,79,50,245,34,130,160,45,77,108,146,154,60,69,147,133,15,62,20,85,129,162,106,169,135,160,40,104,79,162,4,229,115,57,248,129,131,122,
  163,158,24,24,240,227,23,105,228,202,2,130,44,136,150,104,220,155,54,110,196,225,67,135,80,174,84,160,68,232,203,59,199,143,99,124,98,34,54,134,
  207,59,110,209,193,18,4,1,214,172,94,141,103,159,121,6,181,90,13,63,122,225,133,196,224,76,211,52,60,251,204,51,88,61,58,138,227,39,78,224,
  194,197,139,194,205,100,251,182,109,24,28,28,196,241,19,39,0,0,71,143,28,193,246,109,219,112,225,226,69,188,253,206,59,210,123,39,210,198,148,53,
  34,145,251,240,173,231,158,67,79,79,15,29,207,114,156,15,249,191,45,151,203,216,183,119,47,38,38,38,112,107,114,178,75,113,70,36,199,152,132,164,
  179,74,1,73,180,151,44,232,39,203,153,150,25,239,4,28,229,73,198,191,75,163,197,241,247,62,233,190,253,161,234,48,73,136,180,168,57,72,164,182,
  144,54,94,25,149,39,43,210,156,164,238,32,171,124,136,92,5,59,73,152,34,216,247,149,4,74,16,162,192,87,97,154,176,148,88,224,193,6,209,68,
  222,43,92,179,42,213,219,101,131,232,118,187,133,118,187,141,222,222,62,220,185,51,29,157,35,26,216,102,173,78,176,223,81,168,10,3,50,133,169,66,
  6,93,103,90,79,79,14,247,223,191,14,147,147,247,176,184,88,21,206,49,222,128,69,86,45,34,247,159,117,202,36,212,10,214,128,69,166,141,207,83,
  40,146,170,68,166,105,226,208,161,67,0,128,254,254,126,28,62,124,24,47,188,240,2,206,156,57,19,59,187,194,6,63,139,42,136,240,115,170,84,42,
  209,102,192,125,251,246,225,234,213,171,104,181,90,40,151,203,52,128,86,85,21,141,70,3,182,221,68,169,100,64,211,20,56,142,7,93,239,104,139,135,
  115,93,165,10,20,29,106,136,23,169,103,88,48,140,34,211,136,27,2,99,134,145,163,212,61,203,178,168,13,183,109,55,97,154,69,216,118,59,122,182,
  1,28,199,69,185,92,132,231,185,148,95,76,232,39,97,99,100,71,197,37,110,105,221,225,19,147,6,83,66,255,225,255,166,131,70,7,93,149,155,142,
  68,96,16,163,121,116,26,104,125,240,174,130,113,29,101,80,221,106,81,245,83,212,48,24,159,99,217,130,103,217,154,151,129,60,50,106,71,18,160,34,
  162,53,177,251,143,46,42,121,243,246,192,203,49,67,88,46,31,45,249,189,129,208,62,177,59,216,85,169,101,42,217,192,58,168,129,129,208,113,48,0,
  96,2,112,132,220,158,112,66,232,0,28,233,166,77,190,223,113,218,176,219,22,12,67,197,229,203,215,177,48,63,139,251,159,122,20,154,110,96,116,116,
  43,150,150,102,145,207,23,144,203,21,225,184,173,144,114,162,104,17,167,41,128,97,228,161,107,6,42,61,101,228,114,57,218,53,157,116,191,58,19,179,
  147,253,41,138,2,63,240,145,47,244,225,216,99,79,224,248,241,183,80,173,206,64,211,75,33,221,162,189,192,5,82,221,137,66,22,29,99,85,51,66,
  177,247,192,237,10,64,68,135,119,26,194,199,78,70,35,151,139,14,82,63,211,65,46,162,12,201,198,45,90,48,89,199,221,211,211,131,199,143,29,227,
  208,114,19,143,31,59,134,31,254,248,199,168,213,106,153,230,115,214,113,179,77,99,147,83,83,248,231,239,126,55,53,217,228,27,132,100,165,169,237,219,
  182,225,200,35,143,96,114,106,74,248,247,203,69,112,249,242,175,44,1,79,10,4,101,129,20,225,2,147,239,250,234,51,207,160,167,167,7,19,19,19,
  93,141,102,159,39,240,146,205,13,81,121,155,191,239,162,185,147,53,104,204,34,125,151,69,217,133,93,127,89,154,193,72,160,35,162,204,44,55,160,150,
  57,54,242,202,49,50,46,188,8,125,79,163,242,200,170,12,188,227,102,218,62,147,20,192,19,119,181,142,133,177,88,214,142,231,11,179,42,77,113,105,
  59,190,58,161,68,65,115,39,48,225,131,224,16,97,180,209,108,54,98,82,113,29,215,194,32,70,247,136,27,96,116,246,117,215,117,35,106,28,225,67,
  119,238,181,105,170,216,188,121,53,102,102,74,152,154,186,75,247,92,182,210,192,186,23,38,157,129,44,213,137,237,161,34,241,2,143,70,39,237,27,34,
  186,7,219,240,117,229,202,21,28,61,122,20,15,63,252,48,10,133,2,250,250,250,186,230,11,145,168,235,4,153,113,149,158,117,235,214,209,6,176,158,
  158,30,12,14,14,226,250,245,235,232,237,237,141,81,86,108,219,130,231,57,209,88,58,13,216,161,170,135,13,199,177,160,40,26,109,186,235,110,242,139,
  39,246,68,150,46,212,240,54,34,109,106,82,113,38,13,133,46,76,211,64,179,25,38,72,225,189,212,35,84,214,231,168,24,113,42,5,27,15,177,129,
  105,7,253,21,27,154,196,171,42,241,181,210,145,73,140,7,232,225,223,121,84,105,35,57,222,11,50,7,207,162,223,167,5,207,89,246,41,113,236,216,
  189,71,240,160,150,204,245,86,148,232,233,60,2,192,31,12,73,156,145,172,13,31,127,200,75,222,36,22,118,127,134,130,244,129,48,176,14,111,132,197,
  92,139,35,12,168,58,65,142,13,210,129,45,202,76,200,245,55,26,53,252,248,133,31,225,190,251,54,163,90,109,162,94,159,197,47,126,241,51,28,60,
  240,32,190,250,236,23,65,154,82,131,192,141,178,48,3,166,153,131,170,169,180,193,65,215,116,216,142,133,197,249,5,180,154,77,136,124,226,217,251,186,
  122,244,62,172,90,185,30,189,125,69,188,255,254,137,72,39,82,129,162,26,216,189,123,43,238,187,111,53,74,165,39,241,201,199,151,240,208,195,251,145,
  203,235,120,233,55,47,225,206,157,155,81,201,210,192,227,143,63,141,83,167,78,96,100,120,45,86,244,245,226,189,15,222,64,161,208,23,201,219,52,133,
  28,198,14,143,216,67,62,87,70,111,239,42,56,78,11,119,238,220,16,114,132,9,103,47,151,235,131,101,205,75,17,87,154,176,105,42,212,72,98,58,
  20,168,239,69,179,185,36,156,11,102,174,7,15,60,112,8,239,191,247,26,246,239,255,2,238,220,190,14,199,85,96,219,45,84,171,243,169,7,42,143,
  244,202,2,219,32,8,48,56,48,64,223,243,131,127,253,87,216,182,29,91,136,149,74,5,127,243,252,243,0,128,151,94,126,25,207,62,243,12,108,219,
  198,107,175,191,142,169,233,208,150,125,223,222,189,56,116,240,32,253,158,177,179,103,113,242,212,169,8,145,48,240,232,209,163,216,188,105,19,229,63,30,
  63,113,2,19,215,175,99,245,234,213,120,246,233,167,81,175,215,41,2,125,232,224,65,236,219,187,151,142,97,106,122,26,175,189,254,58,45,201,202,174,
  119,205,234,213,56,122,228,8,20,69,193,234,209,81,252,215,127,252,71,124,231,123,223,139,253,253,246,109,219,40,247,242,204,88,104,31,191,121,211,38,
  60,241,248,227,244,249,85,171,85,188,246,198,27,152,155,155,163,232,245,241,19,39,98,101,212,51,99,99,93,170,59,155,54,110,196,227,199,142,33,8,
  2,188,241,230,155,152,156,154,194,163,71,143,98,195,250,245,244,192,39,247,140,172,47,114,240,254,217,215,190,70,117,89,159,125,230,25,124,112,242,36,
  206,126,250,41,6,6,6,240,240,67,15,97,213,202,149,80,85,21,227,19,19,56,121,234,20,106,181,90,102,187,106,81,19,159,168,209,146,13,22,179,
  244,119,240,243,137,253,121,86,93,101,209,123,217,96,85,164,21,206,87,144,216,32,153,63,68,216,128,33,169,223,37,107,149,32,201,132,69,102,222,194,
  74,45,166,233,92,39,29,128,105,227,231,65,18,17,10,45,10,208,121,137,45,214,237,182,115,205,74,151,181,55,65,17,187,155,80,21,6,4,9,40,
  50,221,65,244,84,184,174,131,137,137,171,104,54,235,12,23,59,142,116,171,106,64,173,190,137,13,114,39,169,68,204,253,142,52,43,134,110,189,29,243,
  177,161,161,30,20,139,121,76,77,205,160,86,171,119,221,71,17,143,93,134,226,177,202,4,68,14,142,172,63,150,163,204,39,117,50,20,144,4,206,108,
  133,103,122,122,26,243,243,243,104,181,90,152,159,159,167,138,32,252,89,207,58,38,178,207,127,237,218,181,24,25,25,161,252,109,98,243,173,105,26,10,
  133,2,29,99,167,169,79,163,177,7,75,45,116,93,47,218,39,84,10,238,177,73,152,166,169,48,12,51,150,140,132,14,129,64,173,214,64,185,172,195,
  243,28,212,106,179,76,229,74,167,244,29,66,195,8,145,105,11,249,124,14,142,227,50,177,87,135,154,193,234,41,119,230,148,156,178,193,6,174,172,134,
  180,184,199,135,56,120,242,178,141,132,54,226,73,246,81,196,170,57,34,183,66,50,247,9,218,45,122,255,31,18,60,103,139,29,209,37,103,39,171,218,
  139,76,162,216,61,76,231,35,238,180,146,187,8,26,79,43,195,49,151,132,144,236,222,93,254,147,149,224,249,50,40,123,131,53,173,147,29,137,204,49,
  100,180,15,62,25,96,185,111,162,204,184,187,161,194,67,163,89,199,153,177,177,104,18,4,152,157,155,195,43,175,254,14,165,82,9,186,102,68,214,171,
  109,40,0,84,77,199,232,232,26,60,243,204,83,248,249,79,127,134,193,193,149,248,226,23,31,198,210,98,29,175,190,250,58,90,237,5,40,138,9,32,
  90,224,90,1,57,179,130,118,123,17,134,153,7,2,13,35,43,71,240,133,71,118,192,247,3,228,114,6,102,103,231,160,40,1,138,197,10,214,174,89,
  5,0,24,30,90,129,193,129,94,232,154,139,158,74,25,79,60,113,12,243,11,13,20,242,57,156,61,251,25,86,141,172,192,3,15,60,136,90,221,194,
  222,125,91,48,51,59,135,47,125,233,97,120,190,143,183,222,122,51,108,106,20,53,109,121,225,51,24,28,26,197,145,35,7,81,40,228,240,191,191,251,
  191,209,178,26,177,103,54,48,176,26,229,114,17,142,227,227,225,135,15,226,181,215,222,192,198,141,155,177,184,88,197,141,27,231,133,200,79,161,96,128,
  232,125,239,127,240,75,120,96,239,22,252,240,135,63,193,138,254,149,240,220,54,154,45,23,27,55,109,132,105,42,152,95,168,98,239,158,45,208,85,21,
  219,119,109,193,233,143,92,236,220,179,31,134,230,227,95,126,240,47,176,29,59,179,75,159,104,174,177,8,217,248,196,4,102,103,103,49,56,56,136,255,
  244,15,255,0,0,56,127,225,2,174,93,187,70,3,100,242,122,226,241,199,105,185,241,209,163,71,241,163,23,94,160,193,243,153,177,49,156,60,117,10,
  71,143,28,193,190,189,123,97,89,22,206,140,141,225,137,199,31,199,154,213,171,233,103,20,10,5,60,241,248,227,248,209,11,47,116,41,89,110,218,184,
  17,251,246,238,197,212,244,52,94,122,249,101,236,222,181,11,15,63,244,16,118,108,223,78,3,94,81,48,192,55,134,136,238,205,230,77,155,176,125,219,
  54,186,233,255,255,217,123,243,47,61,170,43,91,112,199,240,13,57,75,169,212,152,18,66,201,36,129,65,18,32,129,109,68,149,11,176,171,129,194,216,
  213,85,148,253,220,174,122,221,110,175,254,173,95,175,254,99,122,189,238,234,85,245,220,30,203,207,6,215,51,178,203,64,185,202,96,151,145,24,148,2,
  36,49,40,37,144,50,53,165,166,252,50,243,27,35,162,127,136,239,70,220,123,226,220,33,82,114,25,153,151,107,105,33,82,153,241,221,136,184,195,62,
  251,236,179,207,149,43,87,48,119,230,12,30,123,244,81,180,59,29,124,247,123,223,195,216,216,24,190,252,244,211,120,96,239,94,236,255,217,207,10,178,
  144,76,158,180,176,144,201,91,68,144,241,208,103,63,139,36,73,240,252,254,253,152,59,115,6,187,118,238,196,212,182,109,120,227,205,55,113,104,122,26,
  95,253,171,191,194,35,127,242,39,248,251,111,125,75,1,223,114,39,47,153,121,174,86,171,120,234,201,39,51,224,46,158,79,173,86,195,243,251,247,103,
  44,118,89,173,173,201,165,132,235,166,101,146,136,152,24,94,157,246,89,215,37,147,142,207,198,92,155,58,140,138,119,108,235,78,201,117,234,210,237,241,
  92,3,22,177,150,76,18,149,149,144,46,92,86,193,204,22,241,25,75,25,68,167,251,190,236,89,77,159,67,146,105,170,115,157,52,15,162,243,177,120,
  133,192,75,150,115,208,159,145,65,242,165,75,243,153,107,67,222,193,80,0,103,209,60,37,150,218,69,171,90,208,244,221,123,133,226,69,217,49,33,142,
  99,12,13,85,113,235,173,155,113,241,226,2,78,157,58,99,180,76,148,11,93,41,168,166,158,242,66,23,45,187,101,8,239,101,157,245,161,184,142,0,
  172,66,47,220,237,118,179,235,207,206,206,226,234,213,171,138,37,29,13,56,197,153,34,186,4,138,175,13,27,54,100,215,19,115,184,82,169,160,94,175,
  161,94,175,100,140,127,14,170,68,32,20,42,125,44,68,33,159,108,147,38,222,113,175,151,146,111,189,94,140,32,128,162,171,78,179,206,169,207,116,24,
  138,172,80,212,31,71,208,15,54,124,73,11,236,161,219,109,246,139,5,19,50,103,101,236,228,49,224,217,236,178,33,246,128,180,16,48,102,246,17,57,
  99,150,43,1,168,204,163,136,139,60,229,103,117,204,179,120,190,194,181,65,213,75,187,131,231,107,195,142,40,116,101,54,99,71,94,246,90,208,64,187,
  26,172,115,105,53,219,129,149,94,187,91,218,207,213,180,209,211,198,47,166,52,44,221,68,229,131,90,84,147,154,90,147,218,10,103,196,203,92,146,236,
  216,178,195,167,215,193,135,31,157,192,114,179,133,118,183,139,225,209,17,12,141,12,97,120,116,24,123,31,220,131,183,14,191,143,221,247,222,141,203,151,
  47,97,126,254,10,62,251,217,251,81,175,215,112,246,204,37,172,223,48,134,36,246,208,110,247,0,15,240,124,160,86,247,241,233,207,236,236,107,187,6,
  241,254,251,239,99,235,214,77,240,60,224,131,153,119,209,238,117,241,71,251,238,197,196,196,42,44,53,22,1,68,216,247,208,125,120,255,253,15,241,169,
  187,239,192,242,114,23,126,224,97,185,121,21,221,118,11,19,235,215,96,195,186,113,124,244,209,9,118,3,109,183,155,8,195,16,87,175,92,198,249,243,
  23,177,109,219,70,236,218,121,47,174,46,54,241,222,123,239,193,243,61,36,49,48,60,178,26,127,242,185,189,248,183,223,30,198,186,181,171,112,235,45,
  119,96,223,195,247,33,142,98,124,251,255,59,135,203,87,47,22,0,193,64,125,0,189,110,140,237,119,62,136,135,62,123,15,58,205,101,236,216,177,19,
  123,31,188,27,73,20,3,158,223,7,217,192,107,175,30,69,156,248,184,121,235,58,84,252,4,189,174,135,106,53,192,208,192,32,182,110,217,140,15,78,
  156,32,82,155,46,219,146,222,198,118,137,255,254,232,217,103,177,237,230,155,177,106,213,42,76,109,219,134,59,119,236,192,157,59,118,224,249,253,251,177,
  208,104,100,191,243,194,139,47,98,161,209,192,87,158,121,6,35,35,35,24,31,31,199,212,182,109,25,11,45,152,99,0,152,156,156,196,209,99,199,48,
  185,105,83,6,44,101,64,238,121,30,198,164,110,88,0,48,115,226,4,126,252,220,115,216,60,57,137,61,123,246,96,231,221,119,103,0,213,38,31,153,
  157,155,195,203,175,188,130,135,247,237,195,220,153,51,248,233,243,207,43,247,126,97,126,30,63,125,254,121,252,249,151,190,132,137,137,9,172,90,181,10,
  39,63,252,16,127,255,173,111,97,199,246,237,216,177,125,59,118,237,76,231,218,232,200,136,178,41,181,59,29,252,95,255,249,63,227,207,191,244,37,172,
  91,183,174,48,30,113,223,11,11,11,56,61,59,11,223,79,51,48,226,223,238,221,189,27,23,46,92,80,116,219,242,154,127,126,255,126,60,243,23,127,
  129,177,177,177,236,57,109,187,249,102,84,171,85,52,155,77,252,224,135,63,68,173,86,195,87,158,121,6,147,155,54,97,104,104,8,141,254,123,185,30,
  110,21,38,13,191,218,0,74,47,27,208,237,73,156,172,130,202,231,138,169,214,68,145,184,152,230,50,7,44,109,90,106,155,29,162,238,58,186,61,220,
  37,152,229,200,27,238,26,186,243,197,12,196,19,37,179,151,31,220,244,48,71,129,109,86,221,52,212,74,127,29,136,22,96,60,237,62,24,64,245,122,
  46,142,77,164,194,115,16,157,40,100,143,144,149,8,38,58,157,35,113,223,2,47,146,200,35,100,186,216,28,252,65,3,142,114,150,210,247,19,172,91,
  55,138,106,53,196,135,31,158,97,11,12,185,238,133,130,52,147,201,51,85,6,209,201,228,28,178,46,90,102,136,229,235,142,140,140,96,235,214,173,104,
  54,155,56,119,238,28,154,205,102,6,76,197,252,88,92,92,196,210,210,82,6,174,57,167,164,40,138,176,184,184,136,219,110,187,13,135,15,31,206,10,
  243,7,7,7,21,205,54,144,96,253,250,113,84,171,61,44,45,93,197,234,213,19,217,53,211,172,182,167,72,65,115,175,248,46,162,40,198,192,192,160,
  2,32,211,127,235,41,255,21,207,33,5,202,9,90,173,6,234,245,85,253,174,130,113,54,207,122,189,168,223,125,178,167,120,34,55,155,203,146,68,39,
  7,245,98,110,164,86,114,62,243,110,19,197,161,35,15,8,98,9,4,171,191,67,125,206,229,78,157,156,4,164,216,89,208,147,130,205,226,122,205,29,
  59,18,38,56,67,102,137,167,7,207,9,9,136,175,29,59,202,125,74,236,216,17,90,249,88,40,71,149,98,83,23,81,160,43,139,227,90,177,174,219,
  220,229,72,130,254,155,154,150,68,166,83,163,12,137,75,11,114,29,69,79,15,14,154,206,165,44,152,46,141,168,179,114,17,147,160,177,112,5,155,55,
  111,198,135,39,223,199,217,219,182,97,120,100,16,119,223,189,29,183,222,122,43,144,116,240,169,187,182,33,1,208,105,117,17,39,9,238,184,99,18,189,
  78,15,97,53,125,23,151,46,94,197,220,220,25,108,218,180,57,187,234,234,213,99,120,229,149,95,226,142,219,111,3,226,24,205,102,15,187,119,223,5,
  207,247,208,105,117,81,31,24,68,165,82,193,225,233,195,88,106,38,168,132,62,46,93,60,131,109,83,219,208,105,183,208,238,111,50,221,110,208,215,118,
  69,204,179,234,34,138,34,236,219,247,16,198,198,134,16,245,18,60,240,153,251,208,235,197,216,189,251,30,248,129,135,168,23,227,204,185,11,24,29,29,
  193,190,135,246,34,8,3,172,94,61,150,62,67,63,215,226,21,39,186,135,90,125,0,113,183,129,159,63,255,115,236,219,247,16,86,141,141,224,228,241,
  227,232,244,18,172,95,55,134,179,167,219,240,130,30,14,29,250,13,90,157,203,56,113,252,61,124,102,223,31,193,247,98,116,90,45,212,43,30,54,108,
  88,151,1,104,57,133,99,178,79,179,165,173,147,36,193,137,147,39,51,121,194,223,124,253,235,168,86,171,88,187,118,173,2,160,217,5,220,255,239,155,
  135,14,225,224,107,175,101,7,142,231,121,89,3,0,0,232,116,187,198,226,206,36,73,112,231,142,29,216,247,208,67,104,52,26,56,122,236,24,94,249,
  245,175,177,239,161,135,74,131,64,14,36,45,44,44,192,243,60,180,59,185,231,232,200,200,8,190,242,204,51,0,128,3,7,15,226,197,151,94,194,19,
  143,63,94,64,29,87,174,92,201,126,87,7,20,186,221,46,70,71,71,177,107,231,78,28,126,235,45,188,253,206,59,56,122,236,88,22,88,172,93,187,
  22,19,19,19,56,119,254,60,46,95,190,204,22,86,186,104,203,197,247,76,93,84,57,80,74,247,17,78,54,33,55,55,145,193,131,203,222,168,107,251,
  94,198,229,67,254,123,25,79,101,157,19,138,13,212,114,4,130,142,101,55,129,88,23,150,219,198,246,208,160,132,22,83,83,157,35,215,6,158,151,161,
  128,5,153,178,27,129,72,87,251,126,88,208,67,171,50,12,149,137,22,154,101,225,141,43,143,87,20,15,170,32,93,72,4,82,86,81,200,1,100,183,
  132,220,75,56,253,125,1,150,85,221,109,2,225,24,33,103,92,85,246,25,89,7,58,1,178,198,198,6,112,215,93,83,56,117,234,60,46,94,188,204,
  2,29,142,165,163,239,158,214,116,80,214,89,128,95,25,124,111,217,178,5,187,118,237,194,194,194,2,166,166,166,48,48,48,128,191,253,219,191,205,116,
  204,226,217,45,47,47,163,221,110,107,221,87,134,134,134,48,56,56,136,197,197,69,108,218,180,9,119,221,117,23,166,167,167,179,86,223,114,0,218,108,
  46,161,213,90,64,181,26,160,94,31,84,2,135,20,116,123,125,201,141,202,82,182,219,77,248,190,215,119,226,136,179,103,159,50,216,81,246,220,243,32,
  34,70,175,215,205,36,21,65,16,64,52,71,20,115,38,12,43,8,2,191,15,172,19,69,251,46,92,94,4,224,150,1,109,202,88,219,37,27,185,237,
  28,109,250,3,201,67,58,86,174,151,207,253,184,176,54,41,214,201,187,48,70,25,118,204,51,185,54,144,91,108,176,162,99,158,101,71,28,46,3,102,
  199,142,170,3,210,245,192,142,158,231,33,228,210,127,52,157,233,202,230,200,15,92,183,137,233,40,119,25,213,235,34,136,116,34,37,198,77,92,222,216,
  244,105,61,125,195,17,250,82,228,67,153,163,252,229,135,207,117,96,146,191,206,157,191,138,155,110,218,138,247,222,155,193,137,147,231,48,62,62,138,59,
  238,24,70,37,136,209,104,244,48,48,144,224,200,219,71,176,102,98,28,139,11,75,184,120,233,28,54,111,217,138,51,115,115,120,255,131,15,144,196,17,
  46,92,156,199,238,93,247,225,222,221,59,49,50,58,130,213,171,87,165,242,130,35,71,113,203,212,54,116,58,75,184,52,127,14,87,231,99,28,154,126,
  27,81,210,67,224,121,56,126,242,4,170,181,65,92,189,60,135,185,179,103,176,123,215,110,52,151,23,49,61,125,8,3,245,251,112,203,212,22,188,251,
  222,59,88,90,94,44,76,172,78,167,139,110,55,194,166,201,241,148,13,104,44,97,126,254,60,170,213,65,140,143,143,244,179,143,9,26,87,47,225,234,
  149,171,120,237,224,27,184,247,190,221,72,146,110,22,125,246,34,126,162,54,22,23,17,69,93,188,251,222,219,72,146,4,67,35,99,152,57,254,62,46,
  95,185,148,189,39,249,93,191,122,224,223,224,251,62,94,122,225,37,52,91,75,120,235,157,105,12,14,212,82,107,67,77,170,151,171,216,215,53,172,17,
  239,110,215,206,157,105,245,55,147,170,188,112,225,130,242,59,2,92,166,198,249,29,92,184,112,1,51,51,51,88,59,49,129,221,187,118,97,247,174,93,
  217,24,132,51,197,236,220,28,54,109,220,136,47,125,241,139,202,250,248,233,243,207,23,230,143,144,73,28,159,153,193,161,233,105,60,250,200,35,165,51,
  57,169,142,158,215,64,211,53,42,164,37,243,243,243,56,52,61,141,123,250,140,119,246,236,12,237,157,229,148,231,129,131,7,209,110,183,177,239,161,135,
  112,239,238,221,56,246,238,187,216,115,255,253,184,235,206,59,113,228,232,81,252,253,183,190,149,57,136,232,108,230,228,103,124,224,224,65,28,57,122,20,
  157,78,7,181,90,45,147,214,0,192,236,220,92,198,62,155,192,164,201,1,192,38,155,144,215,185,220,14,216,212,188,136,22,83,149,37,27,104,229,183,
  203,53,116,197,95,212,233,135,107,52,225,218,104,167,108,163,45,46,104,113,9,254,232,57,164,3,118,252,245,116,126,178,238,129,143,10,240,19,226,15,
  77,61,104,211,198,92,81,148,147,61,42,203,204,59,170,8,111,233,28,156,11,240,228,33,8,242,98,65,193,132,11,157,117,110,115,151,235,162,131,64,
  45,10,3,226,140,116,146,117,209,162,110,40,45,112,75,127,230,166,155,214,97,205,154,85,120,255,253,147,172,140,135,211,171,115,247,36,255,155,144,77,
  8,89,135,92,100,88,169,84,176,184,184,136,229,229,101,236,222,189,27,91,183,110,69,167,211,193,240,240,48,46,93,186,164,204,115,225,178,49,48,48,
  80,88,195,158,231,97,203,150,45,24,28,28,196,210,210,18,90,173,22,182,110,221,138,233,233,105,84,171,85,84,171,85,69,139,29,69,61,4,65,216,
  255,111,165,32,219,76,65,96,200,180,115,247,20,144,156,55,115,73,173,229,132,85,94,165,18,35,138,122,104,181,174,74,133,126,249,187,205,1,109,170,
  171,30,28,12,178,2,84,145,241,16,46,27,156,28,67,176,189,169,203,70,172,20,224,83,87,142,156,8,240,180,222,219,98,62,229,12,55,80,52,99,
  160,69,136,194,62,46,201,228,27,42,72,245,180,22,139,156,222,153,130,114,185,203,32,215,108,165,60,118,76,10,254,227,60,118,68,169,174,187,161,220,
  56,133,130,14,155,237,25,199,144,216,144,188,60,88,225,167,169,99,0,229,69,171,59,208,104,58,129,191,247,0,64,148,189,24,90,60,163,110,208,126,
  54,121,220,210,133,182,166,51,121,58,110,124,124,28,213,90,186,129,132,126,39,107,182,242,47,191,124,25,183,223,190,13,75,141,37,28,125,255,40,146,
  216,135,231,247,65,200,107,175,21,222,199,244,225,55,49,125,248,77,60,242,39,127,138,109,83,55,35,8,6,113,228,232,97,156,59,127,6,221,110,3,
  63,249,199,103,217,119,214,110,45,97,246,204,50,60,207,199,155,135,222,0,0,188,251,238,17,188,247,222,81,163,13,86,90,129,28,225,204,233,89,188,
  115,228,29,156,63,119,1,23,46,205,99,205,154,245,24,28,24,130,7,224,226,197,11,88,106,54,48,52,56,130,165,229,6,230,206,204,97,97,97,1,
  39,63,60,129,173,91,54,161,219,105,177,204,214,98,163,129,56,246,16,6,1,122,81,132,55,223,60,32,49,48,124,240,5,0,203,173,180,248,165,219,
  235,226,202,66,199,120,224,235,188,33,69,26,135,147,121,76,31,62,140,90,173,134,93,59,119,102,223,107,52,26,120,245,192,1,156,158,157,205,228,10,
  9,144,73,36,162,40,194,63,255,242,151,8,130,0,211,135,15,99,116,116,20,59,182,111,207,238,245,87,47,191,140,99,239,190,155,253,253,129,189,123,
  51,169,71,167,211,193,171,7,14,96,238,204,25,76,110,218,164,172,19,33,249,16,172,173,208,61,215,250,14,38,58,150,80,176,115,2,92,138,49,203,
  250,97,238,247,143,207,204,96,106,106,42,3,220,50,104,165,45,121,77,140,107,187,221,206,216,230,145,145,17,236,218,185,19,175,30,56,128,129,129,129,
  76,14,131,254,243,187,116,233,18,171,15,62,114,244,40,30,232,219,88,85,171,85,116,58,29,252,116,255,126,60,176,119,47,38,55,109,2,0,124,112,
  252,56,14,246,215,137,11,48,90,41,243,64,179,27,98,14,149,5,150,220,191,115,251,17,205,134,113,86,151,54,198,214,100,203,68,25,38,93,177,141,
  137,53,231,186,99,218,14,29,234,12,98,107,98,160,251,55,49,54,170,135,229,157,81,80,0,190,242,247,138,210,27,16,215,11,59,123,157,19,67,114,
  74,92,118,209,80,237,237,4,123,151,126,70,144,117,36,76,146,188,251,96,175,151,126,86,250,172,114,16,45,174,77,189,170,5,24,18,173,192,5,48,
  42,234,162,145,157,191,41,248,6,60,47,198,232,104,13,119,222,57,133,15,63,60,131,197,197,101,107,166,129,179,255,164,243,160,211,233,100,114,186,74,
  165,146,101,183,147,36,193,213,171,87,241,198,27,111,224,238,187,239,198,197,139,23,209,108,54,81,175,215,11,160,70,72,64,84,29,111,58,143,134,135,
  135,49,53,53,149,53,115,105,54,155,217,239,215,235,117,212,106,53,116,187,221,108,173,246,122,49,104,61,148,24,119,90,244,23,65,118,213,80,11,147,
  83,203,186,90,205,203,116,213,130,232,17,197,127,73,226,33,138,218,104,183,91,210,92,79,181,214,169,252,76,204,161,212,101,163,221,94,66,175,151,130,
  238,244,29,122,144,93,54,100,239,111,78,110,161,98,170,34,216,164,65,28,149,125,228,178,14,100,115,71,124,158,124,237,60,219,35,62,147,127,223,178,
  199,52,191,166,249,238,130,98,221,169,224,185,24,200,113,193,62,183,143,81,236,168,107,146,230,18,212,115,154,107,0,240,254,207,255,244,159,18,241,203,
  255,229,219,223,54,120,37,219,55,104,187,125,89,94,17,204,185,37,216,111,72,239,215,44,87,119,186,164,113,109,108,7,119,159,92,133,63,55,49,185,
  3,41,138,124,124,243,127,253,95,112,117,97,17,255,248,143,63,69,167,211,148,36,19,73,255,239,49,194,202,32,226,40,66,156,116,16,248,181,190,165,
  78,130,32,168,161,215,107,35,73,186,153,28,226,11,127,250,69,172,25,31,193,119,190,243,29,168,149,214,249,215,208,208,122,108,217,178,5,151,46,93,
  196,185,115,39,72,81,64,17,100,235,210,183,79,60,241,101,120,232,224,249,253,207,179,207,203,118,255,34,157,23,199,49,42,149,60,234,175,84,198,240,
  213,175,60,131,159,60,247,15,184,178,112,69,219,170,219,198,110,153,210,206,166,246,202,43,29,247,232,232,104,38,115,248,238,247,191,143,197,197,197,210,
  227,54,221,15,101,191,76,96,79,215,173,210,181,184,215,4,184,76,21,243,166,205,141,3,77,54,176,170,107,75,110,115,103,48,249,113,203,64,144,219,
  56,175,213,65,200,229,25,187,48,27,186,251,21,254,180,174,236,173,110,46,216,246,66,93,219,122,249,217,209,251,164,207,93,151,253,224,136,25,23,155,
  40,147,235,137,204,58,185,28,134,197,115,193,163,138,36,205,30,144,48,114,20,79,210,84,23,11,17,5,40,73,221,12,4,83,167,118,29,164,254,211,
  42,187,14,229,231,83,102,217,207,154,192,228,183,230,33,142,123,253,54,222,126,159,116,240,178,154,30,1,68,114,135,14,244,217,110,89,166,38,252,168,
  197,152,3,248,126,106,235,119,254,252,85,156,57,115,62,3,191,178,92,198,36,83,210,237,187,2,200,136,189,81,128,218,36,73,240,196,19,79,96,106,
  106,42,173,61,249,209,143,112,230,204,25,5,188,213,235,117,60,245,212,83,88,183,110,29,126,243,155,223,224,141,55,222,200,246,154,181,107,215,226,201,
  39,159,204,156,56,42,149,10,150,150,150,240,252,243,63,197,214,173,55,225,222,123,239,67,28,39,153,188,160,211,105,225,220,185,147,232,245,186,88,179,
  102,11,6,7,135,165,185,222,195,185,115,39,17,199,61,140,143,111,65,181,90,203,124,173,195,16,56,127,254,67,4,65,29,171,86,109,200,214,103,187,
  189,140,165,165,121,196,113,130,129,129,145,126,195,184,30,22,23,115,82,32,12,171,168,84,134,81,173,6,152,159,63,7,213,219,59,150,124,153,211,119,
  154,75,113,18,73,39,156,100,77,85,114,249,69,194,202,156,196,181,105,131,19,85,170,33,55,227,129,161,21,55,175,75,230,27,165,120,198,140,150,174,
  88,80,15,176,249,241,148,199,142,176,58,39,153,36,131,242,189,252,245,215,190,166,50,208,38,6,88,151,18,215,121,125,154,0,142,231,229,166,240,46,
  222,179,101,83,157,98,51,115,121,248,84,231,104,178,152,50,49,48,186,195,170,168,229,140,176,255,103,63,197,67,159,125,8,95,251,218,87,208,110,71,
  232,116,59,128,231,163,90,73,55,165,74,24,100,128,32,138,19,36,17,16,11,59,35,36,64,12,68,113,148,86,33,39,192,214,173,235,208,92,106,227,
  11,95,120,10,157,78,15,157,78,55,91,80,81,18,195,247,18,220,113,251,45,88,189,122,16,157,78,15,39,78,206,33,138,128,78,167,133,229,165,69,
  4,97,29,113,212,129,39,210,208,158,15,15,126,26,133,71,29,196,49,208,139,99,248,94,5,67,195,195,152,24,31,198,189,247,125,14,205,102,3,30,
  60,192,139,81,173,12,192,243,195,126,32,16,161,18,14,33,138,58,232,70,77,196,81,140,90,109,4,72,98,68,113,23,81,47,70,47,234,2,73,136,
  78,183,1,223,15,81,175,15,99,96,32,68,37,244,181,86,73,166,244,47,109,168,162,215,160,155,55,118,221,252,21,155,61,125,247,217,162,179,100,79,
  92,198,205,177,117,52,211,83,214,211,88,102,13,228,241,211,122,7,219,166,67,53,111,148,101,53,5,209,92,250,204,165,35,166,206,37,197,69,90,96,
  10,72,116,41,63,202,182,150,241,72,118,217,175,108,133,112,220,62,82,246,89,184,178,183,69,249,0,172,107,134,54,129,113,125,15,220,33,103,106,232,
  164,211,234,155,14,75,87,38,73,118,210,160,242,11,157,195,70,238,46,64,247,138,132,128,221,220,166,78,118,243,200,139,167,114,249,97,24,6,132,160,
  82,217,58,185,112,81,104,168,213,22,228,113,223,223,217,203,164,27,121,134,14,146,52,67,0,111,143,0,147,162,87,181,28,4,8,11,60,223,7,54,
  108,88,133,241,241,81,188,255,254,135,104,181,218,108,155,119,219,123,165,103,175,108,11,39,223,243,210,210,82,166,123,230,252,163,5,88,141,162,8,183,
  222,122,43,142,29,59,134,229,229,229,204,117,67,232,142,5,48,79,146,24,219,183,223,140,32,240,177,188,188,132,74,165,166,20,5,114,153,205,98,75,
  235,188,253,55,0,44,47,47,41,99,169,84,42,253,206,133,185,221,91,179,185,136,193,193,154,34,55,77,125,149,211,255,182,219,157,126,176,165,222,159,
  12,140,133,223,178,108,123,24,199,17,211,142,30,100,158,37,138,182,157,15,224,85,246,58,39,0,18,163,252,47,191,166,169,31,135,103,193,91,242,58,
  79,152,174,131,158,1,59,122,134,224,56,177,98,71,23,60,224,50,127,179,49,247,199,230,76,147,81,173,47,213,155,112,172,80,49,165,214,211,50,193,
  92,247,187,34,219,211,179,178,61,28,147,170,219,92,117,7,71,58,62,97,185,231,14,222,205,126,164,1,230,230,230,240,15,63,252,7,108,92,191,17,
  107,214,140,99,124,124,2,81,175,135,86,187,13,192,195,197,139,23,208,237,116,224,7,62,60,132,240,17,195,11,61,116,187,109,132,65,21,190,23,32,
  172,132,24,91,189,10,129,31,162,185,116,5,131,3,21,132,126,23,67,163,85,120,65,165,127,223,233,159,53,107,86,163,62,80,239,223,84,140,122,8,
  248,21,15,222,240,0,214,220,126,51,58,173,22,150,150,151,82,214,180,155,166,158,252,192,67,146,140,32,240,83,6,37,189,94,128,177,85,35,168,213,
  171,216,189,115,10,157,118,23,240,61,120,73,140,56,73,208,237,117,128,196,135,7,15,189,40,66,224,167,126,152,9,98,37,77,236,121,49,170,181,1,
  248,158,15,223,75,16,3,168,132,1,186,157,197,108,51,164,105,99,221,130,209,61,111,217,29,193,110,246,78,211,95,238,45,193,27,141,6,254,239,191,
  253,91,101,62,95,203,184,185,159,211,49,235,182,241,201,1,159,174,217,137,105,99,17,12,58,181,61,51,69,250,98,172,242,245,57,35,123,25,144,115,
  96,75,103,45,100,98,122,77,62,194,114,17,23,125,174,69,39,158,196,40,147,208,1,96,219,123,113,5,180,186,123,163,215,149,51,28,156,87,61,231,
  235,239,2,252,229,122,3,151,57,171,179,56,229,158,151,238,189,234,100,90,166,192,81,231,254,228,162,201,214,129,104,10,198,101,87,20,254,51,100,41,
  69,238,92,144,7,103,169,99,6,32,251,51,199,5,231,131,92,118,145,107,94,5,112,162,114,13,81,220,38,152,72,85,142,164,234,162,115,246,26,138,
  222,153,178,238,226,251,162,48,49,205,110,230,207,165,86,243,49,53,53,137,43,87,22,49,55,119,222,106,109,75,65,13,7,172,185,96,44,117,172,136,
  217,249,35,100,118,65,16,96,97,97,1,235,215,175,199,167,62,245,41,28,60,120,16,73,146,96,108,108,172,224,198,209,108,54,80,169,164,146,204,180,
  51,97,69,1,153,242,94,32,239,219,189,94,36,157,67,49,194,48,223,191,210,66,191,28,232,165,69,141,62,150,150,46,247,175,19,163,82,73,253,165,
  155,205,182,36,147,200,179,8,173,86,75,98,141,99,201,103,57,81,216,100,33,53,17,224,60,13,0,35,169,125,183,79,152,230,152,101,112,213,250,9,
  53,128,18,99,162,65,21,253,189,60,72,67,22,212,113,172,178,142,0,144,93,54,210,113,115,186,242,124,61,242,216,209,188,7,155,176,35,245,249,191,
  46,216,17,121,235,61,237,224,108,237,54,185,84,158,73,172,109,102,8,60,227,134,45,31,176,166,168,196,181,235,158,252,61,94,35,27,145,201,152,24,
  31,60,231,159,172,62,175,60,242,62,123,254,44,206,158,63,123,77,169,99,238,221,80,143,206,193,129,65,252,217,159,253,25,38,55,79,226,133,95,188,
  136,15,102,222,239,255,78,29,79,63,253,151,56,118,228,117,188,125,228,176,118,98,201,193,208,45,83,219,241,224,222,221,248,209,143,159,69,167,215,177,
  78,194,116,83,230,217,179,48,168,33,137,187,72,60,32,232,239,16,221,40,178,182,32,230,220,91,232,207,201,0,104,37,204,157,12,184,116,93,230,184,
  214,214,215,50,110,29,115,71,253,140,57,169,144,173,109,53,55,70,91,122,95,14,148,77,64,71,60,7,213,34,170,152,177,209,125,182,45,208,224,230,
  183,240,153,213,109,164,52,35,193,201,76,56,66,128,2,1,174,69,117,25,185,90,25,166,220,228,142,161,219,179,184,198,47,212,241,65,254,55,57,19,
  177,146,14,178,50,96,55,21,141,217,2,85,19,147,111,90,115,186,204,40,55,30,155,181,29,215,59,128,234,154,213,127,243,10,254,208,50,136,206,215,
  121,68,230,146,0,190,126,230,251,155,130,7,95,105,70,145,119,108,243,20,61,179,0,19,162,177,69,158,246,79,129,110,90,180,24,72,12,57,250,231,
  150,215,255,153,168,15,168,101,198,207,203,152,192,52,136,132,98,135,151,3,196,156,201,172,215,43,88,191,126,28,195,195,67,152,153,249,8,189,94,84,
  144,239,112,30,227,92,99,29,157,124,115,121,121,89,113,237,160,239,113,203,150,45,168,86,171,104,183,219,88,92,92,196,228,228,36,166,167,167,209,110,
  183,81,175,215,179,14,132,52,224,79,93,46,2,197,95,90,128,66,181,241,140,96,166,147,254,124,175,42,25,106,81,12,153,246,97,168,99,113,113,17,
  213,106,21,221,238,34,218,237,150,2,74,133,175,116,110,23,151,102,12,124,63,206,220,54,132,124,66,232,216,139,93,4,229,44,132,40,62,20,115,64,
  245,115,22,115,73,14,196,138,251,169,10,158,41,155,173,15,148,189,172,80,50,125,126,17,57,143,76,129,186,154,193,225,199,149,100,178,18,217,155,250,
  122,97,71,57,59,226,142,29,225,188,15,135,20,117,211,3,135,70,109,242,4,229,218,253,202,29,133,92,0,160,110,115,164,135,174,172,171,228,24,8,
  121,140,34,37,97,63,192,212,141,81,238,78,147,126,70,136,36,233,58,167,115,105,7,39,238,224,117,241,31,118,137,186,116,76,145,248,10,130,0,237,
  78,27,211,135,143,97,114,243,36,230,206,166,128,125,96,96,28,235,214,111,195,154,137,97,236,186,247,94,140,173,217,136,211,31,125,132,83,167,222,213,
  2,146,180,177,200,187,56,62,115,172,112,88,235,210,189,212,95,85,97,208,162,118,182,158,58,157,78,118,56,155,186,138,233,82,201,54,143,240,162,39,
  165,109,220,102,57,146,204,90,254,46,198,173,75,105,187,128,77,215,197,47,214,166,78,171,237,226,221,75,247,4,174,29,42,183,23,184,20,135,153,130,
  17,91,215,58,147,134,205,54,6,46,200,49,217,162,113,123,152,46,173,40,31,212,182,98,107,27,163,205,201,137,76,50,12,153,97,179,105,233,185,194,
  33,93,192,98,107,20,163,235,190,104,155,167,116,29,138,49,136,249,106,146,140,184,236,209,42,120,83,165,28,58,9,9,5,216,156,51,7,13,94,100,
  123,59,33,241,16,159,41,64,67,81,67,174,186,23,228,242,142,4,73,226,3,16,217,18,63,203,202,10,178,34,13,142,114,105,73,238,182,145,23,236,
  167,76,115,209,77,65,109,216,145,244,153,243,68,210,78,3,35,35,85,220,125,247,109,56,126,124,22,87,175,46,40,1,155,92,24,40,175,127,249,239,
  242,189,82,25,216,210,210,82,38,139,144,223,113,146,36,168,84,42,184,227,142,59,50,87,12,81,104,88,171,213,208,108,54,179,140,184,188,190,114,151,
  156,16,66,79,156,123,54,71,125,112,93,85,8,151,212,167,190,133,56,142,16,134,85,80,61,111,90,96,152,182,231,246,253,90,246,25,105,39,223,188,
  136,51,138,98,197,211,57,245,129,110,162,213,106,246,63,59,33,178,11,217,213,34,207,86,164,195,140,36,173,186,167,128,255,188,133,56,32,107,219,139,
  141,159,242,192,81,214,92,231,254,205,234,250,167,133,138,34,128,150,247,6,209,104,197,37,216,149,61,162,213,128,138,214,36,36,78,70,20,174,216,145,
  146,40,238,216,209,109,79,241,60,15,190,220,197,135,10,206,69,85,171,142,169,165,160,155,99,35,196,53,104,127,122,202,182,173,212,170,72,213,209,132,
  133,195,58,41,209,7,82,182,216,233,31,61,172,21,10,173,188,215,177,62,188,148,35,209,48,77,190,18,209,185,30,84,220,179,148,223,83,165,82,77,
  89,140,254,247,166,166,110,193,159,126,254,65,12,15,85,49,88,175,98,243,250,85,24,29,174,24,159,149,8,154,104,138,87,214,147,113,133,89,226,154,
  92,202,78,108,134,52,197,107,107,149,172,107,226,115,35,142,251,90,191,100,61,243,74,190,116,133,142,54,151,13,249,143,184,31,122,232,153,180,196,186,
  32,128,206,93,147,126,156,174,65,181,225,65,196,74,114,86,210,92,69,110,79,108,90,127,58,6,71,158,107,212,119,158,155,167,182,76,31,167,225,180,
  221,151,78,191,47,7,57,58,173,177,220,14,215,164,145,52,177,231,220,179,146,93,64,104,234,158,203,32,112,215,19,7,34,117,86,145,201,16,218,145,
  79,62,219,242,131,51,102,65,1,29,167,46,205,173,122,237,22,231,93,175,23,43,29,226,132,191,116,106,33,215,147,62,67,101,32,133,102,56,29,91,
  46,227,200,25,201,88,98,43,253,190,157,153,60,230,152,184,129,64,1,117,2,248,117,187,81,118,78,203,227,200,89,210,36,251,60,32,198,45,183,108,
  196,182,109,91,80,169,132,138,19,10,37,41,232,31,211,121,214,237,118,209,110,183,51,192,44,255,204,170,85,171,80,173,86,179,22,222,189,94,15,213,
  106,21,195,195,195,24,31,95,141,213,171,87,163,213,106,73,204,111,94,108,39,220,59,104,83,20,1,218,196,51,16,127,228,121,35,207,147,244,61,164,
  215,94,92,92,64,146,116,250,215,238,42,141,71,210,247,16,161,82,9,165,177,36,82,246,65,20,10,138,6,42,57,80,237,245,34,244,122,221,126,195,
  25,244,3,24,79,105,146,34,28,53,212,251,240,21,82,35,138,226,62,230,234,21,230,93,26,180,121,44,120,206,51,97,106,22,130,174,67,89,198,34,
  158,17,149,5,138,231,42,130,185,98,182,71,245,127,190,222,216,145,206,53,23,230,122,37,216,49,76,164,141,93,199,238,208,200,68,167,127,226,6,43,
  91,48,153,34,8,51,75,128,130,127,116,222,19,94,108,164,106,234,130,75,27,167,81,120,0,161,197,78,173,131,34,210,40,37,144,38,43,159,190,207,
  55,248,92,152,47,111,250,122,144,164,247,167,22,22,54,92,154,195,85,72,79,211,30,113,28,195,243,69,186,44,125,254,71,142,188,137,217,83,199,241,
  165,47,127,9,7,15,28,192,219,71,222,210,94,147,243,128,212,85,195,155,64,35,39,242,215,233,232,41,56,227,238,75,119,176,94,207,113,203,205,68,
  228,235,31,56,120,16,111,188,249,230,53,143,251,155,223,248,6,122,189,30,126,248,163,31,21,252,139,239,184,253,118,60,188,111,31,60,207,203,116,214,
  242,61,236,188,231,30,60,248,192,3,136,227,24,63,126,238,57,204,207,207,99,114,211,166,204,143,250,123,63,248,1,22,22,22,180,1,156,231,121,89,
  171,241,35,71,143,226,87,47,191,140,93,59,119,226,129,189,123,241,206,209,163,248,205,111,126,99,148,231,184,178,136,156,158,84,190,151,135,247,237,195,
  142,237,219,17,199,49,94,123,253,117,188,241,230,155,25,35,36,222,143,232,238,40,179,89,239,190,247,30,94,61,112,32,43,26,226,178,47,28,227,76,
  181,206,174,236,175,144,75,112,140,178,248,239,19,143,63,142,77,27,55,226,229,87,94,193,187,239,189,167,204,3,206,50,78,190,71,121,174,253,234,229,
  151,113,244,216,177,236,217,52,26,13,124,239,7,63,112,98,166,185,53,199,1,104,42,9,162,251,45,213,136,186,20,86,234,214,27,117,63,161,231,133,
  174,137,141,174,83,34,189,39,90,140,70,27,124,168,215,246,20,139,44,117,175,45,158,57,69,86,75,46,28,164,103,149,186,191,171,186,127,79,250,25,
  193,6,11,55,13,234,51,13,133,53,76,65,96,158,174,79,65,85,144,165,248,211,251,207,217,59,1,174,125,63,200,128,179,88,182,97,232,65,238,28,
  151,62,34,95,97,28,133,52,36,253,108,209,46,60,144,10,12,83,221,236,248,248,32,106,181,155,112,238,220,69,92,185,178,80,112,222,161,181,80,186,
  44,141,236,92,36,192,234,77,55,221,132,15,63,252,48,251,153,90,173,86,0,105,205,102,19,235,215,143,195,243,198,224,251,49,90,173,22,42,149,74,
  38,109,16,1,131,236,49,44,214,68,222,17,49,5,173,65,144,103,58,42,149,10,150,151,19,229,204,234,116,58,168,84,124,52,155,237,236,29,181,90,
  77,212,106,163,232,116,226,44,128,74,117,210,61,196,113,39,179,185,19,128,55,247,109,78,178,118,221,170,254,216,235,203,105,130,236,57,231,235,37,46,
  72,37,212,107,211,236,89,58,39,194,80,110,245,157,219,233,234,44,24,213,150,218,9,155,21,167,227,40,6,254,170,187,76,142,165,60,86,30,34,152,
  247,235,133,29,117,189,58,84,236,168,250,73,235,136,11,93,150,81,254,247,208,180,49,138,10,212,212,218,198,156,246,227,172,183,116,105,63,78,191,199,
  109,198,114,138,89,6,92,185,62,81,101,82,184,86,184,106,180,34,71,241,73,223,135,49,95,108,57,104,78,129,181,45,61,44,247,166,119,161,252,211,
  174,86,126,225,48,213,117,80,147,173,127,76,204,148,122,16,9,15,201,116,227,170,85,107,253,66,197,102,255,119,123,184,178,112,9,63,123,254,103,88,
  92,108,88,117,137,58,173,34,213,20,113,139,138,54,44,225,64,47,199,200,113,27,49,31,9,187,251,117,151,29,183,238,218,123,247,236,65,28,199,152,
  62,124,248,154,198,45,3,99,46,125,14,164,14,44,108,218,72,2,7,95,120,236,49,124,231,123,223,115,2,52,84,227,203,205,89,145,169,16,204,151,
  204,82,83,169,134,13,72,155,164,71,181,90,13,59,182,111,207,0,255,242,242,114,54,79,232,51,160,236,228,142,237,219,49,50,50,130,231,247,239,215,
  202,153,184,140,144,77,235,175,99,28,76,204,132,248,187,60,22,142,8,224,27,104,192,89,247,204,189,127,93,144,195,249,230,139,160,65,176,113,54,55,
  20,23,247,15,174,35,163,105,31,51,1,101,93,86,83,214,95,235,220,64,104,118,73,183,167,200,251,48,101,172,242,113,248,76,102,51,247,227,21,62,
  184,20,68,231,96,193,87,122,27,20,229,28,121,163,20,33,143,160,26,107,193,2,11,230,46,8,60,73,123,42,44,234,60,201,199,89,109,142,162,178,
  135,114,145,160,90,80,152,106,168,61,73,110,135,204,185,67,0,123,245,253,120,138,79,240,208,80,5,91,183,110,64,189,94,197,236,236,57,5,160,112,
  129,151,46,128,146,241,64,179,217,196,29,119,220,129,217,217,89,204,204,204,0,72,253,238,233,154,105,52,22,80,169,196,253,191,47,98,213,170,53,133,
  231,39,7,146,114,182,67,72,60,227,56,55,38,200,45,240,252,126,59,239,60,155,149,6,234,109,37,96,175,84,170,125,118,180,35,21,10,166,192,177,
  94,175,98,121,249,74,6,112,101,57,68,10,180,125,73,90,3,229,103,114,95,233,144,9,170,18,82,152,23,51,192,82,6,192,144,198,22,101,178,33,
  174,200,48,159,19,61,38,56,245,10,22,117,60,137,39,201,52,123,57,118,212,101,214,175,55,118,148,127,199,142,29,227,146,216,145,15,172,67,83,242,
  175,82,169,56,209,217,186,116,122,89,157,162,77,175,72,169,246,107,41,16,19,147,51,103,126,115,224,153,90,206,208,170,77,175,16,149,201,90,171,28,
  104,228,215,81,25,172,80,1,141,84,23,166,43,224,228,52,152,230,162,29,98,27,212,237,34,8,83,93,89,71,170,253,59,123,97,206,122,104,203,155,
  160,73,50,162,99,129,229,166,27,98,131,160,94,168,98,147,162,192,64,60,15,142,73,178,189,247,235,49,110,249,51,126,250,252,243,152,157,155,195,147,
  79,60,129,201,77,155,176,123,215,46,28,234,183,137,253,252,99,143,225,166,45,91,178,159,61,52,61,141,3,7,15,226,193,7,30,192,174,157,59,49,
  115,226,4,94,124,233,165,172,29,247,252,252,60,126,244,236,179,248,230,55,190,1,0,248,206,247,190,135,70,163,129,91,111,185,5,143,62,242,8,58,
  157,14,222,126,231,29,41,150,135,177,67,220,200,200,8,30,124,224,1,204,206,206,22,230,250,234,213,171,241,153,79,127,26,155,39,39,251,58,246,19,
  248,237,171,175,98,105,105,137,45,192,148,159,85,24,134,25,75,77,239,77,102,76,191,247,131,31,224,233,167,158,194,43,191,254,53,142,207,204,40,239,
  171,86,171,225,129,189,123,51,160,60,63,63,143,87,15,28,192,71,167,78,225,203,79,63,157,205,255,175,254,213,95,225,7,63,252,97,198,196,43,239,
  186,255,174,94,61,112,0,211,135,15,103,99,218,60,57,137,181,107,215,102,127,255,237,171,175,102,255,254,192,222,189,56,52,61,141,70,163,129,125,15,
  61,148,118,126,156,157,197,232,232,104,154,133,233,55,134,153,152,152,192,99,143,60,146,53,152,1,128,23,94,124,17,199,103,102,20,159,239,23,95,122,
  9,19,19,19,104,181,90,56,61,59,139,45,155,55,103,99,60,52,61,141,36,73,178,185,241,242,43,175,224,244,236,44,254,252,75,95,66,165,90,197,
  193,131,7,113,124,102,38,187,214,47,94,120,1,51,253,150,243,156,206,189,76,23,44,221,191,115,193,152,248,62,149,169,217,246,90,91,97,160,205,158,
  207,117,141,209,121,46,31,122,46,159,201,89,44,234,159,81,160,176,99,234,30,163,90,219,9,102,185,200,72,201,44,155,124,15,81,198,234,114,129,116,
  106,105,230,43,109,183,101,31,222,124,127,84,89,191,98,144,225,73,246,105,170,3,71,202,112,230,141,89,138,5,141,189,66,128,144,255,140,79,252,125,
  19,178,119,11,189,110,206,118,110,220,56,142,145,145,33,124,248,225,25,116,58,157,66,225,22,149,208,113,65,230,248,248,56,124,63,117,169,168,86,171,
  184,237,182,219,112,252,248,113,0,192,192,192,0,1,199,9,146,164,167,200,48,168,246,94,6,103,113,28,101,231,52,181,121,147,193,181,239,251,104,52,
  174,34,142,123,136,34,31,231,207,159,199,248,248,56,146,164,135,102,179,193,200,115,188,12,56,231,29,33,99,180,90,13,116,187,61,208,86,219,186,142,
  130,178,188,34,117,253,128,36,159,65,65,62,36,187,90,200,22,140,92,161,96,238,99,238,91,189,157,85,119,12,144,115,219,55,202,37,169,191,179,76,
  188,232,92,59,184,32,118,165,216,81,87,64,248,187,192,142,50,174,43,221,237,129,243,128,229,82,244,166,118,201,174,105,96,217,134,139,99,98,221,175,
  231,247,55,148,30,128,106,198,46,123,222,0,42,21,100,17,125,173,86,71,181,94,67,189,54,136,106,181,134,106,181,142,106,197,71,125,96,16,181,90,
  53,77,149,37,34,5,20,1,8,16,69,169,253,78,165,90,3,18,32,78,34,68,113,130,110,187,9,207,247,16,134,117,204,28,127,23,75,75,23,11,
  192,144,234,105,117,105,86,170,47,114,57,36,210,52,95,140,94,148,110,114,181,106,21,203,203,40,48,20,166,212,181,156,10,215,177,104,38,61,107,113,
  76,188,236,65,215,225,138,187,150,203,231,93,239,113,139,127,159,153,153,193,228,166,77,168,213,106,168,215,235,216,181,115,39,110,218,178,5,7,14,30,
  204,90,108,239,218,185,19,167,79,159,206,58,240,137,182,216,147,253,255,30,61,118,172,176,33,140,142,142,226,79,62,247,57,0,105,183,189,123,119,239,
  46,164,254,197,90,224,190,118,237,220,153,177,35,226,171,86,171,225,233,167,158,202,186,14,122,158,135,91,166,166,80,175,215,51,182,148,147,158,200,215,
  188,239,222,123,241,250,27,111,224,245,55,222,192,190,135,30,202,62,231,120,159,25,2,128,135,247,237,131,239,251,56,113,242,100,225,61,62,246,232,163,
  89,183,64,32,109,73,254,216,163,143,226,219,223,253,174,2,160,92,25,109,32,237,146,40,64,253,232,200,8,102,102,102,176,121,114,18,183,76,77,225,
  208,244,52,182,221,124,51,0,96,230,196,9,172,237,183,64,95,59,49,145,129,248,36,73,80,173,86,241,234,129,3,120,248,161,135,50,38,251,244,236,
  44,254,230,235,95,199,195,15,63,140,217,57,53,184,124,244,145,71,140,153,130,215,223,120,67,249,249,43,87,174,224,95,127,245,43,60,250,200,35,216,
  187,103,79,214,109,242,208,244,116,246,156,92,154,58,113,105,73,215,246,217,186,207,160,69,53,38,103,17,83,99,29,26,224,186,72,97,56,70,159,59,
  232,92,125,89,185,224,210,100,211,40,222,91,26,176,235,52,218,114,107,98,176,146,59,161,69,149,11,191,138,153,78,10,110,104,215,62,244,219,116,203,
  173,147,161,200,49,212,98,74,191,160,37,21,64,54,215,164,10,171,50,85,26,32,218,68,139,159,201,219,135,251,10,195,152,3,100,79,105,214,66,193,
  86,46,249,200,25,240,161,161,26,118,236,216,134,211,167,47,96,126,254,98,1,176,152,26,227,140,141,141,97,114,114,50,237,111,208,47,18,172,213,106,
  24,26,26,194,210,210,18,170,213,170,162,241,77,157,120,170,89,235,108,62,51,20,179,153,142,56,142,81,173,134,74,113,35,45,46,76,153,233,54,86,
  173,26,237,51,194,81,225,243,151,150,154,24,25,169,43,246,131,169,182,189,71,88,231,56,99,251,229,54,219,42,120,46,182,236,166,45,192,115,139,183,
  132,72,175,124,34,209,136,37,41,70,194,186,100,232,26,163,240,142,100,190,50,135,139,56,15,204,56,139,0,94,158,75,215,138,29,245,29,14,127,23,
  216,145,175,195,203,10,81,101,141,31,45,204,17,44,128,45,157,39,6,40,83,241,130,85,148,237,109,116,17,131,238,218,174,221,170,108,108,228,182,109,
  159,194,109,119,220,158,54,236,168,164,237,136,171,97,128,74,165,150,142,217,75,80,169,4,240,189,212,7,25,113,130,40,142,50,3,243,94,183,131,78,
  171,131,56,1,130,48,64,156,244,16,6,1,124,63,64,24,84,250,187,74,130,4,9,252,160,130,192,247,211,30,247,3,3,168,212,42,24,168,87,240,
  234,171,255,194,30,36,133,214,144,204,129,229,82,117,90,124,14,113,191,57,75,85,115,72,168,169,216,178,207,222,229,189,148,241,211,229,92,42,92,236,
  193,108,122,232,235,49,110,93,49,212,129,131,7,113,254,252,249,12,0,11,240,54,54,54,134,185,51,103,48,59,55,135,201,77,155,48,185,105,19,54,
  79,78,102,224,147,190,187,45,155,55,103,90,187,191,255,214,183,50,150,213,230,100,208,104,52,112,124,102,166,192,20,11,192,94,169,84,208,233,116,240,
  157,239,125,15,245,122,29,95,121,230,25,76,110,218,132,225,225,225,66,243,128,40,138,16,73,239,96,106,219,54,4,65,128,251,238,189,23,247,221,123,
  175,114,93,25,64,207,204,204,40,128,92,124,85,171,213,12,60,63,191,127,63,102,231,230,50,61,243,212,182,109,248,201,127,251,111,248,218,87,191,154,
  177,216,84,7,238,250,53,115,226,4,30,216,187,23,107,215,174,197,150,205,155,177,110,221,58,92,152,159,199,133,11,23,50,0,61,58,58,154,61,215,
  93,59,119,98,116,116,20,158,231,225,199,207,61,135,93,59,119,98,237,218,181,120,96,239,222,172,53,186,248,175,248,122,126,255,254,140,237,238,118,187,
  248,238,247,191,143,135,247,237,195,212,182,109,24,25,25,81,222,143,56,12,78,156,60,137,163,199,142,225,206,29,59,176,102,98,2,11,11,11,89,11,
  118,87,16,172,147,107,185,110,250,156,158,207,181,168,70,183,126,57,137,146,124,232,201,192,156,254,190,206,139,181,172,171,134,107,67,30,221,126,146,131,
  186,88,57,220,121,176,12,77,221,74,209,157,131,54,172,224,218,133,23,153,61,81,84,22,247,187,13,38,10,136,230,238,55,5,173,121,93,143,0,191,
  189,158,240,45,22,12,116,238,192,33,106,132,210,107,167,54,122,113,156,50,158,121,90,223,147,128,60,164,54,225,114,195,142,226,115,201,129,116,140,205,
  155,215,160,94,15,113,238,220,69,244,122,145,182,62,74,158,31,107,214,172,65,173,86,203,188,160,197,60,25,24,24,64,171,213,202,108,234,104,26,94,
  4,58,149,74,37,235,216,73,89,90,17,232,80,131,4,25,188,138,235,182,219,237,126,113,103,234,233,188,180,116,21,149,202,88,22,128,228,32,215,239,
  75,56,122,74,129,105,145,81,142,251,64,88,117,86,17,69,137,190,31,100,46,42,226,153,6,129,215,151,144,8,176,92,4,163,188,191,58,250,76,188,
  10,214,57,71,14,202,88,235,25,219,226,185,167,214,128,200,146,136,168,0,188,147,68,14,182,125,35,163,92,6,59,210,61,231,119,137,29,117,234,138,
  32,8,144,0,8,229,170,83,91,87,39,185,234,159,43,198,161,69,98,186,205,140,110,232,43,5,61,186,212,160,60,49,60,207,195,93,119,109,195,205,
  91,55,226,208,155,211,104,53,59,136,226,8,237,110,19,113,55,101,145,227,56,109,102,18,69,61,180,91,93,52,91,139,125,230,182,130,86,107,9,94,
  198,50,8,253,116,238,23,233,33,68,130,40,251,188,192,175,3,73,130,24,17,38,198,199,241,213,175,125,165,175,85,82,55,69,121,99,12,130,176,32,
  15,41,3,6,117,96,218,247,253,180,243,31,128,4,129,230,89,133,8,2,187,87,175,235,1,231,242,61,155,118,216,20,92,217,82,183,215,107,220,220,
  58,216,188,121,51,128,212,122,175,211,233,96,223,67,15,97,199,246,237,152,157,155,195,241,227,199,209,233,116,112,231,142,29,217,194,22,140,245,195,251,
  246,161,82,169,96,250,240,225,2,83,236,121,30,218,237,54,251,153,178,4,70,7,34,222,120,243,77,220,50,53,165,200,16,188,254,105,71,89,44,221,
  38,146,57,25,48,63,35,100,27,242,252,144,63,235,180,36,29,113,253,10,130,96,197,140,192,45,253,118,191,240,60,92,185,122,21,173,86,43,99,251,
  31,222,183,47,3,245,242,215,242,242,50,154,205,38,154,205,102,6,238,1,224,175,254,242,47,49,58,58,138,163,199,142,165,140,244,190,125,202,189,137,
  123,190,186,176,144,201,63,218,237,84,11,217,110,183,11,133,77,244,107,221,218,181,41,112,236,245,80,171,213,50,31,91,27,88,94,169,244,205,101,221,
  149,181,61,52,105,222,105,81,165,174,88,145,43,82,116,177,99,212,173,83,91,97,35,103,73,73,159,109,94,220,232,73,96,154,218,131,241,76,52,45,
  212,20,154,90,83,179,21,106,235,41,179,191,2,12,167,154,100,21,68,203,224,95,72,53,132,214,86,232,162,229,118,223,185,230,85,216,37,170,141,87,
  226,184,215,151,19,166,25,202,40,234,245,245,182,9,122,189,110,218,0,75,98,163,133,93,30,71,188,20,129,116,26,16,76,76,140,96,108,108,4,167,
  79,159,199,194,66,67,59,247,196,255,11,134,89,182,160,11,130,0,27,55,174,199,166,77,107,144,36,221,172,30,75,92,39,149,72,228,129,153,176,192,
  163,132,138,104,8,37,59,104,165,247,218,235,203,85,226,108,79,15,67,31,173,86,151,176,188,177,164,91,207,25,215,180,201,74,156,1,224,162,107,73,
  66,220,44,188,204,62,46,117,101,201,223,159,108,71,39,50,18,185,155,76,177,17,74,46,99,2,9,118,124,9,236,235,252,144,193,186,108,208,119,42,
  116,249,38,201,70,190,247,37,133,38,68,234,58,65,6,166,175,39,118,92,201,158,104,199,142,118,16,46,103,4,67,207,247,145,48,5,69,212,46,137,
  234,233,108,155,49,53,90,55,29,26,180,105,129,173,179,159,229,209,128,86,126,122,30,48,119,230,18,126,243,219,223,104,188,126,107,152,186,117,7,102,
  79,159,66,187,117,81,188,198,116,99,200,54,66,241,208,42,253,255,79,129,105,156,196,253,40,55,221,172,162,184,149,141,164,211,75,63,107,121,121,153,
  5,207,121,209,154,94,179,155,47,36,160,56,87,196,216,42,253,72,124,36,219,16,130,160,2,223,15,49,60,188,26,0,176,109,106,7,206,159,31,71,
  165,90,67,181,82,129,31,248,24,24,24,68,37,168,33,8,211,67,224,131,15,142,226,236,153,15,173,205,59,108,224,158,43,234,146,179,28,178,167,171,
  174,40,200,36,191,208,177,108,215,123,220,98,115,126,226,241,199,149,241,31,154,158,70,28,199,216,180,105,19,146,36,193,59,239,188,131,217,185,57,236,
  218,181,43,187,142,239,251,56,62,51,131,7,246,238,205,92,36,142,19,96,71,65,104,173,86,195,55,191,241,13,52,26,13,39,93,169,231,121,232,116,
  58,248,167,23,94,192,255,248,229,47,43,255,126,234,244,105,116,58,29,84,171,85,252,205,215,191,158,221,215,236,220,28,22,22,22,140,141,46,132,94,
  122,98,98,34,99,109,197,179,255,245,111,126,131,83,167,79,91,87,97,167,211,201,24,120,225,14,34,190,127,124,102,166,192,242,218,190,246,238,217,147,
  177,236,9,128,185,185,57,204,207,207,195,243,60,204,156,56,129,93,59,119,102,224,151,62,103,170,251,21,65,128,0,197,175,30,56,128,209,209,81,5,
  60,235,230,88,225,153,145,246,192,226,115,118,237,220,137,137,137,9,52,26,13,156,158,157,197,142,237,219,241,240,190,125,44,91,111,3,140,116,99,167,
  5,160,186,150,223,92,109,65,25,230,219,116,56,113,100,139,107,59,93,174,57,13,101,212,116,146,14,218,166,94,103,197,104,170,172,23,115,66,206,188,
  169,158,206,158,51,19,173,58,50,113,159,151,131,34,153,241,86,11,171,189,204,139,57,101,31,139,13,193,104,79,6,32,40,52,141,18,76,176,72,149,
  231,65,65,94,216,40,92,166,84,64,150,254,110,16,132,153,67,68,238,212,33,192,160,175,92,187,232,228,32,183,33,247,17,4,49,110,190,121,3,206,
  159,175,227,236,217,11,44,155,39,238,169,94,175,103,239,51,245,98,110,163,86,171,96,116,180,134,110,55,109,127,45,207,29,250,238,122,189,30,106,53,
  20,100,22,169,78,61,204,10,4,5,198,88,94,22,94,207,33,186,221,30,60,47,64,183,219,69,173,230,75,237,186,147,190,251,136,215,183,26,132,164,
  117,6,210,215,23,161,215,139,64,117,204,114,209,96,14,32,19,210,194,59,134,236,234,34,2,166,116,47,73,10,174,46,178,73,128,124,125,153,189,22,
  215,52,201,156,242,134,47,17,219,203,131,90,41,114,235,64,221,123,242,251,80,231,190,220,80,237,250,98,71,174,171,236,245,193,142,96,229,70,220,156,
  205,26,245,248,158,7,48,221,219,92,27,161,208,23,64,25,47,151,13,155,110,4,156,17,183,75,154,94,116,204,201,31,190,15,223,15,16,120,21,73,
  107,230,161,82,29,195,218,117,235,177,122,108,4,231,206,207,227,234,149,75,120,236,145,79,227,173,183,198,241,155,223,252,18,64,216,7,202,130,165,168,
  163,94,31,68,183,219,65,183,219,200,62,35,29,75,12,32,132,239,247,138,105,236,190,249,120,179,217,118,46,178,161,247,169,22,39,166,95,27,54,220,
  130,59,239,220,142,129,129,65,248,1,80,175,13,164,247,25,248,121,225,72,224,195,247,60,12,14,164,133,160,251,246,237,74,187,22,69,49,154,173,38,
  16,123,168,84,66,196,113,234,147,57,182,106,12,107,199,7,241,220,63,126,100,100,174,220,53,216,234,191,137,251,144,125,101,41,232,208,181,113,213,177,
  78,244,123,215,115,220,52,221,43,254,123,224,224,65,76,31,62,12,0,56,120,240,32,30,125,228,17,124,254,177,199,208,104,52,112,254,194,5,140,12,
  15,103,85,227,157,78,7,239,28,57,130,221,187,118,97,118,110,14,23,47,94,100,65,66,167,211,193,139,47,189,132,71,31,121,4,237,118,27,175,30,
  56,160,232,110,185,13,70,6,4,151,46,93,194,161,233,105,236,218,185,83,57,84,126,186,127,63,30,216,187,55,147,82,204,156,56,129,87,15,28,208,
  110,98,114,22,224,208,244,52,70,70,70,50,237,48,0,252,234,149,87,112,244,232,81,140,141,141,57,165,197,94,120,241,69,165,136,240,252,249,243,120,
  245,192,1,180,90,173,2,128,118,234,80,216,95,5,71,143,30,197,129,131,7,179,141,243,252,249,243,56,61,59,139,205,147,147,56,61,59,139,70,163,
  97,212,254,2,192,194,194,2,222,60,116,8,187,119,237,194,223,124,253,235,248,232,212,41,204,207,207,99,98,98,66,97,138,99,162,159,44,128,49,38,
  83,49,49,49,145,129,253,95,189,252,50,22,26,13,236,216,190,29,155,54,110,196,61,119,223,141,183,222,126,123,69,27,58,247,92,40,176,226,228,88,
  174,95,114,229,186,141,161,225,214,145,173,35,166,174,219,160,169,129,145,173,149,183,237,203,148,33,16,41,126,153,112,145,155,92,168,179,78,77,127,115,
  7,171,220,209,175,40,167,131,100,93,87,76,195,203,133,129,234,231,233,37,29,130,137,86,153,68,57,45,159,179,214,226,26,242,88,133,219,70,222,6,
  220,147,64,120,174,243,78,217,232,68,145,117,228,247,192,7,92,130,217,159,152,24,193,208,80,29,103,207,94,196,226,226,146,226,164,32,88,72,89,227,
  44,222,87,175,215,69,167,211,234,239,189,106,1,91,238,201,12,197,19,91,222,223,211,110,137,233,125,116,187,93,169,219,99,14,4,163,168,139,78,103,
  185,223,130,187,130,197,197,171,82,70,66,61,163,40,179,220,104,92,149,152,99,159,0,231,28,60,139,49,200,172,176,220,116,135,43,80,227,228,23,185,
  94,56,129,106,89,152,3,103,89,242,161,99,150,101,235,223,226,245,229,181,97,6,207,121,187,120,159,200,140,124,197,42,82,87,67,176,82,236,72,215,
  222,245,197,142,145,2,220,117,142,66,25,38,73,18,120,255,231,127,250,79,137,120,57,223,250,206,119,216,67,218,197,51,149,211,205,217,152,103,27,235,
  193,21,95,216,65,185,234,150,17,199,62,158,122,226,9,84,7,70,240,236,179,169,221,215,159,60,250,4,238,190,235,86,44,92,105,160,23,39,248,151,
  127,57,128,63,249,220,253,88,110,245,240,243,159,255,10,195,67,53,220,180,117,27,70,70,6,176,184,216,193,218,181,171,177,113,227,56,122,221,46,94,
  127,253,109,92,190,114,53,5,4,231,206,162,213,186,210,7,218,3,253,180,90,51,155,164,55,221,116,43,190,252,229,39,241,195,31,254,20,179,179,31,
  112,161,3,146,164,87,240,54,229,34,200,252,185,248,120,230,153,191,70,224,183,112,250,212,28,124,84,112,181,113,17,237,118,27,155,183,220,142,94,183,
  131,91,111,219,140,193,161,65,0,192,165,11,11,24,26,25,64,173,158,2,233,55,95,59,130,171,139,139,24,31,95,141,53,107,198,17,120,192,220,217,
  11,184,121,235,36,230,102,63,194,11,47,253,66,155,30,145,199,37,79,118,151,3,77,118,43,209,45,20,23,111,103,211,1,124,163,142,91,173,210,143,
  88,143,91,157,15,165,41,45,110,250,146,27,36,113,27,42,183,177,173,180,149,181,137,17,149,189,153,77,197,164,38,16,9,0,95,126,250,105,76,76,
  76,224,229,87,94,193,145,163,71,157,247,30,147,22,159,62,3,249,57,209,119,99,42,166,227,60,165,203,234,241,128,212,226,170,82,169,104,199,72,219,
  144,235,52,199,215,131,149,225,64,164,92,164,103,179,146,179,49,78,28,187,205,21,211,114,237,203,77,108,189,62,45,27,21,252,108,213,31,245,140,160,
  160,24,216,232,254,93,5,195,180,40,80,20,10,230,99,205,27,95,136,235,114,1,20,101,42,211,231,194,253,158,151,117,35,148,219,128,171,63,39,138,
  123,229,231,224,145,207,246,148,46,133,178,107,72,81,239,156,74,65,206,156,185,136,11,23,46,42,4,74,181,90,197,125,247,221,135,53,107,214,96,121,
  121,25,213,106,181,223,40,37,192,249,243,167,0,196,240,253,65,140,142,142,23,240,69,163,113,174,111,187,54,140,161,161,145,172,142,164,90,173,98,121,
  121,17,173,214,37,36,73,5,213,234,48,234,245,122,246,153,173,214,34,218,237,70,223,78,115,0,221,174,143,122,189,142,78,167,129,78,167,213,7,215,
  17,42,149,42,130,96,16,149,74,136,197,197,139,253,182,217,49,241,19,246,141,172,179,0,207,42,144,78,50,55,19,121,127,19,90,237,92,142,83,212,
  204,203,158,211,244,255,185,254,7,42,240,77,10,32,80,214,58,155,152,91,10,158,139,231,152,255,59,199,142,38,235,207,235,131,29,139,0,159,27,255,
  215,255,195,127,200,158,66,104,43,42,209,61,80,57,109,169,107,175,202,109,178,182,162,66,151,67,136,183,164,19,7,84,143,56,11,196,136,226,30,162,
  184,139,180,102,178,135,153,247,223,198,212,214,117,72,60,224,194,217,203,248,244,131,247,97,116,108,20,99,171,60,60,253,244,231,49,58,50,136,133,171,
  11,152,59,125,6,15,62,112,39,0,224,236,217,243,120,231,240,59,120,228,243,159,203,54,252,115,231,174,226,135,255,245,71,184,237,214,59,112,255,253,
  59,225,121,62,166,15,31,195,244,161,151,1,36,24,24,72,53,89,203,185,253,5,25,111,100,221,216,105,165,106,181,58,132,53,107,134,241,139,127,250,
  55,92,188,120,21,187,118,239,198,155,211,135,241,224,103,254,24,55,221,180,1,205,118,7,240,67,52,22,26,152,61,53,135,102,51,194,237,219,111,193,
  204,219,239,226,205,233,105,220,118,219,109,248,227,63,222,155,141,229,226,249,75,184,121,235,90,140,142,214,49,59,171,50,147,242,1,124,173,133,122,50,
  3,224,2,92,117,154,36,217,215,83,102,203,110,196,113,115,5,89,2,168,235,14,35,221,253,216,82,93,92,23,81,121,83,49,49,117,43,41,210,48,
  101,154,76,0,212,101,31,146,127,102,239,158,61,136,162,8,31,28,63,142,119,142,28,193,177,119,223,213,54,123,114,101,56,76,173,217,69,224,33,7,
  57,54,176,206,53,7,208,179,68,60,19,47,111,230,212,43,89,158,123,178,103,191,169,131,34,213,57,211,2,25,91,192,65,131,2,250,76,202,6,116,
  58,185,198,74,26,223,172,132,177,150,27,104,201,114,139,252,115,139,45,191,41,27,173,130,2,125,241,161,172,139,86,61,232,61,69,78,146,3,177,156,
  241,83,155,166,232,91,128,203,0,75,5,91,113,95,142,33,251,81,203,210,143,148,49,79,231,92,206,70,231,178,14,209,192,197,87,24,105,90,244,70,
  153,77,207,139,177,97,195,106,172,90,53,138,19,39,78,101,126,228,226,143,44,177,240,125,31,221,110,4,207,11,50,169,1,245,28,23,64,51,93,211,
  42,123,157,215,106,33,147,18,200,242,144,48,12,208,106,37,253,236,240,18,42,149,17,9,188,138,123,10,50,224,155,106,200,115,77,124,110,93,151,122,
  114,203,78,42,170,12,66,6,213,9,153,55,124,17,188,0,215,92,241,159,40,110,76,179,7,1,168,203,6,175,51,23,247,238,105,252,242,193,106,157,
  101,125,117,113,254,130,172,3,119,226,99,37,216,81,206,174,253,110,177,99,164,117,165,98,207,190,36,209,219,216,137,201,76,187,122,81,203,32,29,88,
  208,109,104,212,94,68,215,117,138,78,10,90,80,166,75,39,114,2,244,74,173,222,159,252,62,146,196,195,201,15,63,196,223,255,253,119,177,99,251,29,
  184,253,246,41,108,154,28,71,146,36,88,88,104,224,151,47,188,132,199,159,252,83,84,235,117,108,220,156,250,231,118,90,29,12,12,142,224,228,71,103,
  209,92,94,194,192,224,16,58,237,54,206,157,157,197,190,125,127,132,79,221,181,13,31,188,119,12,97,181,130,125,15,237,194,251,239,189,133,165,165,75,
  24,30,26,67,167,211,197,226,226,21,22,212,232,180,190,244,158,5,243,220,231,68,1,196,232,246,34,84,107,1,238,186,235,86,156,56,241,33,30,220,
  123,55,206,204,158,193,175,94,254,45,254,248,115,15,99,120,104,0,8,43,56,249,209,41,108,216,180,14,155,183,110,129,87,169,224,245,131,7,49,119,
  250,35,220,117,247,78,172,26,27,198,183,191,247,93,0,49,62,253,224,103,177,117,235,22,163,231,48,125,206,114,181,172,11,211,197,207,35,181,88,70,
  7,110,104,209,169,201,67,246,223,103,220,112,2,178,182,113,115,160,68,231,116,96,2,10,114,250,73,103,83,102,147,5,216,138,38,76,77,50,92,191,
  232,161,65,199,104,99,122,41,208,61,112,240,32,171,81,227,222,27,245,22,54,61,95,157,116,64,62,164,93,131,9,211,51,114,181,83,164,65,34,247,
  44,76,115,217,68,132,232,128,241,74,191,92,37,22,166,6,43,186,206,136,38,255,114,46,19,176,146,177,200,238,1,234,187,75,20,182,86,149,122,64,
  115,16,195,169,184,144,54,56,202,1,117,210,119,218,200,253,163,229,2,195,188,243,158,12,202,138,110,40,169,75,3,50,224,156,130,64,175,175,113,22,
  140,114,222,76,38,247,150,134,34,235,16,211,72,60,214,92,151,91,148,118,112,128,174,94,247,113,235,173,91,112,246,236,69,44,45,45,97,124,124,21,
  130,32,200,44,236,242,103,148,250,55,167,93,17,123,25,22,145,141,12,242,194,62,53,56,78,139,16,253,172,21,189,192,48,233,179,2,218,237,101,5,
  224,138,130,192,180,184,15,202,121,20,134,65,214,174,59,45,186,140,50,15,112,1,162,117,172,51,167,137,166,77,81,56,134,55,151,246,36,68,14,130,
  190,68,2,80,189,164,97,100,157,57,240,156,103,60,60,237,53,68,231,196,116,126,37,6,70,218,140,29,249,110,137,110,216,145,6,246,255,222,216,209,
  118,150,249,194,82,37,34,44,132,75,84,47,6,194,233,238,84,35,243,184,112,67,202,32,250,155,63,183,249,201,17,129,124,131,162,178,150,70,53,114,
  101,124,182,9,199,81,191,234,181,141,36,137,49,53,245,41,124,118,223,163,56,241,225,37,60,247,147,23,112,250,163,180,136,107,104,120,24,203,109,31,
  175,191,254,58,150,23,175,96,108,108,0,205,229,101,188,247,238,81,124,116,114,6,113,146,224,249,231,255,9,205,229,37,92,185,188,140,95,255,230,32,
  226,94,3,190,239,225,196,71,23,209,237,120,104,46,47,161,213,76,43,143,195,74,128,78,43,66,28,37,10,211,65,239,203,212,81,71,46,38,201,60,
  59,227,4,72,128,32,168,2,113,140,139,231,207,224,23,191,120,17,163,171,198,176,110,195,77,88,94,110,195,247,1,63,233,34,234,45,162,213,110,161,
  219,110,225,202,197,51,88,88,184,138,19,31,157,192,252,197,75,104,181,59,89,132,254,209,169,115,24,24,28,130,239,87,180,93,244,232,56,197,59,43,
  219,67,94,189,78,98,252,93,219,247,100,31,207,127,223,113,227,154,199,109,250,204,50,160,75,222,92,196,186,228,124,53,233,134,35,119,101,116,97,160,
  57,29,176,233,217,81,176,73,181,219,58,221,58,103,121,37,223,147,45,16,161,123,130,56,60,57,205,188,46,29,72,61,98,87,10,20,203,2,82,219,
  126,96,210,250,155,222,65,25,22,184,236,125,148,117,85,225,8,4,93,198,83,215,205,81,62,7,228,180,186,109,13,186,188,39,53,216,245,156,88,56,
  238,220,18,32,133,95,55,42,32,213,165,157,41,8,147,129,152,184,118,241,250,234,218,206,219,116,67,210,206,22,219,75,11,150,85,30,91,202,242,70,
  253,191,123,200,157,73,84,15,99,49,70,193,224,230,191,155,100,197,145,81,148,142,167,82,241,177,101,203,122,108,219,182,5,227,227,67,232,116,90,138,
  212,45,197,19,62,124,63,204,108,250,68,33,160,220,28,69,126,182,178,139,7,93,183,50,131,153,36,93,180,219,205,140,201,85,231,72,210,7,229,73,
  118,223,113,220,193,242,242,98,118,111,185,92,130,99,136,197,115,68,198,142,167,99,84,193,115,28,199,89,192,32,126,150,206,85,25,60,231,76,127,238,
  162,33,44,2,213,44,18,20,41,136,220,24,165,8,174,61,45,25,36,55,20,50,131,231,180,230,140,98,177,235,133,29,229,181,253,123,195,142,150,179,
  58,204,22,45,244,118,99,38,237,27,151,242,163,155,145,252,16,76,233,209,50,27,174,28,93,219,82,236,94,224,43,133,162,151,47,206,225,190,221,183,
  225,153,103,158,196,114,171,139,213,99,3,248,240,196,41,156,57,115,6,189,94,11,175,189,254,38,94,127,227,53,12,14,12,34,138,34,180,59,109,36,
  73,8,223,247,177,180,228,225,71,255,245,57,60,244,208,67,152,220,178,9,191,252,151,95,98,169,209,192,99,143,126,6,190,239,227,159,246,191,128,184,
  223,37,169,90,169,2,94,132,94,212,86,180,98,84,15,231,146,190,240,253,184,207,154,38,136,226,24,97,232,35,233,179,210,9,128,102,219,199,123,239,
  125,132,63,122,248,94,196,113,130,217,211,103,176,255,231,63,3,0,60,88,249,44,78,157,58,131,127,123,245,64,198,102,123,94,13,213,106,254,222,26,
  139,139,168,86,171,24,30,26,68,99,177,167,68,146,156,174,146,50,38,166,20,188,14,28,232,14,210,178,0,68,142,120,111,164,113,155,116,86,46,191,
  111,98,134,93,211,232,174,102,245,28,51,75,139,65,86,146,182,215,49,161,220,58,40,11,72,229,123,211,201,18,76,236,4,199,112,114,223,43,235,236,
  226,2,228,108,191,35,230,141,75,134,193,214,77,144,178,242,174,62,203,212,31,182,236,251,167,114,39,157,7,123,153,166,44,174,231,137,218,84,69,157,
  219,105,131,18,213,190,141,182,219,230,138,11,117,239,64,238,228,86,156,203,121,51,21,113,13,186,39,228,191,159,40,142,30,162,8,146,211,167,171,242,
  13,181,197,183,208,223,166,186,111,113,38,231,108,116,222,101,145,22,35,10,223,105,72,210,14,159,176,206,185,13,154,124,79,242,125,167,76,112,130,106,
  53,253,92,225,241,44,7,234,105,155,236,212,147,57,101,134,19,130,33,210,123,136,162,36,107,94,34,255,110,171,181,220,7,69,61,44,45,45,194,247,
  211,98,197,110,183,149,129,90,241,88,6,6,234,153,251,137,112,220,16,172,120,218,89,81,200,95,138,237,178,243,32,41,102,158,13,20,16,156,222,123,
  44,1,91,185,181,186,25,60,171,255,175,183,118,21,217,2,221,254,196,53,86,225,192,177,26,232,21,131,69,245,30,174,47,118,148,241,220,239,27,59,
  218,62,47,228,168,116,29,243,193,29,204,180,112,197,198,212,152,124,105,93,216,19,46,133,169,111,41,155,110,2,161,239,163,27,121,125,239,75,224,242,
  213,75,248,225,143,158,197,154,241,213,88,191,110,3,58,173,24,31,205,206,161,219,107,75,215,240,176,220,92,150,38,73,128,56,110,33,142,3,92,188,
  220,192,79,254,219,243,0,60,4,65,13,7,94,63,136,119,223,127,23,3,181,81,156,191,56,143,212,86,174,139,193,193,193,190,127,165,246,245,178,50,
  2,186,161,167,192,79,44,192,38,186,221,54,134,134,6,113,226,228,73,192,243,144,120,62,166,182,110,194,212,45,55,247,211,53,192,234,213,163,240,188,
  10,146,164,139,15,79,158,68,175,187,44,63,89,92,189,122,25,113,188,42,251,206,194,213,179,88,108,44,98,243,230,45,120,235,237,105,37,74,212,177,
  169,101,61,188,175,71,122,88,103,99,151,155,211,223,56,227,166,154,61,147,22,76,62,244,63,251,153,207,224,174,59,239,196,169,211,167,241,179,159,255,
  220,56,94,17,109,235,230,152,174,168,141,187,22,167,33,22,99,218,60,57,137,39,30,127,28,141,70,3,223,251,193,15,180,96,84,124,137,230,47,71,
  143,29,195,191,252,235,191,226,201,39,158,192,230,201,73,252,234,229,151,113,228,232,81,86,198,194,61,31,217,57,66,247,236,108,186,116,241,179,52,229,
  168,147,79,112,247,36,52,157,54,66,97,165,64,154,211,82,235,172,28,185,130,60,83,81,142,232,232,166,203,138,185,102,66,92,188,212,185,253,205,118,
  38,232,236,37,109,50,46,147,247,48,247,115,186,22,227,194,251,159,90,114,81,16,45,154,161,232,159,135,74,160,112,146,14,46,24,145,1,176,144,112,
  164,178,131,64,25,3,189,191,220,177,129,62,75,213,63,56,47,106,203,59,11,10,223,232,92,7,155,72,32,217,39,186,95,89,35,157,59,122,200,82,
  0,234,112,34,91,181,201,44,186,154,118,143,149,238,126,178,4,75,72,56,68,241,162,60,31,82,77,117,42,187,232,118,219,168,84,128,48,172,74,236,
  172,7,217,102,77,102,93,125,63,80,186,9,230,204,186,124,79,137,194,246,10,159,109,206,137,35,239,12,168,62,63,238,157,233,236,233,114,109,190,167,
  45,136,167,160,190,56,231,61,173,222,89,158,159,69,224,44,23,39,210,61,193,67,24,94,95,236,168,115,134,250,247,199,142,110,68,84,232,18,169,115,
  55,114,173,6,214,182,78,122,58,109,30,101,12,205,17,66,186,40,107,245,58,46,204,93,232,47,154,48,91,244,23,47,93,198,197,75,11,16,197,124,
  65,48,132,141,155,182,224,236,153,89,244,122,77,164,122,99,17,69,183,251,63,3,36,73,187,127,141,0,73,226,33,12,135,209,88,92,68,99,113,145,
  188,252,8,157,158,60,241,138,140,179,123,51,146,252,239,157,78,15,67,67,35,104,52,46,227,199,63,250,33,26,141,243,120,233,159,127,129,215,95,27,
  195,224,96,29,173,118,27,129,159,58,124,36,73,130,131,175,253,186,80,89,122,244,232,155,152,59,253,158,242,25,231,206,95,196,182,155,183,225,232,177,
  119,216,244,189,60,225,203,164,71,93,24,172,107,1,180,114,186,230,70,26,183,204,136,187,124,233,52,92,38,22,155,22,83,22,51,27,190,51,19,204,
  129,127,241,253,217,185,57,252,231,255,231,255,113,46,70,126,243,208,161,172,51,95,16,4,138,63,114,25,54,179,76,85,53,247,187,162,209,130,169,40,
  144,122,140,234,188,203,105,145,164,9,208,115,159,163,11,166,93,44,254,40,163,194,21,159,154,2,26,26,132,228,141,70,236,25,26,57,3,177,18,207,
  105,181,81,200,202,10,15,203,118,51,228,206,35,19,219,173,186,37,196,74,113,161,236,66,192,233,154,213,191,23,187,210,169,105,114,207,216,2,60,111,
  188,34,23,165,37,133,130,62,234,253,43,174,39,7,10,41,40,245,179,78,131,2,248,166,255,175,130,189,124,60,185,62,58,253,28,202,186,122,133,0,
  131,58,125,228,224,59,39,145,104,64,148,74,61,42,104,181,58,146,181,92,162,172,191,84,211,157,96,112,112,16,237,118,7,245,122,29,0,208,108,54,
  81,169,4,232,27,40,161,221,110,97,104,104,85,223,78,54,37,144,82,198,217,79,123,55,244,59,255,229,157,8,85,25,139,40,180,148,89,215,220,250,
  79,5,207,180,123,31,93,151,114,139,110,174,160,79,110,216,34,7,29,98,140,174,46,27,52,0,164,76,184,9,60,139,192,38,159,251,62,251,243,166,
  165,181,18,236,232,42,161,252,247,193,142,40,88,35,178,0,58,145,74,67,117,222,158,20,185,235,138,147,76,41,101,27,67,68,219,225,186,110,130,156,
  17,191,168,168,22,47,186,82,169,246,187,144,249,240,188,16,73,146,118,33,186,255,254,207,226,214,219,110,197,165,75,87,240,226,11,191,64,16,36,184,
  117,106,43,102,79,31,71,28,119,37,13,92,212,247,240,12,250,41,171,174,52,2,90,252,32,221,159,231,33,240,60,248,126,13,113,188,12,106,35,83,
  230,89,201,207,236,202,149,6,214,172,89,139,141,155,110,69,148,36,216,188,101,71,206,16,248,21,12,86,3,196,81,11,27,55,173,74,181,86,125,109,
  79,55,106,195,247,66,120,30,80,169,12,160,217,108,96,245,248,86,120,158,135,110,119,9,195,35,171,16,250,29,86,7,36,255,87,6,20,38,192,96,
  154,228,28,16,20,155,187,56,204,105,117,63,55,225,69,51,5,206,87,250,90,198,45,108,209,126,252,220,115,152,159,159,199,223,124,253,235,168,86,171,
  120,241,165,151,48,115,226,4,158,120,252,113,76,110,218,132,31,63,247,28,190,244,197,47,162,217,106,225,219,223,249,14,38,38,38,240,244,83,79,1,
  0,190,245,237,111,163,211,233,224,155,223,248,6,218,237,118,214,49,239,192,193,131,120,237,245,215,241,133,207,127,30,83,219,182,97,250,240,97,188,122,
  224,0,118,239,218,133,189,123,246,224,192,193,131,120,243,208,33,101,108,15,236,221,139,93,59,119,98,126,126,30,87,251,109,175,101,230,87,180,1,7,
  144,141,245,239,254,203,127,193,151,190,248,69,172,90,181,10,63,122,246,89,108,158,156,132,231,121,88,88,88,192,204,137,19,240,60,15,83,219,182,41,
  158,211,141,70,3,47,188,244,18,230,231,231,179,78,139,135,166,167,51,123,168,86,171,133,217,185,57,108,158,156,4,0,92,184,112,1,115,103,206,100,
  13,83,4,3,189,119,207,30,236,218,185,19,157,78,7,223,253,254,247,51,63,232,249,249,121,252,248,185,231,176,243,158,123,240,192,222,189,56,122,236,
  24,94,126,229,21,252,217,147,79,98,211,198,141,120,249,149,87,240,206,145,35,5,102,162,86,171,101,237,179,211,32,178,131,95,189,252,50,142,207,204,
  32,73,18,220,50,53,149,53,173,1,242,14,138,0,176,239,161,135,240,169,187,238,82,238,163,211,233,100,222,209,189,94,15,151,46,93,194,169,211,167,
  149,38,13,54,233,14,183,255,80,201,154,45,171,96,218,204,93,27,0,81,141,158,201,53,195,102,209,164,99,192,77,108,179,28,124,148,89,255,28,179,
  228,106,203,104,146,30,185,156,23,166,102,19,220,1,77,237,40,139,221,9,115,160,145,74,64,60,3,35,239,41,192,210,165,5,120,17,68,251,125,87,
  6,209,142,91,239,25,157,238,147,73,214,114,153,22,52,211,118,209,162,113,74,234,19,157,23,176,9,41,97,127,247,182,2,233,220,169,35,33,157,24,
  19,9,72,122,8,195,162,15,175,8,74,56,151,13,89,50,18,199,9,90,173,101,248,126,53,3,226,149,74,128,229,229,5,214,165,166,215,235,245,11,
  19,243,230,51,237,118,7,3,3,149,254,186,247,160,243,116,206,25,224,88,210,28,139,70,30,194,126,46,215,40,231,243,194,99,36,16,148,117,150,1,
  46,5,240,137,161,209,90,194,190,111,155,100,35,15,162,228,127,47,22,201,150,33,38,86,138,29,101,255,250,223,63,118,244,89,169,151,110,111,9,85,
  79,67,53,157,44,11,189,185,77,146,22,187,169,26,39,223,200,88,233,192,13,7,174,116,169,59,253,195,145,163,51,15,240,18,196,113,216,95,184,93,
  84,42,3,120,226,137,39,177,97,253,56,94,63,248,58,214,76,172,197,3,159,222,135,127,251,245,63,225,197,151,126,6,223,135,98,63,227,121,97,31,
  120,183,51,123,155,52,157,87,65,28,119,225,121,49,60,191,138,77,155,166,112,229,242,5,44,47,95,74,31,110,80,67,167,219,234,123,67,171,204,14,
  32,54,40,57,29,157,64,204,5,83,84,53,55,119,26,159,249,244,3,152,88,59,138,4,253,69,16,199,72,144,32,142,34,4,97,5,64,140,36,138,
  225,249,1,106,3,3,240,18,160,219,233,32,74,98,36,30,16,6,1,252,44,149,148,22,101,212,170,21,204,95,56,175,117,91,224,216,73,93,209,19,
  213,49,113,21,174,212,154,72,110,28,99,106,8,65,109,223,76,115,102,165,227,158,57,113,2,227,227,227,88,59,49,1,32,111,255,60,49,49,129,247,
  63,248,0,107,198,199,177,176,176,128,11,23,46,96,230,196,137,172,157,246,198,13,27,178,235,109,158,156,196,66,31,236,206,156,56,129,227,51,51,89,
  199,188,48,12,177,165,223,26,124,243,228,36,94,237,95,59,73,18,124,112,252,184,178,17,220,117,231,157,89,147,148,137,137,9,76,244,199,20,199,49,
  102,231,230,208,104,52,48,60,60,140,109,55,223,12,207,243,80,173,86,49,115,226,4,186,221,110,246,220,255,104,223,190,236,247,0,224,255,253,187,191,
  67,16,4,120,244,145,71,50,144,59,58,58,138,47,63,253,52,246,238,217,131,253,63,251,89,246,236,228,6,45,116,19,145,129,123,146,36,105,207,206,
  56,198,129,131,7,177,121,114,18,19,19,19,120,242,241,199,49,49,49,129,78,167,131,23,94,122,201,202,36,114,224,232,177,71,31,205,26,194,136,247,
  241,232,35,143,224,194,252,60,70,71,70,148,32,32,73,18,236,188,231,30,180,90,45,28,126,235,173,236,122,244,62,178,206,134,73,130,118,187,157,221,
  135,142,169,53,233,153,233,222,37,75,67,116,32,152,106,199,185,204,130,107,161,34,245,129,213,177,169,58,153,2,199,118,154,138,247,92,101,74,38,105,
  135,201,61,132,27,191,174,66,223,213,166,138,27,19,151,34,230,93,3,2,7,176,174,182,255,166,153,195,98,0,85,212,8,23,245,172,158,194,30,83,
  137,98,16,208,6,24,197,54,224,156,54,186,219,237,33,12,3,82,139,35,52,205,158,196,106,123,18,27,45,127,86,46,105,81,129,180,220,6,58,82,
  210,253,50,80,163,222,213,130,8,145,179,132,105,103,68,143,245,199,23,5,95,113,156,130,213,74,37,47,38,236,118,155,253,34,187,56,11,70,242,130,
  74,17,224,228,22,111,97,216,70,163,209,204,10,0,205,96,49,33,109,179,125,73,6,18,67,28,91,130,53,167,236,174,188,174,5,171,44,186,66,230,
  204,111,82,96,228,139,110,25,200,206,110,209,56,70,150,15,81,135,14,110,254,139,130,72,113,95,156,86,58,93,23,130,64,44,214,182,209,122,163,178,
  216,145,218,134,254,254,177,99,241,157,235,200,134,196,100,99,167,160,236,62,72,225,58,197,153,14,19,90,168,163,219,140,203,176,176,38,61,34,13,0,
  210,177,7,8,195,42,170,213,176,175,151,10,241,185,63,254,19,108,219,54,137,239,124,251,251,56,127,225,12,110,191,237,78,252,15,143,127,30,135,222,
  120,173,31,205,250,136,162,30,54,108,216,138,118,231,42,110,185,101,59,146,36,193,145,35,239,224,214,219,182,163,18,86,49,59,123,10,103,207,124,128,
  209,177,13,24,29,93,143,201,201,181,120,240,193,79,225,213,87,223,192,171,191,125,25,158,23,162,18,134,136,250,93,147,40,96,44,78,38,31,97,88,
  1,208,69,146,132,72,146,174,54,141,126,232,205,215,113,252,253,247,176,220,108,32,138,125,196,81,4,63,160,94,185,62,124,36,240,189,0,95,120,252,
  139,232,182,26,120,229,149,87,224,249,126,10,180,227,8,149,176,214,127,246,61,36,137,135,110,175,215,31,47,159,85,160,150,134,166,148,135,188,32,76,
  239,81,61,32,161,24,176,139,223,165,190,206,46,158,227,215,58,238,183,223,121,7,247,223,119,31,70,70,70,178,174,121,237,118,27,155,39,39,49,54,
  54,134,90,173,134,195,111,189,149,182,146,158,153,193,45,83,83,88,59,49,129,245,235,215,247,91,209,214,48,62,62,158,253,238,7,199,143,99,126,126,
  30,243,243,243,88,219,7,193,162,227,221,196,196,4,42,149,10,38,38,38,112,122,118,22,87,175,94,205,158,139,239,251,88,187,118,109,6,194,95,124,
  233,165,140,253,22,95,71,143,29,195,222,61,123,48,217,103,152,1,96,102,102,38,171,246,142,162,8,51,39,78,224,167,251,247,227,111,190,254,117,0,
  192,234,213,171,49,63,63,143,191,255,214,183,112,231,142,29,184,227,246,219,51,128,57,214,111,113,45,198,112,234,244,105,60,191,127,63,254,227,95,255,
  53,106,181,26,94,61,112,0,167,103,103,241,244,83,79,33,12,67,12,14,14,66,46,70,22,239,227,133,151,94,194,87,158,121,38,3,238,135,166,167,
  209,232,7,20,166,13,75,188,111,161,111,14,195,16,27,55,108,64,146,36,120,126,255,126,204,157,57,163,252,158,232,116,120,122,118,22,207,239,223,159,
  181,205,190,235,206,59,113,248,173,183,50,104,51,59,59,139,159,62,255,60,254,231,191,249,27,84,42,21,28,56,120,16,31,126,244,17,254,252,75,95,
  66,181,86,195,200,200,136,210,197,144,219,183,108,62,209,38,121,140,14,248,153,106,71,86,90,132,168,147,109,216,246,90,113,191,52,224,117,177,234,211,
  29,76,58,219,66,157,190,219,86,19,192,221,43,149,191,112,122,73,238,32,214,177,86,69,87,132,216,96,227,72,83,212,30,202,20,24,166,255,175,202,
  132,84,192,235,41,122,225,98,122,62,81,90,134,171,76,98,194,202,120,114,176,43,103,229,160,248,65,203,122,102,161,27,22,207,53,15,194,168,95,181,
  44,41,145,223,121,164,88,151,138,231,100,146,81,137,64,64,180,57,87,231,139,176,146,75,208,237,118,208,110,247,48,56,56,8,223,71,223,213,35,33,
  228,84,212,7,123,158,228,17,157,62,171,102,115,89,121,206,66,170,33,164,45,50,112,46,90,211,65,145,122,136,49,201,239,87,103,61,169,106,226,85,
  135,21,83,51,48,78,178,193,129,103,221,58,82,231,20,37,28,217,93,69,11,126,109,95,178,245,28,23,64,127,188,177,99,104,101,179,67,63,55,116,
  44,157,134,55,117,7,163,66,109,154,106,212,221,184,206,118,73,238,88,102,218,184,139,197,53,49,60,47,65,167,189,212,103,133,67,76,77,221,132,233,
  195,111,227,252,133,121,0,30,90,173,84,94,241,199,159,219,135,85,171,87,227,216,209,247,81,9,124,220,191,231,30,120,62,16,6,62,162,56,193,93,
  119,221,134,225,161,10,170,213,26,222,120,179,130,243,231,78,227,137,39,191,128,122,109,16,239,29,123,23,94,146,224,200,59,239,244,63,63,66,167,219,
  196,232,232,132,171,56,35,147,134,120,94,79,217,0,10,254,192,72,176,176,184,208,127,46,221,116,147,76,104,33,88,140,8,64,148,244,224,251,85,44,
  44,45,98,169,181,172,252,76,19,45,237,162,144,63,83,164,198,228,103,203,49,34,178,14,210,196,158,201,169,58,249,208,163,198,237,101,139,177,104,70,
  228,90,198,221,235,245,112,230,236,217,140,69,158,159,159,199,123,239,191,143,79,63,248,32,38,55,109,130,231,121,56,61,155,218,31,206,206,205,101,64,
  120,243,228,36,142,28,61,138,209,209,81,220,180,101,11,78,207,206,162,211,233,96,110,110,46,3,193,123,247,236,201,36,16,2,16,223,122,203,45,24,
  29,25,193,209,163,71,21,175,234,56,142,251,242,35,253,215,59,71,142,96,207,253,247,227,150,169,169,140,21,158,57,113,2,190,239,163,82,169,32,8,
  2,180,219,109,116,58,157,236,119,42,97,136,145,145,17,124,229,153,103,0,164,45,202,95,124,233,37,252,217,147,79,102,221,23,197,51,23,218,126,17,
  24,136,107,201,7,169,188,241,136,247,190,118,66,157,251,130,197,151,247,1,78,183,43,222,187,220,112,70,92,179,211,237,106,247,34,211,230,236,245,229,
  41,0,208,106,181,16,134,33,218,237,182,210,192,193,164,181,43,171,213,227,138,98,232,253,113,235,131,147,51,216,252,81,233,103,202,206,18,101,191,86,
  226,40,67,229,87,54,143,113,91,87,73,23,224,190,18,173,180,237,190,109,159,167,43,82,151,199,154,51,143,185,30,86,20,146,209,75,171,122,106,207,
  16,104,196,25,171,75,187,118,202,54,106,41,171,234,73,141,58,82,151,138,32,240,11,44,126,122,46,170,86,98,233,58,128,212,97,208,35,153,88,79,
  98,34,19,169,136,206,151,218,127,123,25,107,172,186,128,248,74,147,150,244,94,83,169,101,165,82,65,179,217,70,165,82,97,36,26,49,106,181,170,66,
  62,229,93,22,5,94,232,34,138,252,254,190,149,186,118,164,99,142,229,144,158,37,74,168,67,137,144,200,200,50,46,185,147,32,149,62,168,5,127,113,
  223,234,15,44,120,150,63,75,45,182,244,10,118,120,186,121,43,131,110,126,142,123,90,137,36,109,215,77,53,211,116,137,232,48,23,119,62,151,89,55,
  38,41,220,199,11,59,218,207,153,48,3,24,43,216,68,185,212,32,253,55,234,209,75,233,121,14,176,233,138,102,76,15,92,247,162,235,245,65,212,170,
  53,52,22,251,142,26,232,33,8,60,92,156,191,10,160,11,207,11,176,188,220,130,231,121,184,227,246,173,184,120,97,30,15,239,187,183,207,244,190,131,
  192,247,112,247,206,59,17,248,192,170,209,1,252,228,185,231,241,249,47,60,130,161,161,65,60,242,216,231,177,97,221,56,218,237,8,39,78,206,97,211,
  198,53,120,248,225,63,198,254,253,63,77,175,13,15,189,168,7,160,138,176,82,197,234,213,171,49,58,58,138,53,107,38,80,27,168,35,142,35,116,59,
  93,44,45,94,69,163,177,136,86,187,137,78,167,139,165,165,6,188,164,135,180,184,49,54,104,254,2,69,167,198,219,197,120,168,15,84,208,235,117,21,
  166,70,119,136,113,207,145,2,12,151,3,152,118,186,227,22,158,181,211,15,251,111,62,187,129,252,46,198,61,51,51,131,7,246,238,197,232,232,40,142,
  28,61,138,51,103,207,194,243,60,220,187,123,119,198,38,11,192,122,188,207,66,87,171,85,204,207,207,163,211,233,96,106,219,54,212,106,53,165,181,244,
  241,153,25,236,221,179,39,211,3,159,158,157,69,187,221,206,216,223,163,199,142,97,199,246,237,216,247,208,67,0,128,255,251,111,255,22,151,46,165,146,
  160,169,109,219,240,205,111,124,35,3,130,226,249,117,187,93,156,56,121,18,83,219,182,33,142,99,188,251,222,123,172,75,1,125,54,2,196,207,207,207,
  227,208,244,52,238,254,212,167,210,247,67,93,45,164,84,150,236,209,105,250,170,86,171,120,96,239,222,126,17,235,107,216,115,255,253,216,181,115,39,102,
  103,103,49,59,55,87,212,154,145,141,243,43,207,60,131,145,145,145,76,15,126,122,110,14,147,27,55,226,203,79,63,173,124,206,243,251,247,99,118,118,
  22,59,239,185,7,155,39,39,241,191,125,243,155,10,51,175,99,117,57,189,30,183,39,201,233,201,178,69,159,38,249,130,14,44,186,50,42,116,190,235,
  44,225,76,21,238,197,148,110,92,170,152,143,30,116,54,130,133,234,88,105,166,200,244,220,108,172,53,101,89,233,189,232,10,21,101,166,142,2,2,93,
  80,86,4,209,50,216,148,237,232,162,66,90,156,62,86,42,53,145,165,13,197,207,243,36,23,139,184,64,64,8,208,156,167,247,33,177,183,197,2,67,
  53,120,5,219,185,80,110,192,34,108,239,210,117,32,0,103,156,57,130,80,57,128,58,255,98,165,139,159,0,144,169,235,133,42,5,200,211,242,41,0,
  246,125,175,223,28,37,191,215,165,165,37,101,63,74,219,147,71,10,184,207,93,77,60,116,187,233,239,183,90,221,76,50,145,190,163,4,194,201,67,102,
  152,69,163,148,116,172,177,148,189,240,11,78,25,116,62,234,88,103,33,33,73,159,143,14,132,219,27,163,240,114,31,40,1,155,220,37,51,159,251,69,
  240,108,146,166,233,100,142,242,218,162,160,211,134,19,77,178,178,143,27,118,52,201,196,196,247,66,211,70,229,34,192,54,109,106,121,122,33,80,54,75,
  151,246,181,58,170,222,228,19,203,177,61,245,90,21,240,125,44,46,53,251,147,43,45,58,88,61,62,14,32,64,88,25,193,218,181,155,225,121,30,62,
  248,96,6,191,126,249,183,248,15,255,211,95,166,12,85,23,216,184,113,28,221,62,219,246,193,7,31,226,196,135,167,112,234,212,71,216,190,99,7,58,
  237,14,222,154,158,198,221,59,119,98,207,222,123,49,127,225,42,238,217,117,27,238,190,103,15,222,126,235,13,172,30,159,192,200,232,8,254,252,47,190,
  138,85,99,3,168,85,2,36,113,140,229,165,37,44,46,46,162,151,68,24,168,85,17,134,91,16,86,171,8,131,10,252,32,64,179,221,69,167,211,69,
  115,185,137,86,187,135,133,43,87,208,88,108,160,213,90,70,171,217,65,171,189,132,229,229,102,26,105,247,109,126,122,189,101,132,225,32,122,189,38,177,
  71,74,16,248,62,162,136,179,79,202,187,110,201,207,147,211,252,232,14,184,149,176,68,166,223,181,95,87,95,108,117,189,199,125,124,102,38,3,178,51,
  39,78,224,226,197,139,152,159,159,199,196,196,68,1,156,205,205,205,225,206,29,59,208,110,183,49,115,226,4,170,213,42,246,238,217,131,145,145,17,204,
  156,56,145,253,92,163,207,102,139,107,120,158,135,163,199,142,97,231,61,247,224,248,204,12,90,173,86,97,60,51,39,78,224,208,244,116,86,68,40,88,
  108,249,235,200,145,35,152,218,182,13,190,239,99,102,102,198,40,91,17,223,63,62,51,131,169,169,41,76,110,218,132,111,126,227,27,56,122,236,24,58,
  157,14,106,213,170,209,185,194,69,22,243,240,190,125,24,25,25,193,169,211,167,241,230,161,67,24,30,30,198,157,59,118,224,225,125,251,240,163,103,159,
  45,174,111,139,147,194,203,47,191,140,7,246,238,85,138,8,95,61,112,0,179,125,102,255,165,127,254,103,60,176,119,47,134,135,135,211,224,119,122,58,
  115,249,224,216,82,93,192,69,53,177,148,221,229,192,23,167,239,231,50,29,58,102,91,102,187,108,0,90,214,77,235,230,185,12,244,93,218,220,82,207,
  116,155,92,194,22,184,154,152,102,42,97,163,141,64,92,109,85,117,231,131,173,91,38,183,223,232,10,18,139,172,173,91,96,81,204,0,36,70,16,93,
  60,3,249,66,192,28,88,123,4,0,123,160,5,96,130,125,78,165,14,126,161,192,80,181,198,83,83,250,189,94,148,129,237,34,115,238,21,186,237,165,
  242,129,68,114,167,240,36,246,215,83,36,33,253,89,12,223,15,20,144,43,99,3,225,11,157,22,228,197,104,181,90,8,130,90,182,214,150,151,151,209,
  235,181,17,69,93,5,120,118,187,93,12,13,13,33,138,82,15,103,209,20,5,8,209,237,118,81,169,4,104,183,59,10,195,45,24,115,234,189,44,128,
  181,74,220,240,192,89,100,227,228,76,2,39,211,16,239,94,196,144,114,192,99,194,68,54,151,13,206,187,89,100,10,84,169,137,12,216,237,251,131,78,
  47,172,35,219,76,103,171,110,47,249,56,99,71,23,31,125,239,255,248,223,255,247,196,239,127,243,91,223,249,142,209,23,214,102,170,79,219,239,114,182,
  79,186,110,81,182,194,19,154,98,21,133,4,114,218,157,163,254,215,78,172,199,51,207,252,37,126,240,15,207,225,194,133,83,0,128,125,159,253,12,118,
  221,123,63,78,157,154,199,170,85,195,8,195,4,167,62,154,197,63,255,242,151,232,118,186,248,31,191,252,20,218,157,14,54,108,220,128,171,87,47,225,
  215,191,62,128,230,114,11,173,78,132,70,99,30,67,67,67,184,115,251,29,248,240,228,44,206,95,60,135,251,118,221,135,102,187,131,123,119,239,196,234,
  241,213,248,229,47,95,193,123,239,127,128,255,248,31,191,6,36,17,222,123,247,40,46,95,186,138,203,151,46,227,202,194,2,26,75,139,82,186,168,31,
  201,248,33,6,6,6,81,175,215,48,56,80,199,208,208,32,234,3,35,24,24,8,49,52,56,132,90,125,16,131,67,131,8,194,16,149,74,45,93,116,
  153,102,171,111,197,3,31,221,94,132,118,183,7,196,9,218,157,38,26,141,22,182,223,190,5,135,222,56,136,55,14,189,225,100,137,101,59,192,169,248,
  159,78,76,57,101,34,82,186,166,235,113,12,251,181,52,229,184,145,199,189,82,139,72,225,166,209,104,52,240,221,239,127,223,248,110,77,114,0,153,57,
  178,21,145,149,1,22,182,72,222,244,59,156,238,216,246,59,186,170,116,186,39,153,158,181,236,141,172,171,196,166,207,128,22,15,210,235,203,190,213,101,
  93,139,202,130,74,29,224,230,138,238,168,52,192,229,250,166,22,218,186,185,108,147,151,184,88,92,165,69,83,61,163,251,7,93,239,212,138,80,119,254,
  172,196,77,72,255,236,35,6,156,203,1,5,216,70,46,106,208,161,219,171,139,141,74,138,103,168,0,198,158,34,203,80,255,173,8,26,132,116,66,14,
  104,100,112,38,10,42,197,248,211,226,191,36,3,210,178,102,86,118,22,17,192,78,150,183,84,42,21,140,143,175,195,226,226,114,102,67,151,119,21,140,
  177,176,112,25,213,106,13,158,87,205,250,42,212,106,21,92,186,116,30,73,18,73,224,209,131,231,85,48,58,58,138,86,171,129,86,171,169,104,211,235,
  245,1,116,58,169,77,157,0,157,162,136,47,151,149,32,211,133,231,82,190,40,147,188,200,253,24,184,238,126,54,173,115,222,33,48,207,12,0,106,1,
  32,149,114,228,207,79,199,80,243,251,156,240,217,86,155,2,37,22,208,105,199,92,20,204,202,242,71,29,118,180,237,33,31,87,236,168,251,250,235,175,
  125,45,251,123,232,51,244,54,189,169,178,169,74,153,81,224,218,44,218,210,4,174,63,71,89,7,238,115,170,149,42,18,36,104,181,150,178,135,253,202,
  175,95,197,165,139,87,176,106,213,24,142,189,125,30,31,205,206,98,185,25,1,232,2,30,240,163,103,127,2,120,1,234,213,16,203,173,165,172,98,85,
  116,109,106,181,90,120,253,205,67,253,207,12,241,230,225,55,1,0,199,222,61,138,193,250,16,150,91,139,240,60,15,75,139,75,152,61,117,14,255,250,
  202,175,173,69,54,189,184,135,198,210,34,150,154,29,204,95,154,103,244,78,85,120,94,154,126,170,85,171,24,168,215,49,56,56,128,32,8,225,7,62,
  194,32,68,173,90,69,181,90,69,173,86,67,165,86,195,232,240,48,214,140,85,17,120,17,106,245,1,54,106,164,105,222,50,222,173,186,195,148,186,41,
  136,3,143,3,20,174,62,183,215,2,34,62,174,227,190,30,95,213,106,21,79,62,254,56,62,56,126,28,7,14,30,196,241,153,25,227,189,186,166,230,
  213,98,163,196,41,53,167,187,182,173,160,75,6,87,66,255,109,242,101,54,141,91,231,119,44,87,242,235,152,94,122,0,136,226,38,23,189,174,203,230,
  75,155,190,112,13,110,92,130,14,250,204,116,141,100,228,241,216,164,28,58,16,204,201,64,228,113,203,129,10,101,227,117,29,50,87,226,3,203,201,192,
  76,173,216,139,197,120,170,135,243,74,26,42,185,239,67,73,95,122,224,51,140,51,237,96,232,43,29,8,169,244,141,115,234,40,54,95,73,52,82,23,
  145,97,204,63,71,252,17,94,207,180,57,134,108,147,39,75,73,228,172,165,106,183,38,2,238,72,186,239,28,192,11,143,100,81,88,41,10,244,196,181,
  163,40,234,55,61,73,10,158,253,105,167,194,212,122,174,217,108,101,235,209,247,227,140,125,150,245,171,190,95,65,183,219,235,51,239,57,43,238,121,192,
  242,242,98,214,181,49,15,4,196,188,21,129,132,218,50,189,219,77,37,148,169,27,72,192,130,228,180,27,98,156,189,131,162,13,26,32,55,144,145,29,
  78,196,179,75,193,116,222,93,82,213,50,235,89,103,250,238,213,160,199,147,58,74,114,146,13,183,179,146,35,34,40,153,192,173,53,185,65,201,141,138,
  29,93,206,206,80,160,113,202,158,208,3,212,196,6,136,107,200,122,24,219,193,32,15,70,176,9,182,226,146,50,27,108,54,169,146,94,22,229,137,137,
  25,132,3,120,127,230,44,128,115,240,125,32,74,106,24,30,30,76,253,38,51,143,74,31,221,110,19,245,129,17,132,97,29,3,131,163,240,131,24,181,
  234,32,170,213,58,170,149,0,126,16,160,86,169,195,11,18,84,170,33,130,32,68,53,172,98,104,104,24,73,18,97,237,186,9,188,246,218,180,54,181,
  81,156,156,49,146,164,35,109,82,50,67,149,183,215,238,246,122,104,47,44,96,129,52,110,209,177,131,79,62,241,20,70,70,71,217,67,163,44,115,168,
  59,100,233,38,76,89,94,185,139,148,46,77,170,107,236,97,99,135,76,213,250,31,231,113,115,160,205,84,128,201,125,191,211,233,224,199,207,61,199,106,
  231,116,169,50,93,90,155,22,215,148,13,170,204,78,5,110,21,212,226,89,83,205,163,110,156,189,94,47,43,62,50,121,23,219,238,65,236,129,244,125,
  232,236,139,184,66,63,219,151,9,232,186,30,46,244,243,117,204,143,233,89,235,2,43,93,31,0,83,32,36,175,29,221,179,114,241,138,119,221,139,68,
  192,64,153,91,151,2,119,89,7,237,194,254,211,32,79,72,109,116,239,145,178,195,121,186,156,182,234,142,178,159,203,181,191,166,128,194,35,18,142,132,
  176,207,94,193,51,90,5,73,94,230,22,66,117,218,212,111,154,50,146,178,133,94,62,46,217,50,81,0,72,181,184,79,120,44,231,173,170,99,69,222,
  144,23,1,70,104,52,22,81,173,214,149,96,45,125,246,66,151,28,97,100,100,0,237,118,27,97,24,98,121,185,89,104,72,82,171,85,16,4,149,254,
  231,196,217,231,9,144,43,131,247,92,54,34,36,20,177,18,212,228,239,217,207,108,234,196,124,80,25,100,100,247,146,183,249,46,182,251,86,165,45,137,
  244,25,242,243,149,247,78,79,242,101,246,180,146,13,157,125,27,39,217,160,191,27,69,61,73,134,103,47,230,211,237,21,242,58,148,11,96,109,242,193,
  27,1,59,210,61,140,251,217,48,251,166,163,222,89,247,129,178,38,210,196,68,217,54,101,151,77,86,231,24,161,99,232,130,192,195,253,123,247,162,82,
  245,49,190,106,29,6,234,85,132,149,10,60,97,255,3,0,73,12,15,49,224,133,64,146,192,243,67,36,136,224,37,49,188,32,76,29,23,227,212,146,
  206,247,124,36,177,135,24,61,244,186,61,36,113,234,246,144,68,9,186,189,38,90,237,43,240,224,227,253,99,71,113,230,236,108,127,36,21,0,29,118,
  18,234,244,114,197,182,177,244,96,83,11,8,121,29,79,8,207,31,128,239,47,20,38,32,77,65,201,233,60,93,250,85,188,51,25,180,208,207,213,165,
  86,105,154,166,8,208,138,5,130,52,229,162,75,51,255,126,199,173,143,122,77,227,230,14,112,83,113,158,107,170,95,199,252,217,54,19,57,136,46,11,
  242,76,107,82,126,110,180,34,154,130,65,27,187,96,147,141,200,169,63,250,115,38,89,142,205,226,200,180,145,155,186,65,154,190,202,28,20,220,123,228,
  88,124,215,177,175,228,16,225,10,222,92,216,127,93,80,83,54,179,36,7,75,156,181,170,110,253,113,77,171,202,100,88,116,182,86,14,43,163,0,124,
  84,39,13,207,153,253,162,64,153,178,209,50,72,87,237,184,228,182,217,73,6,128,243,123,20,12,108,44,73,47,146,2,211,41,216,90,213,205,33,151,
  102,8,16,153,23,175,201,255,238,73,122,223,36,43,72,20,93,14,171,213,42,186,221,110,214,16,75,220,239,242,242,82,198,108,119,58,11,24,28,28,
  235,59,9,69,132,233,21,251,102,148,201,22,100,247,16,25,56,170,172,173,204,4,67,98,166,125,242,59,42,128,74,221,62,2,237,190,164,178,244,242,
  217,154,20,58,254,201,26,101,121,12,121,171,117,189,222,153,39,43,64,130,183,164,192,58,171,117,79,170,103,181,110,254,155,214,153,158,16,188,177,177,
  163,28,52,21,156,132,250,111,57,20,213,239,190,230,128,213,233,37,57,127,84,91,36,79,7,69,171,158,229,84,171,201,131,143,58,73,152,216,141,179,
  103,206,96,246,163,83,216,178,113,13,46,93,152,199,241,163,135,177,220,92,194,82,115,9,173,102,27,221,94,23,97,37,173,42,142,163,110,234,159,236,
  5,168,84,195,188,184,38,9,208,110,46,161,213,238,244,55,156,4,81,148,58,122,36,134,67,85,157,132,185,53,157,217,158,10,138,6,205,190,193,6,
  72,18,31,158,215,213,60,175,8,73,210,70,224,135,164,159,189,24,139,47,45,88,21,140,83,102,75,46,32,227,230,138,204,14,201,86,108,180,93,174,
  46,53,44,167,186,228,8,55,12,67,39,79,220,223,223,184,249,200,220,101,220,58,13,151,77,242,83,214,31,216,5,44,93,171,223,175,9,208,83,123,
  46,19,136,227,10,243,56,73,153,208,170,155,178,59,182,38,30,116,141,137,207,149,3,25,83,215,59,121,126,149,149,67,185,20,178,114,141,5,116,115,
  197,180,255,234,14,26,177,23,151,153,51,101,253,88,87,146,205,112,61,124,117,93,205,196,59,227,138,93,117,103,152,105,79,161,157,76,77,231,163,249,
  122,130,121,166,41,122,213,155,89,62,31,57,16,84,236,82,88,4,199,249,253,209,198,25,113,159,172,200,65,52,47,49,73,164,70,46,80,154,186,200,
  224,57,47,22,76,191,223,235,197,89,119,68,25,196,170,96,58,191,78,187,221,234,159,99,233,207,117,187,221,44,96,202,139,249,124,180,219,109,0,62,
  214,172,89,133,11,23,206,42,115,184,219,237,98,121,185,131,181,107,215,96,121,185,199,50,193,169,182,58,33,93,35,227,130,123,73,174,119,230,117,206,
  92,103,92,209,141,80,102,153,101,192,158,51,191,177,196,94,39,202,56,213,32,135,147,108,36,150,253,68,254,59,143,185,210,226,77,117,30,82,121,155,
  14,115,137,0,71,151,13,163,242,174,27,29,59,230,13,111,162,130,132,77,188,169,208,35,236,19,101,105,92,187,74,149,97,170,92,216,66,151,67,199,
  37,125,186,220,106,226,71,207,61,171,73,33,248,80,45,224,120,139,52,62,176,72,55,34,153,9,214,189,16,155,135,171,252,178,68,58,41,191,55,253,
  152,68,186,138,130,237,130,69,76,16,162,185,180,76,54,236,68,137,70,169,39,47,5,14,130,201,17,19,184,82,169,20,38,161,173,145,129,109,17,112,
  243,204,133,245,185,81,199,45,3,15,25,180,211,40,187,140,181,153,139,196,130,155,163,194,15,249,90,165,16,182,207,118,209,160,234,92,45,116,76,40,
  55,46,186,225,115,204,163,203,243,212,21,213,113,173,113,133,102,111,165,122,252,178,44,177,237,94,138,205,39,86,222,136,192,86,132,199,93,139,123,199,
  242,161,164,147,95,232,158,5,213,143,235,0,181,124,128,235,108,193,108,149,246,242,220,231,82,203,101,129,179,14,236,228,64,88,180,174,78,50,231,11,
  181,209,134,167,213,70,171,115,37,7,192,50,41,194,177,214,28,27,45,60,156,101,47,105,25,160,228,254,206,42,211,25,199,2,192,196,74,91,104,249,
  250,69,48,157,254,123,173,86,69,171,213,203,246,234,116,159,246,176,176,176,172,124,94,20,245,80,169,212,21,43,63,153,212,24,24,168,99,121,121,177,
  255,253,226,123,23,0,87,5,171,73,118,134,231,236,185,79,218,134,155,236,229,84,0,172,2,247,162,183,179,252,190,125,63,64,20,197,125,224,154,192,
  230,178,225,2,156,41,86,40,206,107,61,230,114,169,97,225,214,133,239,251,89,6,225,15,9,59,202,77,221,116,251,71,40,255,3,167,93,145,83,174,
  66,15,122,45,7,170,60,241,57,102,135,218,24,209,65,23,35,136,28,4,174,228,112,87,239,215,71,234,189,12,45,99,33,79,212,226,225,163,86,53,
  115,86,83,186,141,154,211,156,235,54,202,188,24,33,253,204,188,157,107,165,207,56,71,202,231,212,170,1,174,92,110,146,212,162,57,37,67,65,28,45,
  238,50,1,28,122,191,84,26,80,198,239,150,77,159,48,233,226,27,117,220,54,176,38,111,96,92,16,166,3,54,92,202,203,196,202,114,157,33,93,25,
  83,215,244,155,12,110,92,181,210,180,101,171,137,53,144,199,81,86,78,65,63,95,102,160,92,246,21,153,233,84,215,111,82,170,152,83,213,176,38,90,
  128,169,115,23,209,205,53,83,51,3,221,120,196,222,111,178,150,163,123,186,109,239,119,209,57,150,125,87,166,127,179,141,91,119,206,152,44,175,86,218,
  96,135,35,64,138,93,231,104,192,227,177,13,88,76,103,131,78,223,204,101,53,114,123,59,42,31,240,10,108,52,221,39,114,105,8,148,185,154,106,135,
  233,245,0,89,210,34,203,60,218,237,54,186,221,24,213,190,125,102,189,94,195,133,11,115,82,71,63,245,189,44,45,45,145,2,189,84,67,220,110,47,
  19,223,229,188,40,48,119,185,136,251,207,64,4,111,62,27,108,217,128,115,14,220,19,229,121,115,172,51,29,83,46,135,65,214,205,81,6,220,185,173,
  36,95,140,167,107,185,205,129,103,213,83,121,101,152,139,102,4,57,235,203,27,19,59,38,215,84,76,156,181,242,54,85,34,122,22,127,86,23,27,32,
  42,89,176,53,120,176,165,210,212,235,38,164,186,22,86,214,37,31,55,221,244,35,235,102,75,25,225,56,70,191,120,161,99,5,165,46,17,88,254,188,
  253,126,170,77,23,209,113,11,59,206,180,108,242,181,91,237,14,42,149,26,217,36,101,214,89,108,180,21,120,126,136,32,8,80,171,14,33,138,123,232,
  180,23,50,150,219,212,28,133,59,144,138,149,187,158,84,64,179,50,54,211,6,2,77,172,235,202,199,13,182,83,221,245,24,183,105,126,187,180,18,54,
  181,56,151,153,119,153,253,229,174,37,7,110,156,132,226,90,64,131,109,61,218,211,222,137,177,189,243,181,178,227,46,12,179,237,61,82,219,52,221,51,
  48,21,151,234,216,246,50,247,106,10,12,185,123,51,61,123,93,80,224,250,121,166,115,162,236,218,49,145,17,52,8,16,115,157,43,208,117,145,205,152,
  192,180,137,193,42,15,8,124,9,232,202,128,63,129,234,55,204,131,104,153,201,149,9,23,153,33,22,157,9,121,192,39,203,48,82,153,132,218,197,48,
  130,231,5,16,143,80,6,99,180,37,59,77,113,171,129,175,252,60,161,176,193,158,231,161,217,108,194,243,42,88,90,90,66,181,90,133,231,213,149,245,
  158,159,211,98,63,19,237,178,99,105,111,246,52,103,169,170,99,165,1,129,142,241,181,1,103,181,251,32,10,125,21,132,69,158,174,230,72,188,123,174,
  157,182,28,56,233,214,77,46,23,225,240,128,122,157,220,23,250,218,48,151,62,155,113,163,98,71,24,219,140,155,198,147,101,197,228,137,68,15,86,153,
  18,183,21,142,184,54,212,40,235,57,204,85,156,235,92,17,100,173,138,11,69,79,31,30,253,125,46,194,79,23,72,0,209,33,48,237,196,212,210,178,
  141,101,15,124,241,14,30,122,232,49,188,243,206,97,92,190,124,6,105,193,66,146,125,166,24,107,122,56,200,19,188,39,45,30,145,234,138,208,237,196,
  152,88,179,22,107,38,182,98,120,120,20,35,163,35,8,66,31,131,3,195,24,24,8,80,175,13,162,94,15,81,175,215,16,134,33,42,149,106,182,120,
  47,94,186,136,231,126,252,67,80,187,28,83,170,70,207,46,169,58,86,87,143,89,174,32,205,180,120,116,11,123,229,227,198,239,108,220,242,245,233,252,
  161,32,65,167,195,214,205,37,89,207,75,217,95,90,128,102,90,195,38,230,192,197,143,217,164,33,118,209,193,137,107,112,169,70,10,162,108,27,183,91,
  38,170,28,192,215,237,139,58,64,38,179,74,101,155,16,209,223,201,61,115,61,237,97,102,59,120,108,58,64,170,141,228,214,29,221,111,203,250,153,115,
  77,92,108,85,245,180,155,158,142,141,178,101,46,116,197,152,242,126,145,91,168,233,155,75,152,220,170,138,246,102,92,49,116,14,50,139,247,11,101,175,
  215,177,209,178,140,128,43,48,76,139,253,98,77,246,136,118,49,12,164,235,229,89,75,110,173,81,55,135,156,77,77,72,86,150,190,207,164,95,156,31,
  33,8,6,37,135,148,226,26,137,227,8,97,152,160,90,173,98,105,41,119,255,200,65,113,76,58,243,201,128,56,46,248,89,211,159,225,2,78,29,112,
  206,125,157,229,247,38,55,134,73,36,7,146,68,185,223,156,221,79,10,123,107,174,201,230,64,91,62,110,249,122,220,28,82,217,126,56,215,204,152,48,
  23,13,154,76,235,247,147,130,29,67,110,19,228,24,8,29,26,215,165,182,86,82,108,196,109,130,116,177,139,151,200,109,122,28,99,162,139,68,116,12,
  66,78,245,135,72,237,112,192,176,28,49,97,115,125,118,114,148,73,233,169,227,242,48,181,109,61,38,38,246,97,126,254,10,42,181,65,116,218,29,52,
  91,77,116,58,45,196,113,130,56,138,208,238,44,35,12,235,8,250,169,168,176,226,163,94,31,68,181,22,96,100,100,12,129,239,97,104,104,16,19,107,
  70,80,9,125,60,243,204,147,240,61,31,221,110,7,237,102,11,237,86,19,221,110,23,205,246,50,46,157,191,132,197,197,101,196,113,5,205,214,18,122,
  81,7,219,182,77,225,206,187,238,194,208,208,16,150,150,22,11,239,197,198,38,186,176,134,166,231,161,75,95,115,155,1,119,152,210,3,234,227,56,110,
  221,92,165,122,101,58,79,77,169,125,250,57,186,116,151,110,124,43,233,32,105,3,60,166,251,181,21,113,136,159,225,124,153,233,218,209,89,186,217,64,
  162,110,179,150,215,178,171,28,64,231,50,98,3,108,92,251,104,93,26,83,206,26,202,236,31,7,168,197,247,116,129,26,173,78,167,123,170,124,32,153,
  230,182,46,224,226,124,231,233,181,109,140,17,247,236,92,26,235,200,115,221,100,71,197,217,80,202,7,188,173,1,76,153,32,206,94,184,168,243,142,246,
  216,78,134,170,198,55,7,112,190,143,130,222,89,116,38,164,50,65,97,107,38,203,58,210,224,201,87,192,161,12,212,117,160,90,6,112,106,99,144,136,
  169,179,144,247,186,8,64,69,10,120,98,197,159,185,86,27,64,179,217,196,240,240,64,31,16,123,153,69,92,238,123,205,175,251,84,155,29,19,23,19,
  149,52,211,237,79,105,125,72,160,0,214,162,175,115,146,181,40,79,231,182,207,204,125,185,104,159,183,167,19,76,49,159,201,161,25,5,221,58,243,74,
  101,63,93,49,23,221,11,77,197,193,159,20,236,24,202,23,226,124,46,41,48,113,177,160,114,169,10,55,177,135,220,127,117,58,79,155,109,18,151,154,
  231,126,79,108,152,226,191,81,148,106,139,57,64,124,45,192,197,37,93,233,121,85,180,218,61,172,90,53,132,122,45,213,148,213,170,171,224,5,1,124,
  15,8,194,10,144,0,97,88,145,170,255,83,11,159,94,47,66,171,221,68,183,221,65,212,141,176,188,120,9,111,125,248,46,26,139,45,44,47,94,193,
  226,226,18,46,47,44,160,217,110,177,207,114,207,3,143,224,194,236,105,92,184,112,6,11,141,14,182,239,216,161,5,96,182,182,154,28,211,105,94,28,
  69,109,182,142,201,162,166,237,46,192,229,119,55,110,59,163,166,27,183,110,173,228,26,69,115,10,204,198,12,218,64,164,75,176,64,217,95,91,106,203,
  181,59,97,217,34,58,110,77,113,157,19,105,183,63,151,103,197,49,154,212,218,78,0,81,83,129,40,181,167,228,178,10,166,249,162,11,196,229,20,38,
  215,16,133,43,64,229,138,100,117,217,133,50,172,139,203,123,213,105,217,105,160,106,146,187,216,52,241,101,201,9,238,29,152,92,72,116,157,56,117,227,
  167,207,210,84,8,90,220,79,100,22,181,232,215,28,199,121,215,189,20,232,234,214,116,209,143,89,199,70,115,115,68,102,155,115,237,50,181,94,67,214,
  121,80,214,233,234,3,65,191,192,142,83,0,159,234,100,211,117,213,106,181,21,102,87,220,151,239,251,168,84,66,44,46,54,148,127,163,235,129,103,63,
  229,6,50,234,122,44,178,192,233,243,19,50,140,20,60,171,174,30,57,104,79,50,59,62,154,33,224,59,21,38,44,238,16,247,174,119,107,42,102,115,
  41,51,253,187,198,92,84,82,40,95,231,147,132,29,229,175,144,99,27,116,169,216,50,32,176,12,251,170,107,55,107,234,212,37,167,65,117,253,216,185,
  180,153,137,125,83,15,161,152,221,180,117,221,210,116,109,47,77,233,106,202,218,228,215,243,49,50,60,130,119,142,188,141,223,254,246,215,226,142,49,177,
  102,35,38,38,214,161,215,137,48,123,230,56,42,129,143,48,8,16,197,49,122,113,23,72,60,180,59,29,140,142,77,160,185,188,132,102,171,209,191,223,
  33,76,110,190,5,203,203,17,174,94,105,161,215,107,21,128,171,239,87,80,31,88,131,251,238,221,142,171,151,82,0,189,180,180,4,192,195,234,213,171,
  176,176,112,213,10,128,116,77,58,76,237,115,109,30,216,38,192,97,3,115,174,192,237,218,199,141,107,30,55,247,255,212,25,195,69,63,204,109,90,38,
  80,226,154,102,167,17,186,174,232,76,215,38,93,62,164,108,159,41,175,37,110,163,227,154,228,228,77,14,18,101,19,231,180,153,58,6,157,123,102,242,
  62,98,3,207,186,253,207,164,225,230,108,234,228,159,215,201,116,116,190,168,58,54,180,204,123,213,53,84,17,215,167,174,63,226,231,69,215,78,249,186,
  38,32,237,194,214,186,140,205,229,125,200,251,180,90,68,231,173,200,150,79,118,149,209,173,81,155,243,18,247,62,211,57,150,183,94,78,109,207,2,134,
  240,73,140,123,148,204,84,170,107,54,111,31,45,179,209,250,224,46,206,206,136,220,10,47,81,192,182,214,43,55,123,103,177,196,158,171,126,213,114,179,
  19,209,66,59,45,174,140,10,192,118,105,169,1,32,65,171,21,25,125,192,139,1,113,254,204,232,218,227,231,101,140,40,202,221,72,210,119,224,75,44,
  183,90,24,73,11,234,56,224,44,91,212,21,215,31,74,128,103,147,100,227,119,135,185,108,178,201,79,42,118,12,109,55,124,61,12,240,233,117,116,45,
  101,185,131,199,37,77,231,10,162,76,32,196,181,50,147,54,219,224,24,31,93,241,151,206,255,80,101,137,60,12,15,143,96,112,168,134,133,133,70,255,
  51,71,112,223,222,7,241,169,59,111,129,239,3,213,74,5,239,207,76,225,151,47,253,18,221,206,229,194,193,114,219,109,219,177,97,195,58,252,228,39,
  255,128,32,28,193,83,95,252,18,198,199,170,168,214,107,152,155,187,138,131,7,14,97,100,213,16,142,188,117,16,158,215,67,16,14,98,223,31,61,130,
  205,147,27,81,175,85,112,243,45,183,227,189,247,143,99,113,241,50,150,151,91,88,191,110,61,78,158,60,89,10,156,234,138,143,202,154,152,235,14,51,
  215,54,243,174,159,245,251,24,183,173,131,156,204,162,186,234,174,169,78,205,181,117,183,14,224,232,0,28,247,44,4,104,162,224,175,44,224,113,233,14,
  199,201,115,76,12,175,77,47,174,99,65,56,175,232,178,123,32,199,14,235,178,87,180,157,188,14,60,114,223,211,237,171,92,243,0,83,241,14,215,24,
  134,211,40,203,65,139,144,216,200,192,68,23,148,216,186,161,114,227,177,105,37,109,107,221,214,157,179,172,243,140,139,133,160,60,95,77,65,174,236,98,
  32,7,104,170,255,44,10,30,204,121,219,109,254,60,147,45,241,40,160,77,37,6,50,59,107,210,86,139,130,182,24,114,251,112,185,211,160,92,200,95,
  148,22,160,32,91,144,229,32,114,80,16,4,105,22,85,72,46,100,7,13,29,46,80,159,143,120,118,49,51,207,80,96,38,105,246,89,176,201,121,195,
  27,89,187,173,50,243,84,78,84,60,251,101,185,11,152,181,0,163,164,175,216,114,91,111,111,247,187,196,92,156,43,71,153,245,245,135,140,29,67,29,
  203,32,35,116,93,3,138,178,5,48,38,77,27,181,23,161,81,13,167,69,116,177,119,42,123,248,185,166,8,233,24,168,230,134,218,53,233,140,199,139,
  159,147,32,12,124,4,129,143,230,114,170,59,222,176,113,3,246,222,187,29,63,123,126,63,46,94,94,192,142,237,183,98,245,248,24,190,244,165,47,226,
  248,241,19,120,231,237,105,52,155,87,48,182,122,18,139,11,151,48,56,56,136,45,91,214,163,82,29,198,167,62,181,27,94,220,196,223,253,221,183,49,
  52,52,140,199,30,123,4,79,60,249,48,194,208,195,7,239,190,137,161,145,245,248,194,23,30,133,159,180,208,90,58,143,100,124,16,171,134,234,168,134,
  1,218,221,46,102,103,79,97,203,77,55,225,213,3,175,178,126,174,54,64,198,1,93,202,24,186,248,199,186,46,88,23,166,231,227,50,110,147,131,134,
  107,224,103,90,83,220,117,116,25,20,87,93,178,109,188,58,201,128,220,18,93,94,11,148,213,229,128,26,77,155,115,1,79,25,112,171,123,231,220,181,
  168,21,211,74,174,45,155,252,211,107,184,200,143,232,94,35,3,84,147,84,135,211,54,219,138,23,77,62,232,174,76,145,206,58,175,204,28,42,147,93,
  209,173,127,49,207,229,160,146,3,0,166,103,232,210,97,210,4,20,116,158,212,180,173,56,199,174,167,69,124,197,98,176,124,124,42,184,214,61,11,179,
  103,52,50,23,9,234,61,45,207,135,116,63,132,196,62,23,153,81,25,100,11,192,170,155,119,69,192,35,214,118,36,141,59,5,172,189,94,12,81,60,
  23,4,62,130,32,44,128,71,10,40,57,162,74,183,183,136,160,32,101,168,123,89,179,17,21,232,39,253,182,209,161,146,77,226,246,37,185,168,80,55,
  223,57,240,171,174,127,95,11,158,185,101,119,189,49,151,140,183,40,49,163,35,134,62,137,216,209,215,49,59,28,19,66,165,14,92,36,111,210,143,232,
  152,17,206,26,72,151,234,179,29,148,180,43,141,107,107,98,213,47,49,113,2,36,46,224,155,254,46,253,28,185,192,71,124,181,59,77,196,113,130,106,
  173,142,36,73,176,126,221,56,22,22,26,56,126,114,6,87,174,94,192,191,189,250,111,248,183,223,30,198,228,166,113,60,188,239,62,108,88,191,1,91,
  110,186,19,95,249,171,167,177,123,247,125,104,181,22,16,134,1,238,190,235,110,220,114,203,36,22,23,23,225,5,117,196,222,32,254,229,95,15,96,160,
  30,162,219,105,99,124,205,36,158,248,179,199,49,185,113,13,94,253,237,65,252,227,79,255,25,237,78,15,63,251,249,207,209,234,52,225,121,30,206,157,
  191,132,181,107,39,160,22,38,120,82,81,5,191,56,108,17,161,75,251,90,83,49,166,206,245,68,199,26,235,62,247,247,61,110,23,22,203,36,23,225,
  214,143,96,180,40,235,42,111,32,186,162,9,10,114,117,41,79,221,186,160,122,97,241,71,167,159,147,155,106,112,207,183,96,27,20,134,89,131,20,93,
  227,25,249,143,9,44,115,30,223,92,49,140,92,68,35,246,20,219,222,66,223,141,14,128,115,197,138,182,204,138,110,111,230,62,91,87,57,239,242,57,
  226,62,229,118,216,52,67,194,237,201,166,121,104,106,63,158,36,73,191,251,92,172,236,143,186,245,45,95,83,14,126,68,49,156,28,12,200,215,49,101,
  35,228,207,182,21,142,114,99,224,64,130,152,43,220,185,34,159,129,226,179,122,189,30,115,166,38,12,88,44,182,170,166,107,40,191,111,61,192,230,124,
  163,41,232,203,231,186,151,117,7,20,12,177,234,127,236,145,182,213,122,150,149,11,142,60,207,67,181,90,145,138,8,211,110,121,233,123,245,173,107,133,
  206,29,206,11,56,253,19,19,93,114,222,80,37,31,119,254,140,133,227,7,125,38,226,90,226,250,41,216,207,253,166,117,108,57,183,22,242,61,141,250,
  130,115,224,89,118,19,249,247,193,92,186,61,227,147,140,29,67,142,81,88,105,161,134,46,29,231,130,236,117,45,152,109,99,16,135,175,46,229,201,21,
  121,152,152,155,149,184,19,184,165,163,253,66,68,42,31,118,185,239,51,208,233,116,209,235,69,168,86,106,240,60,15,151,47,95,68,181,86,131,231,213,
  144,36,45,0,30,118,238,218,133,168,215,67,16,134,216,179,247,51,88,187,110,24,149,208,7,208,197,219,111,189,139,7,30,184,31,11,11,87,16,134,
  21,68,49,240,232,163,143,225,150,91,111,194,236,233,84,242,49,56,56,136,167,158,250,83,160,215,68,107,185,137,177,85,107,176,176,156,160,90,173,96,
  96,112,8,87,22,174,0,0,206,158,59,143,106,181,134,145,145,49,44,44,92,201,22,174,238,208,180,125,207,166,51,230,230,95,158,146,52,179,65,186,
  121,102,106,139,254,251,30,183,73,62,161,99,208,93,128,166,78,251,44,183,58,213,49,120,58,105,137,206,142,72,175,229,247,172,50,11,155,20,199,5,
  76,186,200,2,40,131,41,111,230,52,221,206,237,3,244,80,160,172,161,173,32,206,133,133,165,29,223,108,89,0,155,4,136,142,223,36,45,210,177,221,
  220,252,181,165,89,117,247,79,171,226,185,179,130,118,27,116,189,55,157,46,158,91,187,226,220,224,154,43,153,230,163,107,151,53,217,181,35,8,2,54,
  211,196,101,28,132,134,92,119,134,170,13,59,100,219,56,213,129,130,74,50,168,59,131,94,27,141,172,19,33,117,215,224,178,10,185,61,156,87,104,148,
  34,179,210,189,94,164,20,201,81,139,54,177,150,162,40,66,187,189,132,209,209,17,244,122,145,22,56,113,207,63,149,108,232,247,67,89,114,161,250,100,
  171,192,150,178,236,84,106,33,119,208,148,11,232,132,230,220,165,163,160,89,106,228,38,217,40,131,185,76,5,123,182,189,190,76,102,232,147,130,29,21,
  0,157,36,9,226,36,73,213,61,164,229,241,74,83,162,46,26,59,238,97,209,23,169,219,240,229,27,151,29,2,116,108,155,13,8,211,195,130,166,33,
  109,7,135,174,160,74,110,90,33,131,103,250,210,250,119,155,93,175,219,237,162,213,106,35,172,164,173,49,143,31,127,15,15,236,189,23,95,248,31,190,
  128,227,31,156,196,212,45,219,112,199,109,55,225,231,251,247,99,221,250,245,216,121,207,221,120,255,221,163,184,229,182,91,49,60,60,130,13,27,183,160,
  211,110,225,212,233,143,112,207,174,187,17,134,62,126,251,155,95,34,72,238,199,248,234,181,56,252,230,33,84,107,53,84,66,224,95,255,229,55,152,220,
  186,25,15,60,240,0,78,126,248,83,92,189,114,9,149,74,190,57,204,159,63,135,94,175,139,53,227,99,88,92,188,42,181,67,77,10,46,1,244,190,
  202,44,8,83,84,107,235,94,38,167,109,184,197,40,87,90,127,156,198,173,187,190,205,146,43,138,34,197,105,194,180,241,209,53,72,189,161,41,104,146,
  217,27,157,148,129,130,20,91,16,66,217,64,157,252,102,165,77,82,76,205,79,104,97,155,252,60,228,103,104,99,251,109,182,103,242,38,203,106,229,12,
  197,135,166,134,2,186,231,66,245,201,46,217,17,93,96,197,177,227,148,189,225,138,117,76,193,9,247,89,58,80,235,18,116,232,130,75,154,202,213,73,
  73,232,103,138,182,195,54,123,63,46,211,194,165,137,117,93,66,233,181,77,197,88,194,166,145,219,47,242,185,133,140,165,84,25,99,185,107,32,36,13,
  47,215,72,167,8,162,243,123,72,50,39,16,249,243,76,231,95,238,236,81,108,223,45,203,141,84,208,42,159,185,9,130,192,67,183,219,70,16,84,208,
  233,180,20,61,179,46,131,171,118,233,147,131,8,222,43,89,180,207,22,159,207,201,80,56,160,107,2,101,242,189,136,49,243,62,240,69,57,137,28,204,
  37,9,221,35,204,146,13,157,4,196,228,185,108,195,92,166,98,57,91,177,179,156,81,249,36,97,199,80,188,126,93,203,86,238,128,52,21,9,137,52,
  156,46,197,232,106,154,77,245,131,182,142,89,101,89,2,91,84,163,219,236,76,126,183,220,223,57,203,36,91,58,33,142,219,184,116,233,34,198,199,39,
  178,251,127,238,185,159,224,145,71,62,135,207,126,122,39,150,151,26,248,197,254,231,241,238,251,31,224,221,247,223,199,107,7,14,162,213,237,96,235,123,
  239,97,98,237,26,156,153,251,8,255,240,253,31,96,121,121,1,231,206,158,193,212,182,155,113,117,97,1,63,253,217,139,217,97,41,223,199,177,99,199,
  48,119,250,35,44,44,46,227,187,223,253,30,122,221,110,118,175,173,86,3,231,206,157,195,77,91,111,198,204,137,19,154,148,147,253,192,119,233,246,103,
  3,40,54,0,170,75,7,209,6,28,31,151,113,115,135,187,112,222,48,69,242,101,89,120,113,40,203,64,182,172,251,128,73,46,64,255,223,86,92,163,
  243,59,54,5,183,174,207,177,12,112,149,153,248,149,102,152,40,115,201,105,108,185,235,187,202,130,116,191,71,83,147,46,233,70,78,210,163,219,139,56,
  57,83,153,96,134,186,83,112,251,185,201,234,209,150,102,117,93,15,174,123,55,87,155,162,11,4,92,58,98,138,195,153,3,252,116,93,228,86,102,129,
  246,94,100,214,151,122,25,171,190,192,126,161,184,80,237,234,151,16,157,46,15,96,82,207,230,160,192,90,235,156,106,100,134,59,93,15,98,206,5,89,
  115,16,142,157,78,127,38,183,178,139,227,14,58,157,142,164,159,150,109,245,60,5,180,230,86,114,32,224,89,181,220,203,239,205,39,63,171,130,230,220,
  9,37,183,1,52,103,37,243,241,167,239,218,103,178,53,40,128,103,58,135,4,59,47,198,67,231,98,94,16,234,73,29,12,221,49,87,153,128,218,132,
  185,76,189,51,184,125,228,147,128,29,67,46,141,169,211,136,185,180,83,148,31,42,23,77,187,30,82,226,26,220,193,80,102,195,228,30,174,173,26,94,
  55,110,46,93,205,165,143,169,173,13,61,196,76,192,77,252,125,254,194,57,108,222,50,133,36,9,225,121,49,218,29,31,255,248,223,126,134,74,48,128,
  110,212,202,152,1,223,31,64,187,151,192,243,234,248,240,212,105,124,120,234,60,146,36,198,162,23,32,8,135,112,241,226,37,236,222,189,11,99,171,55,
  226,234,213,43,8,130,16,190,87,233,119,79,12,144,196,169,101,208,194,98,19,0,208,237,245,0,175,2,15,189,108,60,239,189,55,131,251,238,219,133,
  151,95,254,149,114,143,174,254,141,186,123,228,230,156,108,231,84,102,67,224,38,188,206,239,251,227,52,110,206,130,103,165,110,15,186,245,37,131,72,218,
  188,194,212,94,155,130,112,193,218,185,0,28,186,46,168,172,131,218,128,149,1,202,101,236,193,108,135,134,75,202,81,102,45,116,204,63,23,52,115,204,
  165,238,243,4,195,239,146,122,212,89,44,233,44,157,116,204,248,74,92,45,174,181,149,186,28,104,152,24,98,155,229,167,9,200,234,106,33,108,153,36,
  218,44,197,180,87,184,20,71,218,50,102,174,231,41,125,70,92,7,195,156,41,22,45,174,125,201,133,195,35,140,102,66,2,63,213,41,74,93,143,69,
  121,8,7,132,138,193,176,248,157,184,15,70,19,133,153,46,50,204,114,199,66,10,238,19,163,91,5,117,224,16,90,101,241,121,42,211,156,104,89,92,
  174,219,35,175,249,69,193,150,78,174,45,201,223,173,185,33,82,62,54,85,70,195,187,116,121,48,37,65,175,5,115,209,192,208,21,115,253,119,236,72,
  52,208,28,27,194,165,239,92,244,126,101,210,179,46,135,5,101,23,117,105,71,211,166,110,99,105,184,195,158,3,26,54,97,58,117,222,224,14,9,93,
  229,181,0,229,31,28,63,142,251,238,191,31,95,252,242,95,160,26,86,17,39,17,124,31,240,17,32,142,35,4,97,5,190,159,218,30,121,253,246,172,
  113,28,101,237,87,145,0,9,18,212,42,1,194,48,192,95,252,197,211,104,183,218,128,231,193,247,2,229,157,118,163,24,81,175,7,15,64,28,197,232,
  244,82,45,90,220,75,208,238,196,24,25,173,99,213,170,49,172,91,183,1,231,207,159,213,178,102,174,109,182,185,244,38,101,70,87,234,170,32,31,134,
  220,132,255,56,142,91,222,52,86,202,128,186,68,228,186,34,13,27,40,149,181,169,101,54,49,249,122,50,11,46,63,91,151,134,47,178,14,151,186,135,
  112,65,200,74,217,106,221,151,188,207,80,240,44,107,34,109,206,32,38,54,132,106,171,233,28,164,41,107,121,206,137,185,99,98,140,57,86,198,245,64,
  179,61,75,221,252,225,88,104,155,255,170,203,152,174,213,113,133,11,50,116,89,5,211,89,4,32,11,168,202,6,130,54,79,116,26,80,113,103,138,202,
  90,122,233,185,16,4,210,51,55,101,84,105,243,21,179,172,131,50,183,58,230,146,251,172,116,203,76,136,165,155,58,6,185,249,139,251,122,165,123,132,
  186,46,226,56,201,158,9,175,121,166,5,230,121,243,20,62,147,151,119,76,204,29,80,18,150,153,182,7,167,80,130,9,10,236,69,54,65,124,95,124,
  246,245,196,92,46,46,58,46,235,242,147,140,29,89,42,69,222,196,57,186,93,214,154,114,155,14,231,133,76,15,191,149,128,12,122,24,200,96,198,84,
  221,109,3,207,220,1,32,71,76,38,235,51,249,43,181,185,9,216,168,140,178,37,244,153,202,64,204,247,125,156,57,115,6,211,175,31,68,173,86,193,
  66,39,65,187,185,140,78,187,137,86,167,5,248,21,12,212,106,136,61,31,173,230,50,186,157,54,226,56,70,87,72,47,144,134,227,189,168,139,32,168,
  224,11,127,250,167,56,246,206,219,152,57,249,33,146,36,66,37,172,192,247,43,168,133,1,252,32,68,165,22,160,94,171,163,86,171,3,62,16,134,192,
  240,224,16,134,70,87,163,86,173,34,241,99,92,186,48,15,223,143,173,135,168,41,93,77,193,167,9,60,201,115,197,245,249,151,141,52,63,142,227,46,
  243,69,215,41,101,36,117,5,75,84,63,203,217,156,113,1,4,101,254,202,128,5,10,60,57,125,166,110,3,21,159,43,179,224,166,3,192,133,37,47,
  11,162,203,102,63,92,193,168,109,28,166,32,220,148,138,52,181,149,150,65,183,169,88,181,44,115,165,43,192,225,206,7,186,111,186,52,168,145,127,158,
  118,100,148,53,237,46,62,174,38,205,182,237,189,200,105,122,49,110,23,119,30,46,152,49,233,230,221,51,0,2,192,4,90,93,52,239,23,77,123,32,
  80,230,215,5,72,171,239,37,111,111,157,255,188,10,100,228,236,15,199,8,235,239,79,126,37,162,104,80,212,132,168,18,19,175,240,76,108,235,135,6,
  77,244,153,8,128,205,49,249,58,96,172,91,67,188,238,90,94,179,158,34,55,41,131,185,228,51,203,132,185,92,246,63,147,155,149,235,153,244,135,142,
  29,195,50,233,82,219,97,64,1,34,213,158,150,113,34,40,251,64,56,123,34,78,103,200,217,9,217,10,175,168,198,208,164,19,213,165,32,93,238,137,
  138,232,127,249,242,175,181,45,40,77,239,134,2,249,103,127,252,44,46,95,185,130,50,144,65,199,34,185,76,46,83,69,174,152,3,54,102,120,165,186,
  198,27,105,220,244,221,203,186,201,50,108,175,110,13,233,60,88,185,172,135,13,24,11,22,204,164,89,230,170,154,117,64,64,7,88,56,16,39,179,253,
  28,64,212,189,219,50,140,96,25,0,94,4,12,197,223,235,118,187,206,197,52,46,141,111,100,22,71,23,240,149,77,77,114,158,175,38,139,55,177,175,
  216,230,166,107,193,167,92,208,103,202,14,81,187,65,46,24,227,178,3,174,153,31,238,61,216,152,173,50,107,84,119,222,200,58,112,46,32,114,245,228,
  78,175,21,103,224,181,8,162,121,208,43,175,33,185,72,76,231,43,93,4,210,80,90,131,83,121,152,78,54,164,91,239,66,66,145,102,82,125,173,238,
  59,125,62,106,215,56,234,240,226,251,158,181,192,89,39,219,144,159,87,10,220,124,54,107,167,3,197,166,185,33,88,114,245,221,216,51,84,215,11,115,
  113,228,67,25,204,229,154,41,250,67,198,142,242,191,133,194,129,227,90,211,156,148,109,165,150,64,242,225,167,211,182,152,236,187,116,224,134,179,208,18,
  41,99,186,81,230,134,240,73,233,3,83,254,189,107,73,43,232,210,117,46,7,19,183,208,104,138,132,75,247,93,188,124,217,121,188,52,130,212,53,130,
  177,245,183,231,198,235,178,208,116,218,80,219,130,184,81,199,77,175,41,31,156,46,13,47,116,153,19,29,75,72,1,181,174,25,140,9,152,234,188,60,
  117,13,52,92,116,182,46,243,157,3,96,186,224,192,6,50,203,188,27,29,75,40,203,80,232,125,8,240,44,119,147,211,5,17,186,76,133,206,39,214,
  245,192,40,115,223,166,180,42,13,100,76,133,100,166,230,62,156,54,222,85,87,204,253,123,217,223,213,237,153,92,209,150,139,115,78,89,6,76,7,232,
  76,243,203,101,125,168,215,85,157,36,228,239,11,77,180,12,72,57,194,71,199,70,115,239,66,128,93,209,140,197,198,228,203,44,171,89,154,225,21,44,
  230,116,239,150,178,147,166,206,173,180,121,139,90,104,200,49,195,169,158,92,143,75,138,239,128,119,43,129,177,83,227,74,51,98,101,48,87,89,203,84,
  211,186,163,206,74,174,217,155,63,4,236,232,121,94,58,59,146,4,161,199,44,30,93,187,68,219,2,230,218,8,203,197,66,46,169,0,157,78,205,22,
  241,136,155,231,138,250,184,162,27,83,100,172,59,76,92,180,64,212,55,85,220,135,236,191,235,26,229,153,14,58,10,90,108,213,229,174,154,40,90,24,
  160,171,72,55,1,46,221,65,111,218,208,116,207,199,165,26,247,70,29,183,45,11,162,51,173,23,235,74,216,94,201,89,18,83,119,65,110,158,210,141,
  201,102,161,199,31,68,69,127,91,87,61,171,173,123,163,14,108,233,0,148,204,62,219,60,100,101,7,4,206,129,200,118,48,234,180,116,122,251,48,117,
  79,164,210,5,29,128,226,230,186,204,134,184,120,39,219,230,158,206,33,128,130,62,26,16,148,5,147,174,239,157,219,219,245,254,200,30,27,184,82,102,
  151,211,106,150,181,172,92,9,137,226,42,41,210,217,103,217,190,108,157,31,249,108,27,15,144,139,65,135,27,144,86,229,24,122,230,56,63,127,97,157,
  143,249,92,79,20,137,134,237,25,185,103,127,19,169,216,17,44,43,76,129,167,234,212,161,123,238,124,48,146,95,159,158,243,158,81,83,109,35,80,116,
  133,217,212,186,245,122,98,46,46,16,254,164,96,71,133,129,150,11,80,104,58,201,150,242,213,165,105,76,155,149,41,178,55,165,212,92,61,78,77,105,
  94,157,38,208,230,113,202,165,22,108,214,40,52,74,211,29,2,166,244,132,233,115,202,182,38,183,93,155,99,187,108,172,143,238,231,184,3,193,182,177,
  139,231,79,109,128,92,14,108,78,199,120,35,140,219,54,143,184,67,85,246,140,45,27,96,232,172,197,168,126,150,222,11,215,250,216,6,208,93,14,127,
  155,133,157,92,52,71,89,90,93,144,171,99,90,108,140,172,139,30,149,2,54,155,119,184,248,25,154,250,163,239,80,183,150,233,62,75,175,79,55,119,
  147,245,156,203,62,161,179,200,51,177,77,148,245,211,205,73,206,107,223,246,204,77,65,147,110,44,52,203,98,106,245,174,243,157,165,254,231,114,3,160,
  149,102,30,117,5,194,166,12,144,139,238,211,38,37,41,50,234,118,144,43,190,210,6,40,190,17,72,211,160,149,130,80,234,188,193,49,242,28,35,174,
  98,8,207,250,30,139,222,224,137,38,56,0,97,160,19,112,82,10,221,62,44,108,2,229,49,113,192,156,50,232,249,253,123,133,249,32,58,32,242,153,
  128,132,149,49,113,243,95,87,152,109,194,10,43,193,92,116,125,208,115,225,147,132,29,67,219,33,35,111,170,130,109,211,69,25,220,131,226,30,160,60,
  56,221,6,226,162,57,54,189,92,151,141,199,197,40,157,166,22,56,22,136,86,195,203,140,144,201,50,205,196,132,151,101,119,92,27,81,232,172,162,76,
  135,56,199,216,184,52,66,40,195,158,152,162,67,110,46,202,135,190,142,57,252,184,143,219,52,6,174,13,177,235,103,94,107,51,16,221,253,218,244,115,
  244,251,101,138,195,108,18,26,185,190,192,5,4,234,64,12,101,108,108,155,185,204,2,235,236,149,76,128,73,118,69,16,109,132,109,29,45,77,129,136,
  8,110,116,100,199,181,48,194,166,12,150,109,188,174,110,30,114,42,180,76,1,37,23,228,234,50,64,174,126,205,20,112,112,235,223,68,124,208,247,108,
  218,79,168,47,52,215,241,177,8,214,188,21,173,27,242,19,76,113,161,202,132,234,192,177,231,113,78,42,208,2,98,126,14,21,193,116,113,238,242,239,
  156,50,170,186,57,193,107,231,197,30,234,107,0,46,50,175,105,91,48,47,107,203,93,2,155,60,40,40,62,3,49,38,174,201,202,245,198,92,156,227,
  215,181,98,46,90,44,111,107,178,243,7,141,29,61,47,237,68,232,202,28,216,42,233,109,233,53,121,144,46,135,252,245,212,25,235,152,14,241,192,117,
  158,193,54,0,66,219,53,219,216,55,91,151,157,149,128,231,50,135,164,204,46,218,180,182,220,230,162,235,250,85,126,99,47,70,152,166,119,80,166,69,
  232,141,50,110,110,253,184,56,8,184,6,96,46,65,131,11,240,183,217,179,233,198,35,119,249,43,83,80,194,177,2,182,204,131,11,248,162,191,99,219,
  139,92,129,185,237,57,202,223,163,153,9,238,30,184,198,63,220,190,90,182,201,140,73,206,117,61,44,176,108,227,18,193,131,205,25,128,94,71,246,88,
  183,121,184,115,237,119,117,243,135,91,183,166,110,179,92,70,200,228,42,192,189,127,91,19,25,78,98,232,10,8,120,32,155,3,105,181,233,74,202,188,
  10,112,200,1,105,174,56,79,253,28,61,43,205,17,1,57,75,153,130,202,220,167,89,14,120,96,101,84,77,172,114,241,221,39,165,230,48,119,127,244,
  231,77,89,63,174,97,76,113,126,199,210,121,20,94,55,204,165,251,153,235,141,185,100,25,33,39,35,251,67,199,142,242,127,67,151,130,9,186,185,233,
  218,28,234,38,167,44,125,112,97,125,116,159,35,167,9,77,172,136,46,82,225,12,245,117,54,102,92,36,76,65,179,252,249,81,20,89,171,166,185,231,
  68,93,53,86,90,76,96,106,41,206,217,232,113,108,142,203,181,5,8,40,107,209,38,167,110,77,193,154,11,75,234,10,192,110,132,113,187,6,46,116,
  1,211,67,216,212,44,98,37,86,122,28,16,231,192,139,60,46,57,83,211,235,245,148,231,165,3,109,38,11,33,174,107,166,203,70,203,129,15,223,247,
  217,119,39,64,157,205,155,84,30,143,203,134,43,223,179,206,15,218,100,245,102,146,110,233,0,165,141,77,230,152,54,110,175,213,61,107,155,171,1,213,
  34,211,119,199,185,188,112,45,223,185,64,71,247,188,104,144,70,63,119,37,110,33,244,90,244,115,197,220,22,186,249,50,197,154,156,222,83,183,151,148,
  105,180,98,38,96,146,172,112,45,213,252,82,93,63,160,234,114,115,96,236,50,191,56,86,58,255,127,191,240,189,252,204,139,51,64,157,23,22,22,247,
  94,55,127,253,164,192,64,243,122,227,114,30,205,186,125,160,248,142,245,90,106,190,161,71,232,252,62,93,49,23,37,57,174,55,230,162,107,253,191,99,
  199,4,161,103,73,227,218,162,64,29,16,16,69,78,20,120,152,164,2,92,231,50,142,49,210,109,120,180,133,176,11,219,100,218,200,232,207,11,27,39,
  83,33,131,205,193,128,78,58,211,66,112,97,177,109,173,193,77,54,107,178,78,80,110,39,107,98,69,93,100,14,38,102,212,164,239,163,110,36,174,146,
  10,249,208,188,145,198,77,65,156,88,51,186,150,242,156,79,51,151,210,118,73,65,93,107,132,110,3,82,180,176,205,181,56,81,46,138,180,129,32,151,
  117,99,91,67,66,14,177,18,6,91,23,4,155,88,77,93,209,161,109,63,226,50,95,101,178,49,186,108,71,153,119,195,49,225,186,181,104,11,10,117,
  192,208,150,17,51,129,111,151,12,147,203,154,144,207,34,78,62,85,118,15,161,44,182,60,103,108,89,72,151,32,207,166,77,87,159,105,162,57,67,114,
  32,40,3,73,219,20,163,89,77,10,166,139,224,57,49,182,143,150,127,62,181,198,19,22,98,17,146,4,138,165,156,203,251,212,105,161,213,128,40,42,
  224,13,23,150,151,3,219,233,123,45,58,108,184,226,131,107,193,92,84,206,116,189,49,23,213,241,187,212,7,253,161,99,199,144,174,18,157,6,178,108,
  218,208,166,125,52,53,123,208,129,31,211,191,241,45,78,19,231,131,215,37,157,46,119,99,51,89,14,21,91,165,150,75,125,219,216,45,46,61,227,10,
  116,184,46,138,116,242,152,34,57,221,36,227,180,124,52,117,71,35,72,154,82,118,181,182,162,135,218,141,58,110,14,16,201,25,3,23,86,81,231,68,
  225,146,234,53,165,134,93,128,156,137,73,181,89,222,209,57,40,2,84,142,129,43,3,92,76,5,43,182,103,96,59,28,76,12,150,75,61,3,119,64,
  235,130,60,89,43,232,226,209,172,187,95,157,91,205,74,2,38,83,183,46,46,213,207,177,207,114,230,196,148,61,40,107,61,200,1,159,50,45,221,105,
  166,73,126,230,220,156,93,201,231,113,45,214,185,204,142,110,239,226,64,131,142,28,40,102,11,248,98,189,226,231,21,139,12,117,236,50,119,246,233,192,
  165,220,158,91,128,202,162,91,130,45,139,168,254,188,137,44,224,244,213,122,214,92,13,156,220,50,201,58,22,60,65,28,203,133,143,212,62,180,248,44,
  175,7,230,210,201,40,174,39,230,226,26,38,233,246,159,63,120,236,40,188,231,77,236,134,248,97,81,12,116,61,173,123,116,19,211,230,176,192,201,39,
  116,215,50,45,46,91,235,70,23,32,75,193,7,151,90,183,121,41,202,191,239,2,140,105,17,142,43,203,170,43,96,148,91,0,187,222,191,105,178,149,
  73,153,186,88,151,217,14,190,50,239,236,227,60,110,185,145,10,87,32,102,187,174,88,163,58,246,143,22,125,216,230,153,169,59,231,74,138,202,56,182,
  73,247,59,101,172,144,76,115,215,228,221,174,11,30,56,199,18,219,184,109,99,227,174,37,175,97,91,209,152,107,234,153,107,240,193,121,28,115,204,138,
  235,90,210,249,167,83,185,5,13,86,233,61,154,130,60,211,56,116,160,169,76,141,4,23,4,202,123,171,236,156,164,91,255,186,66,212,50,1,8,119,
  255,186,117,195,145,6,166,53,203,101,112,242,247,236,247,253,141,209,255,59,7,144,19,150,145,78,255,91,12,142,117,153,14,14,76,115,109,195,101,183,
  16,215,238,144,238,210,22,115,246,207,133,201,207,63,219,211,0,231,226,186,206,157,53,220,229,124,101,49,151,205,201,232,122,98,46,151,250,131,79,2,
  118,148,103,129,207,117,152,225,210,98,180,237,171,238,119,202,128,81,215,214,182,116,145,138,162,63,157,158,143,239,20,148,176,27,146,208,179,209,207,214,
  221,175,11,192,213,253,151,142,145,62,3,215,22,201,244,254,76,247,235,18,201,9,125,40,183,64,184,168,82,184,9,232,162,58,29,80,177,217,139,201,
  17,174,206,246,73,0,76,19,0,186,81,198,45,127,201,135,53,237,16,87,38,106,215,253,188,206,150,78,183,94,196,191,233,52,112,58,171,39,238,123,
  166,198,27,116,77,153,14,69,91,81,175,9,236,216,14,33,186,63,152,214,183,107,64,197,49,129,38,38,92,183,247,209,123,48,189,51,122,79,226,119,
  100,77,58,247,156,109,238,37,242,248,226,56,206,246,71,113,93,89,242,192,117,76,164,89,9,170,3,166,239,86,222,151,101,160,163,59,216,100,137,28,
  125,159,226,250,220,207,148,237,176,102,2,15,166,140,159,235,123,211,49,119,84,111,77,245,226,52,67,169,43,124,146,193,29,5,118,38,86,149,254,156,
  112,146,208,21,11,210,123,150,207,236,148,184,137,251,76,116,10,71,228,26,5,122,95,114,129,94,254,61,20,230,142,184,30,253,163,219,147,76,110,14,
  20,240,211,54,227,162,8,51,73,98,134,69,247,88,160,205,237,69,58,134,84,104,127,77,152,107,37,166,3,215,130,185,108,247,241,73,194,142,98,239,
  11,185,31,48,249,14,211,15,208,249,39,187,218,170,185,110,36,182,104,222,165,179,156,252,111,156,174,134,123,128,242,51,209,21,52,232,116,145,215,114,
  207,101,152,12,234,86,192,29,224,166,10,124,174,19,162,28,177,153,216,56,91,10,182,76,241,151,204,64,202,135,182,124,104,234,10,112,110,212,113,115,
  12,239,74,215,133,203,184,117,78,7,38,176,200,177,67,46,13,40,108,109,116,229,247,163,123,87,174,155,102,89,221,57,181,18,115,201,222,152,158,153,
  238,26,54,102,212,69,175,109,250,60,93,48,78,193,40,103,245,102,3,239,98,111,112,149,197,200,107,193,212,117,79,252,215,228,163,109,203,254,113,77,
  28,116,65,154,78,234,64,247,76,46,200,148,27,222,112,89,28,151,172,22,55,111,86,210,118,152,102,166,56,207,115,219,124,115,117,180,40,254,122,34,
  129,103,241,252,18,246,217,217,230,48,87,40,171,74,42,252,12,136,170,255,229,254,158,6,3,81,20,91,125,208,77,146,15,238,124,228,100,30,106,135,
  66,207,33,248,240,172,231,178,174,166,202,212,101,207,85,70,118,61,49,151,237,62,62,137,216,209,231,110,202,116,112,201,236,129,41,205,226,146,142,228,
  190,79,55,107,110,35,52,129,32,202,110,208,107,217,216,45,219,125,113,19,128,190,20,202,242,232,22,66,153,226,37,211,70,40,119,82,163,247,206,165,
  142,229,8,82,28,8,58,0,78,199,42,167,188,232,239,232,34,74,249,48,50,69,223,58,102,136,110,130,189,94,207,153,201,185,17,198,45,207,15,211,
  6,127,45,27,137,206,237,160,204,26,189,214,205,139,178,137,114,166,167,76,193,165,105,60,174,243,66,204,123,93,102,200,198,80,155,238,181,76,139,233,
  149,220,3,231,63,111,98,198,77,146,7,151,121,35,7,136,130,41,212,49,217,220,190,99,99,237,57,150,139,2,126,83,43,98,83,240,71,83,183,92,
  17,174,248,190,236,211,44,143,77,38,23,4,251,94,102,221,233,164,80,101,216,104,121,175,182,157,155,174,231,137,249,223,120,80,152,23,6,38,74,214,
  161,215,139,180,191,67,139,156,77,140,112,145,13,132,49,51,166,251,83,102,143,82,45,232,60,205,125,168,76,124,62,191,101,182,57,33,129,107,196,158,
  243,92,224,230,138,185,76,235,231,119,141,185,108,247,241,73,196,142,62,253,101,19,243,82,102,97,218,38,50,199,226,153,88,45,57,5,71,171,252,203,
  108,100,58,143,68,170,73,52,165,83,93,128,174,24,175,46,245,161,59,124,108,182,46,114,138,206,4,252,108,11,144,251,92,46,37,200,185,120,200,135,
  167,13,88,232,170,199,101,22,136,219,84,117,7,175,208,249,114,69,144,55,218,184,77,0,209,165,21,185,72,115,151,145,240,200,129,147,13,164,209,121,
  106,147,53,153,36,42,244,93,200,193,101,217,195,159,59,60,77,95,84,14,96,98,232,117,239,156,110,206,58,179,127,122,200,155,54,95,106,251,70,229,
  16,54,240,207,29,42,186,64,78,39,251,162,172,138,14,8,175,68,175,204,189,51,57,208,215,101,4,108,197,138,114,134,137,126,126,175,215,99,247,93,
  153,92,224,164,35,34,48,208,129,20,249,236,225,156,84,202,6,35,52,139,192,129,119,250,25,242,250,211,181,45,118,13,132,57,34,33,191,215,8,113,
  28,25,129,180,12,42,169,69,36,39,159,88,185,75,74,82,112,139,82,3,40,183,236,161,238,179,69,147,153,56,230,158,71,2,78,194,146,223,99,162,
  121,38,252,231,113,88,160,12,230,178,145,30,191,107,204,117,189,238,227,15,9,59,134,46,169,67,211,33,238,170,7,44,235,111,76,11,57,174,135,245,
  150,78,154,97,90,96,28,96,230,14,30,93,90,95,87,84,104,179,219,50,9,224,229,107,113,85,195,101,125,142,185,52,134,60,38,185,93,181,110,50,
  185,6,22,244,48,183,165,167,77,139,75,199,12,223,104,227,150,127,215,102,217,35,255,188,174,81,73,153,162,51,27,8,180,217,36,154,82,93,58,70,
  95,102,51,105,32,34,52,115,58,32,41,235,36,117,46,44,174,22,143,166,140,142,169,91,28,189,39,147,55,56,149,166,232,236,203,202,110,234,182,128,
  205,182,135,233,158,85,153,140,66,153,226,67,19,208,115,109,121,175,251,119,46,131,163,43,190,51,221,135,41,104,42,243,76,92,15,101,147,94,156,243,
  179,214,89,32,82,107,82,157,59,129,28,160,219,210,212,197,243,40,7,136,69,73,152,28,180,81,198,61,103,117,211,207,215,119,39,165,50,10,157,70,
  93,150,148,9,203,184,50,5,133,220,117,243,53,105,99,236,77,239,215,203,156,132,56,112,233,90,192,198,49,166,31,7,204,117,173,247,241,135,136,29,
  67,19,21,46,34,94,93,151,25,151,234,80,157,70,75,7,98,117,98,116,219,6,168,235,52,168,123,249,220,65,105,3,0,101,52,108,98,60,50,163,
  40,24,14,142,109,177,85,1,115,233,72,142,157,41,179,249,219,38,169,203,152,86,226,97,45,255,174,172,137,20,239,73,110,185,204,205,31,151,202,228,
  27,113,220,52,11,224,226,231,108,186,150,56,132,93,90,207,114,76,25,119,24,155,54,66,153,213,214,165,207,108,224,73,92,71,94,63,226,217,202,213,
  220,84,119,183,146,13,217,197,53,193,85,207,236,170,57,213,29,244,101,218,158,151,93,187,166,251,51,1,108,93,123,93,90,204,166,43,192,213,221,171,
  172,171,214,237,235,58,175,109,174,40,136,179,248,43,187,254,185,198,11,242,252,227,92,8,86,146,134,230,130,10,91,33,158,233,185,185,120,102,83,146,
  72,6,238,52,120,77,127,222,183,146,8,194,171,151,63,19,245,142,21,65,16,106,173,241,92,64,87,122,77,151,189,17,142,251,93,210,7,225,137,229,
  90,137,114,111,182,98,109,14,87,152,50,23,58,204,101,194,79,255,222,152,203,228,101,111,187,143,63,52,236,40,214,107,104,218,104,185,38,3,174,226,
  117,170,157,209,29,72,182,106,82,46,34,214,57,80,232,88,47,206,63,117,37,69,91,174,93,236,40,107,96,242,76,44,99,217,229,162,103,51,21,38,
  184,128,39,211,247,57,177,61,247,78,229,195,86,110,161,237,18,93,211,5,90,214,199,246,70,28,55,5,186,178,31,169,205,207,153,187,47,219,65,236,
  50,143,109,154,111,1,104,93,138,16,109,95,178,195,2,125,63,58,150,153,126,207,118,63,58,87,2,91,32,163,243,9,55,217,6,150,97,229,93,58,
  33,154,88,226,50,133,161,215,106,147,71,11,116,109,157,242,116,214,107,186,130,86,83,112,235,18,196,148,97,172,228,64,64,119,134,208,204,21,119,111,
  101,93,89,168,173,160,188,86,105,139,112,186,78,77,193,176,169,49,11,93,35,92,70,132,74,72,138,215,137,251,206,27,126,6,96,85,171,54,179,189,
  155,14,80,203,175,74,140,165,120,102,171,141,81,210,192,99,197,92,162,19,75,205,255,142,0,218,118,217,153,201,30,148,35,1,56,204,85,214,119,254,
  223,11,115,149,189,143,63,52,236,40,38,165,7,32,140,251,98,37,79,115,51,186,244,154,107,91,69,46,61,165,139,0,92,205,233,185,38,22,166,13,
  156,74,40,76,47,209,164,41,182,49,42,69,99,120,158,5,144,199,45,51,28,186,251,229,216,41,46,242,162,142,9,28,123,84,198,67,213,101,194,155,
  188,76,101,89,130,139,222,150,122,196,82,96,105,242,251,189,17,199,77,159,63,117,237,40,11,204,92,27,236,184,50,179,166,121,33,223,103,153,96,133,
  50,248,84,63,73,159,133,169,97,4,39,57,90,73,186,206,84,32,76,215,215,74,190,76,237,202,41,96,227,210,164,186,103,80,6,184,185,204,3,93,
  243,16,19,139,102,34,15,196,61,113,251,47,87,192,41,143,129,2,115,221,254,109,203,22,154,220,135,108,103,140,233,221,115,0,183,108,38,64,78,51,
  155,154,80,216,222,175,232,144,171,11,18,116,150,158,242,243,183,167,187,189,194,223,213,150,224,244,222,224,144,145,208,187,25,81,64,43,180,206,114,151,
  191,107,5,202,252,115,246,10,215,201,223,115,210,119,30,73,156,51,76,220,189,219,236,56,117,24,232,247,141,185,56,57,154,233,62,254,96,177,99,191,
  27,80,186,190,60,15,190,33,210,48,49,70,186,65,172,212,223,143,106,135,86,162,235,227,170,244,101,41,5,253,190,173,184,206,101,236,148,41,43,147,
  166,208,105,168,229,47,46,13,194,57,167,232,4,253,101,238,211,22,61,83,80,101,218,248,123,189,158,19,211,201,129,125,202,162,138,180,163,252,30,57,
  7,153,27,105,220,226,179,116,197,119,220,220,166,154,97,174,99,93,25,75,41,142,77,224,198,33,23,197,138,131,89,62,252,197,125,184,116,171,227,126,
  166,219,237,106,231,162,120,214,58,59,36,27,51,73,231,191,205,201,128,130,214,178,235,71,119,15,166,189,209,20,100,115,108,11,245,51,247,28,186,201,
  234,210,144,98,94,233,158,163,73,211,110,115,164,144,89,94,238,121,152,50,22,92,10,183,140,77,97,25,231,20,58,215,109,7,60,231,28,98,98,211,
  108,126,190,166,207,176,185,22,112,215,151,129,135,92,180,169,115,24,113,25,111,241,125,249,70,22,55,73,160,200,218,120,144,171,251,99,154,191,46,127,
  108,115,67,231,21,93,100,205,125,63,128,112,219,16,115,87,126,166,220,254,45,23,124,114,5,253,186,192,203,230,92,244,113,192,92,46,247,241,7,141,
  29,165,251,15,57,211,245,50,58,46,142,69,48,45,106,221,38,91,166,37,170,238,154,98,177,154,34,37,219,61,150,149,117,184,178,13,174,207,208,229,
  94,185,20,189,220,65,107,165,214,94,58,80,69,25,50,87,173,48,5,254,166,102,8,28,235,37,222,161,204,246,184,26,247,223,8,227,214,21,6,233,
  154,50,152,170,157,77,242,16,151,64,78,20,195,81,86,89,215,182,156,254,59,103,197,68,35,120,147,143,180,174,32,210,196,74,80,230,82,100,115,104,
  48,85,38,27,97,179,248,163,122,111,215,66,51,250,254,117,13,150,76,217,37,215,253,193,213,6,74,55,159,202,238,255,43,57,232,76,251,222,74,139,
  160,77,239,187,12,243,108,42,200,165,243,215,5,252,80,73,152,45,144,213,189,27,157,119,176,137,61,166,150,121,182,162,121,177,15,148,63,199,18,45,
  179,188,178,66,75,222,225,98,37,49,108,174,97,246,172,227,21,0,221,20,28,184,116,115,53,157,233,156,204,174,236,58,255,125,98,46,219,125,124,146,
  176,163,207,89,235,184,216,40,209,69,86,134,245,48,29,138,182,8,152,179,116,163,29,230,116,41,33,202,4,154,216,14,27,99,215,235,245,216,138,102,
  46,197,232,210,84,197,102,95,7,228,22,77,46,76,23,253,62,117,5,225,222,25,39,194,167,206,9,186,212,38,199,108,201,172,164,139,46,151,155,7,
  180,82,157,155,204,55,250,184,41,195,75,223,63,181,97,51,53,179,144,127,135,154,220,155,214,29,119,24,115,233,117,23,43,37,29,43,227,194,254,218,
  88,24,29,211,38,2,31,29,43,82,198,121,197,214,169,207,5,52,209,61,74,174,137,224,100,26,186,245,75,189,83,117,149,253,186,181,224,2,236,77,
  29,184,168,228,162,236,30,47,188,207,229,255,23,251,53,125,54,182,53,166,251,190,105,190,216,10,132,92,230,179,206,178,80,62,127,108,30,238,174,65,
  1,181,217,164,146,39,27,64,211,177,119,212,62,144,186,149,148,9,2,245,32,85,239,137,76,159,167,169,91,160,233,61,201,69,249,28,227,93,252,195,
  93,156,203,6,165,250,106,215,241,216,230,18,167,95,54,145,149,54,240,247,113,192,92,186,250,128,79,50,118,244,117,15,136,166,139,117,96,75,199,238,
  184,22,166,232,54,93,155,94,141,78,60,122,232,209,135,82,214,207,148,75,71,203,207,73,46,14,20,52,255,181,78,4,155,102,72,199,0,186,92,95,
  103,117,102,98,135,56,253,17,253,59,183,105,80,207,87,23,223,97,23,6,141,123,15,156,193,252,141,54,110,25,20,153,10,17,185,121,66,3,34,241,
  115,2,76,234,2,51,83,250,218,4,208,77,235,146,50,171,38,175,115,19,184,53,21,53,235,214,111,183,219,101,3,76,91,106,188,12,203,105,27,159,
  45,168,45,195,196,185,180,148,229,10,107,116,45,191,169,28,192,212,128,165,44,251,107,99,136,104,144,32,167,193,77,193,22,221,143,117,141,27,116,115,
  66,204,151,107,97,185,77,204,182,88,171,28,99,43,159,157,156,13,157,137,196,160,243,70,183,191,112,29,101,185,119,164,59,195,203,188,103,91,86,179,
  248,254,197,251,14,178,226,67,249,143,0,217,105,71,191,196,0,128,197,159,56,211,65,139,226,69,250,239,246,185,170,178,206,194,255,153,6,47,156,44,
  206,228,73,79,179,3,242,156,208,101,194,110,68,204,101,218,195,63,73,216,81,190,126,168,19,105,219,16,189,174,64,136,86,20,175,100,115,230,170,241,
  109,155,56,181,87,226,94,62,245,142,53,109,14,166,123,151,211,226,38,246,200,116,224,174,68,54,99,74,127,186,164,78,117,230,236,156,198,142,211,13,
  211,127,19,78,12,166,195,73,28,124,226,48,115,61,104,117,239,81,39,197,184,17,199,45,51,91,186,77,218,166,49,149,15,102,83,33,145,233,96,164,
  207,140,43,122,227,2,18,121,29,80,217,7,253,92,174,160,178,76,192,65,139,204,232,61,85,42,21,118,157,219,130,72,57,109,199,253,158,203,102,206,
  117,6,44,243,229,210,76,198,164,25,52,29,68,46,90,102,221,186,177,5,230,226,153,233,158,183,201,131,89,6,138,186,134,68,174,1,177,45,48,90,
  73,91,96,215,160,73,140,191,140,236,132,3,240,174,62,220,182,0,147,155,255,220,254,160,123,86,20,228,155,50,70,201,202,173,48,72,1,162,78,3,
  46,246,33,31,65,16,18,112,110,243,100,150,153,241,132,245,177,118,169,23,209,189,35,90,211,34,147,93,180,115,165,112,237,145,191,207,129,200,27,1,
  115,137,251,176,21,199,254,33,99,71,101,29,39,73,146,78,197,21,232,102,77,44,134,77,110,112,45,76,173,105,131,51,49,178,101,55,57,211,230,105,
  138,46,203,164,122,202,222,191,173,201,128,14,156,81,160,166,147,24,216,88,120,249,231,109,21,228,153,208,190,15,86,189,146,197,170,220,225,171,75,185,
  220,136,227,150,55,10,29,147,30,134,33,59,142,50,32,155,203,108,152,170,155,93,64,156,24,155,72,201,153,214,134,46,192,176,237,15,220,243,48,181,
  143,182,237,57,84,2,194,217,135,113,251,159,11,131,108,42,52,92,137,181,156,11,35,237,106,249,230,242,89,180,75,159,105,12,242,103,218,214,18,231,
  133,91,236,92,151,20,50,47,38,45,178,173,203,102,217,253,221,244,110,93,10,146,100,38,157,18,42,46,214,163,186,86,227,178,116,75,158,91,182,150,
  202,156,199,183,110,30,216,152,85,83,71,84,46,99,229,58,143,213,6,43,188,15,179,202,86,3,102,157,53,74,175,133,32,8,172,93,74,117,207,139,
  238,123,182,224,77,206,82,232,92,121,62,238,152,139,222,135,222,242,240,147,129,29,19,244,37,28,96,110,216,84,57,46,79,58,29,123,102,114,142,224,
  52,157,92,138,145,106,86,184,241,209,214,214,43,57,152,92,1,173,204,126,233,38,13,103,80,111,122,121,46,7,170,73,143,169,251,127,231,73,160,1,
  223,38,86,148,115,191,208,249,169,202,254,162,84,79,104,251,226,64,103,16,4,218,134,19,55,218,184,229,121,203,177,229,58,221,46,119,31,156,150,187,
  236,193,166,155,19,54,247,10,147,230,140,243,69,149,199,102,27,167,201,40,95,124,79,236,39,38,137,135,169,0,147,91,31,242,30,196,105,201,169,84,
  165,12,88,115,153,63,220,33,67,223,3,231,202,98,123,86,182,96,107,165,76,57,61,0,169,246,217,20,236,153,26,200,152,156,56,228,191,203,159,105,
  42,76,114,45,40,228,206,5,147,174,159,43,136,50,201,119,108,123,137,142,0,160,103,34,71,22,136,245,160,187,95,185,222,130,211,116,115,128,93,252,
  161,204,107,89,32,77,53,252,249,231,23,229,40,197,235,121,240,188,188,123,169,237,44,229,24,78,145,129,20,247,97,243,95,55,117,34,150,155,163,113,
  207,139,238,19,244,158,110,52,204,37,223,135,233,188,248,196,96,199,56,70,200,37,66,116,7,5,157,148,54,6,83,199,66,233,188,136,117,62,180,52,
  213,42,255,142,120,216,114,243,9,83,27,80,202,22,200,135,164,188,32,76,169,37,174,123,13,125,38,186,212,144,75,247,42,249,94,168,33,185,204,214,
  152,82,225,58,123,60,218,66,86,39,101,144,117,193,221,110,87,73,145,187,176,54,58,150,206,101,146,234,26,72,216,154,139,220,104,227,230,28,63,108,
  169,49,93,67,30,186,193,153,216,85,27,3,39,175,17,186,25,154,58,243,25,141,231,45,159,233,234,190,225,242,92,108,239,168,76,38,138,250,232,234,
  52,127,101,83,193,166,67,223,214,162,94,102,208,204,29,229,188,82,146,25,27,75,42,203,144,168,155,140,162,11,116,232,168,168,75,179,234,154,175,232,
  128,13,247,153,84,187,106,146,177,200,193,10,199,132,219,8,8,27,240,213,157,71,242,249,65,107,24,196,231,216,158,163,188,78,109,193,0,45,110,166,
  197,137,114,80,38,206,64,93,163,28,221,254,105,106,168,99,147,77,216,28,66,92,156,99,76,243,93,103,19,106,218,83,185,172,136,77,30,96,243,206,
  23,207,216,228,20,243,113,198,92,242,181,110,228,251,184,38,236,216,239,155,226,121,30,194,64,2,29,255,211,87,191,170,93,52,113,146,100,126,209,84,
  3,236,170,221,210,125,153,64,145,144,152,244,43,8,20,207,106,238,51,68,211,145,108,162,91,2,4,186,185,112,109,91,77,155,130,78,152,206,249,131,
  154,138,0,117,233,49,197,10,13,102,213,87,34,231,179,86,192,124,149,57,244,196,231,121,125,83,113,23,109,37,199,84,233,92,19,232,132,118,213,88,
  222,72,227,182,141,93,94,168,38,102,202,165,101,182,11,136,163,247,96,90,147,220,220,138,227,56,51,153,231,218,192,102,44,62,114,47,205,108,253,72,
  243,214,117,190,154,138,29,57,144,171,211,198,203,94,181,166,122,5,93,151,177,40,142,17,88,90,43,139,125,169,215,235,1,158,135,64,35,247,208,166,
  69,13,107,95,103,181,86,176,92,235,179,38,166,57,231,90,184,227,186,246,228,251,15,194,48,247,104,16,245,8,98,60,186,57,37,158,121,137,38,57,
  73,255,172,72,164,51,139,3,143,217,181,250,93,197,178,6,65,113,156,141,115,37,178,63,93,86,65,167,29,205,230,177,225,124,211,157,59,236,25,24,
  69,8,89,160,65,231,139,122,114,228,235,62,48,218,197,37,164,23,119,162,204,5,79,125,231,134,198,93,182,214,237,226,62,69,97,152,124,45,211,122,
  230,158,9,101,219,229,182,227,220,217,189,18,141,186,110,127,228,18,13,220,154,189,209,48,215,31,202,125,92,11,118,244,185,212,24,7,204,116,209,161,
  139,251,131,156,102,117,137,54,40,45,47,110,220,131,89,183,42,71,237,217,184,52,157,174,56,96,203,85,206,234,211,73,124,106,138,86,155,203,85,231,
  38,246,80,167,179,82,158,143,165,43,158,236,193,195,201,15,184,77,157,166,80,56,107,43,93,90,212,235,143,73,167,245,19,207,173,71,139,132,164,121,
  228,251,62,98,249,51,136,63,175,78,7,41,63,227,27,117,220,58,93,149,60,78,174,177,131,110,222,137,177,208,52,185,41,237,172,59,101,185,53,73,
  211,190,44,104,53,172,205,236,190,61,15,30,213,124,107,246,13,211,220,132,230,89,208,159,151,193,158,71,214,162,108,139,165,115,70,144,217,80,58,103,
  122,81,4,72,207,92,108,188,186,67,32,12,67,5,108,39,73,130,72,146,30,185,62,87,173,29,39,217,179,149,107,245,159,153,182,137,144,0,68,37,
  172,163,92,117,135,33,1,207,242,152,60,6,88,197,66,58,99,153,27,90,73,6,179,95,250,190,159,205,25,165,115,44,213,117,139,20,47,179,119,38,
  46,4,6,119,198,49,246,144,116,191,240,29,0,154,169,160,57,155,99,228,124,142,9,96,206,223,91,130,62,196,32,18,4,115,157,73,54,175,36,18,
  34,159,247,121,51,147,32,228,187,148,10,9,133,73,250,69,51,26,194,163,62,146,214,50,93,151,246,140,64,255,158,251,206,31,186,55,102,114,14,114,
  249,202,215,6,16,197,137,19,94,184,17,49,215,127,199,142,30,66,110,178,21,232,115,41,170,55,165,54,61,207,43,164,205,197,239,217,244,183,156,102,
  134,139,26,229,127,51,177,3,166,34,166,132,28,174,2,56,71,81,4,175,207,134,208,54,178,174,233,60,37,234,209,165,50,4,3,231,251,236,225,80,
  96,75,75,46,98,83,139,75,155,145,190,139,35,73,182,209,246,15,63,155,87,178,167,249,255,12,88,201,233,22,177,181,49,17,109,156,36,8,28,139,
  177,110,148,113,179,209,188,156,197,232,31,82,182,247,109,219,148,56,131,120,118,14,147,162,98,19,99,32,55,152,225,152,130,94,20,41,160,192,4,152,
  11,255,78,214,58,203,30,137,181,230,144,121,200,100,13,100,35,117,90,3,208,203,7,66,73,70,33,192,65,16,4,202,94,73,125,155,11,157,249,52,
  193,150,105,127,51,53,245,17,251,79,34,130,42,234,137,174,187,174,196,188,184,50,109,153,139,134,4,166,226,56,54,202,15,184,102,32,32,103,140,223,
  95,55,137,52,46,157,92,142,99,220,181,117,32,36,128,48,73,237,88,22,215,180,14,109,140,164,46,11,41,157,71,186,51,97,165,128,206,235,179,235,
  249,186,79,140,186,253,76,102,160,105,193,94,88,91,210,61,248,204,124,162,108,111,250,87,183,236,10,231,154,5,70,123,27,247,131,160,252,158,100,57,
  154,151,3,103,164,128,202,181,246,168,44,35,42,220,88,162,40,86,222,101,2,187,219,203,141,128,185,92,26,145,125,210,176,227,255,63,0,218,241,195,
  65,28,3,111,116,0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* TalComponent::bmp00135_png = (const char*) resource_TalComponent_bmp00135_png;
const int TalComponent::bmp00135_pngSize = 134617;
